import React, { Component } from 'react'
// components
import SeatMapView, {
  SEAT_GROUP_TYPES,
  MESSAGES,
  SEAT_STATUSES
} from 'seat-map-view'
// hoc
import { connect } from 'react-redux'
// actions
import { getStatusesStart } from '../../store/actions'
// assets
import seatData from '../../assets/map_samples/venue.json'
import {
  cancelSeatReservationStart,
  reserveSeatStart
} from '../../store/actions/seatMapActions'
import { notify } from '../../store/actions/notificationsActions'
import _forEach from 'lodash/forEach'
import _values from 'lodash/values'
import { jsonValidator } from '../../services/validationService'
import { toast } from 'react-toastify'
class App extends Component {
  constructor(props) {
    super(props)
    this.predefinedSeats = null
    this.seatMapType = null
    this.selectedBlock = null
    this.state = {
      activeGroups: [], // example: ["1"]
      seatsToRecolor: [], // example: [{ seatId: "1-1:1", color: COLORS.HIGHLIGHT}]
      buyerInfo: {}, // example: { "1-1:1": { name: "Name Surname", email: "namenamename@gmail.com", order: "5642649", ticket: "cdsdss"}}
      seatData
    }
  }

  componentDidMount() {
    window.document.addEventListener('readyToDraw', this.showMap)
    window.document.addEventListener('cancelAll', this.cancelAllReservations)
    window.document.addEventListener('unselectBlock', this.unselectBlock)

    if (process.env.NODE_ENV !== 'production') {
      window.seatData = seatData

      window.tierPrices = {}
      window.tierNames = {}
      window.allowMultipleTickets = true
      window.ticket_type_tier_relations = {}
      const eventReadyToDraw = new Event('readyToDraw')
      window.document.dispatchEvent(eventReadyToDraw)
    }
  }

  componentWillUnmount() {
    window.document.removeEventListener('readyToDraw', this.showMap)
    window.document.removeEventListener('cancelAll', this.cancelAllReservations)
  }

  shouldComponentUpdate(nextProps, nextState, nextContext) {
    const {
      reserve: { loading: reserveLoading },
      cancel: { loading: cancelLoading }
    } = this.props
    const {
      reserve: { loading: nextReserveLoading },
      cancel: { loading: nextCancelLoading }
    } = nextProps
    const loading = reserveLoading || cancelLoading
    const nextLoading = nextReserveLoading || nextCancelLoading

    if (loading !== nextLoading) {
      document.body.style.cursor = nextLoading ? 'progress' : 'auto'
      this.setState({ loading: nextLoading })
    }

    return true
  }

  cancelAllReservations = () => {
    const ownReservations = localStorage.getItem('ownReservations')
    const tierId = localStorage.getItem('tierId')
    this.props.cancelSeatReservationStart({
      tierId: parseInt(tierId),
      seatIds: JSON.parse(ownReservations)
    })
  }

  getTierRelationsArray = (data) => {
    let ticketTypeTireRelationsArray = []
    _forEach(data, (item, key) => {
      ticketTypeTireRelationsArray.push(..._values(item))
    })

    return ticketTypeTireRelationsArray
  }

  showMap = () => {
    this.setState({ seatData: window.seatData })
    this.predefinedSeats = window.predefined_seats
    this.seatMapType = window.seatMapType
    this.ticketTypeTireRelationsArray = this.getTierRelationsArray(
      window.ticket_type_tier_relations
    )
    this.tierPrices = window.tierPrices

    this.props.getStatusesStart()
  }

  unselectBlock = () => {
    const previousBlock = this.selectedBlock
    previousBlock && previousBlock.unselect()
  }

  seatClick = (e) => {
    if (e.seat.sgType === SEAT_GROUP_TYPES.STANDING) {
      const {
        seat: { tierId, sgId, sgLabel }
      } = e
      window.document.dispatchEvent(
        new CustomEvent('standingAreaClick', {
          detail: { sgId, tierId, sgLabel }
        })
      )
    } else if (this.seatMapType === 'block') {
      const block = e.seat
      const { availableCount, status, sgId, tierId, label } = block
      if (status === SEAT_STATUSES.AVAILABLE) {
        const previousBlock = this.selectedBlock
        this.selectedBlock = block
        previousBlock && previousBlock.unselect()
        block.select()
        window.document.dispatchEvent(
          new CustomEvent('blockClick', {
            detail: {
              sgId,
              availableCount,
              tierId,
              label,
              previous: previousBlock
                ? {
                    availableCount: previousBlock.availableCount,
                    status: previousBlock.status,
                    sgId: previousBlock.sgId,
                    tierId: previousBlock.tierId,
                    label: previousBlock.label
                  }
                : null
            }
          })
        )
      }
    } else {
      const {
        seat: {
          tierId,
          seatId,
          status,
          setLoading,
          isExternalLink,
          seatExternalLink
        }
      } = e

      if (isExternalLink && seatExternalLink) {
        window.location.href = seatExternalLink
        return
      }

      const previousTierId = localStorage.getItem('tierId')

      if (
        previousTierId &&
        previousTierId !== tierId.toString() &&
        status === SEAT_STATUSES.AVAILABLE &&
        !window.allowMultipleTickets
      ) {
        this.props.notify({
          type: 'confirm',
          message: MESSAGES.GROUP_CHANGE,
          seat: { seatId, tierId, setLoading }
        })
      } else {
        if (status === SEAT_STATUSES.AVAILABLE) {
          this.props.reserveSeatStart({ tierId, seatId })
          setLoading()
        } else if (status === SEAT_STATUSES.RESERVED_BY_ME) {
          this.props.cancelSeatReservationStart({ tierId, seatIds: [seatId] })
          setLoading()
        }
      }
    }
  }

  changeTierId = (newId) => {
    window.localStorage.setItem('tierId', newId)
  }

  onUploadJson = (event) => {
    const jsonFile = event.target.files[0]
    const reader = new FileReader()

    if (jsonFile instanceof File) {
      reader.onload = (event) => {
        const result = JSON.parse(event.target.result)
        if (jsonValidator(result)) {
          this.setState({ seatData: result })
        } else {
          toast.error('Invalid Json', {
            theme: 'colored'
          })
        }
      }
      reader.readAsText(jsonFile)
    }
  }

  render() {
    const {
      reserve: { loading: reserveLoading },
      cancel: { loading: cancelLoading },
      statuses: { data: statusesData, exist: visible }
    } = this.props
    const { seatData } = this.state
    const {
      predefinedSeats,
      changeTierId,
      seatClick,
      ticketTypeTireRelationsArray,
      tierPrices
    } = this
    const parentElement = document.getElementById('seat-reservation-map')
    const disabled = reserveLoading || cancelLoading
    const loading = reserveLoading || cancelLoading

    return (
      <>
        {visible && (
          <SeatMapView
            disabled={disabled}
            loading={loading}
            events={{
              onSeatClick: seatClick,
              onTierIdChange: changeTierId
            }}
            isSelectionOn={true}
            seatData={seatData}
            statuses={statusesData}
            predefinedSeats={predefinedSeats}
            ticketTypeTireRelationsArray={ticketTypeTireRelationsArray}
            tierPrices={tierPrices}
            width={parentElement.clientWidth}
            height={parentElement.clientWidth}
            isBlockMap={this.seatMapType === 'block'}
            isTableMap={this.seatMapType === 'table'}
            buyersInfo={this.state.buyerInfo}
            seatsToRecolor={this.state.seatsToRecolor}
            activeGroups={this.state.activeGroups}
          />
        )}
        {/* <div>
          <label htmlFor='json-file' className='json-file'>
            Upload Json
          </label>
          <input
            id='json-file'
            name='json-file'
            type='file'
            accept='.json'
            hidden
            onChange={this.onUploadJson}
            onClick={(e) => (e.target.value = null)}
          />
        </div> */}
      </>
    )
  }
}

const mapStateToProps = (state) => {
  return {
    statuses: state.seatMapState.statuses,
    reserve: state.seatMapState.reserve,
    cancel: state.seatMapState.cancel
  }
}

const mapDispatchToProps = (dispatch) => ({
  getStatusesStart: () => dispatch(getStatusesStart()),
  reserveSeatStart: (payload) => dispatch(reserveSeatStart(payload)),
  cancelSeatReservationStart: (payload) =>
    dispatch(cancelSeatReservationStart(payload)),
  notify: (payload) => dispatch(notify(payload))
})

const AppWrapper = connect(mapStateToProps, mapDispatchToProps)(App)

export { AppWrapper as App }
