/* eslint-disable no-throw-literal */
import axios from 'axios';
import statuses from '../../mock-data/statuses.json';
import _cloneDeep from 'lodash/cloneDeep'
import { store } from '../../store'

const API_HOST = process.env.NODE_ENV !== 'production' ? process.env.REACT_APP_API_HOST : window.API_HOST;
const EVENT_ID = process.env.NODE_ENV !== 'production' ? process.env.REACT_APP_EVENT_ID : window.EVENT_ID;

export const getStatuses = async () => {
    if(process.env.NODE_ENV !== 'production') {
        return statuses;
    }
    return await axios.get(`${API_HOST}/api/event/${EVENT_ID}/seats/status/`);
}

export const reserveSeat = async (tierId, seatId) => {
    if(process.env.NODE_ENV !== 'production') {
        const storeData = store.getState();
        const statuses = _cloneDeep(storeData.seatMapState.statuses.data);
        const [groupRowId, seatIndex] = seatId.split(":");
        statuses[groupRowId][seatIndex] = "OR";
        const ownReservations = [];

        Object.keys(statuses).forEach(groupRowId => {
            Object.keys(statuses[groupRowId]).forEach(seatIndex => {
                if(statuses[groupRowId][seatIndex] === "OR") {
                    ownReservations.push(`${groupRowId}:${seatIndex}`);
                }
            })
        });
        return { data: { "ownReservations": JSON.stringify(ownReservations) } }; 
    }

    const result = await axios.post(`${API_HOST}/api/event/${EVENT_ID}/seats/reserve/`, {data : {tierId, seatId}})
    if(result.data.error) {
        throw {response: result};
    }
    return result;
}

export const cancelSeatReservation = (tierId, seatIds) => {
    if(process.env.NODE_ENV !== 'production') {
        const storeData = store.getState();
        const statuses = _cloneDeep(storeData.seatMapState.statuses.data);
        seatIds.forEach(seatId => {
          const [groupRowId, seatIndex] = seatId.split(":");
          statuses[groupRowId][seatIndex] = "A";
        });

        const ownReservations = [];
        Object.keys(statuses).forEach(groupRowId => {
          Object.keys(statuses[groupRowId]).forEach(seatIndex => {
            if(statuses[groupRowId][seatIndex] === "OR") {
              ownReservations.push(`${groupRowId}:${seatIndex}`);
            }
          })
        });

        return { data: { "ownReservations": JSON.stringify(ownReservations) } };
    }
    return axios.delete(`${API_HOST}/api/event/${EVENT_ID}/seats/delete-reserved/`, {data : {data : {tierId, seatIds}}})
}
