import update from 'react-addons-update'

import {
    CANCEL_SEAT_RESERVATION_FAIL,
    CANCEL_SEAT_RESERVATION_LOADING,
    CANCEL_SEAT_RESERVATION_SUCCESS,
    GET_STATUSES_FAIL,
    GET_STATUSES_LOADING,
    GET_STATUSES_SUCCESS,
    RESERVE_SEAT_FAIL,
    RESERVE_SEAT_LOADING,
    RESERVE_SEAT_SUCCESS
} from '../actions/actionTypes';
import {SEAT_STATUSES} from "seat-map-view";

const initialState = {
    statuses: {
        loading: false,
        exist: false,
        error: null,
        data: null,
    },
    reserve: {
        loading: false,
        exist: false,
        error: null,
        data: null,
    },
    cancel: {
        loading: false,
        exist: false,
        error: null,
        data: null,
    }
}

const getStatusesLoading = (state, payload) => update(state, {statuses: {$merge: {loading: true, error: null}}})

const getStatusesSuccess = (state, payload) => update(state, {statuses: {$merge: {loading: false, exist: true, data: payload}}})

const getStatusesFail = (state, payload) => update(state, {statuses: {$merge: {loading: false, error: payload }}})


const reserveSeatLoading = (state, payload) => update(state, {reserve: {$merge: {loading: true, error: null}}})

const reserveSeatSuccess = (state, payload) => update(state, {
    reserve: {$merge: {loading: false, exist: true, data: payload}},
    statuses: {data: {[payload.groupRowId]: {[payload.seatIndex]: {$set: SEAT_STATUSES.RESERVED_BY_ME}}}}
})

const reserveSeatFail = (state, payload) => update(state, {reserve: {$merge: {loading: false, error: payload }}})


const cancelSeatReservationLoading = (state, payload) => update(state, {cancel: {$merge: {loading: true, error: null}}})

const cancelSeatReservationSuccess = (state, payload) => payload.canceledSeats.reduce((acm, current) => {
    const {groupRowId, seatIndex} = current
    return update(acm, {
        cancel: {$merge: {loading: false, exist: true, data: payload}},
        statuses: {data: {[groupRowId]: {[seatIndex]: {$set: SEAT_STATUSES.AVAILABLE}}}}
    });
}, state);

const cancelSeatReservationFail = (state, payload) => update(state, {cancel: {$merge: {loading: false, error: payload }}})

const reducer = (state = initialState, action) => {
    switch(action.type) {
        case GET_STATUSES_LOADING: return getStatusesLoading(state, action.payload)
        case GET_STATUSES_SUCCESS: return getStatusesSuccess(state, action.payload)
        case GET_STATUSES_FAIL: return getStatusesFail(state, action.payload)

        case RESERVE_SEAT_LOADING: return reserveSeatLoading(state, action.payload)
        case RESERVE_SEAT_SUCCESS: return reserveSeatSuccess(state, action.payload)
        case RESERVE_SEAT_FAIL: return reserveSeatFail(state, action.payload)

        case CANCEL_SEAT_RESERVATION_LOADING: return cancelSeatReservationLoading(state, action.payload)
        case CANCEL_SEAT_RESERVATION_SUCCESS: return cancelSeatReservationSuccess(state, action.payload)
        case CANCEL_SEAT_RESERVATION_FAIL: return cancelSeatReservationFail(state, action.payload)

        default: return state;
    }
}

export {reducer as seatMapReducer};
