import React__default, { createElement, Fragment, Component } from 'react';
import _get from 'lodash/get';
import _isEmpty from 'lodash/isEmpty';
import _isEqual from 'lodash/isEqual';
import _includes from 'lodash/includes';
import _cloneDeep from 'lodash/cloneDeep';
import _isUndefined from 'lodash/isUndefined';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faWheelchair, faTimes, faMale, faChair } from '@fortawesome/free-solid-svg-icons';
import classNames from 'classnames';
import _map from 'lodash/map';
import _filter from 'lodash/filter';

function _extends() {
  _extends = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };
  return _extends.apply(this, arguments);
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;

  _setPrototypeOf(subClass, superClass);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };
  return _setPrototypeOf(o, p);
}

/**
 * @license
 * Copyright 2010-2021 Three.js Authors
 * SPDX-License-Identifier: MIT
 */
const REVISION = '127';
const MOUSE = { LEFT: 0, MIDDLE: 1, RIGHT: 2, ROTATE: 0, DOLLY: 1, PAN: 2 };
const TOUCH = { ROTATE: 0, PAN: 1, DOLLY_PAN: 2, DOLLY_ROTATE: 3 };
const CullFaceNone = 0;
const CullFaceBack = 1;
const CullFaceFront = 2;
const PCFShadowMap = 1;
const PCFSoftShadowMap = 2;
const VSMShadowMap = 3;
const FrontSide = 0;
const BackSide = 1;
const DoubleSide = 2;
const FlatShading = 1;
const NoBlending = 0;
const NormalBlending = 1;
const AdditiveBlending = 2;
const SubtractiveBlending = 3;
const MultiplyBlending = 4;
const CustomBlending = 5;
const AddEquation = 100;
const SubtractEquation = 101;
const ReverseSubtractEquation = 102;
const MinEquation = 103;
const MaxEquation = 104;
const ZeroFactor = 200;
const OneFactor = 201;
const SrcColorFactor = 202;
const OneMinusSrcColorFactor = 203;
const SrcAlphaFactor = 204;
const OneMinusSrcAlphaFactor = 205;
const DstAlphaFactor = 206;
const OneMinusDstAlphaFactor = 207;
const DstColorFactor = 208;
const OneMinusDstColorFactor = 209;
const SrcAlphaSaturateFactor = 210;
const NeverDepth = 0;
const AlwaysDepth = 1;
const LessDepth = 2;
const LessEqualDepth = 3;
const EqualDepth = 4;
const GreaterEqualDepth = 5;
const GreaterDepth = 6;
const NotEqualDepth = 7;
const MultiplyOperation = 0;
const MixOperation = 1;
const AddOperation = 2;
const NoToneMapping = 0;
const LinearToneMapping = 1;
const ReinhardToneMapping = 2;
const CineonToneMapping = 3;
const ACESFilmicToneMapping = 4;
const CustomToneMapping = 5;

const UVMapping = 300;
const CubeReflectionMapping = 301;
const CubeRefractionMapping = 302;
const EquirectangularReflectionMapping = 303;
const EquirectangularRefractionMapping = 304;
const CubeUVReflectionMapping = 306;
const CubeUVRefractionMapping = 307;
const RepeatWrapping = 1000;
const ClampToEdgeWrapping = 1001;
const MirroredRepeatWrapping = 1002;
const NearestFilter = 1003;
const NearestMipmapNearestFilter = 1004;
const NearestMipmapLinearFilter = 1005;
const LinearFilter = 1006;
const LinearMipmapNearestFilter = 1007;
const LinearMipmapLinearFilter = 1008;
const UnsignedByteType = 1009;
const ByteType = 1010;
const ShortType = 1011;
const UnsignedShortType = 1012;
const IntType = 1013;
const UnsignedIntType = 1014;
const FloatType = 1015;
const HalfFloatType = 1016;
const UnsignedShort4444Type = 1017;
const UnsignedShort5551Type = 1018;
const UnsignedShort565Type = 1019;
const UnsignedInt248Type = 1020;
const AlphaFormat = 1021;
const RGBFormat = 1022;
const RGBAFormat = 1023;
const LuminanceFormat = 1024;
const LuminanceAlphaFormat = 1025;
const DepthFormat = 1026;
const DepthStencilFormat = 1027;
const RedFormat = 1028;
const RedIntegerFormat = 1029;
const RGFormat = 1030;
const RGIntegerFormat = 1031;
const RGBIntegerFormat = 1032;
const RGBAIntegerFormat = 1033;

const RGB_S3TC_DXT1_Format = 33776;
const RGBA_S3TC_DXT1_Format = 33777;
const RGBA_S3TC_DXT3_Format = 33778;
const RGBA_S3TC_DXT5_Format = 33779;
const RGB_PVRTC_4BPPV1_Format = 35840;
const RGB_PVRTC_2BPPV1_Format = 35841;
const RGBA_PVRTC_4BPPV1_Format = 35842;
const RGBA_PVRTC_2BPPV1_Format = 35843;
const RGB_ETC1_Format = 36196;
const RGB_ETC2_Format = 37492;
const RGBA_ETC2_EAC_Format = 37496;
const RGBA_ASTC_4x4_Format = 37808;
const RGBA_ASTC_5x4_Format = 37809;
const RGBA_ASTC_5x5_Format = 37810;
const RGBA_ASTC_6x5_Format = 37811;
const RGBA_ASTC_6x6_Format = 37812;
const RGBA_ASTC_8x5_Format = 37813;
const RGBA_ASTC_8x6_Format = 37814;
const RGBA_ASTC_8x8_Format = 37815;
const RGBA_ASTC_10x5_Format = 37816;
const RGBA_ASTC_10x6_Format = 37817;
const RGBA_ASTC_10x8_Format = 37818;
const RGBA_ASTC_10x10_Format = 37819;
const RGBA_ASTC_12x10_Format = 37820;
const RGBA_ASTC_12x12_Format = 37821;
const RGBA_BPTC_Format = 36492;
const SRGB8_ALPHA8_ASTC_4x4_Format = 37840;
const SRGB8_ALPHA8_ASTC_5x4_Format = 37841;
const SRGB8_ALPHA8_ASTC_5x5_Format = 37842;
const SRGB8_ALPHA8_ASTC_6x5_Format = 37843;
const SRGB8_ALPHA8_ASTC_6x6_Format = 37844;
const SRGB8_ALPHA8_ASTC_8x5_Format = 37845;
const SRGB8_ALPHA8_ASTC_8x6_Format = 37846;
const SRGB8_ALPHA8_ASTC_8x8_Format = 37847;
const SRGB8_ALPHA8_ASTC_10x5_Format = 37848;
const SRGB8_ALPHA8_ASTC_10x6_Format = 37849;
const SRGB8_ALPHA8_ASTC_10x8_Format = 37850;
const SRGB8_ALPHA8_ASTC_10x10_Format = 37851;
const SRGB8_ALPHA8_ASTC_12x10_Format = 37852;
const SRGB8_ALPHA8_ASTC_12x12_Format = 37853;
const LoopOnce = 2200;
const LoopRepeat = 2201;
const LoopPingPong = 2202;
const InterpolateDiscrete = 2300;
const InterpolateLinear = 2301;
const InterpolateSmooth = 2302;
const ZeroCurvatureEnding = 2400;
const ZeroSlopeEnding = 2401;
const WrapAroundEnding = 2402;
const NormalAnimationBlendMode = 2500;
const AdditiveAnimationBlendMode = 2501;
const TrianglesDrawMode = 0;
const LinearEncoding = 3000;
const sRGBEncoding = 3001;
const GammaEncoding = 3007;
const RGBEEncoding = 3002;
const LogLuvEncoding = 3003;
const RGBM7Encoding = 3004;
const RGBM16Encoding = 3005;
const RGBDEncoding = 3006;
const BasicDepthPacking = 3200;
const RGBADepthPacking = 3201;
const TangentSpaceNormalMap = 0;
const ObjectSpaceNormalMap = 1;
const KeepStencilOp = 7680;
const AlwaysStencilFunc = 519;

const StaticDrawUsage = 35044;
const DynamicDrawUsage = 35048;
const GLSL3 = '300 es';

/**
 * https://github.com/mrdoob/eventdispatcher.js/
 */

function EventDispatcher() {}

Object.assign( EventDispatcher.prototype, {

	addEventListener: function ( type, listener ) {

		if ( this._listeners === undefined ) this._listeners = {};

		const listeners = this._listeners;

		if ( listeners[ type ] === undefined ) {

			listeners[ type ] = [];

		}

		if ( listeners[ type ].indexOf( listener ) === - 1 ) {

			listeners[ type ].push( listener );

		}

	},

	hasEventListener: function ( type, listener ) {

		if ( this._listeners === undefined ) return false;

		const listeners = this._listeners;

		return listeners[ type ] !== undefined && listeners[ type ].indexOf( listener ) !== - 1;

	},

	removeEventListener: function ( type, listener ) {

		if ( this._listeners === undefined ) return;

		const listeners = this._listeners;
		const listenerArray = listeners[ type ];

		if ( listenerArray !== undefined ) {

			const index = listenerArray.indexOf( listener );

			if ( index !== - 1 ) {

				listenerArray.splice( index, 1 );

			}

		}

	},

	dispatchEvent: function ( event ) {

		if ( this._listeners === undefined ) return;

		const listeners = this._listeners;
		const listenerArray = listeners[ event.type ];

		if ( listenerArray !== undefined ) {

			event.target = this;

			// Make a copy, in case listeners are removed while iterating.
			const array = listenerArray.slice( 0 );

			for ( let i = 0, l = array.length; i < l; i ++ ) {

				array[ i ].call( this, event );

			}

			event.target = null;

		}

	}

} );

const _lut = [];

for ( let i = 0; i < 256; i ++ ) {

	_lut[ i ] = ( i < 16 ? '0' : '' ) + ( i ).toString( 16 );

}

let _seed = 1234567;

const MathUtils = {

	DEG2RAD: Math.PI / 180,
	RAD2DEG: 180 / Math.PI,

	generateUUID: function () {

		// http://stackoverflow.com/questions/105034/how-to-create-a-guid-uuid-in-javascript/21963136#21963136

		const d0 = Math.random() * 0xffffffff | 0;
		const d1 = Math.random() * 0xffffffff | 0;
		const d2 = Math.random() * 0xffffffff | 0;
		const d3 = Math.random() * 0xffffffff | 0;
		const uuid = _lut[ d0 & 0xff ] + _lut[ d0 >> 8 & 0xff ] + _lut[ d0 >> 16 & 0xff ] + _lut[ d0 >> 24 & 0xff ] + '-' +
			_lut[ d1 & 0xff ] + _lut[ d1 >> 8 & 0xff ] + '-' + _lut[ d1 >> 16 & 0x0f | 0x40 ] + _lut[ d1 >> 24 & 0xff ] + '-' +
			_lut[ d2 & 0x3f | 0x80 ] + _lut[ d2 >> 8 & 0xff ] + '-' + _lut[ d2 >> 16 & 0xff ] + _lut[ d2 >> 24 & 0xff ] +
			_lut[ d3 & 0xff ] + _lut[ d3 >> 8 & 0xff ] + _lut[ d3 >> 16 & 0xff ] + _lut[ d3 >> 24 & 0xff ];

		// .toUpperCase() here flattens concatenated strings to save heap memory space.
		return uuid.toUpperCase();

	},

	clamp: function ( value, min, max ) {

		return Math.max( min, Math.min( max, value ) );

	},

	// compute euclidian modulo of m % n
	// https://en.wikipedia.org/wiki/Modulo_operation

	euclideanModulo: function ( n, m ) {

		return ( ( n % m ) + m ) % m;

	},

	// Linear mapping from range <a1, a2> to range <b1, b2>

	mapLinear: function ( x, a1, a2, b1, b2 ) {

		return b1 + ( x - a1 ) * ( b2 - b1 ) / ( a2 - a1 );

	},

	// https://www.gamedev.net/tutorials/programming/general-and-gameplay-programming/inverse-lerp-a-super-useful-yet-often-overlooked-function-r5230/

	inverseLerp: function ( x, y, value ) {

		if ( x !== y ) {

			return ( value - x ) / ( y - x );

		 } else {

			return 0;

		 }

	},

	// https://en.wikipedia.org/wiki/Linear_interpolation

	lerp: function ( x, y, t ) {

		return ( 1 - t ) * x + t * y;

	},

	// http://www.rorydriscoll.com/2016/03/07/frame-rate-independent-damping-using-lerp/

	damp: function ( x, y, lambda, dt ) {

		return MathUtils.lerp( x, y, 1 - Math.exp( - lambda * dt ) );

	},

	// https://www.desmos.com/calculator/vcsjnyz7x4

	pingpong: function ( x, length = 1 ) {

		return length - Math.abs( MathUtils.euclideanModulo( x, length * 2 ) - length );

	},

	// http://en.wikipedia.org/wiki/Smoothstep

	smoothstep: function ( x, min, max ) {

		if ( x <= min ) return 0;
		if ( x >= max ) return 1;

		x = ( x - min ) / ( max - min );

		return x * x * ( 3 - 2 * x );

	},

	smootherstep: function ( x, min, max ) {

		if ( x <= min ) return 0;
		if ( x >= max ) return 1;

		x = ( x - min ) / ( max - min );

		return x * x * x * ( x * ( x * 6 - 15 ) + 10 );

	},

	// Random integer from <low, high> interval

	randInt: function ( low, high ) {

		return low + Math.floor( Math.random() * ( high - low + 1 ) );

	},

	// Random float from <low, high> interval

	randFloat: function ( low, high ) {

		return low + Math.random() * ( high - low );

	},

	// Random float from <-range/2, range/2> interval

	randFloatSpread: function ( range ) {

		return range * ( 0.5 - Math.random() );

	},

	// Deterministic pseudo-random float in the interval [ 0, 1 ]

	seededRandom: function ( s ) {

		if ( s !== undefined ) _seed = s % 2147483647;

		// Park-Miller algorithm

		_seed = _seed * 16807 % 2147483647;

		return ( _seed - 1 ) / 2147483646;

	},

	degToRad: function ( degrees ) {

		return degrees * MathUtils.DEG2RAD;

	},

	radToDeg: function ( radians ) {

		return radians * MathUtils.RAD2DEG;

	},

	isPowerOfTwo: function ( value ) {

		return ( value & ( value - 1 ) ) === 0 && value !== 0;

	},

	ceilPowerOfTwo: function ( value ) {

		return Math.pow( 2, Math.ceil( Math.log( value ) / Math.LN2 ) );

	},

	floorPowerOfTwo: function ( value ) {

		return Math.pow( 2, Math.floor( Math.log( value ) / Math.LN2 ) );

	},

	setQuaternionFromProperEuler: function ( q, a, b, c, order ) {

		// Intrinsic Proper Euler Angles - see https://en.wikipedia.org/wiki/Euler_angles

		// rotations are applied to the axes in the order specified by 'order'
		// rotation by angle 'a' is applied first, then by angle 'b', then by angle 'c'
		// angles are in radians

		const cos = Math.cos;
		const sin = Math.sin;

		const c2 = cos( b / 2 );
		const s2 = sin( b / 2 );

		const c13 = cos( ( a + c ) / 2 );
		const s13 = sin( ( a + c ) / 2 );

		const c1_3 = cos( ( a - c ) / 2 );
		const s1_3 = sin( ( a - c ) / 2 );

		const c3_1 = cos( ( c - a ) / 2 );
		const s3_1 = sin( ( c - a ) / 2 );

		switch ( order ) {

			case 'XYX':
				q.set( c2 * s13, s2 * c1_3, s2 * s1_3, c2 * c13 );
				break;

			case 'YZY':
				q.set( s2 * s1_3, c2 * s13, s2 * c1_3, c2 * c13 );
				break;

			case 'ZXZ':
				q.set( s2 * c1_3, s2 * s1_3, c2 * s13, c2 * c13 );
				break;

			case 'XZX':
				q.set( c2 * s13, s2 * s3_1, s2 * c3_1, c2 * c13 );
				break;

			case 'YXY':
				q.set( s2 * c3_1, c2 * s13, s2 * s3_1, c2 * c13 );
				break;

			case 'ZYZ':
				q.set( s2 * s3_1, s2 * c3_1, c2 * s13, c2 * c13 );
				break;

			default:
				console.warn( 'THREE.MathUtils: .setQuaternionFromProperEuler() encountered an unknown order: ' + order );

		}

	}

};

class Vector2 {

	constructor( x = 0, y = 0 ) {

		this.x = x;
		this.y = y;

	}

	get width() {

		return this.x;

	}

	set width( value ) {

		this.x = value;

	}

	get height() {

		return this.y;

	}

	set height( value ) {

		this.y = value;

	}

	set( x, y ) {

		this.x = x;
		this.y = y;

		return this;

	}

	setScalar( scalar ) {

		this.x = scalar;
		this.y = scalar;

		return this;

	}

	setX( x ) {

		this.x = x;

		return this;

	}

	setY( y ) {

		this.y = y;

		return this;

	}

	setComponent( index, value ) {

		switch ( index ) {

			case 0: this.x = value; break;
			case 1: this.y = value; break;
			default: throw new Error( 'index is out of range: ' + index );

		}

		return this;

	}

	getComponent( index ) {

		switch ( index ) {

			case 0: return this.x;
			case 1: return this.y;
			default: throw new Error( 'index is out of range: ' + index );

		}

	}

	clone() {

		return new this.constructor( this.x, this.y );

	}

	copy( v ) {

		this.x = v.x;
		this.y = v.y;

		return this;

	}

	add( v, w ) {

		if ( w !== undefined ) {

			console.warn( 'THREE.Vector2: .add() now only accepts one argument. Use .addVectors( a, b ) instead.' );
			return this.addVectors( v, w );

		}

		this.x += v.x;
		this.y += v.y;

		return this;

	}

	addScalar( s ) {

		this.x += s;
		this.y += s;

		return this;

	}

	addVectors( a, b ) {

		this.x = a.x + b.x;
		this.y = a.y + b.y;

		return this;

	}

	addScaledVector( v, s ) {

		this.x += v.x * s;
		this.y += v.y * s;

		return this;

	}

	sub( v, w ) {

		if ( w !== undefined ) {

			console.warn( 'THREE.Vector2: .sub() now only accepts one argument. Use .subVectors( a, b ) instead.' );
			return this.subVectors( v, w );

		}

		this.x -= v.x;
		this.y -= v.y;

		return this;

	}

	subScalar( s ) {

		this.x -= s;
		this.y -= s;

		return this;

	}

	subVectors( a, b ) {

		this.x = a.x - b.x;
		this.y = a.y - b.y;

		return this;

	}

	multiply( v ) {

		this.x *= v.x;
		this.y *= v.y;

		return this;

	}

	multiplyScalar( scalar ) {

		this.x *= scalar;
		this.y *= scalar;

		return this;

	}

	divide( v ) {

		this.x /= v.x;
		this.y /= v.y;

		return this;

	}

	divideScalar( scalar ) {

		return this.multiplyScalar( 1 / scalar );

	}

	applyMatrix3( m ) {

		const x = this.x, y = this.y;
		const e = m.elements;

		this.x = e[ 0 ] * x + e[ 3 ] * y + e[ 6 ];
		this.y = e[ 1 ] * x + e[ 4 ] * y + e[ 7 ];

		return this;

	}

	min( v ) {

		this.x = Math.min( this.x, v.x );
		this.y = Math.min( this.y, v.y );

		return this;

	}

	max( v ) {

		this.x = Math.max( this.x, v.x );
		this.y = Math.max( this.y, v.y );

		return this;

	}

	clamp( min, max ) {

		// assumes min < max, componentwise

		this.x = Math.max( min.x, Math.min( max.x, this.x ) );
		this.y = Math.max( min.y, Math.min( max.y, this.y ) );

		return this;

	}

	clampScalar( minVal, maxVal ) {

		this.x = Math.max( minVal, Math.min( maxVal, this.x ) );
		this.y = Math.max( minVal, Math.min( maxVal, this.y ) );

		return this;

	}

	clampLength( min, max ) {

		const length = this.length();

		return this.divideScalar( length || 1 ).multiplyScalar( Math.max( min, Math.min( max, length ) ) );

	}

	floor() {

		this.x = Math.floor( this.x );
		this.y = Math.floor( this.y );

		return this;

	}

	ceil() {

		this.x = Math.ceil( this.x );
		this.y = Math.ceil( this.y );

		return this;

	}

	round() {

		this.x = Math.round( this.x );
		this.y = Math.round( this.y );

		return this;

	}

	roundToZero() {

		this.x = ( this.x < 0 ) ? Math.ceil( this.x ) : Math.floor( this.x );
		this.y = ( this.y < 0 ) ? Math.ceil( this.y ) : Math.floor( this.y );

		return this;

	}

	negate() {

		this.x = - this.x;
		this.y = - this.y;

		return this;

	}

	dot( v ) {

		return this.x * v.x + this.y * v.y;

	}

	cross( v ) {

		return this.x * v.y - this.y * v.x;

	}

	lengthSq() {

		return this.x * this.x + this.y * this.y;

	}

	length() {

		return Math.sqrt( this.x * this.x + this.y * this.y );

	}

	manhattanLength() {

		return Math.abs( this.x ) + Math.abs( this.y );

	}

	normalize() {

		return this.divideScalar( this.length() || 1 );

	}

	angle() {

		// computes the angle in radians with respect to the positive x-axis

		const angle = Math.atan2( - this.y, - this.x ) + Math.PI;

		return angle;

	}

	distanceTo( v ) {

		return Math.sqrt( this.distanceToSquared( v ) );

	}

	distanceToSquared( v ) {

		const dx = this.x - v.x, dy = this.y - v.y;
		return dx * dx + dy * dy;

	}

	manhattanDistanceTo( v ) {

		return Math.abs( this.x - v.x ) + Math.abs( this.y - v.y );

	}

	setLength( length ) {

		return this.normalize().multiplyScalar( length );

	}

	lerp( v, alpha ) {

		this.x += ( v.x - this.x ) * alpha;
		this.y += ( v.y - this.y ) * alpha;

		return this;

	}

	lerpVectors( v1, v2, alpha ) {

		this.x = v1.x + ( v2.x - v1.x ) * alpha;
		this.y = v1.y + ( v2.y - v1.y ) * alpha;

		return this;

	}

	equals( v ) {

		return ( ( v.x === this.x ) && ( v.y === this.y ) );

	}

	fromArray( array, offset = 0 ) {

		this.x = array[ offset ];
		this.y = array[ offset + 1 ];

		return this;

	}

	toArray( array = [], offset = 0 ) {

		array[ offset ] = this.x;
		array[ offset + 1 ] = this.y;

		return array;

	}

	fromBufferAttribute( attribute, index, offset ) {

		if ( offset !== undefined ) {

			console.warn( 'THREE.Vector2: offset has been removed from .fromBufferAttribute().' );

		}

		this.x = attribute.getX( index );
		this.y = attribute.getY( index );

		return this;

	}

	rotateAround( center, angle ) {

		const c = Math.cos( angle ), s = Math.sin( angle );

		const x = this.x - center.x;
		const y = this.y - center.y;

		this.x = x * c - y * s + center.x;
		this.y = x * s + y * c + center.y;

		return this;

	}

	random() {

		this.x = Math.random();
		this.y = Math.random();

		return this;

	}

}

Vector2.prototype.isVector2 = true;

class Matrix3 {

	constructor() {

		this.elements = [

			1, 0, 0,
			0, 1, 0,
			0, 0, 1

		];

		if ( arguments.length > 0 ) {

			console.error( 'THREE.Matrix3: the constructor no longer reads arguments. use .set() instead.' );

		}

	}

	set( n11, n12, n13, n21, n22, n23, n31, n32, n33 ) {

		const te = this.elements;

		te[ 0 ] = n11; te[ 1 ] = n21; te[ 2 ] = n31;
		te[ 3 ] = n12; te[ 4 ] = n22; te[ 5 ] = n32;
		te[ 6 ] = n13; te[ 7 ] = n23; te[ 8 ] = n33;

		return this;

	}

	identity() {

		this.set(

			1, 0, 0,
			0, 1, 0,
			0, 0, 1

		);

		return this;

	}

	copy( m ) {

		const te = this.elements;
		const me = m.elements;

		te[ 0 ] = me[ 0 ]; te[ 1 ] = me[ 1 ]; te[ 2 ] = me[ 2 ];
		te[ 3 ] = me[ 3 ]; te[ 4 ] = me[ 4 ]; te[ 5 ] = me[ 5 ];
		te[ 6 ] = me[ 6 ]; te[ 7 ] = me[ 7 ]; te[ 8 ] = me[ 8 ];

		return this;

	}

	extractBasis( xAxis, yAxis, zAxis ) {

		xAxis.setFromMatrix3Column( this, 0 );
		yAxis.setFromMatrix3Column( this, 1 );
		zAxis.setFromMatrix3Column( this, 2 );

		return this;

	}

	setFromMatrix4( m ) {

		const me = m.elements;

		this.set(

			me[ 0 ], me[ 4 ], me[ 8 ],
			me[ 1 ], me[ 5 ], me[ 9 ],
			me[ 2 ], me[ 6 ], me[ 10 ]

		);

		return this;

	}

	multiply( m ) {

		return this.multiplyMatrices( this, m );

	}

	premultiply( m ) {

		return this.multiplyMatrices( m, this );

	}

	multiplyMatrices( a, b ) {

		const ae = a.elements;
		const be = b.elements;
		const te = this.elements;

		const a11 = ae[ 0 ], a12 = ae[ 3 ], a13 = ae[ 6 ];
		const a21 = ae[ 1 ], a22 = ae[ 4 ], a23 = ae[ 7 ];
		const a31 = ae[ 2 ], a32 = ae[ 5 ], a33 = ae[ 8 ];

		const b11 = be[ 0 ], b12 = be[ 3 ], b13 = be[ 6 ];
		const b21 = be[ 1 ], b22 = be[ 4 ], b23 = be[ 7 ];
		const b31 = be[ 2 ], b32 = be[ 5 ], b33 = be[ 8 ];

		te[ 0 ] = a11 * b11 + a12 * b21 + a13 * b31;
		te[ 3 ] = a11 * b12 + a12 * b22 + a13 * b32;
		te[ 6 ] = a11 * b13 + a12 * b23 + a13 * b33;

		te[ 1 ] = a21 * b11 + a22 * b21 + a23 * b31;
		te[ 4 ] = a21 * b12 + a22 * b22 + a23 * b32;
		te[ 7 ] = a21 * b13 + a22 * b23 + a23 * b33;

		te[ 2 ] = a31 * b11 + a32 * b21 + a33 * b31;
		te[ 5 ] = a31 * b12 + a32 * b22 + a33 * b32;
		te[ 8 ] = a31 * b13 + a32 * b23 + a33 * b33;

		return this;

	}

	multiplyScalar( s ) {

		const te = this.elements;

		te[ 0 ] *= s; te[ 3 ] *= s; te[ 6 ] *= s;
		te[ 1 ] *= s; te[ 4 ] *= s; te[ 7 ] *= s;
		te[ 2 ] *= s; te[ 5 ] *= s; te[ 8 ] *= s;

		return this;

	}

	determinant() {

		const te = this.elements;

		const a = te[ 0 ], b = te[ 1 ], c = te[ 2 ],
			d = te[ 3 ], e = te[ 4 ], f = te[ 5 ],
			g = te[ 6 ], h = te[ 7 ], i = te[ 8 ];

		return a * e * i - a * f * h - b * d * i + b * f * g + c * d * h - c * e * g;

	}

	invert() {

		const te = this.elements,

			n11 = te[ 0 ], n21 = te[ 1 ], n31 = te[ 2 ],
			n12 = te[ 3 ], n22 = te[ 4 ], n32 = te[ 5 ],
			n13 = te[ 6 ], n23 = te[ 7 ], n33 = te[ 8 ],

			t11 = n33 * n22 - n32 * n23,
			t12 = n32 * n13 - n33 * n12,
			t13 = n23 * n12 - n22 * n13,

			det = n11 * t11 + n21 * t12 + n31 * t13;

		if ( det === 0 ) return this.set( 0, 0, 0, 0, 0, 0, 0, 0, 0 );

		const detInv = 1 / det;

		te[ 0 ] = t11 * detInv;
		te[ 1 ] = ( n31 * n23 - n33 * n21 ) * detInv;
		te[ 2 ] = ( n32 * n21 - n31 * n22 ) * detInv;

		te[ 3 ] = t12 * detInv;
		te[ 4 ] = ( n33 * n11 - n31 * n13 ) * detInv;
		te[ 5 ] = ( n31 * n12 - n32 * n11 ) * detInv;

		te[ 6 ] = t13 * detInv;
		te[ 7 ] = ( n21 * n13 - n23 * n11 ) * detInv;
		te[ 8 ] = ( n22 * n11 - n21 * n12 ) * detInv;

		return this;

	}

	transpose() {

		let tmp;
		const m = this.elements;

		tmp = m[ 1 ]; m[ 1 ] = m[ 3 ]; m[ 3 ] = tmp;
		tmp = m[ 2 ]; m[ 2 ] = m[ 6 ]; m[ 6 ] = tmp;
		tmp = m[ 5 ]; m[ 5 ] = m[ 7 ]; m[ 7 ] = tmp;

		return this;

	}

	getNormalMatrix( matrix4 ) {

		return this.setFromMatrix4( matrix4 ).invert().transpose();

	}

	transposeIntoArray( r ) {

		const m = this.elements;

		r[ 0 ] = m[ 0 ];
		r[ 1 ] = m[ 3 ];
		r[ 2 ] = m[ 6 ];
		r[ 3 ] = m[ 1 ];
		r[ 4 ] = m[ 4 ];
		r[ 5 ] = m[ 7 ];
		r[ 6 ] = m[ 2 ];
		r[ 7 ] = m[ 5 ];
		r[ 8 ] = m[ 8 ];

		return this;

	}

	setUvTransform( tx, ty, sx, sy, rotation, cx, cy ) {

		const c = Math.cos( rotation );
		const s = Math.sin( rotation );

		this.set(
			sx * c, sx * s, - sx * ( c * cx + s * cy ) + cx + tx,
			- sy * s, sy * c, - sy * ( - s * cx + c * cy ) + cy + ty,
			0, 0, 1
		);

		return this;

	}

	scale( sx, sy ) {

		const te = this.elements;

		te[ 0 ] *= sx; te[ 3 ] *= sx; te[ 6 ] *= sx;
		te[ 1 ] *= sy; te[ 4 ] *= sy; te[ 7 ] *= sy;

		return this;

	}

	rotate( theta ) {

		const c = Math.cos( theta );
		const s = Math.sin( theta );

		const te = this.elements;

		const a11 = te[ 0 ], a12 = te[ 3 ], a13 = te[ 6 ];
		const a21 = te[ 1 ], a22 = te[ 4 ], a23 = te[ 7 ];

		te[ 0 ] = c * a11 + s * a21;
		te[ 3 ] = c * a12 + s * a22;
		te[ 6 ] = c * a13 + s * a23;

		te[ 1 ] = - s * a11 + c * a21;
		te[ 4 ] = - s * a12 + c * a22;
		te[ 7 ] = - s * a13 + c * a23;

		return this;

	}

	translate( tx, ty ) {

		const te = this.elements;

		te[ 0 ] += tx * te[ 2 ]; te[ 3 ] += tx * te[ 5 ]; te[ 6 ] += tx * te[ 8 ];
		te[ 1 ] += ty * te[ 2 ]; te[ 4 ] += ty * te[ 5 ]; te[ 7 ] += ty * te[ 8 ];

		return this;

	}

	equals( matrix ) {

		const te = this.elements;
		const me = matrix.elements;

		for ( let i = 0; i < 9; i ++ ) {

			if ( te[ i ] !== me[ i ] ) return false;

		}

		return true;

	}

	fromArray( array, offset = 0 ) {

		for ( let i = 0; i < 9; i ++ ) {

			this.elements[ i ] = array[ i + offset ];

		}

		return this;

	}

	toArray( array = [], offset = 0 ) {

		const te = this.elements;

		array[ offset ] = te[ 0 ];
		array[ offset + 1 ] = te[ 1 ];
		array[ offset + 2 ] = te[ 2 ];

		array[ offset + 3 ] = te[ 3 ];
		array[ offset + 4 ] = te[ 4 ];
		array[ offset + 5 ] = te[ 5 ];

		array[ offset + 6 ] = te[ 6 ];
		array[ offset + 7 ] = te[ 7 ];
		array[ offset + 8 ] = te[ 8 ];

		return array;

	}

	clone() {

		return new this.constructor().fromArray( this.elements );

	}

}

Matrix3.prototype.isMatrix3 = true;

let _canvas;

const ImageUtils = {

	getDataURL: function ( image ) {

		if ( /^data:/i.test( image.src ) ) {

			return image.src;

		}

		if ( typeof HTMLCanvasElement == 'undefined' ) {

			return image.src;

		}

		let canvas;

		if ( image instanceof HTMLCanvasElement ) {

			canvas = image;

		} else {

			if ( _canvas === undefined ) _canvas = document.createElementNS( 'http://www.w3.org/1999/xhtml', 'canvas' );

			_canvas.width = image.width;
			_canvas.height = image.height;

			const context = _canvas.getContext( '2d' );

			if ( image instanceof ImageData ) {

				context.putImageData( image, 0, 0 );

			} else {

				context.drawImage( image, 0, 0, image.width, image.height );

			}

			canvas = _canvas;

		}

		if ( canvas.width > 2048 || canvas.height > 2048 ) {

			console.warn( 'THREE.ImageUtils.getDataURL: Image converted to jpg for performance reasons', image );

			return canvas.toDataURL( 'image/jpeg', 0.6 );

		} else {

			return canvas.toDataURL( 'image/png' );

		}

	}

};

let textureId = 0;

class Texture extends EventDispatcher {

	constructor( image = Texture.DEFAULT_IMAGE, mapping = Texture.DEFAULT_MAPPING, wrapS = ClampToEdgeWrapping, wrapT = ClampToEdgeWrapping, magFilter = LinearFilter, minFilter = LinearMipmapLinearFilter, format = RGBAFormat, type = UnsignedByteType, anisotropy = 1, encoding = LinearEncoding ) {

		super();

		Object.defineProperty( this, 'id', { value: textureId ++ } );

		this.uuid = MathUtils.generateUUID();

		this.name = '';

		this.image = image;
		this.mipmaps = [];

		this.mapping = mapping;

		this.wrapS = wrapS;
		this.wrapT = wrapT;

		this.magFilter = magFilter;
		this.minFilter = minFilter;

		this.anisotropy = anisotropy;

		this.format = format;
		this.internalFormat = null;
		this.type = type;

		this.offset = new Vector2( 0, 0 );
		this.repeat = new Vector2( 1, 1 );
		this.center = new Vector2( 0, 0 );
		this.rotation = 0;

		this.matrixAutoUpdate = true;
		this.matrix = new Matrix3();

		this.generateMipmaps = true;
		this.premultiplyAlpha = false;
		this.flipY = true;
		this.unpackAlignment = 4;	// valid values: 1, 2, 4, 8 (see http://www.khronos.org/opengles/sdk/docs/man/xhtml/glPixelStorei.xml)

		// Values of encoding !== THREE.LinearEncoding only supported on map, envMap and emissiveMap.
		//
		// Also changing the encoding after already used by a Material will not automatically make the Material
		// update. You need to explicitly call Material.needsUpdate to trigger it to recompile.
		this.encoding = encoding;

		this.version = 0;
		this.onUpdate = null;

	}

	updateMatrix() {

		this.matrix.setUvTransform( this.offset.x, this.offset.y, this.repeat.x, this.repeat.y, this.rotation, this.center.x, this.center.y );

	}

	clone() {

		return new this.constructor().copy( this );

	}

	copy( source ) {

		this.name = source.name;

		this.image = source.image;
		this.mipmaps = source.mipmaps.slice( 0 );

		this.mapping = source.mapping;

		this.wrapS = source.wrapS;
		this.wrapT = source.wrapT;

		this.magFilter = source.magFilter;
		this.minFilter = source.minFilter;

		this.anisotropy = source.anisotropy;

		this.format = source.format;
		this.internalFormat = source.internalFormat;
		this.type = source.type;

		this.offset.copy( source.offset );
		this.repeat.copy( source.repeat );
		this.center.copy( source.center );
		this.rotation = source.rotation;

		this.matrixAutoUpdate = source.matrixAutoUpdate;
		this.matrix.copy( source.matrix );

		this.generateMipmaps = source.generateMipmaps;
		this.premultiplyAlpha = source.premultiplyAlpha;
		this.flipY = source.flipY;
		this.unpackAlignment = source.unpackAlignment;
		this.encoding = source.encoding;

		return this;

	}

	toJSON( meta ) {

		const isRootObject = ( meta === undefined || typeof meta === 'string' );

		if ( ! isRootObject && meta.textures[ this.uuid ] !== undefined ) {

			return meta.textures[ this.uuid ];

		}

		const output = {

			metadata: {
				version: 4.5,
				type: 'Texture',
				generator: 'Texture.toJSON'
			},

			uuid: this.uuid,
			name: this.name,

			mapping: this.mapping,

			repeat: [ this.repeat.x, this.repeat.y ],
			offset: [ this.offset.x, this.offset.y ],
			center: [ this.center.x, this.center.y ],
			rotation: this.rotation,

			wrap: [ this.wrapS, this.wrapT ],

			format: this.format,
			type: this.type,
			encoding: this.encoding,

			minFilter: this.minFilter,
			magFilter: this.magFilter,
			anisotropy: this.anisotropy,

			flipY: this.flipY,

			premultiplyAlpha: this.premultiplyAlpha,
			unpackAlignment: this.unpackAlignment

		};

		if ( this.image !== undefined ) {

			// TODO: Move to THREE.Image

			const image = this.image;

			if ( image.uuid === undefined ) {

				image.uuid = MathUtils.generateUUID(); // UGH

			}

			if ( ! isRootObject && meta.images[ image.uuid ] === undefined ) {

				let url;

				if ( Array.isArray( image ) ) {

					// process array of images e.g. CubeTexture

					url = [];

					for ( let i = 0, l = image.length; i < l; i ++ ) {

						// check cube texture with data textures

						if ( image[ i ].isDataTexture ) {

							url.push( serializeImage( image[ i ].image ) );

						} else {

							url.push( serializeImage( image[ i ] ) );

						}

					}

				} else {

					// process single image

					url = serializeImage( image );

				}

				meta.images[ image.uuid ] = {
					uuid: image.uuid,
					url: url
				};

			}

			output.image = image.uuid;

		}

		if ( ! isRootObject ) {

			meta.textures[ this.uuid ] = output;

		}

		return output;

	}

	dispose() {

		this.dispatchEvent( { type: 'dispose' } );

	}

	transformUv( uv ) {

		if ( this.mapping !== UVMapping ) return uv;

		uv.applyMatrix3( this.matrix );

		if ( uv.x < 0 || uv.x > 1 ) {

			switch ( this.wrapS ) {

				case RepeatWrapping:

					uv.x = uv.x - Math.floor( uv.x );
					break;

				case ClampToEdgeWrapping:

					uv.x = uv.x < 0 ? 0 : 1;
					break;

				case MirroredRepeatWrapping:

					if ( Math.abs( Math.floor( uv.x ) % 2 ) === 1 ) {

						uv.x = Math.ceil( uv.x ) - uv.x;

					} else {

						uv.x = uv.x - Math.floor( uv.x );

					}

					break;

			}

		}

		if ( uv.y < 0 || uv.y > 1 ) {

			switch ( this.wrapT ) {

				case RepeatWrapping:

					uv.y = uv.y - Math.floor( uv.y );
					break;

				case ClampToEdgeWrapping:

					uv.y = uv.y < 0 ? 0 : 1;
					break;

				case MirroredRepeatWrapping:

					if ( Math.abs( Math.floor( uv.y ) % 2 ) === 1 ) {

						uv.y = Math.ceil( uv.y ) - uv.y;

					} else {

						uv.y = uv.y - Math.floor( uv.y );

					}

					break;

			}

		}

		if ( this.flipY ) {

			uv.y = 1 - uv.y;

		}

		return uv;

	}

	set needsUpdate( value ) {

		if ( value === true ) this.version ++;

	}

}

Texture.DEFAULT_IMAGE = undefined;
Texture.DEFAULT_MAPPING = UVMapping;

Texture.prototype.isTexture = true;

function serializeImage( image ) {

	if ( ( typeof HTMLImageElement !== 'undefined' && image instanceof HTMLImageElement ) ||
		( typeof HTMLCanvasElement !== 'undefined' && image instanceof HTMLCanvasElement ) ||
		( typeof ImageBitmap !== 'undefined' && image instanceof ImageBitmap ) ) {

		// default images

		return ImageUtils.getDataURL( image );

	} else {

		if ( image.data ) {

			// images of DataTexture

			return {
				data: Array.prototype.slice.call( image.data ),
				width: image.width,
				height: image.height,
				type: image.data.constructor.name
			};

		} else {

			console.warn( 'THREE.Texture: Unable to serialize Texture.' );
			return {};

		}

	}

}

class Vector4 {

	constructor( x = 0, y = 0, z = 0, w = 1 ) {

		this.x = x;
		this.y = y;
		this.z = z;
		this.w = w;

	}

	get width() {

		return this.z;

	}

	set width( value ) {

		this.z = value;

	}

	get height() {

		return this.w;

	}

	set height( value ) {

		this.w = value;

	}

	set( x, y, z, w ) {

		this.x = x;
		this.y = y;
		this.z = z;
		this.w = w;

		return this;

	}

	setScalar( scalar ) {

		this.x = scalar;
		this.y = scalar;
		this.z = scalar;
		this.w = scalar;

		return this;

	}

	setX( x ) {

		this.x = x;

		return this;

	}

	setY( y ) {

		this.y = y;

		return this;

	}

	setZ( z ) {

		this.z = z;

		return this;

	}

	setW( w ) {

		this.w = w;

		return this;

	}

	setComponent( index, value ) {

		switch ( index ) {

			case 0: this.x = value; break;
			case 1: this.y = value; break;
			case 2: this.z = value; break;
			case 3: this.w = value; break;
			default: throw new Error( 'index is out of range: ' + index );

		}

		return this;

	}

	getComponent( index ) {

		switch ( index ) {

			case 0: return this.x;
			case 1: return this.y;
			case 2: return this.z;
			case 3: return this.w;
			default: throw new Error( 'index is out of range: ' + index );

		}

	}

	clone() {

		return new this.constructor( this.x, this.y, this.z, this.w );

	}

	copy( v ) {

		this.x = v.x;
		this.y = v.y;
		this.z = v.z;
		this.w = ( v.w !== undefined ) ? v.w : 1;

		return this;

	}

	add( v, w ) {

		if ( w !== undefined ) {

			console.warn( 'THREE.Vector4: .add() now only accepts one argument. Use .addVectors( a, b ) instead.' );
			return this.addVectors( v, w );

		}

		this.x += v.x;
		this.y += v.y;
		this.z += v.z;
		this.w += v.w;

		return this;

	}

	addScalar( s ) {

		this.x += s;
		this.y += s;
		this.z += s;
		this.w += s;

		return this;

	}

	addVectors( a, b ) {

		this.x = a.x + b.x;
		this.y = a.y + b.y;
		this.z = a.z + b.z;
		this.w = a.w + b.w;

		return this;

	}

	addScaledVector( v, s ) {

		this.x += v.x * s;
		this.y += v.y * s;
		this.z += v.z * s;
		this.w += v.w * s;

		return this;

	}

	sub( v, w ) {

		if ( w !== undefined ) {

			console.warn( 'THREE.Vector4: .sub() now only accepts one argument. Use .subVectors( a, b ) instead.' );
			return this.subVectors( v, w );

		}

		this.x -= v.x;
		this.y -= v.y;
		this.z -= v.z;
		this.w -= v.w;

		return this;

	}

	subScalar( s ) {

		this.x -= s;
		this.y -= s;
		this.z -= s;
		this.w -= s;

		return this;

	}

	subVectors( a, b ) {

		this.x = a.x - b.x;
		this.y = a.y - b.y;
		this.z = a.z - b.z;
		this.w = a.w - b.w;

		return this;

	}

	multiply( v ) {

		this.x *= v.x;
		this.y *= v.y;
		this.z *= v.z;
		this.w *= v.w;

		return this;

	}

	multiplyScalar( scalar ) {

		this.x *= scalar;
		this.y *= scalar;
		this.z *= scalar;
		this.w *= scalar;

		return this;

	}

	applyMatrix4( m ) {

		const x = this.x, y = this.y, z = this.z, w = this.w;
		const e = m.elements;

		this.x = e[ 0 ] * x + e[ 4 ] * y + e[ 8 ] * z + e[ 12 ] * w;
		this.y = e[ 1 ] * x + e[ 5 ] * y + e[ 9 ] * z + e[ 13 ] * w;
		this.z = e[ 2 ] * x + e[ 6 ] * y + e[ 10 ] * z + e[ 14 ] * w;
		this.w = e[ 3 ] * x + e[ 7 ] * y + e[ 11 ] * z + e[ 15 ] * w;

		return this;

	}

	divideScalar( scalar ) {

		return this.multiplyScalar( 1 / scalar );

	}

	setAxisAngleFromQuaternion( q ) {

		// http://www.euclideanspace.com/maths/geometry/rotations/conversions/quaternionToAngle/index.htm

		// q is assumed to be normalized

		this.w = 2 * Math.acos( q.w );

		const s = Math.sqrt( 1 - q.w * q.w );

		if ( s < 0.0001 ) {

			this.x = 1;
			this.y = 0;
			this.z = 0;

		} else {

			this.x = q.x / s;
			this.y = q.y / s;
			this.z = q.z / s;

		}

		return this;

	}

	setAxisAngleFromRotationMatrix( m ) {

		// http://www.euclideanspace.com/maths/geometry/rotations/conversions/matrixToAngle/index.htm

		// assumes the upper 3x3 of m is a pure rotation matrix (i.e, unscaled)

		let angle, x, y, z; // variables for result
		const epsilon = 0.01,		// margin to allow for rounding errors
			epsilon2 = 0.1,		// margin to distinguish between 0 and 180 degrees

			te = m.elements,

			m11 = te[ 0 ], m12 = te[ 4 ], m13 = te[ 8 ],
			m21 = te[ 1 ], m22 = te[ 5 ], m23 = te[ 9 ],
			m31 = te[ 2 ], m32 = te[ 6 ], m33 = te[ 10 ];

		if ( ( Math.abs( m12 - m21 ) < epsilon ) &&
		     ( Math.abs( m13 - m31 ) < epsilon ) &&
		     ( Math.abs( m23 - m32 ) < epsilon ) ) {

			// singularity found
			// first check for identity matrix which must have +1 for all terms
			// in leading diagonal and zero in other terms

			if ( ( Math.abs( m12 + m21 ) < epsilon2 ) &&
			     ( Math.abs( m13 + m31 ) < epsilon2 ) &&
			     ( Math.abs( m23 + m32 ) < epsilon2 ) &&
			     ( Math.abs( m11 + m22 + m33 - 3 ) < epsilon2 ) ) {

				// this singularity is identity matrix so angle = 0

				this.set( 1, 0, 0, 0 );

				return this; // zero angle, arbitrary axis

			}

			// otherwise this singularity is angle = 180

			angle = Math.PI;

			const xx = ( m11 + 1 ) / 2;
			const yy = ( m22 + 1 ) / 2;
			const zz = ( m33 + 1 ) / 2;
			const xy = ( m12 + m21 ) / 4;
			const xz = ( m13 + m31 ) / 4;
			const yz = ( m23 + m32 ) / 4;

			if ( ( xx > yy ) && ( xx > zz ) ) {

				// m11 is the largest diagonal term

				if ( xx < epsilon ) {

					x = 0;
					y = 0.707106781;
					z = 0.707106781;

				} else {

					x = Math.sqrt( xx );
					y = xy / x;
					z = xz / x;

				}

			} else if ( yy > zz ) {

				// m22 is the largest diagonal term

				if ( yy < epsilon ) {

					x = 0.707106781;
					y = 0;
					z = 0.707106781;

				} else {

					y = Math.sqrt( yy );
					x = xy / y;
					z = yz / y;

				}

			} else {

				// m33 is the largest diagonal term so base result on this

				if ( zz < epsilon ) {

					x = 0.707106781;
					y = 0.707106781;
					z = 0;

				} else {

					z = Math.sqrt( zz );
					x = xz / z;
					y = yz / z;

				}

			}

			this.set( x, y, z, angle );

			return this; // return 180 deg rotation

		}

		// as we have reached here there are no singularities so we can handle normally

		let s = Math.sqrt( ( m32 - m23 ) * ( m32 - m23 ) +
			( m13 - m31 ) * ( m13 - m31 ) +
			( m21 - m12 ) * ( m21 - m12 ) ); // used to normalize

		if ( Math.abs( s ) < 0.001 ) s = 1;

		// prevent divide by zero, should not happen if matrix is orthogonal and should be
		// caught by singularity test above, but I've left it in just in case

		this.x = ( m32 - m23 ) / s;
		this.y = ( m13 - m31 ) / s;
		this.z = ( m21 - m12 ) / s;
		this.w = Math.acos( ( m11 + m22 + m33 - 1 ) / 2 );

		return this;

	}

	min( v ) {

		this.x = Math.min( this.x, v.x );
		this.y = Math.min( this.y, v.y );
		this.z = Math.min( this.z, v.z );
		this.w = Math.min( this.w, v.w );

		return this;

	}

	max( v ) {

		this.x = Math.max( this.x, v.x );
		this.y = Math.max( this.y, v.y );
		this.z = Math.max( this.z, v.z );
		this.w = Math.max( this.w, v.w );

		return this;

	}

	clamp( min, max ) {

		// assumes min < max, componentwise

		this.x = Math.max( min.x, Math.min( max.x, this.x ) );
		this.y = Math.max( min.y, Math.min( max.y, this.y ) );
		this.z = Math.max( min.z, Math.min( max.z, this.z ) );
		this.w = Math.max( min.w, Math.min( max.w, this.w ) );

		return this;

	}

	clampScalar( minVal, maxVal ) {

		this.x = Math.max( minVal, Math.min( maxVal, this.x ) );
		this.y = Math.max( minVal, Math.min( maxVal, this.y ) );
		this.z = Math.max( minVal, Math.min( maxVal, this.z ) );
		this.w = Math.max( minVal, Math.min( maxVal, this.w ) );

		return this;

	}

	clampLength( min, max ) {

		const length = this.length();

		return this.divideScalar( length || 1 ).multiplyScalar( Math.max( min, Math.min( max, length ) ) );

	}

	floor() {

		this.x = Math.floor( this.x );
		this.y = Math.floor( this.y );
		this.z = Math.floor( this.z );
		this.w = Math.floor( this.w );

		return this;

	}

	ceil() {

		this.x = Math.ceil( this.x );
		this.y = Math.ceil( this.y );
		this.z = Math.ceil( this.z );
		this.w = Math.ceil( this.w );

		return this;

	}

	round() {

		this.x = Math.round( this.x );
		this.y = Math.round( this.y );
		this.z = Math.round( this.z );
		this.w = Math.round( this.w );

		return this;

	}

	roundToZero() {

		this.x = ( this.x < 0 ) ? Math.ceil( this.x ) : Math.floor( this.x );
		this.y = ( this.y < 0 ) ? Math.ceil( this.y ) : Math.floor( this.y );
		this.z = ( this.z < 0 ) ? Math.ceil( this.z ) : Math.floor( this.z );
		this.w = ( this.w < 0 ) ? Math.ceil( this.w ) : Math.floor( this.w );

		return this;

	}

	negate() {

		this.x = - this.x;
		this.y = - this.y;
		this.z = - this.z;
		this.w = - this.w;

		return this;

	}

	dot( v ) {

		return this.x * v.x + this.y * v.y + this.z * v.z + this.w * v.w;

	}

	lengthSq() {

		return this.x * this.x + this.y * this.y + this.z * this.z + this.w * this.w;

	}

	length() {

		return Math.sqrt( this.x * this.x + this.y * this.y + this.z * this.z + this.w * this.w );

	}

	manhattanLength() {

		return Math.abs( this.x ) + Math.abs( this.y ) + Math.abs( this.z ) + Math.abs( this.w );

	}

	normalize() {

		return this.divideScalar( this.length() || 1 );

	}

	setLength( length ) {

		return this.normalize().multiplyScalar( length );

	}

	lerp( v, alpha ) {

		this.x += ( v.x - this.x ) * alpha;
		this.y += ( v.y - this.y ) * alpha;
		this.z += ( v.z - this.z ) * alpha;
		this.w += ( v.w - this.w ) * alpha;

		return this;

	}

	lerpVectors( v1, v2, alpha ) {

		this.x = v1.x + ( v2.x - v1.x ) * alpha;
		this.y = v1.y + ( v2.y - v1.y ) * alpha;
		this.z = v1.z + ( v2.z - v1.z ) * alpha;
		this.w = v1.w + ( v2.w - v1.w ) * alpha;

		return this;

	}

	equals( v ) {

		return ( ( v.x === this.x ) && ( v.y === this.y ) && ( v.z === this.z ) && ( v.w === this.w ) );

	}

	fromArray( array, offset = 0 ) {

		this.x = array[ offset ];
		this.y = array[ offset + 1 ];
		this.z = array[ offset + 2 ];
		this.w = array[ offset + 3 ];

		return this;

	}

	toArray( array = [], offset = 0 ) {

		array[ offset ] = this.x;
		array[ offset + 1 ] = this.y;
		array[ offset + 2 ] = this.z;
		array[ offset + 3 ] = this.w;

		return array;

	}

	fromBufferAttribute( attribute, index, offset ) {

		if ( offset !== undefined ) {

			console.warn( 'THREE.Vector4: offset has been removed from .fromBufferAttribute().' );

		}

		this.x = attribute.getX( index );
		this.y = attribute.getY( index );
		this.z = attribute.getZ( index );
		this.w = attribute.getW( index );

		return this;

	}

	random() {

		this.x = Math.random();
		this.y = Math.random();
		this.z = Math.random();
		this.w = Math.random();

		return this;

	}

}

Vector4.prototype.isVector4 = true;

/*
 In options, we can specify:
 * Texture parameters for an auto-generated target texture
 * depthBuffer/stencilBuffer: Booleans to indicate if we should generate these buffers
*/
class WebGLRenderTarget extends EventDispatcher {

	constructor( width, height, options ) {

		super();

		this.width = width;
		this.height = height;
		this.depth = 1;

		this.scissor = new Vector4( 0, 0, width, height );
		this.scissorTest = false;

		this.viewport = new Vector4( 0, 0, width, height );

		options = options || {};

		this.texture = new Texture( undefined, options.mapping, options.wrapS, options.wrapT, options.magFilter, options.minFilter, options.format, options.type, options.anisotropy, options.encoding );

		this.texture.image = {};
		this.texture.image.width = width;
		this.texture.image.height = height;
		this.texture.image.depth = 1;

		this.texture.generateMipmaps = options.generateMipmaps !== undefined ? options.generateMipmaps : false;
		this.texture.minFilter = options.minFilter !== undefined ? options.minFilter : LinearFilter;

		this.depthBuffer = options.depthBuffer !== undefined ? options.depthBuffer : true;
		this.stencilBuffer = options.stencilBuffer !== undefined ? options.stencilBuffer : false;
		this.depthTexture = options.depthTexture !== undefined ? options.depthTexture : null;

	}

	setTexture( texture ) {

		texture.image = {
			width: this.width,
			height: this.height,
			depth: this.depth
		};

		this.texture = texture;

	}

	setSize( width, height, depth = 1 ) {

		if ( this.width !== width || this.height !== height || this.depth !== depth ) {

			this.width = width;
			this.height = height;
			this.depth = depth;

			this.texture.image.width = width;
			this.texture.image.height = height;
			this.texture.image.depth = depth;

			this.dispose();

		}

		this.viewport.set( 0, 0, width, height );
		this.scissor.set( 0, 0, width, height );

	}

	clone() {

		return new this.constructor().copy( this );

	}

	copy( source ) {

		this.width = source.width;
		this.height = source.height;
		this.depth = source.depth;

		this.viewport.copy( source.viewport );

		this.texture = source.texture.clone();

		this.depthBuffer = source.depthBuffer;
		this.stencilBuffer = source.stencilBuffer;
		this.depthTexture = source.depthTexture;

		return this;

	}

	dispose() {

		this.dispatchEvent( { type: 'dispose' } );

	}

}

WebGLRenderTarget.prototype.isWebGLRenderTarget = true;

class WebGLMultisampleRenderTarget extends WebGLRenderTarget {

	constructor( width, height, options ) {

		super( width, height, options );

		this.samples = 4;

	}

	copy( source ) {

		super.copy.call( this, source );

		this.samples = source.samples;

		return this;

	}

}

WebGLMultisampleRenderTarget.prototype.isWebGLMultisampleRenderTarget = true;

class Quaternion {

	constructor( x = 0, y = 0, z = 0, w = 1 ) {

		this._x = x;
		this._y = y;
		this._z = z;
		this._w = w;

	}

	static slerp( qa, qb, qm, t ) {

		console.warn( 'THREE.Quaternion: Static .slerp() has been deprecated. Use qm.slerpQuaternions( qa, qb, t ) instead.' );
		return qm.slerpQuaternions( qa, qb, t );

	}

	static slerpFlat( dst, dstOffset, src0, srcOffset0, src1, srcOffset1, t ) {

		// fuzz-free, array-based Quaternion SLERP operation

		let x0 = src0[ srcOffset0 + 0 ],
			y0 = src0[ srcOffset0 + 1 ],
			z0 = src0[ srcOffset0 + 2 ],
			w0 = src0[ srcOffset0 + 3 ];

		const x1 = src1[ srcOffset1 + 0 ],
			y1 = src1[ srcOffset1 + 1 ],
			z1 = src1[ srcOffset1 + 2 ],
			w1 = src1[ srcOffset1 + 3 ];

		if ( t === 0 ) {

			dst[ dstOffset + 0 ] = x0;
			dst[ dstOffset + 1 ] = y0;
			dst[ dstOffset + 2 ] = z0;
			dst[ dstOffset + 3 ] = w0;
			return;

		}

		if ( t === 1 ) {

			dst[ dstOffset + 0 ] = x1;
			dst[ dstOffset + 1 ] = y1;
			dst[ dstOffset + 2 ] = z1;
			dst[ dstOffset + 3 ] = w1;
			return;

		}

		if ( w0 !== w1 || x0 !== x1 || y0 !== y1 || z0 !== z1 ) {

			let s = 1 - t;
			const cos = x0 * x1 + y0 * y1 + z0 * z1 + w0 * w1,
				dir = ( cos >= 0 ? 1 : - 1 ),
				sqrSin = 1 - cos * cos;

			// Skip the Slerp for tiny steps to avoid numeric problems:
			if ( sqrSin > Number.EPSILON ) {

				const sin = Math.sqrt( sqrSin ),
					len = Math.atan2( sin, cos * dir );

				s = Math.sin( s * len ) / sin;
				t = Math.sin( t * len ) / sin;

			}

			const tDir = t * dir;

			x0 = x0 * s + x1 * tDir;
			y0 = y0 * s + y1 * tDir;
			z0 = z0 * s + z1 * tDir;
			w0 = w0 * s + w1 * tDir;

			// Normalize in case we just did a lerp:
			if ( s === 1 - t ) {

				const f = 1 / Math.sqrt( x0 * x0 + y0 * y0 + z0 * z0 + w0 * w0 );

				x0 *= f;
				y0 *= f;
				z0 *= f;
				w0 *= f;

			}

		}

		dst[ dstOffset ] = x0;
		dst[ dstOffset + 1 ] = y0;
		dst[ dstOffset + 2 ] = z0;
		dst[ dstOffset + 3 ] = w0;

	}

	static multiplyQuaternionsFlat( dst, dstOffset, src0, srcOffset0, src1, srcOffset1 ) {

		const x0 = src0[ srcOffset0 ];
		const y0 = src0[ srcOffset0 + 1 ];
		const z0 = src0[ srcOffset0 + 2 ];
		const w0 = src0[ srcOffset0 + 3 ];

		const x1 = src1[ srcOffset1 ];
		const y1 = src1[ srcOffset1 + 1 ];
		const z1 = src1[ srcOffset1 + 2 ];
		const w1 = src1[ srcOffset1 + 3 ];

		dst[ dstOffset ] = x0 * w1 + w0 * x1 + y0 * z1 - z0 * y1;
		dst[ dstOffset + 1 ] = y0 * w1 + w0 * y1 + z0 * x1 - x0 * z1;
		dst[ dstOffset + 2 ] = z0 * w1 + w0 * z1 + x0 * y1 - y0 * x1;
		dst[ dstOffset + 3 ] = w0 * w1 - x0 * x1 - y0 * y1 - z0 * z1;

		return dst;

	}

	get x() {

		return this._x;

	}

	set x( value ) {

		this._x = value;
		this._onChangeCallback();

	}

	get y() {

		return this._y;

	}

	set y( value ) {

		this._y = value;
		this._onChangeCallback();

	}

	get z() {

		return this._z;

	}

	set z( value ) {

		this._z = value;
		this._onChangeCallback();

	}

	get w() {

		return this._w;

	}

	set w( value ) {

		this._w = value;
		this._onChangeCallback();

	}

	set( x, y, z, w ) {

		this._x = x;
		this._y = y;
		this._z = z;
		this._w = w;

		this._onChangeCallback();

		return this;

	}

	clone() {

		return new this.constructor( this._x, this._y, this._z, this._w );

	}

	copy( quaternion ) {

		this._x = quaternion.x;
		this._y = quaternion.y;
		this._z = quaternion.z;
		this._w = quaternion.w;

		this._onChangeCallback();

		return this;

	}

	setFromEuler( euler, update ) {

		if ( ! ( euler && euler.isEuler ) ) {

			throw new Error( 'THREE.Quaternion: .setFromEuler() now expects an Euler rotation rather than a Vector3 and order.' );

		}

		const x = euler._x, y = euler._y, z = euler._z, order = euler._order;

		// http://www.mathworks.com/matlabcentral/fileexchange/
		// 	20696-function-to-convert-between-dcm-euler-angles-quaternions-and-euler-vectors/
		//	content/SpinCalc.m

		const cos = Math.cos;
		const sin = Math.sin;

		const c1 = cos( x / 2 );
		const c2 = cos( y / 2 );
		const c3 = cos( z / 2 );

		const s1 = sin( x / 2 );
		const s2 = sin( y / 2 );
		const s3 = sin( z / 2 );

		switch ( order ) {

			case 'XYZ':
				this._x = s1 * c2 * c3 + c1 * s2 * s3;
				this._y = c1 * s2 * c3 - s1 * c2 * s3;
				this._z = c1 * c2 * s3 + s1 * s2 * c3;
				this._w = c1 * c2 * c3 - s1 * s2 * s3;
				break;

			case 'YXZ':
				this._x = s1 * c2 * c3 + c1 * s2 * s3;
				this._y = c1 * s2 * c3 - s1 * c2 * s3;
				this._z = c1 * c2 * s3 - s1 * s2 * c3;
				this._w = c1 * c2 * c3 + s1 * s2 * s3;
				break;

			case 'ZXY':
				this._x = s1 * c2 * c3 - c1 * s2 * s3;
				this._y = c1 * s2 * c3 + s1 * c2 * s3;
				this._z = c1 * c2 * s3 + s1 * s2 * c3;
				this._w = c1 * c2 * c3 - s1 * s2 * s3;
				break;

			case 'ZYX':
				this._x = s1 * c2 * c3 - c1 * s2 * s3;
				this._y = c1 * s2 * c3 + s1 * c2 * s3;
				this._z = c1 * c2 * s3 - s1 * s2 * c3;
				this._w = c1 * c2 * c3 + s1 * s2 * s3;
				break;

			case 'YZX':
				this._x = s1 * c2 * c3 + c1 * s2 * s3;
				this._y = c1 * s2 * c3 + s1 * c2 * s3;
				this._z = c1 * c2 * s3 - s1 * s2 * c3;
				this._w = c1 * c2 * c3 - s1 * s2 * s3;
				break;

			case 'XZY':
				this._x = s1 * c2 * c3 - c1 * s2 * s3;
				this._y = c1 * s2 * c3 - s1 * c2 * s3;
				this._z = c1 * c2 * s3 + s1 * s2 * c3;
				this._w = c1 * c2 * c3 + s1 * s2 * s3;
				break;

			default:
				console.warn( 'THREE.Quaternion: .setFromEuler() encountered an unknown order: ' + order );

		}

		if ( update !== false ) this._onChangeCallback();

		return this;

	}

	setFromAxisAngle( axis, angle ) {

		// http://www.euclideanspace.com/maths/geometry/rotations/conversions/angleToQuaternion/index.htm

		// assumes axis is normalized

		const halfAngle = angle / 2, s = Math.sin( halfAngle );

		this._x = axis.x * s;
		this._y = axis.y * s;
		this._z = axis.z * s;
		this._w = Math.cos( halfAngle );

		this._onChangeCallback();

		return this;

	}

	setFromRotationMatrix( m ) {

		// http://www.euclideanspace.com/maths/geometry/rotations/conversions/matrixToQuaternion/index.htm

		// assumes the upper 3x3 of m is a pure rotation matrix (i.e, unscaled)

		const te = m.elements,

			m11 = te[ 0 ], m12 = te[ 4 ], m13 = te[ 8 ],
			m21 = te[ 1 ], m22 = te[ 5 ], m23 = te[ 9 ],
			m31 = te[ 2 ], m32 = te[ 6 ], m33 = te[ 10 ],

			trace = m11 + m22 + m33;

		if ( trace > 0 ) {

			const s = 0.5 / Math.sqrt( trace + 1.0 );

			this._w = 0.25 / s;
			this._x = ( m32 - m23 ) * s;
			this._y = ( m13 - m31 ) * s;
			this._z = ( m21 - m12 ) * s;

		} else if ( m11 > m22 && m11 > m33 ) {

			const s = 2.0 * Math.sqrt( 1.0 + m11 - m22 - m33 );

			this._w = ( m32 - m23 ) / s;
			this._x = 0.25 * s;
			this._y = ( m12 + m21 ) / s;
			this._z = ( m13 + m31 ) / s;

		} else if ( m22 > m33 ) {

			const s = 2.0 * Math.sqrt( 1.0 + m22 - m11 - m33 );

			this._w = ( m13 - m31 ) / s;
			this._x = ( m12 + m21 ) / s;
			this._y = 0.25 * s;
			this._z = ( m23 + m32 ) / s;

		} else {

			const s = 2.0 * Math.sqrt( 1.0 + m33 - m11 - m22 );

			this._w = ( m21 - m12 ) / s;
			this._x = ( m13 + m31 ) / s;
			this._y = ( m23 + m32 ) / s;
			this._z = 0.25 * s;

		}

		this._onChangeCallback();

		return this;

	}

	setFromUnitVectors( vFrom, vTo ) {

		// assumes direction vectors vFrom and vTo are normalized

		let r = vFrom.dot( vTo ) + 1;

		if ( r < Number.EPSILON ) {

			// vFrom and vTo point in opposite directions

			r = 0;

			if ( Math.abs( vFrom.x ) > Math.abs( vFrom.z ) ) {

				this._x = - vFrom.y;
				this._y = vFrom.x;
				this._z = 0;
				this._w = r;

			} else {

				this._x = 0;
				this._y = - vFrom.z;
				this._z = vFrom.y;
				this._w = r;

			}

		} else {

			// crossVectors( vFrom, vTo ); // inlined to avoid cyclic dependency on Vector3

			this._x = vFrom.y * vTo.z - vFrom.z * vTo.y;
			this._y = vFrom.z * vTo.x - vFrom.x * vTo.z;
			this._z = vFrom.x * vTo.y - vFrom.y * vTo.x;
			this._w = r;

		}

		return this.normalize();

	}

	angleTo( q ) {

		return 2 * Math.acos( Math.abs( MathUtils.clamp( this.dot( q ), - 1, 1 ) ) );

	}

	rotateTowards( q, step ) {

		const angle = this.angleTo( q );

		if ( angle === 0 ) return this;

		const t = Math.min( 1, step / angle );

		this.slerp( q, t );

		return this;

	}

	identity() {

		return this.set( 0, 0, 0, 1 );

	}

	invert() {

		// quaternion is assumed to have unit length

		return this.conjugate();

	}

	conjugate() {

		this._x *= - 1;
		this._y *= - 1;
		this._z *= - 1;

		this._onChangeCallback();

		return this;

	}

	dot( v ) {

		return this._x * v._x + this._y * v._y + this._z * v._z + this._w * v._w;

	}

	lengthSq() {

		return this._x * this._x + this._y * this._y + this._z * this._z + this._w * this._w;

	}

	length() {

		return Math.sqrt( this._x * this._x + this._y * this._y + this._z * this._z + this._w * this._w );

	}

	normalize() {

		let l = this.length();

		if ( l === 0 ) {

			this._x = 0;
			this._y = 0;
			this._z = 0;
			this._w = 1;

		} else {

			l = 1 / l;

			this._x = this._x * l;
			this._y = this._y * l;
			this._z = this._z * l;
			this._w = this._w * l;

		}

		this._onChangeCallback();

		return this;

	}

	multiply( q, p ) {

		if ( p !== undefined ) {

			console.warn( 'THREE.Quaternion: .multiply() now only accepts one argument. Use .multiplyQuaternions( a, b ) instead.' );
			return this.multiplyQuaternions( q, p );

		}

		return this.multiplyQuaternions( this, q );

	}

	premultiply( q ) {

		return this.multiplyQuaternions( q, this );

	}

	multiplyQuaternions( a, b ) {

		// from http://www.euclideanspace.com/maths/algebra/realNormedAlgebra/quaternions/code/index.htm

		const qax = a._x, qay = a._y, qaz = a._z, qaw = a._w;
		const qbx = b._x, qby = b._y, qbz = b._z, qbw = b._w;

		this._x = qax * qbw + qaw * qbx + qay * qbz - qaz * qby;
		this._y = qay * qbw + qaw * qby + qaz * qbx - qax * qbz;
		this._z = qaz * qbw + qaw * qbz + qax * qby - qay * qbx;
		this._w = qaw * qbw - qax * qbx - qay * qby - qaz * qbz;

		this._onChangeCallback();

		return this;

	}

	slerp( qb, t ) {

		if ( t === 0 ) return this;
		if ( t === 1 ) return this.copy( qb );

		const x = this._x, y = this._y, z = this._z, w = this._w;

		// http://www.euclideanspace.com/maths/algebra/realNormedAlgebra/quaternions/slerp/

		let cosHalfTheta = w * qb._w + x * qb._x + y * qb._y + z * qb._z;

		if ( cosHalfTheta < 0 ) {

			this._w = - qb._w;
			this._x = - qb._x;
			this._y = - qb._y;
			this._z = - qb._z;

			cosHalfTheta = - cosHalfTheta;

		} else {

			this.copy( qb );

		}

		if ( cosHalfTheta >= 1.0 ) {

			this._w = w;
			this._x = x;
			this._y = y;
			this._z = z;

			return this;

		}

		const sqrSinHalfTheta = 1.0 - cosHalfTheta * cosHalfTheta;

		if ( sqrSinHalfTheta <= Number.EPSILON ) {

			const s = 1 - t;
			this._w = s * w + t * this._w;
			this._x = s * x + t * this._x;
			this._y = s * y + t * this._y;
			this._z = s * z + t * this._z;

			this.normalize();
			this._onChangeCallback();

			return this;

		}

		const sinHalfTheta = Math.sqrt( sqrSinHalfTheta );
		const halfTheta = Math.atan2( sinHalfTheta, cosHalfTheta );
		const ratioA = Math.sin( ( 1 - t ) * halfTheta ) / sinHalfTheta,
			ratioB = Math.sin( t * halfTheta ) / sinHalfTheta;

		this._w = ( w * ratioA + this._w * ratioB );
		this._x = ( x * ratioA + this._x * ratioB );
		this._y = ( y * ratioA + this._y * ratioB );
		this._z = ( z * ratioA + this._z * ratioB );

		this._onChangeCallback();

		return this;

	}

	slerpQuaternions( qa, qb, t ) {

		this.copy( qa ).slerp( qb, t );

	}

	equals( quaternion ) {

		return ( quaternion._x === this._x ) && ( quaternion._y === this._y ) && ( quaternion._z === this._z ) && ( quaternion._w === this._w );

	}

	fromArray( array, offset = 0 ) {

		this._x = array[ offset ];
		this._y = array[ offset + 1 ];
		this._z = array[ offset + 2 ];
		this._w = array[ offset + 3 ];

		this._onChangeCallback();

		return this;

	}

	toArray( array = [], offset = 0 ) {

		array[ offset ] = this._x;
		array[ offset + 1 ] = this._y;
		array[ offset + 2 ] = this._z;
		array[ offset + 3 ] = this._w;

		return array;

	}

	fromBufferAttribute( attribute, index ) {

		this._x = attribute.getX( index );
		this._y = attribute.getY( index );
		this._z = attribute.getZ( index );
		this._w = attribute.getW( index );

		return this;

	}

	_onChange( callback ) {

		this._onChangeCallback = callback;

		return this;

	}

	_onChangeCallback() {}

}

Quaternion.prototype.isQuaternion = true;

class Vector3 {

	constructor( x = 0, y = 0, z = 0 ) {

		this.x = x;
		this.y = y;
		this.z = z;

	}

	set( x, y, z ) {

		if ( z === undefined ) z = this.z; // sprite.scale.set(x,y)

		this.x = x;
		this.y = y;
		this.z = z;

		return this;

	}

	setScalar( scalar ) {

		this.x = scalar;
		this.y = scalar;
		this.z = scalar;

		return this;

	}

	setX( x ) {

		this.x = x;

		return this;

	}

	setY( y ) {

		this.y = y;

		return this;

	}

	setZ( z ) {

		this.z = z;

		return this;

	}

	setComponent( index, value ) {

		switch ( index ) {

			case 0: this.x = value; break;
			case 1: this.y = value; break;
			case 2: this.z = value; break;
			default: throw new Error( 'index is out of range: ' + index );

		}

		return this;

	}

	getComponent( index ) {

		switch ( index ) {

			case 0: return this.x;
			case 1: return this.y;
			case 2: return this.z;
			default: throw new Error( 'index is out of range: ' + index );

		}

	}

	clone() {

		return new this.constructor( this.x, this.y, this.z );

	}

	copy( v ) {

		this.x = v.x;
		this.y = v.y;
		this.z = v.z;

		return this;

	}

	add( v, w ) {

		if ( w !== undefined ) {

			console.warn( 'THREE.Vector3: .add() now only accepts one argument. Use .addVectors( a, b ) instead.' );
			return this.addVectors( v, w );

		}

		this.x += v.x;
		this.y += v.y;
		this.z += v.z;

		return this;

	}

	addScalar( s ) {

		this.x += s;
		this.y += s;
		this.z += s;

		return this;

	}

	addVectors( a, b ) {

		this.x = a.x + b.x;
		this.y = a.y + b.y;
		this.z = a.z + b.z;

		return this;

	}

	addScaledVector( v, s ) {

		this.x += v.x * s;
		this.y += v.y * s;
		this.z += v.z * s;

		return this;

	}

	sub( v, w ) {

		if ( w !== undefined ) {

			console.warn( 'THREE.Vector3: .sub() now only accepts one argument. Use .subVectors( a, b ) instead.' );
			return this.subVectors( v, w );

		}

		this.x -= v.x;
		this.y -= v.y;
		this.z -= v.z;

		return this;

	}

	subScalar( s ) {

		this.x -= s;
		this.y -= s;
		this.z -= s;

		return this;

	}

	subVectors( a, b ) {

		this.x = a.x - b.x;
		this.y = a.y - b.y;
		this.z = a.z - b.z;

		return this;

	}

	multiply( v, w ) {

		if ( w !== undefined ) {

			console.warn( 'THREE.Vector3: .multiply() now only accepts one argument. Use .multiplyVectors( a, b ) instead.' );
			return this.multiplyVectors( v, w );

		}

		this.x *= v.x;
		this.y *= v.y;
		this.z *= v.z;

		return this;

	}

	multiplyScalar( scalar ) {

		this.x *= scalar;
		this.y *= scalar;
		this.z *= scalar;

		return this;

	}

	multiplyVectors( a, b ) {

		this.x = a.x * b.x;
		this.y = a.y * b.y;
		this.z = a.z * b.z;

		return this;

	}

	applyEuler( euler ) {

		if ( ! ( euler && euler.isEuler ) ) {

			console.error( 'THREE.Vector3: .applyEuler() now expects an Euler rotation rather than a Vector3 and order.' );

		}

		return this.applyQuaternion( _quaternion$4.setFromEuler( euler ) );

	}

	applyAxisAngle( axis, angle ) {

		return this.applyQuaternion( _quaternion$4.setFromAxisAngle( axis, angle ) );

	}

	applyMatrix3( m ) {

		const x = this.x, y = this.y, z = this.z;
		const e = m.elements;

		this.x = e[ 0 ] * x + e[ 3 ] * y + e[ 6 ] * z;
		this.y = e[ 1 ] * x + e[ 4 ] * y + e[ 7 ] * z;
		this.z = e[ 2 ] * x + e[ 5 ] * y + e[ 8 ] * z;

		return this;

	}

	applyNormalMatrix( m ) {

		return this.applyMatrix3( m ).normalize();

	}

	applyMatrix4( m ) {

		const x = this.x, y = this.y, z = this.z;
		const e = m.elements;

		const w = 1 / ( e[ 3 ] * x + e[ 7 ] * y + e[ 11 ] * z + e[ 15 ] );

		this.x = ( e[ 0 ] * x + e[ 4 ] * y + e[ 8 ] * z + e[ 12 ] ) * w;
		this.y = ( e[ 1 ] * x + e[ 5 ] * y + e[ 9 ] * z + e[ 13 ] ) * w;
		this.z = ( e[ 2 ] * x + e[ 6 ] * y + e[ 10 ] * z + e[ 14 ] ) * w;

		return this;

	}

	applyQuaternion( q ) {

		const x = this.x, y = this.y, z = this.z;
		const qx = q.x, qy = q.y, qz = q.z, qw = q.w;

		// calculate quat * vector

		const ix = qw * x + qy * z - qz * y;
		const iy = qw * y + qz * x - qx * z;
		const iz = qw * z + qx * y - qy * x;
		const iw = - qx * x - qy * y - qz * z;

		// calculate result * inverse quat

		this.x = ix * qw + iw * - qx + iy * - qz - iz * - qy;
		this.y = iy * qw + iw * - qy + iz * - qx - ix * - qz;
		this.z = iz * qw + iw * - qz + ix * - qy - iy * - qx;

		return this;

	}

	project( camera ) {

		return this.applyMatrix4( camera.matrixWorldInverse ).applyMatrix4( camera.projectionMatrix );

	}

	unproject( camera ) {

		return this.applyMatrix4( camera.projectionMatrixInverse ).applyMatrix4( camera.matrixWorld );

	}

	transformDirection( m ) {

		// input: THREE.Matrix4 affine matrix
		// vector interpreted as a direction

		const x = this.x, y = this.y, z = this.z;
		const e = m.elements;

		this.x = e[ 0 ] * x + e[ 4 ] * y + e[ 8 ] * z;
		this.y = e[ 1 ] * x + e[ 5 ] * y + e[ 9 ] * z;
		this.z = e[ 2 ] * x + e[ 6 ] * y + e[ 10 ] * z;

		return this.normalize();

	}

	divide( v ) {

		this.x /= v.x;
		this.y /= v.y;
		this.z /= v.z;

		return this;

	}

	divideScalar( scalar ) {

		return this.multiplyScalar( 1 / scalar );

	}

	min( v ) {

		this.x = Math.min( this.x, v.x );
		this.y = Math.min( this.y, v.y );
		this.z = Math.min( this.z, v.z );

		return this;

	}

	max( v ) {

		this.x = Math.max( this.x, v.x );
		this.y = Math.max( this.y, v.y );
		this.z = Math.max( this.z, v.z );

		return this;

	}

	clamp( min, max ) {

		// assumes min < max, componentwise

		this.x = Math.max( min.x, Math.min( max.x, this.x ) );
		this.y = Math.max( min.y, Math.min( max.y, this.y ) );
		this.z = Math.max( min.z, Math.min( max.z, this.z ) );

		return this;

	}

	clampScalar( minVal, maxVal ) {

		this.x = Math.max( minVal, Math.min( maxVal, this.x ) );
		this.y = Math.max( minVal, Math.min( maxVal, this.y ) );
		this.z = Math.max( minVal, Math.min( maxVal, this.z ) );

		return this;

	}

	clampLength( min, max ) {

		const length = this.length();

		return this.divideScalar( length || 1 ).multiplyScalar( Math.max( min, Math.min( max, length ) ) );

	}

	floor() {

		this.x = Math.floor( this.x );
		this.y = Math.floor( this.y );
		this.z = Math.floor( this.z );

		return this;

	}

	ceil() {

		this.x = Math.ceil( this.x );
		this.y = Math.ceil( this.y );
		this.z = Math.ceil( this.z );

		return this;

	}

	round() {

		this.x = Math.round( this.x );
		this.y = Math.round( this.y );
		this.z = Math.round( this.z );

		return this;

	}

	roundToZero() {

		this.x = ( this.x < 0 ) ? Math.ceil( this.x ) : Math.floor( this.x );
		this.y = ( this.y < 0 ) ? Math.ceil( this.y ) : Math.floor( this.y );
		this.z = ( this.z < 0 ) ? Math.ceil( this.z ) : Math.floor( this.z );

		return this;

	}

	negate() {

		this.x = - this.x;
		this.y = - this.y;
		this.z = - this.z;

		return this;

	}

	dot( v ) {

		return this.x * v.x + this.y * v.y + this.z * v.z;

	}

	// TODO lengthSquared?

	lengthSq() {

		return this.x * this.x + this.y * this.y + this.z * this.z;

	}

	length() {

		return Math.sqrt( this.x * this.x + this.y * this.y + this.z * this.z );

	}

	manhattanLength() {

		return Math.abs( this.x ) + Math.abs( this.y ) + Math.abs( this.z );

	}

	normalize() {

		return this.divideScalar( this.length() || 1 );

	}

	setLength( length ) {

		return this.normalize().multiplyScalar( length );

	}

	lerp( v, alpha ) {

		this.x += ( v.x - this.x ) * alpha;
		this.y += ( v.y - this.y ) * alpha;
		this.z += ( v.z - this.z ) * alpha;

		return this;

	}

	lerpVectors( v1, v2, alpha ) {

		this.x = v1.x + ( v2.x - v1.x ) * alpha;
		this.y = v1.y + ( v2.y - v1.y ) * alpha;
		this.z = v1.z + ( v2.z - v1.z ) * alpha;

		return this;

	}

	cross( v, w ) {

		if ( w !== undefined ) {

			console.warn( 'THREE.Vector3: .cross() now only accepts one argument. Use .crossVectors( a, b ) instead.' );
			return this.crossVectors( v, w );

		}

		return this.crossVectors( this, v );

	}

	crossVectors( a, b ) {

		const ax = a.x, ay = a.y, az = a.z;
		const bx = b.x, by = b.y, bz = b.z;

		this.x = ay * bz - az * by;
		this.y = az * bx - ax * bz;
		this.z = ax * by - ay * bx;

		return this;

	}

	projectOnVector( v ) {

		const denominator = v.lengthSq();

		if ( denominator === 0 ) return this.set( 0, 0, 0 );

		const scalar = v.dot( this ) / denominator;

		return this.copy( v ).multiplyScalar( scalar );

	}

	projectOnPlane( planeNormal ) {

		_vector$c.copy( this ).projectOnVector( planeNormal );

		return this.sub( _vector$c );

	}

	reflect( normal ) {

		// reflect incident vector off plane orthogonal to normal
		// normal is assumed to have unit length

		return this.sub( _vector$c.copy( normal ).multiplyScalar( 2 * this.dot( normal ) ) );

	}

	angleTo( v ) {

		const denominator = Math.sqrt( this.lengthSq() * v.lengthSq() );

		if ( denominator === 0 ) return Math.PI / 2;

		const theta = this.dot( v ) / denominator;

		// clamp, to handle numerical problems

		return Math.acos( MathUtils.clamp( theta, - 1, 1 ) );

	}

	distanceTo( v ) {

		return Math.sqrt( this.distanceToSquared( v ) );

	}

	distanceToSquared( v ) {

		const dx = this.x - v.x, dy = this.y - v.y, dz = this.z - v.z;

		return dx * dx + dy * dy + dz * dz;

	}

	manhattanDistanceTo( v ) {

		return Math.abs( this.x - v.x ) + Math.abs( this.y - v.y ) + Math.abs( this.z - v.z );

	}

	setFromSpherical( s ) {

		return this.setFromSphericalCoords( s.radius, s.phi, s.theta );

	}

	setFromSphericalCoords( radius, phi, theta ) {

		const sinPhiRadius = Math.sin( phi ) * radius;

		this.x = sinPhiRadius * Math.sin( theta );
		this.y = Math.cos( phi ) * radius;
		this.z = sinPhiRadius * Math.cos( theta );

		return this;

	}

	setFromCylindrical( c ) {

		return this.setFromCylindricalCoords( c.radius, c.theta, c.y );

	}

	setFromCylindricalCoords( radius, theta, y ) {

		this.x = radius * Math.sin( theta );
		this.y = y;
		this.z = radius * Math.cos( theta );

		return this;

	}

	setFromMatrixPosition( m ) {

		const e = m.elements;

		this.x = e[ 12 ];
		this.y = e[ 13 ];
		this.z = e[ 14 ];

		return this;

	}

	setFromMatrixScale( m ) {

		const sx = this.setFromMatrixColumn( m, 0 ).length();
		const sy = this.setFromMatrixColumn( m, 1 ).length();
		const sz = this.setFromMatrixColumn( m, 2 ).length();

		this.x = sx;
		this.y = sy;
		this.z = sz;

		return this;

	}

	setFromMatrixColumn( m, index ) {

		return this.fromArray( m.elements, index * 4 );

	}

	setFromMatrix3Column( m, index ) {

		return this.fromArray( m.elements, index * 3 );

	}

	equals( v ) {

		return ( ( v.x === this.x ) && ( v.y === this.y ) && ( v.z === this.z ) );

	}

	fromArray( array, offset = 0 ) {

		this.x = array[ offset ];
		this.y = array[ offset + 1 ];
		this.z = array[ offset + 2 ];

		return this;

	}

	toArray( array = [], offset = 0 ) {

		array[ offset ] = this.x;
		array[ offset + 1 ] = this.y;
		array[ offset + 2 ] = this.z;

		return array;

	}

	fromBufferAttribute( attribute, index, offset ) {

		if ( offset !== undefined ) {

			console.warn( 'THREE.Vector3: offset has been removed from .fromBufferAttribute().' );

		}

		this.x = attribute.getX( index );
		this.y = attribute.getY( index );
		this.z = attribute.getZ( index );

		return this;

	}

	random() {

		this.x = Math.random();
		this.y = Math.random();
		this.z = Math.random();

		return this;

	}

}

Vector3.prototype.isVector3 = true;

const _vector$c = /*@__PURE__*/ new Vector3();
const _quaternion$4 = /*@__PURE__*/ new Quaternion();

class Box3 {

	constructor( min = new Vector3( + Infinity, + Infinity, + Infinity ), max = new Vector3( - Infinity, - Infinity, - Infinity ) ) {

		this.min = min;
		this.max = max;

	}

	set( min, max ) {

		this.min.copy( min );
		this.max.copy( max );

		return this;

	}

	setFromArray( array ) {

		let minX = + Infinity;
		let minY = + Infinity;
		let minZ = + Infinity;

		let maxX = - Infinity;
		let maxY = - Infinity;
		let maxZ = - Infinity;

		for ( let i = 0, l = array.length; i < l; i += 3 ) {

			const x = array[ i ];
			const y = array[ i + 1 ];
			const z = array[ i + 2 ];

			if ( x < minX ) minX = x;
			if ( y < minY ) minY = y;
			if ( z < minZ ) minZ = z;

			if ( x > maxX ) maxX = x;
			if ( y > maxY ) maxY = y;
			if ( z > maxZ ) maxZ = z;

		}

		this.min.set( minX, minY, minZ );
		this.max.set( maxX, maxY, maxZ );

		return this;

	}

	setFromBufferAttribute( attribute ) {

		let minX = + Infinity;
		let minY = + Infinity;
		let minZ = + Infinity;

		let maxX = - Infinity;
		let maxY = - Infinity;
		let maxZ = - Infinity;

		for ( let i = 0, l = attribute.count; i < l; i ++ ) {

			const x = attribute.getX( i );
			const y = attribute.getY( i );
			const z = attribute.getZ( i );

			if ( x < minX ) minX = x;
			if ( y < minY ) minY = y;
			if ( z < minZ ) minZ = z;

			if ( x > maxX ) maxX = x;
			if ( y > maxY ) maxY = y;
			if ( z > maxZ ) maxZ = z;

		}

		this.min.set( minX, minY, minZ );
		this.max.set( maxX, maxY, maxZ );

		return this;

	}

	setFromPoints( points ) {

		this.makeEmpty();

		for ( let i = 0, il = points.length; i < il; i ++ ) {

			this.expandByPoint( points[ i ] );

		}

		return this;

	}

	setFromCenterAndSize( center, size ) {

		const halfSize = _vector$b.copy( size ).multiplyScalar( 0.5 );

		this.min.copy( center ).sub( halfSize );
		this.max.copy( center ).add( halfSize );

		return this;

	}

	setFromObject( object ) {

		this.makeEmpty();

		return this.expandByObject( object );

	}

	clone() {

		return new this.constructor().copy( this );

	}

	copy( box ) {

		this.min.copy( box.min );
		this.max.copy( box.max );

		return this;

	}

	makeEmpty() {

		this.min.x = this.min.y = this.min.z = + Infinity;
		this.max.x = this.max.y = this.max.z = - Infinity;

		return this;

	}

	isEmpty() {

		// this is a more robust check for empty than ( volume <= 0 ) because volume can get positive with two negative axes

		return ( this.max.x < this.min.x ) || ( this.max.y < this.min.y ) || ( this.max.z < this.min.z );

	}

	getCenter( target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Box3: .getCenter() target is now required' );
			target = new Vector3();

		}

		return this.isEmpty() ? target.set( 0, 0, 0 ) : target.addVectors( this.min, this.max ).multiplyScalar( 0.5 );

	}

	getSize( target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Box3: .getSize() target is now required' );
			target = new Vector3();

		}

		return this.isEmpty() ? target.set( 0, 0, 0 ) : target.subVectors( this.max, this.min );

	}

	expandByPoint( point ) {

		this.min.min( point );
		this.max.max( point );

		return this;

	}

	expandByVector( vector ) {

		this.min.sub( vector );
		this.max.add( vector );

		return this;

	}

	expandByScalar( scalar ) {

		this.min.addScalar( - scalar );
		this.max.addScalar( scalar );

		return this;

	}

	expandByObject( object ) {

		// Computes the world-axis-aligned bounding box of an object (including its children),
		// accounting for both the object's, and children's, world transforms

		object.updateWorldMatrix( false, false );

		const geometry = object.geometry;

		if ( geometry !== undefined ) {

			if ( geometry.boundingBox === null ) {

				geometry.computeBoundingBox();

			}

			_box$3.copy( geometry.boundingBox );
			_box$3.applyMatrix4( object.matrixWorld );

			this.union( _box$3 );

		}

		const children = object.children;

		for ( let i = 0, l = children.length; i < l; i ++ ) {

			this.expandByObject( children[ i ] );

		}

		return this;

	}

	containsPoint( point ) {

		return point.x < this.min.x || point.x > this.max.x ||
			point.y < this.min.y || point.y > this.max.y ||
			point.z < this.min.z || point.z > this.max.z ? false : true;

	}

	containsBox( box ) {

		return this.min.x <= box.min.x && box.max.x <= this.max.x &&
			this.min.y <= box.min.y && box.max.y <= this.max.y &&
			this.min.z <= box.min.z && box.max.z <= this.max.z;

	}

	getParameter( point, target ) {

		// This can potentially have a divide by zero if the box
		// has a size dimension of 0.

		if ( target === undefined ) {

			console.warn( 'THREE.Box3: .getParameter() target is now required' );
			target = new Vector3();

		}

		return target.set(
			( point.x - this.min.x ) / ( this.max.x - this.min.x ),
			( point.y - this.min.y ) / ( this.max.y - this.min.y ),
			( point.z - this.min.z ) / ( this.max.z - this.min.z )
		);

	}

	intersectsBox( box ) {

		// using 6 splitting planes to rule out intersections.
		return box.max.x < this.min.x || box.min.x > this.max.x ||
			box.max.y < this.min.y || box.min.y > this.max.y ||
			box.max.z < this.min.z || box.min.z > this.max.z ? false : true;

	}

	intersectsSphere( sphere ) {

		// Find the point on the AABB closest to the sphere center.
		this.clampPoint( sphere.center, _vector$b );

		// If that point is inside the sphere, the AABB and sphere intersect.
		return _vector$b.distanceToSquared( sphere.center ) <= ( sphere.radius * sphere.radius );

	}

	intersectsPlane( plane ) {

		// We compute the minimum and maximum dot product values. If those values
		// are on the same side (back or front) of the plane, then there is no intersection.

		let min, max;

		if ( plane.normal.x > 0 ) {

			min = plane.normal.x * this.min.x;
			max = plane.normal.x * this.max.x;

		} else {

			min = plane.normal.x * this.max.x;
			max = plane.normal.x * this.min.x;

		}

		if ( plane.normal.y > 0 ) {

			min += plane.normal.y * this.min.y;
			max += plane.normal.y * this.max.y;

		} else {

			min += plane.normal.y * this.max.y;
			max += plane.normal.y * this.min.y;

		}

		if ( plane.normal.z > 0 ) {

			min += plane.normal.z * this.min.z;
			max += plane.normal.z * this.max.z;

		} else {

			min += plane.normal.z * this.max.z;
			max += plane.normal.z * this.min.z;

		}

		return ( min <= - plane.constant && max >= - plane.constant );

	}

	intersectsTriangle( triangle ) {

		if ( this.isEmpty() ) {

			return false;

		}

		// compute box center and extents
		this.getCenter( _center );
		_extents.subVectors( this.max, _center );

		// translate triangle to aabb origin
		_v0$2.subVectors( triangle.a, _center );
		_v1$7.subVectors( triangle.b, _center );
		_v2$3.subVectors( triangle.c, _center );

		// compute edge vectors for triangle
		_f0.subVectors( _v1$7, _v0$2 );
		_f1.subVectors( _v2$3, _v1$7 );
		_f2.subVectors( _v0$2, _v2$3 );

		// test against axes that are given by cross product combinations of the edges of the triangle and the edges of the aabb
		// make an axis testing of each of the 3 sides of the aabb against each of the 3 sides of the triangle = 9 axis of separation
		// axis_ij = u_i x f_j (u0, u1, u2 = face normals of aabb = x,y,z axes vectors since aabb is axis aligned)
		let axes = [
			0, - _f0.z, _f0.y, 0, - _f1.z, _f1.y, 0, - _f2.z, _f2.y,
			_f0.z, 0, - _f0.x, _f1.z, 0, - _f1.x, _f2.z, 0, - _f2.x,
			- _f0.y, _f0.x, 0, - _f1.y, _f1.x, 0, - _f2.y, _f2.x, 0
		];
		if ( ! satForAxes( axes, _v0$2, _v1$7, _v2$3, _extents ) ) {

			return false;

		}

		// test 3 face normals from the aabb
		axes = [ 1, 0, 0, 0, 1, 0, 0, 0, 1 ];
		if ( ! satForAxes( axes, _v0$2, _v1$7, _v2$3, _extents ) ) {

			return false;

		}

		// finally testing the face normal of the triangle
		// use already existing triangle edge vectors here
		_triangleNormal.crossVectors( _f0, _f1 );
		axes = [ _triangleNormal.x, _triangleNormal.y, _triangleNormal.z ];

		return satForAxes( axes, _v0$2, _v1$7, _v2$3, _extents );

	}

	clampPoint( point, target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Box3: .clampPoint() target is now required' );
			target = new Vector3();

		}

		return target.copy( point ).clamp( this.min, this.max );

	}

	distanceToPoint( point ) {

		const clampedPoint = _vector$b.copy( point ).clamp( this.min, this.max );

		return clampedPoint.sub( point ).length();

	}

	getBoundingSphere( target ) {

		if ( target === undefined ) {

			console.error( 'THREE.Box3: .getBoundingSphere() target is now required' );
			//target = new Sphere(); // removed to avoid cyclic dependency

		}

		this.getCenter( target.center );

		target.radius = this.getSize( _vector$b ).length() * 0.5;

		return target;

	}

	intersect( box ) {

		this.min.max( box.min );
		this.max.min( box.max );

		// ensure that if there is no overlap, the result is fully empty, not slightly empty with non-inf/+inf values that will cause subsequence intersects to erroneously return valid values.
		if ( this.isEmpty() ) this.makeEmpty();

		return this;

	}

	union( box ) {

		this.min.min( box.min );
		this.max.max( box.max );

		return this;

	}

	applyMatrix4( matrix ) {

		// transform of empty box is an empty box.
		if ( this.isEmpty() ) return this;

		// NOTE: I am using a binary pattern to specify all 2^3 combinations below
		_points[ 0 ].set( this.min.x, this.min.y, this.min.z ).applyMatrix4( matrix ); // 000
		_points[ 1 ].set( this.min.x, this.min.y, this.max.z ).applyMatrix4( matrix ); // 001
		_points[ 2 ].set( this.min.x, this.max.y, this.min.z ).applyMatrix4( matrix ); // 010
		_points[ 3 ].set( this.min.x, this.max.y, this.max.z ).applyMatrix4( matrix ); // 011
		_points[ 4 ].set( this.max.x, this.min.y, this.min.z ).applyMatrix4( matrix ); // 100
		_points[ 5 ].set( this.max.x, this.min.y, this.max.z ).applyMatrix4( matrix ); // 101
		_points[ 6 ].set( this.max.x, this.max.y, this.min.z ).applyMatrix4( matrix ); // 110
		_points[ 7 ].set( this.max.x, this.max.y, this.max.z ).applyMatrix4( matrix ); // 111

		this.setFromPoints( _points );

		return this;

	}

	translate( offset ) {

		this.min.add( offset );
		this.max.add( offset );

		return this;

	}

	equals( box ) {

		return box.min.equals( this.min ) && box.max.equals( this.max );

	}

}

Box3.prototype.isBox3 = true;

const _points = [
	/*@__PURE__*/ new Vector3(),
	/*@__PURE__*/ new Vector3(),
	/*@__PURE__*/ new Vector3(),
	/*@__PURE__*/ new Vector3(),
	/*@__PURE__*/ new Vector3(),
	/*@__PURE__*/ new Vector3(),
	/*@__PURE__*/ new Vector3(),
	/*@__PURE__*/ new Vector3()
];

const _vector$b = /*@__PURE__*/ new Vector3();

const _box$3 = /*@__PURE__*/ new Box3();

// triangle centered vertices

const _v0$2 = /*@__PURE__*/ new Vector3();
const _v1$7 = /*@__PURE__*/ new Vector3();
const _v2$3 = /*@__PURE__*/ new Vector3();

// triangle edge vectors

const _f0 = /*@__PURE__*/ new Vector3();
const _f1 = /*@__PURE__*/ new Vector3();
const _f2 = /*@__PURE__*/ new Vector3();

const _center = /*@__PURE__*/ new Vector3();
const _extents = /*@__PURE__*/ new Vector3();
const _triangleNormal = /*@__PURE__*/ new Vector3();
const _testAxis = /*@__PURE__*/ new Vector3();

function satForAxes( axes, v0, v1, v2, extents ) {

	for ( let i = 0, j = axes.length - 3; i <= j; i += 3 ) {

		_testAxis.fromArray( axes, i );
		// project the aabb onto the seperating axis
		const r = extents.x * Math.abs( _testAxis.x ) + extents.y * Math.abs( _testAxis.y ) + extents.z * Math.abs( _testAxis.z );
		// project all 3 vertices of the triangle onto the seperating axis
		const p0 = v0.dot( _testAxis );
		const p1 = v1.dot( _testAxis );
		const p2 = v2.dot( _testAxis );
		// actual test, basically see if either of the most extreme of the triangle points intersects r
		if ( Math.max( - Math.max( p0, p1, p2 ), Math.min( p0, p1, p2 ) ) > r ) {

			// points of the projected triangle are outside the projected half-length of the aabb
			// the axis is seperating and we can exit
			return false;

		}

	}

	return true;

}

const _box$2 = /*@__PURE__*/ new Box3();
const _v1$6 = /*@__PURE__*/ new Vector3();
const _toFarthestPoint = /*@__PURE__*/ new Vector3();
const _toPoint = /*@__PURE__*/ new Vector3();

class Sphere {

	constructor( center = new Vector3(), radius = - 1 ) {

		this.center = center;
		this.radius = radius;

	}

	set( center, radius ) {

		this.center.copy( center );
		this.radius = radius;

		return this;

	}

	setFromPoints( points, optionalCenter ) {

		const center = this.center;

		if ( optionalCenter !== undefined ) {

			center.copy( optionalCenter );

		} else {

			_box$2.setFromPoints( points ).getCenter( center );

		}

		let maxRadiusSq = 0;

		for ( let i = 0, il = points.length; i < il; i ++ ) {

			maxRadiusSq = Math.max( maxRadiusSq, center.distanceToSquared( points[ i ] ) );

		}

		this.radius = Math.sqrt( maxRadiusSq );

		return this;

	}

	copy( sphere ) {

		this.center.copy( sphere.center );
		this.radius = sphere.radius;

		return this;

	}

	isEmpty() {

		return ( this.radius < 0 );

	}

	makeEmpty() {

		this.center.set( 0, 0, 0 );
		this.radius = - 1;

		return this;

	}

	containsPoint( point ) {

		return ( point.distanceToSquared( this.center ) <= ( this.radius * this.radius ) );

	}

	distanceToPoint( point ) {

		return ( point.distanceTo( this.center ) - this.radius );

	}

	intersectsSphere( sphere ) {

		const radiusSum = this.radius + sphere.radius;

		return sphere.center.distanceToSquared( this.center ) <= ( radiusSum * radiusSum );

	}

	intersectsBox( box ) {

		return box.intersectsSphere( this );

	}

	intersectsPlane( plane ) {

		return Math.abs( plane.distanceToPoint( this.center ) ) <= this.radius;

	}

	clampPoint( point, target ) {

		const deltaLengthSq = this.center.distanceToSquared( point );

		if ( target === undefined ) {

			console.warn( 'THREE.Sphere: .clampPoint() target is now required' );
			target = new Vector3();

		}

		target.copy( point );

		if ( deltaLengthSq > ( this.radius * this.radius ) ) {

			target.sub( this.center ).normalize();
			target.multiplyScalar( this.radius ).add( this.center );

		}

		return target;

	}

	getBoundingBox( target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Sphere: .getBoundingBox() target is now required' );
			target = new Box3();

		}

		if ( this.isEmpty() ) {

			// Empty sphere produces empty bounding box
			target.makeEmpty();
			return target;

		}

		target.set( this.center, this.center );
		target.expandByScalar( this.radius );

		return target;

	}

	applyMatrix4( matrix ) {

		this.center.applyMatrix4( matrix );
		this.radius = this.radius * matrix.getMaxScaleOnAxis();

		return this;

	}

	translate( offset ) {

		this.center.add( offset );

		return this;

	}

	expandByPoint( point ) {

		// from https://github.com/juj/MathGeoLib/blob/2940b99b99cfe575dd45103ef20f4019dee15b54/src/Geometry/Sphere.cpp#L649-L671

		_toPoint.subVectors( point, this.center );

		const lengthSq = _toPoint.lengthSq();

		if ( lengthSq > ( this.radius * this.radius ) ) {

			const length = Math.sqrt( lengthSq );
			const missingRadiusHalf = ( length - this.radius ) * 0.5;

			// Nudge this sphere towards the target point. Add half the missing distance to radius,
			// and the other half to position. This gives a tighter enclosure, instead of if
			// the whole missing distance were just added to radius.

			this.center.add( _toPoint.multiplyScalar( missingRadiusHalf / length ) );
			this.radius += missingRadiusHalf;

		}

		return this;

	}

	union( sphere ) {

		// from https://github.com/juj/MathGeoLib/blob/2940b99b99cfe575dd45103ef20f4019dee15b54/src/Geometry/Sphere.cpp#L759-L769

		// To enclose another sphere into this sphere, we only need to enclose two points:
		// 1) Enclose the farthest point on the other sphere into this sphere.
		// 2) Enclose the opposite point of the farthest point into this sphere.

		_toFarthestPoint.subVectors( sphere.center, this.center ).normalize().multiplyScalar( sphere.radius );

		this.expandByPoint( _v1$6.copy( sphere.center ).add( _toFarthestPoint ) );
		this.expandByPoint( _v1$6.copy( sphere.center ).sub( _toFarthestPoint ) );

		return this;

	}

	equals( sphere ) {

		return sphere.center.equals( this.center ) && ( sphere.radius === this.radius );

	}

	clone() {

		return new this.constructor().copy( this );

	}

}

const _vector$a = /*@__PURE__*/ new Vector3();
const _segCenter = /*@__PURE__*/ new Vector3();
const _segDir = /*@__PURE__*/ new Vector3();
const _diff = /*@__PURE__*/ new Vector3();

const _edge1 = /*@__PURE__*/ new Vector3();
const _edge2 = /*@__PURE__*/ new Vector3();
const _normal$1 = /*@__PURE__*/ new Vector3();

class Ray {

	constructor( origin = new Vector3(), direction = new Vector3( 0, 0, - 1 ) ) {

		this.origin = origin;
		this.direction = direction;

	}

	set( origin, direction ) {

		this.origin.copy( origin );
		this.direction.copy( direction );

		return this;

	}

	copy( ray ) {

		this.origin.copy( ray.origin );
		this.direction.copy( ray.direction );

		return this;

	}

	at( t, target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Ray: .at() target is now required' );
			target = new Vector3();

		}

		return target.copy( this.direction ).multiplyScalar( t ).add( this.origin );

	}

	lookAt( v ) {

		this.direction.copy( v ).sub( this.origin ).normalize();

		return this;

	}

	recast( t ) {

		this.origin.copy( this.at( t, _vector$a ) );

		return this;

	}

	closestPointToPoint( point, target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Ray: .closestPointToPoint() target is now required' );
			target = new Vector3();

		}

		target.subVectors( point, this.origin );

		const directionDistance = target.dot( this.direction );

		if ( directionDistance < 0 ) {

			return target.copy( this.origin );

		}

		return target.copy( this.direction ).multiplyScalar( directionDistance ).add( this.origin );

	}

	distanceToPoint( point ) {

		return Math.sqrt( this.distanceSqToPoint( point ) );

	}

	distanceSqToPoint( point ) {

		const directionDistance = _vector$a.subVectors( point, this.origin ).dot( this.direction );

		// point behind the ray

		if ( directionDistance < 0 ) {

			return this.origin.distanceToSquared( point );

		}

		_vector$a.copy( this.direction ).multiplyScalar( directionDistance ).add( this.origin );

		return _vector$a.distanceToSquared( point );

	}

	distanceSqToSegment( v0, v1, optionalPointOnRay, optionalPointOnSegment ) {

		// from http://www.geometrictools.com/GTEngine/Include/Mathematics/GteDistRaySegment.h
		// It returns the min distance between the ray and the segment
		// defined by v0 and v1
		// It can also set two optional targets :
		// - The closest point on the ray
		// - The closest point on the segment

		_segCenter.copy( v0 ).add( v1 ).multiplyScalar( 0.5 );
		_segDir.copy( v1 ).sub( v0 ).normalize();
		_diff.copy( this.origin ).sub( _segCenter );

		const segExtent = v0.distanceTo( v1 ) * 0.5;
		const a01 = - this.direction.dot( _segDir );
		const b0 = _diff.dot( this.direction );
		const b1 = - _diff.dot( _segDir );
		const c = _diff.lengthSq();
		const det = Math.abs( 1 - a01 * a01 );
		let s0, s1, sqrDist, extDet;

		if ( det > 0 ) {

			// The ray and segment are not parallel.

			s0 = a01 * b1 - b0;
			s1 = a01 * b0 - b1;
			extDet = segExtent * det;

			if ( s0 >= 0 ) {

				if ( s1 >= - extDet ) {

					if ( s1 <= extDet ) {

						// region 0
						// Minimum at interior points of ray and segment.

						const invDet = 1 / det;
						s0 *= invDet;
						s1 *= invDet;
						sqrDist = s0 * ( s0 + a01 * s1 + 2 * b0 ) + s1 * ( a01 * s0 + s1 + 2 * b1 ) + c;

					} else {

						// region 1

						s1 = segExtent;
						s0 = Math.max( 0, - ( a01 * s1 + b0 ) );
						sqrDist = - s0 * s0 + s1 * ( s1 + 2 * b1 ) + c;

					}

				} else {

					// region 5

					s1 = - segExtent;
					s0 = Math.max( 0, - ( a01 * s1 + b0 ) );
					sqrDist = - s0 * s0 + s1 * ( s1 + 2 * b1 ) + c;

				}

			} else {

				if ( s1 <= - extDet ) {

					// region 4

					s0 = Math.max( 0, - ( - a01 * segExtent + b0 ) );
					s1 = ( s0 > 0 ) ? - segExtent : Math.min( Math.max( - segExtent, - b1 ), segExtent );
					sqrDist = - s0 * s0 + s1 * ( s1 + 2 * b1 ) + c;

				} else if ( s1 <= extDet ) {

					// region 3

					s0 = 0;
					s1 = Math.min( Math.max( - segExtent, - b1 ), segExtent );
					sqrDist = s1 * ( s1 + 2 * b1 ) + c;

				} else {

					// region 2

					s0 = Math.max( 0, - ( a01 * segExtent + b0 ) );
					s1 = ( s0 > 0 ) ? segExtent : Math.min( Math.max( - segExtent, - b1 ), segExtent );
					sqrDist = - s0 * s0 + s1 * ( s1 + 2 * b1 ) + c;

				}

			}

		} else {

			// Ray and segment are parallel.

			s1 = ( a01 > 0 ) ? - segExtent : segExtent;
			s0 = Math.max( 0, - ( a01 * s1 + b0 ) );
			sqrDist = - s0 * s0 + s1 * ( s1 + 2 * b1 ) + c;

		}

		if ( optionalPointOnRay ) {

			optionalPointOnRay.copy( this.direction ).multiplyScalar( s0 ).add( this.origin );

		}

		if ( optionalPointOnSegment ) {

			optionalPointOnSegment.copy( _segDir ).multiplyScalar( s1 ).add( _segCenter );

		}

		return sqrDist;

	}

	intersectSphere( sphere, target ) {

		_vector$a.subVectors( sphere.center, this.origin );
		const tca = _vector$a.dot( this.direction );
		const d2 = _vector$a.dot( _vector$a ) - tca * tca;
		const radius2 = sphere.radius * sphere.radius;

		if ( d2 > radius2 ) return null;

		const thc = Math.sqrt( radius2 - d2 );

		// t0 = first intersect point - entrance on front of sphere
		const t0 = tca - thc;

		// t1 = second intersect point - exit point on back of sphere
		const t1 = tca + thc;

		// test to see if both t0 and t1 are behind the ray - if so, return null
		if ( t0 < 0 && t1 < 0 ) return null;

		// test to see if t0 is behind the ray:
		// if it is, the ray is inside the sphere, so return the second exit point scaled by t1,
		// in order to always return an intersect point that is in front of the ray.
		if ( t0 < 0 ) return this.at( t1, target );

		// else t0 is in front of the ray, so return the first collision point scaled by t0
		return this.at( t0, target );

	}

	intersectsSphere( sphere ) {

		return this.distanceSqToPoint( sphere.center ) <= ( sphere.radius * sphere.radius );

	}

	distanceToPlane( plane ) {

		const denominator = plane.normal.dot( this.direction );

		if ( denominator === 0 ) {

			// line is coplanar, return origin
			if ( plane.distanceToPoint( this.origin ) === 0 ) {

				return 0;

			}

			// Null is preferable to undefined since undefined means.... it is undefined

			return null;

		}

		const t = - ( this.origin.dot( plane.normal ) + plane.constant ) / denominator;

		// Return if the ray never intersects the plane

		return t >= 0 ? t : null;

	}

	intersectPlane( plane, target ) {

		const t = this.distanceToPlane( plane );

		if ( t === null ) {

			return null;

		}

		return this.at( t, target );

	}

	intersectsPlane( plane ) {

		// check if the ray lies on the plane first

		const distToPoint = plane.distanceToPoint( this.origin );

		if ( distToPoint === 0 ) {

			return true;

		}

		const denominator = plane.normal.dot( this.direction );

		if ( denominator * distToPoint < 0 ) {

			return true;

		}

		// ray origin is behind the plane (and is pointing behind it)

		return false;

	}

	intersectBox( box, target ) {

		let tmin, tmax, tymin, tymax, tzmin, tzmax;

		const invdirx = 1 / this.direction.x,
			invdiry = 1 / this.direction.y,
			invdirz = 1 / this.direction.z;

		const origin = this.origin;

		if ( invdirx >= 0 ) {

			tmin = ( box.min.x - origin.x ) * invdirx;
			tmax = ( box.max.x - origin.x ) * invdirx;

		} else {

			tmin = ( box.max.x - origin.x ) * invdirx;
			tmax = ( box.min.x - origin.x ) * invdirx;

		}

		if ( invdiry >= 0 ) {

			tymin = ( box.min.y - origin.y ) * invdiry;
			tymax = ( box.max.y - origin.y ) * invdiry;

		} else {

			tymin = ( box.max.y - origin.y ) * invdiry;
			tymax = ( box.min.y - origin.y ) * invdiry;

		}

		if ( ( tmin > tymax ) || ( tymin > tmax ) ) return null;

		// These lines also handle the case where tmin or tmax is NaN
		// (result of 0 * Infinity). x !== x returns true if x is NaN

		if ( tymin > tmin || tmin !== tmin ) tmin = tymin;

		if ( tymax < tmax || tmax !== tmax ) tmax = tymax;

		if ( invdirz >= 0 ) {

			tzmin = ( box.min.z - origin.z ) * invdirz;
			tzmax = ( box.max.z - origin.z ) * invdirz;

		} else {

			tzmin = ( box.max.z - origin.z ) * invdirz;
			tzmax = ( box.min.z - origin.z ) * invdirz;

		}

		if ( ( tmin > tzmax ) || ( tzmin > tmax ) ) return null;

		if ( tzmin > tmin || tmin !== tmin ) tmin = tzmin;

		if ( tzmax < tmax || tmax !== tmax ) tmax = tzmax;

		//return point closest to the ray (positive side)

		if ( tmax < 0 ) return null;

		return this.at( tmin >= 0 ? tmin : tmax, target );

	}

	intersectsBox( box ) {

		return this.intersectBox( box, _vector$a ) !== null;

	}

	intersectTriangle( a, b, c, backfaceCulling, target ) {

		// Compute the offset origin, edges, and normal.

		// from http://www.geometrictools.com/GTEngine/Include/Mathematics/GteIntrRay3Triangle3.h

		_edge1.subVectors( b, a );
		_edge2.subVectors( c, a );
		_normal$1.crossVectors( _edge1, _edge2 );

		// Solve Q + t*D = b1*E1 + b2*E2 (Q = kDiff, D = ray direction,
		// E1 = kEdge1, E2 = kEdge2, N = Cross(E1,E2)) by
		//   |Dot(D,N)|*b1 = sign(Dot(D,N))*Dot(D,Cross(Q,E2))
		//   |Dot(D,N)|*b2 = sign(Dot(D,N))*Dot(D,Cross(E1,Q))
		//   |Dot(D,N)|*t = -sign(Dot(D,N))*Dot(Q,N)
		let DdN = this.direction.dot( _normal$1 );
		let sign;

		if ( DdN > 0 ) {

			if ( backfaceCulling ) return null;
			sign = 1;

		} else if ( DdN < 0 ) {

			sign = - 1;
			DdN = - DdN;

		} else {

			return null;

		}

		_diff.subVectors( this.origin, a );
		const DdQxE2 = sign * this.direction.dot( _edge2.crossVectors( _diff, _edge2 ) );

		// b1 < 0, no intersection
		if ( DdQxE2 < 0 ) {

			return null;

		}

		const DdE1xQ = sign * this.direction.dot( _edge1.cross( _diff ) );

		// b2 < 0, no intersection
		if ( DdE1xQ < 0 ) {

			return null;

		}

		// b1+b2 > 1, no intersection
		if ( DdQxE2 + DdE1xQ > DdN ) {

			return null;

		}

		// Line intersects triangle, check if ray does.
		const QdN = - sign * _diff.dot( _normal$1 );

		// t < 0, no intersection
		if ( QdN < 0 ) {

			return null;

		}

		// Ray intersects triangle.
		return this.at( QdN / DdN, target );

	}

	applyMatrix4( matrix4 ) {

		this.origin.applyMatrix4( matrix4 );
		this.direction.transformDirection( matrix4 );

		return this;

	}

	equals( ray ) {

		return ray.origin.equals( this.origin ) && ray.direction.equals( this.direction );

	}

	clone() {

		return new this.constructor().copy( this );

	}

}

class Matrix4 {

	constructor() {

		this.elements = [

			1, 0, 0, 0,
			0, 1, 0, 0,
			0, 0, 1, 0,
			0, 0, 0, 1

		];

		if ( arguments.length > 0 ) {

			console.error( 'THREE.Matrix4: the constructor no longer reads arguments. use .set() instead.' );

		}

	}

	set( n11, n12, n13, n14, n21, n22, n23, n24, n31, n32, n33, n34, n41, n42, n43, n44 ) {

		const te = this.elements;

		te[ 0 ] = n11; te[ 4 ] = n12; te[ 8 ] = n13; te[ 12 ] = n14;
		te[ 1 ] = n21; te[ 5 ] = n22; te[ 9 ] = n23; te[ 13 ] = n24;
		te[ 2 ] = n31; te[ 6 ] = n32; te[ 10 ] = n33; te[ 14 ] = n34;
		te[ 3 ] = n41; te[ 7 ] = n42; te[ 11 ] = n43; te[ 15 ] = n44;

		return this;

	}

	identity() {

		this.set(

			1, 0, 0, 0,
			0, 1, 0, 0,
			0, 0, 1, 0,
			0, 0, 0, 1

		);

		return this;

	}

	clone() {

		return new Matrix4().fromArray( this.elements );

	}

	copy( m ) {

		const te = this.elements;
		const me = m.elements;

		te[ 0 ] = me[ 0 ]; te[ 1 ] = me[ 1 ]; te[ 2 ] = me[ 2 ]; te[ 3 ] = me[ 3 ];
		te[ 4 ] = me[ 4 ]; te[ 5 ] = me[ 5 ]; te[ 6 ] = me[ 6 ]; te[ 7 ] = me[ 7 ];
		te[ 8 ] = me[ 8 ]; te[ 9 ] = me[ 9 ]; te[ 10 ] = me[ 10 ]; te[ 11 ] = me[ 11 ];
		te[ 12 ] = me[ 12 ]; te[ 13 ] = me[ 13 ]; te[ 14 ] = me[ 14 ]; te[ 15 ] = me[ 15 ];

		return this;

	}

	copyPosition( m ) {

		const te = this.elements, me = m.elements;

		te[ 12 ] = me[ 12 ];
		te[ 13 ] = me[ 13 ];
		te[ 14 ] = me[ 14 ];

		return this;

	}

	setFromMatrix3( m ) {

		const me = m.elements;

		this.set(

			me[ 0 ], me[ 3 ], me[ 6 ], 0,
			me[ 1 ], me[ 4 ], me[ 7 ], 0,
			me[ 2 ], me[ 5 ], me[ 8 ], 0,
			0, 0, 0, 1

		);

		return this;

	}

	extractBasis( xAxis, yAxis, zAxis ) {

		xAxis.setFromMatrixColumn( this, 0 );
		yAxis.setFromMatrixColumn( this, 1 );
		zAxis.setFromMatrixColumn( this, 2 );

		return this;

	}

	makeBasis( xAxis, yAxis, zAxis ) {

		this.set(
			xAxis.x, yAxis.x, zAxis.x, 0,
			xAxis.y, yAxis.y, zAxis.y, 0,
			xAxis.z, yAxis.z, zAxis.z, 0,
			0, 0, 0, 1
		);

		return this;

	}

	extractRotation( m ) {

		// this method does not support reflection matrices

		const te = this.elements;
		const me = m.elements;

		const scaleX = 1 / _v1$5.setFromMatrixColumn( m, 0 ).length();
		const scaleY = 1 / _v1$5.setFromMatrixColumn( m, 1 ).length();
		const scaleZ = 1 / _v1$5.setFromMatrixColumn( m, 2 ).length();

		te[ 0 ] = me[ 0 ] * scaleX;
		te[ 1 ] = me[ 1 ] * scaleX;
		te[ 2 ] = me[ 2 ] * scaleX;
		te[ 3 ] = 0;

		te[ 4 ] = me[ 4 ] * scaleY;
		te[ 5 ] = me[ 5 ] * scaleY;
		te[ 6 ] = me[ 6 ] * scaleY;
		te[ 7 ] = 0;

		te[ 8 ] = me[ 8 ] * scaleZ;
		te[ 9 ] = me[ 9 ] * scaleZ;
		te[ 10 ] = me[ 10 ] * scaleZ;
		te[ 11 ] = 0;

		te[ 12 ] = 0;
		te[ 13 ] = 0;
		te[ 14 ] = 0;
		te[ 15 ] = 1;

		return this;

	}

	makeRotationFromEuler( euler ) {

		if ( ! ( euler && euler.isEuler ) ) {

			console.error( 'THREE.Matrix4: .makeRotationFromEuler() now expects a Euler rotation rather than a Vector3 and order.' );

		}

		const te = this.elements;

		const x = euler.x, y = euler.y, z = euler.z;
		const a = Math.cos( x ), b = Math.sin( x );
		const c = Math.cos( y ), d = Math.sin( y );
		const e = Math.cos( z ), f = Math.sin( z );

		if ( euler.order === 'XYZ' ) {

			const ae = a * e, af = a * f, be = b * e, bf = b * f;

			te[ 0 ] = c * e;
			te[ 4 ] = - c * f;
			te[ 8 ] = d;

			te[ 1 ] = af + be * d;
			te[ 5 ] = ae - bf * d;
			te[ 9 ] = - b * c;

			te[ 2 ] = bf - ae * d;
			te[ 6 ] = be + af * d;
			te[ 10 ] = a * c;

		} else if ( euler.order === 'YXZ' ) {

			const ce = c * e, cf = c * f, de = d * e, df = d * f;

			te[ 0 ] = ce + df * b;
			te[ 4 ] = de * b - cf;
			te[ 8 ] = a * d;

			te[ 1 ] = a * f;
			te[ 5 ] = a * e;
			te[ 9 ] = - b;

			te[ 2 ] = cf * b - de;
			te[ 6 ] = df + ce * b;
			te[ 10 ] = a * c;

		} else if ( euler.order === 'ZXY' ) {

			const ce = c * e, cf = c * f, de = d * e, df = d * f;

			te[ 0 ] = ce - df * b;
			te[ 4 ] = - a * f;
			te[ 8 ] = de + cf * b;

			te[ 1 ] = cf + de * b;
			te[ 5 ] = a * e;
			te[ 9 ] = df - ce * b;

			te[ 2 ] = - a * d;
			te[ 6 ] = b;
			te[ 10 ] = a * c;

		} else if ( euler.order === 'ZYX' ) {

			const ae = a * e, af = a * f, be = b * e, bf = b * f;

			te[ 0 ] = c * e;
			te[ 4 ] = be * d - af;
			te[ 8 ] = ae * d + bf;

			te[ 1 ] = c * f;
			te[ 5 ] = bf * d + ae;
			te[ 9 ] = af * d - be;

			te[ 2 ] = - d;
			te[ 6 ] = b * c;
			te[ 10 ] = a * c;

		} else if ( euler.order === 'YZX' ) {

			const ac = a * c, ad = a * d, bc = b * c, bd = b * d;

			te[ 0 ] = c * e;
			te[ 4 ] = bd - ac * f;
			te[ 8 ] = bc * f + ad;

			te[ 1 ] = f;
			te[ 5 ] = a * e;
			te[ 9 ] = - b * e;

			te[ 2 ] = - d * e;
			te[ 6 ] = ad * f + bc;
			te[ 10 ] = ac - bd * f;

		} else if ( euler.order === 'XZY' ) {

			const ac = a * c, ad = a * d, bc = b * c, bd = b * d;

			te[ 0 ] = c * e;
			te[ 4 ] = - f;
			te[ 8 ] = d * e;

			te[ 1 ] = ac * f + bd;
			te[ 5 ] = a * e;
			te[ 9 ] = ad * f - bc;

			te[ 2 ] = bc * f - ad;
			te[ 6 ] = b * e;
			te[ 10 ] = bd * f + ac;

		}

		// bottom row
		te[ 3 ] = 0;
		te[ 7 ] = 0;
		te[ 11 ] = 0;

		// last column
		te[ 12 ] = 0;
		te[ 13 ] = 0;
		te[ 14 ] = 0;
		te[ 15 ] = 1;

		return this;

	}

	makeRotationFromQuaternion( q ) {

		return this.compose( _zero, q, _one );

	}

	lookAt( eye, target, up ) {

		const te = this.elements;

		_z.subVectors( eye, target );

		if ( _z.lengthSq() === 0 ) {

			// eye and target are in the same position

			_z.z = 1;

		}

		_z.normalize();
		_x.crossVectors( up, _z );

		if ( _x.lengthSq() === 0 ) {

			// up and z are parallel

			if ( Math.abs( up.z ) === 1 ) {

				_z.x += 0.0001;

			} else {

				_z.z += 0.0001;

			}

			_z.normalize();
			_x.crossVectors( up, _z );

		}

		_x.normalize();
		_y.crossVectors( _z, _x );

		te[ 0 ] = _x.x; te[ 4 ] = _y.x; te[ 8 ] = _z.x;
		te[ 1 ] = _x.y; te[ 5 ] = _y.y; te[ 9 ] = _z.y;
		te[ 2 ] = _x.z; te[ 6 ] = _y.z; te[ 10 ] = _z.z;

		return this;

	}

	multiply( m, n ) {

		if ( n !== undefined ) {

			console.warn( 'THREE.Matrix4: .multiply() now only accepts one argument. Use .multiplyMatrices( a, b ) instead.' );
			return this.multiplyMatrices( m, n );

		}

		return this.multiplyMatrices( this, m );

	}

	premultiply( m ) {

		return this.multiplyMatrices( m, this );

	}

	multiplyMatrices( a, b ) {

		const ae = a.elements;
		const be = b.elements;
		const te = this.elements;

		const a11 = ae[ 0 ], a12 = ae[ 4 ], a13 = ae[ 8 ], a14 = ae[ 12 ];
		const a21 = ae[ 1 ], a22 = ae[ 5 ], a23 = ae[ 9 ], a24 = ae[ 13 ];
		const a31 = ae[ 2 ], a32 = ae[ 6 ], a33 = ae[ 10 ], a34 = ae[ 14 ];
		const a41 = ae[ 3 ], a42 = ae[ 7 ], a43 = ae[ 11 ], a44 = ae[ 15 ];

		const b11 = be[ 0 ], b12 = be[ 4 ], b13 = be[ 8 ], b14 = be[ 12 ];
		const b21 = be[ 1 ], b22 = be[ 5 ], b23 = be[ 9 ], b24 = be[ 13 ];
		const b31 = be[ 2 ], b32 = be[ 6 ], b33 = be[ 10 ], b34 = be[ 14 ];
		const b41 = be[ 3 ], b42 = be[ 7 ], b43 = be[ 11 ], b44 = be[ 15 ];

		te[ 0 ] = a11 * b11 + a12 * b21 + a13 * b31 + a14 * b41;
		te[ 4 ] = a11 * b12 + a12 * b22 + a13 * b32 + a14 * b42;
		te[ 8 ] = a11 * b13 + a12 * b23 + a13 * b33 + a14 * b43;
		te[ 12 ] = a11 * b14 + a12 * b24 + a13 * b34 + a14 * b44;

		te[ 1 ] = a21 * b11 + a22 * b21 + a23 * b31 + a24 * b41;
		te[ 5 ] = a21 * b12 + a22 * b22 + a23 * b32 + a24 * b42;
		te[ 9 ] = a21 * b13 + a22 * b23 + a23 * b33 + a24 * b43;
		te[ 13 ] = a21 * b14 + a22 * b24 + a23 * b34 + a24 * b44;

		te[ 2 ] = a31 * b11 + a32 * b21 + a33 * b31 + a34 * b41;
		te[ 6 ] = a31 * b12 + a32 * b22 + a33 * b32 + a34 * b42;
		te[ 10 ] = a31 * b13 + a32 * b23 + a33 * b33 + a34 * b43;
		te[ 14 ] = a31 * b14 + a32 * b24 + a33 * b34 + a34 * b44;

		te[ 3 ] = a41 * b11 + a42 * b21 + a43 * b31 + a44 * b41;
		te[ 7 ] = a41 * b12 + a42 * b22 + a43 * b32 + a44 * b42;
		te[ 11 ] = a41 * b13 + a42 * b23 + a43 * b33 + a44 * b43;
		te[ 15 ] = a41 * b14 + a42 * b24 + a43 * b34 + a44 * b44;

		return this;

	}

	multiplyScalar( s ) {

		const te = this.elements;

		te[ 0 ] *= s; te[ 4 ] *= s; te[ 8 ] *= s; te[ 12 ] *= s;
		te[ 1 ] *= s; te[ 5 ] *= s; te[ 9 ] *= s; te[ 13 ] *= s;
		te[ 2 ] *= s; te[ 6 ] *= s; te[ 10 ] *= s; te[ 14 ] *= s;
		te[ 3 ] *= s; te[ 7 ] *= s; te[ 11 ] *= s; te[ 15 ] *= s;

		return this;

	}

	determinant() {

		const te = this.elements;

		const n11 = te[ 0 ], n12 = te[ 4 ], n13 = te[ 8 ], n14 = te[ 12 ];
		const n21 = te[ 1 ], n22 = te[ 5 ], n23 = te[ 9 ], n24 = te[ 13 ];
		const n31 = te[ 2 ], n32 = te[ 6 ], n33 = te[ 10 ], n34 = te[ 14 ];
		const n41 = te[ 3 ], n42 = te[ 7 ], n43 = te[ 11 ], n44 = te[ 15 ];

		//TODO: make this more efficient
		//( based on http://www.euclideanspace.com/maths/algebra/matrix/functions/inverse/fourD/index.htm )

		return (
			n41 * (
				+ n14 * n23 * n32
				 - n13 * n24 * n32
				 - n14 * n22 * n33
				 + n12 * n24 * n33
				 + n13 * n22 * n34
				 - n12 * n23 * n34
			) +
			n42 * (
				+ n11 * n23 * n34
				 - n11 * n24 * n33
				 + n14 * n21 * n33
				 - n13 * n21 * n34
				 + n13 * n24 * n31
				 - n14 * n23 * n31
			) +
			n43 * (
				+ n11 * n24 * n32
				 - n11 * n22 * n34
				 - n14 * n21 * n32
				 + n12 * n21 * n34
				 + n14 * n22 * n31
				 - n12 * n24 * n31
			) +
			n44 * (
				- n13 * n22 * n31
				 - n11 * n23 * n32
				 + n11 * n22 * n33
				 + n13 * n21 * n32
				 - n12 * n21 * n33
				 + n12 * n23 * n31
			)

		);

	}

	transpose() {

		const te = this.elements;
		let tmp;

		tmp = te[ 1 ]; te[ 1 ] = te[ 4 ]; te[ 4 ] = tmp;
		tmp = te[ 2 ]; te[ 2 ] = te[ 8 ]; te[ 8 ] = tmp;
		tmp = te[ 6 ]; te[ 6 ] = te[ 9 ]; te[ 9 ] = tmp;

		tmp = te[ 3 ]; te[ 3 ] = te[ 12 ]; te[ 12 ] = tmp;
		tmp = te[ 7 ]; te[ 7 ] = te[ 13 ]; te[ 13 ] = tmp;
		tmp = te[ 11 ]; te[ 11 ] = te[ 14 ]; te[ 14 ] = tmp;

		return this;

	}

	setPosition( x, y, z ) {

		const te = this.elements;

		if ( x.isVector3 ) {

			te[ 12 ] = x.x;
			te[ 13 ] = x.y;
			te[ 14 ] = x.z;

		} else {

			te[ 12 ] = x;
			te[ 13 ] = y;
			te[ 14 ] = z;

		}

		return this;

	}

	invert() {

		// based on http://www.euclideanspace.com/maths/algebra/matrix/functions/inverse/fourD/index.htm
		const te = this.elements,

			n11 = te[ 0 ], n21 = te[ 1 ], n31 = te[ 2 ], n41 = te[ 3 ],
			n12 = te[ 4 ], n22 = te[ 5 ], n32 = te[ 6 ], n42 = te[ 7 ],
			n13 = te[ 8 ], n23 = te[ 9 ], n33 = te[ 10 ], n43 = te[ 11 ],
			n14 = te[ 12 ], n24 = te[ 13 ], n34 = te[ 14 ], n44 = te[ 15 ],

			t11 = n23 * n34 * n42 - n24 * n33 * n42 + n24 * n32 * n43 - n22 * n34 * n43 - n23 * n32 * n44 + n22 * n33 * n44,
			t12 = n14 * n33 * n42 - n13 * n34 * n42 - n14 * n32 * n43 + n12 * n34 * n43 + n13 * n32 * n44 - n12 * n33 * n44,
			t13 = n13 * n24 * n42 - n14 * n23 * n42 + n14 * n22 * n43 - n12 * n24 * n43 - n13 * n22 * n44 + n12 * n23 * n44,
			t14 = n14 * n23 * n32 - n13 * n24 * n32 - n14 * n22 * n33 + n12 * n24 * n33 + n13 * n22 * n34 - n12 * n23 * n34;

		const det = n11 * t11 + n21 * t12 + n31 * t13 + n41 * t14;

		if ( det === 0 ) return this.set( 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );

		const detInv = 1 / det;

		te[ 0 ] = t11 * detInv;
		te[ 1 ] = ( n24 * n33 * n41 - n23 * n34 * n41 - n24 * n31 * n43 + n21 * n34 * n43 + n23 * n31 * n44 - n21 * n33 * n44 ) * detInv;
		te[ 2 ] = ( n22 * n34 * n41 - n24 * n32 * n41 + n24 * n31 * n42 - n21 * n34 * n42 - n22 * n31 * n44 + n21 * n32 * n44 ) * detInv;
		te[ 3 ] = ( n23 * n32 * n41 - n22 * n33 * n41 - n23 * n31 * n42 + n21 * n33 * n42 + n22 * n31 * n43 - n21 * n32 * n43 ) * detInv;

		te[ 4 ] = t12 * detInv;
		te[ 5 ] = ( n13 * n34 * n41 - n14 * n33 * n41 + n14 * n31 * n43 - n11 * n34 * n43 - n13 * n31 * n44 + n11 * n33 * n44 ) * detInv;
		te[ 6 ] = ( n14 * n32 * n41 - n12 * n34 * n41 - n14 * n31 * n42 + n11 * n34 * n42 + n12 * n31 * n44 - n11 * n32 * n44 ) * detInv;
		te[ 7 ] = ( n12 * n33 * n41 - n13 * n32 * n41 + n13 * n31 * n42 - n11 * n33 * n42 - n12 * n31 * n43 + n11 * n32 * n43 ) * detInv;

		te[ 8 ] = t13 * detInv;
		te[ 9 ] = ( n14 * n23 * n41 - n13 * n24 * n41 - n14 * n21 * n43 + n11 * n24 * n43 + n13 * n21 * n44 - n11 * n23 * n44 ) * detInv;
		te[ 10 ] = ( n12 * n24 * n41 - n14 * n22 * n41 + n14 * n21 * n42 - n11 * n24 * n42 - n12 * n21 * n44 + n11 * n22 * n44 ) * detInv;
		te[ 11 ] = ( n13 * n22 * n41 - n12 * n23 * n41 - n13 * n21 * n42 + n11 * n23 * n42 + n12 * n21 * n43 - n11 * n22 * n43 ) * detInv;

		te[ 12 ] = t14 * detInv;
		te[ 13 ] = ( n13 * n24 * n31 - n14 * n23 * n31 + n14 * n21 * n33 - n11 * n24 * n33 - n13 * n21 * n34 + n11 * n23 * n34 ) * detInv;
		te[ 14 ] = ( n14 * n22 * n31 - n12 * n24 * n31 - n14 * n21 * n32 + n11 * n24 * n32 + n12 * n21 * n34 - n11 * n22 * n34 ) * detInv;
		te[ 15 ] = ( n12 * n23 * n31 - n13 * n22 * n31 + n13 * n21 * n32 - n11 * n23 * n32 - n12 * n21 * n33 + n11 * n22 * n33 ) * detInv;

		return this;

	}

	scale( v ) {

		const te = this.elements;
		const x = v.x, y = v.y, z = v.z;

		te[ 0 ] *= x; te[ 4 ] *= y; te[ 8 ] *= z;
		te[ 1 ] *= x; te[ 5 ] *= y; te[ 9 ] *= z;
		te[ 2 ] *= x; te[ 6 ] *= y; te[ 10 ] *= z;
		te[ 3 ] *= x; te[ 7 ] *= y; te[ 11 ] *= z;

		return this;

	}

	getMaxScaleOnAxis() {

		const te = this.elements;

		const scaleXSq = te[ 0 ] * te[ 0 ] + te[ 1 ] * te[ 1 ] + te[ 2 ] * te[ 2 ];
		const scaleYSq = te[ 4 ] * te[ 4 ] + te[ 5 ] * te[ 5 ] + te[ 6 ] * te[ 6 ];
		const scaleZSq = te[ 8 ] * te[ 8 ] + te[ 9 ] * te[ 9 ] + te[ 10 ] * te[ 10 ];

		return Math.sqrt( Math.max( scaleXSq, scaleYSq, scaleZSq ) );

	}

	makeTranslation( x, y, z ) {

		this.set(

			1, 0, 0, x,
			0, 1, 0, y,
			0, 0, 1, z,
			0, 0, 0, 1

		);

		return this;

	}

	makeRotationX( theta ) {

		const c = Math.cos( theta ), s = Math.sin( theta );

		this.set(

			1, 0, 0, 0,
			0, c, - s, 0,
			0, s, c, 0,
			0, 0, 0, 1

		);

		return this;

	}

	makeRotationY( theta ) {

		const c = Math.cos( theta ), s = Math.sin( theta );

		this.set(

			 c, 0, s, 0,
			 0, 1, 0, 0,
			- s, 0, c, 0,
			 0, 0, 0, 1

		);

		return this;

	}

	makeRotationZ( theta ) {

		const c = Math.cos( theta ), s = Math.sin( theta );

		this.set(

			c, - s, 0, 0,
			s, c, 0, 0,
			0, 0, 1, 0,
			0, 0, 0, 1

		);

		return this;

	}

	makeRotationAxis( axis, angle ) {

		// Based on http://www.gamedev.net/reference/articles/article1199.asp

		const c = Math.cos( angle );
		const s = Math.sin( angle );
		const t = 1 - c;
		const x = axis.x, y = axis.y, z = axis.z;
		const tx = t * x, ty = t * y;

		this.set(

			tx * x + c, tx * y - s * z, tx * z + s * y, 0,
			tx * y + s * z, ty * y + c, ty * z - s * x, 0,
			tx * z - s * y, ty * z + s * x, t * z * z + c, 0,
			0, 0, 0, 1

		);

		return this;

	}

	makeScale( x, y, z ) {

		this.set(

			x, 0, 0, 0,
			0, y, 0, 0,
			0, 0, z, 0,
			0, 0, 0, 1

		);

		return this;

	}

	makeShear( x, y, z ) {

		this.set(

			1, y, z, 0,
			x, 1, z, 0,
			x, y, 1, 0,
			0, 0, 0, 1

		);

		return this;

	}

	compose( position, quaternion, scale ) {

		const te = this.elements;

		const x = quaternion._x, y = quaternion._y, z = quaternion._z, w = quaternion._w;
		const x2 = x + x,	y2 = y + y, z2 = z + z;
		const xx = x * x2, xy = x * y2, xz = x * z2;
		const yy = y * y2, yz = y * z2, zz = z * z2;
		const wx = w * x2, wy = w * y2, wz = w * z2;

		const sx = scale.x, sy = scale.y, sz = scale.z;

		te[ 0 ] = ( 1 - ( yy + zz ) ) * sx;
		te[ 1 ] = ( xy + wz ) * sx;
		te[ 2 ] = ( xz - wy ) * sx;
		te[ 3 ] = 0;

		te[ 4 ] = ( xy - wz ) * sy;
		te[ 5 ] = ( 1 - ( xx + zz ) ) * sy;
		te[ 6 ] = ( yz + wx ) * sy;
		te[ 7 ] = 0;

		te[ 8 ] = ( xz + wy ) * sz;
		te[ 9 ] = ( yz - wx ) * sz;
		te[ 10 ] = ( 1 - ( xx + yy ) ) * sz;
		te[ 11 ] = 0;

		te[ 12 ] = position.x;
		te[ 13 ] = position.y;
		te[ 14 ] = position.z;
		te[ 15 ] = 1;

		return this;

	}

	decompose( position, quaternion, scale ) {

		const te = this.elements;

		let sx = _v1$5.set( te[ 0 ], te[ 1 ], te[ 2 ] ).length();
		const sy = _v1$5.set( te[ 4 ], te[ 5 ], te[ 6 ] ).length();
		const sz = _v1$5.set( te[ 8 ], te[ 9 ], te[ 10 ] ).length();

		// if determine is negative, we need to invert one scale
		const det = this.determinant();
		if ( det < 0 ) sx = - sx;

		position.x = te[ 12 ];
		position.y = te[ 13 ];
		position.z = te[ 14 ];

		// scale the rotation part
		_m1$2.copy( this );

		const invSX = 1 / sx;
		const invSY = 1 / sy;
		const invSZ = 1 / sz;

		_m1$2.elements[ 0 ] *= invSX;
		_m1$2.elements[ 1 ] *= invSX;
		_m1$2.elements[ 2 ] *= invSX;

		_m1$2.elements[ 4 ] *= invSY;
		_m1$2.elements[ 5 ] *= invSY;
		_m1$2.elements[ 6 ] *= invSY;

		_m1$2.elements[ 8 ] *= invSZ;
		_m1$2.elements[ 9 ] *= invSZ;
		_m1$2.elements[ 10 ] *= invSZ;

		quaternion.setFromRotationMatrix( _m1$2 );

		scale.x = sx;
		scale.y = sy;
		scale.z = sz;

		return this;

	}

	makePerspective( left, right, top, bottom, near, far ) {

		if ( far === undefined ) {

			console.warn( 'THREE.Matrix4: .makePerspective() has been redefined and has a new signature. Please check the docs.' );

		}

		const te = this.elements;
		const x = 2 * near / ( right - left );
		const y = 2 * near / ( top - bottom );

		const a = ( right + left ) / ( right - left );
		const b = ( top + bottom ) / ( top - bottom );
		const c = - ( far + near ) / ( far - near );
		const d = - 2 * far * near / ( far - near );

		te[ 0 ] = x;	te[ 4 ] = 0;	te[ 8 ] = a;	te[ 12 ] = 0;
		te[ 1 ] = 0;	te[ 5 ] = y;	te[ 9 ] = b;	te[ 13 ] = 0;
		te[ 2 ] = 0;	te[ 6 ] = 0;	te[ 10 ] = c;	te[ 14 ] = d;
		te[ 3 ] = 0;	te[ 7 ] = 0;	te[ 11 ] = - 1;	te[ 15 ] = 0;

		return this;

	}

	makeOrthographic( left, right, top, bottom, near, far ) {

		const te = this.elements;
		const w = 1.0 / ( right - left );
		const h = 1.0 / ( top - bottom );
		const p = 1.0 / ( far - near );

		const x = ( right + left ) * w;
		const y = ( top + bottom ) * h;
		const z = ( far + near ) * p;

		te[ 0 ] = 2 * w;	te[ 4 ] = 0;	te[ 8 ] = 0;	te[ 12 ] = - x;
		te[ 1 ] = 0;	te[ 5 ] = 2 * h;	te[ 9 ] = 0;	te[ 13 ] = - y;
		te[ 2 ] = 0;	te[ 6 ] = 0;	te[ 10 ] = - 2 * p;	te[ 14 ] = - z;
		te[ 3 ] = 0;	te[ 7 ] = 0;	te[ 11 ] = 0;	te[ 15 ] = 1;

		return this;

	}

	equals( matrix ) {

		const te = this.elements;
		const me = matrix.elements;

		for ( let i = 0; i < 16; i ++ ) {

			if ( te[ i ] !== me[ i ] ) return false;

		}

		return true;

	}

	fromArray( array, offset = 0 ) {

		for ( let i = 0; i < 16; i ++ ) {

			this.elements[ i ] = array[ i + offset ];

		}

		return this;

	}

	toArray( array = [], offset = 0 ) {

		const te = this.elements;

		array[ offset ] = te[ 0 ];
		array[ offset + 1 ] = te[ 1 ];
		array[ offset + 2 ] = te[ 2 ];
		array[ offset + 3 ] = te[ 3 ];

		array[ offset + 4 ] = te[ 4 ];
		array[ offset + 5 ] = te[ 5 ];
		array[ offset + 6 ] = te[ 6 ];
		array[ offset + 7 ] = te[ 7 ];

		array[ offset + 8 ] = te[ 8 ];
		array[ offset + 9 ] = te[ 9 ];
		array[ offset + 10 ] = te[ 10 ];
		array[ offset + 11 ] = te[ 11 ];

		array[ offset + 12 ] = te[ 12 ];
		array[ offset + 13 ] = te[ 13 ];
		array[ offset + 14 ] = te[ 14 ];
		array[ offset + 15 ] = te[ 15 ];

		return array;

	}

}

Matrix4.prototype.isMatrix4 = true;

const _v1$5 = /*@__PURE__*/ new Vector3();
const _m1$2 = /*@__PURE__*/ new Matrix4();
const _zero = /*@__PURE__*/ new Vector3( 0, 0, 0 );
const _one = /*@__PURE__*/ new Vector3( 1, 1, 1 );
const _x = /*@__PURE__*/ new Vector3();
const _y = /*@__PURE__*/ new Vector3();
const _z = /*@__PURE__*/ new Vector3();

const _matrix$1 = /*@__PURE__*/ new Matrix4();
const _quaternion$3 = /*@__PURE__*/ new Quaternion();

class Euler {

	constructor( x = 0, y = 0, z = 0, order = Euler.DefaultOrder ) {

		this._x = x;
		this._y = y;
		this._z = z;
		this._order = order;

	}

	get x() {

		return this._x;

	}

	set x( value ) {

		this._x = value;
		this._onChangeCallback();

	}

	get y() {

		return this._y;

	}

	set y( value ) {

		this._y = value;
		this._onChangeCallback();

	}

	get z() {

		return this._z;

	}

	set z( value ) {

		this._z = value;
		this._onChangeCallback();

	}

	get order() {

		return this._order;

	}

	set order( value ) {

		this._order = value;
		this._onChangeCallback();

	}

	set( x, y, z, order ) {

		this._x = x;
		this._y = y;
		this._z = z;
		this._order = order || this._order;

		this._onChangeCallback();

		return this;

	}

	clone() {

		return new this.constructor( this._x, this._y, this._z, this._order );

	}

	copy( euler ) {

		this._x = euler._x;
		this._y = euler._y;
		this._z = euler._z;
		this._order = euler._order;

		this._onChangeCallback();

		return this;

	}

	setFromRotationMatrix( m, order, update ) {

		const clamp = MathUtils.clamp;

		// assumes the upper 3x3 of m is a pure rotation matrix (i.e, unscaled)

		const te = m.elements;
		const m11 = te[ 0 ], m12 = te[ 4 ], m13 = te[ 8 ];
		const m21 = te[ 1 ], m22 = te[ 5 ], m23 = te[ 9 ];
		const m31 = te[ 2 ], m32 = te[ 6 ], m33 = te[ 10 ];

		order = order || this._order;

		switch ( order ) {

			case 'XYZ':

				this._y = Math.asin( clamp( m13, - 1, 1 ) );

				if ( Math.abs( m13 ) < 0.9999999 ) {

					this._x = Math.atan2( - m23, m33 );
					this._z = Math.atan2( - m12, m11 );

				} else {

					this._x = Math.atan2( m32, m22 );
					this._z = 0;

				}

				break;

			case 'YXZ':

				this._x = Math.asin( - clamp( m23, - 1, 1 ) );

				if ( Math.abs( m23 ) < 0.9999999 ) {

					this._y = Math.atan2( m13, m33 );
					this._z = Math.atan2( m21, m22 );

				} else {

					this._y = Math.atan2( - m31, m11 );
					this._z = 0;

				}

				break;

			case 'ZXY':

				this._x = Math.asin( clamp( m32, - 1, 1 ) );

				if ( Math.abs( m32 ) < 0.9999999 ) {

					this._y = Math.atan2( - m31, m33 );
					this._z = Math.atan2( - m12, m22 );

				} else {

					this._y = 0;
					this._z = Math.atan2( m21, m11 );

				}

				break;

			case 'ZYX':

				this._y = Math.asin( - clamp( m31, - 1, 1 ) );

				if ( Math.abs( m31 ) < 0.9999999 ) {

					this._x = Math.atan2( m32, m33 );
					this._z = Math.atan2( m21, m11 );

				} else {

					this._x = 0;
					this._z = Math.atan2( - m12, m22 );

				}

				break;

			case 'YZX':

				this._z = Math.asin( clamp( m21, - 1, 1 ) );

				if ( Math.abs( m21 ) < 0.9999999 ) {

					this._x = Math.atan2( - m23, m22 );
					this._y = Math.atan2( - m31, m11 );

				} else {

					this._x = 0;
					this._y = Math.atan2( m13, m33 );

				}

				break;

			case 'XZY':

				this._z = Math.asin( - clamp( m12, - 1, 1 ) );

				if ( Math.abs( m12 ) < 0.9999999 ) {

					this._x = Math.atan2( m32, m22 );
					this._y = Math.atan2( m13, m11 );

				} else {

					this._x = Math.atan2( - m23, m33 );
					this._y = 0;

				}

				break;

			default:

				console.warn( 'THREE.Euler: .setFromRotationMatrix() encountered an unknown order: ' + order );

		}

		this._order = order;

		if ( update !== false ) this._onChangeCallback();

		return this;

	}

	setFromQuaternion( q, order, update ) {

		_matrix$1.makeRotationFromQuaternion( q );

		return this.setFromRotationMatrix( _matrix$1, order, update );

	}

	setFromVector3( v, order ) {

		return this.set( v.x, v.y, v.z, order || this._order );

	}

	reorder( newOrder ) {

		// WARNING: this discards revolution information -bhouston

		_quaternion$3.setFromEuler( this );

		return this.setFromQuaternion( _quaternion$3, newOrder );

	}

	equals( euler ) {

		return ( euler._x === this._x ) && ( euler._y === this._y ) && ( euler._z === this._z ) && ( euler._order === this._order );

	}

	fromArray( array ) {

		this._x = array[ 0 ];
		this._y = array[ 1 ];
		this._z = array[ 2 ];
		if ( array[ 3 ] !== undefined ) this._order = array[ 3 ];

		this._onChangeCallback();

		return this;

	}

	toArray( array = [], offset = 0 ) {

		array[ offset ] = this._x;
		array[ offset + 1 ] = this._y;
		array[ offset + 2 ] = this._z;
		array[ offset + 3 ] = this._order;

		return array;

	}

	toVector3( optionalResult ) {

		if ( optionalResult ) {

			return optionalResult.set( this._x, this._y, this._z );

		} else {

			return new Vector3( this._x, this._y, this._z );

		}

	}

	_onChange( callback ) {

		this._onChangeCallback = callback;

		return this;

	}

	_onChangeCallback() {}

}

Euler.prototype.isEuler = true;

Euler.DefaultOrder = 'XYZ';
Euler.RotationOrders = [ 'XYZ', 'YZX', 'ZXY', 'XZY', 'YXZ', 'ZYX' ];

class Layers {

	constructor() {

		this.mask = 1 | 0;

	}

	set( channel ) {

		this.mask = 1 << channel | 0;

	}

	enable( channel ) {

		this.mask |= 1 << channel | 0;

	}

	enableAll() {

		this.mask = 0xffffffff | 0;

	}

	toggle( channel ) {

		this.mask ^= 1 << channel | 0;

	}

	disable( channel ) {

		this.mask &= ~ ( 1 << channel | 0 );

	}

	disableAll() {

		this.mask = 0;

	}

	test( layers ) {

		return ( this.mask & layers.mask ) !== 0;

	}

}

let _object3DId = 0;

const _v1$4 = new Vector3();
const _q1 = new Quaternion();
const _m1$1 = new Matrix4();
const _target = new Vector3();

const _position$3 = new Vector3();
const _scale$2 = new Vector3();
const _quaternion$2 = new Quaternion();

const _xAxis = new Vector3( 1, 0, 0 );
const _yAxis = new Vector3( 0, 1, 0 );
const _zAxis = new Vector3( 0, 0, 1 );

const _addedEvent = { type: 'added' };
const _removedEvent = { type: 'removed' };

function Object3D() {

	Object.defineProperty( this, 'id', { value: _object3DId ++ } );

	this.uuid = MathUtils.generateUUID();

	this.name = '';
	this.type = 'Object3D';

	this.parent = null;
	this.children = [];

	this.up = Object3D.DefaultUp.clone();

	const position = new Vector3();
	const rotation = new Euler();
	const quaternion = new Quaternion();
	const scale = new Vector3( 1, 1, 1 );

	function onRotationChange() {

		quaternion.setFromEuler( rotation, false );

	}

	function onQuaternionChange() {

		rotation.setFromQuaternion( quaternion, undefined, false );

	}

	rotation._onChange( onRotationChange );
	quaternion._onChange( onQuaternionChange );

	Object.defineProperties( this, {
		position: {
			configurable: true,
			enumerable: true,
			value: position
		},
		rotation: {
			configurable: true,
			enumerable: true,
			value: rotation
		},
		quaternion: {
			configurable: true,
			enumerable: true,
			value: quaternion
		},
		scale: {
			configurable: true,
			enumerable: true,
			value: scale
		},
		modelViewMatrix: {
			value: new Matrix4()
		},
		normalMatrix: {
			value: new Matrix3()
		}
	} );

	this.matrix = new Matrix4();
	this.matrixWorld = new Matrix4();

	this.matrixAutoUpdate = Object3D.DefaultMatrixAutoUpdate;
	this.matrixWorldNeedsUpdate = false;

	this.layers = new Layers();
	this.visible = true;

	this.castShadow = false;
	this.receiveShadow = false;

	this.frustumCulled = true;
	this.renderOrder = 0;

	this.animations = [];

	this.userData = {};

}

Object3D.DefaultUp = new Vector3( 0, 1, 0 );
Object3D.DefaultMatrixAutoUpdate = true;

Object3D.prototype = Object.assign( Object.create( EventDispatcher.prototype ), {

	constructor: Object3D,

	isObject3D: true,

	onBeforeRender: function () {},
	onAfterRender: function () {},

	applyMatrix4: function ( matrix ) {

		if ( this.matrixAutoUpdate ) this.updateMatrix();

		this.matrix.premultiply( matrix );

		this.matrix.decompose( this.position, this.quaternion, this.scale );

	},

	applyQuaternion: function ( q ) {

		this.quaternion.premultiply( q );

		return this;

	},

	setRotationFromAxisAngle: function ( axis, angle ) {

		// assumes axis is normalized

		this.quaternion.setFromAxisAngle( axis, angle );

	},

	setRotationFromEuler: function ( euler ) {

		this.quaternion.setFromEuler( euler, true );

	},

	setRotationFromMatrix: function ( m ) {

		// assumes the upper 3x3 of m is a pure rotation matrix (i.e, unscaled)

		this.quaternion.setFromRotationMatrix( m );

	},

	setRotationFromQuaternion: function ( q ) {

		// assumes q is normalized

		this.quaternion.copy( q );

	},

	rotateOnAxis: function ( axis, angle ) {

		// rotate object on axis in object space
		// axis is assumed to be normalized

		_q1.setFromAxisAngle( axis, angle );

		this.quaternion.multiply( _q1 );

		return this;

	},

	rotateOnWorldAxis: function ( axis, angle ) {

		// rotate object on axis in world space
		// axis is assumed to be normalized
		// method assumes no rotated parent

		_q1.setFromAxisAngle( axis, angle );

		this.quaternion.premultiply( _q1 );

		return this;

	},

	rotateX: function ( angle ) {

		return this.rotateOnAxis( _xAxis, angle );

	},

	rotateY: function ( angle ) {

		return this.rotateOnAxis( _yAxis, angle );

	},

	rotateZ: function ( angle ) {

		return this.rotateOnAxis( _zAxis, angle );

	},

	translateOnAxis: function ( axis, distance ) {

		// translate object by distance along axis in object space
		// axis is assumed to be normalized

		_v1$4.copy( axis ).applyQuaternion( this.quaternion );

		this.position.add( _v1$4.multiplyScalar( distance ) );

		return this;

	},

	translateX: function ( distance ) {

		return this.translateOnAxis( _xAxis, distance );

	},

	translateY: function ( distance ) {

		return this.translateOnAxis( _yAxis, distance );

	},

	translateZ: function ( distance ) {

		return this.translateOnAxis( _zAxis, distance );

	},

	localToWorld: function ( vector ) {

		return vector.applyMatrix4( this.matrixWorld );

	},

	worldToLocal: function ( vector ) {

		return vector.applyMatrix4( _m1$1.copy( this.matrixWorld ).invert() );

	},

	lookAt: function ( x, y, z ) {

		// This method does not support objects having non-uniformly-scaled parent(s)

		if ( x.isVector3 ) {

			_target.copy( x );

		} else {

			_target.set( x, y, z );

		}

		const parent = this.parent;

		this.updateWorldMatrix( true, false );

		_position$3.setFromMatrixPosition( this.matrixWorld );

		if ( this.isCamera || this.isLight ) {

			_m1$1.lookAt( _position$3, _target, this.up );

		} else {

			_m1$1.lookAt( _target, _position$3, this.up );

		}

		this.quaternion.setFromRotationMatrix( _m1$1 );

		if ( parent ) {

			_m1$1.extractRotation( parent.matrixWorld );
			_q1.setFromRotationMatrix( _m1$1 );
			this.quaternion.premultiply( _q1.invert() );

		}

	},

	add: function ( object ) {

		if ( arguments.length > 1 ) {

			for ( let i = 0; i < arguments.length; i ++ ) {

				this.add( arguments[ i ] );

			}

			return this;

		}

		if ( object === this ) {

			console.error( 'THREE.Object3D.add: object can\'t be added as a child of itself.', object );
			return this;

		}

		if ( object && object.isObject3D ) {

			if ( object.parent !== null ) {

				object.parent.remove( object );

			}

			object.parent = this;
			this.children.push( object );

			object.dispatchEvent( _addedEvent );

		} else {

			console.error( 'THREE.Object3D.add: object not an instance of THREE.Object3D.', object );

		}

		return this;

	},

	remove: function ( object ) {

		if ( arguments.length > 1 ) {

			for ( let i = 0; i < arguments.length; i ++ ) {

				this.remove( arguments[ i ] );

			}

			return this;

		}

		const index = this.children.indexOf( object );

		if ( index !== - 1 ) {

			object.parent = null;
			this.children.splice( index, 1 );

			object.dispatchEvent( _removedEvent );

		}

		return this;

	},

	clear: function () {

		for ( let i = 0; i < this.children.length; i ++ ) {

			const object = this.children[ i ];

			object.parent = null;

			object.dispatchEvent( _removedEvent );

		}

		this.children.length = 0;

		return this;


	},

	attach: function ( object ) {

		// adds object as a child of this, while maintaining the object's world transform

		this.updateWorldMatrix( true, false );

		_m1$1.copy( this.matrixWorld ).invert();

		if ( object.parent !== null ) {

			object.parent.updateWorldMatrix( true, false );

			_m1$1.multiply( object.parent.matrixWorld );

		}

		object.applyMatrix4( _m1$1 );

		this.add( object );

		object.updateWorldMatrix( false, true );

		return this;

	},

	getObjectById: function ( id ) {

		return this.getObjectByProperty( 'id', id );

	},

	getObjectByName: function ( name ) {

		return this.getObjectByProperty( 'name', name );

	},

	getObjectByProperty: function ( name, value ) {

		if ( this[ name ] === value ) return this;

		for ( let i = 0, l = this.children.length; i < l; i ++ ) {

			const child = this.children[ i ];
			const object = child.getObjectByProperty( name, value );

			if ( object !== undefined ) {

				return object;

			}

		}

		return undefined;

	},

	getWorldPosition: function ( target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Object3D: .getWorldPosition() target is now required' );
			target = new Vector3();

		}

		this.updateWorldMatrix( true, false );

		return target.setFromMatrixPosition( this.matrixWorld );

	},

	getWorldQuaternion: function ( target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Object3D: .getWorldQuaternion() target is now required' );
			target = new Quaternion();

		}

		this.updateWorldMatrix( true, false );

		this.matrixWorld.decompose( _position$3, target, _scale$2 );

		return target;

	},

	getWorldScale: function ( target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Object3D: .getWorldScale() target is now required' );
			target = new Vector3();

		}

		this.updateWorldMatrix( true, false );

		this.matrixWorld.decompose( _position$3, _quaternion$2, target );

		return target;

	},

	getWorldDirection: function ( target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Object3D: .getWorldDirection() target is now required' );
			target = new Vector3();

		}

		this.updateWorldMatrix( true, false );

		const e = this.matrixWorld.elements;

		return target.set( e[ 8 ], e[ 9 ], e[ 10 ] ).normalize();

	},

	raycast: function () {},

	traverse: function ( callback ) {

		callback( this );

		const children = this.children;

		for ( let i = 0, l = children.length; i < l; i ++ ) {

			children[ i ].traverse( callback );

		}

	},

	traverseVisible: function ( callback ) {

		if ( this.visible === false ) return;

		callback( this );

		const children = this.children;

		for ( let i = 0, l = children.length; i < l; i ++ ) {

			children[ i ].traverseVisible( callback );

		}

	},

	traverseAncestors: function ( callback ) {

		const parent = this.parent;

		if ( parent !== null ) {

			callback( parent );

			parent.traverseAncestors( callback );

		}

	},

	updateMatrix: function () {

		this.matrix.compose( this.position, this.quaternion, this.scale );

		this.matrixWorldNeedsUpdate = true;

	},

	updateMatrixWorld: function ( force ) {

		if ( this.matrixAutoUpdate ) this.updateMatrix();

		if ( this.matrixWorldNeedsUpdate || force ) {

			if ( this.parent === null ) {

				this.matrixWorld.copy( this.matrix );

			} else {

				this.matrixWorld.multiplyMatrices( this.parent.matrixWorld, this.matrix );

			}

			this.matrixWorldNeedsUpdate = false;

			force = true;

		}

		// update children

		const children = this.children;

		for ( let i = 0, l = children.length; i < l; i ++ ) {

			children[ i ].updateMatrixWorld( force );

		}

	},

	updateWorldMatrix: function ( updateParents, updateChildren ) {

		const parent = this.parent;

		if ( updateParents === true && parent !== null ) {

			parent.updateWorldMatrix( true, false );

		}

		if ( this.matrixAutoUpdate ) this.updateMatrix();

		if ( this.parent === null ) {

			this.matrixWorld.copy( this.matrix );

		} else {

			this.matrixWorld.multiplyMatrices( this.parent.matrixWorld, this.matrix );

		}

		// update children

		if ( updateChildren === true ) {

			const children = this.children;

			for ( let i = 0, l = children.length; i < l; i ++ ) {

				children[ i ].updateWorldMatrix( false, true );

			}

		}

	},

	toJSON: function ( meta ) {

		// meta is a string when called from JSON.stringify
		const isRootObject = ( meta === undefined || typeof meta === 'string' );

		const output = {};

		// meta is a hash used to collect geometries, materials.
		// not providing it implies that this is the root object
		// being serialized.
		if ( isRootObject ) {

			// initialize meta obj
			meta = {
				geometries: {},
				materials: {},
				textures: {},
				images: {},
				shapes: {},
				skeletons: {},
				animations: {}
			};

			output.metadata = {
				version: 4.5,
				type: 'Object',
				generator: 'Object3D.toJSON'
			};

		}

		// standard Object3D serialization

		const object = {};

		object.uuid = this.uuid;
		object.type = this.type;

		if ( this.name !== '' ) object.name = this.name;
		if ( this.castShadow === true ) object.castShadow = true;
		if ( this.receiveShadow === true ) object.receiveShadow = true;
		if ( this.visible === false ) object.visible = false;
		if ( this.frustumCulled === false ) object.frustumCulled = false;
		if ( this.renderOrder !== 0 ) object.renderOrder = this.renderOrder;
		if ( JSON.stringify( this.userData ) !== '{}' ) object.userData = this.userData;

		object.layers = this.layers.mask;
		object.matrix = this.matrix.toArray();

		if ( this.matrixAutoUpdate === false ) object.matrixAutoUpdate = false;

		// object specific properties

		if ( this.isInstancedMesh ) {

			object.type = 'InstancedMesh';
			object.count = this.count;
			object.instanceMatrix = this.instanceMatrix.toJSON();
			if ( this.instanceColor !== null ) object.instanceColor = this.instanceColor.toJSON();

		}

		//

		function serialize( library, element ) {

			if ( library[ element.uuid ] === undefined ) {

				library[ element.uuid ] = element.toJSON( meta );

			}

			return element.uuid;

		}

		if ( this.isMesh || this.isLine || this.isPoints ) {

			object.geometry = serialize( meta.geometries, this.geometry );

			const parameters = this.geometry.parameters;

			if ( parameters !== undefined && parameters.shapes !== undefined ) {

				const shapes = parameters.shapes;

				if ( Array.isArray( shapes ) ) {

					for ( let i = 0, l = shapes.length; i < l; i ++ ) {

						const shape = shapes[ i ];

						serialize( meta.shapes, shape );

					}

				} else {

					serialize( meta.shapes, shapes );

				}

			}

		}

		if ( this.isSkinnedMesh ) {

			object.bindMode = this.bindMode;
			object.bindMatrix = this.bindMatrix.toArray();

			if ( this.skeleton !== undefined ) {

				serialize( meta.skeletons, this.skeleton );

				object.skeleton = this.skeleton.uuid;

			}

		}

		if ( this.material !== undefined ) {

			if ( Array.isArray( this.material ) ) {

				const uuids = [];

				for ( let i = 0, l = this.material.length; i < l; i ++ ) {

					uuids.push( serialize( meta.materials, this.material[ i ] ) );

				}

				object.material = uuids;

			} else {

				object.material = serialize( meta.materials, this.material );

			}

		}

		//

		if ( this.children.length > 0 ) {

			object.children = [];

			for ( let i = 0; i < this.children.length; i ++ ) {

				object.children.push( this.children[ i ].toJSON( meta ).object );

			}

		}

		//

		if ( this.animations.length > 0 ) {

			object.animations = [];

			for ( let i = 0; i < this.animations.length; i ++ ) {

				const animation = this.animations[ i ];

				object.animations.push( serialize( meta.animations, animation ) );

			}

		}

		if ( isRootObject ) {

			const geometries = extractFromCache( meta.geometries );
			const materials = extractFromCache( meta.materials );
			const textures = extractFromCache( meta.textures );
			const images = extractFromCache( meta.images );
			const shapes = extractFromCache( meta.shapes );
			const skeletons = extractFromCache( meta.skeletons );
			const animations = extractFromCache( meta.animations );

			if ( geometries.length > 0 ) output.geometries = geometries;
			if ( materials.length > 0 ) output.materials = materials;
			if ( textures.length > 0 ) output.textures = textures;
			if ( images.length > 0 ) output.images = images;
			if ( shapes.length > 0 ) output.shapes = shapes;
			if ( skeletons.length > 0 ) output.skeletons = skeletons;
			if ( animations.length > 0 ) output.animations = animations;

		}

		output.object = object;

		return output;

		// extract data from the cache hash
		// remove metadata on each item
		// and return as array
		function extractFromCache( cache ) {

			const values = [];
			for ( const key in cache ) {

				const data = cache[ key ];
				delete data.metadata;
				values.push( data );

			}

			return values;

		}

	},

	clone: function ( recursive ) {

		return new this.constructor().copy( this, recursive );

	},

	copy: function ( source, recursive = true ) {

		this.name = source.name;

		this.up.copy( source.up );

		this.position.copy( source.position );
		this.rotation.order = source.rotation.order;
		this.quaternion.copy( source.quaternion );
		this.scale.copy( source.scale );

		this.matrix.copy( source.matrix );
		this.matrixWorld.copy( source.matrixWorld );

		this.matrixAutoUpdate = source.matrixAutoUpdate;
		this.matrixWorldNeedsUpdate = source.matrixWorldNeedsUpdate;

		this.layers.mask = source.layers.mask;
		this.visible = source.visible;

		this.castShadow = source.castShadow;
		this.receiveShadow = source.receiveShadow;

		this.frustumCulled = source.frustumCulled;
		this.renderOrder = source.renderOrder;

		this.userData = JSON.parse( JSON.stringify( source.userData ) );

		if ( recursive === true ) {

			for ( let i = 0; i < source.children.length; i ++ ) {

				const child = source.children[ i ];
				this.add( child.clone() );

			}

		}

		return this;

	}

} );

const _vector1 = /*@__PURE__*/ new Vector3();
const _vector2$1 = /*@__PURE__*/ new Vector3();
const _normalMatrix = /*@__PURE__*/ new Matrix3();

class Plane {

	constructor( normal = new Vector3( 1, 0, 0 ), constant = 0 ) {

		// normal is assumed to be normalized

		this.normal = normal;
		this.constant = constant;

	}

	set( normal, constant ) {

		this.normal.copy( normal );
		this.constant = constant;

		return this;

	}

	setComponents( x, y, z, w ) {

		this.normal.set( x, y, z );
		this.constant = w;

		return this;

	}

	setFromNormalAndCoplanarPoint( normal, point ) {

		this.normal.copy( normal );
		this.constant = - point.dot( this.normal );

		return this;

	}

	setFromCoplanarPoints( a, b, c ) {

		const normal = _vector1.subVectors( c, b ).cross( _vector2$1.subVectors( a, b ) ).normalize();

		// Q: should an error be thrown if normal is zero (e.g. degenerate plane)?

		this.setFromNormalAndCoplanarPoint( normal, a );

		return this;

	}

	copy( plane ) {

		this.normal.copy( plane.normal );
		this.constant = plane.constant;

		return this;

	}

	normalize() {

		// Note: will lead to a divide by zero if the plane is invalid.

		const inverseNormalLength = 1.0 / this.normal.length();
		this.normal.multiplyScalar( inverseNormalLength );
		this.constant *= inverseNormalLength;

		return this;

	}

	negate() {

		this.constant *= - 1;
		this.normal.negate();

		return this;

	}

	distanceToPoint( point ) {

		return this.normal.dot( point ) + this.constant;

	}

	distanceToSphere( sphere ) {

		return this.distanceToPoint( sphere.center ) - sphere.radius;

	}

	projectPoint( point, target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Plane: .projectPoint() target is now required' );
			target = new Vector3();

		}

		return target.copy( this.normal ).multiplyScalar( - this.distanceToPoint( point ) ).add( point );

	}

	intersectLine( line, target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Plane: .intersectLine() target is now required' );
			target = new Vector3();

		}

		const direction = line.delta( _vector1 );

		const denominator = this.normal.dot( direction );

		if ( denominator === 0 ) {

			// line is coplanar, return origin
			if ( this.distanceToPoint( line.start ) === 0 ) {

				return target.copy( line.start );

			}

			// Unsure if this is the correct method to handle this case.
			return null;

		}

		const t = - ( line.start.dot( this.normal ) + this.constant ) / denominator;

		if ( t < 0 || t > 1 ) {

			return null;

		}

		return target.copy( direction ).multiplyScalar( t ).add( line.start );

	}

	intersectsLine( line ) {

		// Note: this tests if a line intersects the plane, not whether it (or its end-points) are coplanar with it.

		const startSign = this.distanceToPoint( line.start );
		const endSign = this.distanceToPoint( line.end );

		return ( startSign < 0 && endSign > 0 ) || ( endSign < 0 && startSign > 0 );

	}

	intersectsBox( box ) {

		return box.intersectsPlane( this );

	}

	intersectsSphere( sphere ) {

		return sphere.intersectsPlane( this );

	}

	coplanarPoint( target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Plane: .coplanarPoint() target is now required' );
			target = new Vector3();

		}

		return target.copy( this.normal ).multiplyScalar( - this.constant );

	}

	applyMatrix4( matrix, optionalNormalMatrix ) {

		const normalMatrix = optionalNormalMatrix || _normalMatrix.getNormalMatrix( matrix );

		const referencePoint = this.coplanarPoint( _vector1 ).applyMatrix4( matrix );

		const normal = this.normal.applyMatrix3( normalMatrix ).normalize();

		this.constant = - referencePoint.dot( normal );

		return this;

	}

	translate( offset ) {

		this.constant -= offset.dot( this.normal );

		return this;

	}

	equals( plane ) {

		return plane.normal.equals( this.normal ) && ( plane.constant === this.constant );

	}

	clone() {

		return new this.constructor().copy( this );

	}

}

Plane.prototype.isPlane = true;

const _v0$1 = /*@__PURE__*/ new Vector3();
const _v1$3 = /*@__PURE__*/ new Vector3();
const _v2$2 = /*@__PURE__*/ new Vector3();
const _v3$1 = /*@__PURE__*/ new Vector3();

const _vab = /*@__PURE__*/ new Vector3();
const _vac = /*@__PURE__*/ new Vector3();
const _vbc = /*@__PURE__*/ new Vector3();
const _vap = /*@__PURE__*/ new Vector3();
const _vbp = /*@__PURE__*/ new Vector3();
const _vcp = /*@__PURE__*/ new Vector3();

class Triangle {

	constructor( a = new Vector3(), b = new Vector3(), c = new Vector3() ) {

		this.a = a;
		this.b = b;
		this.c = c;

	}

	static getNormal( a, b, c, target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Triangle: .getNormal() target is now required' );
			target = new Vector3();

		}

		target.subVectors( c, b );
		_v0$1.subVectors( a, b );
		target.cross( _v0$1 );

		const targetLengthSq = target.lengthSq();
		if ( targetLengthSq > 0 ) {

			return target.multiplyScalar( 1 / Math.sqrt( targetLengthSq ) );

		}

		return target.set( 0, 0, 0 );

	}

	// static/instance method to calculate barycentric coordinates
	// based on: http://www.blackpawn.com/texts/pointinpoly/default.html
	static getBarycoord( point, a, b, c, target ) {

		_v0$1.subVectors( c, a );
		_v1$3.subVectors( b, a );
		_v2$2.subVectors( point, a );

		const dot00 = _v0$1.dot( _v0$1 );
		const dot01 = _v0$1.dot( _v1$3 );
		const dot02 = _v0$1.dot( _v2$2 );
		const dot11 = _v1$3.dot( _v1$3 );
		const dot12 = _v1$3.dot( _v2$2 );

		const denom = ( dot00 * dot11 - dot01 * dot01 );

		if ( target === undefined ) {

			console.warn( 'THREE.Triangle: .getBarycoord() target is now required' );
			target = new Vector3();

		}

		// collinear or singular triangle
		if ( denom === 0 ) {

			// arbitrary location outside of triangle?
			// not sure if this is the best idea, maybe should be returning undefined
			return target.set( - 2, - 1, - 1 );

		}

		const invDenom = 1 / denom;
		const u = ( dot11 * dot02 - dot01 * dot12 ) * invDenom;
		const v = ( dot00 * dot12 - dot01 * dot02 ) * invDenom;

		// barycentric coordinates must always sum to 1
		return target.set( 1 - u - v, v, u );

	}

	static containsPoint( point, a, b, c ) {

		this.getBarycoord( point, a, b, c, _v3$1 );

		return ( _v3$1.x >= 0 ) && ( _v3$1.y >= 0 ) && ( ( _v3$1.x + _v3$1.y ) <= 1 );

	}

	static getUV( point, p1, p2, p3, uv1, uv2, uv3, target ) {

		this.getBarycoord( point, p1, p2, p3, _v3$1 );

		target.set( 0, 0 );
		target.addScaledVector( uv1, _v3$1.x );
		target.addScaledVector( uv2, _v3$1.y );
		target.addScaledVector( uv3, _v3$1.z );

		return target;

	}

	static isFrontFacing( a, b, c, direction ) {

		_v0$1.subVectors( c, b );
		_v1$3.subVectors( a, b );

		// strictly front facing
		return ( _v0$1.cross( _v1$3 ).dot( direction ) < 0 ) ? true : false;

	}

	set( a, b, c ) {

		this.a.copy( a );
		this.b.copy( b );
		this.c.copy( c );

		return this;

	}

	setFromPointsAndIndices( points, i0, i1, i2 ) {

		this.a.copy( points[ i0 ] );
		this.b.copy( points[ i1 ] );
		this.c.copy( points[ i2 ] );

		return this;

	}

	clone() {

		return new this.constructor().copy( this );

	}

	copy( triangle ) {

		this.a.copy( triangle.a );
		this.b.copy( triangle.b );
		this.c.copy( triangle.c );

		return this;

	}

	getArea() {

		_v0$1.subVectors( this.c, this.b );
		_v1$3.subVectors( this.a, this.b );

		return _v0$1.cross( _v1$3 ).length() * 0.5;

	}

	getMidpoint( target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Triangle: .getMidpoint() target is now required' );
			target = new Vector3();

		}

		return target.addVectors( this.a, this.b ).add( this.c ).multiplyScalar( 1 / 3 );

	}

	getNormal( target ) {

		return Triangle.getNormal( this.a, this.b, this.c, target );

	}

	getPlane( target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Triangle: .getPlane() target is now required' );
			target = new Plane();

		}

		return target.setFromCoplanarPoints( this.a, this.b, this.c );

	}

	getBarycoord( point, target ) {

		return Triangle.getBarycoord( point, this.a, this.b, this.c, target );

	}

	getUV( point, uv1, uv2, uv3, target ) {

		return Triangle.getUV( point, this.a, this.b, this.c, uv1, uv2, uv3, target );

	}

	containsPoint( point ) {

		return Triangle.containsPoint( point, this.a, this.b, this.c );

	}

	isFrontFacing( direction ) {

		return Triangle.isFrontFacing( this.a, this.b, this.c, direction );

	}

	intersectsBox( box ) {

		return box.intersectsTriangle( this );

	}

	closestPointToPoint( p, target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Triangle: .closestPointToPoint() target is now required' );
			target = new Vector3();

		}

		const a = this.a, b = this.b, c = this.c;
		let v, w;

		// algorithm thanks to Real-Time Collision Detection by Christer Ericson,
		// published by Morgan Kaufmann Publishers, (c) 2005 Elsevier Inc.,
		// under the accompanying license; see chapter 5.1.5 for detailed explanation.
		// basically, we're distinguishing which of the voronoi regions of the triangle
		// the point lies in with the minimum amount of redundant computation.

		_vab.subVectors( b, a );
		_vac.subVectors( c, a );
		_vap.subVectors( p, a );
		const d1 = _vab.dot( _vap );
		const d2 = _vac.dot( _vap );
		if ( d1 <= 0 && d2 <= 0 ) {

			// vertex region of A; barycentric coords (1, 0, 0)
			return target.copy( a );

		}

		_vbp.subVectors( p, b );
		const d3 = _vab.dot( _vbp );
		const d4 = _vac.dot( _vbp );
		if ( d3 >= 0 && d4 <= d3 ) {

			// vertex region of B; barycentric coords (0, 1, 0)
			return target.copy( b );

		}

		const vc = d1 * d4 - d3 * d2;
		if ( vc <= 0 && d1 >= 0 && d3 <= 0 ) {

			v = d1 / ( d1 - d3 );
			// edge region of AB; barycentric coords (1-v, v, 0)
			return target.copy( a ).addScaledVector( _vab, v );

		}

		_vcp.subVectors( p, c );
		const d5 = _vab.dot( _vcp );
		const d6 = _vac.dot( _vcp );
		if ( d6 >= 0 && d5 <= d6 ) {

			// vertex region of C; barycentric coords (0, 0, 1)
			return target.copy( c );

		}

		const vb = d5 * d2 - d1 * d6;
		if ( vb <= 0 && d2 >= 0 && d6 <= 0 ) {

			w = d2 / ( d2 - d6 );
			// edge region of AC; barycentric coords (1-w, 0, w)
			return target.copy( a ).addScaledVector( _vac, w );

		}

		const va = d3 * d6 - d5 * d4;
		if ( va <= 0 && ( d4 - d3 ) >= 0 && ( d5 - d6 ) >= 0 ) {

			_vbc.subVectors( c, b );
			w = ( d4 - d3 ) / ( ( d4 - d3 ) + ( d5 - d6 ) );
			// edge region of BC; barycentric coords (0, 1-w, w)
			return target.copy( b ).addScaledVector( _vbc, w ); // edge region of BC

		}

		// face region
		const denom = 1 / ( va + vb + vc );
		// u = va * denom
		v = vb * denom;
		w = vc * denom;

		return target.copy( a ).addScaledVector( _vab, v ).addScaledVector( _vac, w );

	}

	equals( triangle ) {

		return triangle.a.equals( this.a ) && triangle.b.equals( this.b ) && triangle.c.equals( this.c );

	}

}

let materialId = 0;

function Material() {

	Object.defineProperty( this, 'id', { value: materialId ++ } );

	this.uuid = MathUtils.generateUUID();

	this.name = '';
	this.type = 'Material';

	this.fog = true;

	this.blending = NormalBlending;
	this.side = FrontSide;
	this.vertexColors = false;

	this.opacity = 1;
	this.transparent = false;

	this.blendSrc = SrcAlphaFactor;
	this.blendDst = OneMinusSrcAlphaFactor;
	this.blendEquation = AddEquation;
	this.blendSrcAlpha = null;
	this.blendDstAlpha = null;
	this.blendEquationAlpha = null;

	this.depthFunc = LessEqualDepth;
	this.depthTest = true;
	this.depthWrite = true;

	this.stencilWriteMask = 0xff;
	this.stencilFunc = AlwaysStencilFunc;
	this.stencilRef = 0;
	this.stencilFuncMask = 0xff;
	this.stencilFail = KeepStencilOp;
	this.stencilZFail = KeepStencilOp;
	this.stencilZPass = KeepStencilOp;
	this.stencilWrite = false;

	this.clippingPlanes = null;
	this.clipIntersection = false;
	this.clipShadows = false;

	this.shadowSide = null;

	this.colorWrite = true;

	this.precision = null; // override the renderer's default precision for this material

	this.polygonOffset = false;
	this.polygonOffsetFactor = 0;
	this.polygonOffsetUnits = 0;

	this.dithering = false;

	this.alphaTest = 0;
	this.alphaToCoverage = false;
	this.premultipliedAlpha = false;

	this.visible = true;

	this.toneMapped = true;

	this.userData = {};

	this.version = 0;

}

Material.prototype = Object.assign( Object.create( EventDispatcher.prototype ), {

	constructor: Material,

	isMaterial: true,

	onBeforeCompile: function ( /* shaderobject, renderer */ ) {},

	customProgramCacheKey: function () {

		return this.onBeforeCompile.toString();

	},

	setValues: function ( values ) {

		if ( values === undefined ) return;

		for ( const key in values ) {

			const newValue = values[ key ];

			if ( newValue === undefined ) {

				console.warn( 'THREE.Material: \'' + key + '\' parameter is undefined.' );
				continue;

			}

			// for backward compatability if shading is set in the constructor
			if ( key === 'shading' ) {

				console.warn( 'THREE.' + this.type + ': .shading has been removed. Use the boolean .flatShading instead.' );
				this.flatShading = ( newValue === FlatShading ) ? true : false;
				continue;

			}

			const currentValue = this[ key ];

			if ( currentValue === undefined ) {

				console.warn( 'THREE.' + this.type + ': \'' + key + '\' is not a property of this material.' );
				continue;

			}

			if ( currentValue && currentValue.isColor ) {

				currentValue.set( newValue );

			} else if ( ( currentValue && currentValue.isVector3 ) && ( newValue && newValue.isVector3 ) ) {

				currentValue.copy( newValue );

			} else {

				this[ key ] = newValue;

			}

		}

	},

	toJSON: function ( meta ) {

		const isRoot = ( meta === undefined || typeof meta === 'string' );

		if ( isRoot ) {

			meta = {
				textures: {},
				images: {}
			};

		}

		const data = {
			metadata: {
				version: 4.5,
				type: 'Material',
				generator: 'Material.toJSON'
			}
		};

		// standard Material serialization
		data.uuid = this.uuid;
		data.type = this.type;

		if ( this.name !== '' ) data.name = this.name;

		if ( this.color && this.color.isColor ) data.color = this.color.getHex();

		if ( this.roughness !== undefined ) data.roughness = this.roughness;
		if ( this.metalness !== undefined ) data.metalness = this.metalness;

		if ( this.sheen && this.sheen.isColor ) data.sheen = this.sheen.getHex();
		if ( this.emissive && this.emissive.isColor ) data.emissive = this.emissive.getHex();
		if ( this.emissiveIntensity && this.emissiveIntensity !== 1 ) data.emissiveIntensity = this.emissiveIntensity;

		if ( this.specular && this.specular.isColor ) data.specular = this.specular.getHex();
		if ( this.shininess !== undefined ) data.shininess = this.shininess;
		if ( this.clearcoat !== undefined ) data.clearcoat = this.clearcoat;
		if ( this.clearcoatRoughness !== undefined ) data.clearcoatRoughness = this.clearcoatRoughness;

		if ( this.clearcoatMap && this.clearcoatMap.isTexture ) {

			data.clearcoatMap = this.clearcoatMap.toJSON( meta ).uuid;

		}

		if ( this.clearcoatRoughnessMap && this.clearcoatRoughnessMap.isTexture ) {

			data.clearcoatRoughnessMap = this.clearcoatRoughnessMap.toJSON( meta ).uuid;

		}

		if ( this.clearcoatNormalMap && this.clearcoatNormalMap.isTexture ) {

			data.clearcoatNormalMap = this.clearcoatNormalMap.toJSON( meta ).uuid;
			data.clearcoatNormalScale = this.clearcoatNormalScale.toArray();

		}

		if ( this.map && this.map.isTexture ) data.map = this.map.toJSON( meta ).uuid;
		if ( this.matcap && this.matcap.isTexture ) data.matcap = this.matcap.toJSON( meta ).uuid;
		if ( this.alphaMap && this.alphaMap.isTexture ) data.alphaMap = this.alphaMap.toJSON( meta ).uuid;

		if ( this.lightMap && this.lightMap.isTexture ) {

			data.lightMap = this.lightMap.toJSON( meta ).uuid;
			data.lightMapIntensity = this.lightMapIntensity;

		}

		if ( this.aoMap && this.aoMap.isTexture ) {

			data.aoMap = this.aoMap.toJSON( meta ).uuid;
			data.aoMapIntensity = this.aoMapIntensity;

		}

		if ( this.bumpMap && this.bumpMap.isTexture ) {

			data.bumpMap = this.bumpMap.toJSON( meta ).uuid;
			data.bumpScale = this.bumpScale;

		}

		if ( this.normalMap && this.normalMap.isTexture ) {

			data.normalMap = this.normalMap.toJSON( meta ).uuid;
			data.normalMapType = this.normalMapType;
			data.normalScale = this.normalScale.toArray();

		}

		if ( this.displacementMap && this.displacementMap.isTexture ) {

			data.displacementMap = this.displacementMap.toJSON( meta ).uuid;
			data.displacementScale = this.displacementScale;
			data.displacementBias = this.displacementBias;

		}

		if ( this.roughnessMap && this.roughnessMap.isTexture ) data.roughnessMap = this.roughnessMap.toJSON( meta ).uuid;
		if ( this.metalnessMap && this.metalnessMap.isTexture ) data.metalnessMap = this.metalnessMap.toJSON( meta ).uuid;

		if ( this.emissiveMap && this.emissiveMap.isTexture ) data.emissiveMap = this.emissiveMap.toJSON( meta ).uuid;
		if ( this.specularMap && this.specularMap.isTexture ) data.specularMap = this.specularMap.toJSON( meta ).uuid;

		if ( this.envMap && this.envMap.isTexture ) {

			data.envMap = this.envMap.toJSON( meta ).uuid;
			data.reflectivity = this.reflectivity; // Scale behind envMap
			data.refractionRatio = this.refractionRatio;

			if ( this.combine !== undefined ) data.combine = this.combine;
			if ( this.envMapIntensity !== undefined ) data.envMapIntensity = this.envMapIntensity;

		}

		if ( this.gradientMap && this.gradientMap.isTexture ) {

			data.gradientMap = this.gradientMap.toJSON( meta ).uuid;

		}

		if ( this.size !== undefined ) data.size = this.size;
		if ( this.shadowSide !== null ) data.shadowSide = this.shadowSide;
		if ( this.sizeAttenuation !== undefined ) data.sizeAttenuation = this.sizeAttenuation;

		if ( this.blending !== NormalBlending ) data.blending = this.blending;
		if ( this.side !== FrontSide ) data.side = this.side;
		if ( this.vertexColors ) data.vertexColors = true;

		if ( this.opacity < 1 ) data.opacity = this.opacity;
		if ( this.transparent === true ) data.transparent = this.transparent;

		data.depthFunc = this.depthFunc;
		data.depthTest = this.depthTest;
		data.depthWrite = this.depthWrite;
		data.colorWrite = this.colorWrite;

		data.stencilWrite = this.stencilWrite;
		data.stencilWriteMask = this.stencilWriteMask;
		data.stencilFunc = this.stencilFunc;
		data.stencilRef = this.stencilRef;
		data.stencilFuncMask = this.stencilFuncMask;
		data.stencilFail = this.stencilFail;
		data.stencilZFail = this.stencilZFail;
		data.stencilZPass = this.stencilZPass;

		// rotation (SpriteMaterial)
		if ( this.rotation && this.rotation !== 0 ) data.rotation = this.rotation;

		if ( this.polygonOffset === true ) data.polygonOffset = true;
		if ( this.polygonOffsetFactor !== 0 ) data.polygonOffsetFactor = this.polygonOffsetFactor;
		if ( this.polygonOffsetUnits !== 0 ) data.polygonOffsetUnits = this.polygonOffsetUnits;

		if ( this.linewidth && this.linewidth !== 1 ) data.linewidth = this.linewidth;
		if ( this.dashSize !== undefined ) data.dashSize = this.dashSize;
		if ( this.gapSize !== undefined ) data.gapSize = this.gapSize;
		if ( this.scale !== undefined ) data.scale = this.scale;

		if ( this.dithering === true ) data.dithering = true;

		if ( this.alphaTest > 0 ) data.alphaTest = this.alphaTest;
		if ( this.alphaToCoverage === true ) data.alphaToCoverage = this.alphaToCoverage;
		if ( this.premultipliedAlpha === true ) data.premultipliedAlpha = this.premultipliedAlpha;

		if ( this.wireframe === true ) data.wireframe = this.wireframe;
		if ( this.wireframeLinewidth > 1 ) data.wireframeLinewidth = this.wireframeLinewidth;
		if ( this.wireframeLinecap !== 'round' ) data.wireframeLinecap = this.wireframeLinecap;
		if ( this.wireframeLinejoin !== 'round' ) data.wireframeLinejoin = this.wireframeLinejoin;

		if ( this.morphTargets === true ) data.morphTargets = true;
		if ( this.morphNormals === true ) data.morphNormals = true;
		if ( this.skinning === true ) data.skinning = true;

		if ( this.flatShading === true ) data.flatShading = this.flatShading;

		if ( this.visible === false ) data.visible = false;

		if ( this.toneMapped === false ) data.toneMapped = false;

		if ( JSON.stringify( this.userData ) !== '{}' ) data.userData = this.userData;

		// TODO: Copied from Object3D.toJSON

		function extractFromCache( cache ) {

			const values = [];

			for ( const key in cache ) {

				const data = cache[ key ];
				delete data.metadata;
				values.push( data );

			}

			return values;

		}

		if ( isRoot ) {

			const textures = extractFromCache( meta.textures );
			const images = extractFromCache( meta.images );

			if ( textures.length > 0 ) data.textures = textures;
			if ( images.length > 0 ) data.images = images;

		}

		return data;

	},

	clone: function () {

		return new this.constructor().copy( this );

	},

	copy: function ( source ) {

		this.name = source.name;

		this.fog = source.fog;

		this.blending = source.blending;
		this.side = source.side;
		this.vertexColors = source.vertexColors;

		this.opacity = source.opacity;
		this.transparent = source.transparent;

		this.blendSrc = source.blendSrc;
		this.blendDst = source.blendDst;
		this.blendEquation = source.blendEquation;
		this.blendSrcAlpha = source.blendSrcAlpha;
		this.blendDstAlpha = source.blendDstAlpha;
		this.blendEquationAlpha = source.blendEquationAlpha;

		this.depthFunc = source.depthFunc;
		this.depthTest = source.depthTest;
		this.depthWrite = source.depthWrite;

		this.stencilWriteMask = source.stencilWriteMask;
		this.stencilFunc = source.stencilFunc;
		this.stencilRef = source.stencilRef;
		this.stencilFuncMask = source.stencilFuncMask;
		this.stencilFail = source.stencilFail;
		this.stencilZFail = source.stencilZFail;
		this.stencilZPass = source.stencilZPass;
		this.stencilWrite = source.stencilWrite;

		const srcPlanes = source.clippingPlanes;
		let dstPlanes = null;

		if ( srcPlanes !== null ) {

			const n = srcPlanes.length;
			dstPlanes = new Array( n );

			for ( let i = 0; i !== n; ++ i ) {

				dstPlanes[ i ] = srcPlanes[ i ].clone();

			}

		}

		this.clippingPlanes = dstPlanes;
		this.clipIntersection = source.clipIntersection;
		this.clipShadows = source.clipShadows;

		this.shadowSide = source.shadowSide;

		this.colorWrite = source.colorWrite;

		this.precision = source.precision;

		this.polygonOffset = source.polygonOffset;
		this.polygonOffsetFactor = source.polygonOffsetFactor;
		this.polygonOffsetUnits = source.polygonOffsetUnits;

		this.dithering = source.dithering;

		this.alphaTest = source.alphaTest;
		this.alphaToCoverage = source.alphaToCoverage;
		this.premultipliedAlpha = source.premultipliedAlpha;

		this.visible = source.visible;

		this.toneMapped = source.toneMapped;

		this.userData = JSON.parse( JSON.stringify( source.userData ) );

		return this;

	},

	dispose: function () {

		this.dispatchEvent( { type: 'dispose' } );

	}

} );

Object.defineProperty( Material.prototype, 'needsUpdate', {

	set: function ( value ) {

		if ( value === true ) this.version ++;

	}

} );

const _colorKeywords = { 'aliceblue': 0xF0F8FF, 'antiquewhite': 0xFAEBD7, 'aqua': 0x00FFFF, 'aquamarine': 0x7FFFD4, 'azure': 0xF0FFFF,
	'beige': 0xF5F5DC, 'bisque': 0xFFE4C4, 'black': 0x000000, 'blanchedalmond': 0xFFEBCD, 'blue': 0x0000FF, 'blueviolet': 0x8A2BE2,
	'brown': 0xA52A2A, 'burlywood': 0xDEB887, 'cadetblue': 0x5F9EA0, 'chartreuse': 0x7FFF00, 'chocolate': 0xD2691E, 'coral': 0xFF7F50,
	'cornflowerblue': 0x6495ED, 'cornsilk': 0xFFF8DC, 'crimson': 0xDC143C, 'cyan': 0x00FFFF, 'darkblue': 0x00008B, 'darkcyan': 0x008B8B,
	'darkgoldenrod': 0xB8860B, 'darkgray': 0xA9A9A9, 'darkgreen': 0x006400, 'darkgrey': 0xA9A9A9, 'darkkhaki': 0xBDB76B, 'darkmagenta': 0x8B008B,
	'darkolivegreen': 0x556B2F, 'darkorange': 0xFF8C00, 'darkorchid': 0x9932CC, 'darkred': 0x8B0000, 'darksalmon': 0xE9967A, 'darkseagreen': 0x8FBC8F,
	'darkslateblue': 0x483D8B, 'darkslategray': 0x2F4F4F, 'darkslategrey': 0x2F4F4F, 'darkturquoise': 0x00CED1, 'darkviolet': 0x9400D3,
	'deeppink': 0xFF1493, 'deepskyblue': 0x00BFFF, 'dimgray': 0x696969, 'dimgrey': 0x696969, 'dodgerblue': 0x1E90FF, 'firebrick': 0xB22222,
	'floralwhite': 0xFFFAF0, 'forestgreen': 0x228B22, 'fuchsia': 0xFF00FF, 'gainsboro': 0xDCDCDC, 'ghostwhite': 0xF8F8FF, 'gold': 0xFFD700,
	'goldenrod': 0xDAA520, 'gray': 0x808080, 'green': 0x008000, 'greenyellow': 0xADFF2F, 'grey': 0x808080, 'honeydew': 0xF0FFF0, 'hotpink': 0xFF69B4,
	'indianred': 0xCD5C5C, 'indigo': 0x4B0082, 'ivory': 0xFFFFF0, 'khaki': 0xF0E68C, 'lavender': 0xE6E6FA, 'lavenderblush': 0xFFF0F5, 'lawngreen': 0x7CFC00,
	'lemonchiffon': 0xFFFACD, 'lightblue': 0xADD8E6, 'lightcoral': 0xF08080, 'lightcyan': 0xE0FFFF, 'lightgoldenrodyellow': 0xFAFAD2, 'lightgray': 0xD3D3D3,
	'lightgreen': 0x90EE90, 'lightgrey': 0xD3D3D3, 'lightpink': 0xFFB6C1, 'lightsalmon': 0xFFA07A, 'lightseagreen': 0x20B2AA, 'lightskyblue': 0x87CEFA,
	'lightslategray': 0x778899, 'lightslategrey': 0x778899, 'lightsteelblue': 0xB0C4DE, 'lightyellow': 0xFFFFE0, 'lime': 0x00FF00, 'limegreen': 0x32CD32,
	'linen': 0xFAF0E6, 'magenta': 0xFF00FF, 'maroon': 0x800000, 'mediumaquamarine': 0x66CDAA, 'mediumblue': 0x0000CD, 'mediumorchid': 0xBA55D3,
	'mediumpurple': 0x9370DB, 'mediumseagreen': 0x3CB371, 'mediumslateblue': 0x7B68EE, 'mediumspringgreen': 0x00FA9A, 'mediumturquoise': 0x48D1CC,
	'mediumvioletred': 0xC71585, 'midnightblue': 0x191970, 'mintcream': 0xF5FFFA, 'mistyrose': 0xFFE4E1, 'moccasin': 0xFFE4B5, 'navajowhite': 0xFFDEAD,
	'navy': 0x000080, 'oldlace': 0xFDF5E6, 'olive': 0x808000, 'olivedrab': 0x6B8E23, 'orange': 0xFFA500, 'orangered': 0xFF4500, 'orchid': 0xDA70D6,
	'palegoldenrod': 0xEEE8AA, 'palegreen': 0x98FB98, 'paleturquoise': 0xAFEEEE, 'palevioletred': 0xDB7093, 'papayawhip': 0xFFEFD5, 'peachpuff': 0xFFDAB9,
	'peru': 0xCD853F, 'pink': 0xFFC0CB, 'plum': 0xDDA0DD, 'powderblue': 0xB0E0E6, 'purple': 0x800080, 'rebeccapurple': 0x663399, 'red': 0xFF0000, 'rosybrown': 0xBC8F8F,
	'royalblue': 0x4169E1, 'saddlebrown': 0x8B4513, 'salmon': 0xFA8072, 'sandybrown': 0xF4A460, 'seagreen': 0x2E8B57, 'seashell': 0xFFF5EE,
	'sienna': 0xA0522D, 'silver': 0xC0C0C0, 'skyblue': 0x87CEEB, 'slateblue': 0x6A5ACD, 'slategray': 0x708090, 'slategrey': 0x708090, 'snow': 0xFFFAFA,
	'springgreen': 0x00FF7F, 'steelblue': 0x4682B4, 'tan': 0xD2B48C, 'teal': 0x008080, 'thistle': 0xD8BFD8, 'tomato': 0xFF6347, 'turquoise': 0x40E0D0,
	'violet': 0xEE82EE, 'wheat': 0xF5DEB3, 'white': 0xFFFFFF, 'whitesmoke': 0xF5F5F5, 'yellow': 0xFFFF00, 'yellowgreen': 0x9ACD32 };

const _hslA = { h: 0, s: 0, l: 0 };
const _hslB = { h: 0, s: 0, l: 0 };

function hue2rgb( p, q, t ) {

	if ( t < 0 ) t += 1;
	if ( t > 1 ) t -= 1;
	if ( t < 1 / 6 ) return p + ( q - p ) * 6 * t;
	if ( t < 1 / 2 ) return q;
	if ( t < 2 / 3 ) return p + ( q - p ) * 6 * ( 2 / 3 - t );
	return p;

}

function SRGBToLinear( c ) {

	return ( c < 0.04045 ) ? c * 0.0773993808 : Math.pow( c * 0.9478672986 + 0.0521327014, 2.4 );

}

function LinearToSRGB( c ) {

	return ( c < 0.0031308 ) ? c * 12.92 : 1.055 * ( Math.pow( c, 0.41666 ) ) - 0.055;

}

class Color {

	constructor( r, g, b ) {

		if ( g === undefined && b === undefined ) {

			// r is THREE.Color, hex or string
			return this.set( r );

		}

		return this.setRGB( r, g, b );

	}

	set( value ) {

		if ( value && value.isColor ) {

			this.copy( value );

		} else if ( typeof value === 'number' ) {

			this.setHex( value );

		} else if ( typeof value === 'string' ) {

			this.setStyle( value );

		}

		return this;

	}

	setScalar( scalar ) {

		this.r = scalar;
		this.g = scalar;
		this.b = scalar;

		return this;

	}

	setHex( hex ) {

		hex = Math.floor( hex );

		this.r = ( hex >> 16 & 255 ) / 255;
		this.g = ( hex >> 8 & 255 ) / 255;
		this.b = ( hex & 255 ) / 255;

		return this;

	}

	setRGB( r, g, b ) {

		this.r = r;
		this.g = g;
		this.b = b;

		return this;

	}

	setHSL( h, s, l ) {

		// h,s,l ranges are in 0.0 - 1.0
		h = MathUtils.euclideanModulo( h, 1 );
		s = MathUtils.clamp( s, 0, 1 );
		l = MathUtils.clamp( l, 0, 1 );

		if ( s === 0 ) {

			this.r = this.g = this.b = l;

		} else {

			const p = l <= 0.5 ? l * ( 1 + s ) : l + s - ( l * s );
			const q = ( 2 * l ) - p;

			this.r = hue2rgb( q, p, h + 1 / 3 );
			this.g = hue2rgb( q, p, h );
			this.b = hue2rgb( q, p, h - 1 / 3 );

		}

		return this;

	}

	setStyle( style ) {

		function handleAlpha( string ) {

			if ( string === undefined ) return;

			if ( parseFloat( string ) < 1 ) {

				console.warn( 'THREE.Color: Alpha component of ' + style + ' will be ignored.' );

			}

		}


		let m;

		if ( m = /^((?:rgb|hsl)a?)\(([^\)]*)\)/.exec( style ) ) {

			// rgb / hsl

			let color;
			const name = m[ 1 ];
			const components = m[ 2 ];

			switch ( name ) {

				case 'rgb':
				case 'rgba':

					if ( color = /^\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*(?:,\s*(\d*\.?\d+)\s*)?$/.exec( components ) ) {

						// rgb(255,0,0) rgba(255,0,0,0.5)
						this.r = Math.min( 255, parseInt( color[ 1 ], 10 ) ) / 255;
						this.g = Math.min( 255, parseInt( color[ 2 ], 10 ) ) / 255;
						this.b = Math.min( 255, parseInt( color[ 3 ], 10 ) ) / 255;

						handleAlpha( color[ 4 ] );

						return this;

					}

					if ( color = /^\s*(\d+)\%\s*,\s*(\d+)\%\s*,\s*(\d+)\%\s*(?:,\s*(\d*\.?\d+)\s*)?$/.exec( components ) ) {

						// rgb(100%,0%,0%) rgba(100%,0%,0%,0.5)
						this.r = Math.min( 100, parseInt( color[ 1 ], 10 ) ) / 100;
						this.g = Math.min( 100, parseInt( color[ 2 ], 10 ) ) / 100;
						this.b = Math.min( 100, parseInt( color[ 3 ], 10 ) ) / 100;

						handleAlpha( color[ 4 ] );

						return this;

					}

					break;

				case 'hsl':
				case 'hsla':

					if ( color = /^\s*(\d*\.?\d+)\s*,\s*(\d+)\%\s*,\s*(\d+)\%\s*(?:,\s*(\d*\.?\d+)\s*)?$/.exec( components ) ) {

						// hsl(120,50%,50%) hsla(120,50%,50%,0.5)
						const h = parseFloat( color[ 1 ] ) / 360;
						const s = parseInt( color[ 2 ], 10 ) / 100;
						const l = parseInt( color[ 3 ], 10 ) / 100;

						handleAlpha( color[ 4 ] );

						return this.setHSL( h, s, l );

					}

					break;

			}

		} else if ( m = /^\#([A-Fa-f\d]+)$/.exec( style ) ) {

			// hex color

			const hex = m[ 1 ];
			const size = hex.length;

			if ( size === 3 ) {

				// #ff0
				this.r = parseInt( hex.charAt( 0 ) + hex.charAt( 0 ), 16 ) / 255;
				this.g = parseInt( hex.charAt( 1 ) + hex.charAt( 1 ), 16 ) / 255;
				this.b = parseInt( hex.charAt( 2 ) + hex.charAt( 2 ), 16 ) / 255;

				return this;

			} else if ( size === 6 ) {

				// #ff0000
				this.r = parseInt( hex.charAt( 0 ) + hex.charAt( 1 ), 16 ) / 255;
				this.g = parseInt( hex.charAt( 2 ) + hex.charAt( 3 ), 16 ) / 255;
				this.b = parseInt( hex.charAt( 4 ) + hex.charAt( 5 ), 16 ) / 255;

				return this;

			}

		}

		if ( style && style.length > 0 ) {

			return this.setColorName( style );

		}

		return this;

	}

	setColorName( style ) {

		// color keywords
		const hex = _colorKeywords[ style ];

		if ( hex !== undefined ) {

			// red
			this.setHex( hex );

		} else {

			// unknown color
			console.warn( 'THREE.Color: Unknown color ' + style );

		}

		return this;

	}

	clone() {

		return new this.constructor( this.r, this.g, this.b );

	}

	copy( color ) {

		this.r = color.r;
		this.g = color.g;
		this.b = color.b;

		return this;

	}

	copyGammaToLinear( color, gammaFactor = 2.0 ) {

		this.r = Math.pow( color.r, gammaFactor );
		this.g = Math.pow( color.g, gammaFactor );
		this.b = Math.pow( color.b, gammaFactor );

		return this;

	}

	copyLinearToGamma( color, gammaFactor = 2.0 ) {

		const safeInverse = ( gammaFactor > 0 ) ? ( 1.0 / gammaFactor ) : 1.0;

		this.r = Math.pow( color.r, safeInverse );
		this.g = Math.pow( color.g, safeInverse );
		this.b = Math.pow( color.b, safeInverse );

		return this;

	}

	convertGammaToLinear( gammaFactor ) {

		this.copyGammaToLinear( this, gammaFactor );

		return this;

	}

	convertLinearToGamma( gammaFactor ) {

		this.copyLinearToGamma( this, gammaFactor );

		return this;

	}

	copySRGBToLinear( color ) {

		this.r = SRGBToLinear( color.r );
		this.g = SRGBToLinear( color.g );
		this.b = SRGBToLinear( color.b );

		return this;

	}

	copyLinearToSRGB( color ) {

		this.r = LinearToSRGB( color.r );
		this.g = LinearToSRGB( color.g );
		this.b = LinearToSRGB( color.b );

		return this;

	}

	convertSRGBToLinear() {

		this.copySRGBToLinear( this );

		return this;

	}

	convertLinearToSRGB() {

		this.copyLinearToSRGB( this );

		return this;

	}

	getHex() {

		return ( this.r * 255 ) << 16 ^ ( this.g * 255 ) << 8 ^ ( this.b * 255 ) << 0;

	}

	getHexString() {

		return ( '000000' + this.getHex().toString( 16 ) ).slice( - 6 );

	}

	getHSL( target ) {

		// h,s,l ranges are in 0.0 - 1.0

		if ( target === undefined ) {

			console.warn( 'THREE.Color: .getHSL() target is now required' );
			target = { h: 0, s: 0, l: 0 };

		}

		const r = this.r, g = this.g, b = this.b;

		const max = Math.max( r, g, b );
		const min = Math.min( r, g, b );

		let hue, saturation;
		const lightness = ( min + max ) / 2.0;

		if ( min === max ) {

			hue = 0;
			saturation = 0;

		} else {

			const delta = max - min;

			saturation = lightness <= 0.5 ? delta / ( max + min ) : delta / ( 2 - max - min );

			switch ( max ) {

				case r: hue = ( g - b ) / delta + ( g < b ? 6 : 0 ); break;
				case g: hue = ( b - r ) / delta + 2; break;
				case b: hue = ( r - g ) / delta + 4; break;

			}

			hue /= 6;

		}

		target.h = hue;
		target.s = saturation;
		target.l = lightness;

		return target;

	}

	getStyle() {

		return 'rgb(' + ( ( this.r * 255 ) | 0 ) + ',' + ( ( this.g * 255 ) | 0 ) + ',' + ( ( this.b * 255 ) | 0 ) + ')';

	}

	offsetHSL( h, s, l ) {

		this.getHSL( _hslA );

		_hslA.h += h; _hslA.s += s; _hslA.l += l;

		this.setHSL( _hslA.h, _hslA.s, _hslA.l );

		return this;

	}

	add( color ) {

		this.r += color.r;
		this.g += color.g;
		this.b += color.b;

		return this;

	}

	addColors( color1, color2 ) {

		this.r = color1.r + color2.r;
		this.g = color1.g + color2.g;
		this.b = color1.b + color2.b;

		return this;

	}

	addScalar( s ) {

		this.r += s;
		this.g += s;
		this.b += s;

		return this;

	}

	sub( color ) {

		this.r = Math.max( 0, this.r - color.r );
		this.g = Math.max( 0, this.g - color.g );
		this.b = Math.max( 0, this.b - color.b );

		return this;

	}

	multiply( color ) {

		this.r *= color.r;
		this.g *= color.g;
		this.b *= color.b;

		return this;

	}

	multiplyScalar( s ) {

		this.r *= s;
		this.g *= s;
		this.b *= s;

		return this;

	}

	lerp( color, alpha ) {

		this.r += ( color.r - this.r ) * alpha;
		this.g += ( color.g - this.g ) * alpha;
		this.b += ( color.b - this.b ) * alpha;

		return this;

	}

	lerpColors( color1, color2, alpha ) {

		this.r = color1.r + ( color2.r - color1.r ) * alpha;
		this.g = color1.g + ( color2.g - color1.g ) * alpha;
		this.b = color1.b + ( color2.b - color1.b ) * alpha;

		return this;

	}

	lerpHSL( color, alpha ) {

		this.getHSL( _hslA );
		color.getHSL( _hslB );

		const h = MathUtils.lerp( _hslA.h, _hslB.h, alpha );
		const s = MathUtils.lerp( _hslA.s, _hslB.s, alpha );
		const l = MathUtils.lerp( _hslA.l, _hslB.l, alpha );

		this.setHSL( h, s, l );

		return this;

	}

	equals( c ) {

		return ( c.r === this.r ) && ( c.g === this.g ) && ( c.b === this.b );

	}

	fromArray( array, offset = 0 ) {

		this.r = array[ offset ];
		this.g = array[ offset + 1 ];
		this.b = array[ offset + 2 ];

		return this;

	}

	toArray( array = [], offset = 0 ) {

		array[ offset ] = this.r;
		array[ offset + 1 ] = this.g;
		array[ offset + 2 ] = this.b;

		return array;

	}

	fromBufferAttribute( attribute, index ) {

		this.r = attribute.getX( index );
		this.g = attribute.getY( index );
		this.b = attribute.getZ( index );

		if ( attribute.normalized === true ) {

			// assuming Uint8Array

			this.r /= 255;
			this.g /= 255;
			this.b /= 255;

		}

		return this;

	}

	toJSON() {

		return this.getHex();

	}

}

Color.NAMES = _colorKeywords;

Color.prototype.isColor = true;
Color.prototype.r = 1;
Color.prototype.g = 1;
Color.prototype.b = 1;

/**
 * parameters = {
 *  color: <hex>,
 *  opacity: <float>,
 *  map: new THREE.Texture( <Image> ),
 *
 *  lightMap: new THREE.Texture( <Image> ),
 *  lightMapIntensity: <float>
 *
 *  aoMap: new THREE.Texture( <Image> ),
 *  aoMapIntensity: <float>
 *
 *  specularMap: new THREE.Texture( <Image> ),
 *
 *  alphaMap: new THREE.Texture( <Image> ),
 *
 *  envMap: new THREE.CubeTexture( [posx, negx, posy, negy, posz, negz] ),
 *  combine: THREE.Multiply,
 *  reflectivity: <float>,
 *  refractionRatio: <float>,
 *
 *  depthTest: <bool>,
 *  depthWrite: <bool>,
 *
 *  wireframe: <boolean>,
 *  wireframeLinewidth: <float>,
 *
 *  skinning: <bool>,
 *  morphTargets: <bool>
 * }
 */

class MeshBasicMaterial extends Material {

	constructor( parameters ) {

		super();

		this.type = 'MeshBasicMaterial';

		this.color = new Color( 0xffffff ); // emissive

		this.map = null;

		this.lightMap = null;
		this.lightMapIntensity = 1.0;

		this.aoMap = null;
		this.aoMapIntensity = 1.0;

		this.specularMap = null;

		this.alphaMap = null;

		this.envMap = null;
		this.combine = MultiplyOperation;
		this.reflectivity = 1;
		this.refractionRatio = 0.98;

		this.wireframe = false;
		this.wireframeLinewidth = 1;
		this.wireframeLinecap = 'round';
		this.wireframeLinejoin = 'round';

		this.skinning = false;
		this.morphTargets = false;

		this.setValues( parameters );

	}

	copy( source ) {

		super.copy( source );

		this.color.copy( source.color );

		this.map = source.map;

		this.lightMap = source.lightMap;
		this.lightMapIntensity = source.lightMapIntensity;

		this.aoMap = source.aoMap;
		this.aoMapIntensity = source.aoMapIntensity;

		this.specularMap = source.specularMap;

		this.alphaMap = source.alphaMap;

		this.envMap = source.envMap;
		this.combine = source.combine;
		this.reflectivity = source.reflectivity;
		this.refractionRatio = source.refractionRatio;

		this.wireframe = source.wireframe;
		this.wireframeLinewidth = source.wireframeLinewidth;
		this.wireframeLinecap = source.wireframeLinecap;
		this.wireframeLinejoin = source.wireframeLinejoin;

		this.skinning = source.skinning;
		this.morphTargets = source.morphTargets;

		return this;

	}

}

MeshBasicMaterial.prototype.isMeshBasicMaterial = true;

const _vector$9 = new Vector3();
const _vector2 = new Vector2();

function BufferAttribute( array, itemSize, normalized ) {

	if ( Array.isArray( array ) ) {

		throw new TypeError( 'THREE.BufferAttribute: array should be a Typed Array.' );

	}

	this.name = '';

	this.array = array;
	this.itemSize = itemSize;
	this.count = array !== undefined ? array.length / itemSize : 0;
	this.normalized = normalized === true;

	this.usage = StaticDrawUsage;
	this.updateRange = { offset: 0, count: - 1 };

	this.version = 0;

}

Object.defineProperty( BufferAttribute.prototype, 'needsUpdate', {

	set: function ( value ) {

		if ( value === true ) this.version ++;

	}

} );

Object.assign( BufferAttribute.prototype, {

	isBufferAttribute: true,

	onUploadCallback: function () {},

	setUsage: function ( value ) {

		this.usage = value;

		return this;

	},

	copy: function ( source ) {

		this.name = source.name;
		this.array = new source.array.constructor( source.array );
		this.itemSize = source.itemSize;
		this.count = source.count;
		this.normalized = source.normalized;

		this.usage = source.usage;

		return this;

	},

	copyAt: function ( index1, attribute, index2 ) {

		index1 *= this.itemSize;
		index2 *= attribute.itemSize;

		for ( let i = 0, l = this.itemSize; i < l; i ++ ) {

			this.array[ index1 + i ] = attribute.array[ index2 + i ];

		}

		return this;

	},

	copyArray: function ( array ) {

		this.array.set( array );

		return this;

	},

	copyColorsArray: function ( colors ) {

		const array = this.array;
		let offset = 0;

		for ( let i = 0, l = colors.length; i < l; i ++ ) {

			let color = colors[ i ];

			if ( color === undefined ) {

				console.warn( 'THREE.BufferAttribute.copyColorsArray(): color is undefined', i );
				color = new Color();

			}

			array[ offset ++ ] = color.r;
			array[ offset ++ ] = color.g;
			array[ offset ++ ] = color.b;

		}

		return this;

	},

	copyVector2sArray: function ( vectors ) {

		const array = this.array;
		let offset = 0;

		for ( let i = 0, l = vectors.length; i < l; i ++ ) {

			let vector = vectors[ i ];

			if ( vector === undefined ) {

				console.warn( 'THREE.BufferAttribute.copyVector2sArray(): vector is undefined', i );
				vector = new Vector2();

			}

			array[ offset ++ ] = vector.x;
			array[ offset ++ ] = vector.y;

		}

		return this;

	},

	copyVector3sArray: function ( vectors ) {

		const array = this.array;
		let offset = 0;

		for ( let i = 0, l = vectors.length; i < l; i ++ ) {

			let vector = vectors[ i ];

			if ( vector === undefined ) {

				console.warn( 'THREE.BufferAttribute.copyVector3sArray(): vector is undefined', i );
				vector = new Vector3();

			}

			array[ offset ++ ] = vector.x;
			array[ offset ++ ] = vector.y;
			array[ offset ++ ] = vector.z;

		}

		return this;

	},

	copyVector4sArray: function ( vectors ) {

		const array = this.array;
		let offset = 0;

		for ( let i = 0, l = vectors.length; i < l; i ++ ) {

			let vector = vectors[ i ];

			if ( vector === undefined ) {

				console.warn( 'THREE.BufferAttribute.copyVector4sArray(): vector is undefined', i );
				vector = new Vector4();

			}

			array[ offset ++ ] = vector.x;
			array[ offset ++ ] = vector.y;
			array[ offset ++ ] = vector.z;
			array[ offset ++ ] = vector.w;

		}

		return this;

	},

	applyMatrix3: function ( m ) {

		if ( this.itemSize === 2 ) {

			for ( let i = 0, l = this.count; i < l; i ++ ) {

				_vector2.fromBufferAttribute( this, i );
				_vector2.applyMatrix3( m );

				this.setXY( i, _vector2.x, _vector2.y );

			}

		} else if ( this.itemSize === 3 ) {

			for ( let i = 0, l = this.count; i < l; i ++ ) {

				_vector$9.fromBufferAttribute( this, i );
				_vector$9.applyMatrix3( m );

				this.setXYZ( i, _vector$9.x, _vector$9.y, _vector$9.z );

			}

		}

		return this;

	},

	applyMatrix4: function ( m ) {

		for ( let i = 0, l = this.count; i < l; i ++ ) {

			_vector$9.x = this.getX( i );
			_vector$9.y = this.getY( i );
			_vector$9.z = this.getZ( i );

			_vector$9.applyMatrix4( m );

			this.setXYZ( i, _vector$9.x, _vector$9.y, _vector$9.z );

		}

		return this;

	},

	applyNormalMatrix: function ( m ) {

		for ( let i = 0, l = this.count; i < l; i ++ ) {

			_vector$9.x = this.getX( i );
			_vector$9.y = this.getY( i );
			_vector$9.z = this.getZ( i );

			_vector$9.applyNormalMatrix( m );

			this.setXYZ( i, _vector$9.x, _vector$9.y, _vector$9.z );

		}

		return this;

	},

	transformDirection: function ( m ) {

		for ( let i = 0, l = this.count; i < l; i ++ ) {

			_vector$9.x = this.getX( i );
			_vector$9.y = this.getY( i );
			_vector$9.z = this.getZ( i );

			_vector$9.transformDirection( m );

			this.setXYZ( i, _vector$9.x, _vector$9.y, _vector$9.z );

		}

		return this;

	},

	set: function ( value, offset = 0 ) {

		this.array.set( value, offset );

		return this;

	},

	getX: function ( index ) {

		return this.array[ index * this.itemSize ];

	},

	setX: function ( index, x ) {

		this.array[ index * this.itemSize ] = x;

		return this;

	},

	getY: function ( index ) {

		return this.array[ index * this.itemSize + 1 ];

	},

	setY: function ( index, y ) {

		this.array[ index * this.itemSize + 1 ] = y;

		return this;

	},

	getZ: function ( index ) {

		return this.array[ index * this.itemSize + 2 ];

	},

	setZ: function ( index, z ) {

		this.array[ index * this.itemSize + 2 ] = z;

		return this;

	},

	getW: function ( index ) {

		return this.array[ index * this.itemSize + 3 ];

	},

	setW: function ( index, w ) {

		this.array[ index * this.itemSize + 3 ] = w;

		return this;

	},

	setXY: function ( index, x, y ) {

		index *= this.itemSize;

		this.array[ index + 0 ] = x;
		this.array[ index + 1 ] = y;

		return this;

	},

	setXYZ: function ( index, x, y, z ) {

		index *= this.itemSize;

		this.array[ index + 0 ] = x;
		this.array[ index + 1 ] = y;
		this.array[ index + 2 ] = z;

		return this;

	},

	setXYZW: function ( index, x, y, z, w ) {

		index *= this.itemSize;

		this.array[ index + 0 ] = x;
		this.array[ index + 1 ] = y;
		this.array[ index + 2 ] = z;
		this.array[ index + 3 ] = w;

		return this;

	},

	onUpload: function ( callback ) {

		this.onUploadCallback = callback;

		return this;

	},

	clone: function () {

		return new this.constructor( this.array, this.itemSize ).copy( this );

	},

	toJSON: function () {

		const data = {
			itemSize: this.itemSize,
			type: this.array.constructor.name,
			array: Array.prototype.slice.call( this.array ),
			normalized: this.normalized
		};

		if ( this.name !== '' ) data.name = this.name;
		if ( this.usage !== StaticDrawUsage ) data.usage = this.usage;
		if ( this.updateRange.offset !== 0 || this.updateRange.count !== - 1 ) data.updateRange = this.updateRange;

		return data;

	}

} );

//

function Int8BufferAttribute( array, itemSize, normalized ) {

	BufferAttribute.call( this, new Int8Array( array ), itemSize, normalized );

}

Int8BufferAttribute.prototype = Object.create( BufferAttribute.prototype );
Int8BufferAttribute.prototype.constructor = Int8BufferAttribute;


function Uint8BufferAttribute( array, itemSize, normalized ) {

	BufferAttribute.call( this, new Uint8Array( array ), itemSize, normalized );

}

Uint8BufferAttribute.prototype = Object.create( BufferAttribute.prototype );
Uint8BufferAttribute.prototype.constructor = Uint8BufferAttribute;


function Uint8ClampedBufferAttribute( array, itemSize, normalized ) {

	BufferAttribute.call( this, new Uint8ClampedArray( array ), itemSize, normalized );

}

Uint8ClampedBufferAttribute.prototype = Object.create( BufferAttribute.prototype );
Uint8ClampedBufferAttribute.prototype.constructor = Uint8ClampedBufferAttribute;


function Int16BufferAttribute( array, itemSize, normalized ) {

	BufferAttribute.call( this, new Int16Array( array ), itemSize, normalized );

}

Int16BufferAttribute.prototype = Object.create( BufferAttribute.prototype );
Int16BufferAttribute.prototype.constructor = Int16BufferAttribute;


function Uint16BufferAttribute( array, itemSize, normalized ) {

	BufferAttribute.call( this, new Uint16Array( array ), itemSize, normalized );

}

Uint16BufferAttribute.prototype = Object.create( BufferAttribute.prototype );
Uint16BufferAttribute.prototype.constructor = Uint16BufferAttribute;


function Int32BufferAttribute( array, itemSize, normalized ) {

	BufferAttribute.call( this, new Int32Array( array ), itemSize, normalized );

}

Int32BufferAttribute.prototype = Object.create( BufferAttribute.prototype );
Int32BufferAttribute.prototype.constructor = Int32BufferAttribute;


function Uint32BufferAttribute( array, itemSize, normalized ) {

	BufferAttribute.call( this, new Uint32Array( array ), itemSize, normalized );

}

Uint32BufferAttribute.prototype = Object.create( BufferAttribute.prototype );
Uint32BufferAttribute.prototype.constructor = Uint32BufferAttribute;

function Float16BufferAttribute( array, itemSize, normalized ) {

	BufferAttribute.call( this, new Uint16Array( array ), itemSize, normalized );

}

Float16BufferAttribute.prototype = Object.create( BufferAttribute.prototype );
Float16BufferAttribute.prototype.constructor = Float16BufferAttribute;
Float16BufferAttribute.prototype.isFloat16BufferAttribute = true;

function Float32BufferAttribute( array, itemSize, normalized ) {

	BufferAttribute.call( this, new Float32Array( array ), itemSize, normalized );

}

Float32BufferAttribute.prototype = Object.create( BufferAttribute.prototype );
Float32BufferAttribute.prototype.constructor = Float32BufferAttribute;


function Float64BufferAttribute( array, itemSize, normalized ) {

	BufferAttribute.call( this, new Float64Array( array ), itemSize, normalized );

}

Float64BufferAttribute.prototype = Object.create( BufferAttribute.prototype );
Float64BufferAttribute.prototype.constructor = Float64BufferAttribute;

function arrayMax( array ) {

	if ( array.length === 0 ) return - Infinity;

	let max = array[ 0 ];

	for ( let i = 1, l = array.length; i < l; ++ i ) {

		if ( array[ i ] > max ) max = array[ i ];

	}

	return max;

}

let _id = 0;

const _m1 = new Matrix4();
const _obj = new Object3D();
const _offset = new Vector3();
const _box$1 = new Box3();
const _boxMorphTargets = new Box3();
const _vector$8 = new Vector3();

function BufferGeometry() {

	Object.defineProperty( this, 'id', { value: _id ++ } );

	this.uuid = MathUtils.generateUUID();

	this.name = '';
	this.type = 'BufferGeometry';

	this.index = null;
	this.attributes = {};

	this.morphAttributes = {};
	this.morphTargetsRelative = false;

	this.groups = [];

	this.boundingBox = null;
	this.boundingSphere = null;

	this.drawRange = { start: 0, count: Infinity };

	this.userData = {};

}

BufferGeometry.prototype = Object.assign( Object.create( EventDispatcher.prototype ), {

	constructor: BufferGeometry,

	isBufferGeometry: true,

	getIndex: function () {

		return this.index;

	},

	setIndex: function ( index ) {

		if ( Array.isArray( index ) ) {

			this.index = new ( arrayMax( index ) > 65535 ? Uint32BufferAttribute : Uint16BufferAttribute )( index, 1 );

		} else {

			this.index = index;

		}

		return this;

	},

	getAttribute: function ( name ) {

		return this.attributes[ name ];

	},

	setAttribute: function ( name, attribute ) {

		this.attributes[ name ] = attribute;

		return this;

	},

	deleteAttribute: function ( name ) {

		delete this.attributes[ name ];

		return this;

	},

	hasAttribute: function ( name ) {

		return this.attributes[ name ] !== undefined;

	},

	addGroup: function ( start, count, materialIndex = 0 ) {

		this.groups.push( {

			start: start,
			count: count,
			materialIndex: materialIndex

		} );

	},

	clearGroups: function () {

		this.groups = [];

	},

	setDrawRange: function ( start, count ) {

		this.drawRange.start = start;
		this.drawRange.count = count;

	},

	applyMatrix4: function ( matrix ) {

		const position = this.attributes.position;

		if ( position !== undefined ) {

			position.applyMatrix4( matrix );

			position.needsUpdate = true;

		}

		const normal = this.attributes.normal;

		if ( normal !== undefined ) {

			const normalMatrix = new Matrix3().getNormalMatrix( matrix );

			normal.applyNormalMatrix( normalMatrix );

			normal.needsUpdate = true;

		}

		const tangent = this.attributes.tangent;

		if ( tangent !== undefined ) {

			tangent.transformDirection( matrix );

			tangent.needsUpdate = true;

		}

		if ( this.boundingBox !== null ) {

			this.computeBoundingBox();

		}

		if ( this.boundingSphere !== null ) {

			this.computeBoundingSphere();

		}

		return this;

	},

	rotateX: function ( angle ) {

		// rotate geometry around world x-axis

		_m1.makeRotationX( angle );

		this.applyMatrix4( _m1 );

		return this;

	},

	rotateY: function ( angle ) {

		// rotate geometry around world y-axis

		_m1.makeRotationY( angle );

		this.applyMatrix4( _m1 );

		return this;

	},

	rotateZ: function ( angle ) {

		// rotate geometry around world z-axis

		_m1.makeRotationZ( angle );

		this.applyMatrix4( _m1 );

		return this;

	},

	translate: function ( x, y, z ) {

		// translate geometry

		_m1.makeTranslation( x, y, z );

		this.applyMatrix4( _m1 );

		return this;

	},

	scale: function ( x, y, z ) {

		// scale geometry

		_m1.makeScale( x, y, z );

		this.applyMatrix4( _m1 );

		return this;

	},

	lookAt: function ( vector ) {

		_obj.lookAt( vector );

		_obj.updateMatrix();

		this.applyMatrix4( _obj.matrix );

		return this;

	},

	center: function () {

		this.computeBoundingBox();

		this.boundingBox.getCenter( _offset ).negate();

		this.translate( _offset.x, _offset.y, _offset.z );

		return this;

	},

	setFromPoints: function ( points ) {

		const position = [];

		for ( let i = 0, l = points.length; i < l; i ++ ) {

			const point = points[ i ];
			position.push( point.x, point.y, point.z || 0 );

		}

		this.setAttribute( 'position', new Float32BufferAttribute( position, 3 ) );

		return this;

	},

	computeBoundingBox: function () {

		if ( this.boundingBox === null ) {

			this.boundingBox = new Box3();

		}

		const position = this.attributes.position;
		const morphAttributesPosition = this.morphAttributes.position;

		if ( position && position.isGLBufferAttribute ) {

			console.error( 'THREE.BufferGeometry.computeBoundingBox(): GLBufferAttribute requires a manual bounding box. Alternatively set "mesh.frustumCulled" to "false".', this );

			this.boundingBox.set(
				new Vector3( - Infinity, - Infinity, - Infinity ),
				new Vector3( + Infinity, + Infinity, + Infinity )
			);

			return;

		}

		if ( position !== undefined ) {

			this.boundingBox.setFromBufferAttribute( position );

			// process morph attributes if present

			if ( morphAttributesPosition ) {

				for ( let i = 0, il = morphAttributesPosition.length; i < il; i ++ ) {

					const morphAttribute = morphAttributesPosition[ i ];
					_box$1.setFromBufferAttribute( morphAttribute );

					if ( this.morphTargetsRelative ) {

						_vector$8.addVectors( this.boundingBox.min, _box$1.min );
						this.boundingBox.expandByPoint( _vector$8 );

						_vector$8.addVectors( this.boundingBox.max, _box$1.max );
						this.boundingBox.expandByPoint( _vector$8 );

					} else {

						this.boundingBox.expandByPoint( _box$1.min );
						this.boundingBox.expandByPoint( _box$1.max );

					}

				}

			}

		} else {

			this.boundingBox.makeEmpty();

		}

		if ( isNaN( this.boundingBox.min.x ) || isNaN( this.boundingBox.min.y ) || isNaN( this.boundingBox.min.z ) ) {

			console.error( 'THREE.BufferGeometry.computeBoundingBox(): Computed min/max have NaN values. The "position" attribute is likely to have NaN values.', this );

		}

	},

	computeBoundingSphere: function () {

		if ( this.boundingSphere === null ) {

			this.boundingSphere = new Sphere();

		}

		const position = this.attributes.position;
		const morphAttributesPosition = this.morphAttributes.position;

		if ( position && position.isGLBufferAttribute ) {

			console.error( 'THREE.BufferGeometry.computeBoundingSphere(): GLBufferAttribute requires a manual bounding sphere. Alternatively set "mesh.frustumCulled" to "false".', this );

			this.boundingSphere.set( new Vector3(), Infinity );

			return;

		}

		if ( position ) {

			// first, find the center of the bounding sphere

			const center = this.boundingSphere.center;

			_box$1.setFromBufferAttribute( position );

			// process morph attributes if present

			if ( morphAttributesPosition ) {

				for ( let i = 0, il = morphAttributesPosition.length; i < il; i ++ ) {

					const morphAttribute = morphAttributesPosition[ i ];
					_boxMorphTargets.setFromBufferAttribute( morphAttribute );

					if ( this.morphTargetsRelative ) {

						_vector$8.addVectors( _box$1.min, _boxMorphTargets.min );
						_box$1.expandByPoint( _vector$8 );

						_vector$8.addVectors( _box$1.max, _boxMorphTargets.max );
						_box$1.expandByPoint( _vector$8 );

					} else {

						_box$1.expandByPoint( _boxMorphTargets.min );
						_box$1.expandByPoint( _boxMorphTargets.max );

					}

				}

			}

			_box$1.getCenter( center );

			// second, try to find a boundingSphere with a radius smaller than the
			// boundingSphere of the boundingBox: sqrt(3) smaller in the best case

			let maxRadiusSq = 0;

			for ( let i = 0, il = position.count; i < il; i ++ ) {

				_vector$8.fromBufferAttribute( position, i );

				maxRadiusSq = Math.max( maxRadiusSq, center.distanceToSquared( _vector$8 ) );

			}

			// process morph attributes if present

			if ( morphAttributesPosition ) {

				for ( let i = 0, il = morphAttributesPosition.length; i < il; i ++ ) {

					const morphAttribute = morphAttributesPosition[ i ];
					const morphTargetsRelative = this.morphTargetsRelative;

					for ( let j = 0, jl = morphAttribute.count; j < jl; j ++ ) {

						_vector$8.fromBufferAttribute( morphAttribute, j );

						if ( morphTargetsRelative ) {

							_offset.fromBufferAttribute( position, j );
							_vector$8.add( _offset );

						}

						maxRadiusSq = Math.max( maxRadiusSq, center.distanceToSquared( _vector$8 ) );

					}

				}

			}

			this.boundingSphere.radius = Math.sqrt( maxRadiusSq );

			if ( isNaN( this.boundingSphere.radius ) ) {

				console.error( 'THREE.BufferGeometry.computeBoundingSphere(): Computed radius is NaN. The "position" attribute is likely to have NaN values.', this );

			}

		}

	},

	computeFaceNormals: function () {

		// backwards compatibility

	},

	computeTangents: function () {

		const index = this.index;
		const attributes = this.attributes;

		// based on http://www.terathon.com/code/tangent.html
		// (per vertex tangents)

		if ( index === null ||
			 attributes.position === undefined ||
			 attributes.normal === undefined ||
			 attributes.uv === undefined ) {

			console.error( 'THREE.BufferGeometry: .computeTangents() failed. Missing required attributes (index, position, normal or uv)' );
			return;

		}

		const indices = index.array;
		const positions = attributes.position.array;
		const normals = attributes.normal.array;
		const uvs = attributes.uv.array;

		const nVertices = positions.length / 3;

		if ( attributes.tangent === undefined ) {

			this.setAttribute( 'tangent', new BufferAttribute( new Float32Array( 4 * nVertices ), 4 ) );

		}

		const tangents = attributes.tangent.array;

		const tan1 = [], tan2 = [];

		for ( let i = 0; i < nVertices; i ++ ) {

			tan1[ i ] = new Vector3();
			tan2[ i ] = new Vector3();

		}

		const vA = new Vector3(),
			vB = new Vector3(),
			vC = new Vector3(),

			uvA = new Vector2(),
			uvB = new Vector2(),
			uvC = new Vector2(),

			sdir = new Vector3(),
			tdir = new Vector3();

		function handleTriangle( a, b, c ) {

			vA.fromArray( positions, a * 3 );
			vB.fromArray( positions, b * 3 );
			vC.fromArray( positions, c * 3 );

			uvA.fromArray( uvs, a * 2 );
			uvB.fromArray( uvs, b * 2 );
			uvC.fromArray( uvs, c * 2 );

			vB.sub( vA );
			vC.sub( vA );

			uvB.sub( uvA );
			uvC.sub( uvA );

			const r = 1.0 / ( uvB.x * uvC.y - uvC.x * uvB.y );

			// silently ignore degenerate uv triangles having coincident or colinear vertices

			if ( ! isFinite( r ) ) return;

			sdir.copy( vB ).multiplyScalar( uvC.y ).addScaledVector( vC, - uvB.y ).multiplyScalar( r );
			tdir.copy( vC ).multiplyScalar( uvB.x ).addScaledVector( vB, - uvC.x ).multiplyScalar( r );

			tan1[ a ].add( sdir );
			tan1[ b ].add( sdir );
			tan1[ c ].add( sdir );

			tan2[ a ].add( tdir );
			tan2[ b ].add( tdir );
			tan2[ c ].add( tdir );

		}

		let groups = this.groups;

		if ( groups.length === 0 ) {

			groups = [ {
				start: 0,
				count: indices.length
			} ];

		}

		for ( let i = 0, il = groups.length; i < il; ++ i ) {

			const group = groups[ i ];

			const start = group.start;
			const count = group.count;

			for ( let j = start, jl = start + count; j < jl; j += 3 ) {

				handleTriangle(
					indices[ j + 0 ],
					indices[ j + 1 ],
					indices[ j + 2 ]
				);

			}

		}

		const tmp = new Vector3(), tmp2 = new Vector3();
		const n = new Vector3(), n2 = new Vector3();

		function handleVertex( v ) {

			n.fromArray( normals, v * 3 );
			n2.copy( n );

			const t = tan1[ v ];

			// Gram-Schmidt orthogonalize

			tmp.copy( t );
			tmp.sub( n.multiplyScalar( n.dot( t ) ) ).normalize();

			// Calculate handedness

			tmp2.crossVectors( n2, t );
			const test = tmp2.dot( tan2[ v ] );
			const w = ( test < 0.0 ) ? - 1.0 : 1.0;

			tangents[ v * 4 ] = tmp.x;
			tangents[ v * 4 + 1 ] = tmp.y;
			tangents[ v * 4 + 2 ] = tmp.z;
			tangents[ v * 4 + 3 ] = w;

		}

		for ( let i = 0, il = groups.length; i < il; ++ i ) {

			const group = groups[ i ];

			const start = group.start;
			const count = group.count;

			for ( let j = start, jl = start + count; j < jl; j += 3 ) {

				handleVertex( indices[ j + 0 ] );
				handleVertex( indices[ j + 1 ] );
				handleVertex( indices[ j + 2 ] );

			}

		}

	},

	computeVertexNormals: function () {

		const index = this.index;
		const positionAttribute = this.getAttribute( 'position' );

		if ( positionAttribute !== undefined ) {

			let normalAttribute = this.getAttribute( 'normal' );

			if ( normalAttribute === undefined ) {

				normalAttribute = new BufferAttribute( new Float32Array( positionAttribute.count * 3 ), 3 );
				this.setAttribute( 'normal', normalAttribute );

			} else {

				// reset existing normals to zero

				for ( let i = 0, il = normalAttribute.count; i < il; i ++ ) {

					normalAttribute.setXYZ( i, 0, 0, 0 );

				}

			}

			const pA = new Vector3(), pB = new Vector3(), pC = new Vector3();
			const nA = new Vector3(), nB = new Vector3(), nC = new Vector3();
			const cb = new Vector3(), ab = new Vector3();

			// indexed elements

			if ( index ) {

				for ( let i = 0, il = index.count; i < il; i += 3 ) {

					const vA = index.getX( i + 0 );
					const vB = index.getX( i + 1 );
					const vC = index.getX( i + 2 );

					pA.fromBufferAttribute( positionAttribute, vA );
					pB.fromBufferAttribute( positionAttribute, vB );
					pC.fromBufferAttribute( positionAttribute, vC );

					cb.subVectors( pC, pB );
					ab.subVectors( pA, pB );
					cb.cross( ab );

					nA.fromBufferAttribute( normalAttribute, vA );
					nB.fromBufferAttribute( normalAttribute, vB );
					nC.fromBufferAttribute( normalAttribute, vC );

					nA.add( cb );
					nB.add( cb );
					nC.add( cb );

					normalAttribute.setXYZ( vA, nA.x, nA.y, nA.z );
					normalAttribute.setXYZ( vB, nB.x, nB.y, nB.z );
					normalAttribute.setXYZ( vC, nC.x, nC.y, nC.z );

				}

			} else {

				// non-indexed elements (unconnected triangle soup)

				for ( let i = 0, il = positionAttribute.count; i < il; i += 3 ) {

					pA.fromBufferAttribute( positionAttribute, i + 0 );
					pB.fromBufferAttribute( positionAttribute, i + 1 );
					pC.fromBufferAttribute( positionAttribute, i + 2 );

					cb.subVectors( pC, pB );
					ab.subVectors( pA, pB );
					cb.cross( ab );

					normalAttribute.setXYZ( i + 0, cb.x, cb.y, cb.z );
					normalAttribute.setXYZ( i + 1, cb.x, cb.y, cb.z );
					normalAttribute.setXYZ( i + 2, cb.x, cb.y, cb.z );

				}

			}

			this.normalizeNormals();

			normalAttribute.needsUpdate = true;

		}

	},

	merge: function ( geometry, offset ) {

		if ( ! ( geometry && geometry.isBufferGeometry ) ) {

			console.error( 'THREE.BufferGeometry.merge(): geometry not an instance of THREE.BufferGeometry.', geometry );
			return;

		}

		if ( offset === undefined ) {

			offset = 0;

			console.warn(
				'THREE.BufferGeometry.merge(): Overwriting original geometry, starting at offset=0. '
				+ 'Use BufferGeometryUtils.mergeBufferGeometries() for lossless merge.'
			);

		}

		const attributes = this.attributes;

		for ( const key in attributes ) {

			if ( geometry.attributes[ key ] === undefined ) continue;

			const attribute1 = attributes[ key ];
			const attributeArray1 = attribute1.array;

			const attribute2 = geometry.attributes[ key ];
			const attributeArray2 = attribute2.array;

			const attributeOffset = attribute2.itemSize * offset;
			const length = Math.min( attributeArray2.length, attributeArray1.length - attributeOffset );

			for ( let i = 0, j = attributeOffset; i < length; i ++, j ++ ) {

				attributeArray1[ j ] = attributeArray2[ i ];

			}

		}

		return this;

	},

	normalizeNormals: function () {

		const normals = this.attributes.normal;

		for ( let i = 0, il = normals.count; i < il; i ++ ) {

			_vector$8.fromBufferAttribute( normals, i );

			_vector$8.normalize();

			normals.setXYZ( i, _vector$8.x, _vector$8.y, _vector$8.z );

		}

	},

	toNonIndexed: function () {

		function convertBufferAttribute( attribute, indices ) {

			const array = attribute.array;
			const itemSize = attribute.itemSize;
			const normalized = attribute.normalized;

			const array2 = new array.constructor( indices.length * itemSize );

			let index = 0, index2 = 0;

			for ( let i = 0, l = indices.length; i < l; i ++ ) {

				index = indices[ i ] * itemSize;

				for ( let j = 0; j < itemSize; j ++ ) {

					array2[ index2 ++ ] = array[ index ++ ];

				}

			}

			return new BufferAttribute( array2, itemSize, normalized );

		}

		//

		if ( this.index === null ) {

			console.warn( 'THREE.BufferGeometry.toNonIndexed(): BufferGeometry is already non-indexed.' );
			return this;

		}

		const geometry2 = new BufferGeometry();

		const indices = this.index.array;
		const attributes = this.attributes;

		// attributes

		for ( const name in attributes ) {

			const attribute = attributes[ name ];

			const newAttribute = convertBufferAttribute( attribute, indices );

			geometry2.setAttribute( name, newAttribute );

		}

		// morph attributes

		const morphAttributes = this.morphAttributes;

		for ( const name in morphAttributes ) {

			const morphArray = [];
			const morphAttribute = morphAttributes[ name ]; // morphAttribute: array of Float32BufferAttributes

			for ( let i = 0, il = morphAttribute.length; i < il; i ++ ) {

				const attribute = morphAttribute[ i ];

				const newAttribute = convertBufferAttribute( attribute, indices );

				morphArray.push( newAttribute );

			}

			geometry2.morphAttributes[ name ] = morphArray;

		}

		geometry2.morphTargetsRelative = this.morphTargetsRelative;

		// groups

		const groups = this.groups;

		for ( let i = 0, l = groups.length; i < l; i ++ ) {

			const group = groups[ i ];
			geometry2.addGroup( group.start, group.count, group.materialIndex );

		}

		return geometry2;

	},

	toJSON: function () {

		const data = {
			metadata: {
				version: 4.5,
				type: 'BufferGeometry',
				generator: 'BufferGeometry.toJSON'
			}
		};

		// standard BufferGeometry serialization

		data.uuid = this.uuid;
		data.type = this.type;
		if ( this.name !== '' ) data.name = this.name;
		if ( Object.keys( this.userData ).length > 0 ) data.userData = this.userData;

		if ( this.parameters !== undefined ) {

			const parameters = this.parameters;

			for ( const key in parameters ) {

				if ( parameters[ key ] !== undefined ) data[ key ] = parameters[ key ];

			}

			return data;

		}

		// for simplicity the code assumes attributes are not shared across geometries, see #15811

		data.data = { attributes: {} };

		const index = this.index;

		if ( index !== null ) {

			data.data.index = {
				type: index.array.constructor.name,
				array: Array.prototype.slice.call( index.array )
			};

		}

		const attributes = this.attributes;

		for ( const key in attributes ) {

			const attribute = attributes[ key ];

			data.data.attributes[ key ] = attribute.toJSON( data.data );

		}

		const morphAttributes = {};
		let hasMorphAttributes = false;

		for ( const key in this.morphAttributes ) {

			const attributeArray = this.morphAttributes[ key ];

			const array = [];

			for ( let i = 0, il = attributeArray.length; i < il; i ++ ) {

				const attribute = attributeArray[ i ];

				array.push( attribute.toJSON( data.data ) );

			}

			if ( array.length > 0 ) {

				morphAttributes[ key ] = array;

				hasMorphAttributes = true;

			}

		}

		if ( hasMorphAttributes ) {

			data.data.morphAttributes = morphAttributes;
			data.data.morphTargetsRelative = this.morphTargetsRelative;

		}

		const groups = this.groups;

		if ( groups.length > 0 ) {

			data.data.groups = JSON.parse( JSON.stringify( groups ) );

		}

		const boundingSphere = this.boundingSphere;

		if ( boundingSphere !== null ) {

			data.data.boundingSphere = {
				center: boundingSphere.center.toArray(),
				radius: boundingSphere.radius
			};

		}

		return data;

	},

	clone: function () {

		/*
		 // Handle primitives

		 const parameters = this.parameters;

		 if ( parameters !== undefined ) {

		 const values = [];

		 for ( const key in parameters ) {

		 values.push( parameters[ key ] );

		 }

		 const geometry = Object.create( this.constructor.prototype );
		 this.constructor.apply( geometry, values );
		 return geometry;

		 }

		 return new this.constructor().copy( this );
		 */

		return new BufferGeometry().copy( this );

	},

	copy: function ( source ) {

		// reset

		this.index = null;
		this.attributes = {};
		this.morphAttributes = {};
		this.groups = [];
		this.boundingBox = null;
		this.boundingSphere = null;

		// used for storing cloned, shared data

		const data = {};

		// name

		this.name = source.name;

		// index

		const index = source.index;

		if ( index !== null ) {

			this.setIndex( index.clone( data ) );

		}

		// attributes

		const attributes = source.attributes;

		for ( const name in attributes ) {

			const attribute = attributes[ name ];
			this.setAttribute( name, attribute.clone( data ) );

		}

		// morph attributes

		const morphAttributes = source.morphAttributes;

		for ( const name in morphAttributes ) {

			const array = [];
			const morphAttribute = morphAttributes[ name ]; // morphAttribute: array of Float32BufferAttributes

			for ( let i = 0, l = morphAttribute.length; i < l; i ++ ) {

				array.push( morphAttribute[ i ].clone( data ) );

			}

			this.morphAttributes[ name ] = array;

		}

		this.morphTargetsRelative = source.morphTargetsRelative;

		// groups

		const groups = source.groups;

		for ( let i = 0, l = groups.length; i < l; i ++ ) {

			const group = groups[ i ];
			this.addGroup( group.start, group.count, group.materialIndex );

		}

		// bounding box

		const boundingBox = source.boundingBox;

		if ( boundingBox !== null ) {

			this.boundingBox = boundingBox.clone();

		}

		// bounding sphere

		const boundingSphere = source.boundingSphere;

		if ( boundingSphere !== null ) {

			this.boundingSphere = boundingSphere.clone();

		}

		// draw range

		this.drawRange.start = source.drawRange.start;
		this.drawRange.count = source.drawRange.count;

		// user data

		this.userData = source.userData;

		return this;

	},

	dispose: function () {

		this.dispatchEvent( { type: 'dispose' } );

	}

} );

const _inverseMatrix$2 = new Matrix4();
const _ray$2 = new Ray();
const _sphere$3 = new Sphere();

const _vA$1 = new Vector3();
const _vB$1 = new Vector3();
const _vC$1 = new Vector3();

const _tempA = new Vector3();
const _tempB = new Vector3();
const _tempC = new Vector3();

const _morphA = new Vector3();
const _morphB = new Vector3();
const _morphC = new Vector3();

const _uvA$1 = new Vector2();
const _uvB$1 = new Vector2();
const _uvC$1 = new Vector2();

const _intersectionPoint = new Vector3();
const _intersectionPointWorld = new Vector3();

function Mesh( geometry = new BufferGeometry(), material = new MeshBasicMaterial() ) {

	Object3D.call( this );

	this.type = 'Mesh';

	this.geometry = geometry;
	this.material = material;

	this.updateMorphTargets();

}

Mesh.prototype = Object.assign( Object.create( Object3D.prototype ), {

	constructor: Mesh,

	isMesh: true,

	copy: function ( source ) {

		Object3D.prototype.copy.call( this, source );

		if ( source.morphTargetInfluences !== undefined ) {

			this.morphTargetInfluences = source.morphTargetInfluences.slice();

		}

		if ( source.morphTargetDictionary !== undefined ) {

			this.morphTargetDictionary = Object.assign( {}, source.morphTargetDictionary );

		}

		this.material = source.material;
		this.geometry = source.geometry;

		return this;

	},

	updateMorphTargets: function () {

		const geometry = this.geometry;

		if ( geometry.isBufferGeometry ) {

			const morphAttributes = geometry.morphAttributes;
			const keys = Object.keys( morphAttributes );

			if ( keys.length > 0 ) {

				const morphAttribute = morphAttributes[ keys[ 0 ] ];

				if ( morphAttribute !== undefined ) {

					this.morphTargetInfluences = [];
					this.morphTargetDictionary = {};

					for ( let m = 0, ml = morphAttribute.length; m < ml; m ++ ) {

						const name = morphAttribute[ m ].name || String( m );

						this.morphTargetInfluences.push( 0 );
						this.morphTargetDictionary[ name ] = m;

					}

				}

			}

		} else {

			const morphTargets = geometry.morphTargets;

			if ( morphTargets !== undefined && morphTargets.length > 0 ) {

				console.error( 'THREE.Mesh.updateMorphTargets() no longer supports THREE.Geometry. Use THREE.BufferGeometry instead.' );

			}

		}

	},

	raycast: function ( raycaster, intersects ) {

		const geometry = this.geometry;
		const material = this.material;
		const matrixWorld = this.matrixWorld;

		if ( material === undefined ) return;

		// Checking boundingSphere distance to ray

		if ( geometry.boundingSphere === null ) geometry.computeBoundingSphere();

		_sphere$3.copy( geometry.boundingSphere );
		_sphere$3.applyMatrix4( matrixWorld );

		if ( raycaster.ray.intersectsSphere( _sphere$3 ) === false ) return;

		//

		_inverseMatrix$2.copy( matrixWorld ).invert();
		_ray$2.copy( raycaster.ray ).applyMatrix4( _inverseMatrix$2 );

		// Check boundingBox before continuing

		if ( geometry.boundingBox !== null ) {

			if ( _ray$2.intersectsBox( geometry.boundingBox ) === false ) return;

		}

		let intersection;

		if ( geometry.isBufferGeometry ) {

			const index = geometry.index;
			const position = geometry.attributes.position;
			const morphPosition = geometry.morphAttributes.position;
			const morphTargetsRelative = geometry.morphTargetsRelative;
			const uv = geometry.attributes.uv;
			const uv2 = geometry.attributes.uv2;
			const groups = geometry.groups;
			const drawRange = geometry.drawRange;

			if ( index !== null ) {

				// indexed buffer geometry

				if ( Array.isArray( material ) ) {

					for ( let i = 0, il = groups.length; i < il; i ++ ) {

						const group = groups[ i ];
						const groupMaterial = material[ group.materialIndex ];

						const start = Math.max( group.start, drawRange.start );
						const end = Math.min( ( group.start + group.count ), ( drawRange.start + drawRange.count ) );

						for ( let j = start, jl = end; j < jl; j += 3 ) {

							const a = index.getX( j );
							const b = index.getX( j + 1 );
							const c = index.getX( j + 2 );

							intersection = checkBufferGeometryIntersection( this, groupMaterial, raycaster, _ray$2, position, morphPosition, morphTargetsRelative, uv, uv2, a, b, c );

							if ( intersection ) {

								intersection.faceIndex = Math.floor( j / 3 ); // triangle number in indexed buffer semantics
								intersection.face.materialIndex = group.materialIndex;
								intersects.push( intersection );

							}

						}

					}

				} else {

					const start = Math.max( 0, drawRange.start );
					const end = Math.min( index.count, ( drawRange.start + drawRange.count ) );

					for ( let i = start, il = end; i < il; i += 3 ) {

						const a = index.getX( i );
						const b = index.getX( i + 1 );
						const c = index.getX( i + 2 );

						intersection = checkBufferGeometryIntersection( this, material, raycaster, _ray$2, position, morphPosition, morphTargetsRelative, uv, uv2, a, b, c );

						if ( intersection ) {

							intersection.faceIndex = Math.floor( i / 3 ); // triangle number in indexed buffer semantics
							intersects.push( intersection );

						}

					}

				}

			} else if ( position !== undefined ) {

				// non-indexed buffer geometry

				if ( Array.isArray( material ) ) {

					for ( let i = 0, il = groups.length; i < il; i ++ ) {

						const group = groups[ i ];
						const groupMaterial = material[ group.materialIndex ];

						const start = Math.max( group.start, drawRange.start );
						const end = Math.min( ( group.start + group.count ), ( drawRange.start + drawRange.count ) );

						for ( let j = start, jl = end; j < jl; j += 3 ) {

							const a = j;
							const b = j + 1;
							const c = j + 2;

							intersection = checkBufferGeometryIntersection( this, groupMaterial, raycaster, _ray$2, position, morphPosition, morphTargetsRelative, uv, uv2, a, b, c );

							if ( intersection ) {

								intersection.faceIndex = Math.floor( j / 3 ); // triangle number in non-indexed buffer semantics
								intersection.face.materialIndex = group.materialIndex;
								intersects.push( intersection );

							}

						}

					}

				} else {

					const start = Math.max( 0, drawRange.start );
					const end = Math.min( position.count, ( drawRange.start + drawRange.count ) );

					for ( let i = start, il = end; i < il; i += 3 ) {

						const a = i;
						const b = i + 1;
						const c = i + 2;

						intersection = checkBufferGeometryIntersection( this, material, raycaster, _ray$2, position, morphPosition, morphTargetsRelative, uv, uv2, a, b, c );

						if ( intersection ) {

							intersection.faceIndex = Math.floor( i / 3 ); // triangle number in non-indexed buffer semantics
							intersects.push( intersection );

						}

					}

				}

			}

		} else if ( geometry.isGeometry ) {

			console.error( 'THREE.Mesh.raycast() no longer supports THREE.Geometry. Use THREE.BufferGeometry instead.' );

		}

	}

} );

function checkIntersection( object, material, raycaster, ray, pA, pB, pC, point ) {

	let intersect;

	if ( material.side === BackSide ) {

		intersect = ray.intersectTriangle( pC, pB, pA, true, point );

	} else {

		intersect = ray.intersectTriangle( pA, pB, pC, material.side !== DoubleSide, point );

	}

	if ( intersect === null ) return null;

	_intersectionPointWorld.copy( point );
	_intersectionPointWorld.applyMatrix4( object.matrixWorld );

	const distance = raycaster.ray.origin.distanceTo( _intersectionPointWorld );

	if ( distance < raycaster.near || distance > raycaster.far ) return null;

	return {
		distance: distance,
		point: _intersectionPointWorld.clone(),
		object: object
	};

}

function checkBufferGeometryIntersection( object, material, raycaster, ray, position, morphPosition, morphTargetsRelative, uv, uv2, a, b, c ) {

	_vA$1.fromBufferAttribute( position, a );
	_vB$1.fromBufferAttribute( position, b );
	_vC$1.fromBufferAttribute( position, c );

	const morphInfluences = object.morphTargetInfluences;

	if ( material.morphTargets && morphPosition && morphInfluences ) {

		_morphA.set( 0, 0, 0 );
		_morphB.set( 0, 0, 0 );
		_morphC.set( 0, 0, 0 );

		for ( let i = 0, il = morphPosition.length; i < il; i ++ ) {

			const influence = morphInfluences[ i ];
			const morphAttribute = morphPosition[ i ];

			if ( influence === 0 ) continue;

			_tempA.fromBufferAttribute( morphAttribute, a );
			_tempB.fromBufferAttribute( morphAttribute, b );
			_tempC.fromBufferAttribute( morphAttribute, c );

			if ( morphTargetsRelative ) {

				_morphA.addScaledVector( _tempA, influence );
				_morphB.addScaledVector( _tempB, influence );
				_morphC.addScaledVector( _tempC, influence );

			} else {

				_morphA.addScaledVector( _tempA.sub( _vA$1 ), influence );
				_morphB.addScaledVector( _tempB.sub( _vB$1 ), influence );
				_morphC.addScaledVector( _tempC.sub( _vC$1 ), influence );

			}

		}

		_vA$1.add( _morphA );
		_vB$1.add( _morphB );
		_vC$1.add( _morphC );

	}

	if ( object.isSkinnedMesh && material.skinning ) {

		object.boneTransform( a, _vA$1 );
		object.boneTransform( b, _vB$1 );
		object.boneTransform( c, _vC$1 );

	}

	const intersection = checkIntersection( object, material, raycaster, ray, _vA$1, _vB$1, _vC$1, _intersectionPoint );

	if ( intersection ) {

		if ( uv ) {

			_uvA$1.fromBufferAttribute( uv, a );
			_uvB$1.fromBufferAttribute( uv, b );
			_uvC$1.fromBufferAttribute( uv, c );

			intersection.uv = Triangle.getUV( _intersectionPoint, _vA$1, _vB$1, _vC$1, _uvA$1, _uvB$1, _uvC$1, new Vector2() );

		}

		if ( uv2 ) {

			_uvA$1.fromBufferAttribute( uv2, a );
			_uvB$1.fromBufferAttribute( uv2, b );
			_uvC$1.fromBufferAttribute( uv2, c );

			intersection.uv2 = Triangle.getUV( _intersectionPoint, _vA$1, _vB$1, _vC$1, _uvA$1, _uvB$1, _uvC$1, new Vector2() );

		}

		const face = {
			a: a,
			b: b,
			c: c,
			normal: new Vector3(),
			materialIndex: 0
		};

		Triangle.getNormal( _vA$1, _vB$1, _vC$1, face.normal );

		intersection.face = face;

	}

	return intersection;

}

class BoxGeometry extends BufferGeometry {

	constructor( width = 1, height = 1, depth = 1, widthSegments = 1, heightSegments = 1, depthSegments = 1 ) {

		super();

		this.type = 'BoxGeometry';

		this.parameters = {
			width: width,
			height: height,
			depth: depth,
			widthSegments: widthSegments,
			heightSegments: heightSegments,
			depthSegments: depthSegments
		};

		const scope = this;

		// segments

		widthSegments = Math.floor( widthSegments );
		heightSegments = Math.floor( heightSegments );
		depthSegments = Math.floor( depthSegments );

		// buffers

		const indices = [];
		const vertices = [];
		const normals = [];
		const uvs = [];

		// helper variables

		let numberOfVertices = 0;
		let groupStart = 0;

		// build each side of the box geometry

		buildPlane( 'z', 'y', 'x', - 1, - 1, depth, height, width, depthSegments, heightSegments, 0 ); // px
		buildPlane( 'z', 'y', 'x', 1, - 1, depth, height, - width, depthSegments, heightSegments, 1 ); // nx
		buildPlane( 'x', 'z', 'y', 1, 1, width, depth, height, widthSegments, depthSegments, 2 ); // py
		buildPlane( 'x', 'z', 'y', 1, - 1, width, depth, - height, widthSegments, depthSegments, 3 ); // ny
		buildPlane( 'x', 'y', 'z', 1, - 1, width, height, depth, widthSegments, heightSegments, 4 ); // pz
		buildPlane( 'x', 'y', 'z', - 1, - 1, width, height, - depth, widthSegments, heightSegments, 5 ); // nz

		// build geometry

		this.setIndex( indices );
		this.setAttribute( 'position', new Float32BufferAttribute( vertices, 3 ) );
		this.setAttribute( 'normal', new Float32BufferAttribute( normals, 3 ) );
		this.setAttribute( 'uv', new Float32BufferAttribute( uvs, 2 ) );

		function buildPlane( u, v, w, udir, vdir, width, height, depth, gridX, gridY, materialIndex ) {

			const segmentWidth = width / gridX;
			const segmentHeight = height / gridY;

			const widthHalf = width / 2;
			const heightHalf = height / 2;
			const depthHalf = depth / 2;

			const gridX1 = gridX + 1;
			const gridY1 = gridY + 1;

			let vertexCounter = 0;
			let groupCount = 0;

			const vector = new Vector3();

			// generate vertices, normals and uvs

			for ( let iy = 0; iy < gridY1; iy ++ ) {

				const y = iy * segmentHeight - heightHalf;

				for ( let ix = 0; ix < gridX1; ix ++ ) {

					const x = ix * segmentWidth - widthHalf;

					// set values to correct vector component

					vector[ u ] = x * udir;
					vector[ v ] = y * vdir;
					vector[ w ] = depthHalf;

					// now apply vector to vertex buffer

					vertices.push( vector.x, vector.y, vector.z );

					// set values to correct vector component

					vector[ u ] = 0;
					vector[ v ] = 0;
					vector[ w ] = depth > 0 ? 1 : - 1;

					// now apply vector to normal buffer

					normals.push( vector.x, vector.y, vector.z );

					// uvs

					uvs.push( ix / gridX );
					uvs.push( 1 - ( iy / gridY ) );

					// counters

					vertexCounter += 1;

				}

			}

			// indices

			// 1. you need three indices to draw a single face
			// 2. a single segment consists of two faces
			// 3. so we need to generate six (2*3) indices per segment

			for ( let iy = 0; iy < gridY; iy ++ ) {

				for ( let ix = 0; ix < gridX; ix ++ ) {

					const a = numberOfVertices + ix + gridX1 * iy;
					const b = numberOfVertices + ix + gridX1 * ( iy + 1 );
					const c = numberOfVertices + ( ix + 1 ) + gridX1 * ( iy + 1 );
					const d = numberOfVertices + ( ix + 1 ) + gridX1 * iy;

					// faces

					indices.push( a, b, d );
					indices.push( b, c, d );

					// increase counter

					groupCount += 6;

				}

			}

			// add a group to the geometry. this will ensure multi material support

			scope.addGroup( groupStart, groupCount, materialIndex );

			// calculate new start value for groups

			groupStart += groupCount;

			// update total number of vertices

			numberOfVertices += vertexCounter;

		}

	}

}

/**
 * Uniform Utilities
 */

function cloneUniforms( src ) {

	const dst = {};

	for ( const u in src ) {

		dst[ u ] = {};

		for ( const p in src[ u ] ) {

			const property = src[ u ][ p ];

			if ( property && ( property.isColor ||
				property.isMatrix3 || property.isMatrix4 ||
				property.isVector2 || property.isVector3 || property.isVector4 ||
				property.isTexture || property.isQuaternion ) ) {

				dst[ u ][ p ] = property.clone();

			} else if ( Array.isArray( property ) ) {

				dst[ u ][ p ] = property.slice();

			} else {

				dst[ u ][ p ] = property;

			}

		}

	}

	return dst;

}

function mergeUniforms( uniforms ) {

	const merged = {};

	for ( let u = 0; u < uniforms.length; u ++ ) {

		const tmp = cloneUniforms( uniforms[ u ] );

		for ( const p in tmp ) {

			merged[ p ] = tmp[ p ];

		}

	}

	return merged;

}

// Legacy

const UniformsUtils = { clone: cloneUniforms, merge: mergeUniforms };

var default_vertex = "void main() {\n\tgl_Position = projectionMatrix * modelViewMatrix * vec4( position, 1.0 );\n}";

var default_fragment = "void main() {\n\tgl_FragColor = vec4( 1.0, 0.0, 0.0, 1.0 );\n}";

/**
 * parameters = {
 *  defines: { "label" : "value" },
 *  uniforms: { "parameter1": { value: 1.0 }, "parameter2": { value2: 2 } },
 *
 *  fragmentShader: <string>,
 *  vertexShader: <string>,
 *
 *  wireframe: <boolean>,
 *  wireframeLinewidth: <float>,
 *
 *  lights: <bool>,
 *
 *  skinning: <bool>,
 *  morphTargets: <bool>,
 *  morphNormals: <bool>
 * }
 */

function ShaderMaterial( parameters ) {

	Material.call( this );

	this.type = 'ShaderMaterial';

	this.defines = {};
	this.uniforms = {};

	this.vertexShader = default_vertex;
	this.fragmentShader = default_fragment;

	this.linewidth = 1;

	this.wireframe = false;
	this.wireframeLinewidth = 1;

	this.fog = false; // set to use scene fog
	this.lights = false; // set to use scene lights
	this.clipping = false; // set to use user-defined clipping planes

	this.skinning = false; // set to use skinning attribute streams
	this.morphTargets = false; // set to use morph targets
	this.morphNormals = false; // set to use morph normals

	this.extensions = {
		derivatives: false, // set to use derivatives
		fragDepth: false, // set to use fragment depth values
		drawBuffers: false, // set to use draw buffers
		shaderTextureLOD: false // set to use shader texture LOD
	};

	// When rendered geometry doesn't include these attributes but the material does,
	// use these default values in WebGL. This avoids errors when buffer data is missing.
	this.defaultAttributeValues = {
		'color': [ 1, 1, 1 ],
		'uv': [ 0, 0 ],
		'uv2': [ 0, 0 ]
	};

	this.index0AttributeName = undefined;
	this.uniformsNeedUpdate = false;

	this.glslVersion = null;

	if ( parameters !== undefined ) {

		if ( parameters.attributes !== undefined ) {

			console.error( 'THREE.ShaderMaterial: attributes should now be defined in THREE.BufferGeometry instead.' );

		}

		this.setValues( parameters );

	}

}

ShaderMaterial.prototype = Object.create( Material.prototype );
ShaderMaterial.prototype.constructor = ShaderMaterial;

ShaderMaterial.prototype.isShaderMaterial = true;

ShaderMaterial.prototype.copy = function ( source ) {

	Material.prototype.copy.call( this, source );

	this.fragmentShader = source.fragmentShader;
	this.vertexShader = source.vertexShader;

	this.uniforms = cloneUniforms( source.uniforms );

	this.defines = Object.assign( {}, source.defines );

	this.wireframe = source.wireframe;
	this.wireframeLinewidth = source.wireframeLinewidth;

	this.lights = source.lights;
	this.clipping = source.clipping;

	this.skinning = source.skinning;

	this.morphTargets = source.morphTargets;
	this.morphNormals = source.morphNormals;

	this.extensions = Object.assign( {}, source.extensions );

	this.glslVersion = source.glslVersion;

	return this;

};

ShaderMaterial.prototype.toJSON = function ( meta ) {

	const data = Material.prototype.toJSON.call( this, meta );

	data.glslVersion = this.glslVersion;
	data.uniforms = {};

	for ( const name in this.uniforms ) {

		const uniform = this.uniforms[ name ];
		const value = uniform.value;

		if ( value && value.isTexture ) {

			data.uniforms[ name ] = {
				type: 't',
				value: value.toJSON( meta ).uuid
			};

		} else if ( value && value.isColor ) {

			data.uniforms[ name ] = {
				type: 'c',
				value: value.getHex()
			};

		} else if ( value && value.isVector2 ) {

			data.uniforms[ name ] = {
				type: 'v2',
				value: value.toArray()
			};

		} else if ( value && value.isVector3 ) {

			data.uniforms[ name ] = {
				type: 'v3',
				value: value.toArray()
			};

		} else if ( value && value.isVector4 ) {

			data.uniforms[ name ] = {
				type: 'v4',
				value: value.toArray()
			};

		} else if ( value && value.isMatrix3 ) {

			data.uniforms[ name ] = {
				type: 'm3',
				value: value.toArray()
			};

		} else if ( value && value.isMatrix4 ) {

			data.uniforms[ name ] = {
				type: 'm4',
				value: value.toArray()
			};

		} else {

			data.uniforms[ name ] = {
				value: value
			};

			// note: the array variants v2v, v3v, v4v, m4v and tv are not supported so far

		}

	}

	if ( Object.keys( this.defines ).length > 0 ) data.defines = this.defines;

	data.vertexShader = this.vertexShader;
	data.fragmentShader = this.fragmentShader;

	const extensions = {};

	for ( const key in this.extensions ) {

		if ( this.extensions[ key ] === true ) extensions[ key ] = true;

	}

	if ( Object.keys( extensions ).length > 0 ) data.extensions = extensions;

	return data;

};

function Camera() {

	Object3D.call( this );

	this.type = 'Camera';

	this.matrixWorldInverse = new Matrix4();

	this.projectionMatrix = new Matrix4();
	this.projectionMatrixInverse = new Matrix4();

}

Camera.prototype = Object.assign( Object.create( Object3D.prototype ), {

	constructor: Camera,

	isCamera: true,

	copy: function ( source, recursive ) {

		Object3D.prototype.copy.call( this, source, recursive );

		this.matrixWorldInverse.copy( source.matrixWorldInverse );

		this.projectionMatrix.copy( source.projectionMatrix );
		this.projectionMatrixInverse.copy( source.projectionMatrixInverse );

		return this;

	},

	getWorldDirection: function ( target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Camera: .getWorldDirection() target is now required' );
			target = new Vector3();

		}

		this.updateWorldMatrix( true, false );

		const e = this.matrixWorld.elements;

		return target.set( - e[ 8 ], - e[ 9 ], - e[ 10 ] ).normalize();

	},

	updateMatrixWorld: function ( force ) {

		Object3D.prototype.updateMatrixWorld.call( this, force );

		this.matrixWorldInverse.copy( this.matrixWorld ).invert();

	},

	updateWorldMatrix: function ( updateParents, updateChildren ) {

		Object3D.prototype.updateWorldMatrix.call( this, updateParents, updateChildren );

		this.matrixWorldInverse.copy( this.matrixWorld ).invert();

	},

	clone: function () {

		return new this.constructor().copy( this );

	}

} );

function PerspectiveCamera( fov = 50, aspect = 1, near = 0.1, far = 2000 ) {

	Camera.call( this );

	this.type = 'PerspectiveCamera';

	this.fov = fov;
	this.zoom = 1;

	this.near = near;
	this.far = far;
	this.focus = 10;

	this.aspect = aspect;
	this.view = null;

	this.filmGauge = 35;	// width of the film (default in millimeters)
	this.filmOffset = 0;	// horizontal film offset (same unit as gauge)

	this.updateProjectionMatrix();

}

PerspectiveCamera.prototype = Object.assign( Object.create( Camera.prototype ), {

	constructor: PerspectiveCamera,

	isPerspectiveCamera: true,

	copy: function ( source, recursive ) {

		Camera.prototype.copy.call( this, source, recursive );

		this.fov = source.fov;
		this.zoom = source.zoom;

		this.near = source.near;
		this.far = source.far;
		this.focus = source.focus;

		this.aspect = source.aspect;
		this.view = source.view === null ? null : Object.assign( {}, source.view );

		this.filmGauge = source.filmGauge;
		this.filmOffset = source.filmOffset;

		return this;

	},

	/**
	 * Sets the FOV by focal length in respect to the current .filmGauge.
	 *
	 * The default film gauge is 35, so that the focal length can be specified for
	 * a 35mm (full frame) camera.
	 *
	 * Values for focal length and film gauge must have the same unit.
	 */
	setFocalLength: function ( focalLength ) {

		/** see {@link http://www.bobatkins.com/photography/technical/field_of_view.html} */
		const vExtentSlope = 0.5 * this.getFilmHeight() / focalLength;

		this.fov = MathUtils.RAD2DEG * 2 * Math.atan( vExtentSlope );
		this.updateProjectionMatrix();

	},

	/**
	 * Calculates the focal length from the current .fov and .filmGauge.
	 */
	getFocalLength: function () {

		const vExtentSlope = Math.tan( MathUtils.DEG2RAD * 0.5 * this.fov );

		return 0.5 * this.getFilmHeight() / vExtentSlope;

	},

	getEffectiveFOV: function () {

		return MathUtils.RAD2DEG * 2 * Math.atan(
			Math.tan( MathUtils.DEG2RAD * 0.5 * this.fov ) / this.zoom );

	},

	getFilmWidth: function () {

		// film not completely covered in portrait format (aspect < 1)
		return this.filmGauge * Math.min( this.aspect, 1 );

	},

	getFilmHeight: function () {

		// film not completely covered in landscape format (aspect > 1)
		return this.filmGauge / Math.max( this.aspect, 1 );

	},

	/**
	 * Sets an offset in a larger frustum. This is useful for multi-window or
	 * multi-monitor/multi-machine setups.
	 *
	 * For example, if you have 3x2 monitors and each monitor is 1920x1080 and
	 * the monitors are in grid like this
	 *
	 *   +---+---+---+
	 *   | A | B | C |
	 *   +---+---+---+
	 *   | D | E | F |
	 *   +---+---+---+
	 *
	 * then for each monitor you would call it like this
	 *
	 *   const w = 1920;
	 *   const h = 1080;
	 *   const fullWidth = w * 3;
	 *   const fullHeight = h * 2;
	 *
	 *   --A--
	 *   camera.setViewOffset( fullWidth, fullHeight, w * 0, h * 0, w, h );
	 *   --B--
	 *   camera.setViewOffset( fullWidth, fullHeight, w * 1, h * 0, w, h );
	 *   --C--
	 *   camera.setViewOffset( fullWidth, fullHeight, w * 2, h * 0, w, h );
	 *   --D--
	 *   camera.setViewOffset( fullWidth, fullHeight, w * 0, h * 1, w, h );
	 *   --E--
	 *   camera.setViewOffset( fullWidth, fullHeight, w * 1, h * 1, w, h );
	 *   --F--
	 *   camera.setViewOffset( fullWidth, fullHeight, w * 2, h * 1, w, h );
	 *
	 *   Note there is no reason monitors have to be the same size or in a grid.
	 */
	setViewOffset: function ( fullWidth, fullHeight, x, y, width, height ) {

		this.aspect = fullWidth / fullHeight;

		if ( this.view === null ) {

			this.view = {
				enabled: true,
				fullWidth: 1,
				fullHeight: 1,
				offsetX: 0,
				offsetY: 0,
				width: 1,
				height: 1
			};

		}

		this.view.enabled = true;
		this.view.fullWidth = fullWidth;
		this.view.fullHeight = fullHeight;
		this.view.offsetX = x;
		this.view.offsetY = y;
		this.view.width = width;
		this.view.height = height;

		this.updateProjectionMatrix();

	},

	clearViewOffset: function () {

		if ( this.view !== null ) {

			this.view.enabled = false;

		}

		this.updateProjectionMatrix();

	},

	updateProjectionMatrix: function () {

		const near = this.near;
		let top = near * Math.tan( MathUtils.DEG2RAD * 0.5 * this.fov ) / this.zoom;
		let height = 2 * top;
		let width = this.aspect * height;
		let left = - 0.5 * width;
		const view = this.view;

		if ( this.view !== null && this.view.enabled ) {

			const fullWidth = view.fullWidth,
				fullHeight = view.fullHeight;

			left += view.offsetX * width / fullWidth;
			top -= view.offsetY * height / fullHeight;
			width *= view.width / fullWidth;
			height *= view.height / fullHeight;

		}

		const skew = this.filmOffset;
		if ( skew !== 0 ) left += near * skew / this.getFilmWidth();

		this.projectionMatrix.makePerspective( left, left + width, top, top - height, near, this.far );

		this.projectionMatrixInverse.copy( this.projectionMatrix ).invert();

	},

	toJSON: function ( meta ) {

		const data = Object3D.prototype.toJSON.call( this, meta );

		data.object.fov = this.fov;
		data.object.zoom = this.zoom;

		data.object.near = this.near;
		data.object.far = this.far;
		data.object.focus = this.focus;

		data.object.aspect = this.aspect;

		if ( this.view !== null ) data.object.view = Object.assign( {}, this.view );

		data.object.filmGauge = this.filmGauge;
		data.object.filmOffset = this.filmOffset;

		return data;

	}

} );

const fov = 90, aspect = 1;

class CubeCamera extends Object3D {

	constructor( near, far, renderTarget ) {

		super();

		this.type = 'CubeCamera';

		if ( renderTarget.isWebGLCubeRenderTarget !== true ) {

			console.error( 'THREE.CubeCamera: The constructor now expects an instance of WebGLCubeRenderTarget as third parameter.' );
			return;

		}

		this.renderTarget = renderTarget;

		const cameraPX = new PerspectiveCamera( fov, aspect, near, far );
		cameraPX.layers = this.layers;
		cameraPX.up.set( 0, - 1, 0 );
		cameraPX.lookAt( new Vector3( 1, 0, 0 ) );
		this.add( cameraPX );

		const cameraNX = new PerspectiveCamera( fov, aspect, near, far );
		cameraNX.layers = this.layers;
		cameraNX.up.set( 0, - 1, 0 );
		cameraNX.lookAt( new Vector3( - 1, 0, 0 ) );
		this.add( cameraNX );

		const cameraPY = new PerspectiveCamera( fov, aspect, near, far );
		cameraPY.layers = this.layers;
		cameraPY.up.set( 0, 0, 1 );
		cameraPY.lookAt( new Vector3( 0, 1, 0 ) );
		this.add( cameraPY );

		const cameraNY = new PerspectiveCamera( fov, aspect, near, far );
		cameraNY.layers = this.layers;
		cameraNY.up.set( 0, 0, - 1 );
		cameraNY.lookAt( new Vector3( 0, - 1, 0 ) );
		this.add( cameraNY );

		const cameraPZ = new PerspectiveCamera( fov, aspect, near, far );
		cameraPZ.layers = this.layers;
		cameraPZ.up.set( 0, - 1, 0 );
		cameraPZ.lookAt( new Vector3( 0, 0, 1 ) );
		this.add( cameraPZ );

		const cameraNZ = new PerspectiveCamera( fov, aspect, near, far );
		cameraNZ.layers = this.layers;
		cameraNZ.up.set( 0, - 1, 0 );
		cameraNZ.lookAt( new Vector3( 0, 0, - 1 ) );
		this.add( cameraNZ );

	}

	update( renderer, scene ) {

		if ( this.parent === null ) this.updateMatrixWorld();

		const renderTarget = this.renderTarget;

		const [ cameraPX, cameraNX, cameraPY, cameraNY, cameraPZ, cameraNZ ] = this.children;

		const currentXrEnabled = renderer.xr.enabled;
		const currentRenderTarget = renderer.getRenderTarget();

		renderer.xr.enabled = false;

		const generateMipmaps = renderTarget.texture.generateMipmaps;

		renderTarget.texture.generateMipmaps = false;

		renderer.setRenderTarget( renderTarget, 0 );
		renderer.render( scene, cameraPX );

		renderer.setRenderTarget( renderTarget, 1 );
		renderer.render( scene, cameraNX );

		renderer.setRenderTarget( renderTarget, 2 );
		renderer.render( scene, cameraPY );

		renderer.setRenderTarget( renderTarget, 3 );
		renderer.render( scene, cameraNY );

		renderer.setRenderTarget( renderTarget, 4 );
		renderer.render( scene, cameraPZ );

		renderTarget.texture.generateMipmaps = generateMipmaps;

		renderer.setRenderTarget( renderTarget, 5 );
		renderer.render( scene, cameraNZ );

		renderer.setRenderTarget( currentRenderTarget );

		renderer.xr.enabled = currentXrEnabled;

	}

}

class CubeTexture extends Texture {

	constructor( images, mapping, wrapS, wrapT, magFilter, minFilter, format, type, anisotropy, encoding ) {

		images = images !== undefined ? images : [];
		mapping = mapping !== undefined ? mapping : CubeReflectionMapping;
		format = format !== undefined ? format : RGBFormat;

		super( images, mapping, wrapS, wrapT, magFilter, minFilter, format, type, anisotropy, encoding );

		// Why CubeTexture._needsFlipEnvMap is necessary:
		//
		// By convention -- likely based on the RenderMan spec from the 1990's -- cube maps are specified by WebGL (and three.js)
		// in a coordinate system in which positive-x is to the right when looking up the positive-z axis -- in other words,
		// in a left-handed coordinate system. By continuing this convention, preexisting cube maps continued to render correctly.

		// three.js uses a right-handed coordinate system. So environment maps used in three.js appear to have px and nx swapped
		// and the flag _needsFlipEnvMap controls this conversion. The flip is not required (and thus _needsFlipEnvMap is set to false)
		// when using WebGLCubeRenderTarget.texture as a cube texture.

		this._needsFlipEnvMap = true;

		this.flipY = false;

	}

	get images() {

		return this.image;

	}

	set images( value ) {

		this.image = value;

	}

}

CubeTexture.prototype.isCubeTexture = true;

class WebGLCubeRenderTarget extends WebGLRenderTarget {

	constructor( size, options, dummy ) {

		if ( Number.isInteger( options ) ) {

			console.warn( 'THREE.WebGLCubeRenderTarget: constructor signature is now WebGLCubeRenderTarget( size, options )' );

			options = dummy;

		}

		super( size, size, options );

		options = options || {};

		this.texture = new CubeTexture( undefined, options.mapping, options.wrapS, options.wrapT, options.magFilter, options.minFilter, options.format, options.type, options.anisotropy, options.encoding );

		this.texture.generateMipmaps = options.generateMipmaps !== undefined ? options.generateMipmaps : false;
		this.texture.minFilter = options.minFilter !== undefined ? options.minFilter : LinearFilter;

		this.texture._needsFlipEnvMap = false;

	}

	fromEquirectangularTexture( renderer, texture ) {

		this.texture.type = texture.type;
		this.texture.format = RGBAFormat; // see #18859
		this.texture.encoding = texture.encoding;

		this.texture.generateMipmaps = texture.generateMipmaps;
		this.texture.minFilter = texture.minFilter;
		this.texture.magFilter = texture.magFilter;

		const shader = {

			uniforms: {
				tEquirect: { value: null },
			},

			vertexShader: /* glsl */`

				varying vec3 vWorldDirection;

				vec3 transformDirection( in vec3 dir, in mat4 matrix ) {

					return normalize( ( matrix * vec4( dir, 0.0 ) ).xyz );

				}

				void main() {

					vWorldDirection = transformDirection( position, modelMatrix );

					#include <begin_vertex>
					#include <project_vertex>

				}
			`,

			fragmentShader: /* glsl */`

				uniform sampler2D tEquirect;

				varying vec3 vWorldDirection;

				#include <common>

				void main() {

					vec3 direction = normalize( vWorldDirection );

					vec2 sampleUV = equirectUv( direction );

					gl_FragColor = texture2D( tEquirect, sampleUV );

				}
			`
		};

		const geometry = new BoxGeometry( 5, 5, 5 );

		const material = new ShaderMaterial( {

			name: 'CubemapFromEquirect',

			uniforms: cloneUniforms( shader.uniforms ),
			vertexShader: shader.vertexShader,
			fragmentShader: shader.fragmentShader,
			side: BackSide,
			blending: NoBlending

		} );

		material.uniforms.tEquirect.value = texture;

		const mesh = new Mesh( geometry, material );

		const currentMinFilter = texture.minFilter;

		// Avoid blurred poles
		if ( texture.minFilter === LinearMipmapLinearFilter ) texture.minFilter = LinearFilter;

		const camera = new CubeCamera( 1, 10, this );
		camera.update( renderer, mesh );

		texture.minFilter = currentMinFilter;

		mesh.geometry.dispose();
		mesh.material.dispose();

		return this;

	}

	clear( renderer, color, depth, stencil ) {

		const currentRenderTarget = renderer.getRenderTarget();

		for ( let i = 0; i < 6; i ++ ) {

			renderer.setRenderTarget( this, i );

			renderer.clear( color, depth, stencil );

		}

		renderer.setRenderTarget( currentRenderTarget );

	}

}

WebGLCubeRenderTarget.prototype.isWebGLCubeRenderTarget = true;

class DataTexture extends Texture {

	constructor( data, width, height, format, type, mapping, wrapS, wrapT, magFilter, minFilter, anisotropy, encoding ) {

		super( null, mapping, wrapS, wrapT, magFilter, minFilter, format, type, anisotropy, encoding );

		this.image = { data: data || null, width: width || 1, height: height || 1 };

		this.magFilter = magFilter !== undefined ? magFilter : NearestFilter;
		this.minFilter = minFilter !== undefined ? minFilter : NearestFilter;

		this.generateMipmaps = false;
		this.flipY = false;
		this.unpackAlignment = 1;

		this.needsUpdate = true;

	}

}

DataTexture.prototype.isDataTexture = true;

const _sphere$2 = /*@__PURE__*/ new Sphere();
const _vector$7 = /*@__PURE__*/ new Vector3();

class Frustum {

	constructor( p0 = new Plane(), p1 = new Plane(), p2 = new Plane(), p3 = new Plane(), p4 = new Plane(), p5 = new Plane() ) {

		this.planes = [ p0, p1, p2, p3, p4, p5 ];

	}

	set( p0, p1, p2, p3, p4, p5 ) {

		const planes = this.planes;

		planes[ 0 ].copy( p0 );
		planes[ 1 ].copy( p1 );
		planes[ 2 ].copy( p2 );
		planes[ 3 ].copy( p3 );
		planes[ 4 ].copy( p4 );
		planes[ 5 ].copy( p5 );

		return this;

	}

	copy( frustum ) {

		const planes = this.planes;

		for ( let i = 0; i < 6; i ++ ) {

			planes[ i ].copy( frustum.planes[ i ] );

		}

		return this;

	}

	setFromProjectionMatrix( m ) {

		const planes = this.planes;
		const me = m.elements;
		const me0 = me[ 0 ], me1 = me[ 1 ], me2 = me[ 2 ], me3 = me[ 3 ];
		const me4 = me[ 4 ], me5 = me[ 5 ], me6 = me[ 6 ], me7 = me[ 7 ];
		const me8 = me[ 8 ], me9 = me[ 9 ], me10 = me[ 10 ], me11 = me[ 11 ];
		const me12 = me[ 12 ], me13 = me[ 13 ], me14 = me[ 14 ], me15 = me[ 15 ];

		planes[ 0 ].setComponents( me3 - me0, me7 - me4, me11 - me8, me15 - me12 ).normalize();
		planes[ 1 ].setComponents( me3 + me0, me7 + me4, me11 + me8, me15 + me12 ).normalize();
		planes[ 2 ].setComponents( me3 + me1, me7 + me5, me11 + me9, me15 + me13 ).normalize();
		planes[ 3 ].setComponents( me3 - me1, me7 - me5, me11 - me9, me15 - me13 ).normalize();
		planes[ 4 ].setComponents( me3 - me2, me7 - me6, me11 - me10, me15 - me14 ).normalize();
		planes[ 5 ].setComponents( me3 + me2, me7 + me6, me11 + me10, me15 + me14 ).normalize();

		return this;

	}

	intersectsObject( object ) {

		const geometry = object.geometry;

		if ( geometry.boundingSphere === null ) geometry.computeBoundingSphere();

		_sphere$2.copy( geometry.boundingSphere ).applyMatrix4( object.matrixWorld );

		return this.intersectsSphere( _sphere$2 );

	}

	intersectsSprite( sprite ) {

		_sphere$2.center.set( 0, 0, 0 );
		_sphere$2.radius = 0.7071067811865476;
		_sphere$2.applyMatrix4( sprite.matrixWorld );

		return this.intersectsSphere( _sphere$2 );

	}

	intersectsSphere( sphere ) {

		const planes = this.planes;
		const center = sphere.center;
		const negRadius = - sphere.radius;

		for ( let i = 0; i < 6; i ++ ) {

			const distance = planes[ i ].distanceToPoint( center );

			if ( distance < negRadius ) {

				return false;

			}

		}

		return true;

	}

	intersectsBox( box ) {

		const planes = this.planes;

		for ( let i = 0; i < 6; i ++ ) {

			const plane = planes[ i ];

			// corner at max distance

			_vector$7.x = plane.normal.x > 0 ? box.max.x : box.min.x;
			_vector$7.y = plane.normal.y > 0 ? box.max.y : box.min.y;
			_vector$7.z = plane.normal.z > 0 ? box.max.z : box.min.z;

			if ( plane.distanceToPoint( _vector$7 ) < 0 ) {

				return false;

			}

		}

		return true;

	}

	containsPoint( point ) {

		const planes = this.planes;

		for ( let i = 0; i < 6; i ++ ) {

			if ( planes[ i ].distanceToPoint( point ) < 0 ) {

				return false;

			}

		}

		return true;

	}

	clone() {

		return new this.constructor().copy( this );

	}

}

function WebGLAnimation() {

	let context = null;
	let isAnimating = false;
	let animationLoop = null;
	let requestId = null;

	function onAnimationFrame( time, frame ) {

		animationLoop( time, frame );

		requestId = context.requestAnimationFrame( onAnimationFrame );

	}

	return {

		start: function () {

			if ( isAnimating === true ) return;
			if ( animationLoop === null ) return;

			requestId = context.requestAnimationFrame( onAnimationFrame );

			isAnimating = true;

		},

		stop: function () {

			context.cancelAnimationFrame( requestId );

			isAnimating = false;

		},

		setAnimationLoop: function ( callback ) {

			animationLoop = callback;

		},

		setContext: function ( value ) {

			context = value;

		}

	};

}

function WebGLAttributes( gl, capabilities ) {

	const isWebGL2 = capabilities.isWebGL2;

	const buffers = new WeakMap();

	function createBuffer( attribute, bufferType ) {

		const array = attribute.array;
		const usage = attribute.usage;

		const buffer = gl.createBuffer();

		gl.bindBuffer( bufferType, buffer );
		gl.bufferData( bufferType, array, usage );

		attribute.onUploadCallback();

		let type = 5126;

		if ( array instanceof Float32Array ) {

			type = 5126;

		} else if ( array instanceof Float64Array ) {

			console.warn( 'THREE.WebGLAttributes: Unsupported data buffer format: Float64Array.' );

		} else if ( array instanceof Uint16Array ) {

			if ( attribute.isFloat16BufferAttribute ) {

				if ( isWebGL2 ) {

					type = 5131;

				} else {

					console.warn( 'THREE.WebGLAttributes: Usage of Float16BufferAttribute requires WebGL2.' );

				}

			} else {

				type = 5123;

			}

		} else if ( array instanceof Int16Array ) {

			type = 5122;

		} else if ( array instanceof Uint32Array ) {

			type = 5125;

		} else if ( array instanceof Int32Array ) {

			type = 5124;

		} else if ( array instanceof Int8Array ) {

			type = 5120;

		} else if ( array instanceof Uint8Array ) {

			type = 5121;

		}

		return {
			buffer: buffer,
			type: type,
			bytesPerElement: array.BYTES_PER_ELEMENT,
			version: attribute.version
		};

	}

	function updateBuffer( buffer, attribute, bufferType ) {

		const array = attribute.array;
		const updateRange = attribute.updateRange;

		gl.bindBuffer( bufferType, buffer );

		if ( updateRange.count === - 1 ) {

			// Not using update ranges

			gl.bufferSubData( bufferType, 0, array );

		} else {

			if ( isWebGL2 ) {

				gl.bufferSubData( bufferType, updateRange.offset * array.BYTES_PER_ELEMENT,
					array, updateRange.offset, updateRange.count );

			} else {

				gl.bufferSubData( bufferType, updateRange.offset * array.BYTES_PER_ELEMENT,
					array.subarray( updateRange.offset, updateRange.offset + updateRange.count ) );

			}

			updateRange.count = - 1; // reset range

		}

	}

	//

	function get( attribute ) {

		if ( attribute.isInterleavedBufferAttribute ) attribute = attribute.data;

		return buffers.get( attribute );

	}

	function remove( attribute ) {

		if ( attribute.isInterleavedBufferAttribute ) attribute = attribute.data;

		const data = buffers.get( attribute );

		if ( data ) {

			gl.deleteBuffer( data.buffer );

			buffers.delete( attribute );

		}

	}

	function update( attribute, bufferType ) {

		if ( attribute.isGLBufferAttribute ) {

			const cached = buffers.get( attribute );

			if ( ! cached || cached.version < attribute.version ) {

				buffers.set( attribute, {
					buffer: attribute.buffer,
					type: attribute.type,
					bytesPerElement: attribute.elementSize,
					version: attribute.version
				} );

			}

			return;

		}

		if ( attribute.isInterleavedBufferAttribute ) attribute = attribute.data;

		const data = buffers.get( attribute );

		if ( data === undefined ) {

			buffers.set( attribute, createBuffer( attribute, bufferType ) );

		} else if ( data.version < attribute.version ) {

			updateBuffer( data.buffer, attribute, bufferType );

			data.version = attribute.version;

		}

	}

	return {

		get: get,
		remove: remove,
		update: update

	};

}

class PlaneGeometry extends BufferGeometry {

	constructor( width = 1, height = 1, widthSegments = 1, heightSegments = 1 ) {

		super();
		this.type = 'PlaneGeometry';

		this.parameters = {
			width: width,
			height: height,
			widthSegments: widthSegments,
			heightSegments: heightSegments
		};

		const width_half = width / 2;
		const height_half = height / 2;

		const gridX = Math.floor( widthSegments );
		const gridY = Math.floor( heightSegments );

		const gridX1 = gridX + 1;
		const gridY1 = gridY + 1;

		const segment_width = width / gridX;
		const segment_height = height / gridY;

		//

		const indices = [];
		const vertices = [];
		const normals = [];
		const uvs = [];

		for ( let iy = 0; iy < gridY1; iy ++ ) {

			const y = iy * segment_height - height_half;

			for ( let ix = 0; ix < gridX1; ix ++ ) {

				const x = ix * segment_width - width_half;

				vertices.push( x, - y, 0 );

				normals.push( 0, 0, 1 );

				uvs.push( ix / gridX );
				uvs.push( 1 - ( iy / gridY ) );

			}

		}

		for ( let iy = 0; iy < gridY; iy ++ ) {

			for ( let ix = 0; ix < gridX; ix ++ ) {

				const a = ix + gridX1 * iy;
				const b = ix + gridX1 * ( iy + 1 );
				const c = ( ix + 1 ) + gridX1 * ( iy + 1 );
				const d = ( ix + 1 ) + gridX1 * iy;

				indices.push( a, b, d );
				indices.push( b, c, d );

			}

		}

		this.setIndex( indices );
		this.setAttribute( 'position', new Float32BufferAttribute( vertices, 3 ) );
		this.setAttribute( 'normal', new Float32BufferAttribute( normals, 3 ) );
		this.setAttribute( 'uv', new Float32BufferAttribute( uvs, 2 ) );

	}

}

var alphamap_fragment = "#ifdef USE_ALPHAMAP\n\tdiffuseColor.a *= texture2D( alphaMap, vUv ).g;\n#endif";

var alphamap_pars_fragment = "#ifdef USE_ALPHAMAP\n\tuniform sampler2D alphaMap;\n#endif";

var alphatest_fragment = "#ifdef ALPHATEST\n\tif ( diffuseColor.a < ALPHATEST ) discard;\n#endif";

var aomap_fragment = "#ifdef USE_AOMAP\n\tfloat ambientOcclusion = ( texture2D( aoMap, vUv2 ).r - 1.0 ) * aoMapIntensity + 1.0;\n\treflectedLight.indirectDiffuse *= ambientOcclusion;\n\t#if defined( USE_ENVMAP ) && defined( STANDARD )\n\t\tfloat dotNV = saturate( dot( geometry.normal, geometry.viewDir ) );\n\t\treflectedLight.indirectSpecular *= computeSpecularOcclusion( dotNV, ambientOcclusion, material.specularRoughness );\n\t#endif\n#endif";

var aomap_pars_fragment = "#ifdef USE_AOMAP\n\tuniform sampler2D aoMap;\n\tuniform float aoMapIntensity;\n#endif";

var begin_vertex = "vec3 transformed = vec3( position );";

var beginnormal_vertex = "vec3 objectNormal = vec3( normal );\n#ifdef USE_TANGENT\n\tvec3 objectTangent = vec3( tangent.xyz );\n#endif";

var bsdfs = "vec2 integrateSpecularBRDF( const in float dotNV, const in float roughness ) {\n\tconst vec4 c0 = vec4( - 1, - 0.0275, - 0.572, 0.022 );\n\tconst vec4 c1 = vec4( 1, 0.0425, 1.04, - 0.04 );\n\tvec4 r = roughness * c0 + c1;\n\tfloat a004 = min( r.x * r.x, exp2( - 9.28 * dotNV ) ) * r.x + r.y;\n\treturn vec2( -1.04, 1.04 ) * a004 + r.zw;\n}\nfloat punctualLightIntensityToIrradianceFactor( const in float lightDistance, const in float cutoffDistance, const in float decayExponent ) {\n#if defined ( PHYSICALLY_CORRECT_LIGHTS )\n\tfloat distanceFalloff = 1.0 / max( pow( lightDistance, decayExponent ), 0.01 );\n\tif( cutoffDistance > 0.0 ) {\n\t\tdistanceFalloff *= pow2( saturate( 1.0 - pow4( lightDistance / cutoffDistance ) ) );\n\t}\n\treturn distanceFalloff;\n#else\n\tif( cutoffDistance > 0.0 && decayExponent > 0.0 ) {\n\t\treturn pow( saturate( -lightDistance / cutoffDistance + 1.0 ), decayExponent );\n\t}\n\treturn 1.0;\n#endif\n}\nvec3 BRDF_Diffuse_Lambert( const in vec3 diffuseColor ) {\n\treturn RECIPROCAL_PI * diffuseColor;\n}\nvec3 F_Schlick( const in vec3 specularColor, const in float dotLH ) {\n\tfloat fresnel = exp2( ( -5.55473 * dotLH - 6.98316 ) * dotLH );\n\treturn ( 1.0 - specularColor ) * fresnel + specularColor;\n}\nvec3 F_Schlick_RoughnessDependent( const in vec3 F0, const in float dotNV, const in float roughness ) {\n\tfloat fresnel = exp2( ( -5.55473 * dotNV - 6.98316 ) * dotNV );\n\tvec3 Fr = max( vec3( 1.0 - roughness ), F0 ) - F0;\n\treturn Fr * fresnel + F0;\n}\nfloat G_GGX_Smith( const in float alpha, const in float dotNL, const in float dotNV ) {\n\tfloat a2 = pow2( alpha );\n\tfloat gl = dotNL + sqrt( a2 + ( 1.0 - a2 ) * pow2( dotNL ) );\n\tfloat gv = dotNV + sqrt( a2 + ( 1.0 - a2 ) * pow2( dotNV ) );\n\treturn 1.0 / ( gl * gv );\n}\nfloat G_GGX_SmithCorrelated( const in float alpha, const in float dotNL, const in float dotNV ) {\n\tfloat a2 = pow2( alpha );\n\tfloat gv = dotNL * sqrt( a2 + ( 1.0 - a2 ) * pow2( dotNV ) );\n\tfloat gl = dotNV * sqrt( a2 + ( 1.0 - a2 ) * pow2( dotNL ) );\n\treturn 0.5 / max( gv + gl, EPSILON );\n}\nfloat D_GGX( const in float alpha, const in float dotNH ) {\n\tfloat a2 = pow2( alpha );\n\tfloat denom = pow2( dotNH ) * ( a2 - 1.0 ) + 1.0;\n\treturn RECIPROCAL_PI * a2 / pow2( denom );\n}\nvec3 BRDF_Specular_GGX( const in IncidentLight incidentLight, const in vec3 viewDir, const in vec3 normal, const in vec3 specularColor, const in float roughness ) {\n\tfloat alpha = pow2( roughness );\n\tvec3 halfDir = normalize( incidentLight.direction + viewDir );\n\tfloat dotNL = saturate( dot( normal, incidentLight.direction ) );\n\tfloat dotNV = saturate( dot( normal, viewDir ) );\n\tfloat dotNH = saturate( dot( normal, halfDir ) );\n\tfloat dotLH = saturate( dot( incidentLight.direction, halfDir ) );\n\tvec3 F = F_Schlick( specularColor, dotLH );\n\tfloat G = G_GGX_SmithCorrelated( alpha, dotNL, dotNV );\n\tfloat D = D_GGX( alpha, dotNH );\n\treturn F * ( G * D );\n}\nvec2 LTC_Uv( const in vec3 N, const in vec3 V, const in float roughness ) {\n\tconst float LUT_SIZE = 64.0;\n\tconst float LUT_SCALE = ( LUT_SIZE - 1.0 ) / LUT_SIZE;\n\tconst float LUT_BIAS = 0.5 / LUT_SIZE;\n\tfloat dotNV = saturate( dot( N, V ) );\n\tvec2 uv = vec2( roughness, sqrt( 1.0 - dotNV ) );\n\tuv = uv * LUT_SCALE + LUT_BIAS;\n\treturn uv;\n}\nfloat LTC_ClippedSphereFormFactor( const in vec3 f ) {\n\tfloat l = length( f );\n\treturn max( ( l * l + f.z ) / ( l + 1.0 ), 0.0 );\n}\nvec3 LTC_EdgeVectorFormFactor( const in vec3 v1, const in vec3 v2 ) {\n\tfloat x = dot( v1, v2 );\n\tfloat y = abs( x );\n\tfloat a = 0.8543985 + ( 0.4965155 + 0.0145206 * y ) * y;\n\tfloat b = 3.4175940 + ( 4.1616724 + y ) * y;\n\tfloat v = a / b;\n\tfloat theta_sintheta = ( x > 0.0 ) ? v : 0.5 * inversesqrt( max( 1.0 - x * x, 1e-7 ) ) - v;\n\treturn cross( v1, v2 ) * theta_sintheta;\n}\nvec3 LTC_Evaluate( const in vec3 N, const in vec3 V, const in vec3 P, const in mat3 mInv, const in vec3 rectCoords[ 4 ] ) {\n\tvec3 v1 = rectCoords[ 1 ] - rectCoords[ 0 ];\n\tvec3 v2 = rectCoords[ 3 ] - rectCoords[ 0 ];\n\tvec3 lightNormal = cross( v1, v2 );\n\tif( dot( lightNormal, P - rectCoords[ 0 ] ) < 0.0 ) return vec3( 0.0 );\n\tvec3 T1, T2;\n\tT1 = normalize( V - N * dot( V, N ) );\n\tT2 = - cross( N, T1 );\n\tmat3 mat = mInv * transposeMat3( mat3( T1, T2, N ) );\n\tvec3 coords[ 4 ];\n\tcoords[ 0 ] = mat * ( rectCoords[ 0 ] - P );\n\tcoords[ 1 ] = mat * ( rectCoords[ 1 ] - P );\n\tcoords[ 2 ] = mat * ( rectCoords[ 2 ] - P );\n\tcoords[ 3 ] = mat * ( rectCoords[ 3 ] - P );\n\tcoords[ 0 ] = normalize( coords[ 0 ] );\n\tcoords[ 1 ] = normalize( coords[ 1 ] );\n\tcoords[ 2 ] = normalize( coords[ 2 ] );\n\tcoords[ 3 ] = normalize( coords[ 3 ] );\n\tvec3 vectorFormFactor = vec3( 0.0 );\n\tvectorFormFactor += LTC_EdgeVectorFormFactor( coords[ 0 ], coords[ 1 ] );\n\tvectorFormFactor += LTC_EdgeVectorFormFactor( coords[ 1 ], coords[ 2 ] );\n\tvectorFormFactor += LTC_EdgeVectorFormFactor( coords[ 2 ], coords[ 3 ] );\n\tvectorFormFactor += LTC_EdgeVectorFormFactor( coords[ 3 ], coords[ 0 ] );\n\tfloat result = LTC_ClippedSphereFormFactor( vectorFormFactor );\n\treturn vec3( result );\n}\nvec3 BRDF_Specular_GGX_Environment( const in vec3 viewDir, const in vec3 normal, const in vec3 specularColor, const in float roughness ) {\n\tfloat dotNV = saturate( dot( normal, viewDir ) );\n\tvec2 brdf = integrateSpecularBRDF( dotNV, roughness );\n\treturn specularColor * brdf.x + brdf.y;\n}\nvoid BRDF_Specular_Multiscattering_Environment( const in GeometricContext geometry, const in vec3 specularColor, const in float roughness, inout vec3 singleScatter, inout vec3 multiScatter ) {\n\tfloat dotNV = saturate( dot( geometry.normal, geometry.viewDir ) );\n\tvec3 F = F_Schlick_RoughnessDependent( specularColor, dotNV, roughness );\n\tvec2 brdf = integrateSpecularBRDF( dotNV, roughness );\n\tvec3 FssEss = F * brdf.x + brdf.y;\n\tfloat Ess = brdf.x + brdf.y;\n\tfloat Ems = 1.0 - Ess;\n\tvec3 Favg = specularColor + ( 1.0 - specularColor ) * 0.047619;\tvec3 Fms = FssEss * Favg / ( 1.0 - Ems * Favg );\n\tsingleScatter += FssEss;\n\tmultiScatter += Fms * Ems;\n}\nfloat G_BlinnPhong_Implicit( ) {\n\treturn 0.25;\n}\nfloat D_BlinnPhong( const in float shininess, const in float dotNH ) {\n\treturn RECIPROCAL_PI * ( shininess * 0.5 + 1.0 ) * pow( dotNH, shininess );\n}\nvec3 BRDF_Specular_BlinnPhong( const in IncidentLight incidentLight, const in GeometricContext geometry, const in vec3 specularColor, const in float shininess ) {\n\tvec3 halfDir = normalize( incidentLight.direction + geometry.viewDir );\n\tfloat dotNH = saturate( dot( geometry.normal, halfDir ) );\n\tfloat dotLH = saturate( dot( incidentLight.direction, halfDir ) );\n\tvec3 F = F_Schlick( specularColor, dotLH );\n\tfloat G = G_BlinnPhong_Implicit( );\n\tfloat D = D_BlinnPhong( shininess, dotNH );\n\treturn F * ( G * D );\n}\nfloat GGXRoughnessToBlinnExponent( const in float ggxRoughness ) {\n\treturn ( 2.0 / pow2( ggxRoughness + 0.0001 ) - 2.0 );\n}\nfloat BlinnExponentToGGXRoughness( const in float blinnExponent ) {\n\treturn sqrt( 2.0 / ( blinnExponent + 2.0 ) );\n}\n#if defined( USE_SHEEN )\nfloat D_Charlie(float roughness, float NoH) {\n\tfloat invAlpha = 1.0 / roughness;\n\tfloat cos2h = NoH * NoH;\n\tfloat sin2h = max(1.0 - cos2h, 0.0078125);\treturn (2.0 + invAlpha) * pow(sin2h, invAlpha * 0.5) / (2.0 * PI);\n}\nfloat V_Neubelt(float NoV, float NoL) {\n\treturn saturate(1.0 / (4.0 * (NoL + NoV - NoL * NoV)));\n}\nvec3 BRDF_Specular_Sheen( const in float roughness, const in vec3 L, const in GeometricContext geometry, vec3 specularColor ) {\n\tvec3 N = geometry.normal;\n\tvec3 V = geometry.viewDir;\n\tvec3 H = normalize( V + L );\n\tfloat dotNH = saturate( dot( N, H ) );\n\treturn specularColor * D_Charlie( roughness, dotNH ) * V_Neubelt( dot(N, V), dot(N, L) );\n}\n#endif";

var bumpmap_pars_fragment = "#ifdef USE_BUMPMAP\n\tuniform sampler2D bumpMap;\n\tuniform float bumpScale;\n\tvec2 dHdxy_fwd() {\n\t\tvec2 dSTdx = dFdx( vUv );\n\t\tvec2 dSTdy = dFdy( vUv );\n\t\tfloat Hll = bumpScale * texture2D( bumpMap, vUv ).x;\n\t\tfloat dBx = bumpScale * texture2D( bumpMap, vUv + dSTdx ).x - Hll;\n\t\tfloat dBy = bumpScale * texture2D( bumpMap, vUv + dSTdy ).x - Hll;\n\t\treturn vec2( dBx, dBy );\n\t}\n\tvec3 perturbNormalArb( vec3 surf_pos, vec3 surf_norm, vec2 dHdxy, float faceDirection ) {\n\t\tvec3 vSigmaX = vec3( dFdx( surf_pos.x ), dFdx( surf_pos.y ), dFdx( surf_pos.z ) );\n\t\tvec3 vSigmaY = vec3( dFdy( surf_pos.x ), dFdy( surf_pos.y ), dFdy( surf_pos.z ) );\n\t\tvec3 vN = surf_norm;\n\t\tvec3 R1 = cross( vSigmaY, vN );\n\t\tvec3 R2 = cross( vN, vSigmaX );\n\t\tfloat fDet = dot( vSigmaX, R1 ) * faceDirection;\n\t\tvec3 vGrad = sign( fDet ) * ( dHdxy.x * R1 + dHdxy.y * R2 );\n\t\treturn normalize( abs( fDet ) * surf_norm - vGrad );\n\t}\n#endif";

var clipping_planes_fragment = "#if NUM_CLIPPING_PLANES > 0\n\tvec4 plane;\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < UNION_CLIPPING_PLANES; i ++ ) {\n\t\tplane = clippingPlanes[ i ];\n\t\tif ( dot( vClipPosition, plane.xyz ) > plane.w ) discard;\n\t}\n\t#pragma unroll_loop_end\n\t#if UNION_CLIPPING_PLANES < NUM_CLIPPING_PLANES\n\t\tbool clipped = true;\n\t\t#pragma unroll_loop_start\n\t\tfor ( int i = UNION_CLIPPING_PLANES; i < NUM_CLIPPING_PLANES; i ++ ) {\n\t\t\tplane = clippingPlanes[ i ];\n\t\t\tclipped = ( dot( vClipPosition, plane.xyz ) > plane.w ) && clipped;\n\t\t}\n\t\t#pragma unroll_loop_end\n\t\tif ( clipped ) discard;\n\t#endif\n#endif";

var clipping_planes_pars_fragment = "#if NUM_CLIPPING_PLANES > 0\n\tvarying vec3 vClipPosition;\n\tuniform vec4 clippingPlanes[ NUM_CLIPPING_PLANES ];\n#endif";

var clipping_planes_pars_vertex = "#if NUM_CLIPPING_PLANES > 0\n\tvarying vec3 vClipPosition;\n#endif";

var clipping_planes_vertex = "#if NUM_CLIPPING_PLANES > 0\n\tvClipPosition = - mvPosition.xyz;\n#endif";

var color_fragment = "#if defined( USE_COLOR_ALPHA )\n\tdiffuseColor *= vColor;\n#elif defined( USE_COLOR )\n\tdiffuseColor.rgb *= vColor;\n#endif";

var color_pars_fragment = "#if defined( USE_COLOR_ALPHA )\n\tvarying vec4 vColor;\n#elif defined( USE_COLOR )\n\tvarying vec3 vColor;\n#endif";

var color_pars_vertex = "#if defined( USE_COLOR_ALPHA )\n\tvarying vec4 vColor;\n#elif defined( USE_COLOR ) || defined( USE_INSTANCING_COLOR )\n\tvarying vec3 vColor;\n#endif";

var color_vertex = "#if defined( USE_COLOR_ALPHA )\n\tvColor = vec4( 1.0 );\n#elif defined( USE_COLOR ) || defined( USE_INSTANCING_COLOR )\n\tvColor = vec3( 1.0 );\n#endif\n#ifdef USE_COLOR\n\tvColor *= color;\n#endif\n#ifdef USE_INSTANCING_COLOR\n\tvColor.xyz *= instanceColor.xyz;\n#endif";

var common = "#define PI 3.141592653589793\n#define PI2 6.283185307179586\n#define PI_HALF 1.5707963267948966\n#define RECIPROCAL_PI 0.3183098861837907\n#define RECIPROCAL_PI2 0.15915494309189535\n#define EPSILON 1e-6\n#ifndef saturate\n#define saturate(a) clamp( a, 0.0, 1.0 )\n#endif\n#define whiteComplement(a) ( 1.0 - saturate( a ) )\nfloat pow2( const in float x ) { return x*x; }\nfloat pow3( const in float x ) { return x*x*x; }\nfloat pow4( const in float x ) { float x2 = x*x; return x2*x2; }\nfloat average( const in vec3 color ) { return dot( color, vec3( 0.3333 ) ); }\nhighp float rand( const in vec2 uv ) {\n\tconst highp float a = 12.9898, b = 78.233, c = 43758.5453;\n\thighp float dt = dot( uv.xy, vec2( a,b ) ), sn = mod( dt, PI );\n\treturn fract(sin(sn) * c);\n}\n#ifdef HIGH_PRECISION\n\tfloat precisionSafeLength( vec3 v ) { return length( v ); }\n#else\n\tfloat max3( vec3 v ) { return max( max( v.x, v.y ), v.z ); }\n\tfloat precisionSafeLength( vec3 v ) {\n\t\tfloat maxComponent = max3( abs( v ) );\n\t\treturn length( v / maxComponent ) * maxComponent;\n\t}\n#endif\nstruct IncidentLight {\n\tvec3 color;\n\tvec3 direction;\n\tbool visible;\n};\nstruct ReflectedLight {\n\tvec3 directDiffuse;\n\tvec3 directSpecular;\n\tvec3 indirectDiffuse;\n\tvec3 indirectSpecular;\n};\nstruct GeometricContext {\n\tvec3 position;\n\tvec3 normal;\n\tvec3 viewDir;\n#ifdef CLEARCOAT\n\tvec3 clearcoatNormal;\n#endif\n};\nvec3 transformDirection( in vec3 dir, in mat4 matrix ) {\n\treturn normalize( ( matrix * vec4( dir, 0.0 ) ).xyz );\n}\nvec3 inverseTransformDirection( in vec3 dir, in mat4 matrix ) {\n\treturn normalize( ( vec4( dir, 0.0 ) * matrix ).xyz );\n}\nvec3 projectOnPlane(in vec3 point, in vec3 pointOnPlane, in vec3 planeNormal ) {\n\tfloat distance = dot( planeNormal, point - pointOnPlane );\n\treturn - distance * planeNormal + point;\n}\nfloat sideOfPlane( in vec3 point, in vec3 pointOnPlane, in vec3 planeNormal ) {\n\treturn sign( dot( point - pointOnPlane, planeNormal ) );\n}\nvec3 linePlaneIntersect( in vec3 pointOnLine, in vec3 lineDirection, in vec3 pointOnPlane, in vec3 planeNormal ) {\n\treturn lineDirection * ( dot( planeNormal, pointOnPlane - pointOnLine ) / dot( planeNormal, lineDirection ) ) + pointOnLine;\n}\nmat3 transposeMat3( const in mat3 m ) {\n\tmat3 tmp;\n\ttmp[ 0 ] = vec3( m[ 0 ].x, m[ 1 ].x, m[ 2 ].x );\n\ttmp[ 1 ] = vec3( m[ 0 ].y, m[ 1 ].y, m[ 2 ].y );\n\ttmp[ 2 ] = vec3( m[ 0 ].z, m[ 1 ].z, m[ 2 ].z );\n\treturn tmp;\n}\nfloat linearToRelativeLuminance( const in vec3 color ) {\n\tvec3 weights = vec3( 0.2126, 0.7152, 0.0722 );\n\treturn dot( weights, color.rgb );\n}\nbool isPerspectiveMatrix( mat4 m ) {\n\treturn m[ 2 ][ 3 ] == - 1.0;\n}\nvec2 equirectUv( in vec3 dir ) {\n\tfloat u = atan( dir.z, dir.x ) * RECIPROCAL_PI2 + 0.5;\n\tfloat v = asin( clamp( dir.y, - 1.0, 1.0 ) ) * RECIPROCAL_PI + 0.5;\n\treturn vec2( u, v );\n}";

var cube_uv_reflection_fragment = "#ifdef ENVMAP_TYPE_CUBE_UV\n\t#define cubeUV_maxMipLevel 8.0\n\t#define cubeUV_minMipLevel 4.0\n\t#define cubeUV_maxTileSize 256.0\n\t#define cubeUV_minTileSize 16.0\n\tfloat getFace( vec3 direction ) {\n\t\tvec3 absDirection = abs( direction );\n\t\tfloat face = - 1.0;\n\t\tif ( absDirection.x > absDirection.z ) {\n\t\t\tif ( absDirection.x > absDirection.y )\n\t\t\t\tface = direction.x > 0.0 ? 0.0 : 3.0;\n\t\t\telse\n\t\t\t\tface = direction.y > 0.0 ? 1.0 : 4.0;\n\t\t} else {\n\t\t\tif ( absDirection.z > absDirection.y )\n\t\t\t\tface = direction.z > 0.0 ? 2.0 : 5.0;\n\t\t\telse\n\t\t\t\tface = direction.y > 0.0 ? 1.0 : 4.0;\n\t\t}\n\t\treturn face;\n\t}\n\tvec2 getUV( vec3 direction, float face ) {\n\t\tvec2 uv;\n\t\tif ( face == 0.0 ) {\n\t\t\tuv = vec2( direction.z, direction.y ) / abs( direction.x );\n\t\t} else if ( face == 1.0 ) {\n\t\t\tuv = vec2( - direction.x, - direction.z ) / abs( direction.y );\n\t\t} else if ( face == 2.0 ) {\n\t\t\tuv = vec2( - direction.x, direction.y ) / abs( direction.z );\n\t\t} else if ( face == 3.0 ) {\n\t\t\tuv = vec2( - direction.z, direction.y ) / abs( direction.x );\n\t\t} else if ( face == 4.0 ) {\n\t\t\tuv = vec2( - direction.x, direction.z ) / abs( direction.y );\n\t\t} else {\n\t\t\tuv = vec2( direction.x, direction.y ) / abs( direction.z );\n\t\t}\n\t\treturn 0.5 * ( uv + 1.0 );\n\t}\n\tvec3 bilinearCubeUV( sampler2D envMap, vec3 direction, float mipInt ) {\n\t\tfloat face = getFace( direction );\n\t\tfloat filterInt = max( cubeUV_minMipLevel - mipInt, 0.0 );\n\t\tmipInt = max( mipInt, cubeUV_minMipLevel );\n\t\tfloat faceSize = exp2( mipInt );\n\t\tfloat texelSize = 1.0 / ( 3.0 * cubeUV_maxTileSize );\n\t\tvec2 uv = getUV( direction, face ) * ( faceSize - 1.0 );\n\t\tvec2 f = fract( uv );\n\t\tuv += 0.5 - f;\n\t\tif ( face > 2.0 ) {\n\t\t\tuv.y += faceSize;\n\t\t\tface -= 3.0;\n\t\t}\n\t\tuv.x += face * faceSize;\n\t\tif ( mipInt < cubeUV_maxMipLevel ) {\n\t\t\tuv.y += 2.0 * cubeUV_maxTileSize;\n\t\t}\n\t\tuv.y += filterInt * 2.0 * cubeUV_minTileSize;\n\t\tuv.x += 3.0 * max( 0.0, cubeUV_maxTileSize - 2.0 * faceSize );\n\t\tuv *= texelSize;\n\t\tvec3 tl = envMapTexelToLinear( texture2D( envMap, uv ) ).rgb;\n\t\tuv.x += texelSize;\n\t\tvec3 tr = envMapTexelToLinear( texture2D( envMap, uv ) ).rgb;\n\t\tuv.y += texelSize;\n\t\tvec3 br = envMapTexelToLinear( texture2D( envMap, uv ) ).rgb;\n\t\tuv.x -= texelSize;\n\t\tvec3 bl = envMapTexelToLinear( texture2D( envMap, uv ) ).rgb;\n\t\tvec3 tm = mix( tl, tr, f.x );\n\t\tvec3 bm = mix( bl, br, f.x );\n\t\treturn mix( tm, bm, f.y );\n\t}\n\t#define r0 1.0\n\t#define v0 0.339\n\t#define m0 - 2.0\n\t#define r1 0.8\n\t#define v1 0.276\n\t#define m1 - 1.0\n\t#define r4 0.4\n\t#define v4 0.046\n\t#define m4 2.0\n\t#define r5 0.305\n\t#define v5 0.016\n\t#define m5 3.0\n\t#define r6 0.21\n\t#define v6 0.0038\n\t#define m6 4.0\n\tfloat roughnessToMip( float roughness ) {\n\t\tfloat mip = 0.0;\n\t\tif ( roughness >= r1 ) {\n\t\t\tmip = ( r0 - roughness ) * ( m1 - m0 ) / ( r0 - r1 ) + m0;\n\t\t} else if ( roughness >= r4 ) {\n\t\t\tmip = ( r1 - roughness ) * ( m4 - m1 ) / ( r1 - r4 ) + m1;\n\t\t} else if ( roughness >= r5 ) {\n\t\t\tmip = ( r4 - roughness ) * ( m5 - m4 ) / ( r4 - r5 ) + m4;\n\t\t} else if ( roughness >= r6 ) {\n\t\t\tmip = ( r5 - roughness ) * ( m6 - m5 ) / ( r5 - r6 ) + m5;\n\t\t} else {\n\t\t\tmip = - 2.0 * log2( 1.16 * roughness );\t\t}\n\t\treturn mip;\n\t}\n\tvec4 textureCubeUV( sampler2D envMap, vec3 sampleDir, float roughness ) {\n\t\tfloat mip = clamp( roughnessToMip( roughness ), m0, cubeUV_maxMipLevel );\n\t\tfloat mipF = fract( mip );\n\t\tfloat mipInt = floor( mip );\n\t\tvec3 color0 = bilinearCubeUV( envMap, sampleDir, mipInt );\n\t\tif ( mipF == 0.0 ) {\n\t\t\treturn vec4( color0, 1.0 );\n\t\t} else {\n\t\t\tvec3 color1 = bilinearCubeUV( envMap, sampleDir, mipInt + 1.0 );\n\t\t\treturn vec4( mix( color0, color1, mipF ), 1.0 );\n\t\t}\n\t}\n#endif";

var defaultnormal_vertex = "vec3 transformedNormal = objectNormal;\n#ifdef USE_INSTANCING\n\tmat3 m = mat3( instanceMatrix );\n\ttransformedNormal /= vec3( dot( m[ 0 ], m[ 0 ] ), dot( m[ 1 ], m[ 1 ] ), dot( m[ 2 ], m[ 2 ] ) );\n\ttransformedNormal = m * transformedNormal;\n#endif\ntransformedNormal = normalMatrix * transformedNormal;\n#ifdef FLIP_SIDED\n\ttransformedNormal = - transformedNormal;\n#endif\n#ifdef USE_TANGENT\n\tvec3 transformedTangent = ( modelViewMatrix * vec4( objectTangent, 0.0 ) ).xyz;\n\t#ifdef FLIP_SIDED\n\t\ttransformedTangent = - transformedTangent;\n\t#endif\n#endif";

var displacementmap_pars_vertex = "#ifdef USE_DISPLACEMENTMAP\n\tuniform sampler2D displacementMap;\n\tuniform float displacementScale;\n\tuniform float displacementBias;\n#endif";

var displacementmap_vertex = "#ifdef USE_DISPLACEMENTMAP\n\ttransformed += normalize( objectNormal ) * ( texture2D( displacementMap, vUv ).x * displacementScale + displacementBias );\n#endif";

var emissivemap_fragment = "#ifdef USE_EMISSIVEMAP\n\tvec4 emissiveColor = texture2D( emissiveMap, vUv );\n\temissiveColor.rgb = emissiveMapTexelToLinear( emissiveColor ).rgb;\n\ttotalEmissiveRadiance *= emissiveColor.rgb;\n#endif";

var emissivemap_pars_fragment = "#ifdef USE_EMISSIVEMAP\n\tuniform sampler2D emissiveMap;\n#endif";

var encodings_fragment = "gl_FragColor = linearToOutputTexel( gl_FragColor );";

var encodings_pars_fragment = "\nvec4 LinearToLinear( in vec4 value ) {\n\treturn value;\n}\nvec4 GammaToLinear( in vec4 value, in float gammaFactor ) {\n\treturn vec4( pow( value.rgb, vec3( gammaFactor ) ), value.a );\n}\nvec4 LinearToGamma( in vec4 value, in float gammaFactor ) {\n\treturn vec4( pow( value.rgb, vec3( 1.0 / gammaFactor ) ), value.a );\n}\nvec4 sRGBToLinear( in vec4 value ) {\n\treturn vec4( mix( pow( value.rgb * 0.9478672986 + vec3( 0.0521327014 ), vec3( 2.4 ) ), value.rgb * 0.0773993808, vec3( lessThanEqual( value.rgb, vec3( 0.04045 ) ) ) ), value.a );\n}\nvec4 LinearTosRGB( in vec4 value ) {\n\treturn vec4( mix( pow( value.rgb, vec3( 0.41666 ) ) * 1.055 - vec3( 0.055 ), value.rgb * 12.92, vec3( lessThanEqual( value.rgb, vec3( 0.0031308 ) ) ) ), value.a );\n}\nvec4 RGBEToLinear( in vec4 value ) {\n\treturn vec4( value.rgb * exp2( value.a * 255.0 - 128.0 ), 1.0 );\n}\nvec4 LinearToRGBE( in vec4 value ) {\n\tfloat maxComponent = max( max( value.r, value.g ), value.b );\n\tfloat fExp = clamp( ceil( log2( maxComponent ) ), -128.0, 127.0 );\n\treturn vec4( value.rgb / exp2( fExp ), ( fExp + 128.0 ) / 255.0 );\n}\nvec4 RGBMToLinear( in vec4 value, in float maxRange ) {\n\treturn vec4( value.rgb * value.a * maxRange, 1.0 );\n}\nvec4 LinearToRGBM( in vec4 value, in float maxRange ) {\n\tfloat maxRGB = max( value.r, max( value.g, value.b ) );\n\tfloat M = clamp( maxRGB / maxRange, 0.0, 1.0 );\n\tM = ceil( M * 255.0 ) / 255.0;\n\treturn vec4( value.rgb / ( M * maxRange ), M );\n}\nvec4 RGBDToLinear( in vec4 value, in float maxRange ) {\n\treturn vec4( value.rgb * ( ( maxRange / 255.0 ) / value.a ), 1.0 );\n}\nvec4 LinearToRGBD( in vec4 value, in float maxRange ) {\n\tfloat maxRGB = max( value.r, max( value.g, value.b ) );\n\tfloat D = max( maxRange / maxRGB, 1.0 );\n\tD = clamp( floor( D ) / 255.0, 0.0, 1.0 );\n\treturn vec4( value.rgb * ( D * ( 255.0 / maxRange ) ), D );\n}\nconst mat3 cLogLuvM = mat3( 0.2209, 0.3390, 0.4184, 0.1138, 0.6780, 0.7319, 0.0102, 0.1130, 0.2969 );\nvec4 LinearToLogLuv( in vec4 value ) {\n\tvec3 Xp_Y_XYZp = cLogLuvM * value.rgb;\n\tXp_Y_XYZp = max( Xp_Y_XYZp, vec3( 1e-6, 1e-6, 1e-6 ) );\n\tvec4 vResult;\n\tvResult.xy = Xp_Y_XYZp.xy / Xp_Y_XYZp.z;\n\tfloat Le = 2.0 * log2(Xp_Y_XYZp.y) + 127.0;\n\tvResult.w = fract( Le );\n\tvResult.z = ( Le - ( floor( vResult.w * 255.0 ) ) / 255.0 ) / 255.0;\n\treturn vResult;\n}\nconst mat3 cLogLuvInverseM = mat3( 6.0014, -2.7008, -1.7996, -1.3320, 3.1029, -5.7721, 0.3008, -1.0882, 5.6268 );\nvec4 LogLuvToLinear( in vec4 value ) {\n\tfloat Le = value.z * 255.0 + value.w;\n\tvec3 Xp_Y_XYZp;\n\tXp_Y_XYZp.y = exp2( ( Le - 127.0 ) / 2.0 );\n\tXp_Y_XYZp.z = Xp_Y_XYZp.y / value.y;\n\tXp_Y_XYZp.x = value.x * Xp_Y_XYZp.z;\n\tvec3 vRGB = cLogLuvInverseM * Xp_Y_XYZp.rgb;\n\treturn vec4( max( vRGB, 0.0 ), 1.0 );\n}";

var envmap_fragment = "#ifdef USE_ENVMAP\n\t#ifdef ENV_WORLDPOS\n\t\tvec3 cameraToFrag;\n\t\tif ( isOrthographic ) {\n\t\t\tcameraToFrag = normalize( vec3( - viewMatrix[ 0 ][ 2 ], - viewMatrix[ 1 ][ 2 ], - viewMatrix[ 2 ][ 2 ] ) );\n\t\t} else {\n\t\t\tcameraToFrag = normalize( vWorldPosition - cameraPosition );\n\t\t}\n\t\tvec3 worldNormal = inverseTransformDirection( normal, viewMatrix );\n\t\t#ifdef ENVMAP_MODE_REFLECTION\n\t\t\tvec3 reflectVec = reflect( cameraToFrag, worldNormal );\n\t\t#else\n\t\t\tvec3 reflectVec = refract( cameraToFrag, worldNormal, refractionRatio );\n\t\t#endif\n\t#else\n\t\tvec3 reflectVec = vReflect;\n\t#endif\n\t#ifdef ENVMAP_TYPE_CUBE\n\t\tvec4 envColor = textureCube( envMap, vec3( flipEnvMap * reflectVec.x, reflectVec.yz ) );\n\t#elif defined( ENVMAP_TYPE_CUBE_UV )\n\t\tvec4 envColor = textureCubeUV( envMap, reflectVec, 0.0 );\n\t#else\n\t\tvec4 envColor = vec4( 0.0 );\n\t#endif\n\t#ifndef ENVMAP_TYPE_CUBE_UV\n\t\tenvColor = envMapTexelToLinear( envColor );\n\t#endif\n\t#ifdef ENVMAP_BLENDING_MULTIPLY\n\t\toutgoingLight = mix( outgoingLight, outgoingLight * envColor.xyz, specularStrength * reflectivity );\n\t#elif defined( ENVMAP_BLENDING_MIX )\n\t\toutgoingLight = mix( outgoingLight, envColor.xyz, specularStrength * reflectivity );\n\t#elif defined( ENVMAP_BLENDING_ADD )\n\t\toutgoingLight += envColor.xyz * specularStrength * reflectivity;\n\t#endif\n#endif";

var envmap_common_pars_fragment = "#ifdef USE_ENVMAP\n\tuniform float envMapIntensity;\n\tuniform float flipEnvMap;\n\tuniform int maxMipLevel;\n\t#ifdef ENVMAP_TYPE_CUBE\n\t\tuniform samplerCube envMap;\n\t#else\n\t\tuniform sampler2D envMap;\n\t#endif\n\t\n#endif";

var envmap_pars_fragment = "#ifdef USE_ENVMAP\n\tuniform float reflectivity;\n\t#if defined( USE_BUMPMAP ) || defined( USE_NORMALMAP ) || defined( PHONG )\n\t\t#define ENV_WORLDPOS\n\t#endif\n\t#ifdef ENV_WORLDPOS\n\t\tvarying vec3 vWorldPosition;\n\t\tuniform float refractionRatio;\n\t#else\n\t\tvarying vec3 vReflect;\n\t#endif\n#endif";

var envmap_pars_vertex = "#ifdef USE_ENVMAP\n\t#if defined( USE_BUMPMAP ) || defined( USE_NORMALMAP ) ||defined( PHONG )\n\t\t#define ENV_WORLDPOS\n\t#endif\n\t#ifdef ENV_WORLDPOS\n\t\t\n\t\tvarying vec3 vWorldPosition;\n\t#else\n\t\tvarying vec3 vReflect;\n\t\tuniform float refractionRatio;\n\t#endif\n#endif";

var envmap_vertex = "#ifdef USE_ENVMAP\n\t#ifdef ENV_WORLDPOS\n\t\tvWorldPosition = worldPosition.xyz;\n\t#else\n\t\tvec3 cameraToVertex;\n\t\tif ( isOrthographic ) {\n\t\t\tcameraToVertex = normalize( vec3( - viewMatrix[ 0 ][ 2 ], - viewMatrix[ 1 ][ 2 ], - viewMatrix[ 2 ][ 2 ] ) );\n\t\t} else {\n\t\t\tcameraToVertex = normalize( worldPosition.xyz - cameraPosition );\n\t\t}\n\t\tvec3 worldNormal = inverseTransformDirection( transformedNormal, viewMatrix );\n\t\t#ifdef ENVMAP_MODE_REFLECTION\n\t\t\tvReflect = reflect( cameraToVertex, worldNormal );\n\t\t#else\n\t\t\tvReflect = refract( cameraToVertex, worldNormal, refractionRatio );\n\t\t#endif\n\t#endif\n#endif";

var fog_vertex = "#ifdef USE_FOG\n\tfogDepth = - mvPosition.z;\n#endif";

var fog_pars_vertex = "#ifdef USE_FOG\n\tvarying float fogDepth;\n#endif";

var fog_fragment = "#ifdef USE_FOG\n\t#ifdef FOG_EXP2\n\t\tfloat fogFactor = 1.0 - exp( - fogDensity * fogDensity * fogDepth * fogDepth );\n\t#else\n\t\tfloat fogFactor = smoothstep( fogNear, fogFar, fogDepth );\n\t#endif\n\tgl_FragColor.rgb = mix( gl_FragColor.rgb, fogColor, fogFactor );\n#endif";

var fog_pars_fragment = "#ifdef USE_FOG\n\tuniform vec3 fogColor;\n\tvarying float fogDepth;\n\t#ifdef FOG_EXP2\n\t\tuniform float fogDensity;\n\t#else\n\t\tuniform float fogNear;\n\t\tuniform float fogFar;\n\t#endif\n#endif";

var gradientmap_pars_fragment = "#ifdef USE_GRADIENTMAP\n\tuniform sampler2D gradientMap;\n#endif\nvec3 getGradientIrradiance( vec3 normal, vec3 lightDirection ) {\n\tfloat dotNL = dot( normal, lightDirection );\n\tvec2 coord = vec2( dotNL * 0.5 + 0.5, 0.0 );\n\t#ifdef USE_GRADIENTMAP\n\t\treturn texture2D( gradientMap, coord ).rgb;\n\t#else\n\t\treturn ( coord.x < 0.7 ) ? vec3( 0.7 ) : vec3( 1.0 );\n\t#endif\n}";

var lightmap_fragment = "#ifdef USE_LIGHTMAP\n\tvec4 lightMapTexel= texture2D( lightMap, vUv2 );\n\treflectedLight.indirectDiffuse += PI * lightMapTexelToLinear( lightMapTexel ).rgb * lightMapIntensity;\n#endif";

var lightmap_pars_fragment = "#ifdef USE_LIGHTMAP\n\tuniform sampler2D lightMap;\n\tuniform float lightMapIntensity;\n#endif";

var lights_lambert_vertex = "vec3 diffuse = vec3( 1.0 );\nGeometricContext geometry;\ngeometry.position = mvPosition.xyz;\ngeometry.normal = normalize( transformedNormal );\ngeometry.viewDir = ( isOrthographic ) ? vec3( 0, 0, 1 ) : normalize( -mvPosition.xyz );\nGeometricContext backGeometry;\nbackGeometry.position = geometry.position;\nbackGeometry.normal = -geometry.normal;\nbackGeometry.viewDir = geometry.viewDir;\nvLightFront = vec3( 0.0 );\nvIndirectFront = vec3( 0.0 );\n#ifdef DOUBLE_SIDED\n\tvLightBack = vec3( 0.0 );\n\tvIndirectBack = vec3( 0.0 );\n#endif\nIncidentLight directLight;\nfloat dotNL;\nvec3 directLightColor_Diffuse;\nvIndirectFront += getAmbientLightIrradiance( ambientLightColor );\nvIndirectFront += getLightProbeIrradiance( lightProbe, geometry );\n#ifdef DOUBLE_SIDED\n\tvIndirectBack += getAmbientLightIrradiance( ambientLightColor );\n\tvIndirectBack += getLightProbeIrradiance( lightProbe, backGeometry );\n#endif\n#if NUM_POINT_LIGHTS > 0\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_POINT_LIGHTS; i ++ ) {\n\t\tgetPointDirectLightIrradiance( pointLights[ i ], geometry, directLight );\n\t\tdotNL = dot( geometry.normal, directLight.direction );\n\t\tdirectLightColor_Diffuse = PI * directLight.color;\n\t\tvLightFront += saturate( dotNL ) * directLightColor_Diffuse;\n\t\t#ifdef DOUBLE_SIDED\n\t\t\tvLightBack += saturate( -dotNL ) * directLightColor_Diffuse;\n\t\t#endif\n\t}\n\t#pragma unroll_loop_end\n#endif\n#if NUM_SPOT_LIGHTS > 0\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_SPOT_LIGHTS; i ++ ) {\n\t\tgetSpotDirectLightIrradiance( spotLights[ i ], geometry, directLight );\n\t\tdotNL = dot( geometry.normal, directLight.direction );\n\t\tdirectLightColor_Diffuse = PI * directLight.color;\n\t\tvLightFront += saturate( dotNL ) * directLightColor_Diffuse;\n\t\t#ifdef DOUBLE_SIDED\n\t\t\tvLightBack += saturate( -dotNL ) * directLightColor_Diffuse;\n\t\t#endif\n\t}\n\t#pragma unroll_loop_end\n#endif\n#if NUM_DIR_LIGHTS > 0\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_DIR_LIGHTS; i ++ ) {\n\t\tgetDirectionalDirectLightIrradiance( directionalLights[ i ], geometry, directLight );\n\t\tdotNL = dot( geometry.normal, directLight.direction );\n\t\tdirectLightColor_Diffuse = PI * directLight.color;\n\t\tvLightFront += saturate( dotNL ) * directLightColor_Diffuse;\n\t\t#ifdef DOUBLE_SIDED\n\t\t\tvLightBack += saturate( -dotNL ) * directLightColor_Diffuse;\n\t\t#endif\n\t}\n\t#pragma unroll_loop_end\n#endif\n#if NUM_HEMI_LIGHTS > 0\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_HEMI_LIGHTS; i ++ ) {\n\t\tvIndirectFront += getHemisphereLightIrradiance( hemisphereLights[ i ], geometry );\n\t\t#ifdef DOUBLE_SIDED\n\t\t\tvIndirectBack += getHemisphereLightIrradiance( hemisphereLights[ i ], backGeometry );\n\t\t#endif\n\t}\n\t#pragma unroll_loop_end\n#endif";

var lights_pars_begin = "uniform bool receiveShadow;\nuniform vec3 ambientLightColor;\nuniform vec3 lightProbe[ 9 ];\nvec3 shGetIrradianceAt( in vec3 normal, in vec3 shCoefficients[ 9 ] ) {\n\tfloat x = normal.x, y = normal.y, z = normal.z;\n\tvec3 result = shCoefficients[ 0 ] * 0.886227;\n\tresult += shCoefficients[ 1 ] * 2.0 * 0.511664 * y;\n\tresult += shCoefficients[ 2 ] * 2.0 * 0.511664 * z;\n\tresult += shCoefficients[ 3 ] * 2.0 * 0.511664 * x;\n\tresult += shCoefficients[ 4 ] * 2.0 * 0.429043 * x * y;\n\tresult += shCoefficients[ 5 ] * 2.0 * 0.429043 * y * z;\n\tresult += shCoefficients[ 6 ] * ( 0.743125 * z * z - 0.247708 );\n\tresult += shCoefficients[ 7 ] * 2.0 * 0.429043 * x * z;\n\tresult += shCoefficients[ 8 ] * 0.429043 * ( x * x - y * y );\n\treturn result;\n}\nvec3 getLightProbeIrradiance( const in vec3 lightProbe[ 9 ], const in GeometricContext geometry ) {\n\tvec3 worldNormal = inverseTransformDirection( geometry.normal, viewMatrix );\n\tvec3 irradiance = shGetIrradianceAt( worldNormal, lightProbe );\n\treturn irradiance;\n}\nvec3 getAmbientLightIrradiance( const in vec3 ambientLightColor ) {\n\tvec3 irradiance = ambientLightColor;\n\t#ifndef PHYSICALLY_CORRECT_LIGHTS\n\t\tirradiance *= PI;\n\t#endif\n\treturn irradiance;\n}\n#if NUM_DIR_LIGHTS > 0\n\tstruct DirectionalLight {\n\t\tvec3 direction;\n\t\tvec3 color;\n\t};\n\tuniform DirectionalLight directionalLights[ NUM_DIR_LIGHTS ];\n\tvoid getDirectionalDirectLightIrradiance( const in DirectionalLight directionalLight, const in GeometricContext geometry, out IncidentLight directLight ) {\n\t\tdirectLight.color = directionalLight.color;\n\t\tdirectLight.direction = directionalLight.direction;\n\t\tdirectLight.visible = true;\n\t}\n#endif\n#if NUM_POINT_LIGHTS > 0\n\tstruct PointLight {\n\t\tvec3 position;\n\t\tvec3 color;\n\t\tfloat distance;\n\t\tfloat decay;\n\t};\n\tuniform PointLight pointLights[ NUM_POINT_LIGHTS ];\n\tvoid getPointDirectLightIrradiance( const in PointLight pointLight, const in GeometricContext geometry, out IncidentLight directLight ) {\n\t\tvec3 lVector = pointLight.position - geometry.position;\n\t\tdirectLight.direction = normalize( lVector );\n\t\tfloat lightDistance = length( lVector );\n\t\tdirectLight.color = pointLight.color;\n\t\tdirectLight.color *= punctualLightIntensityToIrradianceFactor( lightDistance, pointLight.distance, pointLight.decay );\n\t\tdirectLight.visible = ( directLight.color != vec3( 0.0 ) );\n\t}\n#endif\n#if NUM_SPOT_LIGHTS > 0\n\tstruct SpotLight {\n\t\tvec3 position;\n\t\tvec3 direction;\n\t\tvec3 color;\n\t\tfloat distance;\n\t\tfloat decay;\n\t\tfloat coneCos;\n\t\tfloat penumbraCos;\n\t};\n\tuniform SpotLight spotLights[ NUM_SPOT_LIGHTS ];\n\tvoid getSpotDirectLightIrradiance( const in SpotLight spotLight, const in GeometricContext geometry, out IncidentLight directLight ) {\n\t\tvec3 lVector = spotLight.position - geometry.position;\n\t\tdirectLight.direction = normalize( lVector );\n\t\tfloat lightDistance = length( lVector );\n\t\tfloat angleCos = dot( directLight.direction, spotLight.direction );\n\t\tif ( angleCos > spotLight.coneCos ) {\n\t\t\tfloat spotEffect = smoothstep( spotLight.coneCos, spotLight.penumbraCos, angleCos );\n\t\t\tdirectLight.color = spotLight.color;\n\t\t\tdirectLight.color *= spotEffect * punctualLightIntensityToIrradianceFactor( lightDistance, spotLight.distance, spotLight.decay );\n\t\t\tdirectLight.visible = true;\n\t\t} else {\n\t\t\tdirectLight.color = vec3( 0.0 );\n\t\t\tdirectLight.visible = false;\n\t\t}\n\t}\n#endif\n#if NUM_RECT_AREA_LIGHTS > 0\n\tstruct RectAreaLight {\n\t\tvec3 color;\n\t\tvec3 position;\n\t\tvec3 halfWidth;\n\t\tvec3 halfHeight;\n\t};\n\tuniform sampler2D ltc_1;\tuniform sampler2D ltc_2;\n\tuniform RectAreaLight rectAreaLights[ NUM_RECT_AREA_LIGHTS ];\n#endif\n#if NUM_HEMI_LIGHTS > 0\n\tstruct HemisphereLight {\n\t\tvec3 direction;\n\t\tvec3 skyColor;\n\t\tvec3 groundColor;\n\t};\n\tuniform HemisphereLight hemisphereLights[ NUM_HEMI_LIGHTS ];\n\tvec3 getHemisphereLightIrradiance( const in HemisphereLight hemiLight, const in GeometricContext geometry ) {\n\t\tfloat dotNL = dot( geometry.normal, hemiLight.direction );\n\t\tfloat hemiDiffuseWeight = 0.5 * dotNL + 0.5;\n\t\tvec3 irradiance = mix( hemiLight.groundColor, hemiLight.skyColor, hemiDiffuseWeight );\n\t\t#ifndef PHYSICALLY_CORRECT_LIGHTS\n\t\t\tirradiance *= PI;\n\t\t#endif\n\t\treturn irradiance;\n\t}\n#endif";

var envmap_physical_pars_fragment = "#if defined( USE_ENVMAP )\n\t#ifdef ENVMAP_MODE_REFRACTION\n\t\tuniform float refractionRatio;\n\t#endif\n\tvec3 getLightProbeIndirectIrradiance( const in GeometricContext geometry, const in int maxMIPLevel ) {\n\t\tvec3 worldNormal = inverseTransformDirection( geometry.normal, viewMatrix );\n\t\t#ifdef ENVMAP_TYPE_CUBE\n\t\t\tvec3 queryVec = vec3( flipEnvMap * worldNormal.x, worldNormal.yz );\n\t\t\t#ifdef TEXTURE_LOD_EXT\n\t\t\t\tvec4 envMapColor = textureCubeLodEXT( envMap, queryVec, float( maxMIPLevel ) );\n\t\t\t#else\n\t\t\t\tvec4 envMapColor = textureCube( envMap, queryVec, float( maxMIPLevel ) );\n\t\t\t#endif\n\t\t\tenvMapColor.rgb = envMapTexelToLinear( envMapColor ).rgb;\n\t\t#elif defined( ENVMAP_TYPE_CUBE_UV )\n\t\t\tvec4 envMapColor = textureCubeUV( envMap, worldNormal, 1.0 );\n\t\t#else\n\t\t\tvec4 envMapColor = vec4( 0.0 );\n\t\t#endif\n\t\treturn PI * envMapColor.rgb * envMapIntensity;\n\t}\n\tfloat getSpecularMIPLevel( const in float roughness, const in int maxMIPLevel ) {\n\t\tfloat maxMIPLevelScalar = float( maxMIPLevel );\n\t\tfloat sigma = PI * roughness * roughness / ( 1.0 + roughness );\n\t\tfloat desiredMIPLevel = maxMIPLevelScalar + log2( sigma );\n\t\treturn clamp( desiredMIPLevel, 0.0, maxMIPLevelScalar );\n\t}\n\tvec3 getLightProbeIndirectRadiance( const in vec3 viewDir, const in vec3 normal, const in float roughness, const in int maxMIPLevel ) {\n\t\t#ifdef ENVMAP_MODE_REFLECTION\n\t\t\tvec3 reflectVec = reflect( -viewDir, normal );\n\t\t\treflectVec = normalize( mix( reflectVec, normal, roughness * roughness) );\n\t\t#else\n\t\t\tvec3 reflectVec = refract( -viewDir, normal, refractionRatio );\n\t\t#endif\n\t\treflectVec = inverseTransformDirection( reflectVec, viewMatrix );\n\t\tfloat specularMIPLevel = getSpecularMIPLevel( roughness, maxMIPLevel );\n\t\t#ifdef ENVMAP_TYPE_CUBE\n\t\t\tvec3 queryReflectVec = vec3( flipEnvMap * reflectVec.x, reflectVec.yz );\n\t\t\t#ifdef TEXTURE_LOD_EXT\n\t\t\t\tvec4 envMapColor = textureCubeLodEXT( envMap, queryReflectVec, specularMIPLevel );\n\t\t\t#else\n\t\t\t\tvec4 envMapColor = textureCube( envMap, queryReflectVec, specularMIPLevel );\n\t\t\t#endif\n\t\t\tenvMapColor.rgb = envMapTexelToLinear( envMapColor ).rgb;\n\t\t#elif defined( ENVMAP_TYPE_CUBE_UV )\n\t\t\tvec4 envMapColor = textureCubeUV( envMap, reflectVec, roughness );\n\t\t#endif\n\t\treturn envMapColor.rgb * envMapIntensity;\n\t}\n#endif";

var lights_toon_fragment = "ToonMaterial material;\nmaterial.diffuseColor = diffuseColor.rgb;";

var lights_toon_pars_fragment = "varying vec3 vViewPosition;\n#ifndef FLAT_SHADED\n\tvarying vec3 vNormal;\n#endif\nstruct ToonMaterial {\n\tvec3 diffuseColor;\n};\nvoid RE_Direct_Toon( const in IncidentLight directLight, const in GeometricContext geometry, const in ToonMaterial material, inout ReflectedLight reflectedLight ) {\n\tvec3 irradiance = getGradientIrradiance( geometry.normal, directLight.direction ) * directLight.color;\n\t#ifndef PHYSICALLY_CORRECT_LIGHTS\n\t\tirradiance *= PI;\n\t#endif\n\treflectedLight.directDiffuse += irradiance * BRDF_Diffuse_Lambert( material.diffuseColor );\n}\nvoid RE_IndirectDiffuse_Toon( const in vec3 irradiance, const in GeometricContext geometry, const in ToonMaterial material, inout ReflectedLight reflectedLight ) {\n\treflectedLight.indirectDiffuse += irradiance * BRDF_Diffuse_Lambert( material.diffuseColor );\n}\n#define RE_Direct\t\t\t\tRE_Direct_Toon\n#define RE_IndirectDiffuse\t\tRE_IndirectDiffuse_Toon\n#define Material_LightProbeLOD( material )\t(0)";

var lights_phong_fragment = "BlinnPhongMaterial material;\nmaterial.diffuseColor = diffuseColor.rgb;\nmaterial.specularColor = specular;\nmaterial.specularShininess = shininess;\nmaterial.specularStrength = specularStrength;";

var lights_phong_pars_fragment = "varying vec3 vViewPosition;\n#ifndef FLAT_SHADED\n\tvarying vec3 vNormal;\n#endif\nstruct BlinnPhongMaterial {\n\tvec3 diffuseColor;\n\tvec3 specularColor;\n\tfloat specularShininess;\n\tfloat specularStrength;\n};\nvoid RE_Direct_BlinnPhong( const in IncidentLight directLight, const in GeometricContext geometry, const in BlinnPhongMaterial material, inout ReflectedLight reflectedLight ) {\n\tfloat dotNL = saturate( dot( geometry.normal, directLight.direction ) );\n\tvec3 irradiance = dotNL * directLight.color;\n\t#ifndef PHYSICALLY_CORRECT_LIGHTS\n\t\tirradiance *= PI;\n\t#endif\n\treflectedLight.directDiffuse += irradiance * BRDF_Diffuse_Lambert( material.diffuseColor );\n\treflectedLight.directSpecular += irradiance * BRDF_Specular_BlinnPhong( directLight, geometry, material.specularColor, material.specularShininess ) * material.specularStrength;\n}\nvoid RE_IndirectDiffuse_BlinnPhong( const in vec3 irradiance, const in GeometricContext geometry, const in BlinnPhongMaterial material, inout ReflectedLight reflectedLight ) {\n\treflectedLight.indirectDiffuse += irradiance * BRDF_Diffuse_Lambert( material.diffuseColor );\n}\n#define RE_Direct\t\t\t\tRE_Direct_BlinnPhong\n#define RE_IndirectDiffuse\t\tRE_IndirectDiffuse_BlinnPhong\n#define Material_LightProbeLOD( material )\t(0)";

var lights_physical_fragment = "PhysicalMaterial material;\nmaterial.diffuseColor = diffuseColor.rgb * ( 1.0 - metalnessFactor );\nvec3 dxy = max( abs( dFdx( geometryNormal ) ), abs( dFdy( geometryNormal ) ) );\nfloat geometryRoughness = max( max( dxy.x, dxy.y ), dxy.z );\nmaterial.specularRoughness = max( roughnessFactor, 0.0525 );material.specularRoughness += geometryRoughness;\nmaterial.specularRoughness = min( material.specularRoughness, 1.0 );\n#ifdef REFLECTIVITY\n\tmaterial.specularColor = mix( vec3( MAXIMUM_SPECULAR_COEFFICIENT * pow2( reflectivity ) ), diffuseColor.rgb, metalnessFactor );\n#else\n\tmaterial.specularColor = mix( vec3( DEFAULT_SPECULAR_COEFFICIENT ), diffuseColor.rgb, metalnessFactor );\n#endif\n#ifdef CLEARCOAT\n\tmaterial.clearcoat = clearcoat;\n\tmaterial.clearcoatRoughness = clearcoatRoughness;\n\t#ifdef USE_CLEARCOATMAP\n\t\tmaterial.clearcoat *= texture2D( clearcoatMap, vUv ).x;\n\t#endif\n\t#ifdef USE_CLEARCOAT_ROUGHNESSMAP\n\t\tmaterial.clearcoatRoughness *= texture2D( clearcoatRoughnessMap, vUv ).y;\n\t#endif\n\tmaterial.clearcoat = saturate( material.clearcoat );\tmaterial.clearcoatRoughness = max( material.clearcoatRoughness, 0.0525 );\n\tmaterial.clearcoatRoughness += geometryRoughness;\n\tmaterial.clearcoatRoughness = min( material.clearcoatRoughness, 1.0 );\n#endif\n#ifdef USE_SHEEN\n\tmaterial.sheenColor = sheen;\n#endif";

var lights_physical_pars_fragment = "struct PhysicalMaterial {\n\tvec3 diffuseColor;\n\tfloat specularRoughness;\n\tvec3 specularColor;\n#ifdef CLEARCOAT\n\tfloat clearcoat;\n\tfloat clearcoatRoughness;\n#endif\n#ifdef USE_SHEEN\n\tvec3 sheenColor;\n#endif\n};\n#define MAXIMUM_SPECULAR_COEFFICIENT 0.16\n#define DEFAULT_SPECULAR_COEFFICIENT 0.04\nfloat clearcoatDHRApprox( const in float roughness, const in float dotNL ) {\n\treturn DEFAULT_SPECULAR_COEFFICIENT + ( 1.0 - DEFAULT_SPECULAR_COEFFICIENT ) * ( pow( 1.0 - dotNL, 5.0 ) * pow( 1.0 - roughness, 2.0 ) );\n}\n#if NUM_RECT_AREA_LIGHTS > 0\n\tvoid RE_Direct_RectArea_Physical( const in RectAreaLight rectAreaLight, const in GeometricContext geometry, const in PhysicalMaterial material, inout ReflectedLight reflectedLight ) {\n\t\tvec3 normal = geometry.normal;\n\t\tvec3 viewDir = geometry.viewDir;\n\t\tvec3 position = geometry.position;\n\t\tvec3 lightPos = rectAreaLight.position;\n\t\tvec3 halfWidth = rectAreaLight.halfWidth;\n\t\tvec3 halfHeight = rectAreaLight.halfHeight;\n\t\tvec3 lightColor = rectAreaLight.color;\n\t\tfloat roughness = material.specularRoughness;\n\t\tvec3 rectCoords[ 4 ];\n\t\trectCoords[ 0 ] = lightPos + halfWidth - halfHeight;\t\trectCoords[ 1 ] = lightPos - halfWidth - halfHeight;\n\t\trectCoords[ 2 ] = lightPos - halfWidth + halfHeight;\n\t\trectCoords[ 3 ] = lightPos + halfWidth + halfHeight;\n\t\tvec2 uv = LTC_Uv( normal, viewDir, roughness );\n\t\tvec4 t1 = texture2D( ltc_1, uv );\n\t\tvec4 t2 = texture2D( ltc_2, uv );\n\t\tmat3 mInv = mat3(\n\t\t\tvec3( t1.x, 0, t1.y ),\n\t\t\tvec3(    0, 1,    0 ),\n\t\t\tvec3( t1.z, 0, t1.w )\n\t\t);\n\t\tvec3 fresnel = ( material.specularColor * t2.x + ( vec3( 1.0 ) - material.specularColor ) * t2.y );\n\t\treflectedLight.directSpecular += lightColor * fresnel * LTC_Evaluate( normal, viewDir, position, mInv, rectCoords );\n\t\treflectedLight.directDiffuse += lightColor * material.diffuseColor * LTC_Evaluate( normal, viewDir, position, mat3( 1.0 ), rectCoords );\n\t}\n#endif\nvoid RE_Direct_Physical( const in IncidentLight directLight, const in GeometricContext geometry, const in PhysicalMaterial material, inout ReflectedLight reflectedLight ) {\n\tfloat dotNL = saturate( dot( geometry.normal, directLight.direction ) );\n\tvec3 irradiance = dotNL * directLight.color;\n\t#ifndef PHYSICALLY_CORRECT_LIGHTS\n\t\tirradiance *= PI;\n\t#endif\n\t#ifdef CLEARCOAT\n\t\tfloat ccDotNL = saturate( dot( geometry.clearcoatNormal, directLight.direction ) );\n\t\tvec3 ccIrradiance = ccDotNL * directLight.color;\n\t\t#ifndef PHYSICALLY_CORRECT_LIGHTS\n\t\t\tccIrradiance *= PI;\n\t\t#endif\n\t\tfloat clearcoatDHR = material.clearcoat * clearcoatDHRApprox( material.clearcoatRoughness, ccDotNL );\n\t\treflectedLight.directSpecular += ccIrradiance * material.clearcoat * BRDF_Specular_GGX( directLight, geometry.viewDir, geometry.clearcoatNormal, vec3( DEFAULT_SPECULAR_COEFFICIENT ), material.clearcoatRoughness );\n\t#else\n\t\tfloat clearcoatDHR = 0.0;\n\t#endif\n\t#ifdef USE_SHEEN\n\t\treflectedLight.directSpecular += ( 1.0 - clearcoatDHR ) * irradiance * BRDF_Specular_Sheen(\n\t\t\tmaterial.specularRoughness,\n\t\t\tdirectLight.direction,\n\t\t\tgeometry,\n\t\t\tmaterial.sheenColor\n\t\t);\n\t#else\n\t\treflectedLight.directSpecular += ( 1.0 - clearcoatDHR ) * irradiance * BRDF_Specular_GGX( directLight, geometry.viewDir, geometry.normal, material.specularColor, material.specularRoughness);\n\t#endif\n\treflectedLight.directDiffuse += ( 1.0 - clearcoatDHR ) * irradiance * BRDF_Diffuse_Lambert( material.diffuseColor );\n}\nvoid RE_IndirectDiffuse_Physical( const in vec3 irradiance, const in GeometricContext geometry, const in PhysicalMaterial material, inout ReflectedLight reflectedLight ) {\n\treflectedLight.indirectDiffuse += irradiance * BRDF_Diffuse_Lambert( material.diffuseColor );\n}\nvoid RE_IndirectSpecular_Physical( const in vec3 radiance, const in vec3 irradiance, const in vec3 clearcoatRadiance, const in GeometricContext geometry, const in PhysicalMaterial material, inout ReflectedLight reflectedLight) {\n\t#ifdef CLEARCOAT\n\t\tfloat ccDotNV = saturate( dot( geometry.clearcoatNormal, geometry.viewDir ) );\n\t\treflectedLight.indirectSpecular += clearcoatRadiance * material.clearcoat * BRDF_Specular_GGX_Environment( geometry.viewDir, geometry.clearcoatNormal, vec3( DEFAULT_SPECULAR_COEFFICIENT ), material.clearcoatRoughness );\n\t\tfloat ccDotNL = ccDotNV;\n\t\tfloat clearcoatDHR = material.clearcoat * clearcoatDHRApprox( material.clearcoatRoughness, ccDotNL );\n\t#else\n\t\tfloat clearcoatDHR = 0.0;\n\t#endif\n\tfloat clearcoatInv = 1.0 - clearcoatDHR;\n\tvec3 singleScattering = vec3( 0.0 );\n\tvec3 multiScattering = vec3( 0.0 );\n\tvec3 cosineWeightedIrradiance = irradiance * RECIPROCAL_PI;\n\tBRDF_Specular_Multiscattering_Environment( geometry, material.specularColor, material.specularRoughness, singleScattering, multiScattering );\n\tvec3 diffuse = material.diffuseColor * ( 1.0 - ( singleScattering + multiScattering ) );\n\treflectedLight.indirectSpecular += clearcoatInv * radiance * singleScattering;\n\treflectedLight.indirectSpecular += multiScattering * cosineWeightedIrradiance;\n\treflectedLight.indirectDiffuse += diffuse * cosineWeightedIrradiance;\n}\n#define RE_Direct\t\t\t\tRE_Direct_Physical\n#define RE_Direct_RectArea\t\tRE_Direct_RectArea_Physical\n#define RE_IndirectDiffuse\t\tRE_IndirectDiffuse_Physical\n#define RE_IndirectSpecular\t\tRE_IndirectSpecular_Physical\nfloat computeSpecularOcclusion( const in float dotNV, const in float ambientOcclusion, const in float roughness ) {\n\treturn saturate( pow( dotNV + ambientOcclusion, exp2( - 16.0 * roughness - 1.0 ) ) - 1.0 + ambientOcclusion );\n}";

var lights_fragment_begin = "\nGeometricContext geometry;\ngeometry.position = - vViewPosition;\ngeometry.normal = normal;\ngeometry.viewDir = ( isOrthographic ) ? vec3( 0, 0, 1 ) : normalize( vViewPosition );\n#ifdef CLEARCOAT\n\tgeometry.clearcoatNormal = clearcoatNormal;\n#endif\nIncidentLight directLight;\n#if ( NUM_POINT_LIGHTS > 0 ) && defined( RE_Direct )\n\tPointLight pointLight;\n\t#if defined( USE_SHADOWMAP ) && NUM_POINT_LIGHT_SHADOWS > 0\n\tPointLightShadow pointLightShadow;\n\t#endif\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_POINT_LIGHTS; i ++ ) {\n\t\tpointLight = pointLights[ i ];\n\t\tgetPointDirectLightIrradiance( pointLight, geometry, directLight );\n\t\t#if defined( USE_SHADOWMAP ) && ( UNROLLED_LOOP_INDEX < NUM_POINT_LIGHT_SHADOWS )\n\t\tpointLightShadow = pointLightShadows[ i ];\n\t\tdirectLight.color *= all( bvec2( directLight.visible, receiveShadow ) ) ? getPointShadow( pointShadowMap[ i ], pointLightShadow.shadowMapSize, pointLightShadow.shadowBias, pointLightShadow.shadowRadius, vPointShadowCoord[ i ], pointLightShadow.shadowCameraNear, pointLightShadow.shadowCameraFar ) : 1.0;\n\t\t#endif\n\t\tRE_Direct( directLight, geometry, material, reflectedLight );\n\t}\n\t#pragma unroll_loop_end\n#endif\n#if ( NUM_SPOT_LIGHTS > 0 ) && defined( RE_Direct )\n\tSpotLight spotLight;\n\t#if defined( USE_SHADOWMAP ) && NUM_SPOT_LIGHT_SHADOWS > 0\n\tSpotLightShadow spotLightShadow;\n\t#endif\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_SPOT_LIGHTS; i ++ ) {\n\t\tspotLight = spotLights[ i ];\n\t\tgetSpotDirectLightIrradiance( spotLight, geometry, directLight );\n\t\t#if defined( USE_SHADOWMAP ) && ( UNROLLED_LOOP_INDEX < NUM_SPOT_LIGHT_SHADOWS )\n\t\tspotLightShadow = spotLightShadows[ i ];\n\t\tdirectLight.color *= all( bvec2( directLight.visible, receiveShadow ) ) ? getShadow( spotShadowMap[ i ], spotLightShadow.shadowMapSize, spotLightShadow.shadowBias, spotLightShadow.shadowRadius, vSpotShadowCoord[ i ] ) : 1.0;\n\t\t#endif\n\t\tRE_Direct( directLight, geometry, material, reflectedLight );\n\t}\n\t#pragma unroll_loop_end\n#endif\n#if ( NUM_DIR_LIGHTS > 0 ) && defined( RE_Direct )\n\tDirectionalLight directionalLight;\n\t#if defined( USE_SHADOWMAP ) && NUM_DIR_LIGHT_SHADOWS > 0\n\tDirectionalLightShadow directionalLightShadow;\n\t#endif\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_DIR_LIGHTS; i ++ ) {\n\t\tdirectionalLight = directionalLights[ i ];\n\t\tgetDirectionalDirectLightIrradiance( directionalLight, geometry, directLight );\n\t\t#if defined( USE_SHADOWMAP ) && ( UNROLLED_LOOP_INDEX < NUM_DIR_LIGHT_SHADOWS )\n\t\tdirectionalLightShadow = directionalLightShadows[ i ];\n\t\tdirectLight.color *= all( bvec2( directLight.visible, receiveShadow ) ) ? getShadow( directionalShadowMap[ i ], directionalLightShadow.shadowMapSize, directionalLightShadow.shadowBias, directionalLightShadow.shadowRadius, vDirectionalShadowCoord[ i ] ) : 1.0;\n\t\t#endif\n\t\tRE_Direct( directLight, geometry, material, reflectedLight );\n\t}\n\t#pragma unroll_loop_end\n#endif\n#if ( NUM_RECT_AREA_LIGHTS > 0 ) && defined( RE_Direct_RectArea )\n\tRectAreaLight rectAreaLight;\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_RECT_AREA_LIGHTS; i ++ ) {\n\t\trectAreaLight = rectAreaLights[ i ];\n\t\tRE_Direct_RectArea( rectAreaLight, geometry, material, reflectedLight );\n\t}\n\t#pragma unroll_loop_end\n#endif\n#if defined( RE_IndirectDiffuse )\n\tvec3 iblIrradiance = vec3( 0.0 );\n\tvec3 irradiance = getAmbientLightIrradiance( ambientLightColor );\n\tirradiance += getLightProbeIrradiance( lightProbe, geometry );\n\t#if ( NUM_HEMI_LIGHTS > 0 )\n\t\t#pragma unroll_loop_start\n\t\tfor ( int i = 0; i < NUM_HEMI_LIGHTS; i ++ ) {\n\t\t\tirradiance += getHemisphereLightIrradiance( hemisphereLights[ i ], geometry );\n\t\t}\n\t\t#pragma unroll_loop_end\n\t#endif\n#endif\n#if defined( RE_IndirectSpecular )\n\tvec3 radiance = vec3( 0.0 );\n\tvec3 clearcoatRadiance = vec3( 0.0 );\n#endif";

var lights_fragment_maps = "#if defined( RE_IndirectDiffuse )\n\t#ifdef USE_LIGHTMAP\n\t\tvec4 lightMapTexel= texture2D( lightMap, vUv2 );\n\t\tvec3 lightMapIrradiance = lightMapTexelToLinear( lightMapTexel ).rgb * lightMapIntensity;\n\t\t#ifndef PHYSICALLY_CORRECT_LIGHTS\n\t\t\tlightMapIrradiance *= PI;\n\t\t#endif\n\t\tirradiance += lightMapIrradiance;\n\t#endif\n\t#if defined( USE_ENVMAP ) && defined( STANDARD ) && defined( ENVMAP_TYPE_CUBE_UV )\n\t\tiblIrradiance += getLightProbeIndirectIrradiance( geometry, maxMipLevel );\n\t#endif\n#endif\n#if defined( USE_ENVMAP ) && defined( RE_IndirectSpecular )\n\tradiance += getLightProbeIndirectRadiance( geometry.viewDir, geometry.normal, material.specularRoughness, maxMipLevel );\n\t#ifdef CLEARCOAT\n\t\tclearcoatRadiance += getLightProbeIndirectRadiance( geometry.viewDir, geometry.clearcoatNormal, material.clearcoatRoughness, maxMipLevel );\n\t#endif\n#endif";

var lights_fragment_end = "#if defined( RE_IndirectDiffuse )\n\tRE_IndirectDiffuse( irradiance, geometry, material, reflectedLight );\n#endif\n#if defined( RE_IndirectSpecular )\n\tRE_IndirectSpecular( radiance, iblIrradiance, clearcoatRadiance, geometry, material, reflectedLight );\n#endif";

var logdepthbuf_fragment = "#if defined( USE_LOGDEPTHBUF ) && defined( USE_LOGDEPTHBUF_EXT )\n\tgl_FragDepthEXT = vIsPerspective == 0.0 ? gl_FragCoord.z : log2( vFragDepth ) * logDepthBufFC * 0.5;\n#endif";

var logdepthbuf_pars_fragment = "#if defined( USE_LOGDEPTHBUF ) && defined( USE_LOGDEPTHBUF_EXT )\n\tuniform float logDepthBufFC;\n\tvarying float vFragDepth;\n\tvarying float vIsPerspective;\n#endif";

var logdepthbuf_pars_vertex = "#ifdef USE_LOGDEPTHBUF\n\t#ifdef USE_LOGDEPTHBUF_EXT\n\t\tvarying float vFragDepth;\n\t\tvarying float vIsPerspective;\n\t#else\n\t\tuniform float logDepthBufFC;\n\t#endif\n#endif";

var logdepthbuf_vertex = "#ifdef USE_LOGDEPTHBUF\n\t#ifdef USE_LOGDEPTHBUF_EXT\n\t\tvFragDepth = 1.0 + gl_Position.w;\n\t\tvIsPerspective = float( isPerspectiveMatrix( projectionMatrix ) );\n\t#else\n\t\tif ( isPerspectiveMatrix( projectionMatrix ) ) {\n\t\t\tgl_Position.z = log2( max( EPSILON, gl_Position.w + 1.0 ) ) * logDepthBufFC - 1.0;\n\t\t\tgl_Position.z *= gl_Position.w;\n\t\t}\n\t#endif\n#endif";

var map_fragment = "#ifdef USE_MAP\n\tvec4 texelColor = texture2D( map, vUv );\n\ttexelColor = mapTexelToLinear( texelColor );\n\tdiffuseColor *= texelColor;\n#endif";

var map_pars_fragment = "#ifdef USE_MAP\n\tuniform sampler2D map;\n#endif";

var map_particle_fragment = "#if defined( USE_MAP ) || defined( USE_ALPHAMAP )\n\tvec2 uv = ( uvTransform * vec3( gl_PointCoord.x, 1.0 - gl_PointCoord.y, 1 ) ).xy;\n#endif\n#ifdef USE_MAP\n\tvec4 mapTexel = texture2D( map, uv );\n\tdiffuseColor *= mapTexelToLinear( mapTexel );\n#endif\n#ifdef USE_ALPHAMAP\n\tdiffuseColor.a *= texture2D( alphaMap, uv ).g;\n#endif";

var map_particle_pars_fragment = "#if defined( USE_MAP ) || defined( USE_ALPHAMAP )\n\tuniform mat3 uvTransform;\n#endif\n#ifdef USE_MAP\n\tuniform sampler2D map;\n#endif\n#ifdef USE_ALPHAMAP\n\tuniform sampler2D alphaMap;\n#endif";

var metalnessmap_fragment = "float metalnessFactor = metalness;\n#ifdef USE_METALNESSMAP\n\tvec4 texelMetalness = texture2D( metalnessMap, vUv );\n\tmetalnessFactor *= texelMetalness.b;\n#endif";

var metalnessmap_pars_fragment = "#ifdef USE_METALNESSMAP\n\tuniform sampler2D metalnessMap;\n#endif";

var morphnormal_vertex = "#ifdef USE_MORPHNORMALS\n\tobjectNormal *= morphTargetBaseInfluence;\n\tobjectNormal += morphNormal0 * morphTargetInfluences[ 0 ];\n\tobjectNormal += morphNormal1 * morphTargetInfluences[ 1 ];\n\tobjectNormal += morphNormal2 * morphTargetInfluences[ 2 ];\n\tobjectNormal += morphNormal3 * morphTargetInfluences[ 3 ];\n#endif";

var morphtarget_pars_vertex = "#ifdef USE_MORPHTARGETS\n\tuniform float morphTargetBaseInfluence;\n\t#ifndef USE_MORPHNORMALS\n\t\tuniform float morphTargetInfluences[ 8 ];\n\t#else\n\t\tuniform float morphTargetInfluences[ 4 ];\n\t#endif\n#endif";

var morphtarget_vertex = "#ifdef USE_MORPHTARGETS\n\ttransformed *= morphTargetBaseInfluence;\n\ttransformed += morphTarget0 * morphTargetInfluences[ 0 ];\n\ttransformed += morphTarget1 * morphTargetInfluences[ 1 ];\n\ttransformed += morphTarget2 * morphTargetInfluences[ 2 ];\n\ttransformed += morphTarget3 * morphTargetInfluences[ 3 ];\n\t#ifndef USE_MORPHNORMALS\n\t\ttransformed += morphTarget4 * morphTargetInfluences[ 4 ];\n\t\ttransformed += morphTarget5 * morphTargetInfluences[ 5 ];\n\t\ttransformed += morphTarget6 * morphTargetInfluences[ 6 ];\n\t\ttransformed += morphTarget7 * morphTargetInfluences[ 7 ];\n\t#endif\n#endif";

var normal_fragment_begin = "float faceDirection = gl_FrontFacing ? 1.0 : - 1.0;\n#ifdef FLAT_SHADED\n\tvec3 fdx = vec3( dFdx( vViewPosition.x ), dFdx( vViewPosition.y ), dFdx( vViewPosition.z ) );\n\tvec3 fdy = vec3( dFdy( vViewPosition.x ), dFdy( vViewPosition.y ), dFdy( vViewPosition.z ) );\n\tvec3 normal = normalize( cross( fdx, fdy ) );\n#else\n\tvec3 normal = normalize( vNormal );\n\t#ifdef DOUBLE_SIDED\n\t\tnormal = normal * faceDirection;\n\t#endif\n\t#ifdef USE_TANGENT\n\t\tvec3 tangent = normalize( vTangent );\n\t\tvec3 bitangent = normalize( vBitangent );\n\t\t#ifdef DOUBLE_SIDED\n\t\t\ttangent = tangent * faceDirection;\n\t\t\tbitangent = bitangent * faceDirection;\n\t\t#endif\n\t\t#if defined( TANGENTSPACE_NORMALMAP ) || defined( USE_CLEARCOAT_NORMALMAP )\n\t\t\tmat3 vTBN = mat3( tangent, bitangent, normal );\n\t\t#endif\n\t#endif\n#endif\nvec3 geometryNormal = normal;";

var normal_fragment_maps = "#ifdef OBJECTSPACE_NORMALMAP\n\tnormal = texture2D( normalMap, vUv ).xyz * 2.0 - 1.0;\n\t#ifdef FLIP_SIDED\n\t\tnormal = - normal;\n\t#endif\n\t#ifdef DOUBLE_SIDED\n\t\tnormal = normal * faceDirection;\n\t#endif\n\tnormal = normalize( normalMatrix * normal );\n#elif defined( TANGENTSPACE_NORMALMAP )\n\tvec3 mapN = texture2D( normalMap, vUv ).xyz * 2.0 - 1.0;\n\tmapN.xy *= normalScale;\n\t#ifdef USE_TANGENT\n\t\tnormal = normalize( vTBN * mapN );\n\t#else\n\t\tnormal = perturbNormal2Arb( -vViewPosition, normal, mapN, faceDirection );\n\t#endif\n#elif defined( USE_BUMPMAP )\n\tnormal = perturbNormalArb( -vViewPosition, normal, dHdxy_fwd(), faceDirection );\n#endif";

var normalmap_pars_fragment = "#ifdef USE_NORMALMAP\n\tuniform sampler2D normalMap;\n\tuniform vec2 normalScale;\n#endif\n#ifdef OBJECTSPACE_NORMALMAP\n\tuniform mat3 normalMatrix;\n#endif\n#if ! defined ( USE_TANGENT ) && ( defined ( TANGENTSPACE_NORMALMAP ) || defined ( USE_CLEARCOAT_NORMALMAP ) )\n\tvec3 perturbNormal2Arb( vec3 eye_pos, vec3 surf_norm, vec3 mapN, float faceDirection ) {\n\t\tvec3 q0 = vec3( dFdx( eye_pos.x ), dFdx( eye_pos.y ), dFdx( eye_pos.z ) );\n\t\tvec3 q1 = vec3( dFdy( eye_pos.x ), dFdy( eye_pos.y ), dFdy( eye_pos.z ) );\n\t\tvec2 st0 = dFdx( vUv.st );\n\t\tvec2 st1 = dFdy( vUv.st );\n\t\tvec3 N = surf_norm;\n\t\tvec3 q1perp = cross( q1, N );\n\t\tvec3 q0perp = cross( N, q0 );\n\t\tvec3 T = q1perp * st0.x + q0perp * st1.x;\n\t\tvec3 B = q1perp * st0.y + q0perp * st1.y;\n\t\tfloat det = max( dot( T, T ), dot( B, B ) );\n\t\tfloat scale = ( det == 0.0 ) ? 0.0 : faceDirection * inversesqrt( det );\n\t\treturn normalize( T * ( mapN.x * scale ) + B * ( mapN.y * scale ) + N * mapN.z );\n\t}\n#endif";

var clearcoat_normal_fragment_begin = "#ifdef CLEARCOAT\n\tvec3 clearcoatNormal = geometryNormal;\n#endif";

var clearcoat_normal_fragment_maps = "#ifdef USE_CLEARCOAT_NORMALMAP\n\tvec3 clearcoatMapN = texture2D( clearcoatNormalMap, vUv ).xyz * 2.0 - 1.0;\n\tclearcoatMapN.xy *= clearcoatNormalScale;\n\t#ifdef USE_TANGENT\n\t\tclearcoatNormal = normalize( vTBN * clearcoatMapN );\n\t#else\n\t\tclearcoatNormal = perturbNormal2Arb( - vViewPosition, clearcoatNormal, clearcoatMapN, faceDirection );\n\t#endif\n#endif";

var clearcoat_pars_fragment = "#ifdef USE_CLEARCOATMAP\n\tuniform sampler2D clearcoatMap;\n#endif\n#ifdef USE_CLEARCOAT_ROUGHNESSMAP\n\tuniform sampler2D clearcoatRoughnessMap;\n#endif\n#ifdef USE_CLEARCOAT_NORMALMAP\n\tuniform sampler2D clearcoatNormalMap;\n\tuniform vec2 clearcoatNormalScale;\n#endif";

var packing = "vec3 packNormalToRGB( const in vec3 normal ) {\n\treturn normalize( normal ) * 0.5 + 0.5;\n}\nvec3 unpackRGBToNormal( const in vec3 rgb ) {\n\treturn 2.0 * rgb.xyz - 1.0;\n}\nconst float PackUpscale = 256. / 255.;const float UnpackDownscale = 255. / 256.;\nconst vec3 PackFactors = vec3( 256. * 256. * 256., 256. * 256., 256. );\nconst vec4 UnpackFactors = UnpackDownscale / vec4( PackFactors, 1. );\nconst float ShiftRight8 = 1. / 256.;\nvec4 packDepthToRGBA( const in float v ) {\n\tvec4 r = vec4( fract( v * PackFactors ), v );\n\tr.yzw -= r.xyz * ShiftRight8;\treturn r * PackUpscale;\n}\nfloat unpackRGBAToDepth( const in vec4 v ) {\n\treturn dot( v, UnpackFactors );\n}\nvec4 pack2HalfToRGBA( vec2 v ) {\n\tvec4 r = vec4( v.x, fract( v.x * 255.0 ), v.y, fract( v.y * 255.0 ));\n\treturn vec4( r.x - r.y / 255.0, r.y, r.z - r.w / 255.0, r.w);\n}\nvec2 unpackRGBATo2Half( vec4 v ) {\n\treturn vec2( v.x + ( v.y / 255.0 ), v.z + ( v.w / 255.0 ) );\n}\nfloat viewZToOrthographicDepth( const in float viewZ, const in float near, const in float far ) {\n\treturn ( viewZ + near ) / ( near - far );\n}\nfloat orthographicDepthToViewZ( const in float linearClipZ, const in float near, const in float far ) {\n\treturn linearClipZ * ( near - far ) - near;\n}\nfloat viewZToPerspectiveDepth( const in float viewZ, const in float near, const in float far ) {\n\treturn (( near + viewZ ) * far ) / (( far - near ) * viewZ );\n}\nfloat perspectiveDepthToViewZ( const in float invClipZ, const in float near, const in float far ) {\n\treturn ( near * far ) / ( ( far - near ) * invClipZ - far );\n}";

var premultiplied_alpha_fragment = "#ifdef PREMULTIPLIED_ALPHA\n\tgl_FragColor.rgb *= gl_FragColor.a;\n#endif";

var project_vertex = "vec4 mvPosition = vec4( transformed, 1.0 );\n#ifdef USE_INSTANCING\n\tmvPosition = instanceMatrix * mvPosition;\n#endif\nmvPosition = modelViewMatrix * mvPosition;\ngl_Position = projectionMatrix * mvPosition;";

var dithering_fragment = "#ifdef DITHERING\n\tgl_FragColor.rgb = dithering( gl_FragColor.rgb );\n#endif";

var dithering_pars_fragment = "#ifdef DITHERING\n\tvec3 dithering( vec3 color ) {\n\t\tfloat grid_position = rand( gl_FragCoord.xy );\n\t\tvec3 dither_shift_RGB = vec3( 0.25 / 255.0, -0.25 / 255.0, 0.25 / 255.0 );\n\t\tdither_shift_RGB = mix( 2.0 * dither_shift_RGB, -2.0 * dither_shift_RGB, grid_position );\n\t\treturn color + dither_shift_RGB;\n\t}\n#endif";

var roughnessmap_fragment = "float roughnessFactor = roughness;\n#ifdef USE_ROUGHNESSMAP\n\tvec4 texelRoughness = texture2D( roughnessMap, vUv );\n\troughnessFactor *= texelRoughness.g;\n#endif";

var roughnessmap_pars_fragment = "#ifdef USE_ROUGHNESSMAP\n\tuniform sampler2D roughnessMap;\n#endif";

var shadowmap_pars_fragment = "#ifdef USE_SHADOWMAP\n\t#if NUM_DIR_LIGHT_SHADOWS > 0\n\t\tuniform sampler2D directionalShadowMap[ NUM_DIR_LIGHT_SHADOWS ];\n\t\tvarying vec4 vDirectionalShadowCoord[ NUM_DIR_LIGHT_SHADOWS ];\n\t\tstruct DirectionalLightShadow {\n\t\t\tfloat shadowBias;\n\t\t\tfloat shadowNormalBias;\n\t\t\tfloat shadowRadius;\n\t\t\tvec2 shadowMapSize;\n\t\t};\n\t\tuniform DirectionalLightShadow directionalLightShadows[ NUM_DIR_LIGHT_SHADOWS ];\n\t#endif\n\t#if NUM_SPOT_LIGHT_SHADOWS > 0\n\t\tuniform sampler2D spotShadowMap[ NUM_SPOT_LIGHT_SHADOWS ];\n\t\tvarying vec4 vSpotShadowCoord[ NUM_SPOT_LIGHT_SHADOWS ];\n\t\tstruct SpotLightShadow {\n\t\t\tfloat shadowBias;\n\t\t\tfloat shadowNormalBias;\n\t\t\tfloat shadowRadius;\n\t\t\tvec2 shadowMapSize;\n\t\t};\n\t\tuniform SpotLightShadow spotLightShadows[ NUM_SPOT_LIGHT_SHADOWS ];\n\t#endif\n\t#if NUM_POINT_LIGHT_SHADOWS > 0\n\t\tuniform sampler2D pointShadowMap[ NUM_POINT_LIGHT_SHADOWS ];\n\t\tvarying vec4 vPointShadowCoord[ NUM_POINT_LIGHT_SHADOWS ];\n\t\tstruct PointLightShadow {\n\t\t\tfloat shadowBias;\n\t\t\tfloat shadowNormalBias;\n\t\t\tfloat shadowRadius;\n\t\t\tvec2 shadowMapSize;\n\t\t\tfloat shadowCameraNear;\n\t\t\tfloat shadowCameraFar;\n\t\t};\n\t\tuniform PointLightShadow pointLightShadows[ NUM_POINT_LIGHT_SHADOWS ];\n\t#endif\n\tfloat texture2DCompare( sampler2D depths, vec2 uv, float compare ) {\n\t\treturn step( compare, unpackRGBAToDepth( texture2D( depths, uv ) ) );\n\t}\n\tvec2 texture2DDistribution( sampler2D shadow, vec2 uv ) {\n\t\treturn unpackRGBATo2Half( texture2D( shadow, uv ) );\n\t}\n\tfloat VSMShadow (sampler2D shadow, vec2 uv, float compare ){\n\t\tfloat occlusion = 1.0;\n\t\tvec2 distribution = texture2DDistribution( shadow, uv );\n\t\tfloat hard_shadow = step( compare , distribution.x );\n\t\tif (hard_shadow != 1.0 ) {\n\t\t\tfloat distance = compare - distribution.x ;\n\t\t\tfloat variance = max( 0.00000, distribution.y * distribution.y );\n\t\t\tfloat softness_probability = variance / (variance + distance * distance );\t\t\tsoftness_probability = clamp( ( softness_probability - 0.3 ) / ( 0.95 - 0.3 ), 0.0, 1.0 );\t\t\tocclusion = clamp( max( hard_shadow, softness_probability ), 0.0, 1.0 );\n\t\t}\n\t\treturn occlusion;\n\t}\n\tfloat getShadow( sampler2D shadowMap, vec2 shadowMapSize, float shadowBias, float shadowRadius, vec4 shadowCoord ) {\n\t\tfloat shadow = 1.0;\n\t\tshadowCoord.xyz /= shadowCoord.w;\n\t\tshadowCoord.z += shadowBias;\n\t\tbvec4 inFrustumVec = bvec4 ( shadowCoord.x >= 0.0, shadowCoord.x <= 1.0, shadowCoord.y >= 0.0, shadowCoord.y <= 1.0 );\n\t\tbool inFrustum = all( inFrustumVec );\n\t\tbvec2 frustumTestVec = bvec2( inFrustum, shadowCoord.z <= 1.0 );\n\t\tbool frustumTest = all( frustumTestVec );\n\t\tif ( frustumTest ) {\n\t\t#if defined( SHADOWMAP_TYPE_PCF )\n\t\t\tvec2 texelSize = vec2( 1.0 ) / shadowMapSize;\n\t\t\tfloat dx0 = - texelSize.x * shadowRadius;\n\t\t\tfloat dy0 = - texelSize.y * shadowRadius;\n\t\t\tfloat dx1 = + texelSize.x * shadowRadius;\n\t\t\tfloat dy1 = + texelSize.y * shadowRadius;\n\t\t\tfloat dx2 = dx0 / 2.0;\n\t\t\tfloat dy2 = dy0 / 2.0;\n\t\t\tfloat dx3 = dx1 / 2.0;\n\t\t\tfloat dy3 = dy1 / 2.0;\n\t\t\tshadow = (\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx0, dy0 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( 0.0, dy0 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx1, dy0 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx2, dy2 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( 0.0, dy2 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx3, dy2 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx0, 0.0 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx2, 0.0 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy, shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx3, 0.0 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx1, 0.0 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx2, dy3 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( 0.0, dy3 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx3, dy3 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx0, dy1 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( 0.0, dy1 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx1, dy1 ), shadowCoord.z )\n\t\t\t) * ( 1.0 / 17.0 );\n\t\t#elif defined( SHADOWMAP_TYPE_PCF_SOFT )\n\t\t\tvec2 texelSize = vec2( 1.0 ) / shadowMapSize;\n\t\t\tfloat dx = texelSize.x;\n\t\t\tfloat dy = texelSize.y;\n\t\t\tvec2 uv = shadowCoord.xy;\n\t\t\tvec2 f = fract( uv * shadowMapSize + 0.5 );\n\t\t\tuv -= f * texelSize;\n\t\t\tshadow = (\n\t\t\t\ttexture2DCompare( shadowMap, uv, shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, uv + vec2( dx, 0.0 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, uv + vec2( 0.0, dy ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, uv + texelSize, shadowCoord.z ) +\n\t\t\t\tmix( texture2DCompare( shadowMap, uv + vec2( -dx, 0.0 ), shadowCoord.z ), \n\t\t\t\t\t texture2DCompare( shadowMap, uv + vec2( 2.0 * dx, 0.0 ), shadowCoord.z ),\n\t\t\t\t\t f.x ) +\n\t\t\t\tmix( texture2DCompare( shadowMap, uv + vec2( -dx, dy ), shadowCoord.z ), \n\t\t\t\t\t texture2DCompare( shadowMap, uv + vec2( 2.0 * dx, dy ), shadowCoord.z ),\n\t\t\t\t\t f.x ) +\n\t\t\t\tmix( texture2DCompare( shadowMap, uv + vec2( 0.0, -dy ), shadowCoord.z ), \n\t\t\t\t\t texture2DCompare( shadowMap, uv + vec2( 0.0, 2.0 * dy ), shadowCoord.z ),\n\t\t\t\t\t f.y ) +\n\t\t\t\tmix( texture2DCompare( shadowMap, uv + vec2( dx, -dy ), shadowCoord.z ), \n\t\t\t\t\t texture2DCompare( shadowMap, uv + vec2( dx, 2.0 * dy ), shadowCoord.z ),\n\t\t\t\t\t f.y ) +\n\t\t\t\tmix( mix( texture2DCompare( shadowMap, uv + vec2( -dx, -dy ), shadowCoord.z ), \n\t\t\t\t\t\t  texture2DCompare( shadowMap, uv + vec2( 2.0 * dx, -dy ), shadowCoord.z ),\n\t\t\t\t\t\t  f.x ),\n\t\t\t\t\t mix( texture2DCompare( shadowMap, uv + vec2( -dx, 2.0 * dy ), shadowCoord.z ), \n\t\t\t\t\t\t  texture2DCompare( shadowMap, uv + vec2( 2.0 * dx, 2.0 * dy ), shadowCoord.z ),\n\t\t\t\t\t\t  f.x ),\n\t\t\t\t\t f.y )\n\t\t\t) * ( 1.0 / 9.0 );\n\t\t#elif defined( SHADOWMAP_TYPE_VSM )\n\t\t\tshadow = VSMShadow( shadowMap, shadowCoord.xy, shadowCoord.z );\n\t\t#else\n\t\t\tshadow = texture2DCompare( shadowMap, shadowCoord.xy, shadowCoord.z );\n\t\t#endif\n\t\t}\n\t\treturn shadow;\n\t}\n\tvec2 cubeToUV( vec3 v, float texelSizeY ) {\n\t\tvec3 absV = abs( v );\n\t\tfloat scaleToCube = 1.0 / max( absV.x, max( absV.y, absV.z ) );\n\t\tabsV *= scaleToCube;\n\t\tv *= scaleToCube * ( 1.0 - 2.0 * texelSizeY );\n\t\tvec2 planar = v.xy;\n\t\tfloat almostATexel = 1.5 * texelSizeY;\n\t\tfloat almostOne = 1.0 - almostATexel;\n\t\tif ( absV.z >= almostOne ) {\n\t\t\tif ( v.z > 0.0 )\n\t\t\t\tplanar.x = 4.0 - v.x;\n\t\t} else if ( absV.x >= almostOne ) {\n\t\t\tfloat signX = sign( v.x );\n\t\t\tplanar.x = v.z * signX + 2.0 * signX;\n\t\t} else if ( absV.y >= almostOne ) {\n\t\t\tfloat signY = sign( v.y );\n\t\t\tplanar.x = v.x + 2.0 * signY + 2.0;\n\t\t\tplanar.y = v.z * signY - 2.0;\n\t\t}\n\t\treturn vec2( 0.125, 0.25 ) * planar + vec2( 0.375, 0.75 );\n\t}\n\tfloat getPointShadow( sampler2D shadowMap, vec2 shadowMapSize, float shadowBias, float shadowRadius, vec4 shadowCoord, float shadowCameraNear, float shadowCameraFar ) {\n\t\tvec2 texelSize = vec2( 1.0 ) / ( shadowMapSize * vec2( 4.0, 2.0 ) );\n\t\tvec3 lightToPosition = shadowCoord.xyz;\n\t\tfloat dp = ( length( lightToPosition ) - shadowCameraNear ) / ( shadowCameraFar - shadowCameraNear );\t\tdp += shadowBias;\n\t\tvec3 bd3D = normalize( lightToPosition );\n\t\t#if defined( SHADOWMAP_TYPE_PCF ) || defined( SHADOWMAP_TYPE_PCF_SOFT ) || defined( SHADOWMAP_TYPE_VSM )\n\t\t\tvec2 offset = vec2( - 1, 1 ) * shadowRadius * texelSize.y;\n\t\t\treturn (\n\t\t\t\ttexture2DCompare( shadowMap, cubeToUV( bd3D + offset.xyy, texelSize.y ), dp ) +\n\t\t\t\ttexture2DCompare( shadowMap, cubeToUV( bd3D + offset.yyy, texelSize.y ), dp ) +\n\t\t\t\ttexture2DCompare( shadowMap, cubeToUV( bd3D + offset.xyx, texelSize.y ), dp ) +\n\t\t\t\ttexture2DCompare( shadowMap, cubeToUV( bd3D + offset.yyx, texelSize.y ), dp ) +\n\t\t\t\ttexture2DCompare( shadowMap, cubeToUV( bd3D, texelSize.y ), dp ) +\n\t\t\t\ttexture2DCompare( shadowMap, cubeToUV( bd3D + offset.xxy, texelSize.y ), dp ) +\n\t\t\t\ttexture2DCompare( shadowMap, cubeToUV( bd3D + offset.yxy, texelSize.y ), dp ) +\n\t\t\t\ttexture2DCompare( shadowMap, cubeToUV( bd3D + offset.xxx, texelSize.y ), dp ) +\n\t\t\t\ttexture2DCompare( shadowMap, cubeToUV( bd3D + offset.yxx, texelSize.y ), dp )\n\t\t\t) * ( 1.0 / 9.0 );\n\t\t#else\n\t\t\treturn texture2DCompare( shadowMap, cubeToUV( bd3D, texelSize.y ), dp );\n\t\t#endif\n\t}\n#endif";

var shadowmap_pars_vertex = "#ifdef USE_SHADOWMAP\n\t#if NUM_DIR_LIGHT_SHADOWS > 0\n\t\tuniform mat4 directionalShadowMatrix[ NUM_DIR_LIGHT_SHADOWS ];\n\t\tvarying vec4 vDirectionalShadowCoord[ NUM_DIR_LIGHT_SHADOWS ];\n\t\tstruct DirectionalLightShadow {\n\t\t\tfloat shadowBias;\n\t\t\tfloat shadowNormalBias;\n\t\t\tfloat shadowRadius;\n\t\t\tvec2 shadowMapSize;\n\t\t};\n\t\tuniform DirectionalLightShadow directionalLightShadows[ NUM_DIR_LIGHT_SHADOWS ];\n\t#endif\n\t#if NUM_SPOT_LIGHT_SHADOWS > 0\n\t\tuniform mat4 spotShadowMatrix[ NUM_SPOT_LIGHT_SHADOWS ];\n\t\tvarying vec4 vSpotShadowCoord[ NUM_SPOT_LIGHT_SHADOWS ];\n\t\tstruct SpotLightShadow {\n\t\t\tfloat shadowBias;\n\t\t\tfloat shadowNormalBias;\n\t\t\tfloat shadowRadius;\n\t\t\tvec2 shadowMapSize;\n\t\t};\n\t\tuniform SpotLightShadow spotLightShadows[ NUM_SPOT_LIGHT_SHADOWS ];\n\t#endif\n\t#if NUM_POINT_LIGHT_SHADOWS > 0\n\t\tuniform mat4 pointShadowMatrix[ NUM_POINT_LIGHT_SHADOWS ];\n\t\tvarying vec4 vPointShadowCoord[ NUM_POINT_LIGHT_SHADOWS ];\n\t\tstruct PointLightShadow {\n\t\t\tfloat shadowBias;\n\t\t\tfloat shadowNormalBias;\n\t\t\tfloat shadowRadius;\n\t\t\tvec2 shadowMapSize;\n\t\t\tfloat shadowCameraNear;\n\t\t\tfloat shadowCameraFar;\n\t\t};\n\t\tuniform PointLightShadow pointLightShadows[ NUM_POINT_LIGHT_SHADOWS ];\n\t#endif\n#endif";

var shadowmap_vertex = "#ifdef USE_SHADOWMAP\n\t#if NUM_DIR_LIGHT_SHADOWS > 0 || NUM_SPOT_LIGHT_SHADOWS > 0 || NUM_POINT_LIGHT_SHADOWS > 0\n\t\tvec3 shadowWorldNormal = inverseTransformDirection( transformedNormal, viewMatrix );\n\t\tvec4 shadowWorldPosition;\n\t#endif\n\t#if NUM_DIR_LIGHT_SHADOWS > 0\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_DIR_LIGHT_SHADOWS; i ++ ) {\n\t\tshadowWorldPosition = worldPosition + vec4( shadowWorldNormal * directionalLightShadows[ i ].shadowNormalBias, 0 );\n\t\tvDirectionalShadowCoord[ i ] = directionalShadowMatrix[ i ] * shadowWorldPosition;\n\t}\n\t#pragma unroll_loop_end\n\t#endif\n\t#if NUM_SPOT_LIGHT_SHADOWS > 0\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_SPOT_LIGHT_SHADOWS; i ++ ) {\n\t\tshadowWorldPosition = worldPosition + vec4( shadowWorldNormal * spotLightShadows[ i ].shadowNormalBias, 0 );\n\t\tvSpotShadowCoord[ i ] = spotShadowMatrix[ i ] * shadowWorldPosition;\n\t}\n\t#pragma unroll_loop_end\n\t#endif\n\t#if NUM_POINT_LIGHT_SHADOWS > 0\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_POINT_LIGHT_SHADOWS; i ++ ) {\n\t\tshadowWorldPosition = worldPosition + vec4( shadowWorldNormal * pointLightShadows[ i ].shadowNormalBias, 0 );\n\t\tvPointShadowCoord[ i ] = pointShadowMatrix[ i ] * shadowWorldPosition;\n\t}\n\t#pragma unroll_loop_end\n\t#endif\n#endif";

var shadowmask_pars_fragment = "float getShadowMask() {\n\tfloat shadow = 1.0;\n\t#ifdef USE_SHADOWMAP\n\t#if NUM_DIR_LIGHT_SHADOWS > 0\n\tDirectionalLightShadow directionalLight;\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_DIR_LIGHT_SHADOWS; i ++ ) {\n\t\tdirectionalLight = directionalLightShadows[ i ];\n\t\tshadow *= receiveShadow ? getShadow( directionalShadowMap[ i ], directionalLight.shadowMapSize, directionalLight.shadowBias, directionalLight.shadowRadius, vDirectionalShadowCoord[ i ] ) : 1.0;\n\t}\n\t#pragma unroll_loop_end\n\t#endif\n\t#if NUM_SPOT_LIGHT_SHADOWS > 0\n\tSpotLightShadow spotLight;\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_SPOT_LIGHT_SHADOWS; i ++ ) {\n\t\tspotLight = spotLightShadows[ i ];\n\t\tshadow *= receiveShadow ? getShadow( spotShadowMap[ i ], spotLight.shadowMapSize, spotLight.shadowBias, spotLight.shadowRadius, vSpotShadowCoord[ i ] ) : 1.0;\n\t}\n\t#pragma unroll_loop_end\n\t#endif\n\t#if NUM_POINT_LIGHT_SHADOWS > 0\n\tPointLightShadow pointLight;\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_POINT_LIGHT_SHADOWS; i ++ ) {\n\t\tpointLight = pointLightShadows[ i ];\n\t\tshadow *= receiveShadow ? getPointShadow( pointShadowMap[ i ], pointLight.shadowMapSize, pointLight.shadowBias, pointLight.shadowRadius, vPointShadowCoord[ i ], pointLight.shadowCameraNear, pointLight.shadowCameraFar ) : 1.0;\n\t}\n\t#pragma unroll_loop_end\n\t#endif\n\t#endif\n\treturn shadow;\n}";

var skinbase_vertex = "#ifdef USE_SKINNING\n\tmat4 boneMatX = getBoneMatrix( skinIndex.x );\n\tmat4 boneMatY = getBoneMatrix( skinIndex.y );\n\tmat4 boneMatZ = getBoneMatrix( skinIndex.z );\n\tmat4 boneMatW = getBoneMatrix( skinIndex.w );\n#endif";

var skinning_pars_vertex = "#ifdef USE_SKINNING\n\tuniform mat4 bindMatrix;\n\tuniform mat4 bindMatrixInverse;\n\t#ifdef BONE_TEXTURE\n\t\tuniform highp sampler2D boneTexture;\n\t\tuniform int boneTextureSize;\n\t\tmat4 getBoneMatrix( const in float i ) {\n\t\t\tfloat j = i * 4.0;\n\t\t\tfloat x = mod( j, float( boneTextureSize ) );\n\t\t\tfloat y = floor( j / float( boneTextureSize ) );\n\t\t\tfloat dx = 1.0 / float( boneTextureSize );\n\t\t\tfloat dy = 1.0 / float( boneTextureSize );\n\t\t\ty = dy * ( y + 0.5 );\n\t\t\tvec4 v1 = texture2D( boneTexture, vec2( dx * ( x + 0.5 ), y ) );\n\t\t\tvec4 v2 = texture2D( boneTexture, vec2( dx * ( x + 1.5 ), y ) );\n\t\t\tvec4 v3 = texture2D( boneTexture, vec2( dx * ( x + 2.5 ), y ) );\n\t\t\tvec4 v4 = texture2D( boneTexture, vec2( dx * ( x + 3.5 ), y ) );\n\t\t\tmat4 bone = mat4( v1, v2, v3, v4 );\n\t\t\treturn bone;\n\t\t}\n\t#else\n\t\tuniform mat4 boneMatrices[ MAX_BONES ];\n\t\tmat4 getBoneMatrix( const in float i ) {\n\t\t\tmat4 bone = boneMatrices[ int(i) ];\n\t\t\treturn bone;\n\t\t}\n\t#endif\n#endif";

var skinning_vertex = "#ifdef USE_SKINNING\n\tvec4 skinVertex = bindMatrix * vec4( transformed, 1.0 );\n\tvec4 skinned = vec4( 0.0 );\n\tskinned += boneMatX * skinVertex * skinWeight.x;\n\tskinned += boneMatY * skinVertex * skinWeight.y;\n\tskinned += boneMatZ * skinVertex * skinWeight.z;\n\tskinned += boneMatW * skinVertex * skinWeight.w;\n\ttransformed = ( bindMatrixInverse * skinned ).xyz;\n#endif";

var skinnormal_vertex = "#ifdef USE_SKINNING\n\tmat4 skinMatrix = mat4( 0.0 );\n\tskinMatrix += skinWeight.x * boneMatX;\n\tskinMatrix += skinWeight.y * boneMatY;\n\tskinMatrix += skinWeight.z * boneMatZ;\n\tskinMatrix += skinWeight.w * boneMatW;\n\tskinMatrix = bindMatrixInverse * skinMatrix * bindMatrix;\n\tobjectNormal = vec4( skinMatrix * vec4( objectNormal, 0.0 ) ).xyz;\n\t#ifdef USE_TANGENT\n\t\tobjectTangent = vec4( skinMatrix * vec4( objectTangent, 0.0 ) ).xyz;\n\t#endif\n#endif";

var specularmap_fragment = "float specularStrength;\n#ifdef USE_SPECULARMAP\n\tvec4 texelSpecular = texture2D( specularMap, vUv );\n\tspecularStrength = texelSpecular.r;\n#else\n\tspecularStrength = 1.0;\n#endif";

var specularmap_pars_fragment = "#ifdef USE_SPECULARMAP\n\tuniform sampler2D specularMap;\n#endif";

var tonemapping_fragment = "#if defined( TONE_MAPPING )\n\tgl_FragColor.rgb = toneMapping( gl_FragColor.rgb );\n#endif";

var tonemapping_pars_fragment = "#ifndef saturate\n#define saturate(a) clamp( a, 0.0, 1.0 )\n#endif\nuniform float toneMappingExposure;\nvec3 LinearToneMapping( vec3 color ) {\n\treturn toneMappingExposure * color;\n}\nvec3 ReinhardToneMapping( vec3 color ) {\n\tcolor *= toneMappingExposure;\n\treturn saturate( color / ( vec3( 1.0 ) + color ) );\n}\nvec3 OptimizedCineonToneMapping( vec3 color ) {\n\tcolor *= toneMappingExposure;\n\tcolor = max( vec3( 0.0 ), color - 0.004 );\n\treturn pow( ( color * ( 6.2 * color + 0.5 ) ) / ( color * ( 6.2 * color + 1.7 ) + 0.06 ), vec3( 2.2 ) );\n}\nvec3 RRTAndODTFit( vec3 v ) {\n\tvec3 a = v * ( v + 0.0245786 ) - 0.000090537;\n\tvec3 b = v * ( 0.983729 * v + 0.4329510 ) + 0.238081;\n\treturn a / b;\n}\nvec3 ACESFilmicToneMapping( vec3 color ) {\n\tconst mat3 ACESInputMat = mat3(\n\t\tvec3( 0.59719, 0.07600, 0.02840 ),\t\tvec3( 0.35458, 0.90834, 0.13383 ),\n\t\tvec3( 0.04823, 0.01566, 0.83777 )\n\t);\n\tconst mat3 ACESOutputMat = mat3(\n\t\tvec3(  1.60475, -0.10208, -0.00327 ),\t\tvec3( -0.53108,  1.10813, -0.07276 ),\n\t\tvec3( -0.07367, -0.00605,  1.07602 )\n\t);\n\tcolor *= toneMappingExposure / 0.6;\n\tcolor = ACESInputMat * color;\n\tcolor = RRTAndODTFit( color );\n\tcolor = ACESOutputMat * color;\n\treturn saturate( color );\n}\nvec3 CustomToneMapping( vec3 color ) { return color; }";

var transmissionmap_fragment = "#ifdef USE_TRANSMISSIONMAP\n\ttotalTransmission *= texture2D( transmissionMap, vUv ).r;\n#endif";

var transmissionmap_pars_fragment = "#ifdef USE_TRANSMISSIONMAP\n\tuniform sampler2D transmissionMap;\n#endif";

var uv_pars_fragment = "#if ( defined( USE_UV ) && ! defined( UVS_VERTEX_ONLY ) )\n\tvarying vec2 vUv;\n#endif";

var uv_pars_vertex = "#ifdef USE_UV\n\t#ifdef UVS_VERTEX_ONLY\n\t\tvec2 vUv;\n\t#else\n\t\tvarying vec2 vUv;\n\t#endif\n\tuniform mat3 uvTransform;\n#endif";

var uv_vertex = "#ifdef USE_UV\n\tvUv = ( uvTransform * vec3( uv, 1 ) ).xy;\n#endif";

var uv2_pars_fragment = "#if defined( USE_LIGHTMAP ) || defined( USE_AOMAP )\n\tvarying vec2 vUv2;\n#endif";

var uv2_pars_vertex = "#if defined( USE_LIGHTMAP ) || defined( USE_AOMAP )\n\tattribute vec2 uv2;\n\tvarying vec2 vUv2;\n\tuniform mat3 uv2Transform;\n#endif";

var uv2_vertex = "#if defined( USE_LIGHTMAP ) || defined( USE_AOMAP )\n\tvUv2 = ( uv2Transform * vec3( uv2, 1 ) ).xy;\n#endif";

var worldpos_vertex = "#if defined( USE_ENVMAP ) || defined( DISTANCE ) || defined ( USE_SHADOWMAP )\n\tvec4 worldPosition = vec4( transformed, 1.0 );\n\t#ifdef USE_INSTANCING\n\t\tworldPosition = instanceMatrix * worldPosition;\n\t#endif\n\tworldPosition = modelMatrix * worldPosition;\n#endif";

var background_frag = "uniform sampler2D t2D;\nvarying vec2 vUv;\nvoid main() {\n\tvec4 texColor = texture2D( t2D, vUv );\n\tgl_FragColor = mapTexelToLinear( texColor );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n}";

var background_vert = "varying vec2 vUv;\nuniform mat3 uvTransform;\nvoid main() {\n\tvUv = ( uvTransform * vec3( uv, 1 ) ).xy;\n\tgl_Position = vec4( position.xy, 1.0, 1.0 );\n}";

var cube_frag = "#include <envmap_common_pars_fragment>\nuniform float opacity;\nvarying vec3 vWorldDirection;\n#include <cube_uv_reflection_fragment>\nvoid main() {\n\tvec3 vReflect = vWorldDirection;\n\t#include <envmap_fragment>\n\tgl_FragColor = envColor;\n\tgl_FragColor.a *= opacity;\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n}";

var cube_vert = "varying vec3 vWorldDirection;\n#include <common>\nvoid main() {\n\tvWorldDirection = transformDirection( position, modelMatrix );\n\t#include <begin_vertex>\n\t#include <project_vertex>\n\tgl_Position.z = gl_Position.w;\n}";

var depth_frag = "#if DEPTH_PACKING == 3200\n\tuniform float opacity;\n#endif\n#include <common>\n#include <packing>\n#include <uv_pars_fragment>\n#include <map_pars_fragment>\n#include <alphamap_pars_fragment>\n#include <logdepthbuf_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvarying vec2 vHighPrecisionZW;\nvoid main() {\n\t#include <clipping_planes_fragment>\n\tvec4 diffuseColor = vec4( 1.0 );\n\t#if DEPTH_PACKING == 3200\n\t\tdiffuseColor.a = opacity;\n\t#endif\n\t#include <map_fragment>\n\t#include <alphamap_fragment>\n\t#include <alphatest_fragment>\n\t#include <logdepthbuf_fragment>\n\tfloat fragCoordZ = 0.5 * vHighPrecisionZW[0] / vHighPrecisionZW[1] + 0.5;\n\t#if DEPTH_PACKING == 3200\n\t\tgl_FragColor = vec4( vec3( 1.0 - fragCoordZ ), opacity );\n\t#elif DEPTH_PACKING == 3201\n\t\tgl_FragColor = packDepthToRGBA( fragCoordZ );\n\t#endif\n}";

var depth_vert = "#include <common>\n#include <uv_pars_vertex>\n#include <displacementmap_pars_vertex>\n#include <morphtarget_pars_vertex>\n#include <skinning_pars_vertex>\n#include <logdepthbuf_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvarying vec2 vHighPrecisionZW;\nvoid main() {\n\t#include <uv_vertex>\n\t#include <skinbase_vertex>\n\t#ifdef USE_DISPLACEMENTMAP\n\t\t#include <beginnormal_vertex>\n\t\t#include <morphnormal_vertex>\n\t\t#include <skinnormal_vertex>\n\t#endif\n\t#include <begin_vertex>\n\t#include <morphtarget_vertex>\n\t#include <skinning_vertex>\n\t#include <displacementmap_vertex>\n\t#include <project_vertex>\n\t#include <logdepthbuf_vertex>\n\t#include <clipping_planes_vertex>\n\tvHighPrecisionZW = gl_Position.zw;\n}";

var distanceRGBA_frag = "#define DISTANCE\nuniform vec3 referencePosition;\nuniform float nearDistance;\nuniform float farDistance;\nvarying vec3 vWorldPosition;\n#include <common>\n#include <packing>\n#include <uv_pars_fragment>\n#include <map_pars_fragment>\n#include <alphamap_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvoid main () {\n\t#include <clipping_planes_fragment>\n\tvec4 diffuseColor = vec4( 1.0 );\n\t#include <map_fragment>\n\t#include <alphamap_fragment>\n\t#include <alphatest_fragment>\n\tfloat dist = length( vWorldPosition - referencePosition );\n\tdist = ( dist - nearDistance ) / ( farDistance - nearDistance );\n\tdist = saturate( dist );\n\tgl_FragColor = packDepthToRGBA( dist );\n}";

var distanceRGBA_vert = "#define DISTANCE\nvarying vec3 vWorldPosition;\n#include <common>\n#include <uv_pars_vertex>\n#include <displacementmap_pars_vertex>\n#include <morphtarget_pars_vertex>\n#include <skinning_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvoid main() {\n\t#include <uv_vertex>\n\t#include <skinbase_vertex>\n\t#ifdef USE_DISPLACEMENTMAP\n\t\t#include <beginnormal_vertex>\n\t\t#include <morphnormal_vertex>\n\t\t#include <skinnormal_vertex>\n\t#endif\n\t#include <begin_vertex>\n\t#include <morphtarget_vertex>\n\t#include <skinning_vertex>\n\t#include <displacementmap_vertex>\n\t#include <project_vertex>\n\t#include <worldpos_vertex>\n\t#include <clipping_planes_vertex>\n\tvWorldPosition = worldPosition.xyz;\n}";

var equirect_frag = "uniform sampler2D tEquirect;\nvarying vec3 vWorldDirection;\n#include <common>\nvoid main() {\n\tvec3 direction = normalize( vWorldDirection );\n\tvec2 sampleUV = equirectUv( direction );\n\tvec4 texColor = texture2D( tEquirect, sampleUV );\n\tgl_FragColor = mapTexelToLinear( texColor );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n}";

var equirect_vert = "varying vec3 vWorldDirection;\n#include <common>\nvoid main() {\n\tvWorldDirection = transformDirection( position, modelMatrix );\n\t#include <begin_vertex>\n\t#include <project_vertex>\n}";

var linedashed_frag = "uniform vec3 diffuse;\nuniform float opacity;\nuniform float dashSize;\nuniform float totalSize;\nvarying float vLineDistance;\n#include <common>\n#include <color_pars_fragment>\n#include <fog_pars_fragment>\n#include <logdepthbuf_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvoid main() {\n\t#include <clipping_planes_fragment>\n\tif ( mod( vLineDistance, totalSize ) > dashSize ) {\n\t\tdiscard;\n\t}\n\tvec3 outgoingLight = vec3( 0.0 );\n\tvec4 diffuseColor = vec4( diffuse, opacity );\n\t#include <logdepthbuf_fragment>\n\t#include <color_fragment>\n\toutgoingLight = diffuseColor.rgb;\n\tgl_FragColor = vec4( outgoingLight, diffuseColor.a );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n\t#include <fog_fragment>\n\t#include <premultiplied_alpha_fragment>\n}";

var linedashed_vert = "uniform float scale;\nattribute float lineDistance;\nvarying float vLineDistance;\n#include <common>\n#include <color_pars_vertex>\n#include <fog_pars_vertex>\n#include <morphtarget_pars_vertex>\n#include <logdepthbuf_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvoid main() {\n\tvLineDistance = scale * lineDistance;\n\t#include <color_vertex>\n\t#include <begin_vertex>\n\t#include <morphtarget_vertex>\n\t#include <project_vertex>\n\t#include <logdepthbuf_vertex>\n\t#include <clipping_planes_vertex>\n\t#include <fog_vertex>\n}";

var meshbasic_frag = "uniform vec3 diffuse;\nuniform float opacity;\n#ifndef FLAT_SHADED\n\tvarying vec3 vNormal;\n#endif\n#include <common>\n#include <dithering_pars_fragment>\n#include <color_pars_fragment>\n#include <uv_pars_fragment>\n#include <uv2_pars_fragment>\n#include <map_pars_fragment>\n#include <alphamap_pars_fragment>\n#include <aomap_pars_fragment>\n#include <lightmap_pars_fragment>\n#include <envmap_common_pars_fragment>\n#include <envmap_pars_fragment>\n#include <cube_uv_reflection_fragment>\n#include <fog_pars_fragment>\n#include <specularmap_pars_fragment>\n#include <logdepthbuf_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvoid main() {\n\t#include <clipping_planes_fragment>\n\tvec4 diffuseColor = vec4( diffuse, opacity );\n\t#include <logdepthbuf_fragment>\n\t#include <map_fragment>\n\t#include <color_fragment>\n\t#include <alphamap_fragment>\n\t#include <alphatest_fragment>\n\t#include <specularmap_fragment>\n\tReflectedLight reflectedLight = ReflectedLight( vec3( 0.0 ), vec3( 0.0 ), vec3( 0.0 ), vec3( 0.0 ) );\n\t#ifdef USE_LIGHTMAP\n\t\n\t\tvec4 lightMapTexel= texture2D( lightMap, vUv2 );\n\t\treflectedLight.indirectDiffuse += lightMapTexelToLinear( lightMapTexel ).rgb * lightMapIntensity;\n\t#else\n\t\treflectedLight.indirectDiffuse += vec3( 1.0 );\n\t#endif\n\t#include <aomap_fragment>\n\treflectedLight.indirectDiffuse *= diffuseColor.rgb;\n\tvec3 outgoingLight = reflectedLight.indirectDiffuse;\n\t#include <envmap_fragment>\n\tgl_FragColor = vec4( outgoingLight, diffuseColor.a );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n\t#include <fog_fragment>\n\t#include <premultiplied_alpha_fragment>\n\t#include <dithering_fragment>\n}";

var meshbasic_vert = "#include <common>\n#include <uv_pars_vertex>\n#include <uv2_pars_vertex>\n#include <envmap_pars_vertex>\n#include <color_pars_vertex>\n#include <fog_pars_vertex>\n#include <morphtarget_pars_vertex>\n#include <skinning_pars_vertex>\n#include <logdepthbuf_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvoid main() {\n\t#include <uv_vertex>\n\t#include <uv2_vertex>\n\t#include <color_vertex>\n\t#include <skinbase_vertex>\n\t#ifdef USE_ENVMAP\n\t#include <beginnormal_vertex>\n\t#include <morphnormal_vertex>\n\t#include <skinnormal_vertex>\n\t#include <defaultnormal_vertex>\n\t#endif\n\t#include <begin_vertex>\n\t#include <morphtarget_vertex>\n\t#include <skinning_vertex>\n\t#include <project_vertex>\n\t#include <logdepthbuf_vertex>\n\t#include <worldpos_vertex>\n\t#include <clipping_planes_vertex>\n\t#include <envmap_vertex>\n\t#include <fog_vertex>\n}";

var meshlambert_frag = "uniform vec3 diffuse;\nuniform vec3 emissive;\nuniform float opacity;\nvarying vec3 vLightFront;\nvarying vec3 vIndirectFront;\n#ifdef DOUBLE_SIDED\n\tvarying vec3 vLightBack;\n\tvarying vec3 vIndirectBack;\n#endif\n#include <common>\n#include <packing>\n#include <dithering_pars_fragment>\n#include <color_pars_fragment>\n#include <uv_pars_fragment>\n#include <uv2_pars_fragment>\n#include <map_pars_fragment>\n#include <alphamap_pars_fragment>\n#include <aomap_pars_fragment>\n#include <lightmap_pars_fragment>\n#include <emissivemap_pars_fragment>\n#include <envmap_common_pars_fragment>\n#include <envmap_pars_fragment>\n#include <cube_uv_reflection_fragment>\n#include <bsdfs>\n#include <lights_pars_begin>\n#include <fog_pars_fragment>\n#include <shadowmap_pars_fragment>\n#include <shadowmask_pars_fragment>\n#include <specularmap_pars_fragment>\n#include <logdepthbuf_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvoid main() {\n\t#include <clipping_planes_fragment>\n\tvec4 diffuseColor = vec4( diffuse, opacity );\n\tReflectedLight reflectedLight = ReflectedLight( vec3( 0.0 ), vec3( 0.0 ), vec3( 0.0 ), vec3( 0.0 ) );\n\tvec3 totalEmissiveRadiance = emissive;\n\t#include <logdepthbuf_fragment>\n\t#include <map_fragment>\n\t#include <color_fragment>\n\t#include <alphamap_fragment>\n\t#include <alphatest_fragment>\n\t#include <specularmap_fragment>\n\t#include <emissivemap_fragment>\n\t#ifdef DOUBLE_SIDED\n\t\treflectedLight.indirectDiffuse += ( gl_FrontFacing ) ? vIndirectFront : vIndirectBack;\n\t#else\n\t\treflectedLight.indirectDiffuse += vIndirectFront;\n\t#endif\n\t#include <lightmap_fragment>\n\treflectedLight.indirectDiffuse *= BRDF_Diffuse_Lambert( diffuseColor.rgb );\n\t#ifdef DOUBLE_SIDED\n\t\treflectedLight.directDiffuse = ( gl_FrontFacing ) ? vLightFront : vLightBack;\n\t#else\n\t\treflectedLight.directDiffuse = vLightFront;\n\t#endif\n\treflectedLight.directDiffuse *= BRDF_Diffuse_Lambert( diffuseColor.rgb ) * getShadowMask();\n\t#include <aomap_fragment>\n\tvec3 outgoingLight = reflectedLight.directDiffuse + reflectedLight.indirectDiffuse + totalEmissiveRadiance;\n\t#include <envmap_fragment>\n\tgl_FragColor = vec4( outgoingLight, diffuseColor.a );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n\t#include <fog_fragment>\n\t#include <premultiplied_alpha_fragment>\n\t#include <dithering_fragment>\n}";

var meshlambert_vert = "#define LAMBERT\nvarying vec3 vLightFront;\nvarying vec3 vIndirectFront;\n#ifdef DOUBLE_SIDED\n\tvarying vec3 vLightBack;\n\tvarying vec3 vIndirectBack;\n#endif\n#include <common>\n#include <uv_pars_vertex>\n#include <uv2_pars_vertex>\n#include <envmap_pars_vertex>\n#include <bsdfs>\n#include <lights_pars_begin>\n#include <color_pars_vertex>\n#include <fog_pars_vertex>\n#include <morphtarget_pars_vertex>\n#include <skinning_pars_vertex>\n#include <shadowmap_pars_vertex>\n#include <logdepthbuf_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvoid main() {\n\t#include <uv_vertex>\n\t#include <uv2_vertex>\n\t#include <color_vertex>\n\t#include <beginnormal_vertex>\n\t#include <morphnormal_vertex>\n\t#include <skinbase_vertex>\n\t#include <skinnormal_vertex>\n\t#include <defaultnormal_vertex>\n\t#include <begin_vertex>\n\t#include <morphtarget_vertex>\n\t#include <skinning_vertex>\n\t#include <project_vertex>\n\t#include <logdepthbuf_vertex>\n\t#include <clipping_planes_vertex>\n\t#include <worldpos_vertex>\n\t#include <envmap_vertex>\n\t#include <lights_lambert_vertex>\n\t#include <shadowmap_vertex>\n\t#include <fog_vertex>\n}";

var meshmatcap_frag = "#define MATCAP\nuniform vec3 diffuse;\nuniform float opacity;\nuniform sampler2D matcap;\nvarying vec3 vViewPosition;\n#ifndef FLAT_SHADED\n\tvarying vec3 vNormal;\n#endif\n#include <common>\n#include <dithering_pars_fragment>\n#include <color_pars_fragment>\n#include <uv_pars_fragment>\n#include <map_pars_fragment>\n#include <alphamap_pars_fragment>\n#include <fog_pars_fragment>\n#include <bumpmap_pars_fragment>\n#include <normalmap_pars_fragment>\n#include <logdepthbuf_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvoid main() {\n\t#include <clipping_planes_fragment>\n\tvec4 diffuseColor = vec4( diffuse, opacity );\n\t#include <logdepthbuf_fragment>\n\t#include <map_fragment>\n\t#include <color_fragment>\n\t#include <alphamap_fragment>\n\t#include <alphatest_fragment>\n\t#include <normal_fragment_begin>\n\t#include <normal_fragment_maps>\n\tvec3 viewDir = normalize( vViewPosition );\n\tvec3 x = normalize( vec3( viewDir.z, 0.0, - viewDir.x ) );\n\tvec3 y = cross( viewDir, x );\n\tvec2 uv = vec2( dot( x, normal ), dot( y, normal ) ) * 0.495 + 0.5;\n\t#ifdef USE_MATCAP\n\t\tvec4 matcapColor = texture2D( matcap, uv );\n\t\tmatcapColor = matcapTexelToLinear( matcapColor );\n\t#else\n\t\tvec4 matcapColor = vec4( 1.0 );\n\t#endif\n\tvec3 outgoingLight = diffuseColor.rgb * matcapColor.rgb;\n\tgl_FragColor = vec4( outgoingLight, diffuseColor.a );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n\t#include <fog_fragment>\n\t#include <premultiplied_alpha_fragment>\n\t#include <dithering_fragment>\n}";

var meshmatcap_vert = "#define MATCAP\nvarying vec3 vViewPosition;\n#ifndef FLAT_SHADED\n\tvarying vec3 vNormal;\n#endif\n#include <common>\n#include <uv_pars_vertex>\n#include <color_pars_vertex>\n#include <displacementmap_pars_vertex>\n#include <fog_pars_vertex>\n#include <morphtarget_pars_vertex>\n#include <skinning_pars_vertex>\n#include <logdepthbuf_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvoid main() {\n\t#include <uv_vertex>\n\t#include <color_vertex>\n\t#include <beginnormal_vertex>\n\t#include <morphnormal_vertex>\n\t#include <skinbase_vertex>\n\t#include <skinnormal_vertex>\n\t#include <defaultnormal_vertex>\n\t#ifndef FLAT_SHADED\n\t\tvNormal = normalize( transformedNormal );\n\t#endif\n\t#include <begin_vertex>\n\t#include <morphtarget_vertex>\n\t#include <skinning_vertex>\n\t#include <displacementmap_vertex>\n\t#include <project_vertex>\n\t#include <logdepthbuf_vertex>\n\t#include <clipping_planes_vertex>\n\t#include <fog_vertex>\n\tvViewPosition = - mvPosition.xyz;\n}";

var meshtoon_frag = "#define TOON\nuniform vec3 diffuse;\nuniform vec3 emissive;\nuniform float opacity;\n#include <common>\n#include <packing>\n#include <dithering_pars_fragment>\n#include <color_pars_fragment>\n#include <uv_pars_fragment>\n#include <uv2_pars_fragment>\n#include <map_pars_fragment>\n#include <alphamap_pars_fragment>\n#include <aomap_pars_fragment>\n#include <lightmap_pars_fragment>\n#include <emissivemap_pars_fragment>\n#include <gradientmap_pars_fragment>\n#include <fog_pars_fragment>\n#include <bsdfs>\n#include <lights_pars_begin>\n#include <lights_toon_pars_fragment>\n#include <shadowmap_pars_fragment>\n#include <bumpmap_pars_fragment>\n#include <normalmap_pars_fragment>\n#include <logdepthbuf_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvoid main() {\n\t#include <clipping_planes_fragment>\n\tvec4 diffuseColor = vec4( diffuse, opacity );\n\tReflectedLight reflectedLight = ReflectedLight( vec3( 0.0 ), vec3( 0.0 ), vec3( 0.0 ), vec3( 0.0 ) );\n\tvec3 totalEmissiveRadiance = emissive;\n\t#include <logdepthbuf_fragment>\n\t#include <map_fragment>\n\t#include <color_fragment>\n\t#include <alphamap_fragment>\n\t#include <alphatest_fragment>\n\t#include <normal_fragment_begin>\n\t#include <normal_fragment_maps>\n\t#include <emissivemap_fragment>\n\t#include <lights_toon_fragment>\n\t#include <lights_fragment_begin>\n\t#include <lights_fragment_maps>\n\t#include <lights_fragment_end>\n\t#include <aomap_fragment>\n\tvec3 outgoingLight = reflectedLight.directDiffuse + reflectedLight.indirectDiffuse + totalEmissiveRadiance;\n\tgl_FragColor = vec4( outgoingLight, diffuseColor.a );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n\t#include <fog_fragment>\n\t#include <premultiplied_alpha_fragment>\n\t#include <dithering_fragment>\n}";

var meshtoon_vert = "#define TOON\nvarying vec3 vViewPosition;\n#ifndef FLAT_SHADED\n\tvarying vec3 vNormal;\n#endif\n#include <common>\n#include <uv_pars_vertex>\n#include <uv2_pars_vertex>\n#include <displacementmap_pars_vertex>\n#include <color_pars_vertex>\n#include <fog_pars_vertex>\n#include <morphtarget_pars_vertex>\n#include <skinning_pars_vertex>\n#include <shadowmap_pars_vertex>\n#include <logdepthbuf_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvoid main() {\n\t#include <uv_vertex>\n\t#include <uv2_vertex>\n\t#include <color_vertex>\n\t#include <beginnormal_vertex>\n\t#include <morphnormal_vertex>\n\t#include <skinbase_vertex>\n\t#include <skinnormal_vertex>\n\t#include <defaultnormal_vertex>\n#ifndef FLAT_SHADED\n\tvNormal = normalize( transformedNormal );\n#endif\n\t#include <begin_vertex>\n\t#include <morphtarget_vertex>\n\t#include <skinning_vertex>\n\t#include <displacementmap_vertex>\n\t#include <project_vertex>\n\t#include <logdepthbuf_vertex>\n\t#include <clipping_planes_vertex>\n\tvViewPosition = - mvPosition.xyz;\n\t#include <worldpos_vertex>\n\t#include <shadowmap_vertex>\n\t#include <fog_vertex>\n}";

var meshphong_frag = "#define PHONG\nuniform vec3 diffuse;\nuniform vec3 emissive;\nuniform vec3 specular;\nuniform float shininess;\nuniform float opacity;\n#include <common>\n#include <packing>\n#include <dithering_pars_fragment>\n#include <color_pars_fragment>\n#include <uv_pars_fragment>\n#include <uv2_pars_fragment>\n#include <map_pars_fragment>\n#include <alphamap_pars_fragment>\n#include <aomap_pars_fragment>\n#include <lightmap_pars_fragment>\n#include <emissivemap_pars_fragment>\n#include <envmap_common_pars_fragment>\n#include <envmap_pars_fragment>\n#include <cube_uv_reflection_fragment>\n#include <fog_pars_fragment>\n#include <bsdfs>\n#include <lights_pars_begin>\n#include <lights_phong_pars_fragment>\n#include <shadowmap_pars_fragment>\n#include <bumpmap_pars_fragment>\n#include <normalmap_pars_fragment>\n#include <specularmap_pars_fragment>\n#include <logdepthbuf_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvoid main() {\n\t#include <clipping_planes_fragment>\n\tvec4 diffuseColor = vec4( diffuse, opacity );\n\tReflectedLight reflectedLight = ReflectedLight( vec3( 0.0 ), vec3( 0.0 ), vec3( 0.0 ), vec3( 0.0 ) );\n\tvec3 totalEmissiveRadiance = emissive;\n\t#include <logdepthbuf_fragment>\n\t#include <map_fragment>\n\t#include <color_fragment>\n\t#include <alphamap_fragment>\n\t#include <alphatest_fragment>\n\t#include <specularmap_fragment>\n\t#include <normal_fragment_begin>\n\t#include <normal_fragment_maps>\n\t#include <emissivemap_fragment>\n\t#include <lights_phong_fragment>\n\t#include <lights_fragment_begin>\n\t#include <lights_fragment_maps>\n\t#include <lights_fragment_end>\n\t#include <aomap_fragment>\n\tvec3 outgoingLight = reflectedLight.directDiffuse + reflectedLight.indirectDiffuse + reflectedLight.directSpecular + reflectedLight.indirectSpecular + totalEmissiveRadiance;\n\t#include <envmap_fragment>\n\tgl_FragColor = vec4( outgoingLight, diffuseColor.a );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n\t#include <fog_fragment>\n\t#include <premultiplied_alpha_fragment>\n\t#include <dithering_fragment>\n}";

var meshphong_vert = "#define PHONG\nvarying vec3 vViewPosition;\n#ifndef FLAT_SHADED\n\tvarying vec3 vNormal;\n#endif\n#include <common>\n#include <uv_pars_vertex>\n#include <uv2_pars_vertex>\n#include <displacementmap_pars_vertex>\n#include <envmap_pars_vertex>\n#include <color_pars_vertex>\n#include <fog_pars_vertex>\n#include <morphtarget_pars_vertex>\n#include <skinning_pars_vertex>\n#include <shadowmap_pars_vertex>\n#include <logdepthbuf_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvoid main() {\n\t#include <uv_vertex>\n\t#include <uv2_vertex>\n\t#include <color_vertex>\n\t#include <beginnormal_vertex>\n\t#include <morphnormal_vertex>\n\t#include <skinbase_vertex>\n\t#include <skinnormal_vertex>\n\t#include <defaultnormal_vertex>\n#ifndef FLAT_SHADED\n\tvNormal = normalize( transformedNormal );\n#endif\n\t#include <begin_vertex>\n\t#include <morphtarget_vertex>\n\t#include <skinning_vertex>\n\t#include <displacementmap_vertex>\n\t#include <project_vertex>\n\t#include <logdepthbuf_vertex>\n\t#include <clipping_planes_vertex>\n\tvViewPosition = - mvPosition.xyz;\n\t#include <worldpos_vertex>\n\t#include <envmap_vertex>\n\t#include <shadowmap_vertex>\n\t#include <fog_vertex>\n}";

var meshphysical_frag = "#define STANDARD\n#ifdef PHYSICAL\n\t#define REFLECTIVITY\n\t#define CLEARCOAT\n\t#define TRANSMISSION\n#endif\nuniform vec3 diffuse;\nuniform vec3 emissive;\nuniform float roughness;\nuniform float metalness;\nuniform float opacity;\n#ifdef TRANSMISSION\n\tuniform float transmission;\n#endif\n#ifdef REFLECTIVITY\n\tuniform float reflectivity;\n#endif\n#ifdef CLEARCOAT\n\tuniform float clearcoat;\n\tuniform float clearcoatRoughness;\n#endif\n#ifdef USE_SHEEN\n\tuniform vec3 sheen;\n#endif\nvarying vec3 vViewPosition;\n#ifndef FLAT_SHADED\n\tvarying vec3 vNormal;\n\t#ifdef USE_TANGENT\n\t\tvarying vec3 vTangent;\n\t\tvarying vec3 vBitangent;\n\t#endif\n#endif\n#include <common>\n#include <packing>\n#include <dithering_pars_fragment>\n#include <color_pars_fragment>\n#include <uv_pars_fragment>\n#include <uv2_pars_fragment>\n#include <map_pars_fragment>\n#include <alphamap_pars_fragment>\n#include <aomap_pars_fragment>\n#include <lightmap_pars_fragment>\n#include <emissivemap_pars_fragment>\n#include <transmissionmap_pars_fragment>\n#include <bsdfs>\n#include <cube_uv_reflection_fragment>\n#include <envmap_common_pars_fragment>\n#include <envmap_physical_pars_fragment>\n#include <fog_pars_fragment>\n#include <lights_pars_begin>\n#include <lights_physical_pars_fragment>\n#include <shadowmap_pars_fragment>\n#include <bumpmap_pars_fragment>\n#include <normalmap_pars_fragment>\n#include <clearcoat_pars_fragment>\n#include <roughnessmap_pars_fragment>\n#include <metalnessmap_pars_fragment>\n#include <logdepthbuf_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvoid main() {\n\t#include <clipping_planes_fragment>\n\tvec4 diffuseColor = vec4( diffuse, opacity );\n\tReflectedLight reflectedLight = ReflectedLight( vec3( 0.0 ), vec3( 0.0 ), vec3( 0.0 ), vec3( 0.0 ) );\n\tvec3 totalEmissiveRadiance = emissive;\n\t#ifdef TRANSMISSION\n\t\tfloat totalTransmission = transmission;\n\t#endif\n\t#include <logdepthbuf_fragment>\n\t#include <map_fragment>\n\t#include <color_fragment>\n\t#include <alphamap_fragment>\n\t#include <alphatest_fragment>\n\t#include <roughnessmap_fragment>\n\t#include <metalnessmap_fragment>\n\t#include <normal_fragment_begin>\n\t#include <normal_fragment_maps>\n\t#include <clearcoat_normal_fragment_begin>\n\t#include <clearcoat_normal_fragment_maps>\n\t#include <emissivemap_fragment>\n\t#include <transmissionmap_fragment>\n\t#include <lights_physical_fragment>\n\t#include <lights_fragment_begin>\n\t#include <lights_fragment_maps>\n\t#include <lights_fragment_end>\n\t#include <aomap_fragment>\n\tvec3 outgoingLight = reflectedLight.directDiffuse + reflectedLight.indirectDiffuse + reflectedLight.directSpecular + reflectedLight.indirectSpecular + totalEmissiveRadiance;\n\t#ifdef TRANSMISSION\n\t\tdiffuseColor.a *= mix( saturate( 1. - totalTransmission + linearToRelativeLuminance( reflectedLight.directSpecular + reflectedLight.indirectSpecular ) ), 1.0, metalness );\n\t#endif\n\tgl_FragColor = vec4( outgoingLight, diffuseColor.a );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n\t#include <fog_fragment>\n\t#include <premultiplied_alpha_fragment>\n\t#include <dithering_fragment>\n}";

var meshphysical_vert = "#define STANDARD\nvarying vec3 vViewPosition;\n#ifndef FLAT_SHADED\n\tvarying vec3 vNormal;\n\t#ifdef USE_TANGENT\n\t\tvarying vec3 vTangent;\n\t\tvarying vec3 vBitangent;\n\t#endif\n#endif\n#include <common>\n#include <uv_pars_vertex>\n#include <uv2_pars_vertex>\n#include <displacementmap_pars_vertex>\n#include <color_pars_vertex>\n#include <fog_pars_vertex>\n#include <morphtarget_pars_vertex>\n#include <skinning_pars_vertex>\n#include <shadowmap_pars_vertex>\n#include <logdepthbuf_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvoid main() {\n\t#include <uv_vertex>\n\t#include <uv2_vertex>\n\t#include <color_vertex>\n\t#include <beginnormal_vertex>\n\t#include <morphnormal_vertex>\n\t#include <skinbase_vertex>\n\t#include <skinnormal_vertex>\n\t#include <defaultnormal_vertex>\n#ifndef FLAT_SHADED\n\tvNormal = normalize( transformedNormal );\n\t#ifdef USE_TANGENT\n\t\tvTangent = normalize( transformedTangent );\n\t\tvBitangent = normalize( cross( vNormal, vTangent ) * tangent.w );\n\t#endif\n#endif\n\t#include <begin_vertex>\n\t#include <morphtarget_vertex>\n\t#include <skinning_vertex>\n\t#include <displacementmap_vertex>\n\t#include <project_vertex>\n\t#include <logdepthbuf_vertex>\n\t#include <clipping_planes_vertex>\n\tvViewPosition = - mvPosition.xyz;\n\t#include <worldpos_vertex>\n\t#include <shadowmap_vertex>\n\t#include <fog_vertex>\n}";

var normal_frag = "#define NORMAL\nuniform float opacity;\n#if defined( FLAT_SHADED ) || defined( USE_BUMPMAP ) || defined( TANGENTSPACE_NORMALMAP )\n\tvarying vec3 vViewPosition;\n#endif\n#ifndef FLAT_SHADED\n\tvarying vec3 vNormal;\n\t#ifdef USE_TANGENT\n\t\tvarying vec3 vTangent;\n\t\tvarying vec3 vBitangent;\n\t#endif\n#endif\n#include <packing>\n#include <uv_pars_fragment>\n#include <bumpmap_pars_fragment>\n#include <normalmap_pars_fragment>\n#include <logdepthbuf_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvoid main() {\n\t#include <clipping_planes_fragment>\n\t#include <logdepthbuf_fragment>\n\t#include <normal_fragment_begin>\n\t#include <normal_fragment_maps>\n\tgl_FragColor = vec4( packNormalToRGB( normal ), opacity );\n}";

var normal_vert = "#define NORMAL\n#if defined( FLAT_SHADED ) || defined( USE_BUMPMAP ) || defined( TANGENTSPACE_NORMALMAP )\n\tvarying vec3 vViewPosition;\n#endif\n#ifndef FLAT_SHADED\n\tvarying vec3 vNormal;\n\t#ifdef USE_TANGENT\n\t\tvarying vec3 vTangent;\n\t\tvarying vec3 vBitangent;\n\t#endif\n#endif\n#include <common>\n#include <uv_pars_vertex>\n#include <displacementmap_pars_vertex>\n#include <morphtarget_pars_vertex>\n#include <skinning_pars_vertex>\n#include <logdepthbuf_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvoid main() {\n\t#include <uv_vertex>\n\t#include <beginnormal_vertex>\n\t#include <morphnormal_vertex>\n\t#include <skinbase_vertex>\n\t#include <skinnormal_vertex>\n\t#include <defaultnormal_vertex>\n#ifndef FLAT_SHADED\n\tvNormal = normalize( transformedNormal );\n\t#ifdef USE_TANGENT\n\t\tvTangent = normalize( transformedTangent );\n\t\tvBitangent = normalize( cross( vNormal, vTangent ) * tangent.w );\n\t#endif\n#endif\n\t#include <begin_vertex>\n\t#include <morphtarget_vertex>\n\t#include <skinning_vertex>\n\t#include <displacementmap_vertex>\n\t#include <project_vertex>\n\t#include <logdepthbuf_vertex>\n\t#include <clipping_planes_vertex>\n#if defined( FLAT_SHADED ) || defined( USE_BUMPMAP ) || defined( TANGENTSPACE_NORMALMAP )\n\tvViewPosition = - mvPosition.xyz;\n#endif\n}";

var points_frag = "uniform vec3 diffuse;\nuniform float opacity;\n#include <common>\n#include <color_pars_fragment>\n#include <map_particle_pars_fragment>\n#include <fog_pars_fragment>\n#include <logdepthbuf_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvoid main() {\n\t#include <clipping_planes_fragment>\n\tvec3 outgoingLight = vec3( 0.0 );\n\tvec4 diffuseColor = vec4( diffuse, opacity );\n\t#include <logdepthbuf_fragment>\n\t#include <map_particle_fragment>\n\t#include <color_fragment>\n\t#include <alphatest_fragment>\n\toutgoingLight = diffuseColor.rgb;\n\tgl_FragColor = vec4( outgoingLight, diffuseColor.a );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n\t#include <fog_fragment>\n\t#include <premultiplied_alpha_fragment>\n}";

var points_vert = "uniform float size;\nuniform float scale;\n#include <common>\n#include <color_pars_vertex>\n#include <fog_pars_vertex>\n#include <morphtarget_pars_vertex>\n#include <logdepthbuf_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvoid main() {\n\t#include <color_vertex>\n\t#include <begin_vertex>\n\t#include <morphtarget_vertex>\n\t#include <project_vertex>\n\tgl_PointSize = size;\n\t#ifdef USE_SIZEATTENUATION\n\t\tbool isPerspective = isPerspectiveMatrix( projectionMatrix );\n\t\tif ( isPerspective ) gl_PointSize *= ( scale / - mvPosition.z );\n\t#endif\n\t#include <logdepthbuf_vertex>\n\t#include <clipping_planes_vertex>\n\t#include <worldpos_vertex>\n\t#include <fog_vertex>\n}";

var shadow_frag = "uniform vec3 color;\nuniform float opacity;\n#include <common>\n#include <packing>\n#include <fog_pars_fragment>\n#include <bsdfs>\n#include <lights_pars_begin>\n#include <shadowmap_pars_fragment>\n#include <shadowmask_pars_fragment>\nvoid main() {\n\tgl_FragColor = vec4( color, opacity * ( 1.0 - getShadowMask() ) );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n\t#include <fog_fragment>\n}";

var shadow_vert = "#include <common>\n#include <fog_pars_vertex>\n#include <shadowmap_pars_vertex>\nvoid main() {\n\t#include <begin_vertex>\n\t#include <project_vertex>\n\t#include <worldpos_vertex>\n\t#include <beginnormal_vertex>\n\t#include <morphnormal_vertex>\n\t#include <skinbase_vertex>\n\t#include <skinnormal_vertex>\n\t#include <defaultnormal_vertex>\n\t#include <shadowmap_vertex>\n\t#include <fog_vertex>\n}";

var sprite_frag = "uniform vec3 diffuse;\nuniform float opacity;\n#include <common>\n#include <uv_pars_fragment>\n#include <map_pars_fragment>\n#include <alphamap_pars_fragment>\n#include <fog_pars_fragment>\n#include <logdepthbuf_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvoid main() {\n\t#include <clipping_planes_fragment>\n\tvec3 outgoingLight = vec3( 0.0 );\n\tvec4 diffuseColor = vec4( diffuse, opacity );\n\t#include <logdepthbuf_fragment>\n\t#include <map_fragment>\n\t#include <alphamap_fragment>\n\t#include <alphatest_fragment>\n\toutgoingLight = diffuseColor.rgb;\n\tgl_FragColor = vec4( outgoingLight, diffuseColor.a );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n\t#include <fog_fragment>\n}";

var sprite_vert = "uniform float rotation;\nuniform vec2 center;\n#include <common>\n#include <uv_pars_vertex>\n#include <fog_pars_vertex>\n#include <logdepthbuf_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvoid main() {\n\t#include <uv_vertex>\n\tvec4 mvPosition = modelViewMatrix * vec4( 0.0, 0.0, 0.0, 1.0 );\n\tvec2 scale;\n\tscale.x = length( vec3( modelMatrix[ 0 ].x, modelMatrix[ 0 ].y, modelMatrix[ 0 ].z ) );\n\tscale.y = length( vec3( modelMatrix[ 1 ].x, modelMatrix[ 1 ].y, modelMatrix[ 1 ].z ) );\n\t#ifndef USE_SIZEATTENUATION\n\t\tbool isPerspective = isPerspectiveMatrix( projectionMatrix );\n\t\tif ( isPerspective ) scale *= - mvPosition.z;\n\t#endif\n\tvec2 alignedPosition = ( position.xy - ( center - vec2( 0.5 ) ) ) * scale;\n\tvec2 rotatedPosition;\n\trotatedPosition.x = cos( rotation ) * alignedPosition.x - sin( rotation ) * alignedPosition.y;\n\trotatedPosition.y = sin( rotation ) * alignedPosition.x + cos( rotation ) * alignedPosition.y;\n\tmvPosition.xy += rotatedPosition;\n\tgl_Position = projectionMatrix * mvPosition;\n\t#include <logdepthbuf_vertex>\n\t#include <clipping_planes_vertex>\n\t#include <fog_vertex>\n}";

const ShaderChunk = {
	alphamap_fragment: alphamap_fragment,
	alphamap_pars_fragment: alphamap_pars_fragment,
	alphatest_fragment: alphatest_fragment,
	aomap_fragment: aomap_fragment,
	aomap_pars_fragment: aomap_pars_fragment,
	begin_vertex: begin_vertex,
	beginnormal_vertex: beginnormal_vertex,
	bsdfs: bsdfs,
	bumpmap_pars_fragment: bumpmap_pars_fragment,
	clipping_planes_fragment: clipping_planes_fragment,
	clipping_planes_pars_fragment: clipping_planes_pars_fragment,
	clipping_planes_pars_vertex: clipping_planes_pars_vertex,
	clipping_planes_vertex: clipping_planes_vertex,
	color_fragment: color_fragment,
	color_pars_fragment: color_pars_fragment,
	color_pars_vertex: color_pars_vertex,
	color_vertex: color_vertex,
	common: common,
	cube_uv_reflection_fragment: cube_uv_reflection_fragment,
	defaultnormal_vertex: defaultnormal_vertex,
	displacementmap_pars_vertex: displacementmap_pars_vertex,
	displacementmap_vertex: displacementmap_vertex,
	emissivemap_fragment: emissivemap_fragment,
	emissivemap_pars_fragment: emissivemap_pars_fragment,
	encodings_fragment: encodings_fragment,
	encodings_pars_fragment: encodings_pars_fragment,
	envmap_fragment: envmap_fragment,
	envmap_common_pars_fragment: envmap_common_pars_fragment,
	envmap_pars_fragment: envmap_pars_fragment,
	envmap_pars_vertex: envmap_pars_vertex,
	envmap_physical_pars_fragment: envmap_physical_pars_fragment,
	envmap_vertex: envmap_vertex,
	fog_vertex: fog_vertex,
	fog_pars_vertex: fog_pars_vertex,
	fog_fragment: fog_fragment,
	fog_pars_fragment: fog_pars_fragment,
	gradientmap_pars_fragment: gradientmap_pars_fragment,
	lightmap_fragment: lightmap_fragment,
	lightmap_pars_fragment: lightmap_pars_fragment,
	lights_lambert_vertex: lights_lambert_vertex,
	lights_pars_begin: lights_pars_begin,
	lights_toon_fragment: lights_toon_fragment,
	lights_toon_pars_fragment: lights_toon_pars_fragment,
	lights_phong_fragment: lights_phong_fragment,
	lights_phong_pars_fragment: lights_phong_pars_fragment,
	lights_physical_fragment: lights_physical_fragment,
	lights_physical_pars_fragment: lights_physical_pars_fragment,
	lights_fragment_begin: lights_fragment_begin,
	lights_fragment_maps: lights_fragment_maps,
	lights_fragment_end: lights_fragment_end,
	logdepthbuf_fragment: logdepthbuf_fragment,
	logdepthbuf_pars_fragment: logdepthbuf_pars_fragment,
	logdepthbuf_pars_vertex: logdepthbuf_pars_vertex,
	logdepthbuf_vertex: logdepthbuf_vertex,
	map_fragment: map_fragment,
	map_pars_fragment: map_pars_fragment,
	map_particle_fragment: map_particle_fragment,
	map_particle_pars_fragment: map_particle_pars_fragment,
	metalnessmap_fragment: metalnessmap_fragment,
	metalnessmap_pars_fragment: metalnessmap_pars_fragment,
	morphnormal_vertex: morphnormal_vertex,
	morphtarget_pars_vertex: morphtarget_pars_vertex,
	morphtarget_vertex: morphtarget_vertex,
	normal_fragment_begin: normal_fragment_begin,
	normal_fragment_maps: normal_fragment_maps,
	normalmap_pars_fragment: normalmap_pars_fragment,
	clearcoat_normal_fragment_begin: clearcoat_normal_fragment_begin,
	clearcoat_normal_fragment_maps: clearcoat_normal_fragment_maps,
	clearcoat_pars_fragment: clearcoat_pars_fragment,
	packing: packing,
	premultiplied_alpha_fragment: premultiplied_alpha_fragment,
	project_vertex: project_vertex,
	dithering_fragment: dithering_fragment,
	dithering_pars_fragment: dithering_pars_fragment,
	roughnessmap_fragment: roughnessmap_fragment,
	roughnessmap_pars_fragment: roughnessmap_pars_fragment,
	shadowmap_pars_fragment: shadowmap_pars_fragment,
	shadowmap_pars_vertex: shadowmap_pars_vertex,
	shadowmap_vertex: shadowmap_vertex,
	shadowmask_pars_fragment: shadowmask_pars_fragment,
	skinbase_vertex: skinbase_vertex,
	skinning_pars_vertex: skinning_pars_vertex,
	skinning_vertex: skinning_vertex,
	skinnormal_vertex: skinnormal_vertex,
	specularmap_fragment: specularmap_fragment,
	specularmap_pars_fragment: specularmap_pars_fragment,
	tonemapping_fragment: tonemapping_fragment,
	tonemapping_pars_fragment: tonemapping_pars_fragment,
	transmissionmap_fragment: transmissionmap_fragment,
	transmissionmap_pars_fragment: transmissionmap_pars_fragment,
	uv_pars_fragment: uv_pars_fragment,
	uv_pars_vertex: uv_pars_vertex,
	uv_vertex: uv_vertex,
	uv2_pars_fragment: uv2_pars_fragment,
	uv2_pars_vertex: uv2_pars_vertex,
	uv2_vertex: uv2_vertex,
	worldpos_vertex: worldpos_vertex,

	background_frag: background_frag,
	background_vert: background_vert,
	cube_frag: cube_frag,
	cube_vert: cube_vert,
	depth_frag: depth_frag,
	depth_vert: depth_vert,
	distanceRGBA_frag: distanceRGBA_frag,
	distanceRGBA_vert: distanceRGBA_vert,
	equirect_frag: equirect_frag,
	equirect_vert: equirect_vert,
	linedashed_frag: linedashed_frag,
	linedashed_vert: linedashed_vert,
	meshbasic_frag: meshbasic_frag,
	meshbasic_vert: meshbasic_vert,
	meshlambert_frag: meshlambert_frag,
	meshlambert_vert: meshlambert_vert,
	meshmatcap_frag: meshmatcap_frag,
	meshmatcap_vert: meshmatcap_vert,
	meshtoon_frag: meshtoon_frag,
	meshtoon_vert: meshtoon_vert,
	meshphong_frag: meshphong_frag,
	meshphong_vert: meshphong_vert,
	meshphysical_frag: meshphysical_frag,
	meshphysical_vert: meshphysical_vert,
	normal_frag: normal_frag,
	normal_vert: normal_vert,
	points_frag: points_frag,
	points_vert: points_vert,
	shadow_frag: shadow_frag,
	shadow_vert: shadow_vert,
	sprite_frag: sprite_frag,
	sprite_vert: sprite_vert
};

/**
 * Uniforms library for shared webgl shaders
 */

const UniformsLib = {

	common: {

		diffuse: { value: new Color( 0xeeeeee ) },
		opacity: { value: 1.0 },

		map: { value: null },
		uvTransform: { value: new Matrix3() },
		uv2Transform: { value: new Matrix3() },

		alphaMap: { value: null },

	},

	specularmap: {

		specularMap: { value: null },

	},

	envmap: {

		envMap: { value: null },
		flipEnvMap: { value: - 1 },
		reflectivity: { value: 1.0 },
		refractionRatio: { value: 0.98 },
		maxMipLevel: { value: 0 }

	},

	aomap: {

		aoMap: { value: null },
		aoMapIntensity: { value: 1 }

	},

	lightmap: {

		lightMap: { value: null },
		lightMapIntensity: { value: 1 }

	},

	emissivemap: {

		emissiveMap: { value: null }

	},

	bumpmap: {

		bumpMap: { value: null },
		bumpScale: { value: 1 }

	},

	normalmap: {

		normalMap: { value: null },
		normalScale: { value: new Vector2( 1, 1 ) }

	},

	displacementmap: {

		displacementMap: { value: null },
		displacementScale: { value: 1 },
		displacementBias: { value: 0 }

	},

	roughnessmap: {

		roughnessMap: { value: null }

	},

	metalnessmap: {

		metalnessMap: { value: null }

	},

	gradientmap: {

		gradientMap: { value: null }

	},

	fog: {

		fogDensity: { value: 0.00025 },
		fogNear: { value: 1 },
		fogFar: { value: 2000 },
		fogColor: { value: new Color( 0xffffff ) }

	},

	lights: {

		ambientLightColor: { value: [] },

		lightProbe: { value: [] },

		directionalLights: { value: [], properties: {
			direction: {},
			color: {}
		} },

		directionalLightShadows: { value: [], properties: {
			shadowBias: {},
			shadowNormalBias: {},
			shadowRadius: {},
			shadowMapSize: {}
		} },

		directionalShadowMap: { value: [] },
		directionalShadowMatrix: { value: [] },

		spotLights: { value: [], properties: {
			color: {},
			position: {},
			direction: {},
			distance: {},
			coneCos: {},
			penumbraCos: {},
			decay: {}
		} },

		spotLightShadows: { value: [], properties: {
			shadowBias: {},
			shadowNormalBias: {},
			shadowRadius: {},
			shadowMapSize: {}
		} },

		spotShadowMap: { value: [] },
		spotShadowMatrix: { value: [] },

		pointLights: { value: [], properties: {
			color: {},
			position: {},
			decay: {},
			distance: {}
		} },

		pointLightShadows: { value: [], properties: {
			shadowBias: {},
			shadowNormalBias: {},
			shadowRadius: {},
			shadowMapSize: {},
			shadowCameraNear: {},
			shadowCameraFar: {}
		} },

		pointShadowMap: { value: [] },
		pointShadowMatrix: { value: [] },

		hemisphereLights: { value: [], properties: {
			direction: {},
			skyColor: {},
			groundColor: {}
		} },

		// TODO (abelnation): RectAreaLight BRDF data needs to be moved from example to main src
		rectAreaLights: { value: [], properties: {
			color: {},
			position: {},
			width: {},
			height: {}
		} },

		ltc_1: { value: null },
		ltc_2: { value: null }

	},

	points: {

		diffuse: { value: new Color( 0xeeeeee ) },
		opacity: { value: 1.0 },
		size: { value: 1.0 },
		scale: { value: 1.0 },
		map: { value: null },
		alphaMap: { value: null },
		uvTransform: { value: new Matrix3() }

	},

	sprite: {

		diffuse: { value: new Color( 0xeeeeee ) },
		opacity: { value: 1.0 },
		center: { value: new Vector2( 0.5, 0.5 ) },
		rotation: { value: 0.0 },
		map: { value: null },
		alphaMap: { value: null },
		uvTransform: { value: new Matrix3() }

	}

};

const ShaderLib = {

	basic: {

		uniforms: mergeUniforms( [
			UniformsLib.common,
			UniformsLib.specularmap,
			UniformsLib.envmap,
			UniformsLib.aomap,
			UniformsLib.lightmap,
			UniformsLib.fog
		] ),

		vertexShader: ShaderChunk.meshbasic_vert,
		fragmentShader: ShaderChunk.meshbasic_frag

	},

	lambert: {

		uniforms: mergeUniforms( [
			UniformsLib.common,
			UniformsLib.specularmap,
			UniformsLib.envmap,
			UniformsLib.aomap,
			UniformsLib.lightmap,
			UniformsLib.emissivemap,
			UniformsLib.fog,
			UniformsLib.lights,
			{
				emissive: { value: new Color( 0x000000 ) }
			}
		] ),

		vertexShader: ShaderChunk.meshlambert_vert,
		fragmentShader: ShaderChunk.meshlambert_frag

	},

	phong: {

		uniforms: mergeUniforms( [
			UniformsLib.common,
			UniformsLib.specularmap,
			UniformsLib.envmap,
			UniformsLib.aomap,
			UniformsLib.lightmap,
			UniformsLib.emissivemap,
			UniformsLib.bumpmap,
			UniformsLib.normalmap,
			UniformsLib.displacementmap,
			UniformsLib.fog,
			UniformsLib.lights,
			{
				emissive: { value: new Color( 0x000000 ) },
				specular: { value: new Color( 0x111111 ) },
				shininess: { value: 30 }
			}
		] ),

		vertexShader: ShaderChunk.meshphong_vert,
		fragmentShader: ShaderChunk.meshphong_frag

	},

	standard: {

		uniforms: mergeUniforms( [
			UniformsLib.common,
			UniformsLib.envmap,
			UniformsLib.aomap,
			UniformsLib.lightmap,
			UniformsLib.emissivemap,
			UniformsLib.bumpmap,
			UniformsLib.normalmap,
			UniformsLib.displacementmap,
			UniformsLib.roughnessmap,
			UniformsLib.metalnessmap,
			UniformsLib.fog,
			UniformsLib.lights,
			{
				emissive: { value: new Color( 0x000000 ) },
				roughness: { value: 1.0 },
				metalness: { value: 0.0 },
				envMapIntensity: { value: 1 } // temporary
			}
		] ),

		vertexShader: ShaderChunk.meshphysical_vert,
		fragmentShader: ShaderChunk.meshphysical_frag

	},

	toon: {

		uniforms: mergeUniforms( [
			UniformsLib.common,
			UniformsLib.aomap,
			UniformsLib.lightmap,
			UniformsLib.emissivemap,
			UniformsLib.bumpmap,
			UniformsLib.normalmap,
			UniformsLib.displacementmap,
			UniformsLib.gradientmap,
			UniformsLib.fog,
			UniformsLib.lights,
			{
				emissive: { value: new Color( 0x000000 ) }
			}
		] ),

		vertexShader: ShaderChunk.meshtoon_vert,
		fragmentShader: ShaderChunk.meshtoon_frag

	},

	matcap: {

		uniforms: mergeUniforms( [
			UniformsLib.common,
			UniformsLib.bumpmap,
			UniformsLib.normalmap,
			UniformsLib.displacementmap,
			UniformsLib.fog,
			{
				matcap: { value: null }
			}
		] ),

		vertexShader: ShaderChunk.meshmatcap_vert,
		fragmentShader: ShaderChunk.meshmatcap_frag

	},

	points: {

		uniforms: mergeUniforms( [
			UniformsLib.points,
			UniformsLib.fog
		] ),

		vertexShader: ShaderChunk.points_vert,
		fragmentShader: ShaderChunk.points_frag

	},

	dashed: {

		uniforms: mergeUniforms( [
			UniformsLib.common,
			UniformsLib.fog,
			{
				scale: { value: 1 },
				dashSize: { value: 1 },
				totalSize: { value: 2 }
			}
		] ),

		vertexShader: ShaderChunk.linedashed_vert,
		fragmentShader: ShaderChunk.linedashed_frag

	},

	depth: {

		uniforms: mergeUniforms( [
			UniformsLib.common,
			UniformsLib.displacementmap
		] ),

		vertexShader: ShaderChunk.depth_vert,
		fragmentShader: ShaderChunk.depth_frag

	},

	normal: {

		uniforms: mergeUniforms( [
			UniformsLib.common,
			UniformsLib.bumpmap,
			UniformsLib.normalmap,
			UniformsLib.displacementmap,
			{
				opacity: { value: 1.0 }
			}
		] ),

		vertexShader: ShaderChunk.normal_vert,
		fragmentShader: ShaderChunk.normal_frag

	},

	sprite: {

		uniforms: mergeUniforms( [
			UniformsLib.sprite,
			UniformsLib.fog
		] ),

		vertexShader: ShaderChunk.sprite_vert,
		fragmentShader: ShaderChunk.sprite_frag

	},

	background: {

		uniforms: {
			uvTransform: { value: new Matrix3() },
			t2D: { value: null },
		},

		vertexShader: ShaderChunk.background_vert,
		fragmentShader: ShaderChunk.background_frag

	},
	/* -------------------------------------------------------------------------
	//	Cube map shader
	 ------------------------------------------------------------------------- */

	cube: {

		uniforms: mergeUniforms( [
			UniformsLib.envmap,
			{
				opacity: { value: 1.0 }
			}
		] ),

		vertexShader: ShaderChunk.cube_vert,
		fragmentShader: ShaderChunk.cube_frag

	},

	equirect: {

		uniforms: {
			tEquirect: { value: null },
		},

		vertexShader: ShaderChunk.equirect_vert,
		fragmentShader: ShaderChunk.equirect_frag

	},

	distanceRGBA: {

		uniforms: mergeUniforms( [
			UniformsLib.common,
			UniformsLib.displacementmap,
			{
				referencePosition: { value: new Vector3() },
				nearDistance: { value: 1 },
				farDistance: { value: 1000 }
			}
		] ),

		vertexShader: ShaderChunk.distanceRGBA_vert,
		fragmentShader: ShaderChunk.distanceRGBA_frag

	},

	shadow: {

		uniforms: mergeUniforms( [
			UniformsLib.lights,
			UniformsLib.fog,
			{
				color: { value: new Color( 0x00000 ) },
				opacity: { value: 1.0 }
			},
		] ),

		vertexShader: ShaderChunk.shadow_vert,
		fragmentShader: ShaderChunk.shadow_frag

	}

};

ShaderLib.physical = {

	uniforms: mergeUniforms( [
		ShaderLib.standard.uniforms,
		{
			clearcoat: { value: 0 },
			clearcoatMap: { value: null },
			clearcoatRoughness: { value: 0 },
			clearcoatRoughnessMap: { value: null },
			clearcoatNormalScale: { value: new Vector2( 1, 1 ) },
			clearcoatNormalMap: { value: null },
			sheen: { value: new Color( 0x000000 ) },
			transmission: { value: 0 },
			transmissionMap: { value: null },
		}
	] ),

	vertexShader: ShaderChunk.meshphysical_vert,
	fragmentShader: ShaderChunk.meshphysical_frag

};

function WebGLBackground( renderer, cubemaps, state, objects, premultipliedAlpha ) {

	const clearColor = new Color( 0x000000 );
	let clearAlpha = 0;

	let planeMesh;
	let boxMesh;

	let currentBackground = null;
	let currentBackgroundVersion = 0;
	let currentTonemapping = null;

	function render( renderList, scene, camera, forceClear ) {

		let background = scene.isScene === true ? scene.background : null;

		if ( background && background.isTexture ) {

			background = cubemaps.get( background );

		}

		// Ignore background in AR
		// TODO: Reconsider this.

		const xr = renderer.xr;
		const session = xr.getSession && xr.getSession();

		if ( session && session.environmentBlendMode === 'additive' ) {

			background = null;

		}

		if ( background === null ) {

			setClear( clearColor, clearAlpha );

		} else if ( background && background.isColor ) {

			setClear( background, 1 );
			forceClear = true;

		}

		if ( renderer.autoClear || forceClear ) {

			renderer.clear( renderer.autoClearColor, renderer.autoClearDepth, renderer.autoClearStencil );

		}

		if ( background && ( background.isCubeTexture || background.mapping === CubeUVReflectionMapping ) ) {

			if ( boxMesh === undefined ) {

				boxMesh = new Mesh(
					new BoxGeometry( 1, 1, 1 ),
					new ShaderMaterial( {
						name: 'BackgroundCubeMaterial',
						uniforms: cloneUniforms( ShaderLib.cube.uniforms ),
						vertexShader: ShaderLib.cube.vertexShader,
						fragmentShader: ShaderLib.cube.fragmentShader,
						side: BackSide,
						depthTest: false,
						depthWrite: false,
						fog: false
					} )
				);

				boxMesh.geometry.deleteAttribute( 'normal' );
				boxMesh.geometry.deleteAttribute( 'uv' );

				boxMesh.onBeforeRender = function ( renderer, scene, camera ) {

					this.matrixWorld.copyPosition( camera.matrixWorld );

				};

				// enable code injection for non-built-in material
				Object.defineProperty( boxMesh.material, 'envMap', {

					get: function () {

						return this.uniforms.envMap.value;

					}

				} );

				objects.update( boxMesh );

			}

			boxMesh.material.uniforms.envMap.value = background;
			boxMesh.material.uniforms.flipEnvMap.value = ( background.isCubeTexture && background._needsFlipEnvMap ) ? - 1 : 1;

			if ( currentBackground !== background ||
				currentBackgroundVersion !== background.version ||
				currentTonemapping !== renderer.toneMapping ) {

				boxMesh.material.needsUpdate = true;

				currentBackground = background;
				currentBackgroundVersion = background.version;
				currentTonemapping = renderer.toneMapping;

			}

			// push to the pre-sorted opaque render list
			renderList.unshift( boxMesh, boxMesh.geometry, boxMesh.material, 0, 0, null );

		} else if ( background && background.isTexture ) {

			if ( planeMesh === undefined ) {

				planeMesh = new Mesh(
					new PlaneGeometry( 2, 2 ),
					new ShaderMaterial( {
						name: 'BackgroundMaterial',
						uniforms: cloneUniforms( ShaderLib.background.uniforms ),
						vertexShader: ShaderLib.background.vertexShader,
						fragmentShader: ShaderLib.background.fragmentShader,
						side: FrontSide,
						depthTest: false,
						depthWrite: false,
						fog: false
					} )
				);

				planeMesh.geometry.deleteAttribute( 'normal' );

				// enable code injection for non-built-in material
				Object.defineProperty( planeMesh.material, 'map', {

					get: function () {

						return this.uniforms.t2D.value;

					}

				} );

				objects.update( planeMesh );

			}

			planeMesh.material.uniforms.t2D.value = background;

			if ( background.matrixAutoUpdate === true ) {

				background.updateMatrix();

			}

			planeMesh.material.uniforms.uvTransform.value.copy( background.matrix );

			if ( currentBackground !== background ||
				currentBackgroundVersion !== background.version ||
				currentTonemapping !== renderer.toneMapping ) {

				planeMesh.material.needsUpdate = true;

				currentBackground = background;
				currentBackgroundVersion = background.version;
				currentTonemapping = renderer.toneMapping;

			}


			// push to the pre-sorted opaque render list
			renderList.unshift( planeMesh, planeMesh.geometry, planeMesh.material, 0, 0, null );

		}

	}

	function setClear( color, alpha ) {

		state.buffers.color.setClear( color.r, color.g, color.b, alpha, premultipliedAlpha );

	}

	return {

		getClearColor: function () {

			return clearColor;

		},
		setClearColor: function ( color, alpha = 1 ) {

			clearColor.set( color );
			clearAlpha = alpha;
			setClear( clearColor, clearAlpha );

		},
		getClearAlpha: function () {

			return clearAlpha;

		},
		setClearAlpha: function ( alpha ) {

			clearAlpha = alpha;
			setClear( clearColor, clearAlpha );

		},
		render: render

	};

}

function WebGLBindingStates( gl, extensions, attributes, capabilities ) {

	const maxVertexAttributes = gl.getParameter( 34921 );

	const extension = capabilities.isWebGL2 ? null : extensions.get( 'OES_vertex_array_object' );
	const vaoAvailable = capabilities.isWebGL2 || extension !== null;

	const bindingStates = {};

	const defaultState = createBindingState( null );
	let currentState = defaultState;

	function setup( object, material, program, geometry, index ) {

		let updateBuffers = false;

		if ( vaoAvailable ) {

			const state = getBindingState( geometry, program, material );

			if ( currentState !== state ) {

				currentState = state;
				bindVertexArrayObject( currentState.object );

			}

			updateBuffers = needsUpdate( geometry, index );

			if ( updateBuffers ) saveCache( geometry, index );

		} else {

			const wireframe = ( material.wireframe === true );

			if ( currentState.geometry !== geometry.id ||
				currentState.program !== program.id ||
				currentState.wireframe !== wireframe ) {

				currentState.geometry = geometry.id;
				currentState.program = program.id;
				currentState.wireframe = wireframe;

				updateBuffers = true;

			}

		}

		if ( object.isInstancedMesh === true ) {

			updateBuffers = true;

		}

		if ( index !== null ) {

			attributes.update( index, 34963 );

		}

		if ( updateBuffers ) {

			setupVertexAttributes( object, material, program, geometry );

			if ( index !== null ) {

				gl.bindBuffer( 34963, attributes.get( index ).buffer );

			}

		}

	}

	function createVertexArrayObject() {

		if ( capabilities.isWebGL2 ) return gl.createVertexArray();

		return extension.createVertexArrayOES();

	}

	function bindVertexArrayObject( vao ) {

		if ( capabilities.isWebGL2 ) return gl.bindVertexArray( vao );

		return extension.bindVertexArrayOES( vao );

	}

	function deleteVertexArrayObject( vao ) {

		if ( capabilities.isWebGL2 ) return gl.deleteVertexArray( vao );

		return extension.deleteVertexArrayOES( vao );

	}

	function getBindingState( geometry, program, material ) {

		const wireframe = ( material.wireframe === true );

		let programMap = bindingStates[ geometry.id ];

		if ( programMap === undefined ) {

			programMap = {};
			bindingStates[ geometry.id ] = programMap;

		}

		let stateMap = programMap[ program.id ];

		if ( stateMap === undefined ) {

			stateMap = {};
			programMap[ program.id ] = stateMap;

		}

		let state = stateMap[ wireframe ];

		if ( state === undefined ) {

			state = createBindingState( createVertexArrayObject() );
			stateMap[ wireframe ] = state;

		}

		return state;

	}

	function createBindingState( vao ) {

		const newAttributes = [];
		const enabledAttributes = [];
		const attributeDivisors = [];

		for ( let i = 0; i < maxVertexAttributes; i ++ ) {

			newAttributes[ i ] = 0;
			enabledAttributes[ i ] = 0;
			attributeDivisors[ i ] = 0;

		}

		return {

			// for backward compatibility on non-VAO support browser
			geometry: null,
			program: null,
			wireframe: false,

			newAttributes: newAttributes,
			enabledAttributes: enabledAttributes,
			attributeDivisors: attributeDivisors,
			object: vao,
			attributes: {},
			index: null

		};

	}

	function needsUpdate( geometry, index ) {

		const cachedAttributes = currentState.attributes;
		const geometryAttributes = geometry.attributes;

		let attributesNum = 0;

		for ( const key in geometryAttributes ) {

			const cachedAttribute = cachedAttributes[ key ];
			const geometryAttribute = geometryAttributes[ key ];

			if ( cachedAttribute === undefined ) return true;

			if ( cachedAttribute.attribute !== geometryAttribute ) return true;

			if ( cachedAttribute.data !== geometryAttribute.data ) return true;

			attributesNum ++;

		}

		if ( currentState.attributesNum !== attributesNum ) return true;

		if ( currentState.index !== index ) return true;

		return false;

	}

	function saveCache( geometry, index ) {

		const cache = {};
		const attributes = geometry.attributes;
		let attributesNum = 0;

		for ( const key in attributes ) {

			const attribute = attributes[ key ];

			const data = {};
			data.attribute = attribute;

			if ( attribute.data ) {

				data.data = attribute.data;

			}

			cache[ key ] = data;

			attributesNum ++;

		}

		currentState.attributes = cache;
		currentState.attributesNum = attributesNum;

		currentState.index = index;

	}

	function initAttributes() {

		const newAttributes = currentState.newAttributes;

		for ( let i = 0, il = newAttributes.length; i < il; i ++ ) {

			newAttributes[ i ] = 0;

		}

	}

	function enableAttribute( attribute ) {

		enableAttributeAndDivisor( attribute, 0 );

	}

	function enableAttributeAndDivisor( attribute, meshPerAttribute ) {

		const newAttributes = currentState.newAttributes;
		const enabledAttributes = currentState.enabledAttributes;
		const attributeDivisors = currentState.attributeDivisors;

		newAttributes[ attribute ] = 1;

		if ( enabledAttributes[ attribute ] === 0 ) {

			gl.enableVertexAttribArray( attribute );
			enabledAttributes[ attribute ] = 1;

		}

		if ( attributeDivisors[ attribute ] !== meshPerAttribute ) {

			const extension = capabilities.isWebGL2 ? gl : extensions.get( 'ANGLE_instanced_arrays' );

			extension[ capabilities.isWebGL2 ? 'vertexAttribDivisor' : 'vertexAttribDivisorANGLE' ]( attribute, meshPerAttribute );
			attributeDivisors[ attribute ] = meshPerAttribute;

		}

	}

	function disableUnusedAttributes() {

		const newAttributes = currentState.newAttributes;
		const enabledAttributes = currentState.enabledAttributes;

		for ( let i = 0, il = enabledAttributes.length; i < il; i ++ ) {

			if ( enabledAttributes[ i ] !== newAttributes[ i ] ) {

				gl.disableVertexAttribArray( i );
				enabledAttributes[ i ] = 0;

			}

		}

	}

	function vertexAttribPointer( index, size, type, normalized, stride, offset ) {

		if ( capabilities.isWebGL2 === true && ( type === 5124 || type === 5125 ) ) {

			gl.vertexAttribIPointer( index, size, type, stride, offset );

		} else {

			gl.vertexAttribPointer( index, size, type, normalized, stride, offset );

		}

	}

	function setupVertexAttributes( object, material, program, geometry ) {

		if ( capabilities.isWebGL2 === false && ( object.isInstancedMesh || geometry.isInstancedBufferGeometry ) ) {

			if ( extensions.get( 'ANGLE_instanced_arrays' ) === null ) return;

		}

		initAttributes();

		const geometryAttributes = geometry.attributes;

		const programAttributes = program.getAttributes();

		const materialDefaultAttributeValues = material.defaultAttributeValues;

		for ( const name in programAttributes ) {

			const programAttribute = programAttributes[ name ];

			if ( programAttribute >= 0 ) {

				const geometryAttribute = geometryAttributes[ name ];

				if ( geometryAttribute !== undefined ) {

					const normalized = geometryAttribute.normalized;
					const size = geometryAttribute.itemSize;

					const attribute = attributes.get( geometryAttribute );

					// TODO Attribute may not be available on context restore

					if ( attribute === undefined ) continue;

					const buffer = attribute.buffer;
					const type = attribute.type;
					const bytesPerElement = attribute.bytesPerElement;

					if ( geometryAttribute.isInterleavedBufferAttribute ) {

						const data = geometryAttribute.data;
						const stride = data.stride;
						const offset = geometryAttribute.offset;

						if ( data && data.isInstancedInterleavedBuffer ) {

							enableAttributeAndDivisor( programAttribute, data.meshPerAttribute );

							if ( geometry._maxInstanceCount === undefined ) {

								geometry._maxInstanceCount = data.meshPerAttribute * data.count;

							}

						} else {

							enableAttribute( programAttribute );

						}

						gl.bindBuffer( 34962, buffer );
						vertexAttribPointer( programAttribute, size, type, normalized, stride * bytesPerElement, offset * bytesPerElement );

					} else {

						if ( geometryAttribute.isInstancedBufferAttribute ) {

							enableAttributeAndDivisor( programAttribute, geometryAttribute.meshPerAttribute );

							if ( geometry._maxInstanceCount === undefined ) {

								geometry._maxInstanceCount = geometryAttribute.meshPerAttribute * geometryAttribute.count;

							}

						} else {

							enableAttribute( programAttribute );

						}

						gl.bindBuffer( 34962, buffer );
						vertexAttribPointer( programAttribute, size, type, normalized, 0, 0 );

					}

				} else if ( name === 'instanceMatrix' ) {

					const attribute = attributes.get( object.instanceMatrix );

					// TODO Attribute may not be available on context restore

					if ( attribute === undefined ) continue;

					const buffer = attribute.buffer;
					const type = attribute.type;

					enableAttributeAndDivisor( programAttribute + 0, 1 );
					enableAttributeAndDivisor( programAttribute + 1, 1 );
					enableAttributeAndDivisor( programAttribute + 2, 1 );
					enableAttributeAndDivisor( programAttribute + 3, 1 );

					gl.bindBuffer( 34962, buffer );

					gl.vertexAttribPointer( programAttribute + 0, 4, type, false, 64, 0 );
					gl.vertexAttribPointer( programAttribute + 1, 4, type, false, 64, 16 );
					gl.vertexAttribPointer( programAttribute + 2, 4, type, false, 64, 32 );
					gl.vertexAttribPointer( programAttribute + 3, 4, type, false, 64, 48 );

				} else if ( name === 'instanceColor' ) {

					const attribute = attributes.get( object.instanceColor );

					// TODO Attribute may not be available on context restore

					if ( attribute === undefined ) continue;

					const buffer = attribute.buffer;
					const type = attribute.type;

					enableAttributeAndDivisor( programAttribute, 1 );

					gl.bindBuffer( 34962, buffer );

					gl.vertexAttribPointer( programAttribute, 3, type, false, 12, 0 );

				} else if ( materialDefaultAttributeValues !== undefined ) {

					const value = materialDefaultAttributeValues[ name ];

					if ( value !== undefined ) {

						switch ( value.length ) {

							case 2:
								gl.vertexAttrib2fv( programAttribute, value );
								break;

							case 3:
								gl.vertexAttrib3fv( programAttribute, value );
								break;

							case 4:
								gl.vertexAttrib4fv( programAttribute, value );
								break;

							default:
								gl.vertexAttrib1fv( programAttribute, value );

						}

					}

				}

			}

		}

		disableUnusedAttributes();

	}

	function dispose() {

		reset();

		for ( const geometryId in bindingStates ) {

			const programMap = bindingStates[ geometryId ];

			for ( const programId in programMap ) {

				const stateMap = programMap[ programId ];

				for ( const wireframe in stateMap ) {

					deleteVertexArrayObject( stateMap[ wireframe ].object );

					delete stateMap[ wireframe ];

				}

				delete programMap[ programId ];

			}

			delete bindingStates[ geometryId ];

		}

	}

	function releaseStatesOfGeometry( geometry ) {

		if ( bindingStates[ geometry.id ] === undefined ) return;

		const programMap = bindingStates[ geometry.id ];

		for ( const programId in programMap ) {

			const stateMap = programMap[ programId ];

			for ( const wireframe in stateMap ) {

				deleteVertexArrayObject( stateMap[ wireframe ].object );

				delete stateMap[ wireframe ];

			}

			delete programMap[ programId ];

		}

		delete bindingStates[ geometry.id ];

	}

	function releaseStatesOfProgram( program ) {

		for ( const geometryId in bindingStates ) {

			const programMap = bindingStates[ geometryId ];

			if ( programMap[ program.id ] === undefined ) continue;

			const stateMap = programMap[ program.id ];

			for ( const wireframe in stateMap ) {

				deleteVertexArrayObject( stateMap[ wireframe ].object );

				delete stateMap[ wireframe ];

			}

			delete programMap[ program.id ];

		}

	}

	function reset() {

		resetDefaultState();

		if ( currentState === defaultState ) return;

		currentState = defaultState;
		bindVertexArrayObject( currentState.object );

	}

	// for backward-compatilibity

	function resetDefaultState() {

		defaultState.geometry = null;
		defaultState.program = null;
		defaultState.wireframe = false;

	}

	return {

		setup: setup,
		reset: reset,
		resetDefaultState: resetDefaultState,
		dispose: dispose,
		releaseStatesOfGeometry: releaseStatesOfGeometry,
		releaseStatesOfProgram: releaseStatesOfProgram,

		initAttributes: initAttributes,
		enableAttribute: enableAttribute,
		disableUnusedAttributes: disableUnusedAttributes

	};

}

function WebGLBufferRenderer( gl, extensions, info, capabilities ) {

	const isWebGL2 = capabilities.isWebGL2;

	let mode;

	function setMode( value ) {

		mode = value;

	}

	function render( start, count ) {

		gl.drawArrays( mode, start, count );

		info.update( count, mode, 1 );

	}

	function renderInstances( start, count, primcount ) {

		if ( primcount === 0 ) return;

		let extension, methodName;

		if ( isWebGL2 ) {

			extension = gl;
			methodName = 'drawArraysInstanced';

		} else {

			extension = extensions.get( 'ANGLE_instanced_arrays' );
			methodName = 'drawArraysInstancedANGLE';

			if ( extension === null ) {

				console.error( 'THREE.WebGLBufferRenderer: using THREE.InstancedBufferGeometry but hardware does not support extension ANGLE_instanced_arrays.' );
				return;

			}

		}

		extension[ methodName ]( mode, start, count, primcount );

		info.update( count, mode, primcount );

	}

	//

	this.setMode = setMode;
	this.render = render;
	this.renderInstances = renderInstances;

}

function WebGLCapabilities( gl, extensions, parameters ) {

	let maxAnisotropy;

	function getMaxAnisotropy() {

		if ( maxAnisotropy !== undefined ) return maxAnisotropy;

		if ( extensions.has( 'EXT_texture_filter_anisotropic' ) === true ) {

			const extension = extensions.get( 'EXT_texture_filter_anisotropic' );

			maxAnisotropy = gl.getParameter( extension.MAX_TEXTURE_MAX_ANISOTROPY_EXT );

		} else {

			maxAnisotropy = 0;

		}

		return maxAnisotropy;

	}

	function getMaxPrecision( precision ) {

		if ( precision === 'highp' ) {

			if ( gl.getShaderPrecisionFormat( 35633, 36338 ).precision > 0 &&
				gl.getShaderPrecisionFormat( 35632, 36338 ).precision > 0 ) {

				return 'highp';

			}

			precision = 'mediump';

		}

		if ( precision === 'mediump' ) {

			if ( gl.getShaderPrecisionFormat( 35633, 36337 ).precision > 0 &&
				gl.getShaderPrecisionFormat( 35632, 36337 ).precision > 0 ) {

				return 'mediump';

			}

		}

		return 'lowp';

	}

	/* eslint-disable no-undef */
	const isWebGL2 = ( typeof WebGL2RenderingContext !== 'undefined' && gl instanceof WebGL2RenderingContext ) ||
		( typeof WebGL2ComputeRenderingContext !== 'undefined' && gl instanceof WebGL2ComputeRenderingContext );
	/* eslint-enable no-undef */

	let precision = parameters.precision !== undefined ? parameters.precision : 'highp';
	const maxPrecision = getMaxPrecision( precision );

	if ( maxPrecision !== precision ) {

		console.warn( 'THREE.WebGLRenderer:', precision, 'not supported, using', maxPrecision, 'instead.' );
		precision = maxPrecision;

	}

	const logarithmicDepthBuffer = parameters.logarithmicDepthBuffer === true;

	const maxTextures = gl.getParameter( 34930 );
	const maxVertexTextures = gl.getParameter( 35660 );
	const maxTextureSize = gl.getParameter( 3379 );
	const maxCubemapSize = gl.getParameter( 34076 );

	const maxAttributes = gl.getParameter( 34921 );
	const maxVertexUniforms = gl.getParameter( 36347 );
	const maxVaryings = gl.getParameter( 36348 );
	const maxFragmentUniforms = gl.getParameter( 36349 );

	const vertexTextures = maxVertexTextures > 0;
	const floatFragmentTextures = isWebGL2 || extensions.has( 'OES_texture_float' );
	const floatVertexTextures = vertexTextures && floatFragmentTextures;

	const maxSamples = isWebGL2 ? gl.getParameter( 36183 ) : 0;

	return {

		isWebGL2: isWebGL2,

		getMaxAnisotropy: getMaxAnisotropy,
		getMaxPrecision: getMaxPrecision,

		precision: precision,
		logarithmicDepthBuffer: logarithmicDepthBuffer,

		maxTextures: maxTextures,
		maxVertexTextures: maxVertexTextures,
		maxTextureSize: maxTextureSize,
		maxCubemapSize: maxCubemapSize,

		maxAttributes: maxAttributes,
		maxVertexUniforms: maxVertexUniforms,
		maxVaryings: maxVaryings,
		maxFragmentUniforms: maxFragmentUniforms,

		vertexTextures: vertexTextures,
		floatFragmentTextures: floatFragmentTextures,
		floatVertexTextures: floatVertexTextures,

		maxSamples: maxSamples

	};

}

function WebGLClipping( properties ) {

	const scope = this;

	let globalState = null,
		numGlobalPlanes = 0,
		localClippingEnabled = false,
		renderingShadows = false;

	const plane = new Plane(),
		viewNormalMatrix = new Matrix3(),

		uniform = { value: null, needsUpdate: false };

	this.uniform = uniform;
	this.numPlanes = 0;
	this.numIntersection = 0;

	this.init = function ( planes, enableLocalClipping, camera ) {

		const enabled =
			planes.length !== 0 ||
			enableLocalClipping ||
			// enable state of previous frame - the clipping code has to
			// run another frame in order to reset the state:
			numGlobalPlanes !== 0 ||
			localClippingEnabled;

		localClippingEnabled = enableLocalClipping;

		globalState = projectPlanes( planes, camera, 0 );
		numGlobalPlanes = planes.length;

		return enabled;

	};

	this.beginShadows = function () {

		renderingShadows = true;
		projectPlanes( null );

	};

	this.endShadows = function () {

		renderingShadows = false;
		resetGlobalState();

	};

	this.setState = function ( material, camera, useCache ) {

		const planes = material.clippingPlanes,
			clipIntersection = material.clipIntersection,
			clipShadows = material.clipShadows;

		const materialProperties = properties.get( material );

		if ( ! localClippingEnabled || planes === null || planes.length === 0 || renderingShadows && ! clipShadows ) {

			// there's no local clipping

			if ( renderingShadows ) {

				// there's no global clipping

				projectPlanes( null );

			} else {

				resetGlobalState();

			}

		} else {

			const nGlobal = renderingShadows ? 0 : numGlobalPlanes,
				lGlobal = nGlobal * 4;

			let dstArray = materialProperties.clippingState || null;

			uniform.value = dstArray; // ensure unique state

			dstArray = projectPlanes( planes, camera, lGlobal, useCache );

			for ( let i = 0; i !== lGlobal; ++ i ) {

				dstArray[ i ] = globalState[ i ];

			}

			materialProperties.clippingState = dstArray;
			this.numIntersection = clipIntersection ? this.numPlanes : 0;
			this.numPlanes += nGlobal;

		}


	};

	function resetGlobalState() {

		if ( uniform.value !== globalState ) {

			uniform.value = globalState;
			uniform.needsUpdate = numGlobalPlanes > 0;

		}

		scope.numPlanes = numGlobalPlanes;
		scope.numIntersection = 0;

	}

	function projectPlanes( planes, camera, dstOffset, skipTransform ) {

		const nPlanes = planes !== null ? planes.length : 0;
		let dstArray = null;

		if ( nPlanes !== 0 ) {

			dstArray = uniform.value;

			if ( skipTransform !== true || dstArray === null ) {

				const flatSize = dstOffset + nPlanes * 4,
					viewMatrix = camera.matrixWorldInverse;

				viewNormalMatrix.getNormalMatrix( viewMatrix );

				if ( dstArray === null || dstArray.length < flatSize ) {

					dstArray = new Float32Array( flatSize );

				}

				for ( let i = 0, i4 = dstOffset; i !== nPlanes; ++ i, i4 += 4 ) {

					plane.copy( planes[ i ] ).applyMatrix4( viewMatrix, viewNormalMatrix );

					plane.normal.toArray( dstArray, i4 );
					dstArray[ i4 + 3 ] = plane.constant;

				}

			}

			uniform.value = dstArray;
			uniform.needsUpdate = true;

		}

		scope.numPlanes = nPlanes;
		scope.numIntersection = 0;

		return dstArray;

	}

}

function WebGLCubeMaps( renderer ) {

	let cubemaps = new WeakMap();

	function mapTextureMapping( texture, mapping ) {

		if ( mapping === EquirectangularReflectionMapping ) {

			texture.mapping = CubeReflectionMapping;

		} else if ( mapping === EquirectangularRefractionMapping ) {

			texture.mapping = CubeRefractionMapping;

		}

		return texture;

	}

	function get( texture ) {

		if ( texture && texture.isTexture ) {

			const mapping = texture.mapping;

			if ( mapping === EquirectangularReflectionMapping || mapping === EquirectangularRefractionMapping ) {

				if ( cubemaps.has( texture ) ) {

					const cubemap = cubemaps.get( texture ).texture;
					return mapTextureMapping( cubemap, texture.mapping );

				} else {

					const image = texture.image;

					if ( image && image.height > 0 ) {

						const currentRenderTarget = renderer.getRenderTarget();

						const renderTarget = new WebGLCubeRenderTarget( image.height / 2 );
						renderTarget.fromEquirectangularTexture( renderer, texture );
						cubemaps.set( texture, renderTarget );

						renderer.setRenderTarget( currentRenderTarget );

						texture.addEventListener( 'dispose', onTextureDispose );

						return mapTextureMapping( renderTarget.texture, texture.mapping );

					} else {

						// image not yet ready. try the conversion next frame

						return null;

					}

				}

			}

		}

		return texture;

	}

	function onTextureDispose( event ) {

		const texture = event.target;

		texture.removeEventListener( 'dispose', onTextureDispose );

		const cubemap = cubemaps.get( texture );

		if ( cubemap !== undefined ) {

			cubemaps.delete( texture );
			cubemap.dispose();

		}

	}

	function dispose() {

		cubemaps = new WeakMap();

	}

	return {
		get: get,
		dispose: dispose
	};

}

function WebGLExtensions( gl ) {

	const extensions = {};

	function getExtension( name ) {

		if ( extensions[ name ] !== undefined ) {

			return extensions[ name ];

		}

		let extension;

		switch ( name ) {

			case 'WEBGL_depth_texture':
				extension = gl.getExtension( 'WEBGL_depth_texture' ) || gl.getExtension( 'MOZ_WEBGL_depth_texture' ) || gl.getExtension( 'WEBKIT_WEBGL_depth_texture' );
				break;

			case 'EXT_texture_filter_anisotropic':
				extension = gl.getExtension( 'EXT_texture_filter_anisotropic' ) || gl.getExtension( 'MOZ_EXT_texture_filter_anisotropic' ) || gl.getExtension( 'WEBKIT_EXT_texture_filter_anisotropic' );
				break;

			case 'WEBGL_compressed_texture_s3tc':
				extension = gl.getExtension( 'WEBGL_compressed_texture_s3tc' ) || gl.getExtension( 'MOZ_WEBGL_compressed_texture_s3tc' ) || gl.getExtension( 'WEBKIT_WEBGL_compressed_texture_s3tc' );
				break;

			case 'WEBGL_compressed_texture_pvrtc':
				extension = gl.getExtension( 'WEBGL_compressed_texture_pvrtc' ) || gl.getExtension( 'WEBKIT_WEBGL_compressed_texture_pvrtc' );
				break;

			default:
				extension = gl.getExtension( name );

		}

		extensions[ name ] = extension;

		return extension;

	}

	return {

		has: function ( name ) {

			return getExtension( name ) !== null;

		},

		init: function ( capabilities ) {

			if ( capabilities.isWebGL2 ) {

				getExtension( 'EXT_color_buffer_float' );

			} else {

				getExtension( 'WEBGL_depth_texture' );
				getExtension( 'OES_texture_float' );
				getExtension( 'OES_texture_half_float' );
				getExtension( 'OES_texture_half_float_linear' );
				getExtension( 'OES_standard_derivatives' );
				getExtension( 'OES_element_index_uint' );
				getExtension( 'OES_vertex_array_object' );
				getExtension( 'ANGLE_instanced_arrays' );

			}

			getExtension( 'OES_texture_float_linear' );
			getExtension( 'EXT_color_buffer_half_float' );

		},

		get: function ( name ) {

			const extension = getExtension( name );

			if ( extension === null ) {

				console.warn( 'THREE.WebGLRenderer: ' + name + ' extension not supported.' );

			}

			return extension;

		}

	};

}

function WebGLGeometries( gl, attributes, info, bindingStates ) {

	const geometries = {};
	const wireframeAttributes = new WeakMap();

	function onGeometryDispose( event ) {

		const geometry = event.target;

		if ( geometry.index !== null ) {

			attributes.remove( geometry.index );

		}

		for ( const name in geometry.attributes ) {

			attributes.remove( geometry.attributes[ name ] );

		}

		geometry.removeEventListener( 'dispose', onGeometryDispose );

		delete geometries[ geometry.id ];

		const attribute = wireframeAttributes.get( geometry );

		if ( attribute ) {

			attributes.remove( attribute );
			wireframeAttributes.delete( geometry );

		}

		bindingStates.releaseStatesOfGeometry( geometry );

		if ( geometry.isInstancedBufferGeometry === true ) {

			delete geometry._maxInstanceCount;

		}

		//

		info.memory.geometries --;

	}

	function get( object, geometry ) {

		if ( geometries[ geometry.id ] === true ) return geometry;

		geometry.addEventListener( 'dispose', onGeometryDispose );

		geometries[ geometry.id ] = true;

		info.memory.geometries ++;

		return geometry;

	}

	function update( geometry ) {

		const geometryAttributes = geometry.attributes;

		// Updating index buffer in VAO now. See WebGLBindingStates.

		for ( const name in geometryAttributes ) {

			attributes.update( geometryAttributes[ name ], 34962 );

		}

		// morph targets

		const morphAttributes = geometry.morphAttributes;

		for ( const name in morphAttributes ) {

			const array = morphAttributes[ name ];

			for ( let i = 0, l = array.length; i < l; i ++ ) {

				attributes.update( array[ i ], 34962 );

			}

		}

	}

	function updateWireframeAttribute( geometry ) {

		const indices = [];

		const geometryIndex = geometry.index;
		const geometryPosition = geometry.attributes.position;
		let version = 0;

		if ( geometryIndex !== null ) {

			const array = geometryIndex.array;
			version = geometryIndex.version;

			for ( let i = 0, l = array.length; i < l; i += 3 ) {

				const a = array[ i + 0 ];
				const b = array[ i + 1 ];
				const c = array[ i + 2 ];

				indices.push( a, b, b, c, c, a );

			}

		} else {

			const array = geometryPosition.array;
			version = geometryPosition.version;

			for ( let i = 0, l = ( array.length / 3 ) - 1; i < l; i += 3 ) {

				const a = i + 0;
				const b = i + 1;
				const c = i + 2;

				indices.push( a, b, b, c, c, a );

			}

		}

		const attribute = new ( arrayMax( indices ) > 65535 ? Uint32BufferAttribute : Uint16BufferAttribute )( indices, 1 );
		attribute.version = version;

		// Updating index buffer in VAO now. See WebGLBindingStates

		//

		const previousAttribute = wireframeAttributes.get( geometry );

		if ( previousAttribute ) attributes.remove( previousAttribute );

		//

		wireframeAttributes.set( geometry, attribute );

	}

	function getWireframeAttribute( geometry ) {

		const currentAttribute = wireframeAttributes.get( geometry );

		if ( currentAttribute ) {

			const geometryIndex = geometry.index;

			if ( geometryIndex !== null ) {

				// if the attribute is obsolete, create a new one

				if ( currentAttribute.version < geometryIndex.version ) {

					updateWireframeAttribute( geometry );

				}

			}

		} else {

			updateWireframeAttribute( geometry );

		}

		return wireframeAttributes.get( geometry );

	}

	return {

		get: get,
		update: update,

		getWireframeAttribute: getWireframeAttribute

	};

}

function WebGLIndexedBufferRenderer( gl, extensions, info, capabilities ) {

	const isWebGL2 = capabilities.isWebGL2;

	let mode;

	function setMode( value ) {

		mode = value;

	}

	let type, bytesPerElement;

	function setIndex( value ) {

		type = value.type;
		bytesPerElement = value.bytesPerElement;

	}

	function render( start, count ) {

		gl.drawElements( mode, count, type, start * bytesPerElement );

		info.update( count, mode, 1 );

	}

	function renderInstances( start, count, primcount ) {

		if ( primcount === 0 ) return;

		let extension, methodName;

		if ( isWebGL2 ) {

			extension = gl;
			methodName = 'drawElementsInstanced';

		} else {

			extension = extensions.get( 'ANGLE_instanced_arrays' );
			methodName = 'drawElementsInstancedANGLE';

			if ( extension === null ) {

				console.error( 'THREE.WebGLIndexedBufferRenderer: using THREE.InstancedBufferGeometry but hardware does not support extension ANGLE_instanced_arrays.' );
				return;

			}

		}

		extension[ methodName ]( mode, count, type, start * bytesPerElement, primcount );

		info.update( count, mode, primcount );

	}

	//

	this.setMode = setMode;
	this.setIndex = setIndex;
	this.render = render;
	this.renderInstances = renderInstances;

}

function WebGLInfo( gl ) {

	const memory = {
		geometries: 0,
		textures: 0
	};

	const render = {
		frame: 0,
		calls: 0,
		triangles: 0,
		points: 0,
		lines: 0
	};

	function update( count, mode, instanceCount ) {

		render.calls ++;

		switch ( mode ) {

			case 4:
				render.triangles += instanceCount * ( count / 3 );
				break;

			case 1:
				render.lines += instanceCount * ( count / 2 );
				break;

			case 3:
				render.lines += instanceCount * ( count - 1 );
				break;

			case 2:
				render.lines += instanceCount * count;
				break;

			case 0:
				render.points += instanceCount * count;
				break;

			default:
				console.error( 'THREE.WebGLInfo: Unknown draw mode:', mode );
				break;

		}

	}

	function reset() {

		render.frame ++;
		render.calls = 0;
		render.triangles = 0;
		render.points = 0;
		render.lines = 0;

	}

	return {
		memory: memory,
		render: render,
		programs: null,
		autoReset: true,
		reset: reset,
		update: update
	};

}

function numericalSort( a, b ) {

	return a[ 0 ] - b[ 0 ];

}

function absNumericalSort( a, b ) {

	return Math.abs( b[ 1 ] ) - Math.abs( a[ 1 ] );

}

function WebGLMorphtargets( gl ) {

	const influencesList = {};
	const morphInfluences = new Float32Array( 8 );

	const workInfluences = [];

	for ( let i = 0; i < 8; i ++ ) {

		workInfluences[ i ] = [ i, 0 ];

	}

	function update( object, geometry, material, program ) {

		const objectInfluences = object.morphTargetInfluences;

		// When object doesn't have morph target influences defined, we treat it as a 0-length array
		// This is important to make sure we set up morphTargetBaseInfluence / morphTargetInfluences

		const length = objectInfluences === undefined ? 0 : objectInfluences.length;

		let influences = influencesList[ geometry.id ];

		if ( influences === undefined ) {

			// initialise list

			influences = [];

			for ( let i = 0; i < length; i ++ ) {

				influences[ i ] = [ i, 0 ];

			}

			influencesList[ geometry.id ] = influences;

		}

		// Collect influences

		for ( let i = 0; i < length; i ++ ) {

			const influence = influences[ i ];

			influence[ 0 ] = i;
			influence[ 1 ] = objectInfluences[ i ];

		}

		influences.sort( absNumericalSort );

		for ( let i = 0; i < 8; i ++ ) {

			if ( i < length && influences[ i ][ 1 ] ) {

				workInfluences[ i ][ 0 ] = influences[ i ][ 0 ];
				workInfluences[ i ][ 1 ] = influences[ i ][ 1 ];

			} else {

				workInfluences[ i ][ 0 ] = Number.MAX_SAFE_INTEGER;
				workInfluences[ i ][ 1 ] = 0;

			}

		}

		workInfluences.sort( numericalSort );

		const morphTargets = material.morphTargets && geometry.morphAttributes.position;
		const morphNormals = material.morphNormals && geometry.morphAttributes.normal;

		let morphInfluencesSum = 0;

		for ( let i = 0; i < 8; i ++ ) {

			const influence = workInfluences[ i ];
			const index = influence[ 0 ];
			const value = influence[ 1 ];

			if ( index !== Number.MAX_SAFE_INTEGER && value ) {

				if ( morphTargets && geometry.getAttribute( 'morphTarget' + i ) !== morphTargets[ index ] ) {

					geometry.setAttribute( 'morphTarget' + i, morphTargets[ index ] );

				}

				if ( morphNormals && geometry.getAttribute( 'morphNormal' + i ) !== morphNormals[ index ] ) {

					geometry.setAttribute( 'morphNormal' + i, morphNormals[ index ] );

				}

				morphInfluences[ i ] = value;
				morphInfluencesSum += value;

			} else {

				if ( morphTargets && geometry.hasAttribute( 'morphTarget' + i ) === true ) {

					geometry.deleteAttribute( 'morphTarget' + i );

				}

				if ( morphNormals && geometry.hasAttribute( 'morphNormal' + i ) === true ) {

					geometry.deleteAttribute( 'morphNormal' + i );

				}

				morphInfluences[ i ] = 0;

			}

		}

		// GLSL shader uses formula baseinfluence * base + sum(target * influence)
		// This allows us to switch between absolute morphs and relative morphs without changing shader code
		// When baseinfluence = 1 - sum(influence), the above is equivalent to sum((target - base) * influence)
		const morphBaseInfluence = geometry.morphTargetsRelative ? 1 : 1 - morphInfluencesSum;

		program.getUniforms().setValue( gl, 'morphTargetBaseInfluence', morphBaseInfluence );
		program.getUniforms().setValue( gl, 'morphTargetInfluences', morphInfluences );

	}

	return {

		update: update

	};

}

function WebGLObjects( gl, geometries, attributes, info ) {

	let updateMap = new WeakMap();

	function update( object ) {

		const frame = info.render.frame;

		const geometry = object.geometry;
		const buffergeometry = geometries.get( object, geometry );

		// Update once per frame

		if ( updateMap.get( buffergeometry ) !== frame ) {

			geometries.update( buffergeometry );

			updateMap.set( buffergeometry, frame );

		}

		if ( object.isInstancedMesh ) {

			if ( object.hasEventListener( 'dispose', onInstancedMeshDispose ) === false ) {

				object.addEventListener( 'dispose', onInstancedMeshDispose );

			}

			attributes.update( object.instanceMatrix, 34962 );

			if ( object.instanceColor !== null ) {

				attributes.update( object.instanceColor, 34962 );

			}

		}

		return buffergeometry;

	}

	function dispose() {

		updateMap = new WeakMap();

	}

	function onInstancedMeshDispose( event ) {

		const instancedMesh = event.target;

		instancedMesh.removeEventListener( 'dispose', onInstancedMeshDispose );

		attributes.remove( instancedMesh.instanceMatrix );

		if ( instancedMesh.instanceColor !== null ) attributes.remove( instancedMesh.instanceColor );

	}

	return {

		update: update,
		dispose: dispose

	};

}

class DataTexture2DArray extends Texture {

	constructor( data = null, width = 1, height = 1, depth = 1 ) {

		super( null );

		this.image = { data, width, height, depth };

		this.magFilter = NearestFilter;
		this.minFilter = NearestFilter;

		this.wrapR = ClampToEdgeWrapping;

		this.generateMipmaps = false;
		this.flipY = false;

		this.needsUpdate = true;

	}

}

DataTexture2DArray.prototype.isDataTexture2DArray = true;

class DataTexture3D extends Texture {

	constructor( data = null, width = 1, height = 1, depth = 1 ) {

		// We're going to add .setXXX() methods for setting properties later.
		// Users can still set in DataTexture3D directly.
		//
		//	const texture = new THREE.DataTexture3D( data, width, height, depth );
		// 	texture.anisotropy = 16;
		//
		// See #14839

		super( null );

		this.image = { data, width, height, depth };

		this.magFilter = NearestFilter;
		this.minFilter = NearestFilter;

		this.wrapR = ClampToEdgeWrapping;

		this.generateMipmaps = false;
		this.flipY = false;

		this.needsUpdate = true;

	}

}

DataTexture3D.prototype.isDataTexture3D = true;

/**
 * Uniforms of a program.
 * Those form a tree structure with a special top-level container for the root,
 * which you get by calling 'new WebGLUniforms( gl, program )'.
 *
 *
 * Properties of inner nodes including the top-level container:
 *
 * .seq - array of nested uniforms
 * .map - nested uniforms by name
 *
 *
 * Methods of all nodes except the top-level container:
 *
 * .setValue( gl, value, [textures] )
 *
 * 		uploads a uniform value(s)
 *  	the 'textures' parameter is needed for sampler uniforms
 *
 *
 * Static methods of the top-level container (textures factorizations):
 *
 * .upload( gl, seq, values, textures )
 *
 * 		sets uniforms in 'seq' to 'values[id].value'
 *
 * .seqWithValue( seq, values ) : filteredSeq
 *
 * 		filters 'seq' entries with corresponding entry in values
 *
 *
 * Methods of the top-level container (textures factorizations):
 *
 * .setValue( gl, name, value, textures )
 *
 * 		sets uniform with  name 'name' to 'value'
 *
 * .setOptional( gl, obj, prop )
 *
 * 		like .set for an optional property of the object
 *
 */

const emptyTexture = new Texture();
const emptyTexture2dArray = new DataTexture2DArray();
const emptyTexture3d = new DataTexture3D();
const emptyCubeTexture = new CubeTexture();

// --- Utilities ---

// Array Caches (provide typed arrays for temporary by size)

const arrayCacheF32 = [];
const arrayCacheI32 = [];

// Float32Array caches used for uploading Matrix uniforms

const mat4array = new Float32Array( 16 );
const mat3array = new Float32Array( 9 );
const mat2array = new Float32Array( 4 );

// Flattening for arrays of vectors and matrices

function flatten( array, nBlocks, blockSize ) {

	const firstElem = array[ 0 ];

	if ( firstElem <= 0 || firstElem > 0 ) return array;
	// unoptimized: ! isNaN( firstElem )
	// see http://jacksondunstan.com/articles/983

	const n = nBlocks * blockSize;
	let r = arrayCacheF32[ n ];

	if ( r === undefined ) {

		r = new Float32Array( n );
		arrayCacheF32[ n ] = r;

	}

	if ( nBlocks !== 0 ) {

		firstElem.toArray( r, 0 );

		for ( let i = 1, offset = 0; i !== nBlocks; ++ i ) {

			offset += blockSize;
			array[ i ].toArray( r, offset );

		}

	}

	return r;

}

function arraysEqual( a, b ) {

	if ( a.length !== b.length ) return false;

	for ( let i = 0, l = a.length; i < l; i ++ ) {

		if ( a[ i ] !== b[ i ] ) return false;

	}

	return true;

}

function copyArray( a, b ) {

	for ( let i = 0, l = b.length; i < l; i ++ ) {

		a[ i ] = b[ i ];

	}

}

// Texture unit allocation

function allocTexUnits( textures, n ) {

	let r = arrayCacheI32[ n ];

	if ( r === undefined ) {

		r = new Int32Array( n );
		arrayCacheI32[ n ] = r;

	}

	for ( let i = 0; i !== n; ++ i ) {

		r[ i ] = textures.allocateTextureUnit();

	}

	return r;

}

// --- Setters ---

// Note: Defining these methods externally, because they come in a bunch
// and this way their names minify.

// Single scalar

function setValueV1f( gl, v ) {

	const cache = this.cache;

	if ( cache[ 0 ] === v ) return;

	gl.uniform1f( this.addr, v );

	cache[ 0 ] = v;

}

// Single float vector (from flat array or THREE.VectorN)

function setValueV2f( gl, v ) {

	const cache = this.cache;

	if ( v.x !== undefined ) {

		if ( cache[ 0 ] !== v.x || cache[ 1 ] !== v.y ) {

			gl.uniform2f( this.addr, v.x, v.y );

			cache[ 0 ] = v.x;
			cache[ 1 ] = v.y;

		}

	} else {

		if ( arraysEqual( cache, v ) ) return;

		gl.uniform2fv( this.addr, v );

		copyArray( cache, v );

	}

}

function setValueV3f( gl, v ) {

	const cache = this.cache;

	if ( v.x !== undefined ) {

		if ( cache[ 0 ] !== v.x || cache[ 1 ] !== v.y || cache[ 2 ] !== v.z ) {

			gl.uniform3f( this.addr, v.x, v.y, v.z );

			cache[ 0 ] = v.x;
			cache[ 1 ] = v.y;
			cache[ 2 ] = v.z;

		}

	} else if ( v.r !== undefined ) {

		if ( cache[ 0 ] !== v.r || cache[ 1 ] !== v.g || cache[ 2 ] !== v.b ) {

			gl.uniform3f( this.addr, v.r, v.g, v.b );

			cache[ 0 ] = v.r;
			cache[ 1 ] = v.g;
			cache[ 2 ] = v.b;

		}

	} else {

		if ( arraysEqual( cache, v ) ) return;

		gl.uniform3fv( this.addr, v );

		copyArray( cache, v );

	}

}

function setValueV4f( gl, v ) {

	const cache = this.cache;

	if ( v.x !== undefined ) {

		if ( cache[ 0 ] !== v.x || cache[ 1 ] !== v.y || cache[ 2 ] !== v.z || cache[ 3 ] !== v.w ) {

			gl.uniform4f( this.addr, v.x, v.y, v.z, v.w );

			cache[ 0 ] = v.x;
			cache[ 1 ] = v.y;
			cache[ 2 ] = v.z;
			cache[ 3 ] = v.w;

		}

	} else {

		if ( arraysEqual( cache, v ) ) return;

		gl.uniform4fv( this.addr, v );

		copyArray( cache, v );

	}

}

// Single matrix (from flat array or THREE.MatrixN)

function setValueM2( gl, v ) {

	const cache = this.cache;
	const elements = v.elements;

	if ( elements === undefined ) {

		if ( arraysEqual( cache, v ) ) return;

		gl.uniformMatrix2fv( this.addr, false, v );

		copyArray( cache, v );

	} else {

		if ( arraysEqual( cache, elements ) ) return;

		mat2array.set( elements );

		gl.uniformMatrix2fv( this.addr, false, mat2array );

		copyArray( cache, elements );

	}

}

function setValueM3( gl, v ) {

	const cache = this.cache;
	const elements = v.elements;

	if ( elements === undefined ) {

		if ( arraysEqual( cache, v ) ) return;

		gl.uniformMatrix3fv( this.addr, false, v );

		copyArray( cache, v );

	} else {

		if ( arraysEqual( cache, elements ) ) return;

		mat3array.set( elements );

		gl.uniformMatrix3fv( this.addr, false, mat3array );

		copyArray( cache, elements );

	}

}

function setValueM4( gl, v ) {

	const cache = this.cache;
	const elements = v.elements;

	if ( elements === undefined ) {

		if ( arraysEqual( cache, v ) ) return;

		gl.uniformMatrix4fv( this.addr, false, v );

		copyArray( cache, v );

	} else {

		if ( arraysEqual( cache, elements ) ) return;

		mat4array.set( elements );

		gl.uniformMatrix4fv( this.addr, false, mat4array );

		copyArray( cache, elements );

	}

}

// Single integer / boolean

function setValueV1i( gl, v ) {

	const cache = this.cache;

	if ( cache[ 0 ] === v ) return;

	gl.uniform1i( this.addr, v );

	cache[ 0 ] = v;

}

// Single integer / boolean vector (from flat array)

function setValueV2i( gl, v ) {

	const cache = this.cache;

	if ( arraysEqual( cache, v ) ) return;

	gl.uniform2iv( this.addr, v );

	copyArray( cache, v );

}

function setValueV3i( gl, v ) {

	const cache = this.cache;

	if ( arraysEqual( cache, v ) ) return;

	gl.uniform3iv( this.addr, v );

	copyArray( cache, v );

}

function setValueV4i( gl, v ) {

	const cache = this.cache;

	if ( arraysEqual( cache, v ) ) return;

	gl.uniform4iv( this.addr, v );

	copyArray( cache, v );

}

// Single unsigned integer

function setValueV1ui( gl, v ) {

	const cache = this.cache;

	if ( cache[ 0 ] === v ) return;

	gl.uniform1ui( this.addr, v );

	cache[ 0 ] = v;

}

// Single unsigned integer vector (from flat array)

function setValueV2ui( gl, v ) {

	const cache = this.cache;

	if ( arraysEqual( cache, v ) ) return;

	gl.uniform2uiv( this.addr, v );

	copyArray( cache, v );

}

function setValueV3ui( gl, v ) {

	const cache = this.cache;

	if ( arraysEqual( cache, v ) ) return;

	gl.uniform3uiv( this.addr, v );

	copyArray( cache, v );

}

function setValueV4ui( gl, v ) {

	const cache = this.cache;

	if ( arraysEqual( cache, v ) ) return;

	gl.uniform4uiv( this.addr, v );

	copyArray( cache, v );

}


// Single texture (2D / Cube)

function setValueT1( gl, v, textures ) {

	const cache = this.cache;
	const unit = textures.allocateTextureUnit();

	if ( cache[ 0 ] !== unit ) {

		gl.uniform1i( this.addr, unit );
		cache[ 0 ] = unit;

	}

	textures.safeSetTexture2D( v || emptyTexture, unit );

}

function setValueT3D1( gl, v, textures ) {

	const cache = this.cache;
	const unit = textures.allocateTextureUnit();

	if ( cache[ 0 ] !== unit ) {

		gl.uniform1i( this.addr, unit );
		cache[ 0 ] = unit;

	}

	textures.setTexture3D( v || emptyTexture3d, unit );

}

function setValueT6( gl, v, textures ) {

	const cache = this.cache;
	const unit = textures.allocateTextureUnit();

	if ( cache[ 0 ] !== unit ) {

		gl.uniform1i( this.addr, unit );
		cache[ 0 ] = unit;

	}

	textures.safeSetTextureCube( v || emptyCubeTexture, unit );

}

function setValueT2DArray1( gl, v, textures ) {

	const cache = this.cache;
	const unit = textures.allocateTextureUnit();

	if ( cache[ 0 ] !== unit ) {

		gl.uniform1i( this.addr, unit );
		cache[ 0 ] = unit;

	}

	textures.setTexture2DArray( v || emptyTexture2dArray, unit );

}

// Helper to pick the right setter for the singular case

function getSingularSetter( type ) {

	switch ( type ) {

		case 0x1406: return setValueV1f; // FLOAT
		case 0x8b50: return setValueV2f; // _VEC2
		case 0x8b51: return setValueV3f; // _VEC3
		case 0x8b52: return setValueV4f; // _VEC4

		case 0x8b5a: return setValueM2; // _MAT2
		case 0x8b5b: return setValueM3; // _MAT3
		case 0x8b5c: return setValueM4; // _MAT4

		case 0x1404: case 0x8b56: return setValueV1i; // INT, BOOL
		case 0x8b53: case 0x8b57: return setValueV2i; // _VEC2
		case 0x8b54: case 0x8b58: return setValueV3i; // _VEC3
		case 0x8b55: case 0x8b59: return setValueV4i; // _VEC4

		case 0x1405: return setValueV1ui; // UINT
		case 0x8dc6: return setValueV2ui; // _VEC2
		case 0x8dc7: return setValueV3ui; // _VEC3
		case 0x8dc8: return setValueV4ui; // _VEC4

		case 0x8b5e: // SAMPLER_2D
		case 0x8d66: // SAMPLER_EXTERNAL_OES
		case 0x8dca: // INT_SAMPLER_2D
		case 0x8dd2: // UNSIGNED_INT_SAMPLER_2D
		case 0x8b62: // SAMPLER_2D_SHADOW
			return setValueT1;

		case 0x8b5f: // SAMPLER_3D
		case 0x8dcb: // INT_SAMPLER_3D
		case 0x8dd3: // UNSIGNED_INT_SAMPLER_3D
			return setValueT3D1;

		case 0x8b60: // SAMPLER_CUBE
		case 0x8dcc: // INT_SAMPLER_CUBE
		case 0x8dd4: // UNSIGNED_INT_SAMPLER_CUBE
		case 0x8dc5: // SAMPLER_CUBE_SHADOW
			return setValueT6;

		case 0x8dc1: // SAMPLER_2D_ARRAY
		case 0x8dcf: // INT_SAMPLER_2D_ARRAY
		case 0x8dd7: // UNSIGNED_INT_SAMPLER_2D_ARRAY
		case 0x8dc4: // SAMPLER_2D_ARRAY_SHADOW
			return setValueT2DArray1;

	}

}


// Array of scalars

function setValueV1fArray( gl, v ) {

	gl.uniform1fv( this.addr, v );

}

// Array of vectors (from flat array or array of THREE.VectorN)

function setValueV2fArray( gl, v ) {

	const data = flatten( v, this.size, 2 );

	gl.uniform2fv( this.addr, data );

}

function setValueV3fArray( gl, v ) {

	const data = flatten( v, this.size, 3 );

	gl.uniform3fv( this.addr, data );

}

function setValueV4fArray( gl, v ) {

	const data = flatten( v, this.size, 4 );

	gl.uniform4fv( this.addr, data );

}

// Array of matrices (from flat array or array of THREE.MatrixN)

function setValueM2Array( gl, v ) {

	const data = flatten( v, this.size, 4 );

	gl.uniformMatrix2fv( this.addr, false, data );

}

function setValueM3Array( gl, v ) {

	const data = flatten( v, this.size, 9 );

	gl.uniformMatrix3fv( this.addr, false, data );

}

function setValueM4Array( gl, v ) {

	const data = flatten( v, this.size, 16 );

	gl.uniformMatrix4fv( this.addr, false, data );

}

// Array of integer / boolean

function setValueV1iArray( gl, v ) {

	gl.uniform1iv( this.addr, v );

}

// Array of integer / boolean vectors (from flat array)

function setValueV2iArray( gl, v ) {

	gl.uniform2iv( this.addr, v );

}

function setValueV3iArray( gl, v ) {

	gl.uniform3iv( this.addr, v );

}

function setValueV4iArray( gl, v ) {

	gl.uniform4iv( this.addr, v );

}

// Array of unsigned integer

function setValueV1uiArray( gl, v ) {

	gl.uniform1uiv( this.addr, v );

}

// Array of unsigned integer vectors (from flat array)

function setValueV2uiArray( gl, v ) {

	gl.uniform2uiv( this.addr, v );

}

function setValueV3uiArray( gl, v ) {

	gl.uniform3uiv( this.addr, v );

}

function setValueV4uiArray( gl, v ) {

	gl.uniform4uiv( this.addr, v );

}


// Array of textures (2D / Cube)

function setValueT1Array( gl, v, textures ) {

	const n = v.length;

	const units = allocTexUnits( textures, n );

	gl.uniform1iv( this.addr, units );

	for ( let i = 0; i !== n; ++ i ) {

		textures.safeSetTexture2D( v[ i ] || emptyTexture, units[ i ] );

	}

}

function setValueT6Array( gl, v, textures ) {

	const n = v.length;

	const units = allocTexUnits( textures, n );

	gl.uniform1iv( this.addr, units );

	for ( let i = 0; i !== n; ++ i ) {

		textures.safeSetTextureCube( v[ i ] || emptyCubeTexture, units[ i ] );

	}

}

// Helper to pick the right setter for a pure (bottom-level) array

function getPureArraySetter( type ) {

	switch ( type ) {

		case 0x1406: return setValueV1fArray; // FLOAT
		case 0x8b50: return setValueV2fArray; // _VEC2
		case 0x8b51: return setValueV3fArray; // _VEC3
		case 0x8b52: return setValueV4fArray; // _VEC4

		case 0x8b5a: return setValueM2Array; // _MAT2
		case 0x8b5b: return setValueM3Array; // _MAT3
		case 0x8b5c: return setValueM4Array; // _MAT4

		case 0x1404: case 0x8b56: return setValueV1iArray; // INT, BOOL
		case 0x8b53: case 0x8b57: return setValueV2iArray; // _VEC2
		case 0x8b54: case 0x8b58: return setValueV3iArray; // _VEC3
		case 0x8b55: case 0x8b59: return setValueV4iArray; // _VEC4

		case 0x1405: return setValueV1uiArray; // UINT
		case 0x8dc6: return setValueV2uiArray; // _VEC2
		case 0x8dc7: return setValueV3uiArray; // _VEC3
		case 0x8dc8: return setValueV4uiArray; // _VEC4

		case 0x8b5e: // SAMPLER_2D
		case 0x8d66: // SAMPLER_EXTERNAL_OES
		case 0x8dca: // INT_SAMPLER_2D
		case 0x8dd2: // UNSIGNED_INT_SAMPLER_2D
		case 0x8b62: // SAMPLER_2D_SHADOW
			return setValueT1Array;

		case 0x8b60: // SAMPLER_CUBE
		case 0x8dcc: // INT_SAMPLER_CUBE
		case 0x8dd4: // UNSIGNED_INT_SAMPLER_CUBE
		case 0x8dc5: // SAMPLER_CUBE_SHADOW
			return setValueT6Array;

	}

}

// --- Uniform Classes ---

function SingleUniform( id, activeInfo, addr ) {

	this.id = id;
	this.addr = addr;
	this.cache = [];
	this.setValue = getSingularSetter( activeInfo.type );

	// this.path = activeInfo.name; // DEBUG

}

function PureArrayUniform( id, activeInfo, addr ) {

	this.id = id;
	this.addr = addr;
	this.cache = [];
	this.size = activeInfo.size;
	this.setValue = getPureArraySetter( activeInfo.type );

	// this.path = activeInfo.name; // DEBUG

}

PureArrayUniform.prototype.updateCache = function ( data ) {

	const cache = this.cache;

	if ( data instanceof Float32Array && cache.length !== data.length ) {

		this.cache = new Float32Array( data.length );

	}

	copyArray( cache, data );

};

function StructuredUniform( id ) {

	this.id = id;

	this.seq = [];
	this.map = {};

}

StructuredUniform.prototype.setValue = function ( gl, value, textures ) {

	const seq = this.seq;

	for ( let i = 0, n = seq.length; i !== n; ++ i ) {

		const u = seq[ i ];
		u.setValue( gl, value[ u.id ], textures );

	}

};

// --- Top-level ---

// Parser - builds up the property tree from the path strings

const RePathPart = /(\w+)(\])?(\[|\.)?/g;

// extracts
// 	- the identifier (member name or array index)
//  - followed by an optional right bracket (found when array index)
//  - followed by an optional left bracket or dot (type of subscript)
//
// Note: These portions can be read in a non-overlapping fashion and
// allow straightforward parsing of the hierarchy that WebGL encodes
// in the uniform names.

function addUniform( container, uniformObject ) {

	container.seq.push( uniformObject );
	container.map[ uniformObject.id ] = uniformObject;

}

function parseUniform( activeInfo, addr, container ) {

	const path = activeInfo.name,
		pathLength = path.length;

	// reset RegExp object, because of the early exit of a previous run
	RePathPart.lastIndex = 0;

	while ( true ) {

		const match = RePathPart.exec( path ),
			matchEnd = RePathPart.lastIndex;

		let id = match[ 1 ];
		const idIsIndex = match[ 2 ] === ']',
			subscript = match[ 3 ];

		if ( idIsIndex ) id = id | 0; // convert to integer

		if ( subscript === undefined || subscript === '[' && matchEnd + 2 === pathLength ) {

			// bare name or "pure" bottom-level array "[0]" suffix

			addUniform( container, subscript === undefined ?
				new SingleUniform( id, activeInfo, addr ) :
				new PureArrayUniform( id, activeInfo, addr ) );

			break;

		} else {

			// step into inner node / create it in case it doesn't exist

			const map = container.map;
			let next = map[ id ];

			if ( next === undefined ) {

				next = new StructuredUniform( id );
				addUniform( container, next );

			}

			container = next;

		}

	}

}

// Root Container

function WebGLUniforms( gl, program ) {

	this.seq = [];
	this.map = {};

	const n = gl.getProgramParameter( program, 35718 );

	for ( let i = 0; i < n; ++ i ) {

		const info = gl.getActiveUniform( program, i ),
			addr = gl.getUniformLocation( program, info.name );

		parseUniform( info, addr, this );

	}

}

WebGLUniforms.prototype.setValue = function ( gl, name, value, textures ) {

	const u = this.map[ name ];

	if ( u !== undefined ) u.setValue( gl, value, textures );

};

WebGLUniforms.prototype.setOptional = function ( gl, object, name ) {

	const v = object[ name ];

	if ( v !== undefined ) this.setValue( gl, name, v );

};


// Static interface

WebGLUniforms.upload = function ( gl, seq, values, textures ) {

	for ( let i = 0, n = seq.length; i !== n; ++ i ) {

		const u = seq[ i ],
			v = values[ u.id ];

		if ( v.needsUpdate !== false ) {

			// note: always updating when .needsUpdate is undefined
			u.setValue( gl, v.value, textures );

		}

	}

};

WebGLUniforms.seqWithValue = function ( seq, values ) {

	const r = [];

	for ( let i = 0, n = seq.length; i !== n; ++ i ) {

		const u = seq[ i ];
		if ( u.id in values ) r.push( u );

	}

	return r;

};

function WebGLShader( gl, type, string ) {

	const shader = gl.createShader( type );

	gl.shaderSource( shader, string );
	gl.compileShader( shader );

	return shader;

}

let programIdCount = 0;

function addLineNumbers( string ) {

	const lines = string.split( '\n' );

	for ( let i = 0; i < lines.length; i ++ ) {

		lines[ i ] = ( i + 1 ) + ': ' + lines[ i ];

	}

	return lines.join( '\n' );

}

function getEncodingComponents( encoding ) {

	switch ( encoding ) {

		case LinearEncoding:
			return [ 'Linear', '( value )' ];
		case sRGBEncoding:
			return [ 'sRGB', '( value )' ];
		case RGBEEncoding:
			return [ 'RGBE', '( value )' ];
		case RGBM7Encoding:
			return [ 'RGBM', '( value, 7.0 )' ];
		case RGBM16Encoding:
			return [ 'RGBM', '( value, 16.0 )' ];
		case RGBDEncoding:
			return [ 'RGBD', '( value, 256.0 )' ];
		case GammaEncoding:
			return [ 'Gamma', '( value, float( GAMMA_FACTOR ) )' ];
		case LogLuvEncoding:
			return [ 'LogLuv', '( value )' ];
		default:
			console.warn( 'THREE.WebGLProgram: Unsupported encoding:', encoding );
			return [ 'Linear', '( value )' ];

	}

}

function getShaderErrors( gl, shader, type ) {

	const status = gl.getShaderParameter( shader, 35713 );
	const log = gl.getShaderInfoLog( shader ).trim();

	if ( status && log === '' ) return '';

	// --enable-privileged-webgl-extension
	// console.log( '**' + type + '**', gl.getExtension( 'WEBGL_debug_shaders' ).getTranslatedShaderSource( shader ) );

	const source = gl.getShaderSource( shader );

	return 'THREE.WebGLShader: gl.getShaderInfoLog() ' + type + '\n' + log + addLineNumbers( source );

}

function getTexelDecodingFunction( functionName, encoding ) {

	const components = getEncodingComponents( encoding );
	return 'vec4 ' + functionName + '( vec4 value ) { return ' + components[ 0 ] + 'ToLinear' + components[ 1 ] + '; }';

}

function getTexelEncodingFunction( functionName, encoding ) {

	const components = getEncodingComponents( encoding );
	return 'vec4 ' + functionName + '( vec4 value ) { return LinearTo' + components[ 0 ] + components[ 1 ] + '; }';

}

function getToneMappingFunction( functionName, toneMapping ) {

	let toneMappingName;

	switch ( toneMapping ) {

		case LinearToneMapping:
			toneMappingName = 'Linear';
			break;

		case ReinhardToneMapping:
			toneMappingName = 'Reinhard';
			break;

		case CineonToneMapping:
			toneMappingName = 'OptimizedCineon';
			break;

		case ACESFilmicToneMapping:
			toneMappingName = 'ACESFilmic';
			break;

		case CustomToneMapping:
			toneMappingName = 'Custom';
			break;

		default:
			console.warn( 'THREE.WebGLProgram: Unsupported toneMapping:', toneMapping );
			toneMappingName = 'Linear';

	}

	return 'vec3 ' + functionName + '( vec3 color ) { return ' + toneMappingName + 'ToneMapping( color ); }';

}

function generateExtensions( parameters ) {

	const chunks = [
		( parameters.extensionDerivatives || parameters.envMapCubeUV || parameters.bumpMap || parameters.tangentSpaceNormalMap || parameters.clearcoatNormalMap || parameters.flatShading || parameters.shaderID === 'physical' ) ? '#extension GL_OES_standard_derivatives : enable' : '',
		( parameters.extensionFragDepth || parameters.logarithmicDepthBuffer ) && parameters.rendererExtensionFragDepth ? '#extension GL_EXT_frag_depth : enable' : '',
		( parameters.extensionDrawBuffers && parameters.rendererExtensionDrawBuffers ) ? '#extension GL_EXT_draw_buffers : require' : '',
		( parameters.extensionShaderTextureLOD || parameters.envMap ) && parameters.rendererExtensionShaderTextureLod ? '#extension GL_EXT_shader_texture_lod : enable' : ''
	];

	return chunks.filter( filterEmptyLine ).join( '\n' );

}

function generateDefines( defines ) {

	const chunks = [];

	for ( const name in defines ) {

		const value = defines[ name ];

		if ( value === false ) continue;

		chunks.push( '#define ' + name + ' ' + value );

	}

	return chunks.join( '\n' );

}

function fetchAttributeLocations( gl, program ) {

	const attributes = {};

	const n = gl.getProgramParameter( program, 35721 );

	for ( let i = 0; i < n; i ++ ) {

		const info = gl.getActiveAttrib( program, i );
		const name = info.name;

		// console.log( 'THREE.WebGLProgram: ACTIVE VERTEX ATTRIBUTE:', name, i );

		attributes[ name ] = gl.getAttribLocation( program, name );

	}

	return attributes;

}

function filterEmptyLine( string ) {

	return string !== '';

}

function replaceLightNums( string, parameters ) {

	return string
		.replace( /NUM_DIR_LIGHTS/g, parameters.numDirLights )
		.replace( /NUM_SPOT_LIGHTS/g, parameters.numSpotLights )
		.replace( /NUM_RECT_AREA_LIGHTS/g, parameters.numRectAreaLights )
		.replace( /NUM_POINT_LIGHTS/g, parameters.numPointLights )
		.replace( /NUM_HEMI_LIGHTS/g, parameters.numHemiLights )
		.replace( /NUM_DIR_LIGHT_SHADOWS/g, parameters.numDirLightShadows )
		.replace( /NUM_SPOT_LIGHT_SHADOWS/g, parameters.numSpotLightShadows )
		.replace( /NUM_POINT_LIGHT_SHADOWS/g, parameters.numPointLightShadows );

}

function replaceClippingPlaneNums( string, parameters ) {

	return string
		.replace( /NUM_CLIPPING_PLANES/g, parameters.numClippingPlanes )
		.replace( /UNION_CLIPPING_PLANES/g, ( parameters.numClippingPlanes - parameters.numClipIntersection ) );

}

// Resolve Includes

const includePattern = /^[ \t]*#include +<([\w\d./]+)>/gm;

function resolveIncludes( string ) {

	return string.replace( includePattern, includeReplacer );

}

function includeReplacer( match, include ) {

	const string = ShaderChunk[ include ];

	if ( string === undefined ) {

		throw new Error( 'Can not resolve #include <' + include + '>' );

	}

	return resolveIncludes( string );

}

// Unroll Loops

const deprecatedUnrollLoopPattern = /#pragma unroll_loop[\s]+?for \( int i \= (\d+)\; i < (\d+)\; i \+\+ \) \{([\s\S]+?)(?=\})\}/g;
const unrollLoopPattern = /#pragma unroll_loop_start\s+for\s*\(\s*int\s+i\s*=\s*(\d+)\s*;\s*i\s*<\s*(\d+)\s*;\s*i\s*\+\+\s*\)\s*{([\s\S]+?)}\s+#pragma unroll_loop_end/g;

function unrollLoops( string ) {

	return string
		.replace( unrollLoopPattern, loopReplacer )
		.replace( deprecatedUnrollLoopPattern, deprecatedLoopReplacer );

}

function deprecatedLoopReplacer( match, start, end, snippet ) {

	console.warn( 'WebGLProgram: #pragma unroll_loop shader syntax is deprecated. Please use #pragma unroll_loop_start syntax instead.' );
	return loopReplacer( match, start, end, snippet );

}

function loopReplacer( match, start, end, snippet ) {

	let string = '';

	for ( let i = parseInt( start ); i < parseInt( end ); i ++ ) {

		string += snippet
			.replace( /\[\s*i\s*\]/g, '[ ' + i + ' ]' )
			.replace( /UNROLLED_LOOP_INDEX/g, i );

	}

	return string;

}

//

function generatePrecision( parameters ) {

	let precisionstring = 'precision ' + parameters.precision + ' float;\nprecision ' + parameters.precision + ' int;';

	if ( parameters.precision === 'highp' ) {

		precisionstring += '\n#define HIGH_PRECISION';

	} else if ( parameters.precision === 'mediump' ) {

		precisionstring += '\n#define MEDIUM_PRECISION';

	} else if ( parameters.precision === 'lowp' ) {

		precisionstring += '\n#define LOW_PRECISION';

	}

	return precisionstring;

}

function generateShadowMapTypeDefine( parameters ) {

	let shadowMapTypeDefine = 'SHADOWMAP_TYPE_BASIC';

	if ( parameters.shadowMapType === PCFShadowMap ) {

		shadowMapTypeDefine = 'SHADOWMAP_TYPE_PCF';

	} else if ( parameters.shadowMapType === PCFSoftShadowMap ) {

		shadowMapTypeDefine = 'SHADOWMAP_TYPE_PCF_SOFT';

	} else if ( parameters.shadowMapType === VSMShadowMap ) {

		shadowMapTypeDefine = 'SHADOWMAP_TYPE_VSM';

	}

	return shadowMapTypeDefine;

}

function generateEnvMapTypeDefine( parameters ) {

	let envMapTypeDefine = 'ENVMAP_TYPE_CUBE';

	if ( parameters.envMap ) {

		switch ( parameters.envMapMode ) {

			case CubeReflectionMapping:
			case CubeRefractionMapping:
				envMapTypeDefine = 'ENVMAP_TYPE_CUBE';
				break;

			case CubeUVReflectionMapping:
			case CubeUVRefractionMapping:
				envMapTypeDefine = 'ENVMAP_TYPE_CUBE_UV';
				break;

		}

	}

	return envMapTypeDefine;

}

function generateEnvMapModeDefine( parameters ) {

	let envMapModeDefine = 'ENVMAP_MODE_REFLECTION';

	if ( parameters.envMap ) {

		switch ( parameters.envMapMode ) {

			case CubeRefractionMapping:
			case CubeUVRefractionMapping:

				envMapModeDefine = 'ENVMAP_MODE_REFRACTION';
				break;

		}

	}

	return envMapModeDefine;

}

function generateEnvMapBlendingDefine( parameters ) {

	let envMapBlendingDefine = 'ENVMAP_BLENDING_NONE';

	if ( parameters.envMap ) {

		switch ( parameters.combine ) {

			case MultiplyOperation:
				envMapBlendingDefine = 'ENVMAP_BLENDING_MULTIPLY';
				break;

			case MixOperation:
				envMapBlendingDefine = 'ENVMAP_BLENDING_MIX';
				break;

			case AddOperation:
				envMapBlendingDefine = 'ENVMAP_BLENDING_ADD';
				break;

		}

	}

	return envMapBlendingDefine;

}

function WebGLProgram( renderer, cacheKey, parameters, bindingStates ) {

	const gl = renderer.getContext();

	const defines = parameters.defines;

	let vertexShader = parameters.vertexShader;
	let fragmentShader = parameters.fragmentShader;

	const shadowMapTypeDefine = generateShadowMapTypeDefine( parameters );
	const envMapTypeDefine = generateEnvMapTypeDefine( parameters );
	const envMapModeDefine = generateEnvMapModeDefine( parameters );
	const envMapBlendingDefine = generateEnvMapBlendingDefine( parameters );


	const gammaFactorDefine = ( renderer.gammaFactor > 0 ) ? renderer.gammaFactor : 1.0;

	const customExtensions = parameters.isWebGL2 ? '' : generateExtensions( parameters );

	const customDefines = generateDefines( defines );

	const program = gl.createProgram();

	let prefixVertex, prefixFragment;
	let versionString = parameters.glslVersion ? '#version ' + parameters.glslVersion + '\n' : '';

	if ( parameters.isRawShaderMaterial ) {

		prefixVertex = [

			customDefines

		].filter( filterEmptyLine ).join( '\n' );

		if ( prefixVertex.length > 0 ) {

			prefixVertex += '\n';

		}

		prefixFragment = [

			customExtensions,
			customDefines

		].filter( filterEmptyLine ).join( '\n' );

		if ( prefixFragment.length > 0 ) {

			prefixFragment += '\n';

		}

	} else {

		prefixVertex = [

			generatePrecision( parameters ),

			'#define SHADER_NAME ' + parameters.shaderName,

			customDefines,

			parameters.instancing ? '#define USE_INSTANCING' : '',
			parameters.instancingColor ? '#define USE_INSTANCING_COLOR' : '',

			parameters.supportsVertexTextures ? '#define VERTEX_TEXTURES' : '',

			'#define GAMMA_FACTOR ' + gammaFactorDefine,

			'#define MAX_BONES ' + parameters.maxBones,
			( parameters.useFog && parameters.fog ) ? '#define USE_FOG' : '',
			( parameters.useFog && parameters.fogExp2 ) ? '#define FOG_EXP2' : '',

			parameters.map ? '#define USE_MAP' : '',
			parameters.envMap ? '#define USE_ENVMAP' : '',
			parameters.envMap ? '#define ' + envMapModeDefine : '',
			parameters.lightMap ? '#define USE_LIGHTMAP' : '',
			parameters.aoMap ? '#define USE_AOMAP' : '',
			parameters.emissiveMap ? '#define USE_EMISSIVEMAP' : '',
			parameters.bumpMap ? '#define USE_BUMPMAP' : '',
			parameters.normalMap ? '#define USE_NORMALMAP' : '',
			( parameters.normalMap && parameters.objectSpaceNormalMap ) ? '#define OBJECTSPACE_NORMALMAP' : '',
			( parameters.normalMap && parameters.tangentSpaceNormalMap ) ? '#define TANGENTSPACE_NORMALMAP' : '',

			parameters.clearcoatMap ? '#define USE_CLEARCOATMAP' : '',
			parameters.clearcoatRoughnessMap ? '#define USE_CLEARCOAT_ROUGHNESSMAP' : '',
			parameters.clearcoatNormalMap ? '#define USE_CLEARCOAT_NORMALMAP' : '',
			parameters.displacementMap && parameters.supportsVertexTextures ? '#define USE_DISPLACEMENTMAP' : '',
			parameters.specularMap ? '#define USE_SPECULARMAP' : '',
			parameters.roughnessMap ? '#define USE_ROUGHNESSMAP' : '',
			parameters.metalnessMap ? '#define USE_METALNESSMAP' : '',
			parameters.alphaMap ? '#define USE_ALPHAMAP' : '',
			parameters.transmissionMap ? '#define USE_TRANSMISSIONMAP' : '',

			parameters.vertexTangents ? '#define USE_TANGENT' : '',
			parameters.vertexColors ? '#define USE_COLOR' : '',
			parameters.vertexAlphas ? '#define USE_COLOR_ALPHA' : '',
			parameters.vertexUvs ? '#define USE_UV' : '',
			parameters.uvsVertexOnly ? '#define UVS_VERTEX_ONLY' : '',

			parameters.flatShading ? '#define FLAT_SHADED' : '',

			parameters.skinning ? '#define USE_SKINNING' : '',
			parameters.useVertexTexture ? '#define BONE_TEXTURE' : '',

			parameters.morphTargets ? '#define USE_MORPHTARGETS' : '',
			parameters.morphNormals && parameters.flatShading === false ? '#define USE_MORPHNORMALS' : '',
			parameters.doubleSided ? '#define DOUBLE_SIDED' : '',
			parameters.flipSided ? '#define FLIP_SIDED' : '',

			parameters.shadowMapEnabled ? '#define USE_SHADOWMAP' : '',
			parameters.shadowMapEnabled ? '#define ' + shadowMapTypeDefine : '',

			parameters.sizeAttenuation ? '#define USE_SIZEATTENUATION' : '',

			parameters.logarithmicDepthBuffer ? '#define USE_LOGDEPTHBUF' : '',
			( parameters.logarithmicDepthBuffer && parameters.rendererExtensionFragDepth ) ? '#define USE_LOGDEPTHBUF_EXT' : '',

			'uniform mat4 modelMatrix;',
			'uniform mat4 modelViewMatrix;',
			'uniform mat4 projectionMatrix;',
			'uniform mat4 viewMatrix;',
			'uniform mat3 normalMatrix;',
			'uniform vec3 cameraPosition;',
			'uniform bool isOrthographic;',

			'#ifdef USE_INSTANCING',

			'	attribute mat4 instanceMatrix;',

			'#endif',

			'#ifdef USE_INSTANCING_COLOR',

			'	attribute vec3 instanceColor;',

			'#endif',

			'attribute vec3 position;',
			'attribute vec3 normal;',
			'attribute vec2 uv;',

			'#ifdef USE_TANGENT',

			'	attribute vec4 tangent;',

			'#endif',

			'#if defined( USE_COLOR_ALPHA )',

			'	attribute vec4 color;',

			'#elif defined( USE_COLOR )',

			'	attribute vec3 color;',

			'#endif',

			'#ifdef USE_MORPHTARGETS',

			'	attribute vec3 morphTarget0;',
			'	attribute vec3 morphTarget1;',
			'	attribute vec3 morphTarget2;',
			'	attribute vec3 morphTarget3;',

			'	#ifdef USE_MORPHNORMALS',

			'		attribute vec3 morphNormal0;',
			'		attribute vec3 morphNormal1;',
			'		attribute vec3 morphNormal2;',
			'		attribute vec3 morphNormal3;',

			'	#else',

			'		attribute vec3 morphTarget4;',
			'		attribute vec3 morphTarget5;',
			'		attribute vec3 morphTarget6;',
			'		attribute vec3 morphTarget7;',

			'	#endif',

			'#endif',

			'#ifdef USE_SKINNING',

			'	attribute vec4 skinIndex;',
			'	attribute vec4 skinWeight;',

			'#endif',

			'\n'

		].filter( filterEmptyLine ).join( '\n' );

		prefixFragment = [

			customExtensions,

			generatePrecision( parameters ),

			'#define SHADER_NAME ' + parameters.shaderName,

			customDefines,

			parameters.alphaTest ? '#define ALPHATEST ' + parameters.alphaTest + ( parameters.alphaTest % 1 ? '' : '.0' ) : '', // add '.0' if integer

			'#define GAMMA_FACTOR ' + gammaFactorDefine,

			( parameters.useFog && parameters.fog ) ? '#define USE_FOG' : '',
			( parameters.useFog && parameters.fogExp2 ) ? '#define FOG_EXP2' : '',

			parameters.map ? '#define USE_MAP' : '',
			parameters.matcap ? '#define USE_MATCAP' : '',
			parameters.envMap ? '#define USE_ENVMAP' : '',
			parameters.envMap ? '#define ' + envMapTypeDefine : '',
			parameters.envMap ? '#define ' + envMapModeDefine : '',
			parameters.envMap ? '#define ' + envMapBlendingDefine : '',
			parameters.lightMap ? '#define USE_LIGHTMAP' : '',
			parameters.aoMap ? '#define USE_AOMAP' : '',
			parameters.emissiveMap ? '#define USE_EMISSIVEMAP' : '',
			parameters.bumpMap ? '#define USE_BUMPMAP' : '',
			parameters.normalMap ? '#define USE_NORMALMAP' : '',
			( parameters.normalMap && parameters.objectSpaceNormalMap ) ? '#define OBJECTSPACE_NORMALMAP' : '',
			( parameters.normalMap && parameters.tangentSpaceNormalMap ) ? '#define TANGENTSPACE_NORMALMAP' : '',
			parameters.clearcoatMap ? '#define USE_CLEARCOATMAP' : '',
			parameters.clearcoatRoughnessMap ? '#define USE_CLEARCOAT_ROUGHNESSMAP' : '',
			parameters.clearcoatNormalMap ? '#define USE_CLEARCOAT_NORMALMAP' : '',
			parameters.specularMap ? '#define USE_SPECULARMAP' : '',
			parameters.roughnessMap ? '#define USE_ROUGHNESSMAP' : '',
			parameters.metalnessMap ? '#define USE_METALNESSMAP' : '',
			parameters.alphaMap ? '#define USE_ALPHAMAP' : '',

			parameters.sheen ? '#define USE_SHEEN' : '',
			parameters.transmissionMap ? '#define USE_TRANSMISSIONMAP' : '',

			parameters.vertexTangents ? '#define USE_TANGENT' : '',
			parameters.vertexColors || parameters.instancingColor ? '#define USE_COLOR' : '',
			parameters.vertexAlphas ? '#define USE_COLOR_ALPHA' : '',
			parameters.vertexUvs ? '#define USE_UV' : '',
			parameters.uvsVertexOnly ? '#define UVS_VERTEX_ONLY' : '',

			parameters.gradientMap ? '#define USE_GRADIENTMAP' : '',

			parameters.flatShading ? '#define FLAT_SHADED' : '',

			parameters.doubleSided ? '#define DOUBLE_SIDED' : '',
			parameters.flipSided ? '#define FLIP_SIDED' : '',

			parameters.shadowMapEnabled ? '#define USE_SHADOWMAP' : '',
			parameters.shadowMapEnabled ? '#define ' + shadowMapTypeDefine : '',

			parameters.premultipliedAlpha ? '#define PREMULTIPLIED_ALPHA' : '',

			parameters.physicallyCorrectLights ? '#define PHYSICALLY_CORRECT_LIGHTS' : '',

			parameters.logarithmicDepthBuffer ? '#define USE_LOGDEPTHBUF' : '',
			( parameters.logarithmicDepthBuffer && parameters.rendererExtensionFragDepth ) ? '#define USE_LOGDEPTHBUF_EXT' : '',

			( ( parameters.extensionShaderTextureLOD || parameters.envMap ) && parameters.rendererExtensionShaderTextureLod ) ? '#define TEXTURE_LOD_EXT' : '',

			'uniform mat4 viewMatrix;',
			'uniform vec3 cameraPosition;',
			'uniform bool isOrthographic;',

			( parameters.toneMapping !== NoToneMapping ) ? '#define TONE_MAPPING' : '',
			( parameters.toneMapping !== NoToneMapping ) ? ShaderChunk[ 'tonemapping_pars_fragment' ] : '', // this code is required here because it is used by the toneMapping() function defined below
			( parameters.toneMapping !== NoToneMapping ) ? getToneMappingFunction( 'toneMapping', parameters.toneMapping ) : '',

			parameters.dithering ? '#define DITHERING' : '',

			ShaderChunk[ 'encodings_pars_fragment' ], // this code is required here because it is used by the various encoding/decoding function defined below
			parameters.map ? getTexelDecodingFunction( 'mapTexelToLinear', parameters.mapEncoding ) : '',
			parameters.matcap ? getTexelDecodingFunction( 'matcapTexelToLinear', parameters.matcapEncoding ) : '',
			parameters.envMap ? getTexelDecodingFunction( 'envMapTexelToLinear', parameters.envMapEncoding ) : '',
			parameters.emissiveMap ? getTexelDecodingFunction( 'emissiveMapTexelToLinear', parameters.emissiveMapEncoding ) : '',
			parameters.lightMap ? getTexelDecodingFunction( 'lightMapTexelToLinear', parameters.lightMapEncoding ) : '',
			getTexelEncodingFunction( 'linearToOutputTexel', parameters.outputEncoding ),

			parameters.depthPacking ? '#define DEPTH_PACKING ' + parameters.depthPacking : '',

			'\n'

		].filter( filterEmptyLine ).join( '\n' );

	}

	vertexShader = resolveIncludes( vertexShader );
	vertexShader = replaceLightNums( vertexShader, parameters );
	vertexShader = replaceClippingPlaneNums( vertexShader, parameters );

	fragmentShader = resolveIncludes( fragmentShader );
	fragmentShader = replaceLightNums( fragmentShader, parameters );
	fragmentShader = replaceClippingPlaneNums( fragmentShader, parameters );

	vertexShader = unrollLoops( vertexShader );
	fragmentShader = unrollLoops( fragmentShader );

	if ( parameters.isWebGL2 && parameters.isRawShaderMaterial !== true ) {

		// GLSL 3.0 conversion for built-in materials and ShaderMaterial

		versionString = '#version 300 es\n';

		prefixVertex = [
			'#define attribute in',
			'#define varying out',
			'#define texture2D texture'
		].join( '\n' ) + '\n' + prefixVertex;

		prefixFragment = [
			'#define varying in',
			( parameters.glslVersion === GLSL3 ) ? '' : 'out highp vec4 pc_fragColor;',
			( parameters.glslVersion === GLSL3 ) ? '' : '#define gl_FragColor pc_fragColor',
			'#define gl_FragDepthEXT gl_FragDepth',
			'#define texture2D texture',
			'#define textureCube texture',
			'#define texture2DProj textureProj',
			'#define texture2DLodEXT textureLod',
			'#define texture2DProjLodEXT textureProjLod',
			'#define textureCubeLodEXT textureLod',
			'#define texture2DGradEXT textureGrad',
			'#define texture2DProjGradEXT textureProjGrad',
			'#define textureCubeGradEXT textureGrad'
		].join( '\n' ) + '\n' + prefixFragment;

	}

	const vertexGlsl = versionString + prefixVertex + vertexShader;
	const fragmentGlsl = versionString + prefixFragment + fragmentShader;

	// console.log( '*VERTEX*', vertexGlsl );
	// console.log( '*FRAGMENT*', fragmentGlsl );

	const glVertexShader = WebGLShader( gl, 35633, vertexGlsl );
	const glFragmentShader = WebGLShader( gl, 35632, fragmentGlsl );

	gl.attachShader( program, glVertexShader );
	gl.attachShader( program, glFragmentShader );

	// Force a particular attribute to index 0.

	if ( parameters.index0AttributeName !== undefined ) {

		gl.bindAttribLocation( program, 0, parameters.index0AttributeName );

	} else if ( parameters.morphTargets === true ) {

		// programs with morphTargets displace position out of attribute 0
		gl.bindAttribLocation( program, 0, 'position' );

	}

	gl.linkProgram( program );

	// check for link errors
	if ( renderer.debug.checkShaderErrors ) {

		const programLog = gl.getProgramInfoLog( program ).trim();
		const vertexLog = gl.getShaderInfoLog( glVertexShader ).trim();
		const fragmentLog = gl.getShaderInfoLog( glFragmentShader ).trim();

		let runnable = true;
		let haveDiagnostics = true;

		if ( gl.getProgramParameter( program, 35714 ) === false ) {

			runnable = false;

			const vertexErrors = getShaderErrors( gl, glVertexShader, 'vertex' );
			const fragmentErrors = getShaderErrors( gl, glFragmentShader, 'fragment' );

			console.error( 'THREE.WebGLProgram: shader error: ', gl.getError(), '35715', gl.getProgramParameter( program, 35715 ), 'gl.getProgramInfoLog', programLog, vertexErrors, fragmentErrors );

		} else if ( programLog !== '' ) {

			console.warn( 'THREE.WebGLProgram: gl.getProgramInfoLog()', programLog );

		} else if ( vertexLog === '' || fragmentLog === '' ) {

			haveDiagnostics = false;

		}

		if ( haveDiagnostics ) {

			this.diagnostics = {

				runnable: runnable,

				programLog: programLog,

				vertexShader: {

					log: vertexLog,
					prefix: prefixVertex

				},

				fragmentShader: {

					log: fragmentLog,
					prefix: prefixFragment

				}

			};

		}

	}

	// Clean up

	// Crashes in iOS9 and iOS10. #18402
	// gl.detachShader( program, glVertexShader );
	// gl.detachShader( program, glFragmentShader );

	gl.deleteShader( glVertexShader );
	gl.deleteShader( glFragmentShader );

	// set up caching for uniform locations

	let cachedUniforms;

	this.getUniforms = function () {

		if ( cachedUniforms === undefined ) {

			cachedUniforms = new WebGLUniforms( gl, program );

		}

		return cachedUniforms;

	};

	// set up caching for attribute locations

	let cachedAttributes;

	this.getAttributes = function () {

		if ( cachedAttributes === undefined ) {

			cachedAttributes = fetchAttributeLocations( gl, program );

		}

		return cachedAttributes;

	};

	// free resource

	this.destroy = function () {

		bindingStates.releaseStatesOfProgram( this );

		gl.deleteProgram( program );
		this.program = undefined;

	};

	//

	this.name = parameters.shaderName;
	this.id = programIdCount ++;
	this.cacheKey = cacheKey;
	this.usedTimes = 1;
	this.program = program;
	this.vertexShader = glVertexShader;
	this.fragmentShader = glFragmentShader;

	return this;

}

function WebGLPrograms( renderer, cubemaps, extensions, capabilities, bindingStates, clipping ) {

	const programs = [];

	const isWebGL2 = capabilities.isWebGL2;
	const logarithmicDepthBuffer = capabilities.logarithmicDepthBuffer;
	const floatVertexTextures = capabilities.floatVertexTextures;
	const maxVertexUniforms = capabilities.maxVertexUniforms;
	const vertexTextures = capabilities.vertexTextures;

	let precision = capabilities.precision;

	const shaderIDs = {
		MeshDepthMaterial: 'depth',
		MeshDistanceMaterial: 'distanceRGBA',
		MeshNormalMaterial: 'normal',
		MeshBasicMaterial: 'basic',
		MeshLambertMaterial: 'lambert',
		MeshPhongMaterial: 'phong',
		MeshToonMaterial: 'toon',
		MeshStandardMaterial: 'physical',
		MeshPhysicalMaterial: 'physical',
		MeshMatcapMaterial: 'matcap',
		LineBasicMaterial: 'basic',
		LineDashedMaterial: 'dashed',
		PointsMaterial: 'points',
		ShadowMaterial: 'shadow',
		SpriteMaterial: 'sprite'
	};

	const parameterNames = [
		'precision', 'isWebGL2', 'supportsVertexTextures', 'outputEncoding', 'instancing', 'instancingColor',
		'map', 'mapEncoding', 'matcap', 'matcapEncoding', 'envMap', 'envMapMode', 'envMapEncoding', 'envMapCubeUV',
		'lightMap', 'lightMapEncoding', 'aoMap', 'emissiveMap', 'emissiveMapEncoding', 'bumpMap', 'normalMap', 'objectSpaceNormalMap', 'tangentSpaceNormalMap', 'clearcoatMap', 'clearcoatRoughnessMap', 'clearcoatNormalMap', 'displacementMap', 'specularMap',
		'roughnessMap', 'metalnessMap', 'gradientMap',
		'alphaMap', 'combine', 'vertexColors', 'vertexAlphas', 'vertexTangents', 'vertexUvs', 'uvsVertexOnly', 'fog', 'useFog', 'fogExp2',
		'flatShading', 'sizeAttenuation', 'logarithmicDepthBuffer', 'skinning',
		'maxBones', 'useVertexTexture', 'morphTargets', 'morphNormals', 'premultipliedAlpha',
		'numDirLights', 'numPointLights', 'numSpotLights', 'numHemiLights', 'numRectAreaLights',
		'numDirLightShadows', 'numPointLightShadows', 'numSpotLightShadows',
		'shadowMapEnabled', 'shadowMapType', 'toneMapping', 'physicallyCorrectLights',
		'alphaTest', 'doubleSided', 'flipSided', 'numClippingPlanes', 'numClipIntersection', 'depthPacking', 'dithering',
		'sheen', 'transmissionMap'
	];

	function getMaxBones( object ) {

		const skeleton = object.skeleton;
		const bones = skeleton.bones;

		if ( floatVertexTextures ) {

			return 1024;

		} else {

			// default for when object is not specified
			// ( for example when prebuilding shader to be used with multiple objects )
			//
			//  - leave some extra space for other uniforms
			//  - limit here is ANGLE's 254 max uniform vectors
			//    (up to 54 should be safe)

			const nVertexUniforms = maxVertexUniforms;
			const nVertexMatrices = Math.floor( ( nVertexUniforms - 20 ) / 4 );

			const maxBones = Math.min( nVertexMatrices, bones.length );

			if ( maxBones < bones.length ) {

				console.warn( 'THREE.WebGLRenderer: Skeleton has ' + bones.length + ' bones. This GPU supports ' + maxBones + '.' );
				return 0;

			}

			return maxBones;

		}

	}

	function getTextureEncodingFromMap( map ) {

		let encoding;

		if ( map && map.isTexture ) {

			encoding = map.encoding;

		} else if ( map && map.isWebGLRenderTarget ) {

			console.warn( 'THREE.WebGLPrograms.getTextureEncodingFromMap: don\'t use render targets as textures. Use their .texture property instead.' );
			encoding = map.texture.encoding;

		} else {

			encoding = LinearEncoding;

		}

		return encoding;

	}

	function getParameters( material, lights, shadows, scene, object ) {

		const fog = scene.fog;
		const environment = material.isMeshStandardMaterial ? scene.environment : null;

		const envMap = cubemaps.get( material.envMap || environment );

		const shaderID = shaderIDs[ material.type ];

		// heuristics to create shader parameters according to lights in the scene
		// (not to blow over maxLights budget)

		const maxBones = object.isSkinnedMesh ? getMaxBones( object ) : 0;

		if ( material.precision !== null ) {

			precision = capabilities.getMaxPrecision( material.precision );

			if ( precision !== material.precision ) {

				console.warn( 'THREE.WebGLProgram.getParameters:', material.precision, 'not supported, using', precision, 'instead.' );

			}

		}

		let vertexShader, fragmentShader;

		if ( shaderID ) {

			const shader = ShaderLib[ shaderID ];

			vertexShader = shader.vertexShader;
			fragmentShader = shader.fragmentShader;

		} else {

			vertexShader = material.vertexShader;
			fragmentShader = material.fragmentShader;

		}

		const currentRenderTarget = renderer.getRenderTarget();

		const parameters = {

			isWebGL2: isWebGL2,

			shaderID: shaderID,
			shaderName: material.type,

			vertexShader: vertexShader,
			fragmentShader: fragmentShader,
			defines: material.defines,

			isRawShaderMaterial: material.isRawShaderMaterial === true,
			glslVersion: material.glslVersion,

			precision: precision,

			instancing: object.isInstancedMesh === true,
			instancingColor: object.isInstancedMesh === true && object.instanceColor !== null,

			supportsVertexTextures: vertexTextures,
			outputEncoding: ( currentRenderTarget !== null ) ? getTextureEncodingFromMap( currentRenderTarget.texture ) : renderer.outputEncoding,
			map: !! material.map,
			mapEncoding: getTextureEncodingFromMap( material.map ),
			matcap: !! material.matcap,
			matcapEncoding: getTextureEncodingFromMap( material.matcap ),
			envMap: !! envMap,
			envMapMode: envMap && envMap.mapping,
			envMapEncoding: getTextureEncodingFromMap( envMap ),
			envMapCubeUV: ( !! envMap ) && ( ( envMap.mapping === CubeUVReflectionMapping ) || ( envMap.mapping === CubeUVRefractionMapping ) ),
			lightMap: !! material.lightMap,
			lightMapEncoding: getTextureEncodingFromMap( material.lightMap ),
			aoMap: !! material.aoMap,
			emissiveMap: !! material.emissiveMap,
			emissiveMapEncoding: getTextureEncodingFromMap( material.emissiveMap ),
			bumpMap: !! material.bumpMap,
			normalMap: !! material.normalMap,
			objectSpaceNormalMap: material.normalMapType === ObjectSpaceNormalMap,
			tangentSpaceNormalMap: material.normalMapType === TangentSpaceNormalMap,
			clearcoatMap: !! material.clearcoatMap,
			clearcoatRoughnessMap: !! material.clearcoatRoughnessMap,
			clearcoatNormalMap: !! material.clearcoatNormalMap,
			displacementMap: !! material.displacementMap,
			roughnessMap: !! material.roughnessMap,
			metalnessMap: !! material.metalnessMap,
			specularMap: !! material.specularMap,
			alphaMap: !! material.alphaMap,

			gradientMap: !! material.gradientMap,

			sheen: !! material.sheen,

			transmissionMap: !! material.transmissionMap,

			combine: material.combine,

			vertexTangents: ( material.normalMap && material.vertexTangents ),
			vertexColors: material.vertexColors,
			vertexAlphas: material.vertexColors === true && object.geometry.attributes.color && object.geometry.attributes.color.itemSize === 4,
			vertexUvs: !! material.map || !! material.bumpMap || !! material.normalMap || !! material.specularMap || !! material.alphaMap || !! material.emissiveMap || !! material.roughnessMap || !! material.metalnessMap || !! material.clearcoatMap || !! material.clearcoatRoughnessMap || !! material.clearcoatNormalMap || !! material.displacementMap || !! material.transmissionMap,
			uvsVertexOnly: ! ( !! material.map || !! material.bumpMap || !! material.normalMap || !! material.specularMap || !! material.alphaMap || !! material.emissiveMap || !! material.roughnessMap || !! material.metalnessMap || !! material.clearcoatNormalMap || !! material.transmissionMap ) && !! material.displacementMap,

			fog: !! fog,
			useFog: material.fog,
			fogExp2: ( fog && fog.isFogExp2 ),

			flatShading: !! material.flatShading,

			sizeAttenuation: material.sizeAttenuation,
			logarithmicDepthBuffer: logarithmicDepthBuffer,

			skinning: material.skinning && maxBones > 0,
			maxBones: maxBones,
			useVertexTexture: floatVertexTextures,

			morphTargets: material.morphTargets,
			morphNormals: material.morphNormals,

			numDirLights: lights.directional.length,
			numPointLights: lights.point.length,
			numSpotLights: lights.spot.length,
			numRectAreaLights: lights.rectArea.length,
			numHemiLights: lights.hemi.length,

			numDirLightShadows: lights.directionalShadowMap.length,
			numPointLightShadows: lights.pointShadowMap.length,
			numSpotLightShadows: lights.spotShadowMap.length,

			numClippingPlanes: clipping.numPlanes,
			numClipIntersection: clipping.numIntersection,

			dithering: material.dithering,

			shadowMapEnabled: renderer.shadowMap.enabled && shadows.length > 0,
			shadowMapType: renderer.shadowMap.type,

			toneMapping: material.toneMapped ? renderer.toneMapping : NoToneMapping,
			physicallyCorrectLights: renderer.physicallyCorrectLights,

			premultipliedAlpha: material.premultipliedAlpha,

			alphaTest: material.alphaTest,
			doubleSided: material.side === DoubleSide,
			flipSided: material.side === BackSide,

			depthPacking: ( material.depthPacking !== undefined ) ? material.depthPacking : false,

			index0AttributeName: material.index0AttributeName,

			extensionDerivatives: material.extensions && material.extensions.derivatives,
			extensionFragDepth: material.extensions && material.extensions.fragDepth,
			extensionDrawBuffers: material.extensions && material.extensions.drawBuffers,
			extensionShaderTextureLOD: material.extensions && material.extensions.shaderTextureLOD,

			rendererExtensionFragDepth: isWebGL2 || extensions.has( 'EXT_frag_depth' ),
			rendererExtensionDrawBuffers: isWebGL2 || extensions.has( 'WEBGL_draw_buffers' ),
			rendererExtensionShaderTextureLod: isWebGL2 || extensions.has( 'EXT_shader_texture_lod' ),

			customProgramCacheKey: material.customProgramCacheKey()

		};

		return parameters;

	}

	function getProgramCacheKey( parameters ) {

		const array = [];

		if ( parameters.shaderID ) {

			array.push( parameters.shaderID );

		} else {

			array.push( parameters.fragmentShader );
			array.push( parameters.vertexShader );

		}

		if ( parameters.defines !== undefined ) {

			for ( const name in parameters.defines ) {

				array.push( name );
				array.push( parameters.defines[ name ] );

			}

		}

		if ( parameters.isRawShaderMaterial === false ) {

			for ( let i = 0; i < parameterNames.length; i ++ ) {

				array.push( parameters[ parameterNames[ i ] ] );

			}

			array.push( renderer.outputEncoding );
			array.push( renderer.gammaFactor );

		}

		array.push( parameters.customProgramCacheKey );

		return array.join();

	}

	function getUniforms( material ) {

		const shaderID = shaderIDs[ material.type ];
		let uniforms;

		if ( shaderID ) {

			const shader = ShaderLib[ shaderID ];
			uniforms = UniformsUtils.clone( shader.uniforms );

		} else {

			uniforms = material.uniforms;

		}

		return uniforms;

	}

	function acquireProgram( parameters, cacheKey ) {

		let program;

		// Check if code has been already compiled
		for ( let p = 0, pl = programs.length; p < pl; p ++ ) {

			const preexistingProgram = programs[ p ];

			if ( preexistingProgram.cacheKey === cacheKey ) {

				program = preexistingProgram;
				++ program.usedTimes;

				break;

			}

		}

		if ( program === undefined ) {

			program = new WebGLProgram( renderer, cacheKey, parameters, bindingStates );
			programs.push( program );

		}

		return program;

	}

	function releaseProgram( program ) {

		if ( -- program.usedTimes === 0 ) {

			// Remove from unordered set
			const i = programs.indexOf( program );
			programs[ i ] = programs[ programs.length - 1 ];
			programs.pop();

			// Free WebGL resources
			program.destroy();

		}

	}

	return {
		getParameters: getParameters,
		getProgramCacheKey: getProgramCacheKey,
		getUniforms: getUniforms,
		acquireProgram: acquireProgram,
		releaseProgram: releaseProgram,
		// Exposed for resource monitoring & error feedback via renderer.info:
		programs: programs
	};

}

function WebGLProperties() {

	let properties = new WeakMap();

	function get( object ) {

		let map = properties.get( object );

		if ( map === undefined ) {

			map = {};
			properties.set( object, map );

		}

		return map;

	}

	function remove( object ) {

		properties.delete( object );

	}

	function update( object, key, value ) {

		properties.get( object )[ key ] = value;

	}

	function dispose() {

		properties = new WeakMap();

	}

	return {
		get: get,
		remove: remove,
		update: update,
		dispose: dispose
	};

}

function painterSortStable( a, b ) {

	if ( a.groupOrder !== b.groupOrder ) {

		return a.groupOrder - b.groupOrder;

	} else if ( a.renderOrder !== b.renderOrder ) {

		return a.renderOrder - b.renderOrder;

	} else if ( a.program !== b.program ) {

		return a.program.id - b.program.id;

	} else if ( a.material.id !== b.material.id ) {

		return a.material.id - b.material.id;

	} else if ( a.z !== b.z ) {

		return a.z - b.z;

	} else {

		return a.id - b.id;

	}

}

function reversePainterSortStable( a, b ) {

	if ( a.groupOrder !== b.groupOrder ) {

		return a.groupOrder - b.groupOrder;

	} else if ( a.renderOrder !== b.renderOrder ) {

		return a.renderOrder - b.renderOrder;

	} else if ( a.z !== b.z ) {

		return b.z - a.z;

	} else {

		return a.id - b.id;

	}

}


function WebGLRenderList( properties ) {

	const renderItems = [];
	let renderItemsIndex = 0;

	const opaque = [];
	const transparent = [];

	const defaultProgram = { id: - 1 };

	function init() {

		renderItemsIndex = 0;

		opaque.length = 0;
		transparent.length = 0;

	}

	function getNextRenderItem( object, geometry, material, groupOrder, z, group ) {

		let renderItem = renderItems[ renderItemsIndex ];
		const materialProperties = properties.get( material );

		if ( renderItem === undefined ) {

			renderItem = {
				id: object.id,
				object: object,
				geometry: geometry,
				material: material,
				program: materialProperties.program || defaultProgram,
				groupOrder: groupOrder,
				renderOrder: object.renderOrder,
				z: z,
				group: group
			};

			renderItems[ renderItemsIndex ] = renderItem;

		} else {

			renderItem.id = object.id;
			renderItem.object = object;
			renderItem.geometry = geometry;
			renderItem.material = material;
			renderItem.program = materialProperties.program || defaultProgram;
			renderItem.groupOrder = groupOrder;
			renderItem.renderOrder = object.renderOrder;
			renderItem.z = z;
			renderItem.group = group;

		}

		renderItemsIndex ++;

		return renderItem;

	}

	function push( object, geometry, material, groupOrder, z, group ) {

		const renderItem = getNextRenderItem( object, geometry, material, groupOrder, z, group );

		( material.transparent === true ? transparent : opaque ).push( renderItem );

	}

	function unshift( object, geometry, material, groupOrder, z, group ) {

		const renderItem = getNextRenderItem( object, geometry, material, groupOrder, z, group );

		( material.transparent === true ? transparent : opaque ).unshift( renderItem );

	}

	function sort( customOpaqueSort, customTransparentSort ) {

		if ( opaque.length > 1 ) opaque.sort( customOpaqueSort || painterSortStable );
		if ( transparent.length > 1 ) transparent.sort( customTransparentSort || reversePainterSortStable );

	}

	function finish() {

		// Clear references from inactive renderItems in the list

		for ( let i = renderItemsIndex, il = renderItems.length; i < il; i ++ ) {

			const renderItem = renderItems[ i ];

			if ( renderItem.id === null ) break;

			renderItem.id = null;
			renderItem.object = null;
			renderItem.geometry = null;
			renderItem.material = null;
			renderItem.program = null;
			renderItem.group = null;

		}

	}

	return {

		opaque: opaque,
		transparent: transparent,

		init: init,
		push: push,
		unshift: unshift,
		finish: finish,

		sort: sort
	};

}

function WebGLRenderLists( properties ) {

	let lists = new WeakMap();

	function get( scene, renderCallDepth ) {

		let list;

		if ( lists.has( scene ) === false ) {

			list = new WebGLRenderList( properties );
			lists.set( scene, [ list ] );

		} else {

			if ( renderCallDepth >= lists.get( scene ).length ) {

				list = new WebGLRenderList( properties );
				lists.get( scene ).push( list );

			} else {

				list = lists.get( scene )[ renderCallDepth ];

			}

		}

		return list;

	}

	function dispose() {

		lists = new WeakMap();

	}

	return {
		get: get,
		dispose: dispose
	};

}

function UniformsCache() {

	const lights = {};

	return {

		get: function ( light ) {

			if ( lights[ light.id ] !== undefined ) {

				return lights[ light.id ];

			}

			let uniforms;

			switch ( light.type ) {

				case 'DirectionalLight':
					uniforms = {
						direction: new Vector3(),
						color: new Color()
					};
					break;

				case 'SpotLight':
					uniforms = {
						position: new Vector3(),
						direction: new Vector3(),
						color: new Color(),
						distance: 0,
						coneCos: 0,
						penumbraCos: 0,
						decay: 0
					};
					break;

				case 'PointLight':
					uniforms = {
						position: new Vector3(),
						color: new Color(),
						distance: 0,
						decay: 0
					};
					break;

				case 'HemisphereLight':
					uniforms = {
						direction: new Vector3(),
						skyColor: new Color(),
						groundColor: new Color()
					};
					break;

				case 'RectAreaLight':
					uniforms = {
						color: new Color(),
						position: new Vector3(),
						halfWidth: new Vector3(),
						halfHeight: new Vector3()
					};
					break;

			}

			lights[ light.id ] = uniforms;

			return uniforms;

		}

	};

}

function ShadowUniformsCache() {

	const lights = {};

	return {

		get: function ( light ) {

			if ( lights[ light.id ] !== undefined ) {

				return lights[ light.id ];

			}

			let uniforms;

			switch ( light.type ) {

				case 'DirectionalLight':
					uniforms = {
						shadowBias: 0,
						shadowNormalBias: 0,
						shadowRadius: 1,
						shadowMapSize: new Vector2()
					};
					break;

				case 'SpotLight':
					uniforms = {
						shadowBias: 0,
						shadowNormalBias: 0,
						shadowRadius: 1,
						shadowMapSize: new Vector2()
					};
					break;

				case 'PointLight':
					uniforms = {
						shadowBias: 0,
						shadowNormalBias: 0,
						shadowRadius: 1,
						shadowMapSize: new Vector2(),
						shadowCameraNear: 1,
						shadowCameraFar: 1000
					};
					break;

				// TODO (abelnation): set RectAreaLight shadow uniforms

			}

			lights[ light.id ] = uniforms;

			return uniforms;

		}

	};

}



let nextVersion = 0;

function shadowCastingLightsFirst( lightA, lightB ) {

	return ( lightB.castShadow ? 1 : 0 ) - ( lightA.castShadow ? 1 : 0 );

}

function WebGLLights( extensions, capabilities ) {

	const cache = new UniformsCache();

	const shadowCache = ShadowUniformsCache();

	const state = {

		version: 0,

		hash: {
			directionalLength: - 1,
			pointLength: - 1,
			spotLength: - 1,
			rectAreaLength: - 1,
			hemiLength: - 1,

			numDirectionalShadows: - 1,
			numPointShadows: - 1,
			numSpotShadows: - 1
		},

		ambient: [ 0, 0, 0 ],
		probe: [],
		directional: [],
		directionalShadow: [],
		directionalShadowMap: [],
		directionalShadowMatrix: [],
		spot: [],
		spotShadow: [],
		spotShadowMap: [],
		spotShadowMatrix: [],
		rectArea: [],
		rectAreaLTC1: null,
		rectAreaLTC2: null,
		point: [],
		pointShadow: [],
		pointShadowMap: [],
		pointShadowMatrix: [],
		hemi: []

	};

	for ( let i = 0; i < 9; i ++ ) state.probe.push( new Vector3() );

	const vector3 = new Vector3();
	const matrix4 = new Matrix4();
	const matrix42 = new Matrix4();

	function setup( lights ) {

		let r = 0, g = 0, b = 0;

		for ( let i = 0; i < 9; i ++ ) state.probe[ i ].set( 0, 0, 0 );

		let directionalLength = 0;
		let pointLength = 0;
		let spotLength = 0;
		let rectAreaLength = 0;
		let hemiLength = 0;

		let numDirectionalShadows = 0;
		let numPointShadows = 0;
		let numSpotShadows = 0;

		lights.sort( shadowCastingLightsFirst );

		for ( let i = 0, l = lights.length; i < l; i ++ ) {

			const light = lights[ i ];

			const color = light.color;
			const intensity = light.intensity;
			const distance = light.distance;

			const shadowMap = ( light.shadow && light.shadow.map ) ? light.shadow.map.texture : null;

			if ( light.isAmbientLight ) {

				r += color.r * intensity;
				g += color.g * intensity;
				b += color.b * intensity;

			} else if ( light.isLightProbe ) {

				for ( let j = 0; j < 9; j ++ ) {

					state.probe[ j ].addScaledVector( light.sh.coefficients[ j ], intensity );

				}

			} else if ( light.isDirectionalLight ) {

				const uniforms = cache.get( light );

				uniforms.color.copy( light.color ).multiplyScalar( light.intensity );

				if ( light.castShadow ) {

					const shadow = light.shadow;

					const shadowUniforms = shadowCache.get( light );

					shadowUniforms.shadowBias = shadow.bias;
					shadowUniforms.shadowNormalBias = shadow.normalBias;
					shadowUniforms.shadowRadius = shadow.radius;
					shadowUniforms.shadowMapSize = shadow.mapSize;

					state.directionalShadow[ directionalLength ] = shadowUniforms;
					state.directionalShadowMap[ directionalLength ] = shadowMap;
					state.directionalShadowMatrix[ directionalLength ] = light.shadow.matrix;

					numDirectionalShadows ++;

				}

				state.directional[ directionalLength ] = uniforms;

				directionalLength ++;

			} else if ( light.isSpotLight ) {

				const uniforms = cache.get( light );

				uniforms.position.setFromMatrixPosition( light.matrixWorld );

				uniforms.color.copy( color ).multiplyScalar( intensity );
				uniforms.distance = distance;

				uniforms.coneCos = Math.cos( light.angle );
				uniforms.penumbraCos = Math.cos( light.angle * ( 1 - light.penumbra ) );
				uniforms.decay = light.decay;

				if ( light.castShadow ) {

					const shadow = light.shadow;

					const shadowUniforms = shadowCache.get( light );

					shadowUniforms.shadowBias = shadow.bias;
					shadowUniforms.shadowNormalBias = shadow.normalBias;
					shadowUniforms.shadowRadius = shadow.radius;
					shadowUniforms.shadowMapSize = shadow.mapSize;

					state.spotShadow[ spotLength ] = shadowUniforms;
					state.spotShadowMap[ spotLength ] = shadowMap;
					state.spotShadowMatrix[ spotLength ] = light.shadow.matrix;

					numSpotShadows ++;

				}

				state.spot[ spotLength ] = uniforms;

				spotLength ++;

			} else if ( light.isRectAreaLight ) {

				const uniforms = cache.get( light );

				// (a) intensity is the total visible light emitted
				//uniforms.color.copy( color ).multiplyScalar( intensity / ( light.width * light.height * Math.PI ) );

				// (b) intensity is the brightness of the light
				uniforms.color.copy( color ).multiplyScalar( intensity );

				uniforms.halfWidth.set( light.width * 0.5, 0.0, 0.0 );
				uniforms.halfHeight.set( 0.0, light.height * 0.5, 0.0 );

				state.rectArea[ rectAreaLength ] = uniforms;

				rectAreaLength ++;

			} else if ( light.isPointLight ) {

				const uniforms = cache.get( light );

				uniforms.color.copy( light.color ).multiplyScalar( light.intensity );
				uniforms.distance = light.distance;
				uniforms.decay = light.decay;

				if ( light.castShadow ) {

					const shadow = light.shadow;

					const shadowUniforms = shadowCache.get( light );

					shadowUniforms.shadowBias = shadow.bias;
					shadowUniforms.shadowNormalBias = shadow.normalBias;
					shadowUniforms.shadowRadius = shadow.radius;
					shadowUniforms.shadowMapSize = shadow.mapSize;
					shadowUniforms.shadowCameraNear = shadow.camera.near;
					shadowUniforms.shadowCameraFar = shadow.camera.far;

					state.pointShadow[ pointLength ] = shadowUniforms;
					state.pointShadowMap[ pointLength ] = shadowMap;
					state.pointShadowMatrix[ pointLength ] = light.shadow.matrix;

					numPointShadows ++;

				}

				state.point[ pointLength ] = uniforms;

				pointLength ++;

			} else if ( light.isHemisphereLight ) {

				const uniforms = cache.get( light );

				uniforms.skyColor.copy( light.color ).multiplyScalar( intensity );
				uniforms.groundColor.copy( light.groundColor ).multiplyScalar( intensity );

				state.hemi[ hemiLength ] = uniforms;

				hemiLength ++;

			}

		}

		if ( rectAreaLength > 0 ) {

			if ( capabilities.isWebGL2 ) {

				// WebGL 2

				state.rectAreaLTC1 = UniformsLib.LTC_FLOAT_1;
				state.rectAreaLTC2 = UniformsLib.LTC_FLOAT_2;

			} else {

				// WebGL 1

				if ( extensions.has( 'OES_texture_float_linear' ) === true ) {

					state.rectAreaLTC1 = UniformsLib.LTC_FLOAT_1;
					state.rectAreaLTC2 = UniformsLib.LTC_FLOAT_2;

				} else if ( extensions.has( 'OES_texture_half_float_linear' ) === true ) {

					state.rectAreaLTC1 = UniformsLib.LTC_HALF_1;
					state.rectAreaLTC2 = UniformsLib.LTC_HALF_2;

				} else {

					console.error( 'THREE.WebGLRenderer: Unable to use RectAreaLight. Missing WebGL extensions.' );

				}

			}

		}

		state.ambient[ 0 ] = r;
		state.ambient[ 1 ] = g;
		state.ambient[ 2 ] = b;

		const hash = state.hash;

		if ( hash.directionalLength !== directionalLength ||
			hash.pointLength !== pointLength ||
			hash.spotLength !== spotLength ||
			hash.rectAreaLength !== rectAreaLength ||
			hash.hemiLength !== hemiLength ||
			hash.numDirectionalShadows !== numDirectionalShadows ||
			hash.numPointShadows !== numPointShadows ||
			hash.numSpotShadows !== numSpotShadows ) {

			state.directional.length = directionalLength;
			state.spot.length = spotLength;
			state.rectArea.length = rectAreaLength;
			state.point.length = pointLength;
			state.hemi.length = hemiLength;

			state.directionalShadow.length = numDirectionalShadows;
			state.directionalShadowMap.length = numDirectionalShadows;
			state.pointShadow.length = numPointShadows;
			state.pointShadowMap.length = numPointShadows;
			state.spotShadow.length = numSpotShadows;
			state.spotShadowMap.length = numSpotShadows;
			state.directionalShadowMatrix.length = numDirectionalShadows;
			state.pointShadowMatrix.length = numPointShadows;
			state.spotShadowMatrix.length = numSpotShadows;

			hash.directionalLength = directionalLength;
			hash.pointLength = pointLength;
			hash.spotLength = spotLength;
			hash.rectAreaLength = rectAreaLength;
			hash.hemiLength = hemiLength;

			hash.numDirectionalShadows = numDirectionalShadows;
			hash.numPointShadows = numPointShadows;
			hash.numSpotShadows = numSpotShadows;

			state.version = nextVersion ++;

		}

	}

	function setupView( lights, camera ) {

		let directionalLength = 0;
		let pointLength = 0;
		let spotLength = 0;
		let rectAreaLength = 0;
		let hemiLength = 0;

		const viewMatrix = camera.matrixWorldInverse;

		for ( let i = 0, l = lights.length; i < l; i ++ ) {

			const light = lights[ i ];

			if ( light.isDirectionalLight ) {

				const uniforms = state.directional[ directionalLength ];

				uniforms.direction.setFromMatrixPosition( light.matrixWorld );
				vector3.setFromMatrixPosition( light.target.matrixWorld );
				uniforms.direction.sub( vector3 );
				uniforms.direction.transformDirection( viewMatrix );

				directionalLength ++;

			} else if ( light.isSpotLight ) {

				const uniforms = state.spot[ spotLength ];

				uniforms.position.setFromMatrixPosition( light.matrixWorld );
				uniforms.position.applyMatrix4( viewMatrix );

				uniforms.direction.setFromMatrixPosition( light.matrixWorld );
				vector3.setFromMatrixPosition( light.target.matrixWorld );
				uniforms.direction.sub( vector3 );
				uniforms.direction.transformDirection( viewMatrix );

				spotLength ++;

			} else if ( light.isRectAreaLight ) {

				const uniforms = state.rectArea[ rectAreaLength ];

				uniforms.position.setFromMatrixPosition( light.matrixWorld );
				uniforms.position.applyMatrix4( viewMatrix );

				// extract local rotation of light to derive width/height half vectors
				matrix42.identity();
				matrix4.copy( light.matrixWorld );
				matrix4.premultiply( viewMatrix );
				matrix42.extractRotation( matrix4 );

				uniforms.halfWidth.set( light.width * 0.5, 0.0, 0.0 );
				uniforms.halfHeight.set( 0.0, light.height * 0.5, 0.0 );

				uniforms.halfWidth.applyMatrix4( matrix42 );
				uniforms.halfHeight.applyMatrix4( matrix42 );

				rectAreaLength ++;

			} else if ( light.isPointLight ) {

				const uniforms = state.point[ pointLength ];

				uniforms.position.setFromMatrixPosition( light.matrixWorld );
				uniforms.position.applyMatrix4( viewMatrix );

				pointLength ++;

			} else if ( light.isHemisphereLight ) {

				const uniforms = state.hemi[ hemiLength ];

				uniforms.direction.setFromMatrixPosition( light.matrixWorld );
				uniforms.direction.transformDirection( viewMatrix );
				uniforms.direction.normalize();

				hemiLength ++;

			}

		}

	}

	return {
		setup: setup,
		setupView: setupView,
		state: state
	};

}

function WebGLRenderState( extensions, capabilities ) {

	const lights = new WebGLLights( extensions, capabilities );

	const lightsArray = [];
	const shadowsArray = [];

	function init() {

		lightsArray.length = 0;
		shadowsArray.length = 0;

	}

	function pushLight( light ) {

		lightsArray.push( light );

	}

	function pushShadow( shadowLight ) {

		shadowsArray.push( shadowLight );

	}

	function setupLights() {

		lights.setup( lightsArray );

	}

	function setupLightsView( camera ) {

		lights.setupView( lightsArray, camera );

	}

	const state = {
		lightsArray: lightsArray,
		shadowsArray: shadowsArray,

		lights: lights
	};

	return {
		init: init,
		state: state,
		setupLights: setupLights,
		setupLightsView: setupLightsView,

		pushLight: pushLight,
		pushShadow: pushShadow
	};

}

function WebGLRenderStates( extensions, capabilities ) {

	let renderStates = new WeakMap();

	function get( scene, renderCallDepth = 0 ) {

		let renderState;

		if ( renderStates.has( scene ) === false ) {

			renderState = new WebGLRenderState( extensions, capabilities );
			renderStates.set( scene, [ renderState ] );

		} else {

			if ( renderCallDepth >= renderStates.get( scene ).length ) {

				renderState = new WebGLRenderState( extensions, capabilities );
				renderStates.get( scene ).push( renderState );

			} else {

				renderState = renderStates.get( scene )[ renderCallDepth ];

			}

		}

		return renderState;

	}

	function dispose() {

		renderStates = new WeakMap();

	}

	return {
		get: get,
		dispose: dispose
	};

}

/**
 * parameters = {
 *
 *  opacity: <float>,
 *
 *  map: new THREE.Texture( <Image> ),
 *
 *  alphaMap: new THREE.Texture( <Image> ),
 *
 *  displacementMap: new THREE.Texture( <Image> ),
 *  displacementScale: <float>,
 *  displacementBias: <float>,
 *
 *  wireframe: <boolean>,
 *  wireframeLinewidth: <float>
 * }
 */

class MeshDepthMaterial extends Material {

	constructor( parameters ) {

		super();

		this.type = 'MeshDepthMaterial';

		this.depthPacking = BasicDepthPacking;

		this.skinning = false;
		this.morphTargets = false;

		this.map = null;

		this.alphaMap = null;

		this.displacementMap = null;
		this.displacementScale = 1;
		this.displacementBias = 0;

		this.wireframe = false;
		this.wireframeLinewidth = 1;

		this.fog = false;

		this.setValues( parameters );

	}

	copy( source ) {

		super.copy( source );

		this.depthPacking = source.depthPacking;

		this.skinning = source.skinning;
		this.morphTargets = source.morphTargets;

		this.map = source.map;

		this.alphaMap = source.alphaMap;

		this.displacementMap = source.displacementMap;
		this.displacementScale = source.displacementScale;
		this.displacementBias = source.displacementBias;

		this.wireframe = source.wireframe;
		this.wireframeLinewidth = source.wireframeLinewidth;

		return this;

	}

}

MeshDepthMaterial.prototype.isMeshDepthMaterial = true;

/**
 * parameters = {
 *
 *  referencePosition: <float>,
 *  nearDistance: <float>,
 *  farDistance: <float>,
 *
 *  skinning: <bool>,
 *  morphTargets: <bool>,
 *
 *  map: new THREE.Texture( <Image> ),
 *
 *  alphaMap: new THREE.Texture( <Image> ),
 *
 *  displacementMap: new THREE.Texture( <Image> ),
 *  displacementScale: <float>,
 *  displacementBias: <float>
 *
 * }
 */

class MeshDistanceMaterial extends Material {

	constructor( parameters ) {

		super();

		this.type = 'MeshDistanceMaterial';

		this.referencePosition = new Vector3();
		this.nearDistance = 1;
		this.farDistance = 1000;

		this.skinning = false;
		this.morphTargets = false;

		this.map = null;

		this.alphaMap = null;

		this.displacementMap = null;
		this.displacementScale = 1;
		this.displacementBias = 0;

		this.fog = false;

		this.setValues( parameters );

	}

	copy( source ) {

		super.copy( source );

		this.referencePosition.copy( source.referencePosition );
		this.nearDistance = source.nearDistance;
		this.farDistance = source.farDistance;

		this.skinning = source.skinning;
		this.morphTargets = source.morphTargets;

		this.map = source.map;

		this.alphaMap = source.alphaMap;

		this.displacementMap = source.displacementMap;
		this.displacementScale = source.displacementScale;
		this.displacementBias = source.displacementBias;

		return this;

	}

}

MeshDistanceMaterial.prototype.isMeshDistanceMaterial = true;

var vsm_frag = "uniform sampler2D shadow_pass;\nuniform vec2 resolution;\nuniform float radius;\n#include <packing>\nvoid main() {\n\tfloat mean = 0.0;\n\tfloat squared_mean = 0.0;\n\tfloat depth = unpackRGBAToDepth( texture2D( shadow_pass, ( gl_FragCoord.xy ) / resolution ) );\n\tfor ( float i = -1.0; i < 1.0 ; i += SAMPLE_RATE) {\n\t\t#ifdef HORIZONTAL_PASS\n\t\t\tvec2 distribution = unpackRGBATo2Half( texture2D( shadow_pass, ( gl_FragCoord.xy + vec2( i, 0.0 ) * radius ) / resolution ) );\n\t\t\tmean += distribution.x;\n\t\t\tsquared_mean += distribution.y * distribution.y + distribution.x * distribution.x;\n\t\t#else\n\t\t\tfloat depth = unpackRGBAToDepth( texture2D( shadow_pass, ( gl_FragCoord.xy + vec2( 0.0, i ) * radius ) / resolution ) );\n\t\t\tmean += depth;\n\t\t\tsquared_mean += depth * depth;\n\t\t#endif\n\t}\n\tmean = mean * HALF_SAMPLE_RATE;\n\tsquared_mean = squared_mean * HALF_SAMPLE_RATE;\n\tfloat std_dev = sqrt( squared_mean - mean * mean );\n\tgl_FragColor = pack2HalfToRGBA( vec2( mean, std_dev ) );\n}";

var vsm_vert = "void main() {\n\tgl_Position = vec4( position, 1.0 );\n}";

function WebGLShadowMap( _renderer, _objects, _capabilities ) {

	let _frustum = new Frustum();

	const _shadowMapSize = new Vector2(),
		_viewportSize = new Vector2(),

		_viewport = new Vector4(),

		_depthMaterials = [],
		_distanceMaterials = [],

		_materialCache = {},

		_maxTextureSize = _capabilities.maxTextureSize;

	const shadowSide = { 0: BackSide, 1: FrontSide, 2: DoubleSide };

	const shadowMaterialVertical = new ShaderMaterial( {

		defines: {
			SAMPLE_RATE: 2.0 / 8.0,
			HALF_SAMPLE_RATE: 1.0 / 8.0
		},

		uniforms: {
			shadow_pass: { value: null },
			resolution: { value: new Vector2() },
			radius: { value: 4.0 }
		},

		vertexShader: vsm_vert,

		fragmentShader: vsm_frag

	} );

	const shadowMaterialHorizontal = shadowMaterialVertical.clone();
	shadowMaterialHorizontal.defines.HORIZONTAL_PASS = 1;

	const fullScreenTri = new BufferGeometry();
	fullScreenTri.setAttribute(
		'position',
		new BufferAttribute(
			new Float32Array( [ - 1, - 1, 0.5, 3, - 1, 0.5, - 1, 3, 0.5 ] ),
			3
		)
	);

	const fullScreenMesh = new Mesh( fullScreenTri, shadowMaterialVertical );

	const scope = this;

	this.enabled = false;

	this.autoUpdate = true;
	this.needsUpdate = false;

	this.type = PCFShadowMap;

	this.render = function ( lights, scene, camera ) {

		if ( scope.enabled === false ) return;
		if ( scope.autoUpdate === false && scope.needsUpdate === false ) return;

		if ( lights.length === 0 ) return;

		const currentRenderTarget = _renderer.getRenderTarget();
		const activeCubeFace = _renderer.getActiveCubeFace();
		const activeMipmapLevel = _renderer.getActiveMipmapLevel();

		const _state = _renderer.state;

		// Set GL state for depth map.
		_state.setBlending( NoBlending );
		_state.buffers.color.setClear( 1, 1, 1, 1 );
		_state.buffers.depth.setTest( true );
		_state.setScissorTest( false );

		// render depth map

		for ( let i = 0, il = lights.length; i < il; i ++ ) {

			const light = lights[ i ];
			const shadow = light.shadow;

			if ( shadow === undefined ) {

				console.warn( 'THREE.WebGLShadowMap:', light, 'has no shadow.' );
				continue;

			}

			if ( shadow.autoUpdate === false && shadow.needsUpdate === false ) continue;

			_shadowMapSize.copy( shadow.mapSize );

			const shadowFrameExtents = shadow.getFrameExtents();

			_shadowMapSize.multiply( shadowFrameExtents );

			_viewportSize.copy( shadow.mapSize );

			if ( _shadowMapSize.x > _maxTextureSize || _shadowMapSize.y > _maxTextureSize ) {

				if ( _shadowMapSize.x > _maxTextureSize ) {

					_viewportSize.x = Math.floor( _maxTextureSize / shadowFrameExtents.x );
					_shadowMapSize.x = _viewportSize.x * shadowFrameExtents.x;
					shadow.mapSize.x = _viewportSize.x;

				}

				if ( _shadowMapSize.y > _maxTextureSize ) {

					_viewportSize.y = Math.floor( _maxTextureSize / shadowFrameExtents.y );
					_shadowMapSize.y = _viewportSize.y * shadowFrameExtents.y;
					shadow.mapSize.y = _viewportSize.y;

				}

			}

			if ( shadow.map === null && ! shadow.isPointLightShadow && this.type === VSMShadowMap ) {

				const pars = { minFilter: LinearFilter, magFilter: LinearFilter, format: RGBAFormat };

				shadow.map = new WebGLRenderTarget( _shadowMapSize.x, _shadowMapSize.y, pars );
				shadow.map.texture.name = light.name + '.shadowMap';

				shadow.mapPass = new WebGLRenderTarget( _shadowMapSize.x, _shadowMapSize.y, pars );

				shadow.camera.updateProjectionMatrix();

			}

			if ( shadow.map === null ) {

				const pars = { minFilter: NearestFilter, magFilter: NearestFilter, format: RGBAFormat };

				shadow.map = new WebGLRenderTarget( _shadowMapSize.x, _shadowMapSize.y, pars );
				shadow.map.texture.name = light.name + '.shadowMap';

				shadow.camera.updateProjectionMatrix();

			}

			_renderer.setRenderTarget( shadow.map );
			_renderer.clear();

			const viewportCount = shadow.getViewportCount();

			for ( let vp = 0; vp < viewportCount; vp ++ ) {

				const viewport = shadow.getViewport( vp );

				_viewport.set(
					_viewportSize.x * viewport.x,
					_viewportSize.y * viewport.y,
					_viewportSize.x * viewport.z,
					_viewportSize.y * viewport.w
				);

				_state.viewport( _viewport );

				shadow.updateMatrices( light, vp );

				_frustum = shadow.getFrustum();

				renderObject( scene, camera, shadow.camera, light, this.type );

			}

			// do blur pass for VSM

			if ( ! shadow.isPointLightShadow && this.type === VSMShadowMap ) {

				VSMPass( shadow, camera );

			}

			shadow.needsUpdate = false;

		}

		scope.needsUpdate = false;

		_renderer.setRenderTarget( currentRenderTarget, activeCubeFace, activeMipmapLevel );

	};

	function VSMPass( shadow, camera ) {

		const geometry = _objects.update( fullScreenMesh );

		// vertical pass

		shadowMaterialVertical.uniforms.shadow_pass.value = shadow.map.texture;
		shadowMaterialVertical.uniforms.resolution.value = shadow.mapSize;
		shadowMaterialVertical.uniforms.radius.value = shadow.radius;
		_renderer.setRenderTarget( shadow.mapPass );
		_renderer.clear();
		_renderer.renderBufferDirect( camera, null, geometry, shadowMaterialVertical, fullScreenMesh, null );

		// horizontal pass

		shadowMaterialHorizontal.uniforms.shadow_pass.value = shadow.mapPass.texture;
		shadowMaterialHorizontal.uniforms.resolution.value = shadow.mapSize;
		shadowMaterialHorizontal.uniforms.radius.value = shadow.radius;
		_renderer.setRenderTarget( shadow.map );
		_renderer.clear();
		_renderer.renderBufferDirect( camera, null, geometry, shadowMaterialHorizontal, fullScreenMesh, null );

	}

	function getDepthMaterialVariant( useMorphing, useSkinning, useInstancing ) {

		const index = useMorphing << 0 | useSkinning << 1 | useInstancing << 2;

		let material = _depthMaterials[ index ];

		if ( material === undefined ) {

			material = new MeshDepthMaterial( {

				depthPacking: RGBADepthPacking,

				morphTargets: useMorphing,
				skinning: useSkinning

			} );

			_depthMaterials[ index ] = material;

		}

		return material;

	}

	function getDistanceMaterialVariant( useMorphing, useSkinning, useInstancing ) {

		const index = useMorphing << 0 | useSkinning << 1 | useInstancing << 2;

		let material = _distanceMaterials[ index ];

		if ( material === undefined ) {

			material = new MeshDistanceMaterial( {

				morphTargets: useMorphing,
				skinning: useSkinning

			} );

			_distanceMaterials[ index ] = material;

		}

		return material;

	}

	function getDepthMaterial( object, geometry, material, light, shadowCameraNear, shadowCameraFar, type ) {

		let result = null;

		let getMaterialVariant = getDepthMaterialVariant;
		let customMaterial = object.customDepthMaterial;

		if ( light.isPointLight === true ) {

			getMaterialVariant = getDistanceMaterialVariant;
			customMaterial = object.customDistanceMaterial;

		}

		if ( customMaterial === undefined ) {

			let useMorphing = false;

			if ( material.morphTargets === true ) {

				useMorphing = geometry.morphAttributes && geometry.morphAttributes.position && geometry.morphAttributes.position.length > 0;

			}

			let useSkinning = false;

			if ( object.isSkinnedMesh === true ) {

				if ( material.skinning === true ) {

					useSkinning = true;

				} else {

					console.warn( 'THREE.WebGLShadowMap: THREE.SkinnedMesh with material.skinning set to false:', object );

				}

			}

			const useInstancing = object.isInstancedMesh === true;

			result = getMaterialVariant( useMorphing, useSkinning, useInstancing );

		} else {

			result = customMaterial;

		}

		if ( _renderer.localClippingEnabled &&
				material.clipShadows === true &&
				material.clippingPlanes.length !== 0 ) {

			// in this case we need a unique material instance reflecting the
			// appropriate state

			const keyA = result.uuid, keyB = material.uuid;

			let materialsForVariant = _materialCache[ keyA ];

			if ( materialsForVariant === undefined ) {

				materialsForVariant = {};
				_materialCache[ keyA ] = materialsForVariant;

			}

			let cachedMaterial = materialsForVariant[ keyB ];

			if ( cachedMaterial === undefined ) {

				cachedMaterial = result.clone();
				materialsForVariant[ keyB ] = cachedMaterial;

			}

			result = cachedMaterial;

		}

		result.visible = material.visible;
		result.wireframe = material.wireframe;

		if ( type === VSMShadowMap ) {

			result.side = ( material.shadowSide !== null ) ? material.shadowSide : material.side;

		} else {

			result.side = ( material.shadowSide !== null ) ? material.shadowSide : shadowSide[ material.side ];

		}

		result.clipShadows = material.clipShadows;
		result.clippingPlanes = material.clippingPlanes;
		result.clipIntersection = material.clipIntersection;

		result.wireframeLinewidth = material.wireframeLinewidth;
		result.linewidth = material.linewidth;

		if ( light.isPointLight === true && result.isMeshDistanceMaterial === true ) {

			result.referencePosition.setFromMatrixPosition( light.matrixWorld );
			result.nearDistance = shadowCameraNear;
			result.farDistance = shadowCameraFar;

		}

		return result;

	}

	function renderObject( object, camera, shadowCamera, light, type ) {

		if ( object.visible === false ) return;

		const visible = object.layers.test( camera.layers );

		if ( visible && ( object.isMesh || object.isLine || object.isPoints ) ) {

			if ( ( object.castShadow || ( object.receiveShadow && type === VSMShadowMap ) ) && ( ! object.frustumCulled || _frustum.intersectsObject( object ) ) ) {

				object.modelViewMatrix.multiplyMatrices( shadowCamera.matrixWorldInverse, object.matrixWorld );

				const geometry = _objects.update( object );
				const material = object.material;

				if ( Array.isArray( material ) ) {

					const groups = geometry.groups;

					for ( let k = 0, kl = groups.length; k < kl; k ++ ) {

						const group = groups[ k ];
						const groupMaterial = material[ group.materialIndex ];

						if ( groupMaterial && groupMaterial.visible ) {

							const depthMaterial = getDepthMaterial( object, geometry, groupMaterial, light, shadowCamera.near, shadowCamera.far, type );

							_renderer.renderBufferDirect( shadowCamera, null, geometry, depthMaterial, object, group );

						}

					}

				} else if ( material.visible ) {

					const depthMaterial = getDepthMaterial( object, geometry, material, light, shadowCamera.near, shadowCamera.far, type );

					_renderer.renderBufferDirect( shadowCamera, null, geometry, depthMaterial, object, null );

				}

			}

		}

		const children = object.children;

		for ( let i = 0, l = children.length; i < l; i ++ ) {

			renderObject( children[ i ], camera, shadowCamera, light, type );

		}

	}

}

function WebGLState( gl, extensions, capabilities ) {

	const isWebGL2 = capabilities.isWebGL2;

	function ColorBuffer() {

		let locked = false;

		const color = new Vector4();
		let currentColorMask = null;
		const currentColorClear = new Vector4( 0, 0, 0, 0 );

		return {

			setMask: function ( colorMask ) {

				if ( currentColorMask !== colorMask && ! locked ) {

					gl.colorMask( colorMask, colorMask, colorMask, colorMask );
					currentColorMask = colorMask;

				}

			},

			setLocked: function ( lock ) {

				locked = lock;

			},

			setClear: function ( r, g, b, a, premultipliedAlpha ) {

				if ( premultipliedAlpha === true ) {

					r *= a; g *= a; b *= a;

				}

				color.set( r, g, b, a );

				if ( currentColorClear.equals( color ) === false ) {

					gl.clearColor( r, g, b, a );
					currentColorClear.copy( color );

				}

			},

			reset: function () {

				locked = false;

				currentColorMask = null;
				currentColorClear.set( - 1, 0, 0, 0 ); // set to invalid state

			}

		};

	}

	function DepthBuffer() {

		let locked = false;

		let currentDepthMask = null;
		let currentDepthFunc = null;
		let currentDepthClear = null;

		return {

			setTest: function ( depthTest ) {

				if ( depthTest ) {

					enable( 2929 );

				} else {

					disable( 2929 );

				}

			},

			setMask: function ( depthMask ) {

				if ( currentDepthMask !== depthMask && ! locked ) {

					gl.depthMask( depthMask );
					currentDepthMask = depthMask;

				}

			},

			setFunc: function ( depthFunc ) {

				if ( currentDepthFunc !== depthFunc ) {

					if ( depthFunc ) {

						switch ( depthFunc ) {

							case NeverDepth:

								gl.depthFunc( 512 );
								break;

							case AlwaysDepth:

								gl.depthFunc( 519 );
								break;

							case LessDepth:

								gl.depthFunc( 513 );
								break;

							case LessEqualDepth:

								gl.depthFunc( 515 );
								break;

							case EqualDepth:

								gl.depthFunc( 514 );
								break;

							case GreaterEqualDepth:

								gl.depthFunc( 518 );
								break;

							case GreaterDepth:

								gl.depthFunc( 516 );
								break;

							case NotEqualDepth:

								gl.depthFunc( 517 );
								break;

							default:

								gl.depthFunc( 515 );

						}

					} else {

						gl.depthFunc( 515 );

					}

					currentDepthFunc = depthFunc;

				}

			},

			setLocked: function ( lock ) {

				locked = lock;

			},

			setClear: function ( depth ) {

				if ( currentDepthClear !== depth ) {

					gl.clearDepth( depth );
					currentDepthClear = depth;

				}

			},

			reset: function () {

				locked = false;

				currentDepthMask = null;
				currentDepthFunc = null;
				currentDepthClear = null;

			}

		};

	}

	function StencilBuffer() {

		let locked = false;

		let currentStencilMask = null;
		let currentStencilFunc = null;
		let currentStencilRef = null;
		let currentStencilFuncMask = null;
		let currentStencilFail = null;
		let currentStencilZFail = null;
		let currentStencilZPass = null;
		let currentStencilClear = null;

		return {

			setTest: function ( stencilTest ) {

				if ( ! locked ) {

					if ( stencilTest ) {

						enable( 2960 );

					} else {

						disable( 2960 );

					}

				}

			},

			setMask: function ( stencilMask ) {

				if ( currentStencilMask !== stencilMask && ! locked ) {

					gl.stencilMask( stencilMask );
					currentStencilMask = stencilMask;

				}

			},

			setFunc: function ( stencilFunc, stencilRef, stencilMask ) {

				if ( currentStencilFunc !== stencilFunc ||
				     currentStencilRef !== stencilRef ||
				     currentStencilFuncMask !== stencilMask ) {

					gl.stencilFunc( stencilFunc, stencilRef, stencilMask );

					currentStencilFunc = stencilFunc;
					currentStencilRef = stencilRef;
					currentStencilFuncMask = stencilMask;

				}

			},

			setOp: function ( stencilFail, stencilZFail, stencilZPass ) {

				if ( currentStencilFail !== stencilFail ||
				     currentStencilZFail !== stencilZFail ||
				     currentStencilZPass !== stencilZPass ) {

					gl.stencilOp( stencilFail, stencilZFail, stencilZPass );

					currentStencilFail = stencilFail;
					currentStencilZFail = stencilZFail;
					currentStencilZPass = stencilZPass;

				}

			},

			setLocked: function ( lock ) {

				locked = lock;

			},

			setClear: function ( stencil ) {

				if ( currentStencilClear !== stencil ) {

					gl.clearStencil( stencil );
					currentStencilClear = stencil;

				}

			},

			reset: function () {

				locked = false;

				currentStencilMask = null;
				currentStencilFunc = null;
				currentStencilRef = null;
				currentStencilFuncMask = null;
				currentStencilFail = null;
				currentStencilZFail = null;
				currentStencilZPass = null;
				currentStencilClear = null;

			}

		};

	}

	//

	const colorBuffer = new ColorBuffer();
	const depthBuffer = new DepthBuffer();
	const stencilBuffer = new StencilBuffer();

	let enabledCapabilities = {};

	let xrFramebuffer = null;
	let currentBoundFramebuffers = {};

	let currentProgram = null;

	let currentBlendingEnabled = false;
	let currentBlending = null;
	let currentBlendEquation = null;
	let currentBlendSrc = null;
	let currentBlendDst = null;
	let currentBlendEquationAlpha = null;
	let currentBlendSrcAlpha = null;
	let currentBlendDstAlpha = null;
	let currentPremultipledAlpha = false;

	let currentFlipSided = null;
	let currentCullFace = null;

	let currentLineWidth = null;

	let currentPolygonOffsetFactor = null;
	let currentPolygonOffsetUnits = null;

	const maxTextures = gl.getParameter( 35661 );

	let lineWidthAvailable = false;
	let version = 0;
	const glVersion = gl.getParameter( 7938 );

	if ( glVersion.indexOf( 'WebGL' ) !== - 1 ) {

		version = parseFloat( /^WebGL (\d)/.exec( glVersion )[ 1 ] );
		lineWidthAvailable = ( version >= 1.0 );

	} else if ( glVersion.indexOf( 'OpenGL ES' ) !== - 1 ) {

		version = parseFloat( /^OpenGL ES (\d)/.exec( glVersion )[ 1 ] );
		lineWidthAvailable = ( version >= 2.0 );

	}

	let currentTextureSlot = null;
	let currentBoundTextures = {};

	const currentScissor = new Vector4( 0, 0, gl.canvas.width, gl.canvas.height );
	const currentViewport = new Vector4( 0, 0, gl.canvas.width, gl.canvas.height );

	function createTexture( type, target, count ) {

		const data = new Uint8Array( 4 ); // 4 is required to match default unpack alignment of 4.
		const texture = gl.createTexture();

		gl.bindTexture( type, texture );
		gl.texParameteri( type, 10241, 9728 );
		gl.texParameteri( type, 10240, 9728 );

		for ( let i = 0; i < count; i ++ ) {

			gl.texImage2D( target + i, 0, 6408, 1, 1, 0, 6408, 5121, data );

		}

		return texture;

	}

	const emptyTextures = {};
	emptyTextures[ 3553 ] = createTexture( 3553, 3553, 1 );
	emptyTextures[ 34067 ] = createTexture( 34067, 34069, 6 );

	// init

	colorBuffer.setClear( 0, 0, 0, 1 );
	depthBuffer.setClear( 1 );
	stencilBuffer.setClear( 0 );

	enable( 2929 );
	depthBuffer.setFunc( LessEqualDepth );

	setFlipSided( false );
	setCullFace( CullFaceBack );
	enable( 2884 );

	setBlending( NoBlending );

	//

	function enable( id ) {

		if ( enabledCapabilities[ id ] !== true ) {

			gl.enable( id );
			enabledCapabilities[ id ] = true;

		}

	}

	function disable( id ) {

		if ( enabledCapabilities[ id ] !== false ) {

			gl.disable( id );
			enabledCapabilities[ id ] = false;

		}

	}

	function bindXRFramebuffer( framebuffer ) {

		if ( framebuffer !== xrFramebuffer ) {

			gl.bindFramebuffer( 36160, framebuffer );

			xrFramebuffer = framebuffer;

		}

	}

	function bindFramebuffer( target, framebuffer ) {

		if ( framebuffer === null && xrFramebuffer !== null ) framebuffer = xrFramebuffer; // use active XR framebuffer if available

		if ( currentBoundFramebuffers[ target ] !== framebuffer ) {

			gl.bindFramebuffer( target, framebuffer );

			currentBoundFramebuffers[ target ] = framebuffer;

		}

	}

	function useProgram( program ) {

		if ( currentProgram !== program ) {

			gl.useProgram( program );

			currentProgram = program;

			return true;

		}

		return false;

	}

	const equationToGL = {
		[ AddEquation ]: 32774,
		[ SubtractEquation ]: 32778,
		[ ReverseSubtractEquation ]: 32779
	};

	if ( isWebGL2 ) {

		equationToGL[ MinEquation ] = 32775;
		equationToGL[ MaxEquation ] = 32776;

	} else {

		const extension = extensions.get( 'EXT_blend_minmax' );

		if ( extension !== null ) {

			equationToGL[ MinEquation ] = extension.MIN_EXT;
			equationToGL[ MaxEquation ] = extension.MAX_EXT;

		}

	}

	const factorToGL = {
		[ ZeroFactor ]: 0,
		[ OneFactor ]: 1,
		[ SrcColorFactor ]: 768,
		[ SrcAlphaFactor ]: 770,
		[ SrcAlphaSaturateFactor ]: 776,
		[ DstColorFactor ]: 774,
		[ DstAlphaFactor ]: 772,
		[ OneMinusSrcColorFactor ]: 769,
		[ OneMinusSrcAlphaFactor ]: 771,
		[ OneMinusDstColorFactor ]: 775,
		[ OneMinusDstAlphaFactor ]: 773
	};

	function setBlending( blending, blendEquation, blendSrc, blendDst, blendEquationAlpha, blendSrcAlpha, blendDstAlpha, premultipliedAlpha ) {

		if ( blending === NoBlending ) {

			if ( currentBlendingEnabled === true ) {

				disable( 3042 );
				currentBlendingEnabled = false;

			}

			return;

		}

		if ( currentBlendingEnabled === false ) {

			enable( 3042 );
			currentBlendingEnabled = true;

		}

		if ( blending !== CustomBlending ) {

			if ( blending !== currentBlending || premultipliedAlpha !== currentPremultipledAlpha ) {

				if ( currentBlendEquation !== AddEquation || currentBlendEquationAlpha !== AddEquation ) {

					gl.blendEquation( 32774 );

					currentBlendEquation = AddEquation;
					currentBlendEquationAlpha = AddEquation;

				}

				if ( premultipliedAlpha ) {

					switch ( blending ) {

						case NormalBlending:
							gl.blendFuncSeparate( 1, 771, 1, 771 );
							break;

						case AdditiveBlending:
							gl.blendFunc( 1, 1 );
							break;

						case SubtractiveBlending:
							gl.blendFuncSeparate( 0, 0, 769, 771 );
							break;

						case MultiplyBlending:
							gl.blendFuncSeparate( 0, 768, 0, 770 );
							break;

						default:
							console.error( 'THREE.WebGLState: Invalid blending: ', blending );
							break;

					}

				} else {

					switch ( blending ) {

						case NormalBlending:
							gl.blendFuncSeparate( 770, 771, 1, 771 );
							break;

						case AdditiveBlending:
							gl.blendFunc( 770, 1 );
							break;

						case SubtractiveBlending:
							gl.blendFunc( 0, 769 );
							break;

						case MultiplyBlending:
							gl.blendFunc( 0, 768 );
							break;

						default:
							console.error( 'THREE.WebGLState: Invalid blending: ', blending );
							break;

					}

				}

				currentBlendSrc = null;
				currentBlendDst = null;
				currentBlendSrcAlpha = null;
				currentBlendDstAlpha = null;

				currentBlending = blending;
				currentPremultipledAlpha = premultipliedAlpha;

			}

			return;

		}

		// custom blending

		blendEquationAlpha = blendEquationAlpha || blendEquation;
		blendSrcAlpha = blendSrcAlpha || blendSrc;
		blendDstAlpha = blendDstAlpha || blendDst;

		if ( blendEquation !== currentBlendEquation || blendEquationAlpha !== currentBlendEquationAlpha ) {

			gl.blendEquationSeparate( equationToGL[ blendEquation ], equationToGL[ blendEquationAlpha ] );

			currentBlendEquation = blendEquation;
			currentBlendEquationAlpha = blendEquationAlpha;

		}

		if ( blendSrc !== currentBlendSrc || blendDst !== currentBlendDst || blendSrcAlpha !== currentBlendSrcAlpha || blendDstAlpha !== currentBlendDstAlpha ) {

			gl.blendFuncSeparate( factorToGL[ blendSrc ], factorToGL[ blendDst ], factorToGL[ blendSrcAlpha ], factorToGL[ blendDstAlpha ] );

			currentBlendSrc = blendSrc;
			currentBlendDst = blendDst;
			currentBlendSrcAlpha = blendSrcAlpha;
			currentBlendDstAlpha = blendDstAlpha;

		}

		currentBlending = blending;
		currentPremultipledAlpha = null;

	}

	function setMaterial( material, frontFaceCW ) {

		material.side === DoubleSide
			? disable( 2884 )
			: enable( 2884 );

		let flipSided = ( material.side === BackSide );
		if ( frontFaceCW ) flipSided = ! flipSided;

		setFlipSided( flipSided );

		( material.blending === NormalBlending && material.transparent === false )
			? setBlending( NoBlending )
			: setBlending( material.blending, material.blendEquation, material.blendSrc, material.blendDst, material.blendEquationAlpha, material.blendSrcAlpha, material.blendDstAlpha, material.premultipliedAlpha );

		depthBuffer.setFunc( material.depthFunc );
		depthBuffer.setTest( material.depthTest );
		depthBuffer.setMask( material.depthWrite );
		colorBuffer.setMask( material.colorWrite );

		const stencilWrite = material.stencilWrite;
		stencilBuffer.setTest( stencilWrite );
		if ( stencilWrite ) {

			stencilBuffer.setMask( material.stencilWriteMask );
			stencilBuffer.setFunc( material.stencilFunc, material.stencilRef, material.stencilFuncMask );
			stencilBuffer.setOp( material.stencilFail, material.stencilZFail, material.stencilZPass );

		}

		setPolygonOffset( material.polygonOffset, material.polygonOffsetFactor, material.polygonOffsetUnits );

		material.alphaToCoverage === true
			? enable( 32926 )
			: disable( 32926 );

	}

	//

	function setFlipSided( flipSided ) {

		if ( currentFlipSided !== flipSided ) {

			if ( flipSided ) {

				gl.frontFace( 2304 );

			} else {

				gl.frontFace( 2305 );

			}

			currentFlipSided = flipSided;

		}

	}

	function setCullFace( cullFace ) {

		if ( cullFace !== CullFaceNone ) {

			enable( 2884 );

			if ( cullFace !== currentCullFace ) {

				if ( cullFace === CullFaceBack ) {

					gl.cullFace( 1029 );

				} else if ( cullFace === CullFaceFront ) {

					gl.cullFace( 1028 );

				} else {

					gl.cullFace( 1032 );

				}

			}

		} else {

			disable( 2884 );

		}

		currentCullFace = cullFace;

	}

	function setLineWidth( width ) {

		if ( width !== currentLineWidth ) {

			if ( lineWidthAvailable ) gl.lineWidth( width );

			currentLineWidth = width;

		}

	}

	function setPolygonOffset( polygonOffset, factor, units ) {

		if ( polygonOffset ) {

			enable( 32823 );

			if ( currentPolygonOffsetFactor !== factor || currentPolygonOffsetUnits !== units ) {

				gl.polygonOffset( factor, units );

				currentPolygonOffsetFactor = factor;
				currentPolygonOffsetUnits = units;

			}

		} else {

			disable( 32823 );

		}

	}

	function setScissorTest( scissorTest ) {

		if ( scissorTest ) {

			enable( 3089 );

		} else {

			disable( 3089 );

		}

	}

	// texture

	function activeTexture( webglSlot ) {

		if ( webglSlot === undefined ) webglSlot = 33984 + maxTextures - 1;

		if ( currentTextureSlot !== webglSlot ) {

			gl.activeTexture( webglSlot );
			currentTextureSlot = webglSlot;

		}

	}

	function bindTexture( webglType, webglTexture ) {

		if ( currentTextureSlot === null ) {

			activeTexture();

		}

		let boundTexture = currentBoundTextures[ currentTextureSlot ];

		if ( boundTexture === undefined ) {

			boundTexture = { type: undefined, texture: undefined };
			currentBoundTextures[ currentTextureSlot ] = boundTexture;

		}

		if ( boundTexture.type !== webglType || boundTexture.texture !== webglTexture ) {

			gl.bindTexture( webglType, webglTexture || emptyTextures[ webglType ] );

			boundTexture.type = webglType;
			boundTexture.texture = webglTexture;

		}

	}

	function unbindTexture() {

		const boundTexture = currentBoundTextures[ currentTextureSlot ];

		if ( boundTexture !== undefined && boundTexture.type !== undefined ) {

			gl.bindTexture( boundTexture.type, null );

			boundTexture.type = undefined;
			boundTexture.texture = undefined;

		}

	}

	function compressedTexImage2D() {

		try {

			gl.compressedTexImage2D.apply( gl, arguments );

		} catch ( error ) {

			console.error( 'THREE.WebGLState:', error );

		}

	}

	function texImage2D() {

		try {

			gl.texImage2D.apply( gl, arguments );

		} catch ( error ) {

			console.error( 'THREE.WebGLState:', error );

		}

	}

	function texImage3D() {

		try {

			gl.texImage3D.apply( gl, arguments );

		} catch ( error ) {

			console.error( 'THREE.WebGLState:', error );

		}

	}

	//

	function scissor( scissor ) {

		if ( currentScissor.equals( scissor ) === false ) {

			gl.scissor( scissor.x, scissor.y, scissor.z, scissor.w );
			currentScissor.copy( scissor );

		}

	}

	function viewport( viewport ) {

		if ( currentViewport.equals( viewport ) === false ) {

			gl.viewport( viewport.x, viewport.y, viewport.z, viewport.w );
			currentViewport.copy( viewport );

		}

	}

	//

	function reset() {

		// reset state

		gl.disable( 3042 );
		gl.disable( 2884 );
		gl.disable( 2929 );
		gl.disable( 32823 );
		gl.disable( 3089 );
		gl.disable( 2960 );
		gl.disable( 32926 );

		gl.blendEquation( 32774 );
		gl.blendFunc( 1, 0 );
		gl.blendFuncSeparate( 1, 0, 1, 0 );

		gl.colorMask( true, true, true, true );
		gl.clearColor( 0, 0, 0, 0 );

		gl.depthMask( true );
		gl.depthFunc( 513 );
		gl.clearDepth( 1 );

		gl.stencilMask( 0xffffffff );
		gl.stencilFunc( 519, 0, 0xffffffff );
		gl.stencilOp( 7680, 7680, 7680 );
		gl.clearStencil( 0 );

		gl.cullFace( 1029 );
		gl.frontFace( 2305 );

		gl.polygonOffset( 0, 0 );

		gl.activeTexture( 33984 );

		gl.bindFramebuffer( 36160, null );

		if ( isWebGL2 === true ) {

			gl.bindFramebuffer( 36009, null );
			gl.bindFramebuffer( 36008, null );

		}

		gl.useProgram( null );

		gl.lineWidth( 1 );

		gl.scissor( 0, 0, gl.canvas.width, gl.canvas.height );
		gl.viewport( 0, 0, gl.canvas.width, gl.canvas.height );

		// reset internals

		enabledCapabilities = {};

		currentTextureSlot = null;
		currentBoundTextures = {};

		xrFramebuffer = null;
		currentBoundFramebuffers = {};

		currentProgram = null;

		currentBlendingEnabled = false;
		currentBlending = null;
		currentBlendEquation = null;
		currentBlendSrc = null;
		currentBlendDst = null;
		currentBlendEquationAlpha = null;
		currentBlendSrcAlpha = null;
		currentBlendDstAlpha = null;
		currentPremultipledAlpha = false;

		currentFlipSided = null;
		currentCullFace = null;

		currentLineWidth = null;

		currentPolygonOffsetFactor = null;
		currentPolygonOffsetUnits = null;

		currentScissor.set( 0, 0, gl.canvas.width, gl.canvas.height );
		currentViewport.set( 0, 0, gl.canvas.width, gl.canvas.height );

		colorBuffer.reset();
		depthBuffer.reset();
		stencilBuffer.reset();

	}

	return {

		buffers: {
			color: colorBuffer,
			depth: depthBuffer,
			stencil: stencilBuffer
		},

		enable: enable,
		disable: disable,

		bindFramebuffer: bindFramebuffer,
		bindXRFramebuffer: bindXRFramebuffer,

		useProgram: useProgram,

		setBlending: setBlending,
		setMaterial: setMaterial,

		setFlipSided: setFlipSided,
		setCullFace: setCullFace,

		setLineWidth: setLineWidth,
		setPolygonOffset: setPolygonOffset,

		setScissorTest: setScissorTest,

		activeTexture: activeTexture,
		bindTexture: bindTexture,
		unbindTexture: unbindTexture,
		compressedTexImage2D: compressedTexImage2D,
		texImage2D: texImage2D,
		texImage3D: texImage3D,

		scissor: scissor,
		viewport: viewport,

		reset: reset

	};

}

function WebGLTextures( _gl, extensions, state, properties, capabilities, utils, info ) {

	const isWebGL2 = capabilities.isWebGL2;
	const maxTextures = capabilities.maxTextures;
	const maxCubemapSize = capabilities.maxCubemapSize;
	const maxTextureSize = capabilities.maxTextureSize;
	const maxSamples = capabilities.maxSamples;

	const _videoTextures = new WeakMap();
	let _canvas;

	// cordova iOS (as of 5.0) still uses UIWebView, which provides OffscreenCanvas,
	// also OffscreenCanvas.getContext("webgl"), but not OffscreenCanvas.getContext("2d")!
	// Some implementations may only implement OffscreenCanvas partially (e.g. lacking 2d).

	let useOffscreenCanvas = false;

	try {

		useOffscreenCanvas = typeof OffscreenCanvas !== 'undefined'
			&& ( new OffscreenCanvas( 1, 1 ).getContext( '2d' ) ) !== null;

	} catch ( err ) {

		// Ignore any errors

	}

	function createCanvas( width, height ) {

		// Use OffscreenCanvas when available. Specially needed in web workers

		return useOffscreenCanvas ?
			new OffscreenCanvas( width, height ) :
			document.createElementNS( 'http://www.w3.org/1999/xhtml', 'canvas' );

	}

	function resizeImage( image, needsPowerOfTwo, needsNewCanvas, maxSize ) {

		let scale = 1;

		// handle case if texture exceeds max size

		if ( image.width > maxSize || image.height > maxSize ) {

			scale = maxSize / Math.max( image.width, image.height );

		}

		// only perform resize if necessary

		if ( scale < 1 || needsPowerOfTwo === true ) {

			// only perform resize for certain image types

			if ( ( typeof HTMLImageElement !== 'undefined' && image instanceof HTMLImageElement ) ||
				( typeof HTMLCanvasElement !== 'undefined' && image instanceof HTMLCanvasElement ) ||
				( typeof ImageBitmap !== 'undefined' && image instanceof ImageBitmap ) ) {

				const floor = needsPowerOfTwo ? MathUtils.floorPowerOfTwo : Math.floor;

				const width = floor( scale * image.width );
				const height = floor( scale * image.height );

				if ( _canvas === undefined ) _canvas = createCanvas( width, height );

				// cube textures can't reuse the same canvas

				const canvas = needsNewCanvas ? createCanvas( width, height ) : _canvas;

				canvas.width = width;
				canvas.height = height;

				const context = canvas.getContext( '2d' );
				context.drawImage( image, 0, 0, width, height );

				console.warn( 'THREE.WebGLRenderer: Texture has been resized from (' + image.width + 'x' + image.height + ') to (' + width + 'x' + height + ').' );

				return canvas;

			} else {

				if ( 'data' in image ) {

					console.warn( 'THREE.WebGLRenderer: Image in DataTexture is too big (' + image.width + 'x' + image.height + ').' );

				}

				return image;

			}

		}

		return image;

	}

	function isPowerOfTwo( image ) {

		return MathUtils.isPowerOfTwo( image.width ) && MathUtils.isPowerOfTwo( image.height );

	}

	function textureNeedsPowerOfTwo( texture ) {

		if ( isWebGL2 ) return false;

		return ( texture.wrapS !== ClampToEdgeWrapping || texture.wrapT !== ClampToEdgeWrapping ) ||
			( texture.minFilter !== NearestFilter && texture.minFilter !== LinearFilter );

	}

	function textureNeedsGenerateMipmaps( texture, supportsMips ) {

		return texture.generateMipmaps && supportsMips &&
			texture.minFilter !== NearestFilter && texture.minFilter !== LinearFilter;

	}

	function generateMipmap( target, texture, width, height ) {

		_gl.generateMipmap( target );

		const textureProperties = properties.get( texture );

		textureProperties.__maxMipLevel = Math.log2( Math.max( width, height ) );

	}

	function getInternalFormat( internalFormatName, glFormat, glType ) {

		if ( isWebGL2 === false ) return glFormat;

		if ( internalFormatName !== null ) {

			if ( _gl[ internalFormatName ] !== undefined ) return _gl[ internalFormatName ];

			console.warn( 'THREE.WebGLRenderer: Attempt to use non-existing WebGL internal format \'' + internalFormatName + '\'' );

		}

		let internalFormat = glFormat;

		if ( glFormat === 6403 ) {

			if ( glType === 5126 ) internalFormat = 33326;
			if ( glType === 5131 ) internalFormat = 33325;
			if ( glType === 5121 ) internalFormat = 33321;

		}

		if ( glFormat === 6407 ) {

			if ( glType === 5126 ) internalFormat = 34837;
			if ( glType === 5131 ) internalFormat = 34843;
			if ( glType === 5121 ) internalFormat = 32849;

		}

		if ( glFormat === 6408 ) {

			if ( glType === 5126 ) internalFormat = 34836;
			if ( glType === 5131 ) internalFormat = 34842;
			if ( glType === 5121 ) internalFormat = 32856;

		}

		if ( internalFormat === 33325 || internalFormat === 33326 ||
			internalFormat === 34842 || internalFormat === 34836 ) {

			extensions.get( 'EXT_color_buffer_float' );

		}

		return internalFormat;

	}

	// Fallback filters for non-power-of-2 textures

	function filterFallback( f ) {

		if ( f === NearestFilter || f === NearestMipmapNearestFilter || f === NearestMipmapLinearFilter ) {

			return 9728;

		}

		return 9729;

	}

	//

	function onTextureDispose( event ) {

		const texture = event.target;

		texture.removeEventListener( 'dispose', onTextureDispose );

		deallocateTexture( texture );

		if ( texture.isVideoTexture ) {

			_videoTextures.delete( texture );

		}

		info.memory.textures --;

	}

	function onRenderTargetDispose( event ) {

		const renderTarget = event.target;

		renderTarget.removeEventListener( 'dispose', onRenderTargetDispose );

		deallocateRenderTarget( renderTarget );

		info.memory.textures --;

	}

	//

	function deallocateTexture( texture ) {

		const textureProperties = properties.get( texture );

		if ( textureProperties.__webglInit === undefined ) return;

		_gl.deleteTexture( textureProperties.__webglTexture );

		properties.remove( texture );

	}

	function deallocateRenderTarget( renderTarget ) {

		const texture = renderTarget.texture;

		const renderTargetProperties = properties.get( renderTarget );
		const textureProperties = properties.get( texture );

		if ( ! renderTarget ) return;

		if ( textureProperties.__webglTexture !== undefined ) {

			_gl.deleteTexture( textureProperties.__webglTexture );

		}

		if ( renderTarget.depthTexture ) {

			renderTarget.depthTexture.dispose();

		}

		if ( renderTarget.isWebGLCubeRenderTarget ) {

			for ( let i = 0; i < 6; i ++ ) {

				_gl.deleteFramebuffer( renderTargetProperties.__webglFramebuffer[ i ] );
				if ( renderTargetProperties.__webglDepthbuffer ) _gl.deleteRenderbuffer( renderTargetProperties.__webglDepthbuffer[ i ] );

			}

		} else {

			_gl.deleteFramebuffer( renderTargetProperties.__webglFramebuffer );
			if ( renderTargetProperties.__webglDepthbuffer ) _gl.deleteRenderbuffer( renderTargetProperties.__webglDepthbuffer );
			if ( renderTargetProperties.__webglMultisampledFramebuffer ) _gl.deleteFramebuffer( renderTargetProperties.__webglMultisampledFramebuffer );
			if ( renderTargetProperties.__webglColorRenderbuffer ) _gl.deleteRenderbuffer( renderTargetProperties.__webglColorRenderbuffer );
			if ( renderTargetProperties.__webglDepthRenderbuffer ) _gl.deleteRenderbuffer( renderTargetProperties.__webglDepthRenderbuffer );

		}

		properties.remove( texture );
		properties.remove( renderTarget );

	}

	//

	let textureUnits = 0;

	function resetTextureUnits() {

		textureUnits = 0;

	}

	function allocateTextureUnit() {

		const textureUnit = textureUnits;

		if ( textureUnit >= maxTextures ) {

			console.warn( 'THREE.WebGLTextures: Trying to use ' + textureUnit + ' texture units while this GPU supports only ' + maxTextures );

		}

		textureUnits += 1;

		return textureUnit;

	}

	//

	function setTexture2D( texture, slot ) {

		const textureProperties = properties.get( texture );

		if ( texture.isVideoTexture ) updateVideoTexture( texture );

		if ( texture.version > 0 && textureProperties.__version !== texture.version ) {

			const image = texture.image;

			if ( image === undefined ) {

				console.warn( 'THREE.WebGLRenderer: Texture marked for update but image is undefined' );

			} else if ( image.complete === false ) {

				console.warn( 'THREE.WebGLRenderer: Texture marked for update but image is incomplete' );

			} else {

				uploadTexture( textureProperties, texture, slot );
				return;

			}

		}

		state.activeTexture( 33984 + slot );
		state.bindTexture( 3553, textureProperties.__webglTexture );

	}

	function setTexture2DArray( texture, slot ) {

		const textureProperties = properties.get( texture );

		if ( texture.version > 0 && textureProperties.__version !== texture.version ) {

			uploadTexture( textureProperties, texture, slot );
			return;

		}

		state.activeTexture( 33984 + slot );
		state.bindTexture( 35866, textureProperties.__webglTexture );

	}

	function setTexture3D( texture, slot ) {

		const textureProperties = properties.get( texture );

		if ( texture.version > 0 && textureProperties.__version !== texture.version ) {

			uploadTexture( textureProperties, texture, slot );
			return;

		}

		state.activeTexture( 33984 + slot );
		state.bindTexture( 32879, textureProperties.__webglTexture );

	}

	function setTextureCube( texture, slot ) {

		const textureProperties = properties.get( texture );

		if ( texture.version > 0 && textureProperties.__version !== texture.version ) {

			uploadCubeTexture( textureProperties, texture, slot );
			return;

		}

		state.activeTexture( 33984 + slot );
		state.bindTexture( 34067, textureProperties.__webglTexture );

	}

	const wrappingToGL = {
		[ RepeatWrapping ]: 10497,
		[ ClampToEdgeWrapping ]: 33071,
		[ MirroredRepeatWrapping ]: 33648
	};

	const filterToGL = {
		[ NearestFilter ]: 9728,
		[ NearestMipmapNearestFilter ]: 9984,
		[ NearestMipmapLinearFilter ]: 9986,

		[ LinearFilter ]: 9729,
		[ LinearMipmapNearestFilter ]: 9985,
		[ LinearMipmapLinearFilter ]: 9987
	};

	function setTextureParameters( textureType, texture, supportsMips ) {

		if ( supportsMips ) {

			_gl.texParameteri( textureType, 10242, wrappingToGL[ texture.wrapS ] );
			_gl.texParameteri( textureType, 10243, wrappingToGL[ texture.wrapT ] );

			if ( textureType === 32879 || textureType === 35866 ) {

				_gl.texParameteri( textureType, 32882, wrappingToGL[ texture.wrapR ] );

			}

			_gl.texParameteri( textureType, 10240, filterToGL[ texture.magFilter ] );
			_gl.texParameteri( textureType, 10241, filterToGL[ texture.minFilter ] );

		} else {

			_gl.texParameteri( textureType, 10242, 33071 );
			_gl.texParameteri( textureType, 10243, 33071 );

			if ( textureType === 32879 || textureType === 35866 ) {

				_gl.texParameteri( textureType, 32882, 33071 );

			}

			if ( texture.wrapS !== ClampToEdgeWrapping || texture.wrapT !== ClampToEdgeWrapping ) {

				console.warn( 'THREE.WebGLRenderer: Texture is not power of two. Texture.wrapS and Texture.wrapT should be set to THREE.ClampToEdgeWrapping.' );

			}

			_gl.texParameteri( textureType, 10240, filterFallback( texture.magFilter ) );
			_gl.texParameteri( textureType, 10241, filterFallback( texture.minFilter ) );

			if ( texture.minFilter !== NearestFilter && texture.minFilter !== LinearFilter ) {

				console.warn( 'THREE.WebGLRenderer: Texture is not power of two. Texture.minFilter should be set to THREE.NearestFilter or THREE.LinearFilter.' );

			}

		}

		if ( extensions.has( 'EXT_texture_filter_anisotropic' ) === true ) {

			const extension = extensions.get( 'EXT_texture_filter_anisotropic' );

			if ( texture.type === FloatType && extensions.has( 'OES_texture_float_linear' ) === false ) return; // verify extension for WebGL 1 and WebGL 2
			if ( isWebGL2 === false && ( texture.type === HalfFloatType && extensions.has( 'OES_texture_half_float_linear' ) === false ) ) return; // verify extension for WebGL 1 only

			if ( texture.anisotropy > 1 || properties.get( texture ).__currentAnisotropy ) {

				_gl.texParameterf( textureType, extension.TEXTURE_MAX_ANISOTROPY_EXT, Math.min( texture.anisotropy, capabilities.getMaxAnisotropy() ) );
				properties.get( texture ).__currentAnisotropy = texture.anisotropy;

			}

		}

	}

	function initTexture( textureProperties, texture ) {

		if ( textureProperties.__webglInit === undefined ) {

			textureProperties.__webglInit = true;

			texture.addEventListener( 'dispose', onTextureDispose );

			textureProperties.__webglTexture = _gl.createTexture();

			info.memory.textures ++;

		}

	}

	function uploadTexture( textureProperties, texture, slot ) {

		let textureType = 3553;

		if ( texture.isDataTexture2DArray ) textureType = 35866;
		if ( texture.isDataTexture3D ) textureType = 32879;

		initTexture( textureProperties, texture );

		state.activeTexture( 33984 + slot );
		state.bindTexture( textureType, textureProperties.__webglTexture );

		_gl.pixelStorei( 37440, texture.flipY );
		_gl.pixelStorei( 37441, texture.premultiplyAlpha );
		_gl.pixelStorei( 3317, texture.unpackAlignment );
		_gl.pixelStorei( 37443, 0 );

		const needsPowerOfTwo = textureNeedsPowerOfTwo( texture ) && isPowerOfTwo( texture.image ) === false;
		const image = resizeImage( texture.image, needsPowerOfTwo, false, maxTextureSize );

		const supportsMips = isPowerOfTwo( image ) || isWebGL2,
			glFormat = utils.convert( texture.format );

		let glType = utils.convert( texture.type ),
			glInternalFormat = getInternalFormat( texture.internalFormat, glFormat, glType );

		setTextureParameters( textureType, texture, supportsMips );

		let mipmap;
		const mipmaps = texture.mipmaps;

		if ( texture.isDepthTexture ) {

			// populate depth texture with dummy data

			glInternalFormat = 6402;

			if ( isWebGL2 ) {

				if ( texture.type === FloatType ) {

					glInternalFormat = 36012;

				} else if ( texture.type === UnsignedIntType ) {

					glInternalFormat = 33190;

				} else if ( texture.type === UnsignedInt248Type ) {

					glInternalFormat = 35056;

				} else {

					glInternalFormat = 33189; // WebGL2 requires sized internalformat for glTexImage2D

				}

			} else {

				if ( texture.type === FloatType ) {

					console.error( 'WebGLRenderer: Floating point depth texture requires WebGL2.' );

				}

			}

			// validation checks for WebGL 1

			if ( texture.format === DepthFormat && glInternalFormat === 6402 ) {

				// The error INVALID_OPERATION is generated by texImage2D if format and internalformat are
				// DEPTH_COMPONENT and type is not UNSIGNED_SHORT or UNSIGNED_INT
				// (https://www.khronos.org/registry/webgl/extensions/WEBGL_depth_texture/)
				if ( texture.type !== UnsignedShortType && texture.type !== UnsignedIntType ) {

					console.warn( 'THREE.WebGLRenderer: Use UnsignedShortType or UnsignedIntType for DepthFormat DepthTexture.' );

					texture.type = UnsignedShortType;
					glType = utils.convert( texture.type );

				}

			}

			if ( texture.format === DepthStencilFormat && glInternalFormat === 6402 ) {

				// Depth stencil textures need the DEPTH_STENCIL internal format
				// (https://www.khronos.org/registry/webgl/extensions/WEBGL_depth_texture/)
				glInternalFormat = 34041;

				// The error INVALID_OPERATION is generated by texImage2D if format and internalformat are
				// DEPTH_STENCIL and type is not UNSIGNED_INT_24_8_WEBGL.
				// (https://www.khronos.org/registry/webgl/extensions/WEBGL_depth_texture/)
				if ( texture.type !== UnsignedInt248Type ) {

					console.warn( 'THREE.WebGLRenderer: Use UnsignedInt248Type for DepthStencilFormat DepthTexture.' );

					texture.type = UnsignedInt248Type;
					glType = utils.convert( texture.type );

				}

			}

			//

			state.texImage2D( 3553, 0, glInternalFormat, image.width, image.height, 0, glFormat, glType, null );

		} else if ( texture.isDataTexture ) {

			// use manually created mipmaps if available
			// if there are no manual mipmaps
			// set 0 level mipmap and then use GL to generate other mipmap levels

			if ( mipmaps.length > 0 && supportsMips ) {

				for ( let i = 0, il = mipmaps.length; i < il; i ++ ) {

					mipmap = mipmaps[ i ];
					state.texImage2D( 3553, i, glInternalFormat, mipmap.width, mipmap.height, 0, glFormat, glType, mipmap.data );

				}

				texture.generateMipmaps = false;
				textureProperties.__maxMipLevel = mipmaps.length - 1;

			} else {

				state.texImage2D( 3553, 0, glInternalFormat, image.width, image.height, 0, glFormat, glType, image.data );
				textureProperties.__maxMipLevel = 0;

			}

		} else if ( texture.isCompressedTexture ) {

			for ( let i = 0, il = mipmaps.length; i < il; i ++ ) {

				mipmap = mipmaps[ i ];

				if ( texture.format !== RGBAFormat && texture.format !== RGBFormat ) {

					if ( glFormat !== null ) {

						state.compressedTexImage2D( 3553, i, glInternalFormat, mipmap.width, mipmap.height, 0, mipmap.data );

					} else {

						console.warn( 'THREE.WebGLRenderer: Attempt to load unsupported compressed texture format in .uploadTexture()' );

					}

				} else {

					state.texImage2D( 3553, i, glInternalFormat, mipmap.width, mipmap.height, 0, glFormat, glType, mipmap.data );

				}

			}

			textureProperties.__maxMipLevel = mipmaps.length - 1;

		} else if ( texture.isDataTexture2DArray ) {

			state.texImage3D( 35866, 0, glInternalFormat, image.width, image.height, image.depth, 0, glFormat, glType, image.data );
			textureProperties.__maxMipLevel = 0;

		} else if ( texture.isDataTexture3D ) {

			state.texImage3D( 32879, 0, glInternalFormat, image.width, image.height, image.depth, 0, glFormat, glType, image.data );
			textureProperties.__maxMipLevel = 0;

		} else {

			// regular Texture (image, video, canvas)

			// use manually created mipmaps if available
			// if there are no manual mipmaps
			// set 0 level mipmap and then use GL to generate other mipmap levels

			if ( mipmaps.length > 0 && supportsMips ) {

				for ( let i = 0, il = mipmaps.length; i < il; i ++ ) {

					mipmap = mipmaps[ i ];
					state.texImage2D( 3553, i, glInternalFormat, glFormat, glType, mipmap );

				}

				texture.generateMipmaps = false;
				textureProperties.__maxMipLevel = mipmaps.length - 1;

			} else {

				state.texImage2D( 3553, 0, glInternalFormat, glFormat, glType, image );
				textureProperties.__maxMipLevel = 0;

			}

		}

		if ( textureNeedsGenerateMipmaps( texture, supportsMips ) ) {

			generateMipmap( textureType, texture, image.width, image.height );

		}

		textureProperties.__version = texture.version;

		if ( texture.onUpdate ) texture.onUpdate( texture );

	}

	function uploadCubeTexture( textureProperties, texture, slot ) {

		if ( texture.image.length !== 6 ) return;

		initTexture( textureProperties, texture );

		state.activeTexture( 33984 + slot );
		state.bindTexture( 34067, textureProperties.__webglTexture );

		_gl.pixelStorei( 37440, texture.flipY );
		_gl.pixelStorei( 37441, texture.premultiplyAlpha );
		_gl.pixelStorei( 3317, texture.unpackAlignment );
		_gl.pixelStorei( 37443, 0 );

		const isCompressed = ( texture && ( texture.isCompressedTexture || texture.image[ 0 ].isCompressedTexture ) );
		const isDataTexture = ( texture.image[ 0 ] && texture.image[ 0 ].isDataTexture );

		const cubeImage = [];

		for ( let i = 0; i < 6; i ++ ) {

			if ( ! isCompressed && ! isDataTexture ) {

				cubeImage[ i ] = resizeImage( texture.image[ i ], false, true, maxCubemapSize );

			} else {

				cubeImage[ i ] = isDataTexture ? texture.image[ i ].image : texture.image[ i ];

			}

		}

		const image = cubeImage[ 0 ],
			supportsMips = isPowerOfTwo( image ) || isWebGL2,
			glFormat = utils.convert( texture.format ),
			glType = utils.convert( texture.type ),
			glInternalFormat = getInternalFormat( texture.internalFormat, glFormat, glType );

		setTextureParameters( 34067, texture, supportsMips );

		let mipmaps;

		if ( isCompressed ) {

			for ( let i = 0; i < 6; i ++ ) {

				mipmaps = cubeImage[ i ].mipmaps;

				for ( let j = 0; j < mipmaps.length; j ++ ) {

					const mipmap = mipmaps[ j ];

					if ( texture.format !== RGBAFormat && texture.format !== RGBFormat ) {

						if ( glFormat !== null ) {

							state.compressedTexImage2D( 34069 + i, j, glInternalFormat, mipmap.width, mipmap.height, 0, mipmap.data );

						} else {

							console.warn( 'THREE.WebGLRenderer: Attempt to load unsupported compressed texture format in .setTextureCube()' );

						}

					} else {

						state.texImage2D( 34069 + i, j, glInternalFormat, mipmap.width, mipmap.height, 0, glFormat, glType, mipmap.data );

					}

				}

			}

			textureProperties.__maxMipLevel = mipmaps.length - 1;

		} else {

			mipmaps = texture.mipmaps;

			for ( let i = 0; i < 6; i ++ ) {

				if ( isDataTexture ) {

					state.texImage2D( 34069 + i, 0, glInternalFormat, cubeImage[ i ].width, cubeImage[ i ].height, 0, glFormat, glType, cubeImage[ i ].data );

					for ( let j = 0; j < mipmaps.length; j ++ ) {

						const mipmap = mipmaps[ j ];
						const mipmapImage = mipmap.image[ i ].image;

						state.texImage2D( 34069 + i, j + 1, glInternalFormat, mipmapImage.width, mipmapImage.height, 0, glFormat, glType, mipmapImage.data );

					}

				} else {

					state.texImage2D( 34069 + i, 0, glInternalFormat, glFormat, glType, cubeImage[ i ] );

					for ( let j = 0; j < mipmaps.length; j ++ ) {

						const mipmap = mipmaps[ j ];

						state.texImage2D( 34069 + i, j + 1, glInternalFormat, glFormat, glType, mipmap.image[ i ] );

					}

				}

			}

			textureProperties.__maxMipLevel = mipmaps.length;

		}

		if ( textureNeedsGenerateMipmaps( texture, supportsMips ) ) {

			// We assume images for cube map have the same size.
			generateMipmap( 34067, texture, image.width, image.height );

		}

		textureProperties.__version = texture.version;

		if ( texture.onUpdate ) texture.onUpdate( texture );

	}

	// Render targets

	// Setup storage for target texture and bind it to correct framebuffer
	function setupFrameBufferTexture( framebuffer, renderTarget, attachment, textureTarget ) {

		const texture = renderTarget.texture;

		const glFormat = utils.convert( texture.format );
		const glType = utils.convert( texture.type );
		const glInternalFormat = getInternalFormat( texture.internalFormat, glFormat, glType );

		if ( textureTarget === 32879 || textureTarget === 35866 ) {

			state.texImage3D( textureTarget, 0, glInternalFormat, renderTarget.width, renderTarget.height, renderTarget.depth, 0, glFormat, glType, null );

		} else {

			state.texImage2D( textureTarget, 0, glInternalFormat, renderTarget.width, renderTarget.height, 0, glFormat, glType, null );

		}

		state.bindFramebuffer( 36160, framebuffer );
		_gl.framebufferTexture2D( 36160, attachment, textureTarget, properties.get( texture ).__webglTexture, 0 );
		state.bindFramebuffer( 36160, null );

	}

	// Setup storage for internal depth/stencil buffers and bind to correct framebuffer
	function setupRenderBufferStorage( renderbuffer, renderTarget, isMultisample ) {

		_gl.bindRenderbuffer( 36161, renderbuffer );

		if ( renderTarget.depthBuffer && ! renderTarget.stencilBuffer ) {

			let glInternalFormat = 33189;

			if ( isMultisample ) {

				const depthTexture = renderTarget.depthTexture;

				if ( depthTexture && depthTexture.isDepthTexture ) {

					if ( depthTexture.type === FloatType ) {

						glInternalFormat = 36012;

					} else if ( depthTexture.type === UnsignedIntType ) {

						glInternalFormat = 33190;

					}

				}

				const samples = getRenderTargetSamples( renderTarget );

				_gl.renderbufferStorageMultisample( 36161, samples, glInternalFormat, renderTarget.width, renderTarget.height );

			} else {

				_gl.renderbufferStorage( 36161, glInternalFormat, renderTarget.width, renderTarget.height );

			}

			_gl.framebufferRenderbuffer( 36160, 36096, 36161, renderbuffer );

		} else if ( renderTarget.depthBuffer && renderTarget.stencilBuffer ) {

			if ( isMultisample ) {

				const samples = getRenderTargetSamples( renderTarget );

				_gl.renderbufferStorageMultisample( 36161, samples, 35056, renderTarget.width, renderTarget.height );

			} else {

				_gl.renderbufferStorage( 36161, 34041, renderTarget.width, renderTarget.height );

			}


			_gl.framebufferRenderbuffer( 36160, 33306, 36161, renderbuffer );

		} else {

			const texture = renderTarget.texture;

			const glFormat = utils.convert( texture.format );
			const glType = utils.convert( texture.type );
			const glInternalFormat = getInternalFormat( texture.internalFormat, glFormat, glType );

			if ( isMultisample ) {

				const samples = getRenderTargetSamples( renderTarget );

				_gl.renderbufferStorageMultisample( 36161, samples, glInternalFormat, renderTarget.width, renderTarget.height );

			} else {

				_gl.renderbufferStorage( 36161, glInternalFormat, renderTarget.width, renderTarget.height );

			}

		}

		_gl.bindRenderbuffer( 36161, null );

	}

	// Setup resources for a Depth Texture for a FBO (needs an extension)
	function setupDepthTexture( framebuffer, renderTarget ) {

		const isCube = ( renderTarget && renderTarget.isWebGLCubeRenderTarget );
		if ( isCube ) throw new Error( 'Depth Texture with cube render targets is not supported' );

		state.bindFramebuffer( 36160, framebuffer );

		if ( ! ( renderTarget.depthTexture && renderTarget.depthTexture.isDepthTexture ) ) {

			throw new Error( 'renderTarget.depthTexture must be an instance of THREE.DepthTexture' );

		}

		// upload an empty depth texture with framebuffer size
		if ( ! properties.get( renderTarget.depthTexture ).__webglTexture ||
				renderTarget.depthTexture.image.width !== renderTarget.width ||
				renderTarget.depthTexture.image.height !== renderTarget.height ) {

			renderTarget.depthTexture.image.width = renderTarget.width;
			renderTarget.depthTexture.image.height = renderTarget.height;
			renderTarget.depthTexture.needsUpdate = true;

		}

		setTexture2D( renderTarget.depthTexture, 0 );

		const webglDepthTexture = properties.get( renderTarget.depthTexture ).__webglTexture;

		if ( renderTarget.depthTexture.format === DepthFormat ) {

			_gl.framebufferTexture2D( 36160, 36096, 3553, webglDepthTexture, 0 );

		} else if ( renderTarget.depthTexture.format === DepthStencilFormat ) {

			_gl.framebufferTexture2D( 36160, 33306, 3553, webglDepthTexture, 0 );

		} else {

			throw new Error( 'Unknown depthTexture format' );

		}

	}

	// Setup GL resources for a non-texture depth buffer
	function setupDepthRenderbuffer( renderTarget ) {

		const renderTargetProperties = properties.get( renderTarget );

		const isCube = ( renderTarget.isWebGLCubeRenderTarget === true );

		if ( renderTarget.depthTexture ) {

			if ( isCube ) throw new Error( 'target.depthTexture not supported in Cube render targets' );

			setupDepthTexture( renderTargetProperties.__webglFramebuffer, renderTarget );

		} else {

			if ( isCube ) {

				renderTargetProperties.__webglDepthbuffer = [];

				for ( let i = 0; i < 6; i ++ ) {

					state.bindFramebuffer( 36160, renderTargetProperties.__webglFramebuffer[ i ] );
					renderTargetProperties.__webglDepthbuffer[ i ] = _gl.createRenderbuffer();
					setupRenderBufferStorage( renderTargetProperties.__webglDepthbuffer[ i ], renderTarget, false );

				}

			} else {

				state.bindFramebuffer( 36160, renderTargetProperties.__webglFramebuffer );
				renderTargetProperties.__webglDepthbuffer = _gl.createRenderbuffer();
				setupRenderBufferStorage( renderTargetProperties.__webglDepthbuffer, renderTarget, false );

			}

		}

		state.bindFramebuffer( 36160, null );

	}

	// Set up GL resources for the render target
	function setupRenderTarget( renderTarget ) {

		const texture = renderTarget.texture;

		const renderTargetProperties = properties.get( renderTarget );
		const textureProperties = properties.get( texture );

		renderTarget.addEventListener( 'dispose', onRenderTargetDispose );

		textureProperties.__webglTexture = _gl.createTexture();
		textureProperties.__version = texture.version;

		info.memory.textures ++;

		const isCube = ( renderTarget.isWebGLCubeRenderTarget === true );
		const isMultisample = ( renderTarget.isWebGLMultisampleRenderTarget === true );
		const isRenderTarget3D = texture.isDataTexture3D || texture.isDataTexture2DArray;
		const supportsMips = isPowerOfTwo( renderTarget ) || isWebGL2;

		// Handles WebGL2 RGBFormat fallback - #18858

		if ( isWebGL2 && texture.format === RGBFormat && ( texture.type === FloatType || texture.type === HalfFloatType ) ) {

			texture.format = RGBAFormat;

			console.warn( 'THREE.WebGLRenderer: Rendering to textures with RGB format is not supported. Using RGBA format instead.' );

		}

		// Setup framebuffer

		if ( isCube ) {

			renderTargetProperties.__webglFramebuffer = [];

			for ( let i = 0; i < 6; i ++ ) {

				renderTargetProperties.__webglFramebuffer[ i ] = _gl.createFramebuffer();

			}

		} else {

			renderTargetProperties.__webglFramebuffer = _gl.createFramebuffer();

			if ( isMultisample ) {

				if ( isWebGL2 ) {

					renderTargetProperties.__webglMultisampledFramebuffer = _gl.createFramebuffer();
					renderTargetProperties.__webglColorRenderbuffer = _gl.createRenderbuffer();

					_gl.bindRenderbuffer( 36161, renderTargetProperties.__webglColorRenderbuffer );

					const glFormat = utils.convert( texture.format );
					const glType = utils.convert( texture.type );
					const glInternalFormat = getInternalFormat( texture.internalFormat, glFormat, glType );
					const samples = getRenderTargetSamples( renderTarget );
					_gl.renderbufferStorageMultisample( 36161, samples, glInternalFormat, renderTarget.width, renderTarget.height );

					state.bindFramebuffer( 36160, renderTargetProperties.__webglMultisampledFramebuffer );
					_gl.framebufferRenderbuffer( 36160, 36064, 36161, renderTargetProperties.__webglColorRenderbuffer );
					_gl.bindRenderbuffer( 36161, null );

					if ( renderTarget.depthBuffer ) {

						renderTargetProperties.__webglDepthRenderbuffer = _gl.createRenderbuffer();
						setupRenderBufferStorage( renderTargetProperties.__webglDepthRenderbuffer, renderTarget, true );

					}

					state.bindFramebuffer( 36160, null );


				} else {

					console.warn( 'THREE.WebGLRenderer: WebGLMultisampleRenderTarget can only be used with WebGL2.' );

				}

			}

		}

		// Setup color buffer

		if ( isCube ) {

			state.bindTexture( 34067, textureProperties.__webglTexture );
			setTextureParameters( 34067, texture, supportsMips );

			for ( let i = 0; i < 6; i ++ ) {

				setupFrameBufferTexture( renderTargetProperties.__webglFramebuffer[ i ], renderTarget, 36064, 34069 + i );

			}

			if ( textureNeedsGenerateMipmaps( texture, supportsMips ) ) {

				generateMipmap( 34067, texture, renderTarget.width, renderTarget.height );

			}

			state.bindTexture( 34067, null );

		} else {

			let glTextureType = 3553;

			if ( isRenderTarget3D ) {

				// Render targets containing layers, i.e: Texture 3D and 2d arrays

				if ( isWebGL2 ) {

					const isTexture3D = texture.isDataTexture3D;
					glTextureType = isTexture3D ? 32879 : 35866;

				} else {

					console.warn( 'THREE.DataTexture3D and THREE.DataTexture2DArray only supported with WebGL2.' );

				}

			}

			state.bindTexture( glTextureType, textureProperties.__webglTexture );
			setTextureParameters( glTextureType, texture, supportsMips );
			setupFrameBufferTexture( renderTargetProperties.__webglFramebuffer, renderTarget, 36064, glTextureType );

			if ( textureNeedsGenerateMipmaps( texture, supportsMips ) ) {

				generateMipmap( 3553, texture, renderTarget.width, renderTarget.height );

			}

			state.bindTexture( 3553, null );

		}

		// Setup depth and stencil buffers

		if ( renderTarget.depthBuffer ) {

			setupDepthRenderbuffer( renderTarget );

		}

	}

	function updateRenderTargetMipmap( renderTarget ) {

		const texture = renderTarget.texture;

		const supportsMips = isPowerOfTwo( renderTarget ) || isWebGL2;

		if ( textureNeedsGenerateMipmaps( texture, supportsMips ) ) {

			const target = renderTarget.isWebGLCubeRenderTarget ? 34067 : 3553;
			const webglTexture = properties.get( texture ).__webglTexture;

			state.bindTexture( target, webglTexture );
			generateMipmap( target, texture, renderTarget.width, renderTarget.height );
			state.bindTexture( target, null );

		}

	}

	function updateMultisampleRenderTarget( renderTarget ) {

		if ( renderTarget.isWebGLMultisampleRenderTarget ) {

			if ( isWebGL2 ) {

				const renderTargetProperties = properties.get( renderTarget );

				state.bindFramebuffer( 36008, renderTargetProperties.__webglMultisampledFramebuffer );
				state.bindFramebuffer( 36009, renderTargetProperties.__webglFramebuffer );

				const width = renderTarget.width;
				const height = renderTarget.height;
				let mask = 16384;

				if ( renderTarget.depthBuffer ) mask |= 256;
				if ( renderTarget.stencilBuffer ) mask |= 1024;

				_gl.blitFramebuffer( 0, 0, width, height, 0, 0, width, height, mask, 9728 );

				state.bindFramebuffer( 36160, renderTargetProperties.__webglMultisampledFramebuffer ); // see #18905

			} else {

				console.warn( 'THREE.WebGLRenderer: WebGLMultisampleRenderTarget can only be used with WebGL2.' );

			}

		}

	}

	function getRenderTargetSamples( renderTarget ) {

		return ( isWebGL2 && renderTarget.isWebGLMultisampleRenderTarget ) ?
			Math.min( maxSamples, renderTarget.samples ) : 0;

	}

	function updateVideoTexture( texture ) {

		const frame = info.render.frame;

		// Check the last frame we updated the VideoTexture

		if ( _videoTextures.get( texture ) !== frame ) {

			_videoTextures.set( texture, frame );
			texture.update();

		}

	}

	// backwards compatibility

	let warnedTexture2D = false;
	let warnedTextureCube = false;

	function safeSetTexture2D( texture, slot ) {

		if ( texture && texture.isWebGLRenderTarget ) {

			if ( warnedTexture2D === false ) {

				console.warn( 'THREE.WebGLTextures.safeSetTexture2D: don\'t use render targets as textures. Use their .texture property instead.' );
				warnedTexture2D = true;

			}

			texture = texture.texture;

		}

		setTexture2D( texture, slot );

	}

	function safeSetTextureCube( texture, slot ) {

		if ( texture && texture.isWebGLCubeRenderTarget ) {

			if ( warnedTextureCube === false ) {

				console.warn( 'THREE.WebGLTextures.safeSetTextureCube: don\'t use cube render targets as textures. Use their .texture property instead.' );
				warnedTextureCube = true;

			}

			texture = texture.texture;

		}


		setTextureCube( texture, slot );

	}

	//

	this.allocateTextureUnit = allocateTextureUnit;
	this.resetTextureUnits = resetTextureUnits;

	this.setTexture2D = setTexture2D;
	this.setTexture2DArray = setTexture2DArray;
	this.setTexture3D = setTexture3D;
	this.setTextureCube = setTextureCube;
	this.setupRenderTarget = setupRenderTarget;
	this.updateRenderTargetMipmap = updateRenderTargetMipmap;
	this.updateMultisampleRenderTarget = updateMultisampleRenderTarget;

	this.safeSetTexture2D = safeSetTexture2D;
	this.safeSetTextureCube = safeSetTextureCube;

}

function WebGLUtils( gl, extensions, capabilities ) {

	const isWebGL2 = capabilities.isWebGL2;

	function convert( p ) {

		let extension;

		if ( p === UnsignedByteType ) return 5121;
		if ( p === UnsignedShort4444Type ) return 32819;
		if ( p === UnsignedShort5551Type ) return 32820;
		if ( p === UnsignedShort565Type ) return 33635;

		if ( p === ByteType ) return 5120;
		if ( p === ShortType ) return 5122;
		if ( p === UnsignedShortType ) return 5123;
		if ( p === IntType ) return 5124;
		if ( p === UnsignedIntType ) return 5125;
		if ( p === FloatType ) return 5126;

		if ( p === HalfFloatType ) {

			if ( isWebGL2 ) return 5131;

			extension = extensions.get( 'OES_texture_half_float' );

			if ( extension !== null ) {

				return extension.HALF_FLOAT_OES;

			} else {

				return null;

			}

		}

		if ( p === AlphaFormat ) return 6406;
		if ( p === RGBFormat ) return 6407;
		if ( p === RGBAFormat ) return 6408;
		if ( p === LuminanceFormat ) return 6409;
		if ( p === LuminanceAlphaFormat ) return 6410;
		if ( p === DepthFormat ) return 6402;
		if ( p === DepthStencilFormat ) return 34041;
		if ( p === RedFormat ) return 6403;

		// WebGL2 formats.

		if ( p === RedIntegerFormat ) return 36244;
		if ( p === RGFormat ) return 33319;
		if ( p === RGIntegerFormat ) return 33320;
		if ( p === RGBIntegerFormat ) return 36248;
		if ( p === RGBAIntegerFormat ) return 36249;

		if ( p === RGB_S3TC_DXT1_Format || p === RGBA_S3TC_DXT1_Format ||
			p === RGBA_S3TC_DXT3_Format || p === RGBA_S3TC_DXT5_Format ) {

			extension = extensions.get( 'WEBGL_compressed_texture_s3tc' );

			if ( extension !== null ) {

				if ( p === RGB_S3TC_DXT1_Format ) return extension.COMPRESSED_RGB_S3TC_DXT1_EXT;
				if ( p === RGBA_S3TC_DXT1_Format ) return extension.COMPRESSED_RGBA_S3TC_DXT1_EXT;
				if ( p === RGBA_S3TC_DXT3_Format ) return extension.COMPRESSED_RGBA_S3TC_DXT3_EXT;
				if ( p === RGBA_S3TC_DXT5_Format ) return extension.COMPRESSED_RGBA_S3TC_DXT5_EXT;

			} else {

				return null;

			}

		}

		if ( p === RGB_PVRTC_4BPPV1_Format || p === RGB_PVRTC_2BPPV1_Format ||
			p === RGBA_PVRTC_4BPPV1_Format || p === RGBA_PVRTC_2BPPV1_Format ) {

			extension = extensions.get( 'WEBGL_compressed_texture_pvrtc' );

			if ( extension !== null ) {

				if ( p === RGB_PVRTC_4BPPV1_Format ) return extension.COMPRESSED_RGB_PVRTC_4BPPV1_IMG;
				if ( p === RGB_PVRTC_2BPPV1_Format ) return extension.COMPRESSED_RGB_PVRTC_2BPPV1_IMG;
				if ( p === RGBA_PVRTC_4BPPV1_Format ) return extension.COMPRESSED_RGBA_PVRTC_4BPPV1_IMG;
				if ( p === RGBA_PVRTC_2BPPV1_Format ) return extension.COMPRESSED_RGBA_PVRTC_2BPPV1_IMG;

			} else {

				return null;

			}

		}

		if ( p === RGB_ETC1_Format ) {

			extension = extensions.get( 'WEBGL_compressed_texture_etc1' );

			if ( extension !== null ) {

				return extension.COMPRESSED_RGB_ETC1_WEBGL;

			} else {

				return null;

			}

		}

		if ( p === RGB_ETC2_Format || p === RGBA_ETC2_EAC_Format ) {

			extension = extensions.get( 'WEBGL_compressed_texture_etc' );

			if ( extension !== null ) {

				if ( p === RGB_ETC2_Format ) return extension.COMPRESSED_RGB8_ETC2;
				if ( p === RGBA_ETC2_EAC_Format ) return extension.COMPRESSED_RGBA8_ETC2_EAC;

			}

		}

		if ( p === RGBA_ASTC_4x4_Format || p === RGBA_ASTC_5x4_Format || p === RGBA_ASTC_5x5_Format ||
			p === RGBA_ASTC_6x5_Format || p === RGBA_ASTC_6x6_Format || p === RGBA_ASTC_8x5_Format ||
			p === RGBA_ASTC_8x6_Format || p === RGBA_ASTC_8x8_Format || p === RGBA_ASTC_10x5_Format ||
			p === RGBA_ASTC_10x6_Format || p === RGBA_ASTC_10x8_Format || p === RGBA_ASTC_10x10_Format ||
			p === RGBA_ASTC_12x10_Format || p === RGBA_ASTC_12x12_Format ||
			p === SRGB8_ALPHA8_ASTC_4x4_Format || p === SRGB8_ALPHA8_ASTC_5x4_Format || p === SRGB8_ALPHA8_ASTC_5x5_Format ||
			p === SRGB8_ALPHA8_ASTC_6x5_Format || p === SRGB8_ALPHA8_ASTC_6x6_Format || p === SRGB8_ALPHA8_ASTC_8x5_Format ||
			p === SRGB8_ALPHA8_ASTC_8x6_Format || p === SRGB8_ALPHA8_ASTC_8x8_Format || p === SRGB8_ALPHA8_ASTC_10x5_Format ||
			p === SRGB8_ALPHA8_ASTC_10x6_Format || p === SRGB8_ALPHA8_ASTC_10x8_Format || p === SRGB8_ALPHA8_ASTC_10x10_Format ||
			p === SRGB8_ALPHA8_ASTC_12x10_Format || p === SRGB8_ALPHA8_ASTC_12x12_Format ) {

			extension = extensions.get( 'WEBGL_compressed_texture_astc' );

			if ( extension !== null ) {

				// TODO Complete?

				return p;

			} else {

				return null;

			}

		}

		if ( p === RGBA_BPTC_Format ) {

			extension = extensions.get( 'EXT_texture_compression_bptc' );

			if ( extension !== null ) {

				// TODO Complete?

				return p;

			} else {

				return null;

			}

		}

		if ( p === UnsignedInt248Type ) {

			if ( isWebGL2 ) return 34042;

			extension = extensions.get( 'WEBGL_depth_texture' );

			if ( extension !== null ) {

				return extension.UNSIGNED_INT_24_8_WEBGL;

			} else {

				return null;

			}

		}

	}

	return { convert: convert };

}

class ArrayCamera extends PerspectiveCamera {

	constructor( array = [] ) {

		super();

		this.cameras = array;

	}

}

ArrayCamera.prototype.isArrayCamera = true;

class Group extends Object3D {

	constructor() {

		super();

		this.type = 'Group';

	}

}

Group.prototype.isGroup = true;

function WebXRController() {

	this._targetRay = null;
	this._grip = null;
	this._hand = null;

}

Object.assign( WebXRController.prototype, {

	constructor: WebXRController,

	getHandSpace: function () {

		if ( this._hand === null ) {

			this._hand = new Group();
			this._hand.matrixAutoUpdate = false;
			this._hand.visible = false;

			this._hand.joints = {};
			this._hand.inputState = { pinching: false };

		}

		return this._hand;

	},

	getTargetRaySpace: function () {

		if ( this._targetRay === null ) {

			this._targetRay = new Group();
			this._targetRay.matrixAutoUpdate = false;
			this._targetRay.visible = false;

		}

		return this._targetRay;

	},

	getGripSpace: function () {

		if ( this._grip === null ) {

			this._grip = new Group();
			this._grip.matrixAutoUpdate = false;
			this._grip.visible = false;

		}

		return this._grip;

	},

	dispatchEvent: function ( event ) {

		if ( this._targetRay !== null ) {

			this._targetRay.dispatchEvent( event );

		}

		if ( this._grip !== null ) {

			this._grip.dispatchEvent( event );

		}

		if ( this._hand !== null ) {

			this._hand.dispatchEvent( event );

		}

		return this;

	},

	disconnect: function ( inputSource ) {

		this.dispatchEvent( { type: 'disconnected', data: inputSource } );

		if ( this._targetRay !== null ) {

			this._targetRay.visible = false;

		}

		if ( this._grip !== null ) {

			this._grip.visible = false;

		}

		if ( this._hand !== null ) {

			this._hand.visible = false;

		}

		return this;

	},

	update: function ( inputSource, frame, referenceSpace ) {

		let inputPose = null;
		let gripPose = null;
		let handPose = null;

		const targetRay = this._targetRay;
		const grip = this._grip;
		const hand = this._hand;

		if ( inputSource && frame.session.visibilityState !== 'visible-blurred' ) {

			if ( targetRay !== null ) {

				inputPose = frame.getPose( inputSource.targetRaySpace, referenceSpace );

				if ( inputPose !== null ) {

					targetRay.matrix.fromArray( inputPose.transform.matrix );
					targetRay.matrix.decompose( targetRay.position, targetRay.rotation, targetRay.scale );

				}

			}

			if ( hand && inputSource.hand ) {

				handPose = true;

				for ( const inputjoint of inputSource.hand.values() ) {

					// Update the joints groups with the XRJoint poses
					const jointPose = frame.getJointPose( inputjoint, referenceSpace );

					if ( hand.joints[ inputjoint.jointName ] === undefined ) {

						// The transform of this joint will be updated with the joint pose on each frame
						const joint = new Group();
						joint.matrixAutoUpdate = false;
						joint.visible = false;
						hand.joints[ inputjoint.jointName ] = joint;
						// ??
						hand.add( joint );

					}

					const joint = hand.joints[ inputjoint.jointName ];

					if ( jointPose !== null ) {

						joint.matrix.fromArray( jointPose.transform.matrix );
						joint.matrix.decompose( joint.position, joint.rotation, joint.scale );
						joint.jointRadius = jointPose.radius;

					}

					joint.visible = jointPose !== null;

				}

				// Custom events

				// Check pinchz
				const indexTip = hand.joints[ 'index-finger-tip' ];
				const thumbTip = hand.joints[ 'thumb-tip' ];
				const distance = indexTip.position.distanceTo( thumbTip.position );

				const distanceToPinch = 0.02;
				const threshold = 0.005;

				if ( hand.inputState.pinching && distance > distanceToPinch + threshold ) {

					hand.inputState.pinching = false;
					this.dispatchEvent( {
						type: 'pinchend',
						handedness: inputSource.handedness,
						target: this
					} );

				} else if ( ! hand.inputState.pinching && distance <= distanceToPinch - threshold ) {

					hand.inputState.pinching = true;
					this.dispatchEvent( {
						type: 'pinchstart',
						handedness: inputSource.handedness,
						target: this
					} );

				}

			} else {

				if ( grip !== null && inputSource.gripSpace ) {

					gripPose = frame.getPose( inputSource.gripSpace, referenceSpace );

					if ( gripPose !== null ) {

						grip.matrix.fromArray( gripPose.transform.matrix );
						grip.matrix.decompose( grip.position, grip.rotation, grip.scale );

					}

				}

			}

		}

		if ( targetRay !== null ) {

			targetRay.visible = ( inputPose !== null );

		}

		if ( grip !== null ) {

			grip.visible = ( gripPose !== null );

		}

		if ( hand !== null ) {

			hand.visible = ( handPose !== null );

		}

		return this;

	}

} );

function WebXRManager( renderer, gl ) {

	const scope = this;
	const state = renderer.state;

	let session = null;

	let framebufferScaleFactor = 1.0;

	let referenceSpace = null;
	let referenceSpaceType = 'local-floor';

	let pose = null;

	const controllers = [];
	const inputSourcesMap = new Map();

	//

	const cameraL = new PerspectiveCamera();
	cameraL.layers.enable( 1 );
	cameraL.viewport = new Vector4();

	const cameraR = new PerspectiveCamera();
	cameraR.layers.enable( 2 );
	cameraR.viewport = new Vector4();

	const cameras = [ cameraL, cameraR ];

	const cameraVR = new ArrayCamera();
	cameraVR.layers.enable( 1 );
	cameraVR.layers.enable( 2 );

	let _currentDepthNear = null;
	let _currentDepthFar = null;

	//

	this.enabled = false;

	this.isPresenting = false;

	this.getController = function ( index ) {

		let controller = controllers[ index ];

		if ( controller === undefined ) {

			controller = new WebXRController();
			controllers[ index ] = controller;

		}

		return controller.getTargetRaySpace();

	};

	this.getControllerGrip = function ( index ) {

		let controller = controllers[ index ];

		if ( controller === undefined ) {

			controller = new WebXRController();
			controllers[ index ] = controller;

		}

		return controller.getGripSpace();

	};

	this.getHand = function ( index ) {

		let controller = controllers[ index ];

		if ( controller === undefined ) {

			controller = new WebXRController();
			controllers[ index ] = controller;

		}

		return controller.getHandSpace();

	};

	//

	function onSessionEvent( event ) {

		const controller = inputSourcesMap.get( event.inputSource );

		if ( controller ) {

			controller.dispatchEvent( { type: event.type, data: event.inputSource } );

		}

	}

	function onSessionEnd() {

		inputSourcesMap.forEach( function ( controller, inputSource ) {

			controller.disconnect( inputSource );

		} );

		inputSourcesMap.clear();

		_currentDepthNear = null;
		_currentDepthFar = null;

		// restore framebuffer/rendering state

		state.bindXRFramebuffer( null );
		renderer.setRenderTarget( renderer.getRenderTarget() );

		//

		animation.stop();

		scope.isPresenting = false;

		scope.dispatchEvent( { type: 'sessionend' } );

	}

	this.setFramebufferScaleFactor = function ( value ) {

		framebufferScaleFactor = value;

		if ( scope.isPresenting === true ) {

			console.warn( 'THREE.WebXRManager: Cannot change framebuffer scale while presenting.' );

		}

	};

	this.setReferenceSpaceType = function ( value ) {

		referenceSpaceType = value;

		if ( scope.isPresenting === true ) {

			console.warn( 'THREE.WebXRManager: Cannot change reference space type while presenting.' );

		}

	};

	this.getReferenceSpace = function () {

		return referenceSpace;

	};

	this.getSession = function () {

		return session;

	};

	this.setSession = async function ( value ) {

		session = value;

		if ( session !== null ) {

			session.addEventListener( 'select', onSessionEvent );
			session.addEventListener( 'selectstart', onSessionEvent );
			session.addEventListener( 'selectend', onSessionEvent );
			session.addEventListener( 'squeeze', onSessionEvent );
			session.addEventListener( 'squeezestart', onSessionEvent );
			session.addEventListener( 'squeezeend', onSessionEvent );
			session.addEventListener( 'end', onSessionEnd );
			session.addEventListener( 'inputsourceschange', onInputSourcesChange );

			const attributes = gl.getContextAttributes();

			if ( attributes.xrCompatible !== true ) {

				await gl.makeXRCompatible();

			}

			const layerInit = {
				antialias: attributes.antialias,
				alpha: attributes.alpha,
				depth: attributes.depth,
				stencil: attributes.stencil,
				framebufferScaleFactor: framebufferScaleFactor
			};

			// eslint-disable-next-line no-undef
			const baseLayer = new XRWebGLLayer( session, gl, layerInit );

			session.updateRenderState( { baseLayer: baseLayer } );

			referenceSpace = await session.requestReferenceSpace( referenceSpaceType );

			animation.setContext( session );
			animation.start();

			scope.isPresenting = true;

			scope.dispatchEvent( { type: 'sessionstart' } );

		}

	};

	function onInputSourcesChange( event ) {

		const inputSources = session.inputSources;

		// Assign inputSources to available controllers

		for ( let i = 0; i < controllers.length; i ++ ) {

			inputSourcesMap.set( inputSources[ i ], controllers[ i ] );

		}

		// Notify disconnected

		for ( let i = 0; i < event.removed.length; i ++ ) {

			const inputSource = event.removed[ i ];
			const controller = inputSourcesMap.get( inputSource );

			if ( controller ) {

				controller.dispatchEvent( { type: 'disconnected', data: inputSource } );
				inputSourcesMap.delete( inputSource );

			}

		}

		// Notify connected

		for ( let i = 0; i < event.added.length; i ++ ) {

			const inputSource = event.added[ i ];
			const controller = inputSourcesMap.get( inputSource );

			if ( controller ) {

				controller.dispatchEvent( { type: 'connected', data: inputSource } );

			}

		}

	}

	//

	const cameraLPos = new Vector3();
	const cameraRPos = new Vector3();

	/**
	 * Assumes 2 cameras that are parallel and share an X-axis, and that
	 * the cameras' projection and world matrices have already been set.
	 * And that near and far planes are identical for both cameras.
	 * Visualization of this technique: https://computergraphics.stackexchange.com/a/4765
	 */
	function setProjectionFromUnion( camera, cameraL, cameraR ) {

		cameraLPos.setFromMatrixPosition( cameraL.matrixWorld );
		cameraRPos.setFromMatrixPosition( cameraR.matrixWorld );

		const ipd = cameraLPos.distanceTo( cameraRPos );

		const projL = cameraL.projectionMatrix.elements;
		const projR = cameraR.projectionMatrix.elements;

		// VR systems will have identical far and near planes, and
		// most likely identical top and bottom frustum extents.
		// Use the left camera for these values.
		const near = projL[ 14 ] / ( projL[ 10 ] - 1 );
		const far = projL[ 14 ] / ( projL[ 10 ] + 1 );
		const topFov = ( projL[ 9 ] + 1 ) / projL[ 5 ];
		const bottomFov = ( projL[ 9 ] - 1 ) / projL[ 5 ];

		const leftFov = ( projL[ 8 ] - 1 ) / projL[ 0 ];
		const rightFov = ( projR[ 8 ] + 1 ) / projR[ 0 ];
		const left = near * leftFov;
		const right = near * rightFov;

		// Calculate the new camera's position offset from the
		// left camera. xOffset should be roughly half `ipd`.
		const zOffset = ipd / ( - leftFov + rightFov );
		const xOffset = zOffset * - leftFov;

		// TODO: Better way to apply this offset?
		cameraL.matrixWorld.decompose( camera.position, camera.quaternion, camera.scale );
		camera.translateX( xOffset );
		camera.translateZ( zOffset );
		camera.matrixWorld.compose( camera.position, camera.quaternion, camera.scale );
		camera.matrixWorldInverse.copy( camera.matrixWorld ).invert();

		// Find the union of the frustum values of the cameras and scale
		// the values so that the near plane's position does not change in world space,
		// although must now be relative to the new union camera.
		const near2 = near + zOffset;
		const far2 = far + zOffset;
		const left2 = left - xOffset;
		const right2 = right + ( ipd - xOffset );
		const top2 = topFov * far / far2 * near2;
		const bottom2 = bottomFov * far / far2 * near2;

		camera.projectionMatrix.makePerspective( left2, right2, top2, bottom2, near2, far2 );

	}

	function updateCamera( camera, parent ) {

		if ( parent === null ) {

			camera.matrixWorld.copy( camera.matrix );

		} else {

			camera.matrixWorld.multiplyMatrices( parent.matrixWorld, camera.matrix );

		}

		camera.matrixWorldInverse.copy( camera.matrixWorld ).invert();

	}

	this.getCamera = function ( camera ) {

		cameraVR.near = cameraR.near = cameraL.near = camera.near;
		cameraVR.far = cameraR.far = cameraL.far = camera.far;

		if ( _currentDepthNear !== cameraVR.near || _currentDepthFar !== cameraVR.far ) {

			// Note that the new renderState won't apply until the next frame. See #18320

			session.updateRenderState( {
				depthNear: cameraVR.near,
				depthFar: cameraVR.far
			} );

			_currentDepthNear = cameraVR.near;
			_currentDepthFar = cameraVR.far;

		}

		const parent = camera.parent;
		const cameras = cameraVR.cameras;

		updateCamera( cameraVR, parent );

		for ( let i = 0; i < cameras.length; i ++ ) {

			updateCamera( cameras[ i ], parent );

		}

		// update camera and its children

		camera.matrixWorld.copy( cameraVR.matrixWorld );
		camera.matrix.copy( cameraVR.matrix );
		camera.matrix.decompose( camera.position, camera.quaternion, camera.scale );

		const children = camera.children;

		for ( let i = 0, l = children.length; i < l; i ++ ) {

			children[ i ].updateMatrixWorld( true );

		}

		// update projection matrix for proper view frustum culling

		if ( cameras.length === 2 ) {

			setProjectionFromUnion( cameraVR, cameraL, cameraR );

		} else {

			// assume single camera setup (AR)

			cameraVR.projectionMatrix.copy( cameraL.projectionMatrix );

		}

		return cameraVR;

	};

	// Animation Loop

	let onAnimationFrameCallback = null;

	function onAnimationFrame( time, frame ) {

		pose = frame.getViewerPose( referenceSpace );

		if ( pose !== null ) {

			const views = pose.views;
			const baseLayer = session.renderState.baseLayer;

			state.bindXRFramebuffer( baseLayer.framebuffer );

			let cameraVRNeedsUpdate = false;

			// check if it's necessary to rebuild cameraVR's camera list

			if ( views.length !== cameraVR.cameras.length ) {

				cameraVR.cameras.length = 0;
				cameraVRNeedsUpdate = true;

			}

			for ( let i = 0; i < views.length; i ++ ) {

				const view = views[ i ];
				const viewport = baseLayer.getViewport( view );

				const camera = cameras[ i ];
				camera.matrix.fromArray( view.transform.matrix );
				camera.projectionMatrix.fromArray( view.projectionMatrix );
				camera.viewport.set( viewport.x, viewport.y, viewport.width, viewport.height );

				if ( i === 0 ) {

					cameraVR.matrix.copy( camera.matrix );

				}

				if ( cameraVRNeedsUpdate === true ) {

					cameraVR.cameras.push( camera );

				}

			}

		}

		//

		const inputSources = session.inputSources;

		for ( let i = 0; i < controllers.length; i ++ ) {

			const controller = controllers[ i ];
			const inputSource = inputSources[ i ];

			controller.update( inputSource, frame, referenceSpace );

		}

		if ( onAnimationFrameCallback ) onAnimationFrameCallback( time, frame );

	}

	const animation = new WebGLAnimation();
	animation.setAnimationLoop( onAnimationFrame );

	this.setAnimationLoop = function ( callback ) {

		onAnimationFrameCallback = callback;

	};

	this.dispose = function () {};

}

Object.assign( WebXRManager.prototype, EventDispatcher.prototype );

function WebGLMaterials( properties ) {

	function refreshFogUniforms( uniforms, fog ) {

		uniforms.fogColor.value.copy( fog.color );

		if ( fog.isFog ) {

			uniforms.fogNear.value = fog.near;
			uniforms.fogFar.value = fog.far;

		} else if ( fog.isFogExp2 ) {

			uniforms.fogDensity.value = fog.density;

		}

	}

	function refreshMaterialUniforms( uniforms, material, pixelRatio, height ) {

		if ( material.isMeshBasicMaterial ) {

			refreshUniformsCommon( uniforms, material );

		} else if ( material.isMeshLambertMaterial ) {

			refreshUniformsCommon( uniforms, material );
			refreshUniformsLambert( uniforms, material );

		} else if ( material.isMeshToonMaterial ) {

			refreshUniformsCommon( uniforms, material );
			refreshUniformsToon( uniforms, material );

		} else if ( material.isMeshPhongMaterial ) {

			refreshUniformsCommon( uniforms, material );
			refreshUniformsPhong( uniforms, material );

		} else if ( material.isMeshStandardMaterial ) {

			refreshUniformsCommon( uniforms, material );

			if ( material.isMeshPhysicalMaterial ) {

				refreshUniformsPhysical( uniforms, material );

			} else {

				refreshUniformsStandard( uniforms, material );

			}

		} else if ( material.isMeshMatcapMaterial ) {

			refreshUniformsCommon( uniforms, material );
			refreshUniformsMatcap( uniforms, material );

		} else if ( material.isMeshDepthMaterial ) {

			refreshUniformsCommon( uniforms, material );
			refreshUniformsDepth( uniforms, material );

		} else if ( material.isMeshDistanceMaterial ) {

			refreshUniformsCommon( uniforms, material );
			refreshUniformsDistance( uniforms, material );

		} else if ( material.isMeshNormalMaterial ) {

			refreshUniformsCommon( uniforms, material );
			refreshUniformsNormal( uniforms, material );

		} else if ( material.isLineBasicMaterial ) {

			refreshUniformsLine( uniforms, material );

			if ( material.isLineDashedMaterial ) {

				refreshUniformsDash( uniforms, material );

			}

		} else if ( material.isPointsMaterial ) {

			refreshUniformsPoints( uniforms, material, pixelRatio, height );

		} else if ( material.isSpriteMaterial ) {

			refreshUniformsSprites( uniforms, material );

		} else if ( material.isShadowMaterial ) {

			uniforms.color.value.copy( material.color );
			uniforms.opacity.value = material.opacity;

		} else if ( material.isShaderMaterial ) {

			material.uniformsNeedUpdate = false; // #15581

		}

	}

	function refreshUniformsCommon( uniforms, material ) {

		uniforms.opacity.value = material.opacity;

		if ( material.color ) {

			uniforms.diffuse.value.copy( material.color );

		}

		if ( material.emissive ) {

			uniforms.emissive.value.copy( material.emissive ).multiplyScalar( material.emissiveIntensity );

		}

		if ( material.map ) {

			uniforms.map.value = material.map;

		}

		if ( material.alphaMap ) {

			uniforms.alphaMap.value = material.alphaMap;

		}

		if ( material.specularMap ) {

			uniforms.specularMap.value = material.specularMap;

		}

		const envMap = properties.get( material ).envMap;

		if ( envMap ) {

			uniforms.envMap.value = envMap;

			uniforms.flipEnvMap.value = ( envMap.isCubeTexture && envMap._needsFlipEnvMap ) ? - 1 : 1;

			uniforms.reflectivity.value = material.reflectivity;
			uniforms.refractionRatio.value = material.refractionRatio;

			const maxMipLevel = properties.get( envMap ).__maxMipLevel;

			if ( maxMipLevel !== undefined ) {

				uniforms.maxMipLevel.value = maxMipLevel;

			}

		}

		if ( material.lightMap ) {

			uniforms.lightMap.value = material.lightMap;
			uniforms.lightMapIntensity.value = material.lightMapIntensity;

		}

		if ( material.aoMap ) {

			uniforms.aoMap.value = material.aoMap;
			uniforms.aoMapIntensity.value = material.aoMapIntensity;

		}

		// uv repeat and offset setting priorities
		// 1. color map
		// 2. specular map
		// 3. displacementMap map
		// 4. normal map
		// 5. bump map
		// 6. roughnessMap map
		// 7. metalnessMap map
		// 8. alphaMap map
		// 9. emissiveMap map
		// 10. clearcoat map
		// 11. clearcoat normal map
		// 12. clearcoat roughnessMap map

		let uvScaleMap;

		if ( material.map ) {

			uvScaleMap = material.map;

		} else if ( material.specularMap ) {

			uvScaleMap = material.specularMap;

		} else if ( material.displacementMap ) {

			uvScaleMap = material.displacementMap;

		} else if ( material.normalMap ) {

			uvScaleMap = material.normalMap;

		} else if ( material.bumpMap ) {

			uvScaleMap = material.bumpMap;

		} else if ( material.roughnessMap ) {

			uvScaleMap = material.roughnessMap;

		} else if ( material.metalnessMap ) {

			uvScaleMap = material.metalnessMap;

		} else if ( material.alphaMap ) {

			uvScaleMap = material.alphaMap;

		} else if ( material.emissiveMap ) {

			uvScaleMap = material.emissiveMap;

		} else if ( material.clearcoatMap ) {

			uvScaleMap = material.clearcoatMap;

		} else if ( material.clearcoatNormalMap ) {

			uvScaleMap = material.clearcoatNormalMap;

		} else if ( material.clearcoatRoughnessMap ) {

			uvScaleMap = material.clearcoatRoughnessMap;

		}

		if ( uvScaleMap !== undefined ) {

			// backwards compatibility
			if ( uvScaleMap.isWebGLRenderTarget ) {

				uvScaleMap = uvScaleMap.texture;

			}

			if ( uvScaleMap.matrixAutoUpdate === true ) {

				uvScaleMap.updateMatrix();

			}

			uniforms.uvTransform.value.copy( uvScaleMap.matrix );

		}

		// uv repeat and offset setting priorities for uv2
		// 1. ao map
		// 2. light map

		let uv2ScaleMap;

		if ( material.aoMap ) {

			uv2ScaleMap = material.aoMap;

		} else if ( material.lightMap ) {

			uv2ScaleMap = material.lightMap;

		}

		if ( uv2ScaleMap !== undefined ) {

			// backwards compatibility
			if ( uv2ScaleMap.isWebGLRenderTarget ) {

				uv2ScaleMap = uv2ScaleMap.texture;

			}

			if ( uv2ScaleMap.matrixAutoUpdate === true ) {

				uv2ScaleMap.updateMatrix();

			}

			uniforms.uv2Transform.value.copy( uv2ScaleMap.matrix );

		}

	}

	function refreshUniformsLine( uniforms, material ) {

		uniforms.diffuse.value.copy( material.color );
		uniforms.opacity.value = material.opacity;

	}

	function refreshUniformsDash( uniforms, material ) {

		uniforms.dashSize.value = material.dashSize;
		uniforms.totalSize.value = material.dashSize + material.gapSize;
		uniforms.scale.value = material.scale;

	}

	function refreshUniformsPoints( uniforms, material, pixelRatio, height ) {

		uniforms.diffuse.value.copy( material.color );
		uniforms.opacity.value = material.opacity;
		uniforms.size.value = material.size * pixelRatio;
		uniforms.scale.value = height * 0.5;

		if ( material.map ) {

			uniforms.map.value = material.map;

		}

		if ( material.alphaMap ) {

			uniforms.alphaMap.value = material.alphaMap;

		}

		// uv repeat and offset setting priorities
		// 1. color map
		// 2. alpha map

		let uvScaleMap;

		if ( material.map ) {

			uvScaleMap = material.map;

		} else if ( material.alphaMap ) {

			uvScaleMap = material.alphaMap;

		}

		if ( uvScaleMap !== undefined ) {

			if ( uvScaleMap.matrixAutoUpdate === true ) {

				uvScaleMap.updateMatrix();

			}

			uniforms.uvTransform.value.copy( uvScaleMap.matrix );

		}

	}

	function refreshUniformsSprites( uniforms, material ) {

		uniforms.diffuse.value.copy( material.color );
		uniforms.opacity.value = material.opacity;
		uniforms.rotation.value = material.rotation;

		if ( material.map ) {

			uniforms.map.value = material.map;

		}

		if ( material.alphaMap ) {

			uniforms.alphaMap.value = material.alphaMap;

		}

		// uv repeat and offset setting priorities
		// 1. color map
		// 2. alpha map

		let uvScaleMap;

		if ( material.map ) {

			uvScaleMap = material.map;

		} else if ( material.alphaMap ) {

			uvScaleMap = material.alphaMap;

		}

		if ( uvScaleMap !== undefined ) {

			if ( uvScaleMap.matrixAutoUpdate === true ) {

				uvScaleMap.updateMatrix();

			}

			uniforms.uvTransform.value.copy( uvScaleMap.matrix );

		}

	}

	function refreshUniformsLambert( uniforms, material ) {

		if ( material.emissiveMap ) {

			uniforms.emissiveMap.value = material.emissiveMap;

		}

	}

	function refreshUniformsPhong( uniforms, material ) {

		uniforms.specular.value.copy( material.specular );
		uniforms.shininess.value = Math.max( material.shininess, 1e-4 ); // to prevent pow( 0.0, 0.0 )

		if ( material.emissiveMap ) {

			uniforms.emissiveMap.value = material.emissiveMap;

		}

		if ( material.bumpMap ) {

			uniforms.bumpMap.value = material.bumpMap;
			uniforms.bumpScale.value = material.bumpScale;
			if ( material.side === BackSide ) uniforms.bumpScale.value *= - 1;

		}

		if ( material.normalMap ) {

			uniforms.normalMap.value = material.normalMap;
			uniforms.normalScale.value.copy( material.normalScale );
			if ( material.side === BackSide ) uniforms.normalScale.value.negate();

		}

		if ( material.displacementMap ) {

			uniforms.displacementMap.value = material.displacementMap;
			uniforms.displacementScale.value = material.displacementScale;
			uniforms.displacementBias.value = material.displacementBias;

		}

	}

	function refreshUniformsToon( uniforms, material ) {

		if ( material.gradientMap ) {

			uniforms.gradientMap.value = material.gradientMap;

		}

		if ( material.emissiveMap ) {

			uniforms.emissiveMap.value = material.emissiveMap;

		}

		if ( material.bumpMap ) {

			uniforms.bumpMap.value = material.bumpMap;
			uniforms.bumpScale.value = material.bumpScale;
			if ( material.side === BackSide ) uniforms.bumpScale.value *= - 1;

		}

		if ( material.normalMap ) {

			uniforms.normalMap.value = material.normalMap;
			uniforms.normalScale.value.copy( material.normalScale );
			if ( material.side === BackSide ) uniforms.normalScale.value.negate();

		}

		if ( material.displacementMap ) {

			uniforms.displacementMap.value = material.displacementMap;
			uniforms.displacementScale.value = material.displacementScale;
			uniforms.displacementBias.value = material.displacementBias;

		}

	}

	function refreshUniformsStandard( uniforms, material ) {

		uniforms.roughness.value = material.roughness;
		uniforms.metalness.value = material.metalness;

		if ( material.roughnessMap ) {

			uniforms.roughnessMap.value = material.roughnessMap;

		}

		if ( material.metalnessMap ) {

			uniforms.metalnessMap.value = material.metalnessMap;

		}

		if ( material.emissiveMap ) {

			uniforms.emissiveMap.value = material.emissiveMap;

		}

		if ( material.bumpMap ) {

			uniforms.bumpMap.value = material.bumpMap;
			uniforms.bumpScale.value = material.bumpScale;
			if ( material.side === BackSide ) uniforms.bumpScale.value *= - 1;

		}

		if ( material.normalMap ) {

			uniforms.normalMap.value = material.normalMap;
			uniforms.normalScale.value.copy( material.normalScale );
			if ( material.side === BackSide ) uniforms.normalScale.value.negate();

		}

		if ( material.displacementMap ) {

			uniforms.displacementMap.value = material.displacementMap;
			uniforms.displacementScale.value = material.displacementScale;
			uniforms.displacementBias.value = material.displacementBias;

		}

		const envMap = properties.get( material ).envMap;

		if ( envMap ) {

			//uniforms.envMap.value = material.envMap; // part of uniforms common
			uniforms.envMapIntensity.value = material.envMapIntensity;

		}

	}

	function refreshUniformsPhysical( uniforms, material ) {

		refreshUniformsStandard( uniforms, material );

		uniforms.reflectivity.value = material.reflectivity; // also part of uniforms common

		uniforms.clearcoat.value = material.clearcoat;
		uniforms.clearcoatRoughness.value = material.clearcoatRoughness;
		if ( material.sheen ) uniforms.sheen.value.copy( material.sheen );

		if ( material.clearcoatMap ) {

			uniforms.clearcoatMap.value = material.clearcoatMap;

		}

		if ( material.clearcoatRoughnessMap ) {

			uniforms.clearcoatRoughnessMap.value = material.clearcoatRoughnessMap;

		}

		if ( material.clearcoatNormalMap ) {

			uniforms.clearcoatNormalScale.value.copy( material.clearcoatNormalScale );
			uniforms.clearcoatNormalMap.value = material.clearcoatNormalMap;

			if ( material.side === BackSide ) {

				uniforms.clearcoatNormalScale.value.negate();

			}

		}

		uniforms.transmission.value = material.transmission;

		if ( material.transmissionMap ) {

			uniforms.transmissionMap.value = material.transmissionMap;

		}

	}

	function refreshUniformsMatcap( uniforms, material ) {

		if ( material.matcap ) {

			uniforms.matcap.value = material.matcap;

		}

		if ( material.bumpMap ) {

			uniforms.bumpMap.value = material.bumpMap;
			uniforms.bumpScale.value = material.bumpScale;
			if ( material.side === BackSide ) uniforms.bumpScale.value *= - 1;

		}

		if ( material.normalMap ) {

			uniforms.normalMap.value = material.normalMap;
			uniforms.normalScale.value.copy( material.normalScale );
			if ( material.side === BackSide ) uniforms.normalScale.value.negate();

		}

		if ( material.displacementMap ) {

			uniforms.displacementMap.value = material.displacementMap;
			uniforms.displacementScale.value = material.displacementScale;
			uniforms.displacementBias.value = material.displacementBias;

		}

	}

	function refreshUniformsDepth( uniforms, material ) {

		if ( material.displacementMap ) {

			uniforms.displacementMap.value = material.displacementMap;
			uniforms.displacementScale.value = material.displacementScale;
			uniforms.displacementBias.value = material.displacementBias;

		}

	}

	function refreshUniformsDistance( uniforms, material ) {

		if ( material.displacementMap ) {

			uniforms.displacementMap.value = material.displacementMap;
			uniforms.displacementScale.value = material.displacementScale;
			uniforms.displacementBias.value = material.displacementBias;

		}

		uniforms.referencePosition.value.copy( material.referencePosition );
		uniforms.nearDistance.value = material.nearDistance;
		uniforms.farDistance.value = material.farDistance;

	}

	function refreshUniformsNormal( uniforms, material ) {

		if ( material.bumpMap ) {

			uniforms.bumpMap.value = material.bumpMap;
			uniforms.bumpScale.value = material.bumpScale;
			if ( material.side === BackSide ) uniforms.bumpScale.value *= - 1;

		}

		if ( material.normalMap ) {

			uniforms.normalMap.value = material.normalMap;
			uniforms.normalScale.value.copy( material.normalScale );
			if ( material.side === BackSide ) uniforms.normalScale.value.negate();

		}

		if ( material.displacementMap ) {

			uniforms.displacementMap.value = material.displacementMap;
			uniforms.displacementScale.value = material.displacementScale;
			uniforms.displacementBias.value = material.displacementBias;

		}

	}

	return {
		refreshFogUniforms: refreshFogUniforms,
		refreshMaterialUniforms: refreshMaterialUniforms
	};

}

function createCanvasElement() {

	const canvas = document.createElementNS( 'http://www.w3.org/1999/xhtml', 'canvas' );
	canvas.style.display = 'block';
	return canvas;

}

function WebGLRenderer( parameters ) {

	parameters = parameters || {};

	const _canvas = parameters.canvas !== undefined ? parameters.canvas : createCanvasElement(),
		_context = parameters.context !== undefined ? parameters.context : null,

		_alpha = parameters.alpha !== undefined ? parameters.alpha : false,
		_depth = parameters.depth !== undefined ? parameters.depth : true,
		_stencil = parameters.stencil !== undefined ? parameters.stencil : true,
		_antialias = parameters.antialias !== undefined ? parameters.antialias : false,
		_premultipliedAlpha = parameters.premultipliedAlpha !== undefined ? parameters.premultipliedAlpha : true,
		_preserveDrawingBuffer = parameters.preserveDrawingBuffer !== undefined ? parameters.preserveDrawingBuffer : false,
		_powerPreference = parameters.powerPreference !== undefined ? parameters.powerPreference : 'default',
		_failIfMajorPerformanceCaveat = parameters.failIfMajorPerformanceCaveat !== undefined ? parameters.failIfMajorPerformanceCaveat : false;

	let currentRenderList = null;
	let currentRenderState = null;

	// render() can be called from within a callback triggered by another render.
	// We track this so that the nested render call gets its list and state isolated from the parent render call.

	const renderListStack = [];
	const renderStateStack = [];

	// public properties

	this.domElement = _canvas;

	// Debug configuration container
	this.debug = {

		/**
		 * Enables error checking and reporting when shader programs are being compiled
		 * @type {boolean}
		 */
		checkShaderErrors: true
	};

	// clearing

	this.autoClear = true;
	this.autoClearColor = true;
	this.autoClearDepth = true;
	this.autoClearStencil = true;

	// scene graph

	this.sortObjects = true;

	// user-defined clipping

	this.clippingPlanes = [];
	this.localClippingEnabled = false;

	// physically based shading

	this.gammaFactor = 2.0;	// for backwards compatibility
	this.outputEncoding = LinearEncoding;

	// physical lights

	this.physicallyCorrectLights = false;

	// tone mapping

	this.toneMapping = NoToneMapping;
	this.toneMappingExposure = 1.0;

	// internal properties

	const _this = this;

	let _isContextLost = false;

	// internal state cache

	let _currentActiveCubeFace = 0;
	let _currentActiveMipmapLevel = 0;
	let _currentRenderTarget = null;
	let _currentMaterialId = - 1;

	let _currentCamera = null;

	const _currentViewport = new Vector4();
	const _currentScissor = new Vector4();
	let _currentScissorTest = null;

	//

	let _width = _canvas.width;
	let _height = _canvas.height;

	let _pixelRatio = 1;
	let _opaqueSort = null;
	let _transparentSort = null;

	const _viewport = new Vector4( 0, 0, _width, _height );
	const _scissor = new Vector4( 0, 0, _width, _height );
	let _scissorTest = false;

	// frustum

	const _frustum = new Frustum();

	// clipping

	let _clippingEnabled = false;
	let _localClippingEnabled = false;

	// camera matrices cache

	const _projScreenMatrix = new Matrix4();

	const _vector3 = new Vector3();

	const _emptyScene = { background: null, fog: null, environment: null, overrideMaterial: null, isScene: true };

	function getTargetPixelRatio() {

		return _currentRenderTarget === null ? _pixelRatio : 1;

	}

	// initialize

	let _gl = _context;

	function getContext( contextNames, contextAttributes ) {

		for ( let i = 0; i < contextNames.length; i ++ ) {

			const contextName = contextNames[ i ];
			const context = _canvas.getContext( contextName, contextAttributes );
			if ( context !== null ) return context;

		}

		return null;

	}

	try {

		const contextAttributes = {
			alpha: _alpha,
			depth: _depth,
			stencil: _stencil,
			antialias: _antialias,
			premultipliedAlpha: _premultipliedAlpha,
			preserveDrawingBuffer: _preserveDrawingBuffer,
			powerPreference: _powerPreference,
			failIfMajorPerformanceCaveat: _failIfMajorPerformanceCaveat
		};

		// event listeners must be registered before WebGL context is created, see #12753

		_canvas.addEventListener( 'webglcontextlost', onContextLost, false );
		_canvas.addEventListener( 'webglcontextrestored', onContextRestore, false );

		if ( _gl === null ) {

			const contextNames = [ 'webgl2', 'webgl', 'experimental-webgl' ];

			if ( _this.isWebGL1Renderer === true ) {

				contextNames.shift();

			}

			_gl = getContext( contextNames, contextAttributes );

			if ( _gl === null ) {

				if ( getContext( contextNames ) ) {

					throw new Error( 'Error creating WebGL context with your selected attributes.' );

				} else {

					throw new Error( 'Error creating WebGL context.' );

				}

			}

		}

		// Some experimental-webgl implementations do not have getShaderPrecisionFormat

		if ( _gl.getShaderPrecisionFormat === undefined ) {

			_gl.getShaderPrecisionFormat = function () {

				return { 'rangeMin': 1, 'rangeMax': 1, 'precision': 1 };

			};

		}

	} catch ( error ) {

		console.error( 'THREE.WebGLRenderer: ' + error.message );
		throw error;

	}

	let extensions, capabilities, state, info;
	let properties, textures, cubemaps, attributes, geometries, objects;
	let programCache, materials, renderLists, renderStates, clipping, shadowMap;

	let background, morphtargets, bufferRenderer, indexedBufferRenderer;

	let utils, bindingStates;

	function initGLContext() {

		extensions = new WebGLExtensions( _gl );

		capabilities = new WebGLCapabilities( _gl, extensions, parameters );

		extensions.init( capabilities );

		utils = new WebGLUtils( _gl, extensions, capabilities );

		state = new WebGLState( _gl, extensions, capabilities );

		info = new WebGLInfo( _gl );
		properties = new WebGLProperties();
		textures = new WebGLTextures( _gl, extensions, state, properties, capabilities, utils, info );
		cubemaps = new WebGLCubeMaps( _this );
		attributes = new WebGLAttributes( _gl, capabilities );
		bindingStates = new WebGLBindingStates( _gl, extensions, attributes, capabilities );
		geometries = new WebGLGeometries( _gl, attributes, info, bindingStates );
		objects = new WebGLObjects( _gl, geometries, attributes, info );
		morphtargets = new WebGLMorphtargets( _gl );
		clipping = new WebGLClipping( properties );
		programCache = new WebGLPrograms( _this, cubemaps, extensions, capabilities, bindingStates, clipping );
		materials = new WebGLMaterials( properties );
		renderLists = new WebGLRenderLists( properties );
		renderStates = new WebGLRenderStates( extensions, capabilities );
		background = new WebGLBackground( _this, cubemaps, state, objects, _premultipliedAlpha );
		shadowMap = new WebGLShadowMap( _this, objects, capabilities );

		bufferRenderer = new WebGLBufferRenderer( _gl, extensions, info, capabilities );
		indexedBufferRenderer = new WebGLIndexedBufferRenderer( _gl, extensions, info, capabilities );

		info.programs = programCache.programs;

		_this.capabilities = capabilities;
		_this.extensions = extensions;
		_this.properties = properties;
		_this.renderLists = renderLists;
		_this.shadowMap = shadowMap;
		_this.state = state;
		_this.info = info;

	}

	initGLContext();

	// xr

	const xr = new WebXRManager( _this, _gl );

	this.xr = xr;

	// API

	this.getContext = function () {

		return _gl;

	};

	this.getContextAttributes = function () {

		return _gl.getContextAttributes();

	};

	this.forceContextLoss = function () {

		const extension = extensions.get( 'WEBGL_lose_context' );
		if ( extension ) extension.loseContext();

	};

	this.forceContextRestore = function () {

		const extension = extensions.get( 'WEBGL_lose_context' );
		if ( extension ) extension.restoreContext();

	};

	this.getPixelRatio = function () {

		return _pixelRatio;

	};

	this.setPixelRatio = function ( value ) {

		if ( value === undefined ) return;

		_pixelRatio = value;

		this.setSize( _width, _height, false );

	};

	this.getSize = function ( target ) {

		if ( target === undefined ) {

			console.warn( 'WebGLRenderer: .getsize() now requires a Vector2 as an argument' );

			target = new Vector2();

		}

		return target.set( _width, _height );

	};

	this.setSize = function ( width, height, updateStyle ) {

		if ( xr.isPresenting ) {

			console.warn( 'THREE.WebGLRenderer: Can\'t change size while VR device is presenting.' );
			return;

		}

		_width = width;
		_height = height;

		_canvas.width = Math.floor( width * _pixelRatio );
		_canvas.height = Math.floor( height * _pixelRatio );

		if ( updateStyle !== false ) {

			_canvas.style.width = width + 'px';
			_canvas.style.height = height + 'px';

		}

		this.setViewport( 0, 0, width, height );

	};

	this.getDrawingBufferSize = function ( target ) {

		if ( target === undefined ) {

			console.warn( 'WebGLRenderer: .getdrawingBufferSize() now requires a Vector2 as an argument' );

			target = new Vector2();

		}

		return target.set( _width * _pixelRatio, _height * _pixelRatio ).floor();

	};

	this.setDrawingBufferSize = function ( width, height, pixelRatio ) {

		_width = width;
		_height = height;

		_pixelRatio = pixelRatio;

		_canvas.width = Math.floor( width * pixelRatio );
		_canvas.height = Math.floor( height * pixelRatio );

		this.setViewport( 0, 0, width, height );

	};

	this.getCurrentViewport = function ( target ) {

		if ( target === undefined ) {

			console.warn( 'WebGLRenderer: .getCurrentViewport() now requires a Vector4 as an argument' );

			target = new Vector4();

		}

		return target.copy( _currentViewport );

	};

	this.getViewport = function ( target ) {

		return target.copy( _viewport );

	};

	this.setViewport = function ( x, y, width, height ) {

		if ( x.isVector4 ) {

			_viewport.set( x.x, x.y, x.z, x.w );

		} else {

			_viewport.set( x, y, width, height );

		}

		state.viewport( _currentViewport.copy( _viewport ).multiplyScalar( _pixelRatio ).floor() );

	};

	this.getScissor = function ( target ) {

		return target.copy( _scissor );

	};

	this.setScissor = function ( x, y, width, height ) {

		if ( x.isVector4 ) {

			_scissor.set( x.x, x.y, x.z, x.w );

		} else {

			_scissor.set( x, y, width, height );

		}

		state.scissor( _currentScissor.copy( _scissor ).multiplyScalar( _pixelRatio ).floor() );

	};

	this.getScissorTest = function () {

		return _scissorTest;

	};

	this.setScissorTest = function ( boolean ) {

		state.setScissorTest( _scissorTest = boolean );

	};

	this.setOpaqueSort = function ( method ) {

		_opaqueSort = method;

	};

	this.setTransparentSort = function ( method ) {

		_transparentSort = method;

	};

	// Clearing

	this.getClearColor = function ( target ) {

		if ( target === undefined ) {

			console.warn( 'WebGLRenderer: .getClearColor() now requires a Color as an argument' );

			target = new Color();

		}

		return target.copy( background.getClearColor() );

	};

	this.setClearColor = function () {

		background.setClearColor.apply( background, arguments );

	};

	this.getClearAlpha = function () {

		return background.getClearAlpha();

	};

	this.setClearAlpha = function () {

		background.setClearAlpha.apply( background, arguments );

	};

	this.clear = function ( color, depth, stencil ) {

		let bits = 0;

		if ( color === undefined || color ) bits |= 16384;
		if ( depth === undefined || depth ) bits |= 256;
		if ( stencil === undefined || stencil ) bits |= 1024;

		_gl.clear( bits );

	};

	this.clearColor = function () {

		this.clear( true, false, false );

	};

	this.clearDepth = function () {

		this.clear( false, true, false );

	};

	this.clearStencil = function () {

		this.clear( false, false, true );

	};

	//

	this.dispose = function () {

		_canvas.removeEventListener( 'webglcontextlost', onContextLost, false );
		_canvas.removeEventListener( 'webglcontextrestored', onContextRestore, false );

		renderLists.dispose();
		renderStates.dispose();
		properties.dispose();
		cubemaps.dispose();
		objects.dispose();
		bindingStates.dispose();

		xr.dispose();

		xr.removeEventListener( 'sessionstart', onXRSessionStart );
		xr.removeEventListener( 'sessionend', onXRSessionEnd );

		animation.stop();

	};

	// Events

	function onContextLost( event ) {

		event.preventDefault();

		console.log( 'THREE.WebGLRenderer: Context Lost.' );

		_isContextLost = true;

	}

	function onContextRestore( /* event */ ) {

		console.log( 'THREE.WebGLRenderer: Context Restored.' );

		_isContextLost = false;

		const infoAutoReset = info.autoReset;
		const shadowMapEnabled = shadowMap.enabled;
		const shadowMapAutoUpdate = shadowMap.autoUpdate;
		const shadowMapNeedsUpdate = shadowMap.needsUpdate;
		const shadowMapType = shadowMap.type;

		initGLContext();

		info.autoReset = infoAutoReset;
		shadowMap.enabled = shadowMapEnabled;
		shadowMap.autoUpdate = shadowMapAutoUpdate;
		shadowMap.needsUpdate = shadowMapNeedsUpdate;
		shadowMap.type = shadowMapType;

	}

	function onMaterialDispose( event ) {

		const material = event.target;

		material.removeEventListener( 'dispose', onMaterialDispose );

		deallocateMaterial( material );

	}

	// Buffer deallocation

	function deallocateMaterial( material ) {

		releaseMaterialProgramReferences( material );

		properties.remove( material );

	}


	function releaseMaterialProgramReferences( material ) {

		const programs = properties.get( material ).programs;

		if ( programs !== undefined ) {

			programs.forEach( function ( program ) {

				programCache.releaseProgram( program );

			} );

		}

	}

	// Buffer rendering

	function renderObjectImmediate( object, program ) {

		object.render( function ( object ) {

			_this.renderBufferImmediate( object, program );

		} );

	}

	this.renderBufferImmediate = function ( object, program ) {

		bindingStates.initAttributes();

		const buffers = properties.get( object );

		if ( object.hasPositions && ! buffers.position ) buffers.position = _gl.createBuffer();
		if ( object.hasNormals && ! buffers.normal ) buffers.normal = _gl.createBuffer();
		if ( object.hasUvs && ! buffers.uv ) buffers.uv = _gl.createBuffer();
		if ( object.hasColors && ! buffers.color ) buffers.color = _gl.createBuffer();

		const programAttributes = program.getAttributes();

		if ( object.hasPositions ) {

			_gl.bindBuffer( 34962, buffers.position );
			_gl.bufferData( 34962, object.positionArray, 35048 );

			bindingStates.enableAttribute( programAttributes.position );
			_gl.vertexAttribPointer( programAttributes.position, 3, 5126, false, 0, 0 );

		}

		if ( object.hasNormals ) {

			_gl.bindBuffer( 34962, buffers.normal );
			_gl.bufferData( 34962, object.normalArray, 35048 );

			bindingStates.enableAttribute( programAttributes.normal );
			_gl.vertexAttribPointer( programAttributes.normal, 3, 5126, false, 0, 0 );

		}

		if ( object.hasUvs ) {

			_gl.bindBuffer( 34962, buffers.uv );
			_gl.bufferData( 34962, object.uvArray, 35048 );

			bindingStates.enableAttribute( programAttributes.uv );
			_gl.vertexAttribPointer( programAttributes.uv, 2, 5126, false, 0, 0 );

		}

		if ( object.hasColors ) {

			_gl.bindBuffer( 34962, buffers.color );
			_gl.bufferData( 34962, object.colorArray, 35048 );

			bindingStates.enableAttribute( programAttributes.color );
			_gl.vertexAttribPointer( programAttributes.color, 3, 5126, false, 0, 0 );

		}

		bindingStates.disableUnusedAttributes();

		_gl.drawArrays( 4, 0, object.count );

		object.count = 0;

	};

	this.renderBufferDirect = function ( camera, scene, geometry, material, object, group ) {

		if ( scene === null ) scene = _emptyScene; // renderBufferDirect second parameter used to be fog (could be null)

		const frontFaceCW = ( object.isMesh && object.matrixWorld.determinant() < 0 );

		const program = setProgram( camera, scene, material, object );

		state.setMaterial( material, frontFaceCW );

		//

		let index = geometry.index;
		const position = geometry.attributes.position;

		//

		if ( index === null ) {

			if ( position === undefined || position.count === 0 ) return;

		} else if ( index.count === 0 ) {

			return;

		}

		//

		let rangeFactor = 1;

		if ( material.wireframe === true ) {

			index = geometries.getWireframeAttribute( geometry );
			rangeFactor = 2;

		}

		if ( material.morphTargets || material.morphNormals ) {

			morphtargets.update( object, geometry, material, program );

		}

		bindingStates.setup( object, material, program, geometry, index );

		let attribute;
		let renderer = bufferRenderer;

		if ( index !== null ) {

			attribute = attributes.get( index );

			renderer = indexedBufferRenderer;
			renderer.setIndex( attribute );

		}

		//

		const dataCount = ( index !== null ) ? index.count : position.count;

		const rangeStart = geometry.drawRange.start * rangeFactor;
		const rangeCount = geometry.drawRange.count * rangeFactor;

		const groupStart = group !== null ? group.start * rangeFactor : 0;
		const groupCount = group !== null ? group.count * rangeFactor : Infinity;

		const drawStart = Math.max( rangeStart, groupStart );
		const drawEnd = Math.min( dataCount, rangeStart + rangeCount, groupStart + groupCount ) - 1;

		const drawCount = Math.max( 0, drawEnd - drawStart + 1 );

		if ( drawCount === 0 ) return;

		//

		if ( object.isMesh ) {

			if ( material.wireframe === true ) {

				state.setLineWidth( material.wireframeLinewidth * getTargetPixelRatio() );
				renderer.setMode( 1 );

			} else {

				renderer.setMode( 4 );

			}

		} else if ( object.isLine ) {

			let lineWidth = material.linewidth;

			if ( lineWidth === undefined ) lineWidth = 1; // Not using Line*Material

			state.setLineWidth( lineWidth * getTargetPixelRatio() );

			if ( object.isLineSegments ) {

				renderer.setMode( 1 );

			} else if ( object.isLineLoop ) {

				renderer.setMode( 2 );

			} else {

				renderer.setMode( 3 );

			}

		} else if ( object.isPoints ) {

			renderer.setMode( 0 );

		} else if ( object.isSprite ) {

			renderer.setMode( 4 );

		}

		if ( object.isInstancedMesh ) {

			renderer.renderInstances( drawStart, drawCount, object.count );

		} else if ( geometry.isInstancedBufferGeometry ) {

			const instanceCount = Math.min( geometry.instanceCount, geometry._maxInstanceCount );

			renderer.renderInstances( drawStart, drawCount, instanceCount );

		} else {

			renderer.render( drawStart, drawCount );

		}

	};

	// Compile

	this.compile = function ( scene, camera ) {

		currentRenderState = renderStates.get( scene );
		currentRenderState.init();

		scene.traverseVisible( function ( object ) {

			if ( object.isLight && object.layers.test( camera.layers ) ) {

				currentRenderState.pushLight( object );

				if ( object.castShadow ) {

					currentRenderState.pushShadow( object );

				}

			}

		} );

		currentRenderState.setupLights();

		scene.traverse( function ( object ) {

			const material = object.material;

			if ( material ) {

				if ( Array.isArray( material ) ) {

					for ( let i = 0; i < material.length; i ++ ) {

						const material2 = material[ i ];

						getProgram( material2, scene, object );

					}

				} else {

					getProgram( material, scene, object );

				}

			}

		} );

	};

	// Animation Loop

	let onAnimationFrameCallback = null;

	function onAnimationFrame( time ) {

		if ( onAnimationFrameCallback ) onAnimationFrameCallback( time );

	}

	function onXRSessionStart() {

		animation.stop();

	}

	function onXRSessionEnd() {

		animation.start();

	}

	const animation = new WebGLAnimation();
	animation.setAnimationLoop( onAnimationFrame );

	if ( typeof window !== 'undefined' ) animation.setContext( window );

	this.setAnimationLoop = function ( callback ) {

		onAnimationFrameCallback = callback;
		xr.setAnimationLoop( callback );

		( callback === null ) ? animation.stop() : animation.start();

	};

	xr.addEventListener( 'sessionstart', onXRSessionStart );
	xr.addEventListener( 'sessionend', onXRSessionEnd );

	// Rendering

	this.render = function ( scene, camera ) {

		let renderTarget, forceClear;

		if ( arguments[ 2 ] !== undefined ) {

			console.warn( 'THREE.WebGLRenderer.render(): the renderTarget argument has been removed. Use .setRenderTarget() instead.' );
			renderTarget = arguments[ 2 ];

		}

		if ( arguments[ 3 ] !== undefined ) {

			console.warn( 'THREE.WebGLRenderer.render(): the forceClear argument has been removed. Use .clear() instead.' );
			forceClear = arguments[ 3 ];

		}

		if ( camera !== undefined && camera.isCamera !== true ) {

			console.error( 'THREE.WebGLRenderer.render: camera is not an instance of THREE.Camera.' );
			return;

		}

		if ( _isContextLost === true ) return;

		// update scene graph

		if ( scene.autoUpdate === true ) scene.updateMatrixWorld();

		// update camera matrices and frustum

		if ( camera.parent === null ) camera.updateMatrixWorld();

		if ( xr.enabled === true && xr.isPresenting === true ) {

			camera = xr.getCamera( camera );

		}

		//
		if ( scene.isScene === true ) scene.onBeforeRender( _this, scene, camera, renderTarget || _currentRenderTarget );

		currentRenderState = renderStates.get( scene, renderStateStack.length );
		currentRenderState.init();

		renderStateStack.push( currentRenderState );

		_projScreenMatrix.multiplyMatrices( camera.projectionMatrix, camera.matrixWorldInverse );
		_frustum.setFromProjectionMatrix( _projScreenMatrix );

		_localClippingEnabled = this.localClippingEnabled;
		_clippingEnabled = clipping.init( this.clippingPlanes, _localClippingEnabled, camera );

		currentRenderList = renderLists.get( scene, renderListStack.length );
		currentRenderList.init();

		renderListStack.push( currentRenderList );

		projectObject( scene, camera, 0, _this.sortObjects );

		currentRenderList.finish();

		if ( _this.sortObjects === true ) {

			currentRenderList.sort( _opaqueSort, _transparentSort );

		}

		//

		if ( _clippingEnabled === true ) clipping.beginShadows();

		const shadowsArray = currentRenderState.state.shadowsArray;

		shadowMap.render( shadowsArray, scene, camera );

		currentRenderState.setupLights();
		currentRenderState.setupLightsView( camera );

		if ( _clippingEnabled === true ) clipping.endShadows();

		//

		if ( this.info.autoReset === true ) this.info.reset();

		if ( renderTarget !== undefined ) {

			this.setRenderTarget( renderTarget );

		}

		//

		background.render( currentRenderList, scene, camera, forceClear );

		// render scene

		const opaqueObjects = currentRenderList.opaque;
		const transparentObjects = currentRenderList.transparent;

		if ( opaqueObjects.length > 0 ) renderObjects( opaqueObjects, scene, camera );
		if ( transparentObjects.length > 0 ) renderObjects( transparentObjects, scene, camera );

		//

		if ( _currentRenderTarget !== null ) {

			// Generate mipmap if we're using any kind of mipmap filtering

			textures.updateRenderTargetMipmap( _currentRenderTarget );

			// resolve multisample renderbuffers to a single-sample texture if necessary

			textures.updateMultisampleRenderTarget( _currentRenderTarget );

		}

		//

		if ( scene.isScene === true ) scene.onAfterRender( _this, scene, camera );

		// Ensure depth buffer writing is enabled so it can be cleared on next render

		state.buffers.depth.setTest( true );
		state.buffers.depth.setMask( true );
		state.buffers.color.setMask( true );

		state.setPolygonOffset( false );

		// _gl.finish();

		bindingStates.resetDefaultState();
		_currentMaterialId = - 1;
		_currentCamera = null;

		renderStateStack.pop();

		if ( renderStateStack.length > 0 ) {

			currentRenderState = renderStateStack[ renderStateStack.length - 1 ];

		} else {

			currentRenderState = null;

		}

		renderListStack.pop();

		if ( renderListStack.length > 0 ) {

			currentRenderList = renderListStack[ renderListStack.length - 1 ];

		} else {

			currentRenderList = null;

		}

	};

	function projectObject( object, camera, groupOrder, sortObjects ) {

		if ( object.visible === false ) return;

		const visible = object.layers.test( camera.layers );

		if ( visible ) {

			if ( object.isGroup ) {

				groupOrder = object.renderOrder;

			} else if ( object.isLOD ) {

				if ( object.autoUpdate === true ) object.update( camera );

			} else if ( object.isLight ) {

				currentRenderState.pushLight( object );

				if ( object.castShadow ) {

					currentRenderState.pushShadow( object );

				}

			} else if ( object.isSprite ) {

				if ( ! object.frustumCulled || _frustum.intersectsSprite( object ) ) {

					if ( sortObjects ) {

						_vector3.setFromMatrixPosition( object.matrixWorld )
							.applyMatrix4( _projScreenMatrix );

					}

					const geometry = objects.update( object );
					const material = object.material;

					if ( material.visible ) {

						currentRenderList.push( object, geometry, material, groupOrder, _vector3.z, null );

					}

				}

			} else if ( object.isImmediateRenderObject ) {

				if ( sortObjects ) {

					_vector3.setFromMatrixPosition( object.matrixWorld )
						.applyMatrix4( _projScreenMatrix );

				}

				currentRenderList.push( object, null, object.material, groupOrder, _vector3.z, null );

			} else if ( object.isMesh || object.isLine || object.isPoints ) {

				if ( object.isSkinnedMesh ) {

					// update skeleton only once in a frame

					if ( object.skeleton.frame !== info.render.frame ) {

						object.skeleton.update();
						object.skeleton.frame = info.render.frame;

					}

				}

				if ( ! object.frustumCulled || _frustum.intersectsObject( object ) ) {

					if ( sortObjects ) {

						_vector3.setFromMatrixPosition( object.matrixWorld )
							.applyMatrix4( _projScreenMatrix );

					}

					const geometry = objects.update( object );
					const material = object.material;

					if ( Array.isArray( material ) ) {

						const groups = geometry.groups;

						for ( let i = 0, l = groups.length; i < l; i ++ ) {

							const group = groups[ i ];
							const groupMaterial = material[ group.materialIndex ];

							if ( groupMaterial && groupMaterial.visible ) {

								currentRenderList.push( object, geometry, groupMaterial, groupOrder, _vector3.z, group );

							}

						}

					} else if ( material.visible ) {

						currentRenderList.push( object, geometry, material, groupOrder, _vector3.z, null );

					}

				}

			}

		}

		const children = object.children;

		for ( let i = 0, l = children.length; i < l; i ++ ) {

			projectObject( children[ i ], camera, groupOrder, sortObjects );

		}

	}

	function renderObjects( renderList, scene, camera ) {

		const overrideMaterial = scene.isScene === true ? scene.overrideMaterial : null;

		for ( let i = 0, l = renderList.length; i < l; i ++ ) {

			const renderItem = renderList[ i ];

			const object = renderItem.object;
			const geometry = renderItem.geometry;
			const material = overrideMaterial === null ? renderItem.material : overrideMaterial;
			const group = renderItem.group;

			if ( camera.isArrayCamera ) {

				const cameras = camera.cameras;

				for ( let j = 0, jl = cameras.length; j < jl; j ++ ) {

					const camera2 = cameras[ j ];

					if ( object.layers.test( camera2.layers ) ) {

						state.viewport( _currentViewport.copy( camera2.viewport ) );

						currentRenderState.setupLightsView( camera2 );

						renderObject( object, scene, camera2, geometry, material, group );

					}

				}

			} else {

				renderObject( object, scene, camera, geometry, material, group );

			}

		}

	}

	function renderObject( object, scene, camera, geometry, material, group ) {

		object.onBeforeRender( _this, scene, camera, geometry, material, group );

		object.modelViewMatrix.multiplyMatrices( camera.matrixWorldInverse, object.matrixWorld );
		object.normalMatrix.getNormalMatrix( object.modelViewMatrix );

		if ( object.isImmediateRenderObject ) {

			const program = setProgram( camera, scene, material, object );

			state.setMaterial( material );

			bindingStates.reset();

			renderObjectImmediate( object, program );

		} else {

			_this.renderBufferDirect( camera, scene, geometry, material, object, group );

		}

		object.onAfterRender( _this, scene, camera, geometry, material, group );

	}

	function getProgram( material, scene, object ) {

		if ( scene.isScene !== true ) scene = _emptyScene; // scene could be a Mesh, Line, Points, ...

		const materialProperties = properties.get( material );

		const lights = currentRenderState.state.lights;
		const shadowsArray = currentRenderState.state.shadowsArray;

		const lightsStateVersion = lights.state.version;

		const parameters = programCache.getParameters( material, lights.state, shadowsArray, scene, object );
		const programCacheKey = programCache.getProgramCacheKey( parameters );

		let programs = materialProperties.programs;

		// always update environment and fog - changing these trigger an getProgram call, but it's possible that the program doesn't change

		materialProperties.environment = material.isMeshStandardMaterial ? scene.environment : null;
		materialProperties.fog = scene.fog;
		materialProperties.envMap = cubemaps.get( material.envMap || materialProperties.environment );

		if ( programs === undefined ) {

			// new material

			material.addEventListener( 'dispose', onMaterialDispose );

			programs = new Map();
			materialProperties.programs = programs;

		}

		let program = programs.get( programCacheKey );

		if ( program !== undefined ) {

			// early out if program and light state is identical

			if ( materialProperties.currentProgram === program && materialProperties.lightsStateVersion === lightsStateVersion ) {

				updateCommonMaterialProperties( material, parameters );

				return program;

			}

		} else {

			parameters.uniforms = programCache.getUniforms( material );

			material.onBeforeCompile( parameters, _this );

			program = programCache.acquireProgram( parameters, programCacheKey );
			programs.set( programCacheKey, program );

			materialProperties.uniforms = parameters.uniforms;

		}

		const uniforms = materialProperties.uniforms;

		if ( ( ! material.isShaderMaterial && ! material.isRawShaderMaterial ) || material.clipping === true ) {

			uniforms.clippingPlanes = clipping.uniform;

		}

		updateCommonMaterialProperties( material, parameters );

		// store the light setup it was created for

		materialProperties.needsLights = materialNeedsLights( material );
		materialProperties.lightsStateVersion = lightsStateVersion;

		if ( materialProperties.needsLights ) {

			// wire up the material to this renderer's lighting state

			uniforms.ambientLightColor.value = lights.state.ambient;
			uniforms.lightProbe.value = lights.state.probe;
			uniforms.directionalLights.value = lights.state.directional;
			uniforms.directionalLightShadows.value = lights.state.directionalShadow;
			uniforms.spotLights.value = lights.state.spot;
			uniforms.spotLightShadows.value = lights.state.spotShadow;
			uniforms.rectAreaLights.value = lights.state.rectArea;
			uniforms.ltc_1.value = lights.state.rectAreaLTC1;
			uniforms.ltc_2.value = lights.state.rectAreaLTC2;
			uniforms.pointLights.value = lights.state.point;
			uniforms.pointLightShadows.value = lights.state.pointShadow;
			uniforms.hemisphereLights.value = lights.state.hemi;

			uniforms.directionalShadowMap.value = lights.state.directionalShadowMap;
			uniforms.directionalShadowMatrix.value = lights.state.directionalShadowMatrix;
			uniforms.spotShadowMap.value = lights.state.spotShadowMap;
			uniforms.spotShadowMatrix.value = lights.state.spotShadowMatrix;
			uniforms.pointShadowMap.value = lights.state.pointShadowMap;
			uniforms.pointShadowMatrix.value = lights.state.pointShadowMatrix;
			// TODO (abelnation): add area lights shadow info to uniforms

		}

		const progUniforms = program.getUniforms();
		const uniformsList = WebGLUniforms.seqWithValue( progUniforms.seq, uniforms );

		materialProperties.currentProgram = program;
		materialProperties.uniformsList = uniformsList;

		return program;

	}

	function updateCommonMaterialProperties( material, parameters ) {

		const materialProperties = properties.get( material );

		materialProperties.outputEncoding = parameters.outputEncoding;
		materialProperties.instancing = parameters.instancing;
		materialProperties.numClippingPlanes = parameters.numClippingPlanes;
		materialProperties.numIntersection = parameters.numClipIntersection;
		materialProperties.vertexAlphas = parameters.vertexAlphas;

	}

	function setProgram( camera, scene, material, object ) {

		if ( scene.isScene !== true ) scene = _emptyScene; // scene could be a Mesh, Line, Points, ...

		textures.resetTextureUnits();

		const fog = scene.fog;
		const environment = material.isMeshStandardMaterial ? scene.environment : null;
		const encoding = ( _currentRenderTarget === null ) ? _this.outputEncoding : _currentRenderTarget.texture.encoding;
		const envMap = cubemaps.get( material.envMap || environment );
		const vertexAlphas = material.vertexColors === true && object.geometry.attributes.color && object.geometry.attributes.color.itemSize === 4;

		const materialProperties = properties.get( material );
		const lights = currentRenderState.state.lights;

		if ( _clippingEnabled === true ) {

			if ( _localClippingEnabled === true || camera !== _currentCamera ) {

				const useCache =
					camera === _currentCamera &&
					material.id === _currentMaterialId;

				// we might want to call this function with some ClippingGroup
				// object instead of the material, once it becomes feasible
				// (#8465, #8379)
				clipping.setState( material, camera, useCache );

			}

		}

		//

		let needsProgramChange = false;

		if ( material.version === materialProperties.__version ) {

			if ( materialProperties.needsLights && ( materialProperties.lightsStateVersion !== lights.state.version ) ) {

				needsProgramChange = true;

			} else if ( materialProperties.outputEncoding !== encoding ) {

				needsProgramChange = true;

			} else if ( object.isInstancedMesh && materialProperties.instancing === false ) {

				needsProgramChange = true;

			} else if ( ! object.isInstancedMesh && materialProperties.instancing === true ) {

				needsProgramChange = true;

			} else if ( materialProperties.envMap !== envMap ) {

				needsProgramChange = true;

			} else if ( material.fog && materialProperties.fog !== fog ) {

				needsProgramChange = true;

			} else if ( materialProperties.numClippingPlanes !== undefined &&
				( materialProperties.numClippingPlanes !== clipping.numPlanes ||
				materialProperties.numIntersection !== clipping.numIntersection ) ) {

				needsProgramChange = true;

			} else if ( materialProperties.vertexAlphas !== vertexAlphas ) {

				needsProgramChange = true;

			}

		} else {

			needsProgramChange = true;
			materialProperties.__version = material.version;

		}

		//

		let program = materialProperties.currentProgram;

		if ( needsProgramChange === true ) {

			program = getProgram( material, scene, object );

		}

		let refreshProgram = false;
		let refreshMaterial = false;
		let refreshLights = false;

		const p_uniforms = program.getUniforms(),
			m_uniforms = materialProperties.uniforms;

		if ( state.useProgram( program.program ) ) {

			refreshProgram = true;
			refreshMaterial = true;
			refreshLights = true;

		}

		if ( material.id !== _currentMaterialId ) {

			_currentMaterialId = material.id;

			refreshMaterial = true;

		}

		if ( refreshProgram || _currentCamera !== camera ) {

			p_uniforms.setValue( _gl, 'projectionMatrix', camera.projectionMatrix );

			if ( capabilities.logarithmicDepthBuffer ) {

				p_uniforms.setValue( _gl, 'logDepthBufFC',
					2.0 / ( Math.log( camera.far + 1.0 ) / Math.LN2 ) );

			}

			if ( _currentCamera !== camera ) {

				_currentCamera = camera;

				// lighting uniforms depend on the camera so enforce an update
				// now, in case this material supports lights - or later, when
				// the next material that does gets activated:

				refreshMaterial = true;		// set to true on material change
				refreshLights = true;		// remains set until update done

			}

			// load material specific uniforms
			// (shader material also gets them for the sake of genericity)

			if ( material.isShaderMaterial ||
				material.isMeshPhongMaterial ||
				material.isMeshToonMaterial ||
				material.isMeshStandardMaterial ||
				material.envMap ) {

				const uCamPos = p_uniforms.map.cameraPosition;

				if ( uCamPos !== undefined ) {

					uCamPos.setValue( _gl,
						_vector3.setFromMatrixPosition( camera.matrixWorld ) );

				}

			}

			if ( material.isMeshPhongMaterial ||
				material.isMeshToonMaterial ||
				material.isMeshLambertMaterial ||
				material.isMeshBasicMaterial ||
				material.isMeshStandardMaterial ||
				material.isShaderMaterial ) {

				p_uniforms.setValue( _gl, 'isOrthographic', camera.isOrthographicCamera === true );

			}

			if ( material.isMeshPhongMaterial ||
				material.isMeshToonMaterial ||
				material.isMeshLambertMaterial ||
				material.isMeshBasicMaterial ||
				material.isMeshStandardMaterial ||
				material.isShaderMaterial ||
				material.isShadowMaterial ||
				material.skinning ) {

				p_uniforms.setValue( _gl, 'viewMatrix', camera.matrixWorldInverse );

			}

		}

		// skinning uniforms must be set even if material didn't change
		// auto-setting of texture unit for bone texture must go before other textures
		// otherwise textures used for skinning can take over texture units reserved for other material textures

		if ( material.skinning ) {

			p_uniforms.setOptional( _gl, object, 'bindMatrix' );
			p_uniforms.setOptional( _gl, object, 'bindMatrixInverse' );

			const skeleton = object.skeleton;

			if ( skeleton ) {

				const bones = skeleton.bones;

				if ( capabilities.floatVertexTextures ) {

					if ( skeleton.boneTexture === null ) {

						// layout (1 matrix = 4 pixels)
						//      RGBA RGBA RGBA RGBA (=> column1, column2, column3, column4)
						//  with  8x8  pixel texture max   16 bones * 4 pixels =  (8 * 8)
						//       16x16 pixel texture max   64 bones * 4 pixels = (16 * 16)
						//       32x32 pixel texture max  256 bones * 4 pixels = (32 * 32)
						//       64x64 pixel texture max 1024 bones * 4 pixels = (64 * 64)


						let size = Math.sqrt( bones.length * 4 ); // 4 pixels needed for 1 matrix
						size = MathUtils.ceilPowerOfTwo( size );
						size = Math.max( size, 4 );

						const boneMatrices = new Float32Array( size * size * 4 ); // 4 floats per RGBA pixel
						boneMatrices.set( skeleton.boneMatrices ); // copy current values

						const boneTexture = new DataTexture( boneMatrices, size, size, RGBAFormat, FloatType );

						skeleton.boneMatrices = boneMatrices;
						skeleton.boneTexture = boneTexture;
						skeleton.boneTextureSize = size;

					}

					p_uniforms.setValue( _gl, 'boneTexture', skeleton.boneTexture, textures );
					p_uniforms.setValue( _gl, 'boneTextureSize', skeleton.boneTextureSize );

				} else {

					p_uniforms.setOptional( _gl, skeleton, 'boneMatrices' );

				}

			}

		}

		if ( refreshMaterial || materialProperties.receiveShadow !== object.receiveShadow ) {

			materialProperties.receiveShadow = object.receiveShadow;
			p_uniforms.setValue( _gl, 'receiveShadow', object.receiveShadow );

		}

		if ( refreshMaterial ) {

			p_uniforms.setValue( _gl, 'toneMappingExposure', _this.toneMappingExposure );

			if ( materialProperties.needsLights ) {

				// the current material requires lighting info

				// note: all lighting uniforms are always set correctly
				// they simply reference the renderer's state for their
				// values
				//
				// use the current material's .needsUpdate flags to set
				// the GL state when required

				markUniformsLightsNeedsUpdate( m_uniforms, refreshLights );

			}

			// refresh uniforms common to several materials

			if ( fog && material.fog ) {

				materials.refreshFogUniforms( m_uniforms, fog );

			}

			materials.refreshMaterialUniforms( m_uniforms, material, _pixelRatio, _height );

			WebGLUniforms.upload( _gl, materialProperties.uniformsList, m_uniforms, textures );

		}

		if ( material.isShaderMaterial && material.uniformsNeedUpdate === true ) {

			WebGLUniforms.upload( _gl, materialProperties.uniformsList, m_uniforms, textures );
			material.uniformsNeedUpdate = false;

		}

		if ( material.isSpriteMaterial ) {

			p_uniforms.setValue( _gl, 'center', object.center );

		}

		// common matrices

		p_uniforms.setValue( _gl, 'modelViewMatrix', object.modelViewMatrix );
		p_uniforms.setValue( _gl, 'normalMatrix', object.normalMatrix );
		p_uniforms.setValue( _gl, 'modelMatrix', object.matrixWorld );

		return program;

	}

	// If uniforms are marked as clean, they don't need to be loaded to the GPU.

	function markUniformsLightsNeedsUpdate( uniforms, value ) {

		uniforms.ambientLightColor.needsUpdate = value;
		uniforms.lightProbe.needsUpdate = value;

		uniforms.directionalLights.needsUpdate = value;
		uniforms.directionalLightShadows.needsUpdate = value;
		uniforms.pointLights.needsUpdate = value;
		uniforms.pointLightShadows.needsUpdate = value;
		uniforms.spotLights.needsUpdate = value;
		uniforms.spotLightShadows.needsUpdate = value;
		uniforms.rectAreaLights.needsUpdate = value;
		uniforms.hemisphereLights.needsUpdate = value;

	}

	function materialNeedsLights( material ) {

		return material.isMeshLambertMaterial || material.isMeshToonMaterial || material.isMeshPhongMaterial ||
			material.isMeshStandardMaterial || material.isShadowMaterial ||
			( material.isShaderMaterial && material.lights === true );

	}

	this.getActiveCubeFace = function () {

		return _currentActiveCubeFace;

	};

	this.getActiveMipmapLevel = function () {

		return _currentActiveMipmapLevel;

	};

	this.getRenderTarget = function () {

		return _currentRenderTarget;

	};

	this.setRenderTarget = function ( renderTarget, activeCubeFace = 0, activeMipmapLevel = 0 ) {

		_currentRenderTarget = renderTarget;
		_currentActiveCubeFace = activeCubeFace;
		_currentActiveMipmapLevel = activeMipmapLevel;

		if ( renderTarget && properties.get( renderTarget ).__webglFramebuffer === undefined ) {

			textures.setupRenderTarget( renderTarget );

		}

		let framebuffer = null;
		let isCube = false;
		let isRenderTarget3D = false;

		if ( renderTarget ) {

			const texture = renderTarget.texture;

			if ( texture.isDataTexture3D || texture.isDataTexture2DArray ) {

				isRenderTarget3D = true;

			}

			const __webglFramebuffer = properties.get( renderTarget ).__webglFramebuffer;

			if ( renderTarget.isWebGLCubeRenderTarget ) {

				framebuffer = __webglFramebuffer[ activeCubeFace ];
				isCube = true;

			} else if ( renderTarget.isWebGLMultisampleRenderTarget ) {

				framebuffer = properties.get( renderTarget ).__webglMultisampledFramebuffer;

			} else {

				framebuffer = __webglFramebuffer;

			}

			_currentViewport.copy( renderTarget.viewport );
			_currentScissor.copy( renderTarget.scissor );
			_currentScissorTest = renderTarget.scissorTest;

		} else {

			_currentViewport.copy( _viewport ).multiplyScalar( _pixelRatio ).floor();
			_currentScissor.copy( _scissor ).multiplyScalar( _pixelRatio ).floor();
			_currentScissorTest = _scissorTest;

		}

		state.bindFramebuffer( 36160, framebuffer );

		state.viewport( _currentViewport );
		state.scissor( _currentScissor );
		state.setScissorTest( _currentScissorTest );

		if ( isCube ) {

			const textureProperties = properties.get( renderTarget.texture );
			_gl.framebufferTexture2D( 36160, 36064, 34069 + activeCubeFace, textureProperties.__webglTexture, activeMipmapLevel );

		} else if ( isRenderTarget3D ) {

			const textureProperties = properties.get( renderTarget.texture );
			const layer = activeCubeFace || 0;
			_gl.framebufferTextureLayer( 36160, 36064, textureProperties.__webglTexture, activeMipmapLevel || 0, layer );

		}

	};

	this.readRenderTargetPixels = function ( renderTarget, x, y, width, height, buffer, activeCubeFaceIndex ) {

		if ( ! ( renderTarget && renderTarget.isWebGLRenderTarget ) ) {

			console.error( 'THREE.WebGLRenderer.readRenderTargetPixels: renderTarget is not THREE.WebGLRenderTarget.' );
			return;

		}

		let framebuffer = properties.get( renderTarget ).__webglFramebuffer;

		if ( renderTarget.isWebGLCubeRenderTarget && activeCubeFaceIndex !== undefined ) {

			framebuffer = framebuffer[ activeCubeFaceIndex ];

		}

		if ( framebuffer ) {

			state.bindFramebuffer( 36160, framebuffer );

			try {

				const texture = renderTarget.texture;
				const textureFormat = texture.format;
				const textureType = texture.type;

				if ( textureFormat !== RGBAFormat && utils.convert( textureFormat ) !== _gl.getParameter( 35739 ) ) {

					console.error( 'THREE.WebGLRenderer.readRenderTargetPixels: renderTarget is not in RGBA or implementation defined format.' );
					return;

				}

				const halfFloatSupportedByExt = ( textureType === HalfFloatType ) && ( extensions.has( 'EXT_color_buffer_half_float' ) || ( capabilities.isWebGL2 && extensions.has( 'EXT_color_buffer_float' ) ) );

				if ( textureType !== UnsignedByteType && utils.convert( textureType ) !== _gl.getParameter( 35738 ) && // Edge and Chrome Mac < 52 (#9513)
					! ( textureType === FloatType && ( capabilities.isWebGL2 || extensions.has( 'OES_texture_float' ) || extensions.has( 'WEBGL_color_buffer_float' ) ) ) && // Chrome Mac >= 52 and Firefox
					! halfFloatSupportedByExt ) {

					console.error( 'THREE.WebGLRenderer.readRenderTargetPixels: renderTarget is not in UnsignedByteType or implementation defined type.' );
					return;

				}

				if ( _gl.checkFramebufferStatus( 36160 ) === 36053 ) {

					// the following if statement ensures valid read requests (no out-of-bounds pixels, see #8604)

					if ( ( x >= 0 && x <= ( renderTarget.width - width ) ) && ( y >= 0 && y <= ( renderTarget.height - height ) ) ) {

						_gl.readPixels( x, y, width, height, utils.convert( textureFormat ), utils.convert( textureType ), buffer );

					}

				} else {

					console.error( 'THREE.WebGLRenderer.readRenderTargetPixels: readPixels from renderTarget failed. Framebuffer not complete.' );

				}

			} finally {

				// restore framebuffer of current render target if necessary

				const framebuffer = ( _currentRenderTarget !== null ) ? properties.get( _currentRenderTarget ).__webglFramebuffer : null;
				state.bindFramebuffer( 36160, framebuffer );

			}

		}

	};

	this.copyFramebufferToTexture = function ( position, texture, level = 0 ) {

		const levelScale = Math.pow( 2, - level );
		const width = Math.floor( texture.image.width * levelScale );
		const height = Math.floor( texture.image.height * levelScale );
		const glFormat = utils.convert( texture.format );

		textures.setTexture2D( texture, 0 );

		_gl.copyTexImage2D( 3553, level, glFormat, position.x, position.y, width, height, 0 );

		state.unbindTexture();

	};

	this.copyTextureToTexture = function ( position, srcTexture, dstTexture, level = 0 ) {

		const width = srcTexture.image.width;
		const height = srcTexture.image.height;
		const glFormat = utils.convert( dstTexture.format );
		const glType = utils.convert( dstTexture.type );

		textures.setTexture2D( dstTexture, 0 );

		// As another texture upload may have changed pixelStorei
		// parameters, make sure they are correct for the dstTexture
		_gl.pixelStorei( 37440, dstTexture.flipY );
		_gl.pixelStorei( 37441, dstTexture.premultiplyAlpha );
		_gl.pixelStorei( 3317, dstTexture.unpackAlignment );

		if ( srcTexture.isDataTexture ) {

			_gl.texSubImage2D( 3553, level, position.x, position.y, width, height, glFormat, glType, srcTexture.image.data );

		} else {

			if ( srcTexture.isCompressedTexture ) {

				_gl.compressedTexSubImage2D( 3553, level, position.x, position.y, srcTexture.mipmaps[ 0 ].width, srcTexture.mipmaps[ 0 ].height, glFormat, srcTexture.mipmaps[ 0 ].data );

			} else {

				_gl.texSubImage2D( 3553, level, position.x, position.y, glFormat, glType, srcTexture.image );

			}

		}

		// Generate mipmaps only when copying level 0
		if ( level === 0 && dstTexture.generateMipmaps ) _gl.generateMipmap( 3553 );

		state.unbindTexture();

	};

	this.copyTextureToTexture3D = function ( sourceBox, position, srcTexture, dstTexture, level = 0 ) {

		if ( _this.isWebGL1Renderer ) {

			console.warn( 'THREE.WebGLRenderer.copyTextureToTexture3D: can only be used with WebGL2.' );
			return;

		}

		const { width, height, data } = srcTexture.image;
		const glFormat = utils.convert( dstTexture.format );
		const glType = utils.convert( dstTexture.type );
		let glTarget;

		if ( dstTexture.isDataTexture3D ) {

			textures.setTexture3D( dstTexture, 0 );
			glTarget = 32879;

		} else if ( dstTexture.isDataTexture2DArray ) {

			textures.setTexture2DArray( dstTexture, 0 );
			glTarget = 35866;

		} else {

			console.warn( 'THREE.WebGLRenderer.copyTextureToTexture3D: only supports THREE.DataTexture3D and THREE.DataTexture2DArray.' );
			return;

		}

		_gl.pixelStorei( 37440, dstTexture.flipY );
		_gl.pixelStorei( 37441, dstTexture.premultiplyAlpha );
		_gl.pixelStorei( 3317, dstTexture.unpackAlignment );

		const unpackRowLen = _gl.getParameter( 3314 );
		const unpackImageHeight = _gl.getParameter( 32878 );
		const unpackSkipPixels = _gl.getParameter( 3316 );
		const unpackSkipRows = _gl.getParameter( 3315 );
		const unpackSkipImages = _gl.getParameter( 32877 );

		_gl.pixelStorei( 3314, width );
		_gl.pixelStorei( 32878, height );
		_gl.pixelStorei( 3316, sourceBox.min.x );
		_gl.pixelStorei( 3315, sourceBox.min.y );
		_gl.pixelStorei( 32877, sourceBox.min.z );

		_gl.texSubImage3D(
			glTarget,
			level,
			position.x,
			position.y,
			position.z,
			sourceBox.max.x - sourceBox.min.x + 1,
			sourceBox.max.y - sourceBox.min.y + 1,
			sourceBox.max.z - sourceBox.min.z + 1,
			glFormat,
			glType,
			data
		);

		_gl.pixelStorei( 3314, unpackRowLen );
		_gl.pixelStorei( 32878, unpackImageHeight );
		_gl.pixelStorei( 3316, unpackSkipPixels );
		_gl.pixelStorei( 3315, unpackSkipRows );
		_gl.pixelStorei( 32877, unpackSkipImages );

		// Generate mipmaps only when copying level 0
		if ( level === 0 && dstTexture.generateMipmaps ) _gl.generateMipmap( glTarget );

		state.unbindTexture();

	};

	this.initTexture = function ( texture ) {

		textures.setTexture2D( texture, 0 );

		state.unbindTexture();

	};

	this.resetState = function () {

		_currentActiveCubeFace = 0;
		_currentActiveMipmapLevel = 0;
		_currentRenderTarget = null;

		state.reset();
		bindingStates.reset();

	};

	if ( typeof __THREE_DEVTOOLS__ !== 'undefined' ) {

		__THREE_DEVTOOLS__.dispatchEvent( new CustomEvent( 'observe', { detail: this } ) ); // eslint-disable-line no-undef

	}

}

class WebGL1Renderer extends WebGLRenderer {}

WebGL1Renderer.prototype.isWebGL1Renderer = true;

class FogExp2 {

	constructor( color, density = 0.00025 ) {

		this.name = '';

		this.color = new Color( color );
		this.density = density;

	}

	clone() {

		return new FogExp2( this.color, this.density );

	}

	toJSON( /* meta */ ) {

		return {
			type: 'FogExp2',
			color: this.color.getHex(),
			density: this.density
		};

	}

}

FogExp2.prototype.isFogExp2 = true;

class Fog {

	constructor( color, near = 1, far = 1000 ) {

		this.name = '';

		this.color = new Color( color );

		this.near = near;
		this.far = far;

	}

	clone() {

		return new Fog( this.color, this.near, this.far );

	}

	toJSON( /* meta */ ) {

		return {
			type: 'Fog',
			color: this.color.getHex(),
			near: this.near,
			far: this.far
		};

	}

}

Fog.prototype.isFog = true;

class Scene extends Object3D {

	constructor() {

		super();

		this.type = 'Scene';

		this.background = null;
		this.environment = null;
		this.fog = null;

		this.overrideMaterial = null;

		this.autoUpdate = true; // checked by the renderer

		if ( typeof __THREE_DEVTOOLS__ !== 'undefined' ) {

			__THREE_DEVTOOLS__.dispatchEvent( new CustomEvent( 'observe', { detail: this } ) ); // eslint-disable-line no-undef

		}

	}

	copy( source, recursive ) {

		super.copy( source, recursive );

		if ( source.background !== null ) this.background = source.background.clone();
		if ( source.environment !== null ) this.environment = source.environment.clone();
		if ( source.fog !== null ) this.fog = source.fog.clone();

		if ( source.overrideMaterial !== null ) this.overrideMaterial = source.overrideMaterial.clone();

		this.autoUpdate = source.autoUpdate;
		this.matrixAutoUpdate = source.matrixAutoUpdate;

		return this;

	}

	toJSON( meta ) {

		const data = super.toJSON( meta );

		if ( this.background !== null ) data.object.background = this.background.toJSON( meta );
		if ( this.environment !== null ) data.object.environment = this.environment.toJSON( meta );
		if ( this.fog !== null ) data.object.fog = this.fog.toJSON();

		return data;

	}

}

Scene.prototype.isScene = true;

function InterleavedBuffer( array, stride ) {

	this.array = array;
	this.stride = stride;
	this.count = array !== undefined ? array.length / stride : 0;

	this.usage = StaticDrawUsage;
	this.updateRange = { offset: 0, count: - 1 };

	this.version = 0;

	this.uuid = MathUtils.generateUUID();

}

Object.defineProperty( InterleavedBuffer.prototype, 'needsUpdate', {

	set: function ( value ) {

		if ( value === true ) this.version ++;

	}

} );

Object.assign( InterleavedBuffer.prototype, {

	isInterleavedBuffer: true,

	onUploadCallback: function () {},

	setUsage: function ( value ) {

		this.usage = value;

		return this;

	},

	copy: function ( source ) {

		this.array = new source.array.constructor( source.array );
		this.count = source.count;
		this.stride = source.stride;
		this.usage = source.usage;

		return this;

	},

	copyAt: function ( index1, attribute, index2 ) {

		index1 *= this.stride;
		index2 *= attribute.stride;

		for ( let i = 0, l = this.stride; i < l; i ++ ) {

			this.array[ index1 + i ] = attribute.array[ index2 + i ];

		}

		return this;

	},

	set: function ( value, offset = 0 ) {

		this.array.set( value, offset );

		return this;

	},

	clone: function ( data ) {

		if ( data.arrayBuffers === undefined ) {

			data.arrayBuffers = {};

		}

		if ( this.array.buffer._uuid === undefined ) {

			this.array.buffer._uuid = MathUtils.generateUUID();

		}

		if ( data.arrayBuffers[ this.array.buffer._uuid ] === undefined ) {

			data.arrayBuffers[ this.array.buffer._uuid ] = this.array.slice( 0 ).buffer;

		}

		const array = new this.array.constructor( data.arrayBuffers[ this.array.buffer._uuid ] );

		const ib = new InterleavedBuffer( array, this.stride );
		ib.setUsage( this.usage );

		return ib;

	},

	onUpload: function ( callback ) {

		this.onUploadCallback = callback;

		return this;

	},

	toJSON: function ( data ) {

		if ( data.arrayBuffers === undefined ) {

			data.arrayBuffers = {};

		}

		// generate UUID for array buffer if necessary

		if ( this.array.buffer._uuid === undefined ) {

			this.array.buffer._uuid = MathUtils.generateUUID();

		}

		if ( data.arrayBuffers[ this.array.buffer._uuid ] === undefined ) {

			data.arrayBuffers[ this.array.buffer._uuid ] = Array.prototype.slice.call( new Uint32Array( this.array.buffer ) );

		}

		//

		return {
			uuid: this.uuid,
			buffer: this.array.buffer._uuid,
			type: this.array.constructor.name,
			stride: this.stride
		};

	}

} );

const _vector$6 = new Vector3();

function InterleavedBufferAttribute( interleavedBuffer, itemSize, offset, normalized ) {

	this.name = '';

	this.data = interleavedBuffer;
	this.itemSize = itemSize;
	this.offset = offset;

	this.normalized = normalized === true;

}

Object.defineProperties( InterleavedBufferAttribute.prototype, {

	count: {

		get: function () {

			return this.data.count;

		}

	},

	array: {

		get: function () {

			return this.data.array;

		}

	},

	needsUpdate: {

		set: function ( value ) {

			this.data.needsUpdate = value;

		}

	}

} );

Object.assign( InterleavedBufferAttribute.prototype, {

	isInterleavedBufferAttribute: true,

	applyMatrix4: function ( m ) {

		for ( let i = 0, l = this.data.count; i < l; i ++ ) {

			_vector$6.x = this.getX( i );
			_vector$6.y = this.getY( i );
			_vector$6.z = this.getZ( i );

			_vector$6.applyMatrix4( m );

			this.setXYZ( i, _vector$6.x, _vector$6.y, _vector$6.z );

		}

		return this;

	},

	applyNormalMatrix: function ( m ) {

		for ( let i = 0, l = this.count; i < l; i ++ ) {

			_vector$6.x = this.getX( i );
			_vector$6.y = this.getY( i );
			_vector$6.z = this.getZ( i );

			_vector$6.applyNormalMatrix( m );

			this.setXYZ( i, _vector$6.x, _vector$6.y, _vector$6.z );

		}

		return this;

	},

	transformDirection: function ( m ) {

		for ( let i = 0, l = this.count; i < l; i ++ ) {

			_vector$6.x = this.getX( i );
			_vector$6.y = this.getY( i );
			_vector$6.z = this.getZ( i );

			_vector$6.transformDirection( m );

			this.setXYZ( i, _vector$6.x, _vector$6.y, _vector$6.z );

		}

		return this;

	},

	setX: function ( index, x ) {

		this.data.array[ index * this.data.stride + this.offset ] = x;

		return this;

	},

	setY: function ( index, y ) {

		this.data.array[ index * this.data.stride + this.offset + 1 ] = y;

		return this;

	},

	setZ: function ( index, z ) {

		this.data.array[ index * this.data.stride + this.offset + 2 ] = z;

		return this;

	},

	setW: function ( index, w ) {

		this.data.array[ index * this.data.stride + this.offset + 3 ] = w;

		return this;

	},

	getX: function ( index ) {

		return this.data.array[ index * this.data.stride + this.offset ];

	},

	getY: function ( index ) {

		return this.data.array[ index * this.data.stride + this.offset + 1 ];

	},

	getZ: function ( index ) {

		return this.data.array[ index * this.data.stride + this.offset + 2 ];

	},

	getW: function ( index ) {

		return this.data.array[ index * this.data.stride + this.offset + 3 ];

	},

	setXY: function ( index, x, y ) {

		index = index * this.data.stride + this.offset;

		this.data.array[ index + 0 ] = x;
		this.data.array[ index + 1 ] = y;

		return this;

	},

	setXYZ: function ( index, x, y, z ) {

		index = index * this.data.stride + this.offset;

		this.data.array[ index + 0 ] = x;
		this.data.array[ index + 1 ] = y;
		this.data.array[ index + 2 ] = z;

		return this;

	},

	setXYZW: function ( index, x, y, z, w ) {

		index = index * this.data.stride + this.offset;

		this.data.array[ index + 0 ] = x;
		this.data.array[ index + 1 ] = y;
		this.data.array[ index + 2 ] = z;
		this.data.array[ index + 3 ] = w;

		return this;

	},

	clone: function ( data ) {

		if ( data === undefined ) {

			console.log( 'THREE.InterleavedBufferAttribute.clone(): Cloning an interlaved buffer attribute will deinterleave buffer data.' );

			const array = [];

			for ( let i = 0; i < this.count; i ++ ) {

				const index = i * this.data.stride + this.offset;

				for ( let j = 0; j < this.itemSize; j ++ ) {

					array.push( this.data.array[ index + j ] );

				}

			}

			return new BufferAttribute( new this.array.constructor( array ), this.itemSize, this.normalized );

		} else {

			if ( data.interleavedBuffers === undefined ) {

				data.interleavedBuffers = {};

			}

			if ( data.interleavedBuffers[ this.data.uuid ] === undefined ) {

				data.interleavedBuffers[ this.data.uuid ] = this.data.clone( data );

			}

			return new InterleavedBufferAttribute( data.interleavedBuffers[ this.data.uuid ], this.itemSize, this.offset, this.normalized );

		}

	},

	toJSON: function ( data ) {

		if ( data === undefined ) {

			console.log( 'THREE.InterleavedBufferAttribute.toJSON(): Serializing an interlaved buffer attribute will deinterleave buffer data.' );

			const array = [];

			for ( let i = 0; i < this.count; i ++ ) {

				const index = i * this.data.stride + this.offset;

				for ( let j = 0; j < this.itemSize; j ++ ) {

					array.push( this.data.array[ index + j ] );

				}

			}

			// deinterleave data and save it as an ordinary buffer attribute for now

			return {
				itemSize: this.itemSize,
				type: this.array.constructor.name,
				array: array,
				normalized: this.normalized
			};

		} else {

			// save as true interlaved attribtue

			if ( data.interleavedBuffers === undefined ) {

				data.interleavedBuffers = {};

			}

			if ( data.interleavedBuffers[ this.data.uuid ] === undefined ) {

				data.interleavedBuffers[ this.data.uuid ] = this.data.toJSON( data );

			}

			return {
				isInterleavedBufferAttribute: true,
				itemSize: this.itemSize,
				data: this.data.uuid,
				offset: this.offset,
				normalized: this.normalized
			};

		}

	}

} );

/**
 * parameters = {
 *  color: <hex>,
 *  map: new THREE.Texture( <Image> ),
 *  alphaMap: new THREE.Texture( <Image> ),
 *  rotation: <float>,
 *  sizeAttenuation: <bool>
 * }
 */

class SpriteMaterial extends Material {

	constructor( parameters ) {

		super();

		this.type = 'SpriteMaterial';

		this.color = new Color( 0xffffff );

		this.map = null;

		this.alphaMap = null;

		this.rotation = 0;

		this.sizeAttenuation = true;

		this.transparent = true;

		this.setValues( parameters );

	}

	copy( source ) {

		super.copy( source );

		this.color.copy( source.color );

		this.map = source.map;

		this.alphaMap = source.alphaMap;

		this.rotation = source.rotation;

		this.sizeAttenuation = source.sizeAttenuation;

		return this;

	}

}

SpriteMaterial.prototype.isSpriteMaterial = true;

let _geometry;

const _intersectPoint = /*@__PURE__*/ new Vector3();
const _worldScale = /*@__PURE__*/ new Vector3();
const _mvPosition = /*@__PURE__*/ new Vector3();

const _alignedPosition = /*@__PURE__*/ new Vector2();
const _rotatedPosition = /*@__PURE__*/ new Vector2();
const _viewWorldMatrix = /*@__PURE__*/ new Matrix4();

const _vA = /*@__PURE__*/ new Vector3();
const _vB = /*@__PURE__*/ new Vector3();
const _vC = /*@__PURE__*/ new Vector3();

const _uvA = /*@__PURE__*/ new Vector2();
const _uvB = /*@__PURE__*/ new Vector2();
const _uvC = /*@__PURE__*/ new Vector2();

class Sprite extends Object3D {

	constructor( material ) {

		super();

		this.type = 'Sprite';

		if ( _geometry === undefined ) {

			_geometry = new BufferGeometry();

			const float32Array = new Float32Array( [
				- 0.5, - 0.5, 0, 0, 0,
				0.5, - 0.5, 0, 1, 0,
				0.5, 0.5, 0, 1, 1,
				- 0.5, 0.5, 0, 0, 1
			] );

			const interleavedBuffer = new InterleavedBuffer( float32Array, 5 );

			_geometry.setIndex( [ 0, 1, 2,	0, 2, 3 ] );
			_geometry.setAttribute( 'position', new InterleavedBufferAttribute( interleavedBuffer, 3, 0, false ) );
			_geometry.setAttribute( 'uv', new InterleavedBufferAttribute( interleavedBuffer, 2, 3, false ) );

		}

		this.geometry = _geometry;
		this.material = ( material !== undefined ) ? material : new SpriteMaterial();

		this.center = new Vector2( 0.5, 0.5 );

	}

	raycast( raycaster, intersects ) {

		if ( raycaster.camera === null ) {

			console.error( 'THREE.Sprite: "Raycaster.camera" needs to be set in order to raycast against sprites.' );

		}

		_worldScale.setFromMatrixScale( this.matrixWorld );

		_viewWorldMatrix.copy( raycaster.camera.matrixWorld );
		this.modelViewMatrix.multiplyMatrices( raycaster.camera.matrixWorldInverse, this.matrixWorld );

		_mvPosition.setFromMatrixPosition( this.modelViewMatrix );

		if ( raycaster.camera.isPerspectiveCamera && this.material.sizeAttenuation === false ) {

			_worldScale.multiplyScalar( - _mvPosition.z );

		}

		const rotation = this.material.rotation;
		let sin, cos;

		if ( rotation !== 0 ) {

			cos = Math.cos( rotation );
			sin = Math.sin( rotation );

		}

		const center = this.center;

		transformVertex( _vA.set( - 0.5, - 0.5, 0 ), _mvPosition, center, _worldScale, sin, cos );
		transformVertex( _vB.set( 0.5, - 0.5, 0 ), _mvPosition, center, _worldScale, sin, cos );
		transformVertex( _vC.set( 0.5, 0.5, 0 ), _mvPosition, center, _worldScale, sin, cos );

		_uvA.set( 0, 0 );
		_uvB.set( 1, 0 );
		_uvC.set( 1, 1 );

		// check first triangle
		let intersect = raycaster.ray.intersectTriangle( _vA, _vB, _vC, false, _intersectPoint );

		if ( intersect === null ) {

			// check second triangle
			transformVertex( _vB.set( - 0.5, 0.5, 0 ), _mvPosition, center, _worldScale, sin, cos );
			_uvB.set( 0, 1 );

			intersect = raycaster.ray.intersectTriangle( _vA, _vC, _vB, false, _intersectPoint );
			if ( intersect === null ) {

				return;

			}

		}

		const distance = raycaster.ray.origin.distanceTo( _intersectPoint );

		if ( distance < raycaster.near || distance > raycaster.far ) return;

		intersects.push( {

			distance: distance,
			point: _intersectPoint.clone(),
			uv: Triangle.getUV( _intersectPoint, _vA, _vB, _vC, _uvA, _uvB, _uvC, new Vector2() ),
			face: null,
			object: this

		} );

	}

	copy( source ) {

		super.copy( source );

		if ( source.center !== undefined ) this.center.copy( source.center );

		this.material = source.material;

		return this;

	}

}

Sprite.prototype.isSprite = true;

function transformVertex( vertexPosition, mvPosition, center, scale, sin, cos ) {

	// compute position in camera space
	_alignedPosition.subVectors( vertexPosition, center ).addScalar( 0.5 ).multiply( scale );

	// to check if rotation is not zero
	if ( sin !== undefined ) {

		_rotatedPosition.x = ( cos * _alignedPosition.x ) - ( sin * _alignedPosition.y );
		_rotatedPosition.y = ( sin * _alignedPosition.x ) + ( cos * _alignedPosition.y );

	} else {

		_rotatedPosition.copy( _alignedPosition );

	}


	vertexPosition.copy( mvPosition );
	vertexPosition.x += _rotatedPosition.x;
	vertexPosition.y += _rotatedPosition.y;

	// transform to world space
	vertexPosition.applyMatrix4( _viewWorldMatrix );

}

const _basePosition = new Vector3();

const _skinIndex = new Vector4();
const _skinWeight = new Vector4();

const _vector$5 = new Vector3();
const _matrix = new Matrix4();

function SkinnedMesh( geometry, material ) {

	Mesh.call( this, geometry, material );

	this.type = 'SkinnedMesh';

	this.bindMode = 'attached';
	this.bindMatrix = new Matrix4();
	this.bindMatrixInverse = new Matrix4();

}

SkinnedMesh.prototype = Object.assign( Object.create( Mesh.prototype ), {

	constructor: SkinnedMesh,

	isSkinnedMesh: true,

	copy: function ( source ) {

		Mesh.prototype.copy.call( this, source );

		this.bindMode = source.bindMode;
		this.bindMatrix.copy( source.bindMatrix );
		this.bindMatrixInverse.copy( source.bindMatrixInverse );

		this.skeleton = source.skeleton;

		return this;

	},

	bind: function ( skeleton, bindMatrix ) {

		this.skeleton = skeleton;

		if ( bindMatrix === undefined ) {

			this.updateMatrixWorld( true );

			this.skeleton.calculateInverses();

			bindMatrix = this.matrixWorld;

		}

		this.bindMatrix.copy( bindMatrix );
		this.bindMatrixInverse.copy( bindMatrix ).invert();

	},

	pose: function () {

		this.skeleton.pose();

	},

	normalizeSkinWeights: function () {

		const vector = new Vector4();

		const skinWeight = this.geometry.attributes.skinWeight;

		for ( let i = 0, l = skinWeight.count; i < l; i ++ ) {

			vector.x = skinWeight.getX( i );
			vector.y = skinWeight.getY( i );
			vector.z = skinWeight.getZ( i );
			vector.w = skinWeight.getW( i );

			const scale = 1.0 / vector.manhattanLength();

			if ( scale !== Infinity ) {

				vector.multiplyScalar( scale );

			} else {

				vector.set( 1, 0, 0, 0 ); // do something reasonable

			}

			skinWeight.setXYZW( i, vector.x, vector.y, vector.z, vector.w );

		}

	},

	updateMatrixWorld: function ( force ) {

		Mesh.prototype.updateMatrixWorld.call( this, force );

		if ( this.bindMode === 'attached' ) {

			this.bindMatrixInverse.copy( this.matrixWorld ).invert();

		} else if ( this.bindMode === 'detached' ) {

			this.bindMatrixInverse.copy( this.bindMatrix ).invert();

		} else {

			console.warn( 'THREE.SkinnedMesh: Unrecognized bindMode: ' + this.bindMode );

		}

	},

	boneTransform: function ( index, target ) {

		const skeleton = this.skeleton;
		const geometry = this.geometry;

		_skinIndex.fromBufferAttribute( geometry.attributes.skinIndex, index );
		_skinWeight.fromBufferAttribute( geometry.attributes.skinWeight, index );

		_basePosition.fromBufferAttribute( geometry.attributes.position, index ).applyMatrix4( this.bindMatrix );

		target.set( 0, 0, 0 );

		for ( let i = 0; i < 4; i ++ ) {

			const weight = _skinWeight.getComponent( i );

			if ( weight !== 0 ) {

				const boneIndex = _skinIndex.getComponent( i );

				_matrix.multiplyMatrices( skeleton.bones[ boneIndex ].matrixWorld, skeleton.boneInverses[ boneIndex ] );

				target.addScaledVector( _vector$5.copy( _basePosition ).applyMatrix4( _matrix ), weight );

			}

		}

		return target.applyMatrix4( this.bindMatrixInverse );

	}

} );

function Bone() {

	Object3D.call( this );

	this.type = 'Bone';

}

Bone.prototype = Object.assign( Object.create( Object3D.prototype ), {

	constructor: Bone,

	isBone: true

} );

const _instanceLocalMatrix = new Matrix4();
const _instanceWorldMatrix = new Matrix4();

const _instanceIntersects = [];

const _mesh = new Mesh();

function InstancedMesh( geometry, material, count ) {

	Mesh.call( this, geometry, material );

	this.instanceMatrix = new BufferAttribute( new Float32Array( count * 16 ), 16 );
	this.instanceColor = null;

	this.count = count;

	this.frustumCulled = false;

}

InstancedMesh.prototype = Object.assign( Object.create( Mesh.prototype ), {

	constructor: InstancedMesh,

	isInstancedMesh: true,

	copy: function ( source ) {

		Mesh.prototype.copy.call( this, source );

		this.instanceMatrix.copy( source.instanceMatrix );

		if ( source.instanceColor !== null ) this.instanceColor = source.instanceColor.clone();

		this.count = source.count;

		return this;

	},

	getColorAt: function ( index, color ) {

		color.fromArray( this.instanceColor.array, index * 3 );

	},

	getMatrixAt: function ( index, matrix ) {

		matrix.fromArray( this.instanceMatrix.array, index * 16 );

	},

	raycast: function ( raycaster, intersects ) {

		const matrixWorld = this.matrixWorld;
		const raycastTimes = this.count;

		_mesh.geometry = this.geometry;
		_mesh.material = this.material;

		if ( _mesh.material === undefined ) return;

		for ( let instanceId = 0; instanceId < raycastTimes; instanceId ++ ) {

			// calculate the world matrix for each instance

			this.getMatrixAt( instanceId, _instanceLocalMatrix );

			_instanceWorldMatrix.multiplyMatrices( matrixWorld, _instanceLocalMatrix );

			// the mesh represents this single instance

			_mesh.matrixWorld = _instanceWorldMatrix;

			_mesh.raycast( raycaster, _instanceIntersects );

			// process the result of raycast

			for ( let i = 0, l = _instanceIntersects.length; i < l; i ++ ) {

				const intersect = _instanceIntersects[ i ];
				intersect.instanceId = instanceId;
				intersect.object = this;
				intersects.push( intersect );

			}

			_instanceIntersects.length = 0;

		}

	},

	setColorAt: function ( index, color ) {

		if ( this.instanceColor === null ) {

			this.instanceColor = new BufferAttribute( new Float32Array( this.count * 3 ), 3 );

		}

		color.toArray( this.instanceColor.array, index * 3 );

	},

	setMatrixAt: function ( index, matrix ) {

		matrix.toArray( this.instanceMatrix.array, index * 16 );

	},

	updateMorphTargets: function () {

	},

	dispose: function () {

		this.dispatchEvent( { type: 'dispose' } );

	}

} );

/**
 * parameters = {
 *  color: <hex>,
 *  opacity: <float>,
 *
 *  linewidth: <float>,
 *  linecap: "round",
 *  linejoin: "round"
 * }
 */

class LineBasicMaterial extends Material {

	constructor( parameters ) {

		super();

		this.type = 'LineBasicMaterial';

		this.color = new Color( 0xffffff );

		this.linewidth = 1;
		this.linecap = 'round';
		this.linejoin = 'round';

		this.morphTargets = false;

		this.setValues( parameters );

	}


	copy( source ) {

		super.copy( source );

		this.color.copy( source.color );

		this.linewidth = source.linewidth;
		this.linecap = source.linecap;
		this.linejoin = source.linejoin;

		this.morphTargets = source.morphTargets;

		return this;

	}

}

LineBasicMaterial.prototype.isLineBasicMaterial = true;

const _start$1 = new Vector3();
const _end$1 = new Vector3();
const _inverseMatrix$1 = new Matrix4();
const _ray$1 = new Ray();
const _sphere$1 = new Sphere();

function Line( geometry = new BufferGeometry(), material = new LineBasicMaterial() ) {

	Object3D.call( this );

	this.type = 'Line';

	this.geometry = geometry;
	this.material = material;

	this.updateMorphTargets();

}

Line.prototype = Object.assign( Object.create( Object3D.prototype ), {

	constructor: Line,

	isLine: true,

	copy: function ( source ) {

		Object3D.prototype.copy.call( this, source );

		this.material = source.material;
		this.geometry = source.geometry;

		return this;

	},

	computeLineDistances: function () {

		const geometry = this.geometry;

		if ( geometry.isBufferGeometry ) {

			// we assume non-indexed geometry

			if ( geometry.index === null ) {

				const positionAttribute = geometry.attributes.position;
				const lineDistances = [ 0 ];

				for ( let i = 1, l = positionAttribute.count; i < l; i ++ ) {

					_start$1.fromBufferAttribute( positionAttribute, i - 1 );
					_end$1.fromBufferAttribute( positionAttribute, i );

					lineDistances[ i ] = lineDistances[ i - 1 ];
					lineDistances[ i ] += _start$1.distanceTo( _end$1 );

				}

				geometry.setAttribute( 'lineDistance', new Float32BufferAttribute( lineDistances, 1 ) );

			} else {

				console.warn( 'THREE.Line.computeLineDistances(): Computation only possible with non-indexed BufferGeometry.' );

			}

		} else if ( geometry.isGeometry ) {

			console.error( 'THREE.Line.computeLineDistances() no longer supports THREE.Geometry. Use THREE.BufferGeometry instead.' );

		}

		return this;

	},

	raycast: function ( raycaster, intersects ) {

		const geometry = this.geometry;
		const matrixWorld = this.matrixWorld;
		const threshold = raycaster.params.Line.threshold;
		const drawRange = geometry.drawRange;

		// Checking boundingSphere distance to ray

		if ( geometry.boundingSphere === null ) geometry.computeBoundingSphere();

		_sphere$1.copy( geometry.boundingSphere );
		_sphere$1.applyMatrix4( matrixWorld );
		_sphere$1.radius += threshold;

		if ( raycaster.ray.intersectsSphere( _sphere$1 ) === false ) return;

		//

		_inverseMatrix$1.copy( matrixWorld ).invert();
		_ray$1.copy( raycaster.ray ).applyMatrix4( _inverseMatrix$1 );

		const localThreshold = threshold / ( ( this.scale.x + this.scale.y + this.scale.z ) / 3 );
		const localThresholdSq = localThreshold * localThreshold;

		const vStart = new Vector3();
		const vEnd = new Vector3();
		const interSegment = new Vector3();
		const interRay = new Vector3();
		const step = this.isLineSegments ? 2 : 1;

		if ( geometry.isBufferGeometry ) {

			const index = geometry.index;
			const attributes = geometry.attributes;
			const positionAttribute = attributes.position;

			if ( index !== null ) {

				const start = Math.max( 0, drawRange.start );
				const end = Math.min( index.count, ( drawRange.start + drawRange.count ) );

				for ( let i = start, l = end - 1; i < l; i += step ) {

					const a = index.getX( i );
					const b = index.getX( i + 1 );

					vStart.fromBufferAttribute( positionAttribute, a );
					vEnd.fromBufferAttribute( positionAttribute, b );

					const distSq = _ray$1.distanceSqToSegment( vStart, vEnd, interRay, interSegment );

					if ( distSq > localThresholdSq ) continue;

					interRay.applyMatrix4( this.matrixWorld ); //Move back to world space for distance calculation

					const distance = raycaster.ray.origin.distanceTo( interRay );

					if ( distance < raycaster.near || distance > raycaster.far ) continue;

					intersects.push( {

						distance: distance,
						// What do we want? intersection point on the ray or on the segment??
						// point: raycaster.ray.at( distance ),
						point: interSegment.clone().applyMatrix4( this.matrixWorld ),
						index: i,
						face: null,
						faceIndex: null,
						object: this

					} );

				}

			} else {

				const start = Math.max( 0, drawRange.start );
				const end = Math.min( positionAttribute.count, ( drawRange.start + drawRange.count ) );

				for ( let i = start, l = end - 1; i < l; i += step ) {

					vStart.fromBufferAttribute( positionAttribute, i );
					vEnd.fromBufferAttribute( positionAttribute, i + 1 );

					const distSq = _ray$1.distanceSqToSegment( vStart, vEnd, interRay, interSegment );

					if ( distSq > localThresholdSq ) continue;

					interRay.applyMatrix4( this.matrixWorld ); //Move back to world space for distance calculation

					const distance = raycaster.ray.origin.distanceTo( interRay );

					if ( distance < raycaster.near || distance > raycaster.far ) continue;

					intersects.push( {

						distance: distance,
						// What do we want? intersection point on the ray or on the segment??
						// point: raycaster.ray.at( distance ),
						point: interSegment.clone().applyMatrix4( this.matrixWorld ),
						index: i,
						face: null,
						faceIndex: null,
						object: this

					} );

				}

			}

		} else if ( geometry.isGeometry ) {

			console.error( 'THREE.Line.raycast() no longer supports THREE.Geometry. Use THREE.BufferGeometry instead.' );

		}

	},

	updateMorphTargets: function () {

		const geometry = this.geometry;

		if ( geometry.isBufferGeometry ) {

			const morphAttributes = geometry.morphAttributes;
			const keys = Object.keys( morphAttributes );

			if ( keys.length > 0 ) {

				const morphAttribute = morphAttributes[ keys[ 0 ] ];

				if ( morphAttribute !== undefined ) {

					this.morphTargetInfluences = [];
					this.morphTargetDictionary = {};

					for ( let m = 0, ml = morphAttribute.length; m < ml; m ++ ) {

						const name = morphAttribute[ m ].name || String( m );

						this.morphTargetInfluences.push( 0 );
						this.morphTargetDictionary[ name ] = m;

					}

				}

			}

		} else {

			const morphTargets = geometry.morphTargets;

			if ( morphTargets !== undefined && morphTargets.length > 0 ) {

				console.error( 'THREE.Line.updateMorphTargets() does not support THREE.Geometry. Use THREE.BufferGeometry instead.' );

			}

		}

	}

} );

const _start = new Vector3();
const _end = new Vector3();

function LineSegments( geometry, material ) {

	Line.call( this, geometry, material );

	this.type = 'LineSegments';

}

LineSegments.prototype = Object.assign( Object.create( Line.prototype ), {

	constructor: LineSegments,

	isLineSegments: true,

	computeLineDistances: function () {

		const geometry = this.geometry;

		if ( geometry.isBufferGeometry ) {

			// we assume non-indexed geometry

			if ( geometry.index === null ) {

				const positionAttribute = geometry.attributes.position;
				const lineDistances = [];

				for ( let i = 0, l = positionAttribute.count; i < l; i += 2 ) {

					_start.fromBufferAttribute( positionAttribute, i );
					_end.fromBufferAttribute( positionAttribute, i + 1 );

					lineDistances[ i ] = ( i === 0 ) ? 0 : lineDistances[ i - 1 ];
					lineDistances[ i + 1 ] = lineDistances[ i ] + _start.distanceTo( _end );

				}

				geometry.setAttribute( 'lineDistance', new Float32BufferAttribute( lineDistances, 1 ) );

			} else {

				console.warn( 'THREE.LineSegments.computeLineDistances(): Computation only possible with non-indexed BufferGeometry.' );

			}

		} else if ( geometry.isGeometry ) {

			console.error( 'THREE.LineSegments.computeLineDistances() no longer supports THREE.Geometry. Use THREE.BufferGeometry instead.' );

		}

		return this;

	}

} );

class LineLoop extends Line {

	constructor( geometry, material ) {

		super( geometry, material );

		this.type = 'LineLoop';

	}

}

LineLoop.prototype.isLineLoop = true;

/**
 * parameters = {
 *  color: <hex>,
 *  opacity: <float>,
 *  map: new THREE.Texture( <Image> ),
 *  alphaMap: new THREE.Texture( <Image> ),
 *
 *  size: <float>,
 *  sizeAttenuation: <bool>
 *
 *  morphTargets: <bool>
 * }
 */

class PointsMaterial extends Material {

	constructor( parameters ) {

		super();

		this.type = 'PointsMaterial';

		this.color = new Color( 0xffffff );

		this.map = null;

		this.alphaMap = null;

		this.size = 1;
		this.sizeAttenuation = true;

		this.morphTargets = false;

		this.setValues( parameters );

	}

	copy( source ) {

		super.copy( source );

		this.color.copy( source.color );

		this.map = source.map;

		this.alphaMap = source.alphaMap;

		this.size = source.size;
		this.sizeAttenuation = source.sizeAttenuation;

		this.morphTargets = source.morphTargets;

		return this;

	}

}

PointsMaterial.prototype.isPointsMaterial = true;

const _inverseMatrix = new Matrix4();
const _ray = new Ray();
const _sphere = new Sphere();
const _position$2 = new Vector3();

function Points( geometry = new BufferGeometry(), material = new PointsMaterial() ) {

	Object3D.call( this );

	this.type = 'Points';

	this.geometry = geometry;
	this.material = material;

	this.updateMorphTargets();

}

Points.prototype = Object.assign( Object.create( Object3D.prototype ), {

	constructor: Points,

	isPoints: true,

	copy: function ( source ) {

		Object3D.prototype.copy.call( this, source );

		this.material = source.material;
		this.geometry = source.geometry;

		return this;

	},

	raycast: function ( raycaster, intersects ) {

		const geometry = this.geometry;
		const matrixWorld = this.matrixWorld;
		const threshold = raycaster.params.Points.threshold;
		const drawRange = geometry.drawRange;

		// Checking boundingSphere distance to ray

		if ( geometry.boundingSphere === null ) geometry.computeBoundingSphere();

		_sphere.copy( geometry.boundingSphere );
		_sphere.applyMatrix4( matrixWorld );
		_sphere.radius += threshold;

		if ( raycaster.ray.intersectsSphere( _sphere ) === false ) return;

		//

		_inverseMatrix.copy( matrixWorld ).invert();
		_ray.copy( raycaster.ray ).applyMatrix4( _inverseMatrix );

		const localThreshold = threshold / ( ( this.scale.x + this.scale.y + this.scale.z ) / 3 );
		const localThresholdSq = localThreshold * localThreshold;

		if ( geometry.isBufferGeometry ) {

			const index = geometry.index;
			const attributes = geometry.attributes;
			const positionAttribute = attributes.position;

			if ( index !== null ) {

				const start = Math.max( 0, drawRange.start );
				const end = Math.min( index.count, ( drawRange.start + drawRange.count ) );

				for ( let i = start, il = end; i < il; i ++ ) {

					const a = index.getX( i );

					_position$2.fromBufferAttribute( positionAttribute, a );

					testPoint( _position$2, a, localThresholdSq, matrixWorld, raycaster, intersects, this );

				}

			} else {

				const start = Math.max( 0, drawRange.start );
				const end = Math.min( positionAttribute.count, ( drawRange.start + drawRange.count ) );

				for ( let i = start, l = end; i < l; i ++ ) {

					_position$2.fromBufferAttribute( positionAttribute, i );

					testPoint( _position$2, i, localThresholdSq, matrixWorld, raycaster, intersects, this );

				}

			}

		} else {

			console.error( 'THREE.Points.raycast() no longer supports THREE.Geometry. Use THREE.BufferGeometry instead.' );

		}

	},

	updateMorphTargets: function () {

		const geometry = this.geometry;

		if ( geometry.isBufferGeometry ) {

			const morphAttributes = geometry.morphAttributes;
			const keys = Object.keys( morphAttributes );

			if ( keys.length > 0 ) {

				const morphAttribute = morphAttributes[ keys[ 0 ] ];

				if ( morphAttribute !== undefined ) {

					this.morphTargetInfluences = [];
					this.morphTargetDictionary = {};

					for ( let m = 0, ml = morphAttribute.length; m < ml; m ++ ) {

						const name = morphAttribute[ m ].name || String( m );

						this.morphTargetInfluences.push( 0 );
						this.morphTargetDictionary[ name ] = m;

					}

				}

			}

		} else {

			const morphTargets = geometry.morphTargets;

			if ( morphTargets !== undefined && morphTargets.length > 0 ) {

				console.error( 'THREE.Points.updateMorphTargets() does not support THREE.Geometry. Use THREE.BufferGeometry instead.' );

			}

		}

	}

} );

function testPoint( point, index, localThresholdSq, matrixWorld, raycaster, intersects, object ) {

	const rayPointDistanceSq = _ray.distanceSqToPoint( point );

	if ( rayPointDistanceSq < localThresholdSq ) {

		const intersectPoint = new Vector3();

		_ray.closestPointToPoint( point, intersectPoint );
		intersectPoint.applyMatrix4( matrixWorld );

		const distance = raycaster.ray.origin.distanceTo( intersectPoint );

		if ( distance < raycaster.near || distance > raycaster.far ) return;

		intersects.push( {

			distance: distance,
			distanceToRay: Math.sqrt( rayPointDistanceSq ),
			point: intersectPoint,
			index: index,
			face: null,
			object: object

		} );

	}

}

class VideoTexture extends Texture {

	constructor( video, mapping, wrapS, wrapT, magFilter, minFilter, format, type, anisotropy ) {

		super( video, mapping, wrapS, wrapT, magFilter, minFilter, format, type, anisotropy );

		this.format = format !== undefined ? format : RGBFormat;

		this.minFilter = minFilter !== undefined ? minFilter : LinearFilter;
		this.magFilter = magFilter !== undefined ? magFilter : LinearFilter;

		this.generateMipmaps = false;

		const scope = this;

		function updateVideo() {

			scope.needsUpdate = true;
			video.requestVideoFrameCallback( updateVideo );

		}

		if ( 'requestVideoFrameCallback' in video ) {

			video.requestVideoFrameCallback( updateVideo );

		}

	}

	clone() {

		return new this.constructor( this.image ).copy( this );

	}

	update() {

		const video = this.image;
		const hasVideoFrameCallback = 'requestVideoFrameCallback' in video;

		if ( hasVideoFrameCallback === false && video.readyState >= video.HAVE_CURRENT_DATA ) {

			this.needsUpdate = true;

		}

	}

}

VideoTexture.prototype.isVideoTexture = true;

class CompressedTexture extends Texture {

	constructor( mipmaps, width, height, format, type, mapping, wrapS, wrapT, magFilter, minFilter, anisotropy, encoding ) {

		super( null, mapping, wrapS, wrapT, magFilter, minFilter, format, type, anisotropy, encoding );

		this.image = { width: width, height: height };
		this.mipmaps = mipmaps;

		// no flipping for cube textures
		// (also flipping doesn't work for compressed textures )

		this.flipY = false;

		// can't generate mipmaps for compressed textures
		// mips must be embedded in DDS files

		this.generateMipmaps = false;

	}

}

CompressedTexture.prototype.isCompressedTexture = true;

class CanvasTexture extends Texture {

	constructor( canvas, mapping, wrapS, wrapT, magFilter, minFilter, format, type, anisotropy ) {

		super( canvas, mapping, wrapS, wrapT, magFilter, minFilter, format, type, anisotropy );

		this.needsUpdate = true;

	}

}

CanvasTexture.prototype.isCanvasTexture = true;

class DepthTexture extends Texture {

	constructor( width, height, type, mapping, wrapS, wrapT, magFilter, minFilter, anisotropy, format ) {

		format = format !== undefined ? format : DepthFormat;

		if ( format !== DepthFormat && format !== DepthStencilFormat ) {

			throw new Error( 'DepthTexture format must be either THREE.DepthFormat or THREE.DepthStencilFormat' );

		}

		if ( type === undefined && format === DepthFormat ) type = UnsignedShortType;
		if ( type === undefined && format === DepthStencilFormat ) type = UnsignedInt248Type;

		super( null, mapping, wrapS, wrapT, magFilter, minFilter, format, type, anisotropy );

		this.image = { width: width, height: height };

		this.magFilter = magFilter !== undefined ? magFilter : NearestFilter;
		this.minFilter = minFilter !== undefined ? minFilter : NearestFilter;

		this.flipY = false;
		this.generateMipmaps	= false;

	}


}

DepthTexture.prototype.isDepthTexture = true;

class CircleGeometry extends BufferGeometry {

	constructor( radius = 1, segments = 8, thetaStart = 0, thetaLength = Math.PI * 2 ) {

		super();

		this.type = 'CircleGeometry';

		this.parameters = {
			radius: radius,
			segments: segments,
			thetaStart: thetaStart,
			thetaLength: thetaLength
		};

		segments = Math.max( 3, segments );

		// buffers

		const indices = [];
		const vertices = [];
		const normals = [];
		const uvs = [];

		// helper variables

		const vertex = new Vector3();
		const uv = new Vector2();

		// center point

		vertices.push( 0, 0, 0 );
		normals.push( 0, 0, 1 );
		uvs.push( 0.5, 0.5 );

		for ( let s = 0, i = 3; s <= segments; s ++, i += 3 ) {

			const segment = thetaStart + s / segments * thetaLength;

			// vertex

			vertex.x = radius * Math.cos( segment );
			vertex.y = radius * Math.sin( segment );

			vertices.push( vertex.x, vertex.y, vertex.z );

			// normal

			normals.push( 0, 0, 1 );

			// uvs

			uv.x = ( vertices[ i ] / radius + 1 ) / 2;
			uv.y = ( vertices[ i + 1 ] / radius + 1 ) / 2;

			uvs.push( uv.x, uv.y );

		}

		// indices

		for ( let i = 1; i <= segments; i ++ ) {

			indices.push( i, i + 1, 0 );

		}

		// build geometry

		this.setIndex( indices );
		this.setAttribute( 'position', new Float32BufferAttribute( vertices, 3 ) );
		this.setAttribute( 'normal', new Float32BufferAttribute( normals, 3 ) );
		this.setAttribute( 'uv', new Float32BufferAttribute( uvs, 2 ) );

	}

}

const _v0 = new Vector3();
const _v1$1 = new Vector3();
const _normal = new Vector3();
const _triangle = new Triangle();

class EdgesGeometry extends BufferGeometry {

	constructor( geometry, thresholdAngle ) {

		super();

		this.type = 'EdgesGeometry';

		this.parameters = {
			thresholdAngle: thresholdAngle
		};

		thresholdAngle = ( thresholdAngle !== undefined ) ? thresholdAngle : 1;

		if ( geometry.isGeometry === true ) {

			console.error( 'THREE.EdgesGeometry no longer supports THREE.Geometry. Use THREE.BufferGeometry instead.' );
			return;

		}

		const precisionPoints = 4;
		const precision = Math.pow( 10, precisionPoints );
		const thresholdDot = Math.cos( MathUtils.DEG2RAD * thresholdAngle );

		const indexAttr = geometry.getIndex();
		const positionAttr = geometry.getAttribute( 'position' );
		const indexCount = indexAttr ? indexAttr.count : positionAttr.count;

		const indexArr = [ 0, 0, 0 ];
		const vertKeys = [ 'a', 'b', 'c' ];
		const hashes = new Array( 3 );

		const edgeData = {};
		const vertices = [];
		for ( let i = 0; i < indexCount; i += 3 ) {

			if ( indexAttr ) {

				indexArr[ 0 ] = indexAttr.getX( i );
				indexArr[ 1 ] = indexAttr.getX( i + 1 );
				indexArr[ 2 ] = indexAttr.getX( i + 2 );

			} else {

				indexArr[ 0 ] = i;
				indexArr[ 1 ] = i + 1;
				indexArr[ 2 ] = i + 2;

			}

			const { a, b, c } = _triangle;
			a.fromBufferAttribute( positionAttr, indexArr[ 0 ] );
			b.fromBufferAttribute( positionAttr, indexArr[ 1 ] );
			c.fromBufferAttribute( positionAttr, indexArr[ 2 ] );
			_triangle.getNormal( _normal );

			// create hashes for the edge from the vertices
			hashes[ 0 ] = `${ Math.round( a.x * precision ) },${ Math.round( a.y * precision ) },${ Math.round( a.z * precision ) }`;
			hashes[ 1 ] = `${ Math.round( b.x * precision ) },${ Math.round( b.y * precision ) },${ Math.round( b.z * precision ) }`;
			hashes[ 2 ] = `${ Math.round( c.x * precision ) },${ Math.round( c.y * precision ) },${ Math.round( c.z * precision ) }`;

			// skip degenerate triangles
			if ( hashes[ 0 ] === hashes[ 1 ] || hashes[ 1 ] === hashes[ 2 ] || hashes[ 2 ] === hashes[ 0 ] ) {

				continue;

			}

			// iterate over every edge
			for ( let j = 0; j < 3; j ++ ) {

				// get the first and next vertex making up the edge
				const jNext = ( j + 1 ) % 3;
				const vecHash0 = hashes[ j ];
				const vecHash1 = hashes[ jNext ];
				const v0 = _triangle[ vertKeys[ j ] ];
				const v1 = _triangle[ vertKeys[ jNext ] ];

				const hash = `${ vecHash0 }_${ vecHash1 }`;
				const reverseHash = `${ vecHash1 }_${ vecHash0 }`;

				if ( reverseHash in edgeData && edgeData[ reverseHash ] ) {

					// if we found a sibling edge add it into the vertex array if
					// it meets the angle threshold and delete the edge from the map.
					if ( _normal.dot( edgeData[ reverseHash ].normal ) <= thresholdDot ) {

						vertices.push( v0.x, v0.y, v0.z );
						vertices.push( v1.x, v1.y, v1.z );

					}

					edgeData[ reverseHash ] = null;

				} else if ( ! ( hash in edgeData ) ) {

					// if we've already got an edge here then skip adding a new one
					edgeData[ hash ] = {

						index0: indexArr[ j ],
						index1: indexArr[ jNext ],
						normal: _normal.clone(),

					};

				}

			}

		}

		// iterate over all remaining, unmatched edges and add them to the vertex array
		for ( const key in edgeData ) {

			if ( edgeData[ key ] ) {

				const { index0, index1 } = edgeData[ key ];
				_v0.fromBufferAttribute( positionAttr, index0 );
				_v1$1.fromBufferAttribute( positionAttr, index1 );

				vertices.push( _v0.x, _v0.y, _v0.z );
				vertices.push( _v1$1.x, _v1$1.y, _v1$1.z );

			}

		}

		this.setAttribute( 'position', new Float32BufferAttribute( vertices, 3 ) );

	}

}

/**
 * Port from https://github.com/mapbox/earcut (v2.2.2)
 */

const Earcut = {

	triangulate: function ( data, holeIndices, dim ) {

		dim = dim || 2;

		const hasHoles = holeIndices && holeIndices.length;
		const outerLen = hasHoles ? holeIndices[ 0 ] * dim : data.length;
		let outerNode = linkedList( data, 0, outerLen, dim, true );
		const triangles = [];

		if ( ! outerNode || outerNode.next === outerNode.prev ) return triangles;

		let minX, minY, maxX, maxY, x, y, invSize;

		if ( hasHoles ) outerNode = eliminateHoles( data, holeIndices, outerNode, dim );

		// if the shape is not too simple, we'll use z-order curve hash later; calculate polygon bbox
		if ( data.length > 80 * dim ) {

			minX = maxX = data[ 0 ];
			minY = maxY = data[ 1 ];

			for ( let i = dim; i < outerLen; i += dim ) {

				x = data[ i ];
				y = data[ i + 1 ];
				if ( x < minX ) minX = x;
				if ( y < minY ) minY = y;
				if ( x > maxX ) maxX = x;
				if ( y > maxY ) maxY = y;

			}

			// minX, minY and invSize are later used to transform coords into integers for z-order calculation
			invSize = Math.max( maxX - minX, maxY - minY );
			invSize = invSize !== 0 ? 1 / invSize : 0;

		}

		earcutLinked( outerNode, triangles, dim, minX, minY, invSize );

		return triangles;

	}

};

// create a circular doubly linked list from polygon points in the specified winding order
function linkedList( data, start, end, dim, clockwise ) {

	let i, last;

	if ( clockwise === ( signedArea( data, start, end, dim ) > 0 ) ) {

		for ( i = start; i < end; i += dim ) last = insertNode( i, data[ i ], data[ i + 1 ], last );

	} else {

		for ( i = end - dim; i >= start; i -= dim ) last = insertNode( i, data[ i ], data[ i + 1 ], last );

	}

	if ( last && equals( last, last.next ) ) {

		removeNode( last );
		last = last.next;

	}

	return last;

}

// eliminate colinear or duplicate points
function filterPoints( start, end ) {

	if ( ! start ) return start;
	if ( ! end ) end = start;

	let p = start,
		again;
	do {

		again = false;

		if ( ! p.steiner && ( equals( p, p.next ) || area( p.prev, p, p.next ) === 0 ) ) {

			removeNode( p );
			p = end = p.prev;
			if ( p === p.next ) break;
			again = true;

		} else {

			p = p.next;

		}

	} while ( again || p !== end );

	return end;

}

// main ear slicing loop which triangulates a polygon (given as a linked list)
function earcutLinked( ear, triangles, dim, minX, minY, invSize, pass ) {

	if ( ! ear ) return;

	// interlink polygon nodes in z-order
	if ( ! pass && invSize ) indexCurve( ear, minX, minY, invSize );

	let stop = ear,
		prev, next;

	// iterate through ears, slicing them one by one
	while ( ear.prev !== ear.next ) {

		prev = ear.prev;
		next = ear.next;

		if ( invSize ? isEarHashed( ear, minX, minY, invSize ) : isEar( ear ) ) {

			// cut off the triangle
			triangles.push( prev.i / dim );
			triangles.push( ear.i / dim );
			triangles.push( next.i / dim );

			removeNode( ear );

			// skipping the next vertex leads to less sliver triangles
			ear = next.next;
			stop = next.next;

			continue;

		}

		ear = next;

		// if we looped through the whole remaining polygon and can't find any more ears
		if ( ear === stop ) {

			// try filtering points and slicing again
			if ( ! pass ) {

				earcutLinked( filterPoints( ear ), triangles, dim, minX, minY, invSize, 1 );

				// if this didn't work, try curing all small self-intersections locally

			} else if ( pass === 1 ) {

				ear = cureLocalIntersections( filterPoints( ear ), triangles, dim );
				earcutLinked( ear, triangles, dim, minX, minY, invSize, 2 );

				// as a last resort, try splitting the remaining polygon into two

			} else if ( pass === 2 ) {

				splitEarcut( ear, triangles, dim, minX, minY, invSize );

			}

			break;

		}

	}

}

// check whether a polygon node forms a valid ear with adjacent nodes
function isEar( ear ) {

	const a = ear.prev,
		b = ear,
		c = ear.next;

	if ( area( a, b, c ) >= 0 ) return false; // reflex, can't be an ear

	// now make sure we don't have other points inside the potential ear
	let p = ear.next.next;

	while ( p !== ear.prev ) {

		if ( pointInTriangle( a.x, a.y, b.x, b.y, c.x, c.y, p.x, p.y ) &&
			area( p.prev, p, p.next ) >= 0 ) return false;
		p = p.next;

	}

	return true;

}

function isEarHashed( ear, minX, minY, invSize ) {

	const a = ear.prev,
		b = ear,
		c = ear.next;

	if ( area( a, b, c ) >= 0 ) return false; // reflex, can't be an ear

	// triangle bbox; min & max are calculated like this for speed
	const minTX = a.x < b.x ? ( a.x < c.x ? a.x : c.x ) : ( b.x < c.x ? b.x : c.x ),
		minTY = a.y < b.y ? ( a.y < c.y ? a.y : c.y ) : ( b.y < c.y ? b.y : c.y ),
		maxTX = a.x > b.x ? ( a.x > c.x ? a.x : c.x ) : ( b.x > c.x ? b.x : c.x ),
		maxTY = a.y > b.y ? ( a.y > c.y ? a.y : c.y ) : ( b.y > c.y ? b.y : c.y );

	// z-order range for the current triangle bbox;
	const minZ = zOrder( minTX, minTY, minX, minY, invSize ),
		maxZ = zOrder( maxTX, maxTY, minX, minY, invSize );

	let p = ear.prevZ,
		n = ear.nextZ;

	// look for points inside the triangle in both directions
	while ( p && p.z >= minZ && n && n.z <= maxZ ) {

		if ( p !== ear.prev && p !== ear.next &&
			pointInTriangle( a.x, a.y, b.x, b.y, c.x, c.y, p.x, p.y ) &&
			area( p.prev, p, p.next ) >= 0 ) return false;
		p = p.prevZ;

		if ( n !== ear.prev && n !== ear.next &&
			pointInTriangle( a.x, a.y, b.x, b.y, c.x, c.y, n.x, n.y ) &&
			area( n.prev, n, n.next ) >= 0 ) return false;
		n = n.nextZ;

	}

	// look for remaining points in decreasing z-order
	while ( p && p.z >= minZ ) {

		if ( p !== ear.prev && p !== ear.next &&
			pointInTriangle( a.x, a.y, b.x, b.y, c.x, c.y, p.x, p.y ) &&
			area( p.prev, p, p.next ) >= 0 ) return false;
		p = p.prevZ;

	}

	// look for remaining points in increasing z-order
	while ( n && n.z <= maxZ ) {

		if ( n !== ear.prev && n !== ear.next &&
			pointInTriangle( a.x, a.y, b.x, b.y, c.x, c.y, n.x, n.y ) &&
			area( n.prev, n, n.next ) >= 0 ) return false;
		n = n.nextZ;

	}

	return true;

}

// go through all polygon nodes and cure small local self-intersections
function cureLocalIntersections( start, triangles, dim ) {

	let p = start;
	do {

		const a = p.prev,
			b = p.next.next;

		if ( ! equals( a, b ) && intersects( a, p, p.next, b ) && locallyInside( a, b ) && locallyInside( b, a ) ) {

			triangles.push( a.i / dim );
			triangles.push( p.i / dim );
			triangles.push( b.i / dim );

			// remove two nodes involved
			removeNode( p );
			removeNode( p.next );

			p = start = b;

		}

		p = p.next;

	} while ( p !== start );

	return filterPoints( p );

}

// try splitting polygon into two and triangulate them independently
function splitEarcut( start, triangles, dim, minX, minY, invSize ) {

	// look for a valid diagonal that divides the polygon into two
	let a = start;
	do {

		let b = a.next.next;
		while ( b !== a.prev ) {

			if ( a.i !== b.i && isValidDiagonal( a, b ) ) {

				// split the polygon in two by the diagonal
				let c = splitPolygon( a, b );

				// filter colinear points around the cuts
				a = filterPoints( a, a.next );
				c = filterPoints( c, c.next );

				// run earcut on each half
				earcutLinked( a, triangles, dim, minX, minY, invSize );
				earcutLinked( c, triangles, dim, minX, minY, invSize );
				return;

			}

			b = b.next;

		}

		a = a.next;

	} while ( a !== start );

}

// link every hole into the outer loop, producing a single-ring polygon without holes
function eliminateHoles( data, holeIndices, outerNode, dim ) {

	const queue = [];
	let i, len, start, end, list;

	for ( i = 0, len = holeIndices.length; i < len; i ++ ) {

		start = holeIndices[ i ] * dim;
		end = i < len - 1 ? holeIndices[ i + 1 ] * dim : data.length;
		list = linkedList( data, start, end, dim, false );
		if ( list === list.next ) list.steiner = true;
		queue.push( getLeftmost( list ) );

	}

	queue.sort( compareX );

	// process holes from left to right
	for ( i = 0; i < queue.length; i ++ ) {

		eliminateHole( queue[ i ], outerNode );
		outerNode = filterPoints( outerNode, outerNode.next );

	}

	return outerNode;

}

function compareX( a, b ) {

	return a.x - b.x;

}

// find a bridge between vertices that connects hole with an outer ring and and link it
function eliminateHole( hole, outerNode ) {

	outerNode = findHoleBridge( hole, outerNode );
	if ( outerNode ) {

		const b = splitPolygon( outerNode, hole );

		// filter collinear points around the cuts
		filterPoints( outerNode, outerNode.next );
		filterPoints( b, b.next );

	}

}

// David Eberly's algorithm for finding a bridge between hole and outer polygon
function findHoleBridge( hole, outerNode ) {

	let p = outerNode;
	const hx = hole.x;
	const hy = hole.y;
	let qx = - Infinity, m;

	// find a segment intersected by a ray from the hole's leftmost point to the left;
	// segment's endpoint with lesser x will be potential connection point
	do {

		if ( hy <= p.y && hy >= p.next.y && p.next.y !== p.y ) {

			const x = p.x + ( hy - p.y ) * ( p.next.x - p.x ) / ( p.next.y - p.y );
			if ( x <= hx && x > qx ) {

				qx = x;
				if ( x === hx ) {

					if ( hy === p.y ) return p;
					if ( hy === p.next.y ) return p.next;

				}

				m = p.x < p.next.x ? p : p.next;

			}

		}

		p = p.next;

	} while ( p !== outerNode );

	if ( ! m ) return null;

	if ( hx === qx ) return m; // hole touches outer segment; pick leftmost endpoint

	// look for points inside the triangle of hole point, segment intersection and endpoint;
	// if there are no points found, we have a valid connection;
	// otherwise choose the point of the minimum angle with the ray as connection point

	const stop = m,
		mx = m.x,
		my = m.y;
	let tanMin = Infinity, tan;

	p = m;

	do {

		if ( hx >= p.x && p.x >= mx && hx !== p.x &&
				pointInTriangle( hy < my ? hx : qx, hy, mx, my, hy < my ? qx : hx, hy, p.x, p.y ) ) {

			tan = Math.abs( hy - p.y ) / ( hx - p.x ); // tangential

			if ( locallyInside( p, hole ) && ( tan < tanMin || ( tan === tanMin && ( p.x > m.x || ( p.x === m.x && sectorContainsSector( m, p ) ) ) ) ) ) {

				m = p;
				tanMin = tan;

			}

		}

		p = p.next;

	} while ( p !== stop );

	return m;

}

// whether sector in vertex m contains sector in vertex p in the same coordinates
function sectorContainsSector( m, p ) {

	return area( m.prev, m, p.prev ) < 0 && area( p.next, m, m.next ) < 0;

}

// interlink polygon nodes in z-order
function indexCurve( start, minX, minY, invSize ) {

	let p = start;
	do {

		if ( p.z === null ) p.z = zOrder( p.x, p.y, minX, minY, invSize );
		p.prevZ = p.prev;
		p.nextZ = p.next;
		p = p.next;

	} while ( p !== start );

	p.prevZ.nextZ = null;
	p.prevZ = null;

	sortLinked( p );

}

// Simon Tatham's linked list merge sort algorithm
// http://www.chiark.greenend.org.uk/~sgtatham/algorithms/listsort.html
function sortLinked( list ) {

	let i, p, q, e, tail, numMerges, pSize, qSize,
		inSize = 1;

	do {

		p = list;
		list = null;
		tail = null;
		numMerges = 0;

		while ( p ) {

			numMerges ++;
			q = p;
			pSize = 0;
			for ( i = 0; i < inSize; i ++ ) {

				pSize ++;
				q = q.nextZ;
				if ( ! q ) break;

			}

			qSize = inSize;

			while ( pSize > 0 || ( qSize > 0 && q ) ) {

				if ( pSize !== 0 && ( qSize === 0 || ! q || p.z <= q.z ) ) {

					e = p;
					p = p.nextZ;
					pSize --;

				} else {

					e = q;
					q = q.nextZ;
					qSize --;

				}

				if ( tail ) tail.nextZ = e;
				else list = e;

				e.prevZ = tail;
				tail = e;

			}

			p = q;

		}

		tail.nextZ = null;
		inSize *= 2;

	} while ( numMerges > 1 );

	return list;

}

// z-order of a point given coords and inverse of the longer side of data bbox
function zOrder( x, y, minX, minY, invSize ) {

	// coords are transformed into non-negative 15-bit integer range
	x = 32767 * ( x - minX ) * invSize;
	y = 32767 * ( y - minY ) * invSize;

	x = ( x | ( x << 8 ) ) & 0x00FF00FF;
	x = ( x | ( x << 4 ) ) & 0x0F0F0F0F;
	x = ( x | ( x << 2 ) ) & 0x33333333;
	x = ( x | ( x << 1 ) ) & 0x55555555;

	y = ( y | ( y << 8 ) ) & 0x00FF00FF;
	y = ( y | ( y << 4 ) ) & 0x0F0F0F0F;
	y = ( y | ( y << 2 ) ) & 0x33333333;
	y = ( y | ( y << 1 ) ) & 0x55555555;

	return x | ( y << 1 );

}

// find the leftmost node of a polygon ring
function getLeftmost( start ) {

	let p = start,
		leftmost = start;
	do {

		if ( p.x < leftmost.x || ( p.x === leftmost.x && p.y < leftmost.y ) ) leftmost = p;
		p = p.next;

	} while ( p !== start );

	return leftmost;

}

// check if a point lies within a convex triangle
function pointInTriangle( ax, ay, bx, by, cx, cy, px, py ) {

	return ( cx - px ) * ( ay - py ) - ( ax - px ) * ( cy - py ) >= 0 &&
			( ax - px ) * ( by - py ) - ( bx - px ) * ( ay - py ) >= 0 &&
			( bx - px ) * ( cy - py ) - ( cx - px ) * ( by - py ) >= 0;

}

// check if a diagonal between two polygon nodes is valid (lies in polygon interior)
function isValidDiagonal( a, b ) {

	return a.next.i !== b.i && a.prev.i !== b.i && ! intersectsPolygon( a, b ) && // dones't intersect other edges
		( locallyInside( a, b ) && locallyInside( b, a ) && middleInside( a, b ) && // locally visible
		( area( a.prev, a, b.prev ) || area( a, b.prev, b ) ) || // does not create opposite-facing sectors
		equals( a, b ) && area( a.prev, a, a.next ) > 0 && area( b.prev, b, b.next ) > 0 ); // special zero-length case

}

// signed area of a triangle
function area( p, q, r ) {

	return ( q.y - p.y ) * ( r.x - q.x ) - ( q.x - p.x ) * ( r.y - q.y );

}

// check if two points are equal
function equals( p1, p2 ) {

	return p1.x === p2.x && p1.y === p2.y;

}

// check if two segments intersect
function intersects( p1, q1, p2, q2 ) {

	const o1 = sign( area( p1, q1, p2 ) );
	const o2 = sign( area( p1, q1, q2 ) );
	const o3 = sign( area( p2, q2, p1 ) );
	const o4 = sign( area( p2, q2, q1 ) );

	if ( o1 !== o2 && o3 !== o4 ) return true; // general case

	if ( o1 === 0 && onSegment( p1, p2, q1 ) ) return true; // p1, q1 and p2 are collinear and p2 lies on p1q1
	if ( o2 === 0 && onSegment( p1, q2, q1 ) ) return true; // p1, q1 and q2 are collinear and q2 lies on p1q1
	if ( o3 === 0 && onSegment( p2, p1, q2 ) ) return true; // p2, q2 and p1 are collinear and p1 lies on p2q2
	if ( o4 === 0 && onSegment( p2, q1, q2 ) ) return true; // p2, q2 and q1 are collinear and q1 lies on p2q2

	return false;

}

// for collinear points p, q, r, check if point q lies on segment pr
function onSegment( p, q, r ) {

	return q.x <= Math.max( p.x, r.x ) && q.x >= Math.min( p.x, r.x ) && q.y <= Math.max( p.y, r.y ) && q.y >= Math.min( p.y, r.y );

}

function sign( num ) {

	return num > 0 ? 1 : num < 0 ? - 1 : 0;

}

// check if a polygon diagonal intersects any polygon segments
function intersectsPolygon( a, b ) {

	let p = a;
	do {

		if ( p.i !== a.i && p.next.i !== a.i && p.i !== b.i && p.next.i !== b.i &&
				intersects( p, p.next, a, b ) ) return true;
		p = p.next;

	} while ( p !== a );

	return false;

}

// check if a polygon diagonal is locally inside the polygon
function locallyInside( a, b ) {

	return area( a.prev, a, a.next ) < 0 ?
		area( a, b, a.next ) >= 0 && area( a, a.prev, b ) >= 0 :
		area( a, b, a.prev ) < 0 || area( a, a.next, b ) < 0;

}

// check if the middle point of a polygon diagonal is inside the polygon
function middleInside( a, b ) {

	let p = a,
		inside = false;
	const px = ( a.x + b.x ) / 2,
		py = ( a.y + b.y ) / 2;
	do {

		if ( ( ( p.y > py ) !== ( p.next.y > py ) ) && p.next.y !== p.y &&
				( px < ( p.next.x - p.x ) * ( py - p.y ) / ( p.next.y - p.y ) + p.x ) )
			inside = ! inside;
		p = p.next;

	} while ( p !== a );

	return inside;

}

// link two polygon vertices with a bridge; if the vertices belong to the same ring, it splits polygon into two;
// if one belongs to the outer ring and another to a hole, it merges it into a single ring
function splitPolygon( a, b ) {

	const a2 = new Node( a.i, a.x, a.y ),
		b2 = new Node( b.i, b.x, b.y ),
		an = a.next,
		bp = b.prev;

	a.next = b;
	b.prev = a;

	a2.next = an;
	an.prev = a2;

	b2.next = a2;
	a2.prev = b2;

	bp.next = b2;
	b2.prev = bp;

	return b2;

}

// create a node and optionally link it with previous one (in a circular doubly linked list)
function insertNode( i, x, y, last ) {

	const p = new Node( i, x, y );

	if ( ! last ) {

		p.prev = p;
		p.next = p;

	} else {

		p.next = last.next;
		p.prev = last;
		last.next.prev = p;
		last.next = p;

	}

	return p;

}

function removeNode( p ) {

	p.next.prev = p.prev;
	p.prev.next = p.next;

	if ( p.prevZ ) p.prevZ.nextZ = p.nextZ;
	if ( p.nextZ ) p.nextZ.prevZ = p.prevZ;

}

function Node( i, x, y ) {

	// vertex index in coordinates array
	this.i = i;

	// vertex coordinates
	this.x = x;
	this.y = y;

	// previous and next vertex nodes in a polygon ring
	this.prev = null;
	this.next = null;

	// z-order curve value
	this.z = null;

	// previous and next nodes in z-order
	this.prevZ = null;
	this.nextZ = null;

	// indicates whether this is a steiner point
	this.steiner = false;

}

function signedArea( data, start, end, dim ) {

	let sum = 0;
	for ( let i = start, j = end - dim; i < end; i += dim ) {

		sum += ( data[ j ] - data[ i ] ) * ( data[ i + 1 ] + data[ j + 1 ] );
		j = i;

	}

	return sum;

}

const ShapeUtils = {

	// calculate area of the contour polygon

	area: function ( contour ) {

		const n = contour.length;
		let a = 0.0;

		for ( let p = n - 1, q = 0; q < n; p = q ++ ) {

			a += contour[ p ].x * contour[ q ].y - contour[ q ].x * contour[ p ].y;

		}

		return a * 0.5;

	},

	isClockWise: function ( pts ) {

		return ShapeUtils.area( pts ) < 0;

	},

	triangulateShape: function ( contour, holes ) {

		const vertices = []; // flat array of vertices like [ x0,y0, x1,y1, x2,y2, ... ]
		const holeIndices = []; // array of hole indices
		const faces = []; // final array of vertex indices like [ [ a,b,d ], [ b,c,d ] ]

		removeDupEndPts( contour );
		addContour( vertices, contour );

		//

		let holeIndex = contour.length;

		holes.forEach( removeDupEndPts );

		for ( let i = 0; i < holes.length; i ++ ) {

			holeIndices.push( holeIndex );
			holeIndex += holes[ i ].length;
			addContour( vertices, holes[ i ] );

		}

		//

		const triangles = Earcut.triangulate( vertices, holeIndices );

		//

		for ( let i = 0; i < triangles.length; i += 3 ) {

			faces.push( triangles.slice( i, i + 3 ) );

		}

		return faces;

	}

};

function removeDupEndPts( points ) {

	const l = points.length;

	if ( l > 2 && points[ l - 1 ].equals( points[ 0 ] ) ) {

		points.pop();

	}

}

function addContour( vertices, contour ) {

	for ( let i = 0; i < contour.length; i ++ ) {

		vertices.push( contour[ i ].x );
		vertices.push( contour[ i ].y );

	}

}

/**
 * Creates extruded geometry from a path shape.
 *
 * parameters = {
 *
 *  curveSegments: <int>, // number of points on the curves
 *  steps: <int>, // number of points for z-side extrusions / used for subdividing segments of extrude spline too
 *  depth: <float>, // Depth to extrude the shape
 *
 *  bevelEnabled: <bool>, // turn on bevel
 *  bevelThickness: <float>, // how deep into the original shape bevel goes
 *  bevelSize: <float>, // how far from shape outline (including bevelOffset) is bevel
 *  bevelOffset: <float>, // how far from shape outline does bevel start
 *  bevelSegments: <int>, // number of bevel layers
 *
 *  extrudePath: <THREE.Curve> // curve to extrude shape along
 *
 *  UVGenerator: <Object> // object that provides UV generator functions
 *
 * }
 */

class ExtrudeGeometry extends BufferGeometry {

	constructor( shapes, options ) {

		super();

		this.type = 'ExtrudeGeometry';

		this.parameters = {
			shapes: shapes,
			options: options
		};

		shapes = Array.isArray( shapes ) ? shapes : [ shapes ];

		const scope = this;

		const verticesArray = [];
		const uvArray = [];

		for ( let i = 0, l = shapes.length; i < l; i ++ ) {

			const shape = shapes[ i ];
			addShape( shape );

		}

		// build geometry

		this.setAttribute( 'position', new Float32BufferAttribute( verticesArray, 3 ) );
		this.setAttribute( 'uv', new Float32BufferAttribute( uvArray, 2 ) );

		this.computeVertexNormals();

		// functions

		function addShape( shape ) {

			const placeholder = [];

			// options

			const curveSegments = options.curveSegments !== undefined ? options.curveSegments : 12;
			const steps = options.steps !== undefined ? options.steps : 1;
			let depth = options.depth !== undefined ? options.depth : 100;

			let bevelEnabled = options.bevelEnabled !== undefined ? options.bevelEnabled : true;
			let bevelThickness = options.bevelThickness !== undefined ? options.bevelThickness : 6;
			let bevelSize = options.bevelSize !== undefined ? options.bevelSize : bevelThickness - 2;
			let bevelOffset = options.bevelOffset !== undefined ? options.bevelOffset : 0;
			let bevelSegments = options.bevelSegments !== undefined ? options.bevelSegments : 3;

			const extrudePath = options.extrudePath;

			const uvgen = options.UVGenerator !== undefined ? options.UVGenerator : WorldUVGenerator;

			// deprecated options

			if ( options.amount !== undefined ) {

				console.warn( 'THREE.ExtrudeBufferGeometry: amount has been renamed to depth.' );
				depth = options.amount;

			}

			//

			let extrudePts, extrudeByPath = false;
			let splineTube, binormal, normal, position2;

			if ( extrudePath ) {

				extrudePts = extrudePath.getSpacedPoints( steps );

				extrudeByPath = true;
				bevelEnabled = false; // bevels not supported for path extrusion

				// SETUP TNB variables

				// TODO1 - have a .isClosed in spline?

				splineTube = extrudePath.computeFrenetFrames( steps, false );

				// console.log(splineTube, 'splineTube', splineTube.normals.length, 'steps', steps, 'extrudePts', extrudePts.length);

				binormal = new Vector3();
				normal = new Vector3();
				position2 = new Vector3();

			}

			// Safeguards if bevels are not enabled

			if ( ! bevelEnabled ) {

				bevelSegments = 0;
				bevelThickness = 0;
				bevelSize = 0;
				bevelOffset = 0;

			}

			// Variables initialization

			const shapePoints = shape.extractPoints( curveSegments );

			let vertices = shapePoints.shape;
			const holes = shapePoints.holes;

			const reverse = ! ShapeUtils.isClockWise( vertices );

			if ( reverse ) {

				vertices = vertices.reverse();

				// Maybe we should also check if holes are in the opposite direction, just to be safe ...

				for ( let h = 0, hl = holes.length; h < hl; h ++ ) {

					const ahole = holes[ h ];

					if ( ShapeUtils.isClockWise( ahole ) ) {

						holes[ h ] = ahole.reverse();

					}

				}

			}


			const faces = ShapeUtils.triangulateShape( vertices, holes );

			/* Vertices */

			const contour = vertices; // vertices has all points but contour has only points of circumference

			for ( let h = 0, hl = holes.length; h < hl; h ++ ) {

				const ahole = holes[ h ];

				vertices = vertices.concat( ahole );

			}


			function scalePt2( pt, vec, size ) {

				if ( ! vec ) console.error( 'THREE.ExtrudeGeometry: vec does not exist' );

				return vec.clone().multiplyScalar( size ).add( pt );

			}

			const vlen = vertices.length, flen = faces.length;


			// Find directions for point movement


			function getBevelVec( inPt, inPrev, inNext ) {

				// computes for inPt the corresponding point inPt' on a new contour
				//   shifted by 1 unit (length of normalized vector) to the left
				// if we walk along contour clockwise, this new contour is outside the old one
				//
				// inPt' is the intersection of the two lines parallel to the two
				//  adjacent edges of inPt at a distance of 1 unit on the left side.

				let v_trans_x, v_trans_y, shrink_by; // resulting translation vector for inPt

				// good reading for geometry algorithms (here: line-line intersection)
				// http://geomalgorithms.com/a05-_intersect-1.html

				const v_prev_x = inPt.x - inPrev.x,
					v_prev_y = inPt.y - inPrev.y;
				const v_next_x = inNext.x - inPt.x,
					v_next_y = inNext.y - inPt.y;

				const v_prev_lensq = ( v_prev_x * v_prev_x + v_prev_y * v_prev_y );

				// check for collinear edges
				const collinear0 = ( v_prev_x * v_next_y - v_prev_y * v_next_x );

				if ( Math.abs( collinear0 ) > Number.EPSILON ) {

					// not collinear

					// length of vectors for normalizing

					const v_prev_len = Math.sqrt( v_prev_lensq );
					const v_next_len = Math.sqrt( v_next_x * v_next_x + v_next_y * v_next_y );

					// shift adjacent points by unit vectors to the left

					const ptPrevShift_x = ( inPrev.x - v_prev_y / v_prev_len );
					const ptPrevShift_y = ( inPrev.y + v_prev_x / v_prev_len );

					const ptNextShift_x = ( inNext.x - v_next_y / v_next_len );
					const ptNextShift_y = ( inNext.y + v_next_x / v_next_len );

					// scaling factor for v_prev to intersection point

					const sf = ( ( ptNextShift_x - ptPrevShift_x ) * v_next_y -
							( ptNextShift_y - ptPrevShift_y ) * v_next_x ) /
						( v_prev_x * v_next_y - v_prev_y * v_next_x );

					// vector from inPt to intersection point

					v_trans_x = ( ptPrevShift_x + v_prev_x * sf - inPt.x );
					v_trans_y = ( ptPrevShift_y + v_prev_y * sf - inPt.y );

					// Don't normalize!, otherwise sharp corners become ugly
					//  but prevent crazy spikes
					const v_trans_lensq = ( v_trans_x * v_trans_x + v_trans_y * v_trans_y );
					if ( v_trans_lensq <= 2 ) {

						return new Vector2( v_trans_x, v_trans_y );

					} else {

						shrink_by = Math.sqrt( v_trans_lensq / 2 );

					}

				} else {

					// handle special case of collinear edges

					let direction_eq = false; // assumes: opposite

					if ( v_prev_x > Number.EPSILON ) {

						if ( v_next_x > Number.EPSILON ) {

							direction_eq = true;

						}

					} else {

						if ( v_prev_x < - Number.EPSILON ) {

							if ( v_next_x < - Number.EPSILON ) {

								direction_eq = true;

							}

						} else {

							if ( Math.sign( v_prev_y ) === Math.sign( v_next_y ) ) {

								direction_eq = true;

							}

						}

					}

					if ( direction_eq ) {

						// console.log("Warning: lines are a straight sequence");
						v_trans_x = - v_prev_y;
						v_trans_y = v_prev_x;
						shrink_by = Math.sqrt( v_prev_lensq );

					} else {

						// console.log("Warning: lines are a straight spike");
						v_trans_x = v_prev_x;
						v_trans_y = v_prev_y;
						shrink_by = Math.sqrt( v_prev_lensq / 2 );

					}

				}

				return new Vector2( v_trans_x / shrink_by, v_trans_y / shrink_by );

			}


			const contourMovements = [];

			for ( let i = 0, il = contour.length, j = il - 1, k = i + 1; i < il; i ++, j ++, k ++ ) {

				if ( j === il ) j = 0;
				if ( k === il ) k = 0;

				//  (j)---(i)---(k)
				// console.log('i,j,k', i, j , k)

				contourMovements[ i ] = getBevelVec( contour[ i ], contour[ j ], contour[ k ] );

			}

			const holesMovements = [];
			let oneHoleMovements, verticesMovements = contourMovements.concat();

			for ( let h = 0, hl = holes.length; h < hl; h ++ ) {

				const ahole = holes[ h ];

				oneHoleMovements = [];

				for ( let i = 0, il = ahole.length, j = il - 1, k = i + 1; i < il; i ++, j ++, k ++ ) {

					if ( j === il ) j = 0;
					if ( k === il ) k = 0;

					//  (j)---(i)---(k)
					oneHoleMovements[ i ] = getBevelVec( ahole[ i ], ahole[ j ], ahole[ k ] );

				}

				holesMovements.push( oneHoleMovements );
				verticesMovements = verticesMovements.concat( oneHoleMovements );

			}


			// Loop bevelSegments, 1 for the front, 1 for the back

			for ( let b = 0; b < bevelSegments; b ++ ) {

				//for ( b = bevelSegments; b > 0; b -- ) {

				const t = b / bevelSegments;
				const z = bevelThickness * Math.cos( t * Math.PI / 2 );
				const bs = bevelSize * Math.sin( t * Math.PI / 2 ) + bevelOffset;

				// contract shape

				for ( let i = 0, il = contour.length; i < il; i ++ ) {

					const vert = scalePt2( contour[ i ], contourMovements[ i ], bs );

					v( vert.x, vert.y, - z );

				}

				// expand holes

				for ( let h = 0, hl = holes.length; h < hl; h ++ ) {

					const ahole = holes[ h ];
					oneHoleMovements = holesMovements[ h ];

					for ( let i = 0, il = ahole.length; i < il; i ++ ) {

						const vert = scalePt2( ahole[ i ], oneHoleMovements[ i ], bs );

						v( vert.x, vert.y, - z );

					}

				}

			}

			const bs = bevelSize + bevelOffset;

			// Back facing vertices

			for ( let i = 0; i < vlen; i ++ ) {

				const vert = bevelEnabled ? scalePt2( vertices[ i ], verticesMovements[ i ], bs ) : vertices[ i ];

				if ( ! extrudeByPath ) {

					v( vert.x, vert.y, 0 );

				} else {

					// v( vert.x, vert.y + extrudePts[ 0 ].y, extrudePts[ 0 ].x );

					normal.copy( splineTube.normals[ 0 ] ).multiplyScalar( vert.x );
					binormal.copy( splineTube.binormals[ 0 ] ).multiplyScalar( vert.y );

					position2.copy( extrudePts[ 0 ] ).add( normal ).add( binormal );

					v( position2.x, position2.y, position2.z );

				}

			}

			// Add stepped vertices...
			// Including front facing vertices

			for ( let s = 1; s <= steps; s ++ ) {

				for ( let i = 0; i < vlen; i ++ ) {

					const vert = bevelEnabled ? scalePt2( vertices[ i ], verticesMovements[ i ], bs ) : vertices[ i ];

					if ( ! extrudeByPath ) {

						v( vert.x, vert.y, depth / steps * s );

					} else {

						// v( vert.x, vert.y + extrudePts[ s - 1 ].y, extrudePts[ s - 1 ].x );

						normal.copy( splineTube.normals[ s ] ).multiplyScalar( vert.x );
						binormal.copy( splineTube.binormals[ s ] ).multiplyScalar( vert.y );

						position2.copy( extrudePts[ s ] ).add( normal ).add( binormal );

						v( position2.x, position2.y, position2.z );

					}

				}

			}


			// Add bevel segments planes

			//for ( b = 1; b <= bevelSegments; b ++ ) {
			for ( let b = bevelSegments - 1; b >= 0; b -- ) {

				const t = b / bevelSegments;
				const z = bevelThickness * Math.cos( t * Math.PI / 2 );
				const bs = bevelSize * Math.sin( t * Math.PI / 2 ) + bevelOffset;

				// contract shape

				for ( let i = 0, il = contour.length; i < il; i ++ ) {

					const vert = scalePt2( contour[ i ], contourMovements[ i ], bs );
					v( vert.x, vert.y, depth + z );

				}

				// expand holes

				for ( let h = 0, hl = holes.length; h < hl; h ++ ) {

					const ahole = holes[ h ];
					oneHoleMovements = holesMovements[ h ];

					for ( let i = 0, il = ahole.length; i < il; i ++ ) {

						const vert = scalePt2( ahole[ i ], oneHoleMovements[ i ], bs );

						if ( ! extrudeByPath ) {

							v( vert.x, vert.y, depth + z );

						} else {

							v( vert.x, vert.y + extrudePts[ steps - 1 ].y, extrudePts[ steps - 1 ].x + z );

						}

					}

				}

			}

			/* Faces */

			// Top and bottom faces

			buildLidFaces();

			// Sides faces

			buildSideFaces();


			/////  Internal functions

			function buildLidFaces() {

				const start = verticesArray.length / 3;

				if ( bevelEnabled ) {

					let layer = 0; // steps + 1
					let offset = vlen * layer;

					// Bottom faces

					for ( let i = 0; i < flen; i ++ ) {

						const face = faces[ i ];
						f3( face[ 2 ] + offset, face[ 1 ] + offset, face[ 0 ] + offset );

					}

					layer = steps + bevelSegments * 2;
					offset = vlen * layer;

					// Top faces

					for ( let i = 0; i < flen; i ++ ) {

						const face = faces[ i ];
						f3( face[ 0 ] + offset, face[ 1 ] + offset, face[ 2 ] + offset );

					}

				} else {

					// Bottom faces

					for ( let i = 0; i < flen; i ++ ) {

						const face = faces[ i ];
						f3( face[ 2 ], face[ 1 ], face[ 0 ] );

					}

					// Top faces

					for ( let i = 0; i < flen; i ++ ) {

						const face = faces[ i ];
						f3( face[ 0 ] + vlen * steps, face[ 1 ] + vlen * steps, face[ 2 ] + vlen * steps );

					}

				}

				scope.addGroup( start, verticesArray.length / 3 - start, 0 );

			}

			// Create faces for the z-sides of the shape

			function buildSideFaces() {

				const start = verticesArray.length / 3;
				let layeroffset = 0;
				sidewalls( contour, layeroffset );
				layeroffset += contour.length;

				for ( let h = 0, hl = holes.length; h < hl; h ++ ) {

					const ahole = holes[ h ];
					sidewalls( ahole, layeroffset );

					//, true
					layeroffset += ahole.length;

				}


				scope.addGroup( start, verticesArray.length / 3 - start, 1 );


			}

			function sidewalls( contour, layeroffset ) {

				let i = contour.length;

				while ( -- i >= 0 ) {

					const j = i;
					let k = i - 1;
					if ( k < 0 ) k = contour.length - 1;

					//console.log('b', i,j, i-1, k,vertices.length);

					for ( let s = 0, sl = ( steps + bevelSegments * 2 ); s < sl; s ++ ) {

						const slen1 = vlen * s;
						const slen2 = vlen * ( s + 1 );

						const a = layeroffset + j + slen1,
							b = layeroffset + k + slen1,
							c = layeroffset + k + slen2,
							d = layeroffset + j + slen2;

						f4( a, b, c, d );

					}

				}

			}

			function v( x, y, z ) {

				placeholder.push( x );
				placeholder.push( y );
				placeholder.push( z );

			}


			function f3( a, b, c ) {

				addVertex( a );
				addVertex( b );
				addVertex( c );

				const nextIndex = verticesArray.length / 3;
				const uvs = uvgen.generateTopUV( scope, verticesArray, nextIndex - 3, nextIndex - 2, nextIndex - 1 );

				addUV( uvs[ 0 ] );
				addUV( uvs[ 1 ] );
				addUV( uvs[ 2 ] );

			}

			function f4( a, b, c, d ) {

				addVertex( a );
				addVertex( b );
				addVertex( d );

				addVertex( b );
				addVertex( c );
				addVertex( d );


				const nextIndex = verticesArray.length / 3;
				const uvs = uvgen.generateSideWallUV( scope, verticesArray, nextIndex - 6, nextIndex - 3, nextIndex - 2, nextIndex - 1 );

				addUV( uvs[ 0 ] );
				addUV( uvs[ 1 ] );
				addUV( uvs[ 3 ] );

				addUV( uvs[ 1 ] );
				addUV( uvs[ 2 ] );
				addUV( uvs[ 3 ] );

			}

			function addVertex( index ) {

				verticesArray.push( placeholder[ index * 3 + 0 ] );
				verticesArray.push( placeholder[ index * 3 + 1 ] );
				verticesArray.push( placeholder[ index * 3 + 2 ] );

			}


			function addUV( vector2 ) {

				uvArray.push( vector2.x );
				uvArray.push( vector2.y );

			}

		}

	}

	toJSON() {

		const data = BufferGeometry.prototype.toJSON.call( this );

		const shapes = this.parameters.shapes;
		const options = this.parameters.options;

		return toJSON$1( shapes, options, data );

	}

}

const WorldUVGenerator = {

	generateTopUV: function ( geometry, vertices, indexA, indexB, indexC ) {

		const a_x = vertices[ indexA * 3 ];
		const a_y = vertices[ indexA * 3 + 1 ];
		const b_x = vertices[ indexB * 3 ];
		const b_y = vertices[ indexB * 3 + 1 ];
		const c_x = vertices[ indexC * 3 ];
		const c_y = vertices[ indexC * 3 + 1 ];

		return [
			new Vector2( a_x, a_y ),
			new Vector2( b_x, b_y ),
			new Vector2( c_x, c_y )
		];

	},

	generateSideWallUV: function ( geometry, vertices, indexA, indexB, indexC, indexD ) {

		const a_x = vertices[ indexA * 3 ];
		const a_y = vertices[ indexA * 3 + 1 ];
		const a_z = vertices[ indexA * 3 + 2 ];
		const b_x = vertices[ indexB * 3 ];
		const b_y = vertices[ indexB * 3 + 1 ];
		const b_z = vertices[ indexB * 3 + 2 ];
		const c_x = vertices[ indexC * 3 ];
		const c_y = vertices[ indexC * 3 + 1 ];
		const c_z = vertices[ indexC * 3 + 2 ];
		const d_x = vertices[ indexD * 3 ];
		const d_y = vertices[ indexD * 3 + 1 ];
		const d_z = vertices[ indexD * 3 + 2 ];

		if ( Math.abs( a_y - b_y ) < 0.01 ) {

			return [
				new Vector2( a_x, 1 - a_z ),
				new Vector2( b_x, 1 - b_z ),
				new Vector2( c_x, 1 - c_z ),
				new Vector2( d_x, 1 - d_z )
			];

		} else {

			return [
				new Vector2( a_y, 1 - a_z ),
				new Vector2( b_y, 1 - b_z ),
				new Vector2( c_y, 1 - c_z ),
				new Vector2( d_y, 1 - d_z )
			];

		}

	}

};

function toJSON$1( shapes, options, data ) {

	data.shapes = [];

	if ( Array.isArray( shapes ) ) {

		for ( let i = 0, l = shapes.length; i < l; i ++ ) {

			const shape = shapes[ i ];

			data.shapes.push( shape.uuid );

		}

	} else {

		data.shapes.push( shapes.uuid );

	}

	if ( options.extrudePath !== undefined ) data.options.extrudePath = options.extrudePath.toJSON();

	return data;

}

/**
 * Parametric Surfaces Geometry
 * based on the brilliant article by @prideout https://prideout.net/blog/old/blog/index.html@p=44.html
 */

function ParametricGeometry( func, slices, stacks ) {

	BufferGeometry.call( this );

	this.type = 'ParametricGeometry';

	this.parameters = {
		func: func,
		slices: slices,
		stacks: stacks
	};

	// buffers

	const indices = [];
	const vertices = [];
	const normals = [];
	const uvs = [];

	const EPS = 0.00001;

	const normal = new Vector3();

	const p0 = new Vector3(), p1 = new Vector3();
	const pu = new Vector3(), pv = new Vector3();

	if ( func.length < 3 ) {

		console.error( 'THREE.ParametricGeometry: Function must now modify a Vector3 as third parameter.' );

	}

	// generate vertices, normals and uvs

	const sliceCount = slices + 1;

	for ( let i = 0; i <= stacks; i ++ ) {

		const v = i / stacks;

		for ( let j = 0; j <= slices; j ++ ) {

			const u = j / slices;

			// vertex

			func( u, v, p0 );
			vertices.push( p0.x, p0.y, p0.z );

			// normal

			// approximate tangent vectors via finite differences

			if ( u - EPS >= 0 ) {

				func( u - EPS, v, p1 );
				pu.subVectors( p0, p1 );

			} else {

				func( u + EPS, v, p1 );
				pu.subVectors( p1, p0 );

			}

			if ( v - EPS >= 0 ) {

				func( u, v - EPS, p1 );
				pv.subVectors( p0, p1 );

			} else {

				func( u, v + EPS, p1 );
				pv.subVectors( p1, p0 );

			}

			// cross product of tangent vectors returns surface normal

			normal.crossVectors( pu, pv ).normalize();
			normals.push( normal.x, normal.y, normal.z );

			// uv

			uvs.push( u, v );

		}

	}

	// generate indices

	for ( let i = 0; i < stacks; i ++ ) {

		for ( let j = 0; j < slices; j ++ ) {

			const a = i * sliceCount + j;
			const b = i * sliceCount + j + 1;
			const c = ( i + 1 ) * sliceCount + j + 1;
			const d = ( i + 1 ) * sliceCount + j;

			// faces one and two

			indices.push( a, b, d );
			indices.push( b, c, d );

		}

	}

	// build geometry

	this.setIndex( indices );
	this.setAttribute( 'position', new Float32BufferAttribute( vertices, 3 ) );
	this.setAttribute( 'normal', new Float32BufferAttribute( normals, 3 ) );
	this.setAttribute( 'uv', new Float32BufferAttribute( uvs, 2 ) );

}

ParametricGeometry.prototype = Object.create( BufferGeometry.prototype );
ParametricGeometry.prototype.constructor = ParametricGeometry;

class RingGeometry extends BufferGeometry {

	constructor( innerRadius = 0.5, outerRadius = 1, thetaSegments = 8, phiSegments = 1, thetaStart = 0, thetaLength = Math.PI * 2 ) {

		super();

		this.type = 'RingGeometry';

		this.parameters = {
			innerRadius: innerRadius,
			outerRadius: outerRadius,
			thetaSegments: thetaSegments,
			phiSegments: phiSegments,
			thetaStart: thetaStart,
			thetaLength: thetaLength
		};

		thetaSegments = Math.max( 3, thetaSegments );
		phiSegments = Math.max( 1, phiSegments );

		// buffers

		const indices = [];
		const vertices = [];
		const normals = [];
		const uvs = [];

		// some helper variables

		let radius = innerRadius;
		const radiusStep = ( ( outerRadius - innerRadius ) / phiSegments );
		const vertex = new Vector3();
		const uv = new Vector2();

		// generate vertices, normals and uvs

		for ( let j = 0; j <= phiSegments; j ++ ) {

			for ( let i = 0; i <= thetaSegments; i ++ ) {

				// values are generate from the inside of the ring to the outside

				const segment = thetaStart + i / thetaSegments * thetaLength;

				// vertex

				vertex.x = radius * Math.cos( segment );
				vertex.y = radius * Math.sin( segment );

				vertices.push( vertex.x, vertex.y, vertex.z );

				// normal

				normals.push( 0, 0, 1 );

				// uv

				uv.x = ( vertex.x / outerRadius + 1 ) / 2;
				uv.y = ( vertex.y / outerRadius + 1 ) / 2;

				uvs.push( uv.x, uv.y );

			}

			// increase the radius for next row of vertices

			radius += radiusStep;

		}

		// indices

		for ( let j = 0; j < phiSegments; j ++ ) {

			const thetaSegmentLevel = j * ( thetaSegments + 1 );

			for ( let i = 0; i < thetaSegments; i ++ ) {

				const segment = i + thetaSegmentLevel;

				const a = segment;
				const b = segment + thetaSegments + 1;
				const c = segment + thetaSegments + 2;
				const d = segment + 1;

				// faces

				indices.push( a, b, d );
				indices.push( b, c, d );

			}

		}

		// build geometry

		this.setIndex( indices );
		this.setAttribute( 'position', new Float32BufferAttribute( vertices, 3 ) );
		this.setAttribute( 'normal', new Float32BufferAttribute( normals, 3 ) );
		this.setAttribute( 'uv', new Float32BufferAttribute( uvs, 2 ) );

	}

}

class ShapeGeometry extends BufferGeometry {

	constructor( shapes, curveSegments = 12 ) {

		super();
		this.type = 'ShapeGeometry';

		this.parameters = {
			shapes: shapes,
			curveSegments: curveSegments
		};

		// buffers

		const indices = [];
		const vertices = [];
		const normals = [];
		const uvs = [];

		// helper variables

		let groupStart = 0;
		let groupCount = 0;

		// allow single and array values for "shapes" parameter

		if ( Array.isArray( shapes ) === false ) {

			addShape( shapes );

		} else {

			for ( let i = 0; i < shapes.length; i ++ ) {

				addShape( shapes[ i ] );

				this.addGroup( groupStart, groupCount, i ); // enables MultiMaterial support

				groupStart += groupCount;
				groupCount = 0;

			}

		}

		// build geometry

		this.setIndex( indices );
		this.setAttribute( 'position', new Float32BufferAttribute( vertices, 3 ) );
		this.setAttribute( 'normal', new Float32BufferAttribute( normals, 3 ) );
		this.setAttribute( 'uv', new Float32BufferAttribute( uvs, 2 ) );


		// helper functions

		function addShape( shape ) {

			const indexOffset = vertices.length / 3;
			const points = shape.extractPoints( curveSegments );

			let shapeVertices = points.shape;
			const shapeHoles = points.holes;

			// check direction of vertices

			if ( ShapeUtils.isClockWise( shapeVertices ) === false ) {

				shapeVertices = shapeVertices.reverse();

			}

			for ( let i = 0, l = shapeHoles.length; i < l; i ++ ) {

				const shapeHole = shapeHoles[ i ];

				if ( ShapeUtils.isClockWise( shapeHole ) === true ) {

					shapeHoles[ i ] = shapeHole.reverse();

				}

			}

			const faces = ShapeUtils.triangulateShape( shapeVertices, shapeHoles );

			// join vertices of inner and outer paths to a single array

			for ( let i = 0, l = shapeHoles.length; i < l; i ++ ) {

				const shapeHole = shapeHoles[ i ];
				shapeVertices = shapeVertices.concat( shapeHole );

			}

			// vertices, normals, uvs

			for ( let i = 0, l = shapeVertices.length; i < l; i ++ ) {

				const vertex = shapeVertices[ i ];

				vertices.push( vertex.x, vertex.y, 0 );
				normals.push( 0, 0, 1 );
				uvs.push( vertex.x, vertex.y ); // world uvs

			}

			// incides

			for ( let i = 0, l = faces.length; i < l; i ++ ) {

				const face = faces[ i ];

				const a = face[ 0 ] + indexOffset;
				const b = face[ 1 ] + indexOffset;
				const c = face[ 2 ] + indexOffset;

				indices.push( a, b, c );
				groupCount += 3;

			}

		}

	}

	toJSON() {

		const data = BufferGeometry.prototype.toJSON.call( this );

		const shapes = this.parameters.shapes;

		return toJSON( shapes, data );

	}

}

function toJSON( shapes, data ) {

	data.shapes = [];

	if ( Array.isArray( shapes ) ) {

		for ( let i = 0, l = shapes.length; i < l; i ++ ) {

			const shape = shapes[ i ];

			data.shapes.push( shape.uuid );

		}

	} else {

		data.shapes.push( shapes.uuid );

	}

	return data;

}

/**
 * Text = 3D Text
 *
 * parameters = {
 *  font: <THREE.Font>, // font
 *
 *  size: <float>, // size of the text
 *  height: <float>, // thickness to extrude text
 *  curveSegments: <int>, // number of points on the curves
 *
 *  bevelEnabled: <bool>, // turn on bevel
 *  bevelThickness: <float>, // how deep into text bevel goes
 *  bevelSize: <float>, // how far from text outline (including bevelOffset) is bevel
 *  bevelOffset: <float> // how far from text outline does bevel start
 * }
 */

class TextGeometry extends ExtrudeGeometry {

	constructor( text, parameters = {} ) {

		const font = parameters.font;

		if ( ! ( font && font.isFont ) ) {

			console.error( 'THREE.TextGeometry: font parameter is not an instance of THREE.Font.' );
			return new BufferGeometry();

		}

		const shapes = font.generateShapes( text, parameters.size );

		// translate parameters to ExtrudeGeometry API

		parameters.depth = parameters.height !== undefined ? parameters.height : 50;

		// defaults

		if ( parameters.bevelThickness === undefined ) parameters.bevelThickness = 10;
		if ( parameters.bevelSize === undefined ) parameters.bevelSize = 8;
		if ( parameters.bevelEnabled === undefined ) parameters.bevelEnabled = false;

		super( shapes, parameters );

		this.type = 'TextGeometry';

	}

}

/**
 * parameters = {
 *  color: <THREE.Color>
 * }
 */

class ShadowMaterial extends Material {

	constructor( parameters ) {

		super();

		this.type = 'ShadowMaterial';

		this.color = new Color( 0x000000 );
		this.transparent = true;

		this.setValues( parameters );

	}

	copy( source ) {

		super.copy( source );

		this.color.copy( source.color );

		return this;

	}

}

ShadowMaterial.prototype.isShadowMaterial = true;

class RawShaderMaterial extends ShaderMaterial {

	constructor( parameters ) {

		super( parameters );

		this.type = 'RawShaderMaterial';

	}

}

RawShaderMaterial.prototype.isRawShaderMaterial = true;

/**
 * parameters = {
 *  color: <hex>,
 *  roughness: <float>,
 *  metalness: <float>,
 *  opacity: <float>,
 *
 *  map: new THREE.Texture( <Image> ),
 *
 *  lightMap: new THREE.Texture( <Image> ),
 *  lightMapIntensity: <float>
 *
 *  aoMap: new THREE.Texture( <Image> ),
 *  aoMapIntensity: <float>
 *
 *  emissive: <hex>,
 *  emissiveIntensity: <float>
 *  emissiveMap: new THREE.Texture( <Image> ),
 *
 *  bumpMap: new THREE.Texture( <Image> ),
 *  bumpScale: <float>,
 *
 *  normalMap: new THREE.Texture( <Image> ),
 *  normalMapType: THREE.TangentSpaceNormalMap,
 *  normalScale: <Vector2>,
 *
 *  displacementMap: new THREE.Texture( <Image> ),
 *  displacementScale: <float>,
 *  displacementBias: <float>,
 *
 *  roughnessMap: new THREE.Texture( <Image> ),
 *
 *  metalnessMap: new THREE.Texture( <Image> ),
 *
 *  alphaMap: new THREE.Texture( <Image> ),
 *
 *  envMap: new THREE.CubeTexture( [posx, negx, posy, negy, posz, negz] ),
 *  envMapIntensity: <float>
 *
 *  refractionRatio: <float>,
 *
 *  wireframe: <boolean>,
 *  wireframeLinewidth: <float>,
 *
 *  skinning: <bool>,
 *  morphTargets: <bool>,
 *  morphNormals: <bool>,
 *
 *  flatShading: <bool>
 * }
 */

function MeshStandardMaterial( parameters ) {

	Material.call( this );

	this.defines = { 'STANDARD': '' };

	this.type = 'MeshStandardMaterial';

	this.color = new Color( 0xffffff ); // diffuse
	this.roughness = 1.0;
	this.metalness = 0.0;

	this.map = null;

	this.lightMap = null;
	this.lightMapIntensity = 1.0;

	this.aoMap = null;
	this.aoMapIntensity = 1.0;

	this.emissive = new Color( 0x000000 );
	this.emissiveIntensity = 1.0;
	this.emissiveMap = null;

	this.bumpMap = null;
	this.bumpScale = 1;

	this.normalMap = null;
	this.normalMapType = TangentSpaceNormalMap;
	this.normalScale = new Vector2( 1, 1 );

	this.displacementMap = null;
	this.displacementScale = 1;
	this.displacementBias = 0;

	this.roughnessMap = null;

	this.metalnessMap = null;

	this.alphaMap = null;

	this.envMap = null;
	this.envMapIntensity = 1.0;

	this.refractionRatio = 0.98;

	this.wireframe = false;
	this.wireframeLinewidth = 1;
	this.wireframeLinecap = 'round';
	this.wireframeLinejoin = 'round';

	this.skinning = false;
	this.morphTargets = false;
	this.morphNormals = false;

	this.flatShading = false;

	this.vertexTangents = false;

	this.setValues( parameters );

}

MeshStandardMaterial.prototype = Object.create( Material.prototype );
MeshStandardMaterial.prototype.constructor = MeshStandardMaterial;

MeshStandardMaterial.prototype.isMeshStandardMaterial = true;

MeshStandardMaterial.prototype.copy = function ( source ) {

	Material.prototype.copy.call( this, source );

	this.defines = { 'STANDARD': '' };

	this.color.copy( source.color );
	this.roughness = source.roughness;
	this.metalness = source.metalness;

	this.map = source.map;

	this.lightMap = source.lightMap;
	this.lightMapIntensity = source.lightMapIntensity;

	this.aoMap = source.aoMap;
	this.aoMapIntensity = source.aoMapIntensity;

	this.emissive.copy( source.emissive );
	this.emissiveMap = source.emissiveMap;
	this.emissiveIntensity = source.emissiveIntensity;

	this.bumpMap = source.bumpMap;
	this.bumpScale = source.bumpScale;

	this.normalMap = source.normalMap;
	this.normalMapType = source.normalMapType;
	this.normalScale.copy( source.normalScale );

	this.displacementMap = source.displacementMap;
	this.displacementScale = source.displacementScale;
	this.displacementBias = source.displacementBias;

	this.roughnessMap = source.roughnessMap;

	this.metalnessMap = source.metalnessMap;

	this.alphaMap = source.alphaMap;

	this.envMap = source.envMap;
	this.envMapIntensity = source.envMapIntensity;

	this.refractionRatio = source.refractionRatio;

	this.wireframe = source.wireframe;
	this.wireframeLinewidth = source.wireframeLinewidth;
	this.wireframeLinecap = source.wireframeLinecap;
	this.wireframeLinejoin = source.wireframeLinejoin;

	this.skinning = source.skinning;
	this.morphTargets = source.morphTargets;
	this.morphNormals = source.morphNormals;

	this.flatShading = source.flatShading;

	this.vertexTangents = source.vertexTangents;

	return this;

};

/**
 * parameters = {
 *  clearcoat: <float>,
 *  clearcoatMap: new THREE.Texture( <Image> ),
 *  clearcoatRoughness: <float>,
 *  clearcoatRoughnessMap: new THREE.Texture( <Image> ),
 *  clearcoatNormalScale: <Vector2>,
 *  clearcoatNormalMap: new THREE.Texture( <Image> ),
 *
 *  reflectivity: <float>,
 *  ior: <float>,
 *
 *  sheen: <Color>,
 *
 *  transmission: <float>,
 *  transmissionMap: new THREE.Texture( <Image> )
 * }
 */

function MeshPhysicalMaterial( parameters ) {

	MeshStandardMaterial.call( this );

	this.defines = {

		'STANDARD': '',
		'PHYSICAL': ''

	};

	this.type = 'MeshPhysicalMaterial';

	this.clearcoat = 0.0;
	this.clearcoatMap = null;
	this.clearcoatRoughness = 0.0;
	this.clearcoatRoughnessMap = null;
	this.clearcoatNormalScale = new Vector2( 1, 1 );
	this.clearcoatNormalMap = null;

	this.reflectivity = 0.5; // maps to F0 = 0.04

	Object.defineProperty( this, 'ior', {
		get: function () {

			return ( 1 + 0.4 * this.reflectivity ) / ( 1 - 0.4 * this.reflectivity );

		},
		set: function ( ior ) {

			this.reflectivity = MathUtils.clamp( 2.5 * ( ior - 1 ) / ( ior + 1 ), 0, 1 );

		}
	} );

	this.sheen = null; // null will disable sheen bsdf

	this.transmission = 0.0;
	this.transmissionMap = null;

	this.setValues( parameters );

}

MeshPhysicalMaterial.prototype = Object.create( MeshStandardMaterial.prototype );
MeshPhysicalMaterial.prototype.constructor = MeshPhysicalMaterial;

MeshPhysicalMaterial.prototype.isMeshPhysicalMaterial = true;

MeshPhysicalMaterial.prototype.copy = function ( source ) {

	MeshStandardMaterial.prototype.copy.call( this, source );

	this.defines = {

		'STANDARD': '',
		'PHYSICAL': ''

	};

	this.clearcoat = source.clearcoat;
	this.clearcoatMap = source.clearcoatMap;
	this.clearcoatRoughness = source.clearcoatRoughness;
	this.clearcoatRoughnessMap = source.clearcoatRoughnessMap;
	this.clearcoatNormalMap = source.clearcoatNormalMap;
	this.clearcoatNormalScale.copy( source.clearcoatNormalScale );

	this.reflectivity = source.reflectivity;

	if ( source.sheen ) {

		this.sheen = ( this.sheen || new Color() ).copy( source.sheen );

	} else {

		this.sheen = null;

	}

	this.transmission = source.transmission;
	this.transmissionMap = source.transmissionMap;

	return this;

};

/**
 * parameters = {
 *  color: <hex>,
 *  specular: <hex>,
 *  shininess: <float>,
 *  opacity: <float>,
 *
 *  map: new THREE.Texture( <Image> ),
 *
 *  lightMap: new THREE.Texture( <Image> ),
 *  lightMapIntensity: <float>
 *
 *  aoMap: new THREE.Texture( <Image> ),
 *  aoMapIntensity: <float>
 *
 *  emissive: <hex>,
 *  emissiveIntensity: <float>
 *  emissiveMap: new THREE.Texture( <Image> ),
 *
 *  bumpMap: new THREE.Texture( <Image> ),
 *  bumpScale: <float>,
 *
 *  normalMap: new THREE.Texture( <Image> ),
 *  normalMapType: THREE.TangentSpaceNormalMap,
 *  normalScale: <Vector2>,
 *
 *  displacementMap: new THREE.Texture( <Image> ),
 *  displacementScale: <float>,
 *  displacementBias: <float>,
 *
 *  specularMap: new THREE.Texture( <Image> ),
 *
 *  alphaMap: new THREE.Texture( <Image> ),
 *
 *  envMap: new THREE.CubeTexture( [posx, negx, posy, negy, posz, negz] ),
 *  combine: THREE.MultiplyOperation,
 *  reflectivity: <float>,
 *  refractionRatio: <float>,
 *
 *  wireframe: <boolean>,
 *  wireframeLinewidth: <float>,
 *
 *  skinning: <bool>,
 *  morphTargets: <bool>,
 *  morphNormals: <bool>,
 *
 *  flatShading: <bool>
 * }
 */

class MeshPhongMaterial extends Material {

	constructor( parameters ) {

		super();

		this.type = 'MeshPhongMaterial';

		this.color = new Color( 0xffffff ); // diffuse
		this.specular = new Color( 0x111111 );
		this.shininess = 30;

		this.map = null;

		this.lightMap = null;
		this.lightMapIntensity = 1.0;

		this.aoMap = null;
		this.aoMapIntensity = 1.0;

		this.emissive = new Color( 0x000000 );
		this.emissiveIntensity = 1.0;
		this.emissiveMap = null;

		this.bumpMap = null;
		this.bumpScale = 1;

		this.normalMap = null;
		this.normalMapType = TangentSpaceNormalMap;
		this.normalScale = new Vector2( 1, 1 );

		this.displacementMap = null;
		this.displacementScale = 1;
		this.displacementBias = 0;

		this.specularMap = null;

		this.alphaMap = null;

		this.envMap = null;
		this.combine = MultiplyOperation;
		this.reflectivity = 1;
		this.refractionRatio = 0.98;

		this.wireframe = false;
		this.wireframeLinewidth = 1;
		this.wireframeLinecap = 'round';
		this.wireframeLinejoin = 'round';

		this.skinning = false;
		this.morphTargets = false;
		this.morphNormals = false;

		this.flatShading = false;

		this.setValues( parameters );

	}

	copy( source ) {

		super.copy( source );

		this.color.copy( source.color );
		this.specular.copy( source.specular );
		this.shininess = source.shininess;

		this.map = source.map;

		this.lightMap = source.lightMap;
		this.lightMapIntensity = source.lightMapIntensity;

		this.aoMap = source.aoMap;
		this.aoMapIntensity = source.aoMapIntensity;

		this.emissive.copy( source.emissive );
		this.emissiveMap = source.emissiveMap;
		this.emissiveIntensity = source.emissiveIntensity;

		this.bumpMap = source.bumpMap;
		this.bumpScale = source.bumpScale;

		this.normalMap = source.normalMap;
		this.normalMapType = source.normalMapType;
		this.normalScale.copy( source.normalScale );

		this.displacementMap = source.displacementMap;
		this.displacementScale = source.displacementScale;
		this.displacementBias = source.displacementBias;

		this.specularMap = source.specularMap;

		this.alphaMap = source.alphaMap;

		this.envMap = source.envMap;
		this.combine = source.combine;
		this.reflectivity = source.reflectivity;
		this.refractionRatio = source.refractionRatio;

		this.wireframe = source.wireframe;
		this.wireframeLinewidth = source.wireframeLinewidth;
		this.wireframeLinecap = source.wireframeLinecap;
		this.wireframeLinejoin = source.wireframeLinejoin;

		this.skinning = source.skinning;
		this.morphTargets = source.morphTargets;
		this.morphNormals = source.morphNormals;

		this.flatShading = source.flatShading;

		return this;

	}

}

MeshPhongMaterial.prototype.isMeshPhongMaterial = true;

/**
 * parameters = {
 *  color: <hex>,
 *
 *  map: new THREE.Texture( <Image> ),
 *  gradientMap: new THREE.Texture( <Image> ),
 *
 *  lightMap: new THREE.Texture( <Image> ),
 *  lightMapIntensity: <float>
 *
 *  aoMap: new THREE.Texture( <Image> ),
 *  aoMapIntensity: <float>
 *
 *  emissive: <hex>,
 *  emissiveIntensity: <float>
 *  emissiveMap: new THREE.Texture( <Image> ),
 *
 *  bumpMap: new THREE.Texture( <Image> ),
 *  bumpScale: <float>,
 *
 *  normalMap: new THREE.Texture( <Image> ),
 *  normalMapType: THREE.TangentSpaceNormalMap,
 *  normalScale: <Vector2>,
 *
 *  displacementMap: new THREE.Texture( <Image> ),
 *  displacementScale: <float>,
 *  displacementBias: <float>,
 *
 *  alphaMap: new THREE.Texture( <Image> ),
 *
 *  wireframe: <boolean>,
 *  wireframeLinewidth: <float>,
 *
 *  skinning: <bool>,
 *  morphTargets: <bool>,
 *  morphNormals: <bool>
 * }
 */

class MeshToonMaterial extends Material {

	constructor( parameters ) {

		super();

		this.defines = { 'TOON': '' };

		this.type = 'MeshToonMaterial';

		this.color = new Color( 0xffffff );

		this.map = null;
		this.gradientMap = null;

		this.lightMap = null;
		this.lightMapIntensity = 1.0;

		this.aoMap = null;
		this.aoMapIntensity = 1.0;

		this.emissive = new Color( 0x000000 );
		this.emissiveIntensity = 1.0;
		this.emissiveMap = null;

		this.bumpMap = null;
		this.bumpScale = 1;

		this.normalMap = null;
		this.normalMapType = TangentSpaceNormalMap;
		this.normalScale = new Vector2( 1, 1 );

		this.displacementMap = null;
		this.displacementScale = 1;
		this.displacementBias = 0;

		this.alphaMap = null;

		this.wireframe = false;
		this.wireframeLinewidth = 1;
		this.wireframeLinecap = 'round';
		this.wireframeLinejoin = 'round';

		this.skinning = false;
		this.morphTargets = false;
		this.morphNormals = false;

		this.setValues( parameters );

	}

	copy( source ) {

		super.copy( source );

		this.color.copy( source.color );

		this.map = source.map;
		this.gradientMap = source.gradientMap;

		this.lightMap = source.lightMap;
		this.lightMapIntensity = source.lightMapIntensity;

		this.aoMap = source.aoMap;
		this.aoMapIntensity = source.aoMapIntensity;

		this.emissive.copy( source.emissive );
		this.emissiveMap = source.emissiveMap;
		this.emissiveIntensity = source.emissiveIntensity;

		this.bumpMap = source.bumpMap;
		this.bumpScale = source.bumpScale;

		this.normalMap = source.normalMap;
		this.normalMapType = source.normalMapType;
		this.normalScale.copy( source.normalScale );

		this.displacementMap = source.displacementMap;
		this.displacementScale = source.displacementScale;
		this.displacementBias = source.displacementBias;

		this.alphaMap = source.alphaMap;

		this.wireframe = source.wireframe;
		this.wireframeLinewidth = source.wireframeLinewidth;
		this.wireframeLinecap = source.wireframeLinecap;
		this.wireframeLinejoin = source.wireframeLinejoin;

		this.skinning = source.skinning;
		this.morphTargets = source.morphTargets;
		this.morphNormals = source.morphNormals;

		return this;

	}

}

MeshToonMaterial.prototype.isMeshToonMaterial = true;

/**
 * parameters = {
 *  opacity: <float>,
 *
 *  bumpMap: new THREE.Texture( <Image> ),
 *  bumpScale: <float>,
 *
 *  normalMap: new THREE.Texture( <Image> ),
 *  normalMapType: THREE.TangentSpaceNormalMap,
 *  normalScale: <Vector2>,
 *
 *  displacementMap: new THREE.Texture( <Image> ),
 *  displacementScale: <float>,
 *  displacementBias: <float>,
 *
 *  wireframe: <boolean>,
 *  wireframeLinewidth: <float>
 *
 *  skinning: <bool>,
 *  morphTargets: <bool>,
 *  morphNormals: <bool>,
 *
 *  flatShading: <bool>
 * }
 */

class MeshNormalMaterial extends Material {

	constructor( parameters ) {

		super();

		this.type = 'MeshNormalMaterial';

		this.bumpMap = null;
		this.bumpScale = 1;

		this.normalMap = null;
		this.normalMapType = TangentSpaceNormalMap;
		this.normalScale = new Vector2( 1, 1 );

		this.displacementMap = null;
		this.displacementScale = 1;
		this.displacementBias = 0;

		this.wireframe = false;
		this.wireframeLinewidth = 1;

		this.fog = false;

		this.skinning = false;
		this.morphTargets = false;
		this.morphNormals = false;

		this.flatShading = false;

		this.setValues( parameters );

	}

	copy( source ) {

		super.copy( source );

		this.bumpMap = source.bumpMap;
		this.bumpScale = source.bumpScale;

		this.normalMap = source.normalMap;
		this.normalMapType = source.normalMapType;
		this.normalScale.copy( source.normalScale );

		this.displacementMap = source.displacementMap;
		this.displacementScale = source.displacementScale;
		this.displacementBias = source.displacementBias;

		this.wireframe = source.wireframe;
		this.wireframeLinewidth = source.wireframeLinewidth;

		this.skinning = source.skinning;
		this.morphTargets = source.morphTargets;
		this.morphNormals = source.morphNormals;

		this.flatShading = source.flatShading;

		return this;

	}

}

MeshNormalMaterial.prototype.isMeshNormalMaterial = true;

/**
 * parameters = {
 *  color: <hex>,
 *  opacity: <float>,
 *
 *  map: new THREE.Texture( <Image> ),
 *
 *  lightMap: new THREE.Texture( <Image> ),
 *  lightMapIntensity: <float>
 *
 *  aoMap: new THREE.Texture( <Image> ),
 *  aoMapIntensity: <float>
 *
 *  emissive: <hex>,
 *  emissiveIntensity: <float>
 *  emissiveMap: new THREE.Texture( <Image> ),
 *
 *  specularMap: new THREE.Texture( <Image> ),
 *
 *  alphaMap: new THREE.Texture( <Image> ),
 *
 *  envMap: new THREE.CubeTexture( [posx, negx, posy, negy, posz, negz] ),
 *  combine: THREE.Multiply,
 *  reflectivity: <float>,
 *  refractionRatio: <float>,
 *
 *  wireframe: <boolean>,
 *  wireframeLinewidth: <float>,
 *
 *  skinning: <bool>,
 *  morphTargets: <bool>,
 *  morphNormals: <bool>
 * }
 */

class MeshLambertMaterial extends Material {

	constructor( parameters ) {

		super();

		this.type = 'MeshLambertMaterial';

		this.color = new Color( 0xffffff ); // diffuse

		this.map = null;

		this.lightMap = null;
		this.lightMapIntensity = 1.0;

		this.aoMap = null;
		this.aoMapIntensity = 1.0;

		this.emissive = new Color( 0x000000 );
		this.emissiveIntensity = 1.0;
		this.emissiveMap = null;

		this.specularMap = null;

		this.alphaMap = null;

		this.envMap = null;
		this.combine = MultiplyOperation;
		this.reflectivity = 1;
		this.refractionRatio = 0.98;

		this.wireframe = false;
		this.wireframeLinewidth = 1;
		this.wireframeLinecap = 'round';
		this.wireframeLinejoin = 'round';

		this.skinning = false;
		this.morphTargets = false;
		this.morphNormals = false;

		this.setValues( parameters );

	}

	copy( source ) {

		super.copy( source );

		this.color.copy( source.color );

		this.map = source.map;

		this.lightMap = source.lightMap;
		this.lightMapIntensity = source.lightMapIntensity;

		this.aoMap = source.aoMap;
		this.aoMapIntensity = source.aoMapIntensity;

		this.emissive.copy( source.emissive );
		this.emissiveMap = source.emissiveMap;
		this.emissiveIntensity = source.emissiveIntensity;

		this.specularMap = source.specularMap;

		this.alphaMap = source.alphaMap;

		this.envMap = source.envMap;
		this.combine = source.combine;
		this.reflectivity = source.reflectivity;
		this.refractionRatio = source.refractionRatio;

		this.wireframe = source.wireframe;
		this.wireframeLinewidth = source.wireframeLinewidth;
		this.wireframeLinecap = source.wireframeLinecap;
		this.wireframeLinejoin = source.wireframeLinejoin;

		this.skinning = source.skinning;
		this.morphTargets = source.morphTargets;
		this.morphNormals = source.morphNormals;

		return this;

	}

}

MeshLambertMaterial.prototype.isMeshLambertMaterial = true;

/**
 * parameters = {
 *  color: <hex>,
 *  opacity: <float>,
 *
 *  matcap: new THREE.Texture( <Image> ),
 *
 *  map: new THREE.Texture( <Image> ),
 *
 *  bumpMap: new THREE.Texture( <Image> ),
 *  bumpScale: <float>,
 *
 *  normalMap: new THREE.Texture( <Image> ),
 *  normalMapType: THREE.TangentSpaceNormalMap,
 *  normalScale: <Vector2>,
 *
 *  displacementMap: new THREE.Texture( <Image> ),
 *  displacementScale: <float>,
 *  displacementBias: <float>,
 *
 *  alphaMap: new THREE.Texture( <Image> ),
 *
 *  skinning: <bool>,
 *  morphTargets: <bool>,
 *  morphNormals: <bool>
 *
 *  flatShading: <bool>
 * }
 */

class MeshMatcapMaterial extends Material {

	constructor( parameters ) {

		super();

		this.defines = { 'MATCAP': '' };

		this.type = 'MeshMatcapMaterial';

		this.color = new Color( 0xffffff ); // diffuse

		this.matcap = null;

		this.map = null;

		this.bumpMap = null;
		this.bumpScale = 1;

		this.normalMap = null;
		this.normalMapType = TangentSpaceNormalMap;
		this.normalScale = new Vector2( 1, 1 );

		this.displacementMap = null;
		this.displacementScale = 1;
		this.displacementBias = 0;

		this.alphaMap = null;

		this.skinning = false;
		this.morphTargets = false;
		this.morphNormals = false;

		this.flatShading = false;

		this.setValues( parameters );

	}


	copy( source ) {

		super.copy( source );

		this.defines = { 'MATCAP': '' };

		this.color.copy( source.color );

		this.matcap = source.matcap;

		this.map = source.map;

		this.bumpMap = source.bumpMap;
		this.bumpScale = source.bumpScale;

		this.normalMap = source.normalMap;
		this.normalMapType = source.normalMapType;
		this.normalScale.copy( source.normalScale );

		this.displacementMap = source.displacementMap;
		this.displacementScale = source.displacementScale;
		this.displacementBias = source.displacementBias;

		this.alphaMap = source.alphaMap;

		this.skinning = source.skinning;
		this.morphTargets = source.morphTargets;
		this.morphNormals = source.morphNormals;

		this.flatShading = source.flatShading;

		return this;

	}

}

MeshMatcapMaterial.prototype.isMeshMatcapMaterial = true;

/**
 * parameters = {
 *  color: <hex>,
 *  opacity: <float>,
 *
 *  linewidth: <float>,
 *
 *  scale: <float>,
 *  dashSize: <float>,
 *  gapSize: <float>
 * }
 */

class LineDashedMaterial extends LineBasicMaterial {

	constructor( parameters ) {

		super();

		this.type = 'LineDashedMaterial';

		this.scale = 1;
		this.dashSize = 3;
		this.gapSize = 1;

		this.setValues( parameters );

	}

	copy( source ) {

		super.copy( source );

		this.scale = source.scale;
		this.dashSize = source.dashSize;
		this.gapSize = source.gapSize;

		return this;

	}

}

LineDashedMaterial.prototype.isLineDashedMaterial = true;

const AnimationUtils = {

	// same as Array.prototype.slice, but also works on typed arrays
	arraySlice: function ( array, from, to ) {

		if ( AnimationUtils.isTypedArray( array ) ) {

			// in ios9 array.subarray(from, undefined) will return empty array
			// but array.subarray(from) or array.subarray(from, len) is correct
			return new array.constructor( array.subarray( from, to !== undefined ? to : array.length ) );

		}

		return array.slice( from, to );

	},

	// converts an array to a specific type
	convertArray: function ( array, type, forceClone ) {

		if ( ! array || // let 'undefined' and 'null' pass
			! forceClone && array.constructor === type ) return array;

		if ( typeof type.BYTES_PER_ELEMENT === 'number' ) {

			return new type( array ); // create typed array

		}

		return Array.prototype.slice.call( array ); // create Array

	},

	isTypedArray: function ( object ) {

		return ArrayBuffer.isView( object ) &&
			! ( object instanceof DataView );

	},

	// returns an array by which times and values can be sorted
	getKeyframeOrder: function ( times ) {

		function compareTime( i, j ) {

			return times[ i ] - times[ j ];

		}

		const n = times.length;
		const result = new Array( n );
		for ( let i = 0; i !== n; ++ i ) result[ i ] = i;

		result.sort( compareTime );

		return result;

	},

	// uses the array previously returned by 'getKeyframeOrder' to sort data
	sortedArray: function ( values, stride, order ) {

		const nValues = values.length;
		const result = new values.constructor( nValues );

		for ( let i = 0, dstOffset = 0; dstOffset !== nValues; ++ i ) {

			const srcOffset = order[ i ] * stride;

			for ( let j = 0; j !== stride; ++ j ) {

				result[ dstOffset ++ ] = values[ srcOffset + j ];

			}

		}

		return result;

	},

	// function for parsing AOS keyframe formats
	flattenJSON: function ( jsonKeys, times, values, valuePropertyName ) {

		let i = 1, key = jsonKeys[ 0 ];

		while ( key !== undefined && key[ valuePropertyName ] === undefined ) {

			key = jsonKeys[ i ++ ];

		}

		if ( key === undefined ) return; // no data

		let value = key[ valuePropertyName ];
		if ( value === undefined ) return; // no data

		if ( Array.isArray( value ) ) {

			do {

				value = key[ valuePropertyName ];

				if ( value !== undefined ) {

					times.push( key.time );
					values.push.apply( values, value ); // push all elements

				}

				key = jsonKeys[ i ++ ];

			} while ( key !== undefined );

		} else if ( value.toArray !== undefined ) {

			// ...assume THREE.Math-ish

			do {

				value = key[ valuePropertyName ];

				if ( value !== undefined ) {

					times.push( key.time );
					value.toArray( values, values.length );

				}

				key = jsonKeys[ i ++ ];

			} while ( key !== undefined );

		} else {

			// otherwise push as-is

			do {

				value = key[ valuePropertyName ];

				if ( value !== undefined ) {

					times.push( key.time );
					values.push( value );

				}

				key = jsonKeys[ i ++ ];

			} while ( key !== undefined );

		}

	},

	subclip: function ( sourceClip, name, startFrame, endFrame, fps = 30 ) {

		const clip = sourceClip.clone();

		clip.name = name;

		const tracks = [];

		for ( let i = 0; i < clip.tracks.length; ++ i ) {

			const track = clip.tracks[ i ];
			const valueSize = track.getValueSize();

			const times = [];
			const values = [];

			for ( let j = 0; j < track.times.length; ++ j ) {

				const frame = track.times[ j ] * fps;

				if ( frame < startFrame || frame >= endFrame ) continue;

				times.push( track.times[ j ] );

				for ( let k = 0; k < valueSize; ++ k ) {

					values.push( track.values[ j * valueSize + k ] );

				}

			}

			if ( times.length === 0 ) continue;

			track.times = AnimationUtils.convertArray( times, track.times.constructor );
			track.values = AnimationUtils.convertArray( values, track.values.constructor );

			tracks.push( track );

		}

		clip.tracks = tracks;

		// find minimum .times value across all tracks in the trimmed clip

		let minStartTime = Infinity;

		for ( let i = 0; i < clip.tracks.length; ++ i ) {

			if ( minStartTime > clip.tracks[ i ].times[ 0 ] ) {

				minStartTime = clip.tracks[ i ].times[ 0 ];

			}

		}

		// shift all tracks such that clip begins at t=0

		for ( let i = 0; i < clip.tracks.length; ++ i ) {

			clip.tracks[ i ].shift( - 1 * minStartTime );

		}

		clip.resetDuration();

		return clip;

	},

	makeClipAdditive: function ( targetClip, referenceFrame = 0, referenceClip = targetClip, fps = 30 ) {

		if ( fps <= 0 ) fps = 30;

		const numTracks = referenceClip.tracks.length;
		const referenceTime = referenceFrame / fps;

		// Make each track's values relative to the values at the reference frame
		for ( let i = 0; i < numTracks; ++ i ) {

			const referenceTrack = referenceClip.tracks[ i ];
			const referenceTrackType = referenceTrack.ValueTypeName;

			// Skip this track if it's non-numeric
			if ( referenceTrackType === 'bool' || referenceTrackType === 'string' ) continue;

			// Find the track in the target clip whose name and type matches the reference track
			const targetTrack = targetClip.tracks.find( function ( track ) {

				return track.name === referenceTrack.name
					&& track.ValueTypeName === referenceTrackType;

			} );

			if ( targetTrack === undefined ) continue;

			let referenceOffset = 0;
			const referenceValueSize = referenceTrack.getValueSize();

			if ( referenceTrack.createInterpolant.isInterpolantFactoryMethodGLTFCubicSpline ) {

				referenceOffset = referenceValueSize / 3;

			}

			let targetOffset = 0;
			const targetValueSize = targetTrack.getValueSize();

			if ( targetTrack.createInterpolant.isInterpolantFactoryMethodGLTFCubicSpline ) {

				targetOffset = targetValueSize / 3;

			}

			const lastIndex = referenceTrack.times.length - 1;
			let referenceValue;

			// Find the value to subtract out of the track
			if ( referenceTime <= referenceTrack.times[ 0 ] ) {

				// Reference frame is earlier than the first keyframe, so just use the first keyframe
				const startIndex = referenceOffset;
				const endIndex = referenceValueSize - referenceOffset;
				referenceValue = AnimationUtils.arraySlice( referenceTrack.values, startIndex, endIndex );

			} else if ( referenceTime >= referenceTrack.times[ lastIndex ] ) {

				// Reference frame is after the last keyframe, so just use the last keyframe
				const startIndex = lastIndex * referenceValueSize + referenceOffset;
				const endIndex = startIndex + referenceValueSize - referenceOffset;
				referenceValue = AnimationUtils.arraySlice( referenceTrack.values, startIndex, endIndex );

			} else {

				// Interpolate to the reference value
				const interpolant = referenceTrack.createInterpolant();
				const startIndex = referenceOffset;
				const endIndex = referenceValueSize - referenceOffset;
				interpolant.evaluate( referenceTime );
				referenceValue = AnimationUtils.arraySlice( interpolant.resultBuffer, startIndex, endIndex );

			}

			// Conjugate the quaternion
			if ( referenceTrackType === 'quaternion' ) {

				const referenceQuat = new Quaternion().fromArray( referenceValue ).normalize().conjugate();
				referenceQuat.toArray( referenceValue );

			}

			// Subtract the reference value from all of the track values

			const numTimes = targetTrack.times.length;
			for ( let j = 0; j < numTimes; ++ j ) {

				const valueStart = j * targetValueSize + targetOffset;

				if ( referenceTrackType === 'quaternion' ) {

					// Multiply the conjugate for quaternion track types
					Quaternion.multiplyQuaternionsFlat(
						targetTrack.values,
						valueStart,
						referenceValue,
						0,
						targetTrack.values,
						valueStart
					);

				} else {

					const valueEnd = targetValueSize - targetOffset * 2;

					// Subtract each value for all other numeric track types
					for ( let k = 0; k < valueEnd; ++ k ) {

						targetTrack.values[ valueStart + k ] -= referenceValue[ k ];

					}

				}

			}

		}

		targetClip.blendMode = AdditiveAnimationBlendMode;

		return targetClip;

	}

};

/**
 * Abstract base class of interpolants over parametric samples.
 *
 * The parameter domain is one dimensional, typically the time or a path
 * along a curve defined by the data.
 *
 * The sample values can have any dimensionality and derived classes may
 * apply special interpretations to the data.
 *
 * This class provides the interval seek in a Template Method, deferring
 * the actual interpolation to derived classes.
 *
 * Time complexity is O(1) for linear access crossing at most two points
 * and O(log N) for random access, where N is the number of positions.
 *
 * References:
 *
 * 		http://www.oodesign.com/template-method-pattern.html
 *
 */

function Interpolant( parameterPositions, sampleValues, sampleSize, resultBuffer ) {

	this.parameterPositions = parameterPositions;
	this._cachedIndex = 0;

	this.resultBuffer = resultBuffer !== undefined ?
		resultBuffer : new sampleValues.constructor( sampleSize );
	this.sampleValues = sampleValues;
	this.valueSize = sampleSize;

}

Object.assign( Interpolant.prototype, {

	evaluate: function ( t ) {

		const pp = this.parameterPositions;
		let i1 = this._cachedIndex,
			t1 = pp[ i1 ],
			t0 = pp[ i1 - 1 ];

		validate_interval: {

			seek: {

				let right;

				linear_scan: {

					//- See http://jsperf.com/comparison-to-undefined/3
					//- slower code:
					//-
					//- 				if ( t >= t1 || t1 === undefined ) {
					forward_scan: if ( ! ( t < t1 ) ) {

						for ( let giveUpAt = i1 + 2; ; ) {

							if ( t1 === undefined ) {

								if ( t < t0 ) break forward_scan;

								// after end

								i1 = pp.length;
								this._cachedIndex = i1;
								return this.afterEnd_( i1 - 1, t, t0 );

							}

							if ( i1 === giveUpAt ) break; // this loop

							t0 = t1;
							t1 = pp[ ++ i1 ];

							if ( t < t1 ) {

								// we have arrived at the sought interval
								break seek;

							}

						}

						// prepare binary search on the right side of the index
						right = pp.length;
						break linear_scan;

					}

					//- slower code:
					//-					if ( t < t0 || t0 === undefined ) {
					if ( ! ( t >= t0 ) ) {

						// looping?

						const t1global = pp[ 1 ];

						if ( t < t1global ) {

							i1 = 2; // + 1, using the scan for the details
							t0 = t1global;

						}

						// linear reverse scan

						for ( let giveUpAt = i1 - 2; ; ) {

							if ( t0 === undefined ) {

								// before start

								this._cachedIndex = 0;
								return this.beforeStart_( 0, t, t1 );

							}

							if ( i1 === giveUpAt ) break; // this loop

							t1 = t0;
							t0 = pp[ -- i1 - 1 ];

							if ( t >= t0 ) {

								// we have arrived at the sought interval
								break seek;

							}

						}

						// prepare binary search on the left side of the index
						right = i1;
						i1 = 0;
						break linear_scan;

					}

					// the interval is valid

					break validate_interval;

				} // linear scan

				// binary search

				while ( i1 < right ) {

					const mid = ( i1 + right ) >>> 1;

					if ( t < pp[ mid ] ) {

						right = mid;

					} else {

						i1 = mid + 1;

					}

				}

				t1 = pp[ i1 ];
				t0 = pp[ i1 - 1 ];

				// check boundary cases, again

				if ( t0 === undefined ) {

					this._cachedIndex = 0;
					return this.beforeStart_( 0, t, t1 );

				}

				if ( t1 === undefined ) {

					i1 = pp.length;
					this._cachedIndex = i1;
					return this.afterEnd_( i1 - 1, t0, t );

				}

			} // seek

			this._cachedIndex = i1;

			this.intervalChanged_( i1, t0, t1 );

		} // validate_interval

		return this.interpolate_( i1, t0, t, t1 );

	},

	settings: null, // optional, subclass-specific settings structure
	// Note: The indirection allows central control of many interpolants.

	// --- Protected interface

	DefaultSettings_: {},

	getSettings_: function () {

		return this.settings || this.DefaultSettings_;

	},

	copySampleValue_: function ( index ) {

		// copies a sample value to the result buffer

		const result = this.resultBuffer,
			values = this.sampleValues,
			stride = this.valueSize,
			offset = index * stride;

		for ( let i = 0; i !== stride; ++ i ) {

			result[ i ] = values[ offset + i ];

		}

		return result;

	},

	// Template methods for derived classes:

	interpolate_: function ( /* i1, t0, t, t1 */ ) {

		throw new Error( 'call to abstract method' );
		// implementations shall return this.resultBuffer

	},

	intervalChanged_: function ( /* i1, t0, t1 */ ) {

		// empty

	}

} );

// DECLARE ALIAS AFTER assign prototype
Object.assign( Interpolant.prototype, {

	//( 0, t, t0 ), returns this.resultBuffer
	beforeStart_: Interpolant.prototype.copySampleValue_,

	//( N-1, tN-1, t ), returns this.resultBuffer
	afterEnd_: Interpolant.prototype.copySampleValue_,

} );

/**
 * Fast and simple cubic spline interpolant.
 *
 * It was derived from a Hermitian construction setting the first derivative
 * at each sample position to the linear slope between neighboring positions
 * over their parameter interval.
 */

function CubicInterpolant( parameterPositions, sampleValues, sampleSize, resultBuffer ) {

	Interpolant.call( this, parameterPositions, sampleValues, sampleSize, resultBuffer );

	this._weightPrev = - 0;
	this._offsetPrev = - 0;
	this._weightNext = - 0;
	this._offsetNext = - 0;

}

CubicInterpolant.prototype = Object.assign( Object.create( Interpolant.prototype ), {

	constructor: CubicInterpolant,

	DefaultSettings_: {

		endingStart: ZeroCurvatureEnding,
		endingEnd: ZeroCurvatureEnding

	},

	intervalChanged_: function ( i1, t0, t1 ) {

		const pp = this.parameterPositions;
		let iPrev = i1 - 2,
			iNext = i1 + 1,

			tPrev = pp[ iPrev ],
			tNext = pp[ iNext ];

		if ( tPrev === undefined ) {

			switch ( this.getSettings_().endingStart ) {

				case ZeroSlopeEnding:

					// f'(t0) = 0
					iPrev = i1;
					tPrev = 2 * t0 - t1;

					break;

				case WrapAroundEnding:

					// use the other end of the curve
					iPrev = pp.length - 2;
					tPrev = t0 + pp[ iPrev ] - pp[ iPrev + 1 ];

					break;

				default: // ZeroCurvatureEnding

					// f''(t0) = 0 a.k.a. Natural Spline
					iPrev = i1;
					tPrev = t1;

			}

		}

		if ( tNext === undefined ) {

			switch ( this.getSettings_().endingEnd ) {

				case ZeroSlopeEnding:

					// f'(tN) = 0
					iNext = i1;
					tNext = 2 * t1 - t0;

					break;

				case WrapAroundEnding:

					// use the other end of the curve
					iNext = 1;
					tNext = t1 + pp[ 1 ] - pp[ 0 ];

					break;

				default: // ZeroCurvatureEnding

					// f''(tN) = 0, a.k.a. Natural Spline
					iNext = i1 - 1;
					tNext = t0;

			}

		}

		const halfDt = ( t1 - t0 ) * 0.5,
			stride = this.valueSize;

		this._weightPrev = halfDt / ( t0 - tPrev );
		this._weightNext = halfDt / ( tNext - t1 );
		this._offsetPrev = iPrev * stride;
		this._offsetNext = iNext * stride;

	},

	interpolate_: function ( i1, t0, t, t1 ) {

		const result = this.resultBuffer,
			values = this.sampleValues,
			stride = this.valueSize,

			o1 = i1 * stride,		o0 = o1 - stride,
			oP = this._offsetPrev, 	oN = this._offsetNext,
			wP = this._weightPrev,	wN = this._weightNext,

			p = ( t - t0 ) / ( t1 - t0 ),
			pp = p * p,
			ppp = pp * p;

		// evaluate polynomials

		const sP = - wP * ppp + 2 * wP * pp - wP * p;
		const s0 = ( 1 + wP ) * ppp + ( - 1.5 - 2 * wP ) * pp + ( - 0.5 + wP ) * p + 1;
		const s1 = ( - 1 - wN ) * ppp + ( 1.5 + wN ) * pp + 0.5 * p;
		const sN = wN * ppp - wN * pp;

		// combine data linearly

		for ( let i = 0; i !== stride; ++ i ) {

			result[ i ] =
					sP * values[ oP + i ] +
					s0 * values[ o0 + i ] +
					s1 * values[ o1 + i ] +
					sN * values[ oN + i ];

		}

		return result;

	}

} );

function LinearInterpolant( parameterPositions, sampleValues, sampleSize, resultBuffer ) {

	Interpolant.call( this, parameterPositions, sampleValues, sampleSize, resultBuffer );

}

LinearInterpolant.prototype = Object.assign( Object.create( Interpolant.prototype ), {

	constructor: LinearInterpolant,

	interpolate_: function ( i1, t0, t, t1 ) {

		const result = this.resultBuffer,
			values = this.sampleValues,
			stride = this.valueSize,

			offset1 = i1 * stride,
			offset0 = offset1 - stride,

			weight1 = ( t - t0 ) / ( t1 - t0 ),
			weight0 = 1 - weight1;

		for ( let i = 0; i !== stride; ++ i ) {

			result[ i ] =
					values[ offset0 + i ] * weight0 +
					values[ offset1 + i ] * weight1;

		}

		return result;

	}

} );

/**
 *
 * Interpolant that evaluates to the sample value at the position preceeding
 * the parameter.
 */

function DiscreteInterpolant( parameterPositions, sampleValues, sampleSize, resultBuffer ) {

	Interpolant.call( this, parameterPositions, sampleValues, sampleSize, resultBuffer );

}

DiscreteInterpolant.prototype = Object.assign( Object.create( Interpolant.prototype ), {

	constructor: DiscreteInterpolant,

	interpolate_: function ( i1 /*, t0, t, t1 */ ) {

		return this.copySampleValue_( i1 - 1 );

	}

} );

class KeyframeTrack {

	constructor( name, times, values, interpolation ) {

		if ( name === undefined ) throw new Error( 'THREE.KeyframeTrack: track name is undefined' );
		if ( times === undefined || times.length === 0 ) throw new Error( 'THREE.KeyframeTrack: no keyframes in track named ' + name );

		this.name = name;

		this.times = AnimationUtils.convertArray( times, this.TimeBufferType );
		this.values = AnimationUtils.convertArray( values, this.ValueBufferType );

		this.setInterpolation( interpolation || this.DefaultInterpolation );

	}

	// Serialization (in static context, because of constructor invocation
	// and automatic invocation of .toJSON):

	static toJSON( track ) {

		const trackType = track.constructor;

		let json;

		// derived classes can define a static toJSON method
		if ( trackType.toJSON !== this.toJSON ) {

			json = trackType.toJSON( track );

		} else {

			// by default, we assume the data can be serialized as-is
			json = {

				'name': track.name,
				'times': AnimationUtils.convertArray( track.times, Array ),
				'values': AnimationUtils.convertArray( track.values, Array )

			};

			const interpolation = track.getInterpolation();

			if ( interpolation !== track.DefaultInterpolation ) {

				json.interpolation = interpolation;

			}

		}

		json.type = track.ValueTypeName; // mandatory

		return json;

	}

	InterpolantFactoryMethodDiscrete( result ) {

		return new DiscreteInterpolant( this.times, this.values, this.getValueSize(), result );

	}

	InterpolantFactoryMethodLinear( result ) {

		return new LinearInterpolant( this.times, this.values, this.getValueSize(), result );

	}

	InterpolantFactoryMethodSmooth( result ) {

		return new CubicInterpolant( this.times, this.values, this.getValueSize(), result );

	}

	setInterpolation( interpolation ) {

		let factoryMethod;

		switch ( interpolation ) {

			case InterpolateDiscrete:

				factoryMethod = this.InterpolantFactoryMethodDiscrete;

				break;

			case InterpolateLinear:

				factoryMethod = this.InterpolantFactoryMethodLinear;

				break;

			case InterpolateSmooth:

				factoryMethod = this.InterpolantFactoryMethodSmooth;

				break;

		}

		if ( factoryMethod === undefined ) {

			const message = 'unsupported interpolation for ' +
				this.ValueTypeName + ' keyframe track named ' + this.name;

			if ( this.createInterpolant === undefined ) {

				// fall back to default, unless the default itself is messed up
				if ( interpolation !== this.DefaultInterpolation ) {

					this.setInterpolation( this.DefaultInterpolation );

				} else {

					throw new Error( message ); // fatal, in this case

				}

			}

			console.warn( 'THREE.KeyframeTrack:', message );
			return this;

		}

		this.createInterpolant = factoryMethod;

		return this;

	}

	getInterpolation() {

		switch ( this.createInterpolant ) {

			case this.InterpolantFactoryMethodDiscrete:

				return InterpolateDiscrete;

			case this.InterpolantFactoryMethodLinear:

				return InterpolateLinear;

			case this.InterpolantFactoryMethodSmooth:

				return InterpolateSmooth;

		}

	}

	getValueSize() {

		return this.values.length / this.times.length;

	}

	// move all keyframes either forwards or backwards in time
	shift( timeOffset ) {

		if ( timeOffset !== 0.0 ) {

			const times = this.times;

			for ( let i = 0, n = times.length; i !== n; ++ i ) {

				times[ i ] += timeOffset;

			}

		}

		return this;

	}

	// scale all keyframe times by a factor (useful for frame <-> seconds conversions)
	scale( timeScale ) {

		if ( timeScale !== 1.0 ) {

			const times = this.times;

			for ( let i = 0, n = times.length; i !== n; ++ i ) {

				times[ i ] *= timeScale;

			}

		}

		return this;

	}

	// removes keyframes before and after animation without changing any values within the range [startTime, endTime].
	// IMPORTANT: We do not shift around keys to the start of the track time, because for interpolated keys this will change their values
	trim( startTime, endTime ) {

		const times = this.times,
			nKeys = times.length;

		let from = 0,
			to = nKeys - 1;

		while ( from !== nKeys && times[ from ] < startTime ) {

			++ from;

		}

		while ( to !== - 1 && times[ to ] > endTime ) {

			-- to;

		}

		++ to; // inclusive -> exclusive bound

		if ( from !== 0 || to !== nKeys ) {

			// empty tracks are forbidden, so keep at least one keyframe
			if ( from >= to ) {

				to = Math.max( to, 1 );
				from = to - 1;

			}

			const stride = this.getValueSize();
			this.times = AnimationUtils.arraySlice( times, from, to );
			this.values = AnimationUtils.arraySlice( this.values, from * stride, to * stride );

		}

		return this;

	}

	// ensure we do not get a GarbageInGarbageOut situation, make sure tracks are at least minimally viable
	validate() {

		let valid = true;

		const valueSize = this.getValueSize();
		if ( valueSize - Math.floor( valueSize ) !== 0 ) {

			console.error( 'THREE.KeyframeTrack: Invalid value size in track.', this );
			valid = false;

		}

		const times = this.times,
			values = this.values,

			nKeys = times.length;

		if ( nKeys === 0 ) {

			console.error( 'THREE.KeyframeTrack: Track is empty.', this );
			valid = false;

		}

		let prevTime = null;

		for ( let i = 0; i !== nKeys; i ++ ) {

			const currTime = times[ i ];

			if ( typeof currTime === 'number' && isNaN( currTime ) ) {

				console.error( 'THREE.KeyframeTrack: Time is not a valid number.', this, i, currTime );
				valid = false;
				break;

			}

			if ( prevTime !== null && prevTime > currTime ) {

				console.error( 'THREE.KeyframeTrack: Out of order keys.', this, i, currTime, prevTime );
				valid = false;
				break;

			}

			prevTime = currTime;

		}

		if ( values !== undefined ) {

			if ( AnimationUtils.isTypedArray( values ) ) {

				for ( let i = 0, n = values.length; i !== n; ++ i ) {

					const value = values[ i ];

					if ( isNaN( value ) ) {

						console.error( 'THREE.KeyframeTrack: Value is not a valid number.', this, i, value );
						valid = false;
						break;

					}

				}

			}

		}

		return valid;

	}

	// removes equivalent sequential keys as common in morph target sequences
	// (0,0,0,0,1,1,1,0,0,0,0,0,0,0) --> (0,0,1,1,0,0)
	optimize() {

		// times or values may be shared with other tracks, so overwriting is unsafe
		const times = AnimationUtils.arraySlice( this.times ),
			values = AnimationUtils.arraySlice( this.values ),
			stride = this.getValueSize(),

			smoothInterpolation = this.getInterpolation() === InterpolateSmooth,

			lastIndex = times.length - 1;

		let writeIndex = 1;

		for ( let i = 1; i < lastIndex; ++ i ) {

			let keep = false;

			const time = times[ i ];
			const timeNext = times[ i + 1 ];

			// remove adjacent keyframes scheduled at the same time

			if ( time !== timeNext && ( i !== 1 || time !== times[ 0 ] ) ) {

				if ( ! smoothInterpolation ) {

					// remove unnecessary keyframes same as their neighbors

					const offset = i * stride,
						offsetP = offset - stride,
						offsetN = offset + stride;

					for ( let j = 0; j !== stride; ++ j ) {

						const value = values[ offset + j ];

						if ( value !== values[ offsetP + j ] ||
							value !== values[ offsetN + j ] ) {

							keep = true;
							break;

						}

					}

				} else {

					keep = true;

				}

			}

			// in-place compaction

			if ( keep ) {

				if ( i !== writeIndex ) {

					times[ writeIndex ] = times[ i ];

					const readOffset = i * stride,
						writeOffset = writeIndex * stride;

					for ( let j = 0; j !== stride; ++ j ) {

						values[ writeOffset + j ] = values[ readOffset + j ];

					}

				}

				++ writeIndex;

			}

		}

		// flush last keyframe (compaction looks ahead)

		if ( lastIndex > 0 ) {

			times[ writeIndex ] = times[ lastIndex ];

			for ( let readOffset = lastIndex * stride, writeOffset = writeIndex * stride, j = 0; j !== stride; ++ j ) {

				values[ writeOffset + j ] = values[ readOffset + j ];

			}

			++ writeIndex;

		}

		if ( writeIndex !== times.length ) {

			this.times = AnimationUtils.arraySlice( times, 0, writeIndex );
			this.values = AnimationUtils.arraySlice( values, 0, writeIndex * stride );

		} else {

			this.times = times;
			this.values = values;

		}

		return this;

	}

	clone() {

		const times = AnimationUtils.arraySlice( this.times, 0 );
		const values = AnimationUtils.arraySlice( this.values, 0 );

		const TypedKeyframeTrack = this.constructor;
		const track = new TypedKeyframeTrack( this.name, times, values );

		// Interpolant argument to constructor is not saved, so copy the factory method directly.
		track.createInterpolant = this.createInterpolant;

		return track;

	}

}

KeyframeTrack.prototype.TimeBufferType = Float32Array;
KeyframeTrack.prototype.ValueBufferType = Float32Array;
KeyframeTrack.prototype.DefaultInterpolation = InterpolateLinear;

/**
 * A Track of Boolean keyframe values.
 */
class BooleanKeyframeTrack extends KeyframeTrack {}

BooleanKeyframeTrack.prototype.ValueTypeName = 'bool';
BooleanKeyframeTrack.prototype.ValueBufferType = Array;
BooleanKeyframeTrack.prototype.DefaultInterpolation = InterpolateDiscrete;
BooleanKeyframeTrack.prototype.InterpolantFactoryMethodLinear = undefined;
BooleanKeyframeTrack.prototype.InterpolantFactoryMethodSmooth = undefined;

/**
 * A Track of keyframe values that represent color.
 */
class ColorKeyframeTrack extends KeyframeTrack {}

ColorKeyframeTrack.prototype.ValueTypeName = 'color';

/**
 * A Track of numeric keyframe values.
 */
class NumberKeyframeTrack extends KeyframeTrack {}

NumberKeyframeTrack.prototype.ValueTypeName = 'number';

/**
 * Spherical linear unit quaternion interpolant.
 */

function QuaternionLinearInterpolant( parameterPositions, sampleValues, sampleSize, resultBuffer ) {

	Interpolant.call( this, parameterPositions, sampleValues, sampleSize, resultBuffer );

}

QuaternionLinearInterpolant.prototype = Object.assign( Object.create( Interpolant.prototype ), {

	constructor: QuaternionLinearInterpolant,

	interpolate_: function ( i1, t0, t, t1 ) {

		const result = this.resultBuffer,
			values = this.sampleValues,
			stride = this.valueSize,

			alpha = ( t - t0 ) / ( t1 - t0 );

		let offset = i1 * stride;

		for ( let end = offset + stride; offset !== end; offset += 4 ) {

			Quaternion.slerpFlat( result, 0, values, offset - stride, values, offset, alpha );

		}

		return result;

	}

} );

/**
 * A Track of quaternion keyframe values.
 */
class QuaternionKeyframeTrack extends KeyframeTrack {

	InterpolantFactoryMethodLinear( result ) {

		return new QuaternionLinearInterpolant( this.times, this.values, this.getValueSize(), result );

	}

}

QuaternionKeyframeTrack.prototype.ValueTypeName = 'quaternion';
// ValueBufferType is inherited
QuaternionKeyframeTrack.prototype.DefaultInterpolation = InterpolateLinear;
QuaternionKeyframeTrack.prototype.InterpolantFactoryMethodSmooth = undefined;

/**
 * A Track that interpolates Strings
 */
class StringKeyframeTrack extends KeyframeTrack {}

StringKeyframeTrack.prototype.ValueTypeName = 'string';
StringKeyframeTrack.prototype.ValueBufferType = Array;
StringKeyframeTrack.prototype.DefaultInterpolation = InterpolateDiscrete;
StringKeyframeTrack.prototype.InterpolantFactoryMethodLinear = undefined;
StringKeyframeTrack.prototype.InterpolantFactoryMethodSmooth = undefined;

/**
 * A Track of vectored keyframe values.
 */
class VectorKeyframeTrack extends KeyframeTrack {}

VectorKeyframeTrack.prototype.ValueTypeName = 'vector';

class AnimationClip {

	constructor( name, duration = - 1, tracks, blendMode = NormalAnimationBlendMode ) {

		this.name = name;
		this.tracks = tracks;
		this.duration = duration;
		this.blendMode = blendMode;

		this.uuid = MathUtils.generateUUID();

		// this means it should figure out its duration by scanning the tracks
		if ( this.duration < 0 ) {

			this.resetDuration();

		}

	}


	static parse( json ) {

		const tracks = [],
			jsonTracks = json.tracks,
			frameTime = 1.0 / ( json.fps || 1.0 );

		for ( let i = 0, n = jsonTracks.length; i !== n; ++ i ) {

			tracks.push( parseKeyframeTrack( jsonTracks[ i ] ).scale( frameTime ) );

		}

		const clip = new this( json.name, json.duration, tracks, json.blendMode );
		clip.uuid = json.uuid;

		return clip;

	}

	static toJSON( clip ) {

		const tracks = [],
			clipTracks = clip.tracks;

		const json = {

			'name': clip.name,
			'duration': clip.duration,
			'tracks': tracks,
			'uuid': clip.uuid,
			'blendMode': clip.blendMode

		};

		for ( let i = 0, n = clipTracks.length; i !== n; ++ i ) {

			tracks.push( KeyframeTrack.toJSON( clipTracks[ i ] ) );

		}

		return json;

	}

	static CreateFromMorphTargetSequence( name, morphTargetSequence, fps, noLoop ) {

		const numMorphTargets = morphTargetSequence.length;
		const tracks = [];

		for ( let i = 0; i < numMorphTargets; i ++ ) {

			let times = [];
			let values = [];

			times.push(
				( i + numMorphTargets - 1 ) % numMorphTargets,
				i,
				( i + 1 ) % numMorphTargets );

			values.push( 0, 1, 0 );

			const order = AnimationUtils.getKeyframeOrder( times );
			times = AnimationUtils.sortedArray( times, 1, order );
			values = AnimationUtils.sortedArray( values, 1, order );

			// if there is a key at the first frame, duplicate it as the
			// last frame as well for perfect loop.
			if ( ! noLoop && times[ 0 ] === 0 ) {

				times.push( numMorphTargets );
				values.push( values[ 0 ] );

			}

			tracks.push(
				new NumberKeyframeTrack(
					'.morphTargetInfluences[' + morphTargetSequence[ i ].name + ']',
					times, values
				).scale( 1.0 / fps ) );

		}

		return new this( name, - 1, tracks );

	}

	static findByName( objectOrClipArray, name ) {

		let clipArray = objectOrClipArray;

		if ( ! Array.isArray( objectOrClipArray ) ) {

			const o = objectOrClipArray;
			clipArray = o.geometry && o.geometry.animations || o.animations;

		}

		for ( let i = 0; i < clipArray.length; i ++ ) {

			if ( clipArray[ i ].name === name ) {

				return clipArray[ i ];

			}

		}

		return null;

	}

	static CreateClipsFromMorphTargetSequences( morphTargets, fps, noLoop ) {

		const animationToMorphTargets = {};

		// tested with https://regex101.com/ on trick sequences
		// such flamingo_flyA_003, flamingo_run1_003, crdeath0059
		const pattern = /^([\w-]*?)([\d]+)$/;

		// sort morph target names into animation groups based
		// patterns like Walk_001, Walk_002, Run_001, Run_002
		for ( let i = 0, il = morphTargets.length; i < il; i ++ ) {

			const morphTarget = morphTargets[ i ];
			const parts = morphTarget.name.match( pattern );

			if ( parts && parts.length > 1 ) {

				const name = parts[ 1 ];

				let animationMorphTargets = animationToMorphTargets[ name ];

				if ( ! animationMorphTargets ) {

					animationToMorphTargets[ name ] = animationMorphTargets = [];

				}

				animationMorphTargets.push( morphTarget );

			}

		}

		const clips = [];

		for ( const name in animationToMorphTargets ) {

			clips.push( this.CreateFromMorphTargetSequence( name, animationToMorphTargets[ name ], fps, noLoop ) );

		}

		return clips;

	}

	// parse the animation.hierarchy format
	static parseAnimation( animation, bones ) {

		if ( ! animation ) {

			console.error( 'THREE.AnimationClip: No animation in JSONLoader data.' );
			return null;

		}

		const addNonemptyTrack = function ( trackType, trackName, animationKeys, propertyName, destTracks ) {

			// only return track if there are actually keys.
			if ( animationKeys.length !== 0 ) {

				const times = [];
				const values = [];

				AnimationUtils.flattenJSON( animationKeys, times, values, propertyName );

				// empty keys are filtered out, so check again
				if ( times.length !== 0 ) {

					destTracks.push( new trackType( trackName, times, values ) );

				}

			}

		};

		const tracks = [];

		const clipName = animation.name || 'default';
		const fps = animation.fps || 30;
		const blendMode = animation.blendMode;

		// automatic length determination in AnimationClip.
		let duration = animation.length || - 1;

		const hierarchyTracks = animation.hierarchy || [];

		for ( let h = 0; h < hierarchyTracks.length; h ++ ) {

			const animationKeys = hierarchyTracks[ h ].keys;

			// skip empty tracks
			if ( ! animationKeys || animationKeys.length === 0 ) continue;

			// process morph targets
			if ( animationKeys[ 0 ].morphTargets ) {

				// figure out all morph targets used in this track
				const morphTargetNames = {};

				let k;

				for ( k = 0; k < animationKeys.length; k ++ ) {

					if ( animationKeys[ k ].morphTargets ) {

						for ( let m = 0; m < animationKeys[ k ].morphTargets.length; m ++ ) {

							morphTargetNames[ animationKeys[ k ].morphTargets[ m ] ] = - 1;

						}

					}

				}

				// create a track for each morph target with all zero
				// morphTargetInfluences except for the keys in which
				// the morphTarget is named.
				for ( const morphTargetName in morphTargetNames ) {

					const times = [];
					const values = [];

					for ( let m = 0; m !== animationKeys[ k ].morphTargets.length; ++ m ) {

						const animationKey = animationKeys[ k ];

						times.push( animationKey.time );
						values.push( ( animationKey.morphTarget === morphTargetName ) ? 1 : 0 );

					}

					tracks.push( new NumberKeyframeTrack( '.morphTargetInfluence[' + morphTargetName + ']', times, values ) );

				}

				duration = morphTargetNames.length * ( fps || 1.0 );

			} else {

				// ...assume skeletal animation

				const boneName = '.bones[' + bones[ h ].name + ']';

				addNonemptyTrack(
					VectorKeyframeTrack, boneName + '.position',
					animationKeys, 'pos', tracks );

				addNonemptyTrack(
					QuaternionKeyframeTrack, boneName + '.quaternion',
					animationKeys, 'rot', tracks );

				addNonemptyTrack(
					VectorKeyframeTrack, boneName + '.scale',
					animationKeys, 'scl', tracks );

			}

		}

		if ( tracks.length === 0 ) {

			return null;

		}

		const clip = new this( clipName, duration, tracks, blendMode );

		return clip;

	}

	resetDuration() {

		const tracks = this.tracks;
		let duration = 0;

		for ( let i = 0, n = tracks.length; i !== n; ++ i ) {

			const track = this.tracks[ i ];

			duration = Math.max( duration, track.times[ track.times.length - 1 ] );

		}

		this.duration = duration;

		return this;

	}

	trim() {

		for ( let i = 0; i < this.tracks.length; i ++ ) {

			this.tracks[ i ].trim( 0, this.duration );

		}

		return this;

	}

	validate() {

		let valid = true;

		for ( let i = 0; i < this.tracks.length; i ++ ) {

			valid = valid && this.tracks[ i ].validate();

		}

		return valid;

	}

	optimize() {

		for ( let i = 0; i < this.tracks.length; i ++ ) {

			this.tracks[ i ].optimize();

		}

		return this;

	}

	clone() {

		const tracks = [];

		for ( let i = 0; i < this.tracks.length; i ++ ) {

			tracks.push( this.tracks[ i ].clone() );

		}

		return new this.constructor( this.name, this.duration, tracks, this.blendMode );

	}

	toJSON() {

		return this.constructor.toJSON( this );

	}

}

function getTrackTypeForValueTypeName( typeName ) {

	switch ( typeName.toLowerCase() ) {

		case 'scalar':
		case 'double':
		case 'float':
		case 'number':
		case 'integer':

			return NumberKeyframeTrack;

		case 'vector':
		case 'vector2':
		case 'vector3':
		case 'vector4':

			return VectorKeyframeTrack;

		case 'color':

			return ColorKeyframeTrack;

		case 'quaternion':

			return QuaternionKeyframeTrack;

		case 'bool':
		case 'boolean':

			return BooleanKeyframeTrack;

		case 'string':

			return StringKeyframeTrack;

	}

	throw new Error( 'THREE.KeyframeTrack: Unsupported typeName: ' + typeName );

}

function parseKeyframeTrack( json ) {

	if ( json.type === undefined ) {

		throw new Error( 'THREE.KeyframeTrack: track type undefined, can not parse' );

	}

	const trackType = getTrackTypeForValueTypeName( json.type );

	if ( json.times === undefined ) {

		const times = [], values = [];

		AnimationUtils.flattenJSON( json.keys, times, values, 'value' );

		json.times = times;
		json.values = values;

	}

	// derived classes can define a static parse method
	if ( trackType.parse !== undefined ) {

		return trackType.parse( json );

	} else {

		// by default, we assume a constructor compatible with the base
		return new trackType( json.name, json.times, json.values, json.interpolation );

	}

}

const Cache = {

	enabled: false,

	files: {},

	add: function ( key, file ) {

		if ( this.enabled === false ) return;

		// console.log( 'THREE.Cache', 'Adding key:', key );

		this.files[ key ] = file;

	},

	get: function ( key ) {

		if ( this.enabled === false ) return;

		// console.log( 'THREE.Cache', 'Checking key:', key );

		return this.files[ key ];

	},

	remove: function ( key ) {

		delete this.files[ key ];

	},

	clear: function () {

		this.files = {};

	}

};

function LoadingManager( onLoad, onProgress, onError ) {

	const scope = this;

	let isLoading = false;
	let itemsLoaded = 0;
	let itemsTotal = 0;
	let urlModifier = undefined;
	const handlers = [];

	// Refer to #5689 for the reason why we don't set .onStart
	// in the constructor

	this.onStart = undefined;
	this.onLoad = onLoad;
	this.onProgress = onProgress;
	this.onError = onError;

	this.itemStart = function ( url ) {

		itemsTotal ++;

		if ( isLoading === false ) {

			if ( scope.onStart !== undefined ) {

				scope.onStart( url, itemsLoaded, itemsTotal );

			}

		}

		isLoading = true;

	};

	this.itemEnd = function ( url ) {

		itemsLoaded ++;

		if ( scope.onProgress !== undefined ) {

			scope.onProgress( url, itemsLoaded, itemsTotal );

		}

		if ( itemsLoaded === itemsTotal ) {

			isLoading = false;

			if ( scope.onLoad !== undefined ) {

				scope.onLoad();

			}

		}

	};

	this.itemError = function ( url ) {

		if ( scope.onError !== undefined ) {

			scope.onError( url );

		}

	};

	this.resolveURL = function ( url ) {

		if ( urlModifier ) {

			return urlModifier( url );

		}

		return url;

	};

	this.setURLModifier = function ( transform ) {

		urlModifier = transform;

		return this;

	};

	this.addHandler = function ( regex, loader ) {

		handlers.push( regex, loader );

		return this;

	};

	this.removeHandler = function ( regex ) {

		const index = handlers.indexOf( regex );

		if ( index !== - 1 ) {

			handlers.splice( index, 2 );

		}

		return this;

	};

	this.getHandler = function ( file ) {

		for ( let i = 0, l = handlers.length; i < l; i += 2 ) {

			const regex = handlers[ i ];
			const loader = handlers[ i + 1 ];

			if ( regex.global ) regex.lastIndex = 0; // see #17920

			if ( regex.test( file ) ) {

				return loader;

			}

		}

		return null;

	};

}

const DefaultLoadingManager = new LoadingManager();

function Loader( manager ) {

	this.manager = ( manager !== undefined ) ? manager : DefaultLoadingManager;

	this.crossOrigin = 'anonymous';
	this.withCredentials = false;
	this.path = '';
	this.resourcePath = '';
	this.requestHeader = {};

}

Object.assign( Loader.prototype, {

	load: function ( /* url, onLoad, onProgress, onError */ ) {},

	loadAsync: function ( url, onProgress ) {

		const scope = this;

		return new Promise( function ( resolve, reject ) {

			scope.load( url, resolve, onProgress, reject );

		} );

	},

	parse: function ( /* data */ ) {},

	setCrossOrigin: function ( crossOrigin ) {

		this.crossOrigin = crossOrigin;
		return this;

	},

	setWithCredentials: function ( value ) {

		this.withCredentials = value;
		return this;

	},

	setPath: function ( path ) {

		this.path = path;
		return this;

	},

	setResourcePath: function ( resourcePath ) {

		this.resourcePath = resourcePath;
		return this;

	},

	setRequestHeader: function ( requestHeader ) {

		this.requestHeader = requestHeader;
		return this;

	}

} );

const loading = {};

function FileLoader( manager ) {

	Loader.call( this, manager );

}

FileLoader.prototype = Object.assign( Object.create( Loader.prototype ), {

	constructor: FileLoader,

	load: function ( url, onLoad, onProgress, onError ) {

		if ( url === undefined ) url = '';

		if ( this.path !== undefined ) url = this.path + url;

		url = this.manager.resolveURL( url );

		const scope = this;

		const cached = Cache.get( url );

		if ( cached !== undefined ) {

			scope.manager.itemStart( url );

			setTimeout( function () {

				if ( onLoad ) onLoad( cached );

				scope.manager.itemEnd( url );

			}, 0 );

			return cached;

		}

		// Check if request is duplicate

		if ( loading[ url ] !== undefined ) {

			loading[ url ].push( {

				onLoad: onLoad,
				onProgress: onProgress,
				onError: onError

			} );

			return;

		}

		// Check for data: URI
		const dataUriRegex = /^data:(.*?)(;base64)?,(.*)$/;
		const dataUriRegexResult = url.match( dataUriRegex );
		let request;

		// Safari can not handle Data URIs through XMLHttpRequest so process manually
		if ( dataUriRegexResult ) {

			const mimeType = dataUriRegexResult[ 1 ];
			const isBase64 = !! dataUriRegexResult[ 2 ];

			let data = dataUriRegexResult[ 3 ];
			data = decodeURIComponent( data );

			if ( isBase64 ) data = atob( data );

			try {

				let response;
				const responseType = ( this.responseType || '' ).toLowerCase();

				switch ( responseType ) {

					case 'arraybuffer':
					case 'blob':

						const view = new Uint8Array( data.length );

						for ( let i = 0; i < data.length; i ++ ) {

							view[ i ] = data.charCodeAt( i );

						}

						if ( responseType === 'blob' ) {

							response = new Blob( [ view.buffer ], { type: mimeType } );

						} else {

							response = view.buffer;

						}

						break;

					case 'document':

						const parser = new DOMParser();
						response = parser.parseFromString( data, mimeType );

						break;

					case 'json':

						response = JSON.parse( data );

						break;

					default: // 'text' or other

						response = data;

						break;

				}

				// Wait for next browser tick like standard XMLHttpRequest event dispatching does
				setTimeout( function () {

					if ( onLoad ) onLoad( response );

					scope.manager.itemEnd( url );

				}, 0 );

			} catch ( error ) {

				// Wait for next browser tick like standard XMLHttpRequest event dispatching does
				setTimeout( function () {

					if ( onError ) onError( error );

					scope.manager.itemError( url );
					scope.manager.itemEnd( url );

				}, 0 );

			}

		} else {

			// Initialise array for duplicate requests

			loading[ url ] = [];

			loading[ url ].push( {

				onLoad: onLoad,
				onProgress: onProgress,
				onError: onError

			} );

			request = new XMLHttpRequest();

			request.open( 'GET', url, true );

			request.addEventListener( 'load', function ( event ) {

				const response = this.response;

				const callbacks = loading[ url ];

				delete loading[ url ];

				if ( this.status === 200 || this.status === 0 ) {

					// Some browsers return HTTP Status 0 when using non-http protocol
					// e.g. 'file://' or 'data://'. Handle as success.

					if ( this.status === 0 ) console.warn( 'THREE.FileLoader: HTTP Status 0 received.' );

					// Add to cache only on HTTP success, so that we do not cache
					// error response bodies as proper responses to requests.
					Cache.add( url, response );

					for ( let i = 0, il = callbacks.length; i < il; i ++ ) {

						const callback = callbacks[ i ];
						if ( callback.onLoad ) callback.onLoad( response );

					}

					scope.manager.itemEnd( url );

				} else {

					for ( let i = 0, il = callbacks.length; i < il; i ++ ) {

						const callback = callbacks[ i ];
						if ( callback.onError ) callback.onError( event );

					}

					scope.manager.itemError( url );
					scope.manager.itemEnd( url );

				}

			}, false );

			request.addEventListener( 'progress', function ( event ) {

				const callbacks = loading[ url ];

				for ( let i = 0, il = callbacks.length; i < il; i ++ ) {

					const callback = callbacks[ i ];
					if ( callback.onProgress ) callback.onProgress( event );

				}

			}, false );

			request.addEventListener( 'error', function ( event ) {

				const callbacks = loading[ url ];

				delete loading[ url ];

				for ( let i = 0, il = callbacks.length; i < il; i ++ ) {

					const callback = callbacks[ i ];
					if ( callback.onError ) callback.onError( event );

				}

				scope.manager.itemError( url );
				scope.manager.itemEnd( url );

			}, false );

			request.addEventListener( 'abort', function ( event ) {

				const callbacks = loading[ url ];

				delete loading[ url ];

				for ( let i = 0, il = callbacks.length; i < il; i ++ ) {

					const callback = callbacks[ i ];
					if ( callback.onError ) callback.onError( event );

				}

				scope.manager.itemError( url );
				scope.manager.itemEnd( url );

			}, false );

			if ( this.responseType !== undefined ) request.responseType = this.responseType;
			if ( this.withCredentials !== undefined ) request.withCredentials = this.withCredentials;

			if ( request.overrideMimeType ) request.overrideMimeType( this.mimeType !== undefined ? this.mimeType : 'text/plain' );

			for ( const header in this.requestHeader ) {

				request.setRequestHeader( header, this.requestHeader[ header ] );

			}

			request.send( null );

		}

		scope.manager.itemStart( url );

		return request;

	},

	setResponseType: function ( value ) {

		this.responseType = value;
		return this;

	},

	setMimeType: function ( value ) {

		this.mimeType = value;
		return this;

	}

} );

/**
 * Abstract Base class to block based textures loader (dds, pvr, ...)
 *
 * Sub classes have to implement the parse() method which will be used in load().
 */

function CompressedTextureLoader( manager ) {

	Loader.call( this, manager );

}

CompressedTextureLoader.prototype = Object.assign( Object.create( Loader.prototype ), {

	constructor: CompressedTextureLoader,

	load: function ( url, onLoad, onProgress, onError ) {

		const scope = this;

		const images = [];

		const texture = new CompressedTexture();

		const loader = new FileLoader( this.manager );
		loader.setPath( this.path );
		loader.setResponseType( 'arraybuffer' );
		loader.setRequestHeader( this.requestHeader );
		loader.setWithCredentials( scope.withCredentials );

		let loaded = 0;

		function loadTexture( i ) {

			loader.load( url[ i ], function ( buffer ) {

				const texDatas = scope.parse( buffer, true );

				images[ i ] = {
					width: texDatas.width,
					height: texDatas.height,
					format: texDatas.format,
					mipmaps: texDatas.mipmaps
				};

				loaded += 1;

				if ( loaded === 6 ) {

					if ( texDatas.mipmapCount === 1 ) texture.minFilter = LinearFilter;

					texture.image = images;
					texture.format = texDatas.format;
					texture.needsUpdate = true;

					if ( onLoad ) onLoad( texture );

				}

			}, onProgress, onError );

		}

		if ( Array.isArray( url ) ) {

			for ( let i = 0, il = url.length; i < il; ++ i ) {

				loadTexture( i );

			}

		} else {

			// compressed cubemap texture stored in a single DDS file

			loader.load( url, function ( buffer ) {

				const texDatas = scope.parse( buffer, true );

				if ( texDatas.isCubemap ) {

					const faces = texDatas.mipmaps.length / texDatas.mipmapCount;

					for ( let f = 0; f < faces; f ++ ) {

						images[ f ] = { mipmaps: [] };

						for ( let i = 0; i < texDatas.mipmapCount; i ++ ) {

							images[ f ].mipmaps.push( texDatas.mipmaps[ f * texDatas.mipmapCount + i ] );
							images[ f ].format = texDatas.format;
							images[ f ].width = texDatas.width;
							images[ f ].height = texDatas.height;

						}

					}

					texture.image = images;

				} else {

					texture.image.width = texDatas.width;
					texture.image.height = texDatas.height;
					texture.mipmaps = texDatas.mipmaps;

				}

				if ( texDatas.mipmapCount === 1 ) {

					texture.minFilter = LinearFilter;

				}

				texture.format = texDatas.format;
				texture.needsUpdate = true;

				if ( onLoad ) onLoad( texture );

			}, onProgress, onError );

		}

		return texture;

	}

} );

class ImageLoader extends Loader {

	constructor( manager ) {

		super( manager );

	}

	load( url, onLoad, onProgress, onError ) {

		if ( this.path !== undefined ) url = this.path + url;

		url = this.manager.resolveURL( url );

		const scope = this;

		const cached = Cache.get( url );

		if ( cached !== undefined ) {

			scope.manager.itemStart( url );

			setTimeout( function () {

				if ( onLoad ) onLoad( cached );

				scope.manager.itemEnd( url );

			}, 0 );

			return cached;

		}

		const image = document.createElementNS( 'http://www.w3.org/1999/xhtml', 'img' );

		function onImageLoad() {

			image.removeEventListener( 'load', onImageLoad, false );
			image.removeEventListener( 'error', onImageError, false );

			Cache.add( url, this );

			if ( onLoad ) onLoad( this );

			scope.manager.itemEnd( url );

		}

		function onImageError( event ) {

			image.removeEventListener( 'load', onImageLoad, false );
			image.removeEventListener( 'error', onImageError, false );

			if ( onError ) onError( event );

			scope.manager.itemError( url );
			scope.manager.itemEnd( url );

		}

		image.addEventListener( 'load', onImageLoad, false );
		image.addEventListener( 'error', onImageError, false );

		if ( url.substr( 0, 5 ) !== 'data:' ) {

			if ( this.crossOrigin !== undefined ) image.crossOrigin = this.crossOrigin;

		}

		scope.manager.itemStart( url );

		image.src = url;

		return image;

	}

}

class CubeTextureLoader extends Loader {

	constructor( manager ) {

		super( manager );

	}

	load( urls, onLoad, onProgress, onError ) {

		const texture = new CubeTexture();

		const loader = new ImageLoader( this.manager );
		loader.setCrossOrigin( this.crossOrigin );
		loader.setPath( this.path );

		let loaded = 0;

		function loadTexture( i ) {

			loader.load( urls[ i ], function ( image ) {

				texture.images[ i ] = image;

				loaded ++;

				if ( loaded === 6 ) {

					texture.needsUpdate = true;

					if ( onLoad ) onLoad( texture );

				}

			}, undefined, onError );

		}

		for ( let i = 0; i < urls.length; ++ i ) {

			loadTexture( i );

		}

		return texture;

	}

}

/**
 * Abstract Base class to load generic binary textures formats (rgbe, hdr, ...)
 *
 * Sub classes have to implement the parse() method which will be used in load().
 */

function DataTextureLoader( manager ) {

	Loader.call( this, manager );

}

DataTextureLoader.prototype = Object.assign( Object.create( Loader.prototype ), {

	constructor: DataTextureLoader,

	load: function ( url, onLoad, onProgress, onError ) {

		const scope = this;

		const texture = new DataTexture();

		const loader = new FileLoader( this.manager );
		loader.setResponseType( 'arraybuffer' );
		loader.setRequestHeader( this.requestHeader );
		loader.setPath( this.path );
		loader.setWithCredentials( scope.withCredentials );
		loader.load( url, function ( buffer ) {

			const texData = scope.parse( buffer );

			if ( ! texData ) return;

			if ( texData.image !== undefined ) {

				texture.image = texData.image;

			} else if ( texData.data !== undefined ) {

				texture.image.width = texData.width;
				texture.image.height = texData.height;
				texture.image.data = texData.data;

			}

			texture.wrapS = texData.wrapS !== undefined ? texData.wrapS : ClampToEdgeWrapping;
			texture.wrapT = texData.wrapT !== undefined ? texData.wrapT : ClampToEdgeWrapping;

			texture.magFilter = texData.magFilter !== undefined ? texData.magFilter : LinearFilter;
			texture.minFilter = texData.minFilter !== undefined ? texData.minFilter : LinearFilter;

			texture.anisotropy = texData.anisotropy !== undefined ? texData.anisotropy : 1;

			if ( texData.encoding !== undefined ) {

				texture.encoding = texData.encoding;

			}

			if ( texData.flipY !== undefined ) {

				texture.flipY = texData.flipY;

			}

			if ( texData.format !== undefined ) {

				texture.format = texData.format;

			}

			if ( texData.type !== undefined ) {

				texture.type = texData.type;

			}

			if ( texData.mipmaps !== undefined ) {

				texture.mipmaps = texData.mipmaps;
				texture.minFilter = LinearMipmapLinearFilter; // presumably...

			}

			if ( texData.mipmapCount === 1 ) {

				texture.minFilter = LinearFilter;

			}

			if ( texData.generateMipmaps !== undefined ) {

				texture.generateMipmaps = texData.generateMipmaps;

			}

			texture.needsUpdate = true;

			if ( onLoad ) onLoad( texture, texData );

		}, onProgress, onError );


		return texture;

	}

} );

function TextureLoader( manager ) {

	Loader.call( this, manager );

}

TextureLoader.prototype = Object.assign( Object.create( Loader.prototype ), {

	constructor: TextureLoader,

	load: function ( url, onLoad, onProgress, onError ) {

		const texture = new Texture();

		const loader = new ImageLoader( this.manager );
		loader.setCrossOrigin( this.crossOrigin );
		loader.setPath( this.path );

		loader.load( url, function ( image ) {

			texture.image = image;

			// JPEGs can't have an alpha channel, so memory can be saved by storing them as RGB.
			const isJPEG = url.search( /\.jpe?g($|\?)/i ) > 0 || url.search( /^data\:image\/jpeg/ ) === 0;

			texture.format = isJPEG ? RGBFormat : RGBAFormat;
			texture.needsUpdate = true;

			if ( onLoad !== undefined ) {

				onLoad( texture );

			}

		}, onProgress, onError );

		return texture;

	}

} );

/**
 * Extensible curve object.
 *
 * Some common of curve methods:
 * .getPoint( t, optionalTarget ), .getTangent( t, optionalTarget )
 * .getPointAt( u, optionalTarget ), .getTangentAt( u, optionalTarget )
 * .getPoints(), .getSpacedPoints()
 * .getLength()
 * .updateArcLengths()
 *
 * This following curves inherit from THREE.Curve:
 *
 * -- 2D curves --
 * THREE.ArcCurve
 * THREE.CubicBezierCurve
 * THREE.EllipseCurve
 * THREE.LineCurve
 * THREE.QuadraticBezierCurve
 * THREE.SplineCurve
 *
 * -- 3D curves --
 * THREE.CatmullRomCurve3
 * THREE.CubicBezierCurve3
 * THREE.LineCurve3
 * THREE.QuadraticBezierCurve3
 *
 * A series of curves can be represented as a THREE.CurvePath.
 *
 **/

function Curve() {

	this.type = 'Curve';

	this.arcLengthDivisions = 200;

}

Object.assign( Curve.prototype, {

	// Virtual base class method to overwrite and implement in subclasses
	//	- t [0 .. 1]

	getPoint: function ( /* t, optionalTarget */ ) {

		console.warn( 'THREE.Curve: .getPoint() not implemented.' );
		return null;

	},

	// Get point at relative position in curve according to arc length
	// - u [0 .. 1]

	getPointAt: function ( u, optionalTarget ) {

		const t = this.getUtoTmapping( u );
		return this.getPoint( t, optionalTarget );

	},

	// Get sequence of points using getPoint( t )

	getPoints: function ( divisions = 5 ) {

		const points = [];

		for ( let d = 0; d <= divisions; d ++ ) {

			points.push( this.getPoint( d / divisions ) );

		}

		return points;

	},

	// Get sequence of points using getPointAt( u )

	getSpacedPoints: function ( divisions = 5 ) {

		const points = [];

		for ( let d = 0; d <= divisions; d ++ ) {

			points.push( this.getPointAt( d / divisions ) );

		}

		return points;

	},

	// Get total curve arc length

	getLength: function () {

		const lengths = this.getLengths();
		return lengths[ lengths.length - 1 ];

	},

	// Get list of cumulative segment lengths

	getLengths: function ( divisions ) {

		if ( divisions === undefined ) divisions = this.arcLengthDivisions;

		if ( this.cacheArcLengths &&
			( this.cacheArcLengths.length === divisions + 1 ) &&
			! this.needsUpdate ) {

			return this.cacheArcLengths;

		}

		this.needsUpdate = false;

		const cache = [];
		let current, last = this.getPoint( 0 );
		let sum = 0;

		cache.push( 0 );

		for ( let p = 1; p <= divisions; p ++ ) {

			current = this.getPoint( p / divisions );
			sum += current.distanceTo( last );
			cache.push( sum );
			last = current;

		}

		this.cacheArcLengths = cache;

		return cache; // { sums: cache, sum: sum }; Sum is in the last element.

	},

	updateArcLengths: function () {

		this.needsUpdate = true;
		this.getLengths();

	},

	// Given u ( 0 .. 1 ), get a t to find p. This gives you points which are equidistant

	getUtoTmapping: function ( u, distance ) {

		const arcLengths = this.getLengths();

		let i = 0;
		const il = arcLengths.length;

		let targetArcLength; // The targeted u distance value to get

		if ( distance ) {

			targetArcLength = distance;

		} else {

			targetArcLength = u * arcLengths[ il - 1 ];

		}

		// binary search for the index with largest value smaller than target u distance

		let low = 0, high = il - 1, comparison;

		while ( low <= high ) {

			i = Math.floor( low + ( high - low ) / 2 ); // less likely to overflow, though probably not issue here, JS doesn't really have integers, all numbers are floats

			comparison = arcLengths[ i ] - targetArcLength;

			if ( comparison < 0 ) {

				low = i + 1;

			} else if ( comparison > 0 ) {

				high = i - 1;

			} else {

				high = i;
				break;

				// DONE

			}

		}

		i = high;

		if ( arcLengths[ i ] === targetArcLength ) {

			return i / ( il - 1 );

		}

		// we could get finer grain at lengths, or use simple interpolation between two points

		const lengthBefore = arcLengths[ i ];
		const lengthAfter = arcLengths[ i + 1 ];

		const segmentLength = lengthAfter - lengthBefore;

		// determine where we are between the 'before' and 'after' points

		const segmentFraction = ( targetArcLength - lengthBefore ) / segmentLength;

		// add that fractional amount to t

		const t = ( i + segmentFraction ) / ( il - 1 );

		return t;

	},

	// Returns a unit vector tangent at t
	// In case any sub curve does not implement its tangent derivation,
	// 2 points a small delta apart will be used to find its gradient
	// which seems to give a reasonable approximation

	getTangent: function ( t, optionalTarget ) {

		const delta = 0.0001;
		let t1 = t - delta;
		let t2 = t + delta;

		// Capping in case of danger

		if ( t1 < 0 ) t1 = 0;
		if ( t2 > 1 ) t2 = 1;

		const pt1 = this.getPoint( t1 );
		const pt2 = this.getPoint( t2 );

		const tangent = optionalTarget || ( ( pt1.isVector2 ) ? new Vector2() : new Vector3() );

		tangent.copy( pt2 ).sub( pt1 ).normalize();

		return tangent;

	},

	getTangentAt: function ( u, optionalTarget ) {

		const t = this.getUtoTmapping( u );
		return this.getTangent( t, optionalTarget );

	},

	computeFrenetFrames: function ( segments, closed ) {

		// see http://www.cs.indiana.edu/pub/techreports/TR425.pdf

		const normal = new Vector3();

		const tangents = [];
		const normals = [];
		const binormals = [];

		const vec = new Vector3();
		const mat = new Matrix4();

		// compute the tangent vectors for each segment on the curve

		for ( let i = 0; i <= segments; i ++ ) {

			const u = i / segments;

			tangents[ i ] = this.getTangentAt( u, new Vector3() );
			tangents[ i ].normalize();

		}

		// select an initial normal vector perpendicular to the first tangent vector,
		// and in the direction of the minimum tangent xyz component

		normals[ 0 ] = new Vector3();
		binormals[ 0 ] = new Vector3();
		let min = Number.MAX_VALUE;
		const tx = Math.abs( tangents[ 0 ].x );
		const ty = Math.abs( tangents[ 0 ].y );
		const tz = Math.abs( tangents[ 0 ].z );

		if ( tx <= min ) {

			min = tx;
			normal.set( 1, 0, 0 );

		}

		if ( ty <= min ) {

			min = ty;
			normal.set( 0, 1, 0 );

		}

		if ( tz <= min ) {

			normal.set( 0, 0, 1 );

		}

		vec.crossVectors( tangents[ 0 ], normal ).normalize();

		normals[ 0 ].crossVectors( tangents[ 0 ], vec );
		binormals[ 0 ].crossVectors( tangents[ 0 ], normals[ 0 ] );


		// compute the slowly-varying normal and binormal vectors for each segment on the curve

		for ( let i = 1; i <= segments; i ++ ) {

			normals[ i ] = normals[ i - 1 ].clone();

			binormals[ i ] = binormals[ i - 1 ].clone();

			vec.crossVectors( tangents[ i - 1 ], tangents[ i ] );

			if ( vec.length() > Number.EPSILON ) {

				vec.normalize();

				const theta = Math.acos( MathUtils.clamp( tangents[ i - 1 ].dot( tangents[ i ] ), - 1, 1 ) ); // clamp for floating pt errors

				normals[ i ].applyMatrix4( mat.makeRotationAxis( vec, theta ) );

			}

			binormals[ i ].crossVectors( tangents[ i ], normals[ i ] );

		}

		// if the curve is closed, postprocess the vectors so the first and last normal vectors are the same

		if ( closed === true ) {

			let theta = Math.acos( MathUtils.clamp( normals[ 0 ].dot( normals[ segments ] ), - 1, 1 ) );
			theta /= segments;

			if ( tangents[ 0 ].dot( vec.crossVectors( normals[ 0 ], normals[ segments ] ) ) > 0 ) {

				theta = - theta;

			}

			for ( let i = 1; i <= segments; i ++ ) {

				// twist a little...
				normals[ i ].applyMatrix4( mat.makeRotationAxis( tangents[ i ], theta * i ) );
				binormals[ i ].crossVectors( tangents[ i ], normals[ i ] );

			}

		}

		return {
			tangents: tangents,
			normals: normals,
			binormals: binormals
		};

	},

	clone: function () {

		return new this.constructor().copy( this );

	},

	copy: function ( source ) {

		this.arcLengthDivisions = source.arcLengthDivisions;

		return this;

	},

	toJSON: function () {

		const data = {
			metadata: {
				version: 4.5,
				type: 'Curve',
				generator: 'Curve.toJSON'
			}
		};

		data.arcLengthDivisions = this.arcLengthDivisions;
		data.type = this.type;

		return data;

	},

	fromJSON: function ( json ) {

		this.arcLengthDivisions = json.arcLengthDivisions;

		return this;

	}

} );

class EllipseCurve extends Curve {

	constructor( aX = 0, aY = 0, xRadius = 1, yRadius = 1, aStartAngle = 0, aEndAngle = Math.PI * 2, aClockwise = false, aRotation = 0 ) {

		super();

		this.type = 'EllipseCurve';

		this.aX = aX;
		this.aY = aY;

		this.xRadius = xRadius;
		this.yRadius = yRadius;

		this.aStartAngle = aStartAngle;
		this.aEndAngle = aEndAngle;

		this.aClockwise = aClockwise;

		this.aRotation = aRotation;

	}

	getPoint( t, optionalTarget ) {

		const point = optionalTarget || new Vector2();

		const twoPi = Math.PI * 2;
		let deltaAngle = this.aEndAngle - this.aStartAngle;
		const samePoints = Math.abs( deltaAngle ) < Number.EPSILON;

		// ensures that deltaAngle is 0 .. 2 PI
		while ( deltaAngle < 0 ) deltaAngle += twoPi;
		while ( deltaAngle > twoPi ) deltaAngle -= twoPi;

		if ( deltaAngle < Number.EPSILON ) {

			if ( samePoints ) {

				deltaAngle = 0;

			} else {

				deltaAngle = twoPi;

			}

		}

		if ( this.aClockwise === true && ! samePoints ) {

			if ( deltaAngle === twoPi ) {

				deltaAngle = - twoPi;

			} else {

				deltaAngle = deltaAngle - twoPi;

			}

		}

		const angle = this.aStartAngle + t * deltaAngle;
		let x = this.aX + this.xRadius * Math.cos( angle );
		let y = this.aY + this.yRadius * Math.sin( angle );

		if ( this.aRotation !== 0 ) {

			const cos = Math.cos( this.aRotation );
			const sin = Math.sin( this.aRotation );

			const tx = x - this.aX;
			const ty = y - this.aY;

			// Rotate the point about the center of the ellipse.
			x = tx * cos - ty * sin + this.aX;
			y = tx * sin + ty * cos + this.aY;

		}

		return point.set( x, y );

	}

	copy( source ) {

		super.copy( source );

		this.aX = source.aX;
		this.aY = source.aY;

		this.xRadius = source.xRadius;
		this.yRadius = source.yRadius;

		this.aStartAngle = source.aStartAngle;
		this.aEndAngle = source.aEndAngle;

		this.aClockwise = source.aClockwise;

		this.aRotation = source.aRotation;

		return this;

	}

	toJSON() {

		const data = super.toJSON();

		data.aX = this.aX;
		data.aY = this.aY;

		data.xRadius = this.xRadius;
		data.yRadius = this.yRadius;

		data.aStartAngle = this.aStartAngle;
		data.aEndAngle = this.aEndAngle;

		data.aClockwise = this.aClockwise;

		data.aRotation = this.aRotation;

		return data;

	}

	fromJSON( json ) {

		super.fromJSON( json );

		this.aX = json.aX;
		this.aY = json.aY;

		this.xRadius = json.xRadius;
		this.yRadius = json.yRadius;

		this.aStartAngle = json.aStartAngle;
		this.aEndAngle = json.aEndAngle;

		this.aClockwise = json.aClockwise;

		this.aRotation = json.aRotation;

		return this;

	}

}

EllipseCurve.prototype.isEllipseCurve = true;

class ArcCurve extends EllipseCurve {

	constructor( aX, aY, aRadius, aStartAngle, aEndAngle, aClockwise ) {

		super( aX, aY, aRadius, aRadius, aStartAngle, aEndAngle, aClockwise );

		this.type = 'ArcCurve';

	}

}

ArcCurve.prototype.isArcCurve = true;

/**
 * Centripetal CatmullRom Curve - which is useful for avoiding
 * cusps and self-intersections in non-uniform catmull rom curves.
 * http://www.cemyuksel.com/research/catmullrom_param/catmullrom.pdf
 *
 * curve.type accepts centripetal(default), chordal and catmullrom
 * curve.tension is used for catmullrom which defaults to 0.5
 */


/*
Based on an optimized c++ solution in
 - http://stackoverflow.com/questions/9489736/catmull-rom-curve-with-no-cusps-and-no-self-intersections/
 - http://ideone.com/NoEbVM

This CubicPoly class could be used for reusing some variables and calculations,
but for three.js curve use, it could be possible inlined and flatten into a single function call
which can be placed in CurveUtils.
*/

function CubicPoly() {

	let c0 = 0, c1 = 0, c2 = 0, c3 = 0;

	/*
	 * Compute coefficients for a cubic polynomial
	 *   p(s) = c0 + c1*s + c2*s^2 + c3*s^3
	 * such that
	 *   p(0) = x0, p(1) = x1
	 *  and
	 *   p'(0) = t0, p'(1) = t1.
	 */
	function init( x0, x1, t0, t1 ) {

		c0 = x0;
		c1 = t0;
		c2 = - 3 * x0 + 3 * x1 - 2 * t0 - t1;
		c3 = 2 * x0 - 2 * x1 + t0 + t1;

	}

	return {

		initCatmullRom: function ( x0, x1, x2, x3, tension ) {

			init( x1, x2, tension * ( x2 - x0 ), tension * ( x3 - x1 ) );

		},

		initNonuniformCatmullRom: function ( x0, x1, x2, x3, dt0, dt1, dt2 ) {

			// compute tangents when parameterized in [t1,t2]
			let t1 = ( x1 - x0 ) / dt0 - ( x2 - x0 ) / ( dt0 + dt1 ) + ( x2 - x1 ) / dt1;
			let t2 = ( x2 - x1 ) / dt1 - ( x3 - x1 ) / ( dt1 + dt2 ) + ( x3 - x2 ) / dt2;

			// rescale tangents for parametrization in [0,1]
			t1 *= dt1;
			t2 *= dt1;

			init( x1, x2, t1, t2 );

		},

		calc: function ( t ) {

			const t2 = t * t;
			const t3 = t2 * t;
			return c0 + c1 * t + c2 * t2 + c3 * t3;

		}

	};

}

//

const tmp = new Vector3();
const px = new CubicPoly(), py = new CubicPoly(), pz = new CubicPoly();

class CatmullRomCurve3 extends Curve {

	constructor( points = [], closed = false, curveType = 'centripetal', tension = 0.5 ) {

		super();

		this.type = 'CatmullRomCurve3';

		this.points = points;
		this.closed = closed;
		this.curveType = curveType;
		this.tension = tension;

	}

	getPoint( t, optionalTarget = new Vector3() ) {

		const point = optionalTarget;

		const points = this.points;
		const l = points.length;

		const p = ( l - ( this.closed ? 0 : 1 ) ) * t;
		let intPoint = Math.floor( p );
		let weight = p - intPoint;

		if ( this.closed ) {

			intPoint += intPoint > 0 ? 0 : ( Math.floor( Math.abs( intPoint ) / l ) + 1 ) * l;

		} else if ( weight === 0 && intPoint === l - 1 ) {

			intPoint = l - 2;
			weight = 1;

		}

		let p0, p3; // 4 points (p1 & p2 defined below)

		if ( this.closed || intPoint > 0 ) {

			p0 = points[ ( intPoint - 1 ) % l ];

		} else {

			// extrapolate first point
			tmp.subVectors( points[ 0 ], points[ 1 ] ).add( points[ 0 ] );
			p0 = tmp;

		}

		const p1 = points[ intPoint % l ];
		const p2 = points[ ( intPoint + 1 ) % l ];

		if ( this.closed || intPoint + 2 < l ) {

			p3 = points[ ( intPoint + 2 ) % l ];

		} else {

			// extrapolate last point
			tmp.subVectors( points[ l - 1 ], points[ l - 2 ] ).add( points[ l - 1 ] );
			p3 = tmp;

		}

		if ( this.curveType === 'centripetal' || this.curveType === 'chordal' ) {

			// init Centripetal / Chordal Catmull-Rom
			const pow = this.curveType === 'chordal' ? 0.5 : 0.25;
			let dt0 = Math.pow( p0.distanceToSquared( p1 ), pow );
			let dt1 = Math.pow( p1.distanceToSquared( p2 ), pow );
			let dt2 = Math.pow( p2.distanceToSquared( p3 ), pow );

			// safety check for repeated points
			if ( dt1 < 1e-4 ) dt1 = 1.0;
			if ( dt0 < 1e-4 ) dt0 = dt1;
			if ( dt2 < 1e-4 ) dt2 = dt1;

			px.initNonuniformCatmullRom( p0.x, p1.x, p2.x, p3.x, dt0, dt1, dt2 );
			py.initNonuniformCatmullRom( p0.y, p1.y, p2.y, p3.y, dt0, dt1, dt2 );
			pz.initNonuniformCatmullRom( p0.z, p1.z, p2.z, p3.z, dt0, dt1, dt2 );

		} else if ( this.curveType === 'catmullrom' ) {

			px.initCatmullRom( p0.x, p1.x, p2.x, p3.x, this.tension );
			py.initCatmullRom( p0.y, p1.y, p2.y, p3.y, this.tension );
			pz.initCatmullRom( p0.z, p1.z, p2.z, p3.z, this.tension );

		}

		point.set(
			px.calc( weight ),
			py.calc( weight ),
			pz.calc( weight )
		);

		return point;

	}

	copy( source ) {

		super.copy( source );

		this.points = [];

		for ( let i = 0, l = source.points.length; i < l; i ++ ) {

			const point = source.points[ i ];

			this.points.push( point.clone() );

		}

		this.closed = source.closed;
		this.curveType = source.curveType;
		this.tension = source.tension;

		return this;

	}

	toJSON() {

		const data = super.toJSON();

		data.points = [];

		for ( let i = 0, l = this.points.length; i < l; i ++ ) {

			const point = this.points[ i ];
			data.points.push( point.toArray() );

		}

		data.closed = this.closed;
		data.curveType = this.curveType;
		data.tension = this.tension;

		return data;

	}

	fromJSON( json ) {

		super.fromJSON( json );

		this.points = [];

		for ( let i = 0, l = json.points.length; i < l; i ++ ) {

			const point = json.points[ i ];
			this.points.push( new Vector3().fromArray( point ) );

		}

		this.closed = json.closed;
		this.curveType = json.curveType;
		this.tension = json.tension;

		return this;

	}

}

CatmullRomCurve3.prototype.isCatmullRomCurve3 = true;

/**
 * Bezier Curves formulas obtained from
 * http://en.wikipedia.org/wiki/Bézier_curve
 */

function CatmullRom( t, p0, p1, p2, p3 ) {

	const v0 = ( p2 - p0 ) * 0.5;
	const v1 = ( p3 - p1 ) * 0.5;
	const t2 = t * t;
	const t3 = t * t2;
	return ( 2 * p1 - 2 * p2 + v0 + v1 ) * t3 + ( - 3 * p1 + 3 * p2 - 2 * v0 - v1 ) * t2 + v0 * t + p1;

}

//

function QuadraticBezierP0( t, p ) {

	const k = 1 - t;
	return k * k * p;

}

function QuadraticBezierP1( t, p ) {

	return 2 * ( 1 - t ) * t * p;

}

function QuadraticBezierP2( t, p ) {

	return t * t * p;

}

function QuadraticBezier( t, p0, p1, p2 ) {

	return QuadraticBezierP0( t, p0 ) + QuadraticBezierP1( t, p1 ) +
		QuadraticBezierP2( t, p2 );

}

//

function CubicBezierP0( t, p ) {

	const k = 1 - t;
	return k * k * k * p;

}

function CubicBezierP1( t, p ) {

	const k = 1 - t;
	return 3 * k * k * t * p;

}

function CubicBezierP2( t, p ) {

	return 3 * ( 1 - t ) * t * t * p;

}

function CubicBezierP3( t, p ) {

	return t * t * t * p;

}

function CubicBezier( t, p0, p1, p2, p3 ) {

	return CubicBezierP0( t, p0 ) + CubicBezierP1( t, p1 ) + CubicBezierP2( t, p2 ) +
		CubicBezierP3( t, p3 );

}

class CubicBezierCurve extends Curve {

	constructor( v0 = new Vector2(), v1 = new Vector2(), v2 = new Vector2(), v3 = new Vector2() ) {

		super();

		this.type = 'CubicBezierCurve';

		this.v0 = v0;
		this.v1 = v1;
		this.v2 = v2;
		this.v3 = v3;

	}

	getPoint( t, optionalTarget = new Vector2() ) {

		const point = optionalTarget;

		const v0 = this.v0, v1 = this.v1, v2 = this.v2, v3 = this.v3;

		point.set(
			CubicBezier( t, v0.x, v1.x, v2.x, v3.x ),
			CubicBezier( t, v0.y, v1.y, v2.y, v3.y )
		);

		return point;

	}

	copy( source ) {

		super.copy( source );

		this.v0.copy( source.v0 );
		this.v1.copy( source.v1 );
		this.v2.copy( source.v2 );
		this.v3.copy( source.v3 );

		return this;

	}

	toJSON() {

		const data = super.toJSON();

		data.v0 = this.v0.toArray();
		data.v1 = this.v1.toArray();
		data.v2 = this.v2.toArray();
		data.v3 = this.v3.toArray();

		return data;

	}

	fromJSON( json ) {

		super.fromJSON( json );

		this.v0.fromArray( json.v0 );
		this.v1.fromArray( json.v1 );
		this.v2.fromArray( json.v2 );
		this.v3.fromArray( json.v3 );

		return this;

	}

}

CubicBezierCurve.prototype.isCubicBezierCurve = true;

class CubicBezierCurve3 extends Curve {

	constructor( v0 = new Vector3(), v1 = new Vector3(), v2 = new Vector3(), v3 = new Vector3() ) {

		super();

		this.type = 'CubicBezierCurve3';

		this.v0 = v0;
		this.v1 = v1;
		this.v2 = v2;
		this.v3 = v3;

	}

	getPoint( t, optionalTarget = new Vector3() ) {

		const point = optionalTarget;

		const v0 = this.v0, v1 = this.v1, v2 = this.v2, v3 = this.v3;

		point.set(
			CubicBezier( t, v0.x, v1.x, v2.x, v3.x ),
			CubicBezier( t, v0.y, v1.y, v2.y, v3.y ),
			CubicBezier( t, v0.z, v1.z, v2.z, v3.z )
		);

		return point;

	}

	copy( source ) {

		super.copy( source );

		this.v0.copy( source.v0 );
		this.v1.copy( source.v1 );
		this.v2.copy( source.v2 );
		this.v3.copy( source.v3 );

		return this;

	}

	toJSON() {

		const data = super.toJSON();

		data.v0 = this.v0.toArray();
		data.v1 = this.v1.toArray();
		data.v2 = this.v2.toArray();
		data.v3 = this.v3.toArray();

		return data;

	}

	fromJSON( json ) {

		super.fromJSON( json );

		this.v0.fromArray( json.v0 );
		this.v1.fromArray( json.v1 );
		this.v2.fromArray( json.v2 );
		this.v3.fromArray( json.v3 );

		return this;

	}

}

CubicBezierCurve3.prototype.isCubicBezierCurve3 = true;

class LineCurve extends Curve {

	constructor( v1 = new Vector2(), v2 = new Vector2() ) {

		super();

		this.type = 'LineCurve';

		this.v1 = v1;
		this.v2 = v2;

	}

	getPoint( t, optionalTarget = new Vector2() ) {

		const point = optionalTarget;

		if ( t === 1 ) {

			point.copy( this.v2 );

		} else {

			point.copy( this.v2 ).sub( this.v1 );
			point.multiplyScalar( t ).add( this.v1 );

		}

		return point;

	}

	// Line curve is linear, so we can overwrite default getPointAt
	getPointAt( u, optionalTarget ) {

		return this.getPoint( u, optionalTarget );

	}

	getTangent( t, optionalTarget ) {

		const tangent = optionalTarget || new Vector2();

		tangent.copy( this.v2 ).sub( this.v1 ).normalize();

		return tangent;

	}

	copy( source ) {

		super.copy( source );

		this.v1.copy( source.v1 );
		this.v2.copy( source.v2 );

		return this;

	}

	toJSON() {

		const data = super.toJSON();

		data.v1 = this.v1.toArray();
		data.v2 = this.v2.toArray();

		return data;

	}

	fromJSON( json ) {

		super.fromJSON( json );

		this.v1.fromArray( json.v1 );
		this.v2.fromArray( json.v2 );

		return this;

	}

}

LineCurve.prototype.isLineCurve = true;

class LineCurve3 extends Curve {

	constructor( v1 = new Vector3(), v2 = new Vector3() ) {

		super();

		this.type = 'LineCurve3';
		this.isLineCurve3 = true;

		this.v1 = v1;
		this.v2 = v2;

	}
	getPoint( t, optionalTarget = new Vector3() ) {

		const point = optionalTarget;

		if ( t === 1 ) {

			point.copy( this.v2 );

		} else {

			point.copy( this.v2 ).sub( this.v1 );
			point.multiplyScalar( t ).add( this.v1 );

		}

		return point;

	}
	// Line curve is linear, so we can overwrite default getPointAt
	getPointAt( u, optionalTarget ) {

		return this.getPoint( u, optionalTarget );

	}
	copy( source ) {

		super.copy( source );

		this.v1.copy( source.v1 );
		this.v2.copy( source.v2 );

		return this;

	}
	toJSON() {

		const data = super.toJSON();

		data.v1 = this.v1.toArray();
		data.v2 = this.v2.toArray();

		return data;

	}
	fromJSON( json ) {

		super.fromJSON( json );

		this.v1.fromArray( json.v1 );
		this.v2.fromArray( json.v2 );

		return this;

	}

}

class QuadraticBezierCurve extends Curve {

	constructor( v0 = new Vector2(), v1 = new Vector2(), v2 = new Vector2() ) {

		super();

		this.type = 'QuadraticBezierCurve';

		this.v0 = v0;
		this.v1 = v1;
		this.v2 = v2;

	}

	getPoint( t, optionalTarget = new Vector2() ) {

		const point = optionalTarget;

		const v0 = this.v0, v1 = this.v1, v2 = this.v2;

		point.set(
			QuadraticBezier( t, v0.x, v1.x, v2.x ),
			QuadraticBezier( t, v0.y, v1.y, v2.y )
		);

		return point;

	}

	copy( source ) {

		super.copy( source );

		this.v0.copy( source.v0 );
		this.v1.copy( source.v1 );
		this.v2.copy( source.v2 );

		return this;

	}

	toJSON() {

		const data = super.toJSON();

		data.v0 = this.v0.toArray();
		data.v1 = this.v1.toArray();
		data.v2 = this.v2.toArray();

		return data;

	}

	fromJSON( json ) {

		super.fromJSON( json );

		this.v0.fromArray( json.v0 );
		this.v1.fromArray( json.v1 );
		this.v2.fromArray( json.v2 );

		return this;

	}

}

QuadraticBezierCurve.prototype.isQuadraticBezierCurve = true;

class QuadraticBezierCurve3 extends Curve {

	constructor( v0 = new Vector3(), v1 = new Vector3(), v2 = new Vector3() ) {

		super();

		this.type = 'QuadraticBezierCurve3';

		this.v0 = v0;
		this.v1 = v1;
		this.v2 = v2;

	}

	getPoint( t, optionalTarget = new Vector3() ) {

		const point = optionalTarget;

		const v0 = this.v0, v1 = this.v1, v2 = this.v2;

		point.set(
			QuadraticBezier( t, v0.x, v1.x, v2.x ),
			QuadraticBezier( t, v0.y, v1.y, v2.y ),
			QuadraticBezier( t, v0.z, v1.z, v2.z )
		);

		return point;

	}

	copy( source ) {

		super.copy( source );

		this.v0.copy( source.v0 );
		this.v1.copy( source.v1 );
		this.v2.copy( source.v2 );

		return this;

	}

	toJSON() {

		const data = super.toJSON();

		data.v0 = this.v0.toArray();
		data.v1 = this.v1.toArray();
		data.v2 = this.v2.toArray();

		return data;

	}

	fromJSON( json ) {

		super.fromJSON( json );

		this.v0.fromArray( json.v0 );
		this.v1.fromArray( json.v1 );
		this.v2.fromArray( json.v2 );

		return this;

	}

}

QuadraticBezierCurve3.prototype.isQuadraticBezierCurve3 = true;

class SplineCurve extends Curve {

	constructor( points = [] ) {

		super();

		this.type = 'SplineCurve';

		this.points = points;

	}

	getPoint( t, optionalTarget = new Vector2() ) {

		const point = optionalTarget;

		const points = this.points;
		const p = ( points.length - 1 ) * t;

		const intPoint = Math.floor( p );
		const weight = p - intPoint;

		const p0 = points[ intPoint === 0 ? intPoint : intPoint - 1 ];
		const p1 = points[ intPoint ];
		const p2 = points[ intPoint > points.length - 2 ? points.length - 1 : intPoint + 1 ];
		const p3 = points[ intPoint > points.length - 3 ? points.length - 1 : intPoint + 2 ];

		point.set(
			CatmullRom( weight, p0.x, p1.x, p2.x, p3.x ),
			CatmullRom( weight, p0.y, p1.y, p2.y, p3.y )
		);

		return point;

	}

	copy( source ) {

		super.copy( source );

		this.points = [];

		for ( let i = 0, l = source.points.length; i < l; i ++ ) {

			const point = source.points[ i ];

			this.points.push( point.clone() );

		}

		return this;

	}

	toJSON() {

		const data = super.toJSON();

		data.points = [];

		for ( let i = 0, l = this.points.length; i < l; i ++ ) {

			const point = this.points[ i ];
			data.points.push( point.toArray() );

		}

		return data;

	}

	fromJSON( json ) {

		super.fromJSON( json );

		this.points = [];

		for ( let i = 0, l = json.points.length; i < l; i ++ ) {

			const point = json.points[ i ];
			this.points.push( new Vector2().fromArray( point ) );

		}

		return this;

	}

}

SplineCurve.prototype.isSplineCurve = true;

var Curves = /*#__PURE__*/Object.freeze({
	__proto__: null,
	ArcCurve: ArcCurve,
	CatmullRomCurve3: CatmullRomCurve3,
	CubicBezierCurve: CubicBezierCurve,
	CubicBezierCurve3: CubicBezierCurve3,
	EllipseCurve: EllipseCurve,
	LineCurve: LineCurve,
	LineCurve3: LineCurve3,
	QuadraticBezierCurve: QuadraticBezierCurve,
	QuadraticBezierCurve3: QuadraticBezierCurve3,
	SplineCurve: SplineCurve
});

/**************************************************************
 *	Curved Path - a curve path is simply a array of connected
 *  curves, but retains the api of a curve
 **************************************************************/

class CurvePath extends Curve {

	constructor() {

		super();

		this.type = 'CurvePath';

		this.curves = [];
		this.autoClose = false; // Automatically closes the path

	}

	add( curve ) {

		this.curves.push( curve );

	}

	closePath() {

		// Add a line curve if start and end of lines are not connected
		const startPoint = this.curves[ 0 ].getPoint( 0 );
		const endPoint = this.curves[ this.curves.length - 1 ].getPoint( 1 );

		if ( ! startPoint.equals( endPoint ) ) {

			this.curves.push( new LineCurve( endPoint, startPoint ) );

		}

	}

	// To get accurate point with reference to
	// entire path distance at time t,
	// following has to be done:

	// 1. Length of each sub path have to be known
	// 2. Locate and identify type of curve
	// 3. Get t for the curve
	// 4. Return curve.getPointAt(t')

	getPoint( t ) {

		const d = t * this.getLength();
		const curveLengths = this.getCurveLengths();
		let i = 0;

		// To think about boundaries points.

		while ( i < curveLengths.length ) {

			if ( curveLengths[ i ] >= d ) {

				const diff = curveLengths[ i ] - d;
				const curve = this.curves[ i ];

				const segmentLength = curve.getLength();
				const u = segmentLength === 0 ? 0 : 1 - diff / segmentLength;

				return curve.getPointAt( u );

			}

			i ++;

		}

		return null;

		// loop where sum != 0, sum > d , sum+1 <d

	}

	// We cannot use the default THREE.Curve getPoint() with getLength() because in
	// THREE.Curve, getLength() depends on getPoint() but in THREE.CurvePath
	// getPoint() depends on getLength

	getLength() {

		const lens = this.getCurveLengths();
		return lens[ lens.length - 1 ];

	}

	// cacheLengths must be recalculated.
	updateArcLengths() {

		this.needsUpdate = true;
		this.cacheLengths = null;
		this.getCurveLengths();

	}

	// Compute lengths and cache them
	// We cannot overwrite getLengths() because UtoT mapping uses it.

	getCurveLengths() {

		// We use cache values if curves and cache array are same length

		if ( this.cacheLengths && this.cacheLengths.length === this.curves.length ) {

			return this.cacheLengths;

		}

		// Get length of sub-curve
		// Push sums into cached array

		const lengths = [];
		let sums = 0;

		for ( let i = 0, l = this.curves.length; i < l; i ++ ) {

			sums += this.curves[ i ].getLength();
			lengths.push( sums );

		}

		this.cacheLengths = lengths;

		return lengths;

	}

	getSpacedPoints( divisions = 40 ) {

		const points = [];

		for ( let i = 0; i <= divisions; i ++ ) {

			points.push( this.getPoint( i / divisions ) );

		}

		if ( this.autoClose ) {

			points.push( points[ 0 ] );

		}

		return points;

	}

	getPoints( divisions = 12 ) {

		const points = [];
		let last;

		for ( let i = 0, curves = this.curves; i < curves.length; i ++ ) {

			const curve = curves[ i ];
			const resolution = ( curve && curve.isEllipseCurve ) ? divisions * 2
				: ( curve && ( curve.isLineCurve || curve.isLineCurve3 ) ) ? 1
					: ( curve && curve.isSplineCurve ) ? divisions * curve.points.length
						: divisions;

			const pts = curve.getPoints( resolution );

			for ( let j = 0; j < pts.length; j ++ ) {

				const point = pts[ j ];

				if ( last && last.equals( point ) ) continue; // ensures no consecutive points are duplicates

				points.push( point );
				last = point;

			}

		}

		if ( this.autoClose && points.length > 1 && ! points[ points.length - 1 ].equals( points[ 0 ] ) ) {

			points.push( points[ 0 ] );

		}

		return points;

	}

	copy( source ) {

		super.copy( source );

		this.curves = [];

		for ( let i = 0, l = source.curves.length; i < l; i ++ ) {

			const curve = source.curves[ i ];

			this.curves.push( curve.clone() );

		}

		this.autoClose = source.autoClose;

		return this;

	}

	toJSON() {

		const data = super.toJSON();

		data.autoClose = this.autoClose;
		data.curves = [];

		for ( let i = 0, l = this.curves.length; i < l; i ++ ) {

			const curve = this.curves[ i ];
			data.curves.push( curve.toJSON() );

		}

		return data;

	}

	fromJSON( json ) {

		super.fromJSON( json );

		this.autoClose = json.autoClose;
		this.curves = [];

		for ( let i = 0, l = json.curves.length; i < l; i ++ ) {

			const curve = json.curves[ i ];
			this.curves.push( new Curves[ curve.type ]().fromJSON( curve ) );

		}

		return this;

	}

}

class Path extends CurvePath {

	constructor( points ) {

		super();
		this.type = 'Path';

		this.currentPoint = new Vector2();

		if ( points ) {

			this.setFromPoints( points );

		}

	}

	setFromPoints( points ) {

		this.moveTo( points[ 0 ].x, points[ 0 ].y );

		for ( let i = 1, l = points.length; i < l; i ++ ) {

			this.lineTo( points[ i ].x, points[ i ].y );

		}

		return this;

	}

	moveTo( x, y ) {

		this.currentPoint.set( x, y ); // TODO consider referencing vectors instead of copying?

		return this;

	}

	lineTo( x, y ) {

		const curve = new LineCurve( this.currentPoint.clone(), new Vector2( x, y ) );
		this.curves.push( curve );

		this.currentPoint.set( x, y );

		return this;

	}

	quadraticCurveTo( aCPx, aCPy, aX, aY ) {

		const curve = new QuadraticBezierCurve(
			this.currentPoint.clone(),
			new Vector2( aCPx, aCPy ),
			new Vector2( aX, aY )
		);

		this.curves.push( curve );

		this.currentPoint.set( aX, aY );

		return this;

	}

	bezierCurveTo( aCP1x, aCP1y, aCP2x, aCP2y, aX, aY ) {

		const curve = new CubicBezierCurve(
			this.currentPoint.clone(),
			new Vector2( aCP1x, aCP1y ),
			new Vector2( aCP2x, aCP2y ),
			new Vector2( aX, aY )
		);

		this.curves.push( curve );

		this.currentPoint.set( aX, aY );

		return this;

	}

	splineThru( pts /*Array of Vector*/ ) {

		const npts = [ this.currentPoint.clone() ].concat( pts );

		const curve = new SplineCurve( npts );
		this.curves.push( curve );

		this.currentPoint.copy( pts[ pts.length - 1 ] );

		return this;

	}

	arc( aX, aY, aRadius, aStartAngle, aEndAngle, aClockwise ) {

		const x0 = this.currentPoint.x;
		const y0 = this.currentPoint.y;

		this.absarc( aX + x0, aY + y0, aRadius,
			aStartAngle, aEndAngle, aClockwise );

		return this;

	}

	absarc( aX, aY, aRadius, aStartAngle, aEndAngle, aClockwise ) {

		this.absellipse( aX, aY, aRadius, aRadius, aStartAngle, aEndAngle, aClockwise );

		return this;

	}

	ellipse( aX, aY, xRadius, yRadius, aStartAngle, aEndAngle, aClockwise, aRotation ) {

		const x0 = this.currentPoint.x;
		const y0 = this.currentPoint.y;

		this.absellipse( aX + x0, aY + y0, xRadius, yRadius, aStartAngle, aEndAngle, aClockwise, aRotation );

		return this;

	}

	absellipse( aX, aY, xRadius, yRadius, aStartAngle, aEndAngle, aClockwise, aRotation ) {

		const curve = new EllipseCurve( aX, aY, xRadius, yRadius, aStartAngle, aEndAngle, aClockwise, aRotation );

		if ( this.curves.length > 0 ) {

			// if a previous curve is present, attempt to join
			const firstPoint = curve.getPoint( 0 );

			if ( ! firstPoint.equals( this.currentPoint ) ) {

				this.lineTo( firstPoint.x, firstPoint.y );

			}

		}

		this.curves.push( curve );

		const lastPoint = curve.getPoint( 1 );
		this.currentPoint.copy( lastPoint );

		return this;

	}

	copy( source ) {

		super.copy( source );

		this.currentPoint.copy( source.currentPoint );

		return this;

	}

	toJSON() {

		const data = super.toJSON();

		data.currentPoint = this.currentPoint.toArray();

		return data;

	}

	fromJSON( json ) {

		super.fromJSON( json );

		this.currentPoint.fromArray( json.currentPoint );

		return this;

	}

}

class Shape extends Path {

	constructor( points ) {

		super( points );

		this.uuid = MathUtils.generateUUID();

		this.type = 'Shape';

		this.holes = [];

	}

	getPointsHoles( divisions ) {

		const holesPts = [];

		for ( let i = 0, l = this.holes.length; i < l; i ++ ) {

			holesPts[ i ] = this.holes[ i ].getPoints( divisions );

		}

		return holesPts;

	}

	// get points of shape and holes (keypoints based on segments parameter)

	extractPoints( divisions ) {

		return {

			shape: this.getPoints( divisions ),
			holes: this.getPointsHoles( divisions )

		};

	}

	copy( source ) {

		super.copy( source );

		this.holes = [];

		for ( let i = 0, l = source.holes.length; i < l; i ++ ) {

			const hole = source.holes[ i ];

			this.holes.push( hole.clone() );

		}

		return this;

	}

	toJSON() {

		const data = super.toJSON();

		data.uuid = this.uuid;
		data.holes = [];

		for ( let i = 0, l = this.holes.length; i < l; i ++ ) {

			const hole = this.holes[ i ];
			data.holes.push( hole.toJSON() );

		}

		return data;

	}

	fromJSON( json ) {

		super.fromJSON( json );

		this.uuid = json.uuid;
		this.holes = [];

		for ( let i = 0, l = json.holes.length; i < l; i ++ ) {

			const hole = json.holes[ i ];
			this.holes.push( new Path().fromJSON( hole ) );

		}

		return this;

	}

}

class Light extends Object3D {

	constructor( color, intensity = 1 ) {

		super();

		this.type = 'Light';

		this.color = new Color( color );
		this.intensity = intensity;

	}

	copy( source ) {

		super.copy( source );

		this.color.copy( source.color );
		this.intensity = source.intensity;

		return this;

	}

	toJSON( meta ) {

		const data = super.toJSON( meta );

		data.object.color = this.color.getHex();
		data.object.intensity = this.intensity;

		if ( this.groundColor !== undefined ) data.object.groundColor = this.groundColor.getHex();

		if ( this.distance !== undefined ) data.object.distance = this.distance;
		if ( this.angle !== undefined ) data.object.angle = this.angle;
		if ( this.decay !== undefined ) data.object.decay = this.decay;
		if ( this.penumbra !== undefined ) data.object.penumbra = this.penumbra;

		if ( this.shadow !== undefined ) data.object.shadow = this.shadow.toJSON();

		return data;

	}

}

Light.prototype.isLight = true;

class HemisphereLight extends Light {

	constructor( skyColor, groundColor, intensity ) {

		super( skyColor, intensity );

		this.type = 'HemisphereLight';

		this.position.copy( Object3D.DefaultUp );
		this.updateMatrix();

		this.groundColor = new Color( groundColor );

	}

	copy( source ) {

		Light.prototype.copy.call( this, source );

		this.groundColor.copy( source.groundColor );

		return this;

	}

}

HemisphereLight.prototype.isHemisphereLight = true;

const _projScreenMatrix$1 = /*@__PURE__*/ new Matrix4();
const _lightPositionWorld$1 = /*@__PURE__*/ new Vector3();
const _lookTarget$1 = /*@__PURE__*/ new Vector3();

class LightShadow {

	constructor( camera ) {

		this.camera = camera;

		this.bias = 0;
		this.normalBias = 0;
		this.radius = 1;

		this.mapSize = new Vector2( 512, 512 );

		this.map = null;
		this.mapPass = null;
		this.matrix = new Matrix4();

		this.autoUpdate = true;
		this.needsUpdate = false;

		this._frustum = new Frustum();
		this._frameExtents = new Vector2( 1, 1 );

		this._viewportCount = 1;

		this._viewports = [

			new Vector4( 0, 0, 1, 1 )

		];

	}

	getViewportCount() {

		return this._viewportCount;

	}

	getFrustum() {

		return this._frustum;

	}

	updateMatrices( light ) {

		const shadowCamera = this.camera;
		const shadowMatrix = this.matrix;

		_lightPositionWorld$1.setFromMatrixPosition( light.matrixWorld );
		shadowCamera.position.copy( _lightPositionWorld$1 );

		_lookTarget$1.setFromMatrixPosition( light.target.matrixWorld );
		shadowCamera.lookAt( _lookTarget$1 );
		shadowCamera.updateMatrixWorld();

		_projScreenMatrix$1.multiplyMatrices( shadowCamera.projectionMatrix, shadowCamera.matrixWorldInverse );
		this._frustum.setFromProjectionMatrix( _projScreenMatrix$1 );

		shadowMatrix.set(
			0.5, 0.0, 0.0, 0.5,
			0.0, 0.5, 0.0, 0.5,
			0.0, 0.0, 0.5, 0.5,
			0.0, 0.0, 0.0, 1.0
		);

		shadowMatrix.multiply( shadowCamera.projectionMatrix );
		shadowMatrix.multiply( shadowCamera.matrixWorldInverse );

	}

	getViewport( viewportIndex ) {

		return this._viewports[ viewportIndex ];

	}

	getFrameExtents() {

		return this._frameExtents;

	}

	copy( source ) {

		this.camera = source.camera.clone();

		this.bias = source.bias;
		this.radius = source.radius;

		this.mapSize.copy( source.mapSize );

		return this;

	}

	clone() {

		return new this.constructor().copy( this );

	}

	toJSON() {

		const object = {};

		if ( this.bias !== 0 ) object.bias = this.bias;
		if ( this.normalBias !== 0 ) object.normalBias = this.normalBias;
		if ( this.radius !== 1 ) object.radius = this.radius;
		if ( this.mapSize.x !== 512 || this.mapSize.y !== 512 ) object.mapSize = this.mapSize.toArray();

		object.camera = this.camera.toJSON( false ).object;
		delete object.camera.matrix;

		return object;

	}

}

class SpotLightShadow extends LightShadow {

	constructor() {

		super( new PerspectiveCamera( 50, 1, 0.5, 500 ) );

		this.focus = 1;

	}

	updateMatrices( light ) {

		const camera = this.camera;

		const fov = MathUtils.RAD2DEG * 2 * light.angle * this.focus;
		const aspect = this.mapSize.width / this.mapSize.height;
		const far = light.distance || camera.far;

		if ( fov !== camera.fov || aspect !== camera.aspect || far !== camera.far ) {

			camera.fov = fov;
			camera.aspect = aspect;
			camera.far = far;
			camera.updateProjectionMatrix();

		}

		super.updateMatrices( light );

	}

	copy( source ) {

		super.copy( source );

		this.focus = source.focus;

		return this;

	}

}

SpotLightShadow.prototype.isSpotLightShadow = true;

class SpotLight extends Light {

	constructor( color, intensity, distance = 0, angle = Math.PI / 3, penumbra = 0, decay = 1 ) {

		super( color, intensity );

		this.type = 'SpotLight';

		this.position.copy( Object3D.DefaultUp );
		this.updateMatrix();

		this.target = new Object3D();

		this.distance = distance;
		this.angle = angle;
		this.penumbra = penumbra;
		this.decay = decay; // for physically correct lights, should be 2.

		this.shadow = new SpotLightShadow();

	}

	get power() {

		// intensity = power per solid angle.
		// ref: equation (17) from https://seblagarde.files.wordpress.com/2015/07/course_notes_moving_frostbite_to_pbr_v32.pdf
		return this.intensity * Math.PI;

	}

	set power( power ) {

		// intensity = power per solid angle.
		// ref: equation (17) from https://seblagarde.files.wordpress.com/2015/07/course_notes_moving_frostbite_to_pbr_v32.pdf
		this.intensity = power / Math.PI;

	}

	copy( source ) {

		super.copy( source );

		this.distance = source.distance;
		this.angle = source.angle;
		this.penumbra = source.penumbra;
		this.decay = source.decay;

		this.target = source.target.clone();

		this.shadow = source.shadow.clone();

		return this;

	}

}

SpotLight.prototype.isSpotLight = true;

const _projScreenMatrix = /*@__PURE__*/ new Matrix4();
const _lightPositionWorld = /*@__PURE__*/ new Vector3();
const _lookTarget = /*@__PURE__*/ new Vector3();

class PointLightShadow extends LightShadow {

	constructor() {

		super( new PerspectiveCamera( 90, 1, 0.5, 500 ) );

		this._frameExtents = new Vector2( 4, 2 );

		this._viewportCount = 6;

		this._viewports = [
			// These viewports map a cube-map onto a 2D texture with the
			// following orientation:
			//
			//  xzXZ
			//   y Y
			//
			// X - Positive x direction
			// x - Negative x direction
			// Y - Positive y direction
			// y - Negative y direction
			// Z - Positive z direction
			// z - Negative z direction

			// positive X
			new Vector4( 2, 1, 1, 1 ),
			// negative X
			new Vector4( 0, 1, 1, 1 ),
			// positive Z
			new Vector4( 3, 1, 1, 1 ),
			// negative Z
			new Vector4( 1, 1, 1, 1 ),
			// positive Y
			new Vector4( 3, 0, 1, 1 ),
			// negative Y
			new Vector4( 1, 0, 1, 1 )
		];

		this._cubeDirections = [
			new Vector3( 1, 0, 0 ), new Vector3( - 1, 0, 0 ), new Vector3( 0, 0, 1 ),
			new Vector3( 0, 0, - 1 ), new Vector3( 0, 1, 0 ), new Vector3( 0, - 1, 0 )
		];

		this._cubeUps = [
			new Vector3( 0, 1, 0 ), new Vector3( 0, 1, 0 ), new Vector3( 0, 1, 0 ),
			new Vector3( 0, 1, 0 ), new Vector3( 0, 0, 1 ),	new Vector3( 0, 0, - 1 )
		];

	}

	updateMatrices( light, viewportIndex = 0 ) {

		const camera = this.camera;
		const shadowMatrix = this.matrix;

		const far = light.distance || camera.far;

		if ( far !== camera.far ) {

			camera.far = far;
			camera.updateProjectionMatrix();

		}

		_lightPositionWorld.setFromMatrixPosition( light.matrixWorld );
		camera.position.copy( _lightPositionWorld );

		_lookTarget.copy( camera.position );
		_lookTarget.add( this._cubeDirections[ viewportIndex ] );
		camera.up.copy( this._cubeUps[ viewportIndex ] );
		camera.lookAt( _lookTarget );
		camera.updateMatrixWorld();

		shadowMatrix.makeTranslation( - _lightPositionWorld.x, - _lightPositionWorld.y, - _lightPositionWorld.z );

		_projScreenMatrix.multiplyMatrices( camera.projectionMatrix, camera.matrixWorldInverse );
		this._frustum.setFromProjectionMatrix( _projScreenMatrix );

	}

}

PointLightShadow.prototype.isPointLightShadow = true;

class PointLight extends Light {

	constructor( color, intensity, distance = 0, decay = 1 ) {

		super( color, intensity );

		this.type = 'PointLight';

		this.distance = distance;
		this.decay = decay; // for physically correct lights, should be 2.

		this.shadow = new PointLightShadow();

	}

	get power() {

		// intensity = power per solid angle.
		// ref: equation (15) from https://seblagarde.files.wordpress.com/2015/07/course_notes_moving_frostbite_to_pbr_v32.pdf
		return this.intensity * 4 * Math.PI;

	}

	set power( power ) {

		// intensity = power per solid angle.
		// ref: equation (15) from https://seblagarde.files.wordpress.com/2015/07/course_notes_moving_frostbite_to_pbr_v32.pdf
		this.intensity = power / ( 4 * Math.PI );

	}

	copy( source ) {

		super.copy( source );

		this.distance = source.distance;
		this.decay = source.decay;

		this.shadow = source.shadow.clone();

		return this;

	}

}

PointLight.prototype.isPointLight = true;

class OrthographicCamera extends Camera {

	constructor( left = - 1, right = 1, top = 1, bottom = - 1, near = 0.1, far = 2000 ) {

		super();

		this.type = 'OrthographicCamera';

		this.zoom = 1;
		this.view = null;

		this.left = left;
		this.right = right;
		this.top = top;
		this.bottom = bottom;

		this.near = near;
		this.far = far;

		this.updateProjectionMatrix();

	}

	copy( source, recursive ) {

		super.copy( source, recursive );

		this.left = source.left;
		this.right = source.right;
		this.top = source.top;
		this.bottom = source.bottom;
		this.near = source.near;
		this.far = source.far;

		this.zoom = source.zoom;
		this.view = source.view === null ? null : Object.assign( {}, source.view );

		return this;

	}

	setViewOffset( fullWidth, fullHeight, x, y, width, height ) {

		if ( this.view === null ) {

			this.view = {
				enabled: true,
				fullWidth: 1,
				fullHeight: 1,
				offsetX: 0,
				offsetY: 0,
				width: 1,
				height: 1
			};

		}

		this.view.enabled = true;
		this.view.fullWidth = fullWidth;
		this.view.fullHeight = fullHeight;
		this.view.offsetX = x;
		this.view.offsetY = y;
		this.view.width = width;
		this.view.height = height;

		this.updateProjectionMatrix();

	}

	clearViewOffset() {

		if ( this.view !== null ) {

			this.view.enabled = false;

		}

		this.updateProjectionMatrix();

	}

	updateProjectionMatrix() {

		const dx = ( this.right - this.left ) / ( 2 * this.zoom );
		const dy = ( this.top - this.bottom ) / ( 2 * this.zoom );
		const cx = ( this.right + this.left ) / 2;
		const cy = ( this.top + this.bottom ) / 2;

		let left = cx - dx;
		let right = cx + dx;
		let top = cy + dy;
		let bottom = cy - dy;

		if ( this.view !== null && this.view.enabled ) {

			const scaleW = ( this.right - this.left ) / this.view.fullWidth / this.zoom;
			const scaleH = ( this.top - this.bottom ) / this.view.fullHeight / this.zoom;

			left += scaleW * this.view.offsetX;
			right = left + scaleW * this.view.width;
			top -= scaleH * this.view.offsetY;
			bottom = top - scaleH * this.view.height;

		}

		this.projectionMatrix.makeOrthographic( left, right, top, bottom, this.near, this.far );

		this.projectionMatrixInverse.copy( this.projectionMatrix ).invert();

	}

	toJSON( meta ) {

		const data = Object3D.prototype.toJSON.call( this, meta );

		data.object.zoom = this.zoom;
		data.object.left = this.left;
		data.object.right = this.right;
		data.object.top = this.top;
		data.object.bottom = this.bottom;
		data.object.near = this.near;
		data.object.far = this.far;

		if ( this.view !== null ) data.object.view = Object.assign( {}, this.view );

		return data;

	}

}

OrthographicCamera.prototype.isOrthographicCamera = true;

class DirectionalLightShadow extends LightShadow {

	constructor() {

		super( new OrthographicCamera( - 5, 5, 5, - 5, 0.5, 500 ) );

	}

}

DirectionalLightShadow.prototype.isDirectionalLightShadow = true;

class DirectionalLight extends Light {

	constructor( color, intensity ) {

		super( color, intensity );

		this.type = 'DirectionalLight';

		this.position.copy( Object3D.DefaultUp );
		this.updateMatrix();

		this.target = new Object3D();

		this.shadow = new DirectionalLightShadow();

	}

	copy( source ) {

		super.copy( source );

		this.target = source.target.clone();
		this.shadow = source.shadow.clone();

		return this;

	}

}

DirectionalLight.prototype.isDirectionalLight = true;

class AmbientLight extends Light {

	constructor( color, intensity ) {

		super( color, intensity );

		this.type = 'AmbientLight';

	}

}

AmbientLight.prototype.isAmbientLight = true;

class RectAreaLight extends Light {

	constructor( color, intensity, width = 10, height = 10 ) {

		super( color, intensity );

		this.type = 'RectAreaLight';

		this.width = width;
		this.height = height;

	}

	copy( source ) {

		super.copy( source );

		this.width = source.width;
		this.height = source.height;

		return this;

	}

	toJSON( meta ) {

		const data = super.toJSON( meta );

		data.object.width = this.width;
		data.object.height = this.height;

		return data;

	}

}

RectAreaLight.prototype.isRectAreaLight = true;

/**
 * Primary reference:
 *   https://graphics.stanford.edu/papers/envmap/envmap.pdf
 *
 * Secondary reference:
 *   https://www.ppsloan.org/publications/StupidSH36.pdf
 */

// 3-band SH defined by 9 coefficients

class SphericalHarmonics3 {

	constructor() {

		this.coefficients = [];

		for ( let i = 0; i < 9; i ++ ) {

			this.coefficients.push( new Vector3() );

		}

	}

	set( coefficients ) {

		for ( let i = 0; i < 9; i ++ ) {

			this.coefficients[ i ].copy( coefficients[ i ] );

		}

		return this;

	}

	zero() {

		for ( let i = 0; i < 9; i ++ ) {

			this.coefficients[ i ].set( 0, 0, 0 );

		}

		return this;

	}

	// get the radiance in the direction of the normal
	// target is a Vector3
	getAt( normal, target ) {

		// normal is assumed to be unit length

		const x = normal.x, y = normal.y, z = normal.z;

		const coeff = this.coefficients;

		// band 0
		target.copy( coeff[ 0 ] ).multiplyScalar( 0.282095 );

		// band 1
		target.addScaledVector( coeff[ 1 ], 0.488603 * y );
		target.addScaledVector( coeff[ 2 ], 0.488603 * z );
		target.addScaledVector( coeff[ 3 ], 0.488603 * x );

		// band 2
		target.addScaledVector( coeff[ 4 ], 1.092548 * ( x * y ) );
		target.addScaledVector( coeff[ 5 ], 1.092548 * ( y * z ) );
		target.addScaledVector( coeff[ 6 ], 0.315392 * ( 3.0 * z * z - 1.0 ) );
		target.addScaledVector( coeff[ 7 ], 1.092548 * ( x * z ) );
		target.addScaledVector( coeff[ 8 ], 0.546274 * ( x * x - y * y ) );

		return target;

	}

	// get the irradiance (radiance convolved with cosine lobe) in the direction of the normal
	// target is a Vector3
	// https://graphics.stanford.edu/papers/envmap/envmap.pdf
	getIrradianceAt( normal, target ) {

		// normal is assumed to be unit length

		const x = normal.x, y = normal.y, z = normal.z;

		const coeff = this.coefficients;

		// band 0
		target.copy( coeff[ 0 ] ).multiplyScalar( 0.886227 ); // π * 0.282095

		// band 1
		target.addScaledVector( coeff[ 1 ], 2.0 * 0.511664 * y ); // ( 2 * π / 3 ) * 0.488603
		target.addScaledVector( coeff[ 2 ], 2.0 * 0.511664 * z );
		target.addScaledVector( coeff[ 3 ], 2.0 * 0.511664 * x );

		// band 2
		target.addScaledVector( coeff[ 4 ], 2.0 * 0.429043 * x * y ); // ( π / 4 ) * 1.092548
		target.addScaledVector( coeff[ 5 ], 2.0 * 0.429043 * y * z );
		target.addScaledVector( coeff[ 6 ], 0.743125 * z * z - 0.247708 ); // ( π / 4 ) * 0.315392 * 3
		target.addScaledVector( coeff[ 7 ], 2.0 * 0.429043 * x * z );
		target.addScaledVector( coeff[ 8 ], 0.429043 * ( x * x - y * y ) ); // ( π / 4 ) * 0.546274

		return target;

	}

	add( sh ) {

		for ( let i = 0; i < 9; i ++ ) {

			this.coefficients[ i ].add( sh.coefficients[ i ] );

		}

		return this;

	}

	addScaledSH( sh, s ) {

		for ( let i = 0; i < 9; i ++ ) {

			this.coefficients[ i ].addScaledVector( sh.coefficients[ i ], s );

		}

		return this;

	}

	scale( s ) {

		for ( let i = 0; i < 9; i ++ ) {

			this.coefficients[ i ].multiplyScalar( s );

		}

		return this;

	}

	lerp( sh, alpha ) {

		for ( let i = 0; i < 9; i ++ ) {

			this.coefficients[ i ].lerp( sh.coefficients[ i ], alpha );

		}

		return this;

	}

	equals( sh ) {

		for ( let i = 0; i < 9; i ++ ) {

			if ( ! this.coefficients[ i ].equals( sh.coefficients[ i ] ) ) {

				return false;

			}

		}

		return true;

	}

	copy( sh ) {

		return this.set( sh.coefficients );

	}

	clone() {

		return new this.constructor().copy( this );

	}

	fromArray( array, offset = 0 ) {

		const coefficients = this.coefficients;

		for ( let i = 0; i < 9; i ++ ) {

			coefficients[ i ].fromArray( array, offset + ( i * 3 ) );

		}

		return this;

	}

	toArray( array = [], offset = 0 ) {

		const coefficients = this.coefficients;

		for ( let i = 0; i < 9; i ++ ) {

			coefficients[ i ].toArray( array, offset + ( i * 3 ) );

		}

		return array;

	}

	// evaluate the basis functions
	// shBasis is an Array[ 9 ]
	static getBasisAt( normal, shBasis ) {

		// normal is assumed to be unit length

		const x = normal.x, y = normal.y, z = normal.z;

		// band 0
		shBasis[ 0 ] = 0.282095;

		// band 1
		shBasis[ 1 ] = 0.488603 * y;
		shBasis[ 2 ] = 0.488603 * z;
		shBasis[ 3 ] = 0.488603 * x;

		// band 2
		shBasis[ 4 ] = 1.092548 * x * y;
		shBasis[ 5 ] = 1.092548 * y * z;
		shBasis[ 6 ] = 0.315392 * ( 3 * z * z - 1 );
		shBasis[ 7 ] = 1.092548 * x * z;
		shBasis[ 8 ] = 0.546274 * ( x * x - y * y );

	}

}

SphericalHarmonics3.prototype.isSphericalHarmonics3 = true;

class LightProbe extends Light {

	constructor( sh = new SphericalHarmonics3(), intensity = 1 ) {

		super( undefined, intensity );

		this.sh = sh;

	}

	copy( source ) {

		super.copy( source );

		this.sh.copy( source.sh );

		return this;

	}

	fromJSON( json ) {

		this.intensity = json.intensity; // TODO: Move this bit to Light.fromJSON();
		this.sh.fromArray( json.sh );

		return this;

	}

	toJSON( meta ) {

		const data = super.toJSON( meta );

		data.object.sh = this.sh.toArray();

		return data;

	}

}

LightProbe.prototype.isLightProbe = true;

const LoaderUtils = {

	decodeText: function ( array ) {

		if ( typeof TextDecoder !== 'undefined' ) {

			return new TextDecoder().decode( array );

		}

		// Avoid the String.fromCharCode.apply(null, array) shortcut, which
		// throws a "maximum call stack size exceeded" error for large arrays.

		let s = '';

		for ( let i = 0, il = array.length; i < il; i ++ ) {

			// Implicitly assumes little-endian.
			s += String.fromCharCode( array[ i ] );

		}

		try {

			// merges multi-byte utf-8 characters.

			return decodeURIComponent( escape( s ) );

		} catch ( e ) { // see #16358

			return s;

		}

	},

	extractUrlBase: function ( url ) {

		const index = url.lastIndexOf( '/' );

		if ( index === - 1 ) return './';

		return url.substr( 0, index + 1 );

	}

};

function InstancedBufferGeometry() {

	BufferGeometry.call( this );

	this.type = 'InstancedBufferGeometry';
	this.instanceCount = Infinity;

}

InstancedBufferGeometry.prototype = Object.assign( Object.create( BufferGeometry.prototype ), {

	constructor: InstancedBufferGeometry,

	isInstancedBufferGeometry: true,

	copy: function ( source ) {

		BufferGeometry.prototype.copy.call( this, source );

		this.instanceCount = source.instanceCount;

		return this;

	},

	clone: function () {

		return new this.constructor().copy( this );

	},

	toJSON: function () {

		const data = BufferGeometry.prototype.toJSON.call( this );

		data.instanceCount = this.instanceCount;

		data.isInstancedBufferGeometry = true;

		return data;

	}

} );

function InstancedBufferAttribute( array, itemSize, normalized, meshPerAttribute ) {

	if ( typeof ( normalized ) === 'number' ) {

		meshPerAttribute = normalized;

		normalized = false;

		console.error( 'THREE.InstancedBufferAttribute: The constructor now expects normalized as the third argument.' );

	}

	BufferAttribute.call( this, array, itemSize, normalized );

	this.meshPerAttribute = meshPerAttribute || 1;

}

InstancedBufferAttribute.prototype = Object.assign( Object.create( BufferAttribute.prototype ), {

	constructor: InstancedBufferAttribute,

	isInstancedBufferAttribute: true,

	copy: function ( source ) {

		BufferAttribute.prototype.copy.call( this, source );

		this.meshPerAttribute = source.meshPerAttribute;

		return this;

	},

	toJSON: function ()	{

		const data = BufferAttribute.prototype.toJSON.call( this );

		data.meshPerAttribute = this.meshPerAttribute;

		data.isInstancedBufferAttribute = true;

		return data;

	}

} );

function ImageBitmapLoader( manager ) {

	if ( typeof createImageBitmap === 'undefined' ) {

		console.warn( 'THREE.ImageBitmapLoader: createImageBitmap() not supported.' );

	}

	if ( typeof fetch === 'undefined' ) {

		console.warn( 'THREE.ImageBitmapLoader: fetch() not supported.' );

	}

	Loader.call( this, manager );

	this.options = { premultiplyAlpha: 'none' };

}

ImageBitmapLoader.prototype = Object.assign( Object.create( Loader.prototype ), {

	constructor: ImageBitmapLoader,

	isImageBitmapLoader: true,

	setOptions: function setOptions( options ) {

		this.options = options;

		return this;

	},

	load: function ( url, onLoad, onProgress, onError ) {

		if ( url === undefined ) url = '';

		if ( this.path !== undefined ) url = this.path + url;

		url = this.manager.resolveURL( url );

		const scope = this;

		const cached = Cache.get( url );

		if ( cached !== undefined ) {

			scope.manager.itemStart( url );

			setTimeout( function () {

				if ( onLoad ) onLoad( cached );

				scope.manager.itemEnd( url );

			}, 0 );

			return cached;

		}

		const fetchOptions = {};
		fetchOptions.credentials = ( this.crossOrigin === 'anonymous' ) ? 'same-origin' : 'include';
		fetchOptions.headers = this.requestHeader;

		fetch( url, fetchOptions ).then( function ( res ) {

			return res.blob();

		} ).then( function ( blob ) {

			return createImageBitmap( blob, Object.assign( scope.options, { colorSpaceConversion: 'none' } ) );

		} ).then( function ( imageBitmap ) {

			Cache.add( url, imageBitmap );

			if ( onLoad ) onLoad( imageBitmap );

			scope.manager.itemEnd( url );

		} ).catch( function ( e ) {

			if ( onError ) onError( e );

			scope.manager.itemError( url );
			scope.manager.itemEnd( url );

		} );

		scope.manager.itemStart( url );

	}

} );

class ShapePath {

	constructor() {

		this.type = 'ShapePath';

		this.color = new Color();

		this.subPaths = [];
		this.currentPath = null;

	}

	moveTo( x, y ) {

		this.currentPath = new Path();
		this.subPaths.push( this.currentPath );
		this.currentPath.moveTo( x, y );

		return this;

	}

	lineTo( x, y ) {

		this.currentPath.lineTo( x, y );

		return this;

	}

	quadraticCurveTo( aCPx, aCPy, aX, aY ) {

		this.currentPath.quadraticCurveTo( aCPx, aCPy, aX, aY );

		return this;

	}

	bezierCurveTo( aCP1x, aCP1y, aCP2x, aCP2y, aX, aY ) {

		this.currentPath.bezierCurveTo( aCP1x, aCP1y, aCP2x, aCP2y, aX, aY );

		return this;

	}

	splineThru( pts ) {

		this.currentPath.splineThru( pts );

		return this;

	}

	toShapes( isCCW, noHoles ) {

		function toShapesNoHoles( inSubpaths ) {

			const shapes = [];

			for ( let i = 0, l = inSubpaths.length; i < l; i ++ ) {

				const tmpPath = inSubpaths[ i ];

				const tmpShape = new Shape();
				tmpShape.curves = tmpPath.curves;

				shapes.push( tmpShape );

			}

			return shapes;

		}

		function isPointInsidePolygon( inPt, inPolygon ) {

			const polyLen = inPolygon.length;

			// inPt on polygon contour => immediate success    or
			// toggling of inside/outside at every single! intersection point of an edge
			//  with the horizontal line through inPt, left of inPt
			//  not counting lowerY endpoints of edges and whole edges on that line
			let inside = false;
			for ( let p = polyLen - 1, q = 0; q < polyLen; p = q ++ ) {

				let edgeLowPt = inPolygon[ p ];
				let edgeHighPt = inPolygon[ q ];

				let edgeDx = edgeHighPt.x - edgeLowPt.x;
				let edgeDy = edgeHighPt.y - edgeLowPt.y;

				if ( Math.abs( edgeDy ) > Number.EPSILON ) {

					// not parallel
					if ( edgeDy < 0 ) {

						edgeLowPt = inPolygon[ q ]; edgeDx = - edgeDx;
						edgeHighPt = inPolygon[ p ]; edgeDy = - edgeDy;

					}

					if ( ( inPt.y < edgeLowPt.y ) || ( inPt.y > edgeHighPt.y ) ) 		continue;

					if ( inPt.y === edgeLowPt.y ) {

						if ( inPt.x === edgeLowPt.x )		return	true;		// inPt is on contour ?
						// continue;				// no intersection or edgeLowPt => doesn't count !!!

					} else {

						const perpEdge = edgeDy * ( inPt.x - edgeLowPt.x ) - edgeDx * ( inPt.y - edgeLowPt.y );
						if ( perpEdge === 0 )				return	true;		// inPt is on contour ?
						if ( perpEdge < 0 ) 				continue;
						inside = ! inside;		// true intersection left of inPt

					}

				} else {

					// parallel or collinear
					if ( inPt.y !== edgeLowPt.y ) 		continue;			// parallel
					// edge lies on the same horizontal line as inPt
					if ( ( ( edgeHighPt.x <= inPt.x ) && ( inPt.x <= edgeLowPt.x ) ) ||
						 ( ( edgeLowPt.x <= inPt.x ) && ( inPt.x <= edgeHighPt.x ) ) )		return	true;	// inPt: Point on contour !
					// continue;

				}

			}

			return	inside;

		}

		const isClockWise = ShapeUtils.isClockWise;

		const subPaths = this.subPaths;
		if ( subPaths.length === 0 ) return [];

		if ( noHoles === true )	return	toShapesNoHoles( subPaths );


		let solid, tmpPath, tmpShape;
		const shapes = [];

		if ( subPaths.length === 1 ) {

			tmpPath = subPaths[ 0 ];
			tmpShape = new Shape();
			tmpShape.curves = tmpPath.curves;
			shapes.push( tmpShape );
			return shapes;

		}

		let holesFirst = ! isClockWise( subPaths[ 0 ].getPoints() );
		holesFirst = isCCW ? ! holesFirst : holesFirst;

		// console.log("Holes first", holesFirst);

		const betterShapeHoles = [];
		const newShapes = [];
		let newShapeHoles = [];
		let mainIdx = 0;
		let tmpPoints;

		newShapes[ mainIdx ] = undefined;
		newShapeHoles[ mainIdx ] = [];

		for ( let i = 0, l = subPaths.length; i < l; i ++ ) {

			tmpPath = subPaths[ i ];
			tmpPoints = tmpPath.getPoints();
			solid = isClockWise( tmpPoints );
			solid = isCCW ? ! solid : solid;

			if ( solid ) {

				if ( ( ! holesFirst ) && ( newShapes[ mainIdx ] ) )	mainIdx ++;

				newShapes[ mainIdx ] = { s: new Shape(), p: tmpPoints };
				newShapes[ mainIdx ].s.curves = tmpPath.curves;

				if ( holesFirst )	mainIdx ++;
				newShapeHoles[ mainIdx ] = [];

				//console.log('cw', i);

			} else {

				newShapeHoles[ mainIdx ].push( { h: tmpPath, p: tmpPoints[ 0 ] } );

				//console.log('ccw', i);

			}

		}

		// only Holes? -> probably all Shapes with wrong orientation
		if ( ! newShapes[ 0 ] )	return	toShapesNoHoles( subPaths );


		if ( newShapes.length > 1 ) {

			let ambiguous = false;
			const toChange = [];

			for ( let sIdx = 0, sLen = newShapes.length; sIdx < sLen; sIdx ++ ) {

				betterShapeHoles[ sIdx ] = [];

			}

			for ( let sIdx = 0, sLen = newShapes.length; sIdx < sLen; sIdx ++ ) {

				const sho = newShapeHoles[ sIdx ];

				for ( let hIdx = 0; hIdx < sho.length; hIdx ++ ) {

					const ho = sho[ hIdx ];
					let hole_unassigned = true;

					for ( let s2Idx = 0; s2Idx < newShapes.length; s2Idx ++ ) {

						if ( isPointInsidePolygon( ho.p, newShapes[ s2Idx ].p ) ) {

							if ( sIdx !== s2Idx )	toChange.push( { froms: sIdx, tos: s2Idx, hole: hIdx } );
							if ( hole_unassigned ) {

								hole_unassigned = false;
								betterShapeHoles[ s2Idx ].push( ho );

							} else {

								ambiguous = true;

							}

						}

					}

					if ( hole_unassigned ) {

						betterShapeHoles[ sIdx ].push( ho );

					}

				}

			}
			// console.log("ambiguous: ", ambiguous);

			if ( toChange.length > 0 ) {

				// console.log("to change: ", toChange);
				if ( ! ambiguous )	newShapeHoles = betterShapeHoles;

			}

		}

		let tmpHoles;

		for ( let i = 0, il = newShapes.length; i < il; i ++ ) {

			tmpShape = newShapes[ i ].s;
			shapes.push( tmpShape );
			tmpHoles = newShapeHoles[ i ];

			for ( let j = 0, jl = tmpHoles.length; j < jl; j ++ ) {

				tmpShape.holes.push( tmpHoles[ j ].h );

			}

		}

		//console.log("shape", shapes);

		return shapes;

	}

}

class Font {

	constructor( data ) {

		this.type = 'Font';

		this.data = data;

	}

	generateShapes( text, size = 100 ) {

		const shapes = [];
		const paths = createPaths( text, size, this.data );

		for ( let p = 0, pl = paths.length; p < pl; p ++ ) {

			Array.prototype.push.apply( shapes, paths[ p ].toShapes() );

		}

		return shapes;

	}

}

function createPaths( text, size, data ) {

	const chars = Array.from( text );
	const scale = size / data.resolution;
	const line_height = ( data.boundingBox.yMax - data.boundingBox.yMin + data.underlineThickness ) * scale;

	const paths = [];

	let offsetX = 0, offsetY = 0;

	for ( let i = 0; i < chars.length; i ++ ) {

		const char = chars[ i ];

		if ( char === '\n' ) {

			offsetX = 0;
			offsetY -= line_height;

		} else {

			const ret = createPath( char, scale, offsetX, offsetY, data );
			offsetX += ret.offsetX;
			paths.push( ret.path );

		}

	}

	return paths;

}

function createPath( char, scale, offsetX, offsetY, data ) {

	const glyph = data.glyphs[ char ] || data.glyphs[ '?' ];

	if ( ! glyph ) {

		console.error( 'THREE.Font: character "' + char + '" does not exists in font family ' + data.familyName + '.' );

		return;

	}

	const path = new ShapePath();

	let x, y, cpx, cpy, cpx1, cpy1, cpx2, cpy2;

	if ( glyph.o ) {

		const outline = glyph._cachedOutline || ( glyph._cachedOutline = glyph.o.split( ' ' ) );

		for ( let i = 0, l = outline.length; i < l; ) {

			const action = outline[ i ++ ];

			switch ( action ) {

				case 'm': // moveTo

					x = outline[ i ++ ] * scale + offsetX;
					y = outline[ i ++ ] * scale + offsetY;

					path.moveTo( x, y );

					break;

				case 'l': // lineTo

					x = outline[ i ++ ] * scale + offsetX;
					y = outline[ i ++ ] * scale + offsetY;

					path.lineTo( x, y );

					break;

				case 'q': // quadraticCurveTo

					cpx = outline[ i ++ ] * scale + offsetX;
					cpy = outline[ i ++ ] * scale + offsetY;
					cpx1 = outline[ i ++ ] * scale + offsetX;
					cpy1 = outline[ i ++ ] * scale + offsetY;

					path.quadraticCurveTo( cpx1, cpy1, cpx, cpy );

					break;

				case 'b': // bezierCurveTo

					cpx = outline[ i ++ ] * scale + offsetX;
					cpy = outline[ i ++ ] * scale + offsetY;
					cpx1 = outline[ i ++ ] * scale + offsetX;
					cpy1 = outline[ i ++ ] * scale + offsetY;
					cpx2 = outline[ i ++ ] * scale + offsetX;
					cpy2 = outline[ i ++ ] * scale + offsetY;

					path.bezierCurveTo( cpx1, cpy1, cpx2, cpy2, cpx, cpy );

					break;

			}

		}

	}

	return { offsetX: glyph.ha * scale, path: path };

}

Font.prototype.isFont = true;

class FontLoader extends Loader {

	constructor( manager ) {

		super( manager );

	}

	load( url, onLoad, onProgress, onError ) {

		const scope = this;

		const loader = new FileLoader( this.manager );
		loader.setPath( this.path );
		loader.setRequestHeader( this.requestHeader );
		loader.setWithCredentials( scope.withCredentials );
		loader.load( url, function ( text ) {

			let json;

			try {

				json = JSON.parse( text );

			} catch ( e ) {

				console.warn( 'THREE.FontLoader: typeface.js support is being deprecated. Use typeface.json instead.' );
				json = JSON.parse( text.substring( 65, text.length - 2 ) );

			}

			const font = scope.parse( json );

			if ( onLoad ) onLoad( font );

		}, onProgress, onError );

	}

	parse( json ) {

		return new Font( json );

	}

}

let _context;

const AudioContext = {

	getContext: function () {

		if ( _context === undefined ) {

			_context = new ( window.AudioContext || window.webkitAudioContext )();

		}

		return _context;

	},

	setContext: function ( value ) {

		_context = value;

	}

};

class AudioLoader extends Loader {

	constructor( manager ) {

		super( manager );

	}

	load( url, onLoad, onProgress, onError ) {

		const scope = this;

		const loader = new FileLoader( this.manager );
		loader.setResponseType( 'arraybuffer' );
		loader.setPath( this.path );
		loader.setRequestHeader( this.requestHeader );
		loader.setWithCredentials( this.withCredentials );
		loader.load( url, function ( buffer ) {

			try {

				// Create a copy of the buffer. The `decodeAudioData` method
				// detaches the buffer when complete, preventing reuse.
				const bufferCopy = buffer.slice( 0 );

				const context = AudioContext.getContext();
				context.decodeAudioData( bufferCopy, function ( audioBuffer ) {

					onLoad( audioBuffer );

				} );

			} catch ( e ) {

				if ( onError ) {

					onError( e );

				} else {

					console.error( e );

				}

				scope.manager.itemError( url );

			}

		}, onProgress, onError );

	}

}

class HemisphereLightProbe extends LightProbe {

	constructor( skyColor, groundColor, intensity = 1 ) {

		super( undefined, intensity );

		const color1 = new Color().set( skyColor );
		const color2 = new Color().set( groundColor );

		const sky = new Vector3( color1.r, color1.g, color1.b );
		const ground = new Vector3( color2.r, color2.g, color2.b );

		// without extra factor of PI in the shader, should = 1 / Math.sqrt( Math.PI );
		const c0 = Math.sqrt( Math.PI );
		const c1 = c0 * Math.sqrt( 0.75 );

		this.sh.coefficients[ 0 ].copy( sky ).add( ground ).multiplyScalar( c0 );
		this.sh.coefficients[ 1 ].copy( sky ).sub( ground ).multiplyScalar( c1 );

	}

}

HemisphereLightProbe.prototype.isHemisphereLightProbe = true;

class AmbientLightProbe extends LightProbe {

	constructor( color, intensity = 1 ) {

		super( undefined, intensity );

		const color1 = new Color().set( color );

		// without extra factor of PI in the shader, would be 2 / Math.sqrt( Math.PI );
		this.sh.coefficients[ 0 ].set( color1.r, color1.g, color1.b ).multiplyScalar( 2 * Math.sqrt( Math.PI ) );

	}

}

AmbientLightProbe.prototype.isAmbientLightProbe = true;

const _eyeRight = new Matrix4();
const _eyeLeft = new Matrix4();

class Audio extends Object3D {

	constructor( listener ) {

		super();

		this.type = 'Audio';

		this.listener = listener;
		this.context = listener.context;

		this.gain = this.context.createGain();
		this.gain.connect( listener.getInput() );

		this.autoplay = false;

		this.buffer = null;
		this.detune = 0;
		this.loop = false;
		this.loopStart = 0;
		this.loopEnd = 0;
		this.offset = 0;
		this.duration = undefined;
		this.playbackRate = 1;
		this.isPlaying = false;
		this.hasPlaybackControl = true;
		this.source = null;
		this.sourceType = 'empty';

		this._startedAt = 0;
		this._progress = 0;
		this._connected = false;

		this.filters = [];

	}

	getOutput() {

		return this.gain;

	}

	setNodeSource( audioNode ) {

		this.hasPlaybackControl = false;
		this.sourceType = 'audioNode';
		this.source = audioNode;
		this.connect();

		return this;

	}

	setMediaElementSource( mediaElement ) {

		this.hasPlaybackControl = false;
		this.sourceType = 'mediaNode';
		this.source = this.context.createMediaElementSource( mediaElement );
		this.connect();

		return this;

	}

	setMediaStreamSource( mediaStream ) {

		this.hasPlaybackControl = false;
		this.sourceType = 'mediaStreamNode';
		this.source = this.context.createMediaStreamSource( mediaStream );
		this.connect();

		return this;

	}

	setBuffer( audioBuffer ) {

		this.buffer = audioBuffer;
		this.sourceType = 'buffer';

		if ( this.autoplay ) this.play();

		return this;

	}

	play( delay = 0 ) {

		if ( this.isPlaying === true ) {

			console.warn( 'THREE.Audio: Audio is already playing.' );
			return;

		}

		if ( this.hasPlaybackControl === false ) {

			console.warn( 'THREE.Audio: this Audio has no playback control.' );
			return;

		}

		this._startedAt = this.context.currentTime + delay;

		const source = this.context.createBufferSource();
		source.buffer = this.buffer;
		source.loop = this.loop;
		source.loopStart = this.loopStart;
		source.loopEnd = this.loopEnd;
		source.onended = this.onEnded.bind( this );
		source.start( this._startedAt, this._progress + this.offset, this.duration );

		this.isPlaying = true;

		this.source = source;

		this.setDetune( this.detune );
		this.setPlaybackRate( this.playbackRate );

		return this.connect();

	}

	pause() {

		if ( this.hasPlaybackControl === false ) {

			console.warn( 'THREE.Audio: this Audio has no playback control.' );
			return;

		}

		if ( this.isPlaying === true ) {

			// update current progress

			this._progress += Math.max( this.context.currentTime - this._startedAt, 0 ) * this.playbackRate;

			if ( this.loop === true ) {

				// ensure _progress does not exceed duration with looped audios

				this._progress = this._progress % ( this.duration || this.buffer.duration );

			}

			this.source.stop();
			this.source.onended = null;

			this.isPlaying = false;

		}

		return this;

	}

	stop() {

		if ( this.hasPlaybackControl === false ) {

			console.warn( 'THREE.Audio: this Audio has no playback control.' );
			return;

		}

		this._progress = 0;

		this.source.stop();
		this.source.onended = null;
		this.isPlaying = false;

		return this;

	}

	connect() {

		if ( this.filters.length > 0 ) {

			this.source.connect( this.filters[ 0 ] );

			for ( let i = 1, l = this.filters.length; i < l; i ++ ) {

				this.filters[ i - 1 ].connect( this.filters[ i ] );

			}

			this.filters[ this.filters.length - 1 ].connect( this.getOutput() );

		} else {

			this.source.connect( this.getOutput() );

		}

		this._connected = true;

		return this;

	}

	disconnect() {

		if ( this.filters.length > 0 ) {

			this.source.disconnect( this.filters[ 0 ] );

			for ( let i = 1, l = this.filters.length; i < l; i ++ ) {

				this.filters[ i - 1 ].disconnect( this.filters[ i ] );

			}

			this.filters[ this.filters.length - 1 ].disconnect( this.getOutput() );

		} else {

			this.source.disconnect( this.getOutput() );

		}

		this._connected = false;

		return this;

	}

	getFilters() {

		return this.filters;

	}

	setFilters( value ) {

		if ( ! value ) value = [];

		if ( this._connected === true ) {

			this.disconnect();
			this.filters = value.slice();
			this.connect();

		} else {

			this.filters = value.slice();

		}

		return this;

	}

	setDetune( value ) {

		this.detune = value;

		if ( this.source.detune === undefined ) return; // only set detune when available

		if ( this.isPlaying === true ) {

			this.source.detune.setTargetAtTime( this.detune, this.context.currentTime, 0.01 );

		}

		return this;

	}

	getDetune() {

		return this.detune;

	}

	getFilter() {

		return this.getFilters()[ 0 ];

	}

	setFilter( filter ) {

		return this.setFilters( filter ? [ filter ] : [] );

	}

	setPlaybackRate( value ) {

		if ( this.hasPlaybackControl === false ) {

			console.warn( 'THREE.Audio: this Audio has no playback control.' );
			return;

		}

		this.playbackRate = value;

		if ( this.isPlaying === true ) {

			this.source.playbackRate.setTargetAtTime( this.playbackRate, this.context.currentTime, 0.01 );

		}

		return this;

	}

	getPlaybackRate() {

		return this.playbackRate;

	}

	onEnded() {

		this.isPlaying = false;

	}

	getLoop() {

		if ( this.hasPlaybackControl === false ) {

			console.warn( 'THREE.Audio: this Audio has no playback control.' );
			return false;

		}

		return this.loop;

	}

	setLoop( value ) {

		if ( this.hasPlaybackControl === false ) {

			console.warn( 'THREE.Audio: this Audio has no playback control.' );
			return;

		}

		this.loop = value;

		if ( this.isPlaying === true ) {

			this.source.loop = this.loop;

		}

		return this;

	}

	setLoopStart( value ) {

		this.loopStart = value;

		return this;

	}

	setLoopEnd( value ) {

		this.loopEnd = value;

		return this;

	}

	getVolume() {

		return this.gain.gain.value;

	}

	setVolume( value ) {

		this.gain.gain.setTargetAtTime( value, this.context.currentTime, 0.01 );

		return this;

	}

}

class AudioAnalyser {

	constructor( audio, fftSize = 2048 ) {

		this.analyser = audio.context.createAnalyser();
		this.analyser.fftSize = fftSize;

		this.data = new Uint8Array( this.analyser.frequencyBinCount );

		audio.getOutput().connect( this.analyser );

	}


	getFrequencyData() {

		this.analyser.getByteFrequencyData( this.data );

		return this.data;

	}

	getAverageFrequency() {

		let value = 0;
		const data = this.getFrequencyData();

		for ( let i = 0; i < data.length; i ++ ) {

			value += data[ i ];

		}

		return value / data.length;

	}

}

class PropertyMixer {

	constructor( binding, typeName, valueSize ) {

		this.binding = binding;
		this.valueSize = valueSize;

		let mixFunction,
			mixFunctionAdditive,
			setIdentity;

		// buffer layout: [ incoming | accu0 | accu1 | orig | addAccu | (optional work) ]
		//
		// interpolators can use .buffer as their .result
		// the data then goes to 'incoming'
		//
		// 'accu0' and 'accu1' are used frame-interleaved for
		// the cumulative result and are compared to detect
		// changes
		//
		// 'orig' stores the original state of the property
		//
		// 'add' is used for additive cumulative results
		//
		// 'work' is optional and is only present for quaternion types. It is used
		// to store intermediate quaternion multiplication results

		switch ( typeName ) {

			case 'quaternion':
				mixFunction = this._slerp;
				mixFunctionAdditive = this._slerpAdditive;
				setIdentity = this._setAdditiveIdentityQuaternion;

				this.buffer = new Float64Array( valueSize * 6 );
				this._workIndex = 5;
				break;

			case 'string':
			case 'bool':
				mixFunction = this._select;

				// Use the regular mix function and for additive on these types,
				// additive is not relevant for non-numeric types
				mixFunctionAdditive = this._select;

				setIdentity = this._setAdditiveIdentityOther;

				this.buffer = new Array( valueSize * 5 );
				break;

			default:
				mixFunction = this._lerp;
				mixFunctionAdditive = this._lerpAdditive;
				setIdentity = this._setAdditiveIdentityNumeric;

				this.buffer = new Float64Array( valueSize * 5 );

		}

		this._mixBufferRegion = mixFunction;
		this._mixBufferRegionAdditive = mixFunctionAdditive;
		this._setIdentity = setIdentity;
		this._origIndex = 3;
		this._addIndex = 4;

		this.cumulativeWeight = 0;
		this.cumulativeWeightAdditive = 0;

		this.useCount = 0;
		this.referenceCount = 0;

	}

	// accumulate data in the 'incoming' region into 'accu<i>'
	accumulate( accuIndex, weight ) {

		// note: happily accumulating nothing when weight = 0, the caller knows
		// the weight and shouldn't have made the call in the first place

		const buffer = this.buffer,
			stride = this.valueSize,
			offset = accuIndex * stride + stride;

		let currentWeight = this.cumulativeWeight;

		if ( currentWeight === 0 ) {

			// accuN := incoming * weight

			for ( let i = 0; i !== stride; ++ i ) {

				buffer[ offset + i ] = buffer[ i ];

			}

			currentWeight = weight;

		} else {

			// accuN := accuN + incoming * weight

			currentWeight += weight;
			const mix = weight / currentWeight;
			this._mixBufferRegion( buffer, offset, 0, mix, stride );

		}

		this.cumulativeWeight = currentWeight;

	}

	// accumulate data in the 'incoming' region into 'add'
	accumulateAdditive( weight ) {

		const buffer = this.buffer,
			stride = this.valueSize,
			offset = stride * this._addIndex;

		if ( this.cumulativeWeightAdditive === 0 ) {

			// add = identity

			this._setIdentity();

		}

		// add := add + incoming * weight

		this._mixBufferRegionAdditive( buffer, offset, 0, weight, stride );
		this.cumulativeWeightAdditive += weight;

	}

	// apply the state of 'accu<i>' to the binding when accus differ
	apply( accuIndex ) {

		const stride = this.valueSize,
			buffer = this.buffer,
			offset = accuIndex * stride + stride,

			weight = this.cumulativeWeight,
			weightAdditive = this.cumulativeWeightAdditive,

			binding = this.binding;

		this.cumulativeWeight = 0;
		this.cumulativeWeightAdditive = 0;

		if ( weight < 1 ) {

			// accuN := accuN + original * ( 1 - cumulativeWeight )

			const originalValueOffset = stride * this._origIndex;

			this._mixBufferRegion(
				buffer, offset, originalValueOffset, 1 - weight, stride );

		}

		if ( weightAdditive > 0 ) {

			// accuN := accuN + additive accuN

			this._mixBufferRegionAdditive( buffer, offset, this._addIndex * stride, 1, stride );

		}

		for ( let i = stride, e = stride + stride; i !== e; ++ i ) {

			if ( buffer[ i ] !== buffer[ i + stride ] ) {

				// value has changed -> update scene graph

				binding.setValue( buffer, offset );
				break;

			}

		}

	}

	// remember the state of the bound property and copy it to both accus
	saveOriginalState() {

		const binding = this.binding;

		const buffer = this.buffer,
			stride = this.valueSize,

			originalValueOffset = stride * this._origIndex;

		binding.getValue( buffer, originalValueOffset );

		// accu[0..1] := orig -- initially detect changes against the original
		for ( let i = stride, e = originalValueOffset; i !== e; ++ i ) {

			buffer[ i ] = buffer[ originalValueOffset + ( i % stride ) ];

		}

		// Add to identity for additive
		this._setIdentity();

		this.cumulativeWeight = 0;
		this.cumulativeWeightAdditive = 0;

	}

	// apply the state previously taken via 'saveOriginalState' to the binding
	restoreOriginalState() {

		const originalValueOffset = this.valueSize * 3;
		this.binding.setValue( this.buffer, originalValueOffset );

	}

	_setAdditiveIdentityNumeric() {

		const startIndex = this._addIndex * this.valueSize;
		const endIndex = startIndex + this.valueSize;

		for ( let i = startIndex; i < endIndex; i ++ ) {

			this.buffer[ i ] = 0;

		}

	}

	_setAdditiveIdentityQuaternion() {

		this._setAdditiveIdentityNumeric();
		this.buffer[ this._addIndex * this.valueSize + 3 ] = 1;

	}

	_setAdditiveIdentityOther() {

		const startIndex = this._origIndex * this.valueSize;
		const targetIndex = this._addIndex * this.valueSize;

		for ( let i = 0; i < this.valueSize; i ++ ) {

			this.buffer[ targetIndex + i ] = this.buffer[ startIndex + i ];

		}

	}


	// mix functions

	_select( buffer, dstOffset, srcOffset, t, stride ) {

		if ( t >= 0.5 ) {

			for ( let i = 0; i !== stride; ++ i ) {

				buffer[ dstOffset + i ] = buffer[ srcOffset + i ];

			}

		}

	}

	_slerp( buffer, dstOffset, srcOffset, t ) {

		Quaternion.slerpFlat( buffer, dstOffset, buffer, dstOffset, buffer, srcOffset, t );

	}

	_slerpAdditive( buffer, dstOffset, srcOffset, t, stride ) {

		const workOffset = this._workIndex * stride;

		// Store result in intermediate buffer offset
		Quaternion.multiplyQuaternionsFlat( buffer, workOffset, buffer, dstOffset, buffer, srcOffset );

		// Slerp to the intermediate result
		Quaternion.slerpFlat( buffer, dstOffset, buffer, dstOffset, buffer, workOffset, t );

	}

	_lerp( buffer, dstOffset, srcOffset, t, stride ) {

		const s = 1 - t;

		for ( let i = 0; i !== stride; ++ i ) {

			const j = dstOffset + i;

			buffer[ j ] = buffer[ j ] * s + buffer[ srcOffset + i ] * t;

		}

	}

	_lerpAdditive( buffer, dstOffset, srcOffset, t, stride ) {

		for ( let i = 0; i !== stride; ++ i ) {

			const j = dstOffset + i;

			buffer[ j ] = buffer[ j ] + buffer[ srcOffset + i ] * t;

		}

	}

}

// Characters [].:/ are reserved for track binding syntax.
const _RESERVED_CHARS_RE = '\\[\\]\\.:\\/';
const _reservedRe = new RegExp( '[' + _RESERVED_CHARS_RE + ']', 'g' );

// Attempts to allow node names from any language. ES5's `\w` regexp matches
// only latin characters, and the unicode \p{L} is not yet supported. So
// instead, we exclude reserved characters and match everything else.
const _wordChar = '[^' + _RESERVED_CHARS_RE + ']';
const _wordCharOrDot = '[^' + _RESERVED_CHARS_RE.replace( '\\.', '' ) + ']';

// Parent directories, delimited by '/' or ':'. Currently unused, but must
// be matched to parse the rest of the track name.
const _directoryRe = /((?:WC+[\/:])*)/.source.replace( 'WC', _wordChar );

// Target node. May contain word characters (a-zA-Z0-9_) and '.' or '-'.
const _nodeRe = /(WCOD+)?/.source.replace( 'WCOD', _wordCharOrDot );

// Object on target node, and accessor. May not contain reserved
// characters. Accessor may contain any character except closing bracket.
const _objectRe = /(?:\.(WC+)(?:\[(.+)\])?)?/.source.replace( 'WC', _wordChar );

// Property and accessor. May not contain reserved characters. Accessor may
// contain any non-bracket characters.
const _propertyRe = /\.(WC+)(?:\[(.+)\])?/.source.replace( 'WC', _wordChar );

const _trackRe = new RegExp( ''
	+ '^'
	+ _directoryRe
	+ _nodeRe
	+ _objectRe
	+ _propertyRe
	+ '$'
);

const _supportedObjectNames = [ 'material', 'materials', 'bones' ];

function Composite( targetGroup, path, optionalParsedPath ) {

	const parsedPath = optionalParsedPath || PropertyBinding.parseTrackName( path );

	this._targetGroup = targetGroup;
	this._bindings = targetGroup.subscribe_( path, parsedPath );

}

Object.assign( Composite.prototype, {

	getValue: function ( array, offset ) {

		this.bind(); // bind all binding

		const firstValidIndex = this._targetGroup.nCachedObjects_,
			binding = this._bindings[ firstValidIndex ];

		// and only call .getValue on the first
		if ( binding !== undefined ) binding.getValue( array, offset );

	},

	setValue: function ( array, offset ) {

		const bindings = this._bindings;

		for ( let i = this._targetGroup.nCachedObjects_, n = bindings.length; i !== n; ++ i ) {

			bindings[ i ].setValue( array, offset );

		}

	},

	bind: function () {

		const bindings = this._bindings;

		for ( let i = this._targetGroup.nCachedObjects_, n = bindings.length; i !== n; ++ i ) {

			bindings[ i ].bind();

		}

	},

	unbind: function () {

		const bindings = this._bindings;

		for ( let i = this._targetGroup.nCachedObjects_, n = bindings.length; i !== n; ++ i ) {

			bindings[ i ].unbind();

		}

	}

} );


function PropertyBinding( rootNode, path, parsedPath ) {

	this.path = path;
	this.parsedPath = parsedPath || PropertyBinding.parseTrackName( path );

	this.node = PropertyBinding.findNode( rootNode, this.parsedPath.nodeName ) || rootNode;

	this.rootNode = rootNode;

}

Object.assign( PropertyBinding, {

	Composite: Composite,

	create: function ( root, path, parsedPath ) {

		if ( ! ( root && root.isAnimationObjectGroup ) ) {

			return new PropertyBinding( root, path, parsedPath );

		} else {

			return new PropertyBinding.Composite( root, path, parsedPath );

		}

	},

	/**
	 * Replaces spaces with underscores and removes unsupported characters from
	 * node names, to ensure compatibility with parseTrackName().
	 *
	 * @param {string} name Node name to be sanitized.
	 * @return {string}
	 */
	sanitizeNodeName: function ( name ) {

		return name.replace( /\s/g, '_' ).replace( _reservedRe, '' );

	},

	parseTrackName: function ( trackName ) {

		const matches = _trackRe.exec( trackName );

		if ( ! matches ) {

			throw new Error( 'PropertyBinding: Cannot parse trackName: ' + trackName );

		}

		const results = {
			// directoryName: matches[ 1 ], // (tschw) currently unused
			nodeName: matches[ 2 ],
			objectName: matches[ 3 ],
			objectIndex: matches[ 4 ],
			propertyName: matches[ 5 ], // required
			propertyIndex: matches[ 6 ]
		};

		const lastDot = results.nodeName && results.nodeName.lastIndexOf( '.' );

		if ( lastDot !== undefined && lastDot !== - 1 ) {

			const objectName = results.nodeName.substring( lastDot + 1 );

			// Object names must be checked against an allowlist. Otherwise, there
			// is no way to parse 'foo.bar.baz': 'baz' must be a property, but
			// 'bar' could be the objectName, or part of a nodeName (which can
			// include '.' characters).
			if ( _supportedObjectNames.indexOf( objectName ) !== - 1 ) {

				results.nodeName = results.nodeName.substring( 0, lastDot );
				results.objectName = objectName;

			}

		}

		if ( results.propertyName === null || results.propertyName.length === 0 ) {

			throw new Error( 'PropertyBinding: can not parse propertyName from trackName: ' + trackName );

		}

		return results;

	},

	findNode: function ( root, nodeName ) {

		if ( ! nodeName || nodeName === '' || nodeName === '.' || nodeName === - 1 || nodeName === root.name || nodeName === root.uuid ) {

			return root;

		}

		// search into skeleton bones.
		if ( root.skeleton ) {

			const bone = root.skeleton.getBoneByName( nodeName );

			if ( bone !== undefined ) {

				return bone;

			}

		}

		// search into node subtree.
		if ( root.children ) {

			const searchNodeSubtree = function ( children ) {

				for ( let i = 0; i < children.length; i ++ ) {

					const childNode = children[ i ];

					if ( childNode.name === nodeName || childNode.uuid === nodeName ) {

						return childNode;

					}

					const result = searchNodeSubtree( childNode.children );

					if ( result ) return result;

				}

				return null;

			};

			const subTreeNode = searchNodeSubtree( root.children );

			if ( subTreeNode ) {

				return subTreeNode;

			}

		}

		return null;

	}

} );

Object.assign( PropertyBinding.prototype, { // prototype, continued

	// these are used to "bind" a nonexistent property
	_getValue_unavailable: function () {},
	_setValue_unavailable: function () {},

	BindingType: {
		Direct: 0,
		EntireArray: 1,
		ArrayElement: 2,
		HasFromToArray: 3
	},

	Versioning: {
		None: 0,
		NeedsUpdate: 1,
		MatrixWorldNeedsUpdate: 2
	},

	GetterByBindingType: [

		function getValue_direct( buffer, offset ) {

			buffer[ offset ] = this.node[ this.propertyName ];

		},

		function getValue_array( buffer, offset ) {

			const source = this.resolvedProperty;

			for ( let i = 0, n = source.length; i !== n; ++ i ) {

				buffer[ offset ++ ] = source[ i ];

			}

		},

		function getValue_arrayElement( buffer, offset ) {

			buffer[ offset ] = this.resolvedProperty[ this.propertyIndex ];

		},

		function getValue_toArray( buffer, offset ) {

			this.resolvedProperty.toArray( buffer, offset );

		}

	],

	SetterByBindingTypeAndVersioning: [

		[
			// Direct

			function setValue_direct( buffer, offset ) {

				this.targetObject[ this.propertyName ] = buffer[ offset ];

			},

			function setValue_direct_setNeedsUpdate( buffer, offset ) {

				this.targetObject[ this.propertyName ] = buffer[ offset ];
				this.targetObject.needsUpdate = true;

			},

			function setValue_direct_setMatrixWorldNeedsUpdate( buffer, offset ) {

				this.targetObject[ this.propertyName ] = buffer[ offset ];
				this.targetObject.matrixWorldNeedsUpdate = true;

			}

		], [

			// EntireArray

			function setValue_array( buffer, offset ) {

				const dest = this.resolvedProperty;

				for ( let i = 0, n = dest.length; i !== n; ++ i ) {

					dest[ i ] = buffer[ offset ++ ];

				}

			},

			function setValue_array_setNeedsUpdate( buffer, offset ) {

				const dest = this.resolvedProperty;

				for ( let i = 0, n = dest.length; i !== n; ++ i ) {

					dest[ i ] = buffer[ offset ++ ];

				}

				this.targetObject.needsUpdate = true;

			},

			function setValue_array_setMatrixWorldNeedsUpdate( buffer, offset ) {

				const dest = this.resolvedProperty;

				for ( let i = 0, n = dest.length; i !== n; ++ i ) {

					dest[ i ] = buffer[ offset ++ ];

				}

				this.targetObject.matrixWorldNeedsUpdate = true;

			}

		], [

			// ArrayElement

			function setValue_arrayElement( buffer, offset ) {

				this.resolvedProperty[ this.propertyIndex ] = buffer[ offset ];

			},

			function setValue_arrayElement_setNeedsUpdate( buffer, offset ) {

				this.resolvedProperty[ this.propertyIndex ] = buffer[ offset ];
				this.targetObject.needsUpdate = true;

			},

			function setValue_arrayElement_setMatrixWorldNeedsUpdate( buffer, offset ) {

				this.resolvedProperty[ this.propertyIndex ] = buffer[ offset ];
				this.targetObject.matrixWorldNeedsUpdate = true;

			}

		], [

			// HasToFromArray

			function setValue_fromArray( buffer, offset ) {

				this.resolvedProperty.fromArray( buffer, offset );

			},

			function setValue_fromArray_setNeedsUpdate( buffer, offset ) {

				this.resolvedProperty.fromArray( buffer, offset );
				this.targetObject.needsUpdate = true;

			},

			function setValue_fromArray_setMatrixWorldNeedsUpdate( buffer, offset ) {

				this.resolvedProperty.fromArray( buffer, offset );
				this.targetObject.matrixWorldNeedsUpdate = true;

			}

		]

	],

	getValue: function getValue_unbound( targetArray, offset ) {

		this.bind();
		this.getValue( targetArray, offset );

		// Note: This class uses a State pattern on a per-method basis:
		// 'bind' sets 'this.getValue' / 'setValue' and shadows the
		// prototype version of these methods with one that represents
		// the bound state. When the property is not found, the methods
		// become no-ops.

	},

	setValue: function getValue_unbound( sourceArray, offset ) {

		this.bind();
		this.setValue( sourceArray, offset );

	},

	// create getter / setter pair for a property in the scene graph
	bind: function () {

		let targetObject = this.node;
		const parsedPath = this.parsedPath;

		const objectName = parsedPath.objectName;
		const propertyName = parsedPath.propertyName;
		let propertyIndex = parsedPath.propertyIndex;

		if ( ! targetObject ) {

			targetObject = PropertyBinding.findNode( this.rootNode, parsedPath.nodeName ) || this.rootNode;

			this.node = targetObject;

		}

		// set fail state so we can just 'return' on error
		this.getValue = this._getValue_unavailable;
		this.setValue = this._setValue_unavailable;

		// ensure there is a value node
		if ( ! targetObject ) {

			console.error( 'THREE.PropertyBinding: Trying to update node for track: ' + this.path + ' but it wasn\'t found.' );
			return;

		}

		if ( objectName ) {

			let objectIndex = parsedPath.objectIndex;

			// special cases were we need to reach deeper into the hierarchy to get the face materials....
			switch ( objectName ) {

				case 'materials':

					if ( ! targetObject.material ) {

						console.error( 'THREE.PropertyBinding: Can not bind to material as node does not have a material.', this );
						return;

					}

					if ( ! targetObject.material.materials ) {

						console.error( 'THREE.PropertyBinding: Can not bind to material.materials as node.material does not have a materials array.', this );
						return;

					}

					targetObject = targetObject.material.materials;

					break;

				case 'bones':

					if ( ! targetObject.skeleton ) {

						console.error( 'THREE.PropertyBinding: Can not bind to bones as node does not have a skeleton.', this );
						return;

					}

					// potential future optimization: skip this if propertyIndex is already an integer
					// and convert the integer string to a true integer.

					targetObject = targetObject.skeleton.bones;

					// support resolving morphTarget names into indices.
					for ( let i = 0; i < targetObject.length; i ++ ) {

						if ( targetObject[ i ].name === objectIndex ) {

							objectIndex = i;
							break;

						}

					}

					break;

				default:

					if ( targetObject[ objectName ] === undefined ) {

						console.error( 'THREE.PropertyBinding: Can not bind to objectName of node undefined.', this );
						return;

					}

					targetObject = targetObject[ objectName ];

			}


			if ( objectIndex !== undefined ) {

				if ( targetObject[ objectIndex ] === undefined ) {

					console.error( 'THREE.PropertyBinding: Trying to bind to objectIndex of objectName, but is undefined.', this, targetObject );
					return;

				}

				targetObject = targetObject[ objectIndex ];

			}

		}

		// resolve property
		const nodeProperty = targetObject[ propertyName ];

		if ( nodeProperty === undefined ) {

			const nodeName = parsedPath.nodeName;

			console.error( 'THREE.PropertyBinding: Trying to update property for track: ' + nodeName +
				'.' + propertyName + ' but it wasn\'t found.', targetObject );
			return;

		}

		// determine versioning scheme
		let versioning = this.Versioning.None;

		this.targetObject = targetObject;

		if ( targetObject.needsUpdate !== undefined ) { // material

			versioning = this.Versioning.NeedsUpdate;

		} else if ( targetObject.matrixWorldNeedsUpdate !== undefined ) { // node transform

			versioning = this.Versioning.MatrixWorldNeedsUpdate;

		}

		// determine how the property gets bound
		let bindingType = this.BindingType.Direct;

		if ( propertyIndex !== undefined ) {

			// access a sub element of the property array (only primitives are supported right now)

			if ( propertyName === 'morphTargetInfluences' ) {

				// potential optimization, skip this if propertyIndex is already an integer, and convert the integer string to a true integer.

				// support resolving morphTarget names into indices.
				if ( ! targetObject.geometry ) {

					console.error( 'THREE.PropertyBinding: Can not bind to morphTargetInfluences because node does not have a geometry.', this );
					return;

				}

				if ( targetObject.geometry.isBufferGeometry ) {

					if ( ! targetObject.geometry.morphAttributes ) {

						console.error( 'THREE.PropertyBinding: Can not bind to morphTargetInfluences because node does not have a geometry.morphAttributes.', this );
						return;

					}

					if ( targetObject.morphTargetDictionary[ propertyIndex ] !== undefined ) {

						propertyIndex = targetObject.morphTargetDictionary[ propertyIndex ];

					}


				} else {

					console.error( 'THREE.PropertyBinding: Can not bind to morphTargetInfluences on THREE.Geometry. Use THREE.BufferGeometry instead.', this );
					return;

				}

			}

			bindingType = this.BindingType.ArrayElement;

			this.resolvedProperty = nodeProperty;
			this.propertyIndex = propertyIndex;

		} else if ( nodeProperty.fromArray !== undefined && nodeProperty.toArray !== undefined ) {

			// must use copy for Object3D.Euler/Quaternion

			bindingType = this.BindingType.HasFromToArray;

			this.resolvedProperty = nodeProperty;

		} else if ( Array.isArray( nodeProperty ) ) {

			bindingType = this.BindingType.EntireArray;

			this.resolvedProperty = nodeProperty;

		} else {

			this.propertyName = propertyName;

		}

		// select getter / setter
		this.getValue = this.GetterByBindingType[ bindingType ];
		this.setValue = this.SetterByBindingTypeAndVersioning[ bindingType ][ versioning ];

	},

	unbind: function () {

		this.node = null;

		// back to the prototype version of getValue / setValue
		// note: avoiding to mutate the shape of 'this' via 'delete'
		this.getValue = this._getValue_unbound;
		this.setValue = this._setValue_unbound;

	}

} );

// DECLARE ALIAS AFTER assign prototype
Object.assign( PropertyBinding.prototype, {

	// initial state of these methods that calls 'bind'
	_getValue_unbound: PropertyBinding.prototype.getValue,
	_setValue_unbound: PropertyBinding.prototype.setValue,

} );

/**
 *
 * A group of objects that receives a shared animation state.
 *
 * Usage:
 *
 *  - Add objects you would otherwise pass as 'root' to the
 *    constructor or the .clipAction method of AnimationMixer.
 *
 *  - Instead pass this object as 'root'.
 *
 *  - You can also add and remove objects later when the mixer
 *    is running.
 *
 * Note:
 *
 *    Objects of this class appear as one object to the mixer,
 *    so cache control of the individual objects must be done
 *    on the group.
 *
 * Limitation:
 *
 *  - The animated properties must be compatible among the
 *    all objects in the group.
 *
 *  - A single property can either be controlled through a
 *    target group or directly, but not both.
 */

class AnimationObjectGroup {

	constructor() {

		this.uuid = MathUtils.generateUUID();

		// cached objects followed by the active ones
		this._objects = Array.prototype.slice.call( arguments );

		this.nCachedObjects_ = 0; // threshold
		// note: read by PropertyBinding.Composite

		const indices = {};
		this._indicesByUUID = indices; // for bookkeeping

		for ( let i = 0, n = arguments.length; i !== n; ++ i ) {

			indices[ arguments[ i ].uuid ] = i;

		}

		this._paths = []; // inside: string
		this._parsedPaths = []; // inside: { we don't care, here }
		this._bindings = []; // inside: Array< PropertyBinding >
		this._bindingsIndicesByPath = {}; // inside: indices in these arrays

		const scope = this;

		this.stats = {

			objects: {
				get total() {

					return scope._objects.length;

				},
				get inUse() {

					return this.total - scope.nCachedObjects_;

				}
			},
			get bindingsPerObject() {

				return scope._bindings.length;

			}

		};

	}

	add() {

		const objects = this._objects,
			indicesByUUID = this._indicesByUUID,
			paths = this._paths,
			parsedPaths = this._parsedPaths,
			bindings = this._bindings,
			nBindings = bindings.length;

		let knownObject = undefined,
			nObjects = objects.length,
			nCachedObjects = this.nCachedObjects_;

		for ( let i = 0, n = arguments.length; i !== n; ++ i ) {

			const object = arguments[ i ],
				uuid = object.uuid;
			let index = indicesByUUID[ uuid ];

			if ( index === undefined ) {

				// unknown object -> add it to the ACTIVE region

				index = nObjects ++;
				indicesByUUID[ uuid ] = index;
				objects.push( object );

				// accounting is done, now do the same for all bindings

				for ( let j = 0, m = nBindings; j !== m; ++ j ) {

					bindings[ j ].push( new PropertyBinding( object, paths[ j ], parsedPaths[ j ] ) );

				}

			} else if ( index < nCachedObjects ) {

				knownObject = objects[ index ];

				// move existing object to the ACTIVE region

				const firstActiveIndex = -- nCachedObjects,
					lastCachedObject = objects[ firstActiveIndex ];

				indicesByUUID[ lastCachedObject.uuid ] = index;
				objects[ index ] = lastCachedObject;

				indicesByUUID[ uuid ] = firstActiveIndex;
				objects[ firstActiveIndex ] = object;

				// accounting is done, now do the same for all bindings

				for ( let j = 0, m = nBindings; j !== m; ++ j ) {

					const bindingsForPath = bindings[ j ],
						lastCached = bindingsForPath[ firstActiveIndex ];

					let binding = bindingsForPath[ index ];

					bindingsForPath[ index ] = lastCached;

					if ( binding === undefined ) {

						// since we do not bother to create new bindings
						// for objects that are cached, the binding may
						// or may not exist

						binding = new PropertyBinding( object, paths[ j ], parsedPaths[ j ] );

					}

					bindingsForPath[ firstActiveIndex ] = binding;

				}

			} else if ( objects[ index ] !== knownObject ) {

				console.error( 'THREE.AnimationObjectGroup: Different objects with the same UUID ' +
					'detected. Clean the caches or recreate your infrastructure when reloading scenes.' );

			} // else the object is already where we want it to be

		} // for arguments

		this.nCachedObjects_ = nCachedObjects;

	}

	remove() {

		const objects = this._objects,
			indicesByUUID = this._indicesByUUID,
			bindings = this._bindings,
			nBindings = bindings.length;

		let nCachedObjects = this.nCachedObjects_;

		for ( let i = 0, n = arguments.length; i !== n; ++ i ) {

			const object = arguments[ i ],
				uuid = object.uuid,
				index = indicesByUUID[ uuid ];

			if ( index !== undefined && index >= nCachedObjects ) {

				// move existing object into the CACHED region

				const lastCachedIndex = nCachedObjects ++,
					firstActiveObject = objects[ lastCachedIndex ];

				indicesByUUID[ firstActiveObject.uuid ] = index;
				objects[ index ] = firstActiveObject;

				indicesByUUID[ uuid ] = lastCachedIndex;
				objects[ lastCachedIndex ] = object;

				// accounting is done, now do the same for all bindings

				for ( let j = 0, m = nBindings; j !== m; ++ j ) {

					const bindingsForPath = bindings[ j ],
						firstActive = bindingsForPath[ lastCachedIndex ],
						binding = bindingsForPath[ index ];

					bindingsForPath[ index ] = firstActive;
					bindingsForPath[ lastCachedIndex ] = binding;

				}

			}

		} // for arguments

		this.nCachedObjects_ = nCachedObjects;

	}

	// remove & forget
	uncache() {

		const objects = this._objects,
			indicesByUUID = this._indicesByUUID,
			bindings = this._bindings,
			nBindings = bindings.length;

		let nCachedObjects = this.nCachedObjects_,
			nObjects = objects.length;

		for ( let i = 0, n = arguments.length; i !== n; ++ i ) {

			const object = arguments[ i ],
				uuid = object.uuid,
				index = indicesByUUID[ uuid ];

			if ( index !== undefined ) {

				delete indicesByUUID[ uuid ];

				if ( index < nCachedObjects ) {

					// object is cached, shrink the CACHED region

					const firstActiveIndex = -- nCachedObjects,
						lastCachedObject = objects[ firstActiveIndex ],
						lastIndex = -- nObjects,
						lastObject = objects[ lastIndex ];

					// last cached object takes this object's place
					indicesByUUID[ lastCachedObject.uuid ] = index;
					objects[ index ] = lastCachedObject;

					// last object goes to the activated slot and pop
					indicesByUUID[ lastObject.uuid ] = firstActiveIndex;
					objects[ firstActiveIndex ] = lastObject;
					objects.pop();

					// accounting is done, now do the same for all bindings

					for ( let j = 0, m = nBindings; j !== m; ++ j ) {

						const bindingsForPath = bindings[ j ],
							lastCached = bindingsForPath[ firstActiveIndex ],
							last = bindingsForPath[ lastIndex ];

						bindingsForPath[ index ] = lastCached;
						bindingsForPath[ firstActiveIndex ] = last;
						bindingsForPath.pop();

					}

				} else {

					// object is active, just swap with the last and pop

					const lastIndex = -- nObjects,
						lastObject = objects[ lastIndex ];

					if ( lastIndex > 0 ) {

						indicesByUUID[ lastObject.uuid ] = index;

					}

					objects[ index ] = lastObject;
					objects.pop();

					// accounting is done, now do the same for all bindings

					for ( let j = 0, m = nBindings; j !== m; ++ j ) {

						const bindingsForPath = bindings[ j ];

						bindingsForPath[ index ] = bindingsForPath[ lastIndex ];
						bindingsForPath.pop();

					}

				} // cached or active

			} // if object is known

		} // for arguments

		this.nCachedObjects_ = nCachedObjects;

	}

	// Internal interface used by befriended PropertyBinding.Composite:

	subscribe_( path, parsedPath ) {

		// returns an array of bindings for the given path that is changed
		// according to the contained objects in the group

		const indicesByPath = this._bindingsIndicesByPath;
		let index = indicesByPath[ path ];
		const bindings = this._bindings;

		if ( index !== undefined ) return bindings[ index ];

		const paths = this._paths,
			parsedPaths = this._parsedPaths,
			objects = this._objects,
			nObjects = objects.length,
			nCachedObjects = this.nCachedObjects_,
			bindingsForPath = new Array( nObjects );

		index = bindings.length;

		indicesByPath[ path ] = index;

		paths.push( path );
		parsedPaths.push( parsedPath );
		bindings.push( bindingsForPath );

		for ( let i = nCachedObjects, n = objects.length; i !== n; ++ i ) {

			const object = objects[ i ];
			bindingsForPath[ i ] = new PropertyBinding( object, path, parsedPath );

		}

		return bindingsForPath;

	}

	unsubscribe_( path ) {

		// tells the group to forget about a property path and no longer
		// update the array previously obtained with 'subscribe_'

		const indicesByPath = this._bindingsIndicesByPath,
			index = indicesByPath[ path ];

		if ( index !== undefined ) {

			const paths = this._paths,
				parsedPaths = this._parsedPaths,
				bindings = this._bindings,
				lastBindingsIndex = bindings.length - 1,
				lastBindings = bindings[ lastBindingsIndex ],
				lastBindingsPath = path[ lastBindingsIndex ];

			indicesByPath[ lastBindingsPath ] = index;

			bindings[ index ] = lastBindings;
			bindings.pop();

			parsedPaths[ index ] = parsedPaths[ lastBindingsIndex ];
			parsedPaths.pop();

			paths[ index ] = paths[ lastBindingsIndex ];
			paths.pop();

		}

	}

}

AnimationObjectGroup.prototype.isAnimationObjectGroup = true;

class AnimationAction {

	constructor( mixer, clip, localRoot = null, blendMode = clip.blendMode ) {

		this._mixer = mixer;
		this._clip = clip;
		this._localRoot = localRoot;
		this.blendMode = blendMode;

		const tracks = clip.tracks,
			nTracks = tracks.length,
			interpolants = new Array( nTracks );

		const interpolantSettings = {
			endingStart: ZeroCurvatureEnding,
			endingEnd: ZeroCurvatureEnding
		};

		for ( let i = 0; i !== nTracks; ++ i ) {

			const interpolant = tracks[ i ].createInterpolant( null );
			interpolants[ i ] = interpolant;
			interpolant.settings = interpolantSettings;

		}

		this._interpolantSettings = interpolantSettings;

		this._interpolants = interpolants; // bound by the mixer

		// inside: PropertyMixer (managed by the mixer)
		this._propertyBindings = new Array( nTracks );

		this._cacheIndex = null; // for the memory manager
		this._byClipCacheIndex = null; // for the memory manager

		this._timeScaleInterpolant = null;
		this._weightInterpolant = null;

		this.loop = LoopRepeat;
		this._loopCount = - 1;

		// global mixer time when the action is to be started
		// it's set back to 'null' upon start of the action
		this._startTime = null;

		// scaled local time of the action
		// gets clamped or wrapped to 0..clip.duration according to loop
		this.time = 0;

		this.timeScale = 1;
		this._effectiveTimeScale = 1;

		this.weight = 1;
		this._effectiveWeight = 1;

		this.repetitions = Infinity; // no. of repetitions when looping

		this.paused = false; // true -> zero effective time scale
		this.enabled = true; // false -> zero effective weight

		this.clampWhenFinished = false;// keep feeding the last frame?

		this.zeroSlopeAtStart = true;// for smooth interpolation w/o separate
		this.zeroSlopeAtEnd = true;// clips for start, loop and end

	}

	// State & Scheduling

	play() {

		this._mixer._activateAction( this );

		return this;

	}

	stop() {

		this._mixer._deactivateAction( this );

		return this.reset();

	}

	reset() {

		this.paused = false;
		this.enabled = true;

		this.time = 0; // restart clip
		this._loopCount = - 1;// forget previous loops
		this._startTime = null;// forget scheduling

		return this.stopFading().stopWarping();

	}

	isRunning() {

		return this.enabled && ! this.paused && this.timeScale !== 0 &&
			this._startTime === null && this._mixer._isActiveAction( this );

	}

	// return true when play has been called
	isScheduled() {

		return this._mixer._isActiveAction( this );

	}

	startAt( time ) {

		this._startTime = time;

		return this;

	}

	setLoop( mode, repetitions ) {

		this.loop = mode;
		this.repetitions = repetitions;

		return this;

	}

	// Weight

	// set the weight stopping any scheduled fading
	// although .enabled = false yields an effective weight of zero, this
	// method does *not* change .enabled, because it would be confusing
	setEffectiveWeight( weight ) {

		this.weight = weight;

		// note: same logic as when updated at runtime
		this._effectiveWeight = this.enabled ? weight : 0;

		return this.stopFading();

	}

	// return the weight considering fading and .enabled
	getEffectiveWeight() {

		return this._effectiveWeight;

	}

	fadeIn( duration ) {

		return this._scheduleFading( duration, 0, 1 );

	}

	fadeOut( duration ) {

		return this._scheduleFading( duration, 1, 0 );

	}

	crossFadeFrom( fadeOutAction, duration, warp ) {

		fadeOutAction.fadeOut( duration );
		this.fadeIn( duration );

		if ( warp ) {

			const fadeInDuration = this._clip.duration,
				fadeOutDuration = fadeOutAction._clip.duration,

				startEndRatio = fadeOutDuration / fadeInDuration,
				endStartRatio = fadeInDuration / fadeOutDuration;

			fadeOutAction.warp( 1.0, startEndRatio, duration );
			this.warp( endStartRatio, 1.0, duration );

		}

		return this;

	}

	crossFadeTo( fadeInAction, duration, warp ) {

		return fadeInAction.crossFadeFrom( this, duration, warp );

	}

	stopFading() {

		const weightInterpolant = this._weightInterpolant;

		if ( weightInterpolant !== null ) {

			this._weightInterpolant = null;
			this._mixer._takeBackControlInterpolant( weightInterpolant );

		}

		return this;

	}

	// Time Scale Control

	// set the time scale stopping any scheduled warping
	// although .paused = true yields an effective time scale of zero, this
	// method does *not* change .paused, because it would be confusing
	setEffectiveTimeScale( timeScale ) {

		this.timeScale = timeScale;
		this._effectiveTimeScale = this.paused ? 0 : timeScale;

		return this.stopWarping();

	}

	// return the time scale considering warping and .paused
	getEffectiveTimeScale() {

		return this._effectiveTimeScale;

	}

	setDuration( duration ) {

		this.timeScale = this._clip.duration / duration;

		return this.stopWarping();

	}

	syncWith( action ) {

		this.time = action.time;
		this.timeScale = action.timeScale;

		return this.stopWarping();

	}

	halt( duration ) {

		return this.warp( this._effectiveTimeScale, 0, duration );

	}

	warp( startTimeScale, endTimeScale, duration ) {

		const mixer = this._mixer,
			now = mixer.time,
			timeScale = this.timeScale;

		let interpolant = this._timeScaleInterpolant;

		if ( interpolant === null ) {

			interpolant = mixer._lendControlInterpolant();
			this._timeScaleInterpolant = interpolant;

		}

		const times = interpolant.parameterPositions,
			values = interpolant.sampleValues;

		times[ 0 ] = now;
		times[ 1 ] = now + duration;

		values[ 0 ] = startTimeScale / timeScale;
		values[ 1 ] = endTimeScale / timeScale;

		return this;

	}

	stopWarping() {

		const timeScaleInterpolant = this._timeScaleInterpolant;

		if ( timeScaleInterpolant !== null ) {

			this._timeScaleInterpolant = null;
			this._mixer._takeBackControlInterpolant( timeScaleInterpolant );

		}

		return this;

	}

	// Object Accessors

	getMixer() {

		return this._mixer;

	}

	getClip() {

		return this._clip;

	}

	getRoot() {

		return this._localRoot || this._mixer._root;

	}

	// Interna

	_update( time, deltaTime, timeDirection, accuIndex ) {

		// called by the mixer

		if ( ! this.enabled ) {

			// call ._updateWeight() to update ._effectiveWeight

			this._updateWeight( time );
			return;

		}

		const startTime = this._startTime;

		if ( startTime !== null ) {

			// check for scheduled start of action

			const timeRunning = ( time - startTime ) * timeDirection;
			if ( timeRunning < 0 || timeDirection === 0 ) {

				return; // yet to come / don't decide when delta = 0

			}

			// start

			this._startTime = null; // unschedule
			deltaTime = timeDirection * timeRunning;

		}

		// apply time scale and advance time

		deltaTime *= this._updateTimeScale( time );
		const clipTime = this._updateTime( deltaTime );

		// note: _updateTime may disable the action resulting in
		// an effective weight of 0

		const weight = this._updateWeight( time );

		if ( weight > 0 ) {

			const interpolants = this._interpolants;
			const propertyMixers = this._propertyBindings;

			switch ( this.blendMode ) {

				case AdditiveAnimationBlendMode:

					for ( let j = 0, m = interpolants.length; j !== m; ++ j ) {

						interpolants[ j ].evaluate( clipTime );
						propertyMixers[ j ].accumulateAdditive( weight );

					}

					break;

				case NormalAnimationBlendMode:
				default:

					for ( let j = 0, m = interpolants.length; j !== m; ++ j ) {

						interpolants[ j ].evaluate( clipTime );
						propertyMixers[ j ].accumulate( accuIndex, weight );

					}

			}

		}

	}

	_updateWeight( time ) {

		let weight = 0;

		if ( this.enabled ) {

			weight = this.weight;
			const interpolant = this._weightInterpolant;

			if ( interpolant !== null ) {

				const interpolantValue = interpolant.evaluate( time )[ 0 ];

				weight *= interpolantValue;

				if ( time > interpolant.parameterPositions[ 1 ] ) {

					this.stopFading();

					if ( interpolantValue === 0 ) {

						// faded out, disable
						this.enabled = false;

					}

				}

			}

		}

		this._effectiveWeight = weight;
		return weight;

	}

	_updateTimeScale( time ) {

		let timeScale = 0;

		if ( ! this.paused ) {

			timeScale = this.timeScale;

			const interpolant = this._timeScaleInterpolant;

			if ( interpolant !== null ) {

				const interpolantValue = interpolant.evaluate( time )[ 0 ];

				timeScale *= interpolantValue;

				if ( time > interpolant.parameterPositions[ 1 ] ) {

					this.stopWarping();

					if ( timeScale === 0 ) {

						// motion has halted, pause
						this.paused = true;

					} else {

						// warp done - apply final time scale
						this.timeScale = timeScale;

					}

				}

			}

		}

		this._effectiveTimeScale = timeScale;
		return timeScale;

	}

	_updateTime( deltaTime ) {

		const duration = this._clip.duration;
		const loop = this.loop;

		let time = this.time + deltaTime;
		let loopCount = this._loopCount;

		const pingPong = ( loop === LoopPingPong );

		if ( deltaTime === 0 ) {

			if ( loopCount === - 1 ) return time;

			return ( pingPong && ( loopCount & 1 ) === 1 ) ? duration - time : time;

		}

		if ( loop === LoopOnce ) {

			if ( loopCount === - 1 ) {

				// just started

				this._loopCount = 0;
				this._setEndings( true, true, false );

			}

			handle_stop: {

				if ( time >= duration ) {

					time = duration;

				} else if ( time < 0 ) {

					time = 0;

				} else {

					this.time = time;

					break handle_stop;

				}

				if ( this.clampWhenFinished ) this.paused = true;
				else this.enabled = false;

				this.time = time;

				this._mixer.dispatchEvent( {
					type: 'finished', action: this,
					direction: deltaTime < 0 ? - 1 : 1
				} );

			}

		} else { // repetitive Repeat or PingPong

			if ( loopCount === - 1 ) {

				// just started

				if ( deltaTime >= 0 ) {

					loopCount = 0;

					this._setEndings( true, this.repetitions === 0, pingPong );

				} else {

					// when looping in reverse direction, the initial
					// transition through zero counts as a repetition,
					// so leave loopCount at -1

					this._setEndings( this.repetitions === 0, true, pingPong );

				}

			}

			if ( time >= duration || time < 0 ) {

				// wrap around

				const loopDelta = Math.floor( time / duration ); // signed
				time -= duration * loopDelta;

				loopCount += Math.abs( loopDelta );

				const pending = this.repetitions - loopCount;

				if ( pending <= 0 ) {

					// have to stop (switch state, clamp time, fire event)

					if ( this.clampWhenFinished ) this.paused = true;
					else this.enabled = false;

					time = deltaTime > 0 ? duration : 0;

					this.time = time;

					this._mixer.dispatchEvent( {
						type: 'finished', action: this,
						direction: deltaTime > 0 ? 1 : - 1
					} );

				} else {

					// keep running

					if ( pending === 1 ) {

						// entering the last round

						const atStart = deltaTime < 0;
						this._setEndings( atStart, ! atStart, pingPong );

					} else {

						this._setEndings( false, false, pingPong );

					}

					this._loopCount = loopCount;

					this.time = time;

					this._mixer.dispatchEvent( {
						type: 'loop', action: this, loopDelta: loopDelta
					} );

				}

			} else {

				this.time = time;

			}

			if ( pingPong && ( loopCount & 1 ) === 1 ) {

				// invert time for the "pong round"

				return duration - time;

			}

		}

		return time;

	}

	_setEndings( atStart, atEnd, pingPong ) {

		const settings = this._interpolantSettings;

		if ( pingPong ) {

			settings.endingStart = ZeroSlopeEnding;
			settings.endingEnd = ZeroSlopeEnding;

		} else {

			// assuming for LoopOnce atStart == atEnd == true

			if ( atStart ) {

				settings.endingStart = this.zeroSlopeAtStart ? ZeroSlopeEnding : ZeroCurvatureEnding;

			} else {

				settings.endingStart = WrapAroundEnding;

			}

			if ( atEnd ) {

				settings.endingEnd = this.zeroSlopeAtEnd ? ZeroSlopeEnding : ZeroCurvatureEnding;

			} else {

				settings.endingEnd 	 = WrapAroundEnding;

			}

		}

	}

	_scheduleFading( duration, weightNow, weightThen ) {

		const mixer = this._mixer, now = mixer.time;
		let interpolant = this._weightInterpolant;

		if ( interpolant === null ) {

			interpolant = mixer._lendControlInterpolant();
			this._weightInterpolant = interpolant;

		}

		const times = interpolant.parameterPositions,
			values = interpolant.sampleValues;

		times[ 0 ] = now;
		values[ 0 ] = weightNow;
		times[ 1 ] = now + duration;
		values[ 1 ] = weightThen;

		return this;

	}

}

class AnimationMixer extends EventDispatcher {

	constructor( root ) {

		super();

		this._root = root;
		this._initMemoryManager();
		this._accuIndex = 0;
		this.time = 0;
		this.timeScale = 1.0;

	}

	_bindAction( action, prototypeAction ) {

		const root = action._localRoot || this._root,
			tracks = action._clip.tracks,
			nTracks = tracks.length,
			bindings = action._propertyBindings,
			interpolants = action._interpolants,
			rootUuid = root.uuid,
			bindingsByRoot = this._bindingsByRootAndName;

		let bindingsByName = bindingsByRoot[ rootUuid ];

		if ( bindingsByName === undefined ) {

			bindingsByName = {};
			bindingsByRoot[ rootUuid ] = bindingsByName;

		}

		for ( let i = 0; i !== nTracks; ++ i ) {

			const track = tracks[ i ],
				trackName = track.name;

			let binding = bindingsByName[ trackName ];

			if ( binding !== undefined ) {

				bindings[ i ] = binding;

			} else {

				binding = bindings[ i ];

				if ( binding !== undefined ) {

					// existing binding, make sure the cache knows

					if ( binding._cacheIndex === null ) {

						++ binding.referenceCount;
						this._addInactiveBinding( binding, rootUuid, trackName );

					}

					continue;

				}

				const path = prototypeAction && prototypeAction.
					_propertyBindings[ i ].binding.parsedPath;

				binding = new PropertyMixer(
					PropertyBinding.create( root, trackName, path ),
					track.ValueTypeName, track.getValueSize() );

				++ binding.referenceCount;
				this._addInactiveBinding( binding, rootUuid, trackName );

				bindings[ i ] = binding;

			}

			interpolants[ i ].resultBuffer = binding.buffer;

		}

	}

	_activateAction( action ) {

		if ( ! this._isActiveAction( action ) ) {

			if ( action._cacheIndex === null ) {

				// this action has been forgotten by the cache, but the user
				// appears to be still using it -> rebind

				const rootUuid = ( action._localRoot || this._root ).uuid,
					clipUuid = action._clip.uuid,
					actionsForClip = this._actionsByClip[ clipUuid ];

				this._bindAction( action,
					actionsForClip && actionsForClip.knownActions[ 0 ] );

				this._addInactiveAction( action, clipUuid, rootUuid );

			}

			const bindings = action._propertyBindings;

			// increment reference counts / sort out state
			for ( let i = 0, n = bindings.length; i !== n; ++ i ) {

				const binding = bindings[ i ];

				if ( binding.useCount ++ === 0 ) {

					this._lendBinding( binding );
					binding.saveOriginalState();

				}

			}

			this._lendAction( action );

		}

	}

	_deactivateAction( action ) {

		if ( this._isActiveAction( action ) ) {

			const bindings = action._propertyBindings;

			// decrement reference counts / sort out state
			for ( let i = 0, n = bindings.length; i !== n; ++ i ) {

				const binding = bindings[ i ];

				if ( -- binding.useCount === 0 ) {

					binding.restoreOriginalState();
					this._takeBackBinding( binding );

				}

			}

			this._takeBackAction( action );

		}

	}

	// Memory manager

	_initMemoryManager() {

		this._actions = []; // 'nActiveActions' followed by inactive ones
		this._nActiveActions = 0;

		this._actionsByClip = {};
		// inside:
		// {
		// 	knownActions: Array< AnimationAction > - used as prototypes
		// 	actionByRoot: AnimationAction - lookup
		// }


		this._bindings = []; // 'nActiveBindings' followed by inactive ones
		this._nActiveBindings = 0;

		this._bindingsByRootAndName = {}; // inside: Map< name, PropertyMixer >


		this._controlInterpolants = []; // same game as above
		this._nActiveControlInterpolants = 0;

		const scope = this;

		this.stats = {

			actions: {
				get total() {

					return scope._actions.length;

				},
				get inUse() {

					return scope._nActiveActions;

				}
			},
			bindings: {
				get total() {

					return scope._bindings.length;

				},
				get inUse() {

					return scope._nActiveBindings;

				}
			},
			controlInterpolants: {
				get total() {

					return scope._controlInterpolants.length;

				},
				get inUse() {

					return scope._nActiveControlInterpolants;

				}
			}

		};

	}

	// Memory management for AnimationAction objects

	_isActiveAction( action ) {

		const index = action._cacheIndex;
		return index !== null && index < this._nActiveActions;

	}

	_addInactiveAction( action, clipUuid, rootUuid ) {

		const actions = this._actions,
			actionsByClip = this._actionsByClip;

		let actionsForClip = actionsByClip[ clipUuid ];

		if ( actionsForClip === undefined ) {

			actionsForClip = {

				knownActions: [ action ],
				actionByRoot: {}

			};

			action._byClipCacheIndex = 0;

			actionsByClip[ clipUuid ] = actionsForClip;

		} else {

			const knownActions = actionsForClip.knownActions;

			action._byClipCacheIndex = knownActions.length;
			knownActions.push( action );

		}

		action._cacheIndex = actions.length;
		actions.push( action );

		actionsForClip.actionByRoot[ rootUuid ] = action;

	}

	_removeInactiveAction( action ) {

		const actions = this._actions,
			lastInactiveAction = actions[ actions.length - 1 ],
			cacheIndex = action._cacheIndex;

		lastInactiveAction._cacheIndex = cacheIndex;
		actions[ cacheIndex ] = lastInactiveAction;
		actions.pop();

		action._cacheIndex = null;


		const clipUuid = action._clip.uuid,
			actionsByClip = this._actionsByClip,
			actionsForClip = actionsByClip[ clipUuid ],
			knownActionsForClip = actionsForClip.knownActions,

			lastKnownAction =
				knownActionsForClip[ knownActionsForClip.length - 1 ],

			byClipCacheIndex = action._byClipCacheIndex;

		lastKnownAction._byClipCacheIndex = byClipCacheIndex;
		knownActionsForClip[ byClipCacheIndex ] = lastKnownAction;
		knownActionsForClip.pop();

		action._byClipCacheIndex = null;


		const actionByRoot = actionsForClip.actionByRoot,
			rootUuid = ( action._localRoot || this._root ).uuid;

		delete actionByRoot[ rootUuid ];

		if ( knownActionsForClip.length === 0 ) {

			delete actionsByClip[ clipUuid ];

		}

		this._removeInactiveBindingsForAction( action );

	}

	_removeInactiveBindingsForAction( action ) {

		const bindings = action._propertyBindings;

		for ( let i = 0, n = bindings.length; i !== n; ++ i ) {

			const binding = bindings[ i ];

			if ( -- binding.referenceCount === 0 ) {

				this._removeInactiveBinding( binding );

			}

		}

	}

	_lendAction( action ) {

		// [ active actions |  inactive actions  ]
		// [  active actions >| inactive actions ]
		//                 s        a
		//                  <-swap->
		//                 a        s

		const actions = this._actions,
			prevIndex = action._cacheIndex,

			lastActiveIndex = this._nActiveActions ++,

			firstInactiveAction = actions[ lastActiveIndex ];

		action._cacheIndex = lastActiveIndex;
		actions[ lastActiveIndex ] = action;

		firstInactiveAction._cacheIndex = prevIndex;
		actions[ prevIndex ] = firstInactiveAction;

	}

	_takeBackAction( action ) {

		// [  active actions  | inactive actions ]
		// [ active actions |< inactive actions  ]
		//        a        s
		//         <-swap->
		//        s        a

		const actions = this._actions,
			prevIndex = action._cacheIndex,

			firstInactiveIndex = -- this._nActiveActions,

			lastActiveAction = actions[ firstInactiveIndex ];

		action._cacheIndex = firstInactiveIndex;
		actions[ firstInactiveIndex ] = action;

		lastActiveAction._cacheIndex = prevIndex;
		actions[ prevIndex ] = lastActiveAction;

	}

	// Memory management for PropertyMixer objects

	_addInactiveBinding( binding, rootUuid, trackName ) {

		const bindingsByRoot = this._bindingsByRootAndName,
			bindings = this._bindings;

		let bindingByName = bindingsByRoot[ rootUuid ];

		if ( bindingByName === undefined ) {

			bindingByName = {};
			bindingsByRoot[ rootUuid ] = bindingByName;

		}

		bindingByName[ trackName ] = binding;

		binding._cacheIndex = bindings.length;
		bindings.push( binding );

	}

	_removeInactiveBinding( binding ) {

		const bindings = this._bindings,
			propBinding = binding.binding,
			rootUuid = propBinding.rootNode.uuid,
			trackName = propBinding.path,
			bindingsByRoot = this._bindingsByRootAndName,
			bindingByName = bindingsByRoot[ rootUuid ],

			lastInactiveBinding = bindings[ bindings.length - 1 ],
			cacheIndex = binding._cacheIndex;

		lastInactiveBinding._cacheIndex = cacheIndex;
		bindings[ cacheIndex ] = lastInactiveBinding;
		bindings.pop();

		delete bindingByName[ trackName ];

		if ( Object.keys( bindingByName ).length === 0 ) {

			delete bindingsByRoot[ rootUuid ];

		}

	}

	_lendBinding( binding ) {

		const bindings = this._bindings,
			prevIndex = binding._cacheIndex,

			lastActiveIndex = this._nActiveBindings ++,

			firstInactiveBinding = bindings[ lastActiveIndex ];

		binding._cacheIndex = lastActiveIndex;
		bindings[ lastActiveIndex ] = binding;

		firstInactiveBinding._cacheIndex = prevIndex;
		bindings[ prevIndex ] = firstInactiveBinding;

	}

	_takeBackBinding( binding ) {

		const bindings = this._bindings,
			prevIndex = binding._cacheIndex,

			firstInactiveIndex = -- this._nActiveBindings,

			lastActiveBinding = bindings[ firstInactiveIndex ];

		binding._cacheIndex = firstInactiveIndex;
		bindings[ firstInactiveIndex ] = binding;

		lastActiveBinding._cacheIndex = prevIndex;
		bindings[ prevIndex ] = lastActiveBinding;

	}


	// Memory management of Interpolants for weight and time scale

	_lendControlInterpolant() {

		const interpolants = this._controlInterpolants,
			lastActiveIndex = this._nActiveControlInterpolants ++;

		let interpolant = interpolants[ lastActiveIndex ];

		if ( interpolant === undefined ) {

			interpolant = new LinearInterpolant(
				new Float32Array( 2 ), new Float32Array( 2 ),
				1, this._controlInterpolantsResultBuffer );

			interpolant.__cacheIndex = lastActiveIndex;
			interpolants[ lastActiveIndex ] = interpolant;

		}

		return interpolant;

	}

	_takeBackControlInterpolant( interpolant ) {

		const interpolants = this._controlInterpolants,
			prevIndex = interpolant.__cacheIndex,

			firstInactiveIndex = -- this._nActiveControlInterpolants,

			lastActiveInterpolant = interpolants[ firstInactiveIndex ];

		interpolant.__cacheIndex = firstInactiveIndex;
		interpolants[ firstInactiveIndex ] = interpolant;

		lastActiveInterpolant.__cacheIndex = prevIndex;
		interpolants[ prevIndex ] = lastActiveInterpolant;

	}

	// return an action for a clip optionally using a custom root target
	// object (this method allocates a lot of dynamic memory in case a
	// previously unknown clip/root combination is specified)
	clipAction( clip, optionalRoot, blendMode ) {

		const root = optionalRoot || this._root,
			rootUuid = root.uuid;

		let clipObject = typeof clip === 'string' ? AnimationClip.findByName( root, clip ) : clip;

		const clipUuid = clipObject !== null ? clipObject.uuid : clip;

		const actionsForClip = this._actionsByClip[ clipUuid ];
		let prototypeAction = null;

		if ( blendMode === undefined ) {

			if ( clipObject !== null ) {

				blendMode = clipObject.blendMode;

			} else {

				blendMode = NormalAnimationBlendMode;

			}

		}

		if ( actionsForClip !== undefined ) {

			const existingAction = actionsForClip.actionByRoot[ rootUuid ];

			if ( existingAction !== undefined && existingAction.blendMode === blendMode ) {

				return existingAction;

			}

			// we know the clip, so we don't have to parse all
			// the bindings again but can just copy
			prototypeAction = actionsForClip.knownActions[ 0 ];

			// also, take the clip from the prototype action
			if ( clipObject === null )
				clipObject = prototypeAction._clip;

		}

		// clip must be known when specified via string
		if ( clipObject === null ) return null;

		// allocate all resources required to run it
		const newAction = new AnimationAction( this, clipObject, optionalRoot, blendMode );

		this._bindAction( newAction, prototypeAction );

		// and make the action known to the memory manager
		this._addInactiveAction( newAction, clipUuid, rootUuid );

		return newAction;

	}

	// get an existing action
	existingAction( clip, optionalRoot ) {

		const root = optionalRoot || this._root,
			rootUuid = root.uuid,

			clipObject = typeof clip === 'string' ?
				AnimationClip.findByName( root, clip ) : clip,

			clipUuid = clipObject ? clipObject.uuid : clip,

			actionsForClip = this._actionsByClip[ clipUuid ];

		if ( actionsForClip !== undefined ) {

			return actionsForClip.actionByRoot[ rootUuid ] || null;

		}

		return null;

	}

	// deactivates all previously scheduled actions
	stopAllAction() {

		const actions = this._actions,
			nActions = this._nActiveActions;

		for ( let i = nActions - 1; i >= 0; -- i ) {

			actions[ i ].stop();

		}

		return this;

	}

	// advance the time and update apply the animation
	update( deltaTime ) {

		deltaTime *= this.timeScale;

		const actions = this._actions,
			nActions = this._nActiveActions,

			time = this.time += deltaTime,
			timeDirection = Math.sign( deltaTime ),

			accuIndex = this._accuIndex ^= 1;

		// run active actions

		for ( let i = 0; i !== nActions; ++ i ) {

			const action = actions[ i ];

			action._update( time, deltaTime, timeDirection, accuIndex );

		}

		// update scene graph

		const bindings = this._bindings,
			nBindings = this._nActiveBindings;

		for ( let i = 0; i !== nBindings; ++ i ) {

			bindings[ i ].apply( accuIndex );

		}

		return this;

	}

	// Allows you to seek to a specific time in an animation.
	setTime( timeInSeconds ) {

		this.time = 0; // Zero out time attribute for AnimationMixer object;
		for ( let i = 0; i < this._actions.length; i ++ ) {

			this._actions[ i ].time = 0; // Zero out time attribute for all associated AnimationAction objects.

		}

		return this.update( timeInSeconds ); // Update used to set exact time. Returns "this" AnimationMixer object.

	}

	// return this mixer's root target object
	getRoot() {

		return this._root;

	}

	// free all resources specific to a particular clip
	uncacheClip( clip ) {

		const actions = this._actions,
			clipUuid = clip.uuid,
			actionsByClip = this._actionsByClip,
			actionsForClip = actionsByClip[ clipUuid ];

		if ( actionsForClip !== undefined ) {

			// note: just calling _removeInactiveAction would mess up the
			// iteration state and also require updating the state we can
			// just throw away

			const actionsToRemove = actionsForClip.knownActions;

			for ( let i = 0, n = actionsToRemove.length; i !== n; ++ i ) {

				const action = actionsToRemove[ i ];

				this._deactivateAction( action );

				const cacheIndex = action._cacheIndex,
					lastInactiveAction = actions[ actions.length - 1 ];

				action._cacheIndex = null;
				action._byClipCacheIndex = null;

				lastInactiveAction._cacheIndex = cacheIndex;
				actions[ cacheIndex ] = lastInactiveAction;
				actions.pop();

				this._removeInactiveBindingsForAction( action );

			}

			delete actionsByClip[ clipUuid ];

		}

	}

	// free all resources specific to a particular root target object
	uncacheRoot( root ) {

		const rootUuid = root.uuid,
			actionsByClip = this._actionsByClip;

		for ( const clipUuid in actionsByClip ) {

			const actionByRoot = actionsByClip[ clipUuid ].actionByRoot,
				action = actionByRoot[ rootUuid ];

			if ( action !== undefined ) {

				this._deactivateAction( action );
				this._removeInactiveAction( action );

			}

		}

		const bindingsByRoot = this._bindingsByRootAndName,
			bindingByName = bindingsByRoot[ rootUuid ];

		if ( bindingByName !== undefined ) {

			for ( const trackName in bindingByName ) {

				const binding = bindingByName[ trackName ];
				binding.restoreOriginalState();
				this._removeInactiveBinding( binding );

			}

		}

	}

	// remove a targeted clip from the cache
	uncacheAction( clip, optionalRoot ) {

		const action = this.existingAction( clip, optionalRoot );

		if ( action !== null ) {

			this._deactivateAction( action );
			this._removeInactiveAction( action );

		}

	}

}

AnimationMixer.prototype._controlInterpolantsResultBuffer = new Float32Array( 1 );

class Uniform {

	constructor( value ) {

		if ( typeof value === 'string' ) {

			console.warn( 'THREE.Uniform: Type parameter is no longer needed.' );
			value = arguments[ 1 ];

		}

		this.value = value;

	}

	clone() {

		return new Uniform( this.value.clone === undefined ? this.value : this.value.clone() );

	}

}

function InstancedInterleavedBuffer( array, stride, meshPerAttribute ) {

	InterleavedBuffer.call( this, array, stride );

	this.meshPerAttribute = meshPerAttribute || 1;

}

InstancedInterleavedBuffer.prototype = Object.assign( Object.create( InterleavedBuffer.prototype ), {

	constructor: InstancedInterleavedBuffer,

	isInstancedInterleavedBuffer: true,

	copy: function ( source ) {

		InterleavedBuffer.prototype.copy.call( this, source );

		this.meshPerAttribute = source.meshPerAttribute;

		return this;

	},

	clone: function ( data ) {

		const ib = InterleavedBuffer.prototype.clone.call( this, data );

		ib.meshPerAttribute = this.meshPerAttribute;

		return ib;

	},

	toJSON: function ( data ) {

		const json = InterleavedBuffer.prototype.toJSON.call( this, data );

		json.isInstancedInterleavedBuffer = true;
		json.meshPerAttribute = this.meshPerAttribute;

		return json;

	}

} );

function GLBufferAttribute( buffer, type, itemSize, elementSize, count ) {

	this.buffer = buffer;
	this.type = type;
	this.itemSize = itemSize;
	this.elementSize = elementSize;
	this.count = count;

	this.version = 0;

}

Object.defineProperty( GLBufferAttribute.prototype, 'needsUpdate', {

	set: function ( value ) {

		if ( value === true ) this.version ++;

	}

} );

Object.assign( GLBufferAttribute.prototype, {

	isGLBufferAttribute: true,

	setBuffer: function ( buffer ) {

		this.buffer = buffer;

		return this;

	},

	setType: function ( type, elementSize ) {

		this.type = type;
		this.elementSize = elementSize;

		return this;

	},

	setItemSize: function ( itemSize ) {

		this.itemSize = itemSize;

		return this;

	},

	setCount: function ( count ) {

		this.count = count;

		return this;

	},

} );

function Raycaster( origin, direction, near = 0, far = Infinity ) {

	this.ray = new Ray( origin, direction );
	// direction is assumed to be normalized (for accurate distance calculations)

	this.near = near;
	this.far = far;
	this.camera = null;
	this.layers = new Layers();

	this.params = {
		Mesh: {},
		Line: { threshold: 1 },
		LOD: {},
		Points: { threshold: 1 },
		Sprite: {}
	};

	Object.defineProperties( this.params, {
		PointCloud: {
			get: function () {

				console.warn( 'THREE.Raycaster: params.PointCloud has been renamed to params.Points.' );
				return this.Points;

			}
		}
	} );

}

function ascSort( a, b ) {

	return a.distance - b.distance;

}

function intersectObject( object, raycaster, intersects, recursive ) {

	if ( object.layers.test( raycaster.layers ) ) {

		object.raycast( raycaster, intersects );

	}

	if ( recursive === true ) {

		const children = object.children;

		for ( let i = 0, l = children.length; i < l; i ++ ) {

			intersectObject( children[ i ], raycaster, intersects, true );

		}

	}

}

Object.assign( Raycaster.prototype, {

	set: function ( origin, direction ) {

		// direction is assumed to be normalized (for accurate distance calculations)

		this.ray.set( origin, direction );

	},

	setFromCamera: function ( coords, camera ) {

		if ( camera && camera.isPerspectiveCamera ) {

			this.ray.origin.setFromMatrixPosition( camera.matrixWorld );
			this.ray.direction.set( coords.x, coords.y, 0.5 ).unproject( camera ).sub( this.ray.origin ).normalize();
			this.camera = camera;

		} else if ( camera && camera.isOrthographicCamera ) {

			this.ray.origin.set( coords.x, coords.y, ( camera.near + camera.far ) / ( camera.near - camera.far ) ).unproject( camera ); // set origin in plane of camera
			this.ray.direction.set( 0, 0, - 1 ).transformDirection( camera.matrixWorld );
			this.camera = camera;

		} else {

			console.error( 'THREE.Raycaster: Unsupported camera type: ' + camera.type );

		}

	},

	intersectObject: function ( object, recursive = false, intersects = [] ) {

		intersectObject( object, this, intersects, recursive );

		intersects.sort( ascSort );

		return intersects;

	},

	intersectObjects: function ( objects, recursive = false, intersects = [] ) {

		for ( let i = 0, l = objects.length; i < l; i ++ ) {

			intersectObject( objects[ i ], this, intersects, recursive );

		}

		intersects.sort( ascSort );

		return intersects;

	}

} );

/**
 * Ref: https://en.wikipedia.org/wiki/Spherical_coordinate_system
 *
 * The polar angle (phi) is measured from the positive y-axis. The positive y-axis is up.
 * The azimuthal angle (theta) is measured from the positive z-axis.
 */

class Spherical {

	constructor( radius = 1, phi = 0, theta = 0 ) {

		this.radius = radius;
		this.phi = phi; // polar angle
		this.theta = theta; // azimuthal angle

		return this;

	}

	set( radius, phi, theta ) {

		this.radius = radius;
		this.phi = phi;
		this.theta = theta;

		return this;

	}

	copy( other ) {

		this.radius = other.radius;
		this.phi = other.phi;
		this.theta = other.theta;

		return this;

	}

	// restrict phi to be betwee EPS and PI-EPS
	makeSafe() {

		const EPS = 0.000001;
		this.phi = Math.max( EPS, Math.min( Math.PI - EPS, this.phi ) );

		return this;

	}

	setFromVector3( v ) {

		return this.setFromCartesianCoords( v.x, v.y, v.z );

	}

	setFromCartesianCoords( x, y, z ) {

		this.radius = Math.sqrt( x * x + y * y + z * z );

		if ( this.radius === 0 ) {

			this.theta = 0;
			this.phi = 0;

		} else {

			this.theta = Math.atan2( x, z );
			this.phi = Math.acos( MathUtils.clamp( y / this.radius, - 1, 1 ) );

		}

		return this;

	}

	clone() {

		return new this.constructor().copy( this );

	}

}

const _vector$4 = /*@__PURE__*/ new Vector2();

class Box2 {

	constructor( min = new Vector2( + Infinity, + Infinity ), max = new Vector2( - Infinity, - Infinity ) ) {

		this.min = min;
		this.max = max;

	}

	set( min, max ) {

		this.min.copy( min );
		this.max.copy( max );

		return this;

	}

	setFromPoints( points ) {

		this.makeEmpty();

		for ( let i = 0, il = points.length; i < il; i ++ ) {

			this.expandByPoint( points[ i ] );

		}

		return this;

	}

	setFromCenterAndSize( center, size ) {

		const halfSize = _vector$4.copy( size ).multiplyScalar( 0.5 );
		this.min.copy( center ).sub( halfSize );
		this.max.copy( center ).add( halfSize );

		return this;

	}

	clone() {

		return new this.constructor().copy( this );

	}

	copy( box ) {

		this.min.copy( box.min );
		this.max.copy( box.max );

		return this;

	}

	makeEmpty() {

		this.min.x = this.min.y = + Infinity;
		this.max.x = this.max.y = - Infinity;

		return this;

	}

	isEmpty() {

		// this is a more robust check for empty than ( volume <= 0 ) because volume can get positive with two negative axes

		return ( this.max.x < this.min.x ) || ( this.max.y < this.min.y );

	}

	getCenter( target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Box2: .getCenter() target is now required' );
			target = new Vector2();

		}

		return this.isEmpty() ? target.set( 0, 0 ) : target.addVectors( this.min, this.max ).multiplyScalar( 0.5 );

	}

	getSize( target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Box2: .getSize() target is now required' );
			target = new Vector2();

		}

		return this.isEmpty() ? target.set( 0, 0 ) : target.subVectors( this.max, this.min );

	}

	expandByPoint( point ) {

		this.min.min( point );
		this.max.max( point );

		return this;

	}

	expandByVector( vector ) {

		this.min.sub( vector );
		this.max.add( vector );

		return this;

	}

	expandByScalar( scalar ) {

		this.min.addScalar( - scalar );
		this.max.addScalar( scalar );

		return this;

	}

	containsPoint( point ) {

		return point.x < this.min.x || point.x > this.max.x ||
			point.y < this.min.y || point.y > this.max.y ? false : true;

	}

	containsBox( box ) {

		return this.min.x <= box.min.x && box.max.x <= this.max.x &&
			this.min.y <= box.min.y && box.max.y <= this.max.y;

	}

	getParameter( point, target ) {

		// This can potentially have a divide by zero if the box
		// has a size dimension of 0.

		if ( target === undefined ) {

			console.warn( 'THREE.Box2: .getParameter() target is now required' );
			target = new Vector2();

		}

		return target.set(
			( point.x - this.min.x ) / ( this.max.x - this.min.x ),
			( point.y - this.min.y ) / ( this.max.y - this.min.y )
		);

	}

	intersectsBox( box ) {

		// using 4 splitting planes to rule out intersections

		return box.max.x < this.min.x || box.min.x > this.max.x ||
			box.max.y < this.min.y || box.min.y > this.max.y ? false : true;

	}

	clampPoint( point, target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Box2: .clampPoint() target is now required' );
			target = new Vector2();

		}

		return target.copy( point ).clamp( this.min, this.max );

	}

	distanceToPoint( point ) {

		const clampedPoint = _vector$4.copy( point ).clamp( this.min, this.max );
		return clampedPoint.sub( point ).length();

	}

	intersect( box ) {

		this.min.max( box.min );
		this.max.min( box.max );

		return this;

	}

	union( box ) {

		this.min.min( box.min );
		this.max.max( box.max );

		return this;

	}

	translate( offset ) {

		this.min.add( offset );
		this.max.add( offset );

		return this;

	}

	equals( box ) {

		return box.min.equals( this.min ) && box.max.equals( this.max );

	}

}

Box2.prototype.isBox2 = true;

const _startP = /*@__PURE__*/ new Vector3();
const _startEnd = /*@__PURE__*/ new Vector3();

class Line3 {

	constructor( start = new Vector3(), end = new Vector3() ) {

		this.start = start;
		this.end = end;

	}

	set( start, end ) {

		this.start.copy( start );
		this.end.copy( end );

		return this;

	}

	copy( line ) {

		this.start.copy( line.start );
		this.end.copy( line.end );

		return this;

	}

	getCenter( target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Line3: .getCenter() target is now required' );
			target = new Vector3();

		}

		return target.addVectors( this.start, this.end ).multiplyScalar( 0.5 );

	}

	delta( target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Line3: .delta() target is now required' );
			target = new Vector3();

		}

		return target.subVectors( this.end, this.start );

	}

	distanceSq() {

		return this.start.distanceToSquared( this.end );

	}

	distance() {

		return this.start.distanceTo( this.end );

	}

	at( t, target ) {

		if ( target === undefined ) {

			console.warn( 'THREE.Line3: .at() target is now required' );
			target = new Vector3();

		}

		return this.delta( target ).multiplyScalar( t ).add( this.start );

	}

	closestPointToPointParameter( point, clampToLine ) {

		_startP.subVectors( point, this.start );
		_startEnd.subVectors( this.end, this.start );

		const startEnd2 = _startEnd.dot( _startEnd );
		const startEnd_startP = _startEnd.dot( _startP );

		let t = startEnd_startP / startEnd2;

		if ( clampToLine ) {

			t = MathUtils.clamp( t, 0, 1 );

		}

		return t;

	}

	closestPointToPoint( point, clampToLine, target ) {

		const t = this.closestPointToPointParameter( point, clampToLine );

		if ( target === undefined ) {

			console.warn( 'THREE.Line3: .closestPointToPoint() target is now required' );
			target = new Vector3();

		}

		return this.delta( target ).multiplyScalar( t ).add( this.start );

	}

	applyMatrix4( matrix ) {

		this.start.applyMatrix4( matrix );
		this.end.applyMatrix4( matrix );

		return this;

	}

	equals( line ) {

		return line.start.equals( this.start ) && line.end.equals( this.end );

	}

	clone() {

		return new this.constructor().copy( this );

	}

}

function ImmediateRenderObject( material ) {

	Object3D.call( this );

	this.material = material;
	this.render = function ( /* renderCallback */ ) {};

	this.hasPositions = false;
	this.hasNormals = false;
	this.hasColors = false;
	this.hasUvs = false;

	this.positionArray = null;
	this.normalArray = null;
	this.colorArray = null;
	this.uvArray = null;

	this.count = 0;

}

ImmediateRenderObject.prototype = Object.create( Object3D.prototype );
ImmediateRenderObject.prototype.constructor = ImmediateRenderObject;

ImmediateRenderObject.prototype.isImmediateRenderObject = true;

const _vector$2 = /*@__PURE__*/ new Vector3();
const _boneMatrix = /*@__PURE__*/ new Matrix4();
const _matrixWorldInv = /*@__PURE__*/ new Matrix4();


class SkeletonHelper extends LineSegments {

	constructor( object ) {

		const bones = getBoneList( object );

		const geometry = new BufferGeometry();

		const vertices = [];
		const colors = [];

		const color1 = new Color( 0, 0, 1 );
		const color2 = new Color( 0, 1, 0 );

		for ( let i = 0; i < bones.length; i ++ ) {

			const bone = bones[ i ];

			if ( bone.parent && bone.parent.isBone ) {

				vertices.push( 0, 0, 0 );
				vertices.push( 0, 0, 0 );
				colors.push( color1.r, color1.g, color1.b );
				colors.push( color2.r, color2.g, color2.b );

			}

		}

		geometry.setAttribute( 'position', new Float32BufferAttribute( vertices, 3 ) );
		geometry.setAttribute( 'color', new Float32BufferAttribute( colors, 3 ) );

		const material = new LineBasicMaterial( { vertexColors: true, depthTest: false, depthWrite: false, toneMapped: false, transparent: true } );

		super( geometry, material );

		this.type = 'SkeletonHelper';
		this.isSkeletonHelper = true;

		this.root = object;
		this.bones = bones;

		this.matrix = object.matrixWorld;
		this.matrixAutoUpdate = false;

	}

	updateMatrixWorld( force ) {

		const bones = this.bones;

		const geometry = this.geometry;
		const position = geometry.getAttribute( 'position' );

		_matrixWorldInv.copy( this.root.matrixWorld ).invert();

		for ( let i = 0, j = 0; i < bones.length; i ++ ) {

			const bone = bones[ i ];

			if ( bone.parent && bone.parent.isBone ) {

				_boneMatrix.multiplyMatrices( _matrixWorldInv, bone.matrixWorld );
				_vector$2.setFromMatrixPosition( _boneMatrix );
				position.setXYZ( j, _vector$2.x, _vector$2.y, _vector$2.z );

				_boneMatrix.multiplyMatrices( _matrixWorldInv, bone.parent.matrixWorld );
				_vector$2.setFromMatrixPosition( _boneMatrix );
				position.setXYZ( j + 1, _vector$2.x, _vector$2.y, _vector$2.z );

				j += 2;

			}

		}

		geometry.getAttribute( 'position' ).needsUpdate = true;

		super.updateMatrixWorld( force );

	}

}


function getBoneList( object ) {

	const boneList = [];

	if ( object && object.isBone ) {

		boneList.push( object );

	}

	for ( let i = 0; i < object.children.length; i ++ ) {

		boneList.push.apply( boneList, getBoneList( object.children[ i ] ) );

	}

	return boneList;

}

class GridHelper extends LineSegments {

	constructor( size = 10, divisions = 10, color1 = 0x444444, color2 = 0x888888 ) {

		color1 = new Color( color1 );
		color2 = new Color( color2 );

		const center = divisions / 2;
		const step = size / divisions;
		const halfSize = size / 2;

		const vertices = [], colors = [];

		for ( let i = 0, j = 0, k = - halfSize; i <= divisions; i ++, k += step ) {

			vertices.push( - halfSize, 0, k, halfSize, 0, k );
			vertices.push( k, 0, - halfSize, k, 0, halfSize );

			const color = i === center ? color1 : color2;

			color.toArray( colors, j ); j += 3;
			color.toArray( colors, j ); j += 3;
			color.toArray( colors, j ); j += 3;
			color.toArray( colors, j ); j += 3;

		}

		const geometry = new BufferGeometry();
		geometry.setAttribute( 'position', new Float32BufferAttribute( vertices, 3 ) );
		geometry.setAttribute( 'color', new Float32BufferAttribute( colors, 3 ) );

		const material = new LineBasicMaterial( { vertexColors: true, toneMapped: false } );

		super( geometry, material );

		this.type = 'GridHelper';

	}

}

const backgroundMaterial = new MeshBasicMaterial( {
	side: BackSide,
	depthWrite: false,
	depthTest: false,
} );
const backgroundBox = new Mesh( new BoxGeometry(), backgroundMaterial );

//

Curve.create = function ( construct, getPoint ) {

	console.log( 'THREE.Curve.create() has been deprecated' );

	construct.prototype = Object.create( Curve.prototype );
	construct.prototype.constructor = construct;
	construct.prototype.getPoint = getPoint;

	return construct;

};

//

Path.prototype.fromPoints = function ( points ) {

	console.warn( 'THREE.Path: .fromPoints() has been renamed to .setFromPoints().' );
	return this.setFromPoints( points );

};

GridHelper.prototype.setColors = function () {

	console.error( 'THREE.GridHelper: setColors() has been deprecated, pass them in the constructor instead.' );

};

SkeletonHelper.prototype.update = function () {

	console.error( 'THREE.SkeletonHelper: update() no longer needs to be called.' );

};

//

Loader.prototype.extractUrlBase = function ( url ) {

	console.warn( 'THREE.Loader: .extractUrlBase() has been deprecated. Use THREE.LoaderUtils.extractUrlBase() instead.' );
	return LoaderUtils.extractUrlBase( url );

};

Loader.Handlers = {

	add: function ( /* regex, loader */ ) {

		console.error( 'THREE.Loader: Handlers.add() has been removed. Use LoadingManager.addHandler() instead.' );

	},

	get: function ( /* file */ ) {

		console.error( 'THREE.Loader: Handlers.get() has been removed. Use LoadingManager.getHandler() instead.' );

	}

};

//

Box2.prototype.center = function ( optionalTarget ) {

	console.warn( 'THREE.Box2: .center() has been renamed to .getCenter().' );
	return this.getCenter( optionalTarget );

};

Box2.prototype.empty = function () {

	console.warn( 'THREE.Box2: .empty() has been renamed to .isEmpty().' );
	return this.isEmpty();

};

Box2.prototype.isIntersectionBox = function ( box ) {

	console.warn( 'THREE.Box2: .isIntersectionBox() has been renamed to .intersectsBox().' );
	return this.intersectsBox( box );

};

Box2.prototype.size = function ( optionalTarget ) {

	console.warn( 'THREE.Box2: .size() has been renamed to .getSize().' );
	return this.getSize( optionalTarget );

};

//

Box3.prototype.center = function ( optionalTarget ) {

	console.warn( 'THREE.Box3: .center() has been renamed to .getCenter().' );
	return this.getCenter( optionalTarget );

};

Box3.prototype.empty = function () {

	console.warn( 'THREE.Box3: .empty() has been renamed to .isEmpty().' );
	return this.isEmpty();

};

Box3.prototype.isIntersectionBox = function ( box ) {

	console.warn( 'THREE.Box3: .isIntersectionBox() has been renamed to .intersectsBox().' );
	return this.intersectsBox( box );

};

Box3.prototype.isIntersectionSphere = function ( sphere ) {

	console.warn( 'THREE.Box3: .isIntersectionSphere() has been renamed to .intersectsSphere().' );
	return this.intersectsSphere( sphere );

};

Box3.prototype.size = function ( optionalTarget ) {

	console.warn( 'THREE.Box3: .size() has been renamed to .getSize().' );
	return this.getSize( optionalTarget );

};

//

Sphere.prototype.empty = function () {

	console.warn( 'THREE.Sphere: .empty() has been renamed to .isEmpty().' );
	return this.isEmpty();

};

//

Frustum.prototype.setFromMatrix = function ( m ) {

	console.warn( 'THREE.Frustum: .setFromMatrix() has been renamed to .setFromProjectionMatrix().' );
	return this.setFromProjectionMatrix( m );

};

//

Line3.prototype.center = function ( optionalTarget ) {

	console.warn( 'THREE.Line3: .center() has been renamed to .getCenter().' );
	return this.getCenter( optionalTarget );

};

//

MathUtils.random16 = function () {

	console.warn( 'THREE.Math: .random16() has been deprecated. Use Math.random() instead.' );
	return Math.random();

};

MathUtils.nearestPowerOfTwo = function ( value ) {

	console.warn( 'THREE.Math: .nearestPowerOfTwo() has been renamed to .floorPowerOfTwo().' );
	return MathUtils.floorPowerOfTwo( value );

};

MathUtils.nextPowerOfTwo = function ( value ) {

	console.warn( 'THREE.Math: .nextPowerOfTwo() has been renamed to .ceilPowerOfTwo().' );
	return MathUtils.ceilPowerOfTwo( value );

};

//

Matrix3.prototype.flattenToArrayOffset = function ( array, offset ) {

	console.warn( 'THREE.Matrix3: .flattenToArrayOffset() has been deprecated. Use .toArray() instead.' );
	return this.toArray( array, offset );

};

Matrix3.prototype.multiplyVector3 = function ( vector ) {

	console.warn( 'THREE.Matrix3: .multiplyVector3() has been removed. Use vector.applyMatrix3( matrix ) instead.' );
	return vector.applyMatrix3( this );

};

Matrix3.prototype.multiplyVector3Array = function ( /* a */ ) {

	console.error( 'THREE.Matrix3: .multiplyVector3Array() has been removed.' );

};

Matrix3.prototype.applyToBufferAttribute = function ( attribute ) {

	console.warn( 'THREE.Matrix3: .applyToBufferAttribute() has been removed. Use attribute.applyMatrix3( matrix ) instead.' );
	return attribute.applyMatrix3( this );

};

Matrix3.prototype.applyToVector3Array = function ( /* array, offset, length */ ) {

	console.error( 'THREE.Matrix3: .applyToVector3Array() has been removed.' );

};

Matrix3.prototype.getInverse = function ( matrix ) {

	console.warn( 'THREE.Matrix3: .getInverse() has been removed. Use matrixInv.copy( matrix ).invert(); instead.' );
	return this.copy( matrix ).invert();

};

//

Matrix4.prototype.extractPosition = function ( m ) {

	console.warn( 'THREE.Matrix4: .extractPosition() has been renamed to .copyPosition().' );
	return this.copyPosition( m );

};

Matrix4.prototype.flattenToArrayOffset = function ( array, offset ) {

	console.warn( 'THREE.Matrix4: .flattenToArrayOffset() has been deprecated. Use .toArray() instead.' );
	return this.toArray( array, offset );

};

Matrix4.prototype.getPosition = function () {

	console.warn( 'THREE.Matrix4: .getPosition() has been removed. Use Vector3.setFromMatrixPosition( matrix ) instead.' );
	return new Vector3().setFromMatrixColumn( this, 3 );

};

Matrix4.prototype.setRotationFromQuaternion = function ( q ) {

	console.warn( 'THREE.Matrix4: .setRotationFromQuaternion() has been renamed to .makeRotationFromQuaternion().' );
	return this.makeRotationFromQuaternion( q );

};

Matrix4.prototype.multiplyToArray = function () {

	console.warn( 'THREE.Matrix4: .multiplyToArray() has been removed.' );

};

Matrix4.prototype.multiplyVector3 = function ( vector ) {

	console.warn( 'THREE.Matrix4: .multiplyVector3() has been removed. Use vector.applyMatrix4( matrix ) instead.' );
	return vector.applyMatrix4( this );

};

Matrix4.prototype.multiplyVector4 = function ( vector ) {

	console.warn( 'THREE.Matrix4: .multiplyVector4() has been removed. Use vector.applyMatrix4( matrix ) instead.' );
	return vector.applyMatrix4( this );

};

Matrix4.prototype.multiplyVector3Array = function ( /* a */ ) {

	console.error( 'THREE.Matrix4: .multiplyVector3Array() has been removed.' );

};

Matrix4.prototype.rotateAxis = function ( v ) {

	console.warn( 'THREE.Matrix4: .rotateAxis() has been removed. Use Vector3.transformDirection( matrix ) instead.' );
	v.transformDirection( this );

};

Matrix4.prototype.crossVector = function ( vector ) {

	console.warn( 'THREE.Matrix4: .crossVector() has been removed. Use vector.applyMatrix4( matrix ) instead.' );
	return vector.applyMatrix4( this );

};

Matrix4.prototype.translate = function () {

	console.error( 'THREE.Matrix4: .translate() has been removed.' );

};

Matrix4.prototype.rotateX = function () {

	console.error( 'THREE.Matrix4: .rotateX() has been removed.' );

};

Matrix4.prototype.rotateY = function () {

	console.error( 'THREE.Matrix4: .rotateY() has been removed.' );

};

Matrix4.prototype.rotateZ = function () {

	console.error( 'THREE.Matrix4: .rotateZ() has been removed.' );

};

Matrix4.prototype.rotateByAxis = function () {

	console.error( 'THREE.Matrix4: .rotateByAxis() has been removed.' );

};

Matrix4.prototype.applyToBufferAttribute = function ( attribute ) {

	console.warn( 'THREE.Matrix4: .applyToBufferAttribute() has been removed. Use attribute.applyMatrix4( matrix ) instead.' );
	return attribute.applyMatrix4( this );

};

Matrix4.prototype.applyToVector3Array = function ( /* array, offset, length */ ) {

	console.error( 'THREE.Matrix4: .applyToVector3Array() has been removed.' );

};

Matrix4.prototype.makeFrustum = function ( left, right, bottom, top, near, far ) {

	console.warn( 'THREE.Matrix4: .makeFrustum() has been removed. Use .makePerspective( left, right, top, bottom, near, far ) instead.' );
	return this.makePerspective( left, right, top, bottom, near, far );

};

Matrix4.prototype.getInverse = function ( matrix ) {

	console.warn( 'THREE.Matrix4: .getInverse() has been removed. Use matrixInv.copy( matrix ).invert(); instead.' );
	return this.copy( matrix ).invert();

};

//

Plane.prototype.isIntersectionLine = function ( line ) {

	console.warn( 'THREE.Plane: .isIntersectionLine() has been renamed to .intersectsLine().' );
	return this.intersectsLine( line );

};

//

Quaternion.prototype.multiplyVector3 = function ( vector ) {

	console.warn( 'THREE.Quaternion: .multiplyVector3() has been removed. Use is now vector.applyQuaternion( quaternion ) instead.' );
	return vector.applyQuaternion( this );

};

Quaternion.prototype.inverse = function ( ) {

	console.warn( 'THREE.Quaternion: .inverse() has been renamed to invert().' );
	return this.invert();

};

//

Ray.prototype.isIntersectionBox = function ( box ) {

	console.warn( 'THREE.Ray: .isIntersectionBox() has been renamed to .intersectsBox().' );
	return this.intersectsBox( box );

};

Ray.prototype.isIntersectionPlane = function ( plane ) {

	console.warn( 'THREE.Ray: .isIntersectionPlane() has been renamed to .intersectsPlane().' );
	return this.intersectsPlane( plane );

};

Ray.prototype.isIntersectionSphere = function ( sphere ) {

	console.warn( 'THREE.Ray: .isIntersectionSphere() has been renamed to .intersectsSphere().' );
	return this.intersectsSphere( sphere );

};

//

Triangle.prototype.area = function () {

	console.warn( 'THREE.Triangle: .area() has been renamed to .getArea().' );
	return this.getArea();

};

Triangle.prototype.barycoordFromPoint = function ( point, target ) {

	console.warn( 'THREE.Triangle: .barycoordFromPoint() has been renamed to .getBarycoord().' );
	return this.getBarycoord( point, target );

};

Triangle.prototype.midpoint = function ( target ) {

	console.warn( 'THREE.Triangle: .midpoint() has been renamed to .getMidpoint().' );
	return this.getMidpoint( target );

};

Triangle.prototypenormal = function ( target ) {

	console.warn( 'THREE.Triangle: .normal() has been renamed to .getNormal().' );
	return this.getNormal( target );

};

Triangle.prototype.plane = function ( target ) {

	console.warn( 'THREE.Triangle: .plane() has been renamed to .getPlane().' );
	return this.getPlane( target );

};

Triangle.barycoordFromPoint = function ( point, a, b, c, target ) {

	console.warn( 'THREE.Triangle: .barycoordFromPoint() has been renamed to .getBarycoord().' );
	return Triangle.getBarycoord( point, a, b, c, target );

};

Triangle.normal = function ( a, b, c, target ) {

	console.warn( 'THREE.Triangle: .normal() has been renamed to .getNormal().' );
	return Triangle.getNormal( a, b, c, target );

};

//

Shape.prototype.extractAllPoints = function ( divisions ) {

	console.warn( 'THREE.Shape: .extractAllPoints() has been removed. Use .extractPoints() instead.' );
	return this.extractPoints( divisions );

};

Shape.prototype.extrude = function ( options ) {

	console.warn( 'THREE.Shape: .extrude() has been removed. Use ExtrudeGeometry() instead.' );
	return new ExtrudeGeometry( this, options );

};

Shape.prototype.makeGeometry = function ( options ) {

	console.warn( 'THREE.Shape: .makeGeometry() has been removed. Use ShapeGeometry() instead.' );
	return new ShapeGeometry( this, options );

};

//

Vector2.prototype.fromAttribute = function ( attribute, index, offset ) {

	console.warn( 'THREE.Vector2: .fromAttribute() has been renamed to .fromBufferAttribute().' );
	return this.fromBufferAttribute( attribute, index, offset );

};

Vector2.prototype.distanceToManhattan = function ( v ) {

	console.warn( 'THREE.Vector2: .distanceToManhattan() has been renamed to .manhattanDistanceTo().' );
	return this.manhattanDistanceTo( v );

};

Vector2.prototype.lengthManhattan = function () {

	console.warn( 'THREE.Vector2: .lengthManhattan() has been renamed to .manhattanLength().' );
	return this.manhattanLength();

};

//

Vector3.prototype.setEulerFromRotationMatrix = function () {

	console.error( 'THREE.Vector3: .setEulerFromRotationMatrix() has been removed. Use Euler.setFromRotationMatrix() instead.' );

};

Vector3.prototype.setEulerFromQuaternion = function () {

	console.error( 'THREE.Vector3: .setEulerFromQuaternion() has been removed. Use Euler.setFromQuaternion() instead.' );

};

Vector3.prototype.getPositionFromMatrix = function ( m ) {

	console.warn( 'THREE.Vector3: .getPositionFromMatrix() has been renamed to .setFromMatrixPosition().' );
	return this.setFromMatrixPosition( m );

};

Vector3.prototype.getScaleFromMatrix = function ( m ) {

	console.warn( 'THREE.Vector3: .getScaleFromMatrix() has been renamed to .setFromMatrixScale().' );
	return this.setFromMatrixScale( m );

};

Vector3.prototype.getColumnFromMatrix = function ( index, matrix ) {

	console.warn( 'THREE.Vector3: .getColumnFromMatrix() has been renamed to .setFromMatrixColumn().' );
	return this.setFromMatrixColumn( matrix, index );

};

Vector3.prototype.applyProjection = function ( m ) {

	console.warn( 'THREE.Vector3: .applyProjection() has been removed. Use .applyMatrix4( m ) instead.' );
	return this.applyMatrix4( m );

};

Vector3.prototype.fromAttribute = function ( attribute, index, offset ) {

	console.warn( 'THREE.Vector3: .fromAttribute() has been renamed to .fromBufferAttribute().' );
	return this.fromBufferAttribute( attribute, index, offset );

};

Vector3.prototype.distanceToManhattan = function ( v ) {

	console.warn( 'THREE.Vector3: .distanceToManhattan() has been renamed to .manhattanDistanceTo().' );
	return this.manhattanDistanceTo( v );

};

Vector3.prototype.lengthManhattan = function () {

	console.warn( 'THREE.Vector3: .lengthManhattan() has been renamed to .manhattanLength().' );
	return this.manhattanLength();

};

//

Vector4.prototype.fromAttribute = function ( attribute, index, offset ) {

	console.warn( 'THREE.Vector4: .fromAttribute() has been renamed to .fromBufferAttribute().' );
	return this.fromBufferAttribute( attribute, index, offset );

};

Vector4.prototype.lengthManhattan = function () {

	console.warn( 'THREE.Vector4: .lengthManhattan() has been renamed to .manhattanLength().' );
	return this.manhattanLength();

};

//

Object3D.prototype.getChildByName = function ( name ) {

	console.warn( 'THREE.Object3D: .getChildByName() has been renamed to .getObjectByName().' );
	return this.getObjectByName( name );

};

Object3D.prototype.renderDepth = function () {

	console.warn( 'THREE.Object3D: .renderDepth has been removed. Use .renderOrder, instead.' );

};

Object3D.prototype.translate = function ( distance, axis ) {

	console.warn( 'THREE.Object3D: .translate() has been removed. Use .translateOnAxis( axis, distance ) instead.' );
	return this.translateOnAxis( axis, distance );

};

Object3D.prototype.getWorldRotation = function () {

	console.error( 'THREE.Object3D: .getWorldRotation() has been removed. Use THREE.Object3D.getWorldQuaternion( target ) instead.' );

};

Object3D.prototype.applyMatrix = function ( matrix ) {

	console.warn( 'THREE.Object3D: .applyMatrix() has been renamed to .applyMatrix4().' );
	return this.applyMatrix4( matrix );

};

Object.defineProperties( Object3D.prototype, {

	eulerOrder: {
		get: function () {

			console.warn( 'THREE.Object3D: .eulerOrder is now .rotation.order.' );
			return this.rotation.order;

		},
		set: function ( value ) {

			console.warn( 'THREE.Object3D: .eulerOrder is now .rotation.order.' );
			this.rotation.order = value;

		}
	},
	useQuaternion: {
		get: function () {

			console.warn( 'THREE.Object3D: .useQuaternion has been removed. The library now uses quaternions by default.' );

		},
		set: function () {

			console.warn( 'THREE.Object3D: .useQuaternion has been removed. The library now uses quaternions by default.' );

		}
	}

} );

Mesh.prototype.setDrawMode = function () {

	console.error( 'THREE.Mesh: .setDrawMode() has been removed. The renderer now always assumes THREE.TrianglesDrawMode. Transform your geometry via BufferGeometryUtils.toTrianglesDrawMode() if necessary.' );

};

Object.defineProperties( Mesh.prototype, {

	drawMode: {
		get: function () {

			console.error( 'THREE.Mesh: .drawMode has been removed. The renderer now always assumes THREE.TrianglesDrawMode.' );
			return TrianglesDrawMode;

		},
		set: function () {

			console.error( 'THREE.Mesh: .drawMode has been removed. The renderer now always assumes THREE.TrianglesDrawMode. Transform your geometry via BufferGeometryUtils.toTrianglesDrawMode() if necessary.' );

		}
	}

} );

SkinnedMesh.prototype.initBones = function () {

	console.error( 'THREE.SkinnedMesh: initBones() has been removed.' );

};

Object.defineProperty( Curve.prototype, '__arcLengthDivisions', {

	get: function () {

		console.warn( 'THREE.Curve: .__arcLengthDivisions is now .arcLengthDivisions.' );
		return this.arcLengthDivisions;

	},
	set: function ( value ) {

		console.warn( 'THREE.Curve: .__arcLengthDivisions is now .arcLengthDivisions.' );
		this.arcLengthDivisions = value;

	}

} );

//

PerspectiveCamera.prototype.setLens = function ( focalLength, filmGauge ) {

	console.warn( 'THREE.PerspectiveCamera.setLens is deprecated. ' +
			'Use .setFocalLength and .filmGauge for a photographic setup.' );

	if ( filmGauge !== undefined ) this.filmGauge = filmGauge;
	this.setFocalLength( focalLength );

};

//

Object.defineProperties( Light.prototype, {
	onlyShadow: {
		set: function () {

			console.warn( 'THREE.Light: .onlyShadow has been removed.' );

		}
	},
	shadowCameraFov: {
		set: function ( value ) {

			console.warn( 'THREE.Light: .shadowCameraFov is now .shadow.camera.fov.' );
			this.shadow.camera.fov = value;

		}
	},
	shadowCameraLeft: {
		set: function ( value ) {

			console.warn( 'THREE.Light: .shadowCameraLeft is now .shadow.camera.left.' );
			this.shadow.camera.left = value;

		}
	},
	shadowCameraRight: {
		set: function ( value ) {

			console.warn( 'THREE.Light: .shadowCameraRight is now .shadow.camera.right.' );
			this.shadow.camera.right = value;

		}
	},
	shadowCameraTop: {
		set: function ( value ) {

			console.warn( 'THREE.Light: .shadowCameraTop is now .shadow.camera.top.' );
			this.shadow.camera.top = value;

		}
	},
	shadowCameraBottom: {
		set: function ( value ) {

			console.warn( 'THREE.Light: .shadowCameraBottom is now .shadow.camera.bottom.' );
			this.shadow.camera.bottom = value;

		}
	},
	shadowCameraNear: {
		set: function ( value ) {

			console.warn( 'THREE.Light: .shadowCameraNear is now .shadow.camera.near.' );
			this.shadow.camera.near = value;

		}
	},
	shadowCameraFar: {
		set: function ( value ) {

			console.warn( 'THREE.Light: .shadowCameraFar is now .shadow.camera.far.' );
			this.shadow.camera.far = value;

		}
	},
	shadowCameraVisible: {
		set: function () {

			console.warn( 'THREE.Light: .shadowCameraVisible has been removed. Use new THREE.CameraHelper( light.shadow.camera ) instead.' );

		}
	},
	shadowBias: {
		set: function ( value ) {

			console.warn( 'THREE.Light: .shadowBias is now .shadow.bias.' );
			this.shadow.bias = value;

		}
	},
	shadowDarkness: {
		set: function () {

			console.warn( 'THREE.Light: .shadowDarkness has been removed.' );

		}
	},
	shadowMapWidth: {
		set: function ( value ) {

			console.warn( 'THREE.Light: .shadowMapWidth is now .shadow.mapSize.width.' );
			this.shadow.mapSize.width = value;

		}
	},
	shadowMapHeight: {
		set: function ( value ) {

			console.warn( 'THREE.Light: .shadowMapHeight is now .shadow.mapSize.height.' );
			this.shadow.mapSize.height = value;

		}
	}
} );

//

Object.defineProperties( BufferAttribute.prototype, {

	length: {
		get: function () {

			console.warn( 'THREE.BufferAttribute: .length has been deprecated. Use .count instead.' );
			return this.array.length;

		}
	},
	dynamic: {
		get: function () {

			console.warn( 'THREE.BufferAttribute: .dynamic has been deprecated. Use .usage instead.' );
			return this.usage === DynamicDrawUsage;

		},
		set: function ( /* value */ ) {

			console.warn( 'THREE.BufferAttribute: .dynamic has been deprecated. Use .usage instead.' );
			this.setUsage( DynamicDrawUsage );

		}
	}

} );

BufferAttribute.prototype.setDynamic = function ( value ) {

	console.warn( 'THREE.BufferAttribute: .setDynamic() has been deprecated. Use .setUsage() instead.' );
	this.setUsage( value === true ? DynamicDrawUsage : StaticDrawUsage );
	return this;

};

BufferAttribute.prototype.copyIndicesArray = function ( /* indices */ ) {

	console.error( 'THREE.BufferAttribute: .copyIndicesArray() has been removed.' );

},

BufferAttribute.prototype.setArray = function ( /* array */ ) {

	console.error( 'THREE.BufferAttribute: .setArray has been removed. Use BufferGeometry .setAttribute to replace/resize attribute buffers' );

};

//

BufferGeometry.prototype.addIndex = function ( index ) {

	console.warn( 'THREE.BufferGeometry: .addIndex() has been renamed to .setIndex().' );
	this.setIndex( index );

};

BufferGeometry.prototype.addAttribute = function ( name, attribute ) {

	console.warn( 'THREE.BufferGeometry: .addAttribute() has been renamed to .setAttribute().' );

	if ( ! ( attribute && attribute.isBufferAttribute ) && ! ( attribute && attribute.isInterleavedBufferAttribute ) ) {

		console.warn( 'THREE.BufferGeometry: .addAttribute() now expects ( name, attribute ).' );

		return this.setAttribute( name, new BufferAttribute( arguments[ 1 ], arguments[ 2 ] ) );

	}

	if ( name === 'index' ) {

		console.warn( 'THREE.BufferGeometry.addAttribute: Use .setIndex() for index attribute.' );
		this.setIndex( attribute );

		return this;

	}

	return this.setAttribute( name, attribute );

};

BufferGeometry.prototype.addDrawCall = function ( start, count, indexOffset ) {

	if ( indexOffset !== undefined ) {

		console.warn( 'THREE.BufferGeometry: .addDrawCall() no longer supports indexOffset.' );

	}

	console.warn( 'THREE.BufferGeometry: .addDrawCall() is now .addGroup().' );
	this.addGroup( start, count );

};

BufferGeometry.prototype.clearDrawCalls = function () {

	console.warn( 'THREE.BufferGeometry: .clearDrawCalls() is now .clearGroups().' );
	this.clearGroups();

};

BufferGeometry.prototype.computeOffsets = function () {

	console.warn( 'THREE.BufferGeometry: .computeOffsets() has been removed.' );

};

BufferGeometry.prototype.removeAttribute = function ( name ) {

	console.warn( 'THREE.BufferGeometry: .removeAttribute() has been renamed to .deleteAttribute().' );

	return this.deleteAttribute( name );

};

BufferGeometry.prototype.applyMatrix = function ( matrix ) {

	console.warn( 'THREE.BufferGeometry: .applyMatrix() has been renamed to .applyMatrix4().' );
	return this.applyMatrix4( matrix );

};

Object.defineProperties( BufferGeometry.prototype, {

	drawcalls: {
		get: function () {

			console.error( 'THREE.BufferGeometry: .drawcalls has been renamed to .groups.' );
			return this.groups;

		}
	},
	offsets: {
		get: function () {

			console.warn( 'THREE.BufferGeometry: .offsets has been renamed to .groups.' );
			return this.groups;

		}
	}

} );

Object.defineProperties( InstancedBufferGeometry.prototype, {

	maxInstancedCount: {
		get: function () {

			console.warn( 'THREE.InstancedBufferGeometry: .maxInstancedCount has been renamed to .instanceCount.' );
			return this.instanceCount;

		},
		set: function ( value ) {

			console.warn( 'THREE.InstancedBufferGeometry: .maxInstancedCount has been renamed to .instanceCount.' );
			this.instanceCount = value;

		}
	}

} );

Object.defineProperties( Raycaster.prototype, {

	linePrecision: {
		get: function () {

			console.warn( 'THREE.Raycaster: .linePrecision has been deprecated. Use .params.Line.threshold instead.' );
			return this.params.Line.threshold;

		},
		set: function ( value ) {

			console.warn( 'THREE.Raycaster: .linePrecision has been deprecated. Use .params.Line.threshold instead.' );
			this.params.Line.threshold = value;

		}
	}

} );

Object.defineProperties( InterleavedBuffer.prototype, {

	dynamic: {
		get: function () {

			console.warn( 'THREE.InterleavedBuffer: .length has been deprecated. Use .usage instead.' );
			return this.usage === DynamicDrawUsage;

		},
		set: function ( value ) {

			console.warn( 'THREE.InterleavedBuffer: .length has been deprecated. Use .usage instead.' );
			this.setUsage( value );

		}
	}

} );

InterleavedBuffer.prototype.setDynamic = function ( value ) {

	console.warn( 'THREE.InterleavedBuffer: .setDynamic() has been deprecated. Use .setUsage() instead.' );
	this.setUsage( value === true ? DynamicDrawUsage : StaticDrawUsage );
	return this;

};

InterleavedBuffer.prototype.setArray = function ( /* array */ ) {

	console.error( 'THREE.InterleavedBuffer: .setArray has been removed. Use BufferGeometry .setAttribute to replace/resize attribute buffers' );

};

//

ExtrudeGeometry.prototype.getArrays = function () {

	console.error( 'THREE.ExtrudeGeometry: .getArrays() has been removed.' );

};

ExtrudeGeometry.prototype.addShapeList = function () {

	console.error( 'THREE.ExtrudeGeometry: .addShapeList() has been removed.' );

};

ExtrudeGeometry.prototype.addShape = function () {

	console.error( 'THREE.ExtrudeGeometry: .addShape() has been removed.' );

};

//

Scene.prototype.dispose = function () {

	console.error( 'THREE.Scene: .dispose() has been removed.' );

};

//

Uniform.prototype.onUpdate = function () {

	console.warn( 'THREE.Uniform: .onUpdate() has been removed. Use object.onBeforeRender() instead.' );
	return this;

};

//

Object.defineProperties( Material.prototype, {

	wrapAround: {
		get: function () {

			console.warn( 'THREE.Material: .wrapAround has been removed.' );

		},
		set: function () {

			console.warn( 'THREE.Material: .wrapAround has been removed.' );

		}
	},

	overdraw: {
		get: function () {

			console.warn( 'THREE.Material: .overdraw has been removed.' );

		},
		set: function () {

			console.warn( 'THREE.Material: .overdraw has been removed.' );

		}
	},

	wrapRGB: {
		get: function () {

			console.warn( 'THREE.Material: .wrapRGB has been removed.' );
			return new Color();

		}
	},

	shading: {
		get: function () {

			console.error( 'THREE.' + this.type + ': .shading has been removed. Use the boolean .flatShading instead.' );

		},
		set: function ( value ) {

			console.warn( 'THREE.' + this.type + ': .shading has been removed. Use the boolean .flatShading instead.' );
			this.flatShading = ( value === FlatShading );

		}
	},

	stencilMask: {
		get: function () {

			console.warn( 'THREE.' + this.type + ': .stencilMask has been removed. Use .stencilFuncMask instead.' );
			return this.stencilFuncMask;

		},
		set: function ( value ) {

			console.warn( 'THREE.' + this.type + ': .stencilMask has been removed. Use .stencilFuncMask instead.' );
			this.stencilFuncMask = value;

		}
	}

} );

Object.defineProperties( MeshPhysicalMaterial.prototype, {

	transparency: {
		get: function () {

			console.warn( 'THREE.MeshPhysicalMaterial: .transparency has been renamed to .transmission.' );
			return this.transmission;

		},
		set: function ( value ) {

			console.warn( 'THREE.MeshPhysicalMaterial: .transparency has been renamed to .transmission.' );
			this.transmission = value;

		}
	}

} );

Object.defineProperties( ShaderMaterial.prototype, {

	derivatives: {
		get: function () {

			console.warn( 'THREE.ShaderMaterial: .derivatives has been moved to .extensions.derivatives.' );
			return this.extensions.derivatives;

		},
		set: function ( value ) {

			console.warn( 'THREE. ShaderMaterial: .derivatives has been moved to .extensions.derivatives.' );
			this.extensions.derivatives = value;

		}
	}

} );

//

WebGLRenderer.prototype.clearTarget = function ( renderTarget, color, depth, stencil ) {

	console.warn( 'THREE.WebGLRenderer: .clearTarget() has been deprecated. Use .setRenderTarget() and .clear() instead.' );
	this.setRenderTarget( renderTarget );
	this.clear( color, depth, stencil );

};

WebGLRenderer.prototype.animate = function ( callback ) {

	console.warn( 'THREE.WebGLRenderer: .animate() is now .setAnimationLoop().' );
	this.setAnimationLoop( callback );

};

WebGLRenderer.prototype.getCurrentRenderTarget = function () {

	console.warn( 'THREE.WebGLRenderer: .getCurrentRenderTarget() is now .getRenderTarget().' );
	return this.getRenderTarget();

};

WebGLRenderer.prototype.getMaxAnisotropy = function () {

	console.warn( 'THREE.WebGLRenderer: .getMaxAnisotropy() is now .capabilities.getMaxAnisotropy().' );
	return this.capabilities.getMaxAnisotropy();

};

WebGLRenderer.prototype.getPrecision = function () {

	console.warn( 'THREE.WebGLRenderer: .getPrecision() is now .capabilities.precision.' );
	return this.capabilities.precision;

};

WebGLRenderer.prototype.resetGLState = function () {

	console.warn( 'THREE.WebGLRenderer: .resetGLState() is now .state.reset().' );
	return this.state.reset();

};

WebGLRenderer.prototype.supportsFloatTextures = function () {

	console.warn( 'THREE.WebGLRenderer: .supportsFloatTextures() is now .extensions.get( \'OES_texture_float\' ).' );
	return this.extensions.get( 'OES_texture_float' );

};

WebGLRenderer.prototype.supportsHalfFloatTextures = function () {

	console.warn( 'THREE.WebGLRenderer: .supportsHalfFloatTextures() is now .extensions.get( \'OES_texture_half_float\' ).' );
	return this.extensions.get( 'OES_texture_half_float' );

};

WebGLRenderer.prototype.supportsStandardDerivatives = function () {

	console.warn( 'THREE.WebGLRenderer: .supportsStandardDerivatives() is now .extensions.get( \'OES_standard_derivatives\' ).' );
	return this.extensions.get( 'OES_standard_derivatives' );

};

WebGLRenderer.prototype.supportsCompressedTextureS3TC = function () {

	console.warn( 'THREE.WebGLRenderer: .supportsCompressedTextureS3TC() is now .extensions.get( \'WEBGL_compressed_texture_s3tc\' ).' );
	return this.extensions.get( 'WEBGL_compressed_texture_s3tc' );

};

WebGLRenderer.prototype.supportsCompressedTexturePVRTC = function () {

	console.warn( 'THREE.WebGLRenderer: .supportsCompressedTexturePVRTC() is now .extensions.get( \'WEBGL_compressed_texture_pvrtc\' ).' );
	return this.extensions.get( 'WEBGL_compressed_texture_pvrtc' );

};

WebGLRenderer.prototype.supportsBlendMinMax = function () {

	console.warn( 'THREE.WebGLRenderer: .supportsBlendMinMax() is now .extensions.get( \'EXT_blend_minmax\' ).' );
	return this.extensions.get( 'EXT_blend_minmax' );

};

WebGLRenderer.prototype.supportsVertexTextures = function () {

	console.warn( 'THREE.WebGLRenderer: .supportsVertexTextures() is now .capabilities.vertexTextures.' );
	return this.capabilities.vertexTextures;

};

WebGLRenderer.prototype.supportsInstancedArrays = function () {

	console.warn( 'THREE.WebGLRenderer: .supportsInstancedArrays() is now .extensions.get( \'ANGLE_instanced_arrays\' ).' );
	return this.extensions.get( 'ANGLE_instanced_arrays' );

};

WebGLRenderer.prototype.enableScissorTest = function ( boolean ) {

	console.warn( 'THREE.WebGLRenderer: .enableScissorTest() is now .setScissorTest().' );
	this.setScissorTest( boolean );

};

WebGLRenderer.prototype.initMaterial = function () {

	console.warn( 'THREE.WebGLRenderer: .initMaterial() has been removed.' );

};

WebGLRenderer.prototype.addPrePlugin = function () {

	console.warn( 'THREE.WebGLRenderer: .addPrePlugin() has been removed.' );

};

WebGLRenderer.prototype.addPostPlugin = function () {

	console.warn( 'THREE.WebGLRenderer: .addPostPlugin() has been removed.' );

};

WebGLRenderer.prototype.updateShadowMap = function () {

	console.warn( 'THREE.WebGLRenderer: .updateShadowMap() has been removed.' );

};

WebGLRenderer.prototype.setFaceCulling = function () {

	console.warn( 'THREE.WebGLRenderer: .setFaceCulling() has been removed.' );

};

WebGLRenderer.prototype.allocTextureUnit = function () {

	console.warn( 'THREE.WebGLRenderer: .allocTextureUnit() has been removed.' );

};

WebGLRenderer.prototype.setTexture = function () {

	console.warn( 'THREE.WebGLRenderer: .setTexture() has been removed.' );

};

WebGLRenderer.prototype.setTexture2D = function () {

	console.warn( 'THREE.WebGLRenderer: .setTexture2D() has been removed.' );

};

WebGLRenderer.prototype.setTextureCube = function () {

	console.warn( 'THREE.WebGLRenderer: .setTextureCube() has been removed.' );

};

WebGLRenderer.prototype.getActiveMipMapLevel = function () {

	console.warn( 'THREE.WebGLRenderer: .getActiveMipMapLevel() is now .getActiveMipmapLevel().' );
	return this.getActiveMipmapLevel();

};

Object.defineProperties( WebGLRenderer.prototype, {

	shadowMapEnabled: {
		get: function () {

			return this.shadowMap.enabled;

		},
		set: function ( value ) {

			console.warn( 'THREE.WebGLRenderer: .shadowMapEnabled is now .shadowMap.enabled.' );
			this.shadowMap.enabled = value;

		}
	},
	shadowMapType: {
		get: function () {

			return this.shadowMap.type;

		},
		set: function ( value ) {

			console.warn( 'THREE.WebGLRenderer: .shadowMapType is now .shadowMap.type.' );
			this.shadowMap.type = value;

		}
	},
	shadowMapCullFace: {
		get: function () {

			console.warn( 'THREE.WebGLRenderer: .shadowMapCullFace has been removed. Set Material.shadowSide instead.' );
			return undefined;

		},
		set: function ( /* value */ ) {

			console.warn( 'THREE.WebGLRenderer: .shadowMapCullFace has been removed. Set Material.shadowSide instead.' );

		}
	},
	context: {
		get: function () {

			console.warn( 'THREE.WebGLRenderer: .context has been removed. Use .getContext() instead.' );
			return this.getContext();

		}
	},
	vr: {
		get: function () {

			console.warn( 'THREE.WebGLRenderer: .vr has been renamed to .xr' );
			return this.xr;

		}
	},
	gammaInput: {
		get: function () {

			console.warn( 'THREE.WebGLRenderer: .gammaInput has been removed. Set the encoding for textures via Texture.encoding instead.' );
			return false;

		},
		set: function () {

			console.warn( 'THREE.WebGLRenderer: .gammaInput has been removed. Set the encoding for textures via Texture.encoding instead.' );

		}
	},
	gammaOutput: {
		get: function () {

			console.warn( 'THREE.WebGLRenderer: .gammaOutput has been removed. Set WebGLRenderer.outputEncoding instead.' );
			return false;

		},
		set: function ( value ) {

			console.warn( 'THREE.WebGLRenderer: .gammaOutput has been removed. Set WebGLRenderer.outputEncoding instead.' );
			this.outputEncoding = ( value === true ) ? sRGBEncoding : LinearEncoding;

		}
	},
	toneMappingWhitePoint: {
		get: function () {

			console.warn( 'THREE.WebGLRenderer: .toneMappingWhitePoint has been removed.' );
			return 1.0;

		},
		set: function () {

			console.warn( 'THREE.WebGLRenderer: .toneMappingWhitePoint has been removed.' );

		}
	},

} );

Object.defineProperties( WebGLShadowMap.prototype, {

	cullFace: {
		get: function () {

			console.warn( 'THREE.WebGLRenderer: .shadowMap.cullFace has been removed. Set Material.shadowSide instead.' );
			return undefined;

		},
		set: function ( /* cullFace */ ) {

			console.warn( 'THREE.WebGLRenderer: .shadowMap.cullFace has been removed. Set Material.shadowSide instead.' );

		}
	},
	renderReverseSided: {
		get: function () {

			console.warn( 'THREE.WebGLRenderer: .shadowMap.renderReverseSided has been removed. Set Material.shadowSide instead.' );
			return undefined;

		},
		set: function () {

			console.warn( 'THREE.WebGLRenderer: .shadowMap.renderReverseSided has been removed. Set Material.shadowSide instead.' );

		}
	},
	renderSingleSided: {
		get: function () {

			console.warn( 'THREE.WebGLRenderer: .shadowMap.renderSingleSided has been removed. Set Material.shadowSide instead.' );
			return undefined;

		},
		set: function () {

			console.warn( 'THREE.WebGLRenderer: .shadowMap.renderSingleSided has been removed. Set Material.shadowSide instead.' );

		}
	}

} );

//

Object.defineProperties( WebGLRenderTarget.prototype, {

	wrapS: {
		get: function () {

			console.warn( 'THREE.WebGLRenderTarget: .wrapS is now .texture.wrapS.' );
			return this.texture.wrapS;

		},
		set: function ( value ) {

			console.warn( 'THREE.WebGLRenderTarget: .wrapS is now .texture.wrapS.' );
			this.texture.wrapS = value;

		}
	},
	wrapT: {
		get: function () {

			console.warn( 'THREE.WebGLRenderTarget: .wrapT is now .texture.wrapT.' );
			return this.texture.wrapT;

		},
		set: function ( value ) {

			console.warn( 'THREE.WebGLRenderTarget: .wrapT is now .texture.wrapT.' );
			this.texture.wrapT = value;

		}
	},
	magFilter: {
		get: function () {

			console.warn( 'THREE.WebGLRenderTarget: .magFilter is now .texture.magFilter.' );
			return this.texture.magFilter;

		},
		set: function ( value ) {

			console.warn( 'THREE.WebGLRenderTarget: .magFilter is now .texture.magFilter.' );
			this.texture.magFilter = value;

		}
	},
	minFilter: {
		get: function () {

			console.warn( 'THREE.WebGLRenderTarget: .minFilter is now .texture.minFilter.' );
			return this.texture.minFilter;

		},
		set: function ( value ) {

			console.warn( 'THREE.WebGLRenderTarget: .minFilter is now .texture.minFilter.' );
			this.texture.minFilter = value;

		}
	},
	anisotropy: {
		get: function () {

			console.warn( 'THREE.WebGLRenderTarget: .anisotropy is now .texture.anisotropy.' );
			return this.texture.anisotropy;

		},
		set: function ( value ) {

			console.warn( 'THREE.WebGLRenderTarget: .anisotropy is now .texture.anisotropy.' );
			this.texture.anisotropy = value;

		}
	},
	offset: {
		get: function () {

			console.warn( 'THREE.WebGLRenderTarget: .offset is now .texture.offset.' );
			return this.texture.offset;

		},
		set: function ( value ) {

			console.warn( 'THREE.WebGLRenderTarget: .offset is now .texture.offset.' );
			this.texture.offset = value;

		}
	},
	repeat: {
		get: function () {

			console.warn( 'THREE.WebGLRenderTarget: .repeat is now .texture.repeat.' );
			return this.texture.repeat;

		},
		set: function ( value ) {

			console.warn( 'THREE.WebGLRenderTarget: .repeat is now .texture.repeat.' );
			this.texture.repeat = value;

		}
	},
	format: {
		get: function () {

			console.warn( 'THREE.WebGLRenderTarget: .format is now .texture.format.' );
			return this.texture.format;

		},
		set: function ( value ) {

			console.warn( 'THREE.WebGLRenderTarget: .format is now .texture.format.' );
			this.texture.format = value;

		}
	},
	type: {
		get: function () {

			console.warn( 'THREE.WebGLRenderTarget: .type is now .texture.type.' );
			return this.texture.type;

		},
		set: function ( value ) {

			console.warn( 'THREE.WebGLRenderTarget: .type is now .texture.type.' );
			this.texture.type = value;

		}
	},
	generateMipmaps: {
		get: function () {

			console.warn( 'THREE.WebGLRenderTarget: .generateMipmaps is now .texture.generateMipmaps.' );
			return this.texture.generateMipmaps;

		},
		set: function ( value ) {

			console.warn( 'THREE.WebGLRenderTarget: .generateMipmaps is now .texture.generateMipmaps.' );
			this.texture.generateMipmaps = value;

		}
	}

} );

//

Audio.prototype.load = function ( file ) {

	console.warn( 'THREE.Audio: .load has been deprecated. Use THREE.AudioLoader instead.' );
	const scope = this;
	const audioLoader = new AudioLoader();
	audioLoader.load( file, function ( buffer ) {

		scope.setBuffer( buffer );

	} );
	return this;

};


AudioAnalyser.prototype.getData = function () {

	console.warn( 'THREE.AudioAnalyser: .getData() is now .getFrequencyData().' );
	return this.getFrequencyData();

};

//

CubeCamera.prototype.updateCubeMap = function ( renderer, scene ) {

	console.warn( 'THREE.CubeCamera: .updateCubeMap() is now .update().' );
	return this.update( renderer, scene );

};

CubeCamera.prototype.clear = function ( renderer, color, depth, stencil ) {

	console.warn( 'THREE.CubeCamera: .clear() is now .renderTarget.clear().' );
	return this.renderTarget.clear( renderer, color, depth, stencil );

};

ImageUtils.crossOrigin = undefined;

ImageUtils.loadTexture = function ( url, mapping, onLoad, onError ) {

	console.warn( 'THREE.ImageUtils.loadTexture has been deprecated. Use THREE.TextureLoader() instead.' );

	const loader = new TextureLoader();
	loader.setCrossOrigin( this.crossOrigin );

	const texture = loader.load( url, onLoad, undefined, onError );

	if ( mapping ) texture.mapping = mapping;

	return texture;

};

ImageUtils.loadTextureCube = function ( urls, mapping, onLoad, onError ) {

	console.warn( 'THREE.ImageUtils.loadTextureCube has been deprecated. Use THREE.CubeTextureLoader() instead.' );

	const loader = new CubeTextureLoader();
	loader.setCrossOrigin( this.crossOrigin );

	const texture = loader.load( urls, onLoad, undefined, onError );

	if ( mapping ) texture.mapping = mapping;

	return texture;

};

ImageUtils.loadCompressedTexture = function () {

	console.error( 'THREE.ImageUtils.loadCompressedTexture has been removed. Use THREE.DDSLoader instead.' );

};

ImageUtils.loadCompressedTextureCube = function () {

	console.error( 'THREE.ImageUtils.loadCompressedTextureCube has been removed. Use THREE.DDSLoader instead.' );

};

if ( typeof __THREE_DEVTOOLS__ !== 'undefined' ) {

	/* eslint-disable no-undef */
	__THREE_DEVTOOLS__.dispatchEvent( new CustomEvent( 'register', { detail: {
		revision: REVISION,
	} } ) );
	/* eslint-enable no-undef */

}

if ( typeof window !== 'undefined' ) {

	if ( window.__THREE__ ) {

		console.warn( 'WARNING: Multiple instances of Three.js being imported.' );

	} else {

		window.__THREE__ = REVISION;

	}

}

// This set of controls performs orbiting, dollying (zooming), and panning.
// Unlike TrackballControls, it maintains the "up" direction object.up (+Y by default).
//
//    Orbit - left mouse / touch: one-finger move
//    Zoom - middle mouse, or mousewheel / touch: two-finger spread or squish
//    Pan - right mouse, or left mouse + ctrl/meta/shiftKey, or arrow keys / touch: two-finger move

var OrbitControls = function ( object, domElement ) {

	if ( domElement === undefined ) console.warn( 'THREE.OrbitControls: The second parameter "domElement" is now mandatory.' );
	if ( domElement === document ) console.error( 'THREE.OrbitControls: "document" should not be used as the target "domElement". Please use "renderer.domElement" instead.' );

	this.object = object;
	this.domElement = domElement;

	// Set to false to disable this control
	this.enabled = true;

	// "target" sets the location of focus, where the object orbits around
	this.target = new Vector3();

	// How far you can dolly in and out ( PerspectiveCamera only )
	this.minDistance = 0;
	this.maxDistance = Infinity;

	// How far you can zoom in and out ( OrthographicCamera only )
	this.minZoom = 0;
	this.maxZoom = Infinity;

	// How far you can orbit vertically, upper and lower limits.
	// Range is 0 to Math.PI radians.
	this.minPolarAngle = 0; // radians
	this.maxPolarAngle = Math.PI; // radians

	// How far you can orbit horizontally, upper and lower limits.
	// If set, the interval [ min, max ] must be a sub-interval of [ - 2 PI, 2 PI ], with ( max - min < 2 PI )
	this.minAzimuthAngle = - Infinity; // radians
	this.maxAzimuthAngle = Infinity; // radians

	// Set to true to enable damping (inertia)
	// If damping is enabled, you must call controls.update() in your animation loop
	this.enableDamping = false;
	this.dampingFactor = 0.05;

	// This option actually enables dollying in and out; left as "zoom" for backwards compatibility.
	// Set to false to disable zooming
	this.enableZoom = true;
	this.zoomSpeed = 1.0;

	// Set to false to disable rotating
	this.enableRotate = true;
	this.rotateSpeed = 1.0;

	// Set to false to disable panning
	this.enablePan = true;
	this.panSpeed = 1.0;
	this.screenSpacePanning = true; // if false, pan orthogonal to world-space direction camera.up
	this.keyPanSpeed = 7.0;	// pixels moved per arrow key push

	// Set to true to automatically rotate around the target
	// If auto-rotate is enabled, you must call controls.update() in your animation loop
	this.autoRotate = false;
	this.autoRotateSpeed = 2.0; // 30 seconds per orbit when fps is 60

	// The four arrow keys
	this.keys = { LEFT: 'ArrowLeft', UP: 'ArrowUp', RIGHT: 'ArrowRight', BOTTOM: 'ArrowDown' };

	// Mouse buttons
	this.mouseButtons = { LEFT: MOUSE.ROTATE, MIDDLE: MOUSE.DOLLY, RIGHT: MOUSE.PAN };

	// Touch fingers
	this.touches = { ONE: TOUCH.ROTATE, TWO: TOUCH.DOLLY_PAN };

	// for reset
	this.target0 = this.target.clone();
	this.position0 = this.object.position.clone();
	this.zoom0 = this.object.zoom;

	// the target DOM element for key events
	this._domElementKeyEvents = null;

	//
	// public methods
	//

	this.getPolarAngle = function () {

		return spherical.phi;

	};

	this.getAzimuthalAngle = function () {

		return spherical.theta;

	};

	this.listenToKeyEvents = function ( domElement ) {

		domElement.addEventListener( 'keydown', onKeyDown );
		this._domElementKeyEvents = domElement;

	};

	this.saveState = function () {

		scope.target0.copy( scope.target );
		scope.position0.copy( scope.object.position );
		scope.zoom0 = scope.object.zoom;

	};

	this.reset = function () {

		scope.target.copy( scope.target0 );
		scope.object.position.copy( scope.position0 );
		scope.object.zoom = scope.zoom0;

		scope.object.updateProjectionMatrix();
		scope.dispatchEvent( changeEvent );

		scope.update();

		state = STATE.NONE;

	};

	// this method is exposed, but perhaps it would be better if we can make it private...
	this.update = function () {

		var offset = new Vector3();

		// so camera.up is the orbit axis
		var quat = new Quaternion().setFromUnitVectors( object.up, new Vector3( 0, 1, 0 ) );
		var quatInverse = quat.clone().invert();

		var lastPosition = new Vector3();
		var lastQuaternion = new Quaternion();

		var twoPI = 2 * Math.PI;

		return function update() {

			var position = scope.object.position;

			offset.copy( position ).sub( scope.target );

			// rotate offset to "y-axis-is-up" space
			offset.applyQuaternion( quat );

			// angle from z-axis around y-axis
			spherical.setFromVector3( offset );

			if ( scope.autoRotate && state === STATE.NONE ) {

				rotateLeft( getAutoRotationAngle() );

			}

			if ( scope.enableDamping ) {

				spherical.theta += sphericalDelta.theta * scope.dampingFactor;
				spherical.phi += sphericalDelta.phi * scope.dampingFactor;

			} else {

				spherical.theta += sphericalDelta.theta;
				spherical.phi += sphericalDelta.phi;

			}

			// restrict theta to be between desired limits

			var min = scope.minAzimuthAngle;
			var max = scope.maxAzimuthAngle;

			if ( isFinite( min ) && isFinite( max ) ) {

				if ( min < - Math.PI ) min += twoPI; else if ( min > Math.PI ) min -= twoPI;

				if ( max < - Math.PI ) max += twoPI; else if ( max > Math.PI ) max -= twoPI;

				if ( min <= max ) {

					spherical.theta = Math.max( min, Math.min( max, spherical.theta ) );

				} else {

					spherical.theta = ( spherical.theta > ( min + max ) / 2 ) ?
						Math.max( min, spherical.theta ) :
						Math.min( max, spherical.theta );

				}

			}

			// restrict phi to be between desired limits
			spherical.phi = Math.max( scope.minPolarAngle, Math.min( scope.maxPolarAngle, spherical.phi ) );

			spherical.makeSafe();


			spherical.radius *= scale;

			// restrict radius to be between desired limits
			spherical.radius = Math.max( scope.minDistance, Math.min( scope.maxDistance, spherical.radius ) );

			// move target to panned location

			if ( scope.enableDamping === true ) {

				scope.target.addScaledVector( panOffset, scope.dampingFactor );

			} else {

				scope.target.add( panOffset );

			}

			offset.setFromSpherical( spherical );

			// rotate offset back to "camera-up-vector-is-up" space
			offset.applyQuaternion( quatInverse );

			position.copy( scope.target ).add( offset );

			scope.object.lookAt( scope.target );

			if ( scope.enableDamping === true ) {

				sphericalDelta.theta *= ( 1 - scope.dampingFactor );
				sphericalDelta.phi *= ( 1 - scope.dampingFactor );

				panOffset.multiplyScalar( 1 - scope.dampingFactor );

			} else {

				sphericalDelta.set( 0, 0, 0 );

				panOffset.set( 0, 0, 0 );

			}

			scale = 1;

			// update condition is:
			// min(camera displacement, camera rotation in radians)^2 > EPS
			// using small-angle approximation cos(x/2) = 1 - x^2 / 8

			if ( zoomChanged ||
				lastPosition.distanceToSquared( scope.object.position ) > EPS ||
				8 * ( 1 - lastQuaternion.dot( scope.object.quaternion ) ) > EPS ) {

				scope.dispatchEvent( changeEvent );

				lastPosition.copy( scope.object.position );
				lastQuaternion.copy( scope.object.quaternion );
				zoomChanged = false;

				return true;

			}

			return false;

		};

	}();

	this.dispose = function () {

		scope.domElement.removeEventListener( 'contextmenu', onContextMenu );

		scope.domElement.removeEventListener( 'pointerdown', onPointerDown );
		scope.domElement.removeEventListener( 'wheel', onMouseWheel );

		scope.domElement.removeEventListener( 'touchstart', onTouchStart );
		scope.domElement.removeEventListener( 'touchend', onTouchEnd );
		scope.domElement.removeEventListener( 'touchmove', onTouchMove );

		scope.domElement.ownerDocument.removeEventListener( 'pointermove', onPointerMove );
		scope.domElement.ownerDocument.removeEventListener( 'pointerup', onPointerUp );


		if ( scope._domElementKeyEvents !== null ) {

			scope._domElementKeyEvents.removeEventListener( 'keydown', onKeyDown );

		}

		//scope.dispatchEvent( { type: 'dispose' } ); // should this be added here?

	};

	//
	// internals
	//

	var scope = this;

	var changeEvent = { type: 'change' };
	var startEvent = { type: 'start' };
	var endEvent = { type: 'end' };

	var STATE = {
		NONE: - 1,
		ROTATE: 0,
		DOLLY: 1,
		PAN: 2,
		TOUCH_ROTATE: 3,
		TOUCH_PAN: 4,
		TOUCH_DOLLY_PAN: 5,
		TOUCH_DOLLY_ROTATE: 6
	};

	var state = STATE.NONE;

	var EPS = 0.000001;

	// current position in spherical coordinates
	var spherical = new Spherical();
	var sphericalDelta = new Spherical();

	var scale = 1;
	var panOffset = new Vector3();
	var zoomChanged = false;

	var rotateStart = new Vector2();
	var rotateEnd = new Vector2();
	var rotateDelta = new Vector2();

	var panStart = new Vector2();
	var panEnd = new Vector2();
	var panDelta = new Vector2();

	var dollyStart = new Vector2();
	var dollyEnd = new Vector2();
	var dollyDelta = new Vector2();

	function getAutoRotationAngle() {

		return 2 * Math.PI / 60 / 60 * scope.autoRotateSpeed;

	}

	function getZoomScale() {

		return Math.pow( 0.95, scope.zoomSpeed );

	}

	function rotateLeft( angle ) {

		sphericalDelta.theta -= angle;

	}

	function rotateUp( angle ) {

		sphericalDelta.phi -= angle;

	}

	var panLeft = function () {

		var v = new Vector3();

		return function panLeft( distance, objectMatrix ) {

			v.setFromMatrixColumn( objectMatrix, 0 ); // get X column of objectMatrix
			v.multiplyScalar( - distance );

			panOffset.add( v );

		};

	}();

	var panUp = function () {

		var v = new Vector3();

		return function panUp( distance, objectMatrix ) {

			if ( scope.screenSpacePanning === true ) {

				v.setFromMatrixColumn( objectMatrix, 1 );

			} else {

				v.setFromMatrixColumn( objectMatrix, 0 );
				v.crossVectors( scope.object.up, v );

			}

			v.multiplyScalar( distance );

			panOffset.add( v );

		};

	}();

	// deltaX and deltaY are in pixels; right and down are positive
	var pan = function () {

		var offset = new Vector3();

		return function pan( deltaX, deltaY ) {

			var element = scope.domElement;

			if ( scope.object.isPerspectiveCamera ) {

				// perspective
				var position = scope.object.position;
				offset.copy( position ).sub( scope.target );
				var targetDistance = offset.length();

				// half of the fov is center to top of screen
				targetDistance *= Math.tan( ( scope.object.fov / 2 ) * Math.PI / 180.0 );

				// we use only clientHeight here so aspect ratio does not distort speed
				panLeft( 2 * deltaX * targetDistance / element.clientHeight, scope.object.matrix );
				panUp( 2 * deltaY * targetDistance / element.clientHeight, scope.object.matrix );

			} else if ( scope.object.isOrthographicCamera ) {

				// orthographic
				panLeft( deltaX * ( scope.object.right - scope.object.left ) / scope.object.zoom / element.clientWidth, scope.object.matrix );
				panUp( deltaY * ( scope.object.top - scope.object.bottom ) / scope.object.zoom / element.clientHeight, scope.object.matrix );

			} else {

				// camera neither orthographic nor perspective
				console.warn( 'WARNING: OrbitControls.js encountered an unknown camera type - pan disabled.' );
				scope.enablePan = false;

			}

		};

	}();

	function dollyOut( dollyScale ) {

		if ( scope.object.isPerspectiveCamera ) {

			scale /= dollyScale;

		} else if ( scope.object.isOrthographicCamera ) {

			scope.object.zoom = Math.max( scope.minZoom, Math.min( scope.maxZoom, scope.object.zoom * dollyScale ) );
			scope.object.updateProjectionMatrix();
			zoomChanged = true;

		} else {

			console.warn( 'WARNING: OrbitControls.js encountered an unknown camera type - dolly/zoom disabled.' );
			scope.enableZoom = false;

		}

	}

	function dollyIn( dollyScale ) {

		if ( scope.object.isPerspectiveCamera ) {

			scale *= dollyScale;

		} else if ( scope.object.isOrthographicCamera ) {

			scope.object.zoom = Math.max( scope.minZoom, Math.min( scope.maxZoom, scope.object.zoom / dollyScale ) );
			scope.object.updateProjectionMatrix();
			zoomChanged = true;

		} else {

			console.warn( 'WARNING: OrbitControls.js encountered an unknown camera type - dolly/zoom disabled.' );
			scope.enableZoom = false;

		}

	}

	//
	// event callbacks - update the object state
	//

	function handleMouseDownRotate( event ) {

		rotateStart.set( event.clientX, event.clientY );

	}

	function handleMouseDownDolly( event ) {

		dollyStart.set( event.clientX, event.clientY );

	}

	function handleMouseDownPan( event ) {

		panStart.set( event.clientX, event.clientY );

	}

	function handleMouseMoveRotate( event ) {

		rotateEnd.set( event.clientX, event.clientY );

		rotateDelta.subVectors( rotateEnd, rotateStart ).multiplyScalar( scope.rotateSpeed );

		var element = scope.domElement;

		rotateLeft( 2 * Math.PI * rotateDelta.x / element.clientHeight ); // yes, height

		rotateUp( 2 * Math.PI * rotateDelta.y / element.clientHeight );

		rotateStart.copy( rotateEnd );

		scope.update();

	}

	function handleMouseMoveDolly( event ) {

		dollyEnd.set( event.clientX, event.clientY );

		dollyDelta.subVectors( dollyEnd, dollyStart );

		if ( dollyDelta.y > 0 ) {

			dollyOut( getZoomScale() );

		} else if ( dollyDelta.y < 0 ) {

			dollyIn( getZoomScale() );

		}

		dollyStart.copy( dollyEnd );

		scope.update();

	}

	function handleMouseMovePan( event ) {

		panEnd.set( event.clientX, event.clientY );

		panDelta.subVectors( panEnd, panStart ).multiplyScalar( scope.panSpeed );

		pan( panDelta.x, panDelta.y );

		panStart.copy( panEnd );

		scope.update();

	}

	function handleMouseWheel( event ) {

		if ( event.deltaY < 0 ) {

			dollyIn( getZoomScale() );

		} else if ( event.deltaY > 0 ) {

			dollyOut( getZoomScale() );

		}

		scope.update();

	}

	function handleKeyDown( event ) {

		var needsUpdate = false;

		switch ( event.code ) {

			case scope.keys.UP:
				pan( 0, scope.keyPanSpeed );
				needsUpdate = true;
				break;

			case scope.keys.BOTTOM:
				pan( 0, - scope.keyPanSpeed );
				needsUpdate = true;
				break;

			case scope.keys.LEFT:
				pan( scope.keyPanSpeed, 0 );
				needsUpdate = true;
				break;

			case scope.keys.RIGHT:
				pan( - scope.keyPanSpeed, 0 );
				needsUpdate = true;
				break;

		}

		if ( needsUpdate ) {

			// prevent the browser from scrolling on cursor keys
			event.preventDefault();

			scope.update();

		}


	}

	function handleTouchStartRotate( event ) {

		if ( event.touches.length == 1 ) {

			rotateStart.set( event.touches[ 0 ].pageX, event.touches[ 0 ].pageY );

		} else {

			var x = 0.5 * ( event.touches[ 0 ].pageX + event.touches[ 1 ].pageX );
			var y = 0.5 * ( event.touches[ 0 ].pageY + event.touches[ 1 ].pageY );

			rotateStart.set( x, y );

		}

	}

	function handleTouchStartPan( event ) {

		if ( event.touches.length == 1 ) {

			panStart.set( event.touches[ 0 ].pageX, event.touches[ 0 ].pageY );

		} else {

			var x = 0.5 * ( event.touches[ 0 ].pageX + event.touches[ 1 ].pageX );
			var y = 0.5 * ( event.touches[ 0 ].pageY + event.touches[ 1 ].pageY );

			panStart.set( x, y );

		}

	}

	function handleTouchStartDolly( event ) {

		var dx = event.touches[ 0 ].pageX - event.touches[ 1 ].pageX;
		var dy = event.touches[ 0 ].pageY - event.touches[ 1 ].pageY;

		var distance = Math.sqrt( dx * dx + dy * dy );

		dollyStart.set( 0, distance );

	}

	function handleTouchStartDollyPan( event ) {

		if ( scope.enableZoom ) handleTouchStartDolly( event );

		if ( scope.enablePan ) handleTouchStartPan( event );

	}

	function handleTouchStartDollyRotate( event ) {

		if ( scope.enableZoom ) handleTouchStartDolly( event );

		if ( scope.enableRotate ) handleTouchStartRotate( event );

	}

	function handleTouchMoveRotate( event ) {

		if ( event.touches.length == 1 ) {

			rotateEnd.set( event.touches[ 0 ].pageX, event.touches[ 0 ].pageY );

		} else {

			var x = 0.5 * ( event.touches[ 0 ].pageX + event.touches[ 1 ].pageX );
			var y = 0.5 * ( event.touches[ 0 ].pageY + event.touches[ 1 ].pageY );

			rotateEnd.set( x, y );

		}

		rotateDelta.subVectors( rotateEnd, rotateStart ).multiplyScalar( scope.rotateSpeed );

		var element = scope.domElement;

		rotateLeft( 2 * Math.PI * rotateDelta.x / element.clientHeight ); // yes, height

		rotateUp( 2 * Math.PI * rotateDelta.y / element.clientHeight );

		rotateStart.copy( rotateEnd );

	}

	function handleTouchMovePan( event ) {

		if ( event.touches.length == 1 ) {

			panEnd.set( event.touches[ 0 ].pageX, event.touches[ 0 ].pageY );

		} else {

			var x = 0.5 * ( event.touches[ 0 ].pageX + event.touches[ 1 ].pageX );
			var y = 0.5 * ( event.touches[ 0 ].pageY + event.touches[ 1 ].pageY );

			panEnd.set( x, y );

		}

		panDelta.subVectors( panEnd, panStart ).multiplyScalar( scope.panSpeed );

		pan( panDelta.x, panDelta.y );

		panStart.copy( panEnd );

	}

	function handleTouchMoveDolly( event ) {

		var dx = event.touches[ 0 ].pageX - event.touches[ 1 ].pageX;
		var dy = event.touches[ 0 ].pageY - event.touches[ 1 ].pageY;

		var distance = Math.sqrt( dx * dx + dy * dy );

		dollyEnd.set( 0, distance );

		dollyDelta.set( 0, Math.pow( dollyEnd.y / dollyStart.y, scope.zoomSpeed ) );

		dollyOut( dollyDelta.y );

		dollyStart.copy( dollyEnd );

	}

	function handleTouchMoveDollyPan( event ) {

		if ( scope.enableZoom ) handleTouchMoveDolly( event );

		if ( scope.enablePan ) handleTouchMovePan( event );

	}

	function handleTouchMoveDollyRotate( event ) {

		if ( scope.enableZoom ) handleTouchMoveDolly( event );

		if ( scope.enableRotate ) handleTouchMoveRotate( event );

	}

	//
	// event handlers - FSM: listen for events and reset state
	//

	function onPointerDown( event ) {

		if ( scope.enabled === false ) return;

		switch ( event.pointerType ) {

			case 'mouse':
			case 'pen':
				onMouseDown( event );
				break;

			// TODO touch

		}

	}

	function onPointerMove( event ) {

		if ( scope.enabled === false ) return;

		switch ( event.pointerType ) {

			case 'mouse':
			case 'pen':
				onMouseMove( event );
				break;

			// TODO touch

		}

	}

	function onPointerUp( event ) {

		switch ( event.pointerType ) {

			case 'mouse':
			case 'pen':
				onMouseUp();
				break;

			// TODO touch

		}

	}

	function onMouseDown( event ) {

		// Prevent the browser from scrolling.
		event.preventDefault();

		// Manually set the focus since calling preventDefault above
		// prevents the browser from setting it automatically.

		scope.domElement.focus ? scope.domElement.focus() : window.focus();

		var mouseAction;

		switch ( event.button ) {

			case 0:

				mouseAction = scope.mouseButtons.LEFT;
				break;

			case 1:

				mouseAction = scope.mouseButtons.MIDDLE;
				break;

			case 2:

				mouseAction = scope.mouseButtons.RIGHT;
				break;

			default:

				mouseAction = - 1;

		}

		switch ( mouseAction ) {

			case MOUSE.DOLLY:

				if ( scope.enableZoom === false ) return;

				handleMouseDownDolly( event );

				state = STATE.DOLLY;

				break;

			case MOUSE.ROTATE:

				if ( event.ctrlKey || event.metaKey || event.shiftKey ) {

					if ( scope.enablePan === false ) return;

					handleMouseDownPan( event );

					state = STATE.PAN;

				} else {

					if ( scope.enableRotate === false ) return;

					handleMouseDownRotate( event );

					state = STATE.ROTATE;

				}

				break;

			case MOUSE.PAN:

				if ( event.ctrlKey || event.metaKey || event.shiftKey ) {

					if ( scope.enableRotate === false ) return;

					handleMouseDownRotate( event );

					state = STATE.ROTATE;

				} else {

					if ( scope.enablePan === false ) return;

					handleMouseDownPan( event );

					state = STATE.PAN;

				}

				break;

			default:

				state = STATE.NONE;

		}

		if ( state !== STATE.NONE ) {

			scope.domElement.ownerDocument.addEventListener( 'pointermove', onPointerMove );
			scope.domElement.ownerDocument.addEventListener( 'pointerup', onPointerUp );

			scope.dispatchEvent( startEvent );

		}

	}

	function onMouseMove( event ) {

		if ( scope.enabled === false ) return;

		event.preventDefault();

		switch ( state ) {

			case STATE.ROTATE:

				if ( scope.enableRotate === false ) return;

				handleMouseMoveRotate( event );

				break;

			case STATE.DOLLY:

				if ( scope.enableZoom === false ) return;

				handleMouseMoveDolly( event );

				break;

			case STATE.PAN:

				if ( scope.enablePan === false ) return;

				handleMouseMovePan( event );

				break;

		}

	}

	function onMouseUp( event ) {

		scope.domElement.ownerDocument.removeEventListener( 'pointermove', onPointerMove );
		scope.domElement.ownerDocument.removeEventListener( 'pointerup', onPointerUp );

		if ( scope.enabled === false ) return;

		scope.dispatchEvent( endEvent );

		state = STATE.NONE;

	}

	function onMouseWheel( event ) {

		if ( scope.enabled === false || scope.enableZoom === false || ( state !== STATE.NONE && state !== STATE.ROTATE ) ) return;

		event.preventDefault();

		scope.dispatchEvent( startEvent );

		handleMouseWheel( event );

		scope.dispatchEvent( endEvent );

	}

	function onKeyDown( event ) {

		if ( scope.enabled === false || scope.enablePan === false ) return;

		handleKeyDown( event );

	}

	function onTouchStart( event ) {

		if ( scope.enabled === false ) return;

		event.preventDefault(); // prevent scrolling

		switch ( event.touches.length ) {

			case 1:

				switch ( scope.touches.ONE ) {

					case TOUCH.ROTATE:

						if ( scope.enableRotate === false ) return;

						handleTouchStartRotate( event );

						state = STATE.TOUCH_ROTATE;

						break;

					case TOUCH.PAN:

						if ( scope.enablePan === false ) return;

						handleTouchStartPan( event );

						state = STATE.TOUCH_PAN;

						break;

					default:

						state = STATE.NONE;

				}

				break;

			case 2:

				switch ( scope.touches.TWO ) {

					case TOUCH.DOLLY_PAN:

						if ( scope.enableZoom === false && scope.enablePan === false ) return;

						handleTouchStartDollyPan( event );

						state = STATE.TOUCH_DOLLY_PAN;

						break;

					case TOUCH.DOLLY_ROTATE:

						if ( scope.enableZoom === false && scope.enableRotate === false ) return;

						handleTouchStartDollyRotate( event );

						state = STATE.TOUCH_DOLLY_ROTATE;

						break;

					default:

						state = STATE.NONE;

				}

				break;

			default:

				state = STATE.NONE;

		}

		if ( state !== STATE.NONE ) {

			scope.dispatchEvent( startEvent );

		}

	}

	function onTouchMove( event ) {

		if ( scope.enabled === false ) return;

		event.preventDefault(); // prevent scrolling

		switch ( state ) {

			case STATE.TOUCH_ROTATE:

				if ( scope.enableRotate === false ) return;

				handleTouchMoveRotate( event );

				scope.update();

				break;

			case STATE.TOUCH_PAN:

				if ( scope.enablePan === false ) return;

				handleTouchMovePan( event );

				scope.update();

				break;

			case STATE.TOUCH_DOLLY_PAN:

				if ( scope.enableZoom === false && scope.enablePan === false ) return;

				handleTouchMoveDollyPan( event );

				scope.update();

				break;

			case STATE.TOUCH_DOLLY_ROTATE:

				if ( scope.enableZoom === false && scope.enableRotate === false ) return;

				handleTouchMoveDollyRotate( event );

				scope.update();

				break;

			default:

				state = STATE.NONE;

		}

	}

	function onTouchEnd( event ) {

		if ( scope.enabled === false ) return;

		scope.dispatchEvent( endEvent );

		state = STATE.NONE;

	}

	function onContextMenu( event ) {

		if ( scope.enabled === false ) return;

		event.preventDefault();

	}

	//

	scope.domElement.addEventListener( 'contextmenu', onContextMenu );

	scope.domElement.addEventListener( 'pointerdown', onPointerDown );
	scope.domElement.addEventListener( 'wheel', onMouseWheel );

	scope.domElement.addEventListener( 'touchstart', onTouchStart );
	scope.domElement.addEventListener( 'touchend', onTouchEnd );
	scope.domElement.addEventListener( 'touchmove', onTouchMove );

	// force an update at start

	this.update();

};

OrbitControls.prototype = Object.create( EventDispatcher.prototype );
OrbitControls.prototype.constructor = OrbitControls;


// This set of controls performs orbiting, dollying (zooming), and panning.
// Unlike TrackballControls, it maintains the "up" direction object.up (+Y by default).
// This is very similar to OrbitControls, another set of touch behavior
//
//    Orbit - right mouse, or left mouse + ctrl/meta/shiftKey / touch: two-finger rotate
//    Zoom - middle mouse, or mousewheel / touch: two-finger spread or squish
//    Pan - left mouse, or arrow keys / touch: one-finger move

var MapControls = function ( object, domElement ) {

	OrbitControls.call( this, object, domElement );

	this.screenSpacePanning = false; // pan orthogonal to world-space direction camera.up

	this.mouseButtons.LEFT = MOUSE.PAN;
	this.mouseButtons.RIGHT = MOUSE.ROTATE;

	this.touches.ONE = TOUCH.PAN;
	this.touches.TWO = TOUCH.DOLLY_ROTATE;

};

MapControls.prototype = Object.create( EventDispatcher.prototype );
MapControls.prototype.constructor = MapControls;

var glyphs = {
	"0": {
		ha: 794,
		x_min: 69,
		x_max: 725,
		o: "m 725 497 q 644 113 725 240 q 396 -14 563 -14 q 153 116 236 -14 q 69 497 69 246 q 150 882 69 756 q 396 1007 231 1007 q 641 876 557 1007 q 725 497 725 745 m 183 497 q 234 182 183 281 q 396 83 285 83 q 559 183 509 83 q 610 497 610 283 q 559 810 610 711 q 396 909 509 909 q 234 811 285 909 q 183 497 183 713 z "
	},
	"1": {
		ha: 794,
		x_min: 127,
		x_max: 485,
		o: "m 485 0 l 375 0 l 375 707 q 380 873 375 795 q 349 844 366 859 q 187 711 331 828 l 127 789 l 390 991 l 485 991 l 485 0 z "
	},
	"2": {
		ha: 794,
		x_min: 68,
		x_max: 720,
		o: "m 720 0 l 68 0 l 68 97 l 329 359 q 486 532 448 480 q 543 632 524 583 q 562 737 562 681 q 514 863 562 817 q 380 909 466 909 q 263 889 319 909 q 140 815 208 869 l 81 892 q 379 1006 218 1006 q 598 934 519 1006 q 677 742 677 863 q 625 555 677 648 q 427 322 572 463 l 210 110 l 210 104 l 720 104 l 720 0 z "
	},
	"3": {
		ha: 794,
		x_min: 64,
		x_max: 713,
		o: "m 682 758 q 629 603 682 663 q 478 522 576 543 l 478 517 q 655 441 597 502 q 713 281 713 380 q 614 63 713 139 q 335 -14 516 -14 q 191 -2 256 -14 q 64 40 125 10 l 64 147 q 201 99 128 115 q 339 82 274 82 q 596 283 596 82 q 313 464 596 464 l 215 464 l 215 561 l 314 561 q 498 612 430 561 q 566 754 566 663 q 516 868 566 827 q 380 909 466 909 q 258 892 315 909 q 126 827 200 874 l 69 903 q 210 978 130 951 q 378 1006 290 1006 q 602 940 522 1006 q 682 758 682 873 z "
	},
	"4": {
		ha: 794,
		x_min: 29,
		x_max: 766,
		o: "m 766 228 l 619 228 l 619 0 l 511 0 l 511 228 l 29 228 l 29 326 l 500 997 l 619 997 l 619 330 l 766 330 l 766 228 m 511 330 l 511 660 q 518 879 511 757 l 513 879 q 452 771 480 814 l 142 330 l 511 330 z "
	},
	"5": {
		ha: 794,
		x_min: 90,
		x_max: 714,
		o: "m 378 606 q 624 528 534 606 q 714 315 714 450 q 616 74 714 161 q 346 -14 518 -14 q 90 40 178 -14 l 90 149 q 208 101 138 118 q 347 83 279 83 q 533 140 467 83 q 599 302 599 196 q 345 510 599 510 q 172 490 280 510 l 114 528 l 151 991 l 644 991 l 644 888 l 248 888 l 222 590 q 378 606 300 606 z "
	},
	"6": {
		ha: 794,
		x_min: 79,
		x_max: 726,
		o: "m 79 424 q 193 861 79 716 q 529 1006 307 1006 q 650 993 606 1006 l 650 896 q 530 913 597 913 q 287 813 371 913 q 195 501 203 714 l 203 501 q 439 618 277 618 q 649 537 572 618 q 726 318 726 456 q 642 75 726 163 q 414 -14 557 -14 q 170 102 260 -14 q 79 424 79 218 m 412 82 q 562 143 509 82 q 615 318 615 203 q 566 473 615 416 q 418 529 516 529 q 306 504 357 529 q 225 435 255 479 q 195 343 195 391 q 222 213 195 273 q 299 117 249 153 q 412 82 349 82 z "
	},
	"7": {
		ha: 794,
		x_min: 64,
		x_max: 724,
		o: "m 193 0 l 604 888 l 64 888 l 64 991 l 724 991 l 724 901 l 318 0 l 193 0 z "
	},
	"8": {
		ha: 794,
		x_min: 71,
		x_max: 722,
		o: "m 396 1006 q 611 943 532 1006 q 690 768 690 880 q 645 635 690 695 q 500 525 600 574 q 671 404 621 467 q 722 257 722 340 q 636 60 722 134 q 400 -14 550 -14 q 156 56 241 -14 q 71 253 71 125 q 278 518 71 423 q 144 632 184 571 q 103 770 103 694 q 183 942 103 878 q 396 1006 262 1006 m 182 250 q 238 123 182 169 q 397 78 295 78 q 555 125 498 78 q 611 256 611 173 q 558 373 611 321 q 374 472 505 424 q 227 376 273 429 q 182 250 182 323 m 395 914 q 262 873 310 914 q 214 765 214 833 q 254 658 214 703 q 401 568 294 613 q 539 656 498 609 q 579 765 579 703 q 530 874 579 833 q 395 914 481 914 z "
	},
	"9": {
		ha: 794,
		x_min: 72,
		x_max: 720,
		o: "m 720 568 q 269 -14 720 -14 q 144 0 191 -14 l 144 97 q 268 79 199 79 q 514 180 431 79 q 604 489 597 281 l 596 489 q 497 403 559 433 q 358 374 435 374 q 149 452 227 374 q 72 672 72 531 q 158 916 72 827 q 386 1006 245 1006 q 563 954 487 1006 q 679 803 638 902 q 720 568 720 703 m 386 909 q 236 847 289 909 q 183 673 183 785 q 232 520 183 576 q 380 464 281 464 q 494 489 442 464 q 576 557 546 514 q 606 648 606 601 q 578 780 606 720 q 500 875 550 840 q 386 909 450 909 z "
	},
	" ": {
		ha: 361,
		x_min: 0,
		x_max: 0,
		o: ""
	},
	"!": {
		ha: 371,
		x_min: 103,
		x_max: 267,
		o: "m 221 273 l 150 273 l 115 991 l 256 991 l 221 273 m 103 72 q 184 164 103 164 q 245 140 224 164 q 267 72 267 117 q 245 4 267 28 q 184 -20 223 -20 q 126 2 149 -20 q 103 72 103 23 z "
	},
	"\"": {
		ha: 557,
		x_min: 90,
		x_max: 467,
		o: "m 216 991 l 189 633 l 118 633 l 90 991 l 216 991 m 467 991 l 439 633 l 368 633 l 340 991 l 467 991 z "
	},
	"#": {
		ha: 897,
		x_min: 35,
		x_max: 861,
		o: "m 665 610 l 621 379 l 812 379 l 812 292 l 604 292 l 547 0 l 454 0 l 511 292 l 306 292 l 250 0 l 158 0 l 212 292 l 35 292 l 35 379 l 229 379 l 275 610 l 87 610 l 87 696 l 290 696 l 346 991 l 440 991 l 385 696 l 591 696 l 648 991 l 739 991 l 682 696 l 861 696 l 861 610 l 665 610 m 322 379 l 528 379 l 572 610 l 367 610 l 322 379 z "
	},
	$: {
		ha: 794,
		x_min: 89,
		x_max: 703,
		o: "m 703 304 q 633 152 703 212 q 440 77 564 92 l 440 -81 l 353 -81 l 353 71 q 205 82 277 71 q 89 115 134 94 l 89 221 q 219 180 145 196 q 353 164 292 164 l 353 463 q 158 565 214 507 q 102 715 102 623 q 171 861 102 804 q 353 930 239 918 l 353 1054 l 440 1054 l 440 932 q 681 882 565 928 l 646 793 q 440 840 545 833 l 440 546 q 600 480 547 512 q 677 406 652 448 q 703 304 703 364 m 587 296 q 557 375 587 345 q 440 435 527 405 l 440 171 q 587 296 587 191 m 216 717 q 247 634 216 665 q 353 575 277 603 l 353 838 q 251 795 286 827 q 216 717 216 764 z "
	},
	"%": {
		ha: 1143,
		x_min: 71,
		x_max: 1072,
		o: "m 164 696 q 189 523 164 581 q 271 465 214 465 q 382 696 382 465 q 271 925 382 925 q 189 868 214 925 q 164 696 164 811 m 475 696 q 423 462 475 541 q 271 383 371 383 q 123 464 176 383 q 71 696 71 545 q 121 928 71 850 q 271 1006 172 1006 q 422 925 369 1006 q 475 696 475 844 m 761 298 q 786 125 761 182 q 868 68 811 68 q 952 124 925 68 q 979 298 979 181 q 952 470 979 414 q 868 526 925 526 q 786 470 811 526 q 761 298 761 414 m 1072 298 q 1020 65 1072 144 q 868 -14 968 -14 q 720 67 772 -14 q 668 298 668 148 q 719 530 668 452 q 868 608 769 608 q 1018 529 964 608 q 1072 298 1072 449 m 897 991 l 347 0 l 248 0 l 798 991 l 897 991 z "
	},
	"&": {
		ha: 1014,
		x_min: 77,
		x_max: 1011,
		o: "m 281 794 q 305 705 281 747 q 389 603 330 663 q 510 697 476 654 q 545 796 545 740 q 510 881 545 848 q 416 914 475 914 q 318 882 356 914 q 281 794 281 849 m 386 87 q 657 192 549 87 l 361 479 q 254 403 286 433 q 208 338 223 373 q 193 260 193 304 q 246 134 193 180 q 386 87 298 87 m 77 257 q 124 413 77 345 q 293 550 171 481 q 215 648 235 614 q 182 717 194 681 q 170 791 170 753 q 236 950 170 893 q 421 1007 302 1007 q 594 950 531 1007 q 657 793 657 894 q 611 659 657 720 q 458 534 565 597 l 734 269 q 795 368 772 311 q 833 492 818 425 l 947 492 q 808 197 901 298 l 1011 0 l 856 0 l 730 121 q 568 18 650 49 q 383 -14 485 -14 q 157 58 237 -14 q 77 257 77 130 z "
	},
	"'": {
		ha: 307,
		x_min: 90,
		x_max: 216,
		o: "m 216 991 l 189 633 l 118 633 l 90 991 l 216 991 z "
	},
	"(": {
		ha: 411,
		x_min: 56,
		x_max: 370,
		o: "m 56 380 q 108 717 56 560 q 260 991 161 873 l 370 991 q 223 704 272 861 q 174 382 174 547 q 224 64 174 219 q 368 -220 274 -92 l 260 -220 q 108 50 160 -104 q 56 380 56 203 z "
	},
	")": {
		ha: 411,
		x_min: 41,
		x_max: 355,
		o: "m 355 380 q 303 48 355 202 q 151 -220 250 -106 l 43 -220 q 187 63 137 -92 q 237 382 237 219 q 188 704 237 547 q 41 991 139 861 l 151 991 q 303 716 251 873 q 355 380 355 559 z "
	},
	"*": {
		ha: 766,
		x_min: 58,
		x_max: 704,
		o: "m 446 1055 l 416 787 l 686 863 l 704 739 l 446 718 l 614 497 l 497 433 l 378 679 l 269 433 l 150 497 l 314 718 l 58 739 l 78 863 l 343 787 l 314 1055 l 446 1055 z "
	},
	"+": {
		ha: 794,
		x_min: 71,
		x_max: 722,
		o: "m 443 536 l 722 536 l 722 443 l 443 443 l 443 154 l 349 154 l 349 443 l 71 443 l 71 536 l 349 536 l 349 827 l 443 827 l 443 536 z "
	},
	",": {
		ha: 340,
		x_min: 43,
		x_max: 248,
		o: "m 237 161 l 248 146 q 197 -12 230 78 q 127 -179 163 -102 l 43 -179 q 83 -5 61 -109 q 114 161 105 99 l 237 161 z "
	},
	"-": {
		ha: 447,
		x_min: 57,
		x_max: 390,
		o: "m 57 321 l 57 424 l 390 424 l 390 321 l 57 321 z "
	},
	".": {
		ha: 370,
		x_min: 103,
		x_max: 267,
		o: "m 103 72 q 124 141 103 117 q 183 164 144 164 q 244 141 222 164 q 267 72 267 117 q 244 4 267 28 q 183 -20 222 -20 q 126 2 149 -20 q 103 72 103 23 z "
	},
	"/": {
		ha: 510,
		x_min: 14,
		x_max: 496,
		o: "m 496 991 l 126 0 l 14 0 l 383 991 l 496 991 z "
	},
	":": {
		ha: 370,
		x_min: 103,
		x_max: 267,
		o: "m 103 72 q 124 141 103 117 q 183 164 144 164 q 244 141 222 164 q 267 72 267 117 q 244 4 267 28 q 183 -20 222 -20 q 126 2 149 -20 q 103 72 103 23 m 103 671 q 183 762 103 762 q 267 671 267 762 q 244 603 267 627 q 183 579 222 579 q 126 601 149 579 q 103 671 103 622 z "
	},
	";": {
		ha: 370,
		x_min: 43,
		x_max: 264,
		o: "m 237 161 l 248 146 q 197 -12 230 78 q 127 -179 163 -102 l 43 -179 q 83 -5 61 -109 q 114 161 105 99 l 237 161 m 100 671 q 180 762 100 762 q 264 671 264 762 q 241 603 264 627 q 180 579 219 579 q 120 603 141 579 q 100 671 100 627 z "
	},
	"<": {
		ha: 794,
		x_min: 71,
		x_max: 722,
		o: "m 722 164 l 71 450 l 71 517 l 722 842 l 722 741 l 192 489 l 722 267 l 722 164 z "
	},
	"=": {
		ha: 794,
		x_min: 81,
		x_max: 711,
		o: "m 81 582 l 81 675 l 711 675 l 711 582 l 81 582 m 81 304 l 81 397 l 711 397 l 711 304 l 81 304 z "
	},
	">": {
		ha: 794,
		x_min: 71,
		x_max: 722,
		o: "m 71 267 l 602 488 l 71 741 l 71 842 l 722 517 l 722 450 l 71 164 l 71 267 z "
	},
	"?": {
		ha: 596,
		x_min: 18,
		x_max: 559,
		o: "m 196 273 l 196 310 q 220 440 196 389 q 311 549 245 492 q 428 666 404 627 q 452 761 452 705 q 407 868 452 830 q 279 905 363 905 q 175 893 226 905 q 58 847 124 880 l 18 939 q 286 1006 146 1006 q 488 942 416 1006 q 559 762 559 878 q 546 675 559 713 q 507 604 533 637 q 396 496 481 570 q 306 399 328 437 q 283 296 283 360 l 283 273 l 196 273 m 163 72 q 244 164 163 164 q 305 140 283 164 q 326 72 326 117 q 304 4 326 28 q 244 -20 283 -20 q 186 2 209 -20 q 163 72 163 23 z "
	},
	"@": {
		ha: 1249,
		x_min: 82,
		x_max: 1166,
		o: "m 1166 494 q 1137 318 1166 398 q 1053 194 1107 238 q 928 150 998 150 q 829 185 869 150 q 782 275 789 220 l 777 275 q 699 183 749 216 q 579 150 648 150 q 420 219 477 150 q 363 408 363 289 q 443 633 363 547 q 653 720 523 720 q 758 711 699 720 q 863 688 816 703 l 846 369 l 846 354 q 936 233 846 233 q 1036 306 998 233 q 1075 496 1075 379 q 1025 711 1075 618 q 882 853 975 803 q 669 903 789 903 q 406 840 518 903 q 235 661 294 777 q 176 392 176 545 q 286 74 176 185 q 600 -37 395 -37 q 896 21 743 -37 l 896 -69 q 600 -126 766 -126 q 218 9 354 -126 q 82 387 82 144 q 155 701 82 564 q 361 915 227 839 q 669 990 496 990 q 929 929 815 990 q 1104 754 1042 867 q 1166 494 1166 642 m 465 406 q 597 233 465 233 q 750 446 738 233 l 760 623 q 653 636 711 636 q 515 575 565 636 q 465 406 465 514 z "
	},
	A: {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 z "
	},
	B: {
		ha: 900,
		x_min: 136,
		x_max: 823,
		o: "m 136 991 l 416 991 q 702 932 614 991 q 790 746 790 873 q 741 601 790 658 q 597 526 692 543 l 597 519 q 823 282 823 481 q 733 75 823 149 q 482 0 644 0 l 136 0 l 136 991 m 252 567 l 441 567 q 617 605 564 567 q 671 734 671 644 q 611 855 671 818 q 421 892 551 892 l 252 892 l 252 567 m 252 469 l 252 98 l 458 98 q 639 145 578 98 q 700 290 700 191 q 638 426 700 382 q 449 469 576 469 l 252 469 z "
	},
	C: {
		ha: 876,
		x_min: 85,
		x_max: 835,
		o: "m 561 903 q 303 794 397 903 q 208 496 208 685 q 299 195 208 301 q 559 89 391 89 q 796 126 663 89 l 796 25 q 542 -14 693 -14 q 204 119 323 -14 q 85 497 85 252 q 142 766 85 650 q 308 943 199 881 q 562 1006 416 1006 q 835 949 718 1006 l 786 850 q 561 903 673 903 z "
	},
	D: {
		ha: 1013,
		x_min: 136,
		x_max: 928,
		o: "m 928 505 q 794 130 928 260 q 411 0 661 0 l 136 0 l 136 991 l 440 991 q 800 863 671 991 q 928 505 928 735 m 806 501 q 708 793 806 695 q 419 892 611 892 l 252 892 l 252 100 l 392 100 q 702 201 598 100 q 806 501 806 302 z "
	},
	E: {
		ha: 772,
		x_min: 136,
		x_max: 689,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 z "
	},
	F: {
		ha: 717,
		x_min: 136,
		x_max: 689,
		o: "m 252 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 526 l 663 526 l 663 424 l 252 424 l 252 0 z "
	},
	G: {
		ha: 1011,
		x_min: 85,
		x_max: 909,
		o: "m 572 519 l 909 519 l 909 37 q 749 -1 831 12 q 561 -14 668 -14 q 210 120 336 -14 q 85 496 85 254 q 147 767 85 650 q 326 944 209 883 q 599 1006 442 1006 q 895 947 758 1006 l 850 846 q 591 903 715 903 q 309 795 410 903 q 207 496 207 687 q 305 191 207 295 q 593 87 403 87 q 794 111 696 87 l 794 416 l 572 416 l 572 519 z "
	},
	H: {
		ha: 1025,
		x_min: 136,
		x_max: 889,
		o: "m 889 0 l 774 0 l 774 467 l 252 467 l 252 0 l 136 0 l 136 991 l 252 991 l 252 570 l 774 570 l 774 991 l 889 991 l 889 0 z "
	},
	J: {
		ha: 371,
		x_min: -109,
		x_max: 244,
		o: "m -8 -261 q -109 -243 -72 -261 l -109 -144 q -8 -158 -60 -158 q 94 -117 59 -158 q 129 0 129 -77 l 129 991 l 244 991 l 244 9 q 179 -190 244 -119 q -8 -261 114 -261 z "
	},
	K: {
		ha: 852,
		x_min: 136,
		x_max: 852,
		o: "m 852 0 l 717 0 l 355 481 l 252 389 l 252 0 l 136 0 l 136 991 l 252 991 l 252 500 l 701 991 l 838 991 l 439 561 l 852 0 z "
	},
	L: {
		ha: 721,
		x_min: 136,
		x_max: 689,
		o: "m 136 0 l 136 991 l 252 991 l 252 104 l 689 104 l 689 0 l 136 0 z "
	},
	M: {
		ha: 1254,
		x_min: 136,
		x_max: 1118,
		o: "m 575 0 l 239 879 l 233 879 q 243 631 243 774 l 243 0 l 136 0 l 136 991 l 310 991 l 624 174 l 629 174 l 946 991 l 1118 991 l 1118 0 l 1003 0 l 1003 639 q 1013 878 1003 749 l 1007 878 l 668 0 l 575 0 z "
	},
	N: {
		ha: 1047,
		x_min: 136,
		x_max: 911,
		o: "m 911 0 l 779 0 l 237 832 l 232 832 q 243 564 243 686 l 243 0 l 136 0 l 136 991 l 267 991 l 807 163 l 812 163 q 806 280 811 181 q 803 422 802 380 l 803 991 l 911 991 l 911 0 z "
	},
	O: {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 207 497 q 293 192 207 296 q 542 87 378 87 q 791 191 707 87 q 875 497 875 295 q 791 801 875 697 q 543 904 707 904 q 293 800 378 904 q 207 497 207 696 z "
	},
	P: {
		ha: 836,
		x_min: 136,
		x_max: 765,
		o: "m 765 703 q 662 471 765 552 q 368 390 559 390 l 252 390 l 252 0 l 136 0 l 136 991 l 393 991 q 765 703 765 991 m 252 489 l 355 489 q 577 538 509 489 q 646 697 646 588 q 581 844 646 795 q 380 892 517 892 l 252 892 l 252 489 z "
	},
	Q: {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 920 180 997 307 q 704 9 844 54 l 940 -236 l 772 -236 l 579 -12 l 542 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 207 497 q 293 192 207 296 q 542 87 378 87 q 791 191 707 87 q 875 497 875 295 q 791 801 875 697 q 543 904 707 904 q 293 800 378 904 q 207 497 207 696 z "
	},
	R: {
		ha: 859,
		x_min: 136,
		x_max: 835,
		o: "m 252 412 l 252 0 l 136 0 l 136 991 l 408 991 q 678 922 591 991 q 765 711 765 852 q 566 446 765 515 l 835 0 l 699 0 l 458 412 l 252 412 m 252 511 l 410 511 q 589 560 532 511 q 646 705 646 608 q 588 847 646 804 q 401 890 530 890 l 252 890 l 252 511 z "
	},
	S: {
		ha: 762,
		x_min: 72,
		x_max: 696,
		o: "m 696 264 q 601 60 696 133 q 343 -14 506 -14 q 72 32 167 -14 l 72 143 q 205 102 133 117 q 347 87 277 87 q 521 131 463 87 q 579 253 579 175 q 558 337 579 304 q 489 398 538 370 q 342 461 441 426 q 144 578 203 511 q 85 755 85 646 q 171 938 85 870 q 399 1006 257 1006 q 671 951 547 1006 l 635 851 q 396 903 512 903 q 253 863 304 903 q 201 754 201 824 q 220 670 201 703 q 284 609 239 637 q 422 549 330 582 q 637 429 578 493 q 696 264 696 366 z "
	},
	T: {
		ha: 768,
		x_min: 12,
		x_max: 755,
		o: "m 441 0 l 326 0 l 326 889 l 12 889 l 12 991 l 755 991 l 755 889 l 441 889 l 441 0 z "
	},
	U: {
		ha: 1011,
		x_min: 126,
		x_max: 885,
		o: "m 885 991 l 885 350 q 783 83 885 180 q 501 -14 680 -14 q 224 84 322 -14 q 126 353 126 182 l 126 991 l 241 991 l 241 345 q 309 154 241 220 q 509 87 377 87 q 702 154 634 87 q 770 346 770 221 l 770 991 l 885 991 z "
	},
	V: {
		ha: 827,
		x_min: 0,
		x_max: 827,
		o: "m 703 991 l 827 991 l 469 0 l 355 0 l 0 991 l 122 991 l 350 350 q 412 135 389 239 q 476 354 437 245 l 703 991 z "
	},
	W: {
		ha: 1286,
		x_min: 18,
		x_max: 1267,
		o: "m 1002 0 l 888 0 l 688 664 q 656 775 673 708 q 637 856 638 842 q 590 660 623 766 l 396 0 l 282 0 l 18 991 l 140 991 l 297 379 q 345 146 330 250 q 399 389 363 270 l 576 991 l 699 991 l 885 383 q 940 146 918 278 q 989 380 953 242 l 1145 991 l 1267 991 l 1002 0 z "
	},
	X: {
		ha: 802,
		x_min: 5,
		x_max: 796,
		o: "m 796 0 l 665 0 l 399 436 l 127 0 l 5 0 l 335 518 l 28 991 l 155 991 l 401 599 l 650 991 l 772 991 l 465 522 l 796 0 z "
	},
	Y: {
		ha: 778,
		x_min: 0,
		x_max: 778,
		o: "m 389 496 l 653 991 l 778 991 l 447 385 l 447 0 l 330 0 l 330 379 l 0 991 l 126 991 l 389 496 z "
	},
	Z: {
		ha: 793,
		x_min: 56,
		x_max: 737,
		o: "m 737 0 l 56 0 l 56 90 l 582 888 l 72 888 l 72 991 l 722 991 l 722 901 l 196 104 l 737 104 l 737 0 z "
	},
	"[": {
		ha: 457,
		x_min: 113,
		x_max: 422,
		o: "m 422 -220 l 113 -220 l 113 991 l 422 991 l 422 896 l 227 896 l 227 -123 l 422 -123 l 422 -220 z "
	},
	"\\": {
		ha: 510,
		x_min: 16,
		x_max: 497,
		o: "m 126 991 l 497 0 l 385 0 l 16 991 l 126 991 z "
	},
	"]": {
		ha: 457,
		x_min: 35,
		x_max: 345,
		o: "m 35 -123 l 231 -123 l 231 896 l 35 896 l 35 991 l 345 991 l 345 -220 l 35 -220 l 35 -123 z "
	},
	"^": {
		ha: 753,
		x_min: 33,
		x_max: 718,
		o: "m 33 374 l 328 999 l 395 999 l 718 374 l 615 374 l 363 879 l 136 374 l 33 374 z "
	},
	_: {
		ha: 623,
		x_min: -3,
		x_max: 625,
		o: "m 625 -214 l -3 -214 l -3 -125 l 625 -125 l 625 -214 z "
	},
	"`": {
		ha: 802,
		x_min: 267,
		x_max: 533,
		o: "m 533 842 l 458 842 q 354 942 414 877 q 267 1050 294 1007 l 267 1064 l 404 1064 q 465 956 426 1017 q 533 859 503 895 l 533 842 z "
	},
	a: {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 z "
	},
	b: {
		ha: 851,
		x_min: 119,
		x_max: 774,
		o: "m 465 755 q 693 655 612 755 q 774 372 774 555 q 692 88 774 189 q 465 -14 610 -14 q 333 13 393 -14 q 232 96 273 40 l 224 96 l 200 0 l 119 0 l 119 1055 l 232 1055 l 232 799 q 227 644 232 713 l 232 644 q 465 755 311 755 m 449 661 q 283 595 334 661 q 232 372 232 529 q 284 148 232 216 q 452 81 336 81 q 606 156 555 81 q 657 374 657 232 q 606 590 657 519 q 449 661 555 661 z "
	},
	c: {
		ha: 661,
		x_min: 78,
		x_max: 615,
		o: "m 416 -14 q 166 86 255 -14 q 78 367 78 185 q 168 655 78 553 q 424 757 258 757 q 531 745 477 757 q 615 718 585 734 l 581 623 q 499 647 543 637 q 421 657 455 657 q 195 368 195 657 q 250 158 195 231 q 414 85 305 85 q 604 125 507 85 l 604 25 q 416 -14 530 -14 z "
	},
	d: {
		ha: 851,
		x_min: 78,
		x_max: 732,
		o: "m 625 100 l 619 100 q 386 -14 541 -14 q 159 86 240 -14 q 78 370 78 186 q 159 655 78 553 q 386 757 241 757 q 618 647 537 757 l 627 647 l 622 701 l 619 753 l 619 1055 l 732 1055 l 732 0 l 640 0 l 625 100 m 400 81 q 567 143 515 81 q 619 346 619 206 l 619 370 q 567 595 619 528 q 399 663 514 663 q 247 586 300 663 q 195 368 195 509 q 247 153 195 226 q 400 81 299 81 z "
	},
	e: {
		ha: 779,
		x_min: 78,
		x_max: 707,
		o: "m 433 -14 q 173 87 269 -14 q 78 366 78 187 q 166 651 78 545 q 404 757 255 757 q 625 665 544 757 q 707 422 707 573 l 707 351 l 195 351 q 261 153 198 220 q 437 85 323 85 q 675 135 557 85 l 675 35 q 562 -2 615 9 q 433 -14 509 -14 m 403 663 q 260 604 313 663 q 197 443 207 546 l 586 443 q 538 606 586 549 q 403 663 491 663 z "
	},
	f: {
		ha: 471,
		x_min: 20,
		x_max: 530,
		o: "m 454 656 l 265 656 l 265 0 l 153 0 l 153 656 l 20 656 l 20 707 l 153 747 l 153 789 q 392 1063 153 1063 q 530 1039 451 1063 l 501 949 q 390 970 436 970 q 296 927 326 970 q 265 791 265 885 l 265 743 l 454 743 l 454 656 z "
	},
	g: {
		ha: 761,
		x_min: 26,
		x_max: 728,
		o: "m 728 743 l 728 672 l 590 656 q 624 594 609 632 q 639 507 639 555 q 564 333 639 398 q 359 268 490 268 q 297 273 326 268 q 225 178 225 235 q 250 133 225 147 q 336 118 275 118 l 468 118 q 653 67 589 118 q 718 -81 718 16 q 619 -269 718 -204 q 330 -334 520 -334 q 105 -279 184 -334 q 26 -126 26 -225 q 70 -9 26 -58 q 192 58 113 41 q 144 98 163 71 q 125 161 125 125 q 146 233 125 202 q 215 292 168 263 q 121 372 157 315 q 85 503 85 429 q 158 691 85 625 q 366 757 231 757 q 471 743 424 757 l 728 743 m 135 -125 q 186 -216 135 -185 q 332 -248 237 -248 q 542 -205 473 -248 q 610 -90 610 -163 q 572 -6 610 -30 q 432 17 535 17 l 297 17 q 178 -20 220 17 q 135 -125 135 -56 m 196 505 q 240 387 196 427 q 363 347 284 347 q 528 507 528 347 q 361 675 528 675 q 239 632 281 675 q 196 505 196 589 z "
	},
	h: {
		ha: 852,
		x_min: 119,
		x_max: 741,
		o: "m 628 0 l 628 481 q 587 616 628 572 q 457 661 545 661 q 286 597 340 661 q 232 389 232 534 l 232 0 l 119 0 l 119 1055 l 232 1055 l 232 736 q 227 640 232 678 l 233 640 q 328 725 267 694 q 468 755 389 755 q 672 691 604 755 q 741 485 741 626 l 741 0 l 628 0 z "
	},
	i: {
		ha: 351,
		x_min: 110,
		x_max: 243,
		o: "m 232 0 l 119 0 l 119 743 l 232 743 l 232 0 m 110 945 q 129 1001 110 983 q 176 1019 148 1019 q 223 1001 203 1019 q 243 945 243 983 q 223 888 243 907 q 176 869 203 869 q 129 888 148 869 q 110 945 110 907 z "
	},
	j: {
		ha: 351,
		x_min: -75,
		x_max: 243,
		o: "m 29 -334 q -75 -317 -35 -334 l -75 -225 q 17 -239 -28 -239 q 95 -210 70 -239 q 119 -122 119 -181 l 119 743 l 232 743 l 232 -114 q 29 -334 232 -334 m 110 945 q 129 1001 110 983 q 176 1019 148 1019 q 223 1001 203 1019 q 243 945 243 983 q 223 888 243 907 q 176 869 203 869 q 129 888 148 869 q 110 945 110 907 z "
	},
	k: {
		ha: 729,
		x_min: 119,
		x_max: 714,
		o: "m 231 380 q 319 489 260 422 l 559 743 l 693 743 l 392 427 l 714 0 l 578 0 l 315 351 l 231 278 l 231 0 l 119 0 l 119 1055 l 231 1055 l 231 496 q 225 380 231 458 l 231 380 z "
	},
	l: {
		ha: 351,
		x_min: 119,
		x_max: 232,
		o: "m 232 0 l 119 0 l 119 1055 l 232 1055 l 232 0 z "
	},
	m: {
		ha: 1292,
		x_min: 119,
		x_max: 1179,
		o: "m 1067 0 l 1067 484 q 1029 617 1067 572 q 911 661 991 661 q 755 601 806 661 q 705 415 705 541 l 705 0 l 593 0 l 593 484 q 555 617 593 572 q 436 661 517 661 q 281 598 330 661 q 232 390 232 534 l 232 0 l 119 0 l 119 743 l 211 743 l 229 642 l 235 642 q 325 726 267 696 q 454 757 382 757 q 682 631 629 757 l 688 631 q 784 723 721 689 q 928 757 847 757 q 1117 692 1054 757 q 1179 485 1179 627 l 1179 0 l 1067 0 z "
	},
	n: {
		ha: 852,
		x_min: 119,
		x_max: 741,
		o: "m 628 0 l 628 481 q 587 616 628 572 q 457 661 545 661 q 286 598 340 661 q 232 390 232 535 l 232 0 l 119 0 l 119 743 l 211 743 l 229 642 l 235 642 q 332 727 269 696 q 471 757 394 757 q 673 692 605 757 q 741 485 741 627 l 741 0 l 628 0 z "
	},
	o: {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 195 372 q 252 155 195 230 q 419 81 309 81 q 587 155 530 81 q 644 372 644 229 q 587 588 644 514 q 418 661 530 661 q 251 589 307 661 q 195 372 195 516 z "
	},
	p: {
		ha: 851,
		x_min: 119,
		x_max: 774,
		o: "m 465 -14 q 333 13 393 -14 q 232 96 273 40 l 224 96 q 232 -28 232 31 l 232 -334 l 119 -334 l 119 743 l 211 743 l 227 642 l 232 642 q 333 730 275 703 q 465 757 391 757 q 693 656 613 757 q 774 372 774 555 q 692 88 774 189 q 465 -14 610 -14 m 449 661 q 284 598 335 661 q 232 397 233 535 l 232 372 q 284 148 232 216 q 452 81 336 81 q 603 159 548 81 q 657 374 657 237 q 603 587 657 513 q 449 661 548 661 z "
	},
	q: {
		ha: 851,
		x_min: 78,
		x_max: 732,
		o: "m 400 81 q 564 141 513 81 q 619 345 616 201 l 619 370 q 566 594 619 526 q 399 663 513 663 q 247 586 300 663 q 195 368 195 509 q 247 154 195 228 q 400 81 298 81 m 383 -14 q 159 87 239 -14 q 78 370 78 189 q 159 654 78 552 q 386 757 241 757 q 621 642 538 757 l 627 642 l 643 743 l 732 743 l 732 -334 l 619 -334 l 619 -16 q 627 100 619 52 l 618 100 q 383 -14 540 -14 z "
	},
	r: {
		ha: 567,
		x_min: 119,
		x_max: 547,
		o: "m 458 757 q 547 749 508 757 l 532 644 q 450 654 486 654 q 296 581 360 654 q 232 399 232 508 l 232 0 l 119 0 l 119 743 l 212 743 l 225 606 l 231 606 q 330 718 272 678 q 458 757 389 757 z "
	},
	s: {
		ha: 663,
		x_min: 72,
		x_max: 599,
		o: "m 599 203 q 522 43 599 99 q 304 -14 444 -14 q 74 33 157 -14 l 74 138 q 189 95 127 111 q 307 79 250 79 q 443 107 395 79 q 490 193 490 136 q 453 268 490 237 q 306 340 415 298 q 158 408 202 379 q 93 473 115 437 q 72 561 72 510 q 146 704 72 652 q 349 757 220 757 q 583 708 469 757 l 543 616 q 340 663 431 663 q 220 637 260 663 q 179 568 179 612 q 194 517 179 538 q 243 477 210 496 q 374 422 277 458 q 552 326 506 374 q 599 203 599 277 z "
	},
	t: {
		ha: 490,
		x_min: 21,
		x_max: 461,
		o: "m 359 79 q 417 84 389 79 q 461 93 445 88 l 461 7 q 407 -8 443 -2 q 343 -14 372 -14 q 127 214 127 -14 l 127 656 l 21 656 l 21 710 l 127 757 l 175 916 l 240 916 l 240 743 l 456 743 l 456 656 l 240 656 l 240 218 q 272 115 240 151 q 359 79 304 79 z "
	},
	u: {
		ha: 852,
		x_min: 111,
		x_max: 733,
		o: "m 225 743 l 225 261 q 267 125 225 170 q 396 81 308 81 q 567 144 513 81 q 621 353 621 208 l 621 743 l 733 743 l 733 0 l 640 0 l 624 100 l 618 100 q 522 16 583 45 q 382 -14 460 -14 q 179 51 246 -14 q 111 257 111 115 l 111 743 l 225 743 z "
	},
	v: {
		ha: 696,
		x_min: 0,
		x_max: 696,
		o: "m 282 0 l 0 743 l 121 743 l 281 302 q 345 102 335 148 l 350 102 q 397 251 357 138 q 575 743 437 363 l 696 743 l 414 0 l 282 0 z "
	},
	w: {
		ha: 1080,
		x_min: 16,
		x_max: 1065,
		o: "m 726 0 l 590 436 q 542 618 577 476 l 536 618 q 489 435 509 499 l 349 0 l 218 0 l 16 743 l 134 743 q 243 317 205 463 q 286 119 281 170 l 292 119 q 316 219 299 158 q 345 317 332 281 l 481 743 l 603 743 l 736 317 q 787 121 774 200 l 793 121 q 807 196 795 145 q 949 743 819 247 l 1065 743 l 860 0 l 726 0 z "
	},
	x: {
		ha: 728,
		x_min: 26,
		x_max: 700,
		o: "m 298 380 l 40 743 l 168 743 l 364 458 l 559 743 l 686 743 l 428 380 l 700 0 l 572 0 l 364 301 l 154 0 l 26 0 l 298 380 z "
	},
	y: {
		ha: 700,
		x_min: 1,
		x_max: 699,
		o: "m 1 743 l 122 743 l 285 319 q 351 110 338 174 l 357 110 q 394 228 366 144 q 578 743 422 312 l 699 743 l 379 -103 q 268 -281 332 -229 q 113 -334 205 -334 q 11 -322 61 -334 l 11 -232 q 94 -240 48 -240 q 260 -110 210 -240 l 301 -4 l 1 743 z "
	},
	z: {
		ha: 650,
		x_min: 56,
		x_max: 595,
		o: "m 595 0 l 56 0 l 56 77 l 461 656 l 81 656 l 81 743 l 585 743 l 585 656 l 184 87 l 595 87 l 595 0 z "
	},
	"{": {
		ha: 526,
		x_min: 41,
		x_max: 478,
		o: "m 322 8 q 362 -92 322 -61 q 478 -125 401 -123 l 478 -220 q 279 -161 349 -218 q 208 1 208 -103 l 208 207 q 165 308 208 277 q 41 338 123 338 l 41 433 q 169 466 130 435 q 208 562 208 497 l 208 770 q 281 933 208 875 q 478 991 355 991 l 478 897 q 322 762 322 893 l 322 562 q 171 390 322 416 l 171 382 q 322 210 322 355 l 322 8 z "
	},
	"|": {
		ha: 765,
		x_min: 335,
		x_max: 431,
		o: "m 335 1055 l 431 1055 l 431 -336 l 335 -336 l 335 1055 z "
	},
	"}": {
		ha: 526,
		x_min: 49,
		x_max: 485,
		o: "m 354 390 q 203 562 203 416 l 203 762 q 49 897 203 893 l 49 991 q 245 932 174 991 q 317 770 317 873 l 317 562 q 357 466 317 496 q 485 433 397 435 l 485 338 q 359 308 402 338 q 317 207 317 277 l 317 1 q 247 -160 317 -102 q 49 -220 178 -218 l 49 -125 q 163 -92 124 -123 q 203 8 203 -61 l 203 210 q 240 328 203 287 q 354 382 277 368 l 354 390 z "
	},
	"~": {
		ha: 794,
		x_min: 71,
		x_max: 722,
		o: "m 229 484 q 150 461 193 484 q 71 401 107 438 l 71 504 q 236 578 138 578 q 320 568 282 578 q 419 533 359 559 q 497 505 464 514 q 562 496 530 496 q 642 517 599 496 q 722 578 686 539 l 722 476 q 557 401 653 401 q 465 413 508 401 q 374 446 422 424 q 292 475 323 467 q 229 484 262 484 z "
	},
	" ": {
		ha: 361,
		x_min: 0,
		x_max: 0,
		o: ""
	},
	"¡": {
		ha: 371,
		x_min: 103,
		x_max: 267,
		o: "m 149 464 l 220 464 l 254 -253 l 114 -253 l 149 464 m 267 667 q 184 575 267 575 q 123 599 144 575 q 103 667 103 623 q 124 734 103 709 q 184 758 146 758 q 243 736 219 758 q 267 667 267 715 z "
	},
	"¢": {
		ha: 794,
		x_min: 129,
		x_max: 669,
		o: "m 659 163 q 488 122 587 126 l 488 -14 l 397 -14 l 397 126 q 194 240 260 148 q 129 503 129 333 q 397 889 129 847 l 397 1006 l 489 1006 l 489 895 q 588 881 540 892 q 669 854 636 870 l 636 760 q 472 794 546 794 q 300 723 355 794 q 245 504 245 651 q 299 291 245 360 q 467 222 353 222 q 659 262 562 222 l 659 163 z "
	},
	"£": {
		ha: 794,
		x_min: 43,
		x_max: 741,
		o: "m 463 1004 q 707 947 591 1004 l 665 857 q 464 909 561 909 q 338 867 380 909 q 296 730 296 825 l 296 530 l 582 530 l 582 444 l 296 444 l 296 294 q 274 180 296 227 q 201 104 252 134 l 741 104 l 741 0 l 43 0 l 43 96 q 182 293 182 127 l 182 444 l 47 444 l 47 530 l 182 530 l 182 745 q 258 935 182 865 q 463 1004 334 1004 z "
	},
	"¤": {
		ha: 794,
		x_min: 83,
		x_max: 710,
		o: "m 125 490 q 175 646 125 573 l 83 741 l 147 803 l 239 713 q 397 762 309 762 q 553 713 484 762 l 646 803 l 710 741 l 619 647 q 669 490 669 570 q 619 332 669 401 l 708 240 l 646 178 l 553 268 q 397 220 484 220 q 239 269 307 220 l 147 179 l 85 241 l 175 334 q 125 490 125 406 m 212 490 q 266 360 212 414 q 397 306 319 306 q 530 360 476 306 q 583 490 583 414 q 529 623 583 568 q 397 677 475 677 q 266 622 319 677 q 212 490 212 566 z "
	},
	"¥": {
		ha: 794,
		x_min: 21,
		x_max: 771,
		o: "m 396 498 l 653 991 l 771 991 l 489 469 l 667 469 l 667 383 l 452 383 l 452 268 l 667 268 l 667 182 l 452 182 l 452 0 l 340 0 l 340 182 l 126 182 l 126 268 l 340 268 l 340 383 l 126 383 l 126 469 l 300 469 l 21 991 l 142 991 l 396 498 z "
	},
	"¦": {
		ha: 765,
		x_min: 335,
		x_max: 431,
		o: "m 335 1055 l 431 1055 l 431 529 l 335 529 l 335 1055 m 335 191 l 431 191 l 431 -336 l 335 -336 l 335 191 z "
	},
	"§": {
		ha: 717,
		x_min: 83,
		x_max: 623,
		o: "m 94 549 q 123 653 94 607 q 205 725 153 700 q 127 790 155 752 q 98 885 98 827 q 169 1014 98 967 q 372 1061 239 1061 q 490 1052 436 1061 q 610 1015 544 1042 l 574 926 q 461 962 507 953 q 364 971 416 971 q 246 951 286 971 q 207 888 207 931 q 249 819 207 847 q 395 753 290 790 q 572 655 521 707 q 623 532 623 604 q 595 423 623 471 q 517 347 567 375 q 621 193 621 292 q 541 46 621 98 q 318 -5 462 -5 q 83 39 170 -5 l 83 139 q 202 99 136 114 q 323 83 268 83 q 462 109 414 83 q 510 183 510 135 q 494 234 510 214 q 441 273 477 254 q 326 322 404 293 q 184 388 230 357 q 117 457 139 418 q 94 549 94 496 m 193 562 q 238 474 193 510 q 396 397 283 439 l 429 385 q 522 514 522 439 q 472 608 522 570 q 297 685 422 646 q 222 638 251 672 q 193 562 193 604 z "
	},
	"¨": {
		ha: 802,
		x_min: 210,
		x_max: 591,
		o: "m 210 945 q 228 996 210 980 q 271 1011 245 1011 q 315 996 296 1011 q 334 945 334 980 q 315 894 334 911 q 271 878 296 878 q 228 894 245 878 q 210 945 210 911 m 468 945 q 486 996 468 980 q 529 1011 504 1011 q 573 996 554 1011 q 591 945 591 980 q 573 894 591 911 q 529 878 554 878 q 486 894 504 878 q 468 945 468 911 z "
	},
	"©": {
		ha: 1156,
		x_min: 68,
		x_max: 1088,
		o: "m 606 718 q 475 659 521 718 q 429 496 429 600 q 472 327 429 382 q 604 272 515 272 q 747 302 663 272 l 747 218 q 681 195 715 205 q 599 186 646 186 q 397 268 467 186 q 326 496 326 349 q 401 721 326 637 q 606 804 476 804 q 772 764 692 804 l 733 684 q 606 718 660 718 m 68 496 q 136 750 68 631 q 322 937 203 869 q 578 1006 441 1006 q 832 938 713 1006 q 1019 751 951 870 q 1088 496 1088 633 q 1022 245 1088 362 q 838 57 956 127 q 578 -14 719 -14 q 319 57 437 -14 q 134 244 200 127 q 68 496 68 361 m 139 496 q 198 276 139 378 q 359 116 257 174 q 578 57 461 57 q 797 116 696 57 q 957 276 898 175 q 1017 496 1017 376 q 958 715 1017 614 q 798 875 899 816 q 578 935 697 935 q 359 876 460 935 q 198 716 258 817 q 139 496 139 615 z "
	},
	"ª": {
		ha: 492,
		x_min: 47,
		x_max: 424,
		o: "m 361 543 l 345 600 q 187 534 282 534 q 85 568 123 534 q 47 671 47 602 q 100 775 47 740 q 264 815 152 811 l 343 818 l 343 844 q 243 935 343 935 q 104 900 175 935 l 75 965 q 243 1003 153 1003 q 377 967 331 1003 q 424 850 424 932 l 424 543 l 361 543 m 131 671 q 207 603 131 603 q 343 725 343 603 l 343 758 l 277 755 q 166 733 201 753 q 131 671 131 714 z "
	},
	"«": {
		ha: 690,
		x_min: 56,
		x_max: 636,
		o: "m 56 374 l 288 650 l 368 603 l 172 366 l 368 127 l 288 79 l 56 355 l 56 374 m 323 374 l 557 650 l 636 603 l 441 366 l 636 127 l 557 79 l 323 355 l 323 374 z "
	},
	"¬": {
		ha: 794,
		x_min: 71,
		x_max: 722,
		o: "m 722 536 l 722 179 l 629 179 l 629 443 l 71 443 l 71 536 l 722 536 z "
	},
	"­": {
		ha: 447,
		x_min: 57,
		x_max: 390,
		o: "m 57 321 l 57 424 l 390 424 l 390 321 l 57 321 z "
	},
	"®": {
		ha: 1156,
		x_min: 68,
		x_max: 1088,
		o: "m 490 517 l 564 517 q 651 545 618 517 q 684 616 684 573 q 654 689 684 667 q 562 711 625 711 l 490 711 l 490 517 m 785 619 q 756 523 785 565 q 675 461 727 481 l 836 193 l 722 193 l 582 433 l 490 433 l 490 193 l 390 193 l 390 798 l 567 798 q 732 753 680 798 q 785 619 785 709 m 68 496 q 136 750 68 631 q 322 937 203 869 q 578 1006 441 1006 q 832 938 713 1006 q 1019 751 951 870 q 1088 496 1088 633 q 1022 245 1088 362 q 838 57 956 127 q 578 -14 719 -14 q 319 57 437 -14 q 134 244 200 127 q 68 496 68 361 m 139 496 q 198 276 139 378 q 359 116 257 174 q 578 57 461 57 q 797 116 696 57 q 957 276 898 175 q 1017 496 1017 376 q 958 715 1017 614 q 798 875 899 816 q 578 935 697 935 q 359 876 460 935 q 198 716 258 817 q 139 496 139 615 z "
	},
	"¯": {
		ha: 694,
		x_min: -4,
		x_max: 699,
		o: "m 699 1055 l -4 1055 l -4 1141 l 699 1141 l 699 1055 z "
	},
	"°": {
		ha: 595,
		x_min: 86,
		x_max: 509,
		o: "m 86 794 q 148 944 86 882 q 297 1006 209 1006 q 447 944 385 1006 q 509 794 509 883 q 481 689 509 737 q 404 612 453 640 q 297 583 354 583 q 148 644 209 583 q 86 794 86 705 m 164 794 q 204 700 164 739 q 298 661 243 661 q 392 700 353 661 q 431 794 431 738 q 392 889 431 851 q 298 928 354 928 q 203 889 242 928 q 164 794 164 850 z "
	},
	"±": {
		ha: 794,
		x_min: 71,
		x_max: 722,
		o: "m 443 536 l 722 536 l 722 443 l 443 443 l 443 154 l 349 154 l 349 443 l 71 443 l 71 536 l 349 536 l 349 827 l 443 827 l 443 536 m 71 1 l 71 94 l 722 94 l 722 1 l 71 1 z "
	},
	"²": {
		ha: 482,
		x_min: 33,
		x_max: 443,
		o: "m 443 397 l 33 397 l 33 468 l 193 624 q 281 715 254 682 q 320 774 309 748 q 332 836 332 800 q 304 906 332 882 q 235 929 277 929 q 166 916 199 929 q 86 869 133 903 l 41 929 q 233 1004 130 1004 q 373 960 323 1004 q 422 840 422 916 q 392 735 422 786 q 262 590 362 684 l 144 478 l 443 478 l 443 397 z "
	},
	"³": {
		ha: 482,
		x_min: 22,
		x_max: 443,
		o: "m 425 851 q 397 762 425 797 q 323 711 370 727 q 443 570 443 680 q 380 434 443 483 q 204 386 318 386 q 22 424 101 386 l 22 507 q 205 461 122 461 q 349 571 349 461 q 192 669 349 669 l 113 669 l 113 742 l 193 742 q 297 769 263 742 q 330 842 330 795 q 303 906 330 883 q 231 929 276 929 q 148 915 186 929 q 72 876 110 900 l 25 937 q 115 986 68 968 q 227 1004 163 1004 q 372 964 319 1004 q 425 851 425 924 z "
	},
	"´": {
		ha: 802,
		x_min: 267,
		x_max: 533,
		o: "m 267 859 q 337 960 299 901 q 396 1064 374 1020 l 533 1064 l 533 1050 q 444 941 503 1006 q 342 842 385 877 l 267 842 l 267 859 z "
	},
	"µ": {
		ha: 860,
		x_min: 119,
		x_max: 741,
		o: "m 232 258 q 404 81 232 81 q 574 145 520 81 q 628 353 628 209 l 628 743 l 741 743 l 741 0 l 648 0 l 631 100 l 624 100 q 393 -14 549 -14 q 232 49 292 -14 l 225 49 q 232 -117 232 -8 l 232 -334 l 119 -334 l 119 743 l 232 743 l 232 258 z "
	},
	"¶": {
		ha: 909,
		x_min: 77,
		x_max: 760,
		o: "m 760 -176 l 682 -176 l 682 985 l 538 985 l 538 -176 l 460 -176 l 460 379 q 361 367 418 367 q 145 452 214 367 q 77 707 77 536 q 151 969 77 883 q 382 1055 224 1055 l 760 1055 l 760 -176 z "
	},
	"·": {
		ha: 370,
		x_min: 103,
		x_max: 267,
		o: "m 103 490 q 124 558 103 535 q 183 582 145 582 q 244 558 222 582 q 267 490 267 535 q 244 422 267 446 q 183 399 222 399 q 126 420 149 399 q 103 490 103 441 z "
	},
	"¸": {
		ha: 315,
		x_min: 25,
		x_max: 296,
		o: "m 296 -196 q 244 -298 296 -262 q 90 -334 192 -334 q 25 -328 56 -334 l 25 -256 q 96 -261 56 -261 q 177 -248 149 -261 q 204 -197 204 -234 q 177 -150 204 -168 q 77 -121 151 -132 l 136 0 l 211 0 l 174 -78 q 296 -196 296 -104 z "
	},
	"¹": {
		ha: 482,
		x_min: 52,
		x_max: 326,
		o: "m 229 991 l 326 991 l 326 397 l 236 397 l 236 790 q 240 913 236 852 q 207 883 225 898 q 97 803 189 867 l 52 868 l 229 991 z "
	},
	"º": {
		ha: 521,
		x_min: 45,
		x_max: 476,
		o: "m 476 770 q 418 596 476 659 q 258 534 360 534 q 102 597 159 534 q 45 770 45 661 q 102 942 45 880 q 261 1003 159 1003 q 420 941 364 1003 q 476 770 476 880 m 127 770 q 158 646 127 687 q 260 604 189 604 q 362 646 331 604 q 393 770 393 687 q 362 893 393 853 q 260 933 331 933 q 159 893 190 933 q 127 770 127 853 z "
	},
	"»": {
		ha: 690,
		x_min: 54,
		x_max: 635,
		o: "m 635 355 l 401 79 l 322 127 l 517 366 l 322 603 l 401 650 l 635 374 l 635 355 m 367 355 l 134 79 l 54 127 l 249 366 l 54 603 l 134 650 l 367 374 l 367 355 z "
	},
	"¼": {
		ha: 1083,
		x_min: 51,
		x_max: 1010,
		o: "m 880 991 l 268 0 l 171 0 l 783 991 l 880 991 m 229 991 l 326 991 l 326 397 l 235 397 l 235 790 q 239 913 235 852 q 206 883 224 898 q 96 803 188 867 l 51 868 l 229 991 m 1010 138 l 925 138 l 925 1 l 827 1 l 827 138 l 554 138 l 554 206 l 831 599 l 925 599 l 925 217 l 1010 217 l 1010 138 m 827 217 l 827 349 q 831 491 827 440 q 819 470 827 483 q 801 441 811 456 q 781 411 791 426 q 763 384 770 395 l 649 217 l 827 217 z "
	},
	"½": {
		ha: 1083,
		x_min: 31,
		x_max: 1017,
		o: "m 834 991 l 222 0 l 125 0 l 737 991 l 834 991 m 209 991 l 306 991 l 306 397 l 216 397 l 216 790 q 220 913 216 852 q 186 883 205 898 q 77 803 168 867 l 31 868 l 209 991 m 1017 1 l 607 1 l 607 71 l 767 227 q 855 318 827 286 q 894 377 883 351 q 905 439 905 403 q 878 509 905 486 q 808 532 851 532 q 740 519 773 532 q 660 473 707 507 l 615 532 q 807 608 704 608 q 946 564 897 608 q 996 444 996 519 q 966 338 996 389 q 836 193 936 287 l 718 81 l 1017 81 l 1017 1 z "
	},
	"¾": {
		ha: 1083,
		x_min: 18,
		x_max: 1064,
		o: "m 420 851 q 393 762 420 797 q 319 711 365 727 q 438 570 438 680 q 376 434 438 483 q 199 386 313 386 q 18 424 96 386 l 18 507 q 201 461 117 461 q 344 571 344 461 q 187 669 344 669 l 108 669 l 108 742 l 189 742 q 292 769 258 742 q 326 842 326 795 q 298 906 326 883 q 226 929 271 929 q 143 915 181 929 q 67 876 105 900 l 20 937 q 111 986 63 968 q 222 1004 158 1004 q 367 964 314 1004 q 420 851 420 924 m 943 991 l 330 0 l 233 0 l 846 991 l 943 991 m 1064 138 l 979 138 l 979 1 l 881 1 l 881 138 l 608 138 l 608 206 l 885 599 l 979 599 l 979 217 l 1064 217 l 1064 138 m 881 217 l 881 349 q 885 491 881 440 q 873 470 882 483 q 855 441 865 456 q 835 411 845 426 q 817 384 825 395 l 703 217 l 881 217 z "
	},
	"¿": {
		ha: 596,
		x_min: 35,
		x_max: 578,
		o: "m 400 464 l 400 429 q 375 296 400 347 q 283 189 349 246 q 181 92 201 117 q 152 40 160 66 q 143 -24 143 14 q 188 -130 143 -92 q 315 -168 233 -168 q 420 -155 370 -168 q 538 -110 471 -142 l 578 -201 q 310 -267 444 -267 q 108 -203 181 -267 q 35 -25 35 -140 q 46 58 35 22 q 80 124 58 94 q 132 182 102 154 q 199 241 162 210 q 289 340 267 301 q 311 443 311 380 l 311 464 l 400 464 m 433 667 q 351 575 433 575 q 290 598 311 575 q 269 667 269 622 q 292 734 269 710 q 351 758 314 758 q 410 736 386 758 q 433 667 433 715 z "
	},
	"À": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 491 1071 l 416 1071 q 312 1171 372 1106 q 224 1279 252 1236 l 224 1293 l 362 1293 q 422 1185 384 1246 q 491 1088 461 1124 l 491 1071 z "
	},
	"Á": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 357 1088 q 427 1190 389 1130 q 486 1293 465 1249 l 623 1293 l 623 1279 q 534 1171 593 1235 q 432 1071 475 1106 l 357 1071 l 357 1088 z "
	},
	"Â": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 205 1086 q 326 1222 292 1179 q 376 1293 361 1265 l 489 1293 q 541 1220 504 1265 q 663 1086 578 1175 l 663 1071 l 582 1071 q 432 1197 522 1108 q 283 1071 340 1106 l 205 1071 l 205 1086 z "
	},
	"Ã": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 537 1072 q 480 1085 508 1072 q 426 1113 452 1097 q 374 1140 399 1128 q 326 1153 349 1153 q 275 1133 292 1153 q 248 1071 258 1112 l 182 1071 q 230 1199 191 1153 q 330 1246 269 1246 q 391 1233 361 1246 q 446 1205 420 1221 q 497 1178 473 1190 q 543 1165 522 1165 q 593 1185 576 1165 q 619 1247 609 1205 l 686 1247 q 639 1119 677 1165 q 537 1072 601 1072 z "
	},
	"Ä": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 247 1174 q 265 1225 247 1209 q 308 1240 283 1240 q 352 1225 334 1240 q 371 1174 371 1209 q 352 1123 371 1140 q 308 1107 334 1107 q 265 1123 283 1107 q 247 1174 247 1140 m 505 1174 q 523 1225 505 1209 q 566 1240 541 1240 q 610 1225 591 1240 q 629 1174 629 1209 q 610 1123 629 1140 q 566 1107 591 1107 q 523 1123 541 1107 q 505 1174 505 1140 z "
	},
	"Å": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 590 1076 q 548 969 590 1010 q 437 929 507 929 q 328 969 369 929 q 288 1075 288 1008 q 329 1180 288 1141 q 437 1219 370 1219 q 548 1179 506 1219 q 590 1076 590 1139 m 517 1075 q 494 1134 517 1113 q 437 1154 472 1154 q 380 1134 403 1154 q 358 1075 358 1113 q 378 1016 358 1037 q 437 996 399 996 q 495 1016 473 996 q 517 1075 517 1037 z "
	},
	"Æ": {
		ha: 1213,
		x_min: -1,
		x_max: 1129,
		o: "m 1129 0 l 621 0 l 621 315 l 272 315 l 118 0 l -1 0 l 472 991 l 1129 991 l 1129 889 l 736 889 l 736 570 l 1103 570 l 1103 468 l 736 468 l 736 103 l 1129 103 l 1129 0 m 318 419 l 621 419 l 621 889 l 541 889 l 318 419 z "
	},
	"Ç": {
		ha: 876,
		x_min: 85,
		x_max: 835,
		o: "m 561 903 q 303 794 397 903 q 208 496 208 685 q 299 195 208 301 q 559 89 391 89 q 796 126 663 89 l 796 25 q 542 -14 693 -14 q 204 119 323 -14 q 85 497 85 252 q 142 766 85 650 q 308 943 199 881 q 562 1006 416 1006 q 835 949 718 1006 l 786 850 q 561 903 673 903 m 644 -196 q 592 -298 644 -262 q 439 -334 541 -334 q 374 -328 404 -334 l 374 -256 q 444 -261 404 -261 q 525 -248 498 -261 q 553 -197 553 -234 q 526 -150 553 -168 q 425 -121 499 -132 l 485 0 l 559 0 l 522 -78 q 644 -196 644 -104 z "
	},
	"È": {
		ha: 772,
		x_min: 136,
		x_max: 689,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 m 484 1071 l 409 1071 q 304 1171 365 1106 q 217 1279 244 1236 l 217 1293 l 355 1293 q 415 1185 376 1246 q 484 1088 454 1124 l 484 1071 z "
	},
	"É": {
		ha: 772,
		x_min: 136,
		x_max: 689,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 m 309 1088 q 379 1190 342 1130 q 439 1293 417 1249 l 576 1293 l 576 1279 q 487 1171 546 1235 q 385 1071 428 1106 l 309 1071 l 309 1088 z "
	},
	"Ê": {
		ha: 772,
		x_min: 136,
		x_max: 689,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 m 178 1086 q 299 1222 264 1179 q 349 1293 334 1265 l 462 1293 q 514 1220 477 1265 q 635 1086 551 1175 l 635 1071 l 555 1071 q 405 1197 495 1108 q 256 1071 313 1106 l 178 1071 l 178 1086 z "
	},
	"Ë": {
		ha: 772,
		x_min: 136,
		x_max: 689,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 m 222 1174 q 240 1225 222 1209 q 283 1240 258 1240 q 327 1225 309 1240 q 346 1174 346 1209 q 327 1123 346 1140 q 283 1107 309 1107 q 240 1123 258 1107 q 222 1174 222 1140 m 480 1174 q 498 1225 480 1209 q 541 1240 516 1240 q 585 1225 566 1240 q 604 1174 604 1209 q 585 1123 604 1140 q 541 1107 566 1107 q 498 1123 516 1107 q 480 1174 480 1140 z "
	},
	"Ð": {
		ha: 1003,
		x_min: 32,
		x_max: 917,
		o: "m 917 505 q 784 130 917 260 q 400 0 650 0 l 136 0 l 136 440 l 32 440 l 32 542 l 136 542 l 136 991 l 431 991 q 788 864 659 991 q 917 505 917 737 m 794 501 q 408 892 794 892 l 252 892 l 252 542 l 509 542 l 509 440 l 252 440 l 252 100 l 380 100 q 794 501 794 100 z "
	},
	"Ñ": {
		ha: 1047,
		x_min: 136,
		x_max: 911,
		o: "m 911 0 l 779 0 l 237 832 l 232 832 q 243 564 243 686 l 243 0 l 136 0 l 136 991 l 267 991 l 807 163 l 812 163 q 806 280 811 181 q 803 422 802 380 l 803 991 l 911 991 l 911 0 m 634 1072 q 577 1085 605 1072 q 523 1113 549 1097 q 471 1140 496 1128 q 423 1153 446 1153 q 372 1133 389 1153 q 345 1071 355 1112 l 279 1071 q 327 1199 288 1153 q 427 1246 366 1246 q 488 1233 458 1246 q 543 1205 517 1221 q 594 1178 570 1190 q 640 1165 618 1165 q 690 1185 673 1165 q 716 1247 706 1205 l 783 1247 q 736 1119 774 1165 q 634 1072 698 1072 z "
	},
	"Ò": {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 207 497 q 293 192 207 296 q 542 87 378 87 q 791 191 707 87 q 875 497 875 295 q 791 801 875 697 q 543 904 707 904 q 293 800 378 904 q 207 497 207 696 m 615 1071 l 541 1071 q 436 1171 496 1106 q 349 1279 376 1236 l 349 1293 l 486 1293 q 547 1185 508 1246 q 615 1088 585 1124 l 615 1071 z "
	},
	"Ó": {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 207 497 q 293 192 207 296 q 542 87 378 87 q 791 191 707 87 q 875 497 875 295 q 791 801 875 697 q 543 904 707 904 q 293 800 378 904 q 207 497 207 696 m 447 1088 q 517 1190 479 1130 q 576 1293 555 1249 l 713 1293 l 713 1279 q 625 1171 684 1235 q 522 1071 566 1106 l 447 1071 l 447 1088 z "
	},
	"Ô": {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 207 497 q 293 192 207 296 q 542 87 378 87 q 791 191 707 87 q 875 497 875 295 q 791 801 875 697 q 543 904 707 904 q 293 800 378 904 q 207 497 207 696 m 304 1086 q 425 1222 390 1179 q 475 1293 459 1265 l 587 1293 q 639 1220 602 1265 q 761 1086 676 1175 l 761 1071 l 680 1071 q 530 1197 621 1108 q 382 1071 438 1106 l 304 1071 l 304 1086 z "
	},
	"Õ": {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 207 497 q 293 192 207 296 q 542 87 378 87 q 791 191 707 87 q 875 497 875 295 q 791 801 875 697 q 543 904 707 904 q 293 800 378 904 q 207 497 207 696 m 639 1072 q 582 1085 610 1072 q 527 1113 554 1097 q 476 1140 500 1128 q 428 1153 451 1153 q 377 1133 394 1153 q 350 1071 359 1112 l 283 1071 q 331 1199 292 1153 q 432 1246 370 1246 q 492 1233 463 1246 q 548 1205 522 1221 q 599 1178 574 1190 q 645 1165 623 1165 q 694 1185 678 1165 q 721 1247 711 1205 l 788 1247 q 741 1119 779 1165 q 639 1072 703 1072 z "
	},
	"Ö": {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 207 497 q 293 192 207 296 q 542 87 378 87 q 791 191 707 87 q 875 497 875 295 q 791 801 875 697 q 543 904 707 904 q 293 800 378 904 q 207 497 207 696 m 354 1174 q 372 1225 354 1209 q 415 1240 390 1240 q 459 1225 441 1240 q 478 1174 478 1209 q 459 1123 478 1140 q 415 1107 441 1107 q 372 1123 390 1107 q 354 1174 354 1140 m 612 1174 q 630 1225 612 1209 q 673 1240 648 1240 q 717 1225 699 1240 q 736 1174 736 1209 q 717 1123 736 1140 q 673 1107 699 1107 q 630 1123 648 1107 q 612 1174 612 1140 z "
	},
	"×": {
		ha: 794,
		x_min: 90,
		x_max: 703,
		o: "m 637 798 l 703 730 l 464 490 l 701 252 l 636 184 l 396 422 l 160 184 l 92 252 l 329 490 l 90 729 l 158 798 l 397 557 l 637 798 z "
	},
	"Ø": {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 282 54 382 -14 l 214 -41 l 132 12 l 205 117 q 85 498 85 251 q 204 874 85 741 q 543 1007 323 1007 q 791 943 685 1007 l 857 1035 l 939 981 l 867 880 q 997 497 997 743 m 875 497 q 800 786 875 682 l 345 143 q 542 87 422 87 q 791 191 707 87 q 875 497 875 295 m 207 497 q 275 215 207 319 l 729 854 q 543 904 657 904 q 293 800 378 904 q 207 497 207 696 z "
	},
	"Ù": {
		ha: 1011,
		x_min: 126,
		x_max: 885,
		o: "m 885 991 l 885 350 q 783 83 885 180 q 501 -14 680 -14 q 224 84 322 -14 q 126 353 126 182 l 126 991 l 241 991 l 241 345 q 309 154 241 220 q 509 87 377 87 q 702 154 634 87 q 770 346 770 221 l 770 991 l 885 991 m 581 1071 l 506 1071 q 401 1171 462 1106 q 314 1279 341 1236 l 314 1293 l 452 1293 q 512 1185 473 1246 q 581 1088 551 1124 l 581 1071 z "
	},
	"Ú": {
		ha: 1011,
		x_min: 126,
		x_max: 885,
		o: "m 885 991 l 885 350 q 783 83 885 180 q 501 -14 680 -14 q 224 84 322 -14 q 126 353 126 182 l 126 991 l 241 991 l 241 345 q 309 154 241 220 q 509 87 377 87 q 702 154 634 87 q 770 346 770 221 l 770 991 l 885 991 m 407 1088 q 477 1190 439 1130 q 536 1293 515 1249 l 673 1293 l 673 1279 q 585 1171 644 1235 q 482 1071 526 1106 l 407 1071 l 407 1088 z "
	},
	"Û": {
		ha: 1011,
		x_min: 126,
		x_max: 885,
		o: "m 885 991 l 885 350 q 783 83 885 180 q 501 -14 680 -14 q 224 84 322 -14 q 126 353 126 182 l 126 991 l 241 991 l 241 345 q 309 154 241 220 q 509 87 377 87 q 702 154 634 87 q 770 346 770 221 l 770 991 l 885 991 m 267 1086 q 387 1222 353 1179 q 437 1293 422 1265 l 550 1293 q 602 1220 565 1265 q 724 1086 639 1175 l 724 1071 l 643 1071 q 493 1197 583 1108 q 345 1071 401 1106 l 267 1071 l 267 1086 z "
	},
	"Ü": {
		ha: 1011,
		x_min: 126,
		x_max: 885,
		o: "m 885 991 l 885 350 q 783 83 885 180 q 501 -14 680 -14 q 224 84 322 -14 q 126 353 126 182 l 126 991 l 241 991 l 241 345 q 309 154 241 220 q 509 87 377 87 q 702 154 634 87 q 770 346 770 221 l 770 991 l 885 991 m 313 1174 q 331 1225 313 1209 q 374 1240 349 1240 q 418 1225 399 1240 q 437 1174 437 1209 q 418 1123 437 1140 q 374 1107 399 1107 q 331 1123 349 1107 q 313 1174 313 1140 m 571 1174 q 589 1225 571 1209 q 632 1240 607 1240 q 676 1225 657 1240 q 694 1174 694 1209 q 676 1123 694 1140 q 632 1107 657 1107 q 589 1123 607 1107 q 571 1174 571 1140 z "
	},
	"Ý": {
		ha: 778,
		x_min: 0,
		x_max: 778,
		o: "m 389 496 l 653 991 l 778 991 l 447 385 l 447 0 l 330 0 l 330 379 l 0 991 l 126 991 l 389 496 m 300 1088 q 370 1190 332 1130 q 429 1293 408 1249 l 566 1293 l 566 1279 q 477 1171 536 1235 q 375 1071 418 1106 l 300 1071 l 300 1088 z "
	},
	"Þ": {
		ha: 848,
		x_min: 136,
		x_max: 777,
		o: "m 777 532 q 674 297 777 378 q 376 216 571 216 l 252 216 l 252 0 l 136 0 l 136 991 l 252 991 l 252 818 l 397 818 q 682 748 588 818 q 777 532 777 677 m 252 315 l 366 315 q 587 364 519 315 q 656 524 656 412 q 591 671 656 625 q 390 718 527 718 l 252 718 l 252 315 z "
	},
	"ß": {
		ha: 864,
		x_min: 119,
		x_max: 800,
		o: "m 711 859 q 614 689 711 767 q 536 619 555 642 q 517 574 517 596 q 526 538 517 552 q 559 504 535 524 q 636 450 583 485 q 766 333 731 386 q 800 211 800 279 q 734 44 800 102 q 547 -14 669 -14 q 347 33 420 -14 l 347 138 q 443 95 390 111 q 545 79 496 79 q 690 203 690 79 q 662 290 690 254 q 559 374 634 326 q 441 471 473 429 q 408 570 408 513 q 432 648 408 612 q 503 720 455 684 q 576 789 554 759 q 597 856 597 820 q 551 939 597 910 q 419 968 505 968 q 232 817 232 968 l 232 0 l 119 0 l 119 815 q 194 999 119 936 q 419 1063 269 1063 q 635 1009 559 1063 q 711 859 711 956 z "
	},
	"à": {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 456 842 l 381 842 q 277 942 337 877 q 189 1050 216 1007 l 189 1064 l 327 1064 q 387 956 349 1017 q 456 859 426 895 l 456 842 z "
	},
	"á": {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 296 859 q 366 960 328 901 q 425 1064 404 1020 l 562 1064 l 562 1050 q 473 941 532 1006 q 371 842 414 877 l 296 842 l 296 859 z "
	},
	"â": {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 155 857 q 275 993 241 949 q 326 1064 310 1036 l 438 1064 q 490 990 453 1036 q 612 857 527 945 l 612 842 l 531 842 q 381 968 471 879 q 233 842 289 877 l 155 842 l 155 857 z "
	},
	"ã": {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 489 843 q 432 856 460 843 q 377 883 404 868 q 326 911 351 899 q 278 924 301 924 q 227 903 244 924 q 200 842 210 883 l 134 842 q 181 970 142 924 q 282 1017 220 1017 q 342 1004 313 1017 q 398 976 372 991 q 449 948 425 961 q 495 936 473 936 q 545 956 528 936 q 571 1018 561 976 l 638 1018 q 591 889 629 936 q 489 843 553 843 z "
	},
	"ä": {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 189 945 q 207 996 189 980 q 250 1011 225 1011 q 295 996 276 1011 q 313 945 313 980 q 295 894 313 911 q 250 878 276 878 q 207 894 225 878 q 189 945 189 911 m 448 945 q 466 996 448 980 q 509 1011 484 1011 q 552 996 534 1011 q 571 945 571 980 q 552 894 571 911 q 509 878 534 878 q 466 894 484 878 q 448 945 448 911 z "
	},
	"å": {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 545 989 q 504 882 545 922 q 393 842 462 842 q 283 881 324 842 q 243 987 243 921 q 284 1093 243 1054 q 393 1132 325 1132 q 503 1092 461 1132 q 545 989 545 1051 m 472 987 q 450 1046 472 1025 q 393 1067 427 1067 q 336 1046 358 1067 q 313 987 313 1025 q 334 929 313 949 q 393 908 354 908 q 450 929 428 908 q 472 987 472 949 z "
	},
	"æ": {
		ha: 1192,
		x_min: 64,
		x_max: 1120,
		o: "m 64 205 q 148 375 64 315 q 404 441 232 436 l 529 446 l 529 492 q 490 621 529 579 q 370 663 450 663 q 161 606 272 663 l 126 692 q 244 738 176 720 q 378 755 311 755 q 522 726 466 755 q 606 632 578 696 q 700 724 642 692 q 832 757 758 757 q 1041 666 962 757 q 1120 425 1120 576 l 1120 353 l 644 353 q 863 85 650 85 q 978 97 924 85 q 1088 135 1031 109 l 1088 35 q 979 -2 1029 9 q 860 -14 928 -14 q 579 144 664 -14 q 457 22 524 58 q 300 -14 391 -14 q 126 44 189 -14 q 64 205 64 102 m 182 204 q 218 110 182 140 q 314 79 254 79 q 469 137 412 79 q 526 298 526 194 l 526 366 l 419 361 q 237 318 293 355 q 182 204 182 281 m 831 663 q 702 606 749 663 q 647 443 654 550 l 999 443 q 956 606 999 549 q 831 663 912 663 z "
	},
	"ç": {
		ha: 661,
		x_min: 78,
		x_max: 615,
		o: "m 416 -14 q 166 86 255 -14 q 78 367 78 185 q 168 655 78 553 q 424 757 258 757 q 531 745 477 757 q 615 718 585 734 l 581 623 q 499 647 543 637 q 421 657 455 657 q 195 368 195 657 q 250 158 195 231 q 414 85 305 85 q 604 125 507 85 l 604 25 q 416 -14 530 -14 m 517 -196 q 465 -298 517 -262 q 311 -334 413 -334 q 246 -328 277 -334 l 246 -256 q 317 -261 277 -261 q 398 -248 370 -261 q 425 -197 425 -234 q 398 -150 425 -168 q 298 -121 372 -132 l 357 0 l 432 0 l 395 -78 q 517 -196 517 -104 z "
	},
	"è": {
		ha: 779,
		x_min: 78,
		x_max: 707,
		o: "m 433 -14 q 173 87 269 -14 q 78 366 78 187 q 166 651 78 545 q 404 757 255 757 q 625 665 544 757 q 707 422 707 573 l 707 351 l 195 351 q 261 153 198 220 q 437 85 323 85 q 675 135 557 85 l 675 35 q 562 -2 615 9 q 433 -14 509 -14 m 403 663 q 260 604 313 663 q 197 443 207 546 l 586 443 q 538 606 586 549 q 403 663 491 663 m 482 842 l 408 842 q 303 942 363 877 q 216 1050 243 1007 l 216 1064 l 353 1064 q 414 956 375 1017 q 482 859 452 895 l 482 842 z "
	},
	"é": {
		ha: 779,
		x_min: 78,
		x_max: 707,
		o: "m 433 -14 q 173 87 269 -14 q 78 366 78 187 q 166 651 78 545 q 404 757 255 757 q 625 665 544 757 q 707 422 707 573 l 707 351 l 195 351 q 261 153 198 220 q 437 85 323 85 q 675 135 557 85 l 675 35 q 562 -2 615 9 q 433 -14 509 -14 m 403 663 q 260 604 313 663 q 197 443 207 546 l 586 443 q 538 606 586 549 q 403 663 491 663 m 319 859 q 390 960 352 901 q 449 1064 427 1020 l 586 1064 l 586 1050 q 497 941 556 1006 q 395 842 438 877 l 319 842 l 319 859 z "
	},
	"ê": {
		ha: 779,
		x_min: 78,
		x_max: 707,
		o: "m 433 -14 q 173 87 269 -14 q 78 366 78 187 q 166 651 78 545 q 404 757 255 757 q 625 665 544 757 q 707 422 707 573 l 707 351 l 195 351 q 261 153 198 220 q 437 85 323 85 q 675 135 557 85 l 675 35 q 562 -2 615 9 q 433 -14 509 -14 m 403 663 q 260 604 313 663 q 197 443 207 546 l 586 443 q 538 606 586 549 q 403 663 491 663 m 176 857 q 296 993 262 949 q 347 1064 331 1036 l 459 1064 q 511 990 474 1036 q 633 857 548 945 l 633 842 l 552 842 q 402 968 492 879 q 254 842 310 877 l 176 842 l 176 857 z "
	},
	"ë": {
		ha: 779,
		x_min: 78,
		x_max: 707,
		o: "m 433 -14 q 173 87 269 -14 q 78 366 78 187 q 166 651 78 545 q 404 757 255 757 q 625 665 544 757 q 707 422 707 573 l 707 351 l 195 351 q 261 153 198 220 q 437 85 323 85 q 675 135 557 85 l 675 35 q 562 -2 615 9 q 433 -14 509 -14 m 403 663 q 260 604 313 663 q 197 443 207 546 l 586 443 q 538 606 586 549 q 403 663 491 663 m 216 945 q 234 996 216 980 q 277 1011 252 1011 q 322 996 303 1011 q 340 945 340 980 q 322 894 340 911 q 277 878 303 878 q 234 894 252 878 q 216 945 216 911 m 475 945 q 493 996 475 980 q 536 1011 511 1011 q 579 996 561 1011 q 598 945 598 980 q 579 894 598 911 q 536 878 561 878 q 493 894 511 878 q 475 945 475 911 z "
	},
	"ì": {
		ha: 351,
		x_min: -26,
		x_max: 241,
		o: "m 232 0 l 119 0 l 119 743 l 232 743 l 232 0 m 241 842 l 166 842 q 62 942 122 877 q -26 1050 1 1007 l -26 1064 l 112 1064 q 172 956 134 1017 q 241 859 211 895 l 241 842 z "
	},
	"í": {
		ha: 351,
		x_min: 115,
		x_max: 381,
		o: "m 232 0 l 119 0 l 119 743 l 232 743 l 232 0 m 115 859 q 185 960 147 901 q 244 1064 222 1020 l 381 1064 l 381 1050 q 292 941 351 1006 q 190 842 233 877 l 115 842 l 115 859 z "
	},
	"î": {
		ha: 351,
		x_min: -52,
		x_max: 405,
		o: "m 232 0 l 119 0 l 119 743 l 232 743 l 232 0 m -52 857 q 68 993 34 949 q 119 1064 103 1036 l 231 1064 q 283 990 246 1036 q 405 857 320 945 l 405 842 l 324 842 q 174 968 264 879 q 26 842 82 877 l -52 842 l -52 857 z "
	},
	"ï": {
		ha: 351,
		x_min: -14,
		x_max: 368,
		o: "m 232 0 l 119 0 l 119 743 l 232 743 l 232 0 m -14 945 q 4 996 -14 980 q 47 1011 22 1011 q 92 996 73 1011 q 111 945 111 980 q 92 894 111 911 q 47 878 73 878 q 4 894 22 878 q -14 945 -14 911 m 245 945 q 263 996 245 980 q 306 1011 281 1011 q 350 996 331 1011 q 368 945 368 980 q 350 894 368 911 q 306 878 331 878 q 263 894 281 878 q 245 945 245 911 z "
	},
	"ð": {
		ha: 828,
		x_min: 77,
		x_max: 761,
		o: "m 761 382 q 672 89 761 191 q 416 -14 584 -14 q 171 78 266 -14 q 77 322 77 169 q 166 567 77 478 q 404 656 255 656 q 625 574 557 656 l 631 576 q 453 851 592 722 l 269 746 l 220 819 l 378 909 q 252 985 315 951 l 298 1064 q 473 964 404 1015 l 635 1057 l 686 985 l 546 904 q 705 672 649 807 q 761 382 761 537 m 647 347 q 586 505 647 447 q 419 562 525 562 q 191 318 191 562 q 250 143 191 205 q 419 81 309 81 q 592 149 538 81 q 647 347 647 217 z "
	},
	"ñ": {
		ha: 852,
		x_min: 119,
		x_max: 741,
		o: "m 628 0 l 628 481 q 587 616 628 572 q 457 661 545 661 q 286 598 340 661 q 232 390 232 535 l 232 0 l 119 0 l 119 743 l 211 743 l 229 642 l 235 642 q 332 727 269 696 q 471 757 394 757 q 673 692 605 757 q 741 485 741 627 l 741 0 l 628 0 m 544 843 q 487 856 515 843 q 432 883 459 868 q 381 911 406 899 q 333 924 356 924 q 282 903 299 924 q 255 842 264 883 l 189 842 q 236 970 197 924 q 337 1017 275 1017 q 397 1004 368 1017 q 453 976 427 991 q 504 948 479 961 q 550 936 528 936 q 600 956 583 936 q 626 1018 616 976 l 693 1018 q 646 889 684 936 q 544 843 608 843 z "
	},
	"ò": {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 195 372 q 252 155 195 230 q 419 81 309 81 q 587 155 530 81 q 644 372 644 229 q 587 588 644 514 q 418 661 530 661 q 251 589 307 661 q 195 372 195 516 m 503 842 l 429 842 q 324 942 385 877 q 237 1050 264 1007 l 237 1064 l 374 1064 q 435 956 396 1017 q 503 859 473 895 l 503 842 z "
	},
	"ó": {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 195 372 q 252 155 195 230 q 419 81 309 81 q 587 155 530 81 q 644 372 644 229 q 587 588 644 514 q 418 661 530 661 q 251 589 307 661 q 195 372 195 516 m 325 859 q 395 960 357 901 q 454 1064 433 1020 l 591 1064 l 591 1050 q 503 941 562 1006 q 400 842 444 877 l 325 842 l 325 859 z "
	},
	"ô": {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 195 372 q 252 155 195 230 q 419 81 309 81 q 587 155 530 81 q 644 372 644 229 q 587 588 644 514 q 418 661 530 661 q 251 589 307 661 q 195 372 195 516 m 191 857 q 312 993 277 949 q 362 1064 347 1036 l 475 1064 q 527 990 490 1036 q 648 857 564 945 l 648 842 l 568 842 q 418 968 508 879 q 269 842 326 877 l 191 842 l 191 857 z "
	},
	"õ": {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 195 372 q 252 155 195 230 q 419 81 309 81 q 587 155 530 81 q 644 372 644 229 q 587 588 644 514 q 418 661 530 661 q 251 589 307 661 q 195 372 195 516 m 524 843 q 467 856 495 843 q 413 883 439 868 q 361 911 386 899 q 313 924 336 924 q 262 903 279 924 q 235 842 245 883 l 169 842 q 217 970 178 924 q 317 1017 256 1017 q 378 1004 349 1017 q 433 976 407 991 q 484 948 460 961 q 530 936 509 936 q 580 956 564 936 q 606 1018 596 976 l 673 1018 q 626 889 665 936 q 524 843 588 843 z "
	},
	"ö": {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 195 372 q 252 155 195 230 q 419 81 309 81 q 587 155 530 81 q 644 372 644 229 q 587 588 644 514 q 418 661 530 661 q 251 589 307 661 q 195 372 195 516 m 228 945 q 246 996 228 980 q 289 1011 264 1011 q 333 996 315 1011 q 352 945 352 980 q 333 894 352 911 q 289 878 315 878 q 246 894 264 878 q 228 945 228 911 m 486 945 q 504 996 486 980 q 547 1011 522 1011 q 591 996 572 1011 q 610 945 610 980 q 591 894 610 911 q 547 878 572 878 q 504 894 522 878 q 486 945 486 911 z "
	},
	"÷": {
		ha: 794,
		x_min: 71,
		x_max: 722,
		o: "m 71 443 l 71 536 l 722 536 l 722 443 l 71 443 m 319 253 q 339 314 319 294 q 396 335 359 335 q 451 314 431 335 q 471 253 471 292 q 451 193 471 214 q 396 171 431 171 q 340 192 361 171 q 319 253 319 214 m 319 726 q 339 788 319 767 q 396 808 359 808 q 451 787 431 808 q 471 726 471 766 q 451 666 471 688 q 396 644 431 644 q 340 666 361 644 q 319 726 319 687 z "
	},
	"ø": {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 236 33 312 -14 l 179 -46 l 102 7 l 165 96 q 78 372 78 199 q 169 655 78 554 q 421 757 260 757 q 604 705 526 757 l 661 786 l 741 734 l 675 644 q 761 372 761 541 m 195 372 q 231 187 195 256 l 546 625 q 418 661 495 661 q 251 589 307 661 q 195 372 195 516 m 644 372 q 610 551 644 484 l 294 115 q 419 81 342 81 q 587 155 530 81 q 644 372 644 229 z "
	},
	"ù": {
		ha: 852,
		x_min: 111,
		x_max: 733,
		o: "m 225 743 l 225 261 q 267 125 225 170 q 396 81 308 81 q 567 144 513 81 q 621 353 621 208 l 621 743 l 733 743 l 733 0 l 640 0 l 624 100 l 618 100 q 522 16 583 45 q 382 -14 460 -14 q 179 51 246 -14 q 111 257 111 115 l 111 743 l 225 743 m 492 842 l 418 842 q 313 942 374 877 q 226 1050 253 1007 l 226 1064 l 363 1064 q 424 956 385 1017 q 492 859 463 895 l 492 842 z "
	},
	"ú": {
		ha: 852,
		x_min: 111,
		x_max: 733,
		o: "m 225 743 l 225 261 q 267 125 225 170 q 396 81 308 81 q 567 144 513 81 q 621 353 621 208 l 621 743 l 733 743 l 733 0 l 640 0 l 624 100 l 618 100 q 522 16 583 45 q 382 -14 460 -14 q 179 51 246 -14 q 111 257 111 115 l 111 743 l 225 743 m 343 859 q 413 960 376 901 q 473 1064 451 1020 l 610 1064 l 610 1050 q 521 941 580 1006 q 418 842 462 877 l 343 842 l 343 859 z "
	},
	"û": {
		ha: 852,
		x_min: 111,
		x_max: 733,
		o: "m 225 743 l 225 261 q 267 125 225 170 q 396 81 308 81 q 567 144 513 81 q 621 353 621 208 l 621 743 l 733 743 l 733 0 l 640 0 l 624 100 l 618 100 q 522 16 583 45 q 382 -14 460 -14 q 179 51 246 -14 q 111 257 111 115 l 111 743 l 225 743 m 194 857 q 315 993 280 949 q 365 1064 349 1036 l 477 1064 q 529 990 492 1036 q 651 857 566 945 l 651 842 l 570 842 q 420 968 511 879 q 272 842 328 877 l 194 842 l 194 857 z "
	},
	"ü": {
		ha: 852,
		x_min: 111,
		x_max: 733,
		o: "m 225 743 l 225 261 q 267 125 225 170 q 396 81 308 81 q 567 144 513 81 q 621 353 621 208 l 621 743 l 733 743 l 733 0 l 640 0 l 624 100 l 618 100 q 522 16 583 45 q 382 -14 460 -14 q 179 51 246 -14 q 111 257 111 115 l 111 743 l 225 743 m 232 945 q 250 996 232 980 q 293 1011 268 1011 q 337 996 319 1011 q 356 945 356 980 q 337 894 356 911 q 293 878 319 878 q 250 894 268 878 q 232 945 232 911 m 490 945 q 508 996 490 980 q 551 1011 526 1011 q 595 996 576 1011 q 614 945 614 980 q 595 894 614 911 q 551 878 576 878 q 508 894 526 878 q 490 945 490 911 z "
	},
	"ý": {
		ha: 700,
		x_min: 1,
		x_max: 699,
		o: "m 1 743 l 122 743 l 285 319 q 351 110 338 174 l 357 110 q 394 228 366 144 q 578 743 422 312 l 699 743 l 379 -103 q 268 -281 332 -229 q 113 -334 205 -334 q 11 -322 61 -334 l 11 -232 q 94 -240 48 -240 q 260 -110 210 -240 l 301 -4 l 1 743 m 279 859 q 349 960 311 901 q 408 1064 387 1020 l 545 1064 l 545 1050 q 456 941 515 1006 q 354 842 397 877 l 279 842 l 279 859 z "
	},
	"þ": {
		ha: 851,
		x_min: 119,
		x_max: 774,
		o: "m 233 643 q 336 730 278 703 q 465 757 393 757 q 692 655 611 757 q 774 372 774 553 q 692 89 774 191 q 465 -14 610 -14 q 232 96 315 -14 l 224 96 l 227 73 q 232 -22 232 20 l 232 -334 l 119 -334 l 119 1055 l 232 1055 l 232 739 q 228 643 232 704 l 233 643 m 450 661 q 285 599 336 661 q 232 400 233 536 l 232 372 q 284 148 232 216 q 452 81 336 81 q 657 374 657 81 q 607 590 657 519 q 450 661 557 661 z "
	},
	"ÿ": {
		ha: 700,
		x_min: 1,
		x_max: 699,
		o: "m 1 743 l 122 743 l 285 319 q 351 110 338 174 l 357 110 q 394 228 366 144 q 578 743 422 312 l 699 743 l 379 -103 q 268 -281 332 -229 q 113 -334 205 -334 q 11 -322 61 -334 l 11 -232 q 94 -240 48 -240 q 260 -110 210 -240 l 301 -4 l 1 743 m 159 945 q 177 996 159 980 q 220 1011 195 1011 q 264 996 245 1011 q 283 945 283 980 q 264 894 283 911 q 220 878 245 878 q 177 894 195 878 q 159 945 159 911 m 417 945 q 435 996 417 980 q 478 1011 453 1011 q 522 996 503 1011 q 541 945 541 980 q 522 894 541 911 q 478 878 503 878 q 435 894 453 878 q 417 945 417 911 z "
	},
	"Ā": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 247 1164 l 654 1164 l 654 1071 l 247 1071 l 247 1164 z "
	},
	"ā": {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 197 935 l 604 935 l 604 842 l 197 842 l 197 935 z "
	},
	"Ă": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 435 1071 q 287 1117 340 1071 q 228 1253 234 1164 l 298 1253 q 316 1200 302 1219 q 355 1173 330 1181 q 437 1164 380 1164 q 538 1185 506 1164 q 578 1253 571 1207 l 648 1253 q 585 1119 642 1167 q 435 1071 528 1071 z "
	},
	"ă": {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 387 842 q 239 888 292 842 q 180 1023 186 935 l 250 1023 q 268 971 254 990 q 307 943 282 952 q 389 935 332 935 q 490 956 458 935 q 530 1023 523 978 l 600 1023 q 537 890 593 938 q 387 842 480 842 z "
	},
	"Ą": {
		ha: 879,
		x_min: 0,
		x_max: 880,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 750 -161 q 814 -225 750 -225 q 880 -216 842 -225 l 880 -290 q 795 -302 836 -302 q 654 -175 654 -302 q 684 -82 654 -127 q 762 0 713 -36 l 843 0 q 773 -82 797 -45 q 750 -161 750 -119 z "
	},
	"ą": {
		ha: 772,
		x_min: 64,
		x_max: 694,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 565 -161 q 629 -225 565 -225 q 694 -216 657 -225 l 694 -290 q 610 -302 650 -302 q 469 -175 469 -302 q 498 -82 469 -127 q 576 0 528 -36 l 658 0 q 588 -82 612 -45 q 565 -161 565 -119 z "
	},
	"Ć": {
		ha: 876,
		x_min: 85,
		x_max: 835,
		o: "m 561 903 q 303 794 397 903 q 208 496 208 685 q 299 195 208 301 q 559 89 391 89 q 796 126 663 89 l 796 25 q 542 -14 693 -14 q 204 119 323 -14 q 85 497 85 252 q 142 766 85 650 q 308 943 199 881 q 562 1006 416 1006 q 835 949 718 1006 l 786 850 q 561 903 673 903 m 446 1088 q 516 1190 478 1130 q 575 1293 553 1249 l 712 1293 l 712 1279 q 623 1171 682 1235 q 521 1071 564 1106 l 446 1071 l 446 1088 z "
	},
	"ć": {
		ha: 661,
		x_min: 78,
		x_max: 615,
		o: "m 416 -14 q 166 86 255 -14 q 78 367 78 185 q 168 655 78 553 q 424 757 258 757 q 531 745 477 757 q 615 718 585 734 l 581 623 q 499 647 543 637 q 421 657 455 657 q 195 368 195 657 q 250 158 195 231 q 414 85 305 85 q 604 125 507 85 l 604 25 q 416 -14 530 -14 m 313 859 q 383 960 345 901 q 442 1064 420 1020 l 579 1064 l 579 1050 q 490 941 549 1006 q 388 842 431 877 l 313 842 l 313 859 z "
	},
	"Ĉ": {
		ha: 876,
		x_min: 85,
		x_max: 835,
		o: "m 561 903 q 303 794 397 903 q 208 496 208 685 q 299 195 208 301 q 559 89 391 89 q 796 126 663 89 l 796 25 q 542 -14 693 -14 q 204 119 323 -14 q 85 497 85 252 q 142 766 85 650 q 308 943 199 881 q 562 1006 416 1006 q 835 949 718 1006 l 786 850 q 561 903 673 903 m 298 1086 q 419 1222 385 1179 q 469 1293 454 1265 l 582 1293 q 634 1220 597 1265 q 755 1086 671 1175 l 755 1071 l 675 1071 q 525 1197 615 1108 q 376 1071 433 1106 l 298 1071 l 298 1086 z "
	},
	"ĉ": {
		ha: 661,
		x_min: 78,
		x_max: 615,
		o: "m 416 -14 q 166 86 255 -14 q 78 367 78 185 q 168 655 78 553 q 424 757 258 757 q 531 745 477 757 q 615 718 585 734 l 581 623 q 499 647 543 637 q 421 657 455 657 q 195 368 195 657 q 250 158 195 231 q 414 85 305 85 q 604 125 507 85 l 604 25 q 416 -14 530 -14 m 152 857 q 273 993 238 949 q 323 1064 307 1036 l 435 1064 q 487 990 450 1036 q 609 857 524 945 l 609 842 l 528 842 q 378 968 469 879 q 230 842 286 877 l 152 842 l 152 857 z "
	},
	"Ċ": {
		ha: 876,
		x_min: 85,
		x_max: 835,
		o: "m 561 903 q 303 794 397 903 q 208 496 208 685 q 299 195 208 301 q 559 89 391 89 q 796 126 663 89 l 796 25 q 542 -14 693 -14 q 204 119 323 -14 q 85 497 85 252 q 142 766 85 650 q 308 943 199 881 q 562 1006 416 1006 q 835 949 718 1006 l 786 850 q 561 903 673 903 m 475 1174 q 494 1231 475 1213 q 542 1249 513 1249 q 589 1230 569 1249 q 608 1174 608 1212 q 589 1117 608 1136 q 542 1099 569 1099 q 494 1117 513 1099 q 475 1174 475 1136 z "
	},
	"ċ": {
		ha: 661,
		x_min: 78,
		x_max: 615,
		o: "m 416 -14 q 166 86 255 -14 q 78 367 78 185 q 168 655 78 553 q 424 757 258 757 q 531 745 477 757 q 615 718 585 734 l 581 623 q 499 647 543 637 q 421 657 455 657 q 195 368 195 657 q 250 158 195 231 q 414 85 305 85 q 604 125 507 85 l 604 25 q 416 -14 530 -14 m 338 945 q 357 1001 338 983 q 404 1019 376 1019 q 451 1001 431 1019 q 471 945 471 983 q 451 888 471 907 q 404 869 431 869 q 357 888 376 869 q 338 945 338 907 z "
	},
	"Č": {
		ha: 876,
		x_min: 85,
		x_max: 835,
		o: "m 561 903 q 303 794 397 903 q 208 496 208 685 q 299 195 208 301 q 559 89 391 89 q 796 126 663 89 l 796 25 q 542 -14 693 -14 q 204 119 323 -14 q 85 497 85 252 q 142 766 85 650 q 308 943 199 881 q 562 1006 416 1006 q 835 949 718 1006 l 786 850 q 561 903 673 903 m 313 1293 l 391 1293 q 539 1165 468 1243 q 689 1293 627 1253 l 770 1293 l 770 1276 l 725 1230 q 596 1071 627 1130 l 484 1071 q 433 1141 468 1099 q 313 1276 399 1184 l 313 1293 z "
	},
	"č": {
		ha: 661,
		x_min: 78,
		x_max: 630,
		o: "m 416 -14 q 166 86 255 -14 q 78 367 78 185 q 168 655 78 553 q 424 757 258 757 q 531 745 477 757 q 615 718 585 734 l 581 623 q 499 647 543 637 q 421 657 455 657 q 195 368 195 657 q 250 158 195 231 q 414 85 305 85 q 604 125 507 85 l 604 25 q 416 -14 530 -14 m 173 1064 l 251 1064 q 399 936 328 1014 q 549 1064 488 1024 l 630 1064 l 630 1047 l 585 1001 q 456 842 488 901 l 344 842 q 294 912 328 869 q 173 1047 259 955 l 173 1064 z "
	},
	"Ď": {
		ha: 1013,
		x_min: 136,
		x_max: 928,
		o: "m 928 505 q 794 130 928 260 q 411 0 661 0 l 136 0 l 136 991 l 440 991 q 800 863 671 991 q 928 505 928 735 m 806 501 q 708 793 806 695 q 419 892 611 892 l 252 892 l 252 100 l 392 100 q 702 201 598 100 q 806 501 806 302 m 241 1293 l 319 1293 q 468 1165 397 1243 q 618 1293 556 1253 l 699 1293 l 699 1276 l 654 1230 q 525 1071 556 1130 l 412 1071 q 362 1141 397 1099 q 241 1276 328 1184 l 241 1293 z "
	},
	"ď": {
		ha: 851,
		x_min: 78,
		x_max: 956,
		o: "m 625 100 l 619 100 q 386 -14 541 -14 q 159 86 240 -14 q 78 370 78 186 q 159 655 78 553 q 386 757 241 757 q 618 647 537 757 l 627 647 l 622 701 l 619 753 l 619 1055 l 732 1055 l 732 0 l 640 0 l 625 100 m 400 81 q 567 143 515 81 q 619 346 619 206 l 619 370 q 567 595 619 528 q 399 663 514 663 q 247 586 300 663 q 195 368 195 509 q 247 153 195 226 q 400 81 299 81 m 795 850 q 822 944 808 887 q 842 1055 835 1000 l 956 1055 l 956 1043 q 918 945 948 1008 q 857 833 888 881 l 795 833 l 795 850 z "
	},
	"Đ": {
		ha: 1003,
		x_min: 32,
		x_max: 917,
		o: "m 917 505 q 784 130 917 260 q 400 0 650 0 l 136 0 l 136 440 l 32 440 l 32 542 l 136 542 l 136 991 l 431 991 q 788 864 659 991 q 917 505 917 737 m 794 501 q 408 892 794 892 l 252 892 l 252 542 l 509 542 l 509 440 l 252 440 l 252 100 l 380 100 q 794 501 794 100 z "
	},
	"đ": {
		ha: 851,
		x_min: 78,
		x_max: 838,
		o: "m 625 100 l 619 100 q 386 -14 541 -14 q 159 86 240 -14 q 78 370 78 186 q 159 655 78 553 q 386 757 241 757 q 618 647 537 757 l 627 647 q 619 753 619 703 l 619 843 l 315 843 l 315 930 l 619 930 l 619 1055 l 732 1055 l 732 930 l 838 930 l 838 843 l 732 843 l 732 0 l 640 0 l 625 100 m 400 81 q 567 143 515 81 q 619 346 619 206 l 619 370 q 567 595 619 528 q 399 663 514 663 q 247 586 300 663 q 195 368 195 509 q 247 153 195 226 q 400 81 299 81 z "
	},
	"Ē": {
		ha: 772,
		x_min: 136,
		x_max: 689,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 m 216 1164 l 623 1164 l 623 1071 l 216 1071 l 216 1164 z "
	},
	"ē": {
		ha: 779,
		x_min: 78,
		x_max: 707,
		o: "m 433 -14 q 173 87 269 -14 q 78 366 78 187 q 166 651 78 545 q 404 757 255 757 q 625 665 544 757 q 707 422 707 573 l 707 351 l 195 351 q 261 153 198 220 q 437 85 323 85 q 675 135 557 85 l 675 35 q 562 -2 615 9 q 433 -14 509 -14 m 403 663 q 260 604 313 663 q 197 443 207 546 l 586 443 q 538 606 586 549 q 403 663 491 663 m 211 935 l 618 935 l 618 842 l 211 842 l 211 935 z "
	},
	"Ĕ": {
		ha: 772,
		x_min: 136,
		x_max: 689,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 m 416 1071 q 269 1117 321 1071 q 210 1253 216 1164 l 280 1253 q 298 1200 284 1219 q 337 1173 312 1181 q 419 1164 361 1164 q 520 1185 488 1164 q 559 1253 553 1207 l 630 1253 q 567 1119 623 1167 q 416 1071 510 1071 z "
	},
	"ĕ": {
		ha: 779,
		x_min: 78,
		x_max: 707,
		o: "m 433 -14 q 173 87 269 -14 q 78 366 78 187 q 166 651 78 545 q 404 757 255 757 q 625 665 544 757 q 707 422 707 573 l 707 351 l 195 351 q 261 153 198 220 q 437 85 323 85 q 675 135 557 85 l 675 35 q 562 -2 615 9 q 433 -14 509 -14 m 403 663 q 260 604 313 663 q 197 443 207 546 l 586 443 q 538 606 586 549 q 403 663 491 663 m 402 842 q 254 888 307 842 q 195 1023 201 935 l 266 1023 q 284 971 270 990 q 322 943 298 952 q 405 935 347 935 q 506 956 473 935 q 545 1023 538 978 l 616 1023 q 552 890 609 938 q 402 842 496 842 z "
	},
	"Ė": {
		ha: 772,
		x_min: 136,
		x_max: 689,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 m 359 1154 q 378 1211 359 1193 q 425 1229 397 1229 q 472 1211 452 1229 q 492 1154 492 1192 q 472 1098 492 1116 q 425 1079 452 1079 q 378 1098 397 1079 q 359 1154 359 1116 z "
	},
	"ė": {
		ha: 779,
		x_min: 78,
		x_max: 707,
		o: "m 433 -14 q 173 87 269 -14 q 78 366 78 187 q 166 651 78 545 q 404 757 255 757 q 625 665 544 757 q 707 422 707 573 l 707 351 l 195 351 q 261 153 198 220 q 437 85 323 85 q 675 135 557 85 l 675 35 q 562 -2 615 9 q 433 -14 509 -14 m 403 663 q 260 604 313 663 q 197 443 207 546 l 586 443 q 538 606 586 549 q 403 663 491 663 m 340 945 q 359 1001 340 983 q 407 1019 378 1019 q 454 1001 434 1019 q 473 945 473 983 q 454 888 473 907 q 407 869 434 869 q 359 888 378 869 q 340 945 340 907 z "
	},
	"Ę": {
		ha: 772,
		x_min: 136,
		x_max: 689,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 m 546 -161 q 610 -225 546 -225 q 675 -216 638 -225 l 675 -290 q 591 -302 631 -302 q 450 -175 450 -302 q 479 -82 450 -127 q 557 0 509 -36 l 639 0 q 569 -82 593 -45 q 546 -161 546 -119 z "
	},
	"ę": {
		ha: 779,
		x_min: 78,
		x_max: 707,
		o: "m 433 -14 q 173 87 269 -14 q 78 366 78 187 q 166 651 78 545 q 404 757 255 757 q 625 665 544 757 q 707 422 707 573 l 707 351 l 195 351 q 261 153 198 220 q 437 85 323 85 q 675 135 557 85 l 675 35 q 562 -2 615 9 q 433 -14 509 -14 m 403 663 q 260 604 313 663 q 197 443 207 546 l 586 443 q 538 606 586 549 q 403 663 491 663 m 537 -140 q 601 -204 537 -204 q 667 -195 629 -204 l 667 -269 q 582 -281 623 -281 q 441 -154 441 -281 q 471 -61 441 -106 q 549 21 500 -15 l 630 21 q 561 -61 584 -24 q 537 -140 537 -98 z "
	},
	"Ě": {
		ha: 772,
		x_min: 136,
		x_max: 689,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 m 193 1293 l 271 1293 q 419 1165 348 1243 q 569 1293 507 1253 l 650 1293 l 650 1276 l 605 1230 q 476 1071 507 1130 l 363 1071 q 313 1141 348 1099 q 193 1276 279 1184 l 193 1293 z "
	},
	"ě": {
		ha: 779,
		x_min: 78,
		x_max: 707,
		o: "m 433 -14 q 173 87 269 -14 q 78 366 78 187 q 166 651 78 545 q 404 757 255 757 q 625 665 544 757 q 707 422 707 573 l 707 351 l 195 351 q 261 153 198 220 q 437 85 323 85 q 675 135 557 85 l 675 35 q 562 -2 615 9 q 433 -14 509 -14 m 403 663 q 260 604 313 663 q 197 443 207 546 l 586 443 q 538 606 586 549 q 403 663 491 663 m 178 1064 l 256 1064 q 405 936 334 1014 q 555 1064 493 1024 l 635 1064 l 635 1047 l 591 1001 q 462 842 493 901 l 349 842 q 299 912 334 869 q 178 1047 264 955 l 178 1064 z "
	},
	"Ĝ": {
		ha: 1011,
		x_min: 85,
		x_max: 909,
		o: "m 572 519 l 909 519 l 909 37 q 749 -1 831 12 q 561 -14 668 -14 q 210 120 336 -14 q 85 496 85 254 q 147 767 85 650 q 326 944 209 883 q 599 1006 442 1006 q 895 947 758 1006 l 850 846 q 591 903 715 903 q 309 795 410 903 q 207 496 207 687 q 305 191 207 295 q 593 87 403 87 q 794 111 696 87 l 794 416 l 572 416 l 572 519 m 340 1086 q 460 1222 426 1179 q 511 1293 495 1265 l 623 1293 q 675 1220 638 1265 q 797 1086 712 1175 l 797 1071 l 716 1071 q 566 1197 656 1108 q 418 1071 474 1106 l 340 1071 l 340 1086 z "
	},
	"ĝ": {
		ha: 761,
		x_min: 26,
		x_max: 728,
		o: "m 728 743 l 728 672 l 590 656 q 624 594 609 632 q 639 507 639 555 q 564 333 639 398 q 359 268 490 268 q 297 273 326 268 q 225 178 225 235 q 250 133 225 147 q 336 118 275 118 l 468 118 q 653 67 589 118 q 718 -81 718 16 q 619 -269 718 -204 q 330 -334 520 -334 q 105 -279 184 -334 q 26 -126 26 -225 q 70 -9 26 -58 q 192 58 113 41 q 144 98 163 71 q 125 161 125 125 q 146 233 125 202 q 215 292 168 263 q 121 372 157 315 q 85 503 85 429 q 158 691 85 625 q 366 757 231 757 q 471 743 424 757 l 728 743 m 135 -125 q 186 -216 135 -185 q 332 -248 237 -248 q 542 -205 473 -248 q 610 -90 610 -163 q 572 -6 610 -30 q 432 17 535 17 l 297 17 q 178 -20 220 17 q 135 -125 135 -56 m 196 505 q 240 387 196 427 q 363 347 284 347 q 528 507 528 347 q 361 675 528 675 q 239 632 281 675 q 196 505 196 589 m 145 857 q 266 993 231 949 q 316 1064 300 1036 l 429 1064 q 480 990 444 1036 q 602 857 517 945 l 602 842 l 522 842 q 372 968 462 879 q 223 842 279 877 l 145 842 l 145 857 z "
	},
	"Ğ": {
		ha: 1011,
		x_min: 85,
		x_max: 909,
		o: "m 572 519 l 909 519 l 909 37 q 749 -1 831 12 q 561 -14 668 -14 q 210 120 336 -14 q 85 496 85 254 q 147 767 85 650 q 326 944 209 883 q 599 1006 442 1006 q 895 947 758 1006 l 850 846 q 591 903 715 903 q 309 795 410 903 q 207 496 207 687 q 305 191 207 295 q 593 87 403 87 q 794 111 696 87 l 794 416 l 572 416 l 572 519 m 579 1071 q 431 1117 484 1071 q 372 1253 378 1164 l 443 1253 q 461 1200 447 1219 q 499 1173 475 1181 q 582 1164 524 1164 q 683 1185 650 1164 q 722 1253 715 1207 l 793 1253 q 729 1119 786 1167 q 579 1071 673 1071 z "
	},
	"ğ": {
		ha: 761,
		x_min: 26,
		x_max: 728,
		o: "m 728 743 l 728 672 l 590 656 q 624 594 609 632 q 639 507 639 555 q 564 333 639 398 q 359 268 490 268 q 297 273 326 268 q 225 178 225 235 q 250 133 225 147 q 336 118 275 118 l 468 118 q 653 67 589 118 q 718 -81 718 16 q 619 -269 718 -204 q 330 -334 520 -334 q 105 -279 184 -334 q 26 -126 26 -225 q 70 -9 26 -58 q 192 58 113 41 q 144 98 163 71 q 125 161 125 125 q 146 233 125 202 q 215 292 168 263 q 121 372 157 315 q 85 503 85 429 q 158 691 85 625 q 366 757 231 757 q 471 743 424 757 l 728 743 m 135 -125 q 186 -216 135 -185 q 332 -248 237 -248 q 542 -205 473 -248 q 610 -90 610 -163 q 572 -6 610 -30 q 432 17 535 17 l 297 17 q 178 -20 220 17 q 135 -125 135 -56 m 196 505 q 240 387 196 427 q 363 347 284 347 q 528 507 528 347 q 361 675 528 675 q 239 632 281 675 q 196 505 196 589 m 372 842 q 224 888 277 842 q 165 1023 171 935 l 235 1023 q 253 971 239 990 q 292 943 267 952 q 374 935 317 935 q 475 956 443 935 q 515 1023 508 978 l 585 1023 q 522 890 578 938 q 372 842 465 842 z "
	},
	"Ġ": {
		ha: 1011,
		x_min: 85,
		x_max: 909,
		o: "m 572 519 l 909 519 l 909 37 q 749 -1 831 12 q 561 -14 668 -14 q 210 120 336 -14 q 85 496 85 254 q 147 767 85 650 q 326 944 209 883 q 599 1006 442 1006 q 895 947 758 1006 l 850 846 q 591 903 715 903 q 309 795 410 903 q 207 496 207 687 q 305 191 207 295 q 593 87 403 87 q 794 111 696 87 l 794 416 l 572 416 l 572 519 m 525 1174 q 544 1231 525 1213 q 591 1249 563 1249 q 638 1230 618 1249 q 658 1174 658 1212 q 638 1117 658 1136 q 591 1099 618 1099 q 544 1117 563 1099 q 525 1174 525 1136 z "
	},
	"ġ": {
		ha: 761,
		x_min: 26,
		x_max: 728,
		o: "m 728 743 l 728 672 l 590 656 q 624 594 609 632 q 639 507 639 555 q 564 333 639 398 q 359 268 490 268 q 297 273 326 268 q 225 178 225 235 q 250 133 225 147 q 336 118 275 118 l 468 118 q 653 67 589 118 q 718 -81 718 16 q 619 -269 718 -204 q 330 -334 520 -334 q 105 -279 184 -334 q 26 -126 26 -225 q 70 -9 26 -58 q 192 58 113 41 q 144 98 163 71 q 125 161 125 125 q 146 233 125 202 q 215 292 168 263 q 121 372 157 315 q 85 503 85 429 q 158 691 85 625 q 366 757 231 757 q 471 743 424 757 l 728 743 m 135 -125 q 186 -216 135 -185 q 332 -248 237 -248 q 542 -205 473 -248 q 610 -90 610 -163 q 572 -6 610 -30 q 432 17 535 17 l 297 17 q 178 -20 220 17 q 135 -125 135 -56 m 196 505 q 240 387 196 427 q 363 347 284 347 q 528 507 528 347 q 361 675 528 675 q 239 632 281 675 q 196 505 196 589 m 304 945 q 323 1001 304 983 q 371 1019 342 1019 q 418 1001 398 1019 q 437 945 437 983 q 418 888 437 907 q 371 869 398 869 q 323 888 342 869 q 304 945 304 907 z "
	},
	"Ģ": {
		ha: 1011,
		x_min: 85,
		x_max: 909,
		o: "m 572 519 l 909 519 l 909 37 q 749 -1 831 12 q 561 -14 668 -14 q 210 120 336 -14 q 85 496 85 254 q 147 767 85 650 q 326 944 209 883 q 599 1006 442 1006 q 895 947 758 1006 l 850 846 q 591 903 715 903 q 309 795 410 903 q 207 496 207 687 q 305 191 207 295 q 593 87 403 87 q 794 111 696 87 l 794 416 l 572 416 l 572 519 m 450 -290 q 487 -184 469 -247 q 509 -85 504 -121 l 623 -85 l 623 -97 q 582 -197 615 -132 q 511 -307 549 -262 l 450 -307 l 450 -290 z "
	},
	"ģ": {
		ha: 761,
		x_min: 26,
		x_max: 728,
		o: "m 728 743 l 728 672 l 590 656 q 624 594 609 632 q 639 507 639 555 q 564 333 639 398 q 359 268 490 268 q 297 273 326 268 q 225 178 225 235 q 250 133 225 147 q 336 118 275 118 l 468 118 q 653 67 589 118 q 718 -81 718 16 q 619 -269 718 -204 q 330 -334 520 -334 q 105 -279 184 -334 q 26 -126 26 -225 q 70 -9 26 -58 q 192 58 113 41 q 144 98 163 71 q 125 161 125 125 q 146 233 125 202 q 215 292 168 263 q 121 372 157 315 q 85 503 85 429 q 158 691 85 625 q 366 757 231 757 q 471 743 424 757 l 728 743 m 135 -125 q 186 -216 135 -185 q 332 -248 237 -248 q 542 -205 473 -248 q 610 -90 610 -163 q 572 -6 610 -30 q 432 17 535 17 l 297 17 q 178 -20 220 17 q 135 -125 135 -56 m 196 505 q 240 387 196 427 q 363 347 284 347 q 528 507 528 347 q 361 675 528 675 q 239 632 281 675 q 196 505 196 589 m 479 1047 q 442 940 460 1006 q 420 842 424 875 l 307 842 l 307 854 q 350 961 317 896 q 417 1064 384 1025 l 479 1064 l 479 1047 z "
	},
	"Ĥ": {
		ha: 1025,
		x_min: 136,
		x_max: 889,
		o: "m 889 0 l 774 0 l 774 467 l 252 467 l 252 0 l 136 0 l 136 991 l 252 991 l 252 570 l 774 570 l 774 991 l 889 991 l 889 0 m 283 1086 q 404 1222 370 1179 q 454 1293 439 1265 l 567 1293 q 619 1220 582 1265 q 741 1086 656 1175 l 741 1071 l 660 1071 q 510 1197 600 1108 q 361 1071 418 1106 l 283 1071 l 283 1086 z "
	},
	"ĥ": {
		ha: 852,
		x_min: 119,
		x_max: 741,
		o: "m 628 0 l 628 481 q 587 616 628 572 q 457 661 545 661 q 286 597 340 661 q 232 389 232 534 l 232 0 l 119 0 l 119 1055 l 232 1055 l 232 736 q 227 640 232 678 l 233 640 q 328 725 267 694 q 468 755 389 755 q 672 691 604 755 q 741 485 741 626 l 741 0 l 628 0 m 203 1124 q 323 1259 289 1216 q 374 1331 358 1303 l 486 1331 q 538 1257 501 1302 q 660 1124 575 1212 l 660 1108 l 579 1108 q 429 1234 519 1145 q 281 1108 337 1143 l 203 1108 l 203 1124 z "
	},
	"Ħ": {
		ha: 1025,
		x_min: 0,
		x_max: 1025,
		o: "m 136 823 l 136 991 l 252 991 l 252 823 l 774 823 l 774 991 l 889 991 l 889 823 l 1025 823 l 1025 728 l 889 728 l 889 0 l 774 0 l 774 467 l 252 467 l 252 0 l 136 0 l 136 728 l 0 728 l 0 823 l 136 823 m 774 570 l 774 728 l 252 728 l 252 570 l 774 570 z "
	},
	"ħ": {
		ha: 852,
		x_min: 14,
		x_max: 741,
		o: "m 628 0 l 628 454 q 587 590 628 545 q 457 635 545 635 q 286 572 339 635 q 232 364 232 509 l 232 0 l 119 0 l 119 843 l 14 843 l 14 929 l 119 929 l 119 1055 l 232 1055 l 232 929 l 536 929 l 536 843 l 232 843 l 232 710 q 227 615 232 653 l 233 615 q 328 700 267 669 q 468 730 389 730 q 672 666 604 730 q 741 458 741 601 l 741 0 l 628 0 z "
	},
	"ĩ": {
		ha: 351,
		x_min: -76,
		x_max: 429,
		o: "m 232 0 l 119 0 l 119 743 l 232 743 l 232 0 m 279 843 q 222 856 250 843 q 168 883 195 868 q 116 911 141 899 q 68 924 92 924 q 17 903 35 924 q -9 842 0 883 l -76 842 q -28 970 -67 924 q 73 1017 11 1017 q 133 1004 104 1017 q 189 976 162 991 q 239 948 215 961 q 286 936 264 936 q 335 956 319 936 q 361 1018 351 976 l 429 1018 q 381 889 420 936 q 279 843 343 843 z "
	},
	"ī": {
		ha: 351,
		x_min: -26,
		x_max: 381,
		o: "m 232 0 l 119 0 l 119 743 l 232 743 l 232 0 m -26 935 l 381 935 l 381 842 l -26 842 l -26 935 z "
	},
	"ĭ": {
		ha: 351,
		x_min: -35,
		x_max: 385,
		o: "m 232 0 l 119 0 l 119 743 l 232 743 l 232 0 m 172 842 q 24 888 77 842 q -35 1023 -29 935 l 35 1023 q 53 971 39 990 q 92 943 67 952 q 174 935 117 935 q 275 956 243 935 q 315 1023 308 978 l 385 1023 q 322 890 378 938 q 172 842 265 842 z "
	},
	"į": {
		ha: 351,
		x_min: 36,
		x_max: 261,
		o: "m 232 0 l 119 0 l 119 743 l 232 743 l 232 0 m 110 945 q 129 1001 110 983 q 176 1019 148 1019 q 223 1001 203 1019 q 243 945 243 983 q 223 888 243 907 q 176 869 203 869 q 129 888 148 869 q 110 945 110 907 m 132 -161 q 195 -225 132 -225 q 261 -216 224 -225 l 261 -290 q 176 -302 217 -302 q 36 -175 36 -302 q 65 -82 36 -127 q 143 0 94 -36 l 224 0 q 155 -82 178 -45 q 132 -161 132 -119 z "
	},
	"ı": {
		ha: 351,
		x_min: 119,
		x_max: 232,
		o: "m 232 0 l 119 0 l 119 743 l 232 743 l 232 0 z "
	},
	"ĳ": {
		ha: 703,
		x_min: 110,
		x_max: 594,
		o: "m 232 0 l 119 0 l 119 743 l 232 743 l 232 0 m 110 945 q 129 1001 110 983 q 176 1019 148 1019 q 223 1001 203 1019 q 243 945 243 983 q 223 888 243 907 q 176 869 203 869 q 129 888 148 869 q 110 945 110 907 m 380 -334 q 276 -317 316 -334 l 276 -225 q 368 -239 323 -239 q 446 -210 421 -239 q 471 -122 471 -181 l 471 743 l 583 743 l 583 -114 q 380 -334 583 -334 m 461 945 q 480 1001 461 983 q 528 1019 499 1019 q 574 1001 555 1019 q 594 945 594 983 q 574 888 594 907 q 528 869 555 869 q 480 888 499 869 q 461 945 461 907 z "
	},
	"Ĵ": {
		ha: 371,
		x_min: -109,
		x_max: 416,
		o: "m -8 -261 q -109 -243 -72 -261 l -109 -144 q -8 -158 -60 -158 q 94 -117 59 -158 q 129 0 129 -77 l 129 991 l 244 991 l 244 9 q 179 -190 244 -119 q -8 -261 114 -261 m -41 1086 q 79 1222 45 1179 q 130 1293 114 1265 l 242 1293 q 294 1220 257 1265 q 416 1086 331 1175 l 416 1071 l 335 1071 q 185 1197 275 1108 q 37 1071 93 1106 l -41 1071 l -41 1086 z "
	},
	"ĵ": {
		ha: 351,
		x_min: -75,
		x_max: 401,
		o: "m 29 -334 q -75 -317 -35 -334 l -75 -225 q 17 -239 -28 -239 q 95 -210 70 -239 q 119 -122 119 -181 l 119 743 l 232 743 l 232 -114 q 29 -334 232 -334 m -56 857 q 64 993 30 949 q 115 1064 99 1036 l 227 1064 q 279 990 242 1036 q 401 857 316 945 l 401 842 l 320 842 q 170 968 260 879 q 22 842 78 877 l -56 842 l -56 857 z "
	},
	"Ķ": {
		ha: 852,
		x_min: 136,
		x_max: 852,
		o: "m 852 0 l 717 0 l 355 481 l 252 389 l 252 0 l 136 0 l 136 991 l 252 991 l 252 500 l 701 991 l 838 991 l 439 561 l 852 0 m 343 -290 q 379 -184 362 -247 q 401 -85 397 -121 l 515 -85 l 515 -97 q 475 -197 508 -132 q 404 -307 441 -262 l 343 -307 l 343 -290 z "
	},
	"ķ": {
		ha: 729,
		x_min: 119,
		x_max: 714,
		o: "m 231 380 q 319 489 260 422 l 559 743 l 693 743 l 392 427 l 714 0 l 578 0 l 315 351 l 231 278 l 231 0 l 119 0 l 119 1055 l 231 1055 l 231 496 q 225 380 231 458 l 231 380 m 279 -290 q 316 -184 298 -247 q 338 -85 333 -121 l 452 -85 l 452 -97 q 411 -197 444 -132 q 340 -307 378 -262 l 279 -307 l 279 -290 z "
	},
	"ĸ": {
		ha: 720,
		x_min: 119,
		x_max: 713,
		o: "m 553 742 l 693 742 l 412 415 l 713 0 l 576 0 l 332 342 l 240 265 l 240 0 l 119 0 l 119 742 l 240 742 l 240 550 q 232 361 240 437 l 553 742 z "
	},
	"Ĺ": {
		ha: 721,
		x_min: 136,
		x_max: 689,
		o: "m 136 0 l 136 991 l 252 991 l 252 104 l 689 104 l 689 0 l 136 0 m 160 1088 q 230 1190 193 1130 q 290 1293 268 1249 l 427 1293 l 427 1279 q 338 1171 397 1235 q 235 1071 279 1106 l 160 1071 l 160 1088 z "
	},
	"ĺ": {
		ha: 351,
		x_min: 111,
		x_max: 377,
		o: "m 232 0 l 119 0 l 119 1055 l 232 1055 l 232 0 m 111 1126 q 181 1228 143 1168 q 240 1332 218 1288 l 377 1332 l 377 1318 q 288 1209 347 1274 q 186 1109 229 1145 l 111 1109 l 111 1126 z "
	},
	"Ļ": {
		ha: 721,
		x_min: 136,
		x_max: 689,
		o: "m 136 0 l 136 991 l 252 991 l 252 104 l 689 104 l 689 0 l 136 0 m 283 -290 q 320 -184 302 -247 q 342 -85 337 -121 l 456 -85 l 456 -97 q 415 -197 448 -132 q 345 -307 382 -262 l 283 -307 l 283 -290 z "
	},
	"ļ": {
		ha: 351,
		x_min: 60,
		x_max: 233,
		o: "m 232 0 l 119 0 l 119 1055 l 232 1055 l 232 0 m 60 -290 q 97 -184 79 -247 q 119 -85 114 -121 l 233 -85 l 233 -97 q 192 -197 225 -132 q 121 -307 159 -262 l 60 -307 l 60 -290 z "
	},
	"Ľ": {
		ha: 721,
		x_min: 136,
		x_max: 689,
		o: "m 136 0 l 136 991 l 252 991 l 252 104 l 689 104 l 689 0 l 136 0 m 460 787 q 486 881 473 824 q 506 992 499 937 l 620 992 l 620 980 q 583 882 612 945 q 521 770 553 818 l 460 770 l 460 787 z "
	},
	"ľ": {
		ha: 351,
		x_min: 119,
		x_max: 456,
		o: "m 232 0 l 119 0 l 119 1055 l 232 1055 l 232 0 m 296 850 q 322 944 309 887 q 342 1055 335 1000 l 456 1055 l 456 1043 q 418 945 448 1008 q 357 833 389 881 l 296 833 l 296 850 z "
	},
	"Ŀ": {
		ha: 721,
		x_min: 136,
		x_max: 689,
		o: "m 136 0 l 136 991 l 252 991 l 252 104 l 689 104 l 689 0 l 136 0 m 460 494 q 479 550 460 532 q 526 568 498 568 q 573 550 553 568 q 593 494 593 532 q 573 437 593 456 q 526 418 553 418 q 479 437 498 418 q 460 494 460 456 z "
	},
	"ŀ": {
		ha: 436,
		x_min: 119,
		x_max: 461,
		o: "m 232 0 l 119 0 l 119 1055 l 232 1055 l 232 0 m 328 462 q 347 518 328 500 q 395 536 366 536 q 441 518 422 536 q 461 462 461 500 q 441 405 461 424 q 395 387 422 387 q 347 405 366 387 q 328 462 328 424 z "
	},
	"Ł": {
		ha: 726,
		x_min: 20,
		x_max: 689,
		o: "m 136 0 l 136 345 l 65 304 l 20 382 l 136 450 l 136 991 l 252 991 l 252 518 l 453 636 l 498 554 l 252 411 l 252 104 l 689 104 l 689 0 l 136 0 z "
	},
	"ł": {
		ha: 363,
		x_min: -3,
		x_max: 374,
		o: "m 232 586 l 325 650 l 374 574 l 232 478 l 232 0 l 119 0 l 119 404 l 45 355 l -3 432 l 119 513 l 119 1055 l 232 1055 l 232 586 z "
	},
	"Ń": {
		ha: 1047,
		x_min: 136,
		x_max: 911,
		o: "m 911 0 l 779 0 l 237 832 l 232 832 q 243 564 243 686 l 243 0 l 136 0 l 136 991 l 267 991 l 807 163 l 812 163 q 806 280 811 181 q 803 422 802 380 l 803 991 l 911 991 l 911 0 m 441 1088 q 512 1190 474 1130 q 571 1293 549 1249 l 708 1293 l 708 1279 q 619 1171 678 1235 q 517 1071 560 1106 l 441 1071 l 441 1088 z "
	},
	"ń": {
		ha: 852,
		x_min: 119,
		x_max: 741,
		o: "m 628 0 l 628 481 q 587 616 628 572 q 457 661 545 661 q 286 598 340 661 q 232 390 232 535 l 232 0 l 119 0 l 119 743 l 211 743 l 229 642 l 235 642 q 332 727 269 696 q 471 757 394 757 q 673 692 605 757 q 741 485 741 627 l 741 0 l 628 0 m 349 859 q 419 960 381 901 q 478 1064 456 1020 l 615 1064 l 615 1050 q 526 941 585 1006 q 424 842 467 877 l 349 842 l 349 859 z "
	},
	"Ņ": {
		ha: 1047,
		x_min: 136,
		x_max: 911,
		o: "m 911 0 l 779 0 l 237 832 l 232 832 q 243 564 243 686 l 243 0 l 136 0 l 136 991 l 267 991 l 807 163 l 812 163 q 806 280 811 181 q 803 422 802 380 l 803 991 l 911 991 l 911 0 m 389 -290 q 426 -184 408 -247 q 448 -85 443 -121 l 562 -85 l 562 -97 q 521 -197 554 -132 q 450 -307 488 -262 l 389 -307 l 389 -290 z "
	},
	"ņ": {
		ha: 852,
		x_min: 119,
		x_max: 741,
		o: "m 628 0 l 628 481 q 587 616 628 572 q 457 661 545 661 q 286 598 340 661 q 232 390 232 535 l 232 0 l 119 0 l 119 743 l 211 743 l 229 642 l 235 642 q 332 727 269 696 q 471 757 394 757 q 673 692 605 757 q 741 485 741 627 l 741 0 l 628 0 m 309 -290 q 345 -184 328 -247 q 367 -85 362 -121 l 481 -85 l 481 -97 q 440 -197 473 -132 q 370 -307 407 -262 l 309 -307 l 309 -290 z "
	},
	"Ň": {
		ha: 1047,
		x_min: 136,
		x_max: 911,
		o: "m 911 0 l 779 0 l 237 832 l 232 832 q 243 564 243 686 l 243 0 l 136 0 l 136 991 l 267 991 l 807 163 l 812 163 q 806 280 811 181 q 803 422 802 380 l 803 991 l 911 991 l 911 0 m 294 1293 l 372 1293 q 521 1165 450 1243 q 671 1293 609 1253 l 751 1293 l 751 1276 l 707 1230 q 578 1071 609 1130 l 465 1071 q 415 1141 450 1099 q 294 1276 380 1184 l 294 1293 z "
	},
	"ň": {
		ha: 852,
		x_min: 119,
		x_max: 741,
		o: "m 628 0 l 628 481 q 587 616 628 572 q 457 661 545 661 q 286 598 340 661 q 232 390 232 535 l 232 0 l 119 0 l 119 743 l 211 743 l 229 642 l 235 642 q 332 727 269 696 q 471 757 394 757 q 673 692 605 757 q 741 485 741 627 l 741 0 l 628 0 m 203 1064 l 281 1064 q 429 936 358 1014 q 579 1064 517 1024 l 660 1064 l 660 1047 l 615 1001 q 486 842 517 901 l 374 842 q 323 912 358 869 q 203 1047 289 955 l 203 1064 z "
	},
	"ŉ": {
		ha: 946,
		x_min: 1,
		x_max: 832,
		o: "m 720 0 l 720 481 q 678 616 720 572 q 549 661 637 661 q 378 598 432 661 q 323 390 323 535 l 323 0 l 211 0 l 211 743 l 302 743 l 321 642 l 326 642 q 423 727 361 696 q 562 757 486 757 q 764 692 696 757 q 832 485 832 627 l 832 0 l 720 0 m 193 991 l 203 977 q 153 819 186 909 q 83 652 119 729 l 1 652 q 70 991 48 845 l 193 991 z "
	},
	"Ŋ": {
		ha: 1047,
		x_min: 136,
		x_max: 911,
		o: "m 657 -261 q 554 -243 591 -261 l 554 -144 q 659 -158 602 -158 q 766 -117 730 -158 q 802 0 802 -75 l 237 832 l 232 832 q 243 546 243 653 l 243 0 l 136 0 l 136 991 l 267 991 l 807 176 l 812 176 q 803 429 803 277 l 803 991 l 911 991 l 911 9 q 845 -192 911 -123 q 657 -261 780 -261 z "
	},
	"ŋ": {
		ha: 852,
		x_min: 119,
		x_max: 741,
		o: "m 546 -334 q 450 -317 488 -334 l 450 -225 q 533 -239 491 -239 q 628 -122 628 -239 l 628 481 q 587 616 628 572 q 457 661 545 661 q 286 598 340 661 q 232 390 232 535 l 232 0 l 119 0 l 119 743 l 211 743 l 229 642 l 236 642 q 332 727 271 697 q 468 757 393 757 q 673 692 606 757 q 741 485 741 627 l 741 -114 q 693 -276 741 -218 q 546 -334 646 -334 z "
	},
	"Ō": {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 207 497 q 293 192 207 296 q 542 87 378 87 q 791 191 707 87 q 875 497 875 295 q 791 801 875 697 q 543 904 707 904 q 293 800 378 904 q 207 497 207 696 m 339 1164 l 746 1164 l 746 1071 l 339 1071 l 339 1164 z "
	},
	"ō": {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 195 372 q 252 155 195 230 q 419 81 309 81 q 587 155 530 81 q 644 372 644 229 q 587 588 644 514 q 418 661 530 661 q 251 589 307 661 q 195 372 195 516 m 216 935 l 623 935 l 623 842 l 216 842 l 216 935 z "
	},
	"Ŏ": {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 207 497 q 293 192 207 296 q 542 87 378 87 q 791 191 707 87 q 875 497 875 295 q 791 801 875 697 q 543 904 707 904 q 293 800 378 904 q 207 497 207 696 m 536 1071 q 389 1117 441 1071 q 330 1253 336 1164 l 400 1253 q 418 1200 404 1219 q 457 1173 432 1181 q 539 1164 481 1164 q 640 1185 608 1164 q 680 1253 673 1207 l 750 1253 q 687 1119 743 1167 q 536 1071 630 1071 z "
	},
	"ŏ": {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 195 372 q 252 155 195 230 q 419 81 309 81 q 587 155 530 81 q 644 372 644 229 q 587 588 644 514 q 418 661 530 661 q 251 589 307 661 q 195 372 195 516 m 415 842 q 267 888 320 842 q 208 1023 214 935 l 279 1023 q 297 971 283 990 q 335 943 311 952 q 418 935 360 935 q 519 956 486 935 q 558 1023 551 978 l 629 1023 q 565 890 622 938 q 415 842 509 842 z "
	},
	"Ő": {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 207 497 q 293 192 207 296 q 542 87 378 87 q 791 191 707 87 q 875 497 875 295 q 791 801 875 697 q 543 904 707 904 q 293 800 378 904 q 207 497 207 696 m 344 1088 q 406 1183 368 1120 q 464 1293 443 1246 l 590 1293 l 590 1279 q 507 1170 565 1236 q 410 1071 449 1103 l 344 1071 l 344 1088 m 586 1088 q 653 1194 619 1134 q 705 1293 688 1254 l 831 1293 l 831 1279 q 748 1170 806 1236 q 651 1071 690 1103 l 586 1071 l 586 1088 z "
	},
	"ő": {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 195 372 q 252 155 195 230 q 419 81 309 81 q 587 155 530 81 q 644 372 644 229 q 587 588 644 514 q 418 661 530 661 q 251 589 307 661 q 195 372 195 516 m 218 859 q 279 954 242 891 q 338 1064 317 1017 l 464 1064 l 464 1050 q 381 941 439 1007 q 283 842 323 874 l 218 842 l 218 859 m 460 859 q 527 965 493 905 q 579 1064 562 1025 l 705 1064 l 705 1050 q 622 941 680 1007 q 525 842 564 874 l 460 842 l 460 859 z "
	},
	"Œ": {
		ha: 1282,
		x_min: 85,
		x_max: 1198,
		o: "m 1198 0 l 677 0 q 546 -14 608 -14 q 204 120 324 -14 q 85 498 85 253 q 203 875 85 743 q 538 1007 321 1007 q 668 991 607 1007 l 1198 991 l 1198 889 l 799 889 l 799 570 l 1173 570 l 1173 468 l 799 468 l 799 103 l 1198 103 l 1198 0 m 550 904 q 294 801 381 904 q 207 497 207 697 q 294 192 207 296 q 549 87 381 87 q 684 110 625 87 l 684 884 q 550 904 625 904 z "
	},
	"œ": {
		ha: 1308,
		x_min: 77,
		x_max: 1236,
		o: "m 970 -14 q 686 146 771 -14 q 573 27 644 67 q 410 -14 503 -14 q 168 90 258 -14 q 77 372 77 193 q 165 654 77 552 q 414 755 254 755 q 572 715 503 755 q 684 597 642 675 q 788 714 723 673 q 939 755 853 755 q 1156 666 1075 755 q 1236 422 1236 576 l 1236 351 l 742 351 q 971 85 747 85 q 1089 97 1035 85 q 1203 135 1144 109 l 1203 35 q 1092 -3 1143 8 q 970 -14 1040 -14 m 195 372 q 246 155 195 229 q 411 81 298 81 q 573 153 522 81 q 625 367 625 225 q 573 589 625 517 q 408 661 520 661 q 245 588 296 661 q 195 372 195 515 m 937 661 q 802 606 851 661 q 745 443 753 550 l 1114 443 q 1069 606 1114 550 q 937 661 1024 661 z "
	},
	"Ŕ": {
		ha: 859,
		x_min: 136,
		x_max: 835,
		o: "m 252 412 l 252 0 l 136 0 l 136 991 l 408 991 q 678 922 591 991 q 765 711 765 852 q 566 446 765 515 l 835 0 l 699 0 l 458 412 l 252 412 m 252 511 l 410 511 q 589 560 532 511 q 646 705 646 608 q 588 847 646 804 q 401 890 530 890 l 252 890 l 252 511 m 349 1088 q 419 1190 381 1130 q 478 1293 456 1249 l 615 1293 l 615 1279 q 526 1171 585 1235 q 424 1071 467 1106 l 349 1071 l 349 1088 z "
	},
	"ŕ": {
		ha: 567,
		x_min: 119,
		x_max: 547,
		o: "m 458 757 q 547 749 508 757 l 532 644 q 450 654 486 654 q 296 581 360 654 q 232 399 232 508 l 232 0 l 119 0 l 119 743 l 212 743 l 225 606 l 231 606 q 330 718 272 678 q 458 757 389 757 m 242 859 q 312 960 275 901 q 372 1064 350 1020 l 509 1064 l 509 1050 q 420 941 479 1006 q 317 842 361 877 l 242 842 l 242 859 z "
	},
	"Ŗ": {
		ha: 859,
		x_min: 136,
		x_max: 835,
		o: "m 252 412 l 252 0 l 136 0 l 136 991 l 408 991 q 678 922 591 991 q 765 711 765 852 q 566 446 765 515 l 835 0 l 699 0 l 458 412 l 252 412 m 252 511 l 410 511 q 589 560 532 511 q 646 705 646 608 q 588 847 646 804 q 401 890 530 890 l 252 890 l 252 511 m 335 -290 q 371 -184 354 -247 q 393 -85 389 -121 l 507 -85 l 507 -97 q 467 -197 500 -132 q 396 -307 433 -262 l 335 -307 l 335 -290 z "
	},
	"ŗ": {
		ha: 567,
		x_min: 65,
		x_max: 547,
		o: "m 458 757 q 547 749 508 757 l 532 644 q 450 654 486 654 q 296 581 360 654 q 232 399 232 508 l 232 0 l 119 0 l 119 743 l 212 743 l 225 606 l 231 606 q 330 718 272 678 q 458 757 389 757 m 65 -290 q 101 -184 84 -247 q 123 -85 119 -121 l 237 -85 l 237 -97 q 197 -197 230 -132 q 126 -307 163 -262 l 65 -307 l 65 -290 z "
	},
	"Ř": {
		ha: 859,
		x_min: 136,
		x_max: 835,
		o: "m 252 412 l 252 0 l 136 0 l 136 991 l 408 991 q 678 922 591 991 q 765 711 765 852 q 566 446 765 515 l 835 0 l 699 0 l 458 412 l 252 412 m 252 511 l 410 511 q 589 560 532 511 q 646 705 646 608 q 588 847 646 804 q 401 890 530 890 l 252 890 l 252 511 m 200 1293 l 278 1293 q 427 1165 355 1243 q 576 1293 515 1253 l 657 1293 l 657 1276 l 612 1230 q 484 1071 515 1130 l 371 1071 q 321 1141 355 1099 q 200 1276 286 1184 l 200 1293 z "
	},
	"ř": {
		ha: 567,
		x_min: 88,
		x_max: 547,
		o: "m 458 757 q 547 749 508 757 l 532 644 q 450 654 486 654 q 296 581 360 654 q 232 399 232 508 l 232 0 l 119 0 l 119 743 l 212 743 l 225 606 l 231 606 q 330 718 272 678 q 458 757 389 757 m 88 1064 l 166 1064 q 315 936 243 1014 q 465 1064 403 1024 l 545 1064 l 545 1047 l 500 1001 q 372 842 403 901 l 259 842 q 209 912 243 869 q 88 1047 174 955 l 88 1064 z "
	},
	"Ś": {
		ha: 762,
		x_min: 72,
		x_max: 696,
		o: "m 696 264 q 601 60 696 133 q 343 -14 506 -14 q 72 32 167 -14 l 72 143 q 205 102 133 117 q 347 87 277 87 q 521 131 463 87 q 579 253 579 175 q 558 337 579 304 q 489 398 538 370 q 342 461 441 426 q 144 578 203 511 q 85 755 85 646 q 171 938 85 870 q 399 1006 257 1006 q 671 951 547 1006 l 635 851 q 396 903 512 903 q 253 863 304 903 q 201 754 201 824 q 220 670 201 703 q 284 609 239 637 q 422 549 330 582 q 637 429 578 493 q 696 264 696 366 m 321 1088 q 391 1190 353 1130 q 450 1293 429 1249 l 587 1293 l 587 1279 q 498 1171 557 1235 q 396 1071 439 1106 l 321 1071 l 321 1088 z "
	},
	"ś": {
		ha: 663,
		x_min: 72,
		x_max: 599,
		o: "m 599 203 q 522 43 599 99 q 304 -14 444 -14 q 74 33 157 -14 l 74 138 q 189 95 127 111 q 307 79 250 79 q 443 107 395 79 q 490 193 490 136 q 453 268 490 237 q 306 340 415 298 q 158 408 202 379 q 93 473 115 437 q 72 561 72 510 q 146 704 72 652 q 349 757 220 757 q 583 708 469 757 l 543 616 q 340 663 431 663 q 220 637 260 663 q 179 568 179 612 q 194 517 179 538 q 243 477 210 496 q 374 422 277 458 q 552 326 506 374 q 599 203 599 277 m 252 859 q 322 960 284 901 q 381 1064 359 1020 l 518 1064 l 518 1050 q 429 941 488 1006 q 327 842 370 877 l 252 842 l 252 859 z "
	},
	"Ŝ": {
		ha: 762,
		x_min: 72,
		x_max: 696,
		o: "m 696 264 q 601 60 696 133 q 343 -14 506 -14 q 72 32 167 -14 l 72 143 q 205 102 133 117 q 347 87 277 87 q 521 131 463 87 q 579 253 579 175 q 558 337 579 304 q 489 398 538 370 q 342 461 441 426 q 144 578 203 511 q 85 755 85 646 q 171 938 85 870 q 399 1006 257 1006 q 671 951 547 1006 l 635 851 q 396 903 512 903 q 253 863 304 903 q 201 754 201 824 q 220 670 201 703 q 284 609 239 637 q 422 549 330 582 q 637 429 578 493 q 696 264 696 366 m 167 1086 q 288 1222 253 1179 q 338 1293 322 1265 l 450 1293 q 502 1220 465 1265 q 624 1086 539 1175 l 624 1071 l 543 1071 q 393 1197 484 1108 q 245 1071 301 1106 l 167 1071 l 167 1086 z "
	},
	"ŝ": {
		ha: 663,
		x_min: 72,
		x_max: 599,
		o: "m 599 203 q 522 43 599 99 q 304 -14 444 -14 q 74 33 157 -14 l 74 138 q 189 95 127 111 q 307 79 250 79 q 443 107 395 79 q 490 193 490 136 q 453 268 490 237 q 306 340 415 298 q 158 408 202 379 q 93 473 115 437 q 72 561 72 510 q 146 704 72 652 q 349 757 220 757 q 583 708 469 757 l 543 616 q 340 663 431 663 q 220 637 260 663 q 179 568 179 612 q 194 517 179 538 q 243 477 210 496 q 374 422 277 458 q 552 326 506 374 q 599 203 599 277 m 111 857 q 231 993 197 949 q 281 1064 266 1036 l 394 1064 q 446 990 409 1036 q 568 857 483 945 l 568 842 l 487 842 q 337 968 427 879 q 189 842 245 877 l 111 842 l 111 857 z "
	},
	"Ş": {
		ha: 762,
		x_min: 72,
		x_max: 696,
		o: "m 696 264 q 601 60 696 133 q 343 -14 506 -14 q 72 32 167 -14 l 72 143 q 205 102 133 117 q 347 87 277 87 q 521 131 463 87 q 579 253 579 175 q 558 337 579 304 q 489 398 538 370 q 342 461 441 426 q 144 578 203 511 q 85 755 85 646 q 171 938 85 870 q 399 1006 257 1006 q 671 951 547 1006 l 635 851 q 396 903 512 903 q 253 863 304 903 q 201 754 201 824 q 220 670 201 703 q 284 609 239 637 q 422 549 330 582 q 637 429 578 493 q 696 264 696 366 m 496 -196 q 444 -298 496 -262 q 290 -334 392 -334 q 225 -328 256 -334 l 225 -256 q 296 -261 256 -261 q 377 -248 349 -261 q 404 -197 404 -234 q 377 -150 404 -168 q 277 -121 351 -132 l 336 0 l 411 0 l 374 -78 q 496 -196 496 -104 z "
	},
	"ş": {
		ha: 663,
		x_min: 72,
		x_max: 599,
		o: "m 599 203 q 522 43 599 99 q 304 -14 444 -14 q 74 33 157 -14 l 74 138 q 189 95 127 111 q 307 79 250 79 q 443 107 395 79 q 490 193 490 136 q 453 268 490 237 q 306 340 415 298 q 158 408 202 379 q 93 473 115 437 q 72 561 72 510 q 146 704 72 652 q 349 757 220 757 q 583 708 469 757 l 543 616 q 340 663 431 663 q 220 637 260 663 q 179 568 179 612 q 194 517 179 538 q 243 477 210 496 q 374 422 277 458 q 552 326 506 374 q 599 203 599 277 m 440 -196 q 388 -298 440 -262 q 235 -334 336 -334 q 170 -328 200 -334 l 170 -256 q 240 -261 200 -261 q 321 -248 294 -261 q 349 -197 349 -234 q 322 -150 349 -168 q 221 -121 295 -132 l 281 0 l 355 0 l 318 -78 q 440 -196 440 -104 z "
	},
	"Š": {
		ha: 762,
		x_min: 72,
		x_max: 696,
		o: "m 696 264 q 601 60 696 133 q 343 -14 506 -14 q 72 32 167 -14 l 72 143 q 205 102 133 117 q 347 87 277 87 q 521 131 463 87 q 579 253 579 175 q 558 337 579 304 q 489 398 538 370 q 342 461 441 426 q 144 578 203 511 q 85 755 85 646 q 171 938 85 870 q 399 1006 257 1006 q 671 951 547 1006 l 635 851 q 396 903 512 903 q 253 863 304 903 q 201 754 201 824 q 220 670 201 703 q 284 609 239 637 q 422 549 330 582 q 637 429 578 493 q 696 264 696 366 m 163 1293 l 241 1293 q 389 1165 318 1243 q 539 1293 477 1253 l 620 1293 l 620 1276 l 575 1230 q 446 1071 477 1130 l 334 1071 q 283 1141 318 1099 q 163 1276 249 1184 l 163 1293 z "
	},
	"š": {
		ha: 663,
		x_min: 72,
		x_max: 599,
		o: "m 599 203 q 522 43 599 99 q 304 -14 444 -14 q 74 33 157 -14 l 74 138 q 189 95 127 111 q 307 79 250 79 q 443 107 395 79 q 490 193 490 136 q 453 268 490 237 q 306 340 415 298 q 158 408 202 379 q 93 473 115 437 q 72 561 72 510 q 146 704 72 652 q 349 757 220 757 q 583 708 469 757 l 543 616 q 340 663 431 663 q 220 637 260 663 q 179 568 179 612 q 194 517 179 538 q 243 477 210 496 q 374 422 277 458 q 552 326 506 374 q 599 203 599 277 m 112 1064 l 190 1064 q 338 936 267 1014 q 488 1064 427 1024 l 569 1064 l 569 1047 l 524 1001 q 395 842 427 901 l 283 842 q 233 912 267 869 q 112 1047 198 955 l 112 1064 z "
	},
	"Ț": {
		ha: 768,
		x_min: 12,
		x_max: 755,
		o: "m 441 0 l 326 0 l 326 889 l 12 889 l 12 991 l 755 991 l 755 889 l 441 889 l 441 0 m 267 -290 q 303 -184 286 -247 q 326 -85 321 -121 l 439 -85 l 439 -97 q 399 -197 432 -132 q 328 -307 366 -262 l 267 -307 l 267 -290 z "
	},
	"ț": {
		ha: 490,
		x_min: 21,
		x_max: 461,
		o: "m 359 79 q 417 84 389 79 q 461 93 445 88 l 461 7 q 407 -8 443 -2 q 343 -14 372 -14 q 127 214 127 -14 l 127 656 l 21 656 l 21 710 l 127 757 l 175 916 l 240 916 l 240 743 l 456 743 l 456 656 l 240 656 l 240 218 q 272 115 240 151 q 359 79 304 79 m 165 -290 q 201 -184 184 -247 q 223 -85 218 -121 l 337 -85 l 337 -97 q 296 -197 330 -132 q 226 -307 263 -262 l 165 -307 l 165 -290 z "
	},
	"Ť": {
		ha: 768,
		x_min: 12,
		x_max: 755,
		o: "m 441 0 l 326 0 l 326 889 l 12 889 l 12 991 l 755 991 l 755 889 l 441 889 l 441 0 m 157 1293 l 235 1293 q 384 1165 313 1243 q 534 1293 472 1253 l 614 1293 l 614 1276 l 570 1230 q 441 1071 472 1130 l 328 1071 q 278 1141 313 1099 q 157 1276 243 1184 l 157 1293 z "
	},
	"ť": {
		ha: 490,
		x_min: 21,
		x_max: 493,
		o: "m 359 79 q 417 84 389 79 q 461 93 445 88 l 461 7 q 407 -8 443 -2 q 343 -14 372 -14 q 127 214 127 -14 l 127 656 l 21 656 l 21 710 l 127 757 l 175 916 l 240 916 l 240 743 l 456 743 l 456 656 l 240 656 l 240 218 q 272 115 240 151 q 359 79 304 79 m 333 850 q 359 944 346 887 q 379 1055 372 1000 l 493 1055 l 493 1043 q 456 945 486 1008 q 394 833 426 881 l 333 833 l 333 850 z "
	},
	"Ŧ": {
		ha: 768,
		x_min: 12,
		x_max: 755,
		o: "m 326 553 l 326 889 l 12 889 l 12 991 l 755 991 l 755 889 l 441 889 l 441 553 l 652 553 l 652 457 l 441 457 l 441 0 l 326 0 l 326 457 l 114 457 l 114 553 l 326 553 z "
	},
	"ŧ": {
		ha: 490,
		x_min: 21,
		x_max: 461,
		o: "m 363 79 q 461 93 420 79 l 461 7 q 404 -8 439 -3 q 339 -14 368 -14 q 127 222 127 -14 l 127 395 l 32 395 l 32 482 l 127 482 l 127 656 l 21 656 l 21 710 l 127 757 l 175 916 l 240 916 l 240 743 l 456 743 l 456 656 l 240 656 l 240 482 l 444 482 l 444 395 l 240 395 l 240 229 q 363 79 240 79 z "
	},
	"Ũ": {
		ha: 1011,
		x_min: 126,
		x_max: 885,
		o: "m 885 991 l 885 350 q 783 83 885 180 q 501 -14 680 -14 q 224 84 322 -14 q 126 353 126 182 l 126 991 l 241 991 l 241 345 q 309 154 241 220 q 509 87 377 87 q 702 154 634 87 q 770 346 770 221 l 770 991 l 885 991 m 610 1072 q 553 1085 581 1072 q 498 1113 525 1097 q 447 1140 471 1128 q 399 1153 422 1153 q 348 1133 365 1153 q 321 1071 330 1112 l 254 1071 q 302 1199 263 1153 q 403 1246 341 1246 q 463 1233 434 1246 q 519 1205 492 1221 q 570 1178 545 1190 q 616 1165 594 1165 q 665 1185 649 1165 q 692 1247 682 1205 l 759 1247 q 712 1119 750 1165 q 610 1072 673 1072 z "
	},
	"ũ": {
		ha: 852,
		x_min: 111,
		x_max: 733,
		o: "m 225 743 l 225 261 q 267 125 225 170 q 396 81 308 81 q 567 144 513 81 q 621 353 621 208 l 621 743 l 733 743 l 733 0 l 640 0 l 624 100 l 618 100 q 522 16 583 45 q 382 -14 460 -14 q 179 51 246 -14 q 111 257 111 115 l 111 743 l 225 743 m 528 843 q 471 856 499 843 q 417 883 444 868 q 365 911 390 899 q 317 924 340 924 q 266 903 283 924 q 239 842 249 883 l 173 842 q 221 970 182 924 q 321 1017 260 1017 q 382 1004 353 1017 q 437 976 411 991 q 488 948 464 961 q 534 936 513 936 q 584 956 568 936 q 610 1018 600 976 l 677 1018 q 630 889 669 936 q 528 843 592 843 z "
	},
	"Ū": {
		ha: 1011,
		x_min: 126,
		x_max: 885,
		o: "m 885 991 l 885 350 q 783 83 885 180 q 501 -14 680 -14 q 224 84 322 -14 q 126 353 126 182 l 126 991 l 241 991 l 241 345 q 309 154 241 220 q 509 87 377 87 q 702 154 634 87 q 770 346 770 221 l 770 991 l 885 991 m 302 1164 l 709 1164 l 709 1071 l 302 1071 l 302 1164 z "
	},
	"ū": {
		ha: 852,
		x_min: 111,
		x_max: 733,
		o: "m 225 743 l 225 261 q 267 125 225 170 q 396 81 308 81 q 567 144 513 81 q 621 353 621 208 l 621 743 l 733 743 l 733 0 l 640 0 l 624 100 l 618 100 q 522 16 583 45 q 382 -14 460 -14 q 179 51 246 -14 q 111 257 111 115 l 111 743 l 225 743 m 221 935 l 628 935 l 628 842 l 221 842 l 221 935 z "
	},
	"Ŭ": {
		ha: 1011,
		x_min: 126,
		x_max: 885,
		o: "m 885 991 l 885 350 q 783 83 885 180 q 501 -14 680 -14 q 224 84 322 -14 q 126 353 126 182 l 126 991 l 241 991 l 241 345 q 309 154 241 220 q 509 87 377 87 q 702 154 634 87 q 770 346 770 221 l 770 991 l 885 991 m 500 1071 q 352 1117 405 1071 q 293 1253 299 1164 l 363 1253 q 381 1200 368 1219 q 420 1173 395 1181 q 503 1164 445 1164 q 604 1185 571 1164 q 643 1253 636 1207 l 713 1253 q 650 1119 707 1167 q 500 1071 593 1071 z "
	},
	"ŭ": {
		ha: 852,
		x_min: 111,
		x_max: 733,
		o: "m 225 743 l 225 261 q 267 125 225 170 q 396 81 308 81 q 567 144 513 81 q 621 353 621 208 l 621 743 l 733 743 l 733 0 l 640 0 l 624 100 l 618 100 q 522 16 583 45 q 382 -14 460 -14 q 179 51 246 -14 q 111 257 111 115 l 111 743 l 225 743 m 418 842 q 270 888 323 842 q 211 1023 217 935 l 281 1023 q 299 971 286 990 q 338 943 313 952 q 420 935 363 935 q 522 956 489 935 q 561 1023 554 978 l 631 1023 q 568 890 625 938 q 418 842 511 842 z "
	},
	"Ů": {
		ha: 1011,
		x_min: 126,
		x_max: 885,
		o: "m 885 991 l 885 350 q 783 83 885 180 q 501 -14 680 -14 q 224 84 322 -14 q 126 353 126 182 l 126 991 l 241 991 l 241 345 q 309 154 241 220 q 509 87 377 87 q 702 154 634 87 q 770 346 770 221 l 770 991 l 885 991 m 657 1218 q 615 1111 657 1152 q 505 1071 574 1071 q 395 1111 436 1071 q 355 1217 355 1150 q 396 1322 355 1283 q 505 1361 437 1361 q 615 1321 573 1361 q 657 1218 657 1280 m 584 1217 q 562 1275 584 1255 q 505 1296 539 1296 q 448 1275 470 1296 q 425 1217 425 1255 q 446 1158 425 1179 q 505 1137 466 1137 q 562 1158 540 1137 q 584 1217 584 1179 z "
	},
	"ů": {
		ha: 852,
		x_min: 111,
		x_max: 733,
		o: "m 225 743 l 225 261 q 267 125 225 170 q 396 81 308 81 q 567 144 513 81 q 621 353 621 208 l 621 743 l 733 743 l 733 0 l 640 0 l 624 100 l 618 100 q 522 16 583 45 q 382 -14 460 -14 q 179 51 246 -14 q 111 257 111 115 l 111 743 l 225 743 m 575 989 q 533 882 575 922 q 422 842 492 842 q 313 881 354 842 q 273 987 273 921 q 314 1093 273 1054 q 422 1132 355 1132 q 533 1092 491 1132 q 575 989 575 1051 m 502 987 q 479 1046 502 1025 q 422 1067 457 1067 q 366 1046 388 1067 q 343 987 343 1025 q 363 929 343 949 q 422 908 384 908 q 480 929 458 908 q 502 987 502 949 z "
	},
	"Ű": {
		ha: 1011,
		x_min: 126,
		x_max: 885,
		o: "m 885 991 l 885 350 q 783 83 885 180 q 501 -14 680 -14 q 224 84 322 -14 q 126 353 126 182 l 126 991 l 241 991 l 241 345 q 309 154 241 220 q 509 87 377 87 q 702 154 634 87 q 770 346 770 221 l 770 991 l 885 991 m 309 1088 q 371 1183 334 1120 q 429 1293 408 1246 l 555 1293 l 555 1279 q 472 1170 530 1236 q 375 1071 414 1103 l 309 1071 l 309 1088 m 551 1088 q 619 1194 585 1134 q 671 1293 653 1254 l 797 1293 l 797 1279 q 714 1170 772 1236 q 616 1071 656 1103 l 551 1071 l 551 1088 z "
	},
	"ű": {
		ha: 852,
		x_min: 111,
		x_max: 733,
		o: "m 225 743 l 225 261 q 267 125 225 170 q 396 81 308 81 q 567 144 513 81 q 621 353 621 208 l 621 743 l 733 743 l 733 0 l 640 0 l 624 100 l 618 100 q 522 16 583 45 q 382 -14 460 -14 q 179 51 246 -14 q 111 257 111 115 l 111 743 l 225 743 m 227 859 q 289 954 252 891 q 347 1064 326 1017 l 473 1064 l 473 1050 q 390 941 448 1007 q 293 842 332 874 l 227 842 l 227 859 m 469 859 q 537 965 503 905 q 589 1064 571 1025 l 715 1064 l 715 1050 q 632 941 690 1007 q 534 842 574 874 l 469 842 l 469 859 z "
	},
	"Ų": {
		ha: 1011,
		x_min: 126,
		x_max: 885,
		o: "m 885 991 l 885 350 q 783 83 885 180 q 501 -14 680 -14 q 224 84 322 -14 q 126 353 126 182 l 126 991 l 241 991 l 241 345 q 309 154 241 220 q 509 87 377 87 q 702 154 634 87 q 770 346 770 221 l 770 991 l 885 991 m 490 -161 q 554 -225 490 -225 q 620 -216 583 -225 l 620 -290 q 535 -302 576 -302 q 395 -175 395 -302 q 424 -82 395 -127 q 502 0 453 -36 l 583 0 q 514 -82 537 -45 q 490 -161 490 -119 z "
	},
	"ų": {
		ha: 852,
		x_min: 111,
		x_max: 763,
		o: "m 225 743 l 225 261 q 267 125 225 170 q 396 81 308 81 q 567 144 513 81 q 621 353 621 208 l 621 743 l 733 743 l 733 0 l 640 0 l 624 100 l 618 100 q 522 16 583 45 q 382 -14 460 -14 q 179 51 246 -14 q 111 257 111 115 l 111 743 l 225 743 m 633 -161 q 697 -225 633 -225 q 763 -216 726 -225 l 763 -290 q 678 -302 719 -302 q 538 -175 538 -302 q 567 -82 538 -127 q 645 0 596 -36 l 726 0 q 657 -82 680 -45 q 633 -161 633 -119 z "
	},
	"Ŵ": {
		ha: 1286,
		x_min: 18,
		x_max: 1267,
		o: "m 1002 0 l 888 0 l 688 664 q 656 775 673 708 q 637 856 638 842 q 590 660 623 766 l 396 0 l 282 0 l 18 991 l 140 991 l 297 379 q 345 146 330 250 q 399 389 363 270 l 576 991 l 699 991 l 885 383 q 940 146 918 278 q 989 380 953 242 l 1145 991 l 1267 991 l 1002 0 m 412 1086 q 533 1222 498 1179 q 583 1293 568 1265 l 696 1293 q 748 1220 711 1265 q 869 1086 785 1175 l 869 1071 l 789 1071 q 639 1197 729 1108 q 490 1071 547 1106 l 412 1071 l 412 1086 z "
	},
	"ŵ": {
		ha: 1080,
		x_min: 16,
		x_max: 1065,
		o: "m 726 0 l 590 436 q 542 618 577 476 l 536 618 q 489 435 509 499 l 349 0 l 218 0 l 16 743 l 134 743 q 243 317 205 463 q 286 119 281 170 l 292 119 q 316 219 299 158 q 345 317 332 281 l 481 743 l 603 743 l 736 317 q 787 121 774 200 l 793 121 q 807 196 795 145 q 949 743 819 247 l 1065 743 l 860 0 l 726 0 m 313 857 q 433 993 399 949 q 484 1064 468 1036 l 596 1064 q 648 990 611 1036 q 770 857 685 945 l 770 842 l 689 842 q 539 968 629 879 q 391 842 447 877 l 313 842 l 313 857 z "
	},
	"Ŷ": {
		ha: 778,
		x_min: 0,
		x_max: 778,
		o: "m 389 496 l 653 991 l 778 991 l 447 385 l 447 0 l 330 0 l 330 379 l 0 991 l 126 991 l 389 496 m 160 1086 q 281 1222 246 1179 q 331 1293 315 1265 l 444 1293 q 495 1220 458 1265 q 617 1086 532 1175 l 617 1071 l 536 1071 q 387 1197 477 1108 q 238 1071 294 1106 l 160 1071 l 160 1086 z "
	},
	"ŷ": {
		ha: 700,
		x_min: 1,
		x_max: 699,
		o: "m 1 743 l 122 743 l 285 319 q 351 110 338 174 l 357 110 q 394 228 366 144 q 578 743 422 312 l 699 743 l 379 -103 q 268 -281 332 -229 q 113 -334 205 -334 q 11 -322 61 -334 l 11 -232 q 94 -240 48 -240 q 260 -110 210 -240 l 301 -4 l 1 743 m 125 857 q 246 993 212 949 q 296 1064 281 1036 l 409 1064 q 461 990 424 1036 q 583 857 498 945 l 583 842 l 502 842 q 352 968 442 879 q 203 842 260 877 l 125 842 l 125 857 z "
	},
	"Ÿ": {
		ha: 778,
		x_min: 0,
		x_max: 778,
		o: "m 389 496 l 653 991 l 778 991 l 447 385 l 447 0 l 330 0 l 330 379 l 0 991 l 126 991 l 389 496 m 199 1174 q 217 1225 199 1209 q 260 1240 235 1240 q 305 1225 286 1240 q 323 1174 323 1209 q 305 1123 323 1140 q 260 1107 286 1107 q 217 1123 235 1107 q 199 1174 199 1140 m 458 1174 q 476 1225 458 1209 q 519 1240 494 1240 q 563 1225 544 1240 q 581 1174 581 1209 q 563 1123 581 1140 q 519 1107 544 1107 q 476 1123 494 1107 q 458 1174 458 1140 z "
	},
	"Ź": {
		ha: 793,
		x_min: 56,
		x_max: 737,
		o: "m 737 0 l 56 0 l 56 90 l 582 888 l 72 888 l 72 991 l 722 991 l 722 901 l 196 104 l 737 104 l 737 0 m 311 1088 q 381 1190 344 1130 q 441 1293 419 1249 l 578 1293 l 578 1279 q 489 1171 548 1235 q 387 1071 430 1106 l 311 1071 l 311 1088 z "
	},
	"ź": {
		ha: 650,
		x_min: 56,
		x_max: 595,
		o: "m 595 0 l 56 0 l 56 77 l 461 656 l 81 656 l 81 743 l 585 743 l 585 656 l 184 87 l 595 87 l 595 0 m 250 859 q 320 960 283 901 q 380 1064 358 1020 l 517 1064 l 517 1050 q 428 941 487 1006 q 326 842 369 877 l 250 842 l 250 859 z "
	},
	"Ż": {
		ha: 793,
		x_min: 56,
		x_max: 737,
		o: "m 737 0 l 56 0 l 56 90 l 582 888 l 72 888 l 72 991 l 722 991 l 722 901 l 196 104 l 737 104 l 737 0 m 330 1174 q 349 1231 330 1213 q 396 1249 368 1249 q 443 1230 423 1249 q 463 1174 463 1212 q 443 1117 463 1136 q 396 1099 423 1099 q 349 1117 368 1099 q 330 1174 330 1136 z "
	},
	"ż": {
		ha: 650,
		x_min: 56,
		x_max: 595,
		o: "m 595 0 l 56 0 l 56 77 l 461 656 l 81 656 l 81 743 l 585 743 l 585 656 l 184 87 l 595 87 l 595 0 m 261 945 q 280 1001 261 983 q 328 1019 299 1019 q 374 1001 355 1019 q 394 945 394 983 q 374 888 394 907 q 328 869 355 869 q 280 888 299 869 q 261 945 261 907 z "
	},
	"Ž": {
		ha: 793,
		x_min: 56,
		x_max: 737,
		o: "m 737 0 l 56 0 l 56 90 l 582 888 l 72 888 l 72 991 l 722 991 l 722 901 l 196 104 l 737 104 l 737 0 m 169 1293 l 247 1293 q 395 1165 324 1243 q 545 1293 484 1253 l 626 1293 l 626 1276 l 581 1230 q 452 1071 484 1130 l 340 1071 q 290 1141 324 1099 q 169 1276 255 1184 l 169 1293 z "
	},
	"ž": {
		ha: 650,
		x_min: 56,
		x_max: 595,
		o: "m 595 0 l 56 0 l 56 77 l 461 656 l 81 656 l 81 743 l 585 743 l 585 656 l 184 87 l 595 87 l 595 0 m 99 1064 l 177 1064 q 326 936 254 1014 q 475 1064 414 1024 l 556 1064 l 556 1047 l 511 1001 q 382 842 414 901 l 270 842 q 220 912 254 869 q 99 1047 185 955 l 99 1064 z "
	},
	"ſ": {
		ha: 444,
		x_min: 119,
		x_max: 496,
		o: "m 232 0 l 119 0 l 119 800 q 363 1063 119 1063 q 496 1038 428 1063 l 467 947 q 358 968 408 968 q 262 926 292 968 q 232 802 232 884 l 232 0 z "
	},
	"ƒ": {
		ha: 802,
		x_min: 132,
		x_max: 710,
		o: "m 222 -334 q 132 -321 176 -334 l 132 -227 q 221 -239 180 -239 q 312 -204 286 -239 q 338 -93 338 -170 l 338 567 l 187 567 l 187 618 l 338 659 l 338 753 q 393 945 338 885 q 571 1006 448 1006 q 710 977 629 1006 l 695 933 l 681 889 q 571 911 612 911 q 477 875 505 911 q 450 755 450 840 l 450 654 l 642 654 l 642 567 l 453 567 l 453 -90 q 399 -274 453 -215 q 222 -334 346 -334 z "
	},
	"Ǻ": {
		ha: 882,
		x_min: 0,
		x_max: 882,
		o: "m 591 970 q 521 842 591 880 l 882 0 l 764 0 l 644 272 l 231 272 l 118 0 l 0 0 l 361 840 q 289 968 289 876 q 330 1075 289 1036 q 439 1114 372 1114 q 549 1074 507 1114 q 591 970 591 1034 m 610 376 l 489 669 q 440 807 472 709 q 414 729 431 774 q 392 667 398 683 l 272 376 l 610 376 m 375 1161 q 443 1252 408 1201 q 493 1331 477 1302 l 631 1331 l 631 1320 q 552 1234 609 1289 q 450 1149 495 1179 l 375 1149 l 375 1161 m 518 968 q 496 1027 518 1006 q 439 1047 473 1047 q 382 1027 404 1047 q 359 968 359 1006 q 380 910 359 930 q 439 889 400 889 q 496 910 475 889 q 518 968 518 930 z "
	},
	"ǻ": {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 319 1189 l 319 1200 q 387 1269 351 1228 q 437 1331 422 1310 l 614 1331 l 614 1322 q 544 1271 600 1306 q 401 1189 488 1235 l 319 1189 m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 428 441 64 431 l 550 446 l 550 492 q 510 621 550 580 q 379 663 469 663 q 257 645 314 663 q 153 606 201 627 l 115 692 q 243 738 172 721 q 382 755 313 755 q 592 696 524 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 545 989 q 503 882 545 922 q 392 842 461 842 q 282 881 323 842 q 241 987 241 920 q 282 1093 241 1054 q 392 1132 323 1132 q 503 1092 460 1132 q 545 989 545 1051 m 471 987 q 449 1046 471 1025 q 392 1067 427 1067 q 335 1046 357 1067 q 313 987 313 1025 q 333 929 313 949 q 392 908 353 908 q 449 929 427 908 q 471 987 471 949 z "
	},
	"Ǽ": {
		ha: 1213,
		x_min: -1,
		x_max: 1129,
		o: "m 1129 0 l 621 0 l 621 315 l 272 315 l 118 0 l -1 0 l 472 991 l 1129 991 l 1129 889 l 736 889 l 736 570 l 1103 570 l 1103 468 l 736 468 l 736 103 l 1129 103 l 1129 0 m 318 419 l 621 419 l 621 889 l 541 889 l 318 419 m 665 1088 q 735 1190 698 1130 q 795 1293 773 1249 l 932 1293 l 932 1279 q 843 1171 902 1235 q 741 1071 784 1106 l 665 1071 l 665 1088 z "
	},
	"ǽ": {
		ha: 1192,
		x_min: 64,
		x_max: 1120,
		o: "m 64 205 q 148 375 64 315 q 404 441 232 436 l 529 446 l 529 492 q 490 621 529 579 q 370 663 450 663 q 161 606 272 663 l 126 692 q 244 738 176 720 q 378 755 311 755 q 522 726 466 755 q 606 632 578 696 q 700 724 642 692 q 832 757 758 757 q 1041 666 962 757 q 1120 425 1120 576 l 1120 353 l 644 353 q 863 85 650 85 q 978 97 924 85 q 1088 135 1031 109 l 1088 35 q 979 -2 1029 9 q 860 -14 928 -14 q 579 144 664 -14 q 457 22 524 58 q 300 -14 391 -14 q 126 44 189 -14 q 64 205 64 102 m 182 204 q 218 110 182 140 q 314 79 254 79 q 469 137 412 79 q 526 298 526 194 l 526 366 l 419 361 q 237 318 293 355 q 182 204 182 281 m 831 663 q 702 606 749 663 q 647 443 654 550 l 999 443 q 956 606 999 549 q 831 663 912 663 m 530 859 q 601 960 563 901 q 660 1064 638 1020 l 797 1064 l 797 1050 q 708 941 767 1006 q 606 842 649 877 l 530 842 l 530 859 z "
	},
	"Ǿ": {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 282 54 382 -14 l 214 -41 l 132 12 l 205 117 q 85 498 85 251 q 204 874 85 741 q 543 1007 323 1007 q 791 943 685 1007 l 857 1035 l 939 981 l 867 880 q 997 497 997 743 m 875 497 q 800 786 875 682 l 345 143 q 542 87 422 87 q 791 191 707 87 q 875 497 875 295 m 207 497 q 275 215 207 319 l 729 854 q 543 904 657 904 q 293 800 378 904 q 207 497 207 696 m 457 1088 q 527 1190 490 1130 q 587 1293 565 1249 l 724 1293 l 724 1279 q 635 1171 694 1235 q 532 1071 576 1106 l 457 1071 l 457 1088 z "
	},
	"ǿ": {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 236 33 312 -14 l 179 -46 l 102 7 l 165 96 q 78 372 78 199 q 169 655 78 554 q 421 757 260 757 q 604 705 526 757 l 661 786 l 741 734 l 675 644 q 761 372 761 541 m 195 372 q 231 187 195 256 l 546 625 q 418 661 495 661 q 251 589 307 661 q 195 372 195 516 m 644 372 q 610 551 644 484 l 294 115 q 419 81 342 81 q 587 155 530 81 q 644 372 644 229 m 325 859 q 395 960 357 901 q 454 1064 433 1020 l 591 1064 l 591 1050 q 503 941 562 1006 q 400 842 444 877 l 325 842 l 325 859 z "
	},
	"Ș": {
		ha: 762,
		x_min: 72,
		x_max: 696,
		o: "m 696 264 q 601 60 696 133 q 343 -14 506 -14 q 72 32 167 -14 l 72 143 q 205 102 133 117 q 347 87 277 87 q 521 131 463 87 q 579 253 579 175 q 558 337 579 304 q 489 398 538 370 q 342 461 441 426 q 144 578 203 511 q 85 755 85 646 q 171 938 85 870 q 399 1006 257 1006 q 671 951 547 1006 l 635 851 q 396 903 512 903 q 253 863 304 903 q 201 754 201 824 q 220 670 201 703 q 284 609 239 637 q 422 549 330 582 q 637 429 578 493 q 696 264 696 366 m 254 -290 q 291 -184 273 -247 q 313 -85 308 -121 l 427 -85 l 427 -97 q 386 -197 419 -132 q 315 -307 353 -262 l 254 -307 l 254 -290 z "
	},
	"ș": {
		ha: 663,
		x_min: 72,
		x_max: 599,
		o: "m 599 203 q 522 43 599 99 q 304 -14 444 -14 q 74 33 157 -14 l 74 138 q 189 95 127 111 q 307 79 250 79 q 443 107 395 79 q 490 193 490 136 q 453 268 490 237 q 306 340 415 298 q 158 408 202 379 q 93 473 115 437 q 72 561 72 510 q 146 704 72 652 q 349 757 220 757 q 583 708 469 757 l 543 616 q 340 663 431 663 q 220 637 260 663 q 179 568 179 612 q 194 517 179 538 q 243 477 210 496 q 374 422 277 458 q 552 326 506 374 q 599 203 599 277 m 202 -290 q 238 -184 221 -247 q 260 -85 256 -121 l 374 -85 l 374 -97 q 334 -197 367 -132 q 263 -307 300 -262 l 202 -307 l 202 -290 z "
	},
	"ˆ": {
		ha: 822,
		x_min: 182,
		x_max: 639,
		o: "m 182 857 q 302 993 268 949 q 353 1064 337 1036 l 465 1064 q 517 990 480 1036 q 639 857 554 945 l 639 842 l 558 842 q 408 968 498 879 q 260 842 316 877 l 182 842 l 182 857 z "
	},
	"ˇ": {
		ha: 822,
		x_min: 182,
		x_max: 639,
		o: "m 182 1064 l 260 1064 q 408 936 337 1014 q 558 1064 496 1024 l 639 1064 l 639 1047 l 594 1001 q 465 842 496 901 l 353 842 q 302 912 337 869 q 182 1047 268 955 l 182 1064 z "
	},
	"ˉ": {
		ha: 815,
		x_min: 204,
		x_max: 611,
		o: "m 204 935 l 611 935 l 611 842 l 204 842 l 204 935 z "
	},
	"˘": {
		ha: 822,
		x_min: 199,
		x_max: 619,
		o: "m 406 842 q 258 888 311 842 q 199 1023 205 935 l 269 1023 q 287 971 273 990 q 326 943 301 952 q 408 935 351 935 q 509 956 477 935 q 549 1023 542 978 l 619 1023 q 556 890 612 938 q 406 842 499 842 z "
	},
	"˙": {
		ha: 351,
		x_min: 110,
		x_max: 243,
		o: "m 110 945 q 129 1001 110 983 q 176 1019 148 1019 q 223 1001 203 1019 q 243 945 243 983 q 223 888 243 907 q 176 869 203 869 q 129 888 148 869 q 110 945 110 907 z "
	},
	"˚": {
		ha: 802,
		x_min: 249,
		x_max: 551,
		o: "m 551 989 q 510 882 551 922 q 399 842 468 842 q 290 881 330 842 q 249 987 249 921 q 290 1093 249 1054 q 399 1132 331 1132 q 509 1092 467 1132 q 551 989 551 1051 m 478 987 q 456 1046 478 1025 q 399 1067 433 1067 q 342 1046 364 1067 q 319 987 319 1025 q 340 929 319 949 q 399 908 360 908 q 456 929 434 908 q 478 987 478 949 z "
	},
	"˛": {
		ha: 273,
		x_min: 25,
		x_max: 250,
		o: "m 121 -161 q 184 -225 121 -225 q 250 -216 213 -225 l 250 -290 q 165 -302 206 -302 q 25 -175 25 -302 q 54 -82 25 -127 q 132 0 83 -36 l 214 0 q 144 -82 168 -45 q 121 -161 121 -119 z "
	},
	"˜": {
		ha: 822,
		x_min: 179,
		x_max: 684,
		o: "m 534 843 q 477 856 505 843 q 423 883 450 868 q 371 911 396 899 q 323 924 347 924 q 272 903 290 924 q 245 842 255 883 l 179 842 q 227 970 188 924 q 328 1017 266 1017 q 388 1004 359 1017 q 444 976 417 991 q 494 948 470 961 q 541 936 519 936 q 590 956 574 936 q 616 1018 606 976 l 684 1018 q 636 889 675 936 q 534 843 598 843 z "
	},
	"˝": {
		ha: 802,
		x_min: 157,
		x_max: 644,
		o: "m 157 859 q 218 954 181 891 q 277 1064 256 1017 l 403 1064 l 403 1050 q 320 941 378 1007 q 222 842 262 874 l 157 842 l 157 859 m 399 859 q 466 965 432 905 q 518 1064 500 1025 l 644 1064 l 644 1050 q 561 941 619 1007 q 464 842 503 874 l 399 842 l 399 859 z "
	},
	"΄": {
		ha: 802,
		x_min: 345,
		x_max: 532,
		o: "m 345 861 q 381 987 363 910 q 407 1120 399 1064 l 532 1120 l 532 1104 q 483 974 519 1054 q 412 842 446 893 l 345 842 l 345 861 z "
	},
	"΅": {
		ha: 802,
		x_min: 192,
		x_max: 610,
		o: "m 347 958 q 412 1164 391 1073 l 541 1164 l 541 1150 q 477 1044 518 1105 q 401 942 436 983 l 347 942 l 347 958 m 192 945 q 210 996 192 980 q 253 1011 227 1011 q 297 994 281 1011 q 314 945 314 976 q 295 894 314 911 q 253 878 277 878 q 210 894 227 878 q 192 945 192 911 m 488 945 q 505 996 488 980 q 549 1011 523 1011 q 593 994 576 1011 q 610 945 610 976 q 591 894 610 911 q 549 878 572 878 q 505 894 523 878 q 488 945 488 911 z "
	},
	"Ά": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 19 790 q 55 916 37 839 q 81 1048 73 993 l 206 1048 l 206 1033 q 157 902 194 983 q 87 770 120 822 l 19 770 l 19 790 z "
	},
	"·": {
		ha: 370,
		x_min: 103,
		x_max: 267,
		o: "m 103 490 q 124 558 103 535 q 183 582 145 582 q 244 558 222 582 q 267 490 267 535 q 244 422 267 446 q 183 399 222 399 q 126 420 149 399 q 103 490 103 441 z "
	},
	"Έ": {
		ha: 859,
		x_min: -30,
		x_max: 774,
		o: "m 774 0 l 221 0 l 221 991 l 774 991 l 774 889 l 336 889 l 336 570 l 747 570 l 747 468 l 336 468 l 336 103 l 774 103 l 774 0 m -30 790 q 6 916 -12 839 q 33 1048 24 993 l 157 1048 l 157 1033 q 108 902 145 983 q 38 770 71 822 l -30 770 l -30 790 z "
	},
	"Ή": {
		ha: 1126,
		x_min: -30,
		x_max: 991,
		o: "m 991 0 l 876 0 l 876 467 l 353 467 l 353 0 l 238 0 l 238 991 l 353 991 l 353 570 l 876 570 l 876 991 l 991 991 l 991 0 m -30 790 q 6 916 -12 839 q 33 1048 24 993 l 157 1048 l 157 1033 q 108 902 145 983 q 38 770 71 822 l -30 770 l -30 790 z "
	},
	"Ό": {
		ha: 1129,
		x_min: -19,
		x_max: 1043,
		o: "m 1043 497 q 923 123 1043 259 q 588 -14 802 -14 q 250 120 369 -14 q 131 498 131 254 q 250 874 131 741 q 589 1007 370 1007 q 923 871 803 1007 q 1043 497 1043 736 m 253 497 q 339 192 253 296 q 588 87 425 87 q 837 191 753 87 q 921 497 921 295 q 837 801 921 697 q 589 904 753 904 q 339 800 425 904 q 253 497 253 696 m -19 790 q 17 916 -1 839 q 43 1048 35 993 l 168 1048 l 168 1033 q 119 902 156 983 q 49 770 82 822 l -19 770 l -19 790 z "
	},
	"Ύ": {
		ha: 958,
		x_min: -30,
		x_max: 958,
		o: "m 569 496 l 833 991 l 958 991 l 627 385 l 627 0 l 511 0 l 511 379 l 180 991 l 307 991 l 569 496 m -30 790 q 6 916 -12 839 q 33 1048 24 993 l 157 1048 l 157 1033 q 108 902 145 983 q 38 770 71 822 l -30 770 l -30 790 z "
	},
	"Ώ": {
		ha: 1129,
		x_min: -19,
		x_max: 1076,
		o: "m 586 904 q 340 817 425 904 q 255 558 255 730 q 314 293 255 406 q 494 90 372 180 l 494 0 l 97 0 l 97 103 l 344 103 q 187 302 241 183 q 133 558 133 420 q 253 888 133 768 q 586 1007 373 1007 q 920 889 800 1007 q 1040 559 1040 772 q 986 302 1040 419 q 830 103 932 184 l 1076 103 l 1076 0 l 679 0 l 679 90 q 860 294 803 181 q 918 558 918 408 q 833 817 918 729 q 586 904 749 904 m -19 790 q 17 916 -1 839 q 43 1048 35 993 l 168 1048 l 168 1033 q 119 902 156 983 q 49 770 82 822 l -19 770 l -19 790 z "
	},
	"ΐ": {
		ha: 471,
		x_min: -16,
		x_max: 447,
		o: "m 227 743 l 227 218 q 251 114 227 148 q 335 79 276 79 q 394 84 360 79 q 447 93 429 88 l 447 7 q 390 -8 426 -2 q 321 -14 355 -14 q 163 43 212 -14 q 114 215 114 100 l 114 743 l 227 743 m 140 958 q 205 1164 184 1073 l 333 1164 l 333 1150 q 270 1044 311 1105 q 194 942 229 983 l 140 942 l 140 958 m -16 945 q 2 996 -16 980 q 45 1011 20 1011 q 90 994 73 1011 q 106 945 106 976 q 88 894 106 911 q 45 878 69 878 q 2 894 20 878 q -16 945 -16 911 m 280 945 q 298 996 280 980 q 341 1011 315 1011 q 386 994 369 1011 q 402 945 402 976 q 384 894 402 911 q 341 878 365 878 q 298 894 315 878 q 280 945 280 911 z "
	},
	"Α": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 z "
	},
	"Β": {
		ha: 900,
		x_min: 136,
		x_max: 823,
		o: "m 136 991 l 416 991 q 702 932 614 991 q 790 746 790 873 q 741 601 790 658 q 597 526 692 543 l 597 519 q 823 282 823 481 q 733 75 823 149 q 482 0 644 0 l 136 0 l 136 991 m 252 567 l 441 567 q 617 605 564 567 q 671 734 671 644 q 611 855 671 818 q 421 892 551 892 l 252 892 l 252 567 m 252 469 l 252 98 l 458 98 q 639 145 578 98 q 700 290 700 191 q 638 426 700 382 q 449 469 576 469 l 252 469 z "
	},
	"Γ": {
		ha: 722,
		x_min: 136,
		x_max: 689,
		o: "m 689 991 l 689 888 l 252 888 l 252 0 l 136 0 l 136 991 l 689 991 z "
	},
	"Δ": {
		ha: 794,
		x_min: 26,
		x_max: 768,
		o: "m 26 71 l 340 991 l 453 991 l 768 69 l 768 0 l 26 0 l 26 71 m 396 860 q 328 625 355 707 l 151 103 l 640 103 l 465 619 q 396 860 419 756 z "
	},
	"Ε": {
		ha: 772,
		x_min: 136,
		x_max: 689,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 z "
	},
	"Ζ": {
		ha: 793,
		x_min: 56,
		x_max: 737,
		o: "m 737 0 l 56 0 l 56 90 l 582 888 l 72 888 l 72 991 l 722 991 l 722 901 l 196 104 l 737 104 l 737 0 z "
	},
	"Η": {
		ha: 1025,
		x_min: 136,
		x_max: 889,
		o: "m 889 0 l 774 0 l 774 467 l 252 467 l 252 0 l 136 0 l 136 991 l 252 991 l 252 570 l 774 570 l 774 991 l 889 991 l 889 0 z "
	},
	"Θ": {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 328 555 l 754 555 l 754 454 l 328 454 l 328 555 m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 207 497 q 292 192 207 296 q 542 87 376 87 q 791 191 707 87 q 875 497 875 294 q 791 802 875 699 q 543 904 707 904 q 292 800 377 904 q 207 497 207 696 z "
	},
	"Κ": {
		ha: 852,
		x_min: 136,
		x_max: 852,
		o: "m 852 0 l 717 0 l 355 481 l 252 389 l 252 0 l 136 0 l 136 991 l 252 991 l 252 500 l 701 991 l 838 991 l 439 561 l 852 0 z "
	},
	"Λ": {
		ha: 838,
		x_min: 0,
		x_max: 838,
		o: "m 838 0 l 714 0 l 490 629 q 416 861 431 800 q 346 631 394 767 l 123 0 l 0 0 l 358 991 l 478 991 l 838 0 z "
	},
	"Μ": {
		ha: 1254,
		x_min: 136,
		x_max: 1118,
		o: "m 575 0 l 239 879 l 233 879 q 243 631 243 774 l 243 0 l 136 0 l 136 991 l 310 991 l 624 174 l 629 174 l 946 991 l 1118 991 l 1118 0 l 1003 0 l 1003 639 q 1013 878 1003 749 l 1007 878 l 668 0 l 575 0 z "
	},
	"Ν": {
		ha: 1047,
		x_min: 136,
		x_max: 911,
		o: "m 911 0 l 779 0 l 237 832 l 232 832 q 243 564 243 686 l 243 0 l 136 0 l 136 991 l 267 991 l 807 163 l 812 163 q 806 280 811 181 q 803 422 802 380 l 803 991 l 911 991 l 911 0 z "
	},
	"Ξ": {
		ha: 768,
		x_min: 49,
		x_max: 720,
		o: "m 132 570 l 636 570 l 636 468 l 132 468 l 132 570 m 77 991 l 692 991 l 692 889 l 77 889 l 77 991 m 720 103 l 720 0 l 49 0 l 49 103 l 720 103 z "
	},
	"Ο": {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 207 497 q 293 192 207 296 q 542 87 378 87 q 791 191 707 87 q 875 497 875 295 q 791 801 875 697 q 543 904 707 904 q 293 800 378 904 q 207 497 207 696 z "
	},
	"Π": {
		ha: 1013,
		x_min: 136,
		x_max: 876,
		o: "m 876 0 l 761 0 l 761 889 l 252 889 l 252 0 l 136 0 l 136 991 l 876 991 l 876 0 z "
	},
	"Ρ": {
		ha: 836,
		x_min: 136,
		x_max: 765,
		o: "m 765 703 q 662 471 765 552 q 368 390 559 390 l 252 390 l 252 0 l 136 0 l 136 991 l 393 991 q 765 703 765 991 m 252 489 l 355 489 q 577 538 509 489 q 646 697 646 588 q 581 844 646 795 q 380 892 517 892 l 252 892 l 252 489 z "
	},
	"Σ": {
		ha: 787,
		x_min: 50,
		x_max: 757,
		o: "m 50 0 l 50 96 l 376 518 l 58 895 l 58 991 l 717 991 l 717 888 l 258 888 l 193 889 l 505 519 l 179 103 l 757 103 l 757 0 l 50 0 z "
	},
	"Τ": {
		ha: 768,
		x_min: 12,
		x_max: 755,
		o: "m 441 0 l 326 0 l 326 889 l 12 889 l 12 991 l 755 991 l 755 889 l 441 889 l 441 0 z "
	},
	"Υ": {
		ha: 778,
		x_min: 0,
		x_max: 778,
		o: "m 389 496 l 653 991 l 778 991 l 447 385 l 447 0 l 330 0 l 330 379 l 0 991 l 126 991 l 389 496 z "
	},
	"Φ": {
		ha: 1108,
		x_min: 72,
		x_max: 1036,
		o: "m 496 1006 l 612 1006 l 612 884 l 660 884 q 861 838 776 884 q 991 707 946 791 q 1036 515 1036 623 q 986 315 1036 403 q 847 183 935 227 q 640 139 760 139 l 612 139 l 612 -14 l 496 -14 l 496 139 l 465 139 q 258 184 346 139 q 121 317 171 229 q 72 514 72 405 q 118 705 72 621 q 249 837 163 789 q 450 884 334 884 l 496 884 l 496 1006 m 612 236 l 629 236 q 841 310 766 236 q 917 517 917 385 q 847 713 917 642 q 652 785 777 785 l 612 785 l 612 236 m 496 785 l 457 785 q 263 713 334 785 q 192 517 192 641 q 267 310 192 383 q 479 236 342 236 l 496 236 l 496 785 z "
	},
	"Χ": {
		ha: 802,
		x_min: 5,
		x_max: 796,
		o: "m 796 0 l 665 0 l 399 436 l 127 0 l 5 0 l 335 518 l 28 991 l 155 991 l 401 599 l 650 991 l 772 991 l 465 522 l 796 0 z "
	},
	"Ψ": {
		ha: 1105,
		x_min: 74,
		x_max: 1032,
		o: "m 610 0 l 494 0 l 494 302 l 464 302 q 258 344 345 302 q 123 469 172 385 q 74 664 74 553 l 74 991 l 192 991 l 192 667 q 262 471 192 539 q 476 403 332 403 l 494 403 l 494 991 l 610 991 l 610 403 l 629 403 q 843 470 772 403 q 913 664 913 537 l 913 991 l 1032 991 l 1032 665 q 983 470 1032 553 q 847 344 935 386 q 640 302 759 302 l 610 302 l 610 0 z "
	},
	"Ω": {
		ha: 1086,
		x_min: 54,
		x_max: 1034,
		o: "m 543 904 q 297 817 382 904 q 212 558 212 730 q 271 293 212 406 q 452 90 330 180 l 452 0 l 54 0 l 54 103 l 301 103 q 144 302 199 183 q 90 558 90 420 q 210 888 90 768 q 543 1007 330 1007 q 877 889 757 1007 q 997 559 997 772 q 943 302 997 419 q 787 103 890 184 l 1034 103 l 1034 0 l 636 0 l 636 90 q 818 294 760 181 q 875 558 875 408 q 790 817 875 729 q 543 904 706 904 z "
	},
	"Ϋ": {
		ha: 778,
		x_min: 0,
		x_max: 778,
		o: "m 389 496 l 653 991 l 778 991 l 447 385 l 447 0 l 330 0 l 330 379 l 0 991 l 126 991 l 389 496 m 198 1174 q 216 1225 198 1209 q 259 1240 234 1240 q 303 1225 285 1240 q 322 1174 322 1209 q 303 1123 322 1140 q 259 1107 285 1107 q 216 1123 234 1107 q 198 1174 198 1140 m 456 1174 q 474 1225 456 1209 q 517 1240 492 1240 q 561 1225 543 1240 q 580 1174 580 1209 q 561 1123 580 1140 q 517 1107 543 1107 q 474 1123 492 1107 q 456 1174 456 1140 z "
	},
	"ά": {
		ha: 848,
		x_min: 78,
		x_max: 829,
		o: "m 401 81 q 567 147 516 81 q 618 361 618 213 l 618 371 q 566 594 618 526 q 400 661 515 661 q 195 368 195 661 q 244 153 195 224 q 401 81 294 81 m 385 -14 q 159 87 239 -14 q 78 370 78 188 q 161 656 78 555 q 396 757 243 757 q 533 728 478 757 q 624 643 587 700 l 632 643 q 676 743 648 705 l 764 743 q 740 615 749 699 q 730 453 730 532 l 730 155 q 787 78 730 78 q 829 85 807 78 l 829 4 q 754 -14 798 -14 q 669 14 699 -14 q 627 100 639 41 l 618 100 q 521 14 578 41 q 385 -14 465 -14 m 364 861 q 400 987 382 910 q 427 1120 418 1064 l 551 1120 l 551 1104 q 502 974 539 1054 q 432 842 465 893 l 364 842 l 364 861 z "
	},
	"έ": {
		ha: 660,
		x_min: 61,
		x_max: 612,
		o: "m 485 435 l 485 339 l 385 339 q 174 207 174 339 q 223 113 174 146 q 372 79 273 79 q 486 93 429 79 q 610 138 542 106 l 610 37 q 367 -14 517 -14 q 143 44 224 -14 q 61 201 61 101 q 98 312 61 268 q 224 385 136 357 l 224 392 q 121 454 157 411 q 85 559 85 497 q 161 703 85 650 q 367 757 237 757 q 489 746 433 757 q 612 704 545 734 l 570 614 q 462 652 506 642 q 364 661 418 661 q 195 551 195 661 q 410 435 195 435 l 485 435 m 307 861 q 343 987 325 910 q 369 1120 361 1064 l 494 1120 l 494 1104 q 445 974 481 1054 q 374 842 408 893 l 307 842 l 307 861 z "
	},
	"ή": {
		ha: 852,
		x_min: 119,
		x_max: 741,
		o: "m 628 -334 l 628 481 q 587 616 628 572 q 457 661 545 661 q 286 598 340 661 q 232 390 232 535 l 232 0 l 119 0 l 119 743 l 211 743 l 229 642 l 235 642 q 332 727 269 696 q 471 757 394 757 q 673 692 605 757 q 741 485 741 627 l 741 -334 l 628 -334 m 385 861 q 421 987 403 910 q 447 1120 439 1064 l 572 1120 l 572 1104 q 523 974 559 1054 q 452 842 486 893 l 385 842 l 385 861 z "
	},
	"ί": {
		ha: 471,
		x_min: 114,
		x_max: 447,
		o: "m 227 743 l 227 218 q 251 114 227 148 q 335 79 276 79 q 394 84 360 79 q 447 93 429 88 l 447 7 q 390 -8 426 -2 q 321 -14 355 -14 q 163 43 212 -14 q 114 215 114 100 l 114 743 l 227 743 m 130 861 q 166 987 149 910 q 193 1120 184 1064 l 317 1120 l 317 1104 q 268 974 305 1054 q 198 842 231 893 l 130 842 l 130 861 z "
	},
	"ΰ": {
		ha: 846,
		x_min: 111,
		x_max: 771,
		o: "m 425 -14 q 190 71 269 -14 q 111 336 111 156 l 111 743 l 224 743 l 224 342 q 277 145 224 212 q 435 79 331 79 q 603 160 548 79 q 657 411 657 241 q 648 575 657 499 q 615 743 638 651 l 728 743 q 762 573 752 646 q 771 406 771 500 q 684 88 771 190 q 425 -14 596 -14 m 387 958 q 452 1164 431 1073 l 581 1164 l 581 1150 q 517 1044 558 1105 q 441 942 476 983 l 387 942 l 387 958 m 232 945 q 250 996 232 980 q 293 1011 267 1011 q 337 994 321 1011 q 354 945 354 976 q 335 894 354 911 q 293 878 317 878 q 250 894 267 878 q 232 945 232 911 m 528 945 q 545 996 528 980 q 589 1011 563 1011 q 633 994 616 1011 q 650 945 650 976 q 631 894 650 911 q 589 878 612 878 q 545 894 563 878 q 528 945 528 911 z "
	},
	"α": {
		ha: 848,
		x_min: 78,
		x_max: 829,
		o: "m 401 81 q 567 147 516 81 q 618 361 618 213 l 618 371 q 566 594 618 526 q 400 661 515 661 q 195 368 195 661 q 244 153 195 224 q 401 81 294 81 m 385 -14 q 159 87 239 -14 q 78 370 78 188 q 161 656 78 555 q 396 757 243 757 q 533 728 478 757 q 624 643 587 700 l 632 643 q 676 743 648 705 l 764 743 q 740 615 749 699 q 730 453 730 532 l 730 155 q 787 78 730 78 q 829 85 807 78 l 829 4 q 754 -14 798 -14 q 669 14 699 -14 q 627 100 639 41 l 618 100 q 521 14 578 41 q 385 -14 465 -14 z "
	},
	"β": {
		ha: 872,
		x_min: 119,
		x_max: 808,
		o: "m 447 1063 q 681 992 596 1063 q 765 798 765 922 q 553 564 765 598 l 553 558 q 808 281 808 530 q 719 63 808 139 q 468 -14 629 -14 q 340 -3 394 -14 q 232 33 286 7 l 232 -334 l 119 -334 l 119 743 q 205 979 119 896 q 447 1063 291 1063 m 446 968 q 285 909 338 968 q 232 739 232 851 l 232 138 q 350 92 295 104 q 463 79 404 79 q 637 130 578 79 q 696 288 696 181 q 631 450 696 394 q 447 505 567 505 l 371 505 l 371 602 l 433 602 q 593 651 538 602 q 648 794 648 701 q 595 923 648 878 q 446 968 543 968 z "
	},
	"γ": {
		ha: 711,
		x_min: 7,
		x_max: 704,
		o: "m 361 -334 l 239 -334 q 260 -169 239 -269 q 311 8 282 -69 l 7 743 l 123 743 l 286 347 q 363 118 350 188 l 368 118 q 386 186 372 138 q 414 278 399 235 q 587 743 429 322 l 704 743 l 429 18 q 379 -165 397 -66 q 361 -334 361 -264 z "
	},
	"δ": {
		ha: 806,
		x_min: 77,
		x_max: 760,
		o: "m 370 635 q 235 742 275 688 q 196 863 196 795 q 262 1002 196 951 q 439 1054 328 1054 q 573 1039 509 1054 q 722 981 637 1023 l 673 889 q 544 946 597 931 q 435 961 490 961 q 344 934 377 961 q 311 863 311 907 q 348 777 311 814 q 497 677 384 739 q 700 518 640 600 q 760 325 760 436 q 667 76 760 165 q 415 -14 575 -14 q 169 71 261 -14 q 77 297 77 155 q 153 506 77 417 q 370 635 230 594 m 643 325 q 601 474 643 412 q 464 586 559 536 q 259 476 324 550 q 195 294 195 403 q 255 139 195 197 q 415 81 315 81 q 584 144 525 81 q 643 325 643 207 z "
	},
	"ε": {
		ha: 660,
		x_min: 61,
		x_max: 612,
		o: "m 485 435 l 485 339 l 385 339 q 174 207 174 339 q 223 113 174 146 q 372 79 273 79 q 486 93 429 79 q 610 138 542 106 l 610 37 q 367 -14 517 -14 q 143 44 224 -14 q 61 201 61 101 q 98 312 61 268 q 224 385 136 357 l 224 392 q 121 454 157 411 q 85 559 85 497 q 161 703 85 650 q 367 757 237 757 q 489 746 433 757 q 612 704 545 734 l 570 614 q 462 652 506 642 q 364 661 418 661 q 195 551 195 661 q 410 435 195 435 l 485 435 z "
	},
	"ζ": {
		ha: 671,
		x_min: 78,
		x_max: 629,
		o: "m 119 960 l 119 1055 l 629 1055 l 629 968 q 386 736 484 846 q 241 529 288 627 q 195 318 195 431 q 215 212 195 252 q 277 147 235 172 q 436 97 319 122 q 583 39 537 76 q 629 -60 629 2 q 543 -272 629 -161 l 431 -272 q 515 -90 515 -163 q 478 -32 515 -52 q 343 7 440 -11 q 142 105 205 31 q 78 311 78 180 q 98 459 78 389 q 156 596 118 530 q 262 740 194 662 q 494 965 330 817 q 283 960 467 960 l 119 960 z "
	},
	"η": {
		ha: 852,
		x_min: 119,
		x_max: 741,
		o: "m 628 -334 l 628 481 q 587 616 628 572 q 457 661 545 661 q 286 598 340 661 q 232 390 232 535 l 232 0 l 119 0 l 119 743 l 211 743 l 229 642 l 235 642 q 332 727 269 696 q 471 757 394 757 q 673 692 605 757 q 741 485 741 627 l 741 -334 l 628 -334 z "
	},
	"θ": {
		ha: 822,
		x_min: 78,
		x_max: 745,
		o: "m 745 529 q 662 120 745 254 q 410 -14 579 -14 q 162 123 247 -14 q 78 529 78 260 q 161 938 78 805 q 410 1071 244 1071 q 660 933 575 1071 q 745 529 745 796 m 410 81 q 574 184 521 81 q 631 496 627 288 l 192 496 q 245 185 195 290 q 410 81 296 81 m 410 977 q 250 879 300 977 q 192 591 199 782 l 629 591 q 570 879 622 782 q 410 977 519 977 z "
	},
	"ι": {
		ha: 471,
		x_min: 114,
		x_max: 447,
		o: "m 227 743 l 227 218 q 251 114 227 148 q 335 79 276 79 q 394 84 360 79 q 447 93 429 88 l 447 7 q 390 -8 426 -2 q 321 -14 355 -14 q 163 43 212 -14 q 114 215 114 100 l 114 743 l 227 743 z "
	},
	"κ": {
		ha: 720,
		x_min: 119,
		x_max: 713,
		o: "m 553 742 l 693 742 l 412 415 l 713 0 l 576 0 l 332 342 l 240 265 l 240 0 l 119 0 l 119 742 l 240 742 l 240 550 q 232 361 240 437 l 553 742 z "
	},
	"λ": {
		ha: 742,
		x_min: -9,
		x_max: 742,
		o: "m -9 0 l 311 732 l 272 842 q 235 923 252 899 q 195 959 218 947 q 139 971 172 971 q 61 962 100 971 l 61 1053 q 150 1064 107 1064 q 253 1044 212 1064 q 323 979 294 1023 q 390 832 353 935 l 636 146 q 663 95 649 113 q 701 78 677 78 q 742 85 720 78 l 742 3 q 668 -14 709 -14 q 590 12 618 -14 q 542 94 562 38 l 436 390 q 364 618 379 553 l 358 618 q 279 397 339 539 l 107 0 l -9 0 z "
	},
	"μ": {
		ha: 860,
		x_min: 119,
		x_max: 741,
		o: "m 232 258 q 404 81 232 81 q 574 145 520 81 q 628 353 628 209 l 628 743 l 741 743 l 741 0 l 648 0 l 631 100 l 624 100 q 393 -14 549 -14 q 232 49 292 -14 l 225 49 q 232 -117 232 -8 l 232 -334 l 119 -334 l 119 743 l 232 743 l 232 258 z "
	},
	"ν": {
		ha: 753,
		x_min: 0,
		x_max: 696,
		o: "m 0 743 l 117 743 l 265 343 q 311 217 283 298 q 350 94 339 136 l 355 94 q 529 364 475 213 q 583 743 583 515 l 696 743 q 626 325 696 497 q 403 0 555 153 l 277 0 l 0 743 z "
	},
	"ξ": {
		ha: 660,
		x_min: 77,
		x_max: 629,
		o: "m 579 511 l 458 511 q 267 451 339 511 q 195 294 195 391 q 212 207 195 240 q 261 153 229 174 q 339 120 293 132 q 441 96 385 108 q 584 39 538 76 q 629 -60 629 1 q 611 -154 629 -104 q 547 -272 593 -205 l 441 -272 q 500 -174 477 -224 q 522 -89 522 -124 q 483 -31 522 -52 q 347 7 444 -10 q 144 98 212 30 q 77 285 77 165 q 130 451 77 380 q 271 549 184 522 l 271 557 q 123 747 123 600 q 171 880 123 827 q 330 968 218 934 q 165 960 243 960 l 119 960 l 119 1055 l 593 1055 l 593 966 l 558 966 q 394 939 470 966 q 275 863 318 912 q 232 750 232 814 q 289 635 232 672 q 464 599 345 599 l 579 599 l 579 511 z "
	},
	"ο": {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 195 372 q 252 155 195 230 q 419 81 309 81 q 587 155 530 81 q 644 372 644 229 q 587 588 644 514 q 418 661 530 661 q 251 589 307 661 q 195 372 195 516 z "
	},
	"π": {
		ha: 903,
		x_min: 17,
		x_max: 860,
		o: "m 779 79 q 838 92 805 79 l 838 3 q 751 -14 808 -14 q 603 158 603 -14 l 603 647 l 279 647 l 279 0 l 167 0 l 167 647 l 17 647 l 17 697 l 114 743 l 860 743 l 860 647 l 715 647 l 715 167 q 733 98 715 117 q 779 79 750 79 z "
	},
	"ρ": {
		ha: 839,
		x_min: 113,
		x_max: 761,
		o: "m 761 372 q 674 87 761 189 q 429 -14 587 -14 q 227 50 308 -14 l 221 50 q 227 -135 227 9 l 227 -334 l 113 -334 l 113 382 q 198 660 113 562 q 441 757 283 757 q 675 654 590 757 q 761 372 761 551 m 436 661 q 278 591 329 661 q 227 379 227 521 l 227 150 q 433 81 309 81 q 593 151 541 81 q 644 372 644 222 q 595 589 644 517 q 436 661 547 661 z "
	},
	"ς": {
		ha: 669,
		x_min: 78,
		x_max: 631,
		o: "m 195 354 q 215 218 195 262 q 283 146 235 173 q 440 96 332 119 q 585 38 541 74 q 629 -60 629 1 q 611 -156 629 -104 q 547 -272 593 -208 l 441 -272 q 501 -174 478 -224 q 524 -90 524 -124 q 506 -47 524 -63 q 452 -18 489 -31 q 349 7 414 -5 q 144 118 210 32 q 78 353 78 204 q 172 652 78 547 q 433 757 265 757 q 541 746 487 757 q 631 718 594 734 l 595 623 q 429 657 507 657 q 252 583 310 657 q 195 354 195 509 z "
	},
	"σ": {
		ha: 851,
		x_min: 78,
		x_max: 818,
		o: "m 760 345 q 718 156 760 238 q 598 31 676 75 q 416 -14 521 -14 q 168 84 257 -14 q 78 354 78 182 q 479 743 78 743 l 818 743 l 818 647 l 639 647 q 760 345 760 534 m 195 354 q 252 152 195 222 q 419 81 310 81 q 585 148 527 81 q 643 338 643 214 q 525 647 643 521 l 481 647 q 262 576 330 647 q 195 354 195 506 z "
	},
	"τ": {
		ha: 657,
		x_min: 12,
		x_max: 621,
		o: "m 621 743 l 621 647 l 328 647 l 328 228 q 467 77 328 77 q 532 81 498 77 q 583 90 565 85 l 583 5 q 522 -11 559 -5 q 452 -17 484 -17 q 271 41 329 -17 q 214 214 214 98 l 214 647 l 12 647 l 12 697 l 113 743 l 621 743 z "
	},
	"υ": {
		ha: 846,
		x_min: 111,
		x_max: 771,
		o: "m 425 -14 q 190 71 269 -14 q 111 336 111 156 l 111 743 l 224 743 l 224 342 q 277 145 224 212 q 435 79 331 79 q 603 160 548 79 q 657 411 657 241 q 648 575 657 499 q 615 743 638 651 l 728 743 q 762 573 752 646 q 771 406 771 500 q 684 88 771 190 q 425 -14 596 -14 z "
	},
	"φ": {
		ha: 997,
		x_min: 78,
		x_max: 920,
		o: "m 436 -334 l 436 -12 q 169 94 260 -5 q 78 376 78 193 q 218 749 78 577 l 307 688 q 219 532 247 608 q 192 372 192 456 q 436 81 192 104 l 436 501 q 492 692 436 628 q 650 757 549 757 q 846 657 772 757 q 920 389 920 558 q 873 179 920 268 q 743 42 827 91 q 547 -12 659 -7 l 547 -334 l 436 -334 m 803 387 q 761 587 803 513 q 650 661 719 661 q 574 621 600 661 q 547 503 547 580 l 547 81 q 736 172 669 92 q 803 387 803 252 z "
	},
	"χ": {
		ha: 758,
		x_min: -14,
		x_max: 749,
		o: "m 121 747 q 184 733 157 747 q 231 687 210 718 q 282 581 252 656 l 380 330 l 596 743 l 718 743 l 428 220 l 557 -109 q 617 -216 589 -191 q 688 -241 645 -241 q 749 -236 718 -241 l 749 -323 q 668 -334 708 -334 q 542 -294 590 -334 q 454 -146 494 -254 l 353 117 l 107 -334 l -14 -334 l 301 233 l 184 534 q 135 629 159 602 q 82 656 111 656 q 39 648 57 656 l 39 736 q 121 747 72 747 z "
	},
	"ψ": {
		ha: 1046,
		x_min: 111,
		x_max: 960,
		o: "m 582 1054 l 582 83 q 778 172 709 94 q 847 389 847 250 q 838 550 847 470 q 804 743 829 629 l 917 743 q 960 393 960 559 q 863 102 960 206 q 582 -12 766 -1 l 582 -334 l 471 -334 l 471 -12 q 201 92 292 -6 q 111 375 111 191 l 111 743 l 224 743 l 224 371 q 285 165 224 239 q 471 82 346 91 l 471 1054 l 582 1054 z "
	},
	"ω": {
		ha: 1074,
		x_min: 78,
		x_max: 996,
		o: "m 339 -14 q 147 87 216 -14 q 78 359 78 187 q 97 552 78 465 q 161 743 115 640 l 278 743 q 213 551 232 637 q 193 358 193 465 q 234 154 193 227 q 347 81 275 81 q 447 129 411 81 q 482 262 482 178 l 482 473 l 591 473 l 591 262 q 627 128 591 176 q 726 81 663 81 q 840 154 799 81 q 880 358 880 227 q 862 549 880 460 q 796 743 843 637 l 913 743 q 976 555 957 644 q 996 359 996 466 q 927 87 996 189 q 734 -14 858 -14 q 539 110 585 -14 l 533 110 q 339 -14 489 -14 z "
	},
	"ϊ": {
		ha: 471,
		x_min: 6,
		x_max: 447,
		o: "m 227 743 l 227 218 q 251 114 227 148 q 335 79 276 79 q 394 84 360 79 q 447 93 429 88 l 447 7 q 390 -8 426 -2 q 321 -14 355 -14 q 163 43 212 -14 q 114 215 114 100 l 114 743 l 227 743 m 6 945 q 24 996 6 980 q 67 1011 42 1011 q 112 996 93 1011 q 130 945 130 980 q 112 894 130 911 q 67 878 93 878 q 24 894 42 878 q 6 945 6 911 m 264 945 q 282 996 264 980 q 326 1011 300 1011 q 369 996 351 1011 q 388 945 388 980 q 369 894 388 911 q 326 878 351 878 q 282 894 300 878 q 264 945 264 911 z "
	},
	"ϋ": {
		ha: 846,
		x_min: 111,
		x_max: 771,
		o: "m 425 -14 q 190 71 269 -14 q 111 336 111 156 l 111 743 l 224 743 l 224 342 q 277 145 224 212 q 435 79 331 79 q 603 160 548 79 q 657 411 657 241 q 648 575 657 499 q 615 743 638 651 l 728 743 q 762 573 752 646 q 771 406 771 500 q 684 88 771 190 q 425 -14 596 -14 m 248 945 q 266 996 248 980 q 309 1011 284 1011 q 354 996 335 1011 q 372 945 372 980 q 354 894 372 911 q 309 878 335 878 q 266 894 284 878 q 248 945 248 911 m 507 945 q 525 996 507 980 q 568 1011 543 1011 q 611 996 593 1011 q 630 945 630 980 q 611 894 630 911 q 568 878 593 878 q 525 894 543 878 q 507 945 507 911 z "
	},
	"ό": {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 195 372 q 252 155 195 230 q 419 81 309 81 q 587 155 530 81 q 644 372 644 229 q 587 588 644 514 q 418 661 530 661 q 251 589 307 661 q 195 372 195 516 m 367 861 q 403 987 385 910 q 429 1120 421 1064 l 554 1120 l 554 1104 q 505 974 542 1054 q 435 842 468 893 l 367 842 l 367 861 z "
	},
	"ύ": {
		ha: 846,
		x_min: 111,
		x_max: 771,
		o: "m 425 -14 q 190 71 269 -14 q 111 336 111 156 l 111 743 l 224 743 l 224 342 q 277 145 224 212 q 435 79 331 79 q 603 160 548 79 q 657 411 657 241 q 648 575 657 499 q 615 743 638 651 l 728 743 q 762 573 752 646 q 771 406 771 500 q 684 88 771 190 q 425 -14 596 -14 m 371 861 q 407 987 389 910 q 433 1120 425 1064 l 558 1120 l 558 1104 q 509 974 546 1054 q 439 842 472 893 l 371 842 l 371 861 z "
	},
	"ώ": {
		ha: 1074,
		x_min: 78,
		x_max: 996,
		o: "m 339 -14 q 147 87 216 -14 q 78 359 78 187 q 97 552 78 465 q 161 743 115 640 l 278 743 q 213 551 232 637 q 193 358 193 465 q 234 154 193 227 q 347 81 275 81 q 447 129 411 81 q 482 262 482 178 l 482 473 l 591 473 l 591 262 q 627 128 591 176 q 726 81 663 81 q 840 154 799 81 q 880 358 880 227 q 862 549 880 460 q 796 743 843 637 l 913 743 q 976 555 957 644 q 996 359 996 466 q 927 87 996 189 q 734 -14 858 -14 q 539 110 585 -14 l 533 110 q 339 -14 489 -14 m 481 861 q 517 987 499 910 q 543 1120 535 1064 l 668 1120 l 668 1104 q 619 974 656 1054 q 549 842 582 893 l 481 842 l 481 861 z "
	},
	"Ё": {
		ha: 772,
		x_min: 136,
		x_max: 689,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 m 236 1174 q 254 1225 236 1209 q 297 1240 272 1240 q 341 1225 323 1240 q 360 1174 360 1209 q 341 1123 360 1140 q 297 1107 323 1107 q 254 1123 272 1107 q 236 1174 236 1140 m 494 1174 q 512 1225 494 1209 q 555 1240 530 1240 q 599 1225 581 1240 q 618 1174 618 1209 q 599 1123 618 1140 q 555 1107 581 1107 q 512 1123 530 1107 q 494 1174 494 1140 z "
	},
	"Ђ": {
		ha: 1019,
		x_min: 12,
		x_max: 913,
		o: "m 661 -14 q 559 1 596 -14 l 559 103 q 659 90 597 90 q 762 132 727 90 q 798 250 798 174 l 798 339 q 753 464 798 426 q 614 503 709 503 l 355 503 l 355 0 l 240 0 l 240 889 l 12 889 l 12 991 l 657 991 l 657 889 l 355 889 l 355 606 l 624 606 q 838 541 763 606 q 913 355 913 476 l 913 258 q 846 58 913 130 q 661 -14 780 -14 z "
	},
	"Ѓ": {
		ha: 722,
		x_min: 136,
		x_max: 689,
		o: "m 689 991 l 689 888 l 252 888 l 252 0 l 136 0 l 136 991 l 689 991 m 328 1088 q 398 1190 360 1130 q 457 1293 435 1249 l 594 1293 l 594 1279 q 505 1171 564 1235 q 403 1071 446 1106 l 328 1071 l 328 1088 z "
	},
	"Є": {
		ha: 888,
		x_min: 85,
		x_max: 848,
		o: "m 566 903 q 321 818 412 903 q 210 570 230 733 l 700 570 l 700 468 l 207 468 q 304 185 214 282 q 564 89 395 89 q 810 126 673 89 l 810 25 q 547 -14 701 -14 q 203 118 322 -14 q 85 497 85 250 q 213 871 85 735 q 567 1007 340 1007 q 848 947 728 1007 l 800 847 q 566 903 686 903 z "
	},
	"Ѕ": {
		ha: 762,
		x_min: 72,
		x_max: 696,
		o: "m 696 264 q 601 60 696 133 q 343 -14 506 -14 q 72 32 167 -14 l 72 143 q 205 102 133 117 q 347 87 277 87 q 521 131 463 87 q 579 253 579 175 q 558 337 579 304 q 489 398 538 370 q 342 461 441 426 q 144 578 203 511 q 85 755 85 646 q 171 938 85 870 q 399 1006 257 1006 q 671 951 547 1006 l 635 851 q 396 903 512 903 q 253 863 304 903 q 201 754 201 824 q 220 670 201 703 q 284 609 239 637 q 422 549 330 582 q 637 429 578 493 q 696 264 696 366 z "
	},
	"Ј": {
		ha: 371,
		x_min: -109,
		x_max: 244,
		o: "m -8 -261 q -109 -243 -72 -261 l -109 -144 q -8 -158 -60 -158 q 94 -117 59 -158 q 129 0 129 -77 l 129 991 l 244 991 l 244 9 q 179 -190 244 -119 q -8 -261 114 -261 z "
	},
	"Љ": {
		ha: 1291,
		x_min: 0,
		x_max: 1239,
		o: "m 1239 289 q 1146 75 1239 149 q 876 0 1053 0 l 654 0 l 654 889 l 407 889 q 340 414 368 591 q 284 152 311 237 q 210 25 257 66 q 90 -16 163 -16 q 0 1 43 -16 l 0 98 q 77 81 34 81 q 131 102 109 81 q 168 158 153 123 q 201 277 182 193 q 243 513 220 362 q 310 991 266 665 l 770 991 l 770 568 l 852 568 q 1239 289 1239 568 m 770 100 l 860 100 q 1056 147 994 100 q 1118 289 1118 194 q 1053 425 1118 382 q 839 467 988 467 l 770 467 l 770 100 z "
	},
	"Њ": {
		ha: 1324,
		x_min: 136,
		x_max: 1272,
		o: "m 1272 289 q 1180 75 1272 149 q 911 0 1088 0 l 688 0 l 688 467 l 252 467 l 252 0 l 136 0 l 136 991 l 252 991 l 252 570 l 688 570 l 688 991 l 804 991 l 804 568 l 886 568 q 1272 289 1272 568 m 804 100 l 895 100 q 1090 147 1027 100 q 1153 289 1153 194 q 1087 424 1153 382 q 873 467 1022 467 l 804 467 l 804 100 z "
	},
	"Ћ": {
		ha: 1019,
		x_min: 12,
		x_max: 913,
		o: "m 355 606 l 627 606 q 839 542 766 606 q 913 355 913 478 l 913 0 l 798 0 l 798 340 q 755 464 798 426 q 618 503 713 503 l 355 503 l 355 0 l 240 0 l 240 889 l 12 889 l 12 991 l 700 991 l 700 889 l 355 889 l 355 606 z "
	},
	"Ќ": {
		ha: 850,
		x_min: 136,
		x_max: 850,
		o: "m 850 0 l 710 0 l 252 503 l 252 0 l 136 0 l 136 991 l 252 991 l 252 511 l 699 991 l 831 991 l 392 510 l 850 0 m 376 1088 q 447 1190 409 1130 q 506 1293 484 1249 l 643 1293 l 643 1279 q 554 1171 613 1235 q 452 1071 495 1106 l 376 1071 l 376 1088 z "
	},
	"Ў": {
		ha: 863,
		x_min: 18,
		x_max: 863,
		o: "m 199 -14 q 66 7 123 -14 l 66 119 q 195 90 130 90 q 314 124 269 90 q 404 253 359 159 l 18 991 l 146 991 l 439 428 q 465 363 456 395 l 471 363 q 497 435 490 420 l 741 991 l 863 991 l 546 282 q 443 98 489 155 q 340 13 397 40 q 199 -14 283 -14 m 442 1071 q 259 1119 317 1071 q 195 1279 201 1168 l 300 1279 q 337 1181 307 1208 q 445 1153 368 1153 q 548 1183 515 1153 q 589 1279 582 1213 l 696 1279 q 627 1121 688 1172 q 442 1071 567 1071 z "
	},
	"Џ": {
		ha: 1013,
		x_min: 136,
		x_max: 876,
		o: "m 876 0 l 561 0 l 561 -258 l 441 -258 l 441 0 l 136 0 l 136 991 l 252 991 l 252 104 l 761 104 l 761 991 l 876 991 l 876 0 z "
	},
	"А": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 z "
	},
	"Б": {
		ha: 851,
		x_min: 136,
		x_max: 779,
		o: "m 779 289 q 691 71 779 141 q 427 0 604 0 l 136 0 l 136 991 l 721 991 l 721 889 l 252 889 l 252 568 l 406 568 q 618 538 536 568 q 740 449 701 508 q 779 289 779 390 m 252 100 l 414 100 q 601 144 543 100 q 660 289 660 188 q 600 426 660 385 q 392 467 541 467 l 252 467 l 252 100 z "
	},
	"В": {
		ha: 900,
		x_min: 136,
		x_max: 823,
		o: "m 136 991 l 416 991 q 702 932 614 991 q 790 746 790 873 q 741 601 790 658 q 597 526 692 543 l 597 519 q 823 282 823 481 q 733 75 823 149 q 482 0 644 0 l 136 0 l 136 991 m 252 567 l 441 567 q 617 605 564 567 q 671 734 671 644 q 611 855 671 818 q 421 892 551 892 l 252 892 l 252 567 m 252 469 l 252 98 l 458 98 q 639 145 578 98 q 700 290 700 191 q 638 426 700 382 q 449 469 576 469 l 252 469 z "
	},
	"Г": {
		ha: 722,
		x_min: 136,
		x_max: 689,
		o: "m 689 991 l 689 888 l 252 888 l 252 0 l 136 0 l 136 991 l 689 991 z "
	},
	"Д": {
		ha: 949,
		x_min: 9,
		x_max: 918,
		o: "m 918 -258 l 808 -258 l 808 0 l 119 0 l 119 -258 l 9 -258 l 9 104 l 86 104 q 265 532 191 280 q 347 991 339 783 l 793 991 l 793 104 l 918 104 l 918 -258 m 677 104 l 677 888 l 452 888 q 370 493 439 724 q 207 104 300 263 l 677 104 z "
	},
	"Е": {
		ha: 772,
		x_min: 136,
		x_max: 689,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 z "
	},
	"Ж": {
		ha: 1173,
		x_min: 1,
		x_max: 1169,
		o: "m 406 510 l 16 991 l 144 991 l 530 511 l 530 991 l 642 991 l 642 511 l 1028 991 l 1157 991 l 766 511 l 1169 0 l 1036 0 l 642 503 l 642 0 l 530 0 l 530 503 l 136 0 l 1 0 l 406 510 z "
	},
	"З": {
		ha: 807,
		x_min: 50,
		x_max: 730,
		o: "m 711 760 q 649 601 711 662 q 478 524 587 540 l 478 518 q 666 440 602 501 q 730 281 730 379 q 627 64 730 142 q 334 -14 523 -14 q 50 40 161 -14 l 50 147 q 191 99 115 116 q 336 82 267 82 q 539 133 471 82 q 608 279 608 184 q 532 417 608 370 q 304 464 456 464 l 157 464 l 157 561 l 298 561 q 514 611 437 561 q 590 750 590 661 q 535 866 590 823 q 387 909 480 909 q 252 892 313 909 q 113 825 192 876 l 56 903 q 209 980 124 954 q 386 1006 294 1006 q 625 939 538 1006 q 711 760 711 872 z "
	},
	"И": {
		ha: 1059,
		x_min: 138,
		x_max: 924,
		o: "m 138 991 l 245 991 l 245 440 q 236 164 245 288 l 241 164 l 798 991 l 924 991 l 924 0 l 815 0 l 815 546 q 827 832 815 682 l 821 832 l 264 0 l 138 0 l 138 991 z "
	},
	"Й": {
		ha: 1059,
		x_min: 138,
		x_max: 924,
		o: "m 138 991 l 245 991 l 245 440 q 236 164 245 288 l 241 164 l 798 991 l 924 991 l 924 0 l 815 0 l 815 546 q 827 832 815 682 l 821 832 l 264 0 l 138 0 l 138 991 m 549 1071 q 366 1119 423 1071 q 301 1279 308 1168 l 407 1279 q 444 1181 413 1208 q 551 1153 475 1153 q 655 1183 621 1153 q 696 1279 688 1213 l 802 1279 q 734 1121 794 1172 q 549 1071 673 1071 z "
	},
	"К": {
		ha: 850,
		x_min: 136,
		x_max: 850,
		o: "m 850 0 l 710 0 l 252 503 l 252 0 l 136 0 l 136 991 l 252 991 l 252 511 l 699 991 l 831 991 l 392 510 l 850 0 z "
	},
	"Л": {
		ha: 978,
		x_min: 0,
		x_max: 842,
		o: "m 842 0 l 726 0 l 726 889 l 404 889 l 383 726 q 310 251 342 401 q 227 42 279 101 q 90 -16 176 -17 q 0 1 40 -16 l 0 98 q 77 81 37 81 q 139 110 113 81 q 187 213 166 140 q 239 489 208 286 q 309 991 271 693 l 842 991 l 842 0 z "
	},
	"М": {
		ha: 1254,
		x_min: 136,
		x_max: 1118,
		o: "m 575 0 l 239 879 l 233 879 q 243 631 243 774 l 243 0 l 136 0 l 136 991 l 310 991 l 624 174 l 629 174 l 946 991 l 1118 991 l 1118 0 l 1003 0 l 1003 639 q 1013 878 1003 749 l 1007 878 l 668 0 l 575 0 z "
	},
	"Н": {
		ha: 1025,
		x_min: 136,
		x_max: 889,
		o: "m 889 0 l 774 0 l 774 467 l 252 467 l 252 0 l 136 0 l 136 991 l 252 991 l 252 570 l 774 570 l 774 991 l 889 991 l 889 0 z "
	},
	"О": {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 207 497 q 293 192 207 296 q 542 87 378 87 q 791 191 707 87 q 875 497 875 295 q 791 801 875 697 q 543 904 707 904 q 293 800 378 904 q 207 497 207 696 z "
	},
	"П": {
		ha: 1013,
		x_min: 136,
		x_max: 876,
		o: "m 876 0 l 761 0 l 761 889 l 252 889 l 252 0 l 136 0 l 136 991 l 876 991 l 876 0 z "
	},
	"Р": {
		ha: 836,
		x_min: 136,
		x_max: 765,
		o: "m 765 703 q 662 471 765 552 q 368 390 559 390 l 252 390 l 252 0 l 136 0 l 136 991 l 393 991 q 765 703 765 991 m 252 489 l 355 489 q 577 538 509 489 q 646 697 646 588 q 581 844 646 795 q 380 892 517 892 l 252 892 l 252 489 z "
	},
	"С": {
		ha: 876,
		x_min: 85,
		x_max: 835,
		o: "m 561 903 q 303 794 397 903 q 208 496 208 685 q 299 195 208 301 q 559 89 391 89 q 796 126 663 89 l 796 25 q 542 -14 693 -14 q 204 119 323 -14 q 85 497 85 252 q 142 766 85 650 q 308 943 199 881 q 562 1006 416 1006 q 835 949 718 1006 l 786 850 q 561 903 673 903 z "
	},
	"Т": {
		ha: 768,
		x_min: 12,
		x_max: 755,
		o: "m 441 0 l 326 0 l 326 889 l 12 889 l 12 991 l 755 991 l 755 889 l 441 889 l 441 0 z "
	},
	"У": {
		ha: 863,
		x_min: 18,
		x_max: 863,
		o: "m 199 -14 q 66 7 123 -14 l 66 119 q 195 90 130 90 q 314 124 269 90 q 404 253 359 159 l 18 991 l 146 991 l 439 428 q 465 363 456 395 l 471 363 q 497 435 490 420 l 741 991 l 863 991 l 546 282 q 443 98 489 155 q 340 13 397 40 q 199 -14 283 -14 z "
	},
	"Ф": {
		ha: 1108,
		x_min: 72,
		x_max: 1036,
		o: "m 496 1006 l 612 1006 l 612 884 l 660 884 q 861 838 776 884 q 991 707 946 791 q 1036 515 1036 623 q 986 315 1036 403 q 847 183 935 227 q 640 139 760 139 l 612 139 l 612 -14 l 496 -14 l 496 139 l 465 139 q 258 184 346 139 q 121 317 171 229 q 72 514 72 405 q 118 705 72 621 q 249 837 163 789 q 450 884 334 884 l 496 884 l 496 1006 m 612 236 l 629 236 q 841 310 766 236 q 917 517 917 385 q 847 713 917 642 q 652 785 777 785 l 612 785 l 612 236 m 496 785 l 457 785 q 263 713 334 785 q 192 517 192 641 q 267 310 192 383 q 479 236 342 236 l 496 236 l 496 785 z "
	},
	"Х": {
		ha: 802,
		x_min: 5,
		x_max: 796,
		o: "m 796 0 l 665 0 l 399 436 l 127 0 l 5 0 l 335 518 l 28 991 l 155 991 l 401 599 l 650 991 l 772 991 l 465 522 l 796 0 z "
	},
	"Ц": {
		ha: 1023,
		x_min: 136,
		x_max: 993,
		o: "m 876 104 l 993 104 l 993 -258 l 884 -258 l 884 0 l 136 0 l 136 991 l 252 991 l 252 104 l 761 104 l 761 991 l 876 991 l 876 104 z "
	},
	"Ч": {
		ha: 965,
		x_min: 115,
		x_max: 829,
		o: "m 829 0 l 714 0 l 714 410 q 546 360 613 374 q 407 347 479 347 q 191 412 267 347 q 115 597 115 476 l 115 991 l 231 991 l 231 611 q 274 490 231 529 q 414 450 317 450 q 539 460 479 450 q 714 507 600 470 l 714 991 l 829 991 l 829 0 z "
	},
	"Ш": {
		ha: 1434,
		x_min: 136,
		x_max: 1297,
		o: "m 1297 0 l 136 0 l 136 991 l 252 991 l 252 104 l 659 104 l 659 991 l 774 991 l 774 104 l 1181 104 l 1181 991 l 1297 991 l 1297 0 z "
	},
	"Щ": {
		ha: 1435,
		x_min: 136,
		x_max: 1392,
		o: "m 136 0 l 136 991 l 252 991 l 252 104 l 647 104 l 647 991 l 764 991 l 764 104 l 1160 104 l 1160 991 l 1275 991 l 1275 104 l 1392 104 l 1392 -258 l 1282 -258 l 1282 0 l 136 0 z "
	},
	"Ъ": {
		ha: 956,
		x_min: 12,
		x_max: 884,
		o: "m 884 289 q 796 75 884 149 q 539 0 708 0 l 240 0 l 240 889 l 12 889 l 12 991 l 355 991 l 355 568 l 521 568 q 791 499 698 568 q 884 289 884 429 m 355 100 l 526 100 q 706 147 649 100 q 764 289 764 194 q 704 424 764 381 q 507 467 645 467 l 355 467 l 355 100 z "
	},
	"Ы": {
		ha: 1185,
		x_min: 136,
		x_max: 1048,
		o: "m 136 0 l 136 991 l 252 991 l 252 568 l 414 568 q 684 499 591 568 q 777 289 777 429 q 689 75 777 149 q 432 0 601 0 l 136 0 m 252 98 l 419 98 q 600 146 542 98 q 657 289 657 194 q 596 424 657 382 q 400 465 536 465 l 252 465 l 252 98 m 1048 0 l 933 0 l 933 991 l 1048 991 l 1048 0 z "
	},
	"Ь": {
		ha: 893,
		x_min: 136,
		x_max: 821,
		o: "m 821 289 q 729 76 821 151 q 467 0 637 0 l 136 0 l 136 991 l 252 991 l 252 568 l 449 568 q 725 497 630 568 q 821 289 821 425 m 252 100 l 454 100 q 701 289 701 100 q 638 424 701 380 q 435 467 574 467 l 252 467 l 252 100 z "
	},
	"Э": {
		ha: 875,
		x_min: 41,
		x_max: 787,
		o: "m 317 903 q 90 851 200 903 l 41 949 q 318 1006 158 1006 q 571 946 465 1006 q 732 776 677 886 q 787 515 787 665 q 664 123 787 260 q 309 -14 541 -14 q 49 25 155 -14 l 49 126 q 163 100 105 112 q 289 89 222 89 q 566 187 473 89 q 665 469 660 285 l 178 469 l 178 572 l 664 572 q 557 815 649 728 q 317 903 465 903 z "
	},
	"Ю": {
		ha: 1458,
		x_min: 136,
		x_max: 1372,
		o: "m 1372 497 q 1256 122 1372 257 q 935 -14 1141 -14 q 615 111 730 -14 q 492 467 499 235 l 252 467 l 252 0 l 136 0 l 136 991 l 252 991 l 252 570 l 493 570 q 623 894 509 781 q 932 1007 737 1007 q 1256 871 1140 1007 q 1372 497 1372 735 m 612 497 q 693 192 612 296 q 930 87 774 87 q 1170 191 1089 87 q 1250 497 1250 295 q 1170 802 1250 699 q 933 904 1090 904 q 694 801 775 904 q 612 497 612 697 z "
	},
	"Я": {
		ha: 884,
		x_min: 35,
		x_max: 747,
		o: "m 431 414 l 171 0 l 35 0 l 313 433 q 154 538 203 468 q 104 715 104 608 q 196 920 104 848 q 475 991 288 991 l 747 991 l 747 0 l 632 0 l 632 414 l 431 414 m 632 892 l 478 892 q 290 849 354 892 q 225 710 225 806 q 482 513 225 513 l 632 513 l 632 892 z "
	},
	"а": {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 z "
	},
	"б": {
		ha: 828,
		x_min: 81,
		x_max: 751,
		o: "m 81 446 q 153 826 81 690 q 380 997 224 963 q 722 1064 574 1038 l 743 965 q 407 903 508 926 q 259 797 309 880 q 205 570 210 713 l 214 570 q 322 659 256 627 q 461 692 389 692 q 675 603 598 692 q 751 357 751 514 q 663 84 751 182 q 415 -14 574 -14 q 170 106 258 -14 q 81 446 81 227 m 425 81 q 632 345 632 81 q 444 596 632 596 q 345 575 393 596 q 257 518 297 553 q 196 446 218 483 q 252 174 196 267 q 425 81 309 81 z "
	},
	"в": {
		ha: 790,
		x_min: 119,
		x_max: 746,
		o: "m 722 557 q 681 447 722 484 q 564 396 639 409 l 564 390 q 702 334 659 377 q 746 216 746 292 q 670 56 746 113 q 447 0 593 0 l 119 0 l 119 743 l 446 743 q 722 557 722 743 m 633 224 q 588 312 633 284 q 436 339 542 339 l 232 339 l 232 94 l 439 94 q 633 224 633 94 m 612 546 q 571 623 612 600 q 444 646 529 646 l 232 646 l 232 436 l 422 436 q 570 462 527 436 q 612 546 612 488 z "
	},
	"г": {
		ha: 595,
		x_min: 119,
		x_max: 567,
		o: "m 567 647 l 232 647 l 232 0 l 119 0 l 119 743 l 567 743 l 567 647 z "
	},
	"д": {
		ha: 794,
		x_min: 28,
		x_max: 765,
		o: "m 765 -257 l 656 -257 l 656 0 l 136 0 l 136 -257 l 28 -257 l 28 97 l 86 97 q 229 387 177 220 q 282 743 280 553 l 659 743 l 659 97 l 765 97 l 765 -257 m 550 97 l 550 654 l 383 654 q 325 350 374 504 q 203 97 276 195 l 550 97 z "
	},
	"е": {
		ha: 779,
		x_min: 78,
		x_max: 707,
		o: "m 433 -14 q 173 87 269 -14 q 78 366 78 187 q 166 651 78 545 q 404 757 255 757 q 625 665 544 757 q 707 422 707 573 l 707 351 l 195 351 q 261 153 198 220 q 437 85 323 85 q 675 135 557 85 l 675 35 q 562 -2 615 9 q 433 -14 509 -14 m 403 663 q 260 604 313 663 q 197 443 207 546 l 586 443 q 538 606 586 549 q 403 663 491 663 z "
	},
	"ж": {
		ha: 1022,
		x_min: 3,
		x_max: 1019,
		o: "m 458 742 l 562 742 l 562 382 l 869 742 l 993 742 l 682 382 l 1019 0 l 889 0 l 562 376 l 562 0 l 458 0 l 458 376 l 132 0 l 3 0 l 339 382 l 29 742 l 153 742 l 458 382 l 458 742 z "
	},
	"з": {
		ha: 671,
		x_min: 46,
		x_max: 607,
		o: "m 261 435 q 472 551 472 435 q 301 661 472 661 q 206 652 249 661 q 94 614 163 642 l 54 705 q 306 757 170 757 q 508 704 434 757 q 582 561 582 652 q 443 397 582 436 l 443 392 q 568 320 528 367 q 607 204 607 274 q 524 44 607 101 q 294 -14 441 -14 q 46 35 134 -14 l 46 138 q 297 79 170 79 q 445 113 395 79 q 494 210 494 146 q 283 339 494 339 l 180 339 l 180 435 l 261 435 z "
	},
	"и": {
		ha: 880,
		x_min: 119,
		x_max: 761,
		o: "m 225 743 l 225 272 l 220 149 l 218 110 l 621 743 l 761 743 l 761 0 l 656 0 l 656 454 l 658 544 l 661 632 l 260 0 l 119 0 l 119 743 l 225 743 z "
	},
	"й": {
		ha: 880,
		x_min: 119,
		x_max: 761,
		o: "m 225 743 l 225 272 l 220 149 l 218 110 l 621 743 l 761 743 l 761 0 l 656 0 l 656 454 l 658 544 l 661 632 l 260 0 l 119 0 l 119 743 l 225 743 m 437 842 q 254 890 312 842 q 190 1050 197 939 l 296 1050 q 333 951 302 979 q 440 924 363 924 q 544 954 510 924 q 585 1050 577 983 l 691 1050 q 623 892 683 943 q 437 842 562 842 z "
	},
	"к": {
		ha: 721,
		x_min: 119,
		x_max: 703,
		o: "m 553 743 l 676 743 l 355 385 l 703 0 l 571 0 l 232 376 l 232 0 l 119 0 l 119 743 l 232 743 l 232 382 l 553 743 z "
	},
	"л": {
		ha: 793,
		x_min: 11,
		x_max: 673,
		o: "m 673 0 l 559 0 l 559 647 l 336 647 q 286 264 318 406 q 201 56 253 122 q 69 -9 149 -9 q 11 -1 33 -9 l 11 82 q 45 78 26 78 q 169 243 123 78 q 239 743 215 408 l 673 743 l 673 0 z "
	},
	"м": {
		ha: 1021,
		x_min: 119,
		x_max: 900,
		o: "m 505 109 l 526 172 l 555 252 l 757 742 l 900 742 l 900 0 l 800 0 l 800 614 l 787 574 l 747 471 l 555 0 l 461 0 l 269 472 q 220 614 233 563 l 220 0 l 119 0 l 119 742 l 257 742 l 452 271 q 505 109 481 196 z "
	},
	"н": {
		ha: 880,
		x_min: 119,
		x_max: 761,
		o: "m 232 743 l 232 432 l 648 432 l 648 743 l 761 743 l 761 0 l 648 0 l 648 335 l 232 335 l 232 0 l 119 0 l 119 743 l 232 743 z "
	},
	"о": {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 195 372 q 252 155 195 230 q 419 81 309 81 q 587 155 530 81 q 644 372 644 229 q 587 588 644 514 q 418 661 530 661 q 251 589 307 661 q 195 372 195 516 z "
	},
	"п": {
		ha: 863,
		x_min: 119,
		x_max: 743,
		o: "m 232 0 l 119 0 l 119 743 l 743 743 l 743 0 l 629 0 l 629 646 l 232 646 l 232 0 z "
	},
	"р": {
		ha: 851,
		x_min: 119,
		x_max: 774,
		o: "m 465 -14 q 333 13 393 -14 q 232 96 273 40 l 224 96 q 232 -28 232 31 l 232 -334 l 119 -334 l 119 743 l 211 743 l 227 642 l 232 642 q 333 730 275 703 q 465 757 391 757 q 693 656 613 757 q 774 372 774 555 q 692 88 774 189 q 465 -14 610 -14 m 449 661 q 284 598 335 661 q 232 397 233 535 l 232 372 q 284 148 232 216 q 452 81 336 81 q 603 159 548 81 q 657 374 657 237 q 603 587 657 513 q 449 661 548 661 z "
	},
	"с": {
		ha: 661,
		x_min: 78,
		x_max: 615,
		o: "m 416 -14 q 166 86 255 -14 q 78 367 78 185 q 168 655 78 553 q 424 757 258 757 q 531 745 477 757 q 615 718 585 734 l 581 623 q 499 647 543 637 q 421 657 455 657 q 195 368 195 657 q 250 158 195 231 q 414 85 305 85 q 604 125 507 85 l 604 25 q 416 -14 530 -14 z "
	},
	"т": {
		ha: 648,
		x_min: 28,
		x_max: 621,
		o: "m 621 647 l 379 647 l 379 0 l 267 0 l 267 647 l 28 647 l 28 743 l 621 743 l 621 647 z "
	},
	"у": {
		ha: 700,
		x_min: 1,
		x_max: 699,
		o: "m 1 743 l 122 743 l 285 319 q 351 110 338 174 l 357 110 q 394 228 366 144 q 578 743 422 312 l 699 743 l 379 -103 q 268 -281 332 -229 q 113 -334 205 -334 q 11 -322 61 -334 l 11 -232 q 94 -240 48 -240 q 260 -110 210 -240 l 301 -4 l 1 743 z "
	},
	"ф": {
		ha: 993,
		x_min: 77,
		x_max: 916,
		o: "m 916 372 q 820 103 916 203 q 551 -11 724 3 l 551 -334 l 440 -334 l 440 -11 q 174 104 272 2 q 77 372 77 207 q 174 641 77 541 q 444 754 271 741 l 444 1055 l 551 1055 l 551 754 q 819 637 722 739 q 916 372 916 535 m 195 372 q 254 168 195 242 q 444 82 314 94 l 444 660 q 257 578 319 648 q 195 372 195 508 m 799 372 q 551 659 799 632 l 551 82 q 740 169 680 95 q 799 372 799 243 z "
	},
	"х": {
		ha: 728,
		x_min: 26,
		x_max: 700,
		o: "m 298 380 l 40 743 l 168 743 l 364 458 l 559 743 l 686 743 l 428 380 l 700 0 l 572 0 l 364 301 l 154 0 l 26 0 l 298 380 z "
	},
	"ц": {
		ha: 869,
		x_min: 119,
		x_max: 844,
		o: "m 844 -257 l 732 -257 l 732 0 l 119 0 l 119 743 l 232 743 l 232 97 l 627 97 l 627 743 l 739 743 l 739 97 l 844 97 l 844 -257 z "
	},
	"ч": {
		ha: 844,
		x_min: 106,
		x_max: 725,
		o: "m 218 743 l 218 472 q 367 342 218 342 q 485 361 429 342 q 612 425 541 380 l 612 743 l 725 743 l 725 0 l 612 0 l 612 336 q 480 268 541 288 q 343 248 420 248 q 169 306 232 248 q 106 464 106 364 l 106 743 l 218 743 z "
	},
	"ш": {
		ha: 1236,
		x_min: 119,
		x_max: 1117,
		o: "m 673 97 l 1003 97 l 1003 743 l 1117 743 l 1117 0 l 119 0 l 119 743 l 232 743 l 232 97 l 561 97 l 561 743 l 673 743 l 673 97 z "
	},
	"щ": {
		ha: 1246,
		x_min: 119,
		x_max: 1222,
		o: "m 673 97 l 1003 97 l 1003 742 l 1116 742 l 1116 97 l 1222 97 l 1222 -256 l 1108 -256 l 1108 0 l 119 0 l 119 742 l 232 742 l 232 97 l 561 97 l 561 742 l 673 742 l 673 97 z "
	},
	"ъ": {
		ha: 965,
		x_min: 28,
		x_max: 888,
		o: "m 378 436 l 590 436 q 815 384 742 436 q 888 227 888 332 q 812 57 888 114 q 587 0 736 0 l 265 0 l 265 647 l 28 647 l 28 743 l 378 743 l 378 436 m 777 220 q 734 311 777 283 q 586 339 692 339 l 378 339 l 378 94 l 590 94 q 777 220 777 94 z "
	},
	"ы": {
		ha: 1069,
		x_min: 119,
		x_max: 950,
		o: "m 232 436 l 435 436 q 645 384 576 436 q 713 227 713 331 q 641 57 713 115 q 428 0 568 0 l 119 0 l 119 743 l 232 743 l 232 436 m 950 0 l 838 0 l 838 743 l 950 743 l 950 0 m 232 339 l 232 94 l 422 94 q 602 220 602 94 q 560 311 602 282 q 419 339 519 339 l 232 339 z "
	},
	"ь": {
		ha: 822,
		x_min: 119,
		x_max: 746,
		o: "m 232 436 l 461 436 q 746 227 746 436 q 672 58 746 117 q 454 0 597 0 l 119 0 l 119 743 l 232 743 l 232 436 m 232 339 l 232 94 l 449 94 q 586 125 538 94 q 633 220 633 157 q 590 310 633 281 q 446 339 546 339 l 232 339 z "
	},
	"э": {
		ha: 684,
		x_min: 39,
		x_max: 606,
		o: "m 232 -14 q 39 25 119 -14 l 39 125 q 127 97 79 109 q 236 85 174 85 q 418 148 354 85 q 489 336 482 211 l 113 336 l 113 432 l 488 432 q 419 603 477 549 q 253 657 362 657 q 81 621 183 657 l 49 715 q 142 745 86 733 q 252 757 197 757 q 515 657 425 757 q 606 371 606 558 q 508 87 606 189 q 232 -14 410 -14 z "
	},
	"ю": {
		ha: 1154,
		x_min: 119,
		x_max: 1076,
		o: "m 1076 372 q 989 88 1076 190 q 750 -14 902 -14 q 521 77 606 -14 q 427 335 436 168 l 232 335 l 232 0 l 119 0 l 119 743 l 232 743 l 232 432 l 428 432 q 527 672 441 587 q 753 757 612 757 q 989 653 902 757 q 1076 372 1076 550 m 543 372 q 593 155 543 229 q 751 81 642 81 q 909 153 859 81 q 960 372 960 225 q 910 588 960 515 q 751 661 861 661 q 593 588 642 661 q 543 372 543 515 z "
	},
	"я": {
		ha: 771,
		x_min: 25,
		x_max: 652,
		o: "m 157 0 l 25 0 l 239 314 q 107 388 153 333 q 61 525 61 442 q 130 685 61 627 q 321 743 198 743 l 652 743 l 652 0 l 539 0 l 539 297 l 351 297 l 157 0 m 171 524 q 358 395 171 395 l 539 395 l 539 646 l 335 646 q 171 524 171 646 z "
	},
	"ё": {
		ha: 779,
		x_min: 78,
		x_max: 707,
		o: "m 433 -14 q 173 87 269 -14 q 78 366 78 187 q 166 651 78 545 q 404 757 255 757 q 625 665 544 757 q 707 422 707 573 l 707 351 l 195 351 q 261 153 198 220 q 437 85 323 85 q 675 135 557 85 l 675 35 q 562 -2 615 9 q 433 -14 509 -14 m 403 663 q 260 604 313 663 q 197 443 207 546 l 586 443 q 538 606 586 549 q 403 663 491 663 m 215 945 q 233 996 215 980 q 276 1011 251 1011 q 320 996 302 1011 q 339 945 339 980 q 320 894 339 911 q 276 878 302 878 q 233 894 251 878 q 215 945 215 911 m 473 945 q 491 996 473 980 q 534 1011 509 1011 q 578 996 559 1011 q 597 945 597 980 q 578 894 597 911 q 534 878 559 878 q 491 894 509 878 q 473 945 473 911 z "
	},
	"ђ": {
		ha: 852,
		x_min: 14,
		x_max: 741,
		o: "m 553 -334 q 464 -317 499 -334 l 464 -224 q 541 -237 503 -237 q 628 -122 628 -237 l 628 454 q 587 590 628 545 q 457 635 545 635 q 287 571 340 635 q 233 364 233 507 l 233 0 l 119 0 l 119 843 l 14 843 l 14 929 l 119 929 l 119 1055 l 232 1055 l 232 929 l 504 929 l 504 843 l 232 843 l 232 710 q 227 615 232 653 l 233 615 q 328 700 267 669 q 468 730 389 730 q 672 666 604 730 q 741 458 741 601 l 741 -113 q 694 -276 741 -218 q 553 -334 648 -334 z "
	},
	"ѓ": {
		ha: 595,
		x_min: 119,
		x_max: 567,
		o: "m 567 647 l 232 647 l 232 0 l 119 0 l 119 743 l 567 743 l 567 647 m 256 859 q 327 960 289 901 q 386 1064 364 1020 l 523 1064 l 523 1050 q 434 941 493 1006 q 332 842 375 877 l 256 842 l 256 859 z "
	},
	"є": {
		ha: 684,
		x_min: 78,
		x_max: 636,
		o: "m 429 -14 q 170 85 261 -14 q 78 368 78 184 q 171 656 78 555 q 435 757 264 757 q 544 746 490 757 q 636 718 597 735 l 603 623 q 432 657 506 657 q 263 602 321 657 q 195 432 205 547 l 570 432 l 570 336 l 193 336 q 257 146 199 208 q 428 85 315 85 q 625 125 523 85 l 625 25 q 429 -14 547 -14 z "
	},
	"ѕ": {
		ha: 663,
		x_min: 72,
		x_max: 599,
		o: "m 599 203 q 522 43 599 99 q 304 -14 444 -14 q 74 33 157 -14 l 74 138 q 189 95 127 111 q 307 79 250 79 q 443 107 395 79 q 490 193 490 136 q 453 268 490 237 q 306 340 415 298 q 158 408 202 379 q 93 473 115 437 q 72 561 72 510 q 146 704 72 652 q 349 757 220 757 q 583 708 469 757 l 543 616 q 340 663 431 663 q 220 637 260 663 q 179 568 179 612 q 194 517 179 538 q 243 477 210 496 q 374 422 277 458 q 552 326 506 374 q 599 203 599 277 z "
	},
	"і": {
		ha: 351,
		x_min: 110,
		x_max: 243,
		o: "m 232 0 l 119 0 l 119 743 l 232 743 l 232 0 m 110 945 q 129 1001 110 983 q 176 1019 148 1019 q 223 1001 203 1019 q 243 945 243 983 q 223 888 243 907 q 176 869 203 869 q 129 888 148 869 q 110 945 110 907 z "
	},
	"ї": {
		ha: 351,
		x_min: -14,
		x_max: 368,
		o: "m 232 0 l 119 0 l 119 743 l 232 743 l 232 0 m -14 945 q 4 996 -14 980 q 47 1011 22 1011 q 92 996 73 1011 q 111 945 111 980 q 92 894 111 911 q 47 878 73 878 q 4 894 22 878 q -14 945 -14 911 m 245 945 q 263 996 245 980 q 306 1011 281 1011 q 350 996 331 1011 q 368 945 368 980 q 350 894 368 911 q 306 878 331 878 q 263 894 281 878 q 245 945 245 911 z "
	},
	"ј": {
		ha: 351,
		x_min: -75,
		x_max: 243,
		o: "m 29 -334 q -75 -317 -35 -334 l -75 -225 q 17 -239 -28 -239 q 95 -210 70 -239 q 119 -122 119 -181 l 119 743 l 232 743 l 232 -114 q 29 -334 232 -334 m 110 945 q 129 1001 110 983 q 176 1019 148 1019 q 223 1001 203 1019 q 243 945 243 983 q 223 888 243 907 q 176 869 203 869 q 129 888 148 869 q 110 945 110 907 z "
	},
	"љ": {
		ha: 1162,
		x_min: 11,
		x_max: 1086,
		o: "m 640 436 l 806 436 q 1018 384 949 436 q 1086 227 1086 331 q 790 0 1086 0 l 511 0 l 511 647 l 336 647 q 248 143 309 296 q 71 -9 186 -9 q 11 -1 33 -9 l 11 82 q 45 78 26 78 q 169 243 123 78 q 239 743 215 408 l 640 743 l 640 436 m 974 220 q 931 310 974 281 q 782 339 889 339 l 625 339 l 625 94 l 786 94 q 974 220 974 94 z "
	},
	"њ": {
		ha: 1231,
		x_min: 119,
		x_max: 1153,
		o: "m 694 742 l 694 435 l 868 435 q 1084 383 1015 435 q 1153 227 1153 331 q 859 0 1153 0 l 576 0 l 576 335 l 236 335 l 236 0 l 119 0 l 119 742 l 236 742 l 236 432 l 579 432 l 579 742 l 694 742 m 692 339 l 692 94 l 854 94 q 1042 220 1042 94 q 998 311 1042 282 q 851 339 955 339 l 692 339 z "
	},
	"ћ": {
		ha: 852,
		x_min: 14,
		x_max: 741,
		o: "m 628 0 l 628 454 q 587 590 628 545 q 457 635 545 635 q 286 572 339 635 q 232 364 232 509 l 232 0 l 119 0 l 119 843 l 14 843 l 14 929 l 119 929 l 119 1055 l 232 1055 l 232 929 l 536 929 l 536 843 l 232 843 l 232 710 q 227 615 232 653 l 233 615 q 328 700 267 669 q 468 730 389 730 q 672 666 604 730 q 741 458 741 601 l 741 0 l 628 0 z "
	},
	"ќ": {
		ha: 721,
		x_min: 119,
		x_max: 703,
		o: "m 553 743 l 676 743 l 355 385 l 703 0 l 571 0 l 232 376 l 232 0 l 119 0 l 119 743 l 232 743 l 232 382 l 553 743 m 301 859 q 371 960 334 901 q 431 1064 409 1020 l 568 1064 l 568 1050 q 479 941 538 1006 q 376 842 420 877 l 301 842 l 301 859 z "
	},
	"ў": {
		ha: 700,
		x_min: 1,
		x_max: 699,
		o: "m 1 743 l 122 743 l 285 319 q 351 110 338 174 l 357 110 q 394 228 366 144 q 578 743 422 312 l 699 743 l 379 -103 q 268 -281 332 -229 q 113 -334 205 -334 q 11 -322 61 -334 l 11 -232 q 94 -240 48 -240 q 260 -110 210 -240 l 301 -4 l 1 743 m 347 842 q 163 890 221 842 q 99 1050 106 939 l 205 1050 q 242 951 211 979 q 349 924 273 924 q 453 954 419 924 q 494 1050 486 983 l 600 1050 q 532 892 592 943 q 347 842 471 842 z "
	},
	"џ": {
		ha: 863,
		x_min: 119,
		x_max: 742,
		o: "m 379 0 l 119 0 l 119 742 l 232 742 l 232 97 l 629 97 l 629 742 l 742 742 l 742 0 l 492 0 l 492 -256 l 379 -256 l 379 0 z "
	},
	"Ґ": {
		ha: 732,
		x_min: 136,
		x_max: 700,
		o: "m 590 991 l 590 1196 l 700 1196 l 700 888 l 252 888 l 252 0 l 136 0 l 136 991 l 590 991 z "
	},
	"ґ": {
		ha: 595,
		x_min: 119,
		x_max: 567,
		o: "m 567 656 l 232 656 l 232 0 l 119 0 l 119 743 l 454 743 l 454 961 l 567 961 l 567 656 z "
	},
	"Ẁ": {
		ha: 1286,
		x_min: 18,
		x_max: 1267,
		o: "m 1002 0 l 888 0 l 688 664 q 656 775 673 708 q 637 856 638 842 q 590 660 623 766 l 396 0 l 282 0 l 18 991 l 140 991 l 297 379 q 345 146 330 250 q 399 389 363 270 l 576 991 l 699 991 l 885 383 q 940 146 918 278 q 989 380 953 242 l 1145 991 l 1267 991 l 1002 0 m 722 1071 l 648 1071 q 543 1171 604 1106 q 456 1279 483 1236 l 456 1293 l 593 1293 q 654 1185 615 1246 q 722 1088 692 1124 l 722 1071 z "
	},
	"ẁ": {
		ha: 1080,
		x_min: 16,
		x_max: 1065,
		o: "m 726 0 l 590 436 q 542 618 577 476 l 536 618 q 489 435 509 499 l 349 0 l 218 0 l 16 743 l 134 743 q 243 317 205 463 q 286 119 281 170 l 292 119 q 316 219 299 158 q 345 317 332 281 l 481 743 l 603 743 l 736 317 q 787 121 774 200 l 793 121 q 807 196 795 145 q 949 743 819 247 l 1065 743 l 860 0 l 726 0 m 611 842 l 536 842 q 432 942 492 877 q 345 1050 372 1007 l 345 1064 l 482 1064 q 543 956 504 1017 q 611 859 581 895 l 611 842 z "
	},
	"Ẃ": {
		ha: 1286,
		x_min: 18,
		x_max: 1267,
		o: "m 1002 0 l 888 0 l 688 664 q 656 775 673 708 q 637 856 638 842 q 590 660 623 766 l 396 0 l 282 0 l 18 991 l 140 991 l 297 379 q 345 146 330 250 q 399 389 363 270 l 576 991 l 699 991 l 885 383 q 940 146 918 278 q 989 380 953 242 l 1145 991 l 1267 991 l 1002 0 m 559 1088 q 630 1190 592 1130 q 689 1293 667 1249 l 826 1293 l 826 1279 q 737 1171 796 1235 q 635 1071 678 1106 l 559 1071 l 559 1088 z "
	},
	"ẃ": {
		ha: 1080,
		x_min: 16,
		x_max: 1065,
		o: "m 726 0 l 590 436 q 542 618 577 476 l 536 618 q 489 435 509 499 l 349 0 l 218 0 l 16 743 l 134 743 q 243 317 205 463 q 286 119 281 170 l 292 119 q 316 219 299 158 q 345 317 332 281 l 481 743 l 603 743 l 736 317 q 787 121 774 200 l 793 121 q 807 196 795 145 q 949 743 819 247 l 1065 743 l 860 0 l 726 0 m 458 859 q 529 960 491 901 q 588 1064 566 1020 l 725 1064 l 725 1050 q 636 941 695 1006 q 534 842 577 877 l 458 842 l 458 859 z "
	},
	"Ẅ": {
		ha: 1286,
		x_min: 18,
		x_max: 1267,
		o: "m 1002 0 l 888 0 l 688 664 q 656 775 673 708 q 637 856 638 842 q 590 660 623 766 l 396 0 l 282 0 l 18 991 l 140 991 l 297 379 q 345 146 330 250 q 399 389 363 270 l 576 991 l 699 991 l 885 383 q 940 146 918 278 q 989 380 953 242 l 1145 991 l 1267 991 l 1002 0 m 451 1174 q 469 1225 451 1209 q 512 1240 487 1240 q 556 1225 538 1240 q 575 1174 575 1209 q 556 1123 575 1140 q 512 1107 538 1107 q 469 1123 487 1107 q 451 1174 451 1140 m 709 1174 q 727 1225 709 1209 q 770 1240 745 1240 q 814 1225 795 1240 q 833 1174 833 1209 q 814 1123 833 1140 q 770 1107 795 1107 q 727 1123 745 1107 q 709 1174 709 1140 z "
	},
	"ẅ": {
		ha: 1080,
		x_min: 16,
		x_max: 1065,
		o: "m 726 0 l 590 436 q 542 618 577 476 l 536 618 q 489 435 509 499 l 349 0 l 218 0 l 16 743 l 134 743 q 243 317 205 463 q 286 119 281 170 l 292 119 q 316 219 299 158 q 345 317 332 281 l 481 743 l 603 743 l 736 317 q 787 121 774 200 l 793 121 q 807 196 795 145 q 949 743 819 247 l 1065 743 l 860 0 l 726 0 m 350 945 q 368 996 350 980 q 411 1011 386 1011 q 455 996 437 1011 q 474 945 474 980 q 455 894 474 911 q 411 878 437 878 q 368 894 386 878 q 350 945 350 911 m 608 945 q 626 996 608 980 q 669 1011 644 1011 q 713 996 694 1011 q 732 945 732 980 q 713 894 732 911 q 669 878 694 878 q 626 894 644 878 q 608 945 608 911 z "
	},
	"Ỳ": {
		ha: 778,
		x_min: 0,
		x_max: 778,
		o: "m 389 496 l 653 991 l 778 991 l 447 385 l 447 0 l 330 0 l 330 379 l 0 991 l 126 991 l 389 496 m 460 1071 l 385 1071 q 281 1171 341 1106 q 193 1279 220 1236 l 193 1293 l 331 1293 q 391 1185 353 1246 q 460 1088 430 1124 l 460 1071 z "
	},
	"ỳ": {
		ha: 700,
		x_min: 1,
		x_max: 699,
		o: "m 1 743 l 122 743 l 285 319 q 351 110 338 174 l 357 110 q 394 228 366 144 q 578 743 422 312 l 699 743 l 379 -103 q 268 -281 332 -229 q 113 -334 205 -334 q 11 -322 61 -334 l 11 -232 q 94 -240 48 -240 q 260 -110 210 -240 l 301 -4 l 1 743 m 425 842 l 351 842 q 246 942 307 877 q 159 1050 186 1007 l 159 1064 l 296 1064 q 357 956 318 1017 q 425 859 395 895 l 425 842 z "
	},
	"–": {
		ha: 694,
		x_min: 56,
		x_max: 639,
		o: "m 56 321 l 56 424 l 639 424 l 639 321 l 56 321 z "
	},
	"—": {
		ha: 1389,
		x_min: 56,
		x_max: 1333,
		o: "m 56 321 l 56 424 l 1333 424 l 1333 321 l 56 321 z "
	},
	"―": {
		ha: 1389,
		x_min: 56,
		x_max: 1333,
		o: "m 56 321 l 56 424 l 1333 424 l 1333 321 l 56 321 z "
	},
	"‗": {
		ha: 571,
		x_min: -3,
		x_max: 574,
		o: "m 574 -314 l -3 -314 l -3 -220 l 574 -220 l 574 -314 m 574 -125 l -3 -125 l -3 -31 l 574 -31 l 574 -125 z "
	},
	"‘": {
		ha: 236,
		x_min: 17,
		x_max: 220,
		o: "m 25 652 l 17 667 q 65 819 32 728 q 136 991 98 909 l 220 991 q 150 652 175 819 l 25 652 z "
	},
	"’": {
		ha: 236,
		x_min: 17,
		x_max: 220,
		o: "m 210 991 l 220 977 q 169 819 202 909 q 100 652 136 729 l 17 652 q 86 991 64 845 l 210 991 z "
	},
	"‚": {
		ha: 340,
		x_min: 43,
		x_max: 248,
		o: "m 237 161 l 248 146 q 197 -12 230 78 q 127 -179 163 -102 l 43 -179 q 83 -5 61 -109 q 114 161 105 99 l 237 161 z "
	},
	"‛": {
		ha: 236,
		x_min: 17,
		x_max: 221,
		o: "m 151 991 q 221 652 176 821 l 138 652 q 70 812 107 715 q 17 977 33 908 l 26 991 l 151 991 z "
	},
	"“": {
		ha: 486,
		x_min: 17,
		x_max: 469,
		o: "m 275 652 l 265 667 q 386 991 303 812 l 469 991 q 429 815 449 913 q 400 652 409 718 l 275 652 m 25 652 l 17 667 q 65 819 32 728 q 136 991 98 909 l 220 991 q 150 652 175 819 l 25 652 z "
	},
	"”": {
		ha: 486,
		x_min: 17,
		x_max: 469,
		o: "m 210 991 l 220 977 q 169 819 202 909 q 100 652 136 729 l 17 652 q 86 991 64 845 l 210 991 m 460 991 l 469 977 q 420 825 453 915 q 350 652 388 734 l 265 652 q 305 824 283 720 q 336 991 328 928 l 460 991 z "
	},
	"„": {
		ha: 562,
		x_min: 17,
		x_max: 469,
		o: "m 210 161 l 220 146 q 169 -11 202 79 q 100 -178 136 -101 l 17 -178 q 86 161 64 15 l 210 161 m 460 161 l 469 146 q 420 -5 453 85 q 350 -178 388 -96 l 265 -178 q 305 -6 283 -111 q 336 161 328 98 l 460 161 z "
	},
	"†": {
		ha: 697,
		x_min: 83,
		x_max: 614,
		o: "m 614 677 l 375 699 l 408 0 l 275 0 l 309 699 l 83 677 l 83 793 l 309 772 l 275 1055 l 408 1055 l 375 772 l 614 793 l 614 677 z "
	},
	"‡": {
		ha: 708,
		x_min: 83,
		x_max: 625,
		o: "m 386 330 l 625 351 l 625 237 l 386 257 l 419 0 l 285 0 l 318 257 l 83 237 l 83 351 l 318 330 l 289 533 l 318 725 l 83 704 l 83 818 l 318 798 l 285 1055 l 419 1055 l 386 798 l 625 818 l 625 704 l 386 725 l 415 533 l 386 330 z "
	},
	"•": {
		ha: 522,
		x_min: 111,
		x_max: 411,
		o: "m 111 507 q 150 632 111 589 q 261 675 188 675 q 372 633 332 675 q 411 507 411 591 q 372 383 411 427 q 261 339 333 339 q 150 384 189 339 q 111 507 111 428 z "
	},
	"…": {
		ha: 1089,
		x_min: 103,
		x_max: 986,
		o: "m 103 72 q 124 141 103 117 q 183 164 144 164 q 244 141 222 164 q 267 72 267 117 q 244 4 267 28 q 183 -20 222 -20 q 126 2 149 -20 q 103 72 103 23 m 463 72 q 483 141 463 117 q 543 164 504 164 q 604 141 582 164 q 626 72 626 117 q 604 4 626 28 q 543 -20 581 -20 q 485 2 508 -20 q 463 72 463 23 m 823 72 q 843 141 823 117 q 903 164 864 164 q 964 141 942 164 q 986 72 986 117 q 964 4 986 28 q 903 -20 941 -20 q 845 2 868 -20 q 823 72 823 23 z "
	},
	"‰": {
		ha: 1670,
		x_min: 68,
		x_max: 1603,
		o: "m 160 696 q 188 523 160 581 q 279 465 216 465 q 401 696 401 465 q 279 925 401 925 q 188 868 216 925 q 160 696 160 811 m 494 696 q 440 461 494 540 q 279 383 385 383 q 123 464 178 383 q 68 696 68 545 q 279 1006 68 1006 q 438 924 382 1006 q 494 696 494 843 m 913 991 l 363 0 l 262 0 l 812 991 l 913 991 m 1268 298 q 1296 125 1268 182 q 1388 68 1324 68 q 1479 124 1449 68 q 1510 298 1510 181 q 1479 470 1510 414 q 1388 526 1449 526 q 1296 470 1324 526 q 1268 298 1268 414 m 1603 298 q 1548 64 1603 142 q 1388 -14 1493 -14 q 1232 67 1287 -14 q 1177 298 1177 149 q 1388 608 1177 608 q 1546 528 1489 608 q 1603 298 1603 448 m 772 298 q 800 125 772 182 q 890 68 827 68 q 982 124 952 68 q 1013 298 1013 181 q 982 470 1013 414 q 890 526 952 526 q 800 470 827 526 q 772 298 772 414 m 1105 298 q 1050 64 1105 142 q 890 -14 996 -14 q 734 67 789 -14 q 679 298 679 148 q 890 608 679 608 q 1049 528 992 608 q 1105 298 1105 448 z "
	},
	"′": {
		ha: 307,
		x_min: 90,
		x_max: 216,
		o: "m 216 991 l 189 633 l 118 633 l 90 991 l 216 991 z "
	},
	"″": {
		ha: 546,
		x_min: 90,
		x_max: 467,
		o: "m 216 991 l 189 633 l 118 633 l 90 991 l 216 991 m 467 991 l 439 633 l 368 633 l 340 991 l 467 991 z "
	},
	"‹": {
		ha: 422,
		x_min: 56,
		x_max: 368,
		o: "m 56 374 l 288 650 l 368 603 l 172 366 l 368 127 l 288 79 l 56 355 l 56 374 z "
	},
	"›": {
		ha: 422,
		x_min: 54,
		x_max: 367,
		o: "m 367 355 l 134 79 l 54 127 l 249 366 l 54 603 l 134 650 l 367 374 l 367 355 z "
	},
	"‼": {
		ha: 675,
		x_min: 103,
		x_max: 571,
		o: "m 221 273 l 150 273 l 115 991 l 256 991 l 221 273 m 103 72 q 184 164 103 164 q 245 140 224 164 q 267 72 267 117 q 245 4 267 28 q 184 -20 223 -20 q 126 2 149 -20 q 103 72 103 23 m 526 273 l 454 273 l 420 991 l 560 991 l 526 273 m 408 72 q 489 164 408 164 q 550 140 528 164 q 571 72 571 117 q 549 4 571 28 q 489 -20 528 -20 q 431 2 454 -20 q 408 72 408 23 z "
	},
	"⁄": {
		ha: 180,
		x_min: -265,
		x_max: 444,
		o: "m 444 991 l -168 0 l -265 0 l 347 991 l 444 991 z "
	},
	"ⁿ": {
		ha: 547,
		x_min: 74,
		x_max: 479,
		o: "m 399 543 l 399 828 q 372 909 399 885 q 292 933 346 933 q 183 898 214 933 q 153 779 153 862 l 153 543 l 74 543 l 74 994 l 139 994 l 149 935 l 155 935 q 304 1003 206 1003 q 479 833 479 1003 l 479 543 l 399 543 z "
	},
	"₣": {
		ha: 794,
		x_min: 66,
		x_max: 718,
		o: "m 298 268 l 507 268 l 507 180 l 298 180 l 298 0 l 186 0 l 186 180 l 66 180 l 66 268 l 186 268 l 186 991 l 718 991 l 718 889 l 298 889 l 298 526 l 692 526 l 692 424 l 298 424 l 298 268 z "
	},
	"₤": {
		ha: 794,
		x_min: 46,
		x_max: 743,
		o: "m 467 1004 q 710 950 603 1004 l 669 861 q 467 909 566 909 q 341 864 384 909 q 298 730 298 818 l 298 604 l 583 604 l 583 517 l 298 517 l 298 404 l 583 404 l 583 317 l 298 317 l 298 294 q 276 180 298 227 q 204 104 254 134 l 743 104 l 743 0 l 46 0 l 46 96 q 186 293 186 128 l 186 317 l 50 317 l 50 404 l 186 404 l 186 517 l 50 517 l 50 604 l 186 604 l 186 745 q 262 935 186 865 q 467 1004 338 1004 z "
	},
	"₧": {
		ha: 1060,
		x_min: 104,
		x_max: 1010,
		o: "m 921 79 q 973 84 944 79 q 1010 92 1002 89 l 1010 7 q 895 -14 969 -14 q 777 33 821 -14 q 733 172 733 79 l 733 486 l 627 486 l 627 541 l 733 587 l 775 717 l 842 717 l 842 574 l 991 574 l 991 486 l 842 486 l 842 187 q 859 107 842 135 q 921 79 877 79 m 628 703 q 534 469 628 549 q 260 390 440 390 l 216 390 l 216 0 l 104 0 l 104 991 l 282 991 q 542 920 456 991 q 628 703 628 848 m 216 489 l 252 489 q 450 538 387 489 q 513 697 513 587 q 454 844 513 795 q 272 892 396 892 l 216 892 l 216 489 z "
	},
	"€": {
		ha: 819,
		x_min: 43,
		x_max: 787,
		o: "m 539 904 q 269 631 323 904 l 615 631 l 615 543 l 260 543 l 258 505 l 258 461 l 260 431 l 574 431 l 574 343 l 271 343 q 365 154 296 221 q 549 87 433 87 q 758 132 654 87 l 758 31 q 543 -14 659 -14 q 284 78 382 -14 q 155 343 186 169 l 43 343 l 43 431 l 146 431 l 144 459 l 144 489 l 146 543 l 43 543 l 43 631 l 154 631 q 279 907 180 808 q 539 1006 378 1006 q 787 940 675 1006 l 739 846 q 539 904 627 904 z "
	},
	"℅": {
		ha: 1145,
		x_min: 96,
		x_max: 1048,
		o: "m 889 991 l 339 0 l 239 0 l 789 991 l 889 991 m 1048 227 q 991 57 1048 119 q 833 -5 934 -5 q 682 57 739 -5 q 624 227 624 120 q 681 396 624 334 q 838 458 737 458 q 991 395 933 458 q 1048 227 1048 332 m 715 227 q 836 65 715 65 q 956 227 956 65 q 836 386 956 386 q 715 227 715 386 m 332 534 q 157 594 219 534 q 96 764 96 654 q 159 938 96 878 q 339 999 223 999 q 469 975 410 999 l 447 905 q 338 926 392 926 q 186 765 186 926 q 335 606 186 606 q 463 628 401 606 l 463 559 q 332 534 410 534 z "
	},
	"ℓ": {
		ha: 722,
		x_min: 81,
		x_max: 627,
		o: "m 432 75 q 562 220 550 75 l 627 220 q 569 47 621 107 q 421 -14 517 -14 q 265 48 319 -14 q 211 224 211 109 l 211 389 q 81 350 146 365 l 81 427 q 211 467 134 441 l 211 804 q 262 952 211 898 q 404 1006 313 1006 q 529 951 484 1006 q 575 803 575 897 q 505 574 575 677 q 317 421 435 471 l 317 229 q 345 117 317 159 q 432 75 372 75 m 490 799 q 404 930 490 930 q 338 901 359 930 q 317 799 317 872 l 317 503 q 490 799 490 595 z "
	},
	"№": {
		ha: 1415,
		x_min: 136,
		x_max: 1348,
		o: "m 829 0 l 703 0 l 233 832 l 228 832 q 239 564 239 637 l 239 0 l 136 0 l 136 991 l 268 991 l 730 163 l 736 163 q 726 422 726 329 l 726 991 l 829 991 l 829 0 m 1348 421 q 1293 248 1348 311 q 1139 186 1238 186 q 989 249 1044 186 q 935 421 935 313 q 989 593 935 532 q 1143 654 1044 654 q 1293 591 1238 654 q 1348 421 1348 528 m 1023 421 q 1051 304 1023 344 q 1141 264 1078 264 q 1230 303 1203 264 q 1257 421 1257 343 q 1230 537 1257 499 q 1141 575 1203 575 q 1051 537 1079 575 q 1023 421 1023 499 m 965 0 l 965 92 l 1312 92 l 1312 0 l 965 0 z "
	},
	"™": {
		ha: 1078,
		x_min: 25,
		x_max: 958,
		o: "m 250 503 l 167 503 l 167 920 l 25 920 l 25 991 l 393 991 l 393 920 l 250 920 l 250 503 m 657 503 l 521 882 l 515 882 l 519 794 l 519 503 l 439 503 l 439 991 l 566 991 l 699 612 l 836 991 l 958 991 l 958 503 l 872 503 l 872 787 l 876 880 l 871 880 l 728 503 l 657 503 z "
	},
	"Ω": {
		ha: 1086,
		x_min: 54,
		x_max: 1034,
		o: "m 543 904 q 297 817 382 904 q 212 558 212 730 q 271 293 212 406 q 452 90 330 180 l 452 0 l 54 0 l 54 103 l 301 103 q 144 302 199 183 q 90 558 90 420 q 210 888 90 768 q 543 1007 330 1007 q 877 889 757 1007 q 997 559 997 772 q 943 302 997 419 q 787 103 890 184 l 1034 103 l 1034 0 l 636 0 l 636 90 q 818 294 760 181 q 875 558 875 408 q 790 817 875 729 q 543 904 706 904 z "
	},
	"℮": {
		ha: 860,
		x_min: 69,
		x_max: 789,
		o: "m 429 -24 q 241 26 323 -24 q 114 164 159 76 q 69 359 69 253 q 116 562 69 475 q 245 696 163 648 q 429 743 328 743 q 615 695 532 743 q 743 559 697 647 q 789 359 789 471 l 228 359 l 228 118 q 317 58 261 82 q 429 35 374 35 q 580 70 518 35 q 697 193 642 106 l 746 165 q 606 18 680 60 q 429 -24 532 -24 m 632 418 l 632 606 q 543 664 598 642 q 428 686 488 686 q 228 607 311 686 l 228 418 l 632 418 z "
	},
	"⅛": {
		ha: 1083,
		x_min: 48,
		x_max: 1033,
		o: "m 854 991 l 241 0 l 144 0 l 757 991 l 854 991 m 226 991 l 323 991 l 323 397 l 233 397 l 233 790 q 237 913 233 852 q 203 883 222 898 q 94 803 185 867 l 48 868 l 226 991 m 827 604 q 963 564 911 604 q 1014 453 1014 524 q 913 313 1014 365 q 1033 156 1033 263 q 977 33 1033 79 q 827 -14 921 -14 q 674 30 728 -14 q 621 152 621 73 q 645 238 621 199 q 728 309 670 277 q 658 371 677 340 q 639 449 639 403 q 692 561 639 518 q 827 604 745 604 m 709 152 q 738 86 709 111 q 825 62 766 62 q 914 86 886 62 q 943 152 943 111 q 911 221 943 195 q 825 267 880 248 l 806 274 q 733 222 757 251 q 709 152 709 194 m 826 530 q 754 509 780 530 q 729 450 729 489 q 752 394 729 418 q 830 348 775 370 q 925 450 925 385 q 899 509 925 489 q 826 530 872 530 z "
	},
	"⅜": {
		ha: 1083,
		x_min: 22,
		x_max: 1047,
		o: "m 901 991 l 289 0 l 192 0 l 804 991 l 901 991 m 842 604 q 977 564 926 604 q 1028 453 1028 524 q 928 313 1028 365 q 1047 156 1047 263 q 991 33 1047 79 q 842 -14 935 -14 q 689 30 743 -14 q 635 152 635 73 q 660 238 635 199 q 742 309 684 277 q 672 371 692 340 q 653 449 653 403 q 706 561 653 518 q 842 604 760 604 m 724 152 q 752 86 724 111 q 839 62 781 62 q 928 86 900 62 q 957 152 957 111 q 925 221 957 195 q 839 267 894 248 l 820 274 q 747 222 771 251 q 724 152 724 194 m 840 530 q 769 509 794 530 q 743 450 743 489 q 766 394 743 418 q 844 348 789 370 q 939 450 939 385 q 913 509 939 489 q 840 530 886 530 m 425 851 q 397 762 425 797 q 323 711 369 727 q 442 570 442 680 q 380 434 442 483 q 203 386 317 386 q 22 424 100 386 l 22 507 q 205 461 121 461 q 348 571 348 461 q 191 669 348 669 l 112 669 l 112 742 l 193 742 q 296 769 262 742 q 330 842 330 795 q 302 906 330 883 q 230 929 275 929 q 147 915 185 929 q 71 876 109 900 l 24 937 q 115 986 67 968 q 226 1004 162 1004 q 371 964 318 1004 q 425 851 425 924 z "
	},
	"⅝": {
		ha: 1083,
		x_min: 48,
		x_max: 1044,
		o: "m 897 991 l 285 0 l 188 0 l 800 991 l 897 991 m 231 765 q 389 716 329 765 q 448 585 448 668 q 391 437 448 490 q 220 385 333 385 q 123 395 170 385 q 48 421 76 406 l 48 510 q 134 474 86 487 q 218 461 181 461 q 319 489 282 461 q 357 578 357 517 q 320 661 357 631 q 214 690 283 690 q 149 685 175 690 q 102 673 123 679 l 62 701 l 85 983 l 420 983 l 420 913 l 162 913 l 149 757 q 231 765 191 765 m 839 604 q 974 564 923 604 q 1025 453 1025 524 q 925 313 1025 365 q 1044 156 1044 263 q 988 33 1044 79 q 839 -14 932 -14 q 686 30 740 -14 q 632 152 632 73 q 657 238 632 199 q 739 309 682 277 q 670 371 689 340 q 650 449 650 403 q 704 561 650 518 q 839 604 757 604 m 721 152 q 749 86 721 111 q 836 62 778 62 q 926 86 897 62 q 954 152 954 111 q 923 221 954 195 q 836 267 891 248 l 817 274 q 745 222 768 251 q 721 152 721 194 m 838 530 q 766 509 791 530 q 741 450 741 489 q 764 394 741 418 q 842 348 787 370 q 937 450 937 385 q 910 509 937 489 q 838 530 884 530 z "
	},
	"⅞": {
		ha: 1083,
		x_min: 72,
		x_max: 1042,
		o: "m 839 991 l 227 0 l 130 0 l 742 991 l 839 991 m 836 604 q 971 564 920 604 q 1023 453 1023 524 q 922 313 1023 365 q 1042 156 1042 263 q 986 33 1042 79 q 836 -14 930 -14 q 683 30 737 -14 q 629 152 629 73 q 654 238 629 199 q 736 309 679 277 q 667 371 686 340 q 648 449 648 403 q 701 561 648 518 q 836 604 754 604 m 718 152 q 747 86 718 111 q 833 62 775 62 q 923 86 895 62 q 951 152 951 111 q 920 221 951 195 q 833 267 888 248 l 814 274 q 742 222 766 251 q 718 152 718 194 m 835 530 q 763 509 789 530 q 738 450 738 489 q 761 394 738 418 q 839 348 784 370 q 934 450 934 385 q 907 509 934 489 q 835 530 881 530 m 143 397 l 380 913 l 72 913 l 72 991 l 477 991 l 477 928 l 239 397 l 143 397 z "
	},
	"∂": {
		ha: 807,
		x_min: 69,
		x_max: 730,
		o: "m 730 633 q 674 300 730 456 q 524 65 617 144 q 313 -14 430 -14 q 133 55 196 -14 q 69 253 69 123 q 115 472 69 368 q 240 630 161 576 q 421 685 319 685 q 536 653 487 685 q 615 571 586 622 l 618 632 q 427 907 618 907 q 336 892 385 907 q 254 854 287 877 l 254 962 q 339 990 286 978 q 441 1003 391 1003 q 657 909 584 1003 q 730 633 730 816 m 319 82 q 440 131 384 82 q 537 268 496 180 q 593 457 578 355 q 532 559 576 523 q 435 596 489 596 q 310 551 366 596 q 220 422 254 507 q 186 250 186 337 q 220 126 186 170 q 319 82 254 82 z "
	},
	"∆": {
		ha: 794,
		x_min: 26,
		x_max: 768,
		o: "m 26 71 l 340 991 l 453 991 l 768 69 l 768 0 l 26 0 l 26 71 m 396 860 q 328 625 355 707 l 151 103 l 640 103 l 465 619 q 396 860 419 756 z "
	},
	"∏": {
		ha: 1026,
		x_min: 136,
		x_max: 890,
		o: "m 775 -336 l 775 888 l 252 888 l 252 -336 l 136 -336 l 136 991 l 890 991 l 890 -336 l 775 -336 z "
	},
	"∑": {
		ha: 876,
		x_min: 52,
		x_max: 844,
		o: "m 52 -336 l 52 -264 l 479 363 l 62 918 l 62 991 l 800 991 l 800 889 l 225 889 l 618 366 l 208 -233 l 844 -233 l 844 -336 l 52 -336 z "
	},
	"−": {
		ha: 794,
		x_min: 71,
		x_max: 722,
		o: "m 71 443 l 71 536 l 722 536 l 722 443 l 71 443 z "
	},
	"√": {
		ha: 762,
		x_min: 25,
		x_max: 822,
		o: "m 422 -9 l 336 -9 l 147 521 l 25 521 l 25 612 l 221 612 l 380 149 l 729 1145 l 822 1145 l 422 -9 z "
	},
	"∞": {
		ha: 979,
		x_min: 81,
		x_max: 899,
		o: "m 899 488 q 842 336 899 399 q 699 273 785 273 q 584 309 635 273 q 488 424 532 345 q 395 315 447 353 q 283 277 343 277 q 138 335 195 277 q 81 490 81 394 q 138 642 81 581 q 283 703 195 703 q 489 554 406 703 q 699 700 573 700 q 844 641 789 700 q 899 488 899 581 m 289 361 q 371 392 334 361 q 444 490 408 422 q 373 588 410 556 q 288 619 336 619 q 202 584 236 619 q 168 489 168 548 q 201 397 168 433 q 289 361 234 361 m 690 616 q 609 585 646 616 q 534 488 572 554 q 606 390 569 422 q 692 358 644 358 q 777 394 743 358 q 811 488 811 431 q 777 581 811 545 q 690 616 743 616 z "
	},
	"∫": {
		ha: 533,
		x_min: 8,
		x_max: 515,
		o: "m 432 1055 q 515 1044 486 1055 l 515 951 q 440 966 482 966 q 321 802 321 966 l 321 -87 q 265 -270 321 -207 q 98 -334 209 -334 q 8 -321 48 -334 l 8 -229 q 89 -244 50 -244 q 212 -79 212 -244 l 212 810 q 432 1055 212 1055 z "
	},
	"≈": {
		ha: 794,
		x_min: 66,
		x_max: 725,
		o: "m 228 347 q 148 326 191 347 q 66 265 105 304 l 66 367 q 240 441 140 441 q 323 432 286 441 q 421 397 361 422 q 504 367 473 375 q 566 358 535 358 q 645 380 602 358 q 725 441 688 403 l 725 339 q 553 265 656 265 q 470 275 507 265 q 372 310 432 285 q 292 339 323 330 q 228 347 261 347 m 228 623 q 146 600 189 623 q 66 541 102 576 l 66 642 q 240 715 138 715 q 326 705 287 715 q 421 671 366 694 q 500 642 468 651 q 566 633 532 633 q 647 656 603 633 q 725 715 690 678 l 725 614 q 553 539 657 539 q 466 550 506 539 q 372 583 426 561 q 287 614 315 606 q 228 623 258 623 z "
	},
	"≠": {
		ha: 794,
		x_min: 71,
		x_max: 722,
		o: "m 258 304 l 71 304 l 71 397 l 301 397 l 387 582 l 71 582 l 71 675 l 431 675 l 522 869 l 607 831 l 533 675 l 722 675 l 722 582 l 490 582 l 403 397 l 722 397 l 722 304 l 359 304 l 271 113 l 186 150 l 258 304 z "
	},
	"≤": {
		ha: 794,
		x_min: 71,
		x_max: 722,
		o: "m 722 164 l 71 450 l 71 517 l 722 842 l 722 741 l 192 489 l 722 267 l 722 164 m 71 1 l 71 94 l 722 94 l 722 1 l 71 1 z "
	},
	"≥": {
		ha: 794,
		x_min: 71,
		x_max: 722,
		o: "m 71 267 l 602 488 l 71 741 l 71 842 l 722 517 l 722 450 l 71 164 l 71 267 m 71 1 l 71 94 l 722 94 l 722 1 l 71 1 z "
	},
	"◊": {
		ha: 810,
		x_min: 75,
		x_max: 736,
		o: "m 75 498 l 380 1000 l 429 1000 l 736 498 l 429 0 l 380 0 l 75 498 m 621 500 l 406 860 l 191 500 l 406 139 l 621 500 z "
	},
	"ﬁ": {
		ha: 822,
		x_min: 20,
		x_max: 713,
		o: "m 454 656 l 265 656 l 265 0 l 153 0 l 153 656 l 20 656 l 20 707 l 153 747 l 153 789 q 392 1063 153 1063 q 530 1039 451 1063 l 501 949 q 390 970 436 970 q 296 927 326 970 q 265 791 265 885 l 265 743 l 454 743 l 454 656 m 703 0 l 590 0 l 590 743 l 703 743 l 703 0 m 581 945 q 600 1001 581 983 q 647 1019 618 1019 q 694 1001 674 1019 q 713 945 713 983 q 694 888 713 907 q 647 869 674 869 q 600 888 618 869 q 581 945 581 907 z "
	},
	"ﬂ": {
		ha: 822,
		x_min: 20,
		x_max: 703,
		o: "m 454 656 l 265 656 l 265 0 l 153 0 l 153 656 l 20 656 l 20 707 l 153 747 l 153 789 q 392 1063 153 1063 q 530 1039 451 1063 l 501 949 q 390 970 436 970 q 296 927 326 970 q 265 791 265 885 l 265 743 l 454 743 l 454 656 m 703 0 l 590 0 l 590 1055 l 703 1055 l 703 0 z "
	},
	"ȷ": {
		ha: 351,
		x_min: -75,
		x_max: 232,
		o: "m 29 -334 q -75 -317 -35 -334 l -75 -225 q 17 -239 -28 -239 q 95 -210 70 -239 q 119 -122 119 -181 l 119 743 l 232 743 l 232 -114 q 29 -334 232 -334 z "
	},
	"⁰": {
		ha: 482,
		x_min: 26,
		x_max: 454,
		o: "m 119 694 q 147 524 119 581 q 239 468 175 468 q 332 524 302 468 q 361 696 361 580 q 332 867 361 811 q 239 922 302 922 q 147 866 175 922 q 119 694 119 810 m 454 696 q 240 386 454 386 q 80 466 133 386 q 26 696 26 546 q 240 1003 26 1003 q 401 924 347 1003 q 454 696 454 846 z "
	},
	"⁴": {
		ha: 482,
		x_min: 14,
		x_max: 469,
		o: "m 469 534 l 385 534 l 385 397 l 286 397 l 286 534 l 14 534 l 14 603 l 290 996 l 385 996 l 385 614 l 469 614 l 469 534 m 286 614 l 286 746 q 290 888 286 837 q 279 866 287 880 q 260 838 271 853 q 240 807 250 823 q 222 781 230 792 l 109 614 l 286 614 z "
	},
	"⁵": {
		ha: 482,
		x_min: 40,
		x_max: 440,
		o: "m 222 765 q 380 716 321 765 q 440 585 440 668 q 382 437 440 490 q 212 385 325 385 q 115 395 162 385 q 40 421 68 406 l 40 510 q 125 474 78 487 q 210 461 173 461 q 311 489 274 461 q 349 578 349 517 q 312 661 349 631 q 205 690 275 690 q 141 685 167 690 q 94 673 115 679 l 54 701 l 77 983 l 412 983 l 412 913 l 154 913 l 140 757 q 222 765 182 765 z "
	},
	"⁶": {
		ha: 482,
		x_min: 28,
		x_max: 457,
		o: "m 28 653 q 102 917 28 830 q 325 1003 176 1003 q 408 993 375 1003 l 408 916 q 317 928 373 928 q 170 876 221 928 q 113 711 119 824 l 118 711 q 176 761 138 741 q 272 781 214 781 q 407 730 357 781 q 457 595 457 680 q 401 441 457 497 q 249 386 345 386 q 86 457 145 386 q 28 653 28 528 m 248 461 q 336 493 302 461 q 370 591 370 524 q 340 676 370 645 q 252 707 310 707 q 156 676 194 707 q 118 606 118 646 q 154 503 118 545 q 248 461 190 461 z "
	},
	"⁷": {
		ha: 482,
		x_min: 39,
		x_max: 444,
		o: "m 110 397 l 347 913 l 39 913 l 39 991 l 444 991 l 444 928 l 205 397 l 110 397 z "
	},
	"⁸": {
		ha: 482,
		x_min: 35,
		x_max: 447,
		o: "m 241 1003 q 377 963 326 1003 q 428 852 428 923 q 328 713 428 764 q 447 555 447 663 q 391 432 447 479 q 241 386 335 386 q 89 429 142 386 q 35 551 35 473 q 59 637 35 598 q 142 708 84 677 q 72 771 92 739 q 53 848 53 802 q 106 960 53 918 q 241 1003 159 1003 m 123 551 q 152 486 123 511 q 239 461 180 461 q 328 486 300 461 q 357 551 357 511 q 325 621 357 594 q 239 667 294 648 l 220 673 q 147 622 171 650 q 123 551 123 593 m 240 929 q 169 909 194 929 q 143 850 143 888 q 166 794 143 818 q 244 747 189 770 q 339 850 339 784 q 313 909 339 888 q 240 929 286 929 z "
	},
	"⁹": {
		ha: 482,
		x_min: 24,
		x_max: 453,
		o: "m 453 742 q 379 472 453 559 q 161 386 305 386 q 72 396 105 386 l 72 475 q 168 461 105 461 q 370 682 355 461 l 363 682 q 300 630 339 648 q 216 612 260 612 q 76 662 127 612 q 24 800 24 711 q 81 948 24 892 q 231 1004 138 1004 q 393 936 334 1004 q 453 742 453 867 m 231 929 q 143 897 176 929 q 111 806 111 865 q 140 719 111 751 q 228 686 169 686 q 324 715 285 686 q 363 786 363 744 q 328 890 363 852 q 231 929 293 929 z "
	},
	" ": {
		ha: 694,
		x_min: 0,
		x_max: 0,
		o: ""
	},
	" ": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: ""
	},
	" ": {
		ha: 694,
		x_min: 0,
		x_max: 0,
		o: ""
	},
	" ": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: ""
	},
	" ": {
		ha: 463,
		x_min: 0,
		x_max: 0,
		o: ""
	},
	" ": {
		ha: 347,
		x_min: 0,
		x_max: 0,
		o: ""
	},
	" ": {
		ha: 232,
		x_min: 0,
		x_max: 0,
		o: ""
	},
	" ": {
		ha: 777,
		x_min: 0,
		x_max: 0,
		o: ""
	},
	" ": {
		ha: 370,
		x_min: 0,
		x_max: 0,
		o: ""
	},
	" ": {
		ha: 278,
		x_min: 0,
		x_max: 0,
		o: ""
	},
	" ": {
		ha: 139,
		x_min: 0,
		x_max: 0,
		o: ""
	},
	"​": {
		ha: 0,
		x_min: 0,
		x_max: 0,
		o: ""
	},
	"﻿": {
		ha: 0,
		x_min: 0,
		x_max: 0,
		o: ""
	},
	"￼": {
		ha: 1389,
		x_min: 57,
		x_max: 1346,
		o: "m 57 823 l 57 1029 l 262 1029 l 262 954 l 132 954 l 132 823 l 57 823 m 1140 954 l 1140 1029 l 1346 1029 l 1346 823 l 1272 823 l 1272 954 l 1140 954 m 57 -260 l 57 -54 l 132 -54 l 132 -186 l 262 -186 l 262 -260 l 57 -260 m 1140 -260 l 1140 -186 l 1272 -186 l 1272 -54 l 1346 -54 l 1346 -260 l 1140 -260 m 875 -260 l 875 -186 l 1060 -186 l 1060 -260 l 875 -260 m 345 -260 l 345 -186 l 528 -186 l 528 -260 l 345 -260 m 345 954 l 345 1029 l 528 1029 l 528 954 l 345 954 m 1346 26 l 1272 26 l 1272 210 l 1346 210 l 1346 26 m 1346 558 l 1272 558 l 1272 742 l 1346 742 l 1346 558 m 610 -260 l 610 -186 l 794 -186 l 794 -260 l 610 -260 m 132 26 l 57 26 l 57 210 l 132 210 l 132 26 m 610 954 l 610 1029 l 794 1029 l 794 954 l 610 954 m 875 954 l 875 1029 l 1060 1029 l 1060 954 l 875 954 m 132 292 l 57 292 l 57 476 l 132 476 l 132 292 m 132 558 l 57 558 l 57 742 l 132 742 l 132 558 m 1346 292 l 1272 292 l 1272 476 l 1346 476 l 1346 292 m 586 427 q 540 275 586 328 q 408 222 494 222 q 276 275 322 222 q 231 427 231 328 q 276 578 231 526 q 408 631 322 631 q 540 578 494 631 q 586 427 586 526 m 317 427 q 408 293 317 293 q 500 427 500 293 q 408 559 500 559 q 317 427 317 559 m 643 625 l 760 625 q 871 603 833 625 q 909 524 909 580 q 894 468 909 490 q 848 440 878 446 l 848 435 q 905 402 890 425 q 921 343 921 379 q 884 258 921 288 q 783 228 847 228 l 643 228 l 643 625 m 726 468 l 771 468 q 814 479 802 468 q 827 513 827 491 q 812 546 827 536 q 766 555 798 555 l 726 555 l 726 468 m 726 401 l 726 297 l 777 297 q 822 312 810 297 q 835 351 835 326 q 774 401 835 401 l 726 401 m 1011 224 q 957 231 976 224 l 957 300 q 1003 294 986 294 q 1061 353 1061 294 l 1061 625 l 1146 625 l 1146 355 q 1111 257 1146 291 q 1011 224 1075 224 z "
	},
	"�": {
		ha: 1389,
		x_min: 57,
		x_max: 1331,
		o: "m 693 1055 l 1331 419 l 693 -216 l 57 419 l 693 1055 m 737 249 l 737 281 q 752 337 737 315 q 811 392 767 359 q 906 490 881 448 q 930 593 930 533 q 867 736 930 684 q 692 789 804 789 q 575 770 638 789 q 464 722 512 751 l 519 602 q 689 656 628 656 q 753 636 732 656 q 774 585 774 616 q 757 523 774 549 q 692 461 741 498 q 615 377 635 415 q 595 289 595 339 l 595 249 l 737 249 m 576 62 q 601 -5 576 20 q 672 -29 625 -29 q 742 -4 717 -29 q 766 62 766 20 q 742 131 766 106 q 672 155 718 155 q 600 131 623 155 q 576 62 576 106 z "
	},
	"ǰ": {
		ha: 351,
		x_min: -75,
		x_max: 406,
		o: "m 29 -334 q -75 -317 -35 -334 l -75 -225 q 17 -239 -28 -239 q 95 -210 70 -239 q 119 -122 119 -181 l 119 743 l 232 743 l 232 -114 q 29 -334 232 -334 m -51 1064 l 27 1064 q 176 936 104 1014 q 326 1064 264 1024 l 406 1064 l 406 1047 l 361 1001 q 233 842 264 901 l 120 842 q 70 912 104 869 q -51 1047 35 955 l -51 1064 z "
	},
	"ʼ": {
		ha: 236,
		x_min: 17,
		x_max: 220,
		o: "m 210 991 l 220 977 q 169 819 202 909 q 100 652 136 729 l 17 652 q 86 991 64 845 l 210 991 z "
	},
	"ϑ": {
		ha: 864,
		x_min: 7,
		x_max: 846,
		o: "m 753 574 q 755 498 755 543 q 658 123 755 259 q 385 -14 560 -14 q 118 220 118 -14 q 129 342 118 258 q 139 440 139 425 q 127 484 139 472 q 93 496 115 496 q 58 491 76 496 q 29 479 39 486 l 7 559 q 127 586 63 586 q 219 554 187 586 q 250 460 250 523 q 240 347 250 436 q 229 218 229 258 q 387 78 229 78 q 642 501 642 78 q 639 574 642 544 q 382 611 488 575 q 222 710 277 646 q 168 863 168 774 q 230 1016 168 960 q 406 1071 292 1071 q 633 967 547 1071 q 749 661 720 863 l 846 661 l 846 574 l 753 574 m 633 661 q 552 891 614 804 q 407 978 490 978 q 311 946 344 978 q 278 863 278 913 q 371 715 278 767 q 633 661 465 663 z "
	},
	"ϒ": {
		ha: 785,
		x_min: 0,
		x_max: 778,
		o: "m 386 486 q 516 789 469 684 q 590 931 564 895 q 648 984 616 968 q 718 1000 679 1000 q 778 990 751 1000 l 778 899 q 739 903 760 903 q 692 884 712 903 q 643 810 672 865 q 572 670 614 755 q 495 505 530 585 q 440 367 461 425 l 440 0 l 323 0 l 323 379 l 0 991 l 126 991 l 386 486 z "
	},
	"ϖ": {
		ha: 1166,
		x_min: 12,
		x_max: 1122,
		o: "m 375 -14 q 181 65 249 -14 q 114 306 114 143 q 205 647 114 469 l 12 647 l 12 697 l 109 743 l 1122 743 l 1122 647 l 953 647 q 1032 306 1032 476 q 964 65 1032 143 q 771 -14 897 -14 q 575 110 621 -14 l 570 110 q 375 -14 524 -14 m 317 647 q 252 470 274 557 q 229 313 229 382 q 266 138 229 195 q 382 81 302 81 q 481 129 445 81 q 518 262 518 178 l 518 390 l 628 390 l 628 262 q 664 130 628 180 q 764 81 701 81 q 880 138 843 81 q 917 313 917 195 q 842 647 917 485 l 317 647 z "
	},
	"Ḿ": {
		ha: 1254,
		x_min: 136,
		x_max: 1118,
		o: "m 575 0 l 239 879 l 233 879 q 243 631 243 774 l 243 0 l 136 0 l 136 991 l 310 991 l 624 174 l 629 174 l 946 991 l 1118 991 l 1118 0 l 1003 0 l 1003 639 q 1013 878 1003 749 l 1007 878 l 668 0 l 575 0 m 546 1089 q 616 1191 578 1131 q 675 1295 654 1251 l 812 1295 l 812 1280 q 724 1172 783 1236 q 621 1072 665 1107 l 546 1072 l 546 1089 z "
	},
	"ḿ": {
		ha: 1292,
		x_min: 119,
		x_max: 1179,
		o: "m 1067 0 l 1067 484 q 1029 617 1067 572 q 911 661 991 661 q 755 601 806 661 q 705 415 705 541 l 705 0 l 593 0 l 593 484 q 555 617 593 572 q 436 661 517 661 q 281 598 330 661 q 232 390 232 534 l 232 0 l 119 0 l 119 743 l 211 743 l 229 642 l 235 642 q 325 726 267 696 q 454 757 382 757 q 682 631 629 757 l 688 631 q 784 723 721 689 q 928 757 847 757 q 1117 692 1054 757 q 1179 485 1179 627 l 1179 0 l 1067 0 m 579 859 q 649 960 612 901 q 709 1064 687 1020 l 846 1064 l 846 1050 q 757 941 816 1006 q 654 842 698 877 l 579 842 l 579 859 z "
	},
	"Ḁ": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 593 -229 q 550 -336 593 -295 q 439 -376 508 -376 q 330 -337 370 -376 q 289 -231 289 -297 q 330 -124 289 -164 q 439 -85 370 -85 q 550 -125 507 -85 q 593 -229 593 -165 m 518 -231 q 496 -172 518 -192 q 439 -151 473 -151 q 382 -172 404 -151 q 359 -231 359 -192 q 380 -289 359 -269 q 439 -310 400 -310 q 496 -289 475 -310 q 518 -231 518 -269 z "
	},
	"ḁ": {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 518 -229 q 476 -336 518 -295 q 364 -376 433 -376 q 255 -337 296 -376 q 214 -231 214 -297 q 255 -124 214 -164 q 364 -85 296 -85 q 475 -125 433 -85 q 518 -229 518 -165 m 444 -231 q 421 -172 444 -192 q 364 -151 399 -151 q 307 -172 330 -151 q 285 -231 285 -192 q 305 -289 285 -269 q 364 -310 326 -310 q 422 -289 400 -310 q 444 -231 444 -269 z "
	},
	"Ὅ": {
		ha: 1097,
		x_min: -196,
		x_max: 1010,
		o: "m 1010 497 q 890 123 1010 259 q 555 -14 770 -14 q 217 120 336 -14 q 98 498 98 254 q 218 874 98 741 q 557 1007 337 1007 q 890 871 770 1007 q 1010 497 1010 736 m 220 497 q 306 192 220 296 q 555 87 392 87 q 804 191 720 87 q 888 497 888 295 q 805 801 888 697 q 557 904 721 904 q 306 800 392 904 q 220 497 220 696 m -6 786 q 60 991 41 909 l 189 991 l 189 978 q 120 867 161 925 q 47 770 80 808 l -6 770 l -6 786 m -196 850 q -35 1002 -196 960 l -35 953 q -117 889 -117 925 q -107 869 -117 876 q -84 857 -96 863 q -61 843 -71 852 q -50 815 -50 834 q -113 765 -50 765 q -173 788 -151 765 q -196 850 -196 811 z "
	},
	"˳": {
		ha: 463,
		x_min: 79,
		x_max: 383,
		o: "m 383 -229 q 341 -336 383 -295 q 229 -376 298 -376 q 120 -337 161 -376 q 79 -231 79 -297 q 120 -124 79 -164 q 229 -85 161 -85 q 340 -125 298 -85 q 383 -229 383 -165 m 309 -231 q 286 -172 309 -192 q 229 -151 264 -151 q 172 -172 195 -151 q 150 -231 150 -192 q 170 -289 150 -269 q 229 -310 191 -310 q 287 -289 265 -310 q 309 -231 309 -269 z "
	},
	"ﬃ": {
		ha: 1295,
		x_min: 20,
		x_max: 1185,
		o: "m 921 656 l 732 656 l 732 0 l 619 0 l 619 656 l 486 656 l 486 707 l 619 747 l 619 789 q 859 1063 619 1063 q 997 1039 918 1063 l 968 949 q 857 970 903 970 q 762 927 793 970 q 732 791 732 885 l 732 743 l 921 743 l 921 656 m 454 656 l 265 656 l 265 0 l 153 0 l 153 656 l 20 656 l 20 707 l 153 747 l 153 789 q 392 1063 153 1063 q 530 1039 451 1063 l 501 949 q 390 970 436 970 q 296 927 326 970 q 265 791 265 885 l 265 743 l 454 743 l 454 656 m 1174 0 l 1061 0 l 1061 743 l 1174 743 l 1174 0 m 1052 945 q 1071 1001 1052 983 q 1118 1019 1090 1019 q 1165 1001 1145 1019 q 1185 945 1185 983 q 1165 888 1185 907 q 1118 869 1145 869 q 1071 888 1090 869 q 1052 945 1052 907 z "
	},
	"ﬄ": {
		ha: 1295,
		x_min: 20,
		x_max: 1174,
		o: "m 921 656 l 732 656 l 732 0 l 619 0 l 619 656 l 486 656 l 486 707 l 619 747 l 619 789 q 859 1063 619 1063 q 997 1039 918 1063 l 968 949 q 857 970 903 970 q 762 927 793 970 q 732 791 732 885 l 732 743 l 921 743 l 921 656 m 454 656 l 265 656 l 265 0 l 153 0 l 153 656 l 20 656 l 20 707 l 153 747 l 153 789 q 392 1063 153 1063 q 530 1039 451 1063 l 501 949 q 390 970 436 970 q 296 927 326 970 q 265 791 265 885 l 265 743 l 454 743 l 454 656 m 1174 0 l 1061 0 l 1061 1055 l 1174 1055 l 1174 0 z "
	},
	"Ơ": {
		ha: 1083,
		x_min: 85,
		x_max: 1109,
		o: "m 996 497 q 875 122 996 257 q 542 -14 755 -14 q 204 120 323 -14 q 85 498 85 253 q 204 875 85 743 q 543 1007 324 1007 q 885 861 764 1007 q 939 892 919 869 q 967 949 958 915 q 977 1055 977 984 l 1100 1055 l 1109 1040 q 1045 877 1090 935 q 930 791 1001 818 q 996 497 996 672 m 207 497 q 292 191 207 295 q 542 87 376 87 q 790 191 707 87 q 873 497 873 294 q 790 801 873 697 q 543 904 707 904 q 293 801 378 904 q 207 497 207 699 z "
	},
	"ơ": {
		ha: 846,
		x_min: 78,
		x_max: 885,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 661 661 568 757 q 715 693 696 671 q 744 749 735 715 q 753 857 753 783 l 875 857 l 885 842 q 823 676 864 736 q 713 591 782 617 q 761 372 761 498 m 195 372 q 248 155 195 229 q 419 81 302 81 q 591 155 538 81 q 644 372 644 229 q 590 588 644 515 q 418 661 536 661 q 248 588 300 661 q 195 372 195 515 z "
	},
	"Ư": {
		ha: 1067,
		x_min: 126,
		x_max: 1125,
		o: "m 885 991 l 885 857 q 948 884 924 863 q 982 943 972 905 q 993 1055 993 981 l 1116 1055 l 1125 1040 q 1044 854 1103 917 q 885 775 986 792 l 885 350 q 788 81 885 175 q 511 -14 690 -14 q 228 84 330 -14 q 126 353 126 181 l 126 991 l 241 991 l 241 345 q 311 154 241 220 q 514 87 380 87 q 704 153 639 87 q 770 346 770 218 l 770 991 l 885 991 z "
	},
	"ư": {
		ha: 924,
		x_min: 111,
		x_max: 970,
		o: "m 225 742 l 225 261 q 267 125 225 170 q 396 81 308 81 q 567 144 513 81 q 621 353 621 208 l 621 742 l 733 742 l 733 660 q 814 711 789 667 q 839 859 839 754 l 960 859 l 970 843 q 888 652 948 717 q 733 578 829 587 l 733 0 l 640 0 l 624 100 l 618 100 q 521 15 583 44 q 385 -14 460 -14 q 179 51 247 -14 q 111 257 111 115 l 111 742 l 225 742 z "
	},
	"̀": {
		ha: 0,
		x_min: -638,
		x_max: -372,
		o: "m -372 842 l -446 842 q -551 942 -490 877 q -638 1050 -611 1007 l -638 1064 l -500 1064 q -440 956 -479 1017 q -372 859 -401 895 l -372 842 z "
	},
	"́": {
		ha: 0,
		x_min: -512,
		x_max: -245,
		o: "m -512 859 q -442 960 -479 901 q -382 1064 -404 1020 l -245 1064 l -245 1050 q -334 941 -275 1006 q -437 842 -393 877 l -512 842 l -512 859 z "
	},
	"̃": {
		ha: 0,
		x_min: -677,
		x_max: -173,
		o: "m -322 843 q -379 856 -351 843 q -434 883 -407 868 q -485 911 -460 899 q -533 924 -510 924 q -584 903 -567 924 q -611 842 -602 883 l -677 842 q -630 970 -669 924 q -529 1017 -591 1017 q -469 1004 -498 1017 q -413 976 -439 991 q -362 948 -387 961 q -316 936 -338 936 q -267 956 -283 936 q -240 1018 -250 976 l -173 1018 q -220 889 -182 936 q -322 843 -258 843 z "
	},
	"̉": {
		ha: 0,
		x_min: -515,
		x_max: -269,
		o: "m -269 1014 q -382 896 -269 919 l -389 819 l -460 819 l -468 939 q -383 965 -410 948 q -357 1011 -357 982 q -379 1054 -357 1040 q -452 1068 -402 1068 q -515 1064 -494 1068 l -515 1132 q -443 1139 -490 1139 q -269 1014 -269 1139 z "
	},
	"̣": {
		ha: 0,
		x_min: -481,
		x_max: -346,
		o: "m -481 -164 q -461 -107 -481 -125 q -412 -89 -441 -89 q -366 -107 -385 -89 q -346 -164 -346 -125 q -366 -220 -346 -201 q -412 -239 -385 -239 q -461 -220 -441 -239 q -481 -164 -481 -201 z "
	},
	"Ѐ": {
		ha: 772,
		x_min: 136,
		x_max: 689,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 m 506 1071 l 431 1071 q 327 1171 387 1106 q 239 1279 267 1236 l 239 1293 l 377 1293 q 437 1185 399 1246 q 506 1088 476 1124 l 506 1071 z "
	},
	"Ѝ": {
		ha: 1059,
		x_min: 138,
		x_max: 924,
		o: "m 138 991 l 245 991 l 245 440 q 236 164 245 288 l 241 164 l 798 991 l 924 991 l 924 0 l 815 0 l 815 546 q 827 832 815 682 l 821 832 l 264 0 l 138 0 l 138 991 m 604 1071 l 529 1071 q 425 1171 485 1106 q 337 1279 364 1236 l 337 1293 l 475 1293 q 535 1185 496 1246 q 604 1088 574 1124 l 604 1071 z "
	},
	"ѐ": {
		ha: 779,
		x_min: 78,
		x_max: 707,
		o: "m 433 -14 q 173 87 269 -14 q 78 366 78 187 q 166 651 78 545 q 404 757 255 757 q 625 665 544 757 q 707 422 707 573 l 707 351 l 195 351 q 261 153 198 220 q 437 85 323 85 q 675 135 557 85 l 675 35 q 562 -2 615 9 q 433 -14 509 -14 m 403 663 q 260 604 313 663 q 197 443 207 546 l 586 443 q 538 606 586 549 q 403 663 491 663 m 484 842 l 409 842 q 304 942 365 877 q 217 1050 244 1007 l 217 1064 l 355 1064 q 415 956 376 1017 q 484 859 454 895 l 484 842 z "
	},
	"ѝ": {
		ha: 880,
		x_min: 119,
		x_max: 761,
		o: "m 225 743 l 225 272 l 220 149 l 218 110 l 621 743 l 761 743 l 761 0 l 656 0 l 656 454 l 658 544 l 661 632 l 260 0 l 119 0 l 119 743 l 225 743 m 509 842 l 434 842 q 330 942 390 877 q 242 1050 269 1007 l 242 1064 l 380 1064 q 440 956 401 1017 q 509 859 479 895 l 509 842 z "
	},
	"Ѡ": {
		ha: 1404,
		x_min: 90,
		x_max: 1314,
		o: "m 979 900 q 907 886 939 900 q 844 851 875 872 l 798 950 q 985 1004 883 1004 q 1226 877 1139 1004 q 1314 519 1314 749 q 1218 127 1314 268 q 949 -14 1122 -14 q 817 2 876 -14 q 703 52 759 17 l 697 52 q 586 2 643 18 q 456 -14 528 -14 q 186 126 282 -14 q 90 519 90 265 q 177 877 90 749 q 418 1004 263 1004 q 606 950 522 1004 l 558 851 q 496 886 528 872 q 424 900 465 900 q 268 799 324 900 q 212 518 212 699 q 282 201 212 314 q 479 87 353 87 q 643 139 574 87 l 643 449 l 758 449 l 758 139 q 924 87 827 87 q 1120 200 1050 87 q 1190 518 1190 312 q 1134 800 1190 699 q 979 900 1078 900 z "
	},
	"ѡ": {
		ha: 1137,
		x_min: 0,
		x_max: 1061,
		o: "m 547 310 q 534 275 541 292 q 406 0 527 258 l 261 0 l 0 743 l 117 743 l 283 272 q 336 81 305 209 l 342 81 q 380 172 355 117 q 501 425 405 227 l 385 743 l 505 743 q 700 173 685 222 q 722 94 715 124 l 728 94 q 897 367 845 219 q 949 743 949 514 l 1061 743 q 995 327 1061 498 q 781 0 929 155 l 657 0 l 572 235 q 547 310 550 297 z "
	},
	"Ѣ": {
		ha: 937,
		x_min: 16,
		x_max: 865,
		o: "m 16 864 l 232 864 l 232 1055 l 349 1055 l 349 864 l 632 864 l 632 764 l 349 764 l 349 568 l 485 568 q 865 289 865 568 q 775 74 865 148 q 509 0 686 0 l 232 0 l 232 764 l 16 764 l 16 864 m 349 98 l 493 98 q 685 145 623 98 q 746 289 746 191 q 683 424 746 382 q 472 465 620 465 l 349 465 l 349 98 z "
	},
	"ѣ": {
		ha: 878,
		x_min: 16,
		x_max: 800,
		o: "m 288 743 l 521 743 l 521 648 l 288 648 l 288 435 l 504 435 q 800 227 800 435 q 725 57 800 114 q 500 0 649 0 l 175 0 l 175 648 l 16 648 l 16 743 l 175 743 l 175 895 l 288 895 l 288 743 m 288 339 l 288 94 l 494 94 q 689 220 689 94 q 643 311 689 282 q 492 339 597 339 l 288 339 z "
	},
	"Ѥ": {
		ha: 1276,
		x_min: 136,
		x_max: 1238,
		o: "m 965 903 q 723 818 811 903 q 614 570 635 733 l 1090 570 l 1090 468 l 611 468 q 706 186 616 283 q 964 89 796 89 q 1200 126 1068 89 l 1200 25 q 946 -14 1097 -14 q 612 111 730 -14 q 489 467 494 236 l 252 467 l 252 0 l 136 0 l 136 991 l 252 991 l 252 570 l 493 570 q 638 891 513 778 q 970 1006 764 1004 q 1238 943 1114 1006 l 1189 844 q 1068 890 1121 877 q 965 903 1015 903 z "
	},
	"ѥ": {
		ha: 1022,
		x_min: 119,
		x_max: 974,
		o: "m 775 -14 q 525 77 616 -14 q 427 335 434 167 l 232 335 l 232 0 l 119 0 l 119 743 l 232 743 l 232 431 l 428 431 q 535 673 444 590 q 778 757 627 757 q 885 746 833 757 q 974 718 937 735 l 940 623 q 778 657 846 657 q 611 602 667 657 q 543 431 554 546 l 907 431 l 907 335 l 542 335 q 605 146 548 208 q 774 85 663 85 q 856 92 815 85 q 962 125 896 100 l 962 25 q 775 -14 888 -14 z "
	},
	"Ѧ": {
		ha: 942,
		x_min: 0,
		x_max: 942,
		o: "m 624 463 l 524 463 l 524 0 l 418 0 l 418 463 l 317 463 l 121 0 l 0 0 l 418 991 l 525 991 l 942 0 l 818 0 l 624 463 m 359 564 l 585 564 l 529 704 q 471 869 491 802 q 411 692 454 803 l 359 564 z "
	},
	"ѧ": {
		ha: 783,
		x_min: 7,
		x_max: 777,
		o: "m 461 743 l 777 0 l 660 0 l 519 332 l 443 332 l 443 0 l 340 0 l 340 332 l 262 332 l 123 0 l 7 0 l 322 743 l 461 743 m 300 427 l 484 427 q 435 547 454 500 q 393 665 416 593 l 387 665 q 300 427 368 603 z "
	},
	"Ѩ": {
		ha: 1279,
		x_min: 136,
		x_max: 1279,
		o: "m 958 467 l 861 467 l 861 0 l 757 0 l 757 467 l 657 467 l 464 0 l 338 0 l 534 467 l 252 467 l 252 0 l 136 0 l 136 991 l 252 991 l 252 570 l 578 570 l 755 991 l 863 991 l 1279 0 l 1152 0 l 958 467 m 700 570 l 916 570 q 826 808 836 779 q 808 869 817 838 q 784 789 795 824 q 700 570 772 753 z "
	},
	"ѩ": {
		ha: 1064,
		x_min: 119,
		x_max: 1055,
		o: "m 742 743 l 1055 0 l 940 0 l 799 335 l 722 335 l 722 0 l 619 0 l 619 335 l 545 335 l 403 0 l 286 0 l 428 335 l 232 335 l 232 0 l 119 0 l 119 743 l 232 743 l 232 431 l 469 431 l 603 743 l 742 743 m 673 665 l 668 665 q 650 610 661 642 q 579 431 640 578 l 761 431 l 673 665 z "
	},
	"Ѫ": {
		ha: 1002,
		x_min: 14,
		x_max: 986,
		o: "m 896 991 l 896 901 l 610 566 q 742 537 690 562 q 828 466 794 513 q 896 307 862 418 l 986 0 l 868 0 l 775 310 q 728 418 751 385 q 671 463 705 450 q 576 476 637 476 l 558 476 l 558 0 l 443 0 l 443 476 l 425 476 q 330 463 363 476 q 274 417 296 450 q 231 310 252 385 l 139 0 l 14 0 l 106 307 q 171 463 138 415 q 255 536 205 511 q 386 566 306 561 l 107 901 l 107 991 l 896 991 m 757 888 l 243 888 l 500 579 l 757 888 z "
	},
	"ѫ": {
		ha: 892,
		x_min: 8,
		x_max: 882,
		o: "m 789 743 l 789 672 l 559 433 q 656 412 618 429 q 719 369 694 396 q 760 306 743 342 q 882 0 777 271 l 765 0 l 675 228 q 632 311 652 287 q 584 345 612 335 q 504 354 555 354 l 497 354 l 497 0 l 393 0 l 393 354 l 386 354 q 307 345 335 354 q 260 315 279 336 q 215 229 241 293 l 123 0 l 8 0 l 97 228 q 130 305 113 269 q 171 367 146 340 q 233 411 195 394 q 329 433 270 428 l 102 672 l 102 743 l 789 743 m 644 648 l 245 648 l 444 432 l 644 648 z "
	},
	"Ѭ": {
		ha: 1353,
		x_min: 136,
		x_max: 1349,
		o: "m 1257 991 l 1257 901 l 972 564 q 1106 535 1054 559 q 1192 462 1158 511 q 1257 307 1226 413 l 1349 0 l 1235 0 l 1141 310 q 1097 414 1120 380 q 1037 461 1073 448 q 937 475 1002 475 l 921 475 l 921 0 l 804 0 l 804 475 l 787 475 q 690 461 724 475 q 633 415 656 448 q 589 310 611 383 l 497 0 l 376 0 l 468 302 q 543 467 505 427 l 252 467 l 252 0 l 136 0 l 136 991 l 252 991 l 252 570 l 745 570 l 468 901 l 468 991 l 1257 991 m 1120 888 l 606 888 l 863 578 l 1120 888 z "
	},
	"ѭ": {
		ha: 1175,
		x_min: 119,
		x_max: 1168,
		o: "m 1075 743 l 1075 672 l 846 432 q 943 411 905 427 q 1005 368 981 394 q 1046 306 1030 341 q 1168 0 1063 270 l 1051 0 l 961 228 q 918 310 938 286 q 869 343 899 334 q 790 353 840 353 l 783 353 l 783 0 l 679 0 l 679 353 l 672 353 q 593 343 621 353 q 546 313 564 334 q 501 229 527 292 l 410 0 l 294 0 l 383 228 q 440 336 415 309 l 232 336 l 232 0 l 119 0 l 119 743 l 232 743 l 232 432 l 615 432 l 387 672 l 387 743 l 1075 743 m 930 648 l 532 648 l 730 436 l 930 648 z "
	},
	"Ѯ": {
		ha: 808,
		x_min: 43,
		x_max: 730,
		o: "m 163 -93 q 192 -147 163 -130 q 282 -164 222 -164 q 429 -159 348 -164 q 558 -155 511 -155 q 711 -182 663 -155 l 711 -294 q 549 -260 657 -260 q 431 -263 503 -260 q 289 -267 360 -267 q 105 -224 168 -267 q 43 -92 43 -182 q 116 38 43 -6 q 347 87 190 81 q 547 140 486 93 q 608 282 608 186 q 304 464 608 464 l 157 464 l 157 561 l 298 561 q 514 611 437 561 q 590 750 590 661 q 535 866 590 823 q 387 909 480 909 q 252 891 315 909 q 114 825 189 872 l 56 903 q 298 999 170 980 q 180 1126 260 1044 l 147 1161 l 147 1179 l 231 1179 q 382 1047 293 1139 q 489 1154 444 1125 q 576 1183 533 1183 q 643 1174 610 1183 l 643 1097 q 597 1104 627 1104 q 530 1076 568 1104 q 460 1000 492 1048 q 646 921 581 985 q 711 760 711 857 q 647 602 711 665 q 467 522 582 540 l 467 517 q 662 440 593 500 q 730 282 730 380 q 634 69 730 141 q 347 -9 538 -3 q 208 -32 254 -13 q 163 -93 163 -50 z "
	},
	"ѯ": {
		ha: 671,
		x_min: 17,
		x_max: 607,
		o: "m 515 921 q 581 911 550 921 l 581 835 q 536 842 564 842 q 470 815 505 842 q 404 746 435 788 q 534 680 487 727 q 582 561 582 633 q 439 397 582 436 l 439 392 q 607 207 607 344 q 525 50 607 106 q 290 -9 443 -7 q 190 -19 227 -11 q 137 -41 153 -26 q 121 -85 121 -55 q 146 -133 121 -118 q 232 -147 172 -147 q 373 -144 290 -147 l 492 -140 q 603 -167 576 -140 l 603 -264 q 560 -243 589 -251 q 494 -235 532 -235 l 373 -238 q 222 -241 285 -241 q 71 -201 125 -241 q 17 -85 17 -161 q 81 36 17 -6 q 268 81 146 79 q 494 212 494 83 q 441 309 494 278 q 281 339 389 339 l 180 339 l 180 435 l 261 435 q 472 551 472 435 q 301 661 472 661 q 202 650 251 661 q 94 614 154 638 l 54 705 q 224 753 138 743 q 92 899 161 832 l 92 917 l 175 917 q 321 789 226 882 q 426 892 380 863 q 515 921 471 921 z "
	},
	"Ѱ": {
		ha: 1105,
		x_min: 74,
		x_max: 1032,
		o: "m 610 0 l 494 0 l 494 302 l 464 302 q 258 344 345 302 q 123 469 172 385 q 74 664 74 553 l 74 991 l 192 991 l 192 667 q 262 471 192 539 q 476 403 332 403 l 494 403 l 494 991 l 610 991 l 610 403 l 629 403 q 843 470 772 403 q 913 664 913 537 l 913 991 l 1032 991 l 1032 665 q 983 470 1032 553 q 847 344 935 386 q 640 302 759 302 l 610 302 l 610 0 z "
	},
	"ѱ": {
		ha: 1046,
		x_min: 111,
		x_max: 960,
		o: "m 582 1054 l 582 83 q 778 172 709 94 q 847 389 847 250 q 838 550 847 470 q 804 743 829 629 l 917 743 q 960 393 960 559 q 863 102 960 206 q 582 -12 766 -1 l 582 -334 l 471 -334 l 471 -12 q 201 92 292 -6 q 111 375 111 191 l 111 743 l 224 743 l 224 371 q 285 165 224 239 q 471 82 346 91 l 471 1054 l 582 1054 z "
	},
	"Ѳ": {
		ha: 1083,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 542 87 q 781 180 697 87 q 873 454 865 273 l 208 454 q 301 180 217 273 q 542 87 386 87 m 543 904 q 306 817 391 904 q 208 555 221 729 l 872 555 q 778 816 861 728 q 543 904 695 904 z "
	},
	"ѳ": {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 419 81 q 582 143 526 81 q 644 335 637 205 l 195 335 q 255 145 201 209 q 419 81 309 81 m 418 661 q 258 603 312 661 q 196 431 205 544 l 643 431 q 578 603 633 545 q 418 661 524 661 z "
	},
	"Ѵ": {
		ha: 869,
		x_min: 0,
		x_max: 917,
		o: "m 847 905 q 781 870 807 905 q 715 713 754 836 l 493 0 l 359 0 l 0 991 l 122 991 l 351 355 q 424 123 400 220 q 493 372 446 221 l 603 724 q 671 905 643 853 q 737 978 700 956 q 835 1000 774 1000 q 917 987 863 1000 l 917 892 q 847 905 879 905 z "
	},
	"ѵ": {
		ha: 703,
		x_min: 0,
		x_max: 736,
		o: "m 275 0 l 0 743 l 118 743 l 271 328 q 351 78 338 146 l 357 78 q 428 339 372 158 l 493 549 q 542 673 518 631 q 597 732 566 715 q 685 750 628 750 q 736 743 715 750 l 736 657 q 690 663 716 663 q 639 634 659 663 q 600 543 619 606 l 432 0 l 275 0 z "
	},
	"Ѷ": {
		ha: 869,
		x_min: 0,
		x_max: 917,
		o: "m 847 905 q 781 870 807 905 q 715 713 754 836 l 493 0 l 359 0 l 0 991 l 122 991 l 351 355 q 424 123 400 220 q 493 372 446 221 l 603 724 q 671 905 643 853 q 737 978 700 956 q 835 1000 774 1000 q 917 987 863 1000 l 917 892 q 847 905 879 905 m 615 1071 l 550 1071 q 455 1168 515 1099 q 370 1279 395 1236 l 370 1293 l 496 1293 q 548 1193 515 1252 q 615 1088 582 1134 l 615 1071 m 374 1071 l 309 1071 q 212 1168 271 1101 q 127 1279 153 1236 l 127 1293 l 254 1293 q 307 1193 273 1252 q 374 1088 340 1134 l 374 1071 z "
	},
	"ѷ": {
		ha: 703,
		x_min: 0,
		x_max: 736,
		o: "m 275 0 l 0 743 l 118 743 l 271 328 q 351 78 338 146 l 357 78 q 428 339 372 158 l 493 549 q 542 673 518 631 q 597 732 566 715 q 685 750 628 750 q 736 743 715 750 l 736 657 q 690 663 716 663 q 639 634 659 663 q 600 543 619 606 l 432 0 l 275 0 m 537 842 l 472 842 q 377 939 437 870 q 292 1050 317 1007 l 292 1064 l 418 1064 q 470 964 437 1023 q 537 859 504 905 l 537 842 m 296 842 l 231 842 q 134 939 193 871 q 50 1050 75 1007 l 50 1064 l 176 1064 q 229 964 195 1023 q 296 859 262 905 l 296 842 z "
	},
	"Ѹ": {
		ha: 1679,
		x_min: 85,
		x_max: 1672,
		o: "m 925 497 q 814 122 925 258 q 505 -14 703 -14 q 194 120 304 -14 q 85 498 85 254 q 194 874 85 741 q 507 1007 304 1007 q 815 871 705 1007 q 925 497 925 736 m 207 497 q 282 192 207 296 q 505 87 358 87 q 728 191 653 87 q 803 497 803 294 q 728 801 803 699 q 507 904 654 904 q 283 802 359 904 q 207 497 207 699 m 977 743 l 1096 743 l 1263 317 q 1329 107 1316 176 l 1335 107 q 1370 222 1342 137 q 1553 743 1398 307 l 1672 743 l 1354 -103 q 1244 -280 1308 -227 q 1088 -334 1180 -334 q 986 -322 1036 -334 l 986 -232 q 1068 -240 1023 -240 q 1172 -205 1132 -240 q 1235 -110 1211 -170 l 1276 -4 l 977 743 z "
	},
	"ѹ": {
		ha: 1474,
		x_min: 78,
		x_max: 1472,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 195 372 q 252 155 195 230 q 419 81 309 81 q 587 155 530 81 q 644 372 644 229 q 587 588 644 514 q 418 661 530 661 q 251 589 307 661 q 195 372 195 516 m 775 743 l 896 743 l 1059 319 q 1125 110 1112 174 l 1131 110 q 1167 228 1139 144 q 1352 743 1196 312 l 1472 743 l 1153 -103 q 1042 -281 1105 -229 q 886 -334 979 -334 q 785 -322 835 -334 l 785 -232 q 868 -240 822 -240 q 1034 -110 984 -240 l 1075 -4 l 775 743 z "
	},
	"Ѻ": {
		ha: 1137,
		x_min: 85,
		x_max: 1053,
		o: "m 1053 497 q 950 158 1053 292 q 668 -7 847 23 q 570 -82 650 -82 q 472 -7 486 -82 q 188 155 290 21 q 85 498 85 288 q 186 837 85 705 q 472 999 288 969 q 570 1072 486 1072 q 667 999 652 1072 q 950 834 848 969 q 1053 497 1053 700 m 207 497 q 275 228 207 332 q 472 97 344 125 q 508 142 484 130 q 570 155 533 155 q 665 97 644 155 q 862 228 793 125 q 930 497 930 332 q 862 765 930 663 q 665 895 793 868 q 570 835 644 835 q 472 893 493 835 q 275 765 344 867 q 207 497 207 663 z "
	},
	"ѻ": {
		ha: 913,
		x_min: 78,
		x_max: 835,
		o: "m 835 372 q 759 116 835 214 q 545 -7 683 18 q 517 -59 538 -43 q 457 -74 495 -74 q 398 -59 418 -74 q 371 -7 377 -45 q 156 117 233 18 q 78 372 78 217 q 154 628 78 530 q 371 750 230 726 q 397 802 376 788 q 457 817 418 817 q 517 802 495 817 q 545 749 538 787 q 758 624 682 723 q 835 372 835 525 m 195 372 q 237 181 195 252 q 372 89 279 110 q 401 129 380 117 q 457 140 421 140 q 543 90 527 140 q 676 182 634 111 q 718 372 718 254 q 542 653 718 611 q 512 614 533 625 q 457 603 492 603 q 401 614 421 603 q 372 654 380 625 q 237 563 279 633 q 195 372 195 492 z "
	},
	"Ѽ": {
		ha: 1388,
		x_min: 85,
		x_max: 1301,
		o: "m 978 1350 l 978 1268 l 966 1268 q 861 1280 909 1268 q 772 1307 813 1293 q 697 1334 732 1322 q 633 1346 663 1346 q 581 1329 602 1346 q 561 1276 561 1311 l 561 1265 l 476 1265 l 476 1284 q 515 1392 476 1354 q 631 1429 555 1429 q 708 1417 670 1429 q 786 1390 746 1404 q 872 1362 827 1375 q 970 1350 917 1350 l 978 1350 m 476 87 q 594 110 536 87 q 693 175 652 132 q 788 112 730 136 q 909 87 846 87 q 1108 201 1037 87 q 1179 518 1179 314 q 1123 800 1179 699 q 968 900 1067 900 q 895 885 927 900 q 833 850 863 871 l 786 949 q 972 1004 868 1004 q 1214 877 1127 1004 q 1301 519 1301 749 q 1204 127 1301 267 q 930 -14 1106 -14 q 802 0 860 -14 q 693 46 745 14 q 586 1 642 16 q 454 -14 529 -14 q 181 126 278 -14 q 85 519 85 267 q 172 878 85 751 q 414 1004 259 1004 q 600 949 518 1004 l 553 850 q 492 885 524 871 q 419 900 460 900 q 263 800 319 900 q 207 518 207 700 q 278 200 207 313 q 476 87 349 87 m 783 1200 q 623 1048 783 1090 l 623 1097 q 704 1161 704 1125 q 693 1181 704 1175 q 671 1193 683 1187 q 648 1207 659 1198 q 637 1235 637 1216 q 700 1284 637 1284 q 761 1261 738 1284 q 783 1200 783 1238 z "
	},
	"ѽ": {
		ha: 1122,
		x_min: 78,
		x_max: 1044,
		o: "m 724 -14 q 559 31 623 -14 q 400 -14 497 -14 q 163 85 248 -14 q 78 367 78 184 q 148 656 78 556 q 345 757 218 757 q 427 746 387 757 q 494 718 467 735 l 456 624 q 347 659 395 659 q 232 586 269 659 q 195 368 195 513 q 401 85 195 85 q 561 139 485 85 q 641 98 603 111 q 724 85 678 85 q 928 368 928 85 q 890 586 928 513 q 775 659 853 659 q 667 624 727 659 l 628 718 q 695 746 655 735 q 778 757 736 757 q 975 656 905 757 q 1044 367 1044 555 q 961 86 1044 186 q 724 -14 877 -14 m 822 1140 l 822 1059 l 811 1059 q 706 1071 754 1059 q 616 1097 657 1083 q 542 1124 576 1112 q 478 1136 507 1136 q 406 1067 406 1136 l 406 1055 l 321 1055 l 321 1076 q 360 1182 321 1144 q 475 1219 399 1219 q 552 1207 514 1219 q 631 1180 591 1194 q 715 1153 671 1165 q 812 1140 760 1140 l 822 1140 m 628 990 q 467 839 628 881 l 467 888 q 547 953 547 915 q 537 972 547 966 q 515 984 527 978 q 492 998 503 989 q 482 1025 482 1006 q 545 1075 482 1075 q 605 1052 583 1075 q 628 990 628 1029 z "
	},
	"Ѿ": {
		ha: 1367,
		x_min: 64,
		x_max: 1301,
		o: "m 962 1218 l 962 1200 l 908 1083 l 886 1083 l 852 1153 l 726 1153 l 693 1083 l 671 1083 l 637 1153 l 510 1153 l 478 1083 l 456 1083 l 401 1200 l 401 1218 l 962 1218 m 968 900 q 896 886 927 900 q 833 851 864 872 l 786 950 q 974 1004 870 1004 q 1215 877 1128 1004 q 1301 519 1301 749 q 1204 126 1301 267 q 932 -14 1107 -14 q 795 3 853 -14 q 685 52 736 19 l 679 52 q 568 2 626 18 q 433 -14 509 -14 q 160 127 257 -14 q 64 519 64 269 q 151 878 64 751 q 393 1004 238 1004 q 581 950 495 1004 l 533 851 q 471 886 503 872 q 399 900 439 900 q 243 799 299 900 q 187 518 187 699 q 258 200 187 313 q 458 87 330 87 q 547 100 503 87 q 625 139 591 113 l 625 449 l 741 449 l 741 139 q 908 87 810 87 q 1107 199 1036 87 q 1179 518 1179 311 q 1123 800 1179 699 q 968 900 1067 900 z "
	},
	"ѿ": {
		ha: 1137,
		x_min: 0,
		x_max: 1061,
		o: "m 818 979 l 818 961 l 762 844 l 742 844 l 708 914 l 581 914 l 547 844 l 526 844 l 493 914 l 366 914 l 332 844 l 311 844 l 257 961 l 257 979 l 818 979 m 547 310 l 521 245 l 406 0 l 261 0 l 0 743 l 117 743 l 283 272 q 338 81 310 197 l 343 81 q 363 132 351 104 q 501 425 375 159 l 385 743 l 505 743 q 700 173 685 222 q 722 94 715 124 l 728 94 q 897 367 845 219 q 949 743 949 514 l 1061 743 q 995 327 1061 498 q 781 0 929 155 l 657 0 l 572 235 q 547 310 550 297 z "
	},
	"Ҁ": {
		ha: 888,
		x_min: 85,
		x_max: 848,
		o: "m 570 903 q 306 794 404 903 q 208 496 208 686 q 298 195 208 296 q 564 93 389 93 q 677 109 639 93 l 677 -334 l 562 -334 l 562 -14 l 549 -14 q 204 118 324 -14 q 85 497 85 250 q 144 765 85 649 q 315 943 203 881 q 572 1006 426 1006 q 848 949 733 1006 l 800 850 q 570 903 684 903 z "
	},
	"ҁ": {
		ha: 677,
		x_min: 78,
		x_max: 631,
		o: "m 427 -14 q 166 84 254 -14 q 78 367 78 181 q 171 655 78 554 q 433 757 263 757 q 543 745 487 757 q 631 718 598 734 l 597 623 q 431 657 501 657 q 253 588 311 657 q 195 368 195 518 q 253 151 195 219 q 427 83 311 83 q 490 89 463 83 q 555 113 517 95 l 555 -334 l 443 -334 l 443 -14 l 427 -14 z "
	},
	"҂": {
		ha: 846,
		x_min: 72,
		x_max: 774,
		o: "m 349 212 l 225 -3 l 143 43 l 267 261 l 72 374 l 117 452 l 313 338 l 452 579 l 256 692 l 301 771 l 497 657 l 623 872 l 703 827 l 578 610 l 774 497 l 728 419 l 533 532 l 395 292 l 589 178 l 545 100 l 349 212 z "
	},
	"҃": {
		ha: 774,
		x_min: 138,
		x_max: 637,
		o: "m 265 857 q 247 810 261 827 q 200 793 233 793 q 152 810 165 793 q 138 863 138 827 q 152 912 138 899 q 203 926 166 926 l 507 926 q 526 974 511 958 q 572 990 541 990 q 622 974 607 990 q 637 922 637 957 q 623 871 637 884 q 571 857 608 857 l 265 857 z "
	},
	"҄": {
		ha: 802,
		x_min: 168,
		x_max: 669,
		o: "m 176 935 q 359 975 258 935 q 515 1014 460 1014 q 630 976 591 1014 q 669 871 669 939 l 669 850 l 585 850 l 585 861 q 513 930 585 930 q 449 918 484 930 q 373 891 414 906 q 284 865 332 877 q 179 852 236 852 l 168 852 l 168 935 l 176 935 z "
	},
	"҅": {
		ha: 802,
		x_min: 325,
		x_max: 486,
		o: "m 325 993 q 348 1054 325 1031 q 408 1078 370 1078 q 471 1026 471 1078 q 460 999 471 1008 q 438 985 450 991 q 416 974 426 980 q 406 954 406 968 q 486 890 406 918 l 486 840 q 325 993 325 884 z "
	},
	"҆": {
		ha: 802,
		x_min: 326,
		x_max: 488,
		o: "m 488 993 q 326 840 488 884 l 326 890 q 407 954 407 918 q 397 974 407 968 q 374 985 387 980 q 352 999 362 991 q 342 1026 342 1008 q 404 1078 342 1078 q 465 1054 442 1078 q 488 993 488 1031 z "
	},
	"҈": {
		ha: 1373,
		x_min: 28,
		x_max: 1346,
		o: "m 770 835 q 746 880 766 865 q 679 895 726 895 q 609 881 626 895 q 589 835 592 867 l 538 835 q 679 966 545 966 q 780 932 742 966 q 823 835 819 898 l 770 835 m 770 -216 q 746 -172 766 -186 q 679 -157 726 -157 q 609 -171 626 -157 q 589 -216 592 -184 l 538 -216 q 575 -120 541 -156 q 679 -85 609 -85 q 780 -119 741 -85 q 823 -216 819 -154 l 770 -216 m 1162 -28 q 1139 17 1159 3 q 1072 32 1118 32 q 1002 19 1019 32 q 982 -28 985 5 l 930 -28 q 968 69 934 33 q 1072 104 1002 104 q 1174 70 1135 104 q 1217 -28 1213 35 l 1162 -28 m 380 -28 q 357 17 377 3 q 290 32 336 32 q 220 19 237 32 q 200 -28 203 5 l 149 -28 q 186 69 152 33 q 290 104 220 104 q 392 70 353 104 q 435 -28 431 35 l 380 -28 m 1162 642 q 1139 687 1159 672 q 1072 701 1118 701 q 1002 688 1019 701 q 982 642 985 675 l 930 642 q 968 738 934 703 q 1072 774 1002 774 q 1174 739 1135 774 q 1217 642 1213 705 l 1162 642 m 380 642 q 357 687 377 672 q 290 701 336 701 q 220 688 237 701 q 200 642 203 675 l 149 642 q 186 738 152 703 q 290 774 220 774 q 392 739 353 774 q 435 642 431 705 l 380 642 m 1292 311 q 1268 356 1289 341 q 1202 371 1248 371 q 1132 357 1149 371 q 1111 311 1114 344 l 1060 311 q 1202 443 1067 443 q 1303 408 1264 443 q 1346 311 1342 372 l 1292 311 m 260 311 q 236 356 256 342 q 170 371 216 371 q 100 358 117 371 q 79 311 83 345 l 28 311 q 65 408 31 372 q 170 443 100 443 q 271 408 232 443 q 314 311 310 374 l 260 311 z "
	},
	"҉": {
		ha: 1328,
		x_min: 28,
		x_max: 1300,
		o: "m 732 -24 l 739 -33 q 704 -143 728 -78 q 656 -261 680 -208 l 590 -261 q 637 -24 626 -111 l 732 -24 m 597 774 l 590 783 q 628 899 603 831 q 673 1011 652 967 l 739 1011 q 692 774 704 862 l 597 774 m 1063 441 l 1072 449 q 1188 411 1120 436 q 1300 366 1256 387 l 1300 300 q 1063 347 1150 336 l 1063 441 m 265 307 l 256 300 q 146 335 211 311 q 28 383 81 359 l 28 449 q 265 401 178 413 l 265 307 m 899 690 l 900 701 q 1010 758 945 728 q 1120 804 1074 787 l 1166 758 q 965 624 1046 684 l 899 690 m 429 31 l 428 20 q 208 -83 323 -41 l 161 -37 q 363 97 281 37 l 429 31 m 334 595 l 322 596 q 220 815 267 691 l 265 863 q 400 661 349 728 l 334 595 m 993 125 l 1004 123 q 1061 11 1031 77 q 1107 -96 1092 -55 l 1061 -143 q 996 -43 1040 -109 q 926 58 952 23 l 993 125 z "
	},
	"Ҋ": {
		ha: 1069,
		x_min: 136,
		x_max: 1047,
		o: "m 136 991 l 245 991 l 245 439 q 239 219 245 359 l 236 163 l 241 163 l 798 991 l 922 991 l 922 103 l 1047 103 l 950 -258 l 817 -258 l 922 0 l 814 0 l 814 546 q 827 832 814 665 l 821 832 l 262 0 l 136 0 l 136 991 m 529 1071 q 346 1119 403 1071 q 282 1279 289 1168 l 387 1279 q 426 1179 394 1206 q 532 1153 457 1153 q 636 1184 603 1153 q 676 1279 670 1216 l 783 1279 q 714 1121 775 1171 q 529 1071 652 1071 z "
	},
	"ҋ": {
		ha: 893,
		x_min: 119,
		x_max: 880,
		o: "m 225 743 l 225 294 q 218 110 225 202 l 621 743 l 761 743 l 761 97 l 880 97 l 793 -256 l 676 -256 l 761 0 l 656 0 l 656 456 q 661 633 656 563 l 258 0 l 119 0 l 119 743 l 225 743 m 453 842 q 270 890 328 842 q 205 1050 212 939 l 311 1050 q 347 952 316 981 q 456 924 377 924 q 559 954 526 924 q 600 1050 593 985 l 707 1050 q 638 892 699 943 q 453 842 578 842 z "
	},
	"Ҍ": {
		ha: 851,
		x_min: 32,
		x_max: 779,
		o: "m 32 865 l 136 865 l 136 991 l 252 991 l 252 865 l 484 865 l 484 764 l 252 764 l 252 568 l 382 568 q 779 289 779 568 q 686 73 779 146 q 408 0 592 0 l 136 0 l 136 764 l 32 764 l 32 865 m 252 98 l 401 98 q 659 289 659 98 q 596 422 659 378 q 397 465 534 465 l 252 465 l 252 98 z "
	},
	"ҍ": {
		ha: 822,
		x_min: 14,
		x_max: 746,
		o: "m 232 889 l 432 889 l 432 802 l 232 802 l 232 436 l 449 436 q 673 384 600 436 q 746 227 746 332 q 670 58 746 115 q 444 0 594 0 l 119 0 l 119 802 l 14 802 l 14 889 l 119 889 l 119 1055 l 232 1055 l 232 889 m 232 339 l 232 94 l 439 94 q 633 220 633 94 q 589 311 633 282 q 436 339 544 339 l 232 339 z "
	},
	"Ҏ": {
		ha: 848,
		x_min: 136,
		x_max: 777,
		o: "m 777 703 q 738 548 777 616 q 625 441 699 480 l 707 335 l 639 278 l 538 408 q 376 390 469 390 l 252 390 l 252 0 l 136 0 l 136 991 l 403 991 q 683 919 589 991 q 777 703 777 846 m 252 489 l 364 489 q 475 496 423 489 l 401 591 l 475 647 l 570 528 q 656 697 656 576 q 590 844 656 795 q 387 892 524 892 l 252 892 l 252 489 z "
	},
	"ҏ": {
		ha: 851,
		x_min: 119,
		x_max: 774,
		o: "m 465 -14 q 333 13 393 -14 q 232 96 273 40 l 224 96 q 232 -28 232 -5 l 232 -334 l 119 -334 l 119 743 l 211 743 l 228 642 l 233 642 q 334 730 277 703 q 465 757 391 757 q 693 656 613 757 q 774 372 774 555 q 650 45 774 143 l 728 -61 l 660 -115 l 571 3 q 465 -14 523 -14 m 449 661 q 284 598 335 661 q 232 397 233 535 l 232 372 q 284 148 232 216 q 452 81 336 81 q 511 89 484 81 l 429 196 l 501 250 l 589 135 q 657 374 657 205 q 606 590 657 519 q 449 661 555 661 z "
	},
	"Ғ": {
		ha: 732,
		x_min: 32,
		x_max: 700,
		o: "m 700 991 l 700 888 l 252 888 l 252 542 l 539 542 l 539 440 l 252 440 l 252 0 l 136 0 l 136 440 l 32 440 l 32 542 l 136 542 l 136 991 l 700 991 z "
	},
	"ғ": {
		ha: 595,
		x_min: 12,
		x_max: 566,
		o: "m 566 648 l 232 648 l 232 415 l 467 415 l 467 329 l 232 329 l 232 0 l 119 0 l 119 329 l 12 329 l 12 415 l 119 415 l 119 743 l 566 743 l 566 648 z "
	},
	"Ҕ": {
		ha: 892,
		x_min: 136,
		x_max: 843,
		o: "m 380 444 q 252 436 313 444 l 252 0 l 136 0 l 136 991 l 707 991 l 707 888 l 252 888 l 252 538 q 395 545 313 545 q 727 430 612 545 q 843 102 843 315 q 746 -230 843 -112 q 473 -347 649 -347 q 375 -340 417 -347 q 285 -314 332 -333 l 285 -211 q 461 -244 368 -244 q 721 102 721 -244 q 634 356 721 268 q 380 444 547 444 z "
	},
	"ҕ": {
		ha: 729,
		x_min: 119,
		x_max: 690,
		o: "m 395 -340 q 228 -300 296 -340 l 228 -199 q 390 -241 307 -241 q 526 -173 480 -241 q 572 41 572 -104 q 512 263 572 193 q 329 334 452 334 q 232 322 282 334 l 232 0 l 119 0 l 119 743 l 571 743 l 571 647 l 232 647 l 232 424 q 328 432 288 432 q 601 336 512 432 q 690 40 690 240 q 613 -241 690 -141 q 395 -340 536 -340 z "
	},
	"Җ": {
		ha: 1236,
		x_min: 1,
		x_max: 1210,
		o: "m 406 510 l 16 991 l 144 991 l 530 511 l 530 991 l 642 991 l 642 511 l 1028 991 l 1157 991 l 766 511 l 1088 104 l 1210 104 l 1210 -258 l 1100 -258 l 1100 0 l 1036 0 l 642 503 l 642 0 l 530 0 l 530 503 l 136 0 l 1 0 l 406 510 z "
	},
	"җ": {
		ha: 1083,
		x_min: 3,
		x_max: 1063,
		o: "m 458 743 l 562 743 l 562 383 l 869 743 l 993 743 l 682 383 l 932 97 l 1063 97 l 1063 -256 l 953 -256 l 953 0 l 889 0 l 562 378 l 562 0 l 458 0 l 458 378 l 132 0 l 3 0 l 339 383 l 29 743 l 153 743 l 458 383 l 458 743 z "
	},
	"Ҙ": {
		ha: 807,
		x_min: 50,
		x_max: 730,
		o: "m 711 760 q 649 601 711 662 q 478 524 587 540 l 478 518 q 666 440 602 501 q 730 281 730 379 q 627 64 730 142 q 334 -14 523 -14 q 50 40 161 -14 l 50 147 q 191 99 115 116 q 336 82 267 82 q 539 133 471 82 q 608 279 608 184 q 532 417 608 370 q 304 464 456 464 l 157 464 l 157 561 l 298 561 q 514 611 437 561 q 590 750 590 661 q 535 866 590 823 q 387 909 480 909 q 252 892 313 909 q 113 825 192 876 l 56 903 q 209 980 124 954 q 386 1006 294 1006 q 625 939 538 1006 q 711 760 711 872 m 385 -161 q 290 0 385 -92 l 374 0 q 481 -171 481 -81 q 446 -268 481 -233 q 345 -302 412 -302 q 267 -292 300 -302 l 267 -218 q 325 -225 288 -225 q 367 -209 350 -225 q 385 -161 385 -193 z "
	},
	"ҙ": {
		ha: 671,
		x_min: 46,
		x_max: 607,
		o: "m 261 435 q 472 551 472 435 q 301 661 472 661 q 206 652 249 661 q 94 614 163 642 l 54 705 q 306 757 170 757 q 508 704 434 757 q 582 561 582 652 q 443 397 582 436 l 443 392 q 568 320 528 367 q 607 204 607 274 q 524 44 607 101 q 294 -14 441 -14 q 46 35 134 -14 l 46 138 q 297 79 170 79 q 445 113 395 79 q 494 210 494 146 q 283 339 494 339 l 180 339 l 180 435 l 261 435 m 330 -161 q 236 0 330 -92 l 319 0 q 427 -171 427 -81 q 392 -268 427 -233 q 290 -302 357 -302 q 212 -292 246 -302 l 212 -218 q 271 -225 234 -225 q 313 -209 296 -225 q 330 -161 330 -193 z "
	},
	"Қ": {
		ha: 918,
		x_min: 136,
		x_max: 897,
		o: "m 781 104 l 897 104 l 897 -258 l 787 -258 l 787 0 l 718 0 l 355 481 l 252 389 l 252 0 l 136 0 l 136 991 l 252 991 l 252 500 l 701 991 l 838 991 l 439 561 l 781 104 z "
	},
	"қ": {
		ha: 757,
		x_min: 119,
		x_max: 736,
		o: "m 553 743 l 676 743 l 355 385 l 615 97 l 736 97 l 736 -257 l 628 -257 l 628 0 l 571 0 l 232 376 l 232 0 l 119 0 l 119 743 l 232 743 l 232 382 l 553 743 z "
	},
	"Ҝ": {
		ha: 852,
		x_min: 136,
		x_max: 852,
		o: "m 336 461 l 252 389 l 252 0 l 136 0 l 136 991 l 252 991 l 252 496 l 336 590 l 336 827 l 421 827 l 421 684 l 700 991 l 838 991 l 439 561 l 852 0 l 717 0 l 421 410 l 421 203 l 336 203 l 336 461 z "
	},
	"ҝ": {
		ha: 741,
		x_min: 119,
		x_max: 734,
		o: "m 313 294 l 232 376 l 232 0 l 119 0 l 119 743 l 232 743 l 232 382 l 313 464 l 313 688 l 401 688 l 401 549 l 585 743 l 708 743 l 401 421 l 401 349 l 734 0 l 603 0 l 400 214 l 400 64 l 313 64 l 313 294 z "
	},
	"Ҟ": {
		ha: 852,
		x_min: 32,
		x_max: 852,
		o: "m 32 871 l 136 871 l 136 991 l 252 991 l 252 871 l 401 871 l 401 768 l 252 768 l 252 496 l 700 991 l 838 991 l 439 561 l 852 0 l 713 0 l 355 481 l 252 390 l 252 0 l 136 0 l 136 768 l 32 768 l 32 871 z "
	},
	"ҟ": {
		ha: 718,
		x_min: 14,
		x_max: 713,
		o: "m 14 929 l 119 929 l 119 1055 l 231 1055 l 231 929 l 489 929 l 489 843 l 231 843 l 231 479 l 229 418 l 227 380 l 232 380 l 244 397 q 295 463 281 447 q 558 743 309 479 l 693 743 l 392 427 l 713 0 l 578 0 l 315 350 l 231 278 l 231 0 l 119 0 l 119 843 l 14 843 l 14 929 z "
	},
	"Ҡ": {
		ha: 957,
		x_min: 11,
		x_max: 957,
		o: "m 11 991 l 355 991 l 355 496 l 804 991 l 942 991 l 543 561 l 957 0 l 821 0 l 460 481 l 355 389 l 355 0 l 240 0 l 240 888 l 11 888 l 11 991 z "
	},
	"ҡ": {
		ha: 854,
		x_min: 28,
		x_max: 848,
		o: "m 28 743 l 376 743 l 376 382 l 699 743 l 822 743 l 501 383 l 848 0 l 717 0 l 376 376 l 376 0 l 265 0 l 265 648 l 28 648 l 28 743 z "
	},
	"Ң": {
		ha: 1036,
		x_min: 136,
		x_max: 999,
		o: "m 889 104 l 999 104 l 999 -258 l 889 -258 l 889 0 l 774 0 l 774 467 l 252 467 l 252 0 l 136 0 l 136 991 l 252 991 l 252 570 l 774 570 l 774 991 l 889 991 l 889 104 z "
	},
	"ң": {
		ha: 900,
		x_min: 119,
		x_max: 863,
		o: "m 232 743 l 232 432 l 648 432 l 648 743 l 761 743 l 761 97 l 863 97 l 863 -256 l 750 -256 l 750 0 l 648 0 l 648 335 l 232 335 l 232 0 l 119 0 l 119 743 l 232 743 z "
	},
	"Ҥ": {
		ha: 1129,
		x_min: 136,
		x_max: 1117,
		o: "m 1117 888 l 889 888 l 889 0 l 772 0 l 772 467 l 252 467 l 252 0 l 136 0 l 136 991 l 252 991 l 252 570 l 772 570 l 772 991 l 1117 991 l 1117 888 z "
	},
	"ҥ": {
		ha: 1022,
		x_min: 119,
		x_max: 999,
		o: "m 232 743 l 232 432 l 648 432 l 648 743 l 999 743 l 999 648 l 761 648 l 761 0 l 648 0 l 648 335 l 232 335 l 232 0 l 119 0 l 119 743 l 232 743 z "
	},
	"Ҧ": {
		ha: 1482,
		x_min: 136,
		x_max: 1409,
		o: "m 842 0 l 726 0 l 726 888 l 252 888 l 252 0 l 136 0 l 136 991 l 842 991 l 842 536 q 972 545 888 545 q 1294 428 1180 545 q 1409 100 1409 311 q 1313 -230 1409 -113 q 1042 -347 1217 -347 q 852 -314 936 -347 l 852 -211 q 1029 -244 943 -244 q 1286 102 1286 -244 q 1208 354 1286 266 q 972 443 1130 443 q 901 440 944 443 q 842 435 858 438 l 842 0 z "
	},
	"ҧ": {
		ha: 1202,
		x_min: 119,
		x_max: 1156,
		o: "m 884 -340 q 729 -300 795 -340 l 729 -199 q 876 -241 803 -241 q 1039 41 1039 -241 q 983 263 1039 193 q 810 334 926 334 q 715 319 764 334 l 715 0 l 602 0 l 602 646 l 232 646 l 232 0 l 119 0 l 119 743 l 715 743 l 715 422 q 811 432 766 432 q 1067 331 978 432 q 1156 40 1156 231 q 1085 -241 1156 -142 q 884 -340 1014 -340 z "
	},
	"Ҩ": {
		ha: 1082,
		x_min: 85,
		x_max: 1021,
		o: "m 993 460 q 946 232 993 337 q 821 69 899 127 q 926 49 865 49 q 1021 64 979 49 l 1021 -40 q 921 -57 983 -57 q 700 11 800 -57 q 533 -14 631 -14 q 204 120 323 -14 q 85 485 85 253 q 196 873 85 740 q 521 1007 308 1007 q 669 983 607 1007 l 637 885 q 519 904 581 904 q 207 490 207 904 q 293 191 207 295 q 539 87 380 87 q 607 94 576 87 q 517 260 549 164 q 486 465 486 357 q 553 721 486 631 q 739 811 621 811 q 927 721 862 811 q 993 460 993 631 m 873 456 q 839 645 873 576 q 741 714 804 714 q 642 646 677 714 q 607 458 607 578 q 638 277 607 363 q 726 134 670 191 q 834 266 795 179 q 873 456 873 352 z "
	},
	"ҩ": {
		ha: 889,
		x_min: 78,
		x_max: 838,
		o: "m 509 338 q 532 219 509 274 q 597 129 555 165 q 672 212 644 159 q 700 340 700 266 q 608 500 700 500 q 534 457 559 500 q 509 338 509 415 m 751 -39 q 564 14 652 -39 q 415 -14 498 -14 q 237 34 314 -14 q 119 167 161 81 q 78 364 78 253 q 162 654 78 551 q 400 757 246 757 q 514 742 462 757 l 489 648 q 399 661 452 661 q 244 590 293 661 q 195 363 195 519 q 252 152 195 223 q 421 81 310 81 q 464 84 446 81 q 486 89 482 87 q 392 342 392 189 q 449 525 392 459 q 608 590 506 590 q 762 526 709 590 q 815 342 815 462 q 779 186 815 257 q 679 72 743 115 q 760 54 714 54 q 838 64 804 54 l 838 -29 q 751 -39 811 -39 z "
	},
	"Ҫ": {
		ha: 876,
		x_min: 85,
		x_max: 835,
		o: "m 561 903 q 303 794 397 903 q 208 496 208 685 q 299 195 208 301 q 559 89 391 89 q 796 126 663 89 l 796 25 q 542 -14 693 -14 q 204 119 323 -14 q 85 497 85 252 q 142 766 85 650 q 308 943 199 881 q 562 1006 416 1006 q 835 949 718 1006 l 786 850 q 561 903 673 903 m 524 -161 q 429 0 524 -92 l 513 0 q 620 -171 620 -81 q 585 -268 620 -233 q 484 -302 551 -302 q 406 -292 439 -302 l 406 -218 q 464 -225 427 -225 q 506 -209 489 -225 q 524 -161 524 -193 z "
	},
	"ҫ": {
		ha: 661,
		x_min: 78,
		x_max: 615,
		o: "m 416 -14 q 166 86 255 -14 q 78 367 78 185 q 168 655 78 553 q 424 757 258 757 q 531 745 477 757 q 615 718 585 734 l 581 623 q 499 647 543 637 q 421 657 455 657 q 195 368 195 657 q 250 158 195 231 q 414 85 305 85 q 604 125 507 85 l 604 25 q 416 -14 530 -14 m 414 -161 q 319 0 414 -92 l 403 0 q 510 -171 510 -81 q 475 -268 510 -233 q 374 -302 441 -302 q 296 -292 330 -302 l 296 -218 q 354 -225 317 -225 q 396 -209 379 -225 q 414 -161 414 -193 z "
	},
	"Ҭ": {
		ha: 768,
		x_min: 11,
		x_max: 755,
		o: "m 325 888 l 11 888 l 11 991 l 755 991 l 755 888 l 441 888 l 441 104 l 551 104 l 551 -258 l 441 -258 l 441 0 l 325 0 l 325 888 z "
	},
	"ҭ": {
		ha: 648,
		x_min: 28,
		x_max: 619,
		o: "m 267 648 l 28 648 l 28 743 l 619 743 l 619 648 l 379 648 l 379 97 l 481 97 l 481 -256 l 368 -256 l 368 0 l 267 0 l 267 648 z "
	},
	"Ү": {
		ha: 778,
		x_min: 0,
		x_max: 778,
		o: "m 389 496 l 653 991 l 778 991 l 447 385 l 447 0 l 330 0 l 330 379 l 0 991 l 126 991 l 389 496 z "
	},
	"ү": {
		ha: 696,
		x_min: 0,
		x_max: 696,
		o: "m 404 -334 l 292 -334 l 292 -3 l 0 743 l 117 743 l 277 328 q 346 111 333 177 l 351 111 q 421 328 374 205 l 579 743 l 696 743 l 404 -3 l 404 -334 z "
	},
	"Ұ": {
		ha: 778,
		x_min: 0,
		x_max: 778,
		o: "m 389 496 l 653 991 l 778 991 l 447 385 l 447 345 l 650 345 l 650 241 l 447 241 l 447 0 l 330 0 l 330 241 l 126 241 l 126 345 l 330 345 l 330 379 l 0 991 l 126 991 l 389 496 z "
	},
	"ұ": {
		ha: 696,
		x_min: 0,
		x_max: 696,
		o: "m 404 -87 l 404 -334 l 292 -334 l 292 -87 l 103 -87 l 103 0 l 290 0 l 0 743 l 117 743 l 277 328 q 346 111 333 177 l 351 111 q 421 328 374 205 l 579 743 l 696 743 l 406 0 l 591 0 l 591 -87 l 404 -87 z "
	},
	"Ҳ": {
		ha: 860,
		x_min: 5,
		x_max: 839,
		o: "m 729 104 l 839 104 l 839 -258 l 729 -258 l 729 0 l 665 0 l 399 436 l 127 0 l 5 0 l 335 518 l 28 991 l 155 991 l 401 599 l 650 991 l 772 991 l 465 522 l 729 104 z "
	},
	"ҳ": {
		ha: 753,
		x_min: 26,
		x_max: 732,
		o: "m 298 380 l 40 743 l 168 743 l 364 458 l 559 743 l 686 743 l 428 380 l 631 97 l 732 97 l 732 -257 l 619 -257 l 619 0 l 572 0 l 364 301 l 154 0 l 26 0 l 298 380 z "
	},
	"Ҵ": {
		ha: 1187,
		x_min: 11,
		x_max: 1156,
		o: "m 1040 104 l 1156 104 l 1156 -258 l 1046 -258 l 1046 0 l 300 0 l 300 888 l 11 888 l 11 991 l 737 991 l 737 888 l 415 888 l 415 104 l 925 104 l 925 991 l 1040 991 l 1040 104 z "
	},
	"ҵ": {
		ha: 996,
		x_min: 28,
		x_max: 971,
		o: "m 603 647 l 358 647 l 358 97 l 753 97 l 753 742 l 865 742 l 865 97 l 971 97 l 971 -256 l 859 -256 l 859 0 l 245 0 l 245 647 l 28 647 l 28 742 l 603 742 l 603 647 z "
	},
	"Ҷ": {
		ha: 961,
		x_min: 115,
		x_max: 939,
		o: "m 829 104 l 939 104 l 939 -258 l 829 -258 l 829 0 l 714 0 l 714 410 q 546 360 613 374 q 407 347 479 347 q 191 412 267 347 q 115 597 115 476 l 115 991 l 231 991 l 231 611 q 274 490 231 529 q 414 450 317 450 q 539 460 479 450 q 714 507 600 470 l 714 991 l 829 991 l 829 104 z "
	},
	"ҷ": {
		ha: 846,
		x_min: 106,
		x_max: 827,
		o: "m 218 743 l 218 472 q 367 342 218 342 q 485 361 429 342 q 612 425 541 380 l 612 743 l 725 743 l 725 97 l 827 97 l 827 -257 l 714 -257 l 714 0 l 612 0 l 612 336 q 480 268 541 288 q 343 248 420 248 q 169 306 232 248 q 106 464 106 364 l 106 743 l 218 743 z "
	},
	"Ҹ": {
		ha: 965,
		x_min: 115,
		x_max: 829,
		o: "m 427 347 q 115 597 115 347 l 115 991 l 231 991 l 231 611 q 276 490 231 530 q 427 450 322 450 l 427 686 l 511 686 l 511 454 q 713 504 602 463 l 713 991 l 829 991 l 829 0 l 713 0 l 713 407 q 511 351 599 363 l 511 144 l 427 144 l 427 347 z "
	},
	"ҹ": {
		ha: 833,
		x_min: 106,
		x_max: 714,
		o: "m 218 743 l 218 472 q 354 342 218 346 l 354 546 l 435 546 l 435 349 q 602 425 511 365 l 602 743 l 714 743 l 714 0 l 602 0 l 602 336 q 435 257 515 275 l 435 89 l 354 89 l 354 248 l 339 248 q 168 305 231 248 q 106 464 106 363 l 106 743 l 218 743 z "
	},
	"Һ": {
		ha: 965,
		x_min: 136,
		x_max: 850,
		o: "m 136 991 l 252 991 l 252 582 q 558 644 425 644 q 774 579 699 644 q 850 395 850 515 l 850 0 l 734 0 l 734 380 q 691 502 734 462 q 551 542 648 542 q 416 530 479 542 q 252 485 353 519 l 252 0 l 136 0 l 136 991 z "
	},
	"һ": {
		ha: 812,
		x_min: 119,
		x_max: 739,
		o: "m 625 0 l 625 269 q 478 400 625 400 q 366 384 418 400 q 232 317 313 367 l 232 0 l 119 0 l 119 743 l 232 743 l 232 406 q 359 473 296 452 q 500 494 422 494 q 675 437 610 494 q 739 278 739 380 l 739 0 l 625 0 z "
	},
	"Ҽ": {
		ha: 1164,
		x_min: 41,
		x_max: 1084,
		o: "m 41 612 q 60 699 41 662 l 158 699 q 144 624 144 665 q 221 554 144 554 l 244 554 l 248 583 q 380 896 267 785 q 682 1007 493 1007 q 984 879 884 1007 q 1084 500 1084 750 l 1084 453 l 366 453 q 464 181 375 274 q 720 89 552 89 q 857 99 788 89 q 1021 139 925 110 l 1021 33 q 868 -3 943 7 q 705 -14 793 -14 q 374 107 492 -14 q 243 453 256 228 q 94 493 146 453 q 41 612 41 533 m 681 904 q 458 813 539 904 q 366 554 376 723 l 961 554 q 892 819 961 734 q 681 904 823 904 z "
	},
	"ҽ": {
		ha: 916,
		x_min: 35,
		x_max: 844,
		o: "m 571 -14 q 313 83 406 -14 q 215 351 219 180 q 35 504 35 354 q 52 585 35 551 l 147 585 q 134 513 134 553 q 205 443 134 443 l 220 443 l 224 467 q 332 680 247 604 q 541 755 416 755 q 763 666 681 755 q 844 422 844 576 l 844 351 l 332 351 q 394 151 336 217 q 570 85 452 85 q 692 96 638 85 q 812 135 746 106 l 812 35 q 700 -2 753 9 q 571 -14 646 -14 m 538 661 q 396 606 447 661 q 335 443 345 550 l 724 443 q 676 605 724 549 q 538 661 629 661 z "
	},
	"Ҿ": {
		ha: 1164,
		x_min: 41,
		x_max: 1084,
		o: "m 629 -8 q 355 129 454 12 q 243 453 256 246 q 94 493 146 453 q 41 612 41 533 q 60 699 41 662 l 158 699 q 144 624 144 665 q 221 554 144 554 l 244 554 l 248 583 q 380 896 267 785 q 682 1007 493 1007 q 984 879 884 1007 q 1084 500 1084 750 l 1084 453 l 366 453 q 464 181 375 274 q 720 89 552 89 q 857 99 788 89 q 1021 139 925 110 l 1021 33 q 742 -12 901 -9 l 742 -258 l 629 -258 l 629 -8 m 681 904 q 458 813 539 904 q 366 554 376 723 l 961 554 q 892 819 961 734 q 681 904 823 904 z "
	},
	"ҿ": {
		ha: 916,
		x_min: 35,
		x_max: 844,
		o: "m 492 -7 q 291 107 362 14 q 215 351 219 199 q 35 504 35 354 q 52 585 35 551 l 147 585 q 134 513 134 553 q 205 443 134 443 l 220 443 l 224 467 q 332 680 247 604 q 541 755 416 755 q 763 666 681 755 q 844 422 844 576 l 844 351 l 332 351 q 394 151 336 217 q 570 85 452 85 q 692 96 638 85 q 812 135 746 106 l 812 35 q 604 -12 716 -9 l 604 -256 l 492 -256 l 492 -7 m 538 661 q 396 606 447 661 q 335 443 345 550 l 724 443 q 676 605 724 549 q 538 661 629 661 z "
	},
	"Ӂ": {
		ha: 1173,
		x_min: 1,
		x_max: 1169,
		o: "m 406 510 l 16 991 l 144 991 l 530 511 l 530 991 l 642 991 l 642 511 l 1028 991 l 1157 991 l 766 511 l 1169 0 l 1036 0 l 642 503 l 642 0 l 530 0 l 530 503 l 136 0 l 1 0 l 406 510 m 581 1072 q 397 1121 455 1072 q 333 1280 340 1169 l 439 1280 q 476 1182 445 1210 q 583 1154 507 1154 q 687 1184 653 1154 q 728 1280 720 1214 l 834 1280 q 766 1123 826 1173 q 581 1072 705 1072 z "
	},
	"ӂ": {
		ha: 1022,
		x_min: 3,
		x_max: 1019,
		o: "m 458 742 l 562 742 l 562 382 l 869 742 l 993 742 l 682 382 l 1019 0 l 889 0 l 562 376 l 562 0 l 458 0 l 458 376 l 132 0 l 3 0 l 339 382 l 29 742 l 153 742 l 458 382 l 458 742 m 507 842 q 324 890 382 842 q 260 1050 267 939 l 366 1050 q 402 951 372 979 q 510 924 433 924 q 613 954 580 924 q 654 1050 647 983 l 761 1050 q 692 892 753 943 q 507 842 632 842 z "
	},
	"Ӄ": {
		ha: 957,
		x_min: 136,
		x_max: 885,
		o: "m 411 431 q 252 410 316 431 l 252 0 l 136 0 l 136 991 l 252 991 l 252 511 l 692 991 l 831 991 l 400 528 l 418 529 q 765 419 645 529 q 885 102 885 309 q 785 -229 885 -111 q 504 -347 685 -347 q 406 -340 448 -347 q 317 -314 364 -334 l 317 -211 q 503 -244 399 -244 q 697 -153 629 -244 q 765 102 765 -62 q 671 345 765 259 q 411 431 577 431 z "
	},
	"ӄ": {
		ha: 762,
		x_min: 119,
		x_max: 717,
		o: "m 231 0 l 119 0 l 119 743 l 231 743 l 231 392 l 558 743 l 682 743 l 372 416 q 631 317 546 414 q 717 40 717 221 q 680 -165 717 -80 q 573 -296 642 -251 q 414 -340 504 -340 q 257 -300 321 -340 l 257 -201 q 325 -231 288 -218 q 410 -244 361 -244 q 553 -171 501 -244 q 604 39 604 -97 q 541 252 604 182 q 349 322 477 322 q 231 306 293 322 l 231 0 z "
	},
	"Ӆ": {
		ha: 981,
		x_min: 0,
		x_max: 966,
		o: "m 842 104 l 966 104 l 869 -258 l 736 -258 l 842 0 l 726 0 l 726 889 l 404 889 l 383 726 q 310 251 342 401 q 227 42 279 101 q 90 -16 176 -17 q 0 1 40 -16 l 0 98 q 77 81 37 81 q 139 110 113 81 q 187 213 166 140 q 239 489 208 286 q 309 991 271 693 l 842 991 l 842 104 z "
	},
	"ӆ": {
		ha: 794,
		x_min: 11,
		x_max: 791,
		o: "m 672 97 l 791 97 l 704 -256 l 587 -256 l 672 0 l 559 0 l 559 646 l 335 646 q 284 262 316 401 q 201 57 252 122 q 69 -8 149 -8 q 11 -1 30 -8 l 11 85 q 45 81 26 81 q 168 241 121 81 q 237 742 214 401 l 672 742 l 672 97 z "
	},
	"Ӈ": {
		ha: 1010,
		x_min: 136,
		x_max: 889,
		o: "m 889 102 q 793 -231 889 -114 q 521 -347 698 -347 q 424 -341 465 -347 q 330 -314 382 -334 l 330 -212 q 505 -245 414 -245 q 774 96 774 -245 l 774 467 l 252 467 l 252 0 l 136 0 l 136 991 l 252 991 l 252 570 l 774 570 l 774 991 l 889 991 l 889 102 z "
	},
	"ӈ": {
		ha: 856,
		x_min: 119,
		x_max: 761,
		o: "m 490 -340 q 338 -301 401 -340 l 338 -200 q 482 -241 413 -241 q 607 -174 567 -241 q 647 33 647 -107 l 647 335 l 232 335 l 232 0 l 119 0 l 119 743 l 232 743 l 232 432 l 647 432 l 647 743 l 761 743 l 761 35 q 691 -244 761 -147 q 490 -340 621 -340 z "
	},
	"Ӊ": {
		ha: 1035,
		x_min: 136,
		x_max: 1014,
		o: "m 889 104 l 1014 104 l 916 -258 l 782 -258 l 889 0 l 774 0 l 774 467 l 252 467 l 252 0 l 136 0 l 136 991 l 252 991 l 252 570 l 774 570 l 774 991 l 889 991 l 889 104 z "
	},
	"ӊ": {
		ha: 907,
		x_min: 119,
		x_max: 880,
		o: "m 232 742 l 232 432 l 648 432 l 648 742 l 761 742 l 761 97 l 880 97 l 793 -256 l 676 -256 l 761 0 l 648 0 l 648 335 l 232 335 l 232 0 l 119 0 l 119 742 l 232 742 z "
	},
	"Ӌ": {
		ha: 965,
		x_min: 115,
		x_max: 829,
		o: "m 829 0 l 714 0 l 714 -258 l 604 -258 l 604 104 l 714 104 l 714 410 q 546 360 613 374 q 407 347 479 347 q 191 412 267 347 q 115 597 115 476 l 115 991 l 231 991 l 231 611 q 274 490 231 529 q 414 450 317 450 q 539 460 479 450 q 714 507 600 470 l 714 991 l 829 991 l 829 0 z "
	},
	"ӌ": {
		ha: 844,
		x_min: 106,
		x_max: 725,
		o: "m 218 743 l 218 472 q 367 342 218 342 q 485 361 429 342 q 612 425 541 380 l 612 743 l 725 743 l 725 0 l 624 0 l 624 -257 l 511 -257 l 511 97 l 612 97 l 612 336 q 480 268 541 288 q 343 248 420 248 q 169 306 232 248 q 106 464 106 364 l 106 743 l 218 743 z "
	},
	"Ӎ": {
		ha: 1255,
		x_min: 136,
		x_max: 1243,
		o: "m 575 0 l 239 879 l 233 879 l 238 793 q 243 631 243 663 l 243 0 l 136 0 l 136 991 l 310 991 l 625 174 l 631 174 l 946 991 l 1118 991 l 1118 104 l 1243 104 l 1146 -258 l 1011 -258 l 1118 0 l 1003 0 l 1003 639 q 1013 878 1003 728 l 1007 878 l 668 0 l 575 0 z "
	},
	"ӎ": {
		ha: 1022,
		x_min: 119,
		x_max: 1019,
		o: "m 505 109 l 526 172 l 555 252 l 757 742 l 900 742 l 900 97 l 1019 97 l 932 -256 l 815 -256 l 900 0 l 800 0 l 800 614 l 787 574 l 747 471 l 555 0 l 461 0 l 269 472 q 220 614 233 563 l 220 0 l 119 0 l 119 742 l 257 742 l 458 252 q 505 109 489 177 z "
	},
	"Ӑ": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 435 1071 q 252 1119 309 1071 q 187 1279 194 1168 l 293 1279 q 330 1181 299 1208 q 437 1153 361 1153 q 541 1183 507 1153 q 582 1279 574 1213 l 688 1279 q 620 1121 680 1172 q 435 1071 559 1071 z "
	},
	"ӑ": {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 380 842 q 197 890 254 842 q 132 1050 139 939 l 238 1050 q 275 951 244 979 q 382 924 306 924 q 486 954 452 924 q 527 1050 519 983 l 633 1050 q 565 892 625 943 q 380 842 505 842 z "
	},
	"Ӓ": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 251 1174 q 269 1225 251 1209 q 312 1240 287 1240 q 356 1225 338 1240 q 375 1174 375 1209 q 356 1123 375 1140 q 312 1107 338 1107 q 269 1123 287 1107 q 251 1174 251 1140 m 509 1174 q 527 1225 509 1209 q 570 1240 545 1240 q 614 1225 595 1240 q 633 1174 633 1209 q 614 1123 633 1140 q 570 1107 595 1107 q 527 1123 545 1107 q 509 1174 509 1140 z "
	},
	"ӓ": {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 201 945 q 219 996 201 980 q 262 1011 237 1011 q 306 996 288 1011 q 325 945 325 980 q 306 894 325 911 q 262 878 288 878 q 219 894 237 878 q 201 945 201 911 m 459 945 q 477 996 459 980 q 520 1011 495 1011 q 564 996 545 1011 q 583 945 583 980 q 564 894 583 911 q 520 878 545 878 q 477 894 495 878 q 459 945 459 911 z "
	},
	"Ӕ": {
		ha: 1213,
		x_min: -1,
		x_max: 1129,
		o: "m 1129 0 l 621 0 l 621 315 l 272 315 l 118 0 l -1 0 l 472 991 l 1129 991 l 1129 889 l 736 889 l 736 570 l 1103 570 l 1103 468 l 736 468 l 736 103 l 1129 103 l 1129 0 m 318 419 l 621 419 l 621 889 l 541 889 l 318 419 z "
	},
	"ӕ": {
		ha: 1192,
		x_min: 64,
		x_max: 1120,
		o: "m 64 205 q 148 375 64 315 q 404 441 232 436 l 529 446 l 529 492 q 490 621 529 579 q 370 663 450 663 q 161 606 272 663 l 126 692 q 244 738 176 720 q 378 755 311 755 q 522 726 466 755 q 606 632 578 696 q 700 724 642 692 q 832 757 758 757 q 1041 666 962 757 q 1120 425 1120 576 l 1120 353 l 644 353 q 863 85 650 85 q 978 97 924 85 q 1088 135 1031 109 l 1088 35 q 979 -2 1029 9 q 860 -14 928 -14 q 579 144 664 -14 q 457 22 524 58 q 300 -14 391 -14 q 126 44 189 -14 q 64 205 64 102 m 182 204 q 218 110 182 140 q 314 79 254 79 q 469 137 412 79 q 526 298 526 194 l 526 366 l 419 361 q 237 318 293 355 q 182 204 182 281 m 831 663 q 702 606 749 663 q 647 443 654 550 l 999 443 q 956 606 999 549 q 831 663 912 663 z "
	},
	"Ӗ": {
		ha: 772,
		x_min: 136,
		x_max: 689,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 m 407 1071 q 224 1119 281 1071 q 159 1279 166 1168 l 265 1279 q 302 1181 271 1208 q 410 1153 333 1153 q 513 1183 479 1153 q 554 1279 547 1213 l 661 1279 q 592 1121 652 1172 q 407 1071 532 1071 z "
	},
	"ӗ": {
		ha: 779,
		x_min: 78,
		x_max: 707,
		o: "m 433 -14 q 173 87 269 -14 q 78 366 78 187 q 166 651 78 545 q 404 757 255 757 q 625 665 544 757 q 707 422 707 573 l 707 351 l 195 351 q 261 153 198 220 q 437 85 323 85 q 675 135 557 85 l 675 35 q 562 -2 615 9 q 433 -14 509 -14 m 403 663 q 260 604 313 663 q 197 443 207 546 l 586 443 q 538 606 586 549 q 403 663 491 663 m 404 842 q 221 890 279 842 q 157 1050 163 939 l 262 1050 q 299 951 269 979 q 407 924 330 924 q 510 954 477 924 q 551 1050 544 983 l 658 1050 q 589 892 650 943 q 404 842 529 842 z "
	},
	"Ә": {
		ha: 1014,
		x_min: 79,
		x_max: 928,
		o: "m 450 904 q 143 852 296 904 l 143 960 q 292 996 221 985 q 454 1007 363 1007 q 803 872 679 1007 q 928 494 928 736 q 810 122 928 257 q 484 -14 693 -14 q 79 493 79 -14 l 79 541 l 806 541 q 705 812 794 721 q 450 904 616 904 m 484 89 q 710 180 626 89 q 806 439 795 271 l 203 439 q 272 174 203 260 q 484 89 341 89 z "
	},
	"ә": {
		ha: 777,
		x_min: 69,
		x_max: 699,
		o: "m 343 757 q 604 655 509 757 q 699 376 699 553 q 611 92 699 197 q 375 -14 523 -14 q 152 77 234 -14 q 69 321 69 168 l 69 392 l 582 392 q 518 594 578 530 q 345 657 457 657 q 222 646 278 657 q 102 607 166 635 l 102 707 q 217 745 162 734 q 343 757 272 757 m 376 81 q 519 137 467 81 q 579 298 571 193 l 191 298 q 238 136 191 192 q 376 81 286 81 z "
	},
	"Ӛ": {
		ha: 1014,
		x_min: 79,
		x_max: 928,
		o: "m 450 904 q 143 852 296 904 l 143 960 q 292 996 221 985 q 454 1007 363 1007 q 803 872 679 1007 q 928 494 928 736 q 810 122 928 257 q 484 -14 693 -14 q 79 493 79 -14 l 79 541 l 806 541 q 705 812 794 721 q 450 904 616 904 m 484 89 q 710 180 626 89 q 806 439 795 271 l 203 439 q 272 174 203 260 q 484 89 341 89 m 309 1174 q 327 1225 309 1209 q 370 1240 345 1240 q 415 1225 396 1240 q 433 1174 433 1209 q 415 1123 433 1140 q 370 1107 396 1107 q 327 1123 345 1107 q 309 1174 309 1140 m 568 1174 q 586 1225 568 1209 q 629 1240 604 1240 q 672 1225 654 1240 q 691 1174 691 1209 q 672 1123 691 1140 q 629 1107 654 1107 q 586 1123 604 1107 q 568 1174 568 1140 z "
	},
	"ӛ": {
		ha: 777,
		x_min: 69,
		x_max: 699,
		o: "m 343 757 q 604 655 509 757 q 699 376 699 553 q 611 92 699 197 q 375 -14 523 -14 q 152 77 234 -14 q 69 321 69 168 l 69 392 l 582 392 q 518 594 578 530 q 345 657 457 657 q 222 646 278 657 q 102 607 166 635 l 102 707 q 217 745 162 734 q 343 757 272 757 m 376 81 q 519 137 467 81 q 579 298 571 193 l 191 298 q 238 136 191 192 q 376 81 286 81 m 195 945 q 213 996 195 980 q 256 1011 231 1011 q 300 996 281 1011 q 319 945 319 980 q 300 894 319 911 q 256 878 281 878 q 213 894 231 878 q 195 945 195 911 m 453 945 q 471 996 453 980 q 514 1011 489 1011 q 558 996 539 1011 q 576 945 576 980 q 558 894 576 911 q 514 878 539 878 q 471 894 489 878 q 453 945 453 911 z "
	},
	"Ӝ": {
		ha: 1173,
		x_min: 1,
		x_max: 1169,
		o: "m 406 510 l 16 991 l 144 991 l 530 511 l 530 991 l 642 991 l 642 511 l 1028 991 l 1157 991 l 766 511 l 1169 0 l 1036 0 l 642 503 l 642 0 l 530 0 l 530 503 l 136 0 l 1 0 l 406 510 m 394 1174 q 412 1225 394 1209 q 455 1240 430 1240 q 499 1225 481 1240 q 518 1174 518 1209 q 499 1123 518 1140 q 455 1107 481 1107 q 412 1123 430 1107 q 394 1174 394 1140 m 652 1174 q 670 1225 652 1209 q 713 1240 688 1240 q 757 1225 739 1240 q 776 1174 776 1209 q 757 1123 776 1140 q 713 1107 739 1107 q 670 1123 688 1107 q 652 1174 652 1140 z "
	},
	"ӝ": {
		ha: 1022,
		x_min: 3,
		x_max: 1019,
		o: "m 458 742 l 562 742 l 562 382 l 869 742 l 993 742 l 682 382 l 1019 0 l 889 0 l 562 376 l 562 0 l 458 0 l 458 376 l 132 0 l 3 0 l 339 382 l 29 742 l 153 742 l 458 382 l 458 742 m 319 945 q 337 996 319 980 q 380 1011 355 1011 q 425 996 406 1011 q 444 945 444 980 q 425 894 444 911 q 380 878 406 878 q 337 894 355 878 q 319 945 319 911 m 578 945 q 596 996 578 980 q 639 1011 614 1011 q 683 996 664 1011 q 701 945 701 980 q 683 894 701 911 q 639 878 664 878 q 596 894 614 878 q 578 945 578 911 z "
	},
	"Ӟ": {
		ha: 807,
		x_min: 50,
		x_max: 730,
		o: "m 711 760 q 649 601 711 662 q 478 524 587 540 l 478 518 q 666 440 602 501 q 730 281 730 379 q 627 64 730 142 q 334 -14 523 -14 q 50 40 161 -14 l 50 147 q 191 99 115 116 q 336 82 267 82 q 539 133 471 82 q 608 279 608 184 q 532 417 608 370 q 304 464 456 464 l 157 464 l 157 561 l 298 561 q 514 611 437 561 q 590 750 590 661 q 535 866 590 823 q 387 909 480 909 q 252 892 313 909 q 113 825 192 876 l 56 903 q 209 980 124 954 q 386 1006 294 1006 q 625 939 538 1006 q 711 760 711 872 m 201 1174 q 219 1225 201 1209 q 262 1240 237 1240 q 306 1225 288 1240 q 325 1174 325 1209 q 306 1123 325 1140 q 262 1107 288 1107 q 219 1123 237 1107 q 201 1174 201 1140 m 459 1174 q 477 1225 459 1209 q 520 1240 495 1240 q 564 1225 545 1240 q 583 1174 583 1209 q 564 1123 583 1140 q 520 1107 545 1107 q 477 1123 495 1107 q 459 1174 459 1140 z "
	},
	"ӟ": {
		ha: 671,
		x_min: 46,
		x_max: 607,
		o: "m 261 435 q 472 551 472 435 q 301 661 472 661 q 206 652 249 661 q 94 614 163 642 l 54 705 q 306 757 170 757 q 508 704 434 757 q 582 561 582 652 q 443 397 582 436 l 443 392 q 568 320 528 367 q 607 204 607 274 q 524 44 607 101 q 294 -14 441 -14 q 46 35 134 -14 l 46 138 q 297 79 170 79 q 445 113 395 79 q 494 210 494 146 q 283 339 494 339 l 180 339 l 180 435 l 261 435 m 136 945 q 154 996 136 980 q 197 1011 172 1011 q 242 996 223 1011 q 260 945 260 980 q 242 894 260 911 q 197 878 223 878 q 154 894 172 878 q 136 945 136 911 m 395 945 q 413 996 395 980 q 456 1011 431 1011 q 499 996 481 1011 q 518 945 518 980 q 499 894 518 911 q 456 878 481 878 q 413 894 431 878 q 395 945 395 911 z "
	},
	"Ӡ": {
		ha: 810,
		x_min: 50,
		x_max: 732,
		o: "m 345 564 q 633 486 534 557 q 732 283 732 414 q 628 65 732 144 q 335 -14 524 -14 q 50 40 161 -14 l 50 147 q 191 99 115 116 q 338 82 266 82 q 541 134 473 82 q 610 283 610 186 q 533 421 610 374 q 306 468 457 468 l 211 468 l 211 564 l 546 888 l 78 888 l 78 991 l 690 991 l 690 897 l 345 564 z "
	},
	"ӡ": {
		ha: 679,
		x_min: 18,
		x_max: 633,
		o: "m 290 314 q 469 271 391 309 q 590 163 547 232 q 633 3 633 95 q 533 -241 633 -148 q 271 -334 433 -334 q 18 -286 112 -334 l 18 -182 q 278 -240 142 -240 q 454 -176 387 -240 q 521 -3 521 -111 q 448 166 521 109 q 239 224 376 224 l 157 224 l 157 301 l 464 647 l 36 647 l 36 743 l 595 743 l 595 660 l 290 314 z "
	},
	"Ӣ": {
		ha: 1059,
		x_min: 138,
		x_max: 924,
		o: "m 138 991 l 245 991 l 245 440 q 236 164 245 288 l 241 164 l 798 991 l 924 991 l 924 0 l 815 0 l 815 546 q 827 832 815 682 l 821 832 l 264 0 l 138 0 l 138 991 m 326 1164 l 733 1164 l 733 1071 l 326 1071 l 326 1164 z "
	},
	"ӣ": {
		ha: 880,
		x_min: 119,
		x_max: 761,
		o: "m 225 743 l 225 272 l 220 149 l 218 110 l 621 743 l 761 743 l 761 0 l 656 0 l 656 454 l 658 544 l 661 632 l 260 0 l 119 0 l 119 743 l 225 743 m 237 935 l 644 935 l 644 842 l 237 842 l 237 935 z "
	},
	"Ӥ": {
		ha: 1059,
		x_min: 138,
		x_max: 924,
		o: "m 138 991 l 245 991 l 245 440 q 236 164 245 288 l 241 164 l 798 991 l 924 991 l 924 0 l 815 0 l 815 546 q 827 832 815 682 l 821 832 l 264 0 l 138 0 l 138 991 m 338 1174 q 356 1225 338 1209 q 399 1240 374 1240 q 444 1225 425 1240 q 463 1174 463 1209 q 444 1123 463 1140 q 399 1107 425 1107 q 356 1123 374 1107 q 338 1174 338 1140 m 597 1174 q 615 1225 597 1209 q 658 1240 633 1240 q 702 1225 683 1240 q 720 1174 720 1209 q 702 1123 720 1140 q 658 1107 683 1107 q 615 1123 633 1107 q 597 1174 597 1140 z "
	},
	"ӥ": {
		ha: 880,
		x_min: 119,
		x_max: 761,
		o: "m 225 743 l 225 272 l 220 149 l 218 110 l 621 743 l 761 743 l 761 0 l 656 0 l 656 454 l 658 544 l 661 632 l 260 0 l 119 0 l 119 743 l 225 743 m 251 945 q 269 996 251 980 q 312 1011 287 1011 q 356 996 338 1011 q 375 945 375 980 q 356 894 375 911 q 312 878 338 878 q 269 894 287 878 q 251 945 251 911 m 509 945 q 527 996 509 980 q 570 1011 545 1011 q 614 996 595 1011 q 633 945 633 980 q 614 894 633 911 q 570 878 595 878 q 527 894 545 878 q 509 945 509 911 z "
	},
	"Ӧ": {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 207 497 q 293 192 207 296 q 542 87 378 87 q 791 191 707 87 q 875 497 875 295 q 791 801 875 697 q 543 904 707 904 q 293 800 378 904 q 207 497 207 696 m 351 1174 q 369 1225 351 1209 q 412 1240 387 1240 q 457 1225 438 1240 q 475 1174 475 1209 q 457 1123 475 1140 q 412 1107 438 1107 q 369 1123 387 1107 q 351 1174 351 1140 m 610 1174 q 628 1225 610 1209 q 671 1240 646 1240 q 714 1225 696 1240 q 733 1174 733 1209 q 714 1123 733 1140 q 671 1107 696 1107 q 628 1123 646 1107 q 610 1174 610 1140 z "
	},
	"ӧ": {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 195 372 q 252 155 195 230 q 419 81 309 81 q 587 155 530 81 q 644 372 644 229 q 587 588 644 514 q 418 661 530 661 q 251 589 307 661 q 195 372 195 516 m 229 945 q 247 996 229 980 q 290 1011 265 1011 q 335 996 316 1011 q 353 945 353 980 q 335 894 353 911 q 290 878 316 878 q 247 894 265 878 q 229 945 229 911 m 488 945 q 506 996 488 980 q 549 1011 524 1011 q 592 996 574 1011 q 611 945 611 980 q 592 894 611 911 q 549 878 574 878 q 506 894 524 878 q 488 945 488 911 z "
	},
	"Ө": {
		ha: 1083,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 542 87 q 781 180 697 87 q 873 454 865 273 l 208 454 q 301 180 217 273 q 542 87 386 87 m 543 904 q 306 817 391 904 q 208 555 221 729 l 872 555 q 778 816 861 728 q 543 904 695 904 z "
	},
	"ө": {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 419 81 q 582 143 526 81 q 644 335 637 205 l 195 335 q 255 145 201 209 q 419 81 309 81 m 418 661 q 258 603 312 661 q 196 431 205 544 l 643 431 q 578 603 633 545 q 418 661 524 661 z "
	},
	"Ӫ": {
		ha: 1083,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 542 87 q 781 180 697 87 q 873 454 865 273 l 208 454 q 301 180 217 273 q 542 87 386 87 m 543 904 q 306 817 391 904 q 208 555 221 729 l 872 555 q 778 816 861 728 q 543 904 695 904 m 351 1174 q 369 1225 351 1209 q 412 1240 387 1240 q 457 1225 438 1240 q 475 1174 475 1209 q 457 1123 475 1140 q 412 1107 438 1107 q 369 1123 387 1107 q 351 1174 351 1140 m 610 1174 q 628 1225 610 1209 q 671 1240 646 1240 q 714 1225 696 1240 q 733 1174 733 1209 q 714 1123 733 1140 q 671 1107 696 1107 q 628 1123 646 1107 q 610 1174 610 1140 z "
	},
	"ӫ": {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 419 81 q 582 143 526 81 q 644 335 637 205 l 195 335 q 255 145 201 209 q 419 81 309 81 m 418 661 q 258 603 312 661 q 196 431 205 544 l 643 431 q 578 603 633 545 q 418 661 524 661 m 228 945 q 246 996 228 980 q 289 1011 264 1011 q 333 996 315 1011 q 352 945 352 980 q 333 894 352 911 q 289 878 315 878 q 246 894 264 878 q 228 945 228 911 m 486 945 q 504 996 486 980 q 547 1011 522 1011 q 591 996 572 1011 q 610 945 610 980 q 591 894 610 911 q 547 878 572 878 q 504 894 522 878 q 486 945 486 911 z "
	},
	"Ӭ": {
		ha: 875,
		x_min: 41,
		x_max: 787,
		o: "m 317 903 q 90 851 200 903 l 41 949 q 318 1006 158 1006 q 571 946 465 1006 q 732 776 677 886 q 787 515 787 665 q 664 123 787 260 q 309 -14 541 -14 q 49 25 155 -14 l 49 126 q 163 100 105 112 q 289 89 222 89 q 566 187 473 89 q 665 469 660 285 l 178 469 l 178 572 l 664 572 q 557 815 649 728 q 317 903 465 903 m 197 1174 q 215 1225 197 1209 q 258 1240 233 1240 q 302 1225 283 1240 q 321 1174 321 1209 q 302 1123 321 1140 q 258 1107 283 1107 q 215 1123 233 1107 q 197 1174 197 1140 m 455 1174 q 473 1225 455 1209 q 516 1240 491 1240 q 560 1225 541 1240 q 578 1174 578 1209 q 560 1123 578 1140 q 516 1107 541 1107 q 473 1123 491 1107 q 455 1174 455 1140 z "
	},
	"ӭ": {
		ha: 684,
		x_min: 39,
		x_max: 606,
		o: "m 232 -14 q 39 25 119 -14 l 39 125 q 127 97 79 109 q 236 85 174 85 q 418 148 354 85 q 489 336 482 211 l 113 336 l 113 432 l 488 432 q 419 603 477 549 q 253 657 362 657 q 81 621 183 657 l 49 715 q 142 745 86 733 q 252 757 197 757 q 515 657 425 757 q 606 371 606 558 q 508 87 606 189 q 232 -14 410 -14 m 132 945 q 150 996 132 980 q 193 1011 168 1011 q 238 996 219 1011 q 256 945 256 980 q 238 894 256 911 q 193 878 219 878 q 150 894 168 878 q 132 945 132 911 m 391 945 q 409 996 391 980 q 452 1011 427 1011 q 495 996 477 1011 q 514 945 514 980 q 495 894 514 911 q 452 878 477 878 q 409 894 427 878 q 391 945 391 911 z "
	},
	"Ӯ": {
		ha: 863,
		x_min: 18,
		x_max: 863,
		o: "m 199 -14 q 66 7 123 -14 l 66 119 q 195 90 130 90 q 314 124 269 90 q 404 253 359 159 l 18 991 l 146 991 l 439 428 q 465 363 456 395 l 471 363 q 497 435 490 420 l 741 991 l 863 991 l 546 282 q 443 98 489 155 q 340 13 397 40 q 199 -14 283 -14 m 236 1164 l 643 1164 l 643 1071 l 236 1071 l 236 1164 z "
	},
	"ӯ": {
		ha: 700,
		x_min: 1,
		x_max: 699,
		o: "m 1 743 l 122 743 l 285 319 q 351 110 338 174 l 357 110 q 394 228 366 144 q 578 743 422 312 l 699 743 l 379 -103 q 268 -281 332 -229 q 113 -334 205 -334 q 11 -322 61 -334 l 11 -232 q 94 -240 48 -240 q 260 -110 210 -240 l 301 -4 l 1 743 m 148 935 l 555 935 l 555 842 l 148 842 l 148 935 z "
	},
	"Ӱ": {
		ha: 863,
		x_min: 18,
		x_max: 863,
		o: "m 199 -14 q 66 7 123 -14 l 66 119 q 195 90 130 90 q 314 124 269 90 q 404 253 359 159 l 18 991 l 146 991 l 439 428 q 465 363 456 395 l 471 363 q 497 435 490 420 l 741 991 l 863 991 l 546 282 q 443 98 489 155 q 340 13 397 40 q 199 -14 283 -14 m 250 1174 q 268 1225 250 1209 q 311 1240 286 1240 q 355 1225 336 1240 q 374 1174 374 1209 q 355 1123 374 1140 q 311 1107 336 1107 q 268 1123 286 1107 q 250 1174 250 1140 m 508 1174 q 526 1225 508 1209 q 569 1240 544 1240 q 613 1225 594 1240 q 631 1174 631 1209 q 613 1123 631 1140 q 569 1107 594 1107 q 526 1123 544 1107 q 508 1174 508 1140 z "
	},
	"ӱ": {
		ha: 700,
		x_min: 1,
		x_max: 699,
		o: "m 1 743 l 122 743 l 285 319 q 351 110 338 174 l 357 110 q 394 228 366 144 q 578 743 422 312 l 699 743 l 379 -103 q 268 -281 332 -229 q 113 -334 205 -334 q 11 -322 61 -334 l 11 -232 q 94 -240 48 -240 q 260 -110 210 -240 l 301 -4 l 1 743 m 160 945 q 178 996 160 980 q 221 1011 196 1011 q 266 996 247 1011 q 284 945 284 980 q 266 894 284 911 q 221 878 247 878 q 178 894 196 878 q 160 945 160 911 m 418 945 q 436 996 418 980 q 479 1011 454 1011 q 523 996 505 1011 q 542 945 542 980 q 523 894 542 911 q 479 878 505 878 q 436 894 454 878 q 418 945 418 911 z "
	},
	"Ӳ": {
		ha: 863,
		x_min: 18,
		x_max: 863,
		o: "m 199 -14 q 66 7 123 -14 l 66 119 q 195 90 130 90 q 314 124 269 90 q 404 253 359 159 l 18 991 l 146 991 l 439 428 q 465 363 456 395 l 471 363 q 497 435 490 420 l 741 991 l 863 991 l 546 282 q 443 98 489 155 q 340 13 397 40 q 199 -14 283 -14 m 252 1088 q 314 1183 277 1120 q 372 1293 351 1246 l 498 1293 l 498 1279 q 415 1170 473 1236 q 318 1071 357 1103 l 252 1071 l 252 1088 m 494 1088 q 562 1194 528 1134 q 614 1293 596 1254 l 740 1293 l 740 1279 q 657 1170 715 1236 q 559 1071 599 1103 l 494 1071 l 494 1088 z "
	},
	"ӳ": {
		ha: 700,
		x_min: 1,
		x_max: 699,
		o: "m 1 743 l 122 743 l 285 319 q 351 110 338 174 l 357 110 q 394 228 366 144 q 578 743 422 312 l 699 743 l 379 -103 q 268 -281 332 -229 q 113 -334 205 -334 q 11 -322 61 -334 l 11 -232 q 94 -240 48 -240 q 260 -110 210 -240 l 301 -4 l 1 743 m 159 859 q 221 954 184 891 q 279 1064 258 1017 l 406 1064 l 406 1050 q 322 941 380 1007 q 225 842 264 874 l 159 842 l 159 859 m 401 859 q 469 965 435 905 q 521 1064 503 1025 l 647 1064 l 647 1050 q 564 941 622 1007 q 467 842 506 874 l 401 842 l 401 859 z "
	},
	"Ӵ": {
		ha: 965,
		x_min: 115,
		x_max: 829,
		o: "m 829 0 l 714 0 l 714 410 q 546 360 613 374 q 407 347 479 347 q 191 412 267 347 q 115 597 115 476 l 115 991 l 231 991 l 231 611 q 274 490 231 529 q 414 450 317 450 q 539 460 479 450 q 714 507 600 470 l 714 991 l 829 991 l 829 0 m 281 1174 q 299 1225 281 1209 q 342 1240 317 1240 q 387 1225 368 1240 q 406 1174 406 1209 q 387 1123 406 1140 q 342 1107 368 1107 q 299 1123 317 1107 q 281 1174 281 1140 m 540 1174 q 558 1225 540 1209 q 601 1240 576 1240 q 645 1225 626 1240 q 663 1174 663 1209 q 645 1123 663 1140 q 601 1107 626 1107 q 558 1123 576 1107 q 540 1174 540 1140 z "
	},
	"ӵ": {
		ha: 844,
		x_min: 106,
		x_max: 725,
		o: "m 218 743 l 218 472 q 367 342 218 342 q 485 361 429 342 q 612 425 541 380 l 612 743 l 725 743 l 725 0 l 612 0 l 612 336 q 480 268 541 288 q 343 248 420 248 q 169 306 232 248 q 106 464 106 364 l 106 743 l 218 743 m 225 945 q 243 996 225 980 q 286 1011 261 1011 q 331 996 312 1011 q 349 945 349 980 q 331 894 349 911 q 286 878 312 878 q 243 894 261 878 q 225 945 225 911 m 484 945 q 502 996 484 980 q 545 1011 519 1011 q 588 996 570 1011 q 607 945 607 980 q 588 894 607 911 q 545 878 570 878 q 502 894 519 878 q 484 945 484 911 z "
	},
	"Ӷ": {
		ha: 732,
		x_min: 136,
		x_max: 700,
		o: "m 136 991 l 700 991 l 700 888 l 252 888 l 252 104 l 361 104 l 361 -258 l 252 -258 l 252 0 l 136 0 l 136 991 z "
	},
	"ӷ": {
		ha: 595,
		x_min: 119,
		x_max: 566,
		o: "m 119 742 l 566 742 l 566 647 l 232 647 l 232 97 l 334 97 l 334 -256 l 221 -256 l 221 0 l 119 0 l 119 742 z "
	},
	"Ӹ": {
		ha: 1185,
		x_min: 136,
		x_max: 1048,
		o: "m 136 0 l 136 991 l 252 991 l 252 568 l 414 568 q 684 499 591 568 q 777 289 777 429 q 689 75 777 149 q 432 0 601 0 l 136 0 m 252 98 l 419 98 q 600 146 542 98 q 657 289 657 194 q 596 424 657 382 q 400 465 536 465 l 252 465 l 252 98 m 1048 0 l 933 0 l 933 991 l 1048 991 l 1048 0 m 401 1174 q 419 1225 401 1209 q 463 1240 437 1240 q 507 1225 488 1240 q 526 1174 526 1209 q 507 1123 526 1140 q 463 1107 488 1107 q 419 1123 437 1107 q 401 1174 401 1140 m 660 1174 q 678 1225 660 1209 q 721 1240 696 1240 q 765 1225 746 1240 q 783 1174 783 1209 q 765 1123 783 1140 q 721 1107 746 1107 q 678 1123 696 1107 q 660 1174 660 1140 z "
	},
	"ӹ": {
		ha: 1069,
		x_min: 119,
		x_max: 950,
		o: "m 232 436 l 435 436 q 645 384 576 436 q 713 227 713 331 q 641 57 713 115 q 428 0 568 0 l 119 0 l 119 743 l 232 743 l 232 436 m 950 0 l 838 0 l 838 743 l 950 743 l 950 0 m 232 339 l 232 94 l 422 94 q 602 220 602 94 q 560 311 602 282 q 419 339 519 339 l 232 339 m 343 945 q 361 996 343 980 q 404 1011 379 1011 q 449 996 430 1011 q 467 945 467 980 q 449 894 467 911 q 404 878 430 878 q 361 894 379 878 q 343 945 343 911 m 602 945 q 620 996 602 980 q 663 1011 637 1011 q 706 996 688 1011 q 725 945 725 980 q 706 894 725 911 q 663 878 688 878 q 620 894 637 878 q 602 945 602 911 z "
	},
	"Ӻ": {
		ha: 732,
		x_min: 32,
		x_max: 700,
		o: "m 700 991 l 700 888 l 252 888 l 252 542 l 539 542 l 539 440 l 252 440 l 252 0 l 136 0 l 136 440 l 32 440 l 32 542 l 136 542 l 136 991 l 700 991 m 350 104 l 350 -92 q 195 -268 350 -268 q 117 -252 157 -268 l 117 -157 q 186 -170 144 -170 q 250 -102 250 -170 l 250 104 l 350 104 z "
	},
	"ӻ": {
		ha: 595,
		x_min: 12,
		x_max: 566,
		o: "m 566 648 l 232 648 l 232 415 l 467 415 l 467 329 l 232 329 l 232 0 l 119 0 l 119 329 l 12 329 l 12 415 l 119 415 l 119 743 l 566 743 l 566 648 m 330 97 l 330 -92 q 175 -268 330 -268 q 96 -252 137 -268 l 96 -157 q 165 -170 124 -170 q 229 -102 229 -170 l 229 97 l 330 97 z "
	},
	"Ӽ": {
		ha: 863,
		x_min: 5,
		x_max: 831,
		o: "m 796 0 l 665 0 l 399 436 l 127 0 l 5 0 l 335 518 l 28 991 l 155 991 l 401 599 l 650 991 l 772 991 l 465 522 l 796 0 m 831 104 l 831 -92 q 676 -268 831 -268 q 597 -252 638 -268 l 597 -157 q 667 -170 625 -170 q 730 -102 730 -170 l 730 104 l 831 104 z "
	},
	"ӽ": {
		ha: 750,
		x_min: 26,
		x_max: 730,
		o: "m 298 380 l 40 743 l 168 743 l 364 458 l 559 743 l 686 743 l 428 380 l 700 0 l 572 0 l 364 301 l 154 0 l 26 0 l 298 380 m 730 97 l 730 -92 q 575 -268 730 -268 q 496 -252 537 -268 l 496 -157 q 566 -170 524 -170 q 629 -102 629 -170 l 629 97 l 730 97 z "
	},
	"Ӿ": {
		ha: 802,
		x_min: 4,
		x_max: 796,
		o: "m 86 578 l 294 578 l 28 991 l 155 991 l 401 597 l 648 991 l 772 991 l 501 578 l 715 578 l 715 475 l 494 475 l 796 0 l 665 0 l 399 436 l 127 0 l 4 0 l 307 475 l 86 475 l 86 578 z "
	},
	"ӿ": {
		ha: 728,
		x_min: 26,
		x_max: 700,
		o: "m 79 428 l 265 428 l 40 743 l 168 743 l 364 458 l 559 743 l 686 743 l 460 428 l 650 428 l 650 340 l 456 340 l 700 0 l 572 0 l 364 301 l 154 0 l 26 0 l 269 340 l 79 340 l 79 428 z "
	},
	"Ԁ": {
		ha: 851,
		x_min: 89,
		x_max: 732,
		o: "m 89 285 q 188 498 89 429 q 482 568 287 568 l 616 568 l 616 991 l 732 991 l 732 0 l 452 0 q 180 72 271 0 q 89 285 89 144 m 616 467 l 490 467 q 274 425 340 467 q 208 286 208 382 q 270 144 208 189 q 469 100 332 100 l 616 100 l 616 467 z "
	},
	"ԁ": {
		ha: 851,
		x_min: 78,
		x_max: 732,
		o: "m 625 100 l 619 100 q 386 -14 541 -14 q 159 86 240 -14 q 78 370 78 186 q 159 655 78 553 q 386 757 241 757 q 618 647 537 757 l 627 647 l 622 701 l 619 753 l 619 1055 l 732 1055 l 732 0 l 640 0 l 625 100 m 400 81 q 567 143 515 81 q 619 346 619 206 l 619 370 q 567 595 619 528 q 399 663 514 663 q 247 586 300 663 q 195 368 195 509 q 247 153 195 226 q 400 81 299 81 z "
	},
	"Ԃ": {
		ha: 1249,
		x_min: 89,
		x_max: 1122,
		o: "m 400 -12 q 168 59 247 -12 q 89 271 89 130 q 190 493 89 418 q 488 568 291 568 l 586 568 l 586 991 l 701 991 l 701 248 q 857 87 701 87 q 966 129 925 87 q 1007 245 1007 171 l 1007 575 l 1122 575 l 1122 244 q 1052 56 1122 126 q 857 -14 982 -14 q 723 14 777 -14 q 635 103 669 42 q 400 -12 558 -12 m 586 465 l 484 465 q 274 418 340 465 q 208 269 208 371 q 404 87 208 87 q 538 128 490 87 q 586 244 586 168 l 586 465 z "
	},
	"ԃ": {
		ha: 1244,
		x_min: 78,
		x_max: 1133,
		o: "m 867 81 q 983 125 947 81 q 1019 262 1019 170 l 1019 475 l 1133 475 l 1133 256 q 1065 53 1133 120 q 869 -14 998 -14 q 728 17 782 -14 q 646 125 675 48 l 640 125 q 527 17 589 48 q 376 -14 464 -14 q 157 87 235 -14 q 78 370 78 188 q 156 654 78 551 q 375 757 235 757 q 501 731 447 757 q 597 647 555 705 l 606 647 q 602 696 604 659 q 599 753 599 734 l 599 1055 l 711 1055 l 711 282 q 748 127 711 174 q 867 81 785 81 m 390 81 q 549 143 500 81 q 599 346 599 206 l 599 370 q 549 594 599 526 q 389 663 498 663 q 195 368 195 663 q 242 152 195 223 q 390 81 289 81 z "
	},
	"Ԅ": {
		ha: 1255,
		x_min: 53,
		x_max: 1129,
		o: "m 292 464 l 155 464 l 155 561 l 286 561 q 489 611 416 561 q 561 750 561 661 q 509 866 561 823 q 370 909 456 909 q 240 891 300 909 q 110 825 180 872 l 53 903 q 199 979 116 951 q 371 1006 283 1006 q 599 939 516 1006 q 682 760 682 873 q 622 602 682 664 q 456 522 562 540 l 456 517 q 700 277 696 482 q 738 132 701 175 q 859 89 774 89 q 977 130 939 89 q 1015 262 1015 171 l 1015 575 l 1129 575 l 1129 256 q 1058 54 1129 121 q 859 -14 987 -14 q 651 57 722 -14 q 579 264 581 128 q 508 415 578 366 q 292 464 439 464 z "
	},
	"ԅ": {
		ha: 1114,
		x_min: 54,
		x_max: 1002,
		o: "m 739 81 q 889 262 889 81 l 889 475 l 1002 475 l 1002 256 q 938 53 1002 119 q 742 -14 875 -14 q 475 212 486 -14 q 424 309 471 279 q 275 339 376 339 l 180 339 l 180 435 l 256 435 q 452 551 452 435 q 288 661 452 661 q 191 649 237 661 q 93 614 145 637 l 54 705 q 167 745 112 732 q 293 757 222 757 q 489 704 418 757 q 561 561 561 652 q 431 397 561 436 l 431 392 q 539 337 498 378 q 585 224 581 296 q 739 81 591 81 z "
	},
	"Ԇ": {
		ha: 868,
		x_min: 53,
		x_max: 836,
		o: "m 610 279 q 532 417 610 370 q 301 464 454 464 l 154 464 l 154 561 l 296 561 q 511 611 435 561 q 587 750 587 661 q 532 866 587 823 q 385 909 476 909 q 247 890 313 909 q 110 825 181 870 l 53 903 q 205 979 119 953 q 380 1006 291 1006 q 615 939 530 1006 q 701 760 701 873 q 637 603 701 665 q 463 524 573 541 l 463 518 q 653 440 587 501 q 720 279 720 380 l 720 104 l 836 104 l 836 -258 l 726 -258 l 726 0 l 610 0 l 610 279 z "
	},
	"ԇ": {
		ha: 726,
		x_min: 54,
		x_max: 705,
		o: "m 492 204 q 282 339 492 339 l 180 339 l 180 435 l 260 435 q 472 551 472 435 q 427 634 472 607 q 301 661 382 661 q 96 614 197 661 l 54 705 q 301 755 163 755 q 503 703 431 755 q 575 559 575 651 q 437 397 575 436 l 437 390 q 561 318 523 365 q 599 203 599 272 l 599 97 l 705 97 l 705 -256 l 593 -256 l 593 0 l 492 0 l 492 204 z "
	},
	"Ԉ": {
		ha: 1362,
		x_min: 0,
		x_max: 1238,
		o: "m 703 889 l 404 889 l 383 726 q 328 347 354 497 q 274 125 302 196 q 202 19 246 54 q 90 -16 158 -16 q 0 1 43 -16 l 0 98 q 77 81 34 81 q 132 102 110 81 q 168 158 153 123 q 202 281 183 193 q 246 534 221 368 q 309 991 271 700 l 818 991 l 818 264 q 856 130 818 171 q 971 89 893 89 q 1085 130 1047 89 q 1124 262 1124 171 l 1124 575 l 1238 575 l 1238 256 q 1168 56 1238 125 q 971 -14 1099 -14 q 770 56 838 -14 q 703 258 703 125 l 703 889 z "
	},
	"ԉ": {
		ha: 1182,
		x_min: 11,
		x_max: 1069,
		o: "m 661 262 q 696 125 661 170 q 812 81 732 81 q 957 261 957 81 l 957 475 l 1069 475 l 1069 256 q 1006 53 1069 119 q 814 -14 943 -14 q 617 53 686 -14 q 549 257 549 119 l 549 646 l 335 646 q 284 262 316 401 q 201 57 252 122 q 69 -8 149 -8 q 11 -1 30 -8 l 11 85 q 45 81 26 81 q 168 241 121 81 q 237 742 214 401 l 661 742 l 661 262 z "
	},
	"Ԋ": {
		ha: 1406,
		x_min: 136,
		x_max: 1279,
		o: "m 861 264 q 899 130 861 171 q 1014 89 936 89 q 1128 130 1090 89 q 1166 262 1166 171 l 1166 575 l 1279 575 l 1279 256 q 1211 56 1279 126 q 1014 -14 1143 -14 q 814 55 882 -14 q 746 258 746 124 l 746 467 l 252 467 l 252 0 l 136 0 l 136 991 l 252 991 l 252 570 l 746 570 l 746 991 l 861 991 l 861 264 z "
	},
	"ԋ": {
		ha: 1268,
		x_min: 119,
		x_max: 1156,
		o: "m 232 743 l 232 432 l 633 432 l 633 743 l 746 743 l 746 264 q 782 126 746 171 q 899 81 818 81 q 1043 262 1043 81 l 1043 475 l 1156 475 l 1156 256 q 1092 53 1156 119 q 899 -14 1029 -14 q 703 54 772 -14 q 633 257 633 121 l 633 335 l 232 335 l 232 0 l 119 0 l 119 743 l 232 743 z "
	},
	"Ԍ": {
		ha: 1050,
		x_min: 85,
		x_max: 972,
		o: "m 590 510 l 972 510 l 972 452 q 868 100 972 214 q 553 -14 764 -14 q 209 122 332 -14 q 85 497 85 258 q 145 768 85 652 q 322 944 206 883 q 596 1006 437 1006 q 758 989 677 1006 q 895 946 838 972 l 850 846 q 716 887 789 871 q 585 903 644 903 q 305 796 404 903 q 207 496 207 689 q 297 195 207 302 q 555 87 388 87 q 851 407 851 87 l 590 407 l 590 510 z "
	},
	"ԍ": {
		ha: 889,
		x_min: 78,
		x_max: 814,
		o: "m 468 390 l 814 390 l 814 345 q 465 -14 814 -14 q 179 86 279 -14 q 78 371 78 186 q 188 655 78 554 q 493 757 297 757 q 755 703 637 757 l 715 614 q 489 664 602 664 q 272 589 350 664 q 195 371 195 513 q 264 155 195 231 q 467 79 333 79 q 641 134 581 79 q 700 294 700 188 l 468 294 l 468 390 z "
	},
	"Ԏ": {
		ha: 986,
		x_min: 11,
		x_max: 860,
		o: "m 11 888 l 11 991 l 746 991 l 746 888 l 431 888 l 431 264 q 471 129 431 171 q 589 87 511 87 q 746 261 746 87 l 746 575 l 860 575 l 860 256 q 788 56 860 125 q 589 -14 717 -14 q 385 56 454 -14 q 315 258 315 126 l 315 888 l 11 888 z "
	},
	"ԏ": {
		ha: 899,
		x_min: 28,
		x_max: 786,
		o: "m 608 647 l 374 647 l 374 262 q 411 125 374 170 q 528 81 448 81 q 673 257 673 81 l 673 475 l 786 475 l 786 256 q 722 53 786 119 q 528 -14 658 -14 q 329 55 397 -14 q 261 257 261 124 l 261 647 l 28 647 l 28 742 l 608 742 l 608 647 z "
	},
	"Ԑ": {
		ha: 810,
		x_min: 75,
		x_max: 754,
		o: "m 106 757 q 195 939 106 871 q 437 1006 285 1006 q 754 908 612 1006 l 690 827 q 558 890 619 873 q 428 908 496 908 q 279 866 333 908 q 225 747 225 825 q 296 611 225 661 q 503 561 367 561 l 650 561 l 650 465 l 505 465 q 276 417 355 465 q 197 275 197 369 q 266 131 197 179 q 458 83 334 83 q 751 146 616 83 l 751 39 q 456 -14 633 -14 q 175 61 274 -14 q 75 269 75 136 q 145 437 75 372 q 343 517 216 502 l 343 522 q 167 600 228 539 q 106 757 106 660 z "
	},
	"ԑ": {
		ha: 660,
		x_min: 61,
		x_max: 612,
		o: "m 485 435 l 485 339 l 385 339 q 174 207 174 339 q 223 113 174 146 q 372 79 273 79 q 486 93 429 79 q 610 138 542 106 l 610 37 q 367 -14 517 -14 q 143 44 224 -14 q 61 201 61 101 q 98 312 61 268 q 224 385 136 357 l 224 392 q 121 454 157 411 q 85 559 85 497 q 161 703 85 650 q 367 757 237 757 q 489 746 433 757 q 612 704 545 734 l 570 614 q 462 652 506 642 q 364 661 418 661 q 195 551 195 661 q 410 435 195 435 l 485 435 z "
	},
	"Ԓ": {
		ha: 972,
		x_min: 0,
		x_max: 941,
		o: "m 842 0 l 726 0 l 726 889 l 404 889 l 383 726 q 310 251 342 401 q 227 42 279 101 q 90 -16 176 -17 q 0 1 40 -16 l 0 98 q 77 81 37 81 q 139 110 113 81 q 187 213 166 140 q 239 489 208 286 q 309 991 271 693 l 842 991 l 842 0 m 941 104 l 941 -92 q 786 -268 941 -268 q 707 -252 748 -268 l 707 -157 q 777 -170 735 -170 q 840 -102 840 -170 l 840 104 l 941 104 z "
	},
	"ԓ": {
		ha: 793,
		x_min: 11,
		x_max: 772,
		o: "m 673 0 l 559 0 l 559 647 l 336 647 q 286 264 318 406 q 201 56 253 122 q 69 -9 149 -9 q 11 -1 33 -9 l 11 82 q 45 78 26 78 q 169 243 123 78 q 239 743 215 408 l 673 743 l 673 0 m 772 97 l 772 -92 q 618 -268 772 -268 q 539 -252 580 -268 l 539 -157 q 608 -170 567 -170 q 672 -102 672 -170 l 672 97 l 772 97 z "
	},
	"Ạ": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 372 -164 q 392 -107 372 -125 q 440 -89 412 -89 q 487 -107 467 -89 q 507 -164 507 -125 q 487 -220 507 -201 q 440 -239 467 -239 q 392 -220 412 -239 q 372 -164 372 -201 z "
	},
	"ạ": {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 296 -164 q 316 -107 296 -125 q 364 -89 336 -89 q 411 -107 391 -89 q 431 -164 431 -125 q 411 -220 431 -201 q 364 -239 391 -239 q 316 -220 336 -239 q 296 -164 296 -201 z "
	},
	"Ả": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 596 1243 q 484 1125 596 1148 l 477 1048 l 406 1048 l 397 1168 q 482 1194 456 1177 q 509 1240 509 1211 q 486 1283 509 1270 q 414 1297 463 1297 q 350 1293 372 1297 l 350 1361 q 422 1368 376 1368 q 596 1243 596 1368 z "
	},
	"ả": {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 538 1014 q 425 896 538 919 l 418 819 l 347 819 l 339 939 q 424 965 397 948 q 450 1011 450 982 q 428 1054 450 1040 q 355 1068 405 1068 q 292 1064 313 1068 l 292 1132 q 364 1139 317 1139 q 538 1014 538 1139 z "
	},
	"Ấ": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 646 1071 l 582 1071 q 439 1183 506 1122 q 296 1071 361 1115 l 232 1071 l 232 1088 l 268 1128 q 379 1264 344 1212 l 498 1264 q 646 1088 543 1200 l 646 1071 m 591 1219 q 677 1357 641 1281 l 794 1357 l 794 1343 q 656 1203 738 1268 l 591 1203 l 591 1219 z "
	},
	"ấ": {
		ha: 772,
		x_min: 64,
		x_max: 739,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 590 842 l 526 842 q 383 954 450 892 q 240 842 306 886 l 176 842 l 176 859 l 212 899 q 323 1035 288 983 l 443 1035 q 590 859 488 971 l 590 842 m 536 990 q 622 1128 585 1052 l 739 1128 l 739 1114 q 601 974 682 1039 l 536 974 l 536 990 z "
	},
	"Ầ": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 643 1071 l 579 1071 q 436 1183 513 1115 q 293 1071 364 1118 l 229 1071 l 229 1086 l 265 1127 q 376 1263 341 1211 l 496 1263 q 643 1086 541 1199 l 643 1071 m 283 1203 l 219 1203 q 81 1343 138 1267 l 81 1357 l 197 1357 q 283 1218 233 1284 l 283 1203 z "
	},
	"ầ": {
		ha: 772,
		x_min: 31,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 593 842 l 529 842 q 386 954 463 886 q 243 842 314 889 l 179 842 l 179 857 l 215 898 q 326 1034 291 981 l 446 1034 q 593 857 490 970 l 593 842 m 233 974 l 169 974 q 31 1114 87 1038 l 31 1128 l 147 1128 q 233 989 182 1055 l 233 974 z "
	},
	"Ẩ": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 637 1071 l 574 1071 q 431 1183 498 1122 q 288 1071 353 1115 l 224 1071 l 224 1088 l 260 1128 q 371 1264 336 1212 l 490 1264 q 637 1088 535 1200 l 637 1071 m 751 1345 q 665 1255 751 1275 l 661 1200 l 607 1200 l 600 1289 q 660 1306 639 1295 q 682 1343 682 1317 q 662 1376 682 1368 q 614 1385 643 1385 q 565 1381 583 1385 l 565 1435 q 619 1439 582 1439 q 751 1345 751 1439 z "
	},
	"ẩ": {
		ha: 772,
		x_min: 64,
		x_max: 710,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 596 842 l 532 842 q 389 954 456 892 q 246 842 312 886 l 182 842 l 182 859 l 218 899 q 330 1035 294 983 l 449 1035 q 596 859 494 971 l 596 842 m 710 1116 q 624 1026 710 1046 l 620 971 l 566 971 l 559 1060 q 619 1077 597 1066 q 640 1114 640 1088 q 621 1147 640 1139 q 572 1156 602 1156 q 524 1152 541 1156 l 524 1206 q 578 1210 541 1210 q 710 1116 710 1210 z "
	},
	"Ẫ": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 533 1307 q 484 1317 508 1307 q 437 1340 460 1327 q 393 1362 414 1352 q 353 1373 372 1373 q 311 1356 326 1373 q 288 1305 297 1339 l 226 1305 q 269 1416 235 1377 q 354 1454 303 1454 q 404 1444 379 1454 q 451 1421 429 1434 q 495 1399 474 1409 q 534 1389 516 1389 q 576 1406 562 1389 q 598 1455 590 1422 l 659 1455 q 618 1346 652 1385 q 533 1307 585 1307 m 648 1071 l 585 1071 q 441 1183 519 1115 q 298 1071 370 1118 l 235 1071 l 235 1086 l 271 1127 q 382 1263 347 1211 l 501 1263 q 648 1086 546 1199 l 648 1071 z "
	},
	"ẫ": {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 476 1078 q 427 1088 451 1078 q 380 1111 403 1098 q 336 1133 357 1123 q 296 1143 315 1143 q 254 1127 269 1143 q 231 1076 240 1110 l 169 1076 q 212 1186 178 1148 q 297 1225 246 1225 q 347 1215 322 1225 q 394 1192 372 1204 q 438 1170 417 1180 q 477 1160 459 1160 q 519 1176 505 1160 q 541 1226 533 1193 l 602 1226 q 561 1117 595 1156 q 476 1078 528 1078 m 591 842 l 528 842 q 385 954 462 886 q 241 842 313 889 l 178 842 l 178 857 l 214 898 q 325 1034 290 981 l 444 1034 q 591 857 489 970 l 591 842 z "
	},
	"Ậ": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 372 -164 q 392 -107 372 -125 q 440 -89 412 -89 q 487 -107 467 -89 q 507 -164 507 -125 q 487 -220 507 -201 q 440 -239 467 -239 q 392 -220 412 -239 q 372 -164 372 -201 m 211 1086 q 332 1222 297 1179 q 382 1293 366 1265 l 494 1293 q 546 1220 509 1265 q 668 1086 583 1175 l 668 1071 l 587 1071 q 437 1197 528 1108 q 289 1071 345 1106 l 211 1071 l 211 1086 z "
	},
	"ậ": {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 296 -164 q 316 -107 296 -125 q 364 -89 336 -89 q 411 -107 391 -89 q 431 -164 431 -125 q 411 -220 431 -201 q 364 -239 391 -239 q 316 -220 336 -239 q 296 -164 296 -201 m 152 857 q 273 993 238 949 q 323 1064 307 1036 l 435 1064 q 487 990 450 1036 q 609 857 524 945 l 609 842 l 528 842 q 378 968 469 879 q 230 842 286 877 l 152 842 l 152 857 z "
	},
	"Ắ": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 397 1263 q 485 1402 452 1333 l 602 1402 l 602 1388 q 463 1247 543 1310 l 397 1247 l 397 1263 m 439 1071 q 242 1247 252 1071 l 311 1247 q 343 1179 317 1198 q 441 1160 369 1160 q 536 1182 507 1160 q 571 1247 566 1204 l 642 1247 q 584 1118 635 1166 q 439 1071 534 1071 z "
	},
	"ắ": {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 342 1034 q 429 1173 396 1104 l 546 1173 l 546 1158 q 407 1018 488 1081 l 342 1018 l 342 1034 m 384 842 q 186 1018 197 842 l 256 1018 q 288 950 262 969 q 385 930 313 930 q 481 952 452 930 q 515 1018 510 975 l 587 1018 q 529 889 579 937 q 384 842 478 842 z "
	},
	"Ằ": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 473 1247 l 410 1247 q 271 1388 329 1310 l 271 1402 l 387 1402 q 473 1263 422 1329 l 473 1247 m 437 1071 q 240 1247 250 1071 l 309 1247 q 341 1179 315 1198 q 439 1160 367 1160 q 534 1182 505 1160 q 569 1247 564 1204 l 640 1247 q 582 1118 633 1166 q 437 1071 532 1071 z "
	},
	"ằ": {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 418 1018 l 355 1018 q 216 1158 274 1081 l 216 1173 l 332 1173 q 418 1034 368 1099 l 418 1018 m 382 842 q 185 1018 195 842 l 254 1018 q 286 950 260 969 q 384 930 312 930 q 479 952 450 930 q 514 1018 509 975 l 585 1018 q 527 889 578 937 q 382 842 477 842 z "
	},
	"Ẳ": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 538 1353 q 452 1265 538 1285 l 448 1238 l 393 1238 l 386 1299 q 447 1316 425 1305 q 469 1352 469 1326 q 450 1385 469 1377 q 401 1394 431 1394 q 351 1390 376 1394 l 351 1443 q 408 1449 366 1449 q 538 1353 538 1449 m 437 1071 q 240 1247 250 1071 l 309 1247 q 341 1179 315 1198 q 439 1160 367 1160 q 534 1182 505 1160 q 569 1247 564 1204 l 640 1247 q 582 1118 633 1166 q 437 1071 532 1071 z "
	},
	"ẳ": {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 489 1124 q 403 1036 489 1056 l 399 1008 l 343 1008 l 336 1069 q 397 1086 375 1076 q 420 1122 420 1097 q 400 1156 420 1147 q 351 1165 381 1165 q 302 1161 326 1165 l 302 1214 q 359 1219 317 1219 q 489 1124 489 1219 m 388 842 q 191 1018 201 842 l 260 1018 q 292 950 266 969 q 389 930 317 930 q 485 952 456 930 q 519 1018 514 975 l 591 1018 q 533 889 583 937 q 388 842 482 842 z "
	},
	"Ẵ": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 532 1305 q 483 1316 507 1305 q 436 1338 458 1326 q 392 1360 413 1350 q 351 1371 370 1371 q 310 1354 324 1371 q 286 1304 296 1338 l 224 1304 q 267 1413 233 1375 q 353 1451 301 1451 q 402 1441 378 1451 q 450 1419 427 1431 q 494 1396 473 1407 q 533 1386 515 1386 q 574 1403 560 1386 q 597 1453 589 1419 l 658 1453 q 617 1344 650 1383 q 532 1305 583 1305 m 433 1071 q 236 1247 246 1071 l 305 1247 q 337 1179 311 1198 q 435 1160 363 1160 q 530 1182 501 1160 q 565 1247 559 1204 l 636 1247 q 578 1118 629 1166 q 433 1071 528 1071 z "
	},
	"ẵ": {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 481 1076 q 432 1086 456 1076 q 385 1109 408 1097 q 341 1131 362 1121 q 300 1141 319 1141 q 259 1125 273 1141 q 235 1075 245 1109 l 174 1075 q 216 1184 182 1145 q 302 1222 250 1222 q 352 1212 327 1222 q 399 1190 376 1202 q 443 1167 422 1177 q 482 1157 464 1157 q 524 1174 509 1157 q 546 1223 538 1190 l 607 1223 q 566 1115 600 1154 q 481 1076 532 1076 m 382 842 q 185 1018 195 842 l 254 1018 q 286 950 260 969 q 384 930 312 930 q 479 952 450 930 q 514 1018 509 975 l 585 1018 q 527 889 578 937 q 382 842 477 842 z "
	},
	"Ặ": {
		ha: 879,
		x_min: 0,
		x_max: 879,
		o: "m 760 0 l 636 315 l 239 315 l 117 0 l 0 0 l 392 996 l 489 996 l 879 0 l 760 0 m 600 419 l 485 726 q 439 869 463 785 q 396 726 424 804 l 279 419 l 600 419 m 436 1083 q 288 1129 341 1083 q 229 1265 235 1176 l 300 1265 q 318 1213 304 1232 q 356 1185 332 1194 q 439 1176 381 1176 q 540 1198 507 1176 q 579 1265 572 1219 l 650 1265 q 586 1131 643 1179 q 436 1083 530 1083 m 372 -164 q 392 -107 372 -125 q 440 -89 412 -89 q 487 -107 467 -89 q 507 -164 507 -125 q 487 -220 507 -201 q 440 -239 467 -239 q 392 -220 412 -239 q 372 -164 372 -201 z "
	},
	"ặ": {
		ha: 772,
		x_min: 64,
		x_max: 660,
		o: "m 576 0 l 554 106 l 549 106 q 438 11 493 36 q 300 -14 382 -14 q 126 43 189 -14 q 64 205 64 100 q 424 441 64 431 l 550 446 l 550 492 q 512 621 550 579 q 392 663 475 663 q 182 606 299 663 l 147 692 q 268 739 202 722 q 399 755 333 755 q 596 696 532 755 q 660 507 660 637 l 660 0 l 576 0 m 322 79 q 487 137 427 79 q 547 298 547 195 l 547 366 l 435 361 q 241 319 300 356 q 182 204 182 282 q 219 111 182 143 q 322 79 256 79 m 378 842 q 231 888 283 842 q 172 1023 178 935 l 242 1023 q 260 971 246 990 q 299 943 274 952 q 381 935 323 935 q 482 956 450 935 q 522 1023 515 978 l 592 1023 q 529 890 585 938 q 378 842 472 842 m 296 -164 q 316 -107 296 -125 q 364 -89 336 -89 q 411 -107 391 -89 q 431 -164 431 -125 q 411 -220 431 -201 q 364 -239 391 -239 q 316 -220 336 -239 q 296 -164 296 -201 z "
	},
	"Ẹ": {
		ha: 772,
		x_min: 136,
		x_max: 689,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 m 345 -164 q 365 -107 345 -125 q 413 -89 385 -89 q 460 -107 440 -89 q 479 -164 479 -125 q 460 -220 479 -201 q 413 -239 440 -239 q 365 -220 385 -239 q 345 -164 345 -201 z "
	},
	"ẹ": {
		ha: 779,
		x_min: 78,
		x_max: 707,
		o: "m 433 -14 q 173 87 269 -14 q 78 366 78 187 q 166 651 78 545 q 404 757 255 757 q 625 665 544 757 q 707 422 707 573 l 707 351 l 195 351 q 261 153 198 220 q 437 85 323 85 q 675 135 557 85 l 675 35 q 562 -2 615 9 q 433 -14 509 -14 m 403 663 q 260 604 313 663 q 197 443 207 546 l 586 443 q 538 606 586 549 q 403 663 491 663 m 338 -164 q 358 -107 338 -125 q 407 -89 378 -89 q 454 -107 434 -89 q 473 -164 473 -125 q 454 -220 473 -201 q 407 -239 434 -239 q 358 -220 378 -239 q 338 -164 338 -201 z "
	},
	"Ẻ": {
		ha: 772,
		x_min: 136,
		x_max: 689,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 m 567 1243 q 454 1125 567 1148 l 448 1048 l 376 1048 l 368 1168 q 453 1194 427 1177 q 479 1240 479 1211 q 457 1283 479 1270 q 385 1297 434 1297 q 321 1293 342 1297 l 321 1361 q 393 1368 347 1368 q 567 1243 567 1368 z "
	},
	"ẻ": {
		ha: 779,
		x_min: 78,
		x_max: 707,
		o: "m 433 -14 q 173 87 269 -14 q 78 366 78 187 q 166 651 78 545 q 404 757 255 757 q 625 665 544 757 q 707 422 707 573 l 707 351 l 195 351 q 261 153 198 220 q 437 85 323 85 q 675 135 557 85 l 675 35 q 562 -2 615 9 q 433 -14 509 -14 m 403 663 q 260 604 313 663 q 197 443 207 546 l 586 443 q 538 606 586 549 q 403 663 491 663 m 562 1014 q 449 896 562 919 l 442 819 l 371 819 l 363 939 q 448 965 421 948 q 474 1011 474 982 q 451 1054 474 1040 q 379 1068 429 1068 q 315 1064 337 1068 l 315 1132 q 388 1139 341 1139 q 562 1014 562 1139 z "
	},
	"Ẽ": {
		ha: 772,
		x_min: 136,
		x_max: 689,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 m 515 1072 q 458 1085 486 1072 q 404 1113 431 1097 q 352 1140 377 1128 q 304 1153 328 1153 q 253 1133 271 1153 q 227 1071 236 1112 l 160 1071 q 208 1199 169 1153 q 309 1246 247 1246 q 369 1233 340 1246 q 425 1205 398 1221 q 475 1178 451 1190 q 522 1165 500 1165 q 571 1185 555 1165 q 597 1247 587 1205 l 665 1247 q 617 1119 656 1165 q 515 1072 579 1072 z "
	},
	"ẽ": {
		ha: 779,
		x_min: 78,
		x_max: 707,
		o: "m 433 -14 q 173 87 269 -14 q 78 366 78 187 q 166 651 78 545 q 404 757 255 757 q 625 665 544 757 q 707 422 707 573 l 707 351 l 195 351 q 261 153 198 220 q 437 85 323 85 q 675 135 557 85 l 675 35 q 562 -2 615 9 q 433 -14 509 -14 m 403 663 q 260 604 313 663 q 197 443 207 546 l 586 443 q 538 606 586 549 q 403 663 491 663 m 502 843 q 445 856 473 843 q 390 883 417 868 q 339 911 363 899 q 291 924 314 924 q 240 903 257 924 q 213 842 222 883 l 146 842 q 194 970 155 924 q 295 1017 233 1017 q 355 1004 326 1017 q 411 976 385 991 q 462 948 437 961 q 508 936 486 936 q 557 956 541 936 q 584 1018 574 976 l 651 1018 q 604 889 642 936 q 502 843 566 843 z "
	},
	"Ế": {
		ha: 772,
		x_min: 136,
		x_max: 770,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 m 621 1071 l 557 1071 q 414 1183 481 1122 q 271 1071 337 1115 l 208 1071 l 208 1088 l 243 1128 q 355 1264 319 1212 l 474 1264 q 621 1088 519 1200 l 621 1071 m 567 1219 q 653 1357 616 1281 l 770 1357 l 770 1343 q 632 1203 713 1268 l 567 1203 l 567 1219 z "
	},
	"ế": {
		ha: 779,
		x_min: 78,
		x_max: 757,
		o: "m 433 -14 q 173 87 269 -14 q 78 366 78 187 q 166 651 78 545 q 404 757 255 757 q 625 665 544 757 q 707 422 707 573 l 707 351 l 195 351 q 261 153 198 220 q 437 85 323 85 q 675 135 557 85 l 675 35 q 562 -2 615 9 q 433 -14 509 -14 m 403 663 q 260 604 313 663 q 197 443 207 546 l 586 443 q 538 606 586 549 q 403 663 491 663 m 608 842 l 545 842 q 401 954 469 892 q 258 842 324 886 l 195 842 l 195 859 l 231 899 q 342 1035 307 983 l 461 1035 q 608 859 506 971 l 608 842 m 554 990 q 640 1128 604 1052 l 757 1128 l 757 1114 q 619 974 701 1039 l 554 974 l 554 990 z "
	},
	"Ề": {
		ha: 772,
		x_min: 63,
		x_max: 689,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 m 625 1071 l 562 1071 q 418 1183 496 1115 q 275 1071 347 1118 l 212 1071 l 212 1086 l 248 1127 q 359 1263 323 1211 l 478 1263 q 625 1086 523 1199 l 625 1071 m 266 1203 l 201 1203 q 63 1343 120 1267 l 63 1357 l 180 1357 q 266 1218 215 1284 l 266 1203 z "
	},
	"ề": {
		ha: 779,
		x_min: 50,
		x_max: 707,
		o: "m 433 -14 q 173 87 269 -14 q 78 366 78 187 q 166 651 78 545 q 404 757 255 757 q 625 665 544 757 q 707 422 707 573 l 707 351 l 195 351 q 261 153 198 220 q 437 85 323 85 q 675 135 557 85 l 675 35 q 562 -2 615 9 q 433 -14 509 -14 m 403 663 q 260 604 313 663 q 197 443 207 546 l 586 443 q 538 606 586 549 q 403 663 491 663 m 612 842 l 549 842 q 406 954 483 886 q 262 842 334 889 l 199 842 l 199 857 l 235 898 q 346 1034 311 981 l 465 1034 q 612 857 510 970 l 612 842 m 253 974 l 189 974 q 50 1114 107 1038 l 50 1128 l 167 1128 q 253 989 202 1055 l 253 974 z "
	},
	"Ể": {
		ha: 772,
		x_min: 136,
		x_max: 733,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 m 619 1071 l 555 1071 q 412 1183 479 1122 q 269 1071 335 1115 l 205 1071 l 205 1088 l 241 1128 q 353 1264 317 1212 l 472 1264 q 619 1088 517 1200 l 619 1071 m 733 1345 q 647 1255 733 1275 l 643 1200 l 589 1200 l 582 1289 q 642 1306 621 1295 q 663 1343 663 1317 q 644 1376 663 1368 q 595 1385 625 1385 q 547 1381 564 1385 l 547 1435 q 601 1439 564 1439 q 733 1345 733 1439 z "
	},
	"ể": {
		ha: 779,
		x_min: 78,
		x_max: 714,
		o: "m 433 -14 q 173 87 269 -14 q 78 366 78 187 q 166 651 78 545 q 404 757 255 757 q 625 665 544 757 q 707 422 707 573 l 707 351 l 195 351 q 261 153 198 220 q 437 85 323 85 q 675 135 557 85 l 675 35 q 562 -2 615 9 q 433 -14 509 -14 m 403 663 q 260 604 313 663 q 197 443 207 546 l 586 443 q 538 606 586 549 q 403 663 491 663 m 600 842 l 536 842 q 393 954 460 892 q 250 842 316 886 l 186 842 l 186 859 l 222 899 q 334 1035 298 983 l 453 1035 q 600 859 498 971 l 600 842 m 714 1116 q 628 1026 714 1046 l 624 971 l 570 971 l 563 1060 q 623 1077 602 1066 q 644 1114 644 1088 q 625 1147 644 1139 q 576 1156 606 1156 q 528 1152 545 1156 l 528 1206 q 582 1210 545 1210 q 714 1116 714 1210 z "
	},
	"Ễ": {
		ha: 772,
		x_min: 136,
		x_max: 689,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 m 503 1307 q 453 1317 477 1307 q 407 1340 429 1327 q 362 1362 384 1352 q 322 1373 341 1373 q 281 1356 295 1373 q 257 1305 267 1339 l 195 1305 q 238 1416 204 1377 q 323 1454 273 1454 q 373 1444 349 1454 q 421 1421 398 1434 q 465 1399 444 1409 q 504 1389 486 1389 q 545 1406 531 1389 q 568 1455 559 1422 l 629 1455 q 588 1346 621 1385 q 503 1307 554 1307 m 618 1071 l 554 1071 q 411 1183 488 1115 q 268 1071 339 1118 l 204 1071 l 204 1086 l 240 1127 q 351 1263 316 1211 l 471 1263 q 618 1086 515 1199 l 618 1071 z "
	},
	"ễ": {
		ha: 779,
		x_min: 78,
		x_max: 707,
		o: "m 433 -14 q 173 87 269 -14 q 78 366 78 187 q 166 651 78 545 q 404 757 255 757 q 625 665 544 757 q 707 422 707 573 l 707 351 l 195 351 q 261 153 198 220 q 437 85 323 85 q 675 135 557 85 l 675 35 q 562 -2 615 9 q 433 -14 509 -14 m 403 663 q 260 604 313 663 q 197 443 207 546 l 586 443 q 538 606 586 549 q 403 663 491 663 m 488 1078 q 438 1088 463 1078 q 392 1111 414 1098 q 348 1133 369 1123 q 307 1143 326 1143 q 266 1127 280 1143 q 242 1076 252 1110 l 180 1076 q 223 1186 189 1148 q 309 1225 258 1225 q 358 1215 334 1225 q 406 1192 383 1204 q 450 1170 429 1180 q 489 1160 471 1160 q 530 1176 516 1160 q 553 1226 545 1193 l 614 1226 q 573 1117 606 1156 q 488 1078 539 1078 m 603 842 l 539 842 q 396 954 473 886 q 253 842 324 889 l 189 842 l 189 857 l 225 898 q 336 1034 301 981 l 456 1034 q 603 857 500 970 l 603 842 z "
	},
	"Ệ": {
		ha: 772,
		x_min: 136,
		x_max: 689,
		o: "m 689 0 l 136 0 l 136 991 l 689 991 l 689 889 l 252 889 l 252 570 l 663 570 l 663 468 l 252 468 l 252 103 l 689 103 l 689 0 m 342 -164 q 362 -107 342 -125 q 411 -89 382 -89 q 458 -107 438 -89 q 477 -164 477 -125 q 458 -220 477 -201 q 411 -239 438 -239 q 362 -220 382 -239 q 342 -164 342 -201 m 183 1086 q 304 1222 269 1179 q 354 1293 338 1265 l 467 1293 q 518 1220 481 1265 q 640 1086 555 1175 l 640 1071 l 559 1071 q 410 1197 500 1108 q 261 1071 317 1106 l 183 1071 l 183 1086 z "
	},
	"ệ": {
		ha: 779,
		x_min: 78,
		x_max: 707,
		o: "m 433 -14 q 173 87 269 -14 q 78 366 78 187 q 166 651 78 545 q 404 757 255 757 q 625 665 544 757 q 707 422 707 573 l 707 351 l 195 351 q 261 153 198 220 q 437 85 323 85 q 675 135 557 85 l 675 35 q 562 -2 615 9 q 433 -14 509 -14 m 403 663 q 260 604 313 663 q 197 443 207 546 l 586 443 q 538 606 586 549 q 403 663 491 663 m 333 -164 q 353 -107 333 -125 q 401 -89 373 -89 q 448 -107 429 -89 q 468 -164 468 -125 q 448 -220 468 -201 q 401 -239 429 -239 q 353 -220 373 -239 q 333 -164 333 -201 m 172 857 q 292 993 258 949 q 342 1064 327 1036 l 455 1064 q 507 990 470 1036 q 629 857 544 945 l 629 842 l 548 842 q 398 968 488 879 q 250 842 306 877 l 172 842 l 172 857 z "
	},
	"ỉ": {
		ha: 351,
		x_min: 83,
		x_max: 330,
		o: "m 232 0 l 119 0 l 119 743 l 232 743 l 232 0 m 330 1014 q 217 896 330 919 l 210 819 l 139 819 l 131 939 q 216 965 189 948 q 242 1011 242 982 q 219 1054 242 1040 q 147 1068 197 1068 q 83 1064 105 1068 l 83 1132 q 156 1139 109 1139 q 330 1014 330 1139 z "
	},
	"ị": {
		ha: 351,
		x_min: 106,
		x_max: 243,
		o: "m 232 0 l 119 0 l 119 743 l 232 743 l 232 0 m 110 945 q 129 1001 110 983 q 176 1019 148 1019 q 223 1001 203 1019 q 243 945 243 983 q 223 888 243 907 q 176 869 203 869 q 129 888 148 869 q 110 945 110 907 m 106 -164 q 126 -107 106 -125 q 175 -89 146 -89 q 222 -107 202 -89 q 241 -164 241 -125 q 222 -220 241 -201 q 175 -239 202 -239 q 126 -220 146 -239 q 106 -164 106 -201 z "
	},
	"Ọ": {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 207 497 q 293 192 207 296 q 542 87 378 87 q 791 191 707 87 q 875 497 875 295 q 791 801 875 697 q 543 904 707 904 q 293 800 378 904 q 207 497 207 696 m 473 -164 q 493 -107 473 -125 q 542 -89 513 -89 q 589 -107 569 -89 q 608 -164 608 -125 q 589 -220 608 -201 q 542 -239 569 -239 q 493 -220 513 -239 q 473 -164 473 -201 z "
	},
	"ọ": {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 195 372 q 252 155 195 230 q 419 81 309 81 q 587 155 530 81 q 644 372 644 229 q 587 588 644 514 q 418 661 530 661 q 251 589 307 661 q 195 372 195 516 m 350 -164 q 370 -107 350 -125 q 418 -89 390 -89 q 465 -107 446 -89 q 485 -164 485 -125 q 465 -220 485 -201 q 418 -239 446 -239 q 370 -220 390 -239 q 350 -164 350 -201 z "
	},
	"Ỏ": {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 207 497 q 293 192 207 296 q 542 87 378 87 q 791 191 707 87 q 875 497 875 295 q 791 801 875 697 q 543 904 707 904 q 293 800 378 904 q 207 497 207 696 m 696 1243 q 583 1125 696 1148 l 576 1048 l 505 1048 l 497 1168 q 582 1194 555 1177 q 608 1240 608 1211 q 586 1283 608 1270 q 513 1297 563 1297 q 450 1293 471 1297 l 450 1361 q 522 1368 475 1368 q 696 1243 696 1368 z "
	},
	"ỏ": {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 195 372 q 252 155 195 230 q 419 81 309 81 q 587 155 530 81 q 644 372 644 229 q 587 588 644 514 q 418 661 530 661 q 251 589 307 661 q 195 372 195 516 m 572 1014 q 460 896 572 919 l 453 819 l 382 819 l 374 939 q 458 965 432 948 q 485 1011 485 982 q 462 1054 485 1040 q 390 1068 439 1068 q 326 1064 348 1068 l 326 1132 q 399 1139 352 1139 q 572 1014 572 1139 z "
	},
	"Ố": {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 207 497 q 293 192 207 296 q 542 87 378 87 q 791 191 707 87 q 875 497 875 295 q 791 801 875 697 q 543 904 707 904 q 293 800 378 904 q 207 497 207 696 m 749 1071 l 685 1071 q 542 1183 609 1122 q 399 1071 465 1115 l 335 1071 l 335 1088 l 371 1128 q 482 1264 447 1212 l 602 1264 q 749 1088 646 1200 l 749 1071 m 694 1219 q 781 1357 744 1281 l 897 1357 l 897 1343 q 760 1203 841 1268 l 694 1203 l 694 1219 z "
	},
	"ố": {
		ha: 839,
		x_min: 78,
		x_max: 774,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 195 372 q 252 155 195 230 q 419 81 309 81 q 587 155 530 81 q 644 372 644 229 q 587 588 644 514 q 418 661 530 661 q 251 589 307 661 q 195 372 195 516 m 625 842 l 562 842 q 418 954 486 892 q 275 842 341 886 l 212 842 l 212 859 l 248 899 q 359 1035 323 983 l 478 1035 q 625 859 523 971 l 625 842 m 571 990 q 657 1128 621 1052 l 774 1128 l 774 1114 q 636 974 718 1039 l 571 974 l 571 990 z "
	},
	"Ồ": {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 207 497 q 293 192 207 296 q 542 87 378 87 q 791 191 707 87 q 875 497 875 295 q 791 801 875 697 q 543 904 707 904 q 293 800 378 904 q 207 497 207 696 m 751 1071 l 688 1071 q 545 1183 622 1115 q 401 1071 473 1118 l 338 1071 l 338 1086 l 374 1127 q 485 1263 450 1211 l 604 1263 q 751 1086 649 1199 l 751 1071 m 392 1203 l 328 1203 q 189 1343 246 1267 l 189 1357 l 306 1357 q 392 1218 341 1284 l 392 1203 z "
	},
	"ồ": {
		ha: 839,
		x_min: 66,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 195 372 q 252 155 195 230 q 419 81 309 81 q 587 155 530 81 q 644 372 644 229 q 587 588 644 514 q 418 661 530 661 q 251 589 307 661 q 195 372 195 516 m 628 842 l 564 842 q 421 954 498 886 q 278 842 349 889 l 214 842 l 214 857 l 250 898 q 361 1034 326 981 l 481 1034 q 628 857 526 970 l 628 842 m 269 974 l 204 974 q 66 1114 123 1038 l 66 1128 l 182 1128 q 269 989 218 1055 l 269 974 z "
	},
	"Ổ": {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 207 497 q 293 192 207 296 q 542 87 378 87 q 791 191 707 87 q 875 497 875 295 q 791 801 875 697 q 543 904 707 904 q 293 800 378 904 q 207 497 207 696 m 747 1071 l 684 1071 q 541 1183 608 1122 q 397 1071 463 1115 l 334 1071 l 334 1088 l 370 1128 q 481 1264 446 1212 l 600 1264 q 747 1088 645 1200 l 747 1071 m 861 1345 q 775 1255 861 1275 l 771 1200 l 717 1200 l 710 1289 q 770 1306 749 1295 q 791 1343 791 1317 q 772 1376 791 1368 q 724 1385 753 1385 q 675 1381 692 1385 l 675 1435 q 729 1439 692 1439 q 861 1345 861 1439 z "
	},
	"ổ": {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 195 372 q 252 155 195 230 q 419 81 309 81 q 587 155 530 81 q 644 372 644 229 q 587 588 644 514 q 418 661 530 661 q 251 589 307 661 q 195 372 195 516 m 625 842 l 562 842 q 418 954 486 892 q 275 842 341 886 l 212 842 l 212 859 l 248 899 q 359 1035 323 983 l 478 1035 q 625 859 523 971 l 625 842 m 739 1116 q 653 1026 739 1046 l 649 971 l 595 971 l 588 1060 q 648 1077 627 1066 q 669 1114 669 1088 q 650 1147 669 1139 q 602 1156 631 1156 q 553 1152 570 1156 l 553 1206 q 607 1210 570 1210 q 739 1116 739 1210 z "
	},
	"Ỗ": {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 207 497 q 293 192 207 296 q 542 87 378 87 q 791 191 707 87 q 875 497 875 295 q 791 801 875 697 q 543 904 707 904 q 293 800 378 904 q 207 497 207 696 m 633 1307 q 584 1317 608 1307 q 537 1340 560 1327 q 493 1362 515 1352 q 453 1373 472 1373 q 412 1356 426 1373 q 388 1305 397 1339 l 326 1305 q 369 1416 335 1377 q 454 1454 404 1454 q 504 1444 479 1454 q 552 1421 529 1434 q 595 1399 574 1409 q 635 1389 616 1389 q 676 1406 662 1389 q 699 1455 690 1422 l 760 1455 q 719 1346 752 1385 q 633 1307 685 1307 m 749 1071 l 685 1071 q 542 1183 619 1115 q 399 1071 470 1118 l 335 1071 l 335 1086 l 371 1127 q 482 1263 447 1211 l 602 1263 q 749 1086 646 1199 l 749 1071 z "
	},
	"ỗ": {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 195 372 q 252 155 195 230 q 419 81 309 81 q 587 155 530 81 q 644 372 644 229 q 587 588 644 514 q 418 661 530 661 q 251 589 307 661 q 195 372 195 516 m 511 1078 q 462 1088 486 1078 q 415 1111 438 1098 q 371 1133 393 1123 q 331 1143 350 1143 q 290 1127 304 1143 q 266 1076 275 1110 l 204 1076 q 247 1186 213 1148 q 332 1225 281 1225 q 382 1215 357 1225 q 430 1192 407 1204 q 473 1170 452 1180 q 513 1160 494 1160 q 554 1176 540 1160 q 576 1226 568 1193 l 637 1226 q 596 1117 630 1156 q 511 1078 563 1078 m 627 842 l 563 842 q 420 954 497 886 q 277 842 348 889 l 213 842 l 213 857 l 249 898 q 360 1034 325 981 l 479 1034 q 627 857 524 970 l 627 842 z "
	},
	"Ộ": {
		ha: 1082,
		x_min: 85,
		x_max: 997,
		o: "m 997 497 q 877 123 997 259 q 542 -14 756 -14 q 204 120 323 -14 q 85 498 85 254 q 204 874 85 741 q 543 1007 323 1007 q 877 871 757 1007 q 997 497 997 736 m 207 497 q 293 192 207 296 q 542 87 378 87 q 791 191 707 87 q 875 497 875 295 q 791 801 875 697 q 543 904 707 904 q 293 800 378 904 q 207 497 207 696 m 473 -164 q 493 -107 473 -125 q 542 -89 513 -89 q 589 -107 569 -89 q 608 -164 608 -125 q 589 -220 608 -201 q 542 -239 569 -239 q 493 -220 513 -239 q 473 -164 473 -201 m 313 1086 q 433 1222 399 1179 q 484 1293 468 1265 l 596 1293 q 648 1220 611 1265 q 770 1086 685 1175 l 770 1071 l 689 1071 q 539 1197 629 1108 q 391 1071 447 1106 l 313 1071 l 313 1086 z "
	},
	"ộ": {
		ha: 839,
		x_min: 78,
		x_max: 761,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 669 653 577 757 q 761 372 761 549 m 195 372 q 252 155 195 230 q 419 81 309 81 q 587 155 530 81 q 644 372 644 229 q 587 588 644 514 q 418 661 530 661 q 251 589 307 661 q 195 372 195 516 m 353 -164 q 373 -107 353 -125 q 421 -89 393 -89 q 468 -107 448 -89 q 488 -164 488 -125 q 468 -220 488 -201 q 421 -239 448 -239 q 373 -220 393 -239 q 353 -164 353 -201 m 191 857 q 312 993 277 949 q 362 1064 347 1036 l 475 1064 q 527 990 490 1036 q 648 857 564 945 l 648 842 l 568 842 q 418 968 508 879 q 269 842 326 877 l 191 842 l 191 857 z "
	},
	"Ớ": {
		ha: 1083,
		x_min: 85,
		x_max: 1109,
		o: "m 996 497 q 875 122 996 257 q 542 -14 755 -14 q 204 120 323 -14 q 85 498 85 253 q 204 875 85 743 q 543 1007 324 1007 q 885 861 764 1007 q 939 892 919 869 q 967 949 958 915 q 977 1055 977 984 l 1100 1055 l 1109 1040 q 1045 877 1090 935 q 930 791 1001 818 q 996 497 996 672 m 207 497 q 292 191 207 295 q 542 87 376 87 q 790 191 707 87 q 873 497 873 294 q 790 801 873 697 q 543 904 707 904 q 293 801 378 904 q 207 497 207 699 m 469 1088 q 539 1190 502 1130 q 599 1293 577 1249 l 736 1293 l 736 1279 q 647 1171 706 1235 q 545 1071 588 1106 l 469 1071 l 469 1088 z "
	},
	"ớ": {
		ha: 846,
		x_min: 78,
		x_max: 885,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 661 661 568 757 q 715 693 696 671 q 744 749 735 715 q 753 857 753 783 l 875 857 l 885 842 q 823 676 864 736 q 713 591 782 617 q 761 372 761 498 m 195 372 q 248 155 195 229 q 419 81 302 81 q 591 155 538 81 q 644 372 644 229 q 590 588 644 515 q 418 661 536 661 q 248 588 300 661 q 195 372 195 515 m 340 859 q 411 960 373 901 q 470 1064 448 1020 l 607 1064 l 607 1050 q 518 941 577 1006 q 416 842 459 877 l 340 842 l 340 859 z "
	},
	"Ờ": {
		ha: 1083,
		x_min: 85,
		x_max: 1109,
		o: "m 996 497 q 875 122 996 257 q 542 -14 755 -14 q 204 120 323 -14 q 85 498 85 253 q 204 875 85 743 q 543 1007 324 1007 q 885 861 764 1007 q 939 892 919 869 q 967 949 958 915 q 977 1055 977 984 l 1100 1055 l 1109 1040 q 1045 877 1090 935 q 930 791 1001 818 q 996 497 996 672 m 207 497 q 292 191 207 295 q 542 87 376 87 q 790 191 707 87 q 873 497 873 294 q 790 801 873 697 q 543 904 707 904 q 293 801 378 904 q 207 497 207 699 m 625 1071 l 550 1071 q 446 1171 506 1106 q 358 1279 385 1236 l 358 1293 l 496 1293 q 556 1185 517 1246 q 625 1088 595 1124 l 625 1071 z "
	},
	"ờ": {
		ha: 846,
		x_min: 78,
		x_max: 885,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 661 661 568 757 q 715 693 696 671 q 744 749 735 715 q 753 857 753 783 l 875 857 l 885 842 q 823 676 864 736 q 713 591 782 617 q 761 372 761 498 m 195 372 q 248 155 195 229 q 419 81 302 81 q 591 155 538 81 q 644 372 644 229 q 590 588 644 515 q 418 661 536 661 q 248 588 300 661 q 195 372 195 515 m 503 842 l 429 842 q 324 942 385 877 q 237 1050 264 1007 l 237 1064 l 374 1064 q 435 956 396 1017 q 503 859 473 895 l 503 842 z "
	},
	"Ở": {
		ha: 1083,
		x_min: 85,
		x_max: 1109,
		o: "m 996 497 q 875 122 996 257 q 542 -14 755 -14 q 204 120 323 -14 q 85 498 85 253 q 204 875 85 743 q 543 1007 324 1007 q 885 861 764 1007 q 939 892 919 869 q 967 949 958 915 q 977 1055 977 984 l 1100 1055 l 1109 1040 q 1045 877 1090 935 q 930 791 1001 818 q 996 497 996 672 m 207 497 q 292 191 207 295 q 542 87 376 87 q 790 191 707 87 q 873 497 873 294 q 790 801 873 697 q 543 904 707 904 q 293 801 378 904 q 207 497 207 699 m 696 1243 q 583 1125 696 1148 l 576 1048 l 505 1048 l 497 1168 q 582 1194 555 1177 q 608 1240 608 1211 q 586 1283 608 1270 q 513 1297 563 1297 q 450 1293 471 1297 l 450 1361 q 522 1368 475 1368 q 696 1243 696 1368 z "
	},
	"ở": {
		ha: 846,
		x_min: 78,
		x_max: 885,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 661 661 568 757 q 715 693 696 671 q 744 749 735 715 q 753 857 753 783 l 875 857 l 885 842 q 823 676 864 736 q 713 591 782 617 q 761 372 761 498 m 195 372 q 248 155 195 229 q 419 81 302 81 q 591 155 538 81 q 644 372 644 229 q 590 588 644 515 q 418 661 536 661 q 248 588 300 661 q 195 372 195 515 m 572 1014 q 460 896 572 919 l 453 819 l 382 819 l 374 939 q 458 965 432 948 q 485 1011 485 982 q 462 1054 485 1040 q 390 1068 439 1068 q 326 1064 348 1068 l 326 1132 q 399 1139 352 1139 q 572 1014 572 1139 z "
	},
	"Ỡ": {
		ha: 1083,
		x_min: 85,
		x_max: 1109,
		o: "m 996 497 q 875 122 996 257 q 542 -14 755 -14 q 204 120 323 -14 q 85 498 85 253 q 204 875 85 743 q 543 1007 324 1007 q 885 861 764 1007 q 939 892 919 869 q 967 949 958 915 q 977 1055 977 984 l 1100 1055 l 1109 1040 q 1045 877 1090 935 q 930 791 1001 818 q 996 497 996 672 m 207 497 q 292 191 207 295 q 542 87 376 87 q 790 191 707 87 q 873 497 873 294 q 790 801 873 697 q 543 904 707 904 q 293 801 378 904 q 207 497 207 699 m 643 1072 q 586 1085 614 1072 q 531 1113 558 1097 q 480 1140 505 1128 q 432 1153 455 1153 q 381 1133 398 1153 q 354 1071 363 1112 l 288 1071 q 335 1199 296 1153 q 436 1246 374 1246 q 496 1233 467 1246 q 552 1205 526 1221 q 603 1178 578 1190 q 649 1165 627 1165 q 699 1185 682 1165 q 725 1247 715 1205 l 792 1247 q 745 1119 783 1165 q 643 1072 707 1072 z "
	},
	"ỡ": {
		ha: 846,
		x_min: 78,
		x_max: 885,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 661 661 568 757 q 715 693 696 671 q 744 749 735 715 q 753 857 753 783 l 875 857 l 885 842 q 823 676 864 736 q 713 591 782 617 q 761 372 761 498 m 195 372 q 248 155 195 229 q 419 81 302 81 q 591 155 538 81 q 644 372 644 229 q 590 588 644 515 q 418 661 536 661 q 248 588 300 661 q 195 372 195 515 m 527 843 q 470 856 498 843 q 415 883 442 868 q 364 911 389 899 q 316 924 339 924 q 265 903 282 924 q 238 842 248 883 l 172 842 q 219 970 180 924 q 320 1017 258 1017 q 380 1004 351 1017 q 436 976 410 991 q 487 948 463 961 q 533 936 511 936 q 583 956 566 936 q 609 1018 599 976 l 676 1018 q 629 889 667 936 q 527 843 591 843 z "
	},
	"Ợ": {
		ha: 1083,
		x_min: 85,
		x_max: 1109,
		o: "m 996 497 q 875 122 996 257 q 542 -14 755 -14 q 204 120 323 -14 q 85 498 85 253 q 204 875 85 743 q 543 1007 324 1007 q 885 861 764 1007 q 939 892 919 869 q 967 949 958 915 q 977 1055 977 984 l 1100 1055 l 1109 1040 q 1045 877 1090 935 q 930 791 1001 818 q 996 497 996 672 m 207 497 q 292 191 207 295 q 542 87 376 87 q 790 191 707 87 q 873 497 873 294 q 790 801 873 697 q 543 904 707 904 q 293 801 378 904 q 207 497 207 699 m 471 -164 q 491 -107 471 -125 q 539 -89 511 -89 q 586 -107 566 -89 q 606 -164 606 -125 q 586 -220 606 -201 q 539 -239 566 -239 q 491 -220 511 -239 q 471 -164 471 -201 z "
	},
	"ợ": {
		ha: 846,
		x_min: 78,
		x_max: 885,
		o: "m 761 372 q 669 89 761 191 q 416 -14 578 -14 q 239 33 317 -14 q 120 168 162 80 q 78 372 78 255 q 169 655 78 554 q 421 757 260 757 q 661 661 568 757 q 715 693 696 671 q 744 749 735 715 q 753 857 753 783 l 875 857 l 885 842 q 823 676 864 736 q 713 591 782 617 q 761 372 761 498 m 195 372 q 248 155 195 229 q 419 81 302 81 q 591 155 538 81 q 644 372 644 229 q 590 588 644 515 q 418 661 536 661 q 248 588 300 661 q 195 372 195 515 m 350 -164 q 370 -107 350 -125 q 418 -89 390 -89 q 465 -107 446 -89 q 485 -164 485 -125 q 465 -220 485 -201 q 418 -239 446 -239 q 370 -220 390 -239 q 350 -164 350 -201 z "
	},
	"Ụ": {
		ha: 1011,
		x_min: 126,
		x_max: 885,
		o: "m 885 991 l 885 350 q 783 83 885 180 q 501 -14 680 -14 q 224 84 322 -14 q 126 353 126 182 l 126 991 l 241 991 l 241 345 q 309 154 241 220 q 509 87 377 87 q 702 154 634 87 q 770 346 770 221 l 770 991 l 885 991 m 437 -164 q 457 -107 437 -125 q 506 -89 477 -89 q 553 -107 533 -89 q 572 -164 572 -125 q 553 -220 572 -201 q 506 -239 533 -239 q 457 -220 477 -239 q 437 -164 437 -201 z "
	},
	"ụ": {
		ha: 852,
		x_min: 111,
		x_max: 733,
		o: "m 225 743 l 225 261 q 267 125 225 170 q 396 81 308 81 q 567 144 513 81 q 621 353 621 208 l 621 743 l 733 743 l 733 0 l 640 0 l 624 100 l 618 100 q 522 16 583 45 q 382 -14 460 -14 q 179 51 246 -14 q 111 257 111 115 l 111 743 l 225 743 m 338 -164 q 358 -107 338 -125 q 407 -89 378 -89 q 454 -107 434 -89 q 473 -164 473 -125 q 454 -220 473 -201 q 407 -239 434 -239 q 358 -220 378 -239 q 338 -164 338 -201 z "
	},
	"Ủ": {
		ha: 1011,
		x_min: 126,
		x_max: 885,
		o: "m 885 991 l 885 350 q 783 83 885 180 q 501 -14 680 -14 q 224 84 322 -14 q 126 353 126 182 l 126 991 l 241 991 l 241 345 q 309 154 241 220 q 509 87 377 87 q 702 154 634 87 q 770 346 770 221 l 770 991 l 885 991 m 656 1243 q 543 1125 656 1148 l 536 1048 l 465 1048 l 457 1168 q 542 1194 515 1177 q 568 1240 568 1211 q 546 1283 568 1270 q 473 1297 523 1297 q 410 1293 431 1297 l 410 1361 q 482 1368 435 1368 q 656 1243 656 1368 z "
	},
	"ủ": {
		ha: 852,
		x_min: 111,
		x_max: 733,
		o: "m 225 743 l 225 261 q 267 125 225 170 q 396 81 308 81 q 567 144 513 81 q 621 353 621 208 l 621 743 l 733 743 l 733 0 l 640 0 l 624 100 l 618 100 q 522 16 583 45 q 382 -14 460 -14 q 179 51 246 -14 q 111 257 111 115 l 111 743 l 225 743 m 570 1014 q 457 896 570 919 l 450 819 l 379 819 l 371 939 q 456 965 429 948 q 482 1011 482 982 q 459 1054 482 1040 q 387 1068 437 1068 q 323 1064 345 1068 l 323 1132 q 396 1139 349 1139 q 570 1014 570 1139 z "
	},
	"Ứ": {
		ha: 1067,
		x_min: 126,
		x_max: 1125,
		o: "m 885 991 l 885 857 q 948 884 924 863 q 982 943 972 905 q 993 1055 993 981 l 1116 1055 l 1125 1040 q 1044 854 1103 917 q 885 775 986 792 l 885 350 q 788 81 885 175 q 511 -14 690 -14 q 228 84 330 -14 q 126 353 126 181 l 126 991 l 241 991 l 241 345 q 311 154 241 220 q 514 87 380 87 q 704 153 639 87 q 770 346 770 218 l 770 991 l 885 991 m 428 1088 q 498 1190 460 1130 q 557 1293 536 1249 l 694 1293 l 694 1279 q 606 1171 665 1235 q 503 1071 547 1106 l 428 1071 l 428 1088 z "
	},
	"ứ": {
		ha: 924,
		x_min: 111,
		x_max: 970,
		o: "m 225 742 l 225 261 q 267 125 225 170 q 396 81 308 81 q 567 144 513 81 q 621 353 621 208 l 621 742 l 733 742 l 733 660 q 814 711 789 667 q 839 859 839 754 l 960 859 l 970 843 q 888 652 948 717 q 733 578 829 587 l 733 0 l 640 0 l 624 100 l 618 100 q 521 15 583 44 q 385 -14 460 -14 q 179 51 247 -14 q 111 257 111 115 l 111 742 l 225 742 m 349 859 q 419 960 381 901 q 478 1064 456 1020 l 615 1064 l 615 1050 q 526 941 585 1006 q 424 842 467 877 l 349 842 l 349 859 z "
	},
	"Ừ": {
		ha: 1067,
		x_min: 126,
		x_max: 1125,
		o: "m 885 991 l 885 857 q 948 884 924 863 q 982 943 972 905 q 993 1055 993 981 l 1116 1055 l 1125 1040 q 1044 854 1103 917 q 885 775 986 792 l 885 350 q 788 81 885 175 q 511 -14 690 -14 q 228 84 330 -14 q 126 353 126 181 l 126 991 l 241 991 l 241 345 q 311 154 241 220 q 514 87 380 87 q 704 153 639 87 q 770 346 770 218 l 770 991 l 885 991 m 594 1071 l 519 1071 q 415 1171 475 1106 q 328 1279 355 1236 l 328 1293 l 465 1293 q 526 1185 487 1246 q 594 1088 564 1124 l 594 1071 z "
	},
	"ừ": {
		ha: 924,
		x_min: 111,
		x_max: 970,
		o: "m 225 742 l 225 261 q 267 125 225 170 q 396 81 308 81 q 567 144 513 81 q 621 353 621 208 l 621 742 l 733 742 l 733 660 q 814 711 789 667 q 839 859 839 754 l 960 859 l 970 843 q 888 652 948 717 q 733 578 829 587 l 733 0 l 640 0 l 624 100 l 618 100 q 521 15 583 44 q 385 -14 460 -14 q 179 51 247 -14 q 111 257 111 115 l 111 742 l 225 742 m 486 842 l 412 842 q 307 942 368 877 q 220 1050 247 1007 l 220 1064 l 357 1064 q 418 956 379 1017 q 486 859 456 895 l 486 842 z "
	},
	"Ử": {
		ha: 1067,
		x_min: 126,
		x_max: 1125,
		o: "m 885 991 l 885 857 q 948 884 924 863 q 982 943 972 905 q 993 1055 993 981 l 1116 1055 l 1125 1040 q 1044 854 1103 917 q 885 775 986 792 l 885 350 q 788 81 885 175 q 511 -14 690 -14 q 228 84 330 -14 q 126 353 126 181 l 126 991 l 241 991 l 241 345 q 311 154 241 220 q 514 87 380 87 q 704 153 639 87 q 770 346 770 218 l 770 991 l 885 991 m 664 1243 q 551 1125 664 1148 l 545 1048 l 473 1048 l 465 1168 q 550 1194 524 1177 q 576 1240 576 1211 q 554 1283 576 1270 q 481 1297 531 1297 q 418 1293 439 1297 l 418 1361 q 490 1368 444 1368 q 664 1243 664 1368 z "
	},
	"ử": {
		ha: 924,
		x_min: 111,
		x_max: 970,
		o: "m 225 742 l 225 261 q 267 125 225 170 q 396 81 308 81 q 567 144 513 81 q 621 353 621 208 l 621 742 l 733 742 l 733 660 q 814 711 789 667 q 839 859 839 754 l 960 859 l 970 843 q 888 652 948 717 q 733 578 829 587 l 733 0 l 640 0 l 624 100 l 618 100 q 521 15 583 44 q 385 -14 460 -14 q 179 51 247 -14 q 111 257 111 115 l 111 742 l 225 742 m 574 1014 q 461 896 574 919 l 454 819 l 383 819 l 375 939 q 460 965 433 948 q 486 1011 486 982 q 464 1054 486 1040 q 391 1068 441 1068 q 328 1064 349 1068 l 328 1132 q 400 1139 353 1139 q 574 1014 574 1139 z "
	},
	"Ữ": {
		ha: 1067,
		x_min: 126,
		x_max: 1125,
		o: "m 885 991 l 885 857 q 948 884 924 863 q 982 943 972 905 q 993 1055 993 981 l 1116 1055 l 1125 1040 q 1044 854 1103 917 q 885 775 986 792 l 885 350 q 788 81 885 175 q 511 -14 690 -14 q 228 84 330 -14 q 126 353 126 181 l 126 991 l 241 991 l 241 345 q 311 154 241 220 q 514 87 380 87 q 704 153 639 87 q 770 346 770 218 l 770 991 l 885 991 m 621 1072 q 564 1085 591 1072 q 509 1113 536 1097 q 457 1140 482 1128 q 410 1153 433 1153 q 358 1133 376 1153 q 332 1071 341 1112 l 265 1071 q 313 1199 274 1153 q 414 1246 352 1246 q 474 1233 445 1246 q 530 1205 503 1221 q 581 1178 556 1190 q 627 1165 605 1165 q 676 1185 660 1165 q 703 1247 692 1205 l 770 1247 q 723 1119 761 1165 q 621 1072 684 1072 z "
	},
	"ữ": {
		ha: 924,
		x_min: 111,
		x_max: 970,
		o: "m 225 742 l 225 261 q 267 125 225 170 q 396 81 308 81 q 567 144 513 81 q 621 353 621 208 l 621 742 l 733 742 l 733 660 q 814 711 789 667 q 839 859 839 754 l 960 859 l 970 843 q 888 652 948 717 q 733 578 829 587 l 733 0 l 640 0 l 624 100 l 618 100 q 521 15 583 44 q 385 -14 460 -14 q 179 51 247 -14 q 111 257 111 115 l 111 742 l 225 742 m 534 843 q 477 856 505 843 q 422 883 449 868 q 371 911 395 899 q 323 924 346 924 q 272 903 289 924 q 245 842 254 883 l 178 842 q 226 970 187 924 q 327 1017 265 1017 q 387 1004 358 1017 q 443 976 416 991 q 494 948 469 961 q 540 936 518 936 q 589 956 573 936 q 616 1018 606 976 l 683 1018 q 636 889 674 936 q 534 843 597 843 z "
	},
	"Ự": {
		ha: 1067,
		x_min: 126,
		x_max: 1125,
		o: "m 885 991 l 885 857 q 948 884 924 863 q 982 943 972 905 q 993 1055 993 981 l 1116 1055 l 1125 1040 q 1044 854 1103 917 q 885 775 986 792 l 885 350 q 788 81 885 175 q 511 -14 690 -14 q 228 84 330 -14 q 126 353 126 181 l 126 991 l 241 991 l 241 345 q 311 154 241 220 q 514 87 380 87 q 704 153 639 87 q 770 346 770 218 l 770 991 l 885 991 m 439 -164 q 459 -107 439 -125 q 507 -89 479 -89 q 554 -107 534 -89 q 574 -164 574 -125 q 554 -220 574 -201 q 507 -239 534 -239 q 459 -220 479 -239 q 439 -164 439 -201 z "
	},
	"ự": {
		ha: 924,
		x_min: 111,
		x_max: 970,
		o: "m 225 742 l 225 261 q 267 125 225 170 q 396 81 308 81 q 567 144 513 81 q 621 353 621 208 l 621 742 l 733 742 l 733 660 q 814 711 789 667 q 839 859 839 754 l 960 859 l 970 843 q 888 652 948 717 q 733 578 829 587 l 733 0 l 640 0 l 624 100 l 618 100 q 521 15 583 44 q 385 -14 460 -14 q 179 51 247 -14 q 111 257 111 115 l 111 742 l 225 742 m 334 -164 q 354 -107 334 -125 q 403 -89 374 -89 q 450 -107 430 -89 q 469 -164 469 -125 q 450 -220 469 -201 q 403 -239 430 -239 q 354 -220 374 -239 q 334 -164 334 -201 z "
	},
	"Ỵ": {
		ha: 778,
		x_min: 0,
		x_max: 778,
		o: "m 389 496 l 653 991 l 778 991 l 447 385 l 447 0 l 330 0 l 330 379 l 0 991 l 126 991 l 389 496 m 319 -164 q 339 -107 319 -125 q 388 -89 359 -89 q 435 -107 415 -89 q 454 -164 454 -125 q 435 -220 454 -201 q 388 -239 415 -239 q 339 -220 359 -239 q 319 -164 319 -201 z "
	},
	"ỵ": {
		ha: 700,
		x_min: 1,
		x_max: 699,
		o: "m 1 743 l 122 743 l 285 319 q 351 110 338 174 l 357 110 q 394 228 366 144 q 578 743 422 312 l 699 743 l 379 -103 q 268 -281 332 -229 q 113 -334 205 -334 q 11 -322 61 -334 l 11 -232 q 94 -240 48 -240 q 260 -110 210 -240 l 301 -4 l 1 743 m 494 -166 q 514 -109 494 -127 q 563 -91 534 -91 q 610 -109 590 -91 q 629 -166 629 -127 q 610 -222 629 -203 q 563 -241 590 -241 q 514 -222 534 -241 q 494 -166 494 -203 z "
	},
	"Ỷ": {
		ha: 778,
		x_min: 0,
		x_max: 778,
		o: "m 389 496 l 653 991 l 778 991 l 447 385 l 447 0 l 330 0 l 330 379 l 0 991 l 126 991 l 389 496 m 541 1243 q 428 1125 541 1148 l 421 1048 l 350 1048 l 342 1168 q 427 1194 400 1177 q 453 1240 453 1211 q 430 1283 453 1270 q 358 1297 408 1297 q 294 1293 316 1297 l 294 1361 q 367 1368 320 1368 q 541 1243 541 1368 z "
	},
	"ỷ": {
		ha: 700,
		x_min: 1,
		x_max: 699,
		o: "m 1 743 l 122 743 l 285 319 q 351 110 338 174 l 357 110 q 394 228 366 144 q 578 743 422 312 l 699 743 l 379 -103 q 268 -281 332 -229 q 113 -334 205 -334 q 11 -322 61 -334 l 11 -232 q 94 -240 48 -240 q 260 -110 210 -240 l 301 -4 l 1 743 m 497 1014 q 385 896 497 919 l 378 819 l 307 819 l 298 939 q 383 965 357 948 q 410 1011 410 982 q 387 1054 410 1040 q 315 1068 364 1068 q 251 1064 273 1068 l 251 1132 q 323 1139 277 1139 q 497 1014 497 1139 z "
	},
	"Ỹ": {
		ha: 778,
		x_min: 0,
		x_max: 778,
		o: "m 389 496 l 653 991 l 778 991 l 447 385 l 447 0 l 330 0 l 330 379 l 0 991 l 126 991 l 389 496 m 492 1072 q 435 1085 463 1072 q 381 1113 408 1097 q 329 1140 354 1128 q 281 1153 304 1153 q 230 1133 248 1153 q 203 1071 213 1112 l 137 1071 q 185 1199 146 1153 q 286 1246 224 1246 q 346 1233 317 1246 q 401 1205 375 1221 q 452 1178 428 1190 q 498 1165 477 1165 q 548 1185 532 1165 q 574 1247 564 1205 l 642 1247 q 594 1119 633 1165 q 492 1072 556 1072 z "
	},
	"ỹ": {
		ha: 700,
		x_min: 1,
		x_max: 699,
		o: "m 1 743 l 122 743 l 285 319 q 351 110 338 174 l 357 110 q 394 228 366 144 q 578 743 422 312 l 699 743 l 379 -103 q 268 -281 332 -229 q 113 -334 205 -334 q 11 -322 61 -334 l 11 -232 q 94 -240 48 -240 q 260 -110 210 -240 l 301 -4 l 1 743 m 454 843 q 397 856 425 843 q 343 883 370 868 q 291 911 316 899 q 243 924 267 924 q 192 903 210 924 q 165 842 175 883 l 99 842 q 147 970 108 924 q 248 1017 186 1017 q 308 1004 279 1017 q 363 976 337 991 q 414 948 390 961 q 460 936 439 936 q 510 956 494 936 q 536 1018 526 976 l 604 1018 q 556 889 595 936 q 454 843 518 843 z "
	},
	"₫": {
		ha: 851,
		x_min: 78,
		x_max: 838,
		o: "m 625 100 l 619 100 q 386 -14 541 -14 q 159 86 240 -14 q 78 370 78 186 q 159 655 78 553 q 386 757 241 757 q 618 647 537 757 l 627 647 q 619 753 619 703 l 619 843 l 315 843 l 315 930 l 619 930 l 619 1055 l 732 1055 l 732 930 l 838 930 l 838 843 l 732 843 l 732 0 l 640 0 l 625 100 m 400 81 q 567 143 515 81 q 619 346 619 206 l 619 370 q 567 595 619 528 q 399 663 514 663 q 247 586 300 663 q 195 368 195 509 q 247 153 195 226 q 400 81 299 81 m 747 -214 l 119 -214 l 119 -125 l 747 -125 l 747 -214 z "
	},
	"̏": {
		ha: 0,
		x_min: -713,
		x_max: -225,
		o: "m -225 842 l -290 842 q -386 939 -326 870 q -471 1050 -446 1007 l -471 1064 l -345 1064 q -292 964 -326 1023 q -225 859 -258 905 l -225 842 m -467 842 l -532 842 q -629 939 -570 871 q -713 1050 -688 1007 l -713 1064 l -586 1064 q -533 964 -567 1023 q -467 859 -500 905 l -467 842 z "
	},
	"ﬀ": {
		ha: 942,
		x_min: 20,
		x_max: 1001,
		o: "m 925 656 l 736 656 l 736 0 l 623 0 l 623 656 l 490 656 l 490 707 l 623 747 l 623 789 q 863 1063 623 1063 q 1001 1039 922 1063 l 972 949 q 861 970 907 970 q 766 927 797 970 q 736 791 736 885 l 736 743 l 925 743 l 925 656 m 454 656 l 265 656 l 265 0 l 153 0 l 153 656 l 20 656 l 20 707 l 153 747 l 153 789 q 392 1063 153 1063 q 530 1039 451 1063 l 501 949 q 390 970 436 970 q 296 927 326 970 q 265 791 265 885 l 265 743 l 454 743 l 454 656 z "
	},
	"℠": {
		ha: 1103,
		x_min: 62,
		x_max: 983,
		o: "m 396 637 q 346 534 396 571 q 211 497 295 497 q 62 519 113 497 l 62 593 q 215 566 134 566 q 315 633 315 566 q 308 669 315 656 q 281 694 300 683 q 205 726 263 705 q 97 787 124 756 q 69 869 69 818 q 117 963 69 928 q 240 999 165 999 q 387 972 325 999 l 364 905 q 240 930 296 930 q 174 915 199 930 q 149 872 149 899 q 155 840 149 852 q 175 819 161 828 q 254 785 190 809 q 369 720 342 751 q 396 637 396 690 m 682 503 l 546 882 l 541 882 l 545 794 l 545 503 l 464 503 l 464 991 l 591 991 l 724 612 l 861 991 l 983 991 l 983 503 l 897 503 l 897 787 l 901 880 l 896 880 l 753 503 l 682 503 z "
	},
	"Ţ": {
		ha: 768,
		x_min: 12,
		x_max: 755,
		o: "m 441 0 l 326 0 l 326 889 l 12 889 l 12 991 l 755 991 l 755 889 l 441 889 l 441 0 m 512 -196 q 460 -298 512 -262 q 307 -334 408 -334 q 241 -328 272 -334 l 241 -256 q 312 -261 272 -261 q 393 -248 366 -261 q 420 -197 420 -234 q 394 -150 420 -168 q 293 -121 367 -132 l 353 0 l 427 0 l 390 -78 q 512 -196 512 -104 z "
	},
	"ţ": {
		ha: 490,
		x_min: 21,
		x_max: 461,
		o: "m 359 79 q 417 84 389 79 q 461 93 445 88 l 461 7 q 407 -8 443 -2 q 343 -14 372 -14 q 127 214 127 -14 l 127 656 l 21 656 l 21 710 l 127 757 l 175 916 l 240 916 l 240 743 l 456 743 l 456 656 l 240 656 l 240 218 q 272 115 240 151 q 359 79 304 79 m 429 -196 q 377 -298 429 -262 q 224 -334 326 -334 q 159 -328 189 -334 l 159 -256 q 229 -261 189 -261 q 310 -248 283 -261 q 338 -197 338 -234 q 311 -150 338 -168 q 210 -121 284 -132 l 270 0 l 345 0 l 307 -78 q 429 -196 429 -104 z "
	},
	I: {
		ha: 387,
		x_min: 136,
		x_max: 252,
		o: "m 136 0 l 136 991 l 252 991 l 252 0 l 136 0 z "
	},
	"Ì": {
		ha: 387,
		x_min: 3,
		x_max: 270,
		o: "m 136 0 l 136 991 l 252 991 l 252 0 l 136 0 m 270 1071 l 195 1071 q 91 1171 151 1106 q 3 1279 31 1236 l 3 1293 l 141 1293 q 201 1185 163 1246 q 270 1088 240 1124 l 270 1071 z "
	},
	"Í": {
		ha: 387,
		x_min: 121,
		x_max: 388,
		o: "m 136 0 l 136 991 l 252 991 l 252 0 l 136 0 m 121 1088 q 192 1190 154 1130 q 251 1293 229 1249 l 388 1293 l 388 1279 q 299 1171 358 1235 q 197 1071 240 1106 l 121 1071 l 121 1088 z "
	},
	"Î": {
		ha: 387,
		x_min: -39,
		x_max: 418,
		o: "m 136 0 l 136 991 l 252 991 l 252 0 l 136 0 m -39 1086 q 82 1222 47 1179 q 132 1293 117 1265 l 245 1293 q 297 1220 260 1265 q 418 1086 334 1175 l 418 1071 l 338 1071 q 188 1197 278 1108 q 39 1071 96 1106 l -39 1071 l -39 1086 z "
	},
	"Ï": {
		ha: 387,
		x_min: 3,
		x_max: 385,
		o: "m 136 0 l 136 991 l 252 991 l 252 0 l 136 0 m 3 1174 q 21 1225 3 1209 q 64 1240 39 1240 q 109 1225 90 1240 q 127 1174 127 1209 q 109 1123 127 1140 q 64 1107 90 1107 q 21 1123 39 1107 q 3 1174 3 1140 m 262 1174 q 280 1225 262 1209 q 323 1240 298 1240 q 367 1225 348 1240 q 385 1174 385 1209 q 367 1123 385 1140 q 323 1107 348 1107 q 280 1123 298 1107 q 262 1174 262 1140 z "
	},
	"Ĩ": {
		ha: 387,
		x_min: -58,
		x_max: 447,
		o: "m 136 0 l 136 991 l 252 991 l 252 0 l 136 0 m 298 1072 q 241 1085 269 1072 q 186 1113 213 1097 q 135 1140 159 1128 q 87 1153 110 1153 q 36 1133 53 1153 q 9 1071 18 1112 l -58 1071 q -10 1199 -49 1153 q 91 1246 29 1246 q 151 1233 122 1246 q 207 1205 180 1221 q 258 1178 233 1190 q 304 1165 282 1165 q 353 1185 337 1165 q 380 1247 370 1205 l 447 1247 q 400 1119 438 1165 q 298 1072 361 1072 z "
	},
	"Ī": {
		ha: 387,
		x_min: -9,
		x_max: 398,
		o: "m 136 0 l 136 991 l 252 991 l 252 0 l 136 0 m -9 1164 l 398 1164 l 398 1071 l -9 1071 l -9 1164 z "
	},
	"Ĭ": {
		ha: 387,
		x_min: -17,
		x_max: 404,
		o: "m 136 0 l 136 991 l 252 991 l 252 0 l 136 0 m 190 1071 q 42 1117 95 1071 q -17 1253 -11 1164 l 54 1253 q 72 1200 58 1219 q 110 1173 85 1181 q 193 1164 135 1164 q 294 1185 261 1164 q 333 1253 326 1207 l 404 1253 q 340 1119 397 1167 q 190 1071 283 1071 z "
	},
	"Į": {
		ha: 387,
		x_min: 58,
		x_max: 283,
		o: "m 136 0 l 136 991 l 252 991 l 252 0 l 136 0 m 154 -161 q 218 -225 154 -225 q 283 -216 246 -225 l 283 -290 q 199 -302 239 -302 q 58 -175 58 -302 q 87 -82 58 -127 q 165 0 117 -36 l 247 0 q 177 -82 201 -45 q 154 -161 154 -119 z "
	},
	"İ": {
		ha: 387,
		x_min: 127,
		x_max: 260,
		o: "m 136 0 l 136 991 l 252 991 l 252 0 l 136 0 m 127 1174 q 146 1231 127 1213 q 193 1249 165 1249 q 240 1230 220 1249 q 260 1174 260 1212 q 240 1117 260 1136 q 193 1099 220 1099 q 146 1117 165 1099 q 127 1174 127 1136 z "
	},
	"Ĳ": {
		ha: 758,
		x_min: 136,
		x_max: 631,
		o: "m 136 0 l 136 991 l 252 991 l 252 0 l 136 0 m 379 -261 q 279 -243 315 -261 l 279 -144 q 379 -158 327 -158 q 481 -117 446 -158 q 516 0 516 -77 l 516 991 l 631 991 l 631 9 q 566 -190 631 -119 q 379 -261 501 -261 z "
	},
	"Ί": {
		ha: 503,
		x_min: -19,
		x_max: 367,
		o: "m 252 0 l 252 991 l 367 991 l 367 0 l 252 0 m -19 790 q 17 916 -1 839 q 43 1048 35 993 l 168 1048 l 168 1033 q 119 902 156 983 q 49 770 82 822 l -19 770 l -19 790 z "
	},
	"Ι": {
		ha: 387,
		x_min: 136,
		x_max: 252,
		o: "m 136 0 l 136 991 l 252 991 l 252 0 l 136 0 z "
	},
	"Ϊ": {
		ha: 387,
		x_min: 3,
		x_max: 385,
		o: "m 136 0 l 136 991 l 252 991 l 252 0 l 136 0 m 3 1174 q 21 1225 3 1209 q 64 1240 39 1240 q 109 1225 90 1240 q 127 1174 127 1209 q 109 1123 127 1140 q 64 1107 90 1107 q 21 1123 39 1107 q 3 1174 3 1140 m 262 1174 q 280 1225 262 1209 q 323 1240 298 1240 q 367 1225 348 1240 q 385 1174 385 1209 q 367 1123 385 1140 q 323 1107 348 1107 q 280 1123 298 1107 q 262 1174 262 1140 z "
	},
	"І": {
		ha: 387,
		x_min: 136,
		x_max: 252,
		o: "m 136 0 l 136 991 l 252 991 l 252 0 l 136 0 z "
	},
	"Ї": {
		ha: 387,
		x_min: 3,
		x_max: 385,
		o: "m 136 0 l 136 991 l 252 991 l 252 0 l 136 0 m 3 1174 q 21 1225 3 1209 q 64 1240 39 1240 q 109 1225 90 1240 q 127 1174 127 1209 q 109 1123 127 1140 q 64 1107 90 1107 q 21 1123 39 1107 q 3 1174 3 1140 m 262 1174 q 280 1225 262 1209 q 323 1240 298 1240 q 367 1225 348 1240 q 385 1174 385 1209 q 367 1123 385 1140 q 323 1107 348 1107 q 280 1123 298 1107 q 262 1174 262 1140 z "
	},
	"Ӏ": {
		ha: 387,
		x_min: 136,
		x_max: 252,
		o: "m 136 0 l 136 991 l 252 991 l 252 0 l 136 0 z "
	},
	"ӏ": {
		ha: 387,
		x_min: 136,
		x_max: 252,
		o: "m 136 0 l 136 991 l 252 991 l 252 0 l 136 0 z "
	},
	"Ỉ": {
		ha: 387,
		x_min: 104,
		x_max: 350,
		o: "m 136 0 l 136 991 l 252 991 l 252 0 l 136 0 m 350 1243 q 237 1125 350 1148 l 231 1048 l 159 1048 l 151 1168 q 236 1194 210 1177 q 262 1240 262 1211 q 240 1283 262 1270 q 168 1297 217 1297 q 104 1293 125 1297 l 104 1361 q 176 1368 130 1368 q 350 1243 350 1368 z "
	},
	"Ị": {
		ha: 387,
		x_min: 125,
		x_max: 260,
		o: "m 136 0 l 136 991 l 252 991 l 252 0 l 136 0 m 125 -164 q 145 -107 125 -125 q 193 -89 165 -89 q 240 -107 220 -89 q 260 -164 260 -125 q 240 -220 260 -201 q 193 -239 220 -239 q 145 -220 165 -239 q 125 -164 125 -201 z "
	}
};
var familyName = "Open Sans";
var ascender = 1485;
var descender = -407;
var underlinePosition = -104;
var underlineThickness = 69;
var boundingBox = {
	yMin: -376,
	xMin: -764,
	yMax: 1455,
	xMax: 1672
};
var resolution = 1000;
var original_font_information = {
	format: 0,
	copyright: "Digitized data copyright © 2010-2011, Google Corporation.",
	fontFamily: "Open Sans",
	fontSubfamily: "Regular",
	uniqueID: "Ascender - Open Sans Build 100",
	fullName: "Open Sans",
	version: "Version 1.10",
	postScriptName: "OpenSans",
	trademark: "Open Sans is a trademark of Google and may be registered in certain jurisdictions.",
	manufacturer: "Ascender Corporation",
	manufacturerURL: "http://www.ascendercorp.com/",
	designerURL: "http://www.ascendercorp.com/typedesigners.html",
	licence: "Licensed under the Apache License, Version 2.0",
	licenceURL: "http://www.apache.org/licenses/LICENSE-2.0"
};
var cssFontWeight = "normal";
var cssFontStyle = "normal";
var open_sans_regular_font = {
	glyphs: glyphs,
	familyName: familyName,
	ascender: ascender,
	descender: descender,
	underlinePosition: underlinePosition,
	underlineThickness: underlineThickness,
	boundingBox: boundingBox,
	resolution: resolution,
	original_font_information: original_font_information,
	cssFontWeight: cssFontWeight,
	cssFontStyle: cssFontStyle
};

var glyphs$1 = {
	"0": {
		ha: 833,
		x_min: 0,
		x_max: 0,
		o: "m 71 646 b 418 141 71 315 190 141 b 762 648 643 141 762 315 b 418 1153 762 977 643 1153 b 71 646 190 1153 71 979 z m 648 648 b 418 236 648 366 572 236 b 187 648 260 236 187 369 b 418 1055 187 928 260 1055 b 648 648 575 1055 648 928 z "
	},
	"1": {
		ha: 909,
		x_min: 0,
		x_max: 0,
		o: "m 445 1142 l 193 979 l 193 871 l 396 998 l 396 247 l 179 247 l 179 152 l 730 152 l 730 247 l 510 247 l 510 1142 z "
	},
	"2": {
		ha: 841,
		x_min: 0,
		x_max: 0,
		o: "m 545 553 b 711 892 662 681 711 779 b 415 1153 711 1053 597 1153 b 90 1031 296 1153 165 1104 l 133 941 b 412 1055 225 1017 317 1055 b 594 887 529 1055 594 996 b 461 621 594 803 553 724 l 114 239 l 114 152 l 751 152 l 751 247 l 263 247 z "
	},
	"3": {
		ha: 806,
		x_min: 0,
		x_max: 0,
		o: "m 732 420 b 553 659 732 543 670 627 b 711 901 651 697 711 787 b 418 1153 711 1055 594 1153 b 90 1031 290 1153 168 1107 l 133 941 b 412 1055 225 1017 317 1055 b 597 890 529 1055 597 996 b 369 703 597 768 515 703 l 277 703 l 277 605 l 391 605 b 618 423 543 605 618 545 b 404 239 618 301 545 239 b 119 353 307 239 209 277 l 73 263 b 407 141 155 187 279 141 b 732 420 608 141 732 247 z "
	},
	"4": {
		ha: 854,
		x_min: 0,
		x_max: 0,
		o: "m 637 1142 l 545 1142 l 68 453 l 68 366 l 524 366 l 524 152 l 637 152 l 637 366 l 787 366 l 787 461 l 637 461 z m 524 461 l 187 461 l 524 947 z "
	},
	"5": {
		ha: 873,
		x_min: 0,
		x_max: 0,
		o: "m 768 458 b 458 773 768 646 646 773 b 250 694 374 773 304 746 l 250 1047 l 727 1047 l 727 1142 l 136 1142 l 136 562 l 214 562 b 439 678 271 637 345 678 b 654 458 570 678 654 589 b 431 239 654 323 570 239 b 149 353 336 239 241 277 l 106 263 b 431 141 187 187 312 141 b 768 458 635 141 768 271 z "
	},
	"6": {
		ha: 844,
		x_min: 0,
		x_max: 0,
		o: "m 768 458 b 458 773 768 643 640 773 b 190 602 334 773 233 708 l 190 624 b 458 1055 190 895 288 1055 b 722 941 548 1055 629 1020 l 765 1031 b 456 1153 684 1107 570 1153 b 76 621 217 1153 76 955 b 448 141 76 315 209 141 b 768 458 632 141 768 277 z m 654 458 b 442 236 654 323 570 236 b 222 458 312 236 222 326 b 442 678 222 586 312 678 b 654 458 567 678 654 589 z "
	},
	"7": {
		ha: 833,
		x_min: 0,
		x_max: 0,
		o: "m 618 1044 l 157 152 l 279 152 l 743 1055 l 743 1142 l 90 1142 l 90 1044 z "
	},
	"8": {
		ha: 833,
		x_min: 0,
		x_max: 0,
		o: "m 770 415 b 567 659 770 534 697 627 b 749 890 681 694 749 779 b 418 1153 749 1047 613 1153 b 84 890 220 1153 84 1047 b 269 659 84 779 149 694 b 62 415 141 627 62 534 b 418 141 62 247 201 141 b 770 415 632 141 770 247 z m 418 692 b 198 882 282 708 198 776 b 418 1058 198 990 279 1058 b 635 882 553 1058 635 990 b 418 692 635 776 551 708 z m 659 420 b 418 236 659 301 572 236 b 176 420 260 236 176 301 b 418 613 176 532 271 597 b 659 420 562 597 659 532 z "
	},
	"9": {
		ha: 827,
		x_min: 0,
		x_max: 0,
		o: "m 760 670 b 385 1153 760 979 624 1153 b 68 833 203 1153 68 1017 b 374 521 68 648 195 521 b 646 689 499 521 602 583 l 646 670 b 377 236 646 396 548 236 b 111 350 288 236 206 271 l 68 263 b 377 141 152 184 266 141 b 760 670 618 141 760 336 z m 613 836 b 393 616 613 705 524 616 b 182 836 266 616 182 703 b 391 1055 182 968 266 1055 b 613 836 524 1055 613 968 z "
	},
	" ": {
		ha: 543,
		x_min: 0,
		x_max: 0,
		o: ""
	},
	"-": {
		ha: 594,
		x_min: 0,
		x_max: 0,
		o: "m 494 572 l 100 572 l 100 477 l 494 477 z "
	},
	".": {
		ha: 323,
		x_min: 0,
		x_max: 0,
		o: "m 233 152 l 233 296 l 90 296 l 90 152 z "
	},
	A: {
		ha: 1020,
		x_min: 0,
		x_max: 0,
		o: "m 559 1142 l 458 1142 l 24 152 l 144 152 l 252 407 l 770 407 l 879 152 l 996 152 z m 730 502 l 293 502 l 510 1009 z "
	},
	B: {
		ha: 985,
		x_min: 0,
		x_max: 0,
		o: "m 863 420 b 684 659 863 543 795 629 b 838 882 781 697 838 776 b 518 1142 838 1047 722 1142 l 122 1142 l 122 152 l 534 152 b 863 420 743 152 863 250 z m 502 1047 b 727 876 651 1047 727 990 b 502 703 727 762 651 703 l 236 703 l 236 1047 z m 749 423 b 524 247 749 304 678 247 l 236 247 l 236 605 l 524 605 b 749 423 675 605 749 545 z "
	},
	C: {
		ha: 968,
		x_min: 0,
		x_max: 0,
		o: "m 87 648 b 545 141 87 336 263 141 b 882 266 681 141 798 182 l 838 353 b 545 239 743 271 656 239 b 203 648 328 239 203 388 b 545 1053 203 909 328 1053 b 838 939 656 1053 743 1023 l 882 1028 b 545 1153 798 1109 681 1153 b 87 648 263 1153 87 958 z "
	},
	D: {
		ha: 1074,
		x_min: 0,
		x_max: 0,
		o: "m 453 152 b 952 648 770 152 952 331 b 453 1142 952 963 770 1142 l 122 1142 l 122 152 z m 236 1044 l 448 1044 b 833 648 705 1044 833 911 b 448 250 833 382 703 250 l 236 250 z "
	},
	E: {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 746 247 l 236 247 l 236 605 l 719 605 l 719 703 l 236 703 l 236 1047 l 746 1047 l 746 1142 l 122 1142 l 122 152 l 746 152 z "
	},
	F: {
		ha: 854,
		x_min: 0,
		x_max: 0,
		o: "m 236 605 l 705 605 l 705 703 l 236 703 l 236 1044 l 732 1044 l 732 1142 l 122 1142 l 122 152 l 236 152 z "
	},
	G: {
		ha: 990,
		x_min: 0,
		x_max: 0,
		o: "m 553 578 l 800 578 l 800 277 b 562 236 724 250 646 236 b 203 648 328 236 203 380 b 551 1055 203 911 328 1055 b 846 939 667 1055 751 1023 l 892 1028 b 551 1153 808 1109 689 1153 b 87 648 263 1153 87 960 b 559 141 87 331 263 141 b 903 209 689 141 827 168 l 903 667 l 553 667 z "
	},
	H: {
		ha: 1063,
		x_min: 0,
		x_max: 0,
		o: "m 236 703 l 236 1142 l 122 1142 l 122 152 l 236 152 l 236 605 l 827 605 l 827 152 l 941 152 l 941 1142 l 827 1142 l 827 703 z "
	},
	I: {
		ha: 358,
		x_min: 0,
		x_max: 0,
		o: "m 236 1142 l 122 1142 l 122 152 l 236 152 z "
	},
	J: {
		ha: 342,
		x_min: 0,
		x_max: 0,
		o: "m -8 -95 b 111 -106 41 -103 79 -106 b 334 165 271 -106 334 -27 l 334 1142 l 220 1142 l 220 168 b 92 -8 220 41 187 -8 b -8 3 68 -8 35 -3 z "
	},
	K: {
		ha: 1017,
		x_min: 0,
		x_max: 0,
		o: "m 868 1142 l 722 1142 l 236 686 l 236 1142 l 122 1142 l 122 152 l 236 152 l 236 624 l 746 152 l 895 152 l 353 659 z "
	},
	L: {
		ha: 854,
		x_min: 0,
		x_max: 0,
		o: "m 732 250 l 236 250 l 236 1142 l 122 1142 l 122 152 l 732 152 z "
	},
	M: {
		ha: 1180,
		x_min: 0,
		x_max: 0,
		o: "m 217 1142 l 122 1142 l 122 152 l 228 152 l 225 882 l 551 152 l 629 152 l 952 887 l 952 152 l 1058 152 l 1058 1142 l 963 1142 l 591 288 z "
	},
	N: {
		ha: 1036,
		x_min: 0,
		x_max: 0,
		o: "m 209 1142 l 122 1142 l 122 152 l 228 152 l 228 947 l 827 152 l 914 152 l 914 1142 l 806 1142 l 806 345 z "
	},
	O: {
		ha: 1074,
		x_min: 0,
		x_max: 0,
		o: "m 87 646 b 537 141 87 336 260 141 b 987 648 811 141 987 336 b 537 1153 987 958 817 1153 b 87 646 258 1153 87 958 z m 868 648 b 537 239 868 388 746 239 b 203 648 326 239 203 388 b 537 1055 203 909 326 1055 b 868 648 746 1055 868 909 z "
	},
	P: {
		ha: 963,
		x_min: 0,
		x_max: 0,
		o: "m 236 152 l 236 586 l 515 586 b 841 865 719 586 841 692 b 515 1142 841 1036 722 1142 l 122 1142 l 122 152 z m 236 1047 l 507 1047 b 727 865 654 1047 727 982 b 507 681 727 746 654 681 l 236 681 z "
	},
	Q: {
		ha: 1074,
		x_min: 0,
		x_max: 0,
		o: "m 882 -41 b 703 168 803 24 743 95 b 987 648 882 233 987 404 b 537 1153 987 958 817 1153 b 87 646 258 1153 87 958 b 537 141 87 336 260 141 b 586 144 548 141 564 141 b 803 -114 643 43 724 -54 z m 537 239 b 203 648 326 239 203 388 b 537 1055 203 909 326 1055 b 868 648 746 1055 868 909 b 537 239 868 388 746 239 z "
	},
	R: {
		ha: 1020,
		x_min: 0,
		x_max: 0,
		o: "m 773 486 b 648 608 746 562 705 600 b 846 865 776 646 846 738 b 515 1142 846 1044 727 1142 l 122 1142 l 122 152 l 236 152 l 236 583 l 483 583 b 656 480 578 583 627 562 l 779 152 l 898 152 z m 236 1047 l 505 1047 b 732 865 662 1047 732 987 b 505 678 732 741 656 678 l 236 678 z "
	},
	S: {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 68 266 b 437 141 157 182 282 141 b 800 420 659 141 800 252 b 629 635 800 534 730 602 b 431 692 581 651 515 673 b 198 871 274 730 198 770 b 445 1055 198 985 290 1055 b 741 939 556 1055 648 1017 l 787 1028 b 445 1153 705 1109 583 1153 b 84 865 228 1153 84 1036 b 250 637 84 749 152 678 b 437 583 298 618 361 602 b 686 415 600 545 684 521 b 437 239 689 304 597 239 b 109 353 304 239 209 271 z "
	},
	T: {
		ha: 838,
		x_min: 0,
		x_max: 0,
		o: "m 477 1044 l 822 1044 l 822 1142 l 16 1142 l 16 1044 l 361 1044 l 361 152 l 477 152 z "
	},
	U: {
		ha: 1015,
		x_min: 0,
		x_max: 0,
		o: "m 114 540 b 507 141 114 277 250 141 b 901 540 765 141 901 279 l 901 1142 l 787 1142 l 787 529 b 507 239 787 336 692 239 b 228 529 323 239 228 336 l 228 1142 l 114 1142 z "
	},
	V: {
		ha: 968,
		x_min: 0,
		x_max: 0,
		o: "m 125 1142 l 3 1142 l 434 152 l 534 152 l 966 1142 l 849 1142 l 486 296 z "
	},
	W: {
		ha: 1544,
		x_min: 0,
		x_max: 0,
		o: "m 819 1139 l 730 1142 l 442 315 l 163 1142 l 46 1142 l 393 152 l 488 152 l 773 960 l 1055 152 l 1150 152 l 1497 1142 l 1383 1142 l 1101 312 z "
	},
	X: {
		ha: 906,
		x_min: 0,
		x_max: 0,
		o: "m 743 1142 l 453 743 l 165 1142 l 30 1142 l 388 659 l 11 152 l 146 152 l 453 572 l 760 152 l 895 152 l 518 656 l 879 1142 z "
	},
	Y: {
		ha: 890,
		x_min: 0,
		x_max: 0,
		o: "m 423 654 l 109 1142 l -24 1142 l 363 553 l 363 152 l 477 152 l 477 551 l 865 1142 l 735 1142 z "
	},
	Z: {
		ha: 838,
		x_min: 0,
		x_max: 0,
		o: "m 762 1142 l 62 1142 l 62 1044 l 629 1044 l 62 236 l 62 152 l 776 152 l 776 247 l 195 247 l 762 1058 z "
	},
	a: {
		ha: 792,
		x_min: 0,
		x_max: 0,
		o: "m 613 730 b 377 868 570 817 486 868 b 68 502 193 868 68 719 b 377 141 68 285 190 141 b 613 279 486 141 572 190 l 613 152 l 724 152 l 724 852 l 613 852 z m 613 505 b 399 236 613 334 537 236 b 182 502 260 236 182 331 b 399 773 182 673 260 773 b 613 505 537 773 613 678 z "
	},
	b: {
		ha: 876,
		x_min: 0,
		x_max: 0,
		o: "m 768 507 b 456 868 768 724 643 868 b 220 727 347 868 260 817 l 220 1175 l 109 1175 l 109 152 l 220 152 l 220 282 b 456 141 263 193 347 141 b 768 507 643 141 768 290 z m 651 507 b 437 236 651 336 570 236 b 220 505 298 236 220 334 b 437 773 220 678 298 773 b 651 507 572 773 651 675 z "
	},
	c: {
		ha: 722,
		x_min: 0,
		x_max: 0,
		o: "m 68 502 b 401 141 68 285 198 141 b 654 233 496 141 594 176 l 616 315 b 412 236 545 260 480 236 b 184 502 269 236 184 334 b 412 773 184 673 269 773 b 616 692 480 773 545 749 l 654 776 b 407 868 597 833 502 868 b 68 502 201 868 68 719 z "
	},
	d: {
		ha: 792,
		x_min: 0,
		x_max: 0,
		o: "m 610 732 b 377 868 570 817 486 868 b 68 507 190 868 68 724 b 377 141 68 290 193 141 b 613 279 486 141 572 190 l 613 152 l 724 152 l 724 1175 l 610 1175 z m 613 505 b 399 236 613 334 537 236 b 182 507 260 236 182 336 b 399 773 182 675 260 773 b 613 505 537 773 613 675 z "
	},
	e: {
		ha: 760,
		x_min: 0,
		x_max: 0,
		o: "m 692 526 b 396 868 692 738 581 868 b 68 505 203 868 68 719 b 418 141 68 279 198 141 b 681 236 515 141 618 179 l 643 317 b 418 236 575 263 499 236 b 182 505 263 236 182 328 l 692 505 z m 184 578 b 396 779 206 705 279 779 b 591 578 513 779 581 711 z "
	},
	f: {
		ha: 458,
		x_min: 0,
		x_max: 0,
		o: "m 258 944 b 377 1082 258 1044 304 1082 b 448 1072 399 1082 423 1080 l 448 1166 b 355 1180 418 1175 385 1180 b 146 958 222 1180 146 1104 l 146 852 l 11 852 l 11 762 l 146 762 l 146 152 l 258 152 l 258 762 l 429 762 l 429 852 l 258 852 z "
	},
	g: {
		ha: 798,
		x_min: 0,
		x_max: 0,
		o: "m 618 730 b 380 868 575 817 491 868 b 68 521 193 868 68 727 b 380 174 68 312 193 174 b 618 312 491 174 575 225 l 618 146 b 415 -65 618 8 551 -65 b 160 5 320 -65 250 -43 l 141 -87 b 415 -157 217 -133 309 -157 b 730 157 621 -157 730 -49 l 730 852 l 618 852 z m 618 521 b 401 266 618 363 537 266 b 182 521 263 266 182 363 b 401 773 182 678 263 773 b 618 521 537 773 618 678 z "
	},
	h: {
		ha: 817,
		x_min: 0,
		x_max: 0,
		o: "m 708 591 b 456 868 708 776 624 868 b 220 738 353 868 266 822 l 220 1175 l 109 1175 l 109 152 l 220 152 l 220 551 b 434 773 220 686 307 773 b 597 586 545 773 597 713 l 597 152 l 708 152 z "
	},
	i: {
		ha: 331,
		x_min: 0,
		x_max: 0,
		o: "m 233 1020 l 233 1147 l 98 1147 l 98 1020 z m 220 852 l 109 852 l 109 152 l 220 152 z "
	},
	j: {
		ha: 309,
		x_min: 0,
		x_max: 0,
		o: "m 233 1020 l 233 1147 l 98 1147 l 98 1020 z m -8 -57 b -76 -46 -30 -57 -52 -52 l -76 -141 b 16 -155 -46 -149 -16 -155 b 220 68 144 -155 220 -76 l 220 852 l 109 852 l 109 81 b -8 -57 109 -11 71 -57 z "
	},
	k: {
		ha: 836,
		x_min: 0,
		x_max: 0,
		o: "m 692 849 l 551 849 l 220 529 l 220 1175 l 109 1175 l 109 152 l 220 152 l 220 488 l 581 152 l 727 152 l 342 515 z "
	},
	l: {
		ha: 328,
		x_min: 0,
		x_max: 0,
		o: "m 220 1175 l 109 1175 l 109 152 l 220 152 z "
	},
	m: {
		ha: 1213,
		x_min: 0,
		x_max: 0,
		o: "m 1115 591 b 884 868 1115 776 1036 868 b 651 732 779 868 692 814 b 437 868 616 825 548 868 b 214 727 334 868 255 817 l 203 852 l 98 852 b 109 654 103 792 109 727 l 109 152 l 220 152 l 220 551 b 412 773 220 689 293 773 b 556 586 513 773 556 713 l 556 152 l 670 152 l 670 551 b 857 773 670 689 741 773 b 1004 586 960 773 1004 716 l 1004 152 l 1115 152 z "
	},
	n: {
		ha: 806,
		x_min: 0,
		x_max: 0,
		o: "m 708 591 b 456 868 708 776 624 868 b 214 724 347 868 258 817 l 203 852 l 98 852 b 109 654 103 792 109 727 l 109 152 l 220 152 l 220 551 b 434 773 220 686 307 773 b 597 586 545 773 597 713 l 597 152 l 708 152 z "
	},
	o: {
		ha: 795,
		x_min: 0,
		x_max: 0,
		o: "m 68 505 b 396 141 68 285 198 141 b 727 505 597 141 727 285 b 396 868 727 724 597 868 b 68 505 198 868 68 724 z m 613 505 b 396 236 613 328 537 236 b 182 505 258 236 182 328 b 396 773 182 675 260 773 b 613 505 532 773 613 675 z "
	},
	p: {
		ha: 865,
		x_min: 0,
		x_max: 0,
		o: "m 768 502 b 456 868 768 719 643 868 b 214 716 342 868 255 811 l 203 852 l 98 852 b 109 654 103 792 109 727 l 109 -149 l 220 -149 l 220 282 b 456 141 263 193 347 141 b 768 502 643 141 768 285 z m 651 502 b 437 236 651 331 572 236 b 220 505 298 236 220 334 b 437 773 220 678 298 773 b 651 502 570 773 651 673 z "
	},
	q: {
		ha: 803,
		x_min: 0,
		x_max: 0,
		o: "m 616 719 b 377 868 575 814 488 868 b 68 502 193 868 68 719 b 377 141 68 285 190 141 b 610 277 486 141 570 190 l 610 -149 l 724 -149 l 724 654 b 735 852 724 727 727 792 l 629 852 z m 613 505 b 399 236 613 334 537 236 b 182 502 260 236 182 331 b 399 773 182 673 260 773 b 613 505 537 773 613 678 z "
	},
	r: {
		ha: 594,
		x_min: 0,
		x_max: 0,
		o: "m 496 860 b 434 868 477 865 456 868 b 214 722 334 868 250 817 l 203 852 l 98 852 b 109 654 103 792 109 727 l 109 152 l 220 152 l 220 551 b 269 703 220 610 239 662 b 420 768 304 746 353 768 b 494 757 448 768 475 762 z "
	},
	s: {
		ha: 684,
		x_min: 0,
		x_max: 0,
		o: "m 65 233 b 345 141 133 174 225 141 b 618 342 513 141 618 220 b 426 532 618 439 556 499 l 309 562 b 201 667 233 578 201 610 b 358 776 201 735 258 776 b 567 694 431 776 502 749 l 605 776 b 358 868 545 836 456 868 b 90 659 195 868 90 781 b 271 472 90 567 155 499 l 391 445 b 510 339 475 423 510 393 b 350 233 510 271 453 233 b 103 315 252 233 179 255 z "
	},
	t: {
		ha: 469,
		x_min: 0,
		x_max: 0,
		o: "m 458 852 l 258 852 l 258 1069 l 146 1031 l 146 852 l 11 852 l 11 762 l 146 762 l 146 363 b 358 141 146 220 225 141 b 456 157 393 141 426 146 l 456 252 b 380 239 431 244 407 239 b 258 380 304 239 258 277 l 258 762 l 458 762 z "
	},
	u: {
		ha: 795,
		x_min: 0,
		x_max: 0,
		o: "m 581 456 b 374 236 581 323 496 236 b 217 420 269 236 217 293 l 217 852 l 103 852 l 103 418 b 355 141 103 236 190 141 b 583 274 456 141 540 187 l 583 152 l 692 152 l 692 852 l 581 852 z "
	},
	v: {
		ha: 738,
		x_min: 0,
		x_max: 0,
		o: "m 136 849 l 16 849 l 315 152 l 420 152 l 722 849 l 610 849 l 372 274 z "
	},
	w: {
		ha: 1207,
		x_min: 0,
		x_max: 0,
		o: "m 656 852 l 556 852 l 353 288 l 155 852 l 38 852 l 296 152 l 404 152 l 602 697 l 803 152 l 911 152 l 1169 852 l 1058 852 l 857 285 z "
	},
	x: {
		ha: 743,
		x_min: 0,
		x_max: 0,
		o: "m 572 852 l 372 594 l 168 852 l 33 852 l 307 513 l 16 152 l 152 152 l 372 426 l 591 152 l 727 152 l 439 513 l 711 852 z "
	},
	y: {
		ha: 738,
		x_min: 0,
		x_max: 0,
		o: "m 372 274 l 136 852 l 16 852 l 312 160 l 285 98 b 201 -16 263 46 233 8 b 52 -76 165 -41 117 -60 l 76 -163 b 396 87 225 -136 328 -71 l 722 852 l 610 852 z "
	},
	z: {
		ha: 730,
		x_min: 0,
		x_max: 0,
		o: "m 646 852 l 84 852 l 84 762 l 510 762 l 71 231 l 71 152 l 659 152 l 659 241 l 209 241 l 646 768 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1362 1059 b 1297 1215 1420 1117 1379 1215 l 92 1215 b 27 1059 10 1215 -31 1117 l 608 478 l 608 -43 l 456 -43 b 347 -152 396 -43 347 -92 b 369 -174 347 -164 357 -174 l 1020 -174 b 1042 -152 1032 -174 1042 -164 b 933 -43 1042 -92 993 -43 l 781 -43 l 781 478 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1389 1128 b 1276 1211 1389 1187 1332 1229 l 408 955 b 347 872 372 943 347 910 l 347 163 b 260 174 320 169 291 174 b 0 0 117 174 0 96 b 260 -174 0 -96 117 -174 b 521 0 404 -174 521 -96 l 521 634 l 1215 837 l 1215 337 b 1128 347 1188 343 1159 347 b 868 174 985 347 868 270 b 1128 0 868 78 985 0 b 1389 174 1272 0 1389 78 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1370 14 l 1099 285 b 1053 304 1087 297 1071 304 l 1009 304 b 1128 651 1084 400 1128 520 b 564 1215 1128 963 876 1215 b 0 651 253 1215 0 963 b 564 87 0 339 253 87 b 911 206 695 87 816 131 l 911 162 b 930 116 911 145 918 128 l 1201 -155 b 1293 -155 1226 -180 1268 -180 l 1370 -78 b 1370 14 1395 -52 1395 -11 z m 564 304 b 217 651 372 304 217 459 b 564 998 217 843 372 998 b 911 651 756 998 911 843 b 564 304 911 459 756 304 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1254 1045 b 748 1009 1105 1172 884 1149 l 694 954 l 641 1009 b 135 1045 505 1149 283 1172 b 108 481 -36 900 -44 639 l 633 -60 b 756 -60 667 -95 722 -95 l 1281 481 b 1254 1045 1433 639 1424 900 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 703 1167 l 526 808 l 130 750 b 82 602 59 740 30 652 l 369 323 l 301 -72 b 427 -164 289 -144 364 -197 l 781 23 l 1136 -164 b 1262 -72 1199 -197 1274 -144 l 1194 323 l 1481 602 b 1433 750 1532 652 1504 740 l 1036 808 l 859 1167 b 703 1167 827 1231 735 1232 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 608 521 b 955 868 799 521 955 676 b 608 1215 955 1060 799 1215 b 260 868 416 1215 260 1060 b 608 521 260 676 416 521 z m 851 434 l 805 434 b 608 391 745 406 678 391 b 410 434 537 391 470 406 l 365 434 b 0 69 163 434 0 271 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 69 b 851 434 1215 271 1052 434 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1324 1042 l 1302 1042 l 1302 987 b 1270 955 1302 970 1287 955 l 1161 955 b 1128 987 1143 955 1128 970 l 1128 1042 l 260 1042 l 260 987 b 228 955 260 970 246 955 l 119 955 b 87 987 101 955 87 970 l 87 1042 l 65 1042 b 0 977 29 1042 0 1013 l 0 65 b 65 0 0 29 29 0 l 87 0 l 87 54 b 119 87 87 72 101 87 l 228 87 b 260 54 246 87 260 72 l 260 0 l 1128 0 l 1128 54 b 1161 87 1128 72 1143 87 l 1270 87 b 1302 54 1287 87 1302 72 l 1302 0 l 1324 0 b 1389 65 1360 0 1389 29 l 1389 977 b 1324 1042 1389 1013 1360 1042 z m 260 206 b 228 174 260 188 246 174 l 119 174 b 87 206 101 174 87 188 l 87 315 b 119 347 87 333 101 347 l 228 347 b 260 315 246 347 260 333 z m 260 467 b 228 434 260 449 246 434 l 119 434 b 87 467 101 434 87 449 l 87 575 b 119 608 87 593 101 608 l 228 608 b 260 575 246 608 260 593 z m 260 727 b 228 694 260 709 246 694 l 119 694 b 87 727 101 694 87 709 l 87 836 b 119 868 87 853 101 868 l 228 868 b 260 836 246 868 260 853 z m 998 163 b 966 130 998 145 984 130 l 423 130 b 391 163 405 130 391 145 l 391 423 b 423 456 391 441 405 456 l 966 456 b 998 423 984 456 998 441 z m 998 618 b 966 586 998 601 984 586 l 423 586 b 391 618 405 586 391 601 l 391 879 b 423 911 391 897 405 911 l 966 911 b 998 879 984 911 998 897 z m 1302 206 b 1270 174 1302 188 1287 174 l 1161 174 b 1128 206 1143 174 1128 188 l 1128 315 b 1161 347 1128 333 1143 347 l 1270 347 b 1302 315 1287 347 1302 333 z m 1302 467 b 1270 434 1302 449 1287 434 l 1161 434 b 1128 467 1143 434 1128 449 l 1128 575 b 1161 608 1128 593 1143 608 l 1270 608 b 1302 575 1287 608 1302 593 z m 1302 727 b 1270 694 1302 709 1287 694 l 1161 694 b 1128 727 1143 694 1128 709 l 1128 836 b 1161 868 1128 853 1143 868 l 1270 868 b 1302 836 1287 868 1302 853 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 803 1128 b 738 1063 767 1128 738 1099 l 738 629 b 803 564 738 593 767 564 l 1324 564 b 1389 629 1360 564 1389 593 l 1389 1063 b 1324 1128 1389 1099 1360 1128 z m 586 1128 l 65 1128 b 0 1063 29 1128 0 1099 l 0 629 b 65 564 0 593 29 564 l 586 564 b 651 629 622 564 651 593 l 651 1063 b 586 1128 651 1099 622 1128 z m 0 412 l 0 -22 b 65 -87 0 -58 29 -87 l 586 -87 b 651 -22 622 -87 651 -58 l 651 412 b 586 477 651 448 622 477 l 65 477 b 0 412 29 477 0 448 z m 803 -87 l 1324 -87 b 1389 -22 1360 -87 1389 -58 l 1389 412 b 1324 477 1389 448 1360 477 l 803 477 b 738 412 767 477 738 448 l 738 -22 b 803 -87 738 -58 767 -87 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 405 1063 b 340 1128 405 1099 376 1128 l 65 1128 b 0 1063 29 1128 0 1099 l 0 846 b 65 781 0 810 29 781 l 340 781 b 405 846 376 781 405 810 z m 897 412 l 897 629 b 832 694 897 665 868 694 l 557 694 b 492 629 521 694 492 665 l 492 412 b 557 347 492 376 521 347 l 832 347 b 897 412 868 347 897 376 z m 984 1063 l 984 846 b 1049 781 984 810 1013 781 l 1324 781 b 1389 846 1360 781 1389 810 l 1389 1063 b 1324 1128 1389 1099 1360 1128 l 1049 1128 b 984 1063 1013 1128 984 1099 z m 897 846 l 897 1063 b 832 1128 897 1099 868 1128 l 557 1128 b 492 1063 521 1128 492 1099 l 492 846 b 557 781 492 810 521 781 l 832 781 b 897 846 868 781 897 810 z m 340 694 l 65 694 b 0 629 29 694 0 665 l 0 412 b 65 347 0 376 29 347 l 340 347 b 405 412 376 347 405 376 l 405 629 b 340 694 405 665 376 694 z m 0 195 l 0 -22 b 65 -87 0 -58 29 -87 l 340 -87 b 405 -22 376 -87 405 -58 l 405 195 b 340 260 405 231 376 260 l 65 260 b 0 195 29 260 0 231 z m 1049 347 l 1324 347 b 1389 412 1360 347 1389 376 l 1389 629 b 1324 694 1389 665 1360 694 l 1049 694 b 984 629 1013 694 984 665 l 984 412 b 1049 347 984 376 1013 347 z m 1049 -87 l 1324 -87 b 1389 -22 1360 -87 1389 -58 l 1389 195 b 1324 260 1389 231 1360 260 l 1049 260 b 984 195 1013 260 984 231 l 984 -22 b 1049 -87 984 -58 1013 -87 z m 492 195 l 492 -22 b 557 -87 492 -58 521 -87 l 832 -87 b 897 -22 868 -87 897 -58 l 897 195 b 832 260 897 231 868 260 l 557 260 b 492 195 521 260 492 231 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 405 629 b 340 694 405 665 376 694 l 65 694 b 0 629 29 694 0 665 l 0 412 b 65 347 0 376 29 347 l 340 347 b 405 412 376 347 405 376 z m 0 195 l 0 -22 b 65 -87 0 -58 29 -87 l 340 -87 b 405 -22 376 -87 405 -58 l 405 195 b 340 260 405 231 376 260 l 65 260 b 0 195 29 260 0 231 z m 340 1128 l 65 1128 b 0 1063 29 1128 0 1099 l 0 846 b 65 781 0 810 29 781 l 340 781 b 405 846 376 781 405 810 l 405 1063 b 340 1128 405 1099 376 1128 z m 557 -87 l 1324 -87 b 1389 -22 1360 -87 1389 -58 l 1389 195 b 1324 260 1389 231 1360 260 l 557 260 b 492 195 521 260 492 231 l 492 -22 b 557 -87 492 -58 521 -87 z m 492 1063 l 492 846 b 557 781 492 810 521 781 l 1324 781 b 1389 846 1360 781 1389 810 l 1389 1063 b 1324 1128 1389 1099 1360 1128 l 557 1128 b 492 1063 521 1128 492 1099 z m 557 347 l 1324 347 b 1389 412 1360 347 1389 376 l 1389 629 b 1324 694 1389 665 1360 694 l 557 694 b 492 629 521 694 492 665 l 492 412 b 557 347 492 376 521 347 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 472 23 b 570 23 499 -4 543 -4 l 1369 822 b 1369 920 1396 849 1396 893 l 1270 1018 b 1172 1018 1243 1045 1199 1045 l 521 367 l 217 671 b 119 671 190 698 146 698 l 20 573 b 20 475 -7 546 -7 502 z "
	},
	"": {
		ha: 955,
		x_min: 0,
		x_max: 0,
		o: "m 658 521 l 930 792 b 930 913 963 826 963 880 l 870 973 b 749 973 836 1007 782 1007 l 477 702 l 206 973 b 85 973 173 1007 119 1007 l 25 913 b 25 792 -8 880 -8 826 l 296 521 l 25 249 b 25 129 -8 216 -8 162 l 85 68 b 206 68 119 35 173 35 l 477 340 l 749 68 b 870 68 782 35 836 35 l 930 129 b 930 249 963 162 963 216 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 825 694 b 792 727 825 712 810 727 l 640 727 l 640 879 b 608 911 640 897 626 911 l 521 911 b 488 879 503 911 488 897 l 488 727 l 336 727 b 304 694 318 727 304 712 l 304 608 b 336 575 304 590 318 575 l 488 575 l 488 423 b 521 391 488 405 503 391 l 608 391 b 640 423 626 391 640 405 l 640 575 l 792 575 b 825 608 810 575 825 590 z m 1370 -78 b 1370 14 1395 -52 1395 -11 l 1099 285 b 1053 304 1087 297 1071 304 l 1009 304 b 1128 651 1084 400 1128 520 b 564 1215 1128 963 876 1215 b 0 651 253 1215 0 963 b 564 87 0 339 253 87 b 911 206 695 87 816 131 l 911 162 b 930 116 911 145 918 128 l 1201 -155 b 1293 -155 1226 -180 1268 -180 z m 933 651 b 564 282 933 447 768 282 b 195 651 360 282 195 447 b 564 1020 195 855 360 1020 b 933 651 768 1020 933 855 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 825 694 b 792 727 825 712 810 727 l 336 727 b 304 694 318 727 304 712 l 304 608 b 336 575 304 590 318 575 l 792 575 b 825 608 810 575 825 590 z m 1370 -78 b 1370 14 1395 -52 1395 -11 l 1099 285 b 1053 304 1087 297 1071 304 l 1009 304 b 1128 651 1084 400 1128 520 b 564 1215 1128 963 876 1215 b 0 651 253 1215 0 963 b 564 87 0 339 253 87 b 911 206 695 87 816 131 l 911 162 b 930 116 911 145 918 128 l 1201 -155 b 1293 -155 1226 -180 1268 -180 z m 933 651 b 564 282 933 447 768 282 b 195 651 360 282 195 447 b 564 1020 195 855 360 1020 b 933 651 768 1020 933 855 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1085 1069 b 991 1047 1054 1091 1010 1081 l 948 971 b 965 887 932 943 939 907 b 1150 524 1083 800 1149 664 b 694 65 1152 266 943 65 b 239 521 441 65 238 271 b 423 887 239 671 311 803 b 441 971 449 907 457 943 l 398 1047 b 303 1068 379 1081 335 1091 b 22 520 133 946 21 746 b 696 -152 22 149 326 -153 b 1367 521 1067 -151 1367 150 b 1085 1069 1367 747 1256 946 z m 803 499 l 803 1150 b 738 1215 803 1186 774 1215 l 651 1215 b 586 1150 615 1215 586 1186 l 586 499 b 651 434 586 463 615 434 l 738 434 b 803 499 774 434 803 463 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 586 434 l 456 434 b 412 391 432 434 412 415 l 412 -130 b 456 -174 412 -154 432 -174 l 586 -174 b 629 -130 610 -174 629 -154 l 629 391 b 586 434 629 415 610 434 z m 239 174 l 109 174 b 65 130 85 174 65 154 l 65 -130 b 109 -174 65 -154 85 -174 l 239 -174 b 282 -130 263 -174 282 -154 l 282 130 b 239 174 282 154 263 174 z m 933 694 l 803 694 b 760 651 779 694 760 675 l 760 -130 b 803 -174 760 -154 779 -174 l 933 -174 b 977 -130 957 -174 977 -154 l 977 651 b 933 694 977 675 957 694 z m 1280 955 l 1150 955 b 1107 911 1126 955 1107 935 l 1107 -130 b 1150 -174 1107 -154 1126 -174 l 1280 -174 b 1324 -130 1304 -174 1324 -154 l 1324 911 b 1280 955 1324 935 1304 955 z m 1628 1215 l 1497 1215 b 1454 1172 1473 1215 1454 1196 l 1454 -130 b 1497 -174 1454 -154 1473 -174 l 1628 -174 b 1671 -130 1652 -174 1671 -154 l 1671 1172 b 1628 1215 1671 1196 1652 1215 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1322 359 l 1207 426 b 1207 616 1218 489 1218 553 l 1322 683 b 1337 721 1335 690 1341 706 b 1189 977 1307 817 1256 905 b 1149 984 1178 988 1162 991 l 1033 917 b 868 1012 984 959 929 991 l 868 1145 b 843 1177 868 1160 857 1174 b 546 1177 743 1199 641 1198 b 521 1145 531 1174 521 1160 l 521 1012 b 356 917 461 990 405 958 l 241 983 b 200 977 227 991 211 988 b 52 720 133 905 82 817 b 67 683 47 706 54 690 l 183 616 b 183 425 171 553 171 488 l 67 359 b 52 321 54 351 48 335 b 200 64 82 224 133 137 b 241 58 211 53 228 50 l 356 125 b 521 29 405 83 461 50 l 521 -104 b 547 -136 521 -119 532 -133 b 843 -136 646 -158 748 -157 b 868 -104 858 -133 868 -119 l 868 29 b 1033 125 929 51 984 83 l 1149 58 b 1189 64 1162 50 1179 53 b 1337 321 1256 136 1307 224 b 1322 359 1341 336 1335 351 z m 694 304 b 477 521 575 304 477 401 b 694 738 477 640 575 738 b 911 521 814 738 911 640 b 694 304 911 401 814 304 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 761 813 l 260 401 l 260 -43 b 304 -87 260 -67 280 -87 l 608 -86 b 651 -43 632 -86 651 -66 l 651 -43 l 651 217 b 694 260 651 241 670 260 l 868 260 b 911 217 892 260 911 241 l 911 -42 b 911 -43 911 -42 911 -43 b 955 -86 911 -67 931 -86 l 955 -86 l 1259 -87 b 1302 -43 1283 -87 1302 -67 l 1302 401 l 802 813 b 781 820 797 817 788 820 b 761 813 775 820 766 817 z m 1551 533 l 1324 720 l 1324 1096 b 1291 1128 1324 1114 1309 1128 l 1139 1128 b 1107 1096 1121 1128 1107 1114 l 1107 899 l 864 1099 b 781 1128 844 1115 807 1128 b 698 1099 755 1128 718 1115 l 12 533 b 0 508 5 528 0 516 b 7 487 0 502 3 492 l 77 403 b 102 391 82 397 93 391 b 122 399 108 391 117 395 l 761 924 b 781 932 766 928 775 932 b 802 924 788 932 797 928 l 1440 399 b 1461 391 1445 395 1454 391 b 1486 403 1469 391 1481 397 l 1555 487 b 1563 508 1559 492 1563 501 b 1551 533 1563 516 1557 528 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 z m 849 244 l 610 418 b 597 444 602 424 597 434 l 597 901 b 629 933 597 919 611 933 l 760 933 b 792 901 777 933 792 919 l 792 527 l 964 402 b 971 356 979 391 982 371 l 895 251 b 849 244 884 237 864 233 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1555 123 l 1176 991 b 1100 1042 1162 1022 1133 1042 l 836 1042 l 842 979 b 821 955 844 966 834 955 l 742 955 b 720 979 729 955 719 966 l 727 1042 l 462 1042 b 387 991 430 1042 400 1022 l 8 123 b 83 0 -17 65 23 0 l 617 0 l 645 265 b 688 304 647 287 666 304 l 875 304 b 918 265 897 304 915 287 l 946 0 l 1480 0 b 1555 123 1540 0 1580 65 z m 706 849 b 728 868 708 859 717 868 l 728 868 l 835 868 b 856 849 846 868 855 860 l 869 730 b 836 694 871 711 856 694 l 726 694 b 694 730 707 694 692 711 z m 856 391 l 706 391 b 663 439 680 391 660 413 l 677 569 b 720 608 679 591 698 608 l 842 608 b 886 569 865 608 883 591 l 899 439 b 856 391 902 413 882 391 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 586 1215 b 521 1150 550 1215 521 1186 l 521 694 l 283 694 b 244 602 234 694 210 636 l 658 189 b 732 189 678 169 711 169 l 1144 602 b 1106 694 1178 636 1154 694 l 868 694 l 868 1150 b 803 1215 868 1186 839 1215 z m 1389 195 b 1324 260 1389 231 1360 260 l 926 260 l 793 127 b 596 127 738 73 650 73 l 463 260 l 65 260 b 0 195 29 260 0 231 l 0 -109 b 65 -174 0 -145 29 -174 l 1324 -174 b 1389 -109 1360 -174 1389 -145 z m 1053 -43 b 998 -98 1053 -73 1028 -98 b 944 -43 968 -98 944 -73 b 998 11 944 -14 968 11 b 1053 -43 1028 11 1053 -14 z m 1226 -43 b 1172 -98 1226 -73 1202 -98 b 1118 -43 1142 -98 1118 -73 b 1172 11 1118 -14 1142 11 b 1226 -43 1202 11 1226 -14 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1541 554 l 1254 984 b 1146 1042 1233 1016 1184 1042 l 417 1042 b 309 984 378 1042 330 1016 l 22 554 b 0 481 10 536 0 503 l 0 481 l 0 130 b 130 0 0 58 58 0 l 1432 0 b 1563 130 1504 0 1563 58 l 1563 481 l 1563 481 b 1541 554 1563 503 1553 536 z m 440 868 l 1122 868 l 1354 521 l 1020 521 l 933 347 l 629 347 l 543 521 l 209 521 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1357 1215 l 1229 1215 l 1229 1215 b 1196 1183 1211 1215 1196 1201 b 1196 1181 1196 1182 1196 1182 l 1207 957 b 695 1194 1096 1087 867 1194 l 694 1194 b 22 520 324 1194 21 891 b 694 -152 22 149 323 -152 l 695 -152 b 1145 21 838 -152 1040 -74 b 1156 46 1151 27 1156 38 b 1147 69 1156 53 1152 63 l 1054 161 b 1031 171 1049 166 1039 171 b 1010 162 1025 171 1015 167 b 695 43 935 97 794 43 b 217 521 431 43 217 257 b 695 998 217 784 431 998 b 1091 787 835 998 1012 904 l 815 800 b 814 800 815 800 814 800 b 781 768 796 800 781 786 l 781 768 l 781 639 b 814 607 781 621 796 607 l 1357 607 b 1390 639 1375 607 1390 621 l 1390 1183 b 1357 1215 1390 1201 1375 1215 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1195 1181 l 1206 957 b 695 1194 1095 1087 866 1194 l 694 1194 b 33 646 365 1194 92 958 b 33 640 33 644 33 642 b 65 608 33 622 47 608 l 198 608 b 230 633 213 608 227 619 b 694 998 279 834 486 998 b 1090 787 834 998 1012 904 l 815 800 b 813 800 814 800 814 800 b 781 768 795 800 781 786 l 781 768 l 781 639 b 813 607 781 621 795 607 l 1356 607 b 1389 639 1374 607 1389 621 l 1389 1183 b 1356 1215 1389 1201 1374 1215 l 1228 1215 b 1195 1183 1210 1215 1195 1201 b 1195 1181 1195 1182 1195 1182 z m 694 43 l 694 43 b 298 254 553 43 376 138 l 574 241 b 576 241 575 241 575 241 b 608 274 594 241 608 256 l 608 274 l 608 402 b 576 435 608 420 594 435 l 33 435 b 0 402 15 435 0 420 l 0 -141 b 33 -174 0 -159 15 -174 l 161 -174 l 161 -174 b 194 -141 179 -174 194 -159 b 194 -139 194 -141 194 -140 l 182 85 b 693 -152 293 -46 522 -152 l 694 -152 b 1355 396 1023 -152 1296 84 b 1355 401 1355 397 1355 400 b 1323 434 1355 419 1341 434 l 1190 434 b 1158 409 1175 434 1161 423 b 694 43 1109 207 902 43 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1259 -87 b 1389 43 1331 -87 1389 -29 l 1389 998 b 1259 1128 1389 1070 1331 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -29 58 -87 z m 347 890 b 456 781 407 890 456 841 b 347 673 456 721 407 673 b 239 781 287 673 239 721 b 347 890 239 841 287 890 z m 347 629 b 456 521 407 629 456 581 b 347 412 456 461 407 412 b 239 521 287 412 239 461 b 347 629 239 581 287 629 z m 347 369 b 456 260 407 369 456 320 b 347 152 456 200 407 152 b 239 260 287 152 239 200 b 347 369 239 320 287 369 z m 1128 738 b 1096 705 1128 720 1114 705 l 553 705 b 521 738 535 705 521 720 l 521 825 b 553 857 521 843 535 857 l 1096 857 b 1128 825 1114 857 1128 843 z m 1128 477 b 1096 445 1128 459 1114 445 l 553 445 b 521 477 535 445 521 459 l 521 564 b 553 597 521 582 535 597 l 1096 597 b 1128 564 1114 597 1128 582 z m 1128 217 b 1096 184 1128 199 1114 184 l 553 184 b 521 217 535 184 521 199 l 521 304 b 553 336 521 322 535 336 l 1096 336 b 1128 304 1114 336 1128 322 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1085 608 l 1020 608 l 1020 803 b 608 1215 1020 1030 835 1215 b 195 803 380 1215 195 1030 l 195 608 l 130 608 b 0 477 58 608 0 549 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 477 b 1085 608 1215 549 1157 608 z m 803 608 l 412 608 l 412 803 b 608 998 412 911 500 998 b 803 803 715 998 803 911 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 948 947 b 500 1042 803 947 683 1042 b 315 1009 432 1042 372 1030 b 326 1063 321 1024 326 1048 b 325 1073 326 1066 325 1070 b 180 1215 320 1150 257 1212 b 22 1063 93 1219 22 1149 b 87 939 22 1012 47 966 l 87 -109 b 152 -174 87 -144 116 -174 l 195 -174 b 260 -109 231 -174 260 -144 l 260 148 b 571 208 337 180 433 208 b 1019 113 716 208 836 113 b 1351 224 1150 113 1254 157 b 1389 296 1375 240 1389 267 l 1389 955 b 1266 1034 1389 1018 1323 1060 b 948 947 1172 990 1058 947 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1128 b 0 434 311 1128 0 818 l 0 304 l 0 304 b 48 226 0 274 21 239 l 87 207 b 391 -87 92 44 226 -87 l 456 -87 b 521 -22 492 -87 521 -58 l 521 456 b 456 521 521 492 492 521 l 391 521 b 174 430 306 521 229 486 l 174 434 b 694 955 174 721 407 955 b 1215 434 982 955 1215 721 l 1215 430 b 998 521 1160 486 1083 521 l 933 521 b 868 456 897 521 868 492 l 868 -22 b 933 -87 868 -58 897 -87 l 998 -87 b 1302 207 1163 -87 1296 44 l 1341 226 b 1389 304 1367 239 1389 274 l 1389 434 b 694 1128 1389 818 1078 1128 z "
	},
	"": {
		ha: 694,
		x_min: 0,
		x_max: 0,
		o: "m 583 1023 l 342 781 l 65 781 b 0 716 29 781 0 752 l 0 326 b 65 260 0 290 29 260 l 342 260 l 583 19 b 694 65 624 -22 694 7 l 694 977 b 583 1023 694 1035 624 1063 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 583 1020 l 342 779 l 65 779 b 0 713 29 779 0 749 l 0 323 b 65 258 0 287 29 258 l 342 258 l 583 16 b 694 62 624 -24 694 4 l 694 974 b 583 1020 694 1032 624 1061 z m 918 727 b 829 701 886 744 846 733 b 855 613 812 670 823 630 b 911 518 890 593 911 557 b 855 424 911 479 890 443 b 829 335 823 406 812 367 b 918 310 847 304 886 292 b 1042 518 994 352 1042 432 b 918 727 1042 605 994 684 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 583 1023 l 342 781 l 65 781 b 0 716 29 781 0 752 l 0 326 b 65 260 0 290 29 260 l 342 260 l 583 19 b 694 65 624 -22 694 7 l 694 977 b 583 1023 694 1035 624 1063 z m 1216 1161 b 1125 1142 1186 1181 1145 1173 b 1144 1051 1105 1112 1114 1071 b 1431 521 1324 933 1431 735 b 1144 -10 1431 307 1324 108 b 1125 -101 1114 -30 1105 -70 b 1216 -119 1144 -130 1185 -140 b 1563 521 1433 23 1563 262 b 1216 1161 1563 779 1433 1019 z m 1302 521 b 1069 945 1302 693 1215 852 b 980 924 1039 964 999 955 b 1000 834 960 894 969 853 b 1172 521 1108 765 1172 648 b 1000 208 1172 394 1108 277 b 980 117 969 188 960 148 b 1069 97 997 89 1037 76 b 1302 521 1215 190 1302 348 z m 918 729 b 829 704 886 747 846 735 b 855 615 812 672 823 633 b 911 521 890 596 911 560 b 855 426 911 482 890 446 b 829 338 823 409 812 369 b 918 312 847 306 886 295 b 1042 521 994 354 1042 434 b 918 729 1042 607 994 687 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 0 608 l 521 608 l 521 1128 l 0 1128 z m 174 955 l 347 955 l 347 781 l 174 781 z m 694 1128 l 694 608 l 1215 608 l 1215 1128 z m 1042 781 l 868 781 l 868 955 l 1042 955 z m 0 -87 l 521 -87 l 521 434 l 0 434 z m 174 260 l 347 260 l 347 87 l 174 87 z m 1128 434 l 1128 347 l 955 347 l 955 434 l 694 434 l 694 -87 l 868 -87 l 868 174 l 955 174 l 955 87 l 1215 87 l 1215 434 z m 1128 0 l 1128 -87 l 1215 -87 l 1215 0 z m 955 0 l 955 -87 l 1042 -87 l 1042 0 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 0 0 l 49 0 l 49 1042 l 0 1042 z m 73 1 l 98 1 l 98 1042 l 73 1042 z m 146 1 l 171 1 l 171 1042 l 146 1042 z m 268 1 l 292 1 l 292 1042 l 268 1042 z m 366 1 l 414 1 l 414 1042 l 366 1042 z m 488 1 l 512 1 l 512 1042 l 488 1042 z m 536 1 l 560 1 l 560 1042 l 536 1042 z m 585 1 l 609 1 l 609 1042 l 585 1042 z m 682 1 l 731 1 l 731 1042 l 682 1042 z m 804 1 l 853 1 l 853 1042 l 804 1042 z m 901 1 l 950 1 l 950 1042 l 901 1042 z m 999 1 l 1048 1 l 1048 1042 l 999 1042 z m 1072 1 l 1121 1 l 1121 1042 l 1072 1042 z m 1194 1 l 1267 1 l 1267 1042 l 1194 1042 z m 1291 1 l 1316 1 l 1316 1042 l 1291 1042 z m 1340 0 l 1389 0 l 1389 1042 l 1340 1042 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 0 531 b 38 439 0 502 17 460 l 613 -135 b 797 -135 664 -186 746 -186 l 1351 418 b 1351 602 1402 469 1402 552 l 776 1177 b 684 1215 755 1198 714 1215 l 130 1215 b 0 1085 58 1215 0 1157 z m 304 1042 b 434 911 376 1042 434 983 b 304 781 434 840 376 781 b 174 911 232 781 174 840 b 304 1042 174 983 232 1042 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1351 602 l 776 1177 b 684 1215 755 1198 714 1215 l 130 1215 b 0 1085 58 1215 0 1157 l 0 531 b 38 439 0 502 17 460 l 613 -135 b 797 -135 664 -186 746 -186 l 1351 418 b 1351 602 1402 469 1402 552 z m 304 781 b 174 911 232 781 174 840 b 304 1042 174 983 232 1042 b 434 911 376 1042 434 983 b 304 781 434 840 376 781 z m 1698 418 b 1698 602 1749 469 1749 552 l 1123 1177 b 1031 1215 1102 1198 1061 1215 l 899 1215 l 1431 683 b 1503 510 1477 637 1503 576 b 1431 338 1503 445 1477 384 l 959 -134 l 960 -135 b 1144 -135 1011 -186 1093 -186 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1215 239 l 1215 1150 b 1150 1215 1215 1186 1186 1215 l 260 1215 b 0 955 117 1215 0 1099 l 0 87 b 260 -174 0 -57 117 -174 l 1150 -174 b 1215 -109 1186 -174 1215 -145 l 1215 -65 b 1191 -14 1215 -45 1206 -26 b 1191 188 1180 27 1180 146 b 1215 239 1206 200 1215 218 z m 347 852 b 363 868 347 861 355 868 l 939 868 b 955 852 948 868 955 861 l 955 798 b 939 781 955 789 948 781 l 363 781 b 347 798 355 781 347 789 z m 347 678 b 363 694 347 687 355 694 l 939 694 b 955 678 948 694 955 687 l 955 624 b 939 608 955 615 948 608 l 363 608 b 347 624 355 608 347 615 z m 1035 0 l 260 0 b 174 87 212 0 174 39 b 260 174 174 135 213 174 l 1035 174 b 1035 0 1029 127 1029 46 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 0 -174 l 521 130 l 1042 -174 l 1042 1085 b 911 1215 1042 1157 983 1215 l 130 1215 b 0 1085 58 1215 0 1157 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1215 694 l 1215 1006 b 1190 1067 1215 1029 1206 1051 l 1067 1190 b 1006 1215 1051 1206 1029 1215 l 260 1215 b 174 1128 212 1215 174 1176 l 174 694 b 0 521 78 694 0 617 l 0 217 b 43 174 0 193 19 174 l 174 174 l 174 -87 b 260 -174 174 -135 212 -174 l 1128 -174 b 1215 -87 1176 -174 1215 -135 l 1215 174 l 1345 174 b 1389 217 1369 174 1389 193 l 1389 521 b 1215 694 1389 617 1311 694 z m 1042 0 l 347 0 l 347 260 l 1042 260 z m 1042 608 l 347 608 l 347 1042 l 868 1042 l 868 911 b 911 868 868 887 887 868 l 1042 868 z m 1172 412 b 1107 477 1136 412 1107 441 b 1172 543 1107 513 1136 543 b 1237 477 1208 543 1237 513 b 1172 412 1237 441 1208 412 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1389 825 b 1259 955 1389 897 1331 955 l 1020 955 l 986 1044 b 865 1128 967 1095 919 1128 l 524 1128 b 402 1044 470 1128 421 1095 l 369 955 l 130 955 b 0 825 58 955 0 897 l 0 43 b 130 -87 0 -28 58 -87 l 1259 -87 b 1389 43 1331 -87 1389 -28 z m 1020 434 b 694 109 1020 254 874 109 b 369 434 515 109 369 254 b 694 760 369 614 515 760 b 1020 434 874 760 1020 614 z m 933 434 b 694 673 933 566 826 673 b 456 434 563 673 456 566 b 694 195 456 302 563 195 b 933 434 826 195 933 302 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1172 87 l 1108 87 l 754 1070 b 672 1128 743 1102 706 1128 l 672 1128 l 544 1128 l 544 1128 b 461 1070 509 1128 473 1102 l 107 87 l 43 87 b 0 43 19 87 0 67 l 0 -43 b 43 -87 0 -67 19 -87 l 391 -87 b 434 -43 415 -87 434 -67 l 434 43 b 391 87 434 67 415 87 l 338 87 l 401 260 l 815 260 l 878 87 l 825 87 b 781 43 801 87 781 67 l 781 -43 b 825 -87 781 -67 801 -87 l 1172 -87 b 1215 -43 1196 -87 1215 -67 l 1215 43 b 1172 87 1215 67 1196 87 z m 480 477 l 608 829 l 736 477 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 905 570 b 978 747 940 612 972 692 b 633 1128 998 954 836 1128 l 92 1128 b 49 1085 68 1128 49 1109 l 49 955 b 92 911 49 931 68 911 l 179 911 l 179 130 l 92 130 b 49 87 68 130 49 111 l 49 -43 b 92 -87 49 -67 68 -87 l 660 -87 b 1043 245 852 -87 1024 54 b 905 570 1055 377 998 495 z m 395 911 l 633 911 b 763 781 705 911 763 853 b 633 651 763 709 705 651 l 395 651 z m 633 130 l 395 130 l 395 434 l 633 434 b 785 282 717 434 785 366 b 633 130 785 198 717 130 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 868 1085 b 825 1128 868 1109 849 1128 l 304 1128 b 260 1085 280 1128 260 1109 l 260 998 b 304 955 260 974 280 955 l 431 955 l 214 87 l 43 87 b 0 43 19 87 0 67 l 0 -43 b 43 -87 0 -67 19 -87 l 564 -87 b 608 -43 588 -87 608 -67 l 608 43 b 564 87 608 67 588 87 l 437 87 l 654 955 l 825 955 b 868 998 849 955 868 974 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 825 1128 l 43 1128 b 0 1085 19 1128 0 1109 l 0 825 b 43 781 0 801 19 781 l 130 781 b 174 825 154 781 174 801 l 174 911 l 326 911 l 326 87 l 217 87 b 174 43 193 87 174 67 l 174 -43 b 217 -87 174 -67 193 -87 l 651 -87 b 694 -43 675 -87 694 -67 l 694 43 b 651 87 694 67 675 87 l 543 87 l 543 911 l 694 911 l 694 825 b 738 781 694 801 714 781 l 825 781 b 868 825 849 781 868 801 l 868 1085 b 825 1128 868 1109 849 1128 z m 1519 217 l 1389 217 l 1389 825 l 1519 825 b 1550 899 1558 825 1577 872 l 1333 1116 b 1302 1128 1326 1123 1312 1128 b 1271 1116 1292 1128 1278 1123 l 1054 899 b 1085 825 1029 873 1043 825 l 1215 825 l 1215 217 l 1085 217 b 1054 143 1046 217 1027 170 l 1271 -74 b 1302 -87 1278 -81 1292 -87 b 1333 -74 1312 -87 1326 -81 l 1550 143 b 1519 217 1575 168 1562 217 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1172 1128 l 43 1128 b 0 1085 19 1128 0 1109 l 0 868 b 43 825 0 844 19 825 l 130 825 b 174 868 154 825 174 844 l 174 911 l 499 911 l 499 608 l 434 608 b 391 564 410 608 391 588 l 391 477 b 434 434 391 453 410 434 l 781 434 b 825 477 805 434 825 453 l 825 564 b 781 608 825 588 805 608 l 716 608 l 716 911 l 1042 911 l 1042 868 b 1085 825 1042 844 1061 825 l 1172 825 b 1215 868 1196 825 1215 844 l 1215 1085 b 1172 1128 1215 1109 1196 1128 z m 986 421 b 911 391 960 447 911 433 l 911 260 l 304 260 l 304 391 b 230 421 304 429 257 448 l 13 204 b 0 174 6 197 0 184 b 13 143 0 164 6 150 l 230 -74 b 304 -43 255 -99 304 -86 l 304 87 l 911 87 l 911 -43 b 986 -74 911 -82 958 -101 l 1203 143 b 1215 174 1210 150 1215 164 b 1203 204 1215 184 1210 197 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 35 260 l 746 260 l 746 260 b 781 295 766 260 781 276 l 781 295 l 781 399 l 781 399 b 746 434 781 418 766 434 l 746 434 l 35 434 l 35 434 b 0 399 16 434 0 418 l 0 399 l 0 295 l 0 295 b 35 260 0 276 16 260 z m 35 955 l 746 955 l 746 955 b 781 990 766 955 781 970 l 781 990 l 781 1094 l 781 1094 b 746 1128 781 1113 766 1128 l 746 1128 l 35 1128 l 35 1128 b 0 1094 16 1128 0 1113 l 0 1094 l 0 990 l 0 990 b 35 955 0 970 16 955 z m 1172 781 l 43 781 b 0 738 19 781 0 762 l 0 651 b 43 608 0 627 19 608 l 1172 608 b 1215 651 1196 608 1215 627 l 1215 738 b 1172 781 1215 762 1196 781 z m 1172 87 l 43 87 b 0 43 19 87 0 67 l 0 -43 b 43 -87 0 -67 19 -87 l 1172 -87 b 1215 -43 1196 -87 1215 -67 l 1215 43 b 1172 87 1215 67 1196 87 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1172 781 l 43 781 b 0 738 19 781 0 762 l 0 651 b 43 608 0 627 19 608 l 1172 608 b 1215 651 1196 608 1215 627 l 1215 738 b 1172 781 1215 762 1196 781 z m 1172 87 l 43 87 b 0 43 19 87 0 67 l 0 -43 b 43 -87 0 -67 19 -87 l 1172 -87 b 1215 -43 1196 -87 1215 -67 l 1215 43 b 1172 87 1215 67 1196 87 z m 293 955 l 922 955 l 922 955 b 955 988 940 955 955 970 l 955 988 l 955 1096 b 922 1128 955 1114 940 1128 l 293 1128 l 293 1128 b 260 1096 275 1128 260 1114 l 260 988 b 293 955 260 970 275 955 z m 922 260 l 922 260 b 955 293 940 260 955 275 l 955 293 l 955 401 b 922 434 955 419 940 434 l 293 434 l 293 434 b 260 401 275 434 260 419 l 260 293 b 293 260 260 275 275 260 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 43 608 l 1172 608 b 1215 651 1196 608 1215 627 l 1215 738 b 1172 781 1215 762 1196 781 l 43 781 b 0 738 19 781 0 762 l 0 651 b 43 608 0 627 19 608 z m 1172 87 l 43 87 b 0 43 19 87 0 67 l 0 -43 b 43 -87 0 -67 19 -87 l 1172 -87 b 1215 -43 1196 -87 1215 -67 l 1215 43 b 1172 87 1215 67 1196 87 z m 1180 1128 l 469 1128 l 469 1128 b 434 1094 450 1128 434 1113 l 434 1094 l 434 990 l 434 990 b 469 955 434 970 450 955 l 469 955 l 1180 955 l 1181 955 b 1215 990 1200 955 1215 970 l 1215 990 l 1215 1094 l 1215 1094 b 1181 1128 1215 1113 1200 1128 z m 1180 434 l 469 434 l 469 434 b 434 399 450 434 434 418 l 434 399 l 434 295 l 434 295 b 469 260 434 276 450 260 l 469 260 l 1180 260 l 1181 260 b 1215 295 1200 260 1215 276 l 1215 295 l 1215 399 l 1215 399 b 1181 434 1215 418 1200 434 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1172 87 l 43 87 b 0 43 19 87 0 67 l 0 -43 b 43 -87 0 -67 19 -87 l 1172 -87 b 1215 -43 1196 -87 1215 -67 l 1215 43 b 1172 87 1215 67 1196 87 z m 1172 434 l 43 434 b 0 391 19 434 0 415 l 0 304 b 43 260 0 280 19 260 l 1172 260 b 1215 304 1196 260 1215 280 l 1215 391 b 1172 434 1215 415 1196 434 z m 1172 781 l 43 781 b 0 738 19 781 0 762 l 0 651 b 43 608 0 627 19 608 l 1172 608 b 1215 651 1196 608 1215 627 l 1215 738 b 1172 781 1215 762 1196 781 z m 1172 1128 l 43 1128 b 0 1085 19 1128 0 1109 l 0 998 b 43 955 0 974 19 955 l 1172 955 b 1215 998 1196 955 1215 974 l 1215 1085 b 1172 1128 1215 1109 1196 1128 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 217 217 l 43 217 b 0 174 19 217 0 198 l 0 0 b 43 -43 0 -24 19 -43 l 217 -43 b 260 0 241 -43 260 -24 l 260 174 b 217 217 260 198 241 217 z m 217 1085 l 43 1085 b 0 1042 19 1085 0 1066 l 0 868 b 43 825 0 844 19 825 l 217 825 b 260 868 241 825 260 844 l 260 1042 b 217 1085 260 1066 241 1085 z m 217 651 l 43 651 b 0 608 19 651 0 632 l 0 434 b 43 391 0 410 19 391 l 217 391 b 260 434 241 391 260 410 l 260 608 b 217 651 260 632 241 651 z m 1345 174 l 477 174 b 434 130 453 174 434 154 l 434 43 b 477 0 434 19 453 0 l 1345 0 b 1389 43 1369 0 1389 19 l 1389 130 b 1345 174 1389 154 1369 174 z m 1345 1042 l 477 1042 b 434 998 453 1042 434 1022 l 434 911 b 477 868 434 887 453 868 l 1345 868 b 1389 911 1369 868 1389 887 l 1389 998 b 1345 1042 1389 1022 1369 1042 z m 1345 608 l 477 608 b 434 564 453 608 434 588 l 434 477 b 477 434 434 453 453 434 l 1345 434 b 1389 477 1369 434 1389 453 l 1389 564 b 1345 608 1389 588 1369 608 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 273 230 b 347 260 300 203 347 222 l 347 781 b 273 812 347 820 300 839 l 13 552 b 0 521 6 544 0 531 b 13 490 0 511 6 497 z m 1172 87 l 43 87 b 0 43 19 87 0 67 l 0 -43 b 43 -87 0 -67 19 -87 l 1172 -87 b 1215 -43 1196 -87 1215 -67 l 1215 43 b 1172 87 1215 67 1196 87 z m 1180 434 l 556 434 l 556 434 b 521 399 536 434 521 418 l 521 399 l 521 295 l 521 295 b 556 260 521 276 536 260 l 556 260 l 1180 260 l 1181 260 b 1215 295 1200 260 1215 276 l 1215 295 l 1215 399 l 1215 399 b 1181 434 1215 418 1200 434 z m 1180 781 l 556 781 l 556 781 b 521 746 536 781 521 766 l 521 746 l 521 642 l 521 642 b 556 608 521 623 536 608 l 556 608 l 1180 608 l 1181 608 b 1215 642 1200 608 1215 623 l 1215 642 l 1215 746 l 1215 746 b 1181 781 1215 766 1200 781 z m 1172 1128 l 43 1128 b 0 1085 19 1128 0 1109 l 0 998 b 43 955 0 974 19 955 l 1172 955 b 1215 998 1196 955 1215 974 l 1215 1085 b 1172 1128 1215 1109 1196 1128 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 74 230 l 334 490 b 347 521 342 497 347 511 b 334 552 347 531 342 545 l 74 812 b 0 781 47 839 0 820 l 0 260 b 74 230 0 222 47 203 z m 1172 87 l 43 87 b 0 43 19 87 0 67 l 0 -43 b 43 -87 0 -67 19 -87 l 1172 -87 b 1215 -43 1196 -87 1215 -67 l 1215 43 b 1172 87 1215 67 1196 87 z m 1180 434 l 556 434 l 556 434 b 521 399 536 434 521 418 l 521 399 l 521 295 l 521 295 b 556 260 521 276 536 260 l 556 260 l 1180 260 l 1181 260 b 1215 295 1200 260 1215 276 l 1215 295 l 1215 399 l 1215 399 b 1181 434 1215 418 1200 434 z m 1180 781 l 556 781 l 556 781 b 521 746 536 781 521 766 l 521 746 l 521 642 l 521 642 b 556 608 521 623 536 608 l 556 608 l 1180 608 l 1181 608 b 1215 642 1200 608 1215 623 l 1215 642 l 1215 746 l 1215 746 b 1181 781 1215 766 1200 781 z m 1172 1128 l 43 1128 b 0 1085 19 1128 0 1109 l 0 998 b 43 955 0 974 19 955 l 1172 955 b 1215 998 1196 955 1215 974 l 1215 1085 b 1172 1128 1215 1109 1196 1128 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 912 1042 l 130 1042 b 0 912 58 1042 0 984 l 0 130 b 130 0 0 58 58 0 l 912 0 b 1042 130 984 0 1042 58 l 1042 912 b 912 1042 1042 984 984 1042 z m 1426 939 l 1128 734 l 1128 307 l 1426 103 b 1563 173 1483 63 1563 103 l 1563 869 b 1426 939 1563 938 1484 979 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1259 0 b 1389 130 1331 0 1389 58 l 1389 911 b 1259 1042 1389 983 1331 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 130 b 130 0 0 58 58 0 z m 304 890 b 456 738 388 890 456 822 b 304 586 456 654 388 586 b 152 738 220 586 152 654 b 304 890 152 822 220 890 z m 174 174 l 174 304 l 368 498 b 414 498 380 511 401 511 l 564 347 l 932 715 b 978 715 945 728 965 728 l 1215 477 l 1215 174 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 467 -146 b 574 -146 493 -183 548 -183 b 1042 694 969 426 1042 484 b 521 1215 1042 982 808 1215 b 0 694 233 1215 0 982 b 467 -146 0 484 73 426 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 22 521 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 b 22 521 323 1194 22 892 z m 694 22 l 694 1020 b 1194 521 970 1020 1194 797 b 694 22 1194 245 970 22 z "
	},
	"": {
		ha: 955,
		x_min: 0,
		x_max: 0,
		o: "m 557 1155 b 398 1155 535 1233 423 1237 b 0 309 271 727 0 611 b 477 -174 0 42 214 -174 b 955 309 741 -174 955 42 b 557 1155 955 613 684 725 z m 477 0 b 174 304 310 0 174 136 b 217 347 174 328 193 347 b 260 304 241 347 260 328 b 477 87 260 184 358 87 b 521 43 501 87 521 67 b 477 0 521 19 501 0 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1092 990 b 1055 990 1082 1000 1065 1000 l 462 397 l 434 145 b 493 87 431 112 459 83 l 744 115 l 1337 707 b 1337 745 1347 718 1347 735 z m 1532 1052 l 1399 1184 b 1249 1184 1358 1225 1291 1225 l 1153 1088 b 1153 1051 1143 1078 1143 1061 l 1398 806 b 1436 806 1408 796 1425 796 l 1532 902 b 1532 1052 1573 943 1573 1010 z m 1042 276 l 1042 0 l 174 0 l 174 868 l 797 868 b 820 878 806 868 814 872 l 929 986 b 905 1042 949 1007 935 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 385 b 1160 408 1215 414 1180 428 l 1051 299 b 1042 276 1045 293 1042 285 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 174 -54 b 206 -87 174 -72 188 -87 l 336 -87 b 369 -54 354 -87 369 -72 l 369 421 l 899 -67 b 1042 0 955 -113 1042 -74 l 1042 1042 b 899 1108 1042 1116 955 1155 l 369 617 l 369 1096 b 336 1128 369 1114 354 1128 l 206 1128 b 174 1096 188 1128 174 1114 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 0 33 b 33 0 0 15 15 0 l 141 0 b 174 33 159 0 174 15 l 174 441 l 639 20 b 781 87 695 -26 781 12 l 781 441 l 1246 20 b 1389 87 1302 -26 1389 12 l 1389 955 b 1246 1022 1389 1029 1302 1068 l 781 597 l 781 955 b 639 1022 781 1029 695 1068 l 174 597 l 174 1009 b 141 1042 174 1027 159 1042 l 33 1042 b 0 1009 15 1042 0 1027 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 31 454 l 552 20 b 694 87 608 -27 694 12 l 694 955 b 552 1022 694 1029 608 1068 l 31 588 b 31 454 -10 553 -10 489 z m 726 454 l 1246 20 b 1389 87 1302 -27 1389 12 l 1389 955 b 1246 1022 1389 1029 1302 1068 l 726 588 b 726 454 684 553 684 489 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1151 633 l 196 1197 b 0 1085 119 1243 0 1199 l 0 -43 b 196 -155 0 -145 110 -206 l 1151 409 b 1151 633 1236 459 1237 583 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 391 -84 b 521 46 463 -84 521 -26 l 521 1001 b 391 1131 521 1073 463 1131 l 130 1131 b 0 1001 58 1131 0 1073 l 0 46 b 130 -84 0 -26 58 -84 z m 1215 46 l 1215 1001 b 1085 1131 1215 1073 1157 1131 l 825 1131 b 694 1001 753 1131 694 1073 l 694 46 b 825 -84 694 -26 753 -84 l 1085 -84 b 1215 46 1157 -84 1215 -26 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1085 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -28 58 -87 l 1085 -87 b 1215 43 1157 -87 1215 -28 l 1215 998 b 1085 1128 1215 1070 1157 1128 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1358 588 l 837 1022 b 694 955 781 1068 694 1029 l 694 87 b 837 20 694 12 781 -27 l 1358 454 b 1358 588 1399 489 1399 553 z m 663 588 l 142 1022 b 0 955 87 1068 0 1029 l 0 87 b 142 20 0 12 87 -27 l 663 454 b 663 588 705 489 705 553 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1389 1009 b 1356 1042 1389 1027 1374 1042 l 1248 1042 b 1215 1009 1230 1042 1215 1027 l 1215 600 l 750 1022 b 608 955 694 1068 608 1029 l 608 600 l 142 1022 b 0 955 87 1068 0 1029 l 0 87 b 142 20 0 12 87 -27 l 608 445 l 608 87 b 750 20 608 12 694 -27 l 1215 445 l 1215 33 b 1248 0 1215 15 1230 0 l 1356 0 b 1389 33 1374 0 1389 15 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1042 1096 b 1009 1128 1042 1114 1027 1128 l 879 1128 b 846 1096 861 1128 846 1114 l 846 620 l 316 1108 b 174 1042 260 1155 174 1116 l 174 0 b 316 -67 174 -74 260 -113 l 846 424 l 846 -54 b 879 -87 846 -72 861 -87 l 1009 -87 b 1042 -54 1027 -87 1042 -72 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1215 174 b 1128 260 1215 222 1176 260 l 87 260 b 0 174 39 260 0 222 l 0 0 b 87 -87 0 -48 39 -87 l 1128 -87 b 1215 0 1176 -87 1215 -48 z m 130 347 l 1085 347 b 1181 565 1198 347 1257 482 l 704 1086 b 512 1086 652 1143 563 1143 l 34 565 b 130 347 -42 482 17 347 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 94 567 b 94 475 68 541 68 500 l 621 -52 b 713 -52 646 -78 687 -78 l 774 9 b 774 101 800 34 800 76 l 357 521 l 775 941 b 774 1033 800 966 800 1007 l 713 1094 b 621 1094 688 1119 646 1119 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 774 475 b 774 567 800 500 800 541 l 247 1094 b 155 1094 222 1119 181 1119 l 94 1033 b 94 941 68 1007 68 966 l 511 521 l 94 101 b 94 9 68 76 68 34 l 155 -52 b 247 -52 181 -78 222 -78 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 z m 1085 445 b 1053 412 1085 427 1070 412 l 803 412 l 803 163 b 770 130 803 145 788 130 l 618 130 b 586 163 601 130 586 145 l 586 412 l 336 412 b 304 445 318 412 304 427 l 304 597 b 336 629 304 615 318 629 l 586 629 l 586 879 b 618 911 586 897 601 911 l 770 911 b 803 879 788 911 803 897 l 803 629 l 1053 629 b 1085 597 1070 629 1085 615 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 z m 336 412 b 304 445 318 412 304 427 l 304 597 b 336 629 304 615 318 629 l 1053 629 b 1085 597 1070 629 1085 615 l 1085 445 b 1053 412 1085 427 1070 412 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 z m 1024 344 b 1024 298 1037 331 1037 311 l 917 191 b 871 191 904 178 884 178 l 694 369 l 518 191 b 472 191 505 178 484 178 l 365 298 b 365 345 352 311 352 332 l 543 521 l 365 697 b 365 744 352 710 352 731 l 472 851 b 518 851 485 864 505 864 l 694 673 l 871 851 b 917 851 884 863 904 863 l 1025 743 b 1025 697 1037 731 1037 710 l 846 521 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1367 521 b 694 1194 1367 892 1066 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 z m 617 165 b 555 165 600 148 572 148 l 273 447 b 273 508 256 464 256 491 l 335 570 b 396 570 351 586 379 586 l 586 379 l 993 787 b 1054 787 1010 803 1037 803 l 1116 725 b 1116 664 1133 708 1133 681 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1367 521 b 694 1194 1367 892 1066 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 z m 712 971 b 1029 694 865 971 1029 852 b 803 405 1029 486 803 482 l 803 401 b 770 369 803 383 788 369 l 618 369 b 586 401 601 369 586 383 l 586 412 b 734 604 586 524 670 568 b 822 696 789 635 822 656 b 698 786 822 750 754 786 b 543 688 625 786 591 750 b 498 683 532 675 512 672 l 404 754 b 396 798 390 765 387 784 b 712 971 470 909 565 971 z m 694 298 b 819 174 763 298 819 242 b 694 49 819 105 763 49 b 570 174 626 49 570 105 b 694 298 570 242 626 298 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 z m 694 895 b 808 781 757 895 808 844 b 694 667 808 718 757 667 b 581 781 632 667 581 718 b 694 895 581 844 632 895 z m 846 206 b 814 174 846 188 832 174 l 575 174 b 543 206 557 174 543 188 l 543 271 b 575 304 543 289 557 304 l 608 304 l 608 477 l 575 477 b 543 510 557 477 543 492 l 543 575 b 575 608 543 593 557 608 l 749 608 b 781 575 767 608 781 593 l 781 304 l 814 304 b 846 271 832 304 846 289 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1356 608 l 1274 608 b 781 1100 1236 862 1035 1063 l 781 1183 b 749 1215 781 1201 767 1215 l 640 1215 b 608 1183 622 1215 608 1201 l 608 1100 b 115 608 354 1063 153 862 l 33 608 b 0 575 15 608 0 593 l 0 467 b 33 434 0 449 15 434 l 115 434 b 608 -59 153 180 354 -21 l 608 -141 b 640 -174 608 -159 622 -174 l 749 -174 b 781 -141 767 -174 781 -159 l 781 -59 b 1274 434 1035 -21 1236 180 l 1356 434 b 1389 467 1374 434 1389 449 l 1389 575 b 1356 608 1389 593 1374 608 z m 781 118 l 781 228 b 749 260 781 246 767 260 l 640 260 b 608 228 622 260 608 246 l 608 118 b 291 434 450 151 325 276 l 401 434 b 434 467 419 434 434 449 l 434 575 b 401 608 434 593 419 608 l 291 608 b 608 924 325 765 450 890 l 608 814 b 640 781 608 796 622 781 l 749 781 b 781 814 767 781 781 796 l 781 924 b 1098 608 939 890 1064 766 l 987 608 b 955 575 969 608 955 593 l 955 467 b 987 434 955 449 969 434 l 1098 434 b 781 118 1064 276 939 151 z m 781 521 b 694 608 781 569 742 608 b 608 521 647 608 608 569 b 694 434 608 473 647 434 b 781 521 742 434 781 473 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 z m 1047 874 b 1103 235 1225 696 1237 425 l 408 930 b 1047 874 599 1064 870 1051 z m 342 168 b 285 807 164 345 152 617 l 981 112 b 342 168 790 -22 519 -10 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 699 8 b 697 101 725 34 724 76 l 371 412 l 1150 412 b 1215 477 1186 412 1215 441 l 1215 564 b 1150 629 1215 600 1186 629 l 371 629 l 697 941 b 699 1034 724 966 724 1008 l 638 1094 b 546 1094 613 1120 572 1120 l 19 567 b 19 475 -7 541 -7 500 l 546 -52 b 638 -52 572 -78 613 -78 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 517 1034 b 518 941 491 1008 491 966 l 845 629 l 65 629 b 0 564 29 629 0 600 l 0 477 b 65 412 0 441 29 412 l 845 412 l 518 101 b 517 8 492 76 491 34 l 577 -52 b 669 -52 602 -78 643 -78 l 1196 475 b 1196 567 1222 500 1222 541 l 669 1094 b 577 1094 644 1120 602 1120 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 95 430 b 188 431 120 404 162 404 l 499 758 l 499 -22 b 564 -87 499 -58 528 -87 l 651 -87 b 716 -22 687 -87 716 -58 l 716 758 l 1027 431 b 1120 430 1053 405 1095 404 l 1181 490 b 1181 582 1206 516 1206 557 l 653 1109 b 562 1109 628 1135 587 1135 l 34 582 b 34 490 9 557 9 516 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1121 612 b 1028 611 1095 638 1053 637 l 716 284 l 716 1063 b 651 1128 716 1099 687 1128 l 564 1128 b 499 1063 528 1128 499 1099 l 499 284 l 188 611 b 95 612 162 637 120 637 l 34 551 b 34 460 9 526 9 485 l 562 -68 b 654 -68 587 -93 628 -93 l 1181 460 b 1181 551 1206 485 1206 526 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1366 700 l 889 1113 b 781 1063 847 1149 781 1119 l 781 846 b 0 341 346 841 0 754 b 226 -77 0 174 107 9 b 302 -27 263 -104 316 -70 b 781 477 179 367 361 471 l 781 239 b 889 189 781 183 847 153 l 1366 602 b 1366 700 1396 628 1396 674 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 0 727 b 33 694 0 709 15 694 l 141 694 b 174 727 159 694 174 709 l 174 955 l 401 955 b 434 987 419 955 434 970 l 434 1096 b 401 1128 434 1114 419 1128 l 65 1128 b 0 1063 29 1128 0 1099 z m 781 1096 l 781 987 b 814 955 781 970 796 955 l 1042 955 l 1042 727 b 1074 694 1042 709 1056 694 l 1183 694 b 1215 727 1201 694 1215 709 l 1215 1063 b 1150 1128 1215 1099 1186 1128 l 814 1128 b 781 1096 796 1128 781 1114 z m 1183 347 l 1074 347 b 1042 315 1056 347 1042 333 l 1042 87 l 814 87 b 781 54 796 87 781 72 l 781 -54 b 814 -87 781 -72 796 -87 l 1150 -87 b 1215 -22 1186 -87 1215 -58 l 1215 315 b 1183 347 1215 333 1201 347 z m 434 -54 l 434 54 b 401 87 434 72 419 87 l 174 87 l 174 315 b 141 347 174 333 159 347 l 33 347 b 0 315 15 347 0 333 l 0 -22 b 65 -87 0 -58 29 -87 l 401 -87 b 434 -54 419 -87 434 -72 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1183 694 b 1215 727 1201 694 1215 709 l 1215 836 b 1183 868 1215 853 1201 868 l 955 868 l 955 1096 b 922 1128 955 1114 940 1128 l 814 1128 b 781 1096 796 1128 781 1114 l 781 760 b 846 694 781 723 810 694 z m 434 760 l 434 1096 b 401 1128 434 1114 419 1128 l 293 1128 b 260 1096 275 1128 260 1114 l 260 868 l 33 868 b 0 836 15 868 0 853 l 0 727 b 33 694 0 709 15 694 l 369 694 b 434 760 405 694 434 723 z m 434 -54 l 434 282 b 369 347 434 318 405 347 l 33 347 b 0 315 15 347 0 333 l 0 206 b 33 174 0 188 15 174 l 260 174 l 260 -54 b 293 -87 260 -72 275 -87 l 401 -87 b 434 -54 419 -87 434 -72 z m 955 -54 l 955 174 l 1183 174 b 1215 206 1201 174 1215 188 l 1215 315 b 1183 347 1215 333 1201 347 l 846 347 b 781 282 810 347 781 318 l 781 -54 b 814 -87 781 -72 796 -87 l 922 -87 b 955 -54 940 -87 955 -72 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1128 651 l 738 651 l 738 1042 b 651 1128 738 1090 699 1128 l 564 1128 b 477 1042 516 1128 477 1090 l 477 651 l 87 651 b 0 564 39 651 0 612 l 0 477 b 87 391 0 429 39 391 l 477 391 l 477 0 b 564 -87 477 -48 516 -87 l 651 -87 b 738 0 699 -87 738 -48 l 738 391 l 1128 391 b 1215 477 1176 391 1215 429 l 1215 564 b 1128 651 1215 612 1176 651 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1128 651 l 87 651 b 0 564 39 651 0 612 l 0 477 b 87 391 0 429 39 391 l 1128 391 b 1215 477 1176 391 1215 429 l 1215 564 b 1128 651 1215 612 1176 651 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1297 309 l 911 521 l 1297 733 b 1322 822 1329 750 1341 791 l 1269 914 b 1179 937 1251 945 1211 956 l 803 709 l 812 1149 b 747 1215 813 1185 784 1215 l 642 1215 b 577 1149 605 1215 576 1185 l 586 709 l 210 937 b 119 914 178 956 138 945 l 67 822 b 92 733 48 791 60 750 l 477 521 l 92 309 b 67 219 60 291 48 251 l 119 128 b 210 105 138 96 178 86 l 586 333 l 577 -107 b 642 -174 576 -144 605 -174 l 747 -174 b 812 -107 784 -174 813 -144 l 803 333 l 1179 105 b 1269 128 1211 86 1251 96 l 1322 219 b 1297 309 1341 251 1329 291 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1367 521 b 694 1194 1367 892 1066 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 z m 694 385 b 819 260 763 385 819 329 b 694 136 819 192 763 136 b 570 260 626 136 570 192 b 694 385 570 329 626 385 z m 576 834 b 608 868 575 852 590 868 l 780 868 b 813 834 799 868 814 852 l 793 465 b 760 434 792 448 778 434 l 629 434 b 596 465 611 434 597 448 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 87 0 b 174 -87 87 -48 126 -87 l 608 -87 l 608 347 l 87 347 z m 781 -87 l 1215 -87 b 1302 0 1263 -87 1302 -48 l 1302 347 l 781 347 z m 1302 781 l 1188 781 b 1215 890 1205 814 1215 850 b 977 1128 1215 1021 1108 1128 b 697 943 864 1128 791 1071 b 418 1128 604 1071 531 1128 b 179 890 286 1128 179 1021 b 206 781 179 850 189 814 l 87 781 b 0 694 39 781 0 742 l 0 477 b 43 434 0 454 20 434 l 1345 434 b 1389 477 1369 434 1389 454 l 1389 694 b 1302 781 1389 742 1350 781 z m 417 781 b 309 890 358 781 309 830 b 417 998 309 950 358 998 b 651 781 471 998 511 989 z m 977 781 l 743 781 b 977 998 882 989 921 998 b 1085 890 1037 998 1085 950 b 977 781 1085 830 1037 781 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1482 1189 b 1405 1192 1466 1223 1423 1224 b 998 955 1321 1046 1170 955 l 781 955 b 260 434 494 955 260 722 b 264 378 260 415 263 397 b 1042 608 438 502 687 608 b 1085 564 1066 608 1085 588 b 1042 521 1085 540 1066 521 b 7 -54 360 521 71 103 b 54 -167 -11 -98 10 -149 b 167 -120 98 -186 149 -164 b 362 126 171 -110 224 10 b 837 -84 450 7 617 -107 b 1563 797 1263 -53 1563 329 b 1482 1189 1563 933 1533 1074 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 586 1151 b 466 1186 586 1215 503 1239 b 608 434 130 695 608 673 b 432 260 608 337 529 259 b 260 437 336 262 260 341 l 260 669 b 148 713 260 727 189 756 b 0 347 75 637 0 506 b 521 -174 0 60 234 -174 b 1042 347 808 -174 1042 60 b 586 1151 1042 809 586 871 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1553 560 b 781 1042 1406 847 1115 1042 b 9 560 448 1042 156 847 b 0 521 4 550 0 532 b 9 481 0 509 4 492 b 781 0 157 194 448 0 b 1553 481 1115 0 1406 194 b 1562 521 1558 492 1562 509 b 1553 560 1562 532 1558 550 z m 781 130 b 391 521 566 130 391 305 b 781 911 391 736 566 911 b 1172 521 997 911 1172 736 l 1172 521 b 781 130 1172 305 997 130 z m 781 781 b 713 771 762 781 731 776 b 738 694 727 752 738 718 b 608 564 738 622 680 564 b 531 589 585 564 550 576 b 522 521 526 571 522 541 b 782 262 522 378 638 262 b 1042 521 925 262 1042 378 b 782 781 1042 665 925 781 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 868 130 b 480 491 662 130 496 289 l 196 711 b 96 560 158 664 124 615 b 87 521 91 550 87 532 b 96 481 87 509 91 492 b 868 0 243 194 535 0 b 1079 28 941 0 1011 11 l 939 137 b 868 130 919 134 888 130 z m 1719 -27 l 1419 204 b 1640 481 1496 269 1594 393 b 1649 521 1645 492 1649 509 b 1640 560 1649 532 1645 550 b 868 1042 1493 847 1202 1042 b 468 939 751 1042 572 996 l 123 1206 b 97 1215 117 1211 105 1215 b 62 1199 85 1215 70 1208 l 9 1130 b 0 1103 4 1123 0 1112 b 17 1069 0 1092 8 1076 l 1613 -164 b 1639 -174 1619 -170 1631 -174 b 1674 -157 1651 -174 1667 -166 l 1727 -88 b 1736 -62 1732 -82 1736 -70 b 1719 -27 1736 -50 1729 -35 z m 1221 358 l 1114 440 b 1128 521 1122 462 1128 498 b 1128 524 1128 522 1128 523 b 871 781 1128 666 1013 781 b 799 771 851 781 819 777 b 825 694 813 752 825 718 b 820 667 824 687 822 675 l 621 822 b 868 911 680 871 791 911 l 868 911 b 1259 521 1084 911 1259 737 l 1259 521 b 1221 358 1259 462 1244 407 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1545 22 l 894 1150 b 668 1150 844 1237 719 1237 l 18 22 b 130 -174 -32 -65 30 -174 l 1432 -174 b 1545 22 1532 -174 1595 -65 z m 781 255 b 906 130 850 255 906 199 b 781 5 906 61 850 5 b 656 130 712 5 656 61 b 781 255 656 199 712 255 z m 663 704 b 695 738 662 722 677 738 l 867 738 b 900 704 886 738 901 722 l 880 335 b 847 304 879 317 864 304 l 715 304 b 683 335 698 304 684 317 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1302 694 l 992 694 l 707 1193 b 669 1215 700 1205 683 1215 l 669 1215 l 492 1215 b 450 1160 463 1215 442 1188 l 583 694 l 304 694 l 187 851 b 152 868 178 862 166 868 l 43 868 b 1 814 15 868 -6 842 l 87 521 l 1 228 b 43 174 -6 200 15 174 l 152 174 b 187 191 166 174 178 180 l 304 347 l 583 347 l 450 -118 b 492 -174 442 -146 463 -174 l 669 -174 b 707 -152 685 -174 699 -165 l 992 347 l 1302 347 b 1563 521 1398 347 1563 425 b 1302 694 1563 617 1398 694 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 694 l 0 694 z m 868 488 b 901 521 868 506 883 521 l 1009 521 b 1042 488 1027 521 1042 506 l 1042 380 b 1009 347 1042 362 1027 347 l 901 347 b 868 380 883 347 868 362 z m 868 141 b 901 174 868 159 883 174 l 1009 174 b 1042 141 1027 174 1042 159 l 1042 33 b 1009 0 1042 15 1027 0 l 901 0 b 868 33 883 0 868 15 z m 521 488 b 553 521 521 506 535 521 l 662 521 b 694 488 680 521 694 506 l 694 380 b 662 347 694 362 680 347 l 553 347 b 521 380 535 347 521 362 z m 521 141 b 553 174 521 159 535 174 l 662 174 b 694 141 680 174 694 159 l 694 33 b 662 0 694 15 680 0 l 553 0 b 521 33 535 0 521 15 z m 174 488 b 206 521 174 506 188 521 l 315 521 b 347 488 333 521 347 506 l 347 380 b 315 347 347 362 333 347 l 206 347 b 174 380 188 347 174 362 z m 174 141 b 206 174 174 159 188 174 l 315 174 b 347 141 333 174 347 159 l 347 33 b 315 0 347 15 333 0 l 206 0 b 174 33 188 0 174 15 z m 1085 1042 l 955 1042 l 955 1172 b 911 1215 955 1196 935 1215 l 825 1215 b 781 1172 801 1215 781 1196 l 781 1042 l 434 1042 l 434 1172 b 391 1215 434 1196 414 1215 l 304 1215 b 260 1172 280 1215 260 1196 l 260 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 781 l 1215 781 l 1215 911 b 1085 1042 1215 983 1157 1042 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1370 241 l 1153 458 b 1042 412 1112 499 1042 470 l 1042 304 l 955 304 l 812 457 l 667 302 l 858 97 b 882 87 864 91 874 87 l 1042 87 l 1042 -22 b 1153 -68 1042 -80 1112 -108 l 1370 149 b 1370 241 1395 175 1395 216 z m 33 738 l 260 738 l 404 584 l 548 739 l 357 945 b 333 955 352 950 341 955 l 333 955 l 33 955 b 0 922 15 955 0 940 l 0 770 b 33 738 0 752 15 738 z m 1042 738 l 1042 629 b 1153 583 1042 571 1112 543 l 1370 800 b 1370 892 1395 826 1395 867 l 1153 1109 b 1042 1063 1112 1150 1042 1121 l 1042 955 l 882 955 b 858 945 874 955 864 950 l 260 304 l 33 304 b 0 271 15 304 0 289 l 0 119 b 33 87 0 101 15 87 l 333 87 b 357 97 342 87 351 91 l 955 738 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1128 b 0 564 311 1128 0 876 b 155 210 0 430 58 307 b 6 -50 121 73 7 -49 b 2 -74 0 -56 -2 -66 b 22 -87 5 -82 13 -87 b 403 53 202 -87 336 -1 b 694 0 492 19 590 0 b 1389 564 1078 0 1389 253 b 694 1128 1389 876 1078 1128 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 445 814 l 445 814 b 477 846 463 814 477 828 b 477 846 477 846 477 846 l 477 1063 b 380 1161 477 1117 434 1161 l 98 1161 b 0 1063 44 1161 0 1117 l 0 846 b 33 814 0 828 15 814 z m 1389 846 l 1389 1063 b 1291 1161 1389 1117 1345 1161 l 1009 1161 b 911 1063 955 1161 911 1117 l 911 846 b 944 814 911 828 926 814 l 1356 814 b 1357 814 1356 814 1357 814 b 1389 846 1374 814 1389 828 z m 944 727 b 911 694 926 727 911 712 l 911 553 b 477 553 911 206 477 206 l 477 694 b 445 727 477 712 463 727 l 33 727 l 33 727 b 0 694 15 727 0 712 l 0 694 b 0 549 1 636 2 585 b 697 -120 0 141 370 -120 b 1388 550 1024 -120 1388 141 b 1388 694 1387 584 1388 639 b 1388 694 1388 694 1388 694 b 1356 727 1388 712 1374 727 l 1356 727 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 654 861 b 562 861 628 887 587 887 l 34 334 b 34 242 9 309 9 267 l 96 180 b 188 180 121 155 162 155 l 608 598 l 1027 180 b 1119 180 1053 155 1094 155 l 1181 242 b 1181 334 1206 267 1206 309 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 562 180 b 654 180 587 155 628 155 l 1181 708 b 1181 800 1206 733 1206 774 l 1119 861 b 1027 861 1094 887 1053 887 l 608 444 l 188 861 b 96 861 162 887 121 887 l 34 800 b 34 708 9 774 9 733 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1708 283 b 1708 375 1733 309 1733 350 l 1679 405 b 1585 403 1653 431 1611 430 l 1476 287 l 1476 890 b 1411 955 1476 926 1447 955 l 750 955 b 704 844 692 955 663 885 l 747 800 b 793 781 758 790 778 781 l 1302 781 l 1302 287 l 1192 403 b 1099 405 1167 430 1125 431 l 1070 375 b 1070 283 1044 350 1044 309 l 1343 10 b 1435 10 1368 -15 1409 -15 z m 989 241 b 943 260 978 252 958 260 l 943 260 l 434 260 l 434 754 l 544 638 b 637 637 569 612 611 611 l 666 666 b 666 758 692 692 692 733 l 393 1032 b 301 1032 368 1057 327 1057 l 28 758 b 28 666 3 733 3 692 l 57 637 b 151 638 83 611 126 612 l 260 754 l 260 152 b 326 87 260 116 290 87 l 986 87 b 1032 198 1044 87 1073 157 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1433 398 l 1561 962 b 1497 1042 1570 1003 1539 1042 l 432 1042 l 407 1163 b 343 1215 401 1194 374 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 1107 b 65 1042 0 1071 29 1042 l 255 1042 l 445 110 b 369 -22 400 84 369 35 b 521 -174 369 -106 437 -174 b 673 -22 605 -174 673 -106 b 627 87 673 21 655 59 l 1196 87 b 1150 -22 1168 59 1150 21 b 1302 -174 1150 -106 1218 -174 b 1454 -22 1386 -174 1454 -106 b 1368 115 1454 38 1419 90 l 1383 181 b 1320 260 1393 222 1362 260 l 592 260 l 574 347 l 1369 347 b 1433 398 1400 347 1426 368 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1259 868 l 738 868 l 564 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 130 b 130 0 0 58 58 0 l 1259 0 b 1389 130 1331 0 1389 58 l 1389 738 b 1259 868 1389 810 1331 868 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1554 423 b 1497 521 1579 466 1548 521 l 412 521 l 412 521 b 262 435 357 521 290 482 l 66 98 b 122 0 41 55 72 0 l 1207 0 l 1207 0 b 1357 86 1262 0 1329 39 z m 412 608 l 1302 608 l 1302 738 b 1172 868 1302 810 1244 868 l 738 868 l 564 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 157 l 187 478 b 412 608 234 558 320 608 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 903 347 l 1007 347 b 1042 382 1024 347 1042 365 l 1042 747 b 1007 781 1042 764 1024 781 l 903 781 b 868 747 885 781 868 764 l 868 382 b 903 347 868 365 885 347 z m 1163 347 l 1267 347 b 1302 382 1285 347 1302 365 l 1302 1007 b 1267 1042 1302 1024 1285 1042 l 1163 1042 b 1128 1007 1146 1042 1128 1024 l 1128 382 b 1163 347 1128 365 1146 347 z m 382 347 l 486 347 b 521 382 503 347 521 365 l 521 573 b 486 608 521 590 503 608 l 382 608 b 347 573 365 608 347 590 l 347 382 b 382 347 347 365 365 347 z m 642 347 l 747 347 b 781 382 764 347 781 365 l 781 920 b 747 955 781 937 764 955 l 642 955 b 608 920 625 955 608 937 l 608 382 b 642 347 608 365 625 347 z m 1345 174 l 174 174 l 174 998 b 130 1042 174 1022 154 1042 l 43 1042 b 0 998 19 1042 0 1022 l 0 87 b 87 0 0 39 39 0 l 1345 0 b 1389 43 1369 0 1389 19 l 1389 130 b 1345 174 1389 154 1369 174 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -28 58 -87 l 1259 -87 b 1389 43 1331 -87 1389 -28 l 1389 998 b 1259 1128 1389 1070 1331 1128 z m 130 1042 l 418 1042 b 434 1025 427 1042 434 1034 l 434 971 b 418 955 434 962 427 955 l 103 955 b 87 971 94 955 87 962 l 87 998 b 130 1042 87 1022 106 1042 z m 1286 781 l 103 781 b 87 798 94 781 87 789 l 87 895 b 103 911 87 904 94 911 l 477 911 l 559 1034 b 573 1042 562 1039 567 1042 l 1259 1042 b 1302 998 1283 1042 1302 1022 l 1302 798 b 1286 781 1302 789 1295 781 z m 694 65 b 369 391 515 65 369 211 b 694 716 369 570 515 716 b 1020 391 874 716 1020 570 b 694 65 1020 211 874 65 z m 694 629 b 456 391 563 629 456 522 b 694 152 456 259 563 152 b 933 391 826 152 933 259 b 694 629 933 522 826 629 z m 564 347 b 521 391 540 347 521 367 b 694 564 521 486 599 564 b 738 521 718 564 738 545 b 694 477 738 497 718 477 b 608 391 647 477 608 438 b 564 347 608 367 588 347 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1389 738 b 911 1215 1389 1002 1176 1215 b 434 738 648 1215 434 1002 b 458 588 434 686 442 635 l 19 149 b 0 103 7 137 0 121 l 0 -109 b 65 -174 0 -144 29 -174 l 369 -174 b 434 -109 405 -174 434 -144 l 434 0 l 543 0 b 608 65 578 0 608 29 l 608 174 l 709 174 l 709 174 b 757 195 725 174 747 183 l 822 269 b 911 260 851 263 881 260 b 1389 738 1175 260 1389 474 z m 911 868 b 1042 998 911 940 970 998 b 1172 868 1114 998 1172 940 b 1042 738 1172 796 1114 738 b 911 868 970 738 911 796 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1389 697 b 1475 697 1418 692 1447 692 l 1497 658 b 1538 644 1506 644 1523 638 b 1625 694 1570 656 1600 673 b 1633 737 1638 704 1641 723 l 1611 776 b 1654 850 1630 797 1644 822 l 1699 850 b 1732 878 1715 850 1729 862 b 1732 979 1737 912 1737 946 b 1699 1007 1729 995 1715 1007 l 1654 1007 b 1611 1081 1644 1034 1630 1059 l 1633 1120 b 1625 1162 1641 1134 1638 1152 b 1538 1213 1600 1184 1570 1201 b 1497 1198 1523 1219 1506 1213 l 1475 1159 b 1389 1159 1447 1165 1418 1165 l 1367 1198 b 1326 1213 1359 1213 1341 1219 b 1239 1162 1294 1201 1265 1184 b 1231 1120 1226 1152 1223 1134 l 1254 1081 b 1210 1007 1235 1059 1220 1034 l 1166 1007 b 1133 978 1149 1007 1135 995 b 1133 878 1127 945 1127 910 b 1166 850 1135 862 1149 850 l 1210 850 b 1254 776 1220 822 1235 797 l 1231 737 b 1239 694 1223 723 1226 704 b 1326 644 1265 673 1294 656 b 1367 658 1341 638 1359 644 z m 1361 857 b 1504 1000 1280 961 1399 1080 b 1361 857 1584 895 1465 776 z m 1048 439 b 1048 611 1058 496 1058 554 l 1139 657 b 1166 736 1167 673 1177 706 b 1052 914 1142 802 1096 862 b 970 929 1032 939 997 945 l 891 883 b 742 969 848 920 797 950 l 742 1060 b 688 1124 742 1092 719 1119 b 482 1124 619 1136 549 1136 b 429 1060 451 1119 429 1092 l 429 969 b 280 883 374 950 323 920 l 201 929 b 119 914 173 944 139 938 b 4 736 75 862 28 801 b 33 657 -7 706 5 673 l 123 611 b 123 439 112 554 112 496 l 33 394 b 4 315 5 378 -7 344 b 119 136 28 249 75 189 b 201 122 139 112 174 106 l 280 167 b 429 81 323 130 374 101 l 429 -10 b 483 -74 429 -42 452 -68 b 689 -74 552 -86 622 -85 b 742 -10 720 -68 742 -41 l 742 81 b 891 167 797 101 848 130 l 970 122 b 1052 136 997 106 1032 112 b 1168 315 1096 189 1144 249 b 1139 394 1179 345 1167 378 z m 729 382 b 442 669 520 221 282 460 b 729 382 651 829 889 591 z m 1389 -114 b 1475 -114 1418 -119 1447 -119 l 1497 -152 b 1538 -167 1506 -167 1523 -173 b 1625 -117 1570 -155 1600 -138 b 1633 -74 1638 -106 1641 -88 l 1611 -35 b 1654 39 1630 -14 1644 12 l 1699 39 b 1732 67 1715 39 1729 51 b 1732 168 1737 101 1737 135 b 1699 196 1729 184 1715 196 l 1654 196 b 1611 270 1644 223 1630 248 l 1633 309 b 1625 352 1641 323 1638 341 b 1538 402 1600 373 1570 390 b 1497 387 1523 408 1506 402 l 1475 349 b 1389 349 1447 354 1418 354 l 1367 387 b 1326 402 1359 402 1341 408 b 1239 352 1294 390 1265 373 b 1231 309 1226 341 1223 323 l 1254 270 b 1210 196 1235 248 1220 223 l 1166 196 b 1133 168 1149 196 1135 184 b 1133 67 1127 134 1127 100 b 1166 39 1135 51 1149 39 l 1210 39 b 1254 -35 1220 12 1235 -14 l 1231 -74 b 1239 -117 1223 -88 1226 -106 b 1326 -167 1265 -138 1294 -155 b 1367 -152 1341 -173 1359 -167 z m 1361 46 b 1504 189 1280 151 1399 270 b 1361 46 1584 85 1465 -34 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1128 694 b 564 1128 1128 934 876 1128 b 0 694 253 1128 0 934 b 103 445 0 601 38 516 b 6 297 67 363 7 298 b 2 273 0 291 -2 282 b 22 260 5 265 13 260 b 262 328 121 260 203 294 b 564 260 350 286 453 260 b 1128 694 876 260 1128 455 z m 1459 98 b 1563 347 1524 168 1563 254 b 1212 749 1563 529 1417 684 b 1215 694 1214 731 1215 713 b 564 174 1215 407 923 174 b 478 179 535 174 506 176 b 998 -87 564 23 764 -87 b 1300 -19 1109 -87 1213 -62 b 1541 -87 1359 -53 1442 -87 b 1561 -74 1549 -87 1557 -82 b 1557 -50 1564 -66 1563 -57 b 1459 98 1556 -49 1496 15 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 781 1215 b 703 1167 750 1215 719 1199 l 526 808 l 130 750 b 82 602 59 740 30 652 l 369 323 l 301 -72 b 427 -163 289 -143 363 -197 l 781 23 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 808 634 l 775 955 l 890 955 b 955 1020 926 955 955 984 l 955 1150 b 890 1215 955 1186 926 1215 l 152 1215 b 87 1150 116 1215 87 1186 l 87 1020 b 152 955 87 984 116 955 l 266 955 l 233 634 b 0 326 102 573 0 463 b 65 260 0 290 29 260 l 434 260 l 434 -22 b 436 -31 434 -25 435 -28 l 501 -162 b 540 -162 509 -178 532 -178 l 605 -31 b 608 -22 607 -29 608 -25 l 608 260 l 977 260 b 1042 326 1013 260 1042 290 b 808 634 1042 464 939 574 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1497 1042 l 1215 1042 l 1215 1150 b 1150 1215 1215 1186 1186 1215 l 412 1215 b 347 1150 376 1215 347 1186 l 347 1042 l 65 1042 b 0 977 29 1042 0 1013 l 0 825 b 168 551 0 728 61 628 b 466 438 253 490 357 451 b 651 239 551 297 651 239 l 651 43 l 521 43 b 347 -109 425 43 347 -13 l 347 -141 b 380 -174 347 -159 362 -174 l 1183 -174 b 1215 -141 1201 -174 1215 -159 l 1215 -109 b 1042 43 1215 -13 1137 43 l 911 43 l 911 239 b 1096 438 911 239 1011 297 b 1395 551 1206 451 1309 490 b 1563 825 1501 628 1563 728 l 1563 977 b 1497 1042 1563 1013 1533 1042 z m 269 692 b 174 825 203 740 174 793 l 174 868 l 348 868 b 382 634 350 780 363 702 b 269 692 342 648 303 668 z m 1389 825 b 1293 692 1389 781 1341 727 b 1180 634 1259 668 1221 648 b 1214 868 1199 702 1212 780 l 1389 868 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 803 174 b 868 239 839 174 868 203 l 868 694 l 1106 694 b 1144 787 1154 694 1178 753 l 731 1200 b 657 1200 711 1220 678 1220 l 245 787 b 283 694 211 753 235 694 l 521 694 l 521 239 b 586 174 521 203 550 174 z m 1389 195 b 1324 260 1389 231 1360 260 l 955 260 l 955 239 b 803 87 955 155 887 87 l 586 87 b 434 239 502 87 434 155 l 434 260 l 65 260 b 0 195 29 260 0 231 l 0 -109 b 65 -174 0 -145 29 -174 l 1324 -174 b 1389 -109 1360 -174 1389 -145 z m 1053 -43 b 998 -98 1053 -73 1028 -98 b 944 -43 968 -98 944 -73 b 998 11 944 -14 968 11 b 1053 -43 1028 11 1053 -14 z m 1226 -43 b 1172 -98 1226 -73 1202 -98 b 1118 -43 1142 -98 1118 -73 b 1172 11 1118 -14 1142 11 b 1226 -43 1202 11 1226 -14 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1327 1153 b 1123 1199 1264 1216 1179 1231 b 252 963 963 1106 631 1343 b 17 92 -127 584 110 252 b 62 -111 -16 36 0 -49 b 266 -157 125 -174 210 -190 b 1137 78 426 -64 758 -301 b 1372 949 1516 458 1279 789 b 1327 1153 1405 1005 1389 1090 z m 662 956 b 259 554 504 917 299 712 b 217 521 254 534 236 521 b 206 522 214 521 210 521 b 175 575 183 528 169 552 b 641 1040 222 764 451 993 b 693 1009 664 1046 687 1032 b 662 956 699 986 685 962 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1338 1149 l 1056 1214 b 982 1176 1026 1221 994 1205 l 852 872 b 870 796 840 846 848 814 l 1035 662 b 554 181 937 454 766 280 l 419 345 b 343 364 401 368 370 375 l 40 234 b 2 159 11 221 -5 190 l 67 -123 b 130 -174 74 -152 100 -174 b 1389 1085 825 -174 1389 389 b 1338 1149 1389 1115 1368 1142 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1085 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -29 58 -87 l 1085 -87 b 1215 43 1157 -87 1215 -29 l 1215 998 b 1085 1128 1215 1070 1157 1128 z m 255 87 b 215 118 236 87 220 100 l 175 295 b 174 304 174 297 174 301 b 198 341 174 319 185 335 l 388 423 b 404 426 392 424 400 426 b 436 411 415 426 429 419 l 520 308 b 820 609 653 370 759 479 l 718 693 b 703 724 709 700 703 714 b 706 740 703 729 704 736 l 787 930 b 825 955 793 944 810 955 b 834 954 827 955 831 954 l 1010 913 b 1042 873 1028 909 1042 891 l 1042 873 b 255 87 1042 439 689 87 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1085 521 l 412 521 l 412 801 b 606 998 412 908 498 997 b 803 803 714 999 803 911 l 803 760 b 868 694 803 723 832 694 l 955 694 b 1020 760 991 694 1020 723 l 1020 803 b 606 1215 1020 1031 834 1216 b 195 799 378 1214 195 1027 l 195 521 l 130 521 b 0 391 58 521 0 463 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 391 b 1085 521 1215 463 1157 521 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 0 43 b 130 -87 0 -28 58 -87 l 1432 -87 b 1563 43 1504 -87 1563 -28 l 1563 521 l 0 521 z m 521 228 b 553 260 521 246 535 260 l 922 260 b 955 228 940 260 955 246 l 955 119 b 922 87 955 101 940 87 l 553 87 b 521 119 535 87 521 101 z m 174 228 b 206 260 174 246 188 260 l 401 260 b 434 228 419 260 434 246 l 434 119 b 401 87 434 101 419 87 l 206 87 b 174 119 188 87 174 101 z m 1563 998 b 1432 1128 1563 1070 1504 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 868 l 1563 868 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 347 87 b 174 261 347 183 270 261 b 0 87 78 261 0 183 b 174 -87 0 -9 78 -87 b 347 87 270 -87 347 -9 z m 824 -41 b 46 737 801 378 465 714 b 0 694 21 738 0 719 l 0 563 b 40 520 0 541 18 521 b 607 -46 344 500 587 258 b 650 -87 608 -69 627 -87 l 781 -87 b 824 -41 805 -87 825 -66 z m 1215 -42 b 45 1128 1192 592 682 1106 b 0 1085 20 1129 0 1110 l 0 955 b 42 911 0 931 19 912 b 998 -45 561 890 977 474 b 1041 -87 999 -68 1018 -87 l 1172 -87 b 1215 -42 1196 -87 1216 -67 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1563 391 b 1432 521 1563 463 1504 521 l 130 521 b 0 391 58 521 0 463 l 0 130 b 130 0 0 58 58 0 l 1432 0 b 1563 130 1504 0 1563 58 z m 1432 608 l 1432 608 b 1516 591 1456 608 1494 600 l 1254 984 b 1146 1042 1233 1016 1184 1042 l 417 1042 b 309 984 378 1042 330 1016 l 47 591 b 130 608 69 600 106 608 l 130 608 z m 1302 347 b 1389 260 1350 347 1389 308 b 1302 174 1389 212 1350 174 b 1215 260 1254 174 1215 212 b 1302 347 1215 308 1254 347 z m 1042 347 b 1128 260 1090 347 1128 308 b 1042 174 1128 212 1090 174 b 955 260 994 174 955 212 b 1042 347 955 308 994 347 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1563 564 b 1476 714 1563 628 1527 684 l 1476 1128 b 1389 1215 1476 1152 1457 1215 b 1335 1196 1370 1215 1350 1209 l 1104 1012 b 694 868 988 919 843 868 l 174 868 b 0 694 78 868 0 790 l 0 434 b 174 260 0 338 78 260 l 265 260 b 259 174 261 232 259 203 b 328 -127 259 66 284 -36 b 405 -174 343 -156 373 -174 l 607 -174 b 677 -36 678 -174 720 -93 b 606 174 633 22 606 95 b 618 260 606 204 611 233 l 694 260 b 1104 117 843 260 988 209 l 1335 -68 b 1389 -87 1348 -78 1372 -87 b 1476 0 1456 -87 1476 -25 l 1476 415 b 1563 564 1527 445 1563 500 z m 1302 181 l 1212 252 b 694 434 1066 370 882 434 l 694 694 b 1212 876 882 694 1066 759 l 1302 948 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1244 521 l 1369 643 b 1340 753 1406 678 1389 741 l 1170 796 l 1218 964 b 1137 1045 1231 1013 1186 1058 l 969 997 l 926 1167 b 816 1196 914 1215 851 1232 l 694 1070 l 572 1196 b 463 1167 538 1232 475 1215 l 420 997 l 251 1045 b 171 964 203 1058 157 1013 l 219 796 l 49 753 b 20 643 0 741 -17 678 l 145 521 l 20 399 b 49 290 -17 364 0 301 l 219 246 l 171 78 b 251 -3 157 30 203 -16 l 420 45 l 463 -125 b 572 -154 475 -175 538 -190 l 694 -29 l 816 -154 b 926 -125 850 -190 914 -174 l 969 45 l 1137 -3 b 1218 78 1186 -16 1231 30 l 1170 246 l 1340 290 b 1369 399 1389 301 1406 364 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1389 674 b 1267 793 1389 737 1331 793 l 863 793 b 901 860 878 815 891 837 b 781 1042 938 946 874 1042 b 673 963 731 1042 694 1012 b 381 753 640 886 459 788 b 304 634 333 731 304 684 l 304 634 l 304 169 b 359 104 304 137 327 109 b 694 1 486 85 526 0 b 738 0 702 1 730 0 b 935 193 875 0 959 60 b 979 362 985 219 1018 300 b 996 555 1046 420 1041 509 l 1267 555 b 1389 674 1333 555 1389 610 z m 260 673 b 195 738 260 709 231 738 l 65 738 b 0 673 29 738 0 709 l 0 152 b 65 87 0 116 29 87 l 195 87 b 260 152 231 87 260 116 z m 184 217 b 130 163 184 187 160 163 b 76 217 100 163 76 187 b 130 271 76 247 100 271 b 184 217 160 271 184 247 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 122 793 b 0 674 58 793 0 737 b 122 555 0 610 56 555 l 392 555 b 409 362 348 509 343 420 b 454 193 371 300 404 219 b 651 0 430 60 514 0 b 694 1 658 0 687 1 b 1030 104 863 0 903 85 b 1085 169 1062 109 1085 137 l 1085 634 l 1085 634 b 1008 753 1085 684 1056 731 b 716 963 929 788 749 886 b 608 1042 695 1012 658 1042 b 488 860 515 1042 451 946 b 526 793 498 837 511 815 z m 1194 738 b 1128 673 1158 738 1128 709 l 1128 152 b 1194 87 1128 116 1158 87 l 1324 87 b 1389 152 1360 87 1389 116 l 1389 673 b 1324 738 1389 709 1360 738 z m 1259 163 b 1204 217 1229 163 1204 187 b 1259 271 1204 247 1229 271 b 1313 217 1289 271 1313 247 b 1259 163 1313 187 1289 163 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 368 1215 b 249 1094 305 1215 249 1157 l 249 689 b 182 727 227 705 204 718 b 0 608 95 764 0 700 b 79 499 0 557 30 520 b 289 207 156 466 253 286 b 408 130 310 159 358 130 l 408 130 l 873 130 b 937 185 905 130 932 154 b 1041 521 957 312 1042 353 b 1042 564 1041 528 1042 557 b 848 761 1042 702 982 786 b 679 806 823 812 741 845 b 487 823 622 873 533 867 l 487 1094 b 368 1215 487 1159 432 1215 z m 369 87 b 304 22 333 87 304 58 l 304 -109 b 369 -174 304 -144 333 -174 l 890 -174 b 955 -109 926 -174 955 -144 l 955 22 b 890 87 955 58 926 87 z m 825 11 b 879 -43 855 11 879 -13 b 825 -98 879 -73 855 -98 b 770 -43 795 -98 770 -73 b 825 11 770 -13 795 11 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 249 -52 b 368 -174 249 -116 305 -174 b 487 -52 432 -174 487 -118 l 487 219 b 679 236 533 174 622 169 b 848 280 741 197 823 230 b 1042 477 982 256 1042 340 b 1041 521 1042 485 1041 513 b 937 856 1042 689 957 729 b 873 911 932 888 905 911 l 408 911 l 408 911 b 289 834 358 911 310 883 b 79 543 253 756 156 575 b 0 434 30 521 0 485 b 182 314 0 341 95 277 b 249 353 204 324 227 337 z m 304 1020 b 369 955 304 984 333 955 l 890 955 b 955 1020 926 955 955 984 l 955 1150 b 890 1215 955 1186 926 1215 l 369 1215 b 304 1150 333 1215 304 1186 z m 879 1085 b 825 1031 879 1055 855 1031 b 770 1085 795 1031 770 1055 b 825 1139 770 1115 795 1139 b 879 1085 855 1139 879 1115 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 z m 773 238 b 774 145 799 212 800 170 l 744 115 b 652 115 719 90 678 90 l 292 475 b 292 567 267 500 267 541 l 652 927 b 744 927 677 952 719 952 l 774 897 b 773 804 800 871 799 829 l 568 608 l 1063 608 b 1128 543 1099 608 1128 579 l 1128 499 b 1063 434 1128 463 1099 434 l 568 434 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 z m 616 804 b 615 897 590 829 589 871 l 645 927 b 737 927 670 952 711 952 l 1097 567 b 1097 475 1122 541 1122 500 l 737 115 b 645 115 712 90 670 90 l 615 145 b 616 238 589 170 590 212 l 821 434 l 326 434 b 260 499 289 434 260 463 l 260 543 b 326 608 260 579 289 608 l 821 608 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 22 521 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 b 22 521 323 1194 22 892 z m 411 442 b 318 441 386 416 344 416 l 288 471 b 288 563 263 496 263 538 l 649 923 b 741 923 674 949 715 949 l 1100 563 b 1100 471 1126 538 1126 497 l 1071 441 b 978 442 1045 416 1003 416 l 781 647 l 781 152 b 716 87 781 116 752 87 l 673 87 b 608 152 637 87 608 116 l 608 647 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1367 521 b 694 1194 1367 892 1066 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 z m 978 599 b 1071 600 1003 626 1045 626 l 1100 570 b 1100 479 1126 545 1126 504 l 740 119 b 648 119 715 93 674 93 l 289 479 b 289 570 263 504 263 545 l 318 600 b 411 599 344 626 386 626 l 608 394 l 608 890 b 673 955 608 926 637 955 l 716 955 b 781 890 752 955 781 926 l 781 394 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 913 781 b 673 1194 873 1023 781 1194 b 433 781 565 1194 472 1023 z m 412 521 b 421 347 412 461 416 403 l 924 347 b 933 521 930 403 933 461 b 924 694 933 581 930 639 l 421 694 b 412 521 416 639 412 581 z m 1293 781 b 865 1165 1216 965 1058 1108 b 1000 781 931 1074 976 936 z m 481 1165 b 52 781 287 1108 130 965 l 345 781 b 481 1165 369 936 414 1074 z m 1322 694 l 1011 694 b 1020 521 1017 637 1020 579 b 1011 347 1020 463 1017 404 l 1322 347 b 1345 521 1337 403 1345 461 b 1322 694 1345 581 1337 639 z m 326 521 b 334 694 326 579 329 637 l 23 694 b 0 521 9 639 0 581 b 23 347 0 461 9 403 l 334 347 b 326 521 329 404 326 463 z m 433 260 b 673 -152 472 18 565 -152 b 913 260 781 -152 873 18 z m 865 -124 b 1293 260 1058 -66 1216 76 l 1000 260 b 865 -124 977 106 931 -32 z m 52 260 b 481 -124 130 76 287 -66 b 345 260 415 -32 369 106 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1377 919 b 1323 934 1371 944 1341 952 l 1121 733 l 937 763 l 906 947 l 1108 1149 b 1093 1204 1126 1167 1117 1198 b 722 1101 964 1236 823 1201 b 630 702 615 993 585 837 l 51 123 b 51 -123 -17 55 -17 -55 b 296 -123 119 -191 229 -191 l 875 456 b 1274 548 1011 410 1166 440 b 1377 919 1375 649 1409 791 z m 174 -65 b 109 0 138 -65 109 -36 b 174 65 109 36 138 65 b 239 0 210 65 239 36 b 174 -65 239 -36 210 -65 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 379 1119 b 356 1129 373 1124 363 1129 b 333 1119 348 1129 338 1124 l 160 947 l 98 1007 b 75 1017 93 1013 83 1017 b 52 1007 68 1017 57 1013 l 10 965 b 0 942 4 959 0 949 b 10 918 0 934 4 924 l 139 790 b 163 780 144 785 155 780 b 186 790 170 780 181 785 l 229 832 l 425 1028 b 434 1051 430 1033 434 1044 b 425 1074 434 1059 430 1069 z m 379 687 b 356 697 373 692 363 697 b 333 687 348 697 338 692 l 160 514 l 98 574 b 75 584 93 580 83 584 b 52 574 68 584 57 580 l 10 532 b 0 509 4 526 0 516 b 10 486 0 501 4 491 l 138 357 b 162 347 144 351 155 347 b 186 357 170 347 181 351 l 229 399 l 425 595 b 434 618 430 600 434 611 b 425 641 434 626 430 636 z m 174 217 b 42 87 102 217 42 159 b 174 -43 42 15 102 -43 b 304 87 245 -43 304 15 b 174 217 304 159 245 217 z m 1345 174 l 564 174 b 521 130 540 174 521 154 l 521 43 b 564 0 521 19 540 0 l 1345 0 b 1389 43 1369 0 1389 19 l 1389 130 b 1345 174 1389 154 1369 174 z m 1345 1042 l 564 1042 b 521 998 540 1042 521 1022 l 521 911 b 564 868 521 887 540 868 l 1345 868 b 1389 911 1369 868 1389 887 l 1389 998 b 1345 1042 1389 1022 1369 1042 z m 1345 608 l 564 608 b 521 564 540 608 521 588 l 521 477 b 564 434 521 453 540 434 l 1345 434 b 1389 477 1369 434 1389 453 l 1389 564 b 1345 608 1389 588 1369 608 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1324 1215 l 65 1215 b 19 1104 7 1215 -22 1145 l 521 602 l 521 43 b 549 -10 521 22 531 2 l 766 -162 b 868 -108 808 -192 868 -161 l 868 602 l 1370 1104 b 1324 1215 1411 1145 1382 1215 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 868 304 b 825 260 868 280 849 260 l 564 260 b 521 304 540 260 521 280 l 521 434 l 0 434 l 0 43 b 130 -87 0 -26 61 -87 l 1259 -87 b 1389 43 1328 -87 1389 -26 l 1389 434 l 868 434 z m 1259 868 l 1042 868 l 1042 998 b 911 1128 1042 1068 981 1128 l 477 1128 b 347 998 408 1128 347 1068 l 347 868 l 130 868 b 0 738 61 868 0 807 l 0 521 l 1389 521 l 1389 738 b 1259 868 1389 807 1328 868 z m 868 868 l 521 868 l 521 955 l 868 955 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 955 60 b 909 171 996 101 967 171 l 770 171 l 770 445 l 1044 445 l 1044 306 b 1155 260 1044 248 1114 219 l 1370 475 b 1370 567 1395 500 1395 541 l 1155 782 b 1044 736 1114 823 1044 794 l 1044 597 l 770 597 l 770 870 l 909 870 b 955 981 967 870 996 940 l 740 1196 b 648 1196 715 1222 674 1222 l 433 981 b 480 870 392 940 422 870 l 618 870 l 618 597 l 345 597 l 345 736 b 234 782 345 794 275 823 l 19 567 b 19 475 -6 542 -6 500 l 234 260 b 345 306 275 219 345 248 l 345 445 l 618 445 l 618 171 l 480 171 b 434 60 422 171 393 101 l 649 -155 b 741 -155 674 -180 715 -180 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 260 608 b 434 781 356 608 434 685 b 260 955 434 877 356 955 b 87 781 165 955 87 877 b 260 608 87 685 165 608 z m 1476 608 b 1649 781 1571 608 1649 685 b 1476 955 1649 877 1571 955 b 1302 781 1380 955 1302 877 b 1476 608 1302 685 1380 608 z m 1563 521 l 1389 521 b 1267 470 1341 521 1298 502 b 1470 174 1376 410 1453 302 l 1649 174 b 1736 260 1697 174 1736 212 l 1736 347 b 1563 521 1736 443 1658 521 z m 868 521 b 1172 825 1036 521 1172 657 b 868 1128 1172 993 1036 1128 b 564 825 700 1128 564 993 b 868 521 564 657 700 521 z m 1076 434 l 1054 434 b 868 391 997 407 935 391 b 682 434 801 391 739 407 l 660 434 b 347 122 487 434 347 294 l 347 43 b 477 -87 347 -28 406 -87 l 1259 -87 b 1389 43 1331 -87 1389 -28 l 1389 122 b 1076 434 1389 294 1249 434 z m 470 470 b 347 521 438 502 395 521 l 174 521 b 0 347 78 521 0 443 l 0 260 b 87 174 0 212 39 174 l 266 174 b 470 470 283 302 360 410 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 886 712 b 830 759 872 727 847 748 b 774 754 813 771 789 769 l 717 697 b 686 616 695 675 685 645 b 704 582 686 604 695 589 b 733 559 712 577 722 569 b 733 283 809 483 809 359 l 550 100 b 273 102 474 24 349 24 b 276 378 197 179 200 302 l 311 414 b 322 459 323 426 327 443 b 295 602 306 506 297 554 b 221 631 294 640 248 657 l 121 530 b 121 -53 -40 369 -40 108 b 704 -53 281 -214 543 -214 l 886 129 b 887 130 886 130 887 130 b 886 712 1046 290 1048 550 z m 1268 1095 b 685 1095 1108 1255 846 1255 l 503 912 b 502 911 503 912 502 912 b 503 329 343 752 341 492 b 559 283 517 315 542 294 b 614 287 576 271 599 272 l 672 344 b 703 425 694 367 704 397 b 684 460 703 437 694 453 b 656 483 676 465 666 473 b 656 759 580 559 580 683 l 839 941 b 1116 940 915 1018 1040 1017 b 1113 663 1191 863 1189 739 l 1078 628 b 1067 583 1066 616 1062 599 b 1094 440 1083 536 1092 488 b 1168 411 1095 402 1141 384 l 1268 512 b 1268 1095 1429 672 1429 934 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1458 601 b 1476 694 1469 630 1476 661 b 1215 955 1476 838 1359 955 b 1071 911 1162 955 1112 939 b 694 1128 996 1041 855 1128 b 260 694 455 1128 260 934 b 261 672 260 687 261 680 b 0 304 109 619 0 474 b 391 -87 0 88 175 -87 l 1389 -87 b 1736 260 1581 -87 1736 69 b 1458 601 1736 428 1617 569 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1186 121 l 868 632 l 868 1042 l 890 1042 b 955 1107 926 1042 955 1071 l 955 1150 b 890 1215 955 1186 926 1215 l 326 1215 b 260 1150 289 1215 260 1186 l 260 1107 b 326 1042 260 1071 289 1042 l 347 1042 l 347 632 l 29 121 b 192 -174 -50 -7 42 -174 l 1023 -174 b 1186 121 1174 -174 1265 -7 z m 374 347 l 505 558 b 521 608 515 572 521 589 l 521 1042 l 694 1042 l 694 608 b 710 558 694 589 700 572 l 841 347 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 754 521 l 1206 972 b 1206 1018 1218 985 1218 1006 b 883 1018 1117 1107 972 1107 l 570 705 l 503 772 b 521 868 514 802 521 834 b 260 1128 521 1012 404 1128 b 0 868 117 1128 0 1012 b 260 608 0 724 117 608 b 297 610 273 608 285 609 l 386 521 l 297 432 b 260 434 285 433 273 434 b 0 174 117 434 0 317 b 260 -87 0 30 117 -87 b 521 174 404 -87 521 30 b 503 269 521 207 514 240 l 570 337 l 883 23 b 1206 23 972 -66 1117 -66 b 1206 69 1218 36 1218 57 z m 260 781 b 174 868 213 781 174 820 b 260 955 174 916 213 955 b 347 868 308 955 347 916 b 260 781 347 820 308 781 z m 260 87 b 174 174 213 87 174 126 b 260 260 174 221 213 260 b 347 174 308 260 347 221 b 260 87 347 126 308 87 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 868 0 l 412 0 b 260 152 329 0 260 68 l 260 955 l 65 955 b 0 890 29 955 0 926 l 0 -109 b 65 -174 0 -144 29 -174 l 803 -174 b 868 -109 839 -174 868 -144 z m 868 933 l 868 1215 l 412 1215 b 347 1150 376 1215 347 1186 l 347 152 b 412 87 347 116 376 87 l 1150 87 b 1215 152 1186 87 1215 116 l 1215 868 l 933 868 b 868 933 897 868 868 897 z m 1196 1017 l 1017 1196 b 971 1215 1007 1207 986 1215 l 955 1215 l 955 955 l 1215 955 l 1215 971 b 1196 1017 1215 986 1207 1007 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 117 -49 b 700 -51 277 -214 538 -216 l 1167 427 b 1166 488 1183 444 1183 472 l 1104 549 b 1043 548 1087 566 1059 565 l 576 71 b 242 72 483 -24 334 -24 b 245 420 149 167 151 323 l 814 1002 b 1002 1002 866 1055 950 1055 b 1002 804 1055 947 1055 859 l 506 297 b 463 298 494 285 475 285 b 465 347 451 311 452 334 l 855 746 b 854 807 872 763 871 790 l 792 868 b 731 867 775 885 748 884 l 341 469 b 337 179 263 389 260 260 b 630 176 417 94 549 93 l 1126 683 b 1126 1123 1245 805 1245 1001 b 690 1123 1006 1246 810 1246 l 121 541 b 117 -49 -38 378 -41 114 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1177 863 l 950 1090 b 858 1128 929 1111 887 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -29 58 -87 l 1085 -87 b 1215 43 1157 -87 1215 -29 l 1215 771 b 1177 863 1215 800 1198 842 z m 608 87 b 434 260 512 87 434 165 b 608 434 434 356 512 434 b 781 260 704 434 781 356 b 608 87 781 165 704 87 z m 868 913 l 868 640 b 836 608 868 622 853 608 l 206 608 b 174 640 188 608 174 622 l 174 922 b 206 955 174 940 188 955 l 826 955 b 849 945 835 955 843 951 l 859 936 b 868 913 864 931 868 920 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1085 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -28 58 -87 l 1085 -87 b 1215 43 1157 -87 1215 -28 l 1215 998 b 1085 1128 1215 1070 1157 1128 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 43 857 l 1172 857 b 1215 901 1196 857 1215 877 l 1215 1009 b 1172 1053 1215 1033 1196 1053 l 43 1053 b 0 1009 19 1053 0 1033 l 0 901 b 43 857 0 877 19 857 z m 43 423 l 1172 423 b 1215 467 1196 423 1215 443 l 1215 575 b 1172 618 1215 599 1196 618 l 43 618 b 0 575 19 618 0 599 l 0 467 b 43 423 0 443 19 423 z m 43 -11 l 1172 -11 b 1215 33 1196 -11 1215 9 l 1215 141 b 1172 184 1215 165 1196 184 l 43 184 b 0 141 19 184 0 165 l 0 33 b 43 -11 0 9 19 -11 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 130 1085 b 0 955 58 1085 0 1027 b 130 825 0 883 58 825 b 260 955 202 825 260 883 b 130 1085 260 1027 202 1085 z m 130 651 b 0 521 58 651 0 593 b 130 391 0 449 58 391 b 260 521 202 391 260 449 b 130 651 260 593 202 651 z m 130 217 b 0 87 58 217 0 159 b 130 -43 0 15 58 -43 b 260 87 202 -43 260 15 b 130 217 260 159 202 217 z m 1345 174 l 477 174 b 434 130 453 174 434 154 l 434 43 b 477 0 434 19 453 0 l 1345 0 b 1389 43 1369 0 1389 19 l 1389 130 b 1345 174 1389 154 1369 174 z m 1345 1042 l 477 1042 b 434 998 453 1042 434 1022 l 434 911 b 477 868 434 887 453 868 l 1345 868 b 1389 911 1369 868 1389 887 l 1389 998 b 1345 1042 1389 1022 1369 1042 z m 1345 608 l 477 608 b 434 564 453 608 434 588 l 434 477 b 477 434 434 453 453 434 l 1345 434 b 1389 477 1369 434 1389 453 l 1389 564 b 1345 608 1389 588 1369 608 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 168 127 l 215 182 b 229 218 223 191 229 207 b 229 221 229 219 229 220 l 229 230 b 198 260 229 250 218 260 l 43 260 b 22 239 31 260 22 251 l 22 195 b 43 174 22 183 31 174 l 105 174 b 75 140 97 165 83 150 l 60 121 b 53 81 49 107 46 94 l 56 76 b 89 54 64 60 73 54 l 102 54 b 145 30 130 54 145 48 b 106 7 145 17 133 7 b 64 16 94 7 75 11 b 22 7 46 26 32 25 l 6 -18 b 14 -61 -4 -35 -2 -50 b 114 -87 34 -74 69 -87 b 245 33 207 -87 245 -25 b 168 127 245 72 221 114 z m 1345 608 l 477 608 b 434 564 453 608 434 588 l 434 477 b 477 434 434 453 453 434 l 1345 434 b 1389 477 1369 434 1389 453 l 1389 564 b 1345 608 1389 588 1369 608 z m 1345 1042 l 477 1042 b 434 998 453 1042 434 1022 l 434 911 b 477 868 434 887 453 868 l 1345 868 b 1389 911 1369 868 1389 887 l 1389 998 b 1345 1042 1389 1022 1369 1042 z m 1345 174 l 477 174 b 434 130 453 174 434 154 l 434 43 b 477 0 434 19 453 0 l 1345 0 b 1389 43 1369 0 1389 19 l 1389 130 b 1345 174 1389 154 1369 174 z m 43 781 l 217 781 b 239 803 229 781 239 791 l 239 846 b 217 868 239 858 229 868 l 174 868 l 174 1107 b 152 1128 174 1119 164 1128 l 87 1128 b 67 1116 79 1128 71 1123 l 46 1073 b 43 1063 44 1071 43 1066 b 65 1042 43 1051 53 1042 l 87 1042 l 87 868 l 43 868 b 22 846 31 868 22 858 l 22 803 b 43 781 22 791 31 781 z m 33 347 l 217 347 b 239 369 229 347 239 357 l 239 412 b 217 434 239 424 229 434 l 112 434 b 243 587 121 462 243 485 b 123 694 243 666 175 694 b 13 644 65 694 31 667 b 20 602 1 628 5 614 l 44 583 b 87 590 59 571 74 577 b 113 600 93 596 105 600 l 113 600 b 138 577 122 600 138 596 b 0 389 138 542 0 517 l 0 378 b 33 347 0 358 14 347 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1345 608 l 797 608 l 561 680 b 477 793 515 695 477 745 b 596 911 477 859 530 911 l 777 911 l 777 911 b 898 837 823 911 877 878 b 937 813 905 823 922 813 b 956 817 942 813 951 815 l 1073 875 b 1097 914 1086 882 1097 900 b 1092 934 1097 920 1095 929 l 1091 936 b 780 1128 1038 1042 898 1128 l 780 1128 l 595 1128 b 260 793 410 1128 260 978 b 262 761 260 784 261 769 b 321 608 267 704 289 652 l 43 608 b 0 564 19 608 0 588 l 0 477 b 43 434 0 453 19 434 l 1345 434 b 1389 477 1369 434 1389 453 l 1389 564 b 1345 608 1389 588 1369 608 z m 857 347 b 911 248 887 328 911 284 b 793 130 911 183 858 130 l 612 130 l 612 130 b 491 205 566 130 512 164 b 452 229 484 218 467 229 b 433 224 447 229 438 227 l 316 166 b 292 127 303 160 292 142 b 297 108 292 122 294 113 l 298 105 b 609 -87 351 -1 490 -87 l 609 -87 l 793 -87 b 1129 249 978 -87 1129 64 b 1127 281 1129 258 1128 272 b 1113 347 1125 300 1119 329 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 87 1042 l 174 1042 l 174 608 b 608 174 174 368 368 174 b 1042 608 847 174 1042 368 l 1042 1042 l 1128 1042 b 1172 1085 1152 1042 1172 1061 l 1172 1172 b 1128 1215 1172 1196 1152 1215 l 738 1215 b 694 1172 714 1215 694 1196 l 694 1085 b 738 1042 694 1061 714 1042 l 825 1042 l 825 608 b 608 391 825 488 727 391 b 391 608 488 391 391 488 l 391 1042 l 477 1042 b 521 1085 501 1042 521 1061 l 521 1172 b 477 1215 521 1196 501 1215 l 87 1215 b 43 1172 63 1215 43 1196 l 43 1085 b 87 1042 43 1061 63 1042 z m 1172 0 l 43 0 b 0 -43 19 0 0 -19 l 0 -130 b 43 -174 0 -154 19 -174 l 1172 -174 b 1215 -130 1196 -174 1215 -154 l 1215 -43 b 1172 0 1215 -19 1196 0 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1259 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -29 58 -87 l 1259 -87 b 1389 43 1331 -87 1389 -29 l 1389 998 b 1259 1128 1389 1070 1331 1128 z m 608 87 l 174 87 l 174 347 l 608 347 z m 608 521 l 174 521 l 174 781 l 608 781 z m 1215 87 l 781 87 l 781 347 l 1215 347 z m 1215 521 l 781 521 l 781 781 l 1215 781 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 608 955 l 651 1042 l 738 1085 l 651 1128 l 608 1215 l 564 1128 l 477 1085 l 564 1042 z m 217 781 l 289 926 l 434 998 l 289 1071 l 217 1215 l 145 1071 l 0 998 l 145 926 z m 1172 434 l 1100 289 l 955 217 l 1100 145 l 1172 0 l 1244 145 l 1389 217 l 1244 289 z m 1363 960 l 1133 1190 b 1072 1215 1116 1207 1094 1215 b 1011 1190 1050 1215 1027 1207 l 25 205 b 25 82 -8 171 -8 116 l 256 -148 b 317 -174 273 -165 295 -174 b 378 -148 339 -174 361 -165 l 1363 837 b 1363 960 1397 871 1397 926 z m 975 663 l 837 801 l 1072 1036 l 1210 898 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1693 260 l 1649 260 l 1649 554 b 1611 646 1649 588 1635 621 l 1340 917 b 1248 955 1316 941 1283 955 l 1128 955 l 1128 1085 b 998 1215 1128 1157 1070 1215 l 130 1215 b 0 1085 58 1215 0 1157 l 0 217 b 130 87 0 145 58 87 l 174 87 b 434 -174 174 -57 290 -174 b 694 87 578 -174 694 -57 l 1042 87 b 1302 -174 1042 -57 1158 -174 b 1563 87 1446 -174 1563 -57 l 1693 87 b 1736 130 1717 87 1736 106 l 1736 217 b 1693 260 1736 241 1717 260 z m 434 -43 b 304 87 362 -43 304 15 b 434 217 304 159 362 217 b 564 87 506 217 564 159 b 434 -43 564 15 506 -43 z m 1302 -43 b 1172 87 1230 -43 1172 15 b 1302 217 1172 159 1230 217 b 1432 87 1374 217 1432 159 b 1302 -43 1432 15 1374 -43 z m 1519 521 l 1128 521 l 1128 825 l 1248 825 l 1519 554 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1649 1042 l 87 1042 b 0 955 39 1042 0 1003 l 0 87 b 87 0 0 39 39 0 l 1649 0 b 1736 87 1697 0 1736 39 l 1736 955 b 1649 1042 1736 1003 1697 1042 z m 130 130 l 130 304 b 304 130 226 304 304 226 z m 130 738 l 130 911 l 304 911 b 130 738 304 816 226 738 z m 868 260 b 651 521 748 260 651 377 b 868 781 651 665 748 781 b 1085 521 988 781 1085 665 b 868 260 1085 377 988 260 z m 1606 130 l 1432 130 b 1606 304 1432 226 1510 304 z m 1606 738 b 1432 911 1510 738 1432 816 l 1606 911 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 85 694 b 47 602 37 694 12 636 l 396 253 b 472 253 417 232 451 232 l 821 602 b 783 694 855 636 831 694 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 783 260 b 821 353 831 260 856 319 l 472 702 b 396 702 451 723 417 723 l 47 353 b 85 260 12 319 37 260 z "
	},
	"": {
		ha: 521,
		x_min: 0,
		x_max: 0,
		o: "m 521 870 b 428 908 521 918 462 942 l 79 559 b 79 482 58 538 58 504 l 428 133 b 521 172 462 99 521 123 z "
	},
	"": {
		ha: 521,
		x_min: 0,
		x_max: 0,
		o: "m 0 172 b 93 133 0 123 58 99 l 442 482 b 442 559 463 504 463 538 l 93 908 b 0 870 58 942 0 918 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1259 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -29 58 -87 l 1259 -87 b 1389 43 1331 -87 1389 -29 l 1389 998 b 1259 1128 1389 1070 1331 1128 z m 608 87 l 174 87 l 174 781 l 608 781 z m 1215 87 l 781 87 l 781 781 l 1215 781 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 111 434 b 65 323 53 434 24 364 l 388 0 b 480 0 413 -26 455 -26 l 803 323 b 757 434 844 364 815 434 z m 803 719 l 480 1042 b 388 1042 455 1067 413 1067 l 65 719 b 111 608 24 678 53 608 l 757 608 b 803 719 815 608 844 678 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 111 434 b 65 323 53 434 24 364 l 388 0 b 480 0 413 -26 455 -26 l 803 323 b 757 434 844 364 815 434 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 757 608 b 803 719 815 608 844 678 l 480 1042 b 388 1042 455 1067 413 1067 l 65 719 b 111 608 24 678 53 608 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1363 698 b 945 389 1302 650 1222 590 b 694 260 890 349 791 259 b 444 389 598 260 502 347 b 26 697 168 590 87 650 b 0 685 15 706 0 698 l 0 130 b 130 0 0 58 58 0 l 1259 0 b 1389 130 1331 0 1389 58 l 1389 685 b 1363 698 1389 698 1373 706 z m 694 347 b 894 460 757 346 848 426 b 1364 809 1254 721 1281 744 b 1389 860 1380 821 1389 840 l 1389 911 b 1259 1042 1389 983 1331 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 860 b 25 809 0 840 9 821 b 495 460 108 744 135 721 b 694 347 541 426 632 346 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 576 607 b 609 639 594 607 609 621 l 609 769 b 576 802 609 787 594 802 l 309 802 b 695 998 396 921 537 998 b 1173 521 959 998 1173 785 b 695 43 1173 257 959 43 b 379 163 574 43 463 89 b 335 162 366 174 347 174 l 242 70 b 244 22 229 56 230 35 b 695 -152 363 -86 521 -152 b 1368 519 1066 -152 1367 148 b 700 1194 1369 891 1072 1191 b 195 971 500 1195 319 1109 l 195 1183 b 163 1215 195 1201 181 1215 l 33 1215 b 0 1183 15 1215 0 1201 l 0 639 b 33 607 0 621 15 607 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1370 674 l 1308 736 b 1216 736 1283 761 1242 761 l 1201 721 l 894 1027 l 909 1043 b 909 1135 935 1068 935 1109 l 848 1196 b 756 1196 823 1222 781 1222 l 418 859 b 418 767 393 833 393 792 l 480 705 b 572 705 505 680 546 680 l 587 721 l 695 613 l 475 393 l 459 409 b 337 409 426 443 371 443 l 25 97 b 25 -25 -8 63 -8 8 l 148 -148 b 271 -148 182 -182 237 -182 l 582 163 b 582 286 616 197 616 252 l 567 301 l 787 521 l 894 414 l 879 398 b 879 306 853 373 853 332 l 940 245 b 1032 245 966 219 1007 219 l 1370 582 b 1370 674 1395 608 1395 649 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 803 781 l 490 781 l 605 1133 b 543 1215 616 1175 585 1215 l 152 1215 b 87 1159 119 1215 92 1191 l 1 508 b 65 434 -5 469 26 434 l 387 434 l 262 -94 b 325 -174 252 -135 284 -174 b 382 -141 348 -174 370 -162 l 859 684 b 803 781 884 727 853 781 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 347 260 l 87 260 b 0 174 39 260 0 222 l 0 -87 b 87 -174 0 -135 39 -174 l 347 -174 b 434 -87 395 -174 434 -135 l 434 174 b 347 260 434 222 395 260 z m 282 477 l 803 477 l 803 347 l 933 347 l 933 477 l 1454 477 l 1454 347 l 1584 347 l 1584 503 b 1480 608 1584 561 1537 608 l 933 608 l 933 781 l 1042 781 b 1128 868 1090 781 1128 820 l 1128 1128 b 1042 1215 1128 1176 1090 1215 l 694 1215 b 608 1128 647 1215 608 1176 l 608 868 b 694 781 608 820 647 781 l 803 781 l 803 608 l 256 608 b 152 503 199 608 152 561 l 152 347 l 282 347 z m 998 260 l 738 260 b 651 174 690 260 651 222 l 651 -87 b 738 -174 651 -135 690 -174 l 998 -174 b 1085 -87 1046 -174 1085 -135 l 1085 174 b 998 260 1085 222 1046 260 z m 1649 260 l 1389 260 b 1302 174 1341 260 1302 222 l 1302 -87 b 1389 -174 1302 -135 1341 -174 l 1649 -174 b 1736 -87 1697 -174 1736 -135 l 1736 174 b 1649 260 1736 222 1697 260 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1562 454 b 868 1080 1484 823 1186 1045 l 868 1128 b 781 1215 868 1176 829 1215 b 694 1128 733 1215 694 1176 l 694 1080 b 1 454 375 1045 80 823 b 52 423 -5 426 24 396 b 482 322 193 572 344 565 b 535 322 496 296 522 299 b 781 521 590 418 657 521 b 1027 322 940 521 1021 334 b 1081 322 1040 299 1066 296 b 1511 423 1219 565 1371 570 b 1562 454 1539 396 1568 426 z m 694 397 l 694 43 b 651 0 694 20 675 0 b 610 29 630 0 615 14 b 499 82 594 74 545 98 b 447 -29 454 66 431 16 b 651 -173 477 -115 559 -173 b 868 44 771 -173 868 -76 l 868 397 b 781 434 843 419 814 434 b 694 397 748 434 720 421 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 347 716 b 499 868 347 800 415 868 l 868 868 l 868 1063 b 803 1128 868 1099 839 1128 l 584 1128 b 434 1215 554 1180 498 1215 b 284 1128 370 1215 314 1180 l 65 1128 b 0 1063 29 1128 0 1099 l 0 152 b 65 87 0 116 29 87 l 347 87 z m 434 1107 b 499 1042 470 1107 499 1078 b 434 977 499 1006 470 977 b 369 1042 398 977 369 1006 b 434 1107 369 1078 398 1107 z m 933 434 b 868 499 897 434 868 463 l 868 781 l 499 781 b 434 716 463 781 434 752 l 434 -109 b 499 -174 434 -144 463 -174 l 1150 -174 b 1215 -109 1186 -174 1215 -144 l 1215 434 z m 1215 537 b 1196 583 1215 552 1207 573 l 1017 762 b 971 781 1007 773 986 781 l 955 781 l 955 521 l 1215 521 z "
	},
	"": {
		ha: 955,
		x_min: 0,
		x_max: 0,
		o: "m 261 -17 b 275 -65 261 -34 266 -51 l 321 -135 b 394 -174 336 -156 368 -174 l 561 -174 b 633 -135 587 -174 619 -156 l 680 -65 b 694 -17 688 -53 694 -32 l 694 87 l 260 87 z m 0 738 b 118 424 0 617 45 508 b 260 176 163 373 233 266 b 260 174 260 175 260 174 l 695 174 b 695 176 695 174 695 175 b 837 424 722 266 792 373 b 955 738 910 508 955 617 b 476 1215 955 1002 740 1216 b 0 738 199 1214 0 990 z m 477 955 b 260 738 358 955 260 858 b 217 694 260 714 241 694 b 174 738 193 694 174 714 b 477 1042 174 905 310 1042 b 521 998 501 1042 521 1022 b 477 955 521 974 501 955 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 608 521 b 955 868 799 521 955 676 b 608 1215 955 1060 799 1215 b 260 868 416 1215 260 1060 b 608 521 260 676 416 521 z m 282 65 b 347 0 282 29 311 0 b 412 65 383 0 412 29 b 347 130 412 101 383 130 b 282 65 311 130 282 101 z m 868 432 l 868 299 b 1042 87 967 279 1042 192 l 1042 -26 b 1007 -69 1042 -47 1027 -65 l 919 -86 b 894 -69 908 -89 896 -81 l 885 -27 b 903 -1 883 -15 891 -3 l 955 9 l 955 87 b 694 82 955 257 694 263 l 694 9 l 747 -1 b 764 -27 758 -4 766 -15 l 755 -69 b 730 -87 753 -81 742 -89 l 645 -75 b 608 -32 624 -72 608 -54 l 608 87 b 782 299 608 192 683 279 l 782 422 b 764 417 776 420 770 419 b 608 390 715 400 662 390 b 452 417 553 390 501 400 b 391 431 432 424 412 428 l 391 210 b 499 65 454 191 499 133 b 347 -87 499 -19 431 -87 b 196 65 264 -87 196 -19 b 304 210 196 133 241 191 l 304 428 b 0 69 132 399 0 250 l 0 -52 b 122 -174 0 -119 55 -174 l 1094 -174 b 1215 -52 1161 -174 1215 -119 l 1215 69 b 868 432 1215 265 1061 423 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1213 911 b 1042 742 1120 910 1044 834 b 1129 588 1041 676 1076 618 l 1129 282 b 825 0 1129 127 993 0 b 521 269 662 0 529 120 b 868 694 719 310 868 485 l 868 1116 b 816 1180 868 1147 846 1174 l 645 1214 b 569 1163 610 1221 576 1198 l 560 1120 b 611 1044 553 1085 576 1051 l 694 1027 l 694 698 b 436 434 694 554 580 436 b 173 695 291 433 173 550 l 173 1027 l 257 1044 b 308 1120 292 1051 315 1085 l 299 1163 b 223 1214 292 1198 258 1221 l 52 1180 b 0 1116 22 1174 0 1147 l 0 694 b 347 269 0 485 149 309 b 825 -174 355 24 566 -174 b 1302 282 1088 -174 1302 31 l 1302 588 b 1389 738 1354 618 1389 674 b 1213 911 1389 835 1310 913 z m 1215 694 b 1172 738 1191 694 1172 714 b 1215 781 1172 762 1191 781 b 1259 738 1239 781 1259 762 b 1215 694 1259 714 1239 694 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 347 -87 l 1042 -87 l 1042 998 b 911 1128 1042 1070 983 1128 l 477 1128 b 347 998 406 1128 347 1070 z m 521 955 l 868 955 l 868 868 l 521 868 z m 1389 738 b 1259 868 1389 810 1331 868 l 1128 868 l 1128 -87 l 1259 -87 b 1389 43 1331 -87 1389 -28 z m 260 -87 l 260 868 l 130 868 b 0 738 58 868 0 810 l 0 43 b 130 -87 0 -28 58 -87 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 608 -174 b 781 0 703 -174 781 -96 l 434 0 b 608 -174 434 -96 512 -174 z m 1192 233 b 1041 651 1140 289 1041 374 b 694 1072 1041 862 894 1031 l 694 1128 b 608 1215 694 1176 656 1215 b 521 1128 560 1215 521 1176 l 521 1072 b 174 651 322 1031 174 862 b 23 233 174 374 76 289 b 0 174 7 215 0 194 b 87 87 0 129 35 87 l 1128 87 b 1215 174 1180 87 1215 129 b 1192 233 1215 194 1208 215 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 521 174 l 1042 174 b 1302 434 1185 174 1302 290 l 1389 434 b 1736 781 1580 434 1736 590 b 1389 1128 1736 973 1580 1128 l 326 1128 b 260 1063 289 1128 260 1099 l 260 434 b 521 174 260 290 377 174 z m 1389 955 b 1563 781 1485 955 1563 877 b 1389 608 1563 685 1485 608 l 1302 608 l 1302 955 z m 1518 -87 b 1616 87 1648 -87 1683 87 l 33 87 b 131 -87 -34 87 2 -87 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1215 -119 b 1183 -87 1215 -101 1201 -87 l 1128 -87 l 1128 890 b 1063 955 1128 926 1099 955 l 825 955 l 825 1150 b 760 1215 825 1186 796 1215 l 456 1215 b 391 1150 420 1215 391 1186 l 391 955 l 152 955 b 87 890 116 955 87 926 l 87 -87 l 33 -87 b 0 -119 15 -87 0 -101 l 0 -174 l 1215 -174 z m 836 694 b 868 662 853 694 868 680 l 868 553 b 836 521 868 535 853 521 l 727 521 b 694 553 709 521 694 535 l 694 662 b 727 694 694 680 709 694 z m 380 521 b 347 553 362 521 347 535 l 347 662 b 380 694 347 680 362 694 l 488 694 b 521 662 506 694 521 680 l 521 553 b 488 521 521 535 506 521 z m 662 174 b 694 141 680 174 694 159 l 694 -87 l 521 -87 l 521 141 b 553 174 521 159 535 174 z m 836 434 b 868 401 853 434 868 419 l 868 293 b 836 260 868 275 853 260 l 727 260 b 694 293 709 260 694 275 l 694 401 b 727 434 694 419 709 434 z m 521 401 l 521 293 b 488 260 521 275 506 260 l 380 260 b 347 293 362 260 347 275 l 347 401 b 380 434 347 419 362 434 l 488 434 b 521 401 506 434 521 419 z m 494 955 b 477 971 485 955 477 962 l 477 1025 b 494 1042 477 1034 485 1042 l 564 1042 l 564 1112 b 581 1128 564 1121 572 1128 l 635 1128 b 651 1112 644 1128 651 1121 l 651 1042 l 722 1042 b 738 1025 731 1042 738 1034 l 738 971 b 722 955 738 962 731 955 l 651 955 l 651 884 b 635 868 651 875 644 868 l 581 868 b 564 884 572 868 564 875 l 564 955 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1693 260 l 1649 260 l 1649 554 b 1611 646 1649 588 1635 621 l 1340 917 b 1248 955 1316 941 1283 955 l 1128 955 l 1128 1085 b 998 1215 1128 1157 1070 1215 l 130 1215 b 0 1085 58 1215 0 1157 l 0 217 b 130 87 0 145 58 87 l 174 87 b 434 -174 174 -57 290 -174 b 694 87 578 -174 694 -57 l 1042 87 b 1302 -174 1042 -57 1158 -174 b 1563 87 1446 -174 1563 -57 l 1693 87 b 1736 130 1717 87 1736 106 l 1736 217 b 1693 260 1736 241 1717 260 z m 434 -43 b 304 87 362 -43 304 15 b 434 217 304 159 362 217 b 564 87 506 217 564 159 b 434 -43 564 15 506 -43 z m 825 629 b 803 608 825 617 815 608 l 651 608 l 651 456 b 629 434 651 444 641 434 l 499 434 b 477 456 487 434 477 444 l 477 608 l 326 608 b 304 629 314 608 304 617 l 304 760 b 326 781 304 771 314 781 l 477 781 l 477 933 b 499 955 477 945 487 955 l 629 955 b 651 933 641 955 651 945 l 651 781 l 803 781 b 825 760 815 781 825 771 z m 1302 -43 b 1172 87 1230 -43 1172 15 b 1302 217 1172 159 1230 217 b 1432 87 1374 217 1432 159 b 1302 -43 1432 15 1374 -43 z m 1519 521 l 1128 521 l 1128 825 l 1248 825 l 1519 554 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 260 -87 l 1128 -87 l 1128 868 l 1042 868 l 1042 998 b 911 1128 1042 1070 983 1128 l 477 1128 b 347 998 406 1128 347 1070 l 347 868 l 260 868 z m 521 955 l 868 955 l 868 868 l 521 868 z m 1389 738 b 1259 868 1389 810 1331 868 l 1215 868 l 1215 -87 l 1259 -87 b 1389 43 1331 -87 1389 -29 z m 174 -87 l 174 868 l 130 868 b 0 738 58 868 0 810 l 0 43 b 130 -87 0 -29 58 -87 z m 955 477 l 955 391 b 911 347 955 367 935 347 l 781 347 l 781 217 b 738 174 781 193 762 174 l 651 174 b 608 217 627 174 608 193 l 608 347 l 477 347 b 434 391 453 347 434 367 l 434 477 b 477 521 434 501 453 521 l 608 521 l 608 651 b 651 694 608 675 627 694 l 738 694 b 781 651 762 694 781 675 l 781 521 l 911 521 b 955 477 935 521 955 501 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1476 608 l 1128 651 l 998 694 l 933 694 l 616 1096 l 723 1096 b 781 1112 755 1096 781 1103 b 723 1128 781 1121 755 1128 l 412 1128 l 412 1096 l 456 1096 l 456 651 l 326 651 l 145 868 l 51 868 l 22 839 l 22 651 l 43 651 l 43 608 l 174 608 l 174 600 l 0 579 l 0 463 l 174 441 l 174 434 l 43 434 l 43 391 l 22 391 l 22 203 l 51 174 l 145 174 l 326 391 l 456 391 l 456 -54 l 412 -54 l 412 -87 l 723 -87 b 781 -71 755 -87 781 -80 b 723 -54 781 -61 755 -54 l 616 -54 l 933 347 l 998 347 l 1128 391 l 1476 434 b 1736 521 1736 492 1736 506 b 1476 608 1736 536 1736 550 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 998 955 l 868 955 l 868 1063 b 803 1128 868 1099 839 1128 l 65 1128 b 0 1063 29 1128 0 1099 l 0 -22 b 65 -87 0 -58 29 -87 l 803 -87 b 868 -22 839 -87 868 -58 l 868 93 l 1087 190 b 1215 388 1165 225 1215 303 l 1215 738 b 998 955 1215 858 1118 955 z m 1042 388 b 1016 349 1042 373 1030 355 l 868 283 l 868 781 l 998 781 b 1042 738 1022 781 1042 762 z m 564 174 b 521 217 540 174 521 193 l 521 825 b 564 868 521 849 540 868 b 608 825 588 868 608 849 l 608 217 b 564 174 608 193 588 174 z m 304 174 b 260 217 280 174 260 193 l 260 825 b 304 868 260 849 280 868 b 347 825 328 868 347 849 l 347 217 b 304 174 347 193 328 174 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1215 998 b 1085 1128 1215 1070 1157 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -29 58 -87 l 1085 -87 b 1215 43 1157 -87 1215 -29 z m 911 868 b 955 825 935 868 955 849 l 955 217 b 911 174 955 193 935 174 l 825 174 b 781 217 801 174 781 193 l 781 434 l 434 434 l 434 217 b 391 174 434 193 415 174 l 304 174 b 260 217 280 174 260 193 l 260 825 b 304 868 260 849 280 868 l 391 868 b 434 825 415 868 434 849 l 434 608 l 781 608 l 781 825 b 825 868 781 849 801 868 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1085 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -28 58 -87 l 1085 -87 b 1215 43 1157 -87 1215 -28 l 1215 998 b 1085 1128 1215 1070 1157 1128 z m 998 445 b 966 412 998 427 984 412 l 716 412 l 716 163 b 684 130 716 145 701 130 l 532 130 b 499 163 514 130 499 145 l 499 412 l 250 412 b 217 445 232 412 217 427 l 217 597 b 250 629 217 615 232 629 l 499 629 l 499 879 b 532 911 499 897 514 911 l 684 911 b 716 879 701 911 716 897 l 716 629 l 966 629 b 998 597 984 629 998 615 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 607 567 b 607 475 581 541 581 500 l 976 106 b 1068 106 1001 80 1042 80 l 1129 167 b 1129 259 1155 193 1155 234 l 868 521 l 1129 783 b 1129 875 1155 808 1155 849 l 1068 936 b 976 936 1042 961 1001 961 z m 86 475 l 455 106 b 547 106 480 80 522 80 l 608 167 b 608 259 634 193 634 234 l 347 521 l 608 782 b 608 874 634 808 634 849 l 547 936 b 455 936 522 961 480 961 l 86 567 b 86 475 60 541 60 500 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 608 475 b 608 567 634 500 634 541 l 239 936 b 147 936 214 961 173 961 l 86 874 b 86 782 61 849 61 808 l 348 521 l 86 259 b 86 167 61 234 61 193 l 148 106 b 240 106 173 80 214 80 z m 1129 567 l 760 936 b 668 936 735 961 694 961 l 607 875 b 607 783 582 849 582 808 l 869 521 l 607 260 b 607 168 582 234 582 193 l 668 106 b 760 106 694 81 735 81 l 1129 475 b 1129 567 1155 500 1155 541 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 480 522 b 388 521 455 547 413 547 l 19 152 b 19 60 -7 127 -7 86 l 81 -1 b 173 -1 106 -26 147 -26 l 434 261 l 696 -1 b 788 -1 721 -26 762 -26 l 849 61 b 849 153 875 86 875 127 z m 388 1042 l 19 674 b 19 582 -7 648 -7 607 l 80 520 b 172 520 106 495 147 495 l 434 782 l 695 520 b 787 520 721 495 762 495 l 849 582 b 849 674 874 607 874 648 l 480 1042 b 388 1042 455 1068 413 1068 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 388 520 b 480 520 413 495 455 495 l 849 889 b 849 981 875 914 875 956 l 787 1042 b 695 1042 762 1068 721 1068 l 434 781 l 172 1042 b 80 1042 147 1068 106 1068 l 19 981 b 19 889 -7 956 -7 914 z m 480 -1 l 849 368 b 849 460 875 394 875 435 l 788 521 b 696 521 762 547 721 547 l 434 260 l 173 522 b 81 522 147 547 106 547 l 19 460 b 19 368 -7 435 -7 394 l 388 -1 b 480 -1 413 -26 455 -26 z "
	},
	"": {
		ha: 694,
		x_min: 0,
		x_max: 0,
		o: "m 86 567 b 86 475 60 541 60 500 l 455 106 b 547 106 480 80 522 80 l 608 167 b 608 259 634 193 634 234 l 347 521 l 608 783 b 608 875 634 808 634 849 l 547 936 b 455 936 522 961 480 961 z "
	},
	"": {
		ha: 694,
		x_min: 0,
		x_max: 0,
		o: "m 608 475 b 608 567 634 500 634 541 l 239 936 b 147 936 214 961 173 961 l 86 874 b 86 782 61 849 61 808 l 348 521 l 86 259 b 86 167 61 234 61 193 l 148 106 b 240 106 173 80 214 80 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 480 782 b 388 782 455 808 413 808 l 19 413 b 19 321 -7 388 -7 346 l 81 260 b 173 260 106 234 147 234 l 434 521 l 696 260 b 788 260 721 234 762 234 l 849 321 b 849 413 875 346 875 388 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 388 260 b 480 260 413 234 455 234 l 849 629 b 849 720 874 654 874 695 l 787 782 b 695 782 762 807 721 807 l 434 520 l 172 782 b 80 782 147 807 106 807 l 19 720 b 19 629 -7 695 -7 654 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1432 1215 l 130 1215 b 0 1085 58 1215 0 1157 l 0 217 b 130 87 0 145 58 87 l 651 87 l 608 -43 l 412 -43 b 347 -109 376 -43 347 -72 b 412 -174 347 -145 376 -174 l 1150 -174 b 1215 -109 1186 -174 1215 -145 b 1150 -43 1215 -72 1186 -43 l 955 -43 l 911 87 l 1432 87 b 1563 217 1504 87 1563 145 l 1563 1085 b 1432 1215 1563 1157 1504 1215 z m 1389 260 l 174 260 l 174 1042 l 1389 1042 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1693 87 l 1035 87 b 946 0 1033 33 995 0 l 781 0 b 692 87 731 0 692 47 l 43 87 b 0 43 20 87 0 67 l 0 0 b 174 -174 0 -95 78 -174 l 1563 -174 b 1736 0 1658 -174 1736 -95 l 1736 43 b 1693 87 1736 67 1717 87 z m 1563 1085 b 1432 1215 1563 1157 1504 1215 l 304 1215 b 174 1085 232 1215 174 1157 l 174 174 l 1563 174 z m 1389 347 l 347 347 l 347 1042 l 1389 1042 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1085 1215 l 130 1215 b 0 1085 58 1215 0 1157 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 1085 b 1085 1215 1215 1157 1157 1215 z m 608 -87 b 521 0 560 -87 521 -48 b 608 87 521 48 560 87 b 694 0 656 87 694 48 b 608 -87 694 -48 656 -87 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 738 1215 l 130 1215 b 0 1085 58 1215 0 1157 l 0 -43 b 130 -174 0 -115 58 -174 l 738 -174 b 868 -43 810 -174 868 -115 l 868 1085 b 738 1215 868 1157 810 1215 z m 434 -87 b 347 0 386 -87 347 -48 b 434 87 347 48 386 87 b 521 0 482 87 521 48 b 434 -87 521 -48 482 -87 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1259 521 l 1042 521 l 1042 694 b 1215 868 1042 790 1120 868 l 1237 868 b 1302 933 1273 868 1302 897 l 1302 1063 b 1237 1128 1302 1099 1273 1128 l 1215 1128 b 781 694 975 1128 781 934 l 781 43 b 911 -87 781 -28 840 -87 l 1259 -87 b 1389 43 1331 -87 1389 -28 l 1389 391 b 1259 521 1389 463 1331 521 z m 477 521 l 260 521 l 260 694 b 434 868 260 790 338 868 l 456 868 b 521 933 492 868 521 897 l 521 1063 b 456 1128 521 1099 492 1128 l 434 1128 b 0 694 194 1128 0 934 l 0 43 b 130 -87 0 -28 58 -87 l 477 -87 b 608 43 549 -87 608 -28 l 608 391 b 477 521 608 463 549 521 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1259 1128 l 911 1128 b 781 998 840 1128 781 1070 l 781 651 b 911 521 781 579 840 521 l 1128 521 l 1128 347 b 955 174 1128 251 1051 174 l 933 174 b 868 109 897 174 868 145 l 868 -22 b 933 -87 868 -58 897 -87 l 955 -87 b 1389 347 1195 -87 1389 107 l 1389 998 b 1259 1128 1389 1070 1331 1128 z m 477 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 651 b 130 521 0 579 58 521 l 347 521 l 347 347 b 174 174 347 251 269 174 l 152 174 b 87 109 116 174 87 145 l 87 -22 b 152 -87 87 -58 116 -87 l 174 -87 b 608 347 413 -87 608 107 l 608 998 b 477 1128 608 1070 549 1128 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 825 1085 b 694 1215 825 1157 766 1215 b 564 1085 623 1215 564 1157 b 694 955 564 1013 623 955 b 825 1085 766 955 825 1013 z m 694 87 b 564 -43 623 87 564 29 b 694 -174 564 -115 623 -174 b 825 -43 766 -174 825 -115 b 694 87 825 29 766 87 z m 1259 651 b 1128 521 1187 651 1128 593 b 1259 391 1128 449 1187 391 b 1389 521 1331 391 1389 449 b 1259 651 1389 593 1331 651 z m 260 521 b 130 651 260 593 202 651 b 0 521 58 651 0 593 b 130 391 0 449 58 391 b 260 521 202 391 260 449 z m 295 252 b 165 122 224 252 165 194 b 295 -8 165 50 224 -8 b 426 122 367 -8 426 50 b 295 252 426 194 367 252 z m 1093 252 b 963 122 1022 252 963 194 b 1093 -8 963 50 1022 -8 b 1224 122 1165 -8 1224 50 b 1093 252 1224 194 1165 252 z m 295 1050 b 165 920 224 1050 165 992 b 295 790 165 848 224 790 b 426 920 367 790 426 848 b 295 1050 426 992 367 1050 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 890 738 b 977 651 938 738 977 699 b 890 564 977 603 938 564 b 803 651 842 564 803 603 b 890 738 803 699 842 738 z m 456 738 b 543 651 504 738 543 699 b 456 564 543 603 504 564 b 369 651 408 564 369 603 b 456 738 369 699 408 738 z m 984 276 b 673 130 907 183 793 130 b 361 276 552 130 439 183 b 428 332 324 320 391 376 b 673 217 489 259 578 217 b 917 332 768 217 857 259 b 984 276 954 376 1021 320 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 890 738 b 977 651 938 738 977 699 b 890 564 977 603 938 564 b 803 651 842 564 803 603 b 890 738 803 699 842 738 z m 456 738 b 543 651 504 738 543 699 b 456 564 543 603 504 564 b 369 651 408 564 369 603 b 456 738 369 699 408 738 z m 917 146 b 673 260 857 219 767 260 b 428 146 578 260 489 219 b 361 202 391 102 325 157 b 673 347 439 294 552 347 b 984 201 793 347 907 294 b 917 146 1021 157 954 102 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 456 738 b 543 651 504 738 543 699 b 456 564 543 603 504 564 b 369 651 408 564 369 603 b 456 738 369 699 408 738 z m 933 217 l 412 217 b 412 304 355 217 355 304 l 933 304 b 933 217 991 304 991 217 z m 890 564 b 803 651 842 564 803 603 b 890 738 803 699 842 738 b 977 651 938 738 977 699 b 890 564 977 603 938 564 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1302 955 l 434 955 b 0 521 194 955 0 761 b 434 87 0 281 194 87 b 744 217 556 87 665 137 l 992 217 b 1302 87 1071 137 1181 87 b 1736 521 1542 87 1736 281 b 1302 955 1736 761 1542 955 z m 694 467 b 662 434 694 449 680 434 l 521 434 l 521 293 b 488 260 521 275 506 260 l 380 260 b 347 293 362 260 347 275 l 347 434 l 206 434 b 174 467 188 434 174 449 l 174 575 b 206 608 174 593 188 608 l 347 608 l 347 749 b 380 781 347 767 362 781 l 488 781 b 521 749 506 781 521 767 l 521 608 l 662 608 b 694 575 680 608 694 593 z m 1194 282 b 1063 412 1122 282 1063 340 b 1194 543 1063 484 1122 543 b 1324 412 1265 543 1324 484 b 1194 282 1324 340 1265 282 z m 1411 499 b 1280 629 1339 499 1280 557 b 1411 760 1280 701 1339 760 b 1541 629 1482 760 1541 701 b 1411 499 1541 557 1482 499 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1432 0 b 1563 130 1504 0 1563 58 l 1563 911 b 1432 1042 1563 983 1504 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 130 b 130 0 0 58 58 0 z m 347 727 b 315 694 347 709 333 694 l 206 694 b 174 727 188 694 174 709 l 174 836 b 206 868 174 853 188 868 l 315 868 b 347 836 333 868 347 853 z m 608 727 b 575 694 608 709 593 694 l 467 694 b 434 727 449 694 434 709 l 434 836 b 467 868 434 853 449 868 l 575 868 b 608 836 593 868 608 853 z m 868 727 b 836 694 868 709 853 694 l 727 694 b 694 727 709 694 694 709 l 694 836 b 727 868 694 853 709 868 l 836 868 b 868 836 853 868 868 853 z m 1128 727 b 1096 694 1128 709 1114 694 l 987 694 b 955 727 969 694 955 709 l 955 836 b 987 868 955 853 969 868 l 1096 868 b 1128 836 1114 868 1128 853 z m 1389 727 b 1356 694 1389 709 1374 694 l 1248 694 b 1215 727 1230 694 1215 709 l 1215 836 b 1248 868 1215 853 1230 868 l 1356 868 b 1389 836 1374 868 1389 853 z m 477 467 b 445 434 477 449 463 434 l 336 434 b 304 467 318 434 304 449 l 304 575 b 336 608 304 593 318 608 l 445 608 b 477 575 463 608 477 593 z m 738 467 b 705 434 738 449 723 434 l 597 434 b 564 467 579 434 564 449 l 564 575 b 597 608 564 593 579 608 l 705 608 b 738 575 723 608 738 593 z m 998 467 b 966 434 998 449 984 434 l 857 434 b 825 467 839 434 825 449 l 825 575 b 857 608 825 593 839 608 l 966 608 b 998 575 984 608 998 593 z m 1259 467 b 1226 434 1259 449 1244 434 l 1118 434 b 1085 467 1100 434 1085 449 l 1085 575 b 1118 608 1085 593 1100 608 l 1226 608 b 1259 575 1244 608 1259 593 z m 347 206 b 315 174 347 188 333 174 l 206 174 b 174 206 188 174 174 188 l 174 315 b 206 347 174 333 188 347 l 315 347 b 347 315 333 347 347 333 z m 1128 206 b 1096 174 1128 188 1114 174 l 467 174 b 434 206 449 174 434 188 l 434 315 b 467 347 434 333 449 347 l 1096 347 b 1128 315 1114 347 1128 333 z m 1389 206 b 1356 174 1389 188 1374 174 l 1248 174 b 1215 206 1230 174 1215 188 l 1215 315 b 1248 347 1215 333 1230 347 l 1356 347 b 1389 315 1374 347 1389 333 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 660 700 l 660 515 b 859 455 731 499 793 473 l 859 640 b 660 700 789 656 725 682 z m 1265 1034 b 948 947 1172 991 1058 947 b 500 1042 803 947 683 1042 b 315 1009 432 1042 372 1030 b 325 1073 323 1029 327 1050 b 180 1215 320 1150 257 1212 b 22 1063 93 1219 22 1149 b 87 939 22 1012 47 966 l 87 -109 b 152 -174 87 -145 116 -174 l 195 -174 b 260 -109 231 -174 260 -145 l 260 148 b 571 208 337 180 433 208 b 1019 113 716 208 836 113 b 1351 224 1150 113 1254 157 b 1389 296 1375 240 1389 267 l 1389 955 b 1265 1034 1389 1018 1323 1060 z m 460 332 b 260 287 390 325 324 310 l 260 479 b 460 526 331 504 389 519 z m 1259 697 b 1059 632 1195 671 1133 644 l 1059 439 b 1259 510 1126 449 1198 471 l 1259 319 b 1059 245 1191 275 1127 252 l 1059 439 b 859 455 986 429 929 435 l 859 272 b 660 330 795 292 731 317 l 660 515 b 460 526 606 527 549 534 l 460 716 b 260 659 399 707 339 688 l 260 850 b 460 910 350 883 396 904 l 460 716 b 660 700 533 726 591 719 l 660 883 b 859 825 724 863 788 838 l 859 640 b 1059 632 924 625 988 621 l 1059 827 b 1259 888 1132 840 1201 864 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 700 475 b 700 567 725 500 725 541 l 173 1094 b 81 1094 147 1119 106 1119 l 19 1033 b 19 941 -6 1007 -6 966 l 437 521 l 19 101 b 19 9 -6 76 -6 34 l 81 -52 b 173 -52 106 -78 147 -78 z m 1736 -22 l 1736 65 b 1671 130 1736 101 1707 130 l 846 130 b 781 65 810 130 781 101 l 781 -22 b 846 -87 781 -58 810 -87 l 1671 -87 b 1736 -22 1707 -87 1736 -58 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 757 -172 b 797 -150 774 -177 792 -167 l 1167 1125 b 1145 1166 1172 1143 1162 1161 l 980 1214 b 939 1192 962 1219 944 1209 l 569 -84 b 591 -124 564 -101 574 -119 z m 447 132 l 565 258 b 563 305 578 271 577 292 l 317 521 l 563 737 b 565 784 577 749 578 770 l 447 910 b 401 911 435 923 414 923 l 10 545 b 10 497 -4 532 -4 510 l 401 131 b 447 132 414 118 435 119 z m 1335 130 l 1726 497 b 1726 544 1740 510 1740 532 l 1335 911 b 1289 910 1322 923 1301 923 l 1171 784 b 1173 737 1158 771 1159 750 l 1419 521 l 1173 305 b 1171 258 1159 292 1158 271 l 1289 132 b 1335 130 1301 119 1322 118 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 370 700 b 370 602 340 674 340 628 l 847 189 b 955 239 889 153 955 183 l 955 467 b 1260 -27 1250 433 1365 307 b 1336 -77 1247 -70 1299 -104 b 1563 341 1455 9 1563 174 b 955 839 1563 697 1305 811 l 955 1063 b 847 1113 955 1119 889 1149 z m 23 602 l 500 189 b 608 239 542 153 608 183 l 608 282 l 313 536 b 260 651 284 561 260 613 l 260 651 l 260 651 b 313 766 260 689 284 741 l 608 1020 l 608 1063 b 500 1113 608 1119 542 1149 l 23 700 b 23 602 -8 674 -7 628 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1206 1206 l 78 685 b 130 434 -52 624 -9 434 l 607 434 l 607 -44 b 859 -96 607 -182 798 -226 l 1379 1032 b 1206 1206 1423 1136 1310 1249 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1324 260 l 1215 260 l 1215 919 l 1376 1080 b 1376 1141 1393 1097 1393 1124 l 1315 1203 b 1253 1203 1298 1220 1270 1220 l 1093 1042 l 521 1042 l 521 781 l 832 781 l 434 383 l 434 1150 b 369 1215 434 1186 405 1215 l 239 1215 b 174 1150 203 1215 174 1186 l 174 1042 l 65 1042 b 0 977 29 1042 0 1013 l 0 846 b 65 781 0 810 29 781 l 174 781 l 174 65 b 239 0 174 29 203 0 l 868 0 l 868 260 l 557 260 l 955 659 l 955 -109 b 1020 -174 955 -144 984 -174 l 1150 -174 b 1215 -109 1186 -174 1215 -144 l 1215 0 l 1324 0 b 1389 65 1360 0 1389 29 l 1389 195 b 1324 260 1389 231 1360 260 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 1042 825 b 825 1042 1042 945 945 1042 b 608 825 705 1042 608 945 b 764 616 608 726 674 643 b 734 516 762 573 752 539 b 503 447 692 464 600 455 b 282 401 426 439 347 432 l 282 791 b 434 998 370 819 434 901 b 217 1215 434 1118 337 1215 b 0 998 97 1215 0 1118 b 152 791 0 901 64 819 l 152 251 b 0 43 64 223 0 141 b 217 -174 0 -76 97 -174 b 434 43 337 -174 434 -76 b 295 246 434 136 377 215 b 336 282 304 260 316 272 b 515 317 380 304 445 310 b 836 435 629 327 759 340 b 894 619 873 482 893 543 b 1042 825 980 648 1042 729 z m 217 1042 b 260 998 241 1042 260 1022 b 217 955 260 974 241 955 b 174 998 193 955 174 974 b 217 1042 174 1022 193 1042 z m 217 0 b 174 43 193 0 174 20 b 217 87 174 67 193 87 b 260 43 241 87 260 67 b 217 0 260 20 241 0 z m 825 868 b 868 825 849 868 868 849 b 825 781 868 801 849 781 b 781 825 801 781 781 801 b 825 868 781 849 801 868 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 825 114 l 717 222 b 671 222 705 234 684 234 l 550 100 b 274 100 474 24 350 24 b 274 377 198 177 198 300 l 395 498 b 395 544 408 511 408 531 l 288 651 b 242 651 275 664 254 664 l 121 530 b 121 -53 -40 369 -40 108 b 704 -53 281 -214 543 -214 l 825 68 b 825 114 838 81 838 101 z m 671 820 b 717 820 684 807 705 807 l 839 941 b 1115 941 915 1017 1039 1017 b 1115 665 1191 865 1191 741 l 994 544 b 994 498 981 531 981 511 l 1101 390 b 1147 390 1114 378 1134 378 l 1268 512 b 1268 1095 1429 672 1429 934 b 685 1095 1108 1255 846 1255 l 564 974 b 564 927 551 961 551 940 z m 1308 -155 l 1370 -93 b 1370 -1 1395 -68 1395 -27 l 173 1196 b 80 1196 147 1222 106 1222 l 19 1135 b 19 1043 -6 1109 -6 1068 l 1216 -155 b 1308 -155 1242 -180 1283 -180 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 548 1215 b 81 968 331 1215 191 1127 b 95 879 61 940 67 900 l 212 790 b 302 802 240 769 280 774 b 527 936 370 887 421 936 b 714 801 610 936 714 882 b 581 662 714 740 663 709 b 357 374 485 609 357 541 l 357 347 b 423 282 357 311 387 282 l 619 282 b 684 347 655 282 684 311 l 684 363 b 1024 799 684 479 1024 484 b 548 1215 1024 1036 778 1215 z m 521 202 b 333 14 417 202 333 118 b 521 -174 333 -89 417 -174 b 709 14 624 -174 709 -89 b 521 202 709 118 624 202 z "
	},
	"": {
		ha: 521,
		x_min: 0,
		x_max: 0,
		o: "m 54 64 b 0 10 24 64 0 40 l 0 -119 b 54 -174 0 -149 24 -174 l 467 -174 b 521 -119 497 -174 521 -149 l 521 10 b 467 64 521 40 497 64 l 412 64 l 412 640 b 358 694 412 670 388 694 l 54 694 b 0 640 24 694 0 670 l 0 511 b 54 456 0 481 24 456 l 109 456 l 109 64 z m 260 1215 b 65 1020 153 1215 65 1128 b 260 825 65 912 153 825 b 456 1020 368 825 456 912 b 260 1215 456 1128 368 1215 z "
	},
	"": {
		ha: 521,
		x_min: 0,
		x_max: 0,
		o: "m 477 43 b 260 260 477 163 380 260 b 43 43 141 260 43 163 b 260 -174 43 -76 141 -174 b 477 43 380 -174 477 -76 z m 69 1147 l 105 409 b 170 347 107 374 136 347 l 350 347 b 415 409 385 347 414 374 l 452 1147 b 387 1215 454 1184 425 1215 l 134 1215 b 69 1147 96 1215 67 1184 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1345 781 l 1302 781 l 1302 1172 b 1259 1215 1302 1196 1283 1215 l 1128 1215 b 1090 1191 1114 1215 1096 1205 l 1046 1105 b 1042 1085 1044 1099 1042 1091 b 1085 1042 1042 1061 1061 1042 l 1128 1042 l 1128 781 l 1085 781 b 1042 738 1061 781 1042 762 l 1042 651 b 1085 608 1042 627 1061 608 l 1345 608 b 1389 651 1369 608 1389 627 l 1389 738 b 1345 781 1389 762 1369 781 z m 911 1042 l 730 1042 l 730 1042 b 694 1023 717 1042 701 1033 l 477 711 l 261 1023 b 225 1042 253 1033 238 1042 l 43 1042 b 0 998 19 1042 0 1022 l 0 868 b 43 825 0 844 19 825 l 134 825 l 345 521 l 134 217 l 43 217 b 0 174 19 217 0 198 l 0 43 b 43 0 0 19 19 0 l 225 0 l 225 0 b 261 19 238 0 254 8 l 478 331 l 694 19 b 730 0 701 8 717 0 l 911 0 b 955 43 935 0 955 19 l 955 174 b 911 217 955 198 935 217 l 821 217 l 610 521 l 821 825 l 911 825 b 955 868 935 825 955 844 l 955 998 b 911 1042 955 1022 935 1042 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1345 0 l 1302 0 l 1302 391 b 1259 434 1302 415 1283 434 l 1128 434 b 1090 410 1114 434 1096 423 l 1046 323 b 1042 304 1044 318 1042 309 b 1085 260 1042 280 1061 260 l 1128 260 l 1128 0 l 1085 0 b 1042 -43 1061 0 1042 -19 l 1042 -130 b 1085 -174 1042 -154 1061 -174 l 1345 -174 b 1389 -130 1369 -174 1389 -154 l 1389 -43 b 1345 0 1389 -19 1369 0 z m 911 1042 l 730 1042 l 730 1042 b 694 1023 717 1042 701 1033 l 477 711 l 261 1023 b 225 1042 253 1033 238 1042 l 43 1042 b 0 998 19 1042 0 1022 l 0 868 b 43 825 0 844 19 825 l 134 825 l 345 521 l 134 217 l 43 217 b 0 174 19 217 0 198 l 0 43 b 43 0 0 19 19 0 l 225 0 l 225 0 b 261 19 238 0 254 8 l 478 331 l 694 19 b 730 0 701 8 717 0 l 911 0 b 955 43 935 0 955 19 l 955 174 b 911 217 955 198 935 217 l 821 217 l 610 521 l 821 825 l 911 825 b 955 868 935 825 955 844 l 955 998 b 911 1042 955 1022 935 1042 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1351 472 b 1351 656 1402 523 1402 605 l 917 1090 b 733 1090 866 1141 783 1141 l 38 396 b 38 212 -13 345 -13 263 l 299 -49 b 391 -87 320 -70 361 -87 l 1356 -87 b 1389 -54 1374 -87 1389 -72 l 1389 54 b 1356 87 1389 72 1374 87 l 965 87 z m 530 642 l 902 269 l 720 87 l 409 87 l 192 304 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1409 432 b 1186 347 1296 432 1248 347 b 1172 825 1024 347 1172 825 b 639 834 1172 825 639 608 b 738 1067 639 931 738 959 b 571 1215 738 1163 662 1215 b 391 1062 477 1215 391 1164 b 477 841 391 950 477 902 b 0 763 477 652 0 763 l 0 -141 b 485 -63 0 -141 485 -252 b 376 158 485 -2 376 46 b 549 311 376 259 455 311 b 716 162 640 311 716 258 b 618 -71 716 54 618 26 b 1109 -141 618 -236 969 -141 b 1179 185 1109 -141 1015 185 b 1414 87 1277 185 1305 87 b 1563 254 1510 87 1563 163 b 1409 432 1563 347 1511 432 z "
	},
	"": {
		ha: 955,
		x_min: 0,
		x_max: 0,
		o: "m 477 260 b 738 521 621 260 738 377 l 738 955 b 477 1215 738 1099 621 1215 b 217 955 334 1215 217 1099 l 217 521 b 477 260 217 377 334 260 z m 911 694 l 868 694 b 825 651 844 694 825 675 l 825 521 b 443 175 825 318 650 155 b 130 536 262 193 130 355 l 130 651 b 87 694 130 675 111 694 l 43 694 b 0 651 19 694 0 675 l 0 542 b 412 49 0 299 174 82 l 412 -43 l 260 -43 b 217 -87 236 -43 217 -63 l 217 -130 b 260 -174 217 -154 236 -174 l 694 -174 b 738 -130 718 -174 738 -154 l 738 -87 b 694 -43 738 -63 718 -43 l 543 -43 l 543 48 b 955 521 775 80 955 280 l 955 651 b 911 694 955 675 935 694 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1719 -27 l 1291 303 b 1345 521 1325 369 1345 442 l 1345 651 b 1302 694 1345 675 1326 694 l 1259 694 b 1215 651 1235 694 1215 675 l 1215 521 b 1186 385 1215 472 1205 426 l 1114 440 b 1128 521 1123 466 1128 492 l 1128 955 b 868 1215 1128 1099 1012 1215 b 608 955 724 1215 608 1099 l 608 832 l 123 1206 b 62 1199 104 1221 77 1217 l 9 1130 b 17 1069 -6 1111 -2 1084 l 1613 -164 b 1674 -157 1632 -179 1659 -176 l 1727 -88 b 1719 -27 1742 -69 1738 -42 z m 1085 -43 l 933 -43 l 933 48 b 1024 71 965 53 995 61 l 889 176 b 833 175 870 175 852 173 b 532 452 682 190 566 307 l 391 561 l 391 542 b 803 49 391 299 564 82 l 803 -43 l 651 -43 b 608 -87 627 -43 608 -63 l 608 -130 b 651 -174 608 -154 627 -174 l 1085 -174 b 1128 -130 1109 -174 1128 -154 l 1128 -87 b 1085 -43 1128 -63 1109 -43 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 33 694 b 0 662 15 694 0 680 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 662 b 1183 694 1215 680 1201 694 z m 1215 814 l 1215 911 b 1085 1042 1215 983 1157 1042 l 955 1042 l 955 1183 b 922 1215 955 1201 940 1215 l 814 1215 b 781 1183 796 1215 781 1201 l 781 1042 l 434 1042 l 434 1183 b 401 1215 434 1201 419 1215 l 293 1215 b 260 1183 275 1215 260 1201 l 260 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 814 b 33 781 0 796 15 781 l 1183 781 b 1215 814 1201 781 1215 796 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1177 1144 l 722 1068 b 694 1020 691 1063 694 1031 l 536 1020 b 391 1215 565 1117 492 1215 b 249 1009 284 1215 211 1108 b 5 762 108 973 44 862 b 41 677 -9 729 8 691 b 126 714 74 664 112 680 b 456 890 204 909 290 890 l 456 816 b 260 561 343 787 260 683 l 260 -109 b 326 -174 260 -144 290 -174 l 716 -174 b 781 -109 752 -174 781 -144 l 781 564 b 586 816 781 686 698 788 l 586 890 l 694 890 b 722 842 694 882 690 847 l 1177 766 b 1215 798 1197 763 1215 778 l 1215 1112 b 1177 1144 1215 1132 1197 1147 z m 391 1020 b 347 1063 367 1020 347 1039 b 391 1107 347 1087 367 1107 b 434 1063 415 1107 434 1087 b 391 1020 434 1039 415 1020 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1370 1163 b 1337 1197 1367 1179 1352 1193 b 1113 1215 1250 1215 1181 1215 b 540 868 833 1215 665 1066 l 257 868 b 141 796 213 868 161 836 l 7 528 b 0 499 3 521 0 508 b 65 434 0 463 29 434 l 65 434 l 347 434 l 286 373 b 260 312 272 359 260 332 b 286 250 260 292 272 264 l 424 112 b 485 87 438 98 465 87 b 547 112 505 87 533 98 l 608 173 l 608 -109 l 608 -109 b 673 -174 608 -144 637 -174 b 702 -167 681 -174 694 -171 l 970 -33 b 1041 83 1009 -13 1041 39 l 1041 84 l 1041 367 b 1389 939 1238 492 1389 661 b 1370 1163 1389 1008 1389 1076 z m 1042 760 b 933 868 982 760 933 808 b 1042 977 933 928 982 977 b 1150 868 1102 977 1150 928 b 1042 760 1150 808 1102 760 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 z m 385 475 b 385 567 360 500 360 541 l 753 935 b 845 935 778 960 819 960 l 891 888 b 891 796 917 863 917 822 l 616 521 l 891 245 b 891 153 917 220 917 179 l 845 107 b 753 107 820 82 779 82 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 z m 1003 567 b 1003 475 1029 541 1029 500 l 636 107 b 544 107 611 82 569 82 l 498 153 b 498 245 472 178 472 220 l 773 521 l 498 796 b 498 888 472 822 472 863 l 544 935 b 636 935 569 960 610 960 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 22 521 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 b 22 521 323 1194 22 892 z m 648 830 b 741 830 674 855 715 855 l 1108 462 b 1108 370 1134 437 1134 396 l 1062 324 b 970 324 1037 299 996 299 l 694 600 l 419 324 b 327 324 394 299 352 299 l 281 370 b 281 462 255 396 255 437 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1367 521 b 694 1194 1367 892 1066 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 z m 741 212 b 648 212 715 186 674 186 l 281 579 b 281 671 255 605 255 646 l 327 718 b 419 718 352 743 393 743 l 694 442 l 970 718 b 1062 718 995 743 1037 743 l 1108 671 b 1108 579 1134 646 1134 605 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 35 260 l 123 260 b 781 -174 182 -18 494 -174 b 1439 260 1069 -174 1380 -18 l 1527 260 b 1550 316 1556 260 1571 295 l 1369 498 b 1322 498 1356 511 1335 511 l 1141 316 b 1164 260 1120 295 1135 260 l 1259 260 b 868 5 1204 113 1029 26 l 868 521 l 1009 521 b 1042 553 1027 521 1042 535 l 1042 662 b 1009 694 1042 680 1027 694 l 868 694 l 868 709 b 1042 955 969 745 1042 841 b 785 1215 1042 1097 927 1213 b 521 955 639 1217 521 1100 b 694 709 521 841 593 745 l 694 694 l 553 694 b 521 662 535 694 521 680 l 521 553 b 553 521 521 535 535 521 l 694 521 l 694 5 b 304 260 535 25 359 113 l 399 260 b 422 316 428 260 442 295 l 240 498 b 194 498 227 511 207 511 l 12 316 b 35 260 -8 295 6 260 z m 781 1042 b 868 955 829 1042 868 1003 b 781 868 868 907 829 868 b 694 955 733 868 694 907 b 781 1042 694 1003 733 1042 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1085 521 l 412 521 l 412 801 b 606 998 412 908 498 997 b 803 803 714 999 803 911 l 803 760 b 868 694 803 723 832 694 l 955 694 b 1020 760 991 694 1020 723 l 1020 803 b 606 1215 1020 1031 834 1216 b 195 799 378 1214 195 1027 l 195 521 l 130 521 b 0 391 58 521 0 463 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 391 b 1085 521 1215 463 1157 521 z m 716 109 b 608 0 716 49 668 0 b 499 109 548 0 499 49 l 499 239 b 608 347 499 299 548 347 b 716 239 668 347 716 299 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 673 22 b 174 521 397 22 174 245 b 673 1020 174 797 397 1020 b 1172 521 949 1020 1172 797 b 673 22 1172 245 949 22 z m 673 868 b 326 521 481 868 326 713 b 673 174 326 329 481 174 b 1020 521 865 174 1020 329 b 673 868 1020 713 865 868 z m 673 347 b 499 521 577 347 499 425 b 673 694 499 617 577 694 b 846 521 768 694 846 617 b 673 347 846 425 768 347 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 890 521 b 694 716 890 629 802 716 b 499 521 586 716 499 629 b 694 326 499 413 586 326 b 890 521 802 326 890 413 z m 1172 716 b 977 521 1064 716 977 629 b 1172 326 977 413 1064 326 b 1367 521 1280 326 1367 413 b 1172 716 1367 629 1280 716 z m 217 716 b 22 521 109 716 22 629 b 217 326 22 413 109 326 b 412 521 325 326 412 413 b 217 716 412 629 325 716 z "
	},
	"": {
		ha: 521,
		x_min: 0,
		x_max: 0,
		o: "m 260 716 b 65 521 152 716 65 629 b 260 326 65 413 152 326 b 456 521 368 326 456 413 b 260 716 456 629 368 716 z m 65 998 b 260 803 65 890 152 803 b 456 998 368 803 456 890 b 260 1194 456 1106 368 1194 b 65 998 152 1194 65 1106 z m 65 43 b 260 -152 65 -65 152 -152 b 456 43 368 -152 456 -65 b 260 239 456 151 368 239 b 65 43 152 239 65 151 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1085 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -29 58 -87 l 1085 -87 b 1215 43 1157 -87 1215 -29 l 1215 998 b 1085 1128 1215 1070 1157 1128 z m 304 87 b 174 217 232 87 174 145 b 304 347 174 289 232 347 b 434 217 376 347 434 289 b 304 87 434 145 376 87 z m 731 87 l 638 87 b 607 116 622 87 608 99 b 202 520 593 333 419 506 b 174 551 186 521 174 535 l 174 644 b 206 675 174 662 188 676 b 762 119 506 659 746 419 b 731 87 763 102 749 87 z m 1011 87 l 917 87 b 886 117 901 87 887 100 b 204 800 871 487 574 785 b 174 831 187 800 174 814 l 174 924 b 206 955 174 941 188 955 b 1042 119 661 938 1025 572 b 1011 87 1042 101 1028 87 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 z m 1008 456 l 531 182 b 434 239 488 158 434 189 l 434 803 b 531 860 434 853 488 884 l 1008 570 b 1008 456 1053 545 1053 480 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1085 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -28 58 -87 l 1085 -87 b 1215 43 1157 -87 1215 -28 l 1215 998 b 1085 1128 1215 1070 1157 1128 z m 250 412 b 217 445 232 412 217 427 l 217 597 b 250 629 217 615 232 629 l 966 629 b 998 597 984 629 998 615 l 998 445 b 966 412 998 427 984 412 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1085 -87 b 1215 43 1157 -87 1215 -29 l 1215 998 b 1085 1128 1215 1070 1157 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -29 58 -87 z m 530 179 b 468 179 513 162 485 162 l 186 461 b 186 523 169 478 169 506 l 248 584 b 309 584 265 601 292 601 l 499 394 l 906 801 b 968 801 923 818 951 818 l 1029 740 b 1029 678 1046 723 1046 695 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1085 -87 b 1215 43 1157 -87 1215 -28 l 1215 998 b 1085 1128 1215 1070 1157 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -28 58 -87 z m 646 733 b 669 732 652 739 662 739 l 820 582 b 820 559 826 575 826 565 l 452 191 l 297 174 b 261 210 276 171 259 189 l 278 365 z m 936 767 b 936 675 961 742 961 701 l 873 613 b 850 613 867 607 856 607 l 700 763 b 700 786 693 770 693 780 l 762 849 b 854 849 787 875 829 875 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1542 734 b 1542 829 1569 760 1569 803 l 1151 1197 b 1042 1150 1110 1236 1042 1208 l 1042 955 b 326 485 645 953 326 875 b 527 96 326 330 421 176 b 595 143 560 71 607 102 b 1042 608 484 512 649 605 l 1042 412 b 1151 365 1042 355 1110 326 z m 1042 187 l 1042 0 l 174 0 l 174 868 l 312 868 b 335 878 319 868 330 873 b 474 980 376 920 423 954 b 459 1042 504 996 493 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 197 b 1172 228 1215 220 1193 236 b 1107 217 1154 222 1125 217 b 1079 219 1099 217 1087 218 b 1042 187 1059 222 1042 207 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 611 582 b 611 459 577 548 577 493 b 734 459 645 426 700 426 b 734 582 768 493 768 548 b 611 582 700 616 645 616 z m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 1015 792 l 836 400 b 793 358 829 384 809 365 l 402 179 b 331 250 356 158 310 205 l 510 641 b 552 684 517 658 536 677 l 944 863 b 1015 792 989 884 1036 837 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1215 998 b 1085 1128 1215 1070 1157 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -28 58 -87 l 1085 -87 b 1215 43 1157 -87 1215 -28 z m 251 617 b 274 673 230 638 245 673 l 941 673 b 964 617 970 673 985 638 l 631 283 b 585 283 618 271 597 271 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 0 43 b 130 -87 0 -29 58 -87 l 1085 -87 b 1215 43 1157 -87 1215 -29 l 1215 998 b 1085 1128 1215 1070 1157 1128 l 130 1128 b 0 998 58 1128 0 1070 z m 964 424 b 941 369 985 404 970 369 l 274 369 b 251 424 245 369 230 404 l 585 758 b 631 758 597 771 618 771 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -29 58 -87 l 1085 -87 b 1215 43 1157 -87 1215 -29 l 1215 998 b 1085 1128 1215 1070 1157 1128 z m 511 164 b 456 187 491 144 456 158 l 456 855 b 511 878 456 884 491 898 l 845 544 b 845 498 858 531 858 511 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 843 93 b 804 118 839 111 822 122 b 680 104 775 112 729 104 b 351 335 527 104 404 198 l 659 335 b 691 361 674 335 688 347 l 708 438 b 677 477 713 458 698 477 l 313 477 b 313 592 309 516 309 554 l 711 592 l 711 592 b 742 618 725 592 739 603 l 760 698 b 728 738 764 719 749 738 l 353 738 b 672 941 409 860 523 941 b 775 930 712 941 750 935 b 813 954 792 927 808 937 l 845 1074 b 820 1114 850 1092 839 1111 b 677 1128 787 1121 734 1128 b 122 738 414 1128 201 965 l 33 738 b 0 705 15 738 0 723 l 0 625 b 33 592 0 607 15 592 l 91 592 b 91 477 88 555 88 513 l 33 477 b 0 445 15 477 0 463 l 0 368 b 33 335 0 350 15 335 l 114 335 b 677 -87 182 90 394 -87 b 843 -66 749 -87 809 -74 b 867 -28 860 -61 870 -45 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 836 260 l 712 260 b 680 228 694 260 680 246 l 680 90 l 347 90 l 347 434 l 575 434 b 608 467 593 434 608 449 l 608 575 b 575 608 608 593 593 608 l 347 608 l 347 780 b 515 935 347 868 414 935 b 671 884 579 935 639 904 b 714 891 685 875 703 878 l 791 987 b 785 1034 803 1002 800 1023 b 510 1128 741 1067 642 1128 b 130 787 288 1128 130 985 l 130 608 l 54 608 b 22 575 36 608 22 593 l 22 467 b 54 434 22 449 36 434 l 130 434 l 130 87 l 33 87 b 0 54 15 87 0 72 l 0 -54 b 33 -87 0 -72 15 -87 l 836 -87 b 868 -54 853 -87 868 -72 l 868 228 b 836 260 868 246 853 260 z "
	},
	"": {
		ha: 781,
		x_min: 0,
		x_max: 0,
		o: "m 567 582 l 275 668 b 217 745 241 678 217 709 b 297 825 217 789 253 825 l 477 825 b 570 796 510 825 543 815 b 623 802 586 785 608 788 l 717 894 b 712 960 736 913 734 944 b 477 1042 646 1012 563 1041 l 477 1172 b 434 1215 477 1196 458 1215 l 347 1215 b 304 1172 323 1215 304 1196 l 304 1042 l 297 1042 b 1 717 124 1042 -15 893 b 229 455 13 592 108 490 l 507 374 b 564 297 541 364 564 332 b 484 217 564 253 528 217 l 304 217 b 212 245 271 217 239 227 b 159 240 195 257 173 254 l 64 148 b 69 81 45 129 48 98 b 304 0 136 29 219 0 l 304 -130 b 347 -174 304 -154 323 -174 l 434 -174 b 477 -130 458 -174 477 -154 l 477 1 b 764 198 604 3 722 78 b 567 582 822 365 725 536 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 836 955 b 868 987 853 955 868 969 l 868 1096 b 836 1128 868 1114 853 1128 l 33 1128 b 0 1096 15 1128 0 1114 l 0 975 b 33 942 0 957 15 942 l 264 942 b 429 868 338 942 395 915 l 33 868 b 0 836 15 868 0 853 l 0 727 b 33 694 0 709 15 694 l 463 694 b 260 535 446 597 374 535 l 33 535 b 0 503 15 535 0 521 l 0 359 b 10 335 0 350 4 341 l 458 -78 b 480 -87 463 -83 473 -87 l 704 -87 b 726 -30 734 -87 748 -50 l 317 347 b 692 694 525 354 673 492 l 836 694 b 868 727 853 694 868 709 l 868 836 b 836 868 868 853 853 868 l 676 868 b 638 955 667 899 654 928 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 953 1128 l 776 1128 b 746 1110 763 1128 752 1121 l 596 803 b 522 608 557 709 522 608 l 519 608 b 445 803 519 608 485 709 l 295 1110 b 266 1128 290 1121 278 1128 l 89 1128 b 60 1081 64 1128 49 1102 l 278 673 l 119 673 b 87 640 101 673 87 658 l 87 553 b 119 521 87 535 101 521 l 359 521 l 412 420 l 412 347 l 119 347 b 87 315 101 347 87 333 l 87 228 b 119 195 87 210 101 195 l 412 195 l 412 -54 b 445 -87 412 -72 427 -87 l 597 -87 b 629 -54 615 -87 629 -72 l 629 195 l 922 195 b 955 228 940 195 955 210 l 955 315 b 922 347 955 333 940 347 l 629 347 l 629 420 l 683 521 l 922 521 b 955 553 940 521 955 535 l 955 640 b 922 673 955 658 940 673 l 764 673 l 981 1081 b 953 1128 993 1102 977 1128 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 649 347 b 1042 740 880 347 1042 509 b 649 1128 1042 972 880 1128 l 206 1128 b 174 1096 188 1128 174 1114 l 174 535 l 33 535 b 0 503 15 535 0 521 l 0 380 b 33 347 0 362 15 347 l 174 347 l 174 260 l 33 260 b 0 228 15 260 0 246 l 0 119 b 33 87 0 101 15 87 l 174 87 l 174 -54 b 206 -87 174 -72 188 -87 l 365 -87 b 398 -54 383 -87 398 -72 l 398 87 l 836 87 b 868 119 853 87 868 101 l 868 228 b 836 260 868 246 853 260 l 398 260 l 398 347 z m 398 942 l 611 942 b 814 740 738 942 814 863 b 608 535 814 616 738 535 l 398 535 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1530 694 b 1563 727 1548 694 1563 709 l 1563 836 b 1530 868 1563 853 1548 868 l 1400 868 l 1450 1087 b 1418 1127 1455 1107 1439 1127 l 1293 1127 b 1262 1101 1278 1127 1265 1116 l 1223 868 l 924 868 l 871 1101 b 839 1127 868 1116 854 1127 l 720 1127 b 688 1101 705 1127 692 1116 l 634 868 l 339 868 l 292 1101 b 260 1127 289 1116 275 1127 l 145 1127 b 114 1087 125 1127 109 1107 l 163 868 l 33 868 b 0 836 15 868 0 853 l 0 727 b 33 694 0 709 15 694 l 202 694 l 221 608 l 33 608 b 0 575 15 608 0 593 l 0 467 b 33 434 0 449 15 434 l 260 434 l 371 -61 b 403 -87 374 -76 388 -87 l 557 -87 b 589 -62 572 -87 585 -76 l 703 434 l 853 434 l 968 -62 b 1000 -87 971 -76 984 -87 l 1154 -87 b 1185 -62 1169 -87 1182 -76 l 1300 434 l 1530 434 b 1563 467 1548 434 1563 449 l 1563 575 b 1530 608 1563 593 1548 608 l 1340 608 l 1360 694 z m 499 288 b 479 160 482 218 480 160 l 476 160 b 457 288 476 160 471 219 l 428 434 l 533 434 z m 573 608 l 392 608 l 374 694 l 594 694 z m 744 608 l 749 631 b 762 694 754 653 759 674 l 794 694 b 807 631 798 674 802 653 l 813 608 z m 1099 288 b 1080 160 1085 219 1080 160 l 1077 160 b 1058 288 1076 160 1074 218 l 1024 434 l 1130 434 z m 1167 608 l 984 608 l 965 694 l 1186 694 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 608 846 l 608 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 -109 b 65 -174 0 -145 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -145 l 1042 781 l 673 781 b 608 846 637 781 608 811 z m 1042 885 b 1023 930 1042 902 1035 918 l 757 1196 b 711 1215 745 1208 728 1215 l 694 1215 l 694 868 l 1042 868 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 608 846 l 608 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 -109 b 65 -174 0 -145 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -145 l 1042 781 l 673 781 b 608 846 637 781 608 811 z m 781 206 b 749 174 781 188 767 174 l 293 174 b 260 206 275 174 260 188 l 260 228 b 293 260 260 246 275 260 l 749 260 b 781 228 767 260 781 246 z m 781 380 b 749 347 781 362 767 347 l 293 347 b 260 380 275 347 260 362 l 260 401 b 293 434 260 419 275 434 l 749 434 b 781 401 767 434 781 419 z m 781 575 l 781 553 b 749 521 781 535 767 521 l 293 521 b 260 553 275 521 260 535 l 260 575 b 293 608 260 593 275 608 l 749 608 b 781 575 767 608 781 593 z m 1042 885 b 1023 930 1042 902 1035 918 l 757 1196 b 711 1215 745 1208 728 1215 l 694 1215 l 694 868 l 1042 868 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 477 260 l 347 260 l 347 1085 b 304 1128 347 1109 328 1128 l 217 1128 b 174 1085 193 1128 174 1109 l 174 260 l 43 260 b 13 186 5 260 -15 214 l 230 -74 b 260 -87 237 -81 251 -87 b 291 -74 270 -87 284 -81 l 508 186 b 477 260 535 214 516 260 z m 1128 434 l 781 434 b 738 391 757 434 738 415 l 738 304 b 781 260 738 280 757 260 l 933 260 l 767 69 b 738 4 751 55 738 26 l 738 4 l 738 -43 b 781 -87 738 -67 757 -87 l 1128 -87 b 1172 -43 1152 -87 1172 -67 l 1172 43 b 1128 87 1172 67 1152 87 l 977 87 l 1143 278 b 1172 343 1159 292 1172 321 l 1172 343 l 1172 391 b 1128 434 1172 415 1152 434 z m 1213 666 l 1052 1100 b 1011 1128 1046 1116 1028 1128 l 1011 1128 l 899 1128 l 899 1128 b 858 1100 882 1128 863 1116 l 697 666 b 694 651 696 662 694 655 b 738 608 694 627 714 608 l 805 608 l 805 608 b 847 638 823 608 841 621 l 859 673 l 1051 673 l 1063 638 b 1104 608 1068 621 1087 608 l 1104 608 l 1172 608 b 1215 651 1196 608 1215 627 b 1213 666 1215 655 1214 662 z m 910 825 l 955 955 l 999 825 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 43 781 l 174 781 l 174 -43 b 217 -87 174 -67 193 -87 l 304 -87 b 347 -43 328 -87 347 -67 l 347 781 l 477 781 b 508 855 516 781 535 828 l 291 1116 b 260 1128 284 1123 270 1128 b 230 1116 250 1128 237 1123 l 13 855 b 43 781 -15 828 5 781 z m 1128 434 l 781 434 b 738 391 757 434 738 415 l 738 304 b 781 260 738 280 757 260 l 933 260 l 767 69 b 738 4 751 55 738 26 l 738 4 l 738 -43 b 781 -87 738 -67 757 -87 l 1128 -87 b 1172 -43 1152 -87 1172 -67 l 1172 43 b 1128 87 1172 67 1152 87 l 977 87 l 1143 278 b 1172 343 1159 292 1172 321 l 1172 343 l 1172 391 b 1128 434 1172 415 1152 434 z m 1213 666 l 1052 1100 b 1011 1128 1046 1116 1028 1128 l 1011 1128 l 899 1128 l 899 1128 b 858 1100 882 1128 863 1116 l 697 666 b 694 651 696 662 694 655 b 738 608 694 627 714 608 l 805 608 l 805 608 b 847 638 823 608 841 621 l 859 673 l 1051 673 l 1063 638 b 1104 608 1068 621 1087 608 l 1104 608 l 1172 608 b 1215 651 1196 608 1215 627 b 1213 666 1215 655 1214 662 z m 910 825 l 955 955 l 999 825 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 825 87 l 651 87 b 608 43 627 87 608 67 l 608 -43 b 651 -87 608 -67 627 -87 l 825 -87 b 868 -43 849 -87 868 -67 l 868 43 b 825 87 868 67 849 87 z m 477 260 l 347 260 l 347 1085 b 304 1128 347 1109 328 1128 l 217 1128 b 174 1085 193 1128 174 1109 l 174 260 l 43 260 b 13 186 5 260 -15 214 l 230 -74 b 260 -87 237 -81 251 -87 b 291 -74 270 -87 284 -81 l 508 186 b 477 260 535 214 516 260 z m 1172 781 l 651 781 b 608 738 627 781 608 762 l 608 651 b 651 608 608 627 627 608 l 1172 608 b 1215 651 1196 608 1215 627 l 1215 738 b 1172 781 1215 762 1196 781 z m 998 434 l 651 434 b 608 391 627 434 608 415 l 608 304 b 651 260 608 280 627 260 l 998 260 b 1042 304 1022 260 1042 280 l 1042 391 b 998 434 1042 415 1022 434 z m 1345 1128 l 651 1128 b 608 1085 627 1128 608 1109 l 608 998 b 651 955 608 974 627 955 l 1345 955 b 1389 998 1369 955 1389 974 l 1389 1085 b 1345 1128 1389 1109 1369 1128 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 825 87 l 651 87 b 608 43 627 87 608 67 l 608 -43 b 651 -87 608 -67 627 -87 l 825 -87 b 868 -43 849 -87 868 -67 l 868 43 b 825 87 868 67 849 87 z m 43 781 l 174 781 l 174 -43 b 217 -87 174 -67 193 -87 l 304 -87 b 347 -43 328 -87 347 -67 l 347 781 l 477 781 b 508 855 516 781 535 828 l 291 1116 b 260 1128 284 1123 270 1128 b 230 1116 250 1128 237 1123 l 13 855 b 43 781 -15 828 5 781 z m 1172 781 l 651 781 b 608 738 627 781 608 762 l 608 651 b 651 608 608 627 627 608 l 1172 608 b 1215 651 1196 608 1215 627 l 1215 738 b 1172 781 1215 762 1196 781 z m 998 434 l 651 434 b 608 391 627 434 608 415 l 608 304 b 651 260 608 280 627 260 l 998 260 b 1042 304 1022 260 1042 280 l 1042 391 b 998 434 1042 415 1022 434 z m 1345 1128 l 651 1128 b 608 1085 627 1128 608 1109 l 608 998 b 651 955 608 974 627 955 l 1345 955 b 1389 998 1369 955 1389 974 l 1389 1085 b 1345 1128 1389 1109 1369 1128 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 825 955 l 868 955 l 868 781 l 825 781 b 781 738 801 781 781 762 l 781 651 b 825 608 781 627 801 608 l 1085 608 b 1128 651 1109 608 1128 627 l 1128 738 b 1085 781 1128 762 1109 781 l 1042 781 l 1042 1085 b 998 1128 1042 1109 1022 1128 l 868 1128 b 829 1105 853 1128 836 1118 l 786 1018 b 781 998 783 1013 781 1004 b 825 955 781 974 801 955 z m 896 513 b 746 366 832 495 765 430 b 933 88 708 228 804 101 b 877 53 919 76 894 60 b 854 -2 856 44 847 19 l 881 -57 b 938 -83 889 -80 915 -93 b 1172 275 1095 -16 1172 84 l 1172 304 b 896 513 1172 443 1041 552 z m 955 250 b 901 304 925 250 901 274 b 955 358 901 334 925 358 b 1009 304 985 358 1009 334 b 955 250 1009 274 985 250 z m 477 260 l 347 260 l 347 1085 b 304 1128 347 1109 328 1128 l 217 1128 b 174 1085 193 1128 174 1109 l 174 260 l 43 260 b 13 186 5 260 -15 214 l 230 -74 b 260 -87 237 -81 251 -87 b 291 -74 270 -87 284 -81 l 508 186 b 477 260 535 214 516 260 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 896 513 b 746 366 832 495 765 430 b 933 88 708 228 804 101 b 877 53 919 76 894 60 b 854 -2 856 44 847 19 l 881 -57 b 938 -83 889 -80 915 -93 b 1172 275 1095 -16 1172 84 l 1172 304 b 896 513 1172 443 1041 552 z m 955 250 b 901 304 925 250 901 274 b 955 358 901 334 925 358 b 1009 304 985 358 1009 334 b 955 250 1009 274 985 250 z m 825 955 l 868 955 l 868 781 l 825 781 b 781 738 801 781 781 762 l 781 651 b 825 608 781 627 801 608 l 1085 608 b 1128 651 1109 608 1128 627 l 1128 738 b 1085 781 1128 762 1109 781 l 1042 781 l 1042 1085 b 998 1128 1042 1109 1022 1128 l 868 1128 b 829 1105 853 1128 836 1118 l 786 1018 b 781 998 783 1013 781 1004 b 825 955 781 974 801 955 z m 291 1116 b 260 1128 284 1123 270 1128 b 230 1116 250 1128 237 1123 l 13 855 b 43 781 -15 828 5 781 l 174 781 l 174 -43 b 217 -87 174 -67 193 -87 l 304 -87 b 347 -43 328 -87 347 -67 l 347 781 l 477 781 b 508 855 516 781 535 828 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 282 608 l 65 608 b 0 543 29 608 0 578 l 0 -109 b 65 -174 0 -144 29 -174 l 282 -174 b 347 -109 318 -174 347 -144 l 347 543 b 282 608 347 578 318 608 z m 174 -65 b 109 0 138 -65 109 -36 b 174 65 109 36 138 65 b 239 0 210 65 239 36 b 174 -65 239 -36 210 -65 z m 1042 994 b 846 1215 1042 1194 911 1215 b 754 1059 792 1215 766 1109 b 685 899 740 1000 726 939 b 444 592 597 810 551 698 b 434 569 438 586 434 577 l 434 -11 b 466 -44 434 -29 448 -43 b 609 -88 509 -44 566 -68 b 933 -174 695 -126 802 -174 l 941 -174 b 1249 -93 1057 -174 1194 -172 b 1266 28 1272 -60 1278 -20 b 1311 231 1310 74 1334 161 b 1336 446 1358 294 1363 383 l 1336 447 b 1389 580 1368 479 1389 532 b 1227 738 1388 663 1318 738 l 951 738 b 1042 994 971 815 1042 879 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 0 1063 l 0 412 b 65 347 0 376 29 347 l 282 347 b 347 412 318 347 347 376 l 347 1063 b 282 1128 347 1099 318 1128 l 65 1128 b 0 1063 29 1128 0 1099 z m 109 521 b 174 586 109 557 138 586 b 239 521 210 586 239 557 b 174 456 239 485 210 456 b 109 521 138 456 109 485 z m 846 -174 b 1042 47 911 -174 1042 -152 b 951 304 1042 162 971 227 l 1227 304 b 1389 461 1318 304 1388 379 b 1336 595 1389 510 1368 562 l 1336 595 b 1311 811 1363 658 1358 747 b 1266 1014 1334 881 1310 967 b 1249 1135 1278 1061 1272 1102 b 941 1215 1194 1214 1057 1215 l 933 1215 b 609 1129 802 1215 695 1168 b 466 1085 566 1110 509 1086 b 434 1053 448 1085 434 1071 l 434 473 l 434 473 b 444 450 434 465 438 455 b 685 143 551 344 597 231 b 754 -17 726 103 740 42 b 846 -174 766 -67 792 -174 z "
	},
	"": {
		ha: 694,
		x_min: 0,
		x_max: 0,
		o: "m 347 1215 b 174 1042 251 1215 174 1138 b 347 868 174 946 251 868 b 521 1042 443 868 521 946 b 347 1215 521 1138 443 1215 z m 671 255 l 541 775 b 477 825 534 803 505 825 l 447 825 b 248 825 385 796 312 795 l 217 825 b 154 775 189 825 161 803 l 24 255 b 87 174 13 213 44 174 l 239 174 l 239 -109 b 304 -174 239 -144 268 -174 l 391 -174 b 456 -109 427 -174 456 -144 l 456 174 l 608 174 b 671 255 650 174 681 213 z "
	},
	"": {
		ha: 521,
		x_min: 0,
		x_max: 0,
		o: "m 260 1215 b 87 1042 165 1215 87 1138 b 260 868 87 946 165 868 b 434 1042 356 868 434 946 b 260 1215 434 1138 356 1215 z m 391 825 l 360 825 b 161 825 298 796 225 795 l 130 825 b 0 694 58 825 0 766 l 0 326 b 65 260 0 290 29 260 l 109 260 l 109 -109 b 174 -174 109 -144 138 -174 l 347 -174 b 412 -109 383 -174 412 -144 l 412 260 l 456 260 b 521 326 492 260 521 290 l 521 694 b 391 825 521 766 463 825 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 781 b 434 521 551 781 434 664 b 694 260 434 377 551 260 b 955 521 838 260 955 377 b 694 781 955 664 838 781 z m 1363 563 l 1106 691 l 1197 964 b 1137 1023 1209 1000 1174 1035 l 865 932 l 736 1189 b 652 1189 719 1224 670 1224 l 524 932 l 251 1023 b 192 964 215 1035 180 1000 l 283 691 l 26 563 b 26 479 -9 546 -9 496 l 283 350 l 192 78 b 251 18 180 41 215 6 l 524 109 l 652 -148 b 736 -148 669 -182 719 -182 l 865 109 l 1137 18 b 1196 78 1174 6 1209 41 l 1105 350 l 1362 479 b 1363 563 1398 496 1398 546 z m 940 275 b 449 275 805 140 584 140 b 449 766 314 411 314 631 b 940 766 584 902 805 902 b 940 275 1075 631 1075 411 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 768 -174 b 1308 84 982 -174 1178 -76 b 1276 136 1327 107 1306 142 b 630 670 939 72 630 330 b 905 1143 630 866 735 1046 b 895 1204 932 1158 925 1198 b 768 1215 860 1210 804 1215 b 74 521 385 1215 74 905 b 768 -174 74 138 384 -174 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 87 0 b 174 -87 87 -48 126 -87 l 1215 -87 b 1302 0 1263 -87 1302 -48 l 1302 781 l 87 781 z m 521 575 b 553 608 521 593 535 608 l 836 608 b 868 575 853 608 868 593 l 868 553 b 836 521 868 535 853 521 l 553 521 b 521 553 535 521 521 535 z m 1302 1128 l 87 1128 b 0 1042 39 1128 0 1090 l 0 911 b 43 868 0 888 20 868 l 1345 868 b 1389 911 1369 868 1389 888 l 1389 1042 b 1302 1128 1389 1090 1350 1128 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1389 432 b 1302 521 1390 481 1351 521 l 1150 521 l 1150 680 l 1277 807 b 1277 929 1311 841 1311 896 b 1154 929 1243 963 1188 963 l 1006 781 l 383 781 l 235 929 b 112 929 201 963 146 963 b 112 807 78 896 78 841 l 239 680 l 239 521 l 87 521 b 0 432 38 521 -1 481 b 89 347 1 384 41 347 l 239 347 l 239 304 b 276 138 239 245 252 188 l 112 -25 b 112 -148 78 -59 78 -114 b 235 -148 146 -182 201 -182 l 383 0 b 629 -87 451 -54 536 -87 l 629 575 b 662 608 629 593 644 608 l 727 608 b 760 575 745 608 760 593 l 760 -87 b 1005 0 853 -87 938 -54 l 1154 -148 b 1277 -148 1188 -182 1243 -182 b 1277 -25 1311 -114 1311 -59 l 1113 138 b 1150 304 1137 188 1150 245 l 1150 347 l 1300 347 b 1389 432 1348 347 1388 384 z m 697 1215 b 393 911 529 1215 393 1079 l 1001 911 b 697 1215 1001 1079 865 1215 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1085 -87 b 1215 43 1157 -87 1215 -29 l 1215 998 b 1085 1128 1215 1070 1157 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -29 58 -87 z m 704 878 b 760 855 724 898 760 884 l 760 187 b 704 164 760 158 724 144 l 370 498 b 370 544 358 511 358 531 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 z m 911 521 b 694 304 911 401 814 304 b 477 521 575 304 477 401 b 694 738 477 640 575 738 b 911 521 814 738 911 640 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1346 169 b 1288 189 1335 191 1309 199 l 1189 139 l 1033 471 b 955 521 1020 498 985 521 l 596 521 l 584 608 l 911 608 b 955 651 935 608 955 627 l 955 738 b 911 781 955 762 935 781 l 559 781 l 546 870 b 694 1046 632 882 697 957 b 529 1215 692 1137 619 1211 b 347 1042 429 1220 347 1140 b 435 422 348 1025 343 1065 b 521 347 441 379 478 347 l 900 347 l 1070 -15 b 1187 -56 1091 -59 1144 -78 l 1365 33 b 1384 91 1386 44 1395 70 z m 845 260 l 778 260 b 477 0 757 113 630 0 b 174 304 310 0 174 136 b 326 567 174 416 235 515 b 301 747 316 638 308 697 b 0 304 125 677 0 505 b 477 -174 0 41 214 -174 b 914 112 672 -174 840 -56 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 1009 521 l 879 521 b 846 490 862 521 846 508 b 391 87 846 217 624 87 l 391 519 l 756 600 b 781 632 770 603 781 617 l 781 743 b 742 775 781 764 762 779 l 391 697 l 391 779 l 756 860 b 781 892 770 863 781 878 l 781 1003 b 742 1035 781 1024 762 1039 l 391 957 l 391 1096 b 358 1128 391 1114 376 1128 l 206 1128 b 174 1096 188 1128 174 1114 l 174 909 l 25 876 b 0 844 11 873 0 858 l 0 733 b 40 701 0 712 19 697 l 174 731 l 174 648 l 25 615 b 0 584 11 612 0 598 l 0 472 b 40 441 0 452 19 436 l 174 470 l 174 -54 b 206 -87 174 -72 188 -87 l 422 -87 b 1042 487 786 -87 1028 124 b 1009 521 1042 506 1028 521 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1608 650 b 1280 716 1518 692 1399 716 l 505 716 b 460 760 492 734 477 749 l 1020 760 b 260 1128 622 842 595 1128 l 260 1128 l 260 781 l 217 781 l 217 1128 b 87 955 145 1128 87 1051 l 87 781 b 0 716 24 781 0 754 l 0 608 b 87 543 0 570 24 543 l 87 499 b 0 434 24 499 0 472 l 0 326 b 87 260 0 288 24 260 l 87 87 b 217 -87 87 -9 145 -87 l 217 260 l 260 260 l 260 -87 l 260 -87 b 1020 282 595 -87 622 200 l 460 282 b 505 326 477 293 492 308 l 1280 326 b 1608 391 1399 326 1518 349 b 1736 521 1688 429 1736 477 b 1608 650 1736 564 1688 613 z m 1324 412 b 1302 434 1312 412 1302 422 l 1302 608 b 1324 629 1302 620 1312 629 b 1324 412 1410 629 1410 412 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1085 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -29 58 -87 l 1085 -87 b 1215 43 1157 -87 1215 -29 l 1215 998 b 1085 1128 1215 1070 1157 1128 z m 483 504 b 174 735 237 683 240 683 l 174 803 b 239 868 174 839 203 868 l 977 868 b 1042 803 1013 868 1042 839 l 1042 735 b 732 504 976 683 978 683 b 608 434 704 484 647 433 b 483 504 568 433 512 483 z m 1042 625 l 1042 239 b 977 174 1042 203 1013 174 l 239 174 b 174 239 203 174 174 203 l 174 625 b 432 434 211 595 264 556 b 608 347 471 406 535 347 b 783 434 681 347 746 407 b 1042 625 951 556 1004 595 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1345 868 b 1332 888 1345 876 1339 885 l 703 1127 b 694 1128 701 1128 697 1128 b 686 1127 692 1128 688 1128 l 57 888 b 43 868 49 885 43 876 l 43 825 b 65 803 43 813 53 803 l 130 803 l 130 770 b 163 738 130 752 145 738 l 1226 738 b 1259 770 1244 738 1259 752 l 1259 803 l 1324 803 b 1345 825 1336 803 1345 813 z m 1280 43 l 109 43 b 43 -22 73 43 43 14 l 43 -65 b 65 -87 43 -77 53 -87 l 1324 -87 b 1345 -65 1336 -87 1345 -77 l 1345 -22 b 1280 43 1345 14 1316 43 z m 260 694 l 260 174 l 163 174 b 130 141 145 174 130 159 l 130 87 l 1259 87 l 1259 141 b 1226 174 1259 159 1244 174 l 1128 174 l 1128 694 l 955 694 l 955 174 l 781 174 l 781 694 l 608 694 l 608 174 l 434 174 l 434 694 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1688 800 l 932 1032 b 805 1032 890 1045 846 1045 l 48 800 b 48 676 -16 780 -16 696 l 180 635 b 131 508 151 600 133 556 b 87 434 105 493 87 466 b 124 364 87 405 102 380 l 55 53 b 97 0 49 26 70 0 l 250 0 b 292 53 277 0 298 26 l 223 364 b 260 434 245 380 260 405 b 218 507 260 465 243 492 b 274 607 220 548 241 584 l 805 444 b 932 444 829 436 876 427 l 1688 676 b 1688 800 1752 696 1752 780 z m 957 361 b 779 361 880 337 814 350 l 386 481 l 347 174 b 868 0 347 78 580 0 b 1389 174 1156 0 1389 78 l 1350 481 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 413 575 b 391 665 403 607 391 665 l 390 665 b 369 575 390 665 378 607 l 339 473 l 442 473 z m 1671 955 l 911 955 l 911 87 l 1671 87 b 1736 152 1707 87 1736 116 l 1736 890 b 1671 955 1736 926 1707 955 z m 1606 629 b 1573 597 1606 611 1591 597 l 1542 597 b 1427 407 1524 533 1484 468 b 1497 358 1449 390 1473 373 b 1509 314 1512 349 1517 330 l 1487 277 b 1442 265 1478 261 1458 256 b 1346 333 1408 286 1375 309 b 1250 265 1316 309 1284 286 b 1204 277 1234 256 1214 261 l 1183 314 b 1194 358 1174 330 1179 349 b 1265 407 1220 374 1243 390 b 1208 477 1244 430 1225 453 b 1218 523 1197 492 1202 514 l 1236 534 l 1256 546 b 1299 536 1270 554 1289 550 b 1346 480 1313 517 1328 498 b 1428 597 1383 518 1411 558 l 1118 597 b 1085 629 1100 597 1085 611 l 1085 673 b 1118 705 1085 691 1100 705 l 1291 705 l 1291 749 b 1324 781 1291 767 1306 781 l 1367 781 b 1400 749 1385 781 1400 767 l 1400 705 l 1573 705 b 1606 673 1591 705 1606 691 z m 0 890 l 0 152 b 65 87 0 116 29 87 l 825 87 l 825 955 l 65 955 b 0 890 29 955 0 926 z m 160 304 l 316 762 b 347 784 320 776 333 784 l 435 784 b 466 762 449 784 461 775 l 622 304 b 591 260 629 282 613 260 l 529 260 b 498 284 515 260 501 271 l 472 370 l 309 370 l 284 284 b 253 260 280 270 267 260 l 191 260 b 160 304 168 260 153 282 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 174 868 l 87 868 b 0 781 39 868 0 829 l 0 -87 b 87 -174 0 -135 39 -174 l 174 -174 b 260 -87 222 -174 260 -135 l 260 781 b 174 868 260 829 222 868 z m 1302 781 l 1302 1006 b 1277 1067 1302 1029 1293 1051 l 1154 1190 b 1093 1215 1138 1206 1116 1215 l 434 1215 b 347 1128 386 1215 347 1176 l 347 -87 b 434 -174 347 -135 386 -174 l 1302 -174 b 1389 -87 1350 -174 1389 -135 l 1389 694 b 1302 781 1389 742 1350 781 z m 781 43 b 738 0 781 19 762 0 l 651 0 b 608 43 627 0 608 19 l 608 130 b 651 174 608 154 627 174 l 738 174 b 781 130 762 174 781 154 z m 781 391 b 738 347 781 367 762 347 l 651 347 b 608 391 627 347 608 367 l 608 477 b 651 521 608 501 627 521 l 738 521 b 781 477 762 521 781 501 z m 1128 43 b 1085 0 1128 19 1109 0 l 998 0 b 955 43 974 0 955 19 l 955 130 b 998 174 955 154 974 174 l 1085 174 b 1128 130 1109 174 1128 154 z m 1128 391 b 1085 347 1128 367 1109 347 l 998 347 b 955 391 974 347 955 367 l 955 477 b 998 521 955 501 974 521 l 1085 521 b 1128 477 1109 521 1128 501 z m 1172 694 l 477 694 l 477 1085 l 1042 1085 l 1042 998 b 1085 955 1042 974 1061 955 l 1172 955 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1183 -87 l 1128 -87 l 1128 1150 b 1063 1215 1128 1186 1099 1215 l 152 1215 b 87 1150 116 1215 87 1186 l 87 -87 l 33 -87 b 0 -119 15 -87 0 -101 l 0 -174 l 1215 -174 l 1215 -119 b 1183 -87 1215 -101 1201 -87 z m 347 1009 b 380 1042 347 1027 362 1042 l 488 1042 b 521 1009 506 1042 521 1027 l 521 901 b 488 868 521 883 506 868 l 380 868 b 347 901 362 868 347 883 z m 347 749 b 380 781 347 767 362 781 l 488 781 b 521 749 506 781 521 767 l 521 640 b 488 608 521 622 506 608 l 380 608 b 347 640 362 608 347 622 z m 488 347 l 380 347 b 347 380 362 347 347 362 l 347 488 b 380 521 347 506 362 521 l 488 521 b 521 488 506 521 521 506 l 521 380 b 488 347 521 362 506 347 z m 694 -87 l 521 -87 l 521 141 b 553 174 521 159 535 174 l 662 174 b 694 141 680 174 694 159 z m 868 380 b 836 347 868 362 853 347 l 727 347 b 694 380 709 347 694 362 l 694 488 b 727 521 694 506 709 521 l 836 521 b 868 488 853 521 868 506 z m 868 640 b 836 608 868 622 853 608 l 727 608 b 694 640 709 608 694 622 l 694 749 b 727 781 694 767 709 781 l 836 781 b 868 749 853 781 868 767 z m 868 901 b 836 868 868 883 853 868 l 727 868 b 694 901 709 868 694 883 l 694 1009 b 727 1042 694 1027 709 1042 l 836 1042 b 868 1009 853 1042 868 1027 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 326 1020 b 521 825 326 912 413 825 b 716 1020 629 825 716 912 b 521 1215 716 1128 629 1215 b 326 1020 413 1215 326 1128 z m 1016 1016 b 893 1016 982 1050 927 1050 l 658 781 l 383 781 l 148 1016 b 25 1016 114 1050 59 1050 b 25 893 -8 982 -8 927 l 282 637 l 282 -87 b 369 -174 282 -135 321 -174 l 412 -174 b 499 -87 460 -174 499 -135 l 499 217 l 543 217 l 543 -87 b 629 -174 543 -135 581 -174 l 673 -174 b 760 -87 721 -174 760 -135 l 760 637 l 1016 893 b 1016 1016 1050 927 1050 982 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 608 b 174 64 479 608 174 275 b 368 -87 174 -30 246 -87 b 694 -19 501 -87 588 -19 b 1021 -87 802 -19 889 -87 b 1215 64 1143 -87 1215 -30 b 694 608 1215 275 910 608 z m 295 642 b 101 778 267 736 180 797 b 9 573 22 759 -19 667 b 203 437 37 479 124 419 b 295 642 282 456 323 548 z m 525 698 b 619 951 609 720 651 834 b 408 1125 586 1069 492 1147 b 315 871 324 1103 282 989 b 525 698 347 754 441 676 z m 1288 778 b 1094 642 1209 797 1122 736 b 1186 437 1066 548 1107 456 b 1380 573 1265 419 1352 479 b 1288 778 1408 667 1367 759 z m 864 698 b 1074 871 948 676 1042 754 b 980 1125 1106 989 1064 1103 b 770 951 897 1147 802 1069 b 864 698 738 834 780 720 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 649 1198 l 84 987 b 0 865 34 968 0 919 l 0 254 b 72 138 0 205 28 160 l 636 -145 b 752 -145 673 -163 716 -163 l 1317 138 b 1389 254 1361 160 1389 205 l 1389 865 b 1304 986 1389 919 1355 967 l 740 1198 b 649 1198 711 1209 678 1209 z m 694 1030 l 1215 834 l 1215 831 l 694 620 l 174 831 l 174 834 z m 781 64 l 781 468 l 1215 644 l 1215 281 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1325 537 l 1063 635 l 1063 929 b 1000 1021 1063 970 1038 1006 l 729 1122 b 660 1122 707 1131 682 1131 l 389 1021 b 325 929 350 1006 325 970 l 325 635 l 63 537 b 0 445 25 522 0 486 l 0 146 b 54 59 0 110 21 76 l 325 -76 b 413 -76 353 -90 385 -90 l 694 65 l 976 -76 b 1064 -76 1004 -90 1036 -90 l 1335 59 b 1389 146 1368 76 1389 110 l 1389 445 b 1325 537 1389 486 1364 522 z m 971 633 l 741 546 l 741 731 l 971 831 z m 418 933 l 694 1037 l 971 933 l 971 931 l 694 819 l 418 931 z m 646 143 l 415 28 l 415 243 l 646 348 z m 646 447 l 369 335 l 92 447 l 92 449 l 369 552 l 646 449 z m 1297 143 l 1066 28 l 1066 243 l 1297 348 z m 1297 447 l 1020 335 l 743 447 l 743 449 l 1020 552 l 1297 449 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 501 505 l 432 804 b 380 836 426 827 403 842 l 81 767 b 67 688 43 759 35 709 l 178 619 l 40 398 b 261 0 -68 225 56 0 l 401 0 b 434 33 419 0 434 15 l 434 141 b 401 174 434 159 419 174 l 260 174 b 187 306 192 174 151 249 l 325 527 l 435 458 b 501 505 468 438 509 467 z m 768 1001 l 880 822 l 770 753 b 783 674 737 732 745 682 l 1082 605 b 1134 637 1105 599 1128 614 l 1203 936 b 1138 983 1212 974 1170 1003 l 1027 914 l 915 1093 b 474 1093 813 1256 576 1256 l 425 1015 b 435 970 415 1000 420 980 l 527 913 b 572 923 543 903 563 908 l 621 1001 b 768 1001 655 1056 735 1054 z m 1349 398 l 1274 518 b 1229 528 1265 533 1245 538 l 1138 471 b 1127 426 1122 461 1118 441 l 1202 306 b 1128 174 1238 249 1196 174 l 868 174 l 868 304 b 794 334 868 342 821 362 l 577 117 b 577 56 560 101 560 73 l 794 -161 b 868 -130 821 -188 868 -169 l 868 0 l 1128 0 b 1349 398 1332 0 1457 225 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1356 738 l 1194 738 l 1149 851 b 867 1042 1102 967 992 1042 l 522 1042 b 240 851 397 1042 287 967 l 195 738 l 33 738 b 1 697 11 738 -4 718 l 17 632 b 49 608 21 618 34 608 l 103 608 b 43 477 67 576 43 530 l 43 347 b 87 233 43 303 60 264 l 87 87 b 174 0 87 39 126 0 l 260 0 b 347 87 308 0 347 39 l 347 174 l 1042 174 l 1042 87 b 1128 0 1042 39 1081 0 l 1215 0 b 1302 87 1263 0 1302 39 l 1302 233 b 1345 347 1329 264 1345 303 l 1345 477 b 1286 608 1345 530 1322 576 l 1340 608 b 1372 632 1355 608 1368 618 l 1388 697 b 1356 738 1393 718 1377 738 z m 401 786 b 522 868 421 836 469 868 l 867 868 b 988 786 920 868 968 836 l 1042 651 l 347 651 z m 260 348 b 174 434 208 348 174 382 b 260 521 174 486 208 521 b 391 391 313 521 391 443 b 260 348 391 339 313 348 z m 1128 348 b 998 391 1076 348 998 339 b 1128 521 998 443 1076 521 b 1215 434 1181 521 1215 486 b 1128 348 1215 382 1181 348 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1253 560 l 1194 790 b 985 955 1168 885 1081 955 l 955 955 l 955 1042 b 868 1128 955 1090 916 1128 l 521 1128 b 434 1042 473 1128 434 1090 l 434 955 l 404 955 b 195 790 308 955 221 885 l 136 560 b 0 391 58 542 0 473 l 0 260 b 87 111 0 196 35 141 l 87 0 b 174 -87 87 -48 126 -87 l 260 -87 b 347 0 308 -87 347 -48 l 347 87 l 1042 87 l 1042 0 b 1128 -87 1042 -48 1081 -87 l 1215 -87 b 1302 0 1263 -87 1302 -48 l 1302 111 b 1389 260 1354 141 1389 196 l 1389 391 b 1253 560 1389 473 1331 542 z m 260 260 b 174 347 212 260 174 299 b 260 434 174 395 212 434 b 347 347 308 434 347 395 b 260 260 347 299 308 260 z m 316 564 l 363 744 b 404 781 369 766 389 781 l 985 781 b 1026 746 1000 781 1020 766 l 1073 564 z m 1128 260 b 1042 347 1081 260 1042 299 b 1128 434 1042 395 1081 434 b 1215 347 1176 434 1215 395 b 1128 260 1215 299 1176 260 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 1026 189 l 810 434 l 893 434 b 949 469 917 434 939 448 b 940 535 959 491 956 516 l 728 781 l 807 781 b 863 818 831 781 853 796 b 852 883 873 840 868 866 l 552 1202 b 489 1202 536 1220 506 1220 l 190 883 b 179 818 174 866 169 840 b 235 781 188 796 210 781 l 314 781 l 102 535 b 93 469 86 516 82 491 b 149 434 103 448 125 434 l 232 434 l 15 189 b 6 123 -1 170 -5 145 b 63 87 16 101 38 87 l 434 87 l 434 20 l 352 -111 b 391 -174 337 -140 358 -174 l 651 -174 b 690 -111 683 -174 704 -140 l 608 20 l 608 87 l 979 87 b 1036 123 1003 87 1026 101 b 1026 189 1046 145 1043 170 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1215 1017 b 608 1215 1215 1126 943 1215 b 0 1017 272 1215 0 1126 l 0 893 b 608 694 0 784 272 694 b 1215 893 943 694 1215 784 z m 1215 738 b 608 606 1085 648 846 606 b 0 738 369 606 131 648 l 0 459 b 608 260 0 350 272 260 b 1215 459 943 260 1215 350 z m 1215 304 b 608 172 1085 214 846 172 b 0 304 369 172 131 214 l 0 25 b 608 -174 0 -84 272 -174 b 1215 25 943 -174 1215 -84 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 493 521 b 488 648 509 548 511 648 b 493 521 480 648 480 564 z m 489 393 b 412 222 468 338 442 275 b 582 282 461 241 518 269 b 489 393 548 308 515 345 z m 234 54 b 328 163 234 52 269 69 b 234 54 310 146 249 97 z m 673 781 b 608 846 637 781 608 811 l 608 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 -109 b 65 -174 0 -145 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -145 l 1042 781 z m 651 315 b 851 296 703 322 813 333 b 797 178 882 265 876 178 b 624 231 751 178 683 199 b 409 168 556 220 479 191 b 244 0 340 49 292 0 b 185 27 229 0 200 9 b 333 212 133 91 260 174 b 334 212 333 212 333 212 b 444 445 366 270 413 370 b 422 654 421 534 409 604 b 552 635 437 707 539 715 b 535 461 567 588 547 511 b 651 315 561 394 597 348 z m 1023 930 l 757 1196 b 711 1215 745 1208 728 1215 l 694 1215 l 694 868 l 1042 868 l 1042 885 b 1023 930 1042 902 1035 918 z m 822 238 b 706 262 833 245 815 270 b 822 238 806 219 822 238 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 608 846 l 608 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 -109 b 65 -174 0 -145 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -145 l 1042 781 l 673 781 b 608 846 637 781 608 811 z m 763 521 l 827 521 b 859 481 848 521 864 502 l 756 25 b 724 0 753 10 739 0 l 621 0 b 590 25 606 0 593 10 b 520 324 520 305 533 245 l 519 324 b 449 25 516 286 512 277 b 418 0 446 10 433 0 l 317 0 b 286 26 302 0 289 11 l 183 481 b 215 521 178 502 194 521 l 281 521 b 313 495 297 521 310 510 b 370 163 356 283 368 197 b 450 496 375 191 390 252 b 482 521 454 511 467 521 l 561 521 b 592 496 576 521 589 510 b 673 145 658 223 671 159 b 731 495 672 175 666 193 b 763 521 734 510 747 521 z m 1042 885 b 1023 930 1042 902 1035 918 l 757 1196 b 711 1215 745 1208 728 1215 l 694 1215 l 694 868 l 1042 868 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 608 846 l 608 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 -109 b 65 -174 0 -145 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -145 l 1042 781 l 673 781 b 608 846 637 781 608 811 z m 771 557 l 608 304 l 771 50 b 743 0 785 28 769 0 l 649 0 b 620 17 637 0 626 7 b 521 203 567 115 521 203 b 422 17 503 163 494 149 b 393 0 416 6 405 0 l 298 0 b 271 50 273 0 257 28 l 435 304 l 271 557 b 298 607 257 579 273 607 l 393 607 b 422 590 405 607 416 601 b 521 404 492 458 476 499 b 620 590 521 404 537 436 b 649 607 626 601 637 607 l 743 607 b 771 557 769 608 785 579 z m 1042 885 b 1023 930 1042 902 1035 918 l 757 1196 b 711 1215 745 1208 728 1215 l 694 1215 l 694 868 l 1042 868 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 525 480 l 450 480 l 450 315 l 523 315 b 580 338 548 315 567 323 b 581 458 607 369 607 428 b 525 480 567 472 549 480 z m 1023 930 l 757 1196 b 711 1215 745 1208 728 1215 l 694 1215 l 694 868 l 1042 868 l 1042 885 b 1023 930 1042 902 1035 918 z m 608 846 l 608 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 -109 b 65 -174 0 -145 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -145 l 1042 781 l 673 781 b 608 846 637 781 608 811 z m 751 398 b 450 188 751 153 511 188 l 450 33 b 417 0 450 15 435 0 l 334 0 b 301 33 316 0 301 15 l 301 575 b 334 607 301 592 316 607 l 554 607 b 751 398 674 607 751 518 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 1042 884 b 1023 931 1042 899 1033 920 l 757 1196 b 711 1215 746 1207 726 1215 l 694 1215 l 694 868 l 1042 868 z m 673 781 b 608 846 637 781 608 811 l 608 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 -109 b 65 -174 0 -144 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -144 l 1042 781 z m 305 738 b 436 608 377 738 436 680 b 305 477 436 536 377 477 b 175 608 233 477 175 536 b 305 738 175 680 233 738 z m 870 87 l 175 87 l 176 218 l 284 326 b 328 324 296 338 316 337 l 436 217 l 716 498 b 762 498 729 511 750 511 l 870 391 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 1023 930 l 757 1196 b 711 1215 745 1208 728 1215 l 694 1215 l 694 868 l 1042 868 l 1042 885 b 1023 930 1042 902 1035 918 z m 348 304 b 260 231 300 304 260 271 b 349 157 260 190 300 157 b 436 231 397 157 436 190 b 348 304 436 271 397 304 z m 608 846 l 608 1215 l 435 1215 l 435 1128 l 348 1128 l 348 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 -109 b 65 -174 0 -145 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -145 l 1042 781 l 673 781 b 608 846 637 781 608 811 z m 260 1128 l 347 1128 l 347 1042 l 260 1042 z m 348 87 b 208 257 258 87 190 169 l 262 521 l 262 608 l 348 608 l 348 694 l 262 694 l 262 781 l 348 781 l 348 868 l 262 868 l 262 955 l 348 955 l 348 1042 l 435 1042 l 435 955 l 348 955 l 348 868 l 435 868 l 435 781 l 348 781 l 348 694 l 435 694 l 435 608 l 348 608 l 348 521 l 408 521 b 440 495 424 521 437 510 l 487 257 b 348 87 505 169 437 87 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 608 846 l 608 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 -109 b 65 -174 0 -145 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -145 l 1042 781 l 673 781 b 608 846 637 781 608 811 z m 434 119 b 378 96 434 90 399 76 l 282 195 l 206 195 b 174 228 188 195 174 210 l 174 380 b 206 412 174 398 188 412 l 282 412 l 378 511 b 434 488 399 532 434 517 z m 524 248 b 524 339 549 274 549 314 b 617 430 464 401 557 492 b 617 158 691 354 691 234 b 524 248 558 97 463 186 z m 757 566 b 757 21 905 414 905 173 b 664 112 698 -39 603 49 b 664 475 762 213 763 374 b 757 566 604 537 697 628 z m 1042 885 b 1023 930 1042 902 1035 918 l 757 1196 b 711 1215 745 1208 728 1215 l 694 1215 l 694 868 l 1042 868 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 1042 884 b 1023 931 1042 899 1033 920 l 757 1196 b 711 1215 745 1208 728 1215 l 694 1215 l 694 868 l 1042 868 z m 608 846 l 608 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 -109 b 65 -174 0 -144 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -144 l 1042 781 l 673 781 b 608 846 637 781 608 811 z m 868 456 l 868 152 b 757 106 868 94 798 65 l 608 255 l 608 152 b 543 87 608 116 578 87 l 239 87 b 174 152 203 87 174 116 l 174 456 b 239 521 174 492 203 521 l 543 521 b 608 456 578 521 608 492 l 608 352 l 757 502 b 868 456 798 542 868 514 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 1042 884 b 1023 931 1042 899 1033 920 l 757 1196 b 711 1215 745 1208 728 1215 l 694 1215 l 694 868 l 1042 868 z m 673 781 b 608 846 637 781 608 811 l 608 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 -109 b 65 -174 0 -144 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -144 l 1042 781 z m 334 129 b 324 124 332 126 327 124 b 314 128 320 124 316 126 l 138 293 b 133 304 135 296 133 300 b 138 315 133 307 135 312 l 314 479 b 324 483 316 482 320 483 b 334 479 327 483 332 481 l 387 422 b 391 412 390 420 391 415 b 386 401 391 408 389 404 l 276 304 l 386 207 b 391 196 389 204 391 199 b 387 185 391 192 390 188 z m 473 -8 l 399 14 b 388 28 393 15 388 22 b 389 32 388 29 389 31 l 556 606 b 570 616 557 612 564 616 b 574 616 571 616 573 616 l 648 594 b 659 580 654 592 659 586 b 658 576 659 579 658 577 l 492 2 b 477 -9 490 -4 484 -9 b 473 -8 476 -9 474 -8 z m 910 293 l 734 128 b 724 124 731 126 727 124 b 713 129 720 124 715 126 l 660 185 b 656 196 658 188 656 192 b 661 207 656 199 658 204 l 771 304 l 661 401 b 656 412 658 404 656 408 b 660 422 656 415 658 420 l 713 479 b 724 483 715 481 720 483 b 734 479 727 483 731 482 l 910 315 b 914 304 912 312 914 307 b 910 293 914 300 912 296 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 z m 1166 869 l 994 697 b 871 820 964 748 921 790 l 1043 992 b 1166 869 1082 963 1137 908 z m 694 260 b 434 521 551 260 434 377 b 694 781 434 665 551 781 b 955 521 838 781 955 665 b 694 260 955 377 838 260 z m 346 992 l 518 820 b 395 697 468 790 425 748 l 223 869 b 346 992 252 908 307 963 z m 223 172 l 395 344 b 518 222 425 294 468 251 l 346 50 b 223 172 307 79 252 134 z m 1043 50 l 871 222 b 994 344 921 251 964 294 l 1166 172 b 1043 50 1137 134 1082 79 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 781 1109 l 781 1064 b 829 1002 781 1035 801 1009 b 1194 521 1040 943 1194 750 b 694 22 1194 245 970 22 b 195 521 419 22 195 245 b 560 1002 195 750 349 943 b 608 1064 588 1009 608 1035 l 608 1109 b 526 1172 608 1152 567 1183 b 22 517 235 1097 20 832 b 696 -152 24 145 324 -153 b 1367 521 1067 -151 1367 150 b 862 1173 1367 835 1153 1098 b 781 1109 821 1183 781 1152 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1291 1207 l 34 481 b 40 364 -15 453 -9 385 l 328 243 l 1108 930 b 1131 907 1123 943 1144 923 l 477 111 l 477 -107 b 593 -150 477 -171 555 -196 l 765 60 l 1103 -82 b 1192 -33 1141 -98 1185 -74 l 1388 1139 b 1291 1207 1397 1194 1338 1234 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1367 522 b 694 1194 1367 893 1066 1194 b 230 1007 514 1194 351 1123 l 96 1141 b 22 1110 68 1169 22 1149 l 22 716 b 65 673 22 692 41 673 l 459 673 b 490 747 498 673 517 719 l 352 885 b 694 1020 442 969 562 1020 b 1194 521 970 1020 1194 797 b 694 22 1194 245 970 22 b 384 130 577 22 469 62 b 298 124 358 150 321 148 l 267 94 b 272 -3 240 66 242 21 b 695 -152 388 -96 535 -152 b 1367 522 1066 -152 1368 151 z m 876 308 l 903 343 b 892 434 925 371 920 412 l 781 520 l 781 803 b 716 868 781 839 752 868 l 673 868 b 608 803 637 868 608 839 l 608 435 l 785 297 b 876 308 813 275 854 280 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1215 955 l 1302 955 b 1345 998 1326 955 1345 974 l 1345 1085 b 1302 1128 1345 1109 1326 1128 l 868 1128 b 825 1085 844 1128 825 1109 l 825 998 b 868 955 825 974 844 955 l 955 955 l 955 608 l 434 608 l 434 955 l 521 955 b 564 998 545 955 564 974 l 564 1085 b 521 1128 564 1109 545 1128 l 87 1128 b 43 1085 63 1128 43 1109 l 43 998 b 87 955 43 974 63 955 l 174 955 l 174 87 l 87 87 b 43 43 63 87 43 67 l 43 -43 b 87 -87 43 -67 63 -87 l 521 -87 b 564 -43 545 -87 564 -67 l 564 43 b 521 87 564 67 545 87 l 434 87 l 434 434 l 955 434 l 955 87 l 868 87 b 825 43 844 87 825 67 l 825 -43 b 868 -87 825 -67 844 -87 l 1302 -87 b 1345 -43 1326 -87 1345 -67 l 1345 43 b 1302 87 1345 67 1326 87 l 1215 87 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1215 1085 b 1172 1128 1215 1109 1196 1128 l 521 1128 b 87 694 281 1128 87 934 b 521 260 87 455 281 260 l 608 260 l 608 -43 b 651 -87 608 -67 627 -87 l 738 -87 b 781 -43 762 -87 781 -67 l 781 955 l 868 955 l 868 -43 b 911 -87 868 -67 887 -87 l 998 -87 b 1042 -43 1022 -87 1042 -67 l 1042 955 l 1172 955 b 1215 998 1196 955 1215 974 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1345 174 l 434 174 l 434 217 b 391 260 434 241 414 260 l 304 260 b 260 217 280 260 260 241 l 260 174 l 43 174 b 0 130 20 174 0 154 l 0 43 b 43 0 0 20 20 0 l 260 0 l 260 -43 b 304 -87 260 -67 280 -87 l 391 -87 b 434 -43 414 -87 434 -67 l 434 0 l 1345 0 b 1389 43 1369 0 1389 20 l 1389 130 b 1345 174 1389 154 1369 174 z m 1345 608 l 1128 608 l 1128 651 b 1085 694 1128 675 1109 694 l 998 694 b 955 651 974 694 955 675 l 955 608 l 43 608 b 0 564 20 608 0 588 l 0 477 b 43 434 0 454 20 434 l 955 434 l 955 391 b 998 347 955 367 974 347 l 1085 347 b 1128 391 1109 347 1128 367 l 1128 434 l 1345 434 b 1389 477 1369 434 1389 454 l 1389 564 b 1345 608 1389 588 1369 608 z m 1345 1042 l 781 1042 l 781 1085 b 738 1128 781 1109 762 1128 l 651 1128 b 608 1085 627 1128 608 1109 l 608 1042 l 43 1042 b 0 998 20 1042 0 1022 l 0 911 b 43 868 0 888 20 868 l 608 868 l 608 825 b 651 781 608 801 627 781 l 738 781 b 781 825 762 781 781 801 l 781 868 l 1345 868 b 1389 911 1369 868 1389 888 l 1389 998 b 1345 1042 1389 1022 1369 1042 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 955 347 b 793 291 894 347 837 326 l 515 464 b 521 521 518 480 521 505 b 515 577 521 537 518 562 l 793 751 b 955 694 837 716 894 694 b 1215 955 1099 694 1215 811 b 955 1215 1215 1099 1099 1215 b 694 955 811 1215 694 1099 b 701 898 694 935 697 917 l 423 725 b 260 781 378 760 322 781 b 0 521 117 781 0 665 b 260 260 0 377 117 260 b 423 317 322 260 378 282 l 701 143 b 694 87 697 128 694 103 l 694 87 b 955 -174 694 -57 811 -174 b 1215 87 1099 -174 1215 -57 b 955 347 1215 231 1099 347 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1215 998 b 1085 1128 1215 1070 1157 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -29 58 -87 l 1085 -87 b 1215 43 1157 -87 1215 -29 z m 825 412 b 977 260 909 412 977 344 b 825 109 977 177 909 109 b 673 260 741 109 673 177 l 673 261 b 677 298 673 271 675 288 l 493 409 b 391 369 466 384 430 369 b 239 521 307 369 239 437 b 391 673 239 605 307 673 b 493 633 430 673 466 658 l 677 744 b 673 781 674 756 673 768 b 825 933 673 865 741 933 b 977 781 909 933 977 865 b 825 629 977 697 909 629 b 722 669 785 629 749 644 l 538 559 b 543 521 540 548 543 531 b 538 483 543 510 540 493 l 722 373 b 825 412 749 397 785 412 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1195 975 l 1149 1021 l 1007 880 l 936 952 b 844 952 910 978 869 978 l 796 905 b 564 955 726 937 647 955 b 0 391 253 955 0 702 b 564 -173 0 79 253 -173 b 1128 391 876 -173 1128 79 b 1079 623 1128 474 1111 552 l 1126 670 b 1126 762 1151 696 1151 737 l 1054 834 z m 1356 1053 l 1291 1053 b 1259 1020 1273 1053 1259 1038 b 1291 987 1259 1002 1273 987 l 1356 987 b 1389 1020 1374 987 1389 1002 b 1356 1053 1389 1038 1374 1053 z m 1194 1215 b 1161 1183 1176 1215 1161 1201 l 1161 1118 b 1194 1085 1161 1100 1176 1085 b 1226 1118 1211 1085 1226 1100 l 1226 1183 b 1194 1215 1226 1201 1211 1215 z m 1286 1066 l 1332 1112 b 1332 1158 1344 1125 1344 1146 b 1286 1158 1319 1171 1298 1171 l 1239 1112 b 1239 1066 1227 1099 1227 1079 b 1286 1066 1252 1053 1273 1053 z m 1102 1066 b 1148 1066 1114 1053 1135 1053 b 1148 1112 1160 1079 1160 1099 l 1102 1158 b 1056 1158 1089 1171 1068 1171 b 1056 1112 1043 1146 1043 1125 z m 1286 974 b 1239 974 1273 987 1252 987 b 1239 928 1227 961 1227 940 l 1286 882 b 1332 882 1298 869 1319 869 b 1332 928 1344 894 1344 915 z m 304 477 b 260 434 304 454 284 434 b 217 477 237 434 217 454 b 477 738 217 621 334 738 b 521 694 501 738 521 718 b 477 651 521 671 501 651 b 304 477 382 651 304 573 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1367 521 b 694 1194 1367 892 1066 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 z m 1237 521 b 1133 201 1237 404 1200 293 l 1112 293 l 881 264 l 782 53 l 863 5 b 525 5 754 -31 635 -31 l 607 53 l 508 264 l 277 293 l 256 201 b 152 521 189 293 152 404 l 152 522 l 223 460 l 393 619 l 349 847 l 255 839 b 528 1037 323 932 418 1002 l 491 951 l 694 838 l 898 951 l 861 1038 b 1134 839 971 1002 1066 932 l 1041 847 l 996 619 l 1166 460 l 1237 522 z m 564 332 l 826 332 l 906 580 l 694 733 l 483 580 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 15 934 b 0 898 7 926 0 910 b 8 872 0 891 3 879 l 125 685 b 167 661 133 672 152 661 b 185 665 172 661 181 663 l 420 759 b 451 805 437 765 451 786 b 451 810 451 806 451 808 l 435 971 b 954 971 600 1031 786 1032 l 938 810 b 938 805 938 808 938 806 b 969 759 938 786 952 765 l 1203 665 b 1222 661 1208 663 1217 661 b 1264 685 1237 661 1256 672 l 1381 872 b 1374 934 1394 892 1391 917 b 15 934 999 1309 390 1309 z m 260 380 l 260 488 b 228 521 260 506 246 521 l 119 521 b 87 488 101 521 87 506 l 87 380 b 119 347 87 362 101 347 l 228 347 b 260 380 246 347 260 362 z m 521 380 l 521 488 b 488 521 521 506 506 521 l 380 521 b 347 488 362 521 347 506 l 347 380 b 380 347 347 362 362 347 l 488 347 b 521 380 506 347 521 362 z m 781 380 l 781 488 b 749 521 781 506 767 521 l 640 521 b 608 488 622 521 608 506 l 608 380 b 640 347 608 362 622 347 l 749 347 b 781 380 767 347 781 362 z m 1042 380 l 1042 488 b 1009 521 1042 506 1027 521 l 901 521 b 868 488 883 521 868 506 l 868 380 b 901 347 868 362 883 347 l 1009 347 b 1042 380 1027 347 1042 362 z m 1302 380 l 1302 488 b 1270 521 1302 506 1288 521 l 1161 521 b 1128 488 1143 521 1128 506 l 1128 380 b 1161 347 1128 362 1143 347 l 1270 347 b 1302 380 1288 347 1302 362 z m 391 119 l 391 228 b 358 260 391 246 376 260 l 250 260 b 217 228 232 260 217 246 l 217 119 b 250 87 217 101 232 87 l 358 87 b 391 119 376 87 391 101 z m 651 119 l 651 228 b 618 260 651 246 636 260 l 510 260 b 477 228 492 260 477 246 l 477 119 b 510 87 477 101 492 87 l 618 87 b 651 119 636 87 651 101 z m 911 119 l 911 228 b 879 260 911 246 897 260 l 770 260 b 738 228 752 260 738 246 l 738 119 b 770 87 738 101 752 87 l 879 87 b 911 119 897 87 911 101 z m 1172 119 l 1172 228 b 1139 260 1172 246 1157 260 l 1031 260 b 998 228 1013 260 998 246 l 998 119 b 1031 87 998 101 1013 87 l 1139 87 b 1172 119 1157 87 1172 101 z m 260 -141 l 260 -33 b 228 0 260 -15 246 0 l 119 0 b 87 -33 101 0 87 -15 l 87 -141 b 119 -174 87 -159 101 -174 l 228 -174 b 260 -141 246 -174 260 -159 z m 1042 -141 l 1042 -33 b 1009 0 1042 -15 1027 0 l 380 0 b 347 -33 362 0 347 -15 l 347 -141 b 380 -174 347 -159 362 -174 l 1009 -174 b 1042 -141 1027 -174 1042 -159 z m 1302 -141 l 1302 -33 b 1270 0 1302 -15 1288 0 l 1161 0 b 1128 -33 1143 0 1128 -15 l 1128 -141 b 1161 -174 1128 -159 1143 -174 l 1270 -174 b 1302 -141 1288 -174 1302 -159 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1128 1085 b 1085 1128 1128 1109 1109 1128 l 911 1128 b 868 1085 887 1128 868 1109 l 868 955 l 1128 955 z m 173 781 b 0 119 167 527 9 471 l 0 0 b 87 -87 0 -48 39 -87 l 347 -87 b 434 0 395 -87 434 -48 l 434 434 l 521 434 l 521 868 l 260 868 b 173 781 212 868 175 829 z m 1216 781 b 1129 868 1214 829 1177 868 l 868 868 l 868 434 l 955 434 l 955 0 b 1042 -87 955 -48 994 -87 l 1302 -87 b 1389 0 1350 -87 1389 -48 l 1389 119 b 1216 781 1380 471 1222 527 z m 477 1128 l 304 1128 b 260 1085 280 1128 260 1109 l 260 955 l 521 955 l 521 1085 b 477 1128 521 1109 501 1128 z m 608 434 l 781 434 l 781 868 l 608 868 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 694 825 l 868 825 l 868 1128 b 781 1215 868 1176 829 1215 b 694 1128 733 1215 694 1176 z m 998 781 l 43 781 b 0 738 19 781 0 762 l 0 651 b 43 608 0 627 19 608 l 87 608 l 87 521 b 434 95 87 311 236 136 l 434 -174 l 608 -174 l 608 95 b 955 521 806 136 955 311 l 955 608 l 998 608 b 1042 651 1022 608 1042 627 l 1042 738 b 998 781 1042 762 1022 781 z m 347 825 l 347 1128 b 260 1215 347 1176 308 1215 b 174 1128 212 1215 174 1176 l 174 825 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1497 1042 l 239 1042 b 174 977 203 1042 174 1013 l 174 955 l 65 955 b 0 890 29 955 0 926 l 0 152 b 152 0 0 68 68 0 l 1432 0 b 1563 130 1504 0 1563 58 l 1563 977 b 1497 1042 1563 1013 1533 1042 z m 152 130 b 130 152 140 130 130 140 l 130 825 l 174 825 l 174 152 b 152 130 174 140 164 130 z m 792 174 l 380 174 b 347 206 362 174 347 188 l 347 228 b 380 260 347 246 362 260 l 792 260 b 825 228 810 260 825 246 l 825 206 b 792 174 825 188 810 174 z m 1356 174 l 944 174 b 911 206 926 174 911 188 l 911 228 b 944 260 911 246 926 260 l 1356 260 b 1389 228 1374 260 1389 246 l 1389 206 b 1356 174 1389 188 1374 174 z m 792 434 l 380 434 b 347 467 362 434 347 449 l 347 488 b 380 521 347 506 362 521 l 792 521 b 825 488 810 521 825 506 l 825 467 b 792 434 825 449 810 434 z m 1356 434 l 944 434 b 911 467 926 434 911 449 l 911 488 b 944 521 911 506 926 521 l 1356 521 b 1389 488 1374 521 1389 506 l 1389 467 b 1356 434 1389 449 1374 434 z m 1356 694 l 380 694 b 347 727 362 694 347 709 l 347 836 b 380 868 347 853 362 868 l 1356 868 b 1389 836 1374 868 1389 853 l 1389 727 b 1356 694 1389 709 1374 694 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1722 795 b 14 795 1242 1240 494 1240 b 13 733 -4 778 -5 750 l 106 641 b 167 640 122 624 149 624 b 1570 640 562 1002 1174 1002 b 1630 641 1587 624 1614 624 l 1723 733 b 1722 795 1741 750 1740 778 z m 868 260 b 694 87 772 260 694 183 b 868 -87 694 -9 772 -87 b 1042 87 964 -87 1042 -9 b 868 260 1042 183 964 260 z m 1418 487 b 318 487 1105 764 631 763 b 317 424 300 471 299 442 l 410 332 b 470 330 426 316 453 315 b 1266 330 698 527 1039 526 b 1326 332 1283 315 1310 316 l 1419 424 b 1418 487 1437 442 1437 471 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1085 1215 l 130 1215 b 0 1085 61 1215 0 1155 l 0 -43 b 130 -174 0 -113 61 -174 l 1085 -174 b 1215 -43 1155 -174 1215 -113 l 1215 1085 b 1085 1215 1215 1155 1155 1215 z m 347 35 b 313 0 347 17 330 0 l 208 0 b 174 35 191 0 174 17 l 174 139 b 208 174 174 156 191 174 l 313 174 b 347 139 330 174 347 156 z m 347 382 b 313 347 347 365 330 347 l 208 347 b 174 382 191 347 174 365 l 174 486 b 208 521 174 503 191 521 l 313 521 b 347 486 330 521 347 503 z m 694 35 b 660 0 694 17 677 0 l 556 0 b 521 35 538 0 521 17 l 521 139 b 556 174 521 156 538 174 l 660 174 b 694 139 677 174 694 156 z m 694 382 b 660 347 694 365 677 347 l 556 347 b 521 382 538 347 521 365 l 521 486 b 556 521 521 503 538 521 l 660 521 b 694 486 677 521 694 503 z m 1042 35 b 1007 0 1042 17 1024 0 l 903 0 b 868 35 885 0 868 17 l 868 486 b 903 521 868 503 885 521 l 1007 521 b 1042 486 1024 521 1042 503 z m 1042 729 b 1007 694 1042 712 1024 694 l 208 694 b 174 729 191 694 174 712 l 174 1007 b 208 1042 174 1024 191 1042 l 1007 1042 b 1042 1007 1024 1042 1042 1024 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1719 -27 l 1474 163 b 1476 174 1474 166 1476 170 b 1452 233 1476 194 1469 215 b 1302 651 1400 289 1302 374 b 955 1072 1302 862 1154 1031 l 955 1128 b 868 1215 955 1176 916 1215 b 781 1128 820 1215 781 1176 l 781 1072 b 517 902 672 1049 579 988 l 123 1206 b 62 1199 104 1221 77 1217 l 9 1130 b 17 1069 -6 1111 -2 1084 l 1613 -164 b 1674 -157 1632 -179 1659 -176 l 1727 -88 b 1719 -27 1742 -69 1738 -42 z m 427 533 b 284 233 403 349 328 280 b 260 174 267 215 260 194 b 347 87 261 129 296 87 l 1004 87 z m 868 -174 b 1042 0 964 -174 1042 -96 l 695 0 b 868 -174 695 -96 772 -174 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1172 1128 l 846 1128 l 821 1179 b 763 1215 811 1199 785 1215 l 763 1215 l 452 1215 l 452 1215 b 394 1179 430 1215 404 1199 l 369 1128 l 43 1128 b 0 1085 19 1128 0 1109 l 0 998 b 43 955 0 974 19 955 l 1172 955 b 1215 998 1196 955 1215 974 l 1215 1085 b 1172 1128 1215 1109 1196 1128 z m 144 -52 b 274 -174 149 -119 207 -174 l 941 -174 b 1071 -52 1009 -174 1067 -119 l 1128 868 l 87 868 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 z m 1012 253 b 714 129 1008 248 904 129 b 322 524 484 129 322 301 b 712 913 322 744 490 913 b 993 807 894 913 989 812 b 1001 785 998 802 1001 792 b 996 768 1001 780 999 772 l 935 674 b 886 667 924 657 901 654 b 718 732 885 668 814 732 b 518 526 593 732 518 641 b 719 310 518 418 587 310 b 897 387 824 310 896 386 b 947 384 911 402 935 401 l 1014 293 b 1020 274 1017 289 1020 280 b 1012 253 1020 267 1016 258 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1194 b 22 521 323 1194 22 893 b 694 -152 22 149 323 -152 b 1062 -43 825 -152 953 -114 b 1077 53 1094 -22 1101 23 l 1049 87 b 964 101 1028 112 992 119 b 694 22 884 49 790 22 b 195 521 419 22 195 246 b 694 1020 195 796 419 1020 b 1194 586 966 1020 1194 864 b 1036 359 1194 481 1136 370 b 999 440 989 360 990 394 l 1063 769 b 999 846 1071 809 1040 846 l 877 846 l 877 846 b 840 814 859 846 842 832 l 840 814 b 678 873 801 862 731 873 b 304 462 475 873 304 704 b 564 175 304 285 404 175 b 768 279 637 175 720 217 b 959 186 793 186 878 186 b 1367 586 1255 186 1367 380 b 694 1194 1367 956 1069 1194 z m 636 368 b 538 478 575 368 538 410 b 697 676 538 600 621 676 b 793 565 757 676 793 634 b 636 368 793 443 702 368 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 138 311 b 87 189 105 279 87 235 l 87 65 l 0 -87 l 87 -174 l 239 -87 l 362 -87 b 485 -36 408 -87 452 -69 l 828 308 l 481 655 z m 1313 1139 b 944 1139 1211 1241 1046 1241 l 735 930 l 700 965 b 608 965 674 991 633 991 l 497 854 b 497 762 471 829 471 788 l 936 323 b 1028 323 961 297 1003 298 l 1139 434 b 1139 526 1164 459 1164 501 l 1103 562 l 1313 771 b 1313 1139 1414 872 1414 1037 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 453 376 b 189 180 344 369 245 328 b 150 153 183 163 168 153 b 0 246 120 153 27 228 b 347 -174 0 23 103 -174 b 694 152 553 -174 694 -55 b 692 177 694 161 693 169 z m 1242 1215 b 1133 1171 1201 1215 1162 1197 b 521 518 579 675 521 664 b 545 413 521 481 530 445 l 718 269 b 778 260 737 264 757 260 b 1351 956 947 260 1045 384 b 1389 1081 1371 995 1389 1037 b 1242 1215 1389 1159 1318 1215 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1215 174 l 1215 391 b 1085 521 1215 463 1157 521 l 1042 521 l 1042 911 l 868 911 l 868 521 l 694 521 l 694 911 l 521 911 l 521 521 l 347 521 l 347 911 l 174 911 l 174 521 l 130 521 b 0 391 58 521 0 463 l 0 174 b 203 260 76 174 85 260 b 406 174 321 260 329 174 b 608 260 480 174 491 260 b 810 174 725 260 735 174 b 1013 260 886 174 895 260 b 1215 174 1130 260 1139 174 z m 1215 -174 l 1215 87 b 1013 174 1098 87 1088 174 b 810 87 937 174 928 87 b 608 174 692 87 684 174 b 406 87 534 174 522 87 b 203 174 288 87 279 174 b 0 87 127 174 118 87 l 0 -174 z m 260 955 b 347 1063 309 955 347 991 b 260 1215 347 1135 293 1215 b 174 1042 260 1104 174 1126 b 260 955 174 994 212 955 z m 608 955 b 694 1063 656 955 694 991 b 608 1215 694 1135 640 1215 b 521 1042 608 1104 521 1126 b 608 955 521 994 559 955 z m 955 955 b 1042 1063 1003 955 1042 991 b 955 1215 1042 1135 987 1215 b 868 1042 955 1104 868 1126 b 955 955 868 994 907 955 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1356 174 l 174 174 l 174 1009 b 141 1042 174 1027 159 1042 l 33 1042 b 0 1009 15 1042 0 1027 l 0 33 b 33 0 0 15 15 0 l 1356 0 b 1389 33 1374 0 1389 15 l 1389 141 b 1356 174 1389 159 1374 174 z m 1011 783 l 781 629 l 550 938 b 496 935 536 956 508 955 l 260 543 l 260 260 l 1302 260 l 1058 770 b 1011 783 1050 787 1027 793 z "
	},
	"": {
		ha: 1476,
		x_min: 0,
		x_max: 0,
		o: "m 1432 434 l 788 434 l 1217 5 b 1277 3 1233 -11 1260 -12 b 1475 386 1382 102 1454 236 b 1432 434 1479 411 1458 434 z m 1389 610 b 783 1215 1366 934 1107 1193 b 738 1171 759 1217 738 1196 l 738 564 l 1345 564 b 1389 610 1370 564 1390 585 z m 608 434 l 608 1078 b 559 1121 608 1104 585 1125 b 0 455 236 1076 -11 793 b 659 -174 12 107 311 -178 b 1026 -54 796 -172 922 -128 b 1030 11 1048 -39 1049 -7 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1345 174 l 174 174 l 174 998 b 130 1042 174 1022 154 1042 l 43 1042 b 0 998 19 1042 0 1022 l 0 87 b 87 0 0 39 39 0 l 1345 0 b 1389 43 1369 0 1389 19 l 1389 130 b 1345 174 1389 154 1369 174 z m 1259 955 l 938 955 b 892 844 880 955 851 885 l 980 756 l 781 557 l 582 756 b 459 756 548 790 493 790 l 273 569 b 273 508 256 553 256 525 l 334 447 b 396 447 351 430 379 430 l 521 572 l 720 373 b 843 373 754 339 809 339 l 1103 633 l 1191 545 b 1302 591 1232 504 1302 533 l 1302 911 b 1259 955 1302 935 1283 955 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1042 1042 l 521 1042 b 0 521 233 1042 0 808 b 521 0 0 233 233 0 l 1042 0 b 1563 521 1329 0 1563 233 b 1042 1042 1563 808 1329 1042 z m 174 521 b 521 868 174 713 329 868 b 868 521 713 868 868 713 b 521 174 868 329 713 174 b 174 521 329 174 174 329 z m 1042 174 l 909 174 b 909 868 1086 371 1086 670 l 1042 868 b 1389 521 1234 868 1389 713 b 1042 174 1389 329 1234 174 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1042 1042 l 521 1042 b 0 521 233 1042 0 808 b 521 0 0 233 233 0 l 1042 0 b 1563 521 1329 0 1563 233 b 1042 1042 1563 808 1329 1042 z m 1042 174 b 694 521 850 174 694 329 b 1042 868 694 713 850 868 b 1389 521 1234 868 1389 713 b 1042 174 1389 329 1233 174 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1390 694 b 1264 671 1346 695 1303 686 l 1054 1011 b 998 1042 1043 1028 1018 1042 l 998 1042 l 825 1042 b 781 998 801 1042 781 1022 l 781 955 b 825 911 781 931 801 911 l 962 911 l 1002 846 l 694 846 l 694 890 b 651 933 694 914 675 933 l 414 933 b 347 867 377 933 347 903 b 412 803 348 831 377 803 l 545 803 l 465 674 b 348 694 428 687 389 694 b 0 349 158 695 1 540 b 347 0 -1 157 155 0 b 684 260 509 0 645 111 l 825 260 b 880 291 845 260 869 274 l 1113 668 l 1154 603 b 1042 346 1085 539 1041 448 b 1385 0 1042 160 1199 2 b 1736 342 1577 -2 1733 151 b 1390 694 1739 534 1583 694 z m 507 494 b 560 391 533 466 552 430 l 442 391 z m 347 130 b 130 347 228 130 130 228 b 347 564 130 467 228 564 b 394 559 363 564 379 562 l 270 360 b 326 260 243 316 275 260 l 546 260 b 347 130 513 184 436 130 z m 788 391 l 692 391 b 577 608 681 477 639 553 l 644 716 l 990 716 z m 1377 131 b 1172 339 1266 136 1176 227 b 1224 489 1170 396 1190 449 l 1345 294 b 1405 280 1358 274 1384 268 l 1442 303 b 1456 363 1462 316 1468 343 l 1335 557 b 1389 564 1350 561 1374 564 l 1389 564 b 1606 347 1509 564 1606 467 b 1377 131 1606 224 1502 124 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1324 868 l 1302 868 l 1302 998 b 694 1215 1302 1120 1033 1215 b 87 998 356 1215 87 1120 l 87 868 l 65 868 b 0 803 29 868 0 839 l 0 586 b 65 521 0 550 29 521 l 87 521 l 87 87 b 174 0 87 39 126 0 l 174 -87 b 260 -174 174 -135 212 -174 l 347 -174 b 434 -87 395 -174 434 -135 l 434 0 l 955 0 l 955 -87 b 1042 -174 955 -135 994 -174 l 1128 -174 b 1215 -87 1176 -174 1215 -135 l 1215 0 l 1233 0 b 1302 69 1276 0 1302 35 l 1302 521 l 1324 521 b 1389 586 1360 521 1389 550 l 1389 803 b 1324 868 1389 839 1360 868 z m 304 130 b 217 217 256 130 217 169 b 304 304 217 265 256 304 b 391 217 352 304 391 265 b 304 130 391 169 352 130 z m 347 434 b 260 521 299 434 260 473 l 260 868 b 347 955 260 916 299 955 l 1042 955 b 1128 868 1090 955 1128 916 l 1128 521 b 1042 434 1128 473 1090 434 z m 1085 130 b 998 217 1037 130 998 169 b 1085 304 998 265 1037 304 b 1172 217 1133 304 1172 265 b 1085 130 1172 169 1133 130 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1259 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 130 b 130 0 0 58 58 0 l 1259 0 b 1389 130 1331 0 1389 58 l 1389 911 b 1259 1042 1389 983 1331 1042 z m 592 435 b 617 432 599 442 611 441 l 669 357 b 668 336 674 351 674 342 b 199 521 523 182 199 249 b 667 711 199 785 530 845 b 670 695 673 705 674 702 l 623 613 b 598 608 617 604 606 602 b 341 523 487 695 341 648 b 592 435 342 393 480 332 z m 1108 435 b 1133 432 1116 442 1127 441 l 1186 357 b 1185 336 1191 351 1190 342 b 716 521 1039 182 716 249 b 1184 711 716 785 1046 845 b 1187 695 1190 705 1191 702 l 1139 613 b 1115 608 1134 604 1123 602 b 858 523 1004 695 858 648 b 1108 435 858 393 996 332 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 673 760 l 673 304 b 716 260 673 280 692 260 l 846 260 b 890 304 870 260 890 280 l 890 760 b 521 1128 890 963 725 1128 l 65 1128 b 0 1063 29 1128 0 1099 l 0 -43 b 43 -87 0 -67 19 -87 l 174 -87 b 217 -43 198 -87 217 -67 l 217 911 l 521 911 b 673 760 605 911 673 843 z m 1172 1128 l 1042 1128 b 998 1085 1018 1128 998 1109 l 998 282 b 846 130 998 198 930 130 l 543 130 l 543 738 b 499 781 543 762 523 781 l 369 781 b 326 738 345 781 326 762 l 326 -22 b 391 -87 326 -58 355 -87 l 846 -87 b 1215 282 1050 -87 1215 78 l 1215 1085 b 1172 1128 1215 1109 1196 1128 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1369 347 b 1433 398 1400 347 1426 368 l 1561 962 b 1497 1042 1570 1003 1539 1042 l 432 1042 l 407 1163 b 343 1215 401 1194 374 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 1107 b 65 1042 0 1071 29 1042 l 255 1042 l 445 110 b 369 -30 397 83 366 30 b 514 -173 373 -107 436 -170 b 673 -22 601 -177 673 -108 b 627 87 673 21 655 59 l 1196 87 b 1150 -26 1167 58 1149 18 b 1300 -174 1152 -107 1219 -172 b 1454 -22 1385 -175 1454 -106 b 1368 115 1454 38 1419 90 l 1383 181 b 1320 260 1393 222 1362 260 l 592 260 l 574 347 z m 1107 760 b 1150 716 1131 760 1150 740 l 1150 673 b 1107 629 1150 649 1131 629 l 977 629 l 977 521 b 933 477 977 497 957 477 l 890 477 b 846 521 866 477 846 497 l 846 629 l 716 629 b 673 673 692 629 673 649 l 673 716 b 716 760 673 740 692 760 l 846 760 l 846 868 b 890 911 846 892 866 911 l 933 911 b 977 868 957 911 977 892 l 977 760 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1369 347 b 1433 398 1400 347 1426 368 l 1561 962 b 1497 1042 1570 1003 1539 1042 l 432 1042 l 407 1163 b 343 1215 401 1194 374 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 1107 b 65 1042 0 1071 29 1042 l 255 1042 l 445 110 b 369 -30 397 83 366 30 b 514 -173 373 -107 436 -170 b 673 -22 601 -177 673 -108 b 627 87 673 21 655 59 l 1196 87 b 1150 -26 1167 58 1149 18 b 1300 -174 1152 -107 1219 -172 b 1454 -22 1385 -175 1454 -106 b 1368 115 1454 38 1419 90 l 1383 181 b 1320 260 1393 222 1362 260 l 592 260 l 574 347 z m 1093 694 b 1116 639 1122 694 1137 659 l 934 457 b 888 457 922 444 901 444 l 707 639 b 730 694 686 659 701 694 l 846 694 l 846 857 b 879 890 846 875 861 890 l 944 890 b 977 857 962 890 977 875 l 977 694 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1347 204 l 1537 394 b 1502 538 1583 440 1564 519 l 1389 575 l 1389 955 b 1302 1042 1389 1003 1350 1042 l 1128 1042 l 1128 1150 b 1063 1215 1128 1186 1099 1215 l 673 1215 b 608 1150 637 1215 608 1186 l 608 1042 l 434 1042 b 347 955 386 1042 347 1003 l 347 575 l 234 538 b 199 394 172 519 153 440 l 389 204 b 65 0 341 86 232 0 b 0 -65 29 0 0 -29 l 0 -109 b 65 -174 0 -144 29 -174 b 454 -12 231 -174 357 -118 b 694 -174 493 -107 586 -174 l 1042 -174 b 1282 -12 1150 -174 1243 -107 b 1671 -174 1379 -118 1505 -174 b 1736 -109 1707 -174 1736 -144 l 1736 -65 b 1671 0 1736 -29 1707 0 b 1347 204 1506 0 1396 84 z m 521 868 l 1215 868 l 1215 631 l 895 734 b 868 738 887 736 876 738 b 841 734 861 738 849 736 l 521 631 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1041 379 l 1106 549 b 1066 608 1117 577 1096 608 l 907 608 b 955 781 937 659 955 718 l 955 782 b 1129 868 1061 803 1129 834 b 939 958 1129 904 1055 936 b 828 1182 914 1047 865 1136 b 721 1206 803 1214 758 1225 l 646 1169 b 569 1169 622 1156 593 1156 l 494 1206 b 387 1182 457 1225 413 1214 b 277 958 350 1136 302 1047 b 87 868 161 936 87 904 b 260 782 87 834 154 803 l 260 781 b 309 608 260 718 279 659 l 153 608 b 113 547 122 608 101 576 l 183 384 b 0 69 74 321 0 204 l 0 -52 b 122 -174 0 -119 55 -174 l 1094 -174 b 1215 -52 1161 -174 1215 -119 l 1215 69 b 1041 379 1215 201 1145 315 z m 477 -87 l 365 434 l 499 347 l 564 239 z m 738 -87 l 651 239 l 716 347 l 851 434 z m 851 723 b 806 632 840 691 832 656 b 633 700 779 607 676 572 b 583 700 625 723 591 723 b 409 632 537 564 431 612 b 365 723 384 656 375 691 b 347 739 362 730 347 738 l 347 768 b 608 752 424 758 513 752 b 868 768 703 752 792 758 l 868 739 b 851 723 868 738 853 730 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1391 694 b 1276 676 1351 695 1312 688 l 1187 825 l 1411 825 b 1476 890 1447 825 1476 854 l 1476 977 b 1411 1042 1476 1013 1447 1042 l 1288 1042 b 1239 1020 1269 1042 1252 1034 l 1138 907 l 1076 1010 b 1020 1042 1064 1030 1043 1042 l 803 1042 b 760 998 779 1042 760 1022 l 760 955 b 803 911 760 931 779 911 l 983 911 l 1035 825 l 618 825 b 347 933 570 887 496 933 l 197 933 b 130 867 160 933 129 903 b 195 803 131 831 160 803 l 347 803 b 477 736 414 803 452 773 l 446 680 b 334 694 411 691 373 696 b 0 355 152 688 4 538 b 347 0 -4 160 153 0 b 684 260 509 0 645 111 l 912 260 b 977 329 949 260 979 291 b 1130 668 971 456 1025 583 l 1163 611 b 1042 345 1089 547 1041 451 b 1386 0 1043 156 1197 2 b 1736 350 1580 -2 1738 156 b 1391 694 1734 539 1580 693 z m 347 130 b 130 347 228 130 130 228 b 347 564 130 467 228 564 b 381 562 359 564 370 563 l 269 357 b 326 260 245 314 276 260 l 546 260 b 347 130 512 184 436 130 z m 1606 335 b 1401 130 1600 225 1511 136 b 1172 347 1276 124 1172 224 b 1232 497 1172 405 1195 458 l 1366 273 b 1426 258 1378 253 1405 246 l 1463 280 b 1478 340 1483 293 1490 320 l 1346 560 b 1389 564 1360 563 1374 564 b 1606 335 1513 564 1612 460 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 998 321 b 955 284 985 306 971 293 l 955 221 b 1259 87 1135 196 1259 145 b 694 -65 1259 3 1006 -65 b 130 87 383 -65 130 3 b 434 221 130 145 254 196 l 434 284 b 391 321 417 293 403 306 b 0 87 160 278 0 190 b 694 -174 0 -57 311 -174 b 1389 87 1078 -174 1389 -57 b 998 321 1389 190 1229 278 z m 694 868 b 868 1042 790 868 868 946 b 694 1215 868 1138 790 1215 b 521 1042 599 1215 521 1138 b 694 868 521 946 599 868 z m 521 347 l 521 87 b 608 0 521 39 560 0 l 781 0 b 868 87 829 0 868 39 l 868 347 b 955 434 916 347 955 386 l 955 694 b 825 825 955 766 897 825 l 793 825 b 694 803 763 811 730 803 b 596 825 659 803 626 811 l 564 825 b 434 694 492 825 434 766 l 434 434 b 521 347 434 386 473 347 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 869 554 l 734 284 b 655 286 718 251 670 253 l 501 629 l 420 434 l 164 434 l 659 -72 b 729 -72 679 -92 710 -92 l 1225 434 l 929 434 z m 1285 1015 l 1278 1022 b 770 1022 1139 1164 910 1164 l 694 944 l 619 1021 b 110 1021 479 1164 250 1164 l 104 1015 b 84 521 -28 880 -34 665 l 362 521 l 459 755 b 539 756 474 790 523 790 l 697 405 l 830 671 b 907 671 846 703 891 703 l 982 521 l 1305 521 b 1285 1015 1423 665 1417 880 z "
	},
	"": {
		ha: 781,
		x_min: 0,
		x_max: 0,
		o: "m 781 738 b 391 1128 781 954 606 1128 b 0 738 175 1128 0 954 b 304 357 0 552 130 396 l 304 217 l 206 217 b 174 184 188 217 174 202 l 174 76 b 206 43 174 58 188 43 l 304 43 l 304 -54 b 336 -87 304 -72 318 -87 l 445 -87 b 477 -54 463 -87 477 -72 l 477 43 l 575 43 b 608 76 593 43 608 58 l 608 184 b 575 217 608 202 593 217 l 477 217 l 477 357 b 781 738 651 396 781 552 z m 174 738 b 391 955 174 857 271 955 b 608 738 510 955 608 857 b 391 521 608 618 510 521 b 174 738 271 521 174 618 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 1009 1042 l 795 1042 b 772 986 766 1042 751 1007 l 818 940 l 599 721 b 391 781 538 759 467 781 b 0 391 175 781 0 606 b 391 0 0 175 175 0 b 781 391 606 0 781 175 b 721 599 781 467 759 538 l 940 818 l 986 772 b 1042 795 1007 751 1042 766 l 1042 1009 b 1009 1042 1042 1027 1027 1042 z m 391 174 b 174 391 271 174 174 271 b 391 608 174 510 271 608 b 608 391 510 608 608 510 b 391 174 608 271 510 174 z "
	},
	"": {
		ha: 781,
		x_min: 0,
		x_max: 0,
		o: "m 781 651 b 642 950 781 771 727 878 b 662 965 649 955 656 960 b 779 1179 729 1023 770 1098 b 747 1215 782 1198 766 1215 l 637 1215 b 605 1189 621 1215 608 1204 b 549 1097 598 1155 579 1123 b 391 1042 507 1061 451 1042 b 232 1097 330 1042 274 1061 b 176 1189 202 1123 183 1155 b 144 1215 173 1204 160 1215 l 34 1215 b 2 1179 15 1215 0 1198 b 119 965 11 1098 52 1023 b 139 949 126 960 132 955 b 0 651 54 878 0 771 b 304 270 0 465 130 310 l 304 130 l 206 130 b 174 98 188 130 174 116 l 174 -11 b 206 -43 174 -29 188 -43 l 304 -43 l 304 -141 b 336 -174 304 -159 318 -174 l 445 -174 b 477 -141 463 -174 477 -159 l 477 -43 l 575 -43 b 608 -11 593 -43 608 -29 l 608 98 b 575 130 608 116 593 130 l 477 130 l 477 270 b 781 651 651 310 781 465 z m 174 651 b 391 868 174 771 271 868 b 608 651 510 868 608 771 b 391 434 608 531 510 434 b 174 651 271 434 174 531 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 1009 1215 l 795 1215 b 772 1160 766 1215 751 1180 l 818 1114 l 599 895 b 391 955 538 933 467 955 b 0 564 175 955 0 780 b 304 183 0 378 130 223 l 304 109 l 206 109 b 174 76 188 109 174 94 l 174 -33 b 206 -65 174 -50 188 -65 l 304 -65 l 304 -141 b 336 -174 304 -159 318 -174 l 445 -174 b 477 -141 463 -174 477 -159 l 477 -65 l 575 -65 b 608 -33 593 -65 608 -50 l 608 76 b 575 109 608 94 593 109 l 477 109 l 477 183 b 781 564 651 223 781 378 b 721 772 781 641 759 712 l 940 991 l 986 945 b 1042 968 1007 925 1042 939 l 1042 1183 b 1009 1215 1042 1201 1027 1215 z m 391 347 b 174 564 271 347 174 445 b 391 781 174 684 271 781 b 608 564 510 781 608 684 b 391 347 608 445 510 347 z "
	},
	"": {
		ha: 1302,
		x_min: 0,
		x_max: 0,
		o: "m 1270 1215 l 1055 1215 b 1032 1160 1026 1215 1012 1180 l 1078 1114 l 859 895 b 651 955 799 933 728 955 b 443 895 575 955 503 933 l 398 940 l 452 993 b 452 1039 465 1006 465 1027 l 375 1116 b 329 1116 362 1129 342 1129 l 275 1063 l 224 1114 l 270 1160 b 247 1215 291 1180 276 1215 l 33 1215 b 0 1183 15 1215 0 1201 l 0 968 b 56 945 0 939 35 925 l 101 991 l 153 940 l 99 886 b 99 840 87 873 87 853 l 176 763 b 222 763 189 750 209 750 l 276 817 l 321 772 b 260 564 282 712 260 641 b 564 183 260 378 390 223 l 564 109 l 467 109 b 434 76 449 109 434 94 l 434 -33 b 467 -65 434 -50 449 -65 l 564 -65 l 564 -141 b 597 -174 564 -159 579 -174 l 705 -174 b 738 -141 723 -174 738 -159 l 738 -65 l 836 -65 b 868 -33 853 -65 868 -50 l 868 76 b 836 109 868 94 853 109 l 738 109 l 738 183 b 1042 564 912 223 1042 378 b 982 772 1042 641 1020 712 l 1201 991 l 1246 945 b 1302 968 1267 925 1302 939 l 1302 1183 b 1270 1215 1302 1201 1287 1215 z m 651 347 b 434 564 531 347 434 445 b 651 781 434 684 531 781 b 868 564 771 781 868 684 b 651 347 868 445 771 347 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 781 738 b 391 1128 781 954 606 1128 b 0 738 175 1128 0 954 b 304 357 0 552 130 396 l 304 217 l 206 217 b 174 184 188 217 174 202 l 174 76 b 206 43 174 58 188 43 l 304 43 l 304 -54 b 336 -87 304 -72 318 -87 l 445 -87 b 477 -54 463 -87 477 -72 l 477 43 l 575 43 b 608 76 593 43 608 58 l 608 184 b 575 217 608 202 593 217 l 477 217 l 477 357 b 781 738 651 396 781 552 z m 174 738 b 391 955 174 857 271 955 b 608 738 510 955 608 857 b 391 521 608 618 510 521 b 174 738 271 521 174 618 z m 1085 357 b 1389 738 1259 396 1389 552 b 998 1128 1389 954 1214 1128 b 756 1045 907 1128 823 1097 b 843 890 795 999 824 947 b 998 955 883 930 938 955 b 1215 738 1118 955 1215 857 b 998 521 1215 618 1118 521 b 843 586 937 521 883 546 b 756 431 824 529 794 476 b 911 357 801 396 854 370 l 911 217 l 814 217 b 781 184 796 217 781 202 l 781 76 b 814 43 781 58 796 43 l 911 43 l 911 -54 b 944 -87 911 -72 926 -87 l 1053 -87 b 1085 -54 1070 -87 1085 -72 l 1085 43 l 1183 43 b 1215 76 1201 43 1215 58 l 1215 184 b 1183 217 1215 202 1201 217 l 1085 217 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 922 1215 l 708 1215 b 685 1160 679 1215 665 1180 l 731 1114 l 599 982 b 391 1042 538 1020 467 1042 b 0 651 175 1042 0 867 b 391 260 0 435 175 260 b 781 651 606 260 781 435 b 721 859 781 728 759 799 l 853 991 l 899 945 b 922 936 906 939 914 936 b 955 968 939 936 955 949 l 955 1183 b 922 1215 955 1201 940 1215 z m 391 434 b 174 651 271 434 174 531 b 391 868 174 771 271 868 b 608 651 510 868 608 771 b 391 434 608 531 510 434 z m 1356 782 l 1142 782 b 1119 726 1113 782 1099 747 l 1165 680 l 1033 548 b 866 606 983 579 927 599 b 816 434 860 547 844 488 b 825 434 819 434 822 434 b 1042 217 944 434 1042 337 b 825 0 1042 98 944 0 b 608 217 705 0 608 98 b 608 226 608 220 608 223 b 436 176 554 198 495 181 b 825 -174 457 -21 623 -174 b 1215 217 1040 -174 1215 1 b 1155 425 1215 294 1193 365 l 1287 557 l 1333 511 b 1356 502 1340 505 1348 502 b 1389 534 1373 502 1389 515 l 1389 749 b 1356 782 1389 767 1374 782 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1530 1215 l 1316 1215 b 1293 1160 1287 1215 1272 1180 l 1338 1114 l 1206 982 b 998 1042 1146 1020 1075 1042 b 756 958 907 1042 823 1010 b 843 803 794 913 824 860 b 998 868 883 843 937 868 b 1215 651 1118 868 1215 771 b 998 434 1215 531 1118 434 b 843 499 938 434 883 459 b 756 344 824 442 795 390 b 998 260 823 292 907 260 b 1389 651 1214 260 1389 435 b 1329 859 1389 728 1367 799 l 1461 991 l 1507 945 b 1530 936 1513 939 1522 936 b 1563 968 1546 936 1563 949 l 1563 1183 b 1530 1215 1563 1201 1548 1215 z m 391 1042 b 0 651 175 1042 0 867 b 304 270 0 465 130 310 l 304 130 l 206 130 b 174 98 188 130 174 116 l 174 -11 b 206 -43 174 -29 188 -43 l 304 -43 l 304 -141 b 336 -174 304 -159 318 -174 l 445 -174 b 477 -141 463 -174 477 -159 l 477 -43 l 575 -43 b 608 -11 593 -43 608 -29 l 608 98 b 575 130 608 116 593 130 l 477 130 l 477 270 b 781 651 651 310 781 465 b 391 1042 781 867 606 1042 z m 391 434 b 174 651 271 434 174 531 b 391 868 174 771 271 868 b 608 651 510 868 608 771 b 391 434 608 531 510 434 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 1009 1042 l 795 1042 b 772 986 766 1042 751 1007 l 818 940 l 770 893 l 732 931 b 686 931 719 944 699 944 l 609 854 b 609 808 596 842 596 821 l 647 770 l 598 721 b 390 781 538 759 467 781 b 0 391 175 781 0 606 b 391 0 0 175 175 0 b 781 391 606 0 781 175 b 721 599 781 467 759 538 l 770 648 l 808 609 b 854 609 821 597 842 597 l 931 686 b 931 732 944 699 944 719 l 893 770 l 940 818 l 986 772 b 1042 795 1007 751 1042 766 l 1042 1009 b 1009 1042 1042 1027 1027 1042 z m 391 174 b 174 391 271 174 174 271 b 391 608 174 510 271 608 b 608 391 510 608 608 510 b 391 174 608 271 510 174 z "
	},
	"": {
		ha: 781,
		x_min: 0,
		x_max: 0,
		o: "m 667 580 b 477 685 613 634 547 669 l 477 754 l 532 754 b 564 786 550 754 564 768 l 564 895 b 532 927 564 913 550 927 l 477 927 l 477 994 l 542 994 b 565 1050 571 994 586 1029 l 414 1201 b 368 1201 401 1214 380 1214 l 216 1050 b 239 994 195 1029 210 994 l 304 994 l 304 927 l 250 927 b 217 895 232 927 217 913 l 217 786 b 250 754 217 768 232 754 l 304 754 l 304 685 b 114 580 234 669 168 634 b 114 28 -38 428 -38 180 b 667 28 267 -125 514 -125 b 667 580 819 180 819 428 z m 544 150 b 237 150 460 66 322 66 b 237 457 153 235 153 372 b 544 457 322 542 460 542 b 544 150 629 373 629 235 z "
	},
	"": {
		ha: 1302,
		x_min: 0,
		x_max: 0,
		o: "m 1292 544 l 1140 696 b 1085 672 1120 716 1085 701 l 1085 608 l 1020 608 l 1020 662 b 987 694 1020 680 1005 694 l 879 694 b 846 662 861 694 846 680 l 846 608 l 771 608 b 667 797 756 677 721 743 b 114 797 514 949 267 949 b 114 245 -38 645 -38 397 b 667 245 267 92 514 92 b 771 434 721 299 756 365 l 846 434 l 846 380 b 879 347 846 362 861 347 l 987 347 b 1020 380 1005 347 1020 362 l 1020 434 l 1085 434 l 1085 369 b 1140 346 1085 340 1120 326 l 1292 498 b 1292 544 1305 511 1305 531 z m 544 367 b 237 367 460 283 322 283 b 237 674 153 452 153 589 b 544 674 322 759 460 759 b 544 367 629 590 629 452 z "
	},
	"": {
		ha: 781,
		x_min: 0,
		x_max: 0,
		o: "m 781 738 b 391 1128 781 954 606 1128 b 0 738 175 1128 0 954 b 304 357 0 552 130 396 l 304 -54 b 336 -87 304 -72 318 -87 l 445 -87 b 477 -54 463 -87 477 -72 l 477 357 b 781 738 651 396 781 552 z m 391 521 b 174 738 271 521 174 618 b 391 955 174 857 271 955 b 608 738 510 955 608 857 b 391 521 608 618 510 521 z "
	},
	"": {
		ha: 781,
		x_min: 0,
		x_max: 0,
		o: "m 391 738 b 608 521 510 738 608 640 b 391 304 608 401 510 304 b 174 521 271 304 174 401 b 391 738 174 640 271 738 z m 391 911 b 0 521 175 911 0 736 b 391 130 0 305 175 130 b 781 521 606 130 781 305 b 391 911 781 736 606 911 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1302 781 b 1389 868 1350 781 1389 820 l 1389 1042 b 1302 1128 1389 1090 1350 1128 l 87 1128 b 0 1042 39 1128 0 1090 l 0 868 b 87 781 0 820 39 781 z m 1172 1020 b 1237 955 1208 1020 1237 991 b 1172 890 1237 919 1208 890 b 1107 955 1136 890 1107 919 b 1172 1020 1107 991 1136 1020 z m 998 1020 b 1063 955 1034 1020 1063 991 b 998 890 1063 919 1034 890 b 933 955 962 890 933 919 b 998 1020 933 991 962 1020 z m 1302 347 b 1389 434 1350 347 1389 386 l 1389 608 b 1302 694 1389 656 1350 694 l 87 694 b 0 608 39 694 0 656 l 0 434 b 87 347 0 386 39 347 z m 1172 586 b 1237 521 1208 586 1237 557 b 1172 456 1237 485 1208 456 b 1107 521 1136 456 1107 485 b 1172 586 1107 557 1136 586 z m 998 586 b 1063 521 1034 586 1063 557 b 998 456 1063 485 1034 456 b 933 521 962 456 933 485 b 998 586 933 557 962 586 z m 1302 -87 b 1389 0 1350 -87 1389 -48 l 1389 174 b 1302 260 1389 222 1350 260 l 87 260 b 0 174 39 260 0 222 l 0 0 b 87 -87 0 -48 39 -87 z m 1172 152 b 1237 87 1208 152 1237 123 b 1172 22 1237 51 1208 22 b 1107 87 1136 22 1107 51 b 1172 152 1107 123 1136 152 z m 998 152 b 1063 87 1034 152 1063 123 b 998 22 1063 51 1034 22 b 933 87 962 22 933 51 b 998 152 933 123 962 152 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1693 651 l 1519 651 l 1519 825 b 1476 868 1519 849 1500 868 l 1389 868 b 1345 825 1365 868 1345 849 l 1345 651 l 1172 651 b 1128 608 1148 651 1128 632 l 1128 521 b 1172 477 1128 497 1148 477 l 1345 477 l 1345 304 b 1389 260 1345 280 1365 260 l 1476 260 b 1519 304 1500 260 1519 280 l 1519 477 l 1693 477 b 1736 521 1717 477 1736 497 l 1736 608 b 1693 651 1736 632 1717 651 z m 608 521 b 955 868 799 521 955 676 b 608 1215 955 1060 799 1215 b 260 868 416 1215 260 1060 b 608 521 260 676 416 521 z m 851 434 l 805 434 b 608 391 745 406 678 391 b 410 434 537 391 470 406 l 365 434 b 0 69 163 434 0 271 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 69 b 851 434 1215 271 1052 434 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1599 564 l 1723 688 b 1723 750 1740 705 1740 733 l 1661 812 b 1599 812 1644 829 1616 829 l 1476 688 l 1352 812 b 1290 812 1335 829 1307 829 l 1228 750 b 1228 688 1211 733 1211 705 l 1352 564 l 1228 441 b 1228 379 1211 423 1211 396 l 1290 317 b 1352 317 1307 300 1335 300 l 1476 441 l 1599 317 b 1661 317 1616 300 1644 300 l 1723 379 b 1723 441 1740 396 1740 423 z m 608 521 b 955 868 799 521 955 676 b 608 1215 955 1060 799 1215 b 260 868 416 1215 260 1060 b 608 521 260 676 416 521 z m 851 434 l 805 434 b 608 391 745 406 678 391 b 410 434 537 391 470 406 l 365 434 b 0 69 163 434 0 271 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 69 b 851 434 1215 271 1052 434 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 477 521 b 694 738 597 521 694 618 b 477 955 694 858 597 955 b 260 738 358 955 260 858 b 477 521 260 618 358 521 z m 1432 868 l 825 868 b 781 825 801 868 781 849 l 781 434 l 174 434 l 174 998 b 130 1042 174 1022 154 1042 l 43 1042 b 0 998 19 1042 0 1022 l 0 43 b 43 0 0 19 19 0 l 130 0 b 174 43 154 0 174 19 l 174 174 l 1563 174 l 1563 43 b 1606 0 1563 19 1582 0 l 1693 0 b 1736 43 1717 0 1736 19 l 1736 564 b 1432 868 1736 732 1600 868 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1215 955 b 868 1215 1215 1099 1044 1215 l 347 1215 b 0 955 174 1215 0 1099 l 0 260 b 353 0 0 119 168 0 l 182 -135 b 195 -174 166 -148 175 -174 l 1020 -174 b 1033 -135 1040 -174 1050 -148 l 863 0 b 1215 260 1048 0 1215 120 z m 1085 586 b 1020 521 1085 550 1056 521 l 195 521 b 130 586 159 521 130 550 l 130 890 b 195 955 130 926 159 955 l 1020 955 b 1085 890 1056 955 1085 926 z m 608 412 b 760 260 692 412 760 344 b 608 109 760 177 692 109 b 456 260 524 109 456 177 b 608 412 456 344 524 412 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1215 955 b 868 1215 1215 1099 1044 1215 l 347 1215 b 0 955 174 1215 0 1099 l 0 260 b 353 0 0 119 168 0 l 182 -135 b 195 -174 166 -148 175 -174 l 1020 -174 b 1033 -135 1040 -174 1050 -148 l 863 0 b 1215 260 1048 0 1215 120 z m 543 586 b 477 521 543 550 513 521 l 195 521 b 130 586 159 521 130 550 l 130 890 b 195 955 130 926 159 955 l 477 955 b 543 890 513 955 543 926 z m 1085 586 b 1020 521 1085 550 1056 521 l 738 521 b 673 586 702 521 673 550 l 673 890 b 738 955 673 926 702 955 l 1020 955 b 1085 890 1056 955 1085 926 z m 955 434 b 1085 304 1027 434 1085 376 b 955 174 1085 232 1027 174 b 825 304 883 174 825 232 b 955 434 825 376 883 434 z m 260 434 b 391 304 332 434 391 376 b 260 174 391 232 332 174 b 130 304 189 174 130 232 b 260 434 130 376 189 434 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1476 781 l 1476 608 l 1563 608 l 1563 434 l 1476 434 l 1476 260 l 174 260 l 174 781 z m 1519 955 l 130 955 b 0 825 58 955 0 897 l 0 217 b 130 87 0 145 58 87 l 1519 87 b 1649 217 1591 87 1649 145 l 1649 260 l 1671 260 b 1736 326 1707 260 1736 290 l 1736 716 b 1671 781 1736 752 1707 781 l 1649 781 l 1649 825 b 1519 955 1649 897 1591 955 z m 1389 694 l 260 694 l 260 347 l 1389 347 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1476 781 l 1476 608 l 1563 608 l 1563 434 l 1476 434 l 1476 260 l 174 260 l 174 781 z m 1519 955 l 130 955 b 0 825 58 955 0 897 l 0 217 b 130 87 0 145 58 87 l 1519 87 b 1649 217 1591 87 1649 145 l 1649 260 l 1671 260 b 1736 326 1707 260 1736 290 l 1736 716 b 1671 781 1736 752 1707 781 l 1649 781 l 1649 825 b 1519 955 1649 897 1591 955 z m 1128 694 l 260 694 l 260 347 l 1128 347 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1476 781 l 1476 608 l 1563 608 l 1563 434 l 1476 434 l 1476 260 l 174 260 l 174 781 z m 1519 955 l 130 955 b 0 825 58 955 0 897 l 0 217 b 130 87 0 145 58 87 l 1519 87 b 1649 217 1591 87 1649 145 l 1649 260 l 1671 260 b 1736 326 1707 260 1736 290 l 1736 716 b 1671 781 1736 752 1707 781 l 1649 781 l 1649 825 b 1519 955 1649 897 1591 955 z m 868 694 l 260 694 l 260 347 l 868 347 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1476 781 l 1476 608 l 1563 608 l 1563 434 l 1476 434 l 1476 260 l 174 260 l 174 781 z m 1519 955 l 130 955 b 0 825 58 955 0 897 l 0 217 b 130 87 0 145 58 87 l 1519 87 b 1649 217 1591 87 1649 145 l 1649 260 l 1671 260 b 1736 326 1707 260 1736 290 l 1736 716 b 1671 781 1736 752 1707 781 l 1649 781 l 1649 825 b 1519 955 1649 897 1591 955 z m 608 694 l 260 694 l 260 347 l 608 347 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1476 781 l 1476 608 l 1563 608 l 1563 434 l 1476 434 l 1476 260 l 174 260 l 174 781 z m 1519 955 l 130 955 b 0 825 58 955 0 897 l 0 217 b 130 87 0 145 58 87 l 1519 87 b 1649 217 1591 87 1649 145 l 1649 260 l 1671 260 b 1736 326 1707 260 1736 290 l 1736 716 b 1671 781 1736 752 1707 781 l 1649 781 l 1649 825 b 1519 955 1649 897 1591 955 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 820 322 b 854 407 862 322 885 377 l 82 1201 b 0 1166 54 1232 0 1210 l 0 0 b 82 -35 0 -42 51 -67 l 317 207 l 461 -143 b 524 -170 472 -168 500 -181 l 658 -112 b 683 -46 682 -101 694 -72 l 532 322 z "
	},
	"": {
		ha: 694,
		x_min: 0,
		x_max: 0,
		o: "m 694 1074 l 694 1183 b 662 1215 694 1200 680 1215 b 347 1112 574 1216 452 1215 b 32 1215 245 1213 127 1216 b 0 1182 14 1215 0 1200 l 0 1075 b 33 1042 0 1057 15 1042 b 260 911 103 1043 260 1033 l 260 608 l 163 608 b 130 575 145 608 130 593 l 130 467 b 163 434 130 449 145 434 l 260 434 l 260 130 b 33 0 260 8 108 0 b 0 -32 15 0 0 -14 l 0 -141 b 32 -173 0 -159 14 -173 b 347 -71 120 -175 243 -174 b 662 -173 450 -172 567 -174 b 694 -141 680 -173 694 -159 l 694 -33 b 662 -1 694 -15 680 0 b 434 130 591 -1 434 8 l 434 434 l 532 434 b 564 467 550 434 564 449 l 564 575 b 532 608 564 593 550 608 l 434 608 l 434 911 b 662 1042 434 1033 586 1042 b 694 1074 680 1042 694 1056 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1302 868 l 1302 955 l 1356 955 b 1389 987 1374 955 1389 969 l 1389 1096 b 1356 1128 1389 1114 1374 1128 l 1248 1128 b 1215 1096 1230 1128 1215 1114 l 1215 1042 l 174 1042 l 174 1096 b 141 1128 174 1114 159 1128 l 33 1128 b 0 1096 15 1128 0 1114 l 0 987 b 33 955 0 969 15 955 l 87 955 l 87 87 l 33 87 b 0 54 15 87 0 72 l 0 -54 b 33 -87 0 -72 15 -87 l 141 -87 b 174 -54 159 -87 174 -72 l 174 0 l 1215 0 l 1215 -54 b 1248 -87 1215 -72 1230 -87 l 1356 -87 b 1389 -54 1374 -87 1389 -72 l 1389 54 b 1356 87 1389 72 1374 87 l 1302 87 z m 260 467 l 260 836 b 293 868 260 853 275 868 l 749 868 b 781 836 767 868 781 853 l 781 467 b 749 434 781 449 767 434 l 293 434 b 260 467 275 434 260 449 z m 1128 206 b 1096 174 1128 188 1114 174 l 640 174 b 608 206 622 174 608 188 l 608 347 l 803 347 b 868 412 839 347 868 376 l 868 608 l 1096 608 b 1128 575 1114 608 1128 593 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 174 347 l 955 347 l 955 277 b 971 260 955 268 962 260 l 1112 260 b 1128 277 1121 260 1128 268 l 1128 418 b 1112 434 1128 427 1121 434 l 1042 434 l 1042 955 l 1112 955 b 1128 971 1121 955 1128 962 l 1128 1112 b 1112 1128 1128 1121 1121 1128 l 971 1128 b 955 1112 962 1128 955 1121 l 955 1042 l 174 1042 l 174 1112 b 157 1128 174 1121 166 1128 l 16 1128 b 0 1112 7 1128 0 1121 l 0 971 b 16 955 0 962 7 955 l 87 955 l 87 434 l 16 434 b 0 418 7 434 0 427 l 0 277 b 16 260 0 268 7 260 l 157 260 b 174 277 166 260 174 268 z m 1476 521 l 1476 608 l 1546 608 b 1563 624 1555 608 1563 615 l 1563 765 b 1546 781 1563 774 1555 781 l 1405 781 b 1389 765 1396 781 1389 774 l 1389 694 l 1107 694 l 1107 499 l 1128 499 b 1194 434 1164 499 1194 470 l 1194 260 b 1128 195 1194 224 1164 195 l 955 195 b 890 260 919 195 890 224 l 890 282 l 521 282 l 521 87 l 450 87 b 434 71 441 87 434 80 l 434 -71 b 450 -87 434 -80 441 -87 l 591 -87 b 608 -71 600 -87 608 -80 l 608 0 l 1389 0 l 1389 -71 b 1405 -87 1389 -80 1396 -87 l 1546 -87 b 1563 -71 1555 -87 1563 -80 l 1563 71 b 1546 87 1563 80 1555 87 l 1476 87 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 846 347 l 1215 347 l 1215 1063 b 1150 1128 1215 1099 1186 1128 l 65 1128 b 0 1063 29 1128 0 1099 l 0 -22 b 65 -87 0 -58 29 -87 l 781 -87 l 781 282 b 846 347 781 318 811 347 z m 1196 198 b 1215 244 1208 210 1215 227 l 1215 260 l 868 260 l 868 -87 l 884 -87 b 930 -68 902 -87 918 -80 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1259 1215 l 477 1215 b 347 1085 406 1215 347 1157 l 347 304 b 477 174 347 232 406 174 l 1259 174 b 1389 304 1331 174 1389 232 l 1389 1085 b 1259 1215 1389 1157 1331 1215 z m 477 87 b 260 304 358 87 260 184 l 260 868 l 130 868 b 0 738 58 868 0 810 l 0 -43 b 130 -174 0 -115 58 -174 l 911 -174 b 1042 -43 983 -174 1042 -115 l 1042 87 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 694 304 l 694 304 b 464 796 694 348 698 328 b 231 796 416 892 279 892 b 0 304 -6 323 0 346 l 0 304 b 347 87 0 184 155 87 b 694 304 539 87 694 184 z m 347 738 l 543 347 l 152 347 z m 1736 304 b 1505 796 1736 348 1740 328 b 1272 796 1457 892 1320 892 b 1042 304 1036 323 1042 346 l 1042 304 b 1389 87 1042 184 1197 87 b 1736 304 1581 87 1736 184 z m 1194 347 l 1389 738 l 1584 347 z m 1432 0 l 955 0 l 955 800 b 1081 955 1019 827 1067 885 l 1432 955 b 1476 998 1456 955 1476 974 l 1476 1085 b 1432 1128 1476 1109 1456 1128 l 1041 1128 b 868 1215 1001 1181 939 1215 b 695 1128 797 1215 735 1181 l 304 1128 b 260 1085 280 1128 260 1109 l 260 998 b 304 955 260 974 280 955 l 655 955 b 781 800 670 885 717 827 l 781 0 l 304 0 b 260 -43 280 0 260 -19 l 260 -130 b 304 -174 260 -154 280 -174 l 1432 -174 b 1476 -130 1456 -174 1476 -154 l 1476 -43 b 1432 0 1476 -19 1456 0 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 977 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 1107 b 65 1042 0 1071 29 1042 b 393 521 65 795 203 587 b 65 0 203 455 65 247 b 0 -65 29 0 0 -29 l 0 -109 b 65 -174 0 -144 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -144 l 1042 -65 b 977 0 1042 -29 1013 0 b 649 521 977 247 838 455 b 977 1042 838 587 977 795 b 1042 1107 1013 1042 1042 1071 l 1042 1150 b 977 1215 1042 1186 1013 1215 z m 803 0 l 239 0 b 521 391 239 210 364 391 b 803 0 678 391 803 210 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 977 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 1107 b 65 1042 0 1071 29 1042 b 393 521 65 795 203 587 b 65 0 203 455 65 247 b 0 -65 29 0 0 -29 l 0 -109 b 65 -174 0 -144 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -144 l 1042 -65 b 977 0 1042 -29 1013 0 b 649 521 977 247 838 455 b 977 1042 838 587 977 795 b 1042 1107 1013 1042 1042 1071 l 1042 1150 b 977 1215 1042 1186 1013 1215 z m 773 174 l 269 174 b 521 391 315 301 410 391 b 773 174 632 391 727 301 z m 773 868 l 269 868 b 239 1042 250 921 239 980 l 803 1042 b 773 868 803 980 792 921 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 977 1042 b 1042 1107 1013 1042 1042 1071 l 1042 1150 b 977 1215 1042 1186 1013 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 1107 b 65 1042 0 1071 29 1042 b 393 521 65 795 203 587 b 65 0 203 455 65 247 b 0 -65 29 0 0 -29 l 0 -109 b 65 -174 0 -144 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -144 l 1042 -65 b 977 0 1042 -29 1013 0 b 649 521 977 247 838 455 b 977 1042 838 587 977 795 z m 521 651 b 239 1042 364 651 239 831 l 803 1042 b 521 651 803 830 677 651 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 977 1042 b 1042 1107 1013 1042 1042 1071 l 1042 1150 b 977 1215 1042 1186 1013 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 1107 b 65 1042 0 1071 29 1042 b 393 521 65 795 203 587 b 65 0 203 455 65 247 b 0 -65 29 0 0 -29 l 0 -109 b 65 -174 0 -144 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -144 l 1042 -65 b 977 0 1042 -29 1013 0 b 649 521 977 247 838 455 b 977 1042 838 587 977 795 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1261 998 b 1128 868 1188 999 1128 941 l 1107 868 l 1107 953 b 979 1085 1107 1024 1050 1084 b 846 955 906 1086 846 1028 l 846 868 l 825 868 l 825 996 b 697 1128 825 1067 768 1127 b 564 998 624 1130 564 1071 l 564 868 l 543 868 l 543 953 b 414 1085 543 1024 486 1084 b 282 955 342 1086 282 1028 l 282 586 l 260 605 l 260 736 b 132 868 260 807 204 867 b 0 738 59 869 0 811 l 0 558 b 87 363 0 483 32 413 l 390 94 b 434 -4 418 69 434 33 l 434 -22 b 499 -87 434 -58 463 -87 l 1150 -87 b 1215 -22 1186 -87 1215 -58 l 1215 -14 b 1236 87 1215 21 1222 55 l 1369 403 b 1389 504 1382 435 1389 469 l 1389 866 b 1261 998 1389 937 1332 997 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1109 868 b 998 757 1048 869 998 818 l 998 521 l 977 521 l 977 999 b 866 1109 977 1060 927 1111 b 760 1001 807 1108 760 1060 l 760 521 l 738 521 l 738 1105 b 627 1215 738 1165 688 1216 b 521 1107 568 1214 521 1166 l 521 521 l 499 521 l 499 996 b 389 1107 499 1057 449 1108 b 282 998 330 1106 282 1057 l 282 358 l 196 476 b 45 500 161 525 93 535 b 21 349 -4 465 -14 397 l 361 -120 b 467 -174 383 -150 430 -174 l 1003 -174 b 1130 -73 1063 -174 1116 -132 l 1201 232 b 1215 352 1209 265 1215 318 l 1215 760 b 1109 868 1215 819 1168 867 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 586 22 b 694 -87 586 -38 635 -87 l 911 -87 l 911 -87 b 936 -84 918 -87 930 -86 l 1305 3 b 1389 109 1354 14 1389 58 l 1389 760 b 1334 854 1389 794 1364 837 l 1030 1027 b 892 1001 984 1054 925 1043 l 824 917 l 308 1119 b 139 1046 241 1146 166 1113 b 213 877 113 979 146 903 l 694 688 l 694 651 l 130 651 b 0 521 58 651 0 593 b 130 391 0 449 58 391 l 694 391 l 694 369 l 608 369 b 499 260 548 369 499 320 b 608 152 499 200 548 152 l 694 152 l 694 130 b 586 22 635 130 586 82 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1042 -87 l 1563 -87 l 1563 228 l 1563 228 b 1522 368 1563 270 1544 332 l 1080 1068 b 970 1128 1059 1101 1010 1128 l 65 1128 b 0 1063 29 1128 0 1099 l 0 1020 b 152 868 0 936 68 868 l 776 868 b 821 800 811 868 834 833 l 763 661 b 683 608 750 632 715 608 l 347 608 b 174 434 251 608 174 530 l 174 412 b 239 347 174 376 203 347 l 639 347 l 639 347 b 708 327 659 347 691 338 l 1011 135 b 1042 80 1028 124 1042 100 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1306 952 b 1175 872 1247 966 1189 930 l 1091 521 l 1061 521 l 1179 994 b 1101 1125 1194 1052 1159 1111 b 969 1046 1042 1140 984 1104 l 838 521 l 797 521 l 656 1131 b 526 1213 643 1190 585 1226 b 445 1083 468 1199 431 1141 l 574 521 l 547 521 l 453 936 b 323 1017 440 994 382 1031 b 241 887 265 1004 228 946 l 347 422 l 347 206 l 183 361 b 30 356 139 402 71 400 b 34 203 -11 313 -9 244 l 397 -138 b 486 -174 421 -161 453 -174 l 1074 -174 b 1199 -80 1132 -174 1182 -136 l 1270 159 b 1281 233 1277 183 1281 208 l 1281 347 b 1288 407 1281 368 1283 388 l 1386 821 b 1306 952 1400 879 1364 938 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1215 564 b 1107 673 1215 624 1167 673 b 998 564 1047 673 998 624 l 977 564 l 977 629 b 868 738 977 689 928 738 b 760 629 808 738 760 689 l 760 564 l 738 564 l 738 673 b 629 781 738 733 689 781 b 521 673 569 781 521 733 l 521 564 l 499 564 l 499 1107 b 391 1215 499 1167 451 1215 b 282 1107 331 1215 282 1167 l 282 358 l 196 476 b 45 500 161 525 93 535 b 21 349 -4 465 -14 397 l 368 -129 b 456 -174 386 -154 425 -174 l 1020 -174 b 1126 -90 1070 -174 1114 -139 l 1212 279 b 1215 304 1214 287 1215 295 z m 521 347 l 521 87 l 499 87 l 499 347 z m 760 347 l 760 87 l 738 87 l 738 347 z m 998 347 l 998 87 l 977 87 l 977 347 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1107 629 b 998 521 1047 629 998 581 l 977 521 l 977 608 b 868 716 977 668 928 716 b 760 608 808 716 760 668 l 760 521 l 738 521 l 738 1085 b 608 1215 738 1157 680 1215 b 477 1085 536 1215 477 1157 l 477 521 l 441 521 l 251 1002 b 83 1076 225 1069 150 1102 b 9 907 16 1050 -17 974 l 212 391 l 128 323 b 101 185 86 290 75 231 l 275 -119 b 369 -174 292 -149 334 -174 l 369 -174 l 1020 -174 b 1126 -90 1070 -174 1114 -139 l 1212 279 b 1215 304 1214 286 1215 297 l 1215 304 l 1215 521 b 1107 629 1215 581 1167 629 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 707 955 l 33 955 b 0 922 15 955 0 940 l 0 805 b 33 773 0 787 15 773 l 263 773 l 263 119 b 296 87 263 101 278 87 l 443 87 b 476 119 461 87 476 101 l 476 773 l 707 773 b 739 805 725 773 739 787 l 739 922 b 707 955 739 940 725 955 z m 1736 122 l 1671 925 b 1638 955 1670 942 1656 955 l 1461 955 b 1430 933 1447 955 1435 946 l 1312 588 b 1268 445 1292 532 1268 445 l 1265 445 b 1222 588 1265 445 1241 532 l 1103 933 b 1072 955 1098 946 1086 955 l 895 955 b 862 925 878 955 864 942 l 796 122 b 829 87 795 103 809 87 l 977 87 b 1009 117 994 87 1008 100 l 1034 478 b 1034 623 1039 543 1034 623 l 1036 623 b 1085 478 1036 623 1065 532 l 1168 248 b 1199 227 1173 235 1185 227 l 1335 227 b 1366 248 1349 227 1361 235 l 1449 478 b 1498 623 1469 532 1498 623 l 1500 623 b 1500 478 1500 623 1495 543 l 1525 117 b 1557 87 1526 100 1540 87 l 1704 87 b 1736 122 1723 87 1737 103 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 774 652 b 679 728 774 704 757 728 l 616 728 l 616 575 l 697 575 b 774 652 747 575 774 602 z m 1367 521 b 694 1194 1367 892 1066 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 z m 986 238 b 929 141 1010 194 978 141 l 862 141 b 805 176 841 141 815 157 l 685 401 l 616 401 l 616 206 b 551 141 616 170 587 141 l 490 141 b 425 206 454 141 425 170 l 425 836 b 490 901 425 872 454 901 l 681 901 b 967 659 882 901 967 806 b 865 457 967 574 928 498 b 986 238 868 451 859 468 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1606 1215 l 130 1215 b 0 1085 58 1215 0 1157 l 0 217 b 130 87 0 145 58 87 l 795 87 l 795 0 l 361 0 b 274 -87 313 0 274 -39 b 361 -174 274 -135 313 -174 l 1403 -174 b 1490 -87 1451 -174 1490 -135 b 1403 0 1490 -39 1451 0 l 969 0 l 969 87 l 1606 87 b 1736 217 1678 87 1736 145 l 1736 1085 b 1606 1215 1736 1157 1678 1215 z m 1563 260 l 174 260 l 174 1042 l 1563 1042 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1183 781 b 1215 814 1201 781 1215 796 l 1215 911 b 1085 1042 1215 983 1157 1042 l 955 1042 l 955 1183 b 922 1215 955 1201 940 1215 l 814 1215 b 781 1183 796 1215 781 1201 l 781 1042 l 434 1042 l 434 1183 b 401 1215 434 1201 419 1215 l 293 1215 b 260 1183 275 1215 260 1201 l 260 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 814 b 33 781 0 796 15 781 z m 33 694 b 0 662 15 694 0 680 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 662 b 1183 694 1215 680 1201 694 z m 890 315 l 890 206 b 857 174 890 188 875 174 l 694 174 l 694 11 b 662 -22 694 -7 680 -22 l 553 -22 b 521 11 535 -22 521 -7 l 521 174 l 358 174 b 326 206 340 174 326 188 l 326 315 b 358 347 326 333 340 347 l 521 347 l 521 510 b 553 543 521 528 535 543 l 662 543 b 694 510 680 543 694 528 l 694 347 l 857 347 b 890 315 875 347 890 333 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1183 781 b 1215 814 1201 781 1215 796 l 1215 911 b 1085 1042 1215 983 1157 1042 l 955 1042 l 955 1183 b 922 1215 955 1201 940 1215 l 814 1215 b 781 1183 796 1215 781 1201 l 781 1042 l 434 1042 l 434 1183 b 401 1215 434 1201 419 1215 l 293 1215 b 260 1183 275 1215 260 1201 l 260 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 814 b 33 781 0 796 15 781 z m 33 694 b 0 662 15 694 0 680 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 662 b 1183 694 1215 680 1201 694 z m 857 174 l 358 174 b 326 206 340 174 326 188 l 326 315 b 358 347 326 333 340 347 l 857 347 b 890 315 875 347 890 333 l 890 206 b 857 174 890 188 875 174 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1183 781 b 1215 814 1201 781 1215 796 l 1215 911 b 1085 1042 1215 983 1157 1042 l 955 1042 l 955 1183 b 922 1215 955 1201 940 1215 l 814 1215 b 781 1183 796 1215 781 1201 l 781 1042 l 434 1042 l 434 1183 b 401 1215 434 1201 419 1215 l 293 1215 b 260 1183 275 1215 260 1201 l 260 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 814 b 33 781 0 796 15 781 z m 33 694 b 0 662 15 694 0 680 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 662 b 1183 694 1215 680 1201 694 z m 731 260 l 861 130 b 861 84 874 117 874 97 l 784 7 b 738 7 771 -6 751 -6 l 608 138 l 477 7 b 431 7 464 -6 444 -6 l 354 84 b 354 130 342 97 342 117 l 485 260 l 354 391 b 354 437 342 404 342 424 l 431 514 b 477 514 444 527 464 527 l 608 383 l 738 514 b 784 514 751 527 771 527 l 861 437 b 861 391 874 424 874 404 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1183 781 b 1215 814 1201 781 1215 796 l 1215 911 b 1085 1042 1215 983 1157 1042 l 955 1042 l 955 1183 b 922 1215 955 1201 940 1215 l 814 1215 b 781 1183 796 1215 781 1201 l 781 1042 l 434 1042 l 434 1183 b 401 1215 434 1201 419 1215 l 293 1215 b 260 1183 275 1215 260 1201 l 260 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 814 b 33 781 0 796 15 781 z m 33 694 b 0 662 15 694 0 680 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 662 b 1183 694 1215 680 1201 694 z m 937 434 b 936 388 949 421 949 401 l 549 3 b 503 4 536 -9 515 -9 l 279 230 b 279 276 266 242 266 263 l 356 352 b 402 352 369 365 389 365 l 527 226 l 814 511 b 860 511 827 524 848 524 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1289 771 l 911 531 l 911 716 b 811 771 911 767 855 799 l 434 531 l 434 1063 b 369 1128 434 1099 405 1128 l 65 1128 b 0 1063 29 1128 0 1099 l 0 -22 b 65 -87 0 -58 29 -87 l 1324 -87 b 1389 -22 1360 -87 1389 -58 l 1389 716 b 1289 771 1389 767 1332 799 z "
	},
	"": {
		ha: 781,
		x_min: 0,
		x_max: 0,
		o: "m 304 356 l 304 -70 l 364 -159 b 418 -159 376 -178 405 -178 l 477 -70 l 477 356 b 391 347 449 350 420 347 b 304 356 361 347 332 350 z m 391 1215 b 0 825 175 1215 0 1040 b 391 434 0 609 175 434 b 781 825 606 434 781 609 b 391 1215 781 1040 606 1215 z m 391 1009 b 206 825 289 1009 206 926 b 174 792 206 807 192 792 b 141 825 156 792 141 807 b 391 1074 141 962 253 1074 b 423 1042 409 1074 423 1060 b 391 1009 423 1024 409 1009 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1376 986 l 1259 1103 b 1197 1128 1242 1119 1220 1128 l 781 1128 l 781 1172 b 738 1215 781 1196 762 1215 l 651 1215 b 608 1172 627 1215 608 1196 l 608 1128 l 152 1128 b 87 1063 116 1128 87 1099 l 87 846 b 152 781 87 810 116 781 l 1197 781 b 1259 807 1220 781 1242 790 l 1376 924 b 1376 986 1393 941 1393 969 z m 608 -130 b 651 -174 608 -154 627 -174 l 738 -174 b 781 -130 762 -174 781 -154 l 781 174 l 608 174 z m 1237 608 l 781 608 l 781 694 l 608 694 l 608 608 l 192 608 b 130 582 169 608 147 598 l 13 465 b 13 403 -4 448 -4 420 l 130 286 b 192 260 146 270 169 260 l 1237 260 b 1302 326 1273 260 1302 290 l 1302 543 b 1237 608 1302 578 1273 608 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 0 896 l 0 -43 b 60 -84 0 -74 31 -95 l 434 87 l 434 1128 l 55 977 b 0 896 24 965 0 929 z m 521 87 l 1042 -87 l 1042 955 l 521 1128 z m 1503 1125 l 1128 955 l 1128 -87 l 1508 65 b 1563 146 1538 77 1563 113 l 1563 1085 b 1503 1125 1563 1116 1531 1137 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1215 1215 l 174 1215 b 0 1042 78 1215 0 1137 l 0 260 b 174 87 0 165 78 87 l 434 87 l 434 -141 b 486 -167 434 -168 464 -183 l 825 87 l 1215 87 b 1389 260 1311 87 1389 165 l 1389 1042 b 1215 1215 1389 1137 1311 1215 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 z m 651 304 b 608 260 651 280 632 260 l 477 260 b 434 304 454 260 434 280 l 434 738 b 477 781 434 762 454 781 l 608 781 b 651 738 632 781 651 762 z m 955 304 b 911 260 955 280 935 260 l 781 260 b 738 304 757 260 738 280 l 738 738 b 781 781 738 762 757 781 l 911 781 b 955 738 935 781 955 762 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 z m 955 304 b 911 260 955 280 935 260 l 477 260 b 434 304 454 260 434 280 l 434 738 b 477 781 434 762 454 781 l 911 781 b 955 738 935 781 955 762 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 955 781 l 955 868 b 608 1215 955 1060 799 1215 b 260 868 416 1215 260 1060 l 260 781 l 0 781 l 0 43 b 217 -174 0 -76 97 -174 l 998 -174 b 1215 43 1118 -174 1215 -76 l 1215 781 z m 434 868 b 608 1042 434 964 512 1042 b 781 868 703 1042 781 964 l 781 781 l 434 781 z m 868 543 b 803 608 832 543 803 572 b 868 673 803 644 832 673 b 933 608 904 673 933 644 b 868 543 933 572 904 543 z m 347 543 b 282 608 311 543 282 572 b 347 673 282 644 311 673 b 412 608 383 673 412 644 b 347 543 412 572 383 543 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1563 629 b 1497 694 1563 665 1533 694 l 1315 694 l 1025 1093 b 904 1112 997 1131 943 1140 b 885 991 865 1084 856 1029 l 1100 694 l 462 694 l 678 991 b 659 1112 706 1029 697 1084 b 537 1093 620 1140 566 1131 l 248 694 l 65 694 b 0 629 29 694 0 665 l 0 586 b 65 521 0 550 29 521 l 87 521 l 158 25 b 287 -87 167 -39 222 -87 l 1276 -87 b 1405 25 1341 -87 1396 -39 l 1476 521 l 1497 521 b 1563 586 1533 521 1563 550 z m 846 152 b 781 87 846 116 817 87 b 716 152 745 87 716 116 l 716 456 b 781 521 716 492 745 521 b 846 456 817 521 846 492 z m 1150 152 b 1085 87 1150 116 1121 87 b 1020 152 1049 87 1020 116 l 1020 456 b 1085 521 1020 492 1049 521 b 1150 456 1121 521 1150 492 z m 543 152 b 477 87 543 116 513 87 b 412 152 441 87 412 116 l 412 456 b 477 521 412 492 441 521 b 543 456 513 521 543 492 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1195 721 l 1215 830 b 1183 868 1218 850 1203 868 l 980 868 l 1019 1090 b 987 1128 1023 1110 1008 1128 l 877 1128 b 845 1102 862 1128 848 1116 l 803 868 l 536 868 l 576 1090 b 543 1128 579 1110 564 1128 l 433 1128 b 401 1102 418 1128 404 1116 l 360 868 l 145 868 b 113 841 130 868 116 856 l 94 733 b 126 694 90 713 105 694 l 329 694 l 266 347 l 52 347 b 20 320 37 347 23 335 l 1 212 b 33 174 -3 192 12 174 l 235 174 l 196 -49 b 228 -87 192 -68 208 -87 l 338 -87 b 370 -60 353 -87 367 -75 l 412 174 l 679 174 l 640 -49 b 672 -87 636 -68 652 -87 l 782 -87 b 814 -60 797 -87 811 -75 l 856 174 l 1070 174 b 1102 200 1085 174 1100 186 l 1122 309 b 1090 347 1125 329 1110 347 l 887 347 l 949 694 l 1163 694 b 1195 721 1178 694 1193 706 z m 710 347 l 443 347 l 505 694 l 772 694 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 304 608 b 608 911 472 608 608 744 b 304 1215 608 1079 472 1215 b 0 911 136 1215 0 1079 b 304 608 0 744 136 608 z m 304 1042 b 434 911 376 1042 434 983 b 304 781 434 840 376 781 b 174 911 232 781 174 840 b 304 1042 174 983 232 1042 z m 911 434 b 608 130 744 434 608 298 b 911 -174 608 -38 744 -174 b 1215 130 1079 -174 1215 -38 b 911 434 1215 298 1079 434 z m 911 0 b 781 130 840 0 781 58 b 911 260 781 202 840 260 b 1042 130 983 260 1042 202 b 911 0 1042 58 983 0 z m 1064 1215 b 1011 1187 1043 1215 1023 1204 l 13 -71 b 66 -173 -18 -114 13 -173 l 157 -173 b 210 -145 175 -173 199 -161 l 1203 1112 b 1150 1215 1234 1156 1203 1215 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1085 b 1259 521 1006 1085 1259 833 b 694 -43 1259 209 1006 -43 b 130 521 383 -43 130 209 b 694 1085 130 833 383 1085 z m 694 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 z m 694 1042 b 174 521 407 1042 174 808 b 694 0 174 233 407 0 b 1215 521 982 0 1215 233 b 694 1042 1215 808 982 1042 z m 694 922 b 792 825 748 922 792 879 b 694 727 792 771 748 727 b 597 825 641 727 597 771 b 694 922 597 879 641 922 z m 1014 656 b 791 614 936 638 863 622 b 859 191 793 339 824 280 b 831 128 869 166 856 138 b 768 156 806 118 778 131 b 708 369 744 216 722 266 l 681 369 b 621 156 667 266 645 216 b 558 128 611 131 583 118 b 530 191 533 138 520 166 b 598 614 564 280 596 339 b 375 656 526 622 453 638 b 343 709 352 662 337 685 b 395 741 348 732 372 746 b 994 741 657 679 732 679 b 1046 709 1017 746 1041 732 b 1014 656 1052 685 1037 662 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 1031 -170 b 1042 -152 1037 -167 1042 -159 b 1039 -141 1042 -149 1040 -144 l 697 423 b 661 397 689 413 673 401 l 1001 -163 b 1020 -174 1005 -169 1013 -174 b 1031 -170 1023 -174 1028 -172 z m 387 363 l 299 120 l 397 -120 b 510 -167 415 -164 466 -185 b 558 -54 555 -149 576 -98 z m 260 977 b 380 1096 326 977 380 1030 b 260 1215 380 1162 326 1215 b 141 1096 194 1215 141 1162 b 260 977 141 1030 194 977 z m 680 518 l 355 930 b 304 955 342 946 323 955 l 304 955 l 217 955 l 217 955 b 166 930 198 955 178 946 l 0 717 l 0 457 b 66 391 0 421 30 390 b 130 456 102 391 130 420 l 130 672 l 174 728 l 174 346 l 27 -57 b 79 -168 11 -102 34 -152 b 190 -116 124 -185 174 -162 l 369 375 l 369 666 l 284 774 b 281 781 282 776 281 779 b 292 792 281 787 286 792 b 301 788 295 792 299 790 l 578 437 b 670 426 603 406 644 406 b 680 518 698 449 703 490 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 442 568 b 431 609 438 580 435 595 b 421 568 428 595 424 580 l 396 485 l 466 485 z m 894 655 l 854 655 l 854 387 l 894 387 b 1019 522 974 387 1019 435 b 894 655 1019 609 971 655 z m 1389 911 b 1259 1042 1389 983 1331 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 130 b 130 0 0 58 58 0 l 1259 0 b 1389 130 1331 0 1389 58 z m 666 303 b 635 260 673 282 657 260 l 556 260 l 556 260 b 525 284 542 260 528 271 l 501 365 l 361 365 l 338 284 b 307 260 334 271 320 260 l 228 260 b 197 303 206 260 190 282 l 352 759 b 383 781 356 771 370 781 l 480 781 b 511 759 493 781 507 771 z m 1167 522 b 898 260 1167 361 1064 260 l 743 260 b 710 293 725 260 710 275 l 710 749 b 743 781 710 767 725 781 l 898 781 b 1167 522 1064 781 1167 682 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 264 -160 b 297 -174 272 -167 286 -174 b 322 -167 304 -174 315 -170 l 498 -57 b 519 -17 510 -49 519 -31 b 516 0 519 -12 518 -4 l 428 220 b 385 249 422 236 402 249 b 380 249 384 249 382 249 l 229 234 b 229 721 173 389 172 563 l 380 706 b 385 706 382 706 384 706 b 428 735 402 706 422 719 l 516 955 b 519 972 518 959 519 967 b 498 1011 519 986 510 1004 l 322 1121 b 264 1115 303 1133 279 1130 b 264 -160 -88 763 -88 192 z m 670 956 b 620 963 660 974 635 978 l 603 948 b 597 909 593 938 590 922 b 607 868 603 898 607 880 b 597 827 607 856 603 838 b 603 788 590 814 593 799 l 620 773 b 670 780 635 758 660 762 b 670 956 702 834 702 902 z m 919 1204 b 872 1206 907 1218 885 1219 l 856 1191 b 853 1147 844 1179 843 1160 b 853 589 988 985 988 751 b 856 545 843 576 844 557 l 872 530 b 919 532 885 517 907 518 b 919 1204 1082 727 1082 1009 z m 795 1082 b 746 1085 783 1097 760 1099 l 730 1070 b 727 1027 718 1059 717 1040 b 727 709 799 933 799 803 b 730 666 717 696 718 677 l 746 651 b 795 654 760 637 783 639 b 795 1082 893 780 892 956 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 347 521 b 174 694 347 617 269 694 b 0 521 78 694 0 617 b 174 347 0 425 78 347 b 347 521 269 347 347 425 z m 174 174 b 87 87 126 174 87 135 b 174 0 87 39 126 0 b 260 87 222 0 260 39 b 174 174 260 135 222 174 z m 174 1128 b 0 955 78 1128 0 1051 b 174 781 0 859 78 781 b 347 955 269 781 347 859 b 174 1128 347 1051 269 1128 z m 608 608 b 521 521 560 608 521 569 b 608 434 521 473 560 434 b 694 521 656 434 694 473 b 608 608 694 569 656 608 z m 608 174 b 521 87 560 174 521 135 b 608 0 521 39 560 0 b 694 87 656 0 694 39 b 608 174 694 135 656 174 z m 608 1128 b 434 955 512 1128 434 1051 b 608 781 434 859 512 781 b 781 955 704 781 781 859 b 608 1128 781 1051 704 1128 z m 1215 608 b 1128 521 1167 608 1128 569 b 1215 434 1128 473 1167 434 b 1302 521 1263 434 1302 473 b 1215 608 1302 569 1263 608 z m 1215 174 b 1128 87 1167 174 1128 135 b 1215 0 1128 39 1167 0 b 1302 87 1263 0 1302 39 b 1215 174 1302 135 1263 174 z m 1215 1128 b 1042 955 1119 1128 1042 1051 b 1215 781 1042 859 1119 781 b 1389 955 1311 781 1389 859 b 1215 1128 1389 1051 1311 1128 z m 1649 608 b 1563 521 1601 608 1563 569 b 1649 434 1563 473 1601 434 b 1736 521 1697 434 1736 473 b 1649 608 1736 569 1697 608 z m 1649 174 b 1563 87 1601 174 1563 135 b 1649 0 1563 39 1601 0 b 1736 87 1697 0 1736 39 b 1649 174 1736 135 1697 174 z m 1649 1042 b 1563 955 1601 1042 1563 1003 b 1649 868 1563 907 1601 868 b 1736 955 1697 868 1736 907 b 1649 1042 1736 1003 1697 1042 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 586 510 b 651 575 586 546 615 575 b 716 510 687 575 716 546 b 792 434 716 468 750 434 b 868 510 834 434 868 468 b 651 727 868 630 771 727 b 434 510 531 727 434 630 b 510 434 434 468 468 434 b 586 510 552 434 586 468 z m 651 987 b 174 510 388 987 174 773 b 250 434 174 468 208 434 b 326 510 292 434 326 468 b 651 836 326 689 472 836 b 977 510 831 836 977 689 b 781 120 977 306 784 319 l 781 119 b 640 -22 781 42 718 -22 b 564 -98 598 -22 564 -56 b 640 -174 564 -140 598 -174 b 933 119 802 -174 933 -43 b 1128 510 935 212 1128 250 b 651 987 1128 773 914 987 z m 434 347 b 347 260 386 347 347 308 b 434 174 347 212 386 174 b 521 260 482 174 521 212 b 434 347 521 308 482 347 z m 87 0 b 0 -87 39 0 0 -39 b 87 -174 0 -135 39 -174 b 174 -87 135 -174 174 -135 b 87 0 174 -39 135 0 z m 1389 510 b 1389 522 1389 514 1389 518 b 890 1211 1384 834 1183 1111 b 793 1164 850 1225 807 1204 b 841 1067 780 1124 801 1081 b 1237 520 1074 988 1233 768 l 1237 519 b 1237 510 1237 516 1237 513 b 1313 434 1237 468 1271 434 b 1389 510 1355 434 1389 468 z m 415 24 l 198 241 l 106 149 l 323 -68 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 788 702 b 885 736 824 685 867 700 b 851 833 903 772 888 815 b 613 863 772 872 683 879 b 732 904 648 884 689 899 b 795 985 772 909 800 945 b 714 1048 790 1025 754 1053 b 513 972 640 1039 572 1012 b 601 1068 537 1007 567 1039 b 611 1170 632 1094 637 1139 b 509 1180 586 1201 540 1205 b 293 782 387 1079 310 937 l 181 580 l 24 491 b 5 427 2 478 -6 450 l 126 185 b 190 164 137 163 165 152 l 371 255 b 627 231 371 255 625 231 b 845 369 720 231 805 286 b 781 473 868 418 831 473 b 845 576 831 473 868 527 b 613 713 806 657 728 713 b 788 702 668 733 733 730 z m 714 514 b 778 473 726 488 751 473 b 714 431 751 472 726 457 b 627 376 698 397 665 376 b 530 473 579 376 530 414 b 530 473 530 473 530 473 b 627 569 530 526 573 569 l 627 569 b 714 514 665 569 698 548 z m 1731 615 l 1610 856 b 1546 877 1599 878 1571 890 l 1366 787 l 1114 810 b 891 672 1011 810 929 752 b 955 569 868 624 905 569 b 891 466 905 569 868 514 b 1124 329 930 384 1008 329 b 948 339 1068 309 1003 312 b 851 306 913 356 869 341 b 885 209 833 270 848 227 b 1124 179 964 170 1053 163 b 1004 137 1088 157 1047 142 b 941 56 964 133 936 96 b 1022 -6 947 9 995 -13 b 1223 70 1096 3 1164 30 b 1135 -26 1199 35 1169 2 b 1125 -128 1104 -52 1099 -97 b 1227 -138 1151 -160 1197 -163 b 1443 260 1349 -38 1426 105 l 1555 461 l 1712 551 b 1731 615 1734 564 1742 592 z m 1109 666 b 1206 569 1157 666 1206 627 b 1206 569 1206 569 1206 569 b 1110 473 1206 516 1163 473 l 1109 473 b 1022 528 1072 473 1038 494 b 959 569 1010 553 985 568 b 1022 611 985 570 1010 585 b 1109 666 1038 644 1072 666 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 586 510 b 651 575 586 546 615 575 b 716 510 687 575 716 546 b 792 434 716 468 750 434 b 868 510 834 434 868 468 b 651 727 868 630 771 727 b 434 510 531 727 434 630 b 510 434 434 468 468 434 b 586 510 552 434 586 468 z m 651 987 b 174 510 388 987 174 773 b 250 434 174 468 208 434 b 326 510 292 434 326 468 b 651 836 326 689 472 836 b 977 510 831 836 977 689 b 781 120 977 306 784 319 l 781 119 b 640 -22 781 42 718 -22 b 564 -98 598 -22 564 -56 b 640 -174 564 -140 598 -174 b 933 119 802 -174 933 -43 b 1128 510 935 212 1128 250 b 651 987 1128 773 914 987 z m 1379 1129 l 1303 1206 b 1257 1206 1290 1218 1269 1218 l 1021 970 b 1021 924 1008 957 1008 936 l 1097 847 b 1143 847 1110 834 1131 834 l 1379 1083 b 1379 1129 1392 1096 1392 1116 z m 458 361 b 412 361 446 374 425 374 l 10 -41 b 10 -87 -3 -54 -3 -75 l 86 -164 b 132 -164 99 -177 120 -177 l 535 239 b 535 285 548 251 548 272 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 248 -98 b 326 -174 249 -140 283 -174 l 617 -174 b 702 -164 641 -174 679 -169 l 920 -112 b 992 -22 962 -102 992 -65 l 992 361 b 954 436 992 387 975 421 l 619 680 b 511 663 585 705 536 697 b 528 554 486 628 493 579 l 612 493 l 155 493 b 78 417 113 493 78 459 b 157 338 77 373 113 338 l 496 338 l 496 322 l 78 322 b 0 246 35 322 1 289 b 79 167 -1 203 36 167 l 496 167 l 496 152 l 153 152 b 76 76 111 152 76 118 b 155 -3 75 32 111 -3 l 496 -3 l 496 -19 l 327 -19 b 248 -98 284 -19 247 -54 z m 447 515 l 546 515 l 515 537 b 507 690 465 573 460 648 l 471 735 b 360 749 445 770 394 777 b 348 641 328 723 322 675 z m 1215 520 l 1202 934 b 1122 1009 1200 977 1164 1010 b 1047 929 1079 1007 1045 972 l 1050 825 l 769 1185 b 662 1200 743 1219 695 1225 b 648 1089 627 1174 621 1123 l 856 821 l 844 812 l 587 1142 b 479 1156 561 1175 513 1181 b 466 1045 445 1130 439 1080 l 722 716 l 710 707 l 499 977 b 391 992 473 1011 425 1017 b 378 881 357 966 351 915 l 519 700 b 632 697 553 722 598 722 l 967 454 b 1014 361 996 432 1014 398 l 1014 315 l 1179 444 b 1215 517 1199 459 1215 492 b 1215 520 1215 518 1215 519 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1544 587 b 781 1020 1391 846 1106 1020 b 558 992 704 1020 629 1010 l 414 1187 b 324 1204 394 1217 353 1224 l 288 1179 b 272 1088 259 1158 252 1117 l 389 930 b 18 587 243 858 109 741 b 18 587 18 587 18 587 b 18 455 -6 547 -6 496 b 487 71 122 278 289 140 l 150 553 b 227 650 169 581 203 625 l 551 187 l 551 187 l 661 30 b 781 22 694 25 748 22 b 875 27 807 22 849 24 l 360 763 b 465 821 387 781 435 808 l 1148 -146 b 1239 -162 1169 -175 1209 -182 l 1274 -137 b 1290 -46 1304 -116 1311 -76 l 1173 112 l 1173 112 b 1544 455 1328 188 1458 308 b 1563 521 1554 472 1563 501 b 1544 587 1563 540 1554 570 z m 1058 277 l 1006 351 b 1030 654 1073 439 1081 558 l 1030 654 b 914 537 1030 589 978 537 b 798 654 849 537 798 590 b 914 770 798 718 850 770 l 914 770 b 698 791 847 806 769 812 l 647 864 b 781 890 688 881 734 890 b 1150 521 985 890 1150 725 b 1058 277 1150 427 1115 342 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1179 1042 l 946 1042 b 888 1019 924 1042 904 1034 l 621 775 b 620 774 621 774 620 774 b 614 622 575 731 576 664 b 767 614 649 584 721 574 b 768 615 767 615 767 615 l 984 814 b 1046 811 1002 830 1030 828 b 1043 750 1062 793 1061 766 l 972 685 l 1367 364 b 1389 343 1375 358 1382 350 l 1389 868 l 1241 1016 b 1179 1042 1225 1032 1202 1042 z m 1476 868 l 1476 260 b 1563 173 1476 212 1514 173 l 1736 173 l 1736 868 z m 1606 260 b 1563 304 1582 260 1563 280 b 1606 347 1563 327 1582 347 b 1649 304 1630 347 1649 327 b 1606 260 1649 280 1630 260 z m 0 174 l 174 174 b 260 260 222 174 260 212 l 260 868 l 0 868 z m 130 347 b 174 304 154 347 174 327 b 130 260 174 280 154 260 b 87 304 106 260 87 280 b 130 347 87 328 106 347 z m 1313 296 l 908 625 l 826 551 b 550 563 746 477 622 484 b 562 838 477 642 483 766 l 784 1042 l 557 1042 b 496 1016 534 1042 512 1032 l 347 868 l 347 261 l 397 261 l 642 39 b 887 64 717 -22 826 -11 l 887 64 l 936 22 b 1077 37 979 -13 1042 -6 l 1163 142 l 1177 130 b 1299 142 1214 100 1269 105 l 1325 174 b 1313 296 1356 212 1350 266 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1389 -43 l 1389 671 b 1339 773 1389 706 1367 752 b 894 1103 1271 826 1216 869 b 694 1215 848 1136 757 1216 b 495 1103 632 1216 541 1136 b 50 773 173 869 117 826 b 0 671 22 752 0 706 l 0 -43 b 130 -174 0 -115 58 -174 l 1259 -174 b 1389 -43 1331 -174 1389 -115 z m 1211 490 l 1235 454 b 1239 442 1237 451 1239 446 b 1230 424 1239 436 1235 428 b 945 216 1168 379 1080 314 b 694 87 890 176 791 86 b 444 216 597 86 499 176 b 159 424 309 314 221 379 b 150 442 154 428 150 436 b 154 454 150 446 151 451 l 178 490 b 209 495 185 500 199 502 b 495 286 271 449 359 385 b 694 174 541 253 631 173 b 894 286 757 173 848 253 b 1180 495 1030 385 1118 449 b 1211 490 1190 502 1204 500 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1183 781 b 1215 814 1201 781 1215 796 l 1215 922 b 1183 955 1215 940 1201 955 l 1128 955 l 1128 1085 b 998 1215 1128 1157 1070 1215 l 130 1215 b 0 1085 58 1215 0 1157 l 0 -43 b 130 -174 0 -115 58 -174 l 998 -174 b 1128 -43 1070 -174 1128 -115 l 1128 87 l 1183 87 b 1215 119 1201 87 1215 101 l 1215 228 b 1183 260 1215 246 1201 260 l 1128 260 l 1128 434 l 1183 434 b 1215 467 1201 434 1215 449 l 1215 575 b 1183 608 1215 593 1201 608 l 1128 608 l 1128 781 z m 564 868 b 738 694 660 868 738 790 b 564 521 738 599 660 521 b 391 694 468 521 391 599 b 564 868 391 790 468 868 z m 868 226 b 807 174 868 197 841 174 l 321 174 b 260 226 288 174 260 197 l 260 278 b 443 434 260 364 342 434 l 456 434 b 564 412 490 420 526 412 b 672 434 602 412 639 420 l 686 434 b 868 278 786 434 868 364 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1432 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -28 58 -87 l 1432 -87 b 1563 43 1504 -87 1563 -28 l 1563 998 b 1432 1128 1563 1070 1504 1128 z m 477 868 b 651 694 573 868 651 790 b 477 521 651 599 573 521 b 304 694 382 521 304 599 b 477 868 304 790 382 868 z m 781 226 b 720 174 781 197 754 174 l 234 174 b 174 226 201 174 174 197 l 174 278 b 356 434 174 364 255 434 l 369 434 b 477 412 403 420 439 412 b 585 434 516 412 552 420 l 599 434 b 781 278 700 434 781 364 z m 1389 369 b 1367 347 1389 357 1379 347 l 977 347 b 955 369 965 347 955 357 l 955 412 b 977 434 955 424 965 434 l 1367 434 b 1389 412 1379 434 1389 424 z m 1389 543 b 1367 521 1389 531 1379 521 l 977 521 b 955 543 965 521 955 531 l 955 586 b 977 608 955 598 965 608 l 1367 608 b 1389 586 1379 608 1389 598 z m 1389 716 b 1367 694 1389 704 1379 694 l 977 694 b 955 716 965 694 955 704 l 955 760 b 977 781 955 771 965 781 l 1367 781 b 1389 760 1379 781 1389 771 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 673 933 b 911 694 805 933 911 826 b 673 456 911 563 805 456 b 434 694 541 456 434 563 b 673 933 434 826 541 933 z m 673 0 b 275 185 514 0 371 72 b 543 347 326 281 426 347 b 562 344 549 347 556 346 b 673 326 597 333 634 326 b 784 344 712 326 749 333 b 803 347 790 346 796 347 b 1070 185 919 347 1019 281 b 673 0 975 72 832 0 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 911 1215 l 130 1215 b 0 1085 58 1215 0 1157 l 0 -43 b 130 -174 0 -115 58 -174 l 911 -174 b 1042 -43 983 -174 1042 -115 l 1042 1085 b 911 1215 1042 1157 983 1215 z m 391 1128 l 651 1128 b 694 1085 675 1128 694 1109 b 651 1042 694 1061 675 1042 l 391 1042 b 347 1085 367 1042 347 1061 b 391 1128 347 1109 367 1128 z m 521 781 b 694 608 617 781 694 703 b 521 434 694 512 617 434 b 347 608 425 434 347 512 b 521 781 347 703 425 781 z m 825 139 b 764 87 825 110 798 87 l 278 87 b 217 139 244 87 217 110 l 217 191 b 399 347 217 277 299 347 l 413 347 b 521 326 446 333 483 326 b 629 347 559 326 596 333 l 642 347 b 825 191 743 347 825 277 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1432 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 955 l 1563 955 l 1563 998 b 1432 1128 1563 1070 1504 1128 z m 0 43 b 130 -87 0 -28 58 -87 l 1432 -87 b 1563 43 1504 -87 1563 -28 l 1563 868 l 0 868 z m 955 673 b 977 694 955 685 965 694 l 1367 694 b 1389 673 1379 694 1389 685 l 1389 629 b 1367 608 1389 617 1379 608 l 977 608 b 955 629 965 608 955 617 z m 955 499 b 977 521 955 511 965 521 l 1367 521 b 1389 499 1379 521 1389 511 l 1389 456 b 1367 434 1389 444 1379 434 l 977 434 b 955 456 965 434 955 444 z m 955 326 b 977 347 955 337 965 347 l 1367 347 b 1389 326 1379 347 1389 337 l 1389 282 b 1367 260 1389 270 1379 260 l 977 260 b 955 282 965 260 955 270 z m 477 694 b 651 521 573 694 651 617 b 477 347 651 425 573 347 b 304 521 382 347 304 425 b 477 694 304 617 382 694 z m 182 141 b 347 260 205 210 270 260 l 369 260 b 477 239 403 247 439 239 b 585 260 516 239 552 247 l 608 260 b 773 141 685 260 750 210 b 731 87 782 114 759 87 l 224 87 b 182 141 196 87 173 114 z "
	},
	"": {
		ha: 694,
		x_min: 0,
		x_max: 0,
		o: "m 608 955 b 347 1215 608 1099 491 1215 b 87 955 203 1215 87 1099 l 87 403 b 0 175 33 342 0 263 b 345 -174 -1 -15 154 -172 b 347 -174 346 -174 346 -174 b 694 174 539 -174 694 -18 b 608 403 694 262 662 342 z m 347 -43 l 346 -43 b 130 175 227 -43 130 56 b 217 354 131 267 182 315 l 217 955 b 347 1085 217 1027 275 1085 b 477 955 419 1085 477 1027 l 477 354 b 564 174 512 315 564 267 b 347 -43 564 54 467 -43 z m 521 174 b 434 324 521 238 486 294 l 434 955 b 347 1042 434 1003 395 1042 b 260 955 299 1042 260 1003 l 260 324 b 174 174 209 294 174 238 b 347 0 174 78 251 0 b 521 174 443 0 521 78 z "
	},
	"": {
		ha: 694,
		x_min: 0,
		x_max: 0,
		o: "m 521 174 b 434 324 521 238 486 294 l 434 781 b 347 868 434 829 395 868 b 260 781 299 868 260 829 l 260 324 b 174 174 209 294 174 238 b 347 0 174 78 251 0 b 521 174 443 0 521 78 z m 608 403 l 608 955 b 347 1215 608 1099 491 1215 b 87 955 203 1215 87 1099 l 87 403 b 0 175 33 342 0 263 b 345 -174 -1 -15 154 -172 b 347 -174 346 -174 346 -174 b 694 174 539 -174 694 -18 b 608 403 694 262 662 342 z m 564 174 b 347 -43 564 54 467 -43 l 346 -43 b 130 175 227 -43 130 56 b 217 354 131 267 182 315 l 217 955 b 347 1085 217 1027 275 1085 b 477 955 419 1085 477 1027 l 477 354 b 564 174 512 315 564 267 z "
	},
	"": {
		ha: 694,
		x_min: 0,
		x_max: 0,
		o: "m 521 174 b 434 324 521 238 486 294 l 434 608 b 347 694 434 656 395 694 b 260 608 299 694 260 656 l 260 324 b 174 174 209 294 174 238 b 347 0 174 78 251 0 b 521 174 443 0 521 78 z m 608 403 l 608 955 b 347 1215 608 1099 491 1215 b 87 955 203 1215 87 1099 l 87 403 b 0 175 33 342 0 263 b 345 -174 -1 -15 154 -172 b 347 -174 346 -174 346 -174 b 694 174 539 -174 694 -18 b 608 403 694 262 662 342 z m 564 174 b 347 -43 564 54 467 -43 l 346 -43 b 130 175 227 -43 130 56 b 217 354 131 267 182 315 l 217 955 b 347 1085 217 1027 275 1085 b 477 955 419 1085 477 1027 l 477 354 b 564 174 512 315 564 267 z "
	},
	"": {
		ha: 694,
		x_min: 0,
		x_max: 0,
		o: "m 521 174 b 434 324 521 238 486 294 l 434 434 b 347 521 434 482 395 521 b 260 434 299 521 260 482 l 260 324 b 174 174 209 294 174 238 b 347 0 174 78 251 0 b 521 174 443 0 521 78 z m 608 403 l 608 955 b 347 1215 608 1099 491 1215 b 87 955 203 1215 87 1099 l 87 403 b 0 175 33 342 0 263 b 345 -174 -1 -15 154 -172 b 347 -174 346 -174 346 -174 b 694 174 539 -174 694 -18 b 608 403 694 262 662 342 z m 564 174 b 347 -43 564 54 467 -43 l 346 -43 b 130 175 227 -43 130 56 b 217 354 131 267 182 315 l 217 955 b 347 1085 217 1027 275 1085 b 477 955 419 1085 477 1027 l 477 354 b 564 174 512 315 564 267 z "
	},
	"": {
		ha: 694,
		x_min: 0,
		x_max: 0,
		o: "m 521 174 b 347 347 521 269 443 347 b 174 174 251 347 174 269 b 347 0 174 78 251 0 b 521 174 443 0 521 78 z m 608 403 l 608 955 b 347 1215 608 1099 491 1215 b 87 955 203 1215 87 1099 l 87 403 b 0 175 33 342 0 263 b 345 -174 -1 -15 154 -172 b 347 -174 346 -174 346 -174 b 694 174 539 -174 694 -18 b 608 403 694 262 662 342 z m 564 174 b 347 -43 564 54 467 -43 l 346 -43 b 130 175 227 -43 130 56 b 217 354 131 267 182 315 l 217 955 b 347 1085 217 1027 275 1085 b 477 955 419 1085 477 1027 l 477 354 b 564 174 512 315 564 267 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1057 847 b 1057 940 1082 873 1082 914 l 1026 970 b 934 970 1001 996 960 996 l 934 970 b 645 1027 852 1039 742 1058 b 385 1128 576 1090 485 1128 b 0 744 172 1128 0 956 l 0 -87 l 174 -87 l 174 744 b 385 955 174 860 268 955 b 497 923 426 955 464 943 b 506 542 410 810 413 651 l 505 541 b 505 449 480 516 480 475 l 536 419 b 628 419 562 393 603 393 z m 1042 651 b 998 694 1042 675 1022 694 b 955 651 974 694 955 675 b 998 608 955 627 974 608 b 1042 651 1022 608 1042 627 z m 1128 651 b 1172 608 1128 627 1148 608 b 1215 651 1196 608 1215 627 b 1172 694 1215 675 1196 694 b 1128 651 1148 694 1128 675 z m 1389 651 b 1345 694 1389 675 1369 694 b 1302 651 1322 694 1302 675 b 1345 608 1302 627 1322 608 b 1389 651 1369 608 1389 627 z m 955 564 b 911 608 955 588 935 608 b 868 564 887 608 868 588 b 911 521 868 540 887 521 b 955 564 935 521 955 540 z m 1085 608 b 1042 564 1061 608 1042 588 b 1085 521 1042 540 1061 521 b 1128 564 1109 521 1128 540 b 1085 608 1128 588 1109 608 z m 1302 564 b 1259 608 1302 588 1283 608 b 1215 564 1235 608 1215 588 b 1259 521 1215 540 1235 521 b 1302 564 1283 521 1302 540 z m 868 477 b 825 521 868 501 849 521 b 781 477 801 521 781 501 b 825 434 781 453 801 434 b 868 477 849 434 868 453 z m 955 477 b 998 434 955 453 974 434 b 1042 477 1022 434 1042 453 b 998 521 1042 501 1022 521 b 955 477 974 521 955 501 z m 1215 477 b 1172 521 1215 501 1196 521 b 1128 477 1148 521 1128 501 b 1172 434 1128 453 1148 434 b 1215 477 1196 434 1215 453 z m 868 391 b 911 347 868 367 887 347 b 955 391 935 347 955 367 b 911 434 955 415 935 434 b 868 391 887 434 868 415 z m 1128 391 b 1085 434 1128 415 1109 434 b 1042 391 1061 434 1042 415 b 1085 347 1042 367 1061 347 b 1128 391 1109 347 1128 367 z m 868 304 b 825 347 868 328 849 347 b 781 304 801 347 781 328 b 825 260 781 280 801 260 b 868 304 849 260 868 280 z m 1042 304 b 998 347 1042 328 1022 347 b 955 304 974 347 955 328 b 998 260 955 280 974 260 b 1042 304 1022 260 1042 280 z m 955 217 b 911 260 955 241 935 260 b 868 217 887 260 868 241 b 911 174 868 193 887 174 b 955 217 935 174 955 193 z m 868 130 b 825 174 868 154 849 174 b 781 130 801 174 781 154 b 825 87 781 106 801 87 b 868 130 849 87 868 106 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1324 521 l 217 521 l 217 911 b 304 998 217 959 256 998 b 377 958 335 998 362 982 b 396 733 332 890 337 796 b 397 688 384 720 385 700 l 428 657 b 474 657 440 645 461 645 l 731 915 b 731 961 744 928 744 949 l 701 992 b 656 993 689 1004 669 1005 b 482 1035 608 1038 542 1051 b 304 1128 443 1091 378 1128 b 87 911 184 1128 87 1031 l 87 521 l 65 521 b 0 456 29 521 0 492 l 0 412 b 65 347 0 376 29 347 l 87 347 l 87 260 b 174 66 87 183 120 114 l 174 -22 b 239 -87 174 -58 203 -87 l 282 -87 b 347 -22 318 -87 347 -58 l 347 0 l 1042 0 l 1042 -22 b 1107 -87 1042 -58 1071 -87 l 1150 -87 b 1215 -22 1186 -87 1215 -58 l 1215 66 b 1302 260 1269 114 1302 183 l 1302 347 l 1324 347 b 1389 412 1360 347 1389 376 l 1389 456 b 1324 521 1389 492 1360 521 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 725 -110 b 781 250 746 -32 781 161 b 608 369 781 346 697 369 b 434 250 519 369 434 346 b 490 -110 434 160 469 -33 b 608 -174 504 -162 556 -174 b 725 -110 659 -174 711 -162 z m 426 433 b 347 629 375 482 344 552 b 597 879 353 764 462 873 b 868 618 746 884 868 765 b 790 433 868 546 838 480 b 791 408 782 425 783 414 b 849 339 817 389 836 366 b 875 334 854 329 867 326 b 998 628 953 407 1001 512 b 619 1009 993 835 826 1003 b 217 618 398 1015 217 838 b 341 334 217 506 265 405 b 366 339 349 326 361 329 b 424 408 379 366 399 389 b 426 433 432 414 433 425 z m 608 1215 b 0 608 272 1215 0 944 b 341 61 0 364 143 158 b 364 78 353 56 366 65 b 350 199 358 120 352 162 b 342 211 349 204 346 208 b 130 611 214 297 129 444 b 606 1085 132 872 345 1084 b 1085 608 870 1086 1085 871 b 866 207 1085 440 998 292 b 851 78 864 168 858 124 b 874 61 849 65 862 56 b 1215 608 1072 158 1215 363 b 608 1215 1215 943 944 1215 z m 608 781 b 434 608 512 781 434 704 b 608 434 434 512 512 434 b 781 608 704 434 781 512 b 608 781 781 704 704 781 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1259 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -28 58 -87 l 1259 -87 b 1389 43 1331 -87 1389 -28 l 1389 998 b 1259 1128 1389 1070 1331 1128 z m 1215 694 l 174 694 l 174 922 b 206 955 174 940 188 955 l 1183 955 b 1215 922 1201 955 1215 940 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1259 260 l 130 260 b 0 130 58 260 0 202 l 0 43 b 130 -87 0 -28 58 -87 l 1259 -87 b 1389 43 1331 -87 1389 -28 l 1389 130 b 1259 260 1389 202 1331 260 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1389 1085 b 1259 1215 1389 1157 1331 1215 l 477 1215 b 347 1085 406 1215 347 1157 l 347 955 l 911 955 b 1128 738 1031 955 1128 857 l 1128 174 l 1259 174 b 1389 304 1331 174 1389 232 z m 1042 738 b 911 868 1042 810 983 868 l 130 868 b 0 738 58 868 0 810 l 0 -43 b 130 -174 0 -115 58 -174 l 911 -174 b 1042 -43 983 -174 1042 -115 z m 857 662 l 857 521 l 174 521 l 174 662 b 206 694 174 680 188 694 l 825 694 b 857 662 843 694 857 680 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1128 1085 b 998 1215 1128 1157 1070 1215 l 391 1215 b 260 1085 319 1215 260 1157 l 260 -43 b 391 -174 260 -115 319 -174 l 998 -174 b 1128 -43 1070 -174 1128 -115 z m 1389 928 b 1373 944 1389 937 1382 944 l 1324 944 l 1324 960 b 1308 977 1324 969 1316 977 l 1194 977 l 1194 846 l 1308 846 b 1324 863 1316 846 1324 854 l 1324 879 l 1373 879 b 1389 895 1382 879 1389 886 z m 1389 667 b 1373 684 1389 676 1382 684 l 1324 684 l 1324 700 b 1308 716 1324 709 1316 716 l 1194 716 l 1194 586 l 1308 586 b 1324 602 1316 586 1324 593 l 1324 618 l 1373 618 b 1389 635 1382 618 1389 626 z m 1389 407 b 1373 423 1389 416 1382 423 l 1324 423 l 1324 439 b 1308 456 1324 448 1316 456 l 1194 456 l 1194 326 l 1308 326 b 1324 342 1316 326 1324 333 l 1324 358 l 1373 358 b 1389 374 1382 358 1389 365 z m 1389 146 b 1373 163 1389 155 1382 163 l 1324 163 l 1324 179 b 1308 195 1324 188 1316 195 l 1194 195 l 1194 65 l 1308 65 b 1324 81 1316 65 1324 72 l 1324 98 l 1373 98 b 1389 114 1382 98 1389 105 z m 81 195 b 65 179 72 195 65 188 l 65 163 l 16 163 b 0 146 7 163 0 155 l 0 114 b 16 98 0 105 7 98 l 65 98 l 65 81 b 81 65 65 72 72 65 l 195 65 l 195 195 z m 81 456 b 65 439 72 456 65 448 l 65 423 l 16 423 b 0 407 7 423 0 416 l 0 374 b 16 358 0 365 7 358 l 65 358 l 65 342 b 81 326 65 333 72 326 l 195 326 l 195 456 z m 81 716 b 65 700 72 716 65 709 l 65 684 l 16 684 b 0 667 7 684 0 676 l 0 635 b 16 618 0 626 7 618 l 65 618 l 65 602 b 81 586 65 593 72 586 l 195 586 l 195 716 z m 81 977 b 65 960 72 977 65 969 l 65 944 l 16 944 b 0 928 7 944 0 937 l 0 895 b 16 879 0 886 7 879 l 65 879 l 65 863 b 81 846 65 854 72 846 l 195 846 l 195 977 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1194 279 l 1103 332 l 1173 351 b 1203 402 1195 357 1209 380 l 1192 442 b 1141 472 1186 465 1163 478 l 949 420 l 775 521 l 948 621 l 1140 569 b 1192 599 1163 563 1186 577 l 1203 639 b 1173 691 1208 662 1195 685 l 1102 710 l 1194 763 b 1210 820 1214 774 1221 800 l 1168 892 b 1110 908 1156 913 1130 919 l 1019 855 l 1038 926 b 1008 977 1044 948 1030 971 l 968 988 b 916 958 945 994 922 980 l 865 766 l 694 668 l 694 878 l 840 1024 b 840 1085 857 1041 857 1068 l 809 1116 b 748 1116 793 1133 765 1133 l 694 1062 l 694 1172 b 651 1215 694 1196 675 1215 l 564 1215 b 521 1172 540 1215 521 1196 l 521 1062 l 467 1116 b 406 1116 451 1133 423 1133 l 375 1085 b 375 1024 358 1068 358 1041 l 521 878 l 521 668 l 350 766 l 299 958 b 248 988 293 980 270 994 l 207 977 b 178 926 185 971 172 948 l 197 855 l 105 908 b 48 892 85 920 59 913 l 6 820 b 21 763 -6 800 1 774 l 113 710 l 42 691 b 12 639 20 685 7 662 l 23 599 b 75 569 29 577 52 563 l 267 621 l 440 521 l 267 421 l 75 472 b 23 443 52 478 29 465 l 12 402 b 42 351 7 380 20 357 l 113 332 l 21 279 b 5 222 1 267 -6 242 l 47 149 b 105 133 59 129 85 122 l 196 186 l 177 116 b 207 65 171 94 185 71 l 247 54 b 299 83 270 48 293 61 l 350 275 l 521 374 l 521 163 l 375 18 b 375 -44 358 1 358 -27 l 406 -74 b 467 -74 422 -91 450 -91 l 520 -21 l 520 -130 b 564 -174 520 -154 540 -174 l 650 -174 b 694 -130 674 -174 694 -154 l 694 -21 l 747 -74 b 809 -74 764 -91 792 -91 l 839 -43 b 839 18 856 -27 856 1 l 694 164 l 694 374 l 865 276 l 916 84 b 968 54 922 61 945 48 l 1008 65 b 1038 116 1030 71 1044 94 l 1019 187 l 1110 134 b 1168 149 1130 122 1156 129 l 1210 222 b 1194 279 1221 242 1214 267 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1302 1129 b 684 1051 1153 1278 855 1222 b 606 616 551 918 535 753 l 24 94 b 22 -13 -7 66 -8 17 l 161 -152 b 268 -149 191 -182 240 -180 l 790 433 b 1225 511 927 361 1091 377 b 1302 1129 1396 681 1452 980 z "
	},
	"": {
		ha: 1128,
		x_min: 0,
		x_max: 0,
		o: "m 564 1174 b 434 1171 555 1229 441 1230 l 434 788 b 390 788 431 779 393 779 b 369 1173 387 857 369 1165 b 239 1173 360 1229 248 1229 b 217 788 238 1166 221 857 b 174 788 215 779 177 779 l 174 1171 b 44 1174 167 1230 52 1229 b 0 825 42 1161 0 918 b 187 541 0 683 75 582 l 152 -105 b 217 -174 150 -142 180 -174 l 391 -174 b 456 -105 428 -174 458 -142 l 421 541 b 608 825 532 582 608 683 b 564 1174 608 918 566 1161 z m 887 399 l 847 -103 b 911 -174 843 -141 873 -174 l 1063 -174 b 1128 -109 1099 -174 1128 -145 l 1128 1150 b 1063 1215 1128 1186 1099 1215 b 887 399 840 1215 463 731 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 693 1194 b 230 1007 513 1193 350 1122 l 133 1104 b 22 1058 92 1145 22 1116 l 22 694 b 87 629 22 658 51 629 l 450 629 b 497 740 508 629 538 699 l 383 854 b 690 977 467 932 575 976 b 1150 517 941 979 1152 776 b 694 65 1148 271 949 65 b 394 178 583 65 477 105 b 350 176 381 189 362 188 l 242 69 b 244 21 229 56 230 34 b 694 -152 363 -86 521 -152 b 1367 521 1066 -152 1367 149 b 693 1194 1367 892 1064 1194 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 87 -43 b 217 -174 87 -115 145 -174 l 998 -174 b 1128 -43 1070 -174 1128 -115 l 1128 868 l 87 868 z m 825 651 b 868 694 825 675 844 694 b 911 651 892 694 911 675 l 911 43 b 868 0 911 19 892 0 b 825 43 844 0 825 19 z m 564 651 b 608 694 564 675 584 694 b 651 651 632 694 651 675 l 651 43 b 608 0 651 19 632 0 b 564 43 584 0 564 19 z m 304 651 b 347 694 304 675 323 694 b 391 651 371 694 391 675 l 391 43 b 347 0 391 19 371 0 b 304 43 323 0 304 19 z m 1172 1128 l 846 1128 l 821 1179 b 763 1215 811 1199 785 1215 l 763 1215 l 452 1215 l 452 1215 b 394 1179 430 1215 404 1199 l 369 1128 l 43 1128 b 0 1085 19 1128 0 1109 l 0 998 b 43 955 0 974 19 955 l 1172 955 b 1215 998 1196 955 1215 974 l 1215 1085 b 1172 1128 1215 1109 1196 1128 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1006 854 l 892 740 b 938 629 851 699 880 629 l 1302 629 b 1367 694 1338 629 1367 658 l 1367 1058 b 1256 1104 1367 1116 1297 1145 l 1159 1007 b 694 1194 1038 1123 875 1194 b 33 646 366 1194 92 958 b 65 608 30 626 45 608 l 221 608 b 252 632 236 608 249 618 b 694 977 303 832 484 976 b 1006 854 811 977 921 933 z m 87 412 b 22 347 51 412 22 383 l 22 -16 b 133 -62 22 -74 92 -103 l 230 34 b 694 -152 351 -81 514 -152 b 1356 396 1023 -152 1297 84 b 1324 434 1359 416 1344 434 l 1168 434 b 1136 409 1153 434 1140 424 b 695 65 1086 209 905 65 b 383 188 578 65 468 109 l 497 301 b 450 412 538 342 508 412 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1172 391 b 1081 697 1172 504 1139 610 l 1083 699 l 1162 779 b 1162 825 1175 792 1175 812 l 1086 902 b 1039 902 1073 914 1052 914 l 965 827 b 694 948 889 889 796 932 l 694 1042 l 770 1042 b 803 1074 788 1042 803 1056 l 803 1183 b 770 1215 803 1201 788 1215 l 445 1215 b 412 1183 427 1215 412 1201 l 412 1074 b 445 1042 412 1056 427 1042 l 521 1042 l 521 948 b 43 391 250 907 43 673 b 608 -174 43 79 296 -174 b 1172 391 919 -174 1172 79 z m 694 293 b 662 260 694 275 680 260 l 553 260 b 521 293 535 260 521 275 l 521 704 b 553 736 521 722 535 736 l 662 736 b 694 704 680 736 694 722 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1348 475 b 1348 567 1373 500 1373 541 l 892 1023 b 781 977 852 1063 781 1035 l 781 716 l 412 716 b 347 651 376 716 347 687 l 347 391 b 412 326 347 355 376 326 l 781 326 l 781 65 b 892 19 781 7 852 -22 z m 521 33 l 521 141 b 488 174 521 159 506 174 l 260 174 b 174 260 212 174 174 212 l 174 781 b 260 868 174 829 212 868 l 488 868 b 521 901 506 868 521 883 l 521 1009 b 488 1042 521 1027 506 1042 l 260 1042 b 0 781 117 1042 0 925 l 0 260 b 260 0 0 117 117 0 l 488 0 b 521 33 506 0 521 15 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1128 0 b 1389 260 1272 0 1389 117 l 1389 781 b 1128 1042 1389 925 1272 1042 l 901 1042 b 868 1009 883 1042 868 1027 l 868 901 b 901 868 868 883 883 868 l 1128 868 b 1215 781 1176 868 1215 829 l 1215 260 b 1128 174 1215 212 1176 174 l 901 174 b 868 141 883 174 868 159 l 868 33 b 901 0 868 15 883 0 z m 1001 545 l 545 1001 b 434 955 505 1042 434 1013 l 434 694 l 65 694 b 0 629 29 694 0 665 l 0 369 b 65 304 0 333 29 304 l 434 304 l 434 43 b 545 -3 434 -15 505 -43 l 1001 453 b 1001 545 1026 479 1026 520 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 696 1194 b 22 521 325 1194 22 892 b 694 -152 22 149 323 -152 b 1145 21 868 -152 1026 -86 b 1147 69 1159 34 1160 56 l 1039 176 b 995 178 1027 188 1007 189 b 694 65 911 105 806 65 b 239 517 440 65 241 271 b 698 977 237 776 448 979 b 1006 854 814 976 922 932 l 892 740 b 938 629 851 699 880 629 l 1302 629 b 1367 694 1338 629 1367 658 l 1367 1058 b 1256 1104 1367 1116 1297 1145 l 1159 1007 b 696 1194 1039 1122 875 1193 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1225 214 b 1302 369 1271 250 1302 305 b 1107 564 1302 477 1215 564 l 1069 564 b 1128 694 1105 596 1128 642 b 955 868 1128 790 1051 868 l 939 868 b 955 955 949 895 955 924 b 694 1215 955 1099 838 1215 b 653 1211 680 1215 667 1213 b 694 1085 679 1176 694 1132 b 477 868 694 965 597 868 l 434 868 b 260 694 338 868 260 790 b 320 564 260 642 284 596 l 282 564 b 87 369 174 564 87 477 b 164 214 87 305 117 250 b 0 22 71 199 0 119 b 195 -174 0 -86 87 -174 l 1194 -174 b 1389 22 1302 -174 1389 -86 b 1225 214 1389 119 1318 199 z m 521 521 b 608 434 569 521 608 482 b 521 347 608 386 569 347 b 434 434 473 347 434 386 b 521 521 434 482 473 521 z m 954 144 b 694 0 925 68 795 0 b 435 144 594 0 464 68 b 457 174 430 158 441 174 l 932 174 b 954 144 948 174 959 158 z m 868 347 b 781 434 820 347 781 386 b 868 521 781 482 820 521 b 955 434 916 521 955 482 b 868 347 955 386 916 347 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1302 87 l 391 87 b 174 304 271 87 174 184 l 174 868 l 130 868 b 0 738 58 868 0 810 l 0 43 b 130 -87 0 -29 58 -87 l 1172 -87 b 1302 43 1244 -87 1302 -29 z m 1563 304 l 1563 998 b 1432 1128 1563 1070 1504 1128 l 391 1128 b 260 998 319 1128 260 1070 l 260 304 b 391 174 260 232 319 174 l 1432 174 b 1563 304 1504 174 1563 232 z m 694 868 b 564 738 694 796 636 738 b 434 868 492 738 434 796 b 564 998 434 940 492 998 b 694 868 636 998 694 940 z m 434 477 l 585 628 b 631 628 597 641 618 641 l 738 521 l 1105 888 b 1151 888 1118 901 1139 901 l 1389 651 l 1389 347 l 434 347 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1351 830 b 1351 1014 1402 881 1402 963 l 1188 1177 b 1003 1177 1137 1228 1054 1228 l 878 1052 b 878 1006 866 1039 866 1019 l 1179 705 b 1226 705 1192 692 1213 692 z m 771 945 l 59 232 l 1 -97 b 76 -173 -7 -142 32 -180 l 406 -115 l 1118 597 b 1118 643 1131 610 1131 631 l 817 945 b 771 945 804 957 784 957 z m 337 293 b 337 347 322 308 322 332 l 754 765 b 808 765 769 780 793 780 b 808 711 823 750 823 726 l 390 293 b 337 293 375 278 352 278 z m 239 65 l 369 65 l 369 -33 l 194 -64 l 110 20 l 140 195 l 239 195 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 789 962 l 35 209 l 0 -101 b 72 -173 -4 -143 31 -178 l 382 -139 l 1136 615 z m 1351 1014 l 1188 1177 b 1003 1177 1137 1228 1054 1228 l 850 1024 l 1197 676 l 1351 830 b 1351 1014 1402 881 1402 963 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1351 1014 l 1188 1177 b 1003 1177 1137 1228 1054 1228 l 850 1024 l 1197 676 l 1351 830 b 1351 1014 1402 881 1402 963 z m 681 1070 b 528 1070 639 1112 570 1112 l 206 748 b 206 686 189 731 189 703 l 267 625 b 328 625 284 608 311 608 l 605 901 l 666 840 l 255 429 b 0 -101 133 307 19 70 l 0 -101 b 72 -173 -4 -143 31 -178 b 602 81 243 -154 481 -40 l 1136 615 l 911 840 z "
	},
	"": {
		ha: 694,
		x_min: 0,
		x_max: 0,
		o: "m 456 277 l 456 1096 b 423 1128 456 1114 441 1128 l 271 1128 b 239 1096 253 1128 239 1114 l 239 277 l 114 277 b 68 166 56 277 27 207 l 301 -68 b 393 -68 327 -93 368 -93 l 627 166 b 581 277 668 207 639 277 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 364 412 l 1183 412 b 1215 445 1201 412 1215 427 l 1215 597 b 1183 629 1215 615 1201 629 l 364 629 l 364 754 b 253 800 364 812 294 841 l 19 567 b 19 475 -6 541 -6 500 l 253 241 b 364 287 294 200 364 229 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 852 629 l 33 629 b 0 597 15 629 0 615 l 0 445 b 33 412 0 427 15 412 l 852 412 l 852 287 b 963 241 852 229 922 200 l 1196 475 b 1196 567 1222 500 1222 541 l 963 800 b 852 754 922 841 852 812 z "
	},
	"": {
		ha: 694,
		x_min: 0,
		x_max: 0,
		o: "m 239 765 l 239 -54 b 271 -87 239 -72 253 -87 l 423 -87 b 456 -54 441 -87 456 -72 l 456 765 l 581 765 b 627 876 639 765 668 835 l 393 1109 b 301 1109 368 1135 327 1135 l 68 876 b 114 765 27 835 56 765 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1215 282 b 1104 328 1215 340 1145 369 l 1006 230 l 715 521 l 1006 812 l 1104 713 b 1215 760 1145 672 1215 701 l 1215 1063 l 1215 1064 b 1150 1128 1215 1099 1186 1128 l 1150 1128 l 846 1128 b 800 1017 788 1128 759 1058 l 899 919 l 608 628 l 317 919 l 415 1017 b 369 1128 456 1058 427 1128 l 65 1128 l 65 1128 b 0 1064 29 1128 0 1099 l 0 1063 l 0 760 b 111 713 0 701 70 672 l 209 812 l 500 521 l 209 230 l 111 328 b 0 282 70 369 0 340 l 0 -22 l 0 -22 b 65 -87 0 -58 29 -87 l 65 -87 l 369 -87 b 415 24 427 -87 456 -17 l 317 122 l 608 413 l 898 123 l 800 24 b 846 -87 759 -17 788 -87 l 1150 -87 l 1150 -87 b 1215 -22 1186 -87 1215 -58 l 1215 -22 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 1042 911 b 911 1042 1042 983 983 1042 l 694 1042 b 521 1215 694 1137 617 1215 b 347 1042 425 1215 347 1137 l 130 1042 b 0 911 58 1042 0 983 l 0 -43 b 130 -174 0 -115 58 -174 l 911 -174 b 1042 -43 983 -174 1042 -115 z m 521 1107 b 586 1042 557 1107 586 1078 b 521 977 586 1006 557 977 b 456 1042 485 977 456 1006 b 521 1107 456 1078 485 1107 z m 781 798 b 765 781 781 789 774 781 l 277 781 b 260 798 268 781 260 789 l 260 852 b 277 868 260 861 268 868 l 765 868 b 781 852 774 868 781 861 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1025 754 l 1025 629 l 364 629 l 364 754 b 253 800 364 812 294 841 l 19 567 b 19 475 -6 541 -6 500 l 253 241 b 364 287 294 200 364 229 l 364 412 l 1025 412 l 1025 287 b 1136 241 1025 229 1095 200 l 1370 475 b 1370 567 1395 500 1395 541 l 1136 800 b 1025 754 1095 841 1025 812 z "
	},
	"": {
		ha: 694,
		x_min: 0,
		x_max: 0,
		o: "m 581 190 l 456 190 l 456 852 l 581 852 b 627 963 639 852 668 922 l 393 1196 b 301 1196 368 1222 327 1222 l 68 963 b 114 852 27 922 56 852 l 239 852 l 239 190 l 114 190 b 68 79 56 190 27 120 l 301 -155 b 393 -155 327 -180 368 -180 l 627 79 b 581 190 668 120 639 190 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1367 521 b 694 1194 1367 892 1066 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 z m 575 836 b 608 868 575 853 590 868 l 781 868 b 814 836 799 868 814 853 l 814 521 l 1006 521 b 1029 465 1035 521 1050 486 l 717 155 b 671 155 704 142 684 142 l 360 465 b 383 521 339 486 354 521 l 575 521 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 z m 1009 640 b 1042 608 1027 640 1042 626 l 1042 434 b 1009 401 1042 416 1027 401 l 694 401 l 694 209 b 639 186 694 180 659 166 l 329 498 b 329 544 316 511 316 531 l 639 856 b 694 833 659 876 694 862 l 694 640 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1194 b 22 521 323 1194 22 892 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 z m 380 401 b 347 434 362 401 347 416 l 347 608 b 380 640 347 626 362 640 l 694 640 l 694 833 b 750 856 694 862 729 876 l 1060 544 b 1060 498 1073 531 1073 511 l 750 186 b 694 209 730 165 694 180 l 694 401 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 22 521 b 694 -152 22 149 323 -152 b 1367 521 1066 -152 1367 149 b 694 1194 1367 892 1066 1194 b 22 521 323 1194 22 892 z m 814 206 b 781 174 814 188 799 174 l 608 174 b 575 206 590 174 575 188 l 575 521 l 383 521 b 360 576 354 521 339 556 l 672 887 b 718 887 684 899 705 899 l 1029 576 b 1006 521 1050 556 1035 521 l 814 521 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1563 1150 b 1497 1215 1563 1186 1533 1215 l 1150 1215 b 1104 1104 1092 1215 1063 1145 l 1201 1007 l 540 347 b 540 255 515 321 515 280 l 602 193 b 694 193 627 168 668 168 l 1354 854 l 1451 757 b 1563 803 1492 716 1563 745 z m 1104 481 l 1061 437 b 1042 391 1050 427 1042 406 l 1042 391 l 1042 0 l 174 0 l 174 868 l 890 868 b 936 887 905 868 925 877 l 979 931 b 933 1042 1020 972 991 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 435 b 1104 481 1215 493 1145 522 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1215 998 b 1085 1128 1215 1070 1157 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -29 58 -87 l 1085 -87 b 1215 43 1157 -87 1215 -29 z m 977 955 b 1042 890 1013 955 1042 926 l 1042 586 b 931 540 1042 528 971 499 l 844 627 l 313 96 b 267 96 301 84 280 84 l 183 181 b 183 227 170 193 170 214 l 714 757 l 627 844 b 673 955 586 885 615 955 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 0 760 b 65 694 0 724 29 694 l 1042 694 l 1042 564 b 1153 518 1042 506 1112 477 l 1370 735 b 1370 827 1395 761 1395 802 l 1153 1044 b 1042 998 1112 1085 1042 1056 l 1042 868 l 65 868 b 0 803 29 868 0 839 z m 1324 347 l 347 347 l 347 477 b 236 523 347 535 277 564 l 19 306 b 19 214 -6 281 -6 240 l 236 -3 b 347 43 277 -43 347 -15 l 347 174 l 1324 174 b 1389 239 1360 174 1389 203 l 1389 282 b 1324 347 1389 318 1360 347 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1458 601 b 1476 694 1469 630 1476 661 b 1215 955 1476 838 1359 955 b 1071 911 1162 955 1112 939 b 694 1128 996 1041 855 1128 b 260 694 455 1128 260 934 b 261 672 260 687 261 680 b 0 304 109 619 0 474 b 391 -87 0 88 175 -87 l 1389 -87 b 1736 260 1581 -87 1736 69 b 1458 601 1736 428 1617 569 z m 1098 360 l 812 74 b 751 74 795 57 767 57 l 465 360 b 495 434 437 387 457 434 l 673 434 l 673 738 b 716 781 673 762 692 781 l 846 781 b 890 738 870 781 890 762 l 890 434 l 1067 434 b 1098 360 1106 434 1125 387 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1458 601 b 1476 694 1469 630 1476 661 b 1215 955 1476 838 1359 955 b 1071 911 1162 955 1112 939 b 694 1128 996 1041 855 1128 b 260 694 455 1128 260 934 b 261 672 260 687 261 680 b 0 304 109 619 0 474 b 391 -87 0 88 175 -87 l 1389 -87 b 1736 260 1581 -87 1736 69 b 1458 601 1736 428 1617 569 z m 1067 434 l 890 434 l 890 130 b 846 87 890 106 870 87 l 716 87 b 673 130 692 87 673 106 l 673 434 l 495 434 b 465 508 457 434 437 481 l 751 794 b 812 794 767 811 795 811 l 1098 508 b 1067 434 1125 481 1106 434 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1317 1215 l 1101 1215 l 1288 781 l 1563 781 z m 970 1215 l 593 1215 l 405 781 l 1157 781 z m 245 1215 l 0 781 l 274 781 l 462 1215 z m 0 694 l 593 3 b 607 12 600 -4 611 3 l 273 694 z m 402 694 l 774 -169 b 789 -169 777 -175 786 -175 l 1160 694 z m 956 12 b 969 3 952 3 963 -4 l 1563 694 l 1289 694 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 851 151 b 803 260 889 193 859 260 l 629 260 l 629 1150 b 564 1215 629 1186 600 1215 l 33 1215 b 10 1160 4 1215 -11 1180 l 162 1008 b 185 998 167 1003 177 998 l 185 998 l 412 998 l 412 260 l 239 260 b 191 151 182 260 153 193 l 473 -153 b 569 -153 499 -181 543 -181 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 851 891 l 569 1194 b 473 1194 543 1222 499 1222 l 191 891 b 239 781 153 849 182 781 l 412 781 l 412 43 l 185 43 l 185 43 b 162 34 177 43 167 39 l 10 -118 b 33 -174 -11 -139 4 -174 l 564 -174 b 629 -109 600 -174 629 -144 l 629 781 l 803 781 b 851 891 859 781 889 849 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1149 1215 b 738 799 921 1214 738 1027 l 738 608 l 130 608 b 0 477 58 608 0 549 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 477 b 1085 608 1215 549 1157 608 l 955 608 l 955 801 b 1148 998 955 908 1041 997 b 1345 803 1257 999 1345 911 l 1345 586 b 1411 521 1345 550 1375 521 l 1497 521 b 1563 586 1533 521 1563 550 l 1563 803 b 1149 1215 1563 1031 1377 1216 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 467 -146 b 574 -146 493 -183 548 -183 b 1042 694 969 426 1042 484 b 521 1215 1042 982 808 1215 b 0 694 233 1215 0 982 b 467 -146 0 484 73 426 z m 521 477 b 304 694 401 477 304 575 b 521 911 304 814 401 911 b 738 694 641 911 738 814 b 521 477 738 575 641 477 z "
	},
	"": {
		ha: 955,
		x_min: 0,
		x_max: 0,
		o: "m 911 694 l 868 694 b 825 651 844 694 825 675 l 825 521 b 443 175 825 318 650 155 b 130 536 262 193 130 355 l 130 651 b 87 694 130 675 111 694 l 43 694 b 0 651 19 694 0 675 l 0 542 b 412 49 0 299 174 82 l 412 -43 l 260 -43 b 217 -87 236 -43 217 -63 l 217 -130 b 260 -174 217 -154 236 -174 l 694 -174 b 738 -130 718 -174 738 -154 l 738 -87 b 694 -43 738 -63 718 -43 l 543 -43 l 543 48 b 955 521 775 80 955 280 l 955 651 b 911 694 955 675 935 694 z m 477 260 b 738 521 621 260 738 377 l 506 521 b 477 543 490 521 477 531 l 477 586 b 506 608 477 598 490 608 l 738 608 l 738 694 l 506 694 b 477 716 490 694 477 704 l 477 760 b 506 781 477 772 490 781 l 738 781 l 738 868 l 506 868 b 477 890 490 868 477 878 l 477 933 b 506 955 477 945 490 955 l 738 955 b 477 1215 738 1099 621 1215 b 217 955 334 1215 217 1099 l 217 521 b 477 260 217 377 334 260 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 738 1215 l 130 1215 b 0 1085 58 1215 0 1157 l 0 -43 b 130 -174 0 -115 58 -174 l 738 -174 b 868 -43 810 -174 868 -115 l 868 1085 b 738 1215 868 1157 810 1215 z m 434 -87 b 347 0 386 -87 347 -48 b 434 87 347 48 386 87 b 521 0 482 87 521 48 b 434 -87 521 -48 482 -87 z m 738 206 b 705 174 738 188 723 174 l 163 174 b 130 206 145 174 130 188 l 130 1053 b 163 1085 130 1070 145 1085 l 705 1085 b 738 1053 723 1085 738 1070 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 955 434 l 911 434 l 911 673 b 890 694 911 685 902 694 l 853 694 b 817 684 840 694 828 691 l 775 656 b 766 638 770 652 766 644 b 769 626 766 634 767 629 l 793 590 b 811 580 797 584 805 580 b 823 584 815 580 820 582 l 825 584 l 825 434 l 781 434 b 760 412 769 434 760 424 l 760 369 b 781 347 760 357 769 347 l 955 347 b 977 369 967 347 977 357 l 977 412 b 955 434 977 424 967 434 z m 1649 1042 l 87 1042 b 0 955 39 1042 0 1003 l 0 87 b 87 0 0 39 39 0 l 1649 0 b 1736 87 1697 0 1736 39 l 1736 955 b 1649 1042 1736 1003 1697 1042 z m 130 130 l 130 304 b 304 130 226 304 304 226 z m 130 738 l 130 911 l 304 911 b 130 738 304 816 226 738 z m 868 217 b 608 521 724 217 608 353 b 868 825 608 689 724 825 b 1128 521 1012 825 1128 689 b 868 217 1128 353 1012 217 z m 1606 130 l 1432 130 b 1606 304 1432 226 1510 304 z m 1606 738 b 1432 911 1510 738 1432 816 l 1606 911 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 728 181 l 593 345 b 517 364 575 368 544 375 l 213 234 b 176 159 184 221 168 190 l 241 -123 b 304 -174 247 -152 273 -174 b 1036 62 577 -174 830 -86 l 819 230 b 728 181 789 212 759 195 z m 1719 -27 l 1289 305 b 1563 1085 1459 520 1563 790 b 1512 1149 1563 1115 1542 1142 l 1230 1214 b 1155 1176 1199 1221 1168 1205 l 1025 872 b 1044 796 1014 846 1021 814 l 1208 662 b 1083 464 1175 591 1133 525 l 123 1206 b 62 1198 104 1221 77 1217 l 9 1130 b 17 1069 -5 1111 -2 1084 l 1613 -164 b 1674 -157 1632 -179 1659 -176 l 1727 -88 b 1719 -27 1742 -69 1738 -42 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 911 1215 l 130 1215 b 0 1085 58 1215 0 1157 l 0 -43 b 130 -174 0 -115 58 -174 l 911 -174 b 1042 -43 983 -174 1042 -115 l 1042 1085 b 911 1215 1042 1157 983 1215 z m 521 868 b 694 694 617 868 694 790 b 521 521 694 599 617 521 b 347 694 425 521 347 599 b 521 868 347 790 425 868 z m 825 226 b 764 174 825 197 798 174 l 278 174 b 217 226 244 174 217 197 l 217 278 b 399 434 217 364 299 434 l 413 434 b 521 412 446 420 483 412 b 629 434 559 412 596 420 l 642 434 b 825 278 743 434 825 364 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 23 700 b 23 602 -8 674 -7 628 l 500 189 b 608 239 542 153 608 183 l 608 477 b 1087 -27 1028 471 1210 367 b 1163 -77 1073 -70 1126 -104 b 1389 341 1282 9 1389 174 b 608 846 1389 754 1043 841 l 608 1063 b 500 1113 608 1119 542 1149 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1265 988 l 745 1205 b 695 1215 731 1211 709 1215 b 645 1205 680 1215 658 1211 l 124 988 b 43 868 75 968 43 921 b 644 -164 43 330 354 -43 b 744 -164 676 -177 712 -177 b 1345 868 977 -67 1345 268 b 1265 988 1345 921 1314 968 z m 695 5 l 694 1038 l 1172 839 b 695 5 1163 429 949 131 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1085 1215 l 130 1215 b 0 1085 58 1215 0 1157 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 1085 b 1085 1215 1215 1157 1157 1215 z m 608 -87 b 521 0 560 -87 521 -48 b 608 87 521 48 560 87 b 694 0 656 87 694 48 b 608 -87 694 -48 656 -87 z m 1085 206 b 1053 174 1085 188 1070 174 l 163 174 b 130 206 145 174 130 188 l 130 1053 b 163 1085 130 1070 145 1085 l 1053 1085 b 1085 1053 1070 1085 1085 1070 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 781 1128 b 0 347 350 1128 0 779 b 106 -46 0 204 39 70 b 180 -87 121 -72 150 -87 l 1382 -87 b 1457 -46 1412 -87 1441 -72 b 1563 347 1524 70 1563 204 b 781 1128 1563 779 1213 1128 z m 781 955 b 863 891 821 955 853 927 b 854 873 860 885 856 879 l 829 798 b 781 781 815 788 799 781 b 694 868 733 781 694 820 b 781 955 694 916 733 955 z m 260 174 b 174 260 212 174 174 212 b 260 347 174 308 212 347 b 347 260 308 347 347 308 b 260 174 347 212 308 174 z m 391 608 b 304 694 343 608 304 647 b 391 781 304 742 343 781 b 477 694 439 781 477 742 b 391 608 477 647 439 608 z m 1060 804 l 894 305 b 955 174 931 273 955 226 b 931 87 955 142 946 112 l 632 87 b 608 174 617 112 608 142 b 770 346 608 266 680 340 l 937 845 b 1019 886 948 879 985 898 b 1060 804 1053 875 1071 838 z m 1100 649 l 1142 775 b 1172 781 1151 779 1161 781 b 1259 694 1220 781 1259 742 b 1172 608 1259 646 1220 608 b 1100 649 1141 608 1115 625 z m 1302 174 b 1215 260 1254 174 1215 212 b 1302 347 1215 308 1254 347 b 1389 260 1350 347 1389 308 b 1302 174 1389 212 1350 174 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 347 781 l 347 260 l 1215 260 l 1215 781 z m 1432 521 b 1563 651 1432 593 1491 651 l 1563 911 b 1432 1042 1563 983 1504 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 651 b 130 521 72 651 130 593 b 0 391 130 449 72 391 l 0 130 b 130 0 0 58 58 0 l 1432 0 b 1563 130 1504 0 1563 58 l 1563 391 b 1432 521 1491 391 1432 449 z m 1302 803 l 1302 239 b 1237 174 1302 203 1273 174 l 326 174 b 260 239 290 174 260 203 l 260 803 b 326 868 260 839 290 868 l 1237 868 b 1302 803 1273 868 1302 839 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 434 b 1085 825 910 434 1085 609 b 694 1215 1085 1040 910 1215 b 304 825 479 1215 304 1040 b 694 434 304 609 479 434 z m 1042 347 l 892 347 b 694 304 832 320 765 304 b 497 347 624 304 557 320 l 347 347 b 0 0 155 347 0 192 l 0 -43 b 130 -174 0 -115 58 -174 l 1259 -174 b 1389 -43 1331 -174 1389 -115 l 1389 0 b 1042 347 1389 192 1233 347 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1259 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -28 58 -87 l 1259 -87 b 1389 43 1331 -87 1389 -28 l 1389 998 b 1259 1128 1389 1070 1331 1128 z m 1032 340 b 1032 293 1045 327 1045 306 l 922 183 b 875 183 909 170 888 170 l 694 365 l 514 183 b 467 183 501 170 480 170 l 357 293 b 357 340 344 306 344 327 l 539 521 l 357 701 b 357 748 344 714 344 735 l 467 858 b 514 858 480 871 501 871 l 694 676 l 875 858 b 922 858 888 871 909 871 l 1032 748 b 1032 701 1045 735 1045 714 l 850 521 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 1000 228 l 1078 266 b 1197 101 1108 204 1148 149 b 1345 521 1290 216 1345 362 b 1198 940 1345 680 1290 825 b 1080 779 1150 893 1110 840 l 1002 817 b 1140 1004 1037 888 1083 950 b 673 1194 1019 1121 854 1194 b 206 1005 491 1194 327 1121 b 343 818 262 951 309 888 l 266 780 b 148 941 236 840 196 894 b 0 521 55 825 0 680 b 148 101 0 362 55 216 b 268 267 197 149 238 204 l 346 229 b 206 37 311 157 264 93 b 673 -152 327 -80 491 -152 b 1139 37 854 -152 1018 -80 b 1000 228 1082 92 1035 156 z m 380 315 l 298 342 b 296 705 338 467 332 596 l 378 732 b 380 315 419 608 427 461 z m 967 731 l 1050 704 b 1048 340 1014 595 1007 467 l 965 314 b 967 731 918 460 926 608 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 576 1187 b 244 1041 457 1170 342 1122 l 454 831 b 576 1187 530 929 572 1052 z m 673 613 l 1101 1041 b 706 1194 986 1136 847 1187 b 547 739 703 1020 648 862 z m 152 949 b 6 618 72 852 23 736 b 362 739 142 622 265 664 z m 891 395 b 1345 554 1013 496 1172 551 b 1193 949 1338 695 1288 834 l 765 521 z m 673 429 l 244 0 b 639 -152 359 -94 498 -145 b 798 303 643 22 698 180 z m 1193 93 b 1339 424 1274 190 1322 305 b 983 302 1203 420 1080 378 z m 455 646 b 0 487 332 546 173 490 b 152 93 7 346 57 208 l 581 521 z m 770 -146 b 1101 0 888 -129 1003 -80 l 891 210 b 770 -146 816 113 774 -10 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 326 694 b 239 781 278 694 239 733 b 326 868 239 829 278 868 b 412 781 374 868 412 829 b 326 694 412 733 374 694 z m 499 955 b 586 1042 499 1003 538 1042 b 673 955 634 1042 673 1003 b 586 868 673 907 634 868 b 499 955 538 868 499 907 z m 629 564 b 543 651 581 564 543 603 b 629 738 543 699 581 738 b 716 651 677 738 716 699 b 629 564 716 603 677 564 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 201 651 l 174 651 b 130 608 150 651 130 632 l 130 564 b 174 521 130 540 150 521 l 217 521 b 174 174 217 424 197 268 l 521 174 b 478 521 497 268 478 424 l 521 521 b 564 564 545 521 564 540 l 564 608 b 521 651 564 632 545 651 l 494 651 l 586 896 b 588 911 587 900 588 907 b 545 955 588 935 569 955 l 545 955 l 391 955 l 391 1042 l 456 1042 b 477 1063 468 1042 477 1051 l 477 1107 b 456 1128 477 1119 468 1128 l 391 1128 l 391 1194 b 369 1215 391 1206 381 1215 l 326 1215 b 304 1194 314 1215 304 1206 l 304 1128 l 239 1128 b 217 1107 227 1128 217 1119 l 217 1063 b 239 1042 217 1051 227 1042 l 304 1042 l 304 955 l 149 955 l 149 955 b 106 911 125 955 106 935 b 109 896 106 907 107 900 z m 670 -31 l 608 0 l 608 43 b 564 87 608 67 588 87 l 130 87 b 87 43 106 87 87 67 l 87 0 l 24 -31 b 0 -70 11 -38 0 -55 l 0 -130 b 43 -174 0 -154 19 -174 l 651 -174 b 694 -130 675 -174 694 -154 l 694 -70 l 694 -70 b 670 -31 694 -55 684 -38 z m 922 397 l 913 174 l 1257 174 l 1248 397 l 1315 453 b 1345 519 1332 467 1345 497 l 1345 519 l 1345 678 b 1329 694 1345 687 1338 694 l 1258 694 b 1241 678 1249 694 1241 687 l 1241 608 l 1174 608 l 1174 678 b 1158 694 1174 687 1167 694 l 1012 694 b 996 678 1003 694 996 687 l 996 608 l 929 608 l 929 678 b 913 694 929 687 922 694 l 841 694 b 825 678 832 694 825 687 l 825 519 b 855 453 825 497 838 467 z m 1042 391 b 1085 434 1042 415 1061 434 b 1128 391 1109 434 1128 415 l 1128 304 l 1042 304 z m 1365 -31 l 1302 0 l 1302 43 b 1259 87 1302 67 1283 87 l 911 87 b 868 43 887 87 868 67 l 868 0 l 805 -31 b 781 -70 792 -38 781 -55 l 781 -130 b 825 -174 781 -154 801 -174 l 1345 -174 b 1389 -130 1369 -174 1389 -154 l 1389 -70 l 1389 -70 b 1365 -31 1389 -55 1378 -38 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 22 434 b 174 205 22 294 82 234 l 174 87 l 694 87 l 694 205 b 846 434 786 234 846 294 b 774 713 846 517 817 616 l 502 440 b 487 434 498 437 491 434 b 471 440 482 434 475 437 l 440 471 b 434 486 437 475 434 481 b 440 502 434 491 437 498 l 733 795 b 541 1046 677 897 607 990 b 608 1128 579 1055 608 1088 b 521 1215 608 1176 569 1215 l 347 1215 b 260 1128 299 1215 260 1176 b 327 1046 260 1088 289 1055 b 22 434 184 926 22 633 z m 825 0 l 43 0 b 0 -43 19 0 0 -19 l 0 -130 b 43 -174 0 -154 19 -174 l 825 -174 b 868 -130 849 -174 868 -154 l 868 -43 b 825 0 868 -19 849 0 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1215 l 521 1215 l 521 1041 l 694 1041 z m 0 1041 l 0 868 l 174 868 l 174 1041 z m 347 1215 l 174 1215 l 174 1041 l 347 1041 z m 521 521 l 521 347 l 694 347 l 694 521 z m 0 694 l 0 521 l 174 521 l 174 694 z m 1041 1215 l 868 1215 l 868 1041 l 1041 1041 z m 1388 1215 l 1215 1215 l 1215 1041 l 1388 1041 z m 347 521 l 174 521 l 174 347 l 347 347 z m 1388 0 l 1388 174 l 1215 174 l 1215 0 z m 1388 347 l 1388 521 l 1215 521 l 1215 347 z m 1041 -174 l 1215 -174 l 1215 0 l 1041 0 z m 1388 694 l 1388 868 l 1215 868 l 1215 694 z m 347 -174 l 521 -174 l 521 0 l 347 0 z m 0 -174 l 174 -174 l 174 0 l 0 0 z m 694 -174 l 868 -174 l 868 0 l 694 0 z m 0 347 l 0 173 l 174 173 l 174 347 z m 868 868 l 868 1041 l 694 1041 l 694 868 z m 694 520 l 868 520 l 868 694 l 694 694 z m 521 173 l 521 0 l 694 0 l 694 173 z m 868 347 l 1041 347 l 1041 520 l 868 520 z m 868 694 l 1041 694 l 1041 868 l 868 868 z m 868 173 l 868 0 l 1041 0 l 1041 173 z m 174 173 l 174 0 l 347 0 l 347 173 z m 521 868 l 521 1041 l 347 1041 l 347 868 z m 1041 173 l 1215 173 l 1215 347 l 1041 347 z m 694 694 l 694 868 l 521 868 l 521 694 z m 1041 1041 l 1041 868 l 1215 868 l 1215 1041 z m 694 347 l 694 173 l 868 173 l 868 347 z m 521 694 l 347 694 l 347 521 l 521 521 z m 1041 521 l 1215 521 l 1215 694 l 1041 694 z m 347 868 l 174 868 l 174 694 l 347 694 z m 347 347 l 347 173 l 521 173 l 521 347 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1085 0 l 130 0 b 87 -43 106 0 87 -19 l 87 -130 b 130 -174 87 -154 106 -174 l 1085 -174 b 1128 -130 1109 -174 1128 -154 l 1128 -43 b 1085 0 1128 -19 1109 0 z m 1128 781 l 694 781 l 694 911 l 803 911 b 825 933 815 911 825 921 l 825 1063 b 803 1085 825 1075 815 1085 l 694 1085 l 694 1194 b 673 1215 694 1206 685 1215 l 543 1215 b 521 1194 531 1215 521 1206 l 521 1085 l 412 1085 b 391 1063 400 1085 391 1075 l 391 933 b 412 911 391 921 400 911 l 521 911 l 521 781 l 87 781 b 0 694 39 781 0 742 b 4 669 0 687 2 676 l 202 87 l 1013 87 l 1211 669 b 1215 694 1213 676 1215 687 b 1128 781 1215 742 1176 781 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 52 476 l 162 427 b 197 420 171 423 187 420 b 229 426 206 420 221 422 l 264 440 b 315 495 285 448 308 473 l 340 579 b 374 621 344 595 359 614 l 434 651 l 434 515 l 434 514 b 362 398 434 470 402 418 l 207 320 b 87 126 141 287 87 200 l 87 126 l 87 87 l 954 87 l 954 608 b 434 1128 954 895 721 1128 l 33 1128 b 0 1096 15 1128 0 1114 l 0 1096 b 5 1075 0 1090 2 1081 l 43 998 l 19 974 b 0 928 9 963 0 943 l 0 928 l 0 556 l 0 555 b 52 476 0 524 23 489 z m 141 868 b 195 814 171 868 195 844 b 141 760 195 784 171 760 b 87 814 111 760 87 784 b 141 868 87 844 111 868 z m 998 0 l 43 0 b 0 -43 19 0 0 -19 l 0 -130 b 43 -174 0 -154 19 -174 l 998 -174 b 1042 -130 1022 -174 1042 -154 l 1042 -43 b 998 0 1042 -19 1022 0 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 285 608 l 217 608 b 174 564 193 608 174 588 l 174 477 b 217 434 174 453 193 434 l 260 434 l 260 419 b 195 87 260 300 249 184 l 673 87 b 608 419 619 184 608 300 l 608 434 l 651 434 b 694 477 675 434 694 453 l 694 564 b 651 608 694 588 675 608 l 583 608 b 716 846 663 657 716 745 b 434 1128 716 1002 590 1128 b 152 846 278 1128 152 1002 b 285 608 152 745 205 657 z m 825 0 l 43 0 b 0 -43 19 0 0 -19 l 0 -130 b 43 -174 0 -154 19 -174 l 825 -174 b 868 -130 849 -174 868 -154 l 868 -43 b 825 0 868 -19 849 0 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 911 b 846 1063 778 911 846 980 b 694 1215 846 1147 778 1215 b 543 1063 611 1215 543 1147 b 694 911 543 980 611 911 z m 1172 0 l 217 0 b 174 -43 193 0 174 -19 l 174 -130 b 217 -174 174 -154 193 -174 l 1172 -174 b 1215 -130 1196 -174 1215 -154 l 1215 -43 b 1172 0 1215 -19 1196 0 z m 1370 716 l 1292 759 b 1232 746 1272 772 1246 766 b 1128 694 1210 718 1164 694 b 1104 697 1121 694 1110 695 b 998 832 1041 708 998 768 b 962 868 998 852 982 868 l 857 868 b 822 841 840 868 825 857 b 694 738 810 784 753 738 b 567 841 636 738 579 784 b 532 868 564 857 549 868 l 427 868 l 427 868 b 391 832 407 868 391 852 b 269 695 391 762 339 699 b 260 694 267 695 263 694 b 157 747 225 694 178 718 b 97 759 142 766 117 772 l 19 716 b 0 679 9 708 0 692 b 5 660 0 674 2 665 l 283 87 l 1106 87 l 1384 660 b 1389 680 1387 665 1389 674 b 1370 716 1389 692 1380 709 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 998 1128 l 846 1128 b 803 1085 822 1128 803 1109 l 803 955 l 673 955 l 673 1085 b 629 1128 673 1109 653 1128 l 412 1128 b 369 1085 388 1128 369 1109 l 369 955 l 239 955 l 239 1085 b 196 1128 239 1109 220 1128 l 43 1128 b 0 1085 19 1128 0 1109 l 0 608 l 174 521 b 138 87 174 390 169 263 l 904 87 b 868 521 872 263 868 391 l 1042 608 l 1042 1085 b 998 1128 1042 1109 1022 1128 z m 608 347 l 434 347 l 434 521 b 521 608 434 569 473 608 b 608 521 569 608 608 569 z m 998 0 l 43 0 b 0 -43 19 0 0 -19 l 0 -130 b 43 -174 0 -154 19 -174 l 998 -174 b 1042 -130 1022 -174 1042 -154 l 1042 -43 b 998 0 1042 -19 1022 0 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 282 955 l 152 955 b 87 890 116 955 87 926 l 87 608 l 22 608 b 0 586 10 608 0 598 l 0 456 b 22 434 0 444 10 434 l 87 434 l 87 152 b 152 87 87 116 116 87 l 282 87 b 347 152 318 87 347 116 l 347 890 b 282 955 347 926 318 955 z m 1714 608 l 1649 608 l 1649 890 b 1584 955 1649 926 1620 955 l 1454 955 b 1389 890 1418 955 1389 926 l 1389 152 b 1454 87 1389 116 1418 87 l 1584 87 b 1649 152 1620 87 1649 116 l 1649 434 l 1714 434 b 1736 456 1726 434 1736 444 l 1736 586 b 1714 608 1736 598 1726 608 z m 1237 1128 l 1107 1128 b 1042 1063 1071 1128 1042 1099 l 1042 608 l 694 608 l 694 1063 b 629 1128 694 1099 665 1128 l 499 1128 b 434 1063 463 1128 434 1099 l 434 -22 b 499 -87 434 -58 463 -87 l 629 -87 b 694 -22 665 -87 694 -58 l 694 434 l 1042 434 l 1042 -22 b 1107 -87 1042 -58 1071 -87 l 1237 -87 b 1302 -22 1273 -87 1302 -58 l 1302 1063 b 1237 1128 1302 1099 1273 1128 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 1306 1052 b 1205 1153 1293 1101 1254 1140 b 921 1194 1140 1171 1039 1191 l 1345 768 b 1306 1052 1343 886 1323 987 z m 39 -10 b 141 -111 52 -59 91 -98 b 425 -152 205 -129 307 -149 l 0 273 b 39 -10 2 155 22 55 z m 11 447 l 598 -141 b 1334 595 957 -88 1273 181 l 748 1183 b 11 447 388 1130 73 861 z m 872 782 b 903 782 881 790 894 790 l 933 751 b 933 720 942 743 942 729 l 857 644 l 933 567 b 933 536 942 559 942 545 l 903 506 b 872 506 894 497 881 497 l 795 582 l 734 521 l 811 444 b 811 413 819 436 819 422 l 780 383 b 750 383 772 374 758 374 l 673 460 l 611 398 l 688 321 b 688 291 697 313 697 299 l 658 260 b 627 260 649 252 635 252 l 550 337 l 473 260 b 443 260 465 252 451 252 l 412 291 b 412 321 404 299 404 313 l 489 398 l 412 475 b 412 505 404 483 404 497 l 443 536 b 473 536 451 544 465 544 l 550 459 l 611 521 l 535 597 b 535 628 526 606 526 620 l 565 659 b 596 659 574 667 588 667 l 673 582 l 734 643 l 657 720 b 657 751 649 728 649 742 l 688 781 b 719 781 696 790 710 790 l 795 704 z "
	},
	"": {
		ha: 1128,
		x_min: 0,
		x_max: 0,
		o: "m 260 87 b 347 0 260 39 299 0 l 391 0 b 477 -87 439 0 477 -39 l 477 -141 b 510 -174 477 -159 492 -174 l 618 -174 b 651 -141 636 -174 651 -159 l 651 -87 b 738 0 651 -39 690 0 l 781 0 b 868 87 829 0 868 39 z m 1128 651 b 564 1215 1128 963 876 1215 b 0 651 253 1215 0 963 b 264 174 0 450 106 273 l 864 174 b 1128 651 1023 273 1128 450 z m 640 532 b 556 592 601 532 569 557 b 670 706 627 566 696 633 b 730 622 704 693 730 661 b 640 532 730 572 690 532 z m 773 405 b 683 315 773 355 733 315 b 600 375 644 315 612 340 b 713 489 671 349 739 416 b 773 405 748 476 773 444 z m 947 578 b 857 489 947 529 907 489 b 773 549 818 489 786 514 b 887 662 844 523 913 589 b 947 578 921 650 947 617 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 0 781 b 694 521 0 637 311 521 b 1389 781 1078 521 1389 637 b 694 1042 1389 925 1078 1042 b 0 781 311 1042 0 925 z m 0 558 l 0 260 b 694 0 0 117 311 0 b 1389 260 1078 0 1389 117 l 1389 558 b 0 558 1081 335 308 335 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 696 627 b 253 516 696 627 415 646 b 105 252 193 469 144 365 l 269 312 b 286 286 284 317 297 299 l 30 -18 b 0 -169 10 -106 0 -169 b 723 -76 0 -169 561 -206 b 931 331 886 53 931 331 z m 1341 261 b 1125 44 1222 261 1125 164 b 1341 -173 1125 -75 1222 -173 b 1558 44 1461 -173 1558 -75 b 1341 261 1558 164 1461 261 z m 1727 1131 l 1673 1199 b 1612 1206 1658 1218 1631 1221 l 981 723 l 889 840 b 838 834 875 857 848 854 l 770 686 l 1005 390 l 1165 423 b 1182 471 1186 428 1195 454 l 1089 588 l 1720 1070 b 1727 1131 1739 1085 1742 1112 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1389 -174 l 1389 1215 l 0 1215 l 0 -174 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1346 411 b 1218 1044 1431 622 1389 873 b 394 1044 990 1272 621 1272 l 242 892 l 815 318 b 1346 411 940 487 1172 528 z m 755 195 l 157 792 b 204 401 83 669 98 508 l 298 308 l 18 65 b 15 -13 -5 45 -7 9 l 160 -158 b 238 -155 182 -180 218 -179 l 480 125 l 577 28 b 768 -62 631 -26 698 -55 b 738 87 749 -16 738 34 b 755 195 738 125 745 160 z m 1128 347 b 868 87 985 347 868 231 b 1128 -174 868 -57 985 -174 b 1389 87 1272 -174 1389 -57 b 1128 347 1389 231 1272 347 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 628 555 b 566 842 626 637 598 766 b 77 253 320 727 139 509 b 231 33 107 184 176 85 b 628 555 285 220 462 453 z m 528 919 b 372 1111 493 979 423 1065 b 33 400 125 976 -21 696 b 528 919 123 625 300 812 z m 1036 472 b 772 1189 1060 742 962 1007 b 504 1166 732 1193 632 1204 b 757 562 638 1026 752 755 b 1036 472 830 522 955 481 z m 698 446 b 481 249 628 403 530 315 b 1233 119 703 93 981 45 b 1348 363 1279 180 1330 289 b 1128 335 1288 348 1190 335 b 698 446 980 335 833 373 z m 432 175 b 344 -53 390 105 362 28 b 695 -152 434 -108 591 -152 b 1128 7 831 -152 1025 -81 b 432 175 822 -37 577 74 z m 921 1153 b 1123 466 1071 963 1144 716 b 1125 466 1123 466 1124 466 b 1366 504 1193 466 1301 483 b 1367 520 1366 509 1367 515 b 921 1153 1367 813 1181 1060 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1128 911 b 1042 825 1081 911 1042 872 l 1042 629 b 1020 608 1042 617 1032 608 l 977 608 b 955 629 965 608 955 617 l 955 1042 b 868 1128 955 1089 916 1128 b 781 1042 820 1128 781 1089 l 781 629 b 760 608 781 617 771 608 l 716 608 b 694 629 704 608 694 617 l 694 1128 b 608 1215 694 1176 655 1215 b 521 1128 560 1215 521 1176 l 521 629 b 499 608 521 617 511 608 l 456 608 b 434 629 444 608 434 617 l 434 1042 b 347 1128 434 1089 395 1128 b 260 1042 299 1128 260 1089 l 260 388 l 196 476 b 45 500 161 525 93 535 b 21 348 -4 465 -14 397 l 362 -120 b 467 -174 386 -154 425 -174 l 1003 -174 b 1130 -74 1063 -174 1116 -132 l 1201 232 b 1215 347 1210 269 1215 309 l 1215 825 b 1128 911 1215 872 1176 911 z m 477 87 b 434 130 454 87 434 106 b 477 174 434 154 454 174 b 521 130 501 174 521 154 b 477 87 521 106 501 87 z m 477 347 b 434 391 454 347 434 367 b 477 434 434 414 454 434 b 521 391 501 434 521 414 b 477 347 521 367 501 347 z m 651 0 b 608 43 627 0 608 20 b 651 87 608 67 627 87 b 694 43 675 87 694 67 b 651 0 694 20 675 0 z m 651 260 b 608 304 627 260 608 280 b 651 347 608 328 627 347 b 694 304 675 347 694 328 b 651 260 694 280 675 260 z m 825 174 b 781 217 801 174 781 193 b 825 260 781 241 801 260 b 868 217 849 260 868 241 b 825 174 868 193 849 174 z m 911 0 b 868 43 888 0 868 20 b 911 87 868 67 888 87 b 955 43 935 87 955 67 b 911 0 955 20 935 0 z m 998 347 b 955 391 974 347 955 367 b 998 434 955 414 974 434 b 1042 391 1022 434 1042 414 b 998 347 1042 367 1022 347 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 0 781 l 0 260 b 174 87 0 165 78 87 l 434 87 l 434 955 l 174 955 b 0 781 78 955 0 877 z m 1563 955 l 1302 955 l 1302 87 l 1563 87 b 1736 260 1658 87 1736 165 l 1736 781 b 1563 955 1736 877 1658 955 z m 521 87 l 1215 87 l 1215 955 l 521 955 z m 998 716 b 1063 651 1034 716 1063 687 b 998 586 1063 615 1034 586 b 933 651 962 586 933 615 b 998 716 933 687 962 716 z m 998 456 b 1063 391 1034 456 1063 427 b 998 326 1063 355 1034 326 b 933 391 962 326 933 355 b 998 456 933 427 962 456 z m 738 716 b 803 651 774 716 803 687 b 738 586 803 615 774 586 b 673 651 702 586 673 615 b 738 716 673 687 702 716 z m 738 456 b 803 391 774 456 803 427 b 738 326 803 355 774 326 b 673 391 702 326 673 355 b 738 456 673 427 702 456 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1382 715 l 1245 1126 b 1121 1215 1227 1179 1178 1215 l 738 1215 l 738 694 l 1385 694 b 1382 715 1384 701 1384 708 z m 651 1215 l 267 1215 b 144 1126 211 1215 162 1179 l 7 715 b 4 694 5 708 5 701 l 651 694 z m 0 608 l 0 -43 b 130 -174 0 -115 58 -174 l 1259 -174 b 1389 -43 1331 -174 1389 -115 l 1389 608 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1519 434 l 1302 434 l 1302 174 l 1215 231 l 1128 174 l 1128 434 l 911 434 b 868 391 888 434 868 414 l 868 -130 b 911 -174 868 -154 888 -174 l 1519 -174 b 1563 -130 1543 -174 1563 -154 l 1563 391 b 1519 434 1563 414 1543 434 z m 477 608 l 1085 608 b 1128 651 1109 608 1128 627 l 1128 1172 b 1085 1215 1128 1196 1109 1215 l 868 1215 l 868 955 l 781 1013 l 694 955 l 694 1215 l 477 1215 b 434 1172 454 1215 434 1196 l 434 651 b 477 608 434 627 454 608 z m 651 434 l 434 434 l 434 174 l 347 231 l 260 174 l 260 434 l 43 434 b 0 391 20 434 0 414 l 0 -130 b 43 -174 0 -154 20 -174 l 651 -174 b 694 -130 675 -174 694 -154 l 694 391 b 651 434 694 414 675 434 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1259 868 l 1042 868 l 1042 998 b 911 1128 1042 1070 983 1128 l 477 1128 b 347 998 406 1128 347 1070 l 347 868 l 130 868 b 0 738 58 868 0 810 l 0 -43 b 130 -174 0 -115 58 -174 l 1259 -174 b 1389 -43 1331 -174 1389 -115 l 1389 738 b 1259 868 1389 810 1331 868 z m 521 955 l 868 955 l 868 868 l 521 868 z m 955 282 b 933 260 955 270 945 260 l 781 260 l 781 109 b 760 87 781 97 771 87 l 629 87 b 608 109 617 87 608 97 l 608 260 l 456 260 b 434 282 444 260 434 270 l 434 412 b 456 434 434 424 444 434 l 608 434 l 608 586 b 629 608 608 598 617 608 l 760 608 b 781 586 771 608 781 598 l 781 434 l 933 434 b 955 412 945 434 955 424 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 521 1215 b 0 400 216 940 0 616 b 521 -174 0 62 214 -174 b 1042 400 827 -174 1042 62 b 521 1215 1042 617 824 942 z m 521 0 b 260 257 368 0 260 106 b 521 694 260 294 273 424 b 781 257 769 424 781 294 b 521 0 781 106 674 0 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1506 401 l 1151 909 b 896 1042 1090 996 994 1042 b 719 986 835 1042 773 1023 b 606 840 665 948 627 897 b 304 1128 598 1000 467 1128 b 0 825 136 1128 0 993 l 0 217 b 304 -87 0 49 136 -87 b 608 217 472 -87 608 49 l 608 621 b 642 554 617 598 627 575 l 998 46 b 1253 -87 1058 -41 1155 -87 b 1430 -31 1314 -87 1376 -69 b 1506 401 1571 68 1605 261 z m 434 521 l 174 521 l 174 825 b 304 955 174 897 232 955 b 434 825 376 955 434 897 z m 962 399 l 785 653 b 762 755 764 683 755 719 b 818 843 768 791 788 822 b 896 868 841 860 868 868 b 1008 810 941 868 983 846 l 1186 556 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 911 1042 l 694 1042 b 521 1215 694 1137 617 1215 b 347 1042 425 1215 347 1137 l 130 1042 b 0 911 58 1042 0 983 l 0 -43 b 130 -174 0 -115 58 -174 l 911 -174 b 1042 -43 983 -174 1042 -115 l 1042 911 b 911 1042 1042 983 983 1042 z m 521 1107 b 586 1042 557 1107 586 1078 b 521 977 586 1006 557 977 b 456 1042 485 977 456 1006 b 521 1107 456 1078 485 1107 z m 850 478 l 462 93 b 416 94 449 81 428 81 l 192 320 b 192 366 179 332 179 353 l 269 442 b 315 442 282 455 302 455 l 440 316 l 727 601 b 773 601 740 614 761 614 l 850 524 b 850 478 863 511 862 491 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 911 1042 l 694 1042 b 521 1215 694 1137 617 1215 b 347 1042 425 1215 347 1137 l 130 1042 b 0 911 58 1042 0 983 l 0 -43 b 130 -174 0 -115 58 -174 l 911 -174 b 1042 -43 983 -174 1042 -115 l 1042 911 b 911 1042 1042 983 983 1042 z m 260 65 b 195 130 224 65 195 94 b 260 195 195 166 224 195 b 326 130 296 195 326 166 b 260 65 326 94 296 65 z m 260 326 b 195 391 224 326 195 355 b 260 456 195 427 224 456 b 326 391 296 456 326 427 b 260 326 326 355 296 326 z m 260 586 b 195 651 224 586 195 615 b 260 716 195 687 224 716 b 326 651 296 716 326 687 b 260 586 326 615 296 586 z m 521 1107 b 586 1042 557 1107 586 1078 b 521 977 586 1006 557 977 b 456 1042 485 977 456 1006 b 521 1107 456 1078 485 1107 z m 868 109 b 846 87 868 97 858 87 l 456 87 b 434 109 444 87 434 97 l 434 152 b 456 174 434 164 444 174 l 846 174 b 868 152 858 174 868 164 z m 868 369 b 846 347 868 357 858 347 l 456 347 b 434 369 444 347 434 357 l 434 412 b 456 434 434 424 444 434 l 846 434 b 868 412 858 434 868 424 z m 868 629 b 846 608 868 617 858 608 l 456 608 b 434 629 444 608 434 617 l 434 673 b 456 694 434 685 444 694 l 846 694 b 868 673 858 694 868 685 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1345 521 b 1389 564 1369 521 1389 540 b 1345 608 1389 588 1369 608 b 1302 564 1322 608 1302 588 b 1345 521 1302 540 1322 521 z m 868 738 b 1107 977 1000 738 1107 845 b 868 1215 1107 1108 1000 1215 b 629 977 736 1215 629 1108 b 868 738 629 845 736 738 z m 162 228 b 279 201 190 186 242 180 b 564 333 323 227 431 287 l 564 87 l 1172 87 l 1172 333 b 1458 201 1305 287 1414 227 b 1574 228 1494 179 1546 187 l 1622 300 b 1595 422 1646 336 1643 394 b 1456 497 1563 442 1514 468 b 1219 592 1380 372 1187 448 b 868 651 1110 626 990 651 b 434 564 713 651 562 612 b 219 466 433 455 305 392 b 141 423 189 450 161 435 b 114 300 94 394 90 336 z m 998 282 b 1063 217 1034 282 1063 253 b 998 152 1063 181 1034 152 b 933 217 962 152 933 181 b 998 282 933 253 962 282 z m 738 543 b 803 477 774 543 803 514 b 738 412 803 441 774 412 b 673 477 702 412 673 441 b 738 543 673 514 702 543 z m 304 521 b 347 564 328 521 347 540 b 304 608 347 588 328 608 b 260 564 280 608 260 588 b 304 521 260 540 280 521 z m 1693 0 l 43 0 b 0 -43 20 0 0 -20 l 0 -130 b 43 -174 0 -154 20 -174 l 1693 -174 b 1736 -130 1717 -174 1736 -154 l 1736 -43 b 1693 0 1736 -20 1717 0 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 0 -125 b 43 -173 -3 -151 17 -173 l 131 -174 b 174 -136 153 -174 171 -158 b 184 -87 176 -123 179 -106 l 1031 -87 b 1040 -136 1035 -106 1039 -123 b 1084 -174 1043 -158 1062 -174 l 1171 -173 b 1214 -125 1197 -173 1218 -151 b 892 438 1202 -22 1145 225 b 733 342 844 405 791 373 b 779 304 750 330 764 317 l 433 304 b 647 443 491 353 561 400 b 1215 1167 1114 675 1199 1037 b 1172 1215 1218 1192 1198 1215 l 1084 1215 b 1041 1178 1062 1215 1044 1199 b 1031 1128 1039 1164 1036 1147 l 184 1128 b 175 1178 180 1147 177 1164 b 131 1215 172 1199 153 1216 l 44 1215 b 0 1167 18 1215 -3 1192 b 434 521 14 1050 85 749 b 0 -125 85 292 14 -9 z m 608 620 b 433 738 540 657 482 697 l 782 738 b 608 620 734 697 676 656 z m 963 955 b 906 868 948 927 929 898 l 309 868 b 252 955 287 898 268 927 z m 252 87 b 309 174 268 115 287 144 l 904 174 b 962 87 927 144 946 115 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 798 462 b 932 395 847 448 892 426 l 1370 541 b 1398 596 1393 549 1405 573 l 1234 1087 b 1179 1115 1226 1110 1202 1122 l 1013 1059 l 1103 790 l 939 735 l 849 1005 l 682 949 b 654 894 659 942 647 917 z m 1560 330 l 1533 412 b 1478 440 1525 435 1500 447 l 899 247 b 699 347 852 306 780 345 l 430 1156 b 347 1215 418 1191 385 1215 l 43 1215 b 0 1172 20 1215 0 1196 l 0 1085 b 43 1042 0 1061 20 1042 l 285 1042 l 535 291 b 437 46 464 236 422 146 b 647 -169 454 -61 540 -151 b 954 82 810 -198 951 -75 l 1533 275 b 1560 330 1555 283 1568 307 z m 694 -43 b 564 87 623 -43 564 15 b 694 217 564 159 623 217 b 825 87 766 217 825 159 b 694 -43 825 15 766 -43 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 564 347 l 1606 347 b 1649 391 1630 347 1649 367 l 1649 1085 b 1606 1128 1649 1109 1630 1128 l 1215 1128 l 1215 781 l 1085 868 l 955 781 l 955 1128 l 564 1128 b 521 1085 540 1128 521 1109 l 521 391 b 564 347 521 367 540 347 z m 1693 174 l 347 174 l 347 1172 b 304 1215 347 1196 328 1215 l 43 1215 b 0 1172 20 1215 0 1196 l 0 1085 b 43 1042 0 1061 20 1042 l 174 1042 l 174 43 b 217 0 174 20 193 0 l 442 0 b 434 -43 437 -14 434 -28 b 564 -174 434 -115 492 -174 b 694 -43 636 -174 694 -115 b 687 0 694 -28 691 -14 l 1223 0 b 1216 -43 1219 -14 1216 -28 b 1346 -174 1216 -115 1274 -174 b 1476 -43 1418 -174 1476 -115 b 1468 0 1476 -28 1473 -14 l 1693 0 b 1736 43 1717 0 1736 20 l 1736 130 b 1693 174 1736 154 1717 174 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 1023 930 l 757 1196 b 711 1215 745 1208 728 1215 l 694 1215 l 694 868 l 1042 868 l 1042 885 b 1023 930 1042 902 1035 918 z m 608 846 l 608 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 -109 b 65 -174 0 -145 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -145 l 1042 781 l 673 781 b 608 846 637 781 608 811 z m 781 412 l 781 282 b 760 260 781 270 771 260 l 608 260 l 608 109 b 586 87 608 97 598 87 l 456 87 b 434 109 444 87 434 97 l 434 260 l 282 260 b 260 282 270 260 260 270 l 260 412 b 282 434 260 424 270 434 l 434 434 l 434 586 b 456 608 434 598 444 608 l 586 608 b 608 586 598 608 608 598 l 608 434 l 760 434 b 781 412 771 434 781 424 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 781 846 l 781 1215 l 239 1215 b 174 1150 203 1215 174 1186 l 174 521 l 22 521 b 0 499 10 521 0 511 l 0 456 b 22 434 0 444 10 434 l 404 434 b 423 422 412 434 420 429 l 477 314 l 632 623 b 670 623 639 639 662 639 l 764 434 l 955 434 b 998 391 979 434 998 414 b 955 347 998 367 979 347 l 711 347 l 651 467 l 497 159 b 458 159 489 143 466 143 l 364 347 l 174 347 l 174 -109 b 239 -174 174 -145 203 -174 l 1150 -174 b 1215 -109 1186 -174 1215 -145 l 1215 781 l 846 781 b 781 846 811 781 781 811 z m 1196 930 l 931 1196 b 885 1215 919 1208 902 1215 l 868 1215 l 868 868 l 1215 868 l 1215 885 b 1196 930 1215 902 1208 918 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 0 998 l 0 43 b 130 -87 0 -28 58 -87 l 260 -87 l 260 1128 l 130 1128 b 0 998 58 1128 0 1070 z m 347 -87 l 1215 -87 l 1215 1128 l 347 1128 z m 521 586 b 543 608 521 598 531 608 l 694 608 l 694 760 b 716 781 694 771 704 781 l 846 781 b 868 760 858 781 868 771 l 868 608 l 1020 608 b 1042 586 1032 608 1042 598 l 1042 456 b 1020 434 1042 444 1032 434 l 868 434 l 868 282 b 846 260 868 270 858 260 l 716 260 b 694 282 704 260 694 270 l 694 434 l 543 434 b 521 456 531 434 521 444 z m 1432 1128 l 1302 1128 l 1302 -87 l 1432 -87 b 1563 43 1504 -87 1563 -28 l 1563 998 b 1432 1128 1563 1070 1504 1128 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1476 955 l 1128 955 l 1128 1128 b 1042 1215 1128 1176 1090 1215 l 521 1215 b 434 1128 473 1215 434 1176 l 434 955 l 87 955 b 0 868 39 955 0 916 l 0 -130 b 43 -174 0 -154 20 -174 l 1519 -174 b 1563 -130 1543 -174 1563 -154 l 1563 868 b 1476 955 1563 916 1524 955 z m 434 33 b 401 0 434 15 419 0 l 293 0 b 260 33 275 0 260 15 l 260 141 b 293 174 260 159 275 174 l 401 174 b 434 141 419 174 434 159 z m 434 380 b 401 347 434 362 419 347 l 293 347 b 260 380 275 347 260 362 l 260 488 b 293 521 260 506 275 521 l 401 521 b 434 488 419 521 434 506 z m 868 33 b 836 0 868 15 853 0 l 727 0 b 694 33 709 0 694 15 l 694 141 b 727 174 694 159 709 174 l 836 174 b 868 141 853 174 868 159 z m 868 380 b 836 347 868 362 853 347 l 727 347 b 694 380 709 347 694 362 l 694 488 b 727 521 694 506 709 521 l 836 521 b 868 488 853 521 868 506 z m 911 841 b 895 825 911 832 904 825 l 825 825 l 825 754 b 808 738 825 745 817 738 l 754 738 b 738 754 745 738 738 745 l 738 825 l 667 825 b 651 841 658 825 651 832 l 651 895 b 667 911 651 904 658 911 l 738 911 l 738 982 b 754 998 738 991 745 998 l 808 998 b 825 982 817 998 825 991 l 825 911 l 895 911 b 911 895 904 911 911 904 z m 1302 33 b 1270 0 1302 15 1287 0 l 1161 0 b 1128 33 1143 0 1128 15 l 1128 141 b 1161 174 1128 159 1143 174 l 1270 174 b 1302 141 1287 174 1302 159 z m 1302 380 b 1270 347 1302 362 1287 347 l 1161 347 b 1128 380 1143 347 1128 362 l 1128 488 b 1161 521 1128 506 1143 521 l 1270 521 b 1302 488 1287 521 1302 506 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1215 b 0 521 311 1215 0 904 b 694 -174 0 137 311 -174 b 1389 521 1078 -174 1389 137 b 694 1215 1389 904 1078 1215 z m 998 195 b 977 174 998 183 988 174 l 846 174 b 825 195 834 174 825 183 l 825 434 l 564 434 l 564 195 b 543 174 564 183 554 174 l 412 174 b 391 195 400 174 391 183 l 391 846 b 412 868 391 858 400 868 l 543 868 b 564 846 554 868 564 858 l 564 608 l 825 608 l 825 846 b 846 868 825 858 834 868 l 977 868 b 998 846 988 868 998 858 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1432 1042 l 1042 1042 l 1042 781 l 521 781 l 521 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 -43 b 130 -174 0 -115 58 -174 l 1432 -174 b 1563 -43 1504 -174 1563 -115 l 1563 911 b 1432 1042 1563 983 1504 1042 z m 781 608 b 955 434 877 608 955 530 b 781 260 955 338 877 260 b 608 434 685 260 608 338 b 781 608 608 530 685 608 z m 1034 0 l 528 0 b 486 54 500 0 477 27 b 651 174 508 123 574 174 l 673 174 b 781 152 707 160 743 152 b 889 174 819 152 856 160 l 911 174 b 1077 54 988 174 1054 123 b 1034 0 1085 27 1063 0 z m 955 1128 b 868 1215 955 1176 916 1215 l 694 1215 b 608 1128 646 1215 608 1176 l 608 868 l 955 868 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 911 1042 l 694 1042 b 521 1215 694 1137 617 1215 b 347 1042 425 1215 347 1137 l 130 1042 b 0 911 58 1042 0 983 l 0 -43 b 130 -174 0 -115 58 -174 l 911 -174 b 1042 -43 983 -174 1042 -115 l 1042 911 b 911 1042 1042 983 983 1042 z m 521 1107 b 586 1042 557 1107 586 1078 b 521 977 586 1006 557 977 b 456 1042 485 977 456 1006 b 521 1107 456 1078 485 1107 z m 781 282 b 760 260 781 270 771 260 l 608 260 l 608 109 b 586 87 608 97 598 87 l 456 87 b 434 109 444 87 434 97 l 434 260 l 282 260 b 260 282 270 260 260 270 l 260 412 b 282 434 260 424 270 434 l 434 434 l 434 586 b 456 608 434 598 444 608 l 586 608 b 608 586 598 608 608 598 l 608 434 l 760 434 b 781 412 771 434 781 424 z m 781 803 b 760 781 781 791 771 781 l 282 781 b 260 803 270 781 260 791 l 260 846 b 282 868 260 858 270 868 l 760 868 b 781 846 771 868 781 858 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 391 521 l 1345 521 b 1389 564 1369 521 1389 540 l 1389 1172 b 1345 1215 1389 1196 1369 1215 l 1042 1215 l 1042 868 l 868 955 l 694 868 l 694 1215 l 391 1215 b 347 1172 367 1215 347 1196 l 347 564 b 391 521 347 540 367 521 z m 1693 174 b 1736 217 1717 174 1736 193 l 1736 304 b 1693 347 1736 328 1717 347 l 43 347 b 0 304 20 347 0 328 l 0 217 b 43 174 0 193 20 174 l 174 174 l 174 0 l 43 0 b 0 -43 20 0 0 -20 l 0 -130 b 43 -174 0 -154 20 -174 l 1693 -174 b 1736 -130 1717 -174 1736 -154 l 1736 -43 b 1693 0 1736 -20 1717 0 l 1563 0 l 1563 174 z m 781 0 l 347 0 l 347 174 l 781 174 z m 1389 0 l 955 0 l 955 174 l 1389 174 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 304 1128 b 0 825 136 1128 0 993 l 0 217 b 304 -87 0 49 136 -87 b 608 217 472 -87 608 49 l 608 825 b 304 1128 608 993 472 1128 z m 434 521 l 174 521 l 174 825 b 304 955 174 897 232 955 b 434 825 376 955 434 897 z m 813 601 b 780 599 803 611 787 610 b 823 39 657 430 670 192 b 1383 -4 975 -113 1213 -127 b 1385 30 1394 4 1394 20 z m 1436 653 b 876 696 1284 806 1046 819 b 874 663 866 688 865 672 l 1446 91 b 1479 93 1456 81 1472 82 b 1436 653 1602 263 1589 500 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 87 694 l 412 694 b 434 673 424 694 434 685 l 434 629 b 412 608 434 617 424 608 l 87 608 l 87 434 l 412 434 b 434 412 424 434 434 424 l 434 369 b 412 347 434 357 424 347 l 87 347 l 87 174 l 412 174 b 434 152 424 174 434 164 l 434 109 b 412 87 434 97 424 87 l 87 87 l 87 -87 b 174 -174 87 -135 126 -174 l 868 -174 b 955 -87 916 -174 955 -135 l 955 868 l 87 868 z m 977 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 1020 b 65 955 0 984 29 955 l 977 955 b 1042 1020 1012 955 1042 984 l 1042 1150 b 977 1215 1042 1186 1012 1215 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 977 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 1020 b 65 955 0 984 29 955 l 977 955 b 1042 1020 1012 955 1042 984 l 1042 1150 b 977 1215 1042 1186 1012 1215 z m 87 -87 b 174 -174 87 -135 126 -174 l 868 -174 b 955 -87 916 -174 955 -135 l 955 868 l 87 868 z m 260 412 b 282 434 260 424 270 434 l 434 434 l 434 586 b 456 608 434 598 444 608 l 586 608 b 608 586 598 608 608 598 l 608 434 l 760 434 b 781 412 771 434 781 424 l 781 282 b 760 260 781 270 771 260 l 608 260 l 608 109 b 586 87 608 97 598 87 l 456 87 b 434 109 444 87 434 97 l 434 260 l 282 260 b 260 282 270 260 260 270 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1432 608 l 738 608 b 694 564 714 608 694 588 l 694 174 l 174 174 l 174 825 b 130 868 174 849 154 868 l 43 868 b 0 825 20 868 0 849 l 0 -130 b 43 -174 0 -154 20 -174 l 130 -174 b 174 -130 154 -174 174 -154 l 174 0 l 1563 0 l 1563 -130 b 1606 -174 1563 -154 1582 -174 l 1693 -174 b 1736 -130 1717 -174 1736 -154 l 1736 304 b 1432 608 1736 472 1600 608 z m 369 955 l 711 955 l 786 805 b 863 805 802 773 847 773 l 998 1075 l 1058 955 l 1389 955 b 1432 998 1413 955 1432 974 b 1389 1042 1432 1022 1413 1042 l 1112 1042 l 1037 1191 b 959 1191 1021 1223 975 1223 l 825 922 l 771 1030 b 751 1042 767 1037 760 1042 l 369 1042 b 347 1020 357 1042 347 1032 l 347 977 b 369 955 347 965 357 955 z m 434 260 b 608 434 530 260 608 338 b 434 608 608 530 530 608 b 260 434 338 608 260 530 b 434 260 260 338 338 260 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1693 260 l 1649 260 l 1649 554 b 1611 646 1649 588 1635 621 l 1340 917 b 1248 955 1316 941 1283 955 l 1128 955 l 1128 1085 b 998 1215 1128 1157 1070 1215 l 304 1215 b 174 1085 232 1215 174 1157 l 174 955 l 22 955 b 0 933 10 955 0 945 l 0 890 b 22 868 0 878 10 868 l 760 868 b 781 846 771 868 781 858 l 781 803 b 760 781 781 791 771 781 l 109 781 b 87 760 97 781 87 771 l 87 716 b 109 694 87 704 97 694 l 673 694 b 694 673 685 694 694 685 l 694 629 b 673 608 694 617 685 608 l 22 608 b 0 586 10 608 0 598 l 0 543 b 22 521 0 531 10 521 l 586 521 b 608 499 598 521 608 511 l 608 456 b 586 434 608 444 598 434 l 174 434 l 174 87 b 434 -174 174 -57 290 -174 b 694 87 578 -174 694 -57 l 1042 87 b 1302 -174 1042 -57 1158 -174 b 1563 87 1446 -174 1563 -57 l 1693 87 b 1736 130 1717 87 1736 106 l 1736 217 b 1693 260 1736 241 1717 260 z m 434 -43 b 304 87 362 -43 304 15 b 434 217 304 159 362 217 b 564 87 506 217 564 159 b 434 -43 564 15 506 -43 z m 1302 -43 b 1172 87 1230 -43 1172 15 b 1302 217 1172 159 1230 217 b 1432 87 1374 217 1432 159 b 1302 -43 1432 15 1374 -43 z m 1519 521 l 1128 521 l 1128 825 l 1248 825 l 1519 554 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1714 260 l 1584 260 b 1563 239 1572 260 1563 251 l 1563 -152 b 1584 -174 1563 -164 1572 -174 l 1714 -174 b 1736 -152 1726 -174 1736 -164 l 1736 239 b 1714 260 1736 251 1726 260 z m 1501 979 b 1476 1025 1485 989 1476 1006 l 1476 1194 b 1454 1215 1476 1206 1466 1215 l 1324 1215 b 1302 1194 1312 1215 1302 1206 l 1302 1025 b 1380 874 1302 965 1330 907 b 1563 538 1494 800 1563 674 l 1563 456 b 1584 434 1563 444 1572 434 l 1714 434 b 1736 456 1726 434 1736 444 l 1736 538 b 1501 979 1736 716 1648 880 z m 1172 260 l 130 260 b 0 130 58 260 0 202 l 0 -43 b 130 -174 0 -115 58 -174 l 1172 -174 b 1215 -130 1196 -174 1215 -154 l 1215 217 b 1172 260 1215 241 1196 260 z m 1085 -43 l 608 -43 l 608 130 l 1085 130 z m 1323 831 b 1215 1025 1258 876 1215 946 l 1215 1194 b 1194 1215 1215 1206 1206 1215 l 1063 1215 b 1042 1194 1051 1215 1042 1206 l 1042 1013 b 1205 724 1042 895 1108 792 b 1302 538 1266 681 1302 612 l 1302 456 b 1324 434 1302 444 1312 434 l 1454 434 b 1476 456 1466 434 1476 444 l 1476 538 b 1323 831 1476 655 1419 764 z m 1454 260 l 1324 260 b 1302 239 1312 260 1302 251 l 1302 -152 b 1324 -174 1302 -164 1312 -174 l 1454 -174 b 1476 -152 1466 -174 1476 -164 l 1476 239 b 1454 260 1476 251 1466 260 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 547 741 l 698 590 b 698 559 706 581 706 567 l 667 528 b 636 528 659 520 645 520 l 485 680 l 362 557 l 514 406 b 514 375 522 397 522 383 l 483 344 b 452 344 475 336 461 336 l 301 496 l 229 424 b 167 247 183 377 160 313 l 186 74 l 6 -106 b 6 -136 -2 -114 -2 -128 l 37 -167 b 68 -167 45 -176 59 -176 l 247 13 l 420 -7 b 597 56 485 -14 550 8 l 1091 549 l 723 917 z m 1383 994 l 1168 1209 b 1137 1209 1159 1217 1146 1217 l 1107 1178 b 1107 1148 1098 1170 1098 1156 l 1183 1071 l 1060 948 l 907 1102 l 861 1148 b 830 1148 852 1156 838 1156 l 738 1056 b 738 1025 730 1047 730 1034 l 784 979 l 1152 610 l 1198 564 b 1229 564 1207 556 1221 556 l 1321 656 b 1321 687 1329 665 1329 679 l 1122 887 l 1245 1009 l 1321 933 b 1352 933 1330 924 1344 924 l 1383 963 b 1383 994 1391 972 1391 986 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 434 694 b 0 329 214 694 34 536 b 23 304 -2 316 9 304 l 846 304 b 868 329 859 304 870 316 b 434 694 834 536 654 694 z m 845 217 l 23 217 b 0 192 9 217 -2 205 b 434 -174 34 -15 214 -174 b 868 192 654 -174 834 -15 b 845 217 870 205 859 217 z m 1610 1089 b 1048 1132 1456 1242 1219 1255 b 1046 1099 1038 1124 1037 1108 l 1619 525 b 1653 528 1629 516 1645 517 b 1610 1089 1776 698 1763 936 z m 985 1037 b 951 1035 975 1047 959 1046 b 994 474 828 865 841 627 b 1556 431 1148 320 1386 307 b 1558 464 1567 438 1567 455 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1293 1160 b 936 1132 1192 1243 1034 1231 l 812 1007 l 949 870 b 949 840 957 862 957 848 l 918 809 b 887 809 910 801 896 801 l 751 946 l 628 823 l 765 686 b 765 656 773 678 773 664 l 734 625 b 703 625 726 617 712 617 l 567 761 l 445 638 l 581 502 b 581 471 589 493 589 480 l 550 441 b 519 441 541 432 528 432 l 383 577 l 260 453 l 260 179 l 19 -62 b 19 -154 -7 -88 -7 -129 b 111 -154 44 -180 86 -180 l 352 87 l 623 87 l 1313 774 b 1293 1160 1408 869 1427 1050 z "
	},
	"": {
		ha: 1302,
		x_min: 0,
		x_max: 0,
		o: "m 1296 710 l 840 1166 b 809 1166 831 1174 817 1174 l 717 1074 b 717 1043 708 1065 708 1051 l 747 1013 l 90 357 b 64 -33 -16 252 -33 80 b 277 -130 120 -98 198 -130 b 474 -48 348 -130 420 -103 l 1143 618 l 1173 588 b 1204 588 1181 580 1195 580 l 1296 680 b 1296 710 1304 688 1304 702 z m 863 521 l 437 521 l 838 921 l 1051 709 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 195 1042 l 260 1042 l 260 391 b 477 174 260 271 358 174 b 694 391 597 174 694 271 l 694 1042 l 760 1042 b 781 1063 771 1042 781 1051 l 781 1194 b 760 1215 781 1206 771 1215 l 195 1215 b 174 1194 183 1215 174 1206 l 174 1063 b 195 1042 174 1051 183 1042 z m 391 1042 l 564 1042 l 564 781 l 391 781 z m 1693 0 l 43 0 b 0 -43 20 0 0 -20 l 0 -130 b 43 -174 0 -154 20 -174 l 1693 -174 b 1736 -130 1717 -174 1736 -154 l 1736 -43 b 1693 0 1736 -20 1717 0 z m 977 1042 l 1042 1042 l 1042 391 b 1259 174 1042 271 1139 174 b 1476 391 1378 174 1476 271 l 1476 1042 l 1541 1042 b 1563 1063 1553 1042 1563 1051 l 1563 1194 b 1541 1215 1563 1206 1553 1215 l 977 1215 b 955 1194 965 1215 955 1206 l 955 1063 b 977 1042 955 1051 965 1042 z m 1172 1042 l 1345 1042 l 1345 781 l 1172 781 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1367 260 l 370 260 b 348 239 358 260 348 251 l 348 109 b 370 87 348 97 358 87 l 1367 87 b 1389 109 1379 87 1389 97 l 1389 239 b 1367 260 1389 251 1379 260 z m 1367 0 l 369 0 b 347 -22 357 0 347 -10 l 347 -152 b 369 -174 347 -164 357 -174 l 1367 -174 b 1389 -152 1379 -174 1389 -164 l 1389 -22 b 1367 0 1389 -10 1379 0 z m 1367 521 l 371 521 b 349 499 359 521 349 511 l 349 369 b 370 347 349 357 358 347 l 1367 347 b 1389 369 1379 347 1389 357 l 1389 499 b 1367 521 1389 511 1379 521 z m 1656 898 l 918 1205 b 868 1215 905 1211 882 1215 b 818 1205 854 1215 831 1211 l 80 898 b 0 778 32 878 0 830 l 0 -152 b 22 -174 0 -164 10 -174 l 239 -174 b 260 -152 251 -174 260 -164 l 260 521 b 349 608 260 569 300 608 l 1387 608 b 1476 521 1436 608 1476 569 l 1476 -152 b 1497 -174 1476 -164 1485 -174 l 1714 -174 b 1736 -152 1726 -174 1736 -164 l 1736 778 b 1656 898 1736 830 1704 878 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1215 1042 l 1145 1042 b 1215 781 1189 965 1215 876 b 694 260 1215 494 982 260 b 174 781 407 260 174 494 b 244 1042 174 876 200 965 l 174 1042 b 0 868 78 1042 0 964 l 0 0 b 174 -174 0 -96 78 -174 l 1215 -174 b 1389 0 1311 -174 1389 -96 l 1389 868 b 1215 1042 1389 964 1311 1042 z m 694 347 b 1128 781 934 347 1128 542 b 694 1215 1128 1021 934 1215 b 260 781 455 1215 260 1021 b 694 347 260 542 455 347 z m 694 759 l 785 972 b 842 995 794 994 820 1004 b 865 938 864 985 874 960 l 773 725 b 803 651 791 705 803 680 b 694 543 803 591 754 543 b 586 651 635 543 586 591 b 694 759 586 711 634 759 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 651 174 b 608 130 627 174 608 154 b 651 87 608 106 627 87 b 694 130 675 87 694 106 b 651 174 694 154 675 174 z m 1085 87 b 1128 130 1109 87 1128 106 b 1085 174 1128 154 1109 174 b 1042 130 1061 174 1042 154 b 1085 87 1042 106 1061 87 z m 1693 1215 l 43 1215 b 0 1172 20 1215 0 1196 l 0 1085 b 43 1042 0 1061 20 1042 l 1693 1042 b 1736 1085 1717 1042 1736 1061 l 1736 1172 b 1693 1215 1736 1196 1717 1215 z m 1693 0 l 1563 0 l 1563 955 l 174 955 l 174 0 l 43 0 b 0 -43 20 0 0 -20 l 0 -130 b 43 -174 0 -154 20 -174 l 1693 -174 b 1736 -130 1717 -174 1736 -154 l 1736 -43 b 1693 0 1736 -20 1717 0 z m 1302 543 b 1280 521 1302 531 1292 521 l 911 521 l 911 434 l 1194 434 b 1215 412 1206 434 1215 424 l 1215 369 b 1194 347 1215 357 1206 347 l 911 347 l 911 260 l 1085 260 b 1215 130 1157 260 1215 202 b 1085 0 1215 58 1157 0 b 955 130 1013 0 955 58 l 955 174 l 781 174 l 781 130 b 651 0 781 58 723 0 b 521 130 579 0 521 58 b 651 260 521 202 579 260 l 825 260 l 825 347 l 543 347 b 521 369 531 347 521 357 l 521 412 b 543 434 521 424 531 434 l 825 434 l 825 521 l 456 521 b 434 543 444 521 434 531 l 434 586 b 456 608 434 598 444 608 l 825 608 l 825 694 l 543 694 b 521 716 531 694 521 704 l 521 760 b 543 781 521 771 531 781 l 825 781 l 825 846 b 846 868 825 858 834 868 l 890 868 b 911 846 902 868 911 858 l 911 781 l 1194 781 b 1215 760 1206 781 1215 771 l 1215 716 b 1194 694 1215 704 1206 694 l 911 694 l 911 608 l 1280 608 b 1302 586 1292 608 1302 598 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1155 521 b 1042 584 1109 521 1066 545 l 868 873 l 694 584 b 581 521 670 545 627 521 b 545 526 569 521 557 522 l 174 632 l 174 149 b 239 65 174 109 201 75 l 826 -82 b 910 -82 853 -88 882 -88 l 1497 65 b 1563 149 1535 75 1563 110 l 1563 632 l 1191 526 b 1155 521 1179 522 1167 521 z m 1731 825 l 1592 1104 b 1546 1128 1583 1121 1565 1131 l 868 1042 l 1117 629 b 1167 609 1127 612 1148 604 l 1704 763 b 1731 825 1731 770 1744 800 z m 144 1104 l 5 825 b 32 763 -8 800 5 770 l 569 610 b 619 629 588 604 609 612 l 868 1042 l 189 1128 b 144 1104 171 1130 153 1121 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1128 b 0 564 311 1128 0 876 b 155 210 0 430 58 307 b 6 -50 121 73 7 -49 b 2 -74 0 -56 -2 -66 b 22 -87 5 -82 13 -87 b 403 53 202 -87 336 -1 b 694 0 492 19 590 0 b 1389 564 1078 0 1389 253 b 694 1128 1389 876 1078 1128 z m 347 477 b 260 564 299 477 260 516 b 347 651 260 612 299 651 b 434 564 395 651 434 612 b 347 477 434 516 395 477 z m 694 477 b 608 564 646 477 608 516 b 694 651 608 612 646 651 b 781 564 742 651 781 612 b 694 477 781 516 742 477 z m 1042 477 b 955 564 994 477 955 516 b 1042 651 955 612 994 651 b 1128 564 1090 651 1128 612 b 1042 477 1128 516 1090 477 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 174 564 b 328 210 174 430 232 307 b 180 -50 294 73 181 -49 b 176 -74 174 -56 172 -66 b 195 -87 179 -82 187 -87 b 577 53 375 -87 510 -1 b 868 0 665 19 764 0 b 1081 27 942 0 1014 10 l 198 710 b 174 564 183 663 174 615 z m 1719 -27 l 1409 212 b 1563 564 1504 309 1563 431 b 868 1128 1563 876 1252 1128 b 409 986 691 1128 531 1074 l 123 1206 b 62 1198 104 1221 77 1217 l 9 1130 b 17 1069 -5 1111 -2 1084 l 1613 -164 b 1674 -157 1632 -179 1659 -176 l 1727 -88 b 1719 -27 1742 -69 1738 -42 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 434 608 l 434 434 l 1302 434 l 1302 608 b 1476 781 1302 703 1380 781 l 1563 781 b 1302 1042 1563 925 1446 1042 l 434 1042 b 174 781 290 1042 174 925 l 260 781 b 434 608 356 781 434 703 z m 1563 694 l 1476 694 b 1389 608 1428 694 1389 656 l 1389 347 l 347 347 l 347 608 b 260 694 347 656 308 694 l 174 694 b 0 521 78 694 0 617 b 87 371 0 457 35 401 l 87 43 b 130 0 87 20 106 0 l 304 0 b 347 43 328 0 347 20 l 347 87 l 1389 87 l 1389 43 b 1432 0 1389 20 1408 0 l 1606 0 b 1649 43 1630 0 1649 20 l 1649 371 b 1736 521 1701 401 1736 457 b 1563 694 1736 617 1658 694 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 87 b 1259 651 1006 87 1259 339 b 694 1215 1259 963 1006 1215 b 130 651 383 1215 130 963 b 694 87 130 339 383 87 z m 634 951 l 634 997 b 679 1042 634 1022 654 1042 l 709 1042 b 754 997 734 1042 754 1022 l 754 951 b 871 909 796 948 837 934 b 874 862 886 898 888 875 l 830 820 b 792 818 820 810 804 810 b 744 831 777 827 761 831 l 638 831 b 594 782 614 831 594 809 b 627 734 594 760 608 740 l 796 683 b 914 520 866 662 914 595 b 754 351 914 428 843 353 l 754 305 b 709 260 754 280 734 260 l 679 260 b 634 305 654 260 634 280 l 634 352 b 517 393 592 354 551 368 b 514 440 502 404 500 427 l 558 482 b 596 484 569 492 584 492 b 645 471 611 475 627 471 l 750 471 b 794 520 774 471 794 493 b 761 568 794 543 781 562 l 592 619 b 474 782 523 640 474 707 b 634 951 474 874 546 949 z m 1302 260 l 1214 260 b 1016 87 1161 190 1093 131 l 1189 87 b 1215 65 1203 87 1215 77 l 1215 22 b 1189 0 1215 10 1203 0 l 200 0 b 174 22 185 0 174 10 l 174 65 b 200 87 174 77 185 87 l 373 87 b 175 260 296 131 228 190 l 87 260 b 0 174 39 260 0 222 l 0 -87 b 87 -174 0 -135 39 -174 l 1302 -174 b 1389 -87 1350 -174 1389 -135 l 1389 174 b 1302 260 1389 222 1350 260 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 781 762 l 781 838 b 634 1178 705 936 653 1053 b 559 1200 629 1214 583 1229 b 426 965 502 1133 457 1053 b 781 762 519 862 643 790 z m 1085 1042 b 868 824 965 1042 868 944 l 868 663 b 236 1102 585 680 345 854 b 157 1104 221 1136 173 1138 b 87 790 112 1009 87 903 b 318 286 87 598 179 419 b 423 197 353 252 388 223 l 33 100 b 7 33 4 92 -9 59 b 423 -174 54 -40 171 -163 b 483 -152 444 -174 466 -167 l 659 0 l 868 0 b 1302 434 1108 0 1302 194 l 1302 868 l 1389 1042 z m 1085 781 b 1042 824 1061 781 1042 801 b 1085 868 1042 848 1061 868 b 1128 824 1109 868 1128 848 b 1085 781 1128 801 1109 781 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1533 325 b 1418 325 1501 354 1452 352 l 1167 125 b 1059 87 1137 100 1098 87 l 738 87 b 694 130 714 87 694 106 b 738 174 694 154 714 174 l 950 174 b 1041 246 993 174 1034 203 b 955 347 1050 300 1008 347 l 521 347 b 320 276 448 347 377 322 l 194 174 l 43 174 b 0 130 20 174 0 154 l 0 -130 b 43 -174 0 -154 20 -174 l 1011 -174 b 1120 -136 1051 -174 1089 -160 l 1530 193 b 1533 325 1571 225 1574 288 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 747 536 b 816 536 766 516 797 516 l 1111 846 b 1096 1168 1197 936 1192 1085 b 812 1147 1013 1240 888 1227 l 781 1115 l 751 1147 b 467 1168 675 1227 550 1240 b 451 846 371 1085 366 936 z m 1533 325 b 1418 325 1501 354 1452 352 l 1167 125 b 1059 87 1137 100 1098 87 l 738 87 b 694 130 714 87 694 106 b 738 174 694 154 714 174 l 950 174 b 1041 246 993 174 1034 203 b 955 347 1050 300 1008 347 l 521 347 b 320 276 448 347 377 322 l 194 174 l 43 174 b 0 130 20 174 0 154 l 0 -130 b 43 -174 0 -154 20 -174 l 1011 -174 b 1120 -136 1051 -174 1089 -160 l 1530 193 b 1533 325 1571 225 1574 288 z "
	},
	"": {
		ha: 1476,
		x_min: 0,
		x_max: 0,
		o: "m 699 824 l 834 785 b 851 763 844 782 851 773 b 828 740 851 751 841 740 l 739 740 b 711 746 729 740 720 742 b 673 741 698 752 683 751 l 626 693 b 630 643 611 679 613 655 b 716 612 655 626 685 616 l 716 564 b 759 521 716 540 735 521 l 803 521 b 846 564 827 521 846 540 l 846 612 b 980 783 928 622 991 696 b 863 912 972 845 924 895 l 728 951 b 711 973 718 954 711 963 b 734 996 711 986 722 996 l 823 996 b 851 990 833 996 842 994 b 889 995 864 984 879 985 l 937 1043 b 933 1093 951 1057 949 1082 b 846 1124 907 1110 878 1120 l 846 1172 b 803 1215 846 1196 827 1215 l 760 1215 b 716 1172 736 1215 716 1196 l 716 1124 b 582 953 634 1114 572 1040 b 699 824 590 891 639 841 z m 1448 325 b 1339 325 1418 354 1371 352 l 1102 125 b 1000 87 1073 100 1037 87 l 697 87 b 656 130 674 87 656 106 b 697 174 656 154 674 174 l 897 174 b 983 246 938 174 976 203 b 902 347 991 300 951 347 l 492 347 b 302 276 423 347 356 322 l 183 174 l 41 174 b 0 130 18 174 0 154 l 0 -130 b 41 -174 0 -154 18 -174 l 955 -174 b 1057 -136 992 -174 1028 -160 l 1445 193 b 1448 325 1484 225 1487 288 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 556 590 b 434 608 527 629 472 636 b 417 486 396 579 388 524 l 520 348 b 516 292 533 331 531 307 l 481 257 b 417 260 463 239 433 240 l 174 552 l 174 955 b 87 1042 174 1003 135 1042 b 0 955 39 1042 0 1003 l 0 362 b 28 281 0 333 10 304 l 311 -83 b 339 -142 324 -101 334 -121 b 381 -174 344 -161 361 -174 l 738 -174 b 781 -130 762 -174 781 -154 l 781 174 b 712 382 781 249 757 322 z m 1649 1042 b 1563 955 1601 1042 1563 1003 l 1563 552 l 1319 260 b 1255 257 1303 240 1273 239 l 1220 292 b 1216 348 1205 307 1203 331 l 1319 486 b 1302 608 1348 524 1340 579 b 1181 590 1264 636 1209 629 l 1024 382 b 955 174 979 322 955 249 l 955 -130 b 998 -174 955 -154 974 -174 l 1356 -174 b 1397 -142 1375 -174 1392 -161 b 1425 -83 1402 -121 1412 -101 l 1708 281 b 1736 362 1726 304 1736 333 l 1736 955 b 1649 1042 1736 1003 1697 1042 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1324 694 l 911 694 l 911 543 b 716 347 911 435 824 347 b 521 543 608 347 521 435 l 521 872 l 345 767 b 260 618 292 735 260 679 l 260 490 l 43 364 b 12 246 2 340 -12 287 l 229 -130 b 347 -162 253 -172 306 -186 l 628 0 l 998 0 b 1172 174 1094 0 1172 78 l 1215 174 b 1302 260 1263 174 1302 212 l 1302 434 l 1324 434 b 1389 499 1360 434 1389 463 l 1389 629 b 1324 694 1389 665 1360 694 z m 1724 796 l 1507 1172 b 1389 1204 1484 1213 1430 1227 l 1108 1042 l 831 1042 b 739 1015 799 1042 767 1032 l 648 959 b 608 885 623 943 608 915 l 608 543 b 716 434 608 483 656 434 b 825 543 776 434 825 483 l 825 781 l 1324 781 b 1476 629 1408 781 1476 713 l 1476 552 l 1693 677 b 1724 796 1734 701 1748 754 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1389 741 b 959 1171 1364 946 1176 1100 b 1128 694 1058 1064 1128 898 l 1205 694 l 888 343 b 868 347 882 345 875 347 l 738 347 l 738 694 l 1042 694 b 694 1215 1042 1007 855 1215 b 347 694 534 1215 347 1007 l 651 694 l 651 347 l 521 347 b 500 343 514 347 507 345 l 184 694 l 260 694 b 430 1171 260 898 331 1064 b 0 741 213 1099 25 945 b 44 694 -3 716 19 694 l 67 694 l 438 282 b 434 260 436 275 434 268 l 434 -87 b 521 -174 434 -135 473 -174 l 868 -174 b 955 -87 916 -174 955 -135 l 955 260 b 950 282 955 268 952 275 l 1321 694 l 1345 694 b 1389 741 1370 694 1392 716 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 347 955 b 477 1085 419 955 477 1013 b 347 1215 477 1157 419 1215 b 217 1085 275 1215 217 1157 b 347 955 217 1013 275 955 z m 1389 955 b 1519 1085 1461 955 1519 1013 b 1389 1215 1519 1157 1461 1215 b 1259 1085 1317 1215 1259 1157 b 1389 955 1259 1013 1317 955 z m 1730 -55 l 1611 244 l 1499 118 l 1494 69 l 1569 -119 b 1649 -174 1582 -153 1615 -174 b 1682 -167 1660 -174 1671 -172 b 1730 -55 1726 -149 1748 -99 z m 1637 515 l 1587 732 b 1485 854 1574 786 1536 832 b 1333 845 1435 876 1379 872 b 1203 684 1271 809 1225 751 l 1172 592 l 1128 564 l 1128 825 b 1085 868 1128 849 1109 868 l 651 868 b 608 825 627 868 608 849 l 608 564 l 564 592 l 533 684 b 403 845 511 752 464 809 b 251 854 356 872 301 876 b 149 732 199 832 161 786 l 99 515 b 138 361 87 460 101 403 l 320 155 l 348 -96 b 434 -174 353 -141 390 -174 b 444 -173 437 -174 440 -173 b 520 -77 491 -168 525 -125 l 493 174 b 451 269 489 210 474 243 l 333 402 l 381 593 l 399 538 b 466 449 410 504 432 474 l 604 361 b 651 347 617 353 637 349 l 1085 347 b 1132 361 1099 349 1119 353 l 1270 449 b 1337 538 1305 474 1326 503 l 1355 593 l 1403 402 l 1286 269 b 1243 174 1262 243 1247 210 l 1216 -77 b 1293 -173 1211 -125 1245 -168 b 1302 -174 1296 -173 1299 -174 b 1388 -96 1346 -174 1383 -141 l 1416 155 l 1598 361 b 1637 515 1635 403 1650 460 z m 126 244 l 6 -55 b 55 -167 -12 -99 10 -149 b 167 -119 100 -185 150 -163 l 243 69 l 237 118 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1519 608 l 1439 608 b 1338 750 1415 662 1380 710 l 1389 955 l 1302 955 b 1104 862 1222 955 1152 918 b 1042 868 1083 865 1063 868 l 694 868 b 269 521 484 868 310 719 l 152 521 b 88 599 112 521 80 557 b 155 651 94 630 123 651 l 157 651 b 174 667 166 651 174 658 l 174 722 b 157 738 174 731 166 738 b 1 606 80 738 11 683 b 152 434 -11 513 62 434 l 260 434 b 434 89 260 292 329 168 l 434 -130 b 477 -174 434 -154 454 -174 l 651 -174 b 694 -130 675 -174 694 -154 l 694 0 l 1042 0 l 1042 -130 b 1085 -174 1042 -154 1061 -174 l 1259 -174 b 1302 -130 1283 -174 1302 -154 l 1302 89 b 1387 174 1334 113 1363 141 l 1519 174 b 1563 217 1543 174 1563 193 l 1563 564 b 1519 608 1563 588 1543 608 z m 1172 434 b 1128 477 1148 434 1128 454 b 1172 521 1128 501 1148 521 b 1215 477 1196 521 1215 501 b 1172 434 1215 454 1196 434 z m 694 955 l 1042 955 b 1085 953 1056 955 1071 954 b 1085 955 1085 954 1085 954 b 825 1215 1085 1099 968 1215 b 564 955 681 1215 564 1099 b 566 938 564 949 566 944 b 694 955 607 949 650 955 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 17 10 b 29 -87 -9 -19 -4 -65 l 215 -162 b 300 -152 242 -181 279 -176 l 549 124 l 334 363 z m 1199 10 b 397 905 1199 10 407 891 b 608 954 439 928 506 954 b 818 905 709 954 776 928 b 666 730 809 890 666 730 l 881 492 l 973 595 b 1004 909 1051 682 1064 809 l 886 1108 b 840 1157 874 1128 859 1144 b 375 1157 730 1232 495 1238 b 330 1108 356 1144 341 1128 l 211 910 b 243 595 207 903 111 741 l 916 -152 b 1000 -162 937 -176 974 -180 l 1187 -87 b 1199 10 1219 -65 1225 -19 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1128 347 l 868 347 b 781 434 820 347 781 386 b 868 521 781 482 820 521 l 1128 521 b 1389 955 1128 521 1389 811 b 1128 1215 1389 1099 1272 1215 b 868 955 985 1215 868 1099 b 991 694 868 886 928 783 l 868 694 b 608 434 725 694 608 578 b 868 174 608 291 725 174 l 1128 174 b 1215 87 1176 174 1215 135 b 1128 0 1215 39 1176 0 l 503 0 b 375 -174 460 -67 412 -129 l 1128 -174 b 1389 87 1272 -174 1389 -57 b 1128 347 1389 230 1272 347 z m 1128 1042 b 1215 955 1176 1042 1215 1003 b 1128 868 1215 907 1176 868 b 1042 955 1080 868 1042 907 b 1128 1042 1042 1003 1080 1042 z m 260 521 b 0 260 117 521 0 404 b 260 -174 0 117 260 -174 b 521 260 260 -174 521 117 b 260 521 521 404 404 521 z m 260 174 b 174 260 212 174 174 212 b 260 347 174 308 212 347 b 347 260 308 347 347 308 b 260 174 347 212 308 174 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 174 955 l 0 955 b 608 347 0 619 272 347 l 608 -43 b 651 -87 608 -67 627 -87 l 738 -87 b 781 -43 762 -87 781 -67 l 781 347 b 174 955 781 683 509 955 z m 1215 1128 b 684 816 987 1128 788 1002 b 844 524 760 734 815 634 b 1389 1128 1150 556 1389 814 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1345 1042 l 347 1042 l 347 1172 b 304 1215 347 1196 328 1215 l 217 1215 b 174 1172 193 1215 174 1196 l 174 1042 l 43 1042 b 0 998 20 1042 0 1022 l 0 911 b 43 868 0 888 20 868 l 174 868 l 174 -130 b 217 -174 174 -154 193 -174 l 304 -174 b 347 -130 328 -174 347 -154 l 347 868 l 1345 868 b 1389 911 1369 868 1389 888 l 1389 998 b 1345 1042 1389 1022 1369 1042 z m 434 174 l 1302 174 l 1302 781 l 434 781 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 0 521 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 b 0 521 301 1194 0 892 z m 543 651 b 456 564 543 603 504 564 b 369 651 408 564 369 603 b 456 738 369 699 408 738 b 543 651 504 738 543 699 z m 972 606 b 807 606 932 642 847 642 l 781 583 b 728 613 759 563 723 582 b 890 727 738 681 820 727 b 1053 613 960 727 1042 681 b 999 583 1057 582 1022 563 z m 428 331 b 673 217 489 259 578 217 b 917 331 768 217 857 259 b 984 276 954 375 1021 320 b 673 130 907 183 793 130 b 361 276 552 130 439 183 b 428 331 325 320 392 376 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 608 694 b 434 521 512 694 434 617 b 608 347 434 425 512 347 b 781 521 703 347 781 425 b 608 694 781 617 703 694 z m 1693 87 l 1032 87 b 1215 521 1145 197 1215 351 b 608 1128 1215 856 943 1128 b 0 521 272 1128 0 856 b 608 -87 0 185 272 -87 l 1693 -87 b 1736 -43 1717 -87 1736 -67 l 1736 43 b 1693 87 1736 67 1717 87 z m 608 260 b 347 521 464 260 347 377 b 608 781 347 665 464 781 b 868 521 751 781 868 665 b 608 260 868 377 751 260 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 136 196 b 189 166 142 173 166 160 l 776 323 b 807 376 799 329 813 353 l 672 879 b 619 910 666 903 642 916 l 410 854 l 477 602 l 309 557 l 242 809 l 32 753 b 2 700 9 747 -5 723 z m 1042 1215 b 955 1128 994 1215 955 1176 l 955 251 l 16 -5 b 1 -32 4 -9 -2 -21 l 35 -158 b 62 -173 38 -169 50 -176 l 1130 119 b 1432 -174 1136 -44 1268 -174 b 1736 130 1600 -174 1736 -38 l 1736 1215 z m 1432 0 b 1302 130 1360 0 1302 58 b 1432 260 1302 202 1360 260 b 1563 130 1504 260 1563 202 b 1432 0 1563 58 1504 0 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1685 572 l 1527 730 b 1404 781 1494 763 1450 781 l 1302 781 l 1302 1042 b 1215 1128 1302 1090 1263 1128 l 87 1128 b 0 1042 39 1128 0 1090 l 0 130 b 217 -87 0 10 97 -87 b 391 1 288 -87 351 -52 b 564 -87 430 -52 493 -87 b 781 130 684 -87 781 10 b 777 174 781 145 780 160 l 1220 174 b 1215 130 1217 160 1215 145 b 1432 -87 1215 10 1312 -87 b 1649 130 1552 -87 1649 10 b 1645 174 1649 145 1648 160 l 1693 174 b 1736 217 1717 174 1736 193 l 1736 449 b 1685 572 1736 495 1718 539 z m 217 43 b 130 130 169 43 130 82 b 217 217 130 178 169 217 b 304 130 265 217 304 178 b 217 43 304 82 265 43 z m 564 43 b 477 130 516 43 477 82 b 564 217 477 178 516 217 b 651 130 612 217 651 178 b 564 43 651 82 612 43 z m 1302 651 l 1404 651 b 1434 638 1415 651 1426 646 l 1552 521 l 1302 521 z m 1432 43 b 1345 130 1385 43 1345 82 b 1432 217 1345 178 1385 217 b 1519 130 1480 217 1519 178 b 1432 43 1519 82 1480 43 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1719 -27 l 1570 88 b 1649 172 1612 92 1649 125 l 1649 869 b 1513 939 1649 939 1570 979 l 1215 734 l 1215 362 l 1128 429 l 1128 912 b 999 1042 1128 984 1070 1042 l 336 1042 l 123 1206 b 62 1198 104 1221 77 1217 l 9 1130 b 17 1069 -5 1111 -2 1084 l 116 993 l 1128 210 l 1613 -164 b 1674 -157 1632 -179 1659 -176 l 1727 -88 b 1719 -27 1742 -69 1738 -42 z m 87 130 b 216 0 87 58 145 0 l 999 0 b 1079 28 1029 0 1057 11 l 87 796 z "
	},
	"": {
		ha: 781,
		x_min: 0,
		x_max: 0,
		o: "m 586 -43 l 477 -43 l 477 274 b 780 693 663 318 798 491 l 736 1176 b 694 1215 734 1198 716 1215 l 88 1215 b 45 1176 65 1215 47 1198 l 2 693 b 304 274 -16 491 118 318 l 304 -43 l 195 -43 b 87 -152 135 -43 87 -92 b 109 -174 87 -164 97 -174 l 673 -174 b 694 -152 685 -174 694 -164 b 586 -43 694 -92 646 -43 z "
	},
	"": {
		ha: 10829,
		x_min: 0,
		x_max: 0,
		o: "m 1233 1215 l 156 1215 b 0 1060 70 1215 0 1145 l 0 -18 b 156 -174 0 -103 70 -174 l 1233 -174 b 1389 -18 1319 -174 1389 -103 l 1389 1060 b 1233 1215 1389 1145 1319 1215 z m 1073 334 b 1049 310 1073 321 1062 315 b 882 267 997 288 942 267 b 440 260 773 267 696 389 l 440 120 b 316 120 440 38 316 38 l 316 798 b 275 880 291 817 275 847 b 379 984 275 937 321 984 b 482 880 436 984 482 937 b 442 798 482 847 467 817 l 442 741 b 838 751 543 774 620 835 b 1046 793 910 720 1025 793 b 1074 769 1061 793 1074 782 l 1074 334 z m 2248 806 b 2247 975 2357 806 2355 975 l 1792 975 b 1684 867 1733 975 1684 927 l 1684 161 b 1876 161 1684 35 1876 38 l 1876 448 l 2238 448 b 2238 616 2348 448 2346 616 l 1876 616 l 1876 806 z m 2766 743 b 2766 58 2307 743 2307 58 b 2766 743 3227 58 3227 743 z m 2766 211 b 2766 590 2545 211 2543 590 b 2766 211 2990 590 2990 211 z m 3776 66 b 3862 151 3777 66 3863 69 l 3862 535 b 3422 642 3862 806 3539 782 l 3422 651 b 3251 651 3422 764 3251 760 l 3251 151 b 3422 151 3251 38 3422 39 l 3422 513 b 3691 469 3494 611 3691 638 l 3691 151 b 3776 66 3691 104 3729 66 z m 4264 58 b 4326 211 4469 58 4411 211 b 4252 282 4322 211 4252 202 l 4252 577 l 4319 577 b 4319 727 4416 577 4415 727 l 4252 727 l 4252 828 b 4079 828 4252 943 4079 940 l 4079 727 l 4036 727 b 4036 577 3937 727 3940 577 l 4079 577 l 4079 237 b 4264 58 4079 120 4144 58 z m 5404 882 b 5093 882 5351 1018 5145 1017 b 4823 169 4802 134 4823 196 b 5019 130 4823 53 4981 30 l 5055 227 l 5442 227 l 5477 130 b 5672 169 5515 27 5672 56 b 5404 882 5672 196 5687 150 z m 5108 396 l 5248 783 l 5389 396 z m 6712 628 b 6549 674 6746 738 6577 778 l 6442 288 l 6319 666 b 6126 666 6288 760 6156 755 l 6003 288 l 5896 674 b 5733 628 5868 780 5699 735 l 5880 150 b 6109 150 5914 38 6076 39 l 6224 518 l 6339 150 b 6567 150 6371 39 6533 38 z m 7459 424 b 7135 743 7459 539 7367 743 b 6798 401 6937 743 6798 590 b 7146 58 6798 192 6948 58 b 7399 166 7232 58 7399 90 b 7292 227 7399 216 7341 252 b 6978 346 7158 156 6996 204 l 7380 346 b 7459 424 7424 346 7459 382 z m 6974 462 b 7296 462 7000 661 7283 648 z m 7833 58 b 8116 267 8014 58 8116 149 b 7729 547 8116 531 7729 450 b 7832 612 7729 583 7770 612 b 8012 565 7939 612 7971 565 b 8068 663 8065 565 8098 623 b 7831 743 8032 716 7916 743 b 7563 537 7659 743 7563 647 b 7951 261 7563 281 7951 369 b 7841 191 7951 221 7916 191 b 7630 253 7730 191 7675 253 b 7571 145 7578 253 7538 191 b 7833 58 7612 87 7748 58 z m 8557 743 b 8557 58 8098 743 8098 58 b 8557 743 9019 58 9017 743 z m 8557 211 b 8557 590 8335 211 8335 590 b 8557 211 8781 590 8781 211 z m 9851 151 b 9936 66 9851 63 9945 66 b 10022 152 9986 66 10024 105 l 10022 546 b 9830 743 10022 680 9951 743 b 9606 624 9731 743 9644 683 b 9425 743 9582 699 9521 743 b 9212 642 9326 743 9239 681 l 9212 651 b 9041 651 9212 764 9041 760 l 9041 151 b 9213 151 9041 38 9213 39 l 9213 514 b 9445 486 9284 615 9445 628 l 9445 151 b 9618 151 9445 37 9618 39 l 9618 515 b 9851 486 9684 611 9851 632 z m 10829 424 b 10509 743 10829 539 10741 743 b 10163 401 10311 743 10163 590 b 10520 58 10163 192 10321 58 b 10769 166 10606 58 10769 90 b 10662 227 10769 216 10711 252 b 10343 346 10528 156 10362 204 l 10750 346 b 10829 424 10794 346 10828 382 z m 10339 462 b 10665 462 10366 664 10654 647 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1719 -27 l 1057 485 b 1259 825 1177 551 1259 678 b 868 1215 1259 1040 1084 1215 b 491 922 686 1215 534 1090 l 123 1206 b 62 1198 104 1221 77 1217 l 9 1130 b 17 1069 -5 1111 -2 1084 l 1613 -164 b 1674 -157 1632 -179 1659 -176 l 1727 -88 b 1719 -27 1742 -69 1738 -42 z m 538 347 b 174 -17 337 347 174 184 l 174 -43 b 304 -174 174 -115 232 -174 l 1341 -174 l 667 347 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 174 608 l 210 608 b 608 347 277 454 430 347 b 1005 608 786 347 938 454 l 1042 608 b 1085 651 1066 608 1085 627 l 1085 911 b 1042 955 1085 935 1066 955 l 1005 955 b 608 1215 938 1108 786 1215 b 210 955 430 1215 277 1108 l 174 955 b 130 911 150 955 130 935 l 130 651 b 174 608 130 627 150 608 z m 282 846 b 412 955 282 906 340 955 l 803 955 b 933 846 875 955 933 906 l 933 781 b 673 521 933 637 817 521 l 543 521 b 282 781 399 521 282 637 z m 477 651 l 510 749 l 608 781 l 510 814 l 477 911 l 445 814 l 347 781 l 445 749 z m 889 343 b 608 260 808 291 711 260 b 327 343 504 260 408 291 b 0 -17 144 324 0 171 l 0 -43 b 130 -174 0 -115 58 -174 l 347 -174 l 347 0 b 434 87 347 48 386 87 l 781 87 b 868 0 829 87 868 48 l 868 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 -17 b 889 343 1215 171 1072 324 z m 738 0 b 694 -43 714 0 694 -20 b 738 -87 694 -67 714 -87 b 781 -43 762 -87 781 -67 b 738 0 781 -20 762 0 z m 477 0 b 434 -43 454 0 434 -20 l 434 -174 l 521 -174 l 521 -43 b 477 0 521 -20 501 0 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 608 521 b 955 868 799 521 955 676 b 608 1215 955 1060 799 1215 b 260 868 416 1215 260 1060 b 608 521 260 676 416 521 z m 851 434 l 805 434 b 608 391 745 406 678 391 b 410 434 537 391 470 406 l 365 434 b 0 69 163 434 0 271 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 69 b 851 434 1215 271 1052 434 z m 1727 782 l 1651 859 b 1606 859 1639 871 1619 871 l 1322 577 l 1198 701 b 1153 701 1186 714 1165 714 l 1076 626 b 1076 580 1064 613 1064 593 l 1298 357 b 1343 356 1310 344 1331 344 l 1727 737 b 1727 782 1739 750 1739 770 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1345 608 b 955 217 1130 608 955 433 b 1345 -174 955 1 1130 -174 b 1736 217 1561 -174 1736 1 b 1345 608 1736 433 1561 608 z m 1519 200 b 1493 174 1519 186 1507 174 l 1328 174 b 1302 200 1314 174 1302 186 l 1302 408 b 1328 434 1302 422 1314 434 l 1363 434 b 1389 408 1377 434 1389 422 l 1389 260 l 1493 260 b 1519 234 1507 260 1519 248 z m 868 217 b 917 427 868 292 886 364 b 851 434 896 431 873 434 l 805 434 b 608 391 745 406 678 391 b 410 434 537 391 470 406 l 365 434 b 0 69 163 434 0 271 l 0 -43 b 130 -174 0 -115 58 -174 l 1072 -174 b 868 217 949 -87 868 56 z m 608 521 b 955 868 799 521 955 676 b 608 1215 955 1060 799 1215 b 260 868 416 1215 260 1060 b 608 521 260 676 416 521 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1656 203 b 1656 318 1663 241 1663 280 l 1726 359 b 1735 382 1734 363 1738 373 b 1645 537 1717 440 1686 493 b 1621 541 1639 544 1629 546 l 1551 501 b 1450 559 1521 526 1487 546 l 1450 639 b 1435 659 1450 649 1444 657 b 1255 659 1375 672 1313 672 b 1240 639 1246 657 1240 649 l 1240 559 b 1140 501 1203 546 1169 526 l 1070 541 b 1045 537 1062 546 1052 544 b 955 382 1005 493 974 440 b 964 359 953 373 956 363 l 1034 318 b 1034 203 1027 280 1027 241 l 964 162 b 955 139 956 158 953 148 b 1045 -17 974 81 1005 28 b 1070 -20 1052 -23 1062 -25 l 1140 20 b 1240 -38 1169 -5 1203 -25 l 1240 -119 b 1255 -138 1240 -128 1246 -136 b 1435 -138 1316 -151 1377 -151 b 1450 -119 1444 -136 1450 -128 l 1450 -38 b 1551 20 1487 -25 1521 -5 l 1621 -20 b 1645 -17 1628 -25 1639 -23 b 1735 139 1686 27 1717 81 b 1726 162 1738 148 1734 158 z m 1345 129 b 1214 260 1273 129 1214 188 b 1345 392 1214 333 1273 392 b 1477 260 1418 392 1477 333 b 1345 129 1477 188 1418 129 z m 608 521 b 955 868 799 521 955 676 b 608 1215 955 1060 799 1215 b 260 868 416 1215 260 1060 b 608 521 260 676 416 521 z m 1153 -94 b 1135 -83 1147 -90 1141 -87 l 1114 -95 b 1060 -110 1097 -105 1079 -110 b 982 -76 1031 -110 1002 -97 b 873 113 932 -22 894 43 b 921 237 858 161 878 212 l 943 250 b 943 271 943 257 943 264 l 921 283 b 873 407 878 308 858 359 b 882 431 875 415 879 423 b 851 434 871 432 861 434 l 805 434 b 608 391 745 406 678 391 b 410 434 537 391 470 406 l 365 434 b 0 69 163 434 0 271 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1159 -151 1112 -174 1138 -165 b 1153 -119 1156 -140 1153 -130 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 608 521 b 955 868 799 521 955 676 b 608 1215 955 1060 799 1215 b 260 868 416 1215 260 1060 b 608 521 260 676 416 521 z m 851 434 l 805 434 b 608 391 745 406 678 391 b 410 434 537 391 470 406 l 365 434 b 0 69 163 434 0 271 l 0 -43 b 130 -174 0 -115 58 -174 l 876 -174 b 869 -116 869 -155 867 -136 l 887 49 l 891 79 l 912 101 l 1122 311 b 851 434 1055 386 959 434 z m 974 40 l 955 -126 b 1003 -173 952 -153 975 -177 l 1168 -155 l 1542 219 l 1348 414 z m 1717 486 l 1614 589 b 1523 589 1589 614 1548 614 l 1420 486 l 1409 475 l 1604 280 l 1717 394 b 1717 486 1742 419 1742 460 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 521 521 b 825 825 689 521 825 657 b 521 1128 825 993 689 1128 b 217 825 353 1128 217 993 b 521 521 217 657 353 521 z m 729 434 l 707 434 b 521 391 650 407 588 391 b 335 434 454 391 392 407 l 313 434 b 0 122 140 434 0 294 l 0 43 b 130 -87 0 -28 58 -87 l 911 -87 b 1042 43 983 -87 1042 -28 l 1042 122 b 729 434 1042 294 902 434 z m 1302 521 b 1563 781 1446 521 1563 637 b 1302 1042 1563 925 1446 1042 b 1042 781 1158 1042 1042 925 b 1302 521 1042 637 1158 521 z m 1432 434 l 1422 434 b 1302 412 1384 421 1344 412 b 1182 434 1260 412 1220 421 l 1172 434 b 1021 392 1117 434 1066 418 b 1128 122 1087 321 1128 226 l 1128 17 b 1127 0 1128 11 1127 6 l 1606 0 b 1736 130 1678 0 1736 58 b 1432 434 1736 298 1600 434 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 866 346 l 608 87 l 349 346 b 0 -17 155 337 0 178 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 -17 b 866 346 1215 178 1060 337 z m 37 999 l 54 995 l 54 836 b 22 781 35 825 22 805 b 52 728 22 758 34 739 l 9 559 b 30 521 5 540 15 521 l 144 521 b 164 559 158 521 169 540 l 122 728 b 152 781 139 739 152 758 b 119 836 152 805 138 825 l 119 979 l 298 936 b 260 781 275 889 260 837 b 608 434 260 589 416 434 b 955 781 799 434 955 589 b 917 936 955 837 940 889 l 1178 999 b 1178 1084 1227 1011 1227 1072 l 662 1209 b 554 1209 626 1217 589 1217 l 37 1085 b 37 999 -12 1073 -12 1011 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 608 521 b 955 868 799 521 955 676 b 608 1215 955 1060 799 1215 b 260 868 416 1215 260 1060 b 608 521 260 676 416 521 z m 868 347 b 890 430 868 372 878 409 b 851 434 877 431 864 434 l 805 434 b 608 391 753 410 665 391 b 410 434 550 391 462 410 l 365 434 b 0 69 163 434 0 271 l 0 -43 b 130 -174 0 -115 58 -174 l 892 -174 b 868 -87 879 -151 868 -113 z m 1649 434 l 1563 434 l 1563 651 b 1345 868 1563 771 1465 868 b 1128 651 1226 868 1128 771 l 1128 434 l 1042 434 b 955 347 994 434 955 395 l 955 -87 b 1042 -174 955 -135 994 -174 l 1649 -174 b 1736 -87 1697 -174 1736 -135 l 1736 347 b 1649 434 1736 395 1697 434 z m 1345 43 b 1259 130 1298 43 1259 82 b 1345 217 1259 178 1298 217 b 1432 130 1393 217 1432 178 b 1345 43 1432 82 1393 43 z m 1432 434 l 1259 434 l 1259 651 b 1345 738 1259 699 1298 738 b 1432 651 1393 738 1432 699 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1693 651 l 1172 651 b 1128 608 1148 651 1128 632 l 1128 521 b 1172 477 1128 497 1148 477 l 1693 477 b 1736 521 1717 477 1736 497 l 1736 608 b 1693 651 1736 632 1717 651 z m 608 521 b 955 868 799 521 955 676 b 608 1215 955 1060 799 1215 b 260 868 416 1215 260 1060 b 608 521 260 676 416 521 z m 851 434 l 805 434 b 608 391 745 406 678 391 b 410 434 537 391 470 406 l 365 434 b 0 69 163 434 0 271 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 69 b 851 434 1215 271 1052 434 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 883 431 l 608 156 l 333 431 b 0 69 146 414 0 260 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 69 b 883 431 1215 260 1069 414 z m 87 694 b 275 775 161 694 227 726 b 608 521 315 629 448 521 b 955 868 799 521 955 676 b 608 1215 955 1060 799 1215 b 297 1021 471 1215 354 1136 b 87 1128 250 1086 174 1128 b 204 911 87 1038 133 958 b 87 694 133 865 87 785 z m 477 955 l 738 955 b 825 868 786 955 825 916 l 391 868 b 477 955 391 916 429 955 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1688 480 l 1376 602 b 1315 602 1364 606 1341 612 l 1003 480 b 955 412 974 468 955 442 b 1315 -168 955 110 1141 -100 b 1376 -168 1341 -178 1364 -172 b 1736 412 1515 -114 1736 75 b 1688 480 1736 442 1717 468 z m 1345 -39 l 1345 474 l 1605 373 b 1345 -39 1589 136 1439 5 z m 608 521 b 955 868 799 521 955 676 b 608 1215 955 1060 799 1215 b 260 868 416 1215 260 1060 b 608 521 260 676 416 521 z m 868 412 b 871 432 868 419 870 425 b 851 434 864 432 858 434 l 805 434 b 608 391 745 406 678 391 b 410 434 537 391 470 406 l 365 434 b 0 69 163 434 0 271 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1137 -163 1104 -174 1121 -170 b 868 412 991 -46 868 154 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1719 -27 l 983 542 b 1215 868 1118 589 1215 717 b 868 1215 1215 1060 1060 1215 b 524 897 686 1215 538 1075 l 123 1206 b 62 1198 104 1221 77 1217 l 9 1130 b 17 1069 -5 1111 -2 1084 l 1613 -164 b 1674 -157 1632 -179 1659 -176 l 1727 -88 b 1719 -27 1742 -69 1738 -42 z m 260 69 l 260 -43 b 391 -174 260 -115 319 -174 l 1341 -174 l 563 428 b 260 69 391 398 260 250 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1711 225 l 1466 470 b 1343 521 1433 503 1389 521 l 1128 521 b 1041 434 1080 521 1041 482 l 1041 219 b 1092 97 1041 173 1059 129 l 1337 -148 b 1459 -148 1370 -182 1426 -182 l 1710 103 b 1711 225 1745 137 1745 192 z m 1215 282 b 1150 347 1179 282 1150 311 b 1215 413 1150 384 1179 413 b 1280 347 1251 413 1280 384 b 1215 282 1280 312 1251 282 z m 608 521 b 955 868 799 521 955 677 b 608 1215 955 1060 799 1215 b 260 868 416 1215 260 1060 b 608 521 260 677 416 521 z m 954 219 l 954 418 b 851 435 921 428 887 435 l 805 435 b 608 391 745 407 678 391 b 410 435 537 391 470 407 l 365 435 b 0 70 163 434 0 271 l 0 -43 b 130 -173 0 -115 58 -173 l 1085 -173 b 1188 -122 1127 -173 1164 -153 l 1031 35 b 954 219 981 84 954 150 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 608 521 b 955 868 799 521 955 676 b 608 1215 955 1060 799 1215 b 260 868 416 1215 260 1060 b 608 521 260 676 416 521 z m 868 432 l 738 -87 l 651 282 l 738 434 l 477 434 l 564 282 l 477 -87 l 348 432 b 0 69 154 423 0 265 l 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 69 b 868 432 1215 265 1061 423 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1656 289 b 1656 405 1663 328 1663 367 l 1726 445 b 1735 468 1734 450 1738 460 b 1645 624 1717 527 1686 580 b 1621 628 1639 631 1629 633 l 1551 588 b 1450 645 1521 613 1487 632 l 1450 726 b 1435 745 1450 735 1444 744 b 1255 745 1375 759 1313 758 b 1240 726 1246 744 1240 735 l 1240 645 b 1140 588 1203 632 1169 613 l 1070 628 b 1045 624 1062 633 1052 631 b 955 468 1005 580 974 527 b 964 445 953 460 956 450 l 1034 405 b 1034 289 1027 367 1027 328 l 964 249 b 955 226 956 244 953 235 b 1045 70 974 167 1005 114 b 1070 66 1052 63 1062 62 l 1140 107 b 1240 49 1169 82 1203 62 l 1240 -32 b 1255 -51 1240 -41 1246 -49 b 1435 -51 1316 -65 1377 -64 b 1450 -32 1444 -49 1450 -41 l 1450 49 b 1551 107 1487 62 1521 82 l 1621 66 b 1645 70 1628 62 1639 63 b 1735 226 1686 114 1717 167 b 1726 249 1738 235 1734 244 z m 1345 216 b 1214 347 1273 216 1214 275 b 1345 479 1214 420 1273 479 b 1477 347 1418 479 1477 420 b 1345 216 1477 275 1418 216 z m 260 608 b 434 781 356 608 434 685 b 260 955 434 877 356 955 b 87 781 165 955 87 877 b 260 608 87 685 165 608 z m 868 521 b 883 522 873 521 878 522 b 982 683 906 581 939 637 b 1060 717 1002 705 1030 717 b 1113 703 1079 717 1097 712 l 1135 690 b 1141 694 1137 692 1139 693 b 1172 824 1160 734 1172 778 b 868 1128 1172 992 1036 1128 b 564 825 700 1128 564 993 b 868 521 564 657 700 521 z m 1153 -7 b 1135 4 1147 -4 1141 0 b 1060 -23 1113 -9 1093 -23 b 982 11 1031 -23 1002 -10 b 873 200 932 65 894 130 b 943 337 844 294 940 335 b 943 358 943 344 943 351 l 921 370 b 895 392 911 376 903 384 b 868 391 886 392 877 391 b 683 434 802 391 739 407 l 660 434 b 347 122 487 434 347 294 l 347 43 b 477 -87 347 -28 406 -87 l 1170 -87 b 1153 -32 1160 -71 1153 -52 z m 470 470 b 347 521 438 502 395 521 l 174 521 b 0 347 78 521 0 443 l 0 260 b 87 174 0 212 39 174 l 266 174 b 470 470 283 302 360 410 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1432 0 l 955 0 l 955 800 b 1073 928 1010 824 1053 871 l 1431 1048 b 1458 1103 1453 1056 1466 1081 l 1430 1186 b 1375 1213 1423 1208 1398 1221 l 1056 1106 b 868 1215 1018 1171 949 1215 b 651 998 748 1215 651 1118 b 654 971 651 989 653 980 l 306 854 b 278 799 283 847 271 822 l 306 717 b 361 690 313 694 338 682 l 746 819 b 781 800 757 811 769 805 l 781 -130 b 825 -174 781 -154 801 -174 l 1432 -174 b 1476 -130 1456 -174 1476 -154 l 1476 -43 b 1432 0 1476 -19 1456 0 z m 1736 391 b 1505 883 1736 435 1740 414 b 1272 883 1457 979 1320 979 b 1042 391 1036 410 1042 433 l 1042 391 b 1389 174 1042 271 1197 174 b 1736 391 1581 174 1736 271 z m 1194 434 l 1389 825 l 1584 434 z m 464 536 b 231 536 416 632 279 632 b 0 43 -6 63 0 86 l 0 43 b 347 -174 0 -76 155 -174 b 694 43 539 -174 694 -76 l 694 43 b 464 536 694 87 698 67 z m 152 87 l 347 477 l 543 87 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 260 -43 l 260 -130 b 304 -174 260 -154 280 -174 l 911 -174 b 955 -130 935 -174 955 -154 l 955 800 b 990 819 967 805 979 811 l 1375 690 b 1430 717 1398 682 1423 694 l 1458 799 b 1431 854 1466 822 1453 847 l 1082 971 b 1085 998 1083 980 1085 989 b 868 1215 1085 1118 988 1215 b 681 1106 788 1215 718 1171 l 361 1213 b 306 1186 338 1221 313 1208 l 278 1103 b 305 1048 271 1081 283 1056 l 664 928 b 781 800 683 871 726 824 l 781 0 l 304 0 b 260 -43 280 0 260 -19 z m 0 391 b 347 174 0 271 155 174 b 694 391 539 174 694 271 l 694 391 b 464 883 694 433 700 410 b 231 883 416 979 279 979 b 0 391 -4 414 0 435 z m 152 434 l 347 825 l 543 434 z m 1042 43 l 1042 43 b 1389 -174 1042 -76 1197 -174 b 1736 43 1581 -174 1736 -76 l 1736 43 b 1505 536 1736 86 1742 63 b 1272 536 1457 632 1320 632 b 1042 43 1038 67 1042 87 z m 1194 87 l 1389 477 l 1584 87 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1128 174 l 434 174 b 260 0 338 174 260 96 l 260 -87 b 347 -174 260 -135 299 -174 l 1215 -174 b 1302 -87 1263 -174 1302 -135 l 1302 0 b 1128 174 1302 96 1224 174 z m 781 -87 b 694 0 733 -87 694 -48 b 781 87 694 48 733 87 b 868 0 829 87 868 48 b 781 -87 868 -48 829 -87 z m 890 1042 l 1342 1042 l 1389 1215 l 130 1215 b 0 1085 58 1215 0 1157 l 0 651 b 130 521 0 579 58 521 l 410 521 l 434 260 l 1128 260 l 1176 434 l 890 434 b 868 456 878 434 868 444 l 868 499 b 890 521 868 511 878 521 l 1199 521 l 1247 694 l 890 694 b 868 716 878 694 868 704 l 868 760 b 890 781 868 772 878 781 l 1271 781 l 1318 955 l 890 955 b 868 977 878 955 868 965 l 868 1020 b 890 1042 868 1032 878 1042 z m 174 694 l 174 1042 l 363 1042 l 395 694 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1471 1128 b 844 978 1322 1120 1027 1089 b 825 942 832 970 825 956 l 825 -45 b 888 -82 825 -77 859 -96 b 1481 45 1075 13 1347 38 b 1562 129 1527 48 1562 85 l 1562 1045 b 1471 1128 1563 1093 1521 1131 z m 718 978 b 92 1128 536 1089 240 1120 b 0 1045 42 1131 0 1093 l 0 129 b 81 45 0 85 36 48 b 675 -82 216 38 487 13 b 738 -45 704 -96 738 -77 l 738 942 b 718 978 738 957 731 970 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 409 694 l 501 694 b 541 752 531 694 551 724 b 521 868 528 789 521 828 b 541 984 521 908 528 947 b 501 1042 551 1012 531 1042 l 409 1042 b 368 1011 390 1042 373 1029 b 347 868 354 965 347 917 b 368 725 347 819 354 771 b 409 694 373 707 390 694 z m 244 1152 b 206 1215 259 1181 239 1215 l 110 1215 b 70 1190 93 1215 77 1206 b 0 868 26 1092 0 983 b 72 546 0 801 8 683 b 112 521 80 530 95 521 l 207 521 b 244 584 239 521 260 555 b 244 1152 109 837 199 1067 z m 1666 1190 b 1626 1215 1659 1206 1643 1215 l 1530 1215 b 1492 1152 1497 1215 1476 1181 b 1492 585 1541 1059 1623 828 b 1530 521 1477 556 1497 521 l 1626 521 b 1666 546 1643 521 1659 531 b 1736 868 1711 644 1736 753 b 1666 1190 1736 983 1711 1092 z m 1327 1042 l 1235 1042 b 1195 984 1205 1042 1185 1012 b 1215 868 1208 947 1215 908 b 1195 752 1215 828 1208 789 b 1235 694 1185 724 1205 694 l 1327 694 b 1368 725 1346 694 1363 707 b 1389 868 1382 771 1389 819 b 1368 1011 1389 917 1382 965 b 1327 1042 1363 1029 1346 1042 z m 1011 770 b 1042 868 1030 798 1042 832 b 868 1042 1042 964 964 1042 b 695 868 772 1042 695 964 b 725 770 695 832 706 798 l 371 -80 b 394 -137 362 -102 372 -128 l 474 -170 b 531 -147 496 -179 522 -169 l 664 174 l 1072 174 l 1205 -147 b 1262 -170 1214 -169 1240 -179 l 1342 -137 b 1365 -80 1364 -128 1374 -102 z m 737 347 l 868 662 l 999 347 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 696 627 b 253 516 696 627 415 646 b 105 252 193 469 144 365 l 268 312 b 287 286 284 317 297 299 l 30 -18 b 0 -169 10 -106 0 -169 b 723 -76 0 -169 561 -206 b 931 331 886 53 931 331 z m 1727 1131 l 1673 1199 b 1612 1206 1658 1218 1631 1221 l 981 724 l 889 840 b 838 834 875 858 848 854 l 770 686 l 1005 390 l 1164 424 b 1182 471 1186 428 1195 454 l 1089 588 l 1720 1070 b 1727 1131 1738 1085 1742 1112 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 260 1042 l 1476 1042 l 1476 87 l 1649 87 l 1649 1107 b 1541 1215 1649 1167 1601 1215 l 195 1215 b 87 1107 135 1215 87 1167 l 87 87 l 260 87 z m 1693 0 l 1302 0 l 1302 174 l 781 174 l 781 0 l 43 0 b 0 -43 19 0 0 -19 l 0 -130 b 43 -174 0 -154 19 -174 l 1693 -174 b 1736 -130 1717 -174 1736 -154 l 1736 -43 b 1693 0 1736 -19 1717 0 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 564 260 b 545 257 558 260 551 259 b 434 239 510 246 473 239 b 323 257 395 239 358 246 b 304 260 317 259 310 260 b 0 -45 135 260 -1 123 b 130 -174 0 -116 59 -174 l 738 -174 b 868 -45 809 -174 868 -116 b 564 260 869 123 733 260 z m 434 347 b 694 608 578 347 694 464 b 434 868 694 751 578 868 b 174 608 290 868 174 751 b 434 347 174 464 290 347 z m 1606 1215 l 564 1215 b 434 1081 492 1215 434 1155 l 434 955 b 608 907 498 955 556 936 l 608 1042 l 1563 1042 l 1563 260 l 1389 260 l 1389 434 l 1042 434 l 1042 260 l 835 260 b 943 87 887 215 925 155 l 1606 87 b 1736 221 1678 87 1736 147 l 1736 1081 b 1606 1215 1736 1155 1678 1215 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1260 546 l 955 729 l 955 868 l 1085 868 b 1128 911 1109 868 1128 887 l 1128 998 b 1085 1042 1128 1022 1109 1042 l 955 1042 l 955 1172 b 911 1215 955 1196 935 1215 l 825 1215 b 781 1172 801 1215 781 1196 l 781 1042 l 651 1042 b 608 998 627 1042 608 1022 l 608 911 b 651 868 608 887 627 868 l 781 868 l 781 729 l 476 546 b 434 472 453 532 434 499 l 434 -174 l 694 -174 l 694 87 b 868 260 694 183 772 260 b 1042 87 964 260 1042 183 l 1042 -174 l 1302 -174 l 1302 472 b 1260 546 1302 502 1286 530 z m 0 141 l 0 -130 b 43 -174 0 -154 19 -174 l 347 -174 l 347 347 l 53 221 b 0 141 24 209 0 173 z m 1684 221 l 1389 347 l 1389 -174 l 1693 -174 b 1736 -130 1717 -174 1736 -154 l 1736 141 b 1684 221 1736 176 1715 207 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 0 116 l 0 0 b 521 -174 0 -96 233 -174 b 1042 0 808 -174 1042 -96 l 1042 116 b 521 0 930 37 725 0 b 0 116 317 0 112 37 z m 868 868 b 1389 1042 1156 868 1389 946 b 868 1215 1389 1137 1156 1215 b 347 1042 581 1215 347 1137 b 868 868 347 946 581 868 z m 0 400 l 0 260 b 521 87 0 165 233 87 b 1042 260 808 87 1042 165 l 1042 400 b 521 260 930 308 725 260 b 0 400 317 260 112 308 z m 1128 371 b 1389 521 1284 401 1389 457 l 1389 637 b 1128 543 1326 592 1233 562 z m 521 781 b 0 564 233 781 0 684 b 521 347 0 444 233 347 b 1042 564 808 347 1042 444 b 521 781 1042 684 808 781 z m 1116 629 b 1389 781 1278 658 1389 715 l 1389 897 b 953 784 1293 829 1127 792 b 1116 629 1033 745 1092 693 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 239 521 l 152 521 b 673 1042 152 808 385 1042 l 673 955 b 239 521 433 955 239 760 z m 673 260 b 412 521 529 260 412 377 b 673 781 412 665 529 781 b 933 521 817 781 933 665 b 673 260 933 377 817 260 z m 673 608 b 586 521 625 608 586 569 b 673 434 586 473 625 434 b 760 521 721 434 760 473 b 673 608 760 569 721 608 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1476 1128 l 1431 1128 b 1259 1215 1392 1181 1329 1215 b 1042 998 1139 1215 1042 1118 l 1042 941 l 33 148 b 0 82 15 134 0 105 b 119 9 0 21 64 -18 l 448 174 l 710 174 l 831 -152 b 872 -172 837 -169 855 -178 l 933 -149 b 953 -108 950 -143 959 -125 l 849 174 l 955 174 b 970 174 960 174 965 174 l 1091 -152 b 1133 -172 1097 -169 1116 -178 l 1194 -149 b 1213 -108 1211 -143 1219 -125 l 1101 195 b 1476 694 1318 258 1476 458 l 1476 911 l 1736 955 b 1476 1128 1736 1051 1620 1128 z m 1259 933 b 1194 998 1223 933 1194 962 b 1259 1063 1194 1034 1223 1063 b 1324 998 1295 1063 1324 1034 b 1259 933 1324 962 1295 933 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1432 0 l 304 0 b 260 -43 280 0 260 -20 l 260 -130 b 304 -174 260 -154 280 -174 l 1432 -174 b 1476 -130 1456 -174 1476 -154 l 1476 -43 b 1432 0 1476 -20 1456 0 z m 1606 868 b 1476 738 1534 868 1476 810 b 1488 684 1476 719 1480 701 l 1291 566 b 1171 598 1249 541 1195 556 l 950 985 b 998 1085 979 1009 998 1044 b 868 1215 998 1157 940 1215 b 738 1085 796 1215 738 1157 b 786 985 738 1044 757 1009 l 565 598 b 445 566 541 556 486 541 l 249 684 b 261 738 256 700 261 719 b 130 868 261 810 202 868 b 0 738 59 868 0 810 b 130 608 0 666 58 608 b 151 610 137 608 144 609 l 347 87 l 1389 87 l 1585 610 b 1606 608 1592 609 1599 608 b 1736 738 1678 608 1736 666 b 1606 868 1736 810 1678 868 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1606 694 l 1284 694 b 1238 452 1318 614 1303 518 l 868 83 l 868 -43 b 998 -174 868 -115 926 -174 l 1606 -174 b 1736 -43 1678 -174 1736 -115 l 1736 564 b 1606 694 1736 636 1678 694 z m 1302 195 b 1237 260 1266 195 1237 224 b 1302 326 1237 296 1266 326 b 1367 260 1338 326 1367 296 b 1302 195 1367 224 1338 195 z m 1176 702 l 702 1176 b 514 1176 650 1228 566 1228 l 39 702 b 39 514 -13 650 -13 566 l 514 39 b 702 39 565 -13 650 -13 l 1176 514 b 1176 702 1228 566 1228 650 z m 260 543 b 195 608 224 543 195 572 b 260 673 195 644 224 673 b 326 608 296 673 326 644 b 260 543 326 572 296 543 z m 608 195 b 543 260 572 195 543 224 b 608 326 543 296 572 326 b 673 260 644 326 673 296 b 608 195 673 224 644 195 z m 608 543 b 543 608 572 543 543 572 b 608 673 543 644 572 673 b 673 608 644 673 673 644 b 608 543 673 572 644 543 z m 608 890 b 543 955 572 890 543 919 b 608 1020 543 991 572 1020 b 673 955 644 1020 673 991 b 608 890 673 919 644 890 z m 955 543 b 890 608 919 543 890 572 b 955 673 890 644 919 673 b 1020 608 991 673 1020 644 b 955 543 1020 572 991 543 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1042 1128 l 174 1128 b 0 955 78 1128 0 1051 l 0 87 b 174 -87 0 -9 78 -87 l 1042 -87 b 1215 87 1138 -87 1215 -9 l 1215 955 b 1042 1128 1215 1051 1138 1128 z m 347 174 b 260 260 299 174 260 212 b 347 347 260 308 299 347 b 434 260 395 347 434 308 b 347 174 434 212 395 174 z m 347 694 b 260 781 299 694 260 733 b 347 868 260 829 299 868 b 434 781 395 868 434 829 b 347 694 434 733 395 694 z m 608 434 b 521 521 560 434 521 473 b 608 608 521 569 560 608 b 694 521 656 608 694 569 b 608 434 694 473 656 434 z m 868 174 b 781 260 820 174 781 212 b 868 347 781 308 820 347 b 955 260 916 347 955 308 b 868 174 955 212 916 174 z m 868 694 b 781 781 820 694 781 733 b 868 868 781 829 820 868 b 955 781 916 868 955 829 b 868 694 955 733 916 694 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1042 1128 l 174 1128 b 0 955 78 1128 0 1051 l 0 87 b 174 -87 0 -9 78 -87 l 1042 -87 b 1215 87 1138 -87 1215 -9 l 1215 955 b 1042 1128 1215 1051 1138 1128 z m 347 174 b 260 260 299 174 260 212 b 347 347 260 308 299 347 b 434 260 395 347 434 308 b 347 174 434 212 395 174 z m 347 694 b 260 781 299 694 260 733 b 347 868 260 829 299 868 b 434 781 395 868 434 829 b 347 694 434 733 395 694 z m 868 174 b 781 260 820 174 781 212 b 868 347 781 308 820 347 b 955 260 916 347 955 308 b 868 174 955 212 916 174 z m 868 694 b 781 781 820 694 781 733 b 868 868 781 829 820 868 b 955 781 916 868 955 829 b 868 694 955 733 916 694 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1042 1128 l 174 1128 b 0 955 78 1128 0 1051 l 0 87 b 174 -87 0 -9 78 -87 l 1042 -87 b 1215 87 1138 -87 1215 -9 l 1215 955 b 1042 1128 1215 1051 1138 1128 z m 608 434 b 521 521 560 434 521 473 b 608 608 521 569 560 608 b 694 521 656 608 694 569 b 608 434 694 473 656 434 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1042 1128 l 174 1128 b 0 955 78 1128 0 1051 l 0 87 b 174 -87 0 -9 78 -87 l 1042 -87 b 1215 87 1138 -87 1215 -9 l 1215 955 b 1042 1128 1215 1051 1138 1128 z m 347 174 b 260 260 299 174 260 212 b 347 347 260 308 299 347 b 434 260 395 347 434 308 b 347 174 434 212 395 174 z m 347 434 b 260 521 299 434 260 473 b 347 608 260 569 299 608 b 434 521 395 608 434 569 b 347 434 434 473 395 434 z m 347 694 b 260 781 299 694 260 733 b 347 868 260 829 299 868 b 434 781 395 868 434 829 b 347 694 434 733 395 694 z m 868 174 b 781 260 820 174 781 212 b 868 347 781 308 820 347 b 955 260 916 347 955 308 b 868 174 955 212 916 174 z m 868 434 b 781 521 820 434 781 473 b 868 608 781 569 820 608 b 955 521 916 608 955 569 b 868 434 955 473 916 434 z m 868 694 b 781 781 820 694 781 733 b 868 868 781 829 820 868 b 955 781 916 868 955 829 b 868 694 955 733 916 694 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1042 1128 l 174 1128 b 0 955 78 1128 0 1051 l 0 87 b 174 -87 0 -9 78 -87 l 1042 -87 b 1215 87 1138 -87 1215 -9 l 1215 955 b 1042 1128 1215 1051 1138 1128 z m 347 694 b 260 781 299 694 260 733 b 347 868 260 829 299 868 b 434 781 395 868 434 829 b 347 694 434 733 395 694 z m 608 434 b 521 521 560 434 521 473 b 608 608 521 569 560 608 b 694 521 656 608 694 569 b 608 434 694 473 656 434 z m 868 174 b 781 260 820 174 781 212 b 868 347 781 308 820 347 b 955 260 916 347 955 308 b 868 174 955 212 916 174 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1042 1128 l 174 1128 b 0 955 78 1128 0 1051 l 0 87 b 174 -87 0 -9 78 -87 l 1042 -87 b 1215 87 1138 -87 1215 -9 l 1215 955 b 1042 1128 1215 1051 1138 1128 z m 347 694 b 260 781 299 694 260 733 b 347 868 260 829 299 868 b 434 781 395 868 434 829 b 347 694 434 733 395 694 z m 868 174 b 781 260 820 174 781 212 b 868 347 781 308 820 347 b 955 260 916 347 955 308 b 868 174 955 212 916 174 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 608 260 b 434 87 512 260 434 183 b 608 -87 434 -9 512 -87 b 781 87 704 -87 781 -9 b 608 260 781 183 704 260 z m 608 781 b 781 955 704 781 781 859 b 608 1128 781 1051 704 1128 b 434 955 512 1128 434 1051 b 608 781 434 859 512 781 z m 1128 651 l 87 651 b 0 564 39 651 0 612 l 0 477 b 87 391 0 429 39 391 l 1128 391 b 1215 477 1176 391 1215 429 l 1215 564 b 1128 651 1215 612 1176 651 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1693 0 l 1389 0 l 1389 1077 b 1259 1215 1389 1153 1330 1215 l 477 1215 b 347 1077 406 1215 347 1153 l 347 0 l 43 0 b 0 -43 19 0 0 -19 l 0 -130 b 43 -174 0 -154 19 -174 l 1693 -174 b 1736 -130 1717 -174 1736 -154 l 1736 -43 b 1693 0 1736 -19 1717 0 z m 1128 434 b 1042 521 1081 434 1042 473 b 1128 608 1042 569 1081 608 b 1215 521 1176 608 1215 569 b 1128 434 1215 473 1176 434 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1693 0 l 1476 0 l 1476 908 b 1345 1042 1476 981 1417 1042 l 1042 1042 l 1042 868 l 1302 868 l 1302 -174 l 1693 -174 b 1736 -130 1717 -174 1736 -154 l 1736 -43 b 1693 0 1736 -19 1717 0 z m 847 1213 l 326 1078 b 260 990 288 1068 260 1032 l 260 0 l 43 0 b 0 -43 19 0 0 -19 l 0 -130 b 43 -174 0 -154 19 -174 l 955 -174 l 955 1125 b 847 1213 955 1184 902 1227 z m 716 434 b 651 521 680 434 651 473 b 716 608 651 569 680 608 b 781 521 752 608 781 569 b 716 434 781 473 752 434 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1128 391 l 87 391 b 0 304 39 391 0 352 l 0 217 b 87 130 0 169 39 130 l 1128 130 b 1215 217 1176 130 1215 169 l 1215 304 b 1128 391 1215 352 1176 391 z m 1128 911 l 87 911 b 0 825 39 911 0 873 l 0 738 b 87 651 0 690 39 651 l 1128 651 b 1215 738 1176 651 1215 690 l 1215 825 b 1128 911 1215 873 1176 911 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1267 1094 b 583 1024 1098 1263 829 1269 b 350 791 370 811 417 859 b 178 219 186 628 159 382 l 662 702 b 723 702 679 719 706 719 b 723 641 740 685 740 658 l 19 -62 b 19 -155 -6 -88 -6 -129 b 111 -155 45 -180 86 -180 l 291 25 b 960 174 432 -18 757 -25 l 694 174 l 1094 307 b 1220 434 1230 442 1193 405 l 954 434 l 1311 553 b 1267 1094 1435 755 1404 957 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1211 951 b 998 1128 1193 1052 1105 1128 b 785 951 892 1128 804 1052 b 0 84 344 909 -1 537 b 174 -87 0 -11 79 -87 l 998 -87 b 1042 -43 1022 -87 1042 -67 b 955 43 1042 5 1003 43 l 739 43 l 836 175 b 745 464 902 273 864 414 b 549 434 680 491 605 476 l 460 367 b 399 376 441 353 414 357 b 408 437 385 395 389 423 l 502 508 b 842 508 602 583 742 583 b 952 255 871 486 966 402 l 1274 -87 l 1519 -87 b 1563 -43 1543 -87 1563 -67 b 1476 43 1563 5 1524 43 l 1330 43 l 1063 327 l 1482 562 b 1563 700 1526 587 1563 649 l 1563 700 b 1480 839 1563 758 1531 811 b 1211 951 1370 899 1211 951 z m 998 846 b 933 911 962 846 933 876 b 998 977 933 947 962 977 b 1063 911 1034 977 1063 947 b 998 846 1063 876 1034 846 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 911 0 l 43 0 b 0 -43 20 0 0 -20 l 0 -130 b 43 -174 0 -154 20 -174 l 911 -174 b 955 -130 935 -174 955 -154 l 955 -43 b 911 0 955 -20 935 0 z m 1338 924 l 1118 1144 b 1057 1144 1101 1161 1074 1161 l 1026 1113 b 1026 1052 1009 1096 1009 1069 l 1128 950 l 1128 781 b 1259 632 1128 705 1185 642 l 1259 195 b 1194 130 1259 160 1229 130 b 1128 195 1158 130 1128 160 l 1128 282 b 890 521 1128 414 1022 521 l 868 521 l 868 1042 b 694 1215 868 1137 790 1215 l 260 1215 b 87 1042 165 1215 87 1137 l 87 87 l 868 87 l 868 391 l 890 391 b 998 282 950 391 998 342 l 998 207 b 1173 1 998 104 1072 11 b 1389 195 1290 -11 1389 81 l 1389 801 b 1338 924 1389 847 1370 892 z m 694 694 l 260 694 l 260 1042 l 694 1042 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1557 455 l 1434 948 b 1318 1099 1418 1012 1376 1067 b 1127 1116 1260 1131 1190 1137 l 1086 1102 b 1058 1048 1063 1095 1051 1070 l 1072 1006 b 1127 979 1080 984 1104 971 l 1163 991 b 1253 986 1192 1001 1225 1000 b 1308 916 1281 972 1300 947 l 1412 499 b 1192 533 1352 517 1277 533 b 881 460 1098 533 991 514 l 682 460 b 370 533 571 514 465 533 b 150 499 285 533 210 517 l 254 916 b 310 986 262 947 282 972 b 400 991 337 1001 370 1001 l 435 979 b 490 1006 458 971 483 984 l 504 1048 b 477 1102 512 1070 499 1095 l 435 1116 b 245 1099 372 1137 303 1131 b 128 948 187 1067 144 1012 l 5 455 b 0 413 2 443 0 424 l 0 222 b 313 -87 0 52 140 -87 l 413 -87 b 725 199 577 -87 713 38 l 733 304 l 830 304 l 838 199 b 1149 -87 850 38 986 -87 l 1250 -87 b 1563 222 1423 -87 1563 52 l 1563 413 b 1557 455 1563 427 1561 441 z m 552 212 b 413 87 546 142 486 87 l 313 87 b 174 222 236 87 174 148 l 174 324 b 370 359 223 342 291 359 b 560 324 435 359 499 347 z m 1389 222 b 1250 87 1389 147 1327 87 l 1149 87 b 1011 212 1077 87 1016 142 l 1002 324 b 1192 359 1064 347 1127 359 b 1389 324 1271 359 1340 342 l 1389 222 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 992 646 l 161 1034 b 45 991 117 1054 65 1035 l 8 913 b 50 797 -12 869 7 817 l 643 521 l 50 244 b 8 129 7 224 -12 172 l 45 50 b 160 8 65 7 117 -12 l 992 396 b 1042 474 1019 409 1042 444 l 1042 567 b 992 646 1042 601 1022 632 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 150 923 l 626 738 l 150 554 b 93 443 100 537 74 488 l 125 361 b 249 309 144 316 199 293 l 1066 634 b 1128 716 1103 646 1128 679 l 1128 759 b 1066 841 1128 796 1103 829 l 248 1167 b 125 1115 198 1183 143 1160 l 93 1034 b 150 923 75 989 100 939 z m 1150 130 l 65 130 b 0 65 29 130 0 101 l 0 -65 b 65 -130 0 -101 29 -130 l 1150 -130 b 1215 -65 1186 -130 1215 -101 l 1215 65 b 1150 130 1215 101 1186 130 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 825 174 l 1563 174 b 1649 260 1610 174 1649 212 b 1042 868 1649 596 1377 868 l 1042 1042 l 1519 1042 b 1563 1085 1543 1042 1563 1061 l 1563 1172 b 1519 1215 1563 1196 1543 1215 l 391 1215 b 347 1172 367 1215 347 1196 l 347 1085 b 391 1042 347 1061 367 1042 l 868 1042 l 868 868 l 304 868 l 187 1024 b 152 1042 178 1035 166 1042 l 43 1042 b 1 988 15 1042 -6 1015 l 87 694 l 521 521 l 755 208 b 825 174 770 189 801 174 z m 1128 685 b 1467 347 1298 650 1432 517 l 1128 347 z m 1724 -29 l 1663 31 b 1602 31 1647 48 1619 47 b 1535 0 1583 13 1565 0 l 651 0 b 608 -43 627 0 608 -19 l 608 -130 b 651 -174 608 -154 627 -174 l 1535 -174 b 1721 -94 1576 -174 1631 -172 b 1724 -29 1740 -77 1741 -47 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1278 955 b 868 742 1099 955 958 843 b 458 955 778 843 637 955 b 0 521 206 955 0 760 b 458 87 0 282 206 87 b 868 300 637 87 778 199 b 1278 87 958 199 1099 87 b 1736 521 1530 87 1736 282 b 1278 955 1736 760 1530 955 z m 458 347 b 260 521 349 347 260 425 b 458 694 260 617 349 694 b 713 521 562 694 657 597 b 458 347 658 446 562 347 z m 1278 347 b 1023 521 1174 347 1079 445 b 1278 694 1078 596 1175 694 b 1476 521 1387 694 1476 617 b 1278 347 1476 425 1387 347 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1562 624 b 1241 911 1553 788 1406 911 l 1216 911 b 815 1037 1073 911 933 956 b 400 1115 701 1115 555 1150 b 0 608 351 1104 0 1003 b 260 157 0 415 105 247 l 260 -43 b 304 -87 260 -67 280 -87 l 347 -87 b 391 -43 371 -87 391 -67 l 391 104 b 521 87 432 93 476 87 b 564 90 536 87 550 89 l 564 -43 b 608 -87 564 -67 584 -87 l 651 -87 b 694 -43 675 -87 694 -67 l 694 118 b 804 171 733 132 770 149 b 1217 304 928 252 1068 304 b 1279 306 1286 304 1260 305 l 1481 -64 b 1519 -87 1489 -78 1503 -87 b 1530 -85 1523 -87 1526 -86 b 1562 -43 1549 -81 1562 -63 b 1562 624 1562 -43 1563 608 z m 1259 543 b 1193 608 1223 543 1193 572 b 1259 673 1193 644 1223 673 b 1324 608 1295 673 1324 644 b 1259 543 1324 572 1295 543 z m 1476 127 l 1367 325 b 1476 396 1409 341 1445 365 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 991 245 l 399 521 l 992 797 b 1034 913 1035 818 1054 869 l 997 992 b 881 1034 977 1035 925 1054 l 50 646 b 0 567 22 633 0 598 l 0 474 b 50 396 0 441 20 410 l 881 8 b 997 50 925 -12 976 7 l 1034 129 b 991 245 1054 173 1035 225 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 149 634 l 967 309 b 1090 361 1017 293 1072 316 l 1123 443 b 1065 554 1141 488 1115 537 l 589 738 l 1065 923 b 1123 1034 1115 939 1141 989 l 1090 1115 b 968 1167 1072 1160 1017 1183 l 149 841 b 87 759 112 829 87 796 l 87 716 b 149 634 87 679 112 646 z m 1150 130 l 65 130 b 0 65 29 130 0 101 l 0 -65 b 65 -130 0 -101 29 -130 l 1150 -130 b 1215 -65 1186 -130 1215 -101 l 1215 65 b 1150 130 1215 101 1186 130 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1736 860 l 1736 955 b 1649 1042 1736 1003 1697 1042 l 87 1042 b 0 955 39 1042 0 1003 l 0 860 b 87 738 50 842 87 794 b 0 616 87 681 50 634 l 0 347 l 1736 347 l 1736 616 b 1649 738 1686 634 1649 681 b 1736 860 1649 794 1686 842 z m 608 521 l 434 521 l 434 868 l 608 868 z m 955 521 l 781 521 l 781 868 l 955 868 z m 1302 521 l 1128 521 l 1128 868 l 1302 868 z m 0 0 l 174 0 l 174 72 b 217 116 174 96 193 116 b 260 72 241 116 260 96 l 260 0 l 608 0 l 608 72 b 651 116 608 96 627 116 b 694 72 675 116 694 96 l 694 0 l 1042 0 l 1042 72 b 1085 116 1042 96 1061 116 b 1128 72 1109 116 1128 96 l 1128 0 l 1476 0 l 1476 72 b 1519 116 1476 96 1495 116 b 1563 72 1543 116 1563 96 l 1563 0 l 1736 0 l 1736 260 l 0 260 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1719 -27 l 1292 303 b 1345 521 1326 368 1345 442 l 1345 651 b 1302 694 1345 675 1326 694 l 1259 694 b 1215 651 1235 694 1215 675 l 1215 521 b 1186 385 1215 472 1205 426 l 1114 440 b 1128 521 1123 466 1128 492 l 1010 521 l 898 608 l 1128 608 l 1128 694 l 897 694 b 868 716 881 694 868 704 l 868 760 b 897 781 868 772 881 781 l 1128 781 l 1128 868 l 897 868 b 868 890 881 868 868 878 l 868 933 b 897 955 868 945 881 955 l 1128 955 b 868 1215 1128 1099 1012 1215 b 608 955 724 1215 608 1099 l 608 832 l 123 1206 b 62 1199 104 1221 77 1217 l 9 1130 b 17 1069 -6 1111 -2 1084 l 1613 -164 b 1674 -157 1632 -179 1659 -176 l 1727 -88 b 1719 -27 1742 -69 1738 -42 z m 1085 -43 l 933 -43 l 933 48 b 1025 70 965 53 996 60 l 889 176 b 833 175 870 175 852 173 b 532 452 682 190 566 307 l 391 561 l 391 542 b 803 49 391 299 564 82 l 803 -43 l 651 -43 b 608 -87 627 -43 608 -63 l 608 -130 b 651 -174 608 -154 627 -174 l 1085 -174 b 1128 -130 1109 -174 1128 -154 l 1128 -87 b 1085 -43 1128 -63 1109 -43 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1685 1068 b 1369 1128 1580 1112 1474 1128 b 367 959 1035 1128 701 959 b 116 997 283 959 199 970 b 87 1001 106 1000 97 1001 b 0 915 41 1001 0 965 l 0 54 b 51 -26 0 20 20 -13 b 367 -87 156 -70 262 -87 b 1369 82 701 -87 1035 82 b 1621 45 1453 82 1537 72 b 1649 41 1630 42 1639 41 b 1736 127 1695 41 1736 77 l 1736 988 b 1685 1068 1736 1022 1716 1054 z m 130 857 b 300 832 185 843 242 836 b 130 693 284 753 214 693 z m 130 83 l 130 213 b 303 47 223 213 299 139 b 130 83 242 51 185 64 z m 868 260 b 651 521 748 260 651 377 b 868 781 651 665 748 781 b 1085 521 988 781 1085 665 b 868 260 1085 377 988 260 z m 1606 185 b 1459 208 1558 197 1509 204 b 1606 342 1474 279 1533 332 z m 1606 826 b 1455 993 1522 836 1457 906 b 1606 958 1508 987 1558 976 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1685 1068 b 1369 1128 1580 1112 1474 1128 b 367 959 1035 1128 701 959 b 116 997 283 959 199 970 b 87 1001 106 1000 97 1001 b 0 915 41 1001 0 965 l 0 54 b 51 -26 0 20 20 -13 b 367 -87 156 -70 262 -87 b 1369 82 701 -87 1035 82 b 1621 45 1453 82 1537 72 b 1649 41 1630 42 1639 41 b 1736 127 1695 41 1736 77 l 1736 988 b 1685 1068 1736 1022 1716 1054 z m 868 260 b 651 521 748 260 651 377 b 868 781 651 665 748 781 b 1085 521 988 781 1085 665 b 868 260 1085 377 988 260 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 0 0 b 87 -87 0 -48 39 -87 l 1649 -87 b 1736 0 1697 -87 1736 -48 l 1736 868 l 0 868 z m 1215 564 b 1259 608 1215 588 1235 608 l 1519 608 b 1563 564 1543 608 1563 588 l 1563 477 b 1519 434 1563 453 1543 434 l 1259 434 b 1215 477 1235 434 1215 453 z m 1215 239 b 1237 260 1215 251 1225 260 l 1541 260 b 1563 239 1553 260 1563 251 l 1563 195 b 1541 174 1563 183 1553 174 l 1237 174 b 1215 195 1225 174 1215 183 z m 174 499 b 195 521 174 511 183 521 l 1020 521 b 1042 499 1032 521 1042 511 l 1042 456 b 1020 434 1042 444 1032 434 l 195 434 b 174 456 183 434 174 444 z m 174 239 b 195 260 174 251 183 260 l 673 260 b 694 239 685 260 694 251 l 694 195 b 673 174 694 183 685 174 l 195 174 b 174 195 183 174 174 183 z m 1693 1128 l 43 1128 b 0 1085 19 1128 0 1109 l 0 955 l 1736 955 l 1736 1085 b 1693 1128 1736 1109 1717 1128 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1649 1128 l 87 1128 b 0 1042 39 1128 0 1090 l 0 0 b 87 -87 0 -48 39 -87 l 1649 -87 b 1736 0 1697 -87 1736 -48 l 1736 1042 b 1649 1128 1736 1090 1697 1128 z m 477 326 l 477 282 b 456 260 477 270 468 260 l 412 260 b 391 282 400 260 391 270 l 391 326 b 306 357 360 328 330 339 b 304 390 295 365 294 381 l 336 420 b 363 422 343 428 355 428 b 398 412 374 416 386 412 l 474 412 b 506 448 492 412 506 428 b 483 483 506 464 497 478 l 361 519 b 275 637 310 534 275 583 b 391 759 275 703 327 758 l 391 803 b 412 825 391 815 400 825 l 456 825 b 477 803 468 825 477 815 l 477 759 b 563 728 508 757 538 747 b 564 695 573 720 574 704 l 532 665 b 505 663 525 657 513 657 b 470 673 494 669 482 673 l 394 673 b 362 637 376 673 362 657 b 385 602 362 621 371 607 l 508 566 b 593 448 558 551 593 502 b 477 326 593 382 542 328 z m 1128 369 b 1107 347 1128 357 1119 347 l 803 347 b 781 369 791 347 781 357 l 781 412 b 803 434 781 424 791 434 l 1107 434 b 1128 412 1119 434 1128 424 z m 1563 369 b 1541 347 1563 357 1553 347 l 1324 347 b 1302 369 1312 347 1302 357 l 1302 412 b 1324 434 1302 424 1312 434 l 1541 434 b 1563 412 1553 434 1563 424 z m 1563 629 b 1541 608 1563 617 1553 608 l 803 608 b 781 629 791 608 781 617 l 781 673 b 803 694 781 685 791 694 l 1541 694 b 1563 673 1553 694 1563 685 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1128 651 b 1215 738 1176 651 1215 690 l 1215 825 b 1128 911 1215 873 1176 911 l 1064 911 l 1204 1093 b 1196 1154 1219 1112 1215 1139 l 1127 1206 b 1067 1198 1108 1221 1081 1217 l 844 911 l 87 911 b 0 825 39 911 0 873 l 0 738 b 87 651 0 690 39 651 l 642 651 l 440 391 l 87 391 b 0 304 39 391 0 352 l 0 217 b 87 130 0 169 39 130 l 237 130 l 97 -51 b 105 -112 82 -70 86 -97 l 174 -165 b 234 -157 193 -179 220 -176 l 457 130 l 1128 130 b 1215 217 1176 130 1215 169 l 1215 304 b 1128 391 1215 352 1176 391 l 659 391 l 861 651 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 554 1202 b 14 664 285 1149 67 932 b 716 -168 -86 156 371 -222 b 831 80 828 -151 883 -20 b 997 347 769 203 858 347 l 1213 347 b 1389 524 1310 347 1389 428 b 554 1202 1388 952 999 1288 z m 260 347 b 174 434 212 347 174 386 b 260 521 174 482 212 521 b 347 434 308 521 347 482 b 260 347 347 386 308 347 z m 347 694 b 260 781 299 694 260 733 b 347 868 260 829 299 868 b 434 781 395 868 434 829 b 347 694 434 733 395 694 z m 694 868 b 608 955 646 868 608 907 b 694 1042 608 1003 646 1042 b 781 955 742 1042 781 1003 b 694 868 781 907 742 868 z m 1042 694 b 955 781 994 694 955 733 b 1042 868 955 829 994 868 b 1128 781 1090 868 1128 829 b 1042 694 1128 733 1090 694 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1085 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -28 58 -87 l 1085 -87 b 1215 43 1157 -87 1215 -28 l 1215 998 b 1085 1128 1215 1070 1157 1128 z m 651 347 l 521 347 l 521 217 b 477 174 521 193 501 174 l 391 174 b 347 217 367 174 347 193 l 347 825 b 391 868 347 849 367 868 l 651 868 b 911 608 795 868 911 751 b 651 347 911 464 795 347 z m 651 694 l 521 694 l 521 521 l 651 521 b 738 608 699 521 738 560 b 651 694 738 655 699 694 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 296 745 b 296 991 364 813 364 923 b 51 991 229 1059 119 1059 b 51 745 -17 923 -17 813 b 296 745 119 677 229 677 z m 991 296 b 745 296 923 364 813 364 b 745 51 677 229 677 119 b 991 51 813 -17 923 -17 b 991 296 1059 119 1059 229 z m 986 924 l 924 986 b 801 986 890 1019 835 1019 l 56 240 b 56 117 22 206 22 151 l 117 56 b 240 56 151 22 206 22 l 986 801 b 986 924 1019 835 1019 890 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1042 347 l 694 347 b 608 260 647 347 608 308 l 608 -87 b 694 -174 608 -135 647 -174 l 1042 -174 b 1128 -87 1090 -174 1128 -135 l 1128 260 b 1042 347 1128 308 1090 347 z m 521 1128 b 434 1215 521 1176 482 1215 l 87 1215 b 0 1128 39 1215 0 1176 l 0 781 b 87 694 0 733 39 694 l 346 694 l 545 347 b 694 434 575 399 630 434 l 695 434 l 521 739 l 521 868 l 1128 868 l 1128 1042 l 521 1042 z m 1649 1215 l 1302 1215 b 1215 1128 1254 1215 1215 1176 l 1215 781 b 1302 694 1215 733 1254 694 l 1649 694 b 1736 781 1697 694 1736 733 l 1736 1128 b 1649 1215 1736 1176 1697 1215 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 972 1207 l 868 1085 l 720 1207 b 694 1215 714 1211 702 1215 b 668 1207 686 1215 675 1211 l 521 1085 l 373 1207 b 347 1215 367 1211 355 1215 b 321 1207 339 1215 328 1211 l 174 1085 l 69 1207 b 0 1172 41 1228 0 1208 l 0 -130 b 69 -165 0 -166 41 -186 l 174 -43 l 321 -165 b 347 -174 328 -170 339 -174 b 373 -165 355 -174 367 -170 l 521 -43 l 668 -165 b 694 -174 675 -170 686 -174 b 720 -165 702 -174 714 -170 l 868 -43 l 972 -165 b 1042 -130 1001 -186 1042 -166 l 1042 1172 b 972 1207 1042 1208 1001 1228 z m 868 239 b 846 217 868 227 858 217 l 195 217 b 174 239 183 217 174 227 l 174 282 b 195 304 174 294 183 304 l 846 304 b 868 282 858 304 868 294 z m 868 499 b 846 477 868 487 858 477 l 195 477 b 174 499 183 477 174 487 l 174 543 b 195 564 174 554 183 564 l 846 564 b 868 543 858 564 868 554 z m 868 760 b 846 738 868 748 858 738 l 195 738 b 174 760 183 738 174 748 l 174 803 b 195 825 174 815 183 825 l 846 825 b 868 803 858 825 868 815 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 0 521 l 0 174 b 87 87 0 126 39 87 l 174 87 l 174 608 l 87 608 b 0 521 39 608 0 569 z m 1259 955 l 955 955 l 955 1128 b 868 1215 955 1176 916 1215 b 781 1128 820 1215 781 1176 l 781 955 l 477 955 b 260 738 358 955 260 858 l 260 0 b 434 -174 260 -96 338 -174 l 1302 -174 b 1476 0 1398 -174 1476 -96 l 1476 738 b 1259 955 1476 858 1379 955 z m 694 87 l 521 87 l 521 174 l 694 174 z m 608 412 b 499 521 548 412 499 461 b 608 629 499 581 548 629 b 716 521 668 629 716 581 b 608 412 716 461 668 412 z m 955 87 l 781 87 l 781 174 l 955 174 z m 1215 87 l 1042 87 l 1042 174 l 1215 174 z m 1128 412 b 1020 521 1069 412 1020 461 b 1128 629 1020 581 1069 629 b 1237 521 1188 629 1237 581 b 1128 412 1237 461 1188 412 z m 1649 608 l 1563 608 l 1563 87 l 1649 87 b 1736 174 1697 87 1736 126 l 1736 521 b 1649 608 1736 569 1697 608 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1724 762 l 1509 1129 b 1391 1160 1485 1170 1432 1184 l 1203 1054 l 1365 779 b 1357 750 1371 768 1368 755 l 1320 728 b 1290 736 1310 722 1296 726 l 1128 1012 l 979 927 l 1054 798 b 1047 769 1060 788 1057 775 l 1009 748 b 980 756 999 742 986 746 l 904 884 l 754 799 l 916 524 b 908 495 922 514 919 500 l 871 473 b 841 481 861 467 847 471 l 680 757 l 530 672 l 605 544 b 598 515 611 533 608 521 l 560 493 b 531 501 550 487 537 491 l 455 630 l 305 545 l 467 269 b 459 240 473 259 470 246 l 422 219 b 392 227 412 213 398 217 l 230 502 l 43 396 b 11 280 2 372 -12 321 l 227 -88 b 345 -119 251 -129 304 -142 l 1693 646 b 1724 762 1734 669 1748 721 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 434 434 l 282 434 b 260 456 270 434 260 444 l 260 499 b 282 521 260 511 270 521 l 434 521 l 434 694 l 282 694 b 260 716 270 694 260 704 l 260 760 b 282 781 260 772 270 781 l 434 781 l 434 955 l 282 955 b 260 977 270 955 260 965 l 260 1020 b 282 1042 260 1032 270 1042 l 434 1042 l 434 1128 b 347 1215 434 1176 395 1215 l 87 1215 b 0 1128 39 1215 0 1176 l 0 -87 b 4 -108 0 -94 2 -101 l 434 322 z m 1302 260 l 1215 260 l 1215 109 b 1194 87 1215 97 1206 87 l 1150 87 b 1128 109 1138 87 1128 97 l 1128 260 l 955 260 l 955 109 b 933 87 955 97 945 87 l 890 87 b 868 109 878 87 868 97 l 868 260 l 694 260 l 694 109 b 673 87 694 97 685 87 l 629 87 b 608 109 617 87 608 97 l 608 260 l 495 260 l 66 -169 b 87 -174 73 -171 79 -174 l 1302 -174 b 1389 -87 1350 -174 1389 -135 l 1389 174 b 1302 260 1389 222 1350 260 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1476 868 l 1345 868 l 1345 629 b 1324 608 1345 617 1336 608 l 1280 608 b 1259 629 1268 608 1259 617 l 1259 868 l 1085 868 l 1085 629 b 1063 608 1085 617 1075 608 l 1020 608 b 998 629 1008 608 998 617 l 998 868 l 825 868 l 825 629 b 803 608 825 617 815 608 l 760 608 b 738 629 748 608 738 617 l 738 868 l 564 868 l 564 629 b 543 608 564 617 555 608 l 499 608 b 477 629 487 608 477 617 l 477 868 l 304 868 l 304 629 b 282 608 304 617 294 608 l 239 608 b 217 629 227 608 217 617 l 217 868 l 87 868 b 0 781 39 868 0 829 l 0 260 b 87 174 0 212 39 174 l 1476 174 b 1563 260 1524 174 1563 212 l 1563 781 b 1476 868 1563 829 1524 868 z "
	},
	"": {
		ha: 694,
		x_min: 0,
		x_max: 0,
		o: "m 456 87 b 434 109 444 87 434 97 l 434 152 b 456 174 434 164 444 174 l 694 174 l 694 347 l 456 347 b 434 369 444 347 434 357 l 434 412 b 456 434 434 424 444 434 l 694 434 l 694 608 l 456 608 b 434 629 444 608 434 617 l 434 673 b 456 694 434 685 444 694 l 694 694 l 694 868 l 456 868 b 434 890 444 868 434 878 l 434 933 b 456 955 434 945 444 955 l 694 955 l 694 1128 b 608 1215 694 1176 656 1215 l 87 1215 b 0 1128 39 1215 0 1176 l 0 -87 b 87 -174 0 -135 39 -174 l 608 -174 b 694 -87 656 -174 694 -135 l 694 87 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 0 608 l 0 -130 b 43 -174 0 -154 19 -174 l 260 -174 l 260 694 l 87 694 b 0 608 39 694 0 656 z m 977 738 l 911 738 l 911 846 b 890 868 911 858 902 868 l 846 868 b 825 846 834 868 825 858 l 825 673 b 846 651 825 661 834 651 l 977 651 b 998 673 989 651 998 661 l 998 716 b 977 738 998 728 989 738 z m 1350 911 l 916 1201 b 868 1215 904 1209 883 1215 b 820 1201 854 1215 832 1209 l 386 911 b 347 839 365 897 347 865 l 347 -174 l 694 -174 l 694 217 b 738 260 694 241 714 260 l 998 260 b 1042 217 1022 260 1042 241 l 1042 -174 l 1389 -174 l 1389 839 b 1350 911 1389 868 1374 895 z m 868 521 b 651 738 748 521 651 618 b 868 955 651 858 748 955 b 1085 738 988 955 1085 858 b 868 521 1085 618 988 521 z m 1649 694 l 1476 694 l 1476 -174 l 1693 -174 b 1736 -130 1717 -174 1736 -154 l 1736 608 b 1649 694 1736 656 1697 694 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1215 1215 l 868 955 l 868 787 l 643 561 b 695 521 661 550 679 537 b 735 469 710 505 723 488 l 960 694 l 1128 694 l 1389 1042 z m 347 460 l 30 142 b 30 -1 -10 102 -10 38 l 173 -144 b 316 -144 212 -183 276 -183 l 633 174 b 633 460 712 253 712 381 b 347 460 554 539 426 539 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 521 781 l 608 781 l 608 1128 l 521 1128 b 347 955 425 1128 347 1051 b 521 781 347 859 425 781 z m 0 87 b 174 -87 0 -9 78 -87 l 260 -87 l 260 260 l 174 260 b 0 87 78 260 0 183 z m 915 434 b 631 347 821 434 709 398 b 347 260 564 303 511 260 l 347 -87 l 503 -130 b 721 -171 574 -150 647 -166 b 986 -154 810 -177 899 -173 b 1389 174 1283 -90 1389 51 b 915 434 1389 347 1161 434 z m 1333 1196 b 1068 1213 1246 1214 1157 1219 b 851 1172 994 1207 922 1192 l 694 1128 l 694 781 b 979 694 858 781 911 739 b 1263 608 1056 643 1168 608 b 1736 868 1508 608 1736 694 b 1333 1196 1736 991 1630 1132 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1215 b 0 608 311 1215 0 943 b 256 136 0 417 100 248 b 293 55 282 118 298 87 l 267 -124 b 310 -174 264 -150 284 -174 l 521 -174 l 521 -22 b 543 0 521 -10 531 0 l 586 0 b 608 -22 598 0 608 -10 l 608 -174 l 781 -174 l 781 -22 b 803 0 781 -10 791 0 l 846 0 b 868 -22 858 0 868 -10 l 868 -174 l 1079 -174 b 1121 -124 1105 -174 1125 -150 l 1096 55 b 1133 136 1091 87 1106 118 b 1389 608 1289 248 1389 417 b 694 1215 1389 943 1078 1215 z m 434 347 b 260 521 338 347 260 425 b 434 694 260 617 338 694 b 608 521 530 694 608 617 b 434 347 608 425 530 347 z m 955 347 b 781 521 859 347 781 425 b 955 694 781 617 859 694 b 1128 521 1051 694 1128 617 b 955 347 1128 425 1051 347 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 260 391 b 304 347 260 367 280 347 l 623 347 l 362 608 l 304 608 b 260 564 280 608 260 588 z m 694 1215 b 0 521 311 1215 0 904 b 694 -174 0 137 311 -174 b 1389 521 1078 -174 1389 137 b 694 1215 1389 904 1078 1215 z m 694 0 b 174 521 407 0 174 234 b 270 822 174 633 210 737 l 996 97 b 694 0 911 36 807 0 z m 817 521 l 1042 521 l 1042 434 l 904 434 z m 1118 219 l 991 347 l 1085 347 b 1128 391 1109 347 1128 367 l 1128 564 b 1085 608 1128 588 1109 608 l 730 608 l 393 945 b 694 1042 478 1006 582 1042 b 1215 521 982 1042 1215 808 b 1118 219 1215 409 1179 305 z m 870 868 b 785 938 827 868 792 898 b 764 955 783 948 774 955 l 720 955 b 698 929 706 955 696 943 b 870 781 710 846 783 781 b 955 711 912 781 947 751 b 976 694 957 701 966 694 l 1020 694 b 1041 720 1033 694 1043 707 b 870 868 1029 804 957 868 z "
	},
	"": {
		ha: 1671,
		x_min: 0,
		x_max: 0,
		o: "m 1633 894 l 1457 1175 b 1383 1215 1441 1200 1413 1215 l 288 1215 b 214 1175 258 1215 230 1200 l 38 894 b 197 524 -53 748 28 547 b 235 521 210 522 222 521 b 435 611 315 521 386 556 b 635 521 484 556 555 521 b 835 611 715 521 786 556 b 1035 521 884 556 955 521 b 1236 611 1116 521 1187 556 b 1436 521 1285 556 1356 521 b 1473 524 1449 521 1461 522 b 1633 894 1643 546 1724 748 z m 1436 434 b 1356 444 1409 434 1382 438 l 1356 174 l 315 174 l 315 444 b 235 434 289 438 262 434 b 186 437 218 434 202 435 b 141 447 171 439 156 443 l 141 -87 b 228 -174 141 -135 180 -174 l 1443 -174 b 1530 -87 1491 -174 1530 -135 l 1530 447 b 1486 437 1516 443 1501 439 b 1436 434 1469 435 1453 434 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 868 174 l 347 174 l 347 608 l 174 608 l 174 -87 b 260 -174 174 -135 212 -174 l 955 -174 b 1042 -87 1003 -174 1042 -135 l 1042 608 l 868 608 z m 1721 830 l 1490 1177 b 1418 1215 1474 1201 1447 1215 l 318 1215 b 246 1177 289 1215 262 1201 l 15 830 b 87 694 -24 772 18 694 l 1649 694 b 1721 830 1718 694 1760 772 z m 1389 -130 b 1432 -174 1389 -154 1408 -174 l 1519 -174 b 1563 -130 1543 -174 1563 -154 l 1563 608 l 1389 608 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 43 868 l 1172 868 b 1215 911 1196 868 1215 887 l 1215 1085 b 1172 1128 1215 1109 1196 1128 l 43 1128 b 0 1085 19 1128 0 1109 l 0 911 b 43 868 0 887 19 868 z m 1345 651 l 217 651 b 174 608 193 651 174 632 l 174 434 b 217 391 174 410 193 391 l 1345 391 b 1389 434 1369 391 1389 410 l 1389 608 b 1345 651 1389 632 1369 651 z m 1172 174 l 43 174 b 0 130 19 174 0 154 l 0 -43 b 43 -87 0 -67 19 -87 l 1172 -87 b 1215 -43 1196 -87 1215 -67 l 1215 130 b 1172 174 1215 154 1196 174 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 510 644 l 388 521 l 510 398 l 633 521 z m 817 705 l 694 828 l 572 705 l 694 582 z m 572 337 l 694 214 l 817 337 l 694 459 z m 694 1215 b 0 521 311 1215 0 904 b 694 -174 0 137 311 -174 b 1389 521 1078 -174 1389 137 b 694 1215 1389 904 1078 1215 z m 1201 413 l 1170 383 b 1139 383 1162 374 1148 374 l 1063 459 l 940 337 l 1032 245 l 1078 291 b 1109 291 1087 299 1100 299 l 1139 260 b 1139 229 1148 252 1148 238 l 1093 183 l 1139 137 b 1139 107 1148 129 1148 115 l 1109 76 b 1078 76 1100 67 1087 67 l 1032 122 l 986 76 b 955 76 978 67 964 67 l 925 107 b 925 137 916 115 916 129 l 971 183 l 879 275 l 756 153 l 833 76 b 833 45 841 67 841 54 l 802 14 b 771 14 793 6 780 6 l 694 91 l 618 14 b 587 14 609 6 595 6 l 556 45 b 556 76 548 54 548 67 l 633 153 l 510 275 l 418 183 l 464 137 b 464 107 473 129 473 115 l 434 76 b 403 76 425 67 411 67 l 357 122 l 311 76 b 280 76 302 67 289 67 l 250 107 b 250 137 241 115 241 129 l 296 183 l 250 229 b 250 260 241 238 241 251 l 280 291 b 311 291 289 299 302 299 l 357 245 l 449 337 l 326 459 l 249 383 b 219 383 241 374 227 374 l 188 413 b 188 444 180 422 180 436 l 265 521 l 188 598 b 188 628 180 606 180 620 l 219 659 b 249 659 227 667 241 667 l 326 582 l 449 705 l 357 797 l 311 751 b 280 751 302 743 289 743 l 249 782 b 249 812 241 790 241 804 l 295 858 l 249 904 b 249 935 241 913 241 927 l 280 966 b 311 966 289 974 302 974 l 357 920 l 403 966 b 434 966 411 974 425 974 l 464 935 b 464 904 473 927 473 913 l 418 858 l 510 766 l 633 889 l 556 966 b 556 996 548 974 548 988 l 587 1027 b 618 1027 595 1036 609 1036 l 694 950 l 771 1027 b 802 1027 780 1036 793 1036 l 833 997 b 833 966 841 988 841 974 l 756 889 l 879 766 l 971 858 l 925 904 b 925 935 916 913 916 927 l 955 966 b 986 966 964 974 978 974 l 1032 920 l 1078 966 b 1109 966 1087 974 1100 974 l 1139 935 b 1139 904 1148 927 1148 913 l 1093 858 l 1139 812 b 1139 782 1148 804 1148 790 l 1109 751 b 1078 751 1100 743 1087 743 l 1032 797 l 940 705 l 1063 582 l 1139 659 b 1170 659 1148 667 1162 667 l 1201 628 b 1201 598 1209 620 1209 606 l 1124 521 l 1201 444 b 1207 429 1204 441 1207 434 b 1201 413 1207 424 1204 417 z m 756 521 l 879 398 l 1001 521 l 879 644 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1363 633 l 1241 756 b 1179 781 1224 772 1202 781 l 1042 781 l 1042 998 b 911 1128 1042 1070 983 1128 l 477 1128 b 347 998 406 1128 347 1070 l 347 781 l 210 781 b 148 756 187 781 164 772 l 25 633 b 0 572 9 617 0 595 l 0 347 l 347 347 l 347 391 b 391 434 347 415 367 434 l 477 434 b 521 391 501 434 521 415 l 521 347 l 868 347 l 868 391 b 911 434 868 415 887 434 l 998 434 b 1042 391 1022 434 1042 415 l 1042 347 l 1389 347 l 1389 572 b 1363 633 1389 595 1380 617 z m 868 781 l 521 781 l 521 955 l 868 955 z m 1042 217 b 998 174 1042 193 1022 174 l 911 174 b 868 217 887 174 868 193 l 868 260 l 521 260 l 521 217 b 477 174 521 193 501 174 l 391 174 b 347 217 367 174 347 193 l 347 260 l 0 260 l 0 0 b 87 -87 0 -48 39 -87 l 1302 -87 b 1389 0 1350 -87 1389 -48 l 1389 260 l 1042 260 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1712 954 l 1184 1215 b 868 1087 1130 1140 1009 1087 b 552 1215 727 1087 607 1140 l 24 954 b 4 895 2 943 -6 917 l 160 585 b 218 565 170 563 196 554 l 371 640 b 434 601 400 654 434 633 l 434 -87 b 521 -174 434 -135 473 -174 l 1215 -174 b 1302 -87 1263 -174 1302 -135 l 1302 601 b 1364 640 1302 633 1335 655 l 1518 565 b 1576 585 1539 554 1565 563 l 1731 895 b 1712 954 1742 917 1734 943 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 564 955 b 694 1085 636 955 694 1013 b 564 1215 694 1157 636 1215 b 434 1085 492 1215 434 1157 b 564 955 434 1013 492 955 z m 821 550 l 757 582 l 731 662 b 454 868 691 783 580 868 b 201 800 356 868 302 841 b 66 674 142 776 94 731 l 48 637 b 86 521 27 595 44 543 b 202 560 129 500 180 517 l 220 597 b 264 639 229 616 245 631 l 337 668 l 296 503 b 336 344 282 447 297 387 l 499 166 b 539 91 518 145 532 119 l 589 -108 b 694 -171 601 -154 648 -183 b 757 -66 741 -159 769 -112 l 697 176 b 657 251 690 204 676 230 l 533 386 l 580 572 l 595 527 b 681 427 609 484 640 448 l 744 395 b 859 434 786 374 838 391 b 821 550 880 476 863 529 z m 200 169 b 161 110 191 147 178 127 l 26 -26 b 26 -148 -8 -59 -8 -114 b 148 -148 59 -182 114 -182 l 309 13 b 348 71 326 29 339 49 l 384 163 b 256 308 234 326 279 276 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1251 868 l 217 868 b 174 911 193 868 174 887 b 217 955 174 935 193 955 l 1259 955 b 1302 998 1283 955 1302 974 b 1172 1128 1302 1070 1244 1128 l 174 1128 b 0 955 78 1128 0 1051 l 0 87 b 174 -87 0 -9 78 -87 l 1251 -87 b 1389 43 1327 -87 1389 -28 l 1389 738 b 1251 868 1389 810 1327 868 z m 1128 304 b 1042 391 1081 304 1042 343 b 1128 477 1042 439 1081 477 b 1215 391 1176 477 1215 439 b 1128 304 1215 343 1176 304 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 369 564 b 397 628 369 589 380 612 l 313 653 b 284 707 290 660 277 684 b 338 736 291 730 315 743 l 555 671 b 584 617 578 664 591 640 b 543 586 579 598 562 586 b 538 586 541 586 540 586 b 543 564 540 579 543 572 b 456 477 543 516 504 477 b 369 564 408 477 369 516 z m 825 146 b 521 146 749 237 596 237 b 454 202 484 102 417 157 b 672 304 508 267 588 304 b 891 202 757 304 837 267 b 825 146 929 157 861 102 z m 1032 653 l 948 628 b 977 564 965 612 977 589 b 890 477 977 516 938 477 b 803 564 842 477 803 516 b 808 586 803 572 806 579 b 803 586 806 586 805 586 b 762 617 785 586 767 598 b 791 671 755 640 768 664 l 1008 736 b 1062 707 1031 743 1055 730 b 1032 653 1069 684 1056 660 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1519 0 l 1476 0 l 1476 955 l 87 955 l 87 0 l 43 0 b 0 -43 19 0 0 -19 l 0 -130 b 43 -174 0 -154 19 -174 l 477 -174 b 521 -130 501 -174 521 -154 l 521 347 b 781 608 521 491 637 608 b 1042 347 925 608 1042 491 l 1042 -87 l 1042 -130 b 1085 -174 1042 -154 1061 -174 l 1519 -174 b 1563 -130 1543 -174 1563 -154 l 1563 -43 b 1519 0 1563 -19 1543 0 z m 1519 1215 l 43 1215 b 0 1172 19 1215 0 1196 l 0 1085 b 43 1042 0 1061 19 1042 l 1519 1042 b 1563 1085 1543 1042 1563 1061 l 1563 1172 b 1519 1215 1563 1196 1543 1215 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 864 651 l 758 651 b 719 460 754 578 740 512 b 864 651 794 496 849 566 z m 864 738 b 719 929 849 823 794 893 b 758 738 740 877 754 811 z m 608 951 b 544 738 587 931 551 858 l 671 738 b 608 951 664 858 629 931 z m 496 929 b 352 738 421 893 366 823 l 458 738 b 496 929 462 811 475 877 z m 496 460 b 458 651 475 512 462 578 l 352 651 b 496 460 366 566 421 496 z m 671 651 l 544 651 b 608 438 551 531 587 458 b 671 651 629 458 664 531 z m 1215 243 l 1215 1146 b 1146 1215 1215 1189 1189 1215 l 260 1215 b 0 955 113 1215 0 1102 l 0 87 b 260 -174 0 -61 113 -174 l 1146 -174 b 1215 -104 1181 -174 1215 -148 l 1215 -61 b 1189 -9 1215 -43 1207 -26 b 1189 191 1181 35 1181 156 b 1215 243 1207 200 1215 217 z m 608 1042 b 955 694 799 1042 955 886 b 608 347 955 503 799 347 b 260 694 416 347 260 503 b 608 1042 260 886 416 1042 z m 1042 0 l 260 0 b 174 87 208 0 174 35 b 260 174 174 139 217 174 l 1042 174 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 263 232 b 195 264 240 255 252 249 b 126 300 170 271 147 284 l 3 -2 b 45 -62 -9 -31 14 -63 l 188 -56 l 286 -160 b 358 -147 308 -183 346 -176 l 499 200 b 403 174 470 183 437 174 b 263 232 351 174 301 194 z m 1038 -2 l 915 300 b 846 264 895 284 872 271 b 778 232 789 248 802 255 b 638 174 741 194 691 174 b 542 200 604 174 572 183 l 684 -147 b 755 -160 696 -176 734 -183 l 854 -56 l 997 -62 b 1038 -2 1028 -63 1050 -31 z m 713 293 b 819 348 755 335 760 332 b 896 426 856 358 886 388 b 966 550 916 503 911 494 b 995 658 994 578 1005 619 b 995 801 974 735 974 724 b 966 908 1005 839 994 880 b 896 1032 911 965 916 955 b 819 1111 886 1071 856 1101 b 697 1183 743 1132 752 1126 b 591 1212 669 1211 629 1222 b 450 1212 516 1191 526 1191 b 345 1183 413 1222 373 1211 b 223 1111 290 1126 299 1132 b 146 1032 185 1101 156 1071 b 75 908 126 955 131 965 b 47 801 48 880 37 839 b 47 658 67 724 67 735 b 75 550 37 619 48 578 b 146 426 131 494 126 503 b 223 348 156 388 185 358 b 328 293 284 331 288 334 b 463 277 364 256 420 250 b 521 294 478 286 504 294 b 579 277 538 294 564 286 b 713 293 622 250 678 256 z m 265 738 b 521 998 265 882 380 998 b 777 738 662 998 777 882 b 521 477 777 594 662 477 b 265 738 380 477 265 594 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1563 1042 l 557 1042 l 557 1042 b 434 991 517 1042 462 1019 l 25 582 b 25 459 -8 548 -8 493 l 434 51 b 557 0 467 18 511 0 l 1563 0 b 1736 174 1658 0 1736 78 l 1736 868 b 1563 1042 1736 964 1658 1042 z m 1333 352 b 1333 291 1350 336 1350 308 l 1271 230 b 1210 230 1254 213 1227 213 l 1042 398 l 873 230 b 812 230 856 213 829 213 l 751 291 b 751 352 734 308 734 336 l 919 521 l 751 689 b 751 751 734 706 734 734 l 812 812 b 873 812 829 829 856 829 l 1042 644 l 1210 812 b 1271 812 1227 829 1254 829 l 1333 751 b 1333 689 1350 734 1350 706 l 1164 521 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 998 1128 l 738 1128 b 651 1042 690 1128 651 1090 l 651 781 b 738 694 651 733 690 694 l 998 694 b 1085 781 1046 694 1085 733 l 1085 1042 b 998 1128 1085 1090 1046 1128 z m 564 977 l 334 977 b 174 1085 309 1040 246 1085 b 0 911 78 1085 0 1007 b 174 738 0 816 78 738 b 334 846 246 738 309 783 l 551 846 b 253 434 400 758 291 609 l 387 434 b 580 710 418 547 487 643 b 564 781 570 731 564 756 z m 434 347 l 174 347 b 87 260 126 347 87 308 l 87 0 b 174 -87 87 -48 126 -87 l 434 -87 b 521 0 482 -87 521 -48 l 521 260 b 434 347 521 308 482 347 z m 1563 1085 b 1402 977 1490 1085 1428 1040 l 1172 977 l 1172 781 b 1156 710 1172 756 1166 731 b 1349 434 1249 643 1318 547 l 1483 434 b 1185 846 1446 609 1336 758 l 1402 846 b 1563 738 1428 783 1490 738 b 1736 911 1658 738 1736 816 b 1563 1085 1736 1007 1658 1085 z m 1563 347 l 1302 347 b 1215 260 1254 347 1215 308 l 1215 0 b 1302 -87 1215 -48 1254 -87 l 1563 -87 b 1649 0 1610 -87 1649 -48 l 1649 260 b 1563 347 1649 308 1610 347 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 821 -174 b 972 -85 883 -174 941 -139 b 1042 174 1016 -9 1042 79 b 967 441 1042 272 1014 363 l 1085 559 l 1111 534 b 1172 534 1127 517 1155 517 l 1203 564 b 1203 626 1220 581 1220 609 l 1060 769 b 998 769 1043 786 1015 786 l 968 738 b 968 676 951 721 951 693 l 993 651 l 886 544 b 781 624 855 575 820 602 l 781 1042 l 825 1042 b 868 1086 848 1042 868 1062 l 868 1172 b 825 1215 868 1196 849 1215 l 217 1215 b 174 1171 193 1215 174 1195 l 174 1085 b 217 1042 174 1061 193 1042 l 260 1042 l 260 624 b 0 174 105 534 0 366 b 69 -85 0 79 25 -9 b 221 -174 101 -139 158 -174 z m 326 512 l 391 549 l 391 1042 l 651 1042 l 651 549 l 716 512 b 869 347 783 473 836 415 l 172 347 b 326 512 206 415 258 473 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 955 1215 l 87 1215 b 0 1128 39 1215 0 1176 l 0 521 l 1042 521 l 1042 1128 b 955 1215 1042 1176 1003 1215 z m 0 347 b 174 174 0 251 78 174 l 347 174 l 347 0 b 521 -174 347 -96 425 -174 b 694 0 617 -174 694 -96 l 694 174 l 868 174 b 1042 347 964 174 1042 251 l 1042 434 l 0 434 z m 521 65 b 586 0 557 65 586 36 b 521 -65 586 -36 557 -65 b 456 0 485 -65 456 -36 b 521 65 456 36 485 65 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1324 868 l 1302 868 l 1302 998 b 694 1215 1302 1120 1033 1215 b 87 998 356 1215 87 1120 l 87 868 l 65 868 b 0 803 29 868 0 839 l 0 586 b 65 521 0 550 29 521 l 87 521 l 87 87 b 174 0 87 39 126 0 l 174 -87 b 260 -174 174 -135 212 -174 l 347 -174 b 434 -87 395 -174 434 -135 l 434 0 l 955 0 l 955 -87 b 1042 -174 955 -135 994 -174 l 1128 -174 b 1215 -87 1176 -174 1215 -135 l 1215 0 l 1233 0 b 1302 69 1276 0 1302 35 l 1302 521 l 1324 521 b 1389 586 1360 521 1389 550 l 1389 803 b 1324 868 1389 839 1360 868 z m 434 1020 b 456 1042 434 1032 444 1042 l 933 1042 b 955 1020 945 1042 955 1032 l 955 977 b 933 955 955 965 945 955 l 456 955 b 434 977 444 955 434 965 z m 304 130 b 217 217 256 130 217 169 b 304 304 217 265 256 304 b 391 217 352 304 391 265 b 304 130 391 169 352 130 z m 651 434 l 347 434 b 260 521 299 434 260 473 l 260 781 b 347 868 260 829 299 868 l 651 868 z m 738 434 l 738 868 l 1042 868 b 1128 781 1090 868 1128 829 l 1128 521 b 1042 434 1128 473 1090 434 z m 1085 130 b 998 217 1037 130 998 169 b 1085 304 998 265 1037 304 b 1172 217 1133 304 1172 265 b 1085 130 1172 169 1133 130 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1366 238 b 1157 308 1362 240 1278 284 b 1388 757 1332 512 1385 747 b 1376 797 1391 772 1387 787 b 1345 810 1368 806 1357 810 b 1336 809 1342 810 1339 810 b 901 593 1326 807 1101 756 b 901 605 901 597 901 601 b 731 1195 901 927 738 1184 b 694 1215 724 1206 708 1215 l 694 1215 b 658 1195 680 1215 666 1208 b 488 605 651 1184 488 927 b 488 593 488 601 488 597 b 53 809 287 756 63 807 b 43 810 50 810 46 810 b 13 797 32 810 21 806 b 0 767 6 790 0 777 b 1 757 0 764 0 760 b 231 308 3 747 57 512 b 23 238 111 284 27 240 b 0 200 10 231 0 214 b 23 161 0 185 10 168 b 385 75 30 158 188 75 b 433 76 402 75 418 76 b 387 -30 402 16 388 -28 b 398 -74 382 -46 386 -63 b 428 -87 405 -81 418 -87 b 441 -85 432 -87 438 -86 b 651 22 446 -83 544 -52 l 651 -152 b 673 -174 651 -164 661 -174 l 716 -174 b 738 -152 728 -174 738 -164 l 738 22 b 948 -85 845 -52 943 -83 b 960 -87 951 -86 957 -87 b 991 -74 970 -87 984 -81 b 1002 -30 1003 -63 1007 -46 b 956 76 1001 -28 987 16 b 1004 75 971 75 987 75 b 1366 161 1201 75 1359 158 b 1389 200 1380 169 1389 184 b 1366 238 1389 216 1380 231 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1370 741 l 1262 849 b 1171 849 1237 874 1196 874 l 521 199 l 219 501 b 127 501 193 527 152 527 l 19 394 b 19 301 -7 368 -7 327 l 475 -155 b 567 -155 500 -180 541 -180 l 1370 649 b 1370 741 1395 675 1395 716 z m 490 454 b 551 454 507 436 535 436 l 1116 1018 b 1116 1080 1133 1035 1133 1063 l 993 1203 b 932 1203 976 1220 949 1220 l 521 792 l 371 942 b 309 942 354 959 326 959 l 186 819 b 186 758 170 802 170 775 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 803 -43 l 651 -43 l 651 296 l 1109 754 b 1062 868 1151 796 1121 868 l 67 868 b 20 754 7 868 -22 796 l 477 296 l 477 -43 l 326 -43 b 217 -152 266 -43 217 -92 b 239 -174 217 -164 227 -174 l 890 -174 b 911 -152 902 -174 911 -164 b 803 -43 911 -92 863 -43 z m 1172 1215 b 805 955 1002 1215 859 1106 l 948 955 b 1172 1085 993 1032 1076 1085 b 1432 825 1315 1085 1432 968 b 1172 564 1432 681 1315 564 b 1065 588 1134 564 1098 573 l 970 492 b 1172 434 1029 456 1098 434 b 1563 825 1388 434 1563 609 b 1172 1215 1563 1040 1388 1215 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 781 861 l 781 911 l 825 911 b 868 955 849 911 868 931 l 868 998 b 825 1042 868 1022 849 1042 l 564 1042 b 521 998 540 1042 521 1022 l 521 955 b 564 911 521 931 540 911 l 608 911 l 608 861 b 87 260 313 819 87 566 l 1302 260 b 781 861 1302 566 1076 819 z m 1345 174 l 43 174 b 0 130 19 174 0 154 l 0 43 b 43 0 0 19 19 0 l 1345 0 b 1389 43 1369 0 1389 19 l 1389 130 b 1345 174 1389 154 1369 174 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1384 524 l 1352 731 b 1251 929 1342 795 1296 883 l 1102 1077 b 905 1178 1049 1131 980 1166 l 697 1211 b 479 1176 622 1223 546 1211 l 291 1081 b 134 924 234 1052 164 981 l 39 737 b 0 574 18 695 0 621 b 4 518 0 558 2 533 l 37 311 b 138 113 47 247 92 158 l 286 -36 b 484 -136 332 -81 420 -126 l 692 -169 b 910 -135 766 -181 843 -169 l 1098 -39 b 1254 118 1155 -10 1225 60 l 1350 304 b 1384 524 1384 372 1396 449 z m 477 217 b 391 304 429 217 391 256 b 477 391 391 352 429 391 b 564 304 525 391 564 352 b 477 217 564 256 525 217 z m 564 651 b 477 738 516 651 477 690 b 564 825 477 786 516 825 b 651 738 612 825 651 786 b 564 651 651 690 612 651 z m 998 304 b 911 391 950 304 911 343 b 998 477 911 439 950 477 b 1085 391 1046 477 1085 439 b 998 304 1085 343 1046 304 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1385 521 b 1042 868 1195 524 1042 678 b 695 1211 851 868 697 1021 b 479 1176 621 1222 545 1210 l 291 1081 b 134 924 234 1052 164 981 l 39 737 b 0 574 18 695 0 622 b 4 518 0 558 2 533 l 37 311 b 138 113 47 247 92 158 l 287 -36 b 484 -136 332 -81 420 -126 l 692 -169 b 910 -135 767 -181 843 -169 l 1098 -39 b 1255 118 1155 -10 1225 60 l 1350 305 b 1385 521 1384 371 1396 447 z m 477 217 b 391 304 429 217 391 256 b 477 391 391 352 429 391 b 564 304 525 391 564 352 b 477 217 564 256 525 217 z m 564 651 b 477 738 516 651 477 690 b 564 825 477 786 516 825 b 651 738 612 825 651 786 b 564 651 651 690 612 651 z m 998 304 b 911 391 950 304 911 343 b 998 477 911 439 950 477 b 1085 391 1046 477 1085 439 b 998 304 1085 343 1046 304 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1324 260 l 1215 260 l 1215 955 b 1128 1042 1215 1003 1176 1042 l 521 1042 l 521 781 l 955 781 l 955 -109 b 1020 -174 955 -144 984 -174 l 1150 -174 b 1215 -109 1186 -174 1215 -144 l 1215 0 l 1324 0 b 1389 65 1360 0 1389 29 l 1389 195 b 1324 260 1389 231 1360 260 z m 434 1150 b 369 1215 434 1186 405 1215 l 239 1215 b 174 1150 203 1215 174 1186 l 174 1042 l 65 1042 b 0 977 29 1042 0 1013 l 0 846 b 65 781 0 810 29 781 l 174 781 l 174 87 b 260 0 174 39 212 0 l 868 0 l 868 260 l 434 260 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1649 955 l 87 955 b 0 868 39 955 0 916 l 0 174 b 87 87 0 126 39 87 l 1649 87 b 1736 174 1697 87 1736 126 l 1736 868 b 1649 955 1736 916 1697 955 z m 825 260 b 803 239 825 248 815 239 l 195 239 b 174 260 183 239 174 248 l 174 282 b 195 304 174 294 183 304 l 803 304 b 825 282 815 304 825 294 z m 195 434 l 195 477 b 217 499 195 489 205 499 l 260 499 b 282 477 272 499 282 489 l 282 434 b 260 412 282 422 272 412 l 217 412 b 195 434 205 412 195 422 z m 369 434 l 369 477 b 391 499 369 489 379 499 l 434 499 b 456 477 446 499 456 489 l 456 434 b 434 412 456 422 446 412 l 391 412 b 369 434 379 412 369 422 z m 543 434 l 543 477 b 564 499 543 489 552 499 l 608 499 b 629 477 620 499 629 489 l 629 434 b 608 412 629 422 620 412 l 564 412 b 543 434 552 412 543 422 z m 716 434 l 716 477 b 738 499 716 489 726 499 l 781 499 b 803 477 793 499 803 489 l 803 434 b 781 412 803 422 793 412 l 738 412 b 716 434 726 412 716 422 z m 825 608 b 781 564 825 584 805 564 l 217 564 b 174 608 193 564 174 584 l 174 738 b 217 781 174 762 193 781 l 781 781 b 825 738 805 781 825 762 z m 1563 260 b 1541 239 1563 248 1553 239 l 933 239 b 911 260 921 239 911 248 l 911 282 b 933 304 911 294 921 304 l 1541 304 b 1563 282 1553 304 1563 294 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 412 633 l 334 555 b 273 617 294 515 232 576 l 351 694 l 273 772 b 334 834 232 813 294 874 l 412 756 l 490 834 b 551 772 531 874 592 813 l 474 694 l 551 617 b 490 555 593 575 530 515 z m 673 87 b 499 260 577 87 499 165 b 673 434 499 356 577 434 b 846 260 768 434 846 356 b 673 87 846 165 768 87 z m 1072 617 b 1011 555 1114 575 1051 515 l 933 633 l 855 555 b 794 617 815 515 753 576 l 872 694 l 794 772 b 855 834 753 813 815 874 l 933 756 l 1011 834 b 1072 772 1052 874 1113 813 l 994 694 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1240 281 b 1016 148 1172 226 1097 180 l 1163 -108 l 1310 -170 b 1370 -135 1336 -181 1366 -164 l 1389 23 z m 1355 537 b 1339 598 1368 558 1360 586 l 1262 640 b 1205 625 1242 652 1217 644 b 694 347 1093 453 901 347 b 506 379 630 347 566 359 l 689 695 b 694 694 691 695 693 694 b 700 695 696 694 698 695 l 839 455 b 1063 588 923 479 1000 525 l 922 831 b 955 955 942 868 955 910 b 694 1215 955 1099 838 1215 b 434 955 551 1215 434 1099 b 467 831 434 910 447 868 l 281 510 b 190 616 247 542 216 577 b 132 630 177 635 152 642 l 57 587 b 42 526 36 574 28 546 b 193 358 84 462 136 406 l 0 23 l 19 -135 b 79 -170 23 -164 53 -181 l 226 -108 l 418 225 b 694 174 505 192 598 174 b 1355 537 963 174 1212 312 z m 694 1042 b 781 955 742 1042 781 1003 b 694 868 781 907 742 868 b 608 955 647 868 608 907 b 694 1042 608 1003 647 1042 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1243 887 l 1520 1054 b 1550 1173 1561 1079 1575 1132 b 1431 1203 1525 1214 1472 1228 l 995 941 b 781 955 936 949 841 955 b 0 608 350 955 0 799 l 0 171 b 174 -47 0 89 65 13 l 174 215 b 260 302 174 263 213 302 b 347 215 308 302 347 263 l 347 -117 b 694 -174 449 -147 567 -167 l 694 128 b 781 215 694 176 734 215 b 868 128 829 215 868 176 l 868 -174 b 1215 -117 996 -167 1114 -148 l 1215 215 b 1302 302 1215 262 1254 302 b 1389 215 1350 302 1389 262 l 1389 -47 b 1563 171 1497 13 1563 89 l 1563 608 b 1243 887 1562 723 1436 824 z m 781 391 b 130 608 422 391 130 488 b 781 825 130 727 422 825 b 800 824 788 825 794 824 l 563 682 b 533 563 522 657 509 604 b 652 533 559 520 613 510 l 1094 798 b 1432 608 1295 761 1432 690 b 781 391 1432 488 1141 391 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 781 1128 b 0 781 350 1128 0 973 l 0 260 b 781 -87 0 69 350 -87 b 1563 260 1213 -87 1563 69 l 1563 781 b 781 1128 1563 973 1213 1128 z m 556 699 b 479 589 544 654 517 616 b 130 781 272 625 130 698 b 467 971 130 863 266 934 l 536 851 b 556 699 563 805 570 751 z m 781 564 b 616 572 724 564 669 567 b 781 694 638 642 703 694 b 946 572 859 694 925 642 b 781 564 894 567 838 564 z m 955 955 b 781 781 955 859 877 781 b 608 955 686 781 608 859 l 608 990 b 781 998 663 995 721 998 b 955 990 841 998 900 995 z m 1082 589 b 1005 699 1045 616 1017 654 b 1025 851 991 751 998 805 l 1095 971 b 1432 781 1296 935 1432 863 b 1082 589 1432 698 1290 625 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1389 1215 b 193 435 1248 1206 262 1112 b 176 217 184 363 179 290 l 662 702 b 723 702 679 719 706 719 b 723 641 740 685 740 658 l 19 -62 b 19 -155 -6 -88 -6 -129 b 111 -155 45 -180 86 -180 l 266 0 b 608 20 380 1 495 7 b 968 174 753 35 871 92 l 694 174 l 1092 306 b 1175 434 1122 347 1150 390 l 954 434 l 1243 578 b 1389 1215 1357 855 1384 1144 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 608 846 l 608 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 -109 b 65 -174 0 -145 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -145 l 1042 781 l 673 781 b 608 846 637 781 608 811 z m 174 1020 b 195 1042 174 1032 183 1042 l 412 1042 b 434 1020 424 1042 434 1032 l 434 977 b 412 955 434 965 424 955 l 195 955 b 174 977 183 955 174 965 z m 174 846 b 195 868 174 858 183 868 l 412 868 b 434 846 424 868 434 858 l 434 803 b 412 781 434 791 424 781 l 195 781 b 174 803 183 781 174 791 z m 697 174 l 825 174 b 868 130 849 174 868 154 b 825 87 868 106 849 87 l 697 87 b 592 152 652 87 612 112 b 564 169 584 168 570 169 b 537 152 559 169 545 168 l 516 111 b 477 87 510 98 492 87 l 477 87 b 474 87 476 87 475 87 b 436 117 457 88 442 100 l 391 253 l 362 167 b 251 87 346 119 301 87 l 217 87 b 174 130 193 87 174 106 b 217 174 174 154 193 174 l 251 174 b 279 194 264 174 275 182 l 329 343 b 391 387 338 369 363 387 b 452 343 419 387 443 369 l 490 230 b 669 191 544 274 637 256 b 697 174 675 180 685 174 z m 1023 930 l 757 1196 b 711 1215 745 1208 728 1215 l 694 1215 l 694 868 l 1042 868 l 1042 885 b 1023 930 1042 902 1035 918 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 608 846 l 608 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 -109 b 65 -174 0 -145 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -145 l 1042 781 l 673 781 b 608 846 637 781 608 811 z m 815 273 l 553 13 b 488 13 535 -5 506 -5 l 227 273 b 257 347 199 300 218 347 l 434 347 l 434 564 b 477 608 434 588 453 608 l 564 608 b 608 564 588 608 608 588 l 608 347 l 784 347 b 815 273 823 347 843 300 z m 1023 930 l 757 1196 b 711 1215 745 1208 728 1215 l 694 1215 l 694 868 l 1042 868 l 1042 885 b 1023 930 1042 902 1035 918 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1042 885 b 1023 930 1042 902 1035 918 l 757 1196 b 711 1215 745 1208 728 1215 l 694 1215 l 694 868 l 1042 868 z m 1549 380 l 1289 641 b 1215 611 1262 669 1215 649 l 1215 434 l 1041 434 l 1041 260 l 1215 260 l 1215 84 b 1289 53 1215 45 1262 26 l 1549 315 b 1549 380 1567 333 1567 362 z m 521 304 l 521 391 b 564 434 521 414 540 434 l 1042 434 l 1042 781 l 673 781 b 608 846 637 781 608 811 l 608 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 -109 b 65 -174 0 -145 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -145 l 1042 260 l 564 260 b 521 304 540 260 521 280 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 43 434 b 0 391 20 434 0 414 l 0 304 b 43 260 0 280 20 260 l 347 260 l 347 434 z m 1370 930 l 1104 1196 b 1058 1215 1092 1208 1076 1215 l 1042 1215 l 1042 868 l 1389 868 l 1389 885 b 1370 930 1389 902 1382 918 z m 955 846 l 955 1215 l 412 1215 b 347 1150 376 1215 347 1186 l 347 434 l 694 434 l 694 611 b 769 642 694 650 741 669 l 1028 380 b 1028 315 1046 362 1046 333 l 768 53 b 694 84 741 26 694 45 l 694 260 l 347 260 l 347 -109 b 412 -174 347 -145 376 -174 l 1324 -174 b 1389 -109 1360 -174 1389 -145 l 1389 781 l 1020 781 b 955 846 984 781 955 811 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 781 521 l 260 521 l 260 347 l 781 347 z m 1023 930 l 757 1196 b 711 1215 745 1208 728 1215 l 694 1215 l 694 868 l 1042 868 l 1042 885 b 1023 930 1042 902 1035 918 z m 608 846 l 608 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 -109 b 65 -174 0 -145 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -145 l 1042 781 l 673 781 b 608 846 637 781 608 811 z m 174 1020 b 195 1042 174 1032 183 1042 l 412 1042 b 434 1020 424 1042 434 1032 l 434 977 b 412 955 434 965 424 955 l 195 955 b 174 977 183 955 174 965 z m 174 846 b 195 868 174 858 183 868 l 412 868 b 434 846 424 868 434 858 l 434 803 b 412 781 434 791 424 781 l 195 781 b 174 803 183 781 174 791 z m 868 22 b 846 0 868 10 858 0 l 629 0 b 608 22 617 0 608 10 l 608 65 b 629 87 608 77 617 87 l 846 87 b 868 65 858 87 868 77 z m 868 564 l 868 304 b 825 260 868 280 849 260 l 217 260 b 174 304 193 260 174 280 l 174 564 b 217 608 174 588 193 608 l 825 608 b 868 564 849 608 868 588 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 1023 930 l 757 1196 b 711 1215 745 1208 728 1215 l 694 1215 l 694 868 l 1042 868 l 1042 885 b 1023 930 1042 902 1035 918 z m 608 846 l 608 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 -109 b 65 -174 0 -145 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -145 l 1042 781 l 673 781 b 608 846 637 781 608 811 z m 174 1020 b 195 1042 174 1032 183 1042 l 412 1042 b 434 1020 424 1042 434 1032 l 434 977 b 412 955 434 965 424 955 l 195 955 b 174 977 183 955 174 965 z m 174 803 l 174 846 b 195 868 174 858 183 868 l 412 868 b 434 846 424 868 434 858 l 434 803 b 412 781 434 791 424 781 l 195 781 b 174 803 183 781 174 791 z m 564 87 l 564 22 b 543 0 564 10 555 0 l 499 0 b 477 22 487 0 477 10 l 477 88 b 392 118 447 89 417 100 b 391 151 382 126 381 142 l 423 182 b 450 184 430 189 441 189 b 485 174 461 177 473 174 l 561 174 b 593 209 579 174 593 190 b 569 244 593 226 583 240 l 447 281 b 362 398 397 296 362 344 b 477 521 362 465 413 519 l 477 586 b 499 608 477 598 487 608 l 543 608 b 564 586 554 608 564 598 l 564 520 b 649 489 595 518 625 508 b 651 456 660 481 660 465 l 619 426 b 591 424 611 419 600 418 b 557 434 581 431 569 434 l 480 434 b 448 398 463 434 448 418 b 472 364 448 382 458 368 l 594 327 b 680 209 645 312 680 264 b 564 87 680 143 628 89 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 608 846 l 608 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 -109 b 65 -174 0 -145 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -145 l 1042 781 l 673 781 b 608 846 637 781 608 811 z m 794 359 l 824 329 b 824 267 841 312 841 284 l 743 186 l 825 105 b 825 43 842 88 842 60 l 794 13 b 733 13 777 -4 750 -4 l 651 94 l 569 13 b 508 13 553 -4 525 -4 l 477 43 b 477 105 460 60 460 88 l 559 186 l 398 347 l 347 347 l 347 217 b 304 174 347 193 328 174 l 260 174 b 217 217 236 174 217 193 l 217 651 b 260 694 217 675 236 694 l 477 694 b 651 521 573 694 651 617 b 560 369 651 455 614 399 l 651 278 l 732 359 b 794 359 749 376 777 376 z m 477 477 b 521 521 501 477 521 497 b 477 564 521 545 501 564 l 347 564 l 347 477 z m 1042 885 b 1023 930 1042 902 1035 918 l 757 1196 b 711 1215 745 1208 728 1215 l 694 1215 l 694 868 l 1042 868 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 592 65 b 564 82 584 81 570 82 b 537 66 559 82 545 81 l 516 24 b 436 30 499 -11 448 -7 l 391 167 l 362 80 b 251 0 346 32 301 0 l 217 0 b 174 43 193 0 174 19 b 217 87 174 67 193 87 l 251 87 b 279 108 264 87 275 95 l 329 256 b 391 300 338 282 363 300 b 452 256 419 300 443 282 l 490 143 b 669 104 544 187 637 169 b 694 87 674 94 684 88 l 694 310 l 1042 655 l 1042 781 l 673 781 b 608 846 637 781 608 811 l 608 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 -109 b 65 -174 0 -145 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -145 l 1042 0 l 694 0 b 592 65 651 1 611 25 z m 1042 885 b 1023 930 1042 902 1035 918 l 757 1196 b 711 1215 745 1208 728 1215 l 694 1215 l 694 868 l 1042 868 z m 781 274 l 781 87 l 968 87 l 1407 528 l 1223 713 z m 1542 761 l 1456 848 b 1358 848 1429 875 1385 875 l 1284 774 l 1468 590 l 1542 664 b 1542 761 1569 691 1569 734 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 608 846 l 608 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 -109 b 65 -174 0 -145 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -145 l 1042 781 l 673 781 b 608 846 637 781 608 811 z m 784 260 l 608 260 l 608 43 b 564 0 608 19 588 0 l 477 0 b 434 43 453 0 434 19 l 434 260 l 257 260 b 227 335 218 260 199 307 l 488 594 b 553 594 506 612 535 612 l 815 335 b 784 260 843 307 823 260 z m 1023 930 l 757 1196 b 711 1215 745 1208 728 1215 l 694 1215 l 694 868 l 1042 868 l 1042 885 b 1023 930 1042 902 1035 918 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1363 626 l 800 1190 b 739 1215 783 1207 761 1215 b 677 1190 716 1215 694 1207 l 456 969 l 222 1202 b 161 1202 205 1219 178 1219 l 100 1141 b 100 1080 83 1124 83 1097 l 333 846 l 76 589 b 76 221 -25 487 -25 322 l 394 -97 b 578 -174 445 -148 512 -174 b 762 -97 645 -174 712 -148 l 1363 504 b 1363 626 1397 538 1397 593 z m 1048 434 l 179 434 b 199 466 183 444 189 456 l 235 502 l 456 723 l 615 564 b 738 564 649 530 704 530 b 738 687 772 598 772 653 l 579 846 l 739 1006 l 1179 565 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1389 347 b 1215 0 1389 347 1215 96 b 1389 -174 1215 -96 1293 -174 b 1563 0 1485 -174 1563 -96 b 1389 347 1563 96 1389 347 z m 1363 626 l 800 1190 b 739 1215 783 1207 761 1215 b 677 1190 716 1215 694 1207 l 456 969 l 222 1202 b 161 1202 205 1219 178 1219 l 100 1141 b 100 1080 83 1124 83 1097 l 333 846 l 76 589 b 76 221 -25 487 -25 322 l 394 -97 b 578 -174 445 -148 512 -174 b 762 -97 645 -174 712 -148 l 1363 504 b 1363 626 1397 538 1397 593 z m 1048 434 l 179 434 b 199 466 183 444 189 456 l 235 502 l 456 723 l 615 564 b 738 564 649 530 704 530 b 738 687 772 598 772 653 l 579 846 l 739 1006 l 1179 565 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 695 548 b 630 483 659 548 630 519 b 555 -91 633 287 608 98 b 617 -174 547 -117 560 -174 b 680 -126 646 -174 672 -155 b 760 483 717 4 764 214 b 695 548 760 519 731 548 z m 692 770 b 413 496 526 770 410 642 b 376 109 415 366 402 235 b 427 32 369 74 392 39 b 504 82 462 25 497 47 b 543 498 532 218 545 358 b 691 640 542 568 597 640 b 847 489 775 638 846 571 b 818 100 849 359 839 228 b 872 25 812 64 836 31 b 946 79 926 16 944 66 b 977 491 969 214 979 353 b 692 770 975 642 847 767 z m 392 823 b 301 814 364 846 323 842 b 195 510 231 729 194 621 b 171 206 197 408 189 306 b 223 131 164 171 188 137 b 299 183 278 121 297 171 b 326 512 318 291 327 402 b 402 732 324 593 351 669 b 392 823 424 760 420 801 z m 689 993 b 565 979 647 994 605 989 b 517 901 530 971 508 936 b 595 852 525 866 560 844 b 687 862 625 859 656 863 b 1064 497 892 859 1061 695 b 1049 189 1066 394 1060 291 b 1106 117 1045 153 1070 121 b 1178 174 1151 112 1175 149 b 1194 460 1187 253 1194 381 b 1194 499 1194 471 1194 488 b 689 993 1190 767 963 988 z m 1373 663 b 1295 713 1365 698 1331 721 b 1246 635 1260 705 1238 670 b 1259 468 1259 577 1259 534 b 1323 402 1258 432 1287 403 l 1324 402 b 1389 467 1359 402 1388 431 b 1373 663 1389 527 1390 586 z m 1264 909 b 686 1215 1132 1097 916 1212 b 191 1017 496 1218 321 1148 b 0 546 65 891 -2 724 l 0 488 b 63 421 -1 452 27 422 b 65 421 64 421 65 421 b 130 484 100 421 129 449 l 131 548 b 283 925 128 691 183 825 b 684 1085 388 1031 531 1087 b 1158 834 872 1082 1049 989 b 1248 818 1178 805 1219 798 b 1264 909 1278 839 1285 880 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 887 955 b 311 679 643 955 430 806 l 75 858 b 1 819 42 883 -7 857 l 67 521 l 1 223 b 75 183 -7 185 42 158 l 311 362 b 887 87 430 235 643 87 b 1563 521 1260 87 1563 434 b 887 955 1563 608 1260 955 z m 1124 456 b 1059 521 1089 456 1059 485 b 1124 586 1059 557 1089 586 b 1190 521 1160 586 1190 557 b 1124 456 1190 485 1160 456 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 933 673 b 868 608 897 673 868 644 b 933 543 868 572 897 543 b 998 608 969 543 998 572 b 933 673 998 644 969 673 z m 412 673 b 347 608 376 673 347 644 b 412 543 347 572 376 543 b 477 608 448 543 477 572 b 412 673 477 644 448 673 z m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 217 608 b 412 803 217 716 304 803 b 608 608 520 803 608 716 b 412 412 608 500 520 412 b 217 608 304 412 217 500 z m 846 130 l 499 130 b 499 217 442 130 442 217 l 846 217 b 846 130 904 217 904 130 z m 933 412 b 738 608 825 412 738 500 b 933 803 738 716 825 803 b 1128 608 1041 803 1128 716 b 933 412 1128 500 1041 412 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 369 651 b 456 738 369 699 408 738 b 543 651 504 738 543 699 b 456 564 543 603 504 564 b 369 651 408 564 369 603 z m 877 154 b 673 195 792 180 716 195 b 468 154 630 195 553 180 b 413 203 437 144 407 171 b 673 369 432 311 576 369 b 933 203 770 369 914 311 b 877 154 938 171 908 144 z m 890 564 b 803 651 842 564 803 603 b 890 738 803 699 842 738 b 977 651 938 738 977 699 b 890 564 977 603 938 564 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1362 1059 b 1297 1215 1420 1117 1379 1215 l 92 1215 b 27 1059 10 1215 -31 1117 l 608 478 l 608 -43 l 456 -43 b 347 -152 396 -43 347 -92 b 369 -174 347 -164 357 -174 l 1020 -174 b 1042 -152 1032 -174 1042 -164 b 933 -43 1042 -92 993 -43 l 781 -43 l 781 478 z m 1204 1085 l 1074 955 l 315 955 l 185 1085 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 1107 609 l 1107 590 b 1084 553 1107 574 1098 560 l 1042 532 b 1023 527 1037 529 1028 527 b 1000 534 1016 527 1005 530 l 950 567 b 927 574 944 571 934 574 b 914 572 923 574 917 573 l 906 570 b 885 507 880 561 869 530 l 921 453 b 956 434 928 441 942 434 l 978 434 b 1020 392 1001 434 1020 415 l 1020 361 b 1012 336 1020 352 1017 343 l 961 268 b 953 251 957 263 954 257 l 941 189 b 928 166 940 180 935 172 b 861 87 907 146 877 111 l 825 33 b 763 0 813 15 785 0 b 695 42 734 0 708 16 b 673 137 683 67 673 109 l 673 137 l 673 218 b 631 260 673 242 654 260 l 560 260 b 456 304 521 260 484 276 b 412 409 432 328 412 375 l 412 447 b 472 565 412 493 434 537 l 546 621 b 635 651 568 638 608 651 l 638 651 b 704 635 661 651 683 646 l 744 615 b 776 613 754 610 765 610 l 904 656 b 933 696 922 662 933 678 b 891 738 933 719 914 738 l 864 738 b 834 750 853 738 842 742 l 815 769 b 785 781 808 776 795 781 l 785 781 l 541 781 b 499 823 518 781 499 800 l 499 835 b 531 876 499 855 512 871 l 570 886 b 595 903 580 888 589 895 l 617 936 b 652 955 625 948 638 955 l 718 955 b 760 997 741 955 760 974 l 760 1056 b 1199 651 974 1021 1147 860 l 1149 651 b 1107 609 1126 651 1107 632 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 896 224 b 865 193 885 213 874 202 b 849 164 857 185 851 175 b 836 118 844 149 841 133 l 788 -9 b 673 -22 751 -17 712 -22 l 673 53 b 611 192 677 87 652 151 b 586 253 595 208 586 230 l 586 340 b 541 416 586 371 569 400 b 409 487 502 437 447 467 b 323 546 378 502 349 522 l 321 547 b 272 604 305 561 284 587 b 178 742 246 641 205 703 b 458 1019 234 866 334 965 l 523 986 b 586 1025 552 972 586 993 l 586 1056 b 652 1062 608 1059 630 1061 l 729 986 b 729 924 746 969 746 941 l 716 911 l 688 883 b 688 853 680 875 680 861 l 701 840 b 701 809 709 832 709 818 l 679 788 b 664 781 676 784 669 781 l 664 781 l 639 781 b 624 775 634 781 628 779 l 597 749 b 591 733 594 745 591 738 b 593 724 591 731 592 726 l 635 639 b 616 608 643 625 632 608 l 601 608 b 586 613 595 608 590 610 l 561 635 b 533 645 554 641 542 645 b 519 643 529 645 523 644 l 435 615 b 412 584 422 611 412 597 l 412 584 b 430 555 412 572 419 561 l 460 540 b 543 521 486 528 514 521 b 629 434 571 521 604 447 l 810 434 b 872 409 833 434 856 425 l 909 372 b 933 313 922 358 933 332 l 933 313 b 896 224 933 284 917 244 z m 1131 471 b 1093 498 1115 475 1102 485 l 1044 572 b 1033 608 1038 581 1033 597 b 1044 644 1033 619 1038 635 l 1097 723 b 1122 746 1103 733 1112 741 l 1157 763 b 1215 521 1194 690 1215 608 b 1210 452 1215 497 1213 474 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 642 542 b 605 521 634 529 620 521 l 604 521 b 573 508 592 521 581 516 l 558 493 b 558 462 549 484 549 471 l 573 447 b 586 416 581 439 586 428 l 586 391 b 543 347 586 367 567 347 l 526 347 b 487 371 510 347 495 356 l 426 494 b 391 500 419 507 402 510 l 338 447 b 308 434 330 439 319 434 l 138 434 b 130 521 133 462 130 491 b 673 1063 130 820 374 1063 b 840 1037 731 1063 787 1054 l 704 932 b 706 895 690 923 691 902 l 736 880 b 760 841 750 873 760 858 l 760 629 b 738 608 760 617 750 608 l 730 608 b 710 620 721 608 714 612 b 689 620 706 628 694 629 z m 1107 243 b 1094 212 1107 231 1102 220 l 1068 186 b 1037 174 1060 178 1049 174 l 996 174 b 966 186 985 174 974 178 l 930 222 b 879 243 919 233 896 243 b 861 241 874 243 866 242 l 804 226 b 793 225 800 225 797 225 l 765 225 b 734 238 754 225 743 230 l 702 270 b 696 285 699 274 696 280 l 696 285 l 696 313 b 709 333 696 322 701 330 l 816 376 b 831 384 821 378 827 381 l 896 430 b 908 434 899 432 904 434 l 908 434 l 941 434 b 961 421 950 434 958 429 l 976 386 b 986 379 977 382 982 379 l 996 379 b 1006 387 1001 379 1005 382 l 1018 426 b 1028 434 1019 431 1023 434 l 1045 434 b 1055 423 1051 434 1055 429 l 1055 388 b 1062 373 1055 382 1058 377 l 1094 340 b 1107 310 1102 332 1107 321 l 1107 243 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 391 130 l 369 130 b 282 217 321 130 282 169 l 282 239 l 391 239 z m 391 282 l 282 282 l 282 304 b 369 391 282 352 321 391 l 391 391 z m 369 651 b 456 738 369 699 408 738 b 543 651 504 738 543 699 b 456 564 543 603 504 564 b 369 651 408 564 369 603 z m 564 130 l 434 130 l 434 239 l 564 239 z m 564 282 l 434 282 l 434 391 l 564 391 z m 738 130 l 608 130 l 608 239 l 738 239 z m 738 282 l 608 282 l 608 391 l 738 391 z m 911 130 l 781 130 l 781 239 l 911 239 z m 911 282 l 781 282 l 781 391 l 911 391 z m 890 564 b 803 651 842 564 803 603 b 890 738 803 699 842 738 b 977 651 938 738 977 699 b 890 564 977 603 938 564 z m 1063 217 b 977 130 1063 169 1025 130 l 955 130 l 955 239 l 1063 239 z m 1063 282 l 955 282 l 955 391 l 977 391 b 1063 304 1025 391 1063 352 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 890 738 b 977 651 938 738 977 699 b 890 564 977 603 938 564 b 803 651 842 564 803 603 b 890 738 803 699 842 738 z m 456 738 b 543 651 504 738 543 699 b 456 564 543 603 504 564 b 369 651 408 564 369 603 b 456 738 369 699 408 738 z m 673 43 b 283 296 508 43 308 147 b 339 345 277 329 308 355 b 673 304 421 319 543 304 b 1007 345 803 304 925 319 b 1063 296 1037 355 1068 329 b 673 43 1038 147 837 43 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 846 844 b 934 844 866 875 913 876 b 977 694 968 795 975 744 b 934 544 975 645 968 594 b 846 544 914 514 867 513 b 803 694 812 594 805 645 b 846 844 804 744 812 795 z m 412 844 b 500 844 432 875 479 876 b 543 694 534 795 541 744 b 500 544 541 645 534 594 b 412 544 480 514 433 513 b 369 694 378 594 371 645 b 412 844 370 744 378 795 z m 673 43 b 283 296 508 43 308 147 b 339 345 277 329 308 355 b 673 304 421 319 543 304 b 1007 345 803 304 925 319 b 1063 296 1038 355 1068 329 b 673 43 1038 147 837 43 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 890 803 b 1042 609 954 803 1033 723 b 1001 597 1044 586 1012 577 l 975 643 b 890 702 955 680 923 702 b 805 643 857 702 825 680 l 779 597 b 738 609 768 577 736 586 b 890 803 747 723 825 803 z m 456 803 b 608 609 520 803 599 723 b 567 597 610 586 578 577 l 541 643 b 456 702 521 680 489 702 b 371 643 423 702 391 680 l 345 597 b 304 609 333 577 302 586 b 456 803 313 723 391 803 z m 673 43 b 283 296 508 43 308 147 b 339 345 277 329 308 355 b 673 304 421 319 543 304 b 1007 345 803 304 925 319 b 1063 296 1038 355 1068 329 b 673 43 1038 147 837 43 z "
	},
	"": {
		ha: 1367,
		x_min: 0,
		x_max: 0,
		o: "m 1237 868 b 1367 996 1309 868 1367 925 b 1254 1207 1367 1050 1290 1159 b 1220 1207 1246 1218 1228 1218 b 1107 996 1184 1159 1107 1050 b 1237 868 1107 925 1165 868 z m 1237 781 b 1020 996 1117 781 1020 877 b 1042 1083 1020 1007 1021 1034 b 673 1194 936 1153 809 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 1290 788 1345 616 1325 706 b 1237 781 1273 784 1255 781 z m 890 803 b 1042 609 954 803 1033 723 b 1001 597 1044 586 1012 577 l 975 643 b 890 702 955 680 923 702 b 805 643 857 702 825 680 l 779 597 b 738 609 768 577 736 586 b 890 803 747 723 825 803 z m 456 803 b 608 609 520 803 599 723 b 567 597 610 586 578 577 l 541 643 b 456 702 521 680 489 702 b 371 643 423 702 391 680 l 345 597 b 304 609 333 577 302 586 b 456 803 313 723 391 803 z m 673 43 b 283 296 508 43 308 147 b 339 345 277 329 308 355 b 673 304 421 319 543 304 b 1007 345 803 304 925 319 b 1063 296 1038 355 1068 328 b 673 43 1038 147 837 43 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 245 717 b 367 777 263 765 318 790 l 386 771 l 392 791 b 504 866 405 840 454 875 b 582 746 562 857 598 801 l 530 556 b 501 540 527 544 514 537 l 311 589 b 245 717 256 604 224 662 z m 673 43 b 283 296 508 43 308 147 b 339 345 277 329 308 355 b 673 304 421 319 543 304 b 1007 345 803 304 925 319 b 1063 296 1038 355 1068 329 b 673 43 1038 147 837 43 z m 1035 589 l 844 540 b 816 556 832 537 819 544 l 763 746 b 841 867 748 801 783 857 b 954 791 892 875 940 840 l 959 772 l 978 777 b 1100 718 1028 790 1082 765 b 1035 589 1121 662 1090 604 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 764 679 l 981 809 b 1023 760 1013 828 1047 788 l 932 651 l 1023 542 b 981 493 1047 514 1013 474 l 764 623 b 764 679 744 636 744 666 z m 322 760 b 364 809 299 788 333 828 l 581 679 b 581 623 602 666 602 636 l 364 493 b 322 542 333 474 299 514 l 413 651 z m 673 43 b 283 296 508 43 308 147 b 339 345 277 329 308 355 b 673 304 421 319 543 304 b 1007 345 803 304 925 319 b 1063 296 1038 355 1068 328 b 673 43 1038 147 837 43 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1111 912 b 1351 984 1172 920 1310 944 b 1349 1175 1403 1037 1402 1122 b 1158 1177 1296 1228 1210 1229 b 1085 937 1118 1137 1094 999 b 1111 912 1083 922 1096 910 z m 278 130 b 38 57 216 121 78 97 b 40 -133 -14 5 -13 -80 b 231 -135 94 -186 179 -187 b 304 104 271 -95 295 43 b 278 130 306 119 293 132 z m 1123 826 b 999 950 1034 815 991 894 b 1031 1103 1009 1015 1019 1064 b 219 997 773 1252 439 1217 b 112 185 -1 777 -36 442 b 266 216 151 196 200 206 b 390 92 356 227 397 148 b 358 -61 380 27 369 -22 b 1170 45 616 -210 950 -175 b 1277 857 1391 265 1425 600 b 1123 826 1238 846 1188 835 z m 694 928 b 758 923 703 964 754 959 l 771 781 l 913 768 b 917 704 949 765 953 713 l 672 643 b 633 682 648 637 627 659 z m 447 303 l 434 445 l 292 458 b 287 522 256 461 252 513 l 533 583 b 572 544 557 589 578 567 l 511 298 b 447 303 502 263 451 267 z m 1032 183 b 716 46 953 104 827 46 b 577 86 664 46 616 59 b 583 160 551 105 554 145 b 848 367 659 200 756 275 b 1055 632 940 459 1015 556 b 1129 638 1070 661 1110 665 b 1032 183 1217 515 1148 299 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 257 757 l 351 771 l 393 856 b 431 856 401 872 423 872 l 473 771 l 568 757 b 579 721 585 754 592 734 l 511 655 l 527 560 b 497 538 530 543 512 530 l 412 582 l 327 538 b 298 560 312 530 295 543 l 314 654 l 245 721 b 257 757 232 734 240 754 z m 673 43 b 283 296 508 43 308 147 b 339 345 277 329 308 355 b 673 304 421 319 543 304 b 1007 345 803 304 925 319 b 1063 296 1038 355 1068 329 b 673 43 1038 147 837 43 z m 1101 721 l 1032 655 l 1048 560 b 1018 538 1051 543 1034 530 l 933 582 l 848 538 b 818 560 833 530 816 543 l 835 654 l 766 721 b 777 757 754 734 761 754 l 872 770 l 914 856 b 952 856 922 872 944 872 l 994 770 l 1089 757 b 1101 721 1106 754 1113 734 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 278 521 b 38 448 216 512 78 488 b 40 257 -14 396 -13 311 b 231 256 94 204 179 203 b 304 495 271 296 295 434 b 278 521 306 510 293 523 z m 1698 448 b 1458 521 1657 488 1519 512 b 1432 495 1443 523 1430 510 b 1505 255 1441 433 1465 295 b 1695 257 1557 203 1642 204 b 1698 448 1749 310 1750 396 z m 1347 483 b 1471 607 1336 555 1398 617 b 1536 595 1495 603 1516 599 b 868 1194 1499 932 1214 1194 b 200 595 522 1194 237 932 b 266 607 220 599 241 603 b 390 483 356 617 397 538 b 292 194 358 259 308 210 b 284 187 290 192 287 190 b 868 -152 400 -15 618 -152 b 1452 187 1118 -152 1336 -15 b 1444 194 1449 190 1446 192 b 1347 483 1428 210 1379 259 z m 1085 803 b 1237 609 1150 803 1228 723 b 1197 597 1239 586 1208 577 l 1171 643 b 1085 702 1150 680 1119 702 b 1000 643 1052 702 1021 680 l 974 597 b 934 609 963 577 932 586 b 1085 803 942 723 1021 803 z m 651 803 b 803 609 716 803 794 723 b 763 597 805 586 774 577 l 737 643 b 651 702 716 680 685 702 b 566 643 618 702 587 680 l 540 597 b 500 609 529 577 498 586 b 651 803 508 723 586 803 z m 868 43 b 478 296 704 43 503 147 b 534 345 473 328 503 355 b 868 304 616 319 738 304 b 1202 345 998 304 1120 319 b 1258 296 1233 355 1263 329 b 868 43 1233 147 1032 43 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 437 -109 0 232 182 -13 b 412 0 422 -75 412 -39 l 412 123 b 283 296 345 167 294 227 b 339 345 277 329 308 355 b 673 304 421 319 543 304 b 1007 345 803 304 925 319 b 1063 296 1038 355 1068 329 b 933 123 1051 227 1000 167 l 933 0 b 909 -109 933 -39 924 -75 b 1345 521 1164 -13 1345 232 b 673 1194 1345 892 1044 1194 z m 456 564 b 369 651 408 564 369 603 b 456 738 369 699 408 738 b 543 651 504 738 543 699 b 456 564 543 603 504 564 z m 890 564 b 803 651 842 564 803 603 b 890 738 803 699 842 738 b 977 651 938 738 977 699 b 890 564 977 603 938 564 z m 795 199 b 701 151 756 217 711 193 l 696 130 b 649 130 691 105 655 105 l 645 151 b 551 199 635 193 590 217 b 499 174 548 198 551 200 l 499 3 b 671 -173 499 -93 575 -172 b 847 0 767 -175 847 -96 l 847 174 b 795 199 794 200 797 198 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 795 199 b 701 151 756 217 711 193 l 696 130 b 649 130 691 105 655 105 l 645 151 b 551 199 635 193 590 217 b 499 174 548 198 551 200 l 499 3 b 671 -173 499 -93 575 -172 b 847 0 767 -175 847 -96 l 847 174 b 795 199 794 200 797 198 z m 673 1194 b 0 521 301 1194 0 892 b 437 -109 0 232 182 -13 b 412 0 422 -75 412 -39 l 412 123 b 283 296 345 167 294 227 b 339 345 277 329 308 355 b 673 304 421 319 543 304 b 1007 345 803 304 925 319 b 1063 296 1038 355 1068 329 b 933 123 1051 227 1000 167 l 933 0 b 909 -109 933 -39 924 -75 b 1345 521 1164 -13 1345 232 b 673 1194 1345 892 1044 1194 z m 581 623 l 364 493 b 322 542 333 474 299 514 l 413 651 l 322 760 b 364 809 299 788 333 828 l 581 679 b 581 623 602 666 602 636 z m 1023 542 b 981 493 1047 514 1013 474 l 764 623 b 764 679 743 636 743 666 l 981 809 b 1023 760 1013 828 1046 788 l 932 651 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 933 716 b 868 651 897 716 868 687 b 933 586 868 615 897 586 b 998 651 969 586 998 615 b 933 716 998 687 969 716 z m 673 1194 b 0 521 301 1194 0 892 b 437 -109 0 232 182 -13 b 412 0 422 -75 412 -39 l 412 123 b 283 296 345 167 294 227 b 339 345 277 329 308 355 b 673 304 421 319 543 304 b 1007 345 803 304 925 319 b 1063 296 1038 355 1068 329 b 933 123 1051 227 1000 167 l 933 0 b 909 -109 933 -39 924 -75 b 1345 521 1164 -13 1345 232 b 673 1194 1345 892 1044 1194 z m 521 583 l 495 606 b 330 606 455 642 370 642 l 304 583 b 250 613 281 563 245 582 b 413 727 261 681 343 727 b 575 613 482 727 564 681 b 521 583 579 582 544 563 z m 933 477 b 760 651 837 477 760 555 b 933 825 760 747 837 825 b 1107 651 1029 825 1107 747 b 933 477 1107 555 1029 477 z m 795 199 b 701 151 756 217 711 193 l 696 130 b 649 130 691 105 655 105 l 645 151 b 551 199 635 193 590 217 b 499 174 548 198 551 200 l 499 3 b 671 -173 499 -93 575 -172 b 847 0 767 -175 847 -96 l 847 174 b 795 199 794 200 797 198 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 0 521 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 b 0 521 301 1194 0 892 z m 543 651 b 456 564 543 603 504 564 b 369 651 408 564 369 603 b 456 738 369 699 408 738 b 543 651 504 738 543 699 z m 998 583 l 972 606 b 807 606 932 642 847 642 l 781 583 b 728 613 759 563 723 582 b 890 727 738 681 820 727 b 1053 613 960 727 1042 681 b 998 583 1057 583 1021 563 z m 339 345 b 673 304 421 319 543 304 b 1007 345 803 304 925 319 b 1063 296 1037 355 1068 329 b 673 43 1038 147 837 43 b 283 296 508 43 308 147 b 339 345 277 329 308 355 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 260 434 l 87 434 b 0 347 39 434 0 395 l 0 174 b 87 87 0 126 39 87 l 260 87 b 347 174 308 87 347 126 l 347 347 b 260 434 347 395 308 434 z m 694 434 l 521 434 b 434 347 473 434 434 395 l 434 174 b 521 87 434 126 473 87 l 694 87 b 781 174 742 87 781 126 l 781 347 b 694 434 781 395 742 434 z m 1128 434 l 955 434 b 868 347 907 434 868 395 l 868 174 b 955 87 868 126 907 87 l 1128 87 b 1215 174 1176 87 1215 126 l 1215 347 b 1128 434 1215 395 1176 434 z m 260 955 l 87 955 b 0 868 39 955 0 916 l 0 694 b 87 608 0 647 39 608 l 260 608 b 347 694 308 608 347 647 l 347 868 b 260 955 347 916 308 955 z m 694 955 l 521 955 b 434 868 473 955 434 916 l 434 694 b 521 608 434 647 473 608 l 694 608 b 781 694 742 608 781 647 l 781 868 b 694 955 781 916 742 955 z m 1128 955 l 955 955 b 868 868 907 955 868 916 l 868 694 b 955 608 868 647 907 608 l 1128 608 b 1215 694 1176 608 1215 647 l 1215 868 b 1128 955 1215 916 1176 955 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 260 1128 l 87 1128 b 0 1042 39 1128 0 1090 l 0 868 b 87 781 0 820 39 781 l 260 781 b 347 868 308 781 347 820 l 347 1042 b 260 1128 347 1090 308 1128 z m 260 694 l 87 694 b 0 608 39 694 0 656 l 0 434 b 87 347 0 386 39 347 l 260 347 b 347 434 308 347 347 386 l 347 608 b 260 694 347 656 308 694 z m 260 260 l 87 260 b 0 174 39 260 0 222 l 0 0 b 87 -87 0 -48 39 -87 l 260 -87 b 347 0 308 -87 347 -48 l 347 174 b 260 260 347 222 308 260 z m 781 1128 l 608 1128 b 521 1042 560 1128 521 1090 l 521 868 b 608 781 521 820 560 781 l 781 781 b 868 868 829 781 868 820 l 868 1042 b 781 1128 868 1090 829 1128 z m 781 694 l 608 694 b 521 608 560 694 521 656 l 521 434 b 608 347 521 386 560 347 l 781 347 b 868 434 829 347 868 386 l 868 608 b 781 694 868 656 829 694 z m 781 260 l 608 260 b 521 174 560 260 521 222 l 521 0 b 608 -87 521 -48 560 -87 l 781 -87 b 868 0 829 -87 868 -48 l 868 174 b 781 260 868 222 829 260 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 434 434 l 391 434 b 217 260 295 434 217 356 l 217 87 b 391 -87 217 -9 295 -87 l 434 -87 b 521 0 482 -87 521 -48 l 521 347 b 434 434 521 395 482 434 z m 998 434 l 955 434 b 868 347 907 434 868 395 l 868 0 b 955 -87 868 -48 907 -87 l 998 -87 b 1172 87 1094 -87 1172 -9 l 1172 260 b 998 434 1172 356 1094 434 z m 694 1128 b 0 434 306 1128 12 805 l 0 130 b 43 87 0 106 19 87 l 87 87 b 130 130 111 87 130 106 l 130 434 b 694 998 130 745 383 998 b 1259 434 1006 998 1259 745 l 1259 130 b 1302 87 1259 106 1278 87 l 1345 87 b 1389 130 1369 87 1389 106 l 1389 434 b 694 1128 1376 805 1083 1128 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 521 651 b 434 738 521 699 482 738 l 391 738 b 217 564 295 738 217 660 l 217 434 b 391 260 217 338 295 260 l 434 260 b 521 347 482 260 521 299 z m 998 260 b 1172 434 1094 260 1172 338 l 1172 564 b 998 738 1172 660 1094 738 l 955 738 b 868 651 907 738 868 699 l 868 347 b 955 260 868 299 907 260 z m 694 1215 b 0 521 307 1215 12 893 l 0 477 b 43 434 0 453 19 434 l 87 434 b 130 477 111 434 130 453 l 130 521 b 694 1085 130 832 383 1085 b 1259 521 1006 1085 1259 832 l 1258 521 b 1259 71 1259 514 1259 71 b 1144 -43 1259 8 1207 -43 l 868 -43 b 738 87 868 29 810 87 l 651 87 b 521 -43 579 87 521 29 b 651 -174 521 -115 579 -174 l 1144 -174 b 1389 71 1279 -174 1389 -64 l 1389 521 b 694 1215 1376 893 1082 1215 z "
	},
	"": {
		ha: 1476,
		x_min: 0,
		x_max: 0,
		o: "m 0 -87 l 271 -174 l 367 -77 l 185 104 z m 338 564 b 304 490 319 548 304 515 b 309 461 304 482 306 469 l 344 345 l 206 207 l 467 -54 l 605 84 l 721 48 b 824 77 758 37 799 48 l 921 190 l 451 660 z m 1432 1000 l 1261 1172 b 1057 1178 1205 1227 1116 1230 l 517 717 l 977 256 l 1438 796 b 1432 1000 1490 855 1488 945 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1124 733 b 1166 694 1126 711 1145 694 l 1210 694 b 1253 744 1236 694 1256 718 b 1117 1008 1242 850 1193 946 b 1031 1176 1070 1046 1039 1107 b 988 1215 1028 1199 1010 1215 l 944 1215 b 901 1166 919 1215 898 1192 b 1037 902 913 1059 962 963 b 1124 733 1084 863 1115 802 z m 831 733 b 873 694 833 711 852 694 l 917 694 b 961 744 943 694 963 718 b 825 1008 949 850 900 946 b 738 1176 777 1046 746 1107 b 695 1215 735 1199 717 1215 l 651 1215 b 608 1166 626 1215 605 1192 b 744 902 620 1059 669 963 b 831 733 791 863 822 802 z m 1302 521 l 694 521 l 394 747 b 289 781 368 766 321 781 l 289 781 l 174 781 b 0 608 78 781 0 704 l 0 0 b 174 -174 0 -96 78 -174 l 1215 -174 b 1389 0 1311 -174 1389 -96 l 1389 434 b 1302 521 1389 482 1350 521 z m 347 22 b 326 0 347 10 338 0 l 282 0 b 260 22 270 0 260 10 l 260 326 b 282 347 260 338 270 347 l 326 347 b 347 326 338 347 347 338 z m 608 22 b 586 0 608 10 598 0 l 543 0 b 521 22 531 0 521 10 l 521 326 b 543 347 521 338 531 347 l 586 347 b 608 326 598 347 608 338 z m 868 22 b 846 0 868 10 858 0 l 803 0 b 781 22 791 0 781 10 l 781 326 b 803 347 781 338 791 347 l 846 347 b 868 326 858 347 868 338 z m 1128 22 b 1107 0 1128 10 1119 0 l 1063 0 b 1042 22 1051 0 1042 10 l 1042 326 b 1063 347 1042 338 1051 347 l 1107 347 b 1128 326 1119 347 1128 338 z m 174 868 b 347 1042 270 868 347 946 b 174 1215 347 1138 270 1215 b 0 1042 78 1215 0 1138 b 174 868 0 946 78 868 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1519 1042 b 1563 1085 1543 1042 1563 1061 l 1563 1172 b 1519 1215 1563 1196 1543 1215 l 43 1215 b 0 1172 19 1215 0 1196 l 0 1085 b 43 1042 0 1061 19 1042 l 87 1042 l 87 0 l 43 0 b 0 -43 19 0 0 -19 l 0 -130 b 43 -174 0 -154 19 -174 l 694 -174 l 694 43 b 738 87 694 67 714 87 l 825 87 b 868 43 849 87 868 67 l 868 -174 l 1519 -174 b 1563 -130 1543 -174 1563 -154 l 1563 -43 b 1519 0 1563 -19 1543 0 l 1476 0 l 1476 1042 z m 694 920 b 729 955 694 937 712 955 l 833 955 b 868 920 851 955 868 937 l 868 816 b 833 781 868 799 851 781 l 729 781 b 694 816 712 781 694 799 z m 694 660 b 729 694 694 677 712 694 l 833 694 b 868 660 851 694 868 677 l 868 556 b 833 521 868 538 851 521 l 729 521 b 694 556 712 521 694 538 z m 347 920 b 382 955 347 937 365 955 l 486 955 b 521 920 503 955 521 937 l 521 816 b 486 781 521 799 503 781 l 382 781 b 347 816 365 781 347 799 z m 486 521 l 382 521 b 347 556 365 521 347 538 l 347 660 b 382 694 347 677 365 694 l 486 694 b 521 660 503 694 521 677 l 521 556 b 486 521 521 538 503 521 z m 521 174 b 781 434 521 317 637 434 b 1042 174 925 434 1042 317 z m 1215 556 b 1181 521 1215 538 1198 521 l 1076 521 b 1042 556 1059 521 1042 538 l 1042 660 b 1076 694 1042 677 1059 694 l 1181 694 b 1215 660 1198 694 1215 677 z m 1215 816 b 1181 781 1215 799 1198 781 l 1076 781 b 1042 816 1059 781 1042 799 l 1042 920 b 1076 955 1042 937 1059 955 l 1181 955 b 1215 920 1198 955 1215 937 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1205 724 b 1302 538 1266 681 1302 612 l 1302 456 b 1324 434 1302 444 1312 434 l 1454 434 b 1476 456 1466 434 1476 444 l 1476 538 b 1323 831 1476 655 1419 764 b 1215 1025 1258 876 1215 946 l 1215 1194 b 1194 1215 1215 1206 1206 1215 l 1063 1215 b 1042 1194 1051 1215 1042 1206 l 1042 1013 b 1205 724 1042 895 1108 792 z m 529 241 b 0 43 342 211 162 145 b 757 -174 227 -98 489 -174 l 997 -174 l 691 183 b 529 241 651 230 590 251 z m 1501 979 b 1476 1025 1485 989 1476 1006 l 1476 1194 b 1454 1215 1476 1206 1466 1215 l 1324 1215 b 1302 1194 1312 1215 1302 1206 l 1302 1025 b 1380 874 1302 965 1330 907 b 1563 538 1494 800 1563 674 l 1563 456 b 1584 434 1563 444 1572 434 l 1714 434 b 1736 456 1726 434 1736 444 l 1736 538 b 1501 979 1736 715 1648 880 z m 979 260 b 740 260 886 260 743 260 l 1059 -113 b 1191 -174 1088 -146 1147 -174 l 1191 -174 l 1431 -174 l 1111 200 b 979 260 1082 233 1023 260 z m 1671 260 l 1172 260 l 1492 -113 b 1624 -174 1521 -146 1580 -174 l 1624 -174 l 1671 -174 b 1736 -109 1707 -174 1736 -144 l 1736 195 b 1671 260 1736 231 1707 260 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 456 564 b 369 651 408 564 369 603 b 456 738 369 699 408 738 b 543 651 504 738 543 699 b 456 564 543 603 504 564 z m 825 141 b 631 22 825 89 747 28 b 621 63 608 20 598 54 l 667 83 b 723 141 702 98 723 120 b 667 200 723 163 702 185 l 621 219 b 621 259 605 226 604 252 l 667 279 b 723 337 702 294 723 315 b 667 396 723 359 702 381 l 621 415 b 631 457 598 425 608 460 b 825 337 747 450 825 390 b 729 240 825 302 788 263 b 825 141 788 215 825 176 z m 890 564 b 803 651 842 564 803 603 b 890 738 803 699 842 738 b 977 651 938 738 977 699 b 890 564 977 603 938 564 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 567 597 l 541 643 b 456 702 520 680 489 702 b 370 643 422 702 391 680 l 345 597 b 304 609 333 577 302 586 b 456 803 313 723 391 803 b 608 609 521 803 599 723 b 567 597 609 586 578 577 z m 825 141 b 631 22 825 89 747 28 b 621 63 608 20 598 54 l 667 83 b 723 141 702 98 723 120 b 667 200 723 163 702 185 l 621 219 b 621 259 605 226 604 252 l 667 279 b 723 337 702 294 723 315 b 667 396 723 359 702 381 l 621 415 b 631 457 598 425 608 460 b 825 337 747 450 825 390 b 729 240 825 302 788 263 b 825 141 788 215 825 176 z m 1001 597 l 975 643 b 890 702 954 680 923 702 b 804 643 856 702 825 680 l 779 597 b 738 609 767 577 736 586 b 890 803 747 723 825 803 b 1042 609 955 803 1033 723 b 1001 597 1043 586 1012 577 z "
	},
	"": {
		ha: 1367,
		x_min: 0,
		x_max: 0,
		o: "m 1359 123 b 1215 194 1338 180 1274 209 l 1192 188 l 1186 211 b 1053 300 1170 269 1113 310 b 961 157 985 289 943 222 l 1023 -67 b 1057 -86 1027 -81 1042 -90 l 1282 -28 b 1359 123 1347 -11 1384 59 z m 878 134 b 902 296 862 189 871 248 b 1249 285 983 422 1170 420 b 1299 277 1266 284 1283 281 b 1345 521 1329 352 1345 435 b 673 1194 1345 892 1044 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 942 -95 769 -152 860 -132 b 878 134 942 -93 878 134 z m 456 564 b 369 651 408 564 369 603 b 456 738 369 699 408 738 b 543 651 504 738 543 699 b 456 564 543 603 504 564 z m 781 141 b 587 22 781 89 703 28 b 578 63 564 20 554 54 l 624 83 b 680 141 659 98 680 120 b 624 200 680 163 659 185 l 578 219 b 578 259 561 226 562 253 l 624 279 b 680 337 659 294 680 315 b 624 396 680 359 659 381 l 578 415 b 587 457 554 425 565 460 b 781 337 703 450 781 390 b 686 240 781 302 745 263 b 781 141 745 215 781 176 z m 825 627 b 771 656 802 607 766 626 b 933 770 782 724 864 770 b 1096 656 1003 770 1085 724 b 1042 627 1101 626 1065 606 l 1016 650 b 851 650 976 685 891 685 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 890 781 b 977 694 938 781 977 742 b 890 608 977 646 938 608 b 803 694 842 608 803 646 b 890 781 803 742 842 781 z m 456 781 b 543 694 504 781 543 742 b 456 608 543 646 504 608 b 369 694 408 608 369 646 b 456 781 369 742 408 781 z m 694 43 l 651 43 b 263 385 452 43 288 193 b 307 434 260 411 280 434 l 1039 434 b 1082 385 1065 434 1085 411 b 694 43 1058 193 894 43 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 738 653 b 890 846 747 767 825 846 b 1042 653 954 846 1033 767 b 1001 640 1044 629 1012 620 l 975 687 b 890 745 955 724 923 745 b 805 687 857 745 825 724 l 779 640 b 738 653 767 620 736 629 z m 304 653 b 456 846 313 767 391 846 b 608 653 520 846 599 767 b 567 640 610 629 578 620 l 541 687 b 456 745 521 724 489 745 b 371 687 423 745 391 724 l 345 640 b 304 653 333 620 302 630 z m 1082 385 b 694 43 1058 193 894 43 l 651 43 b 263 385 452 43 288 193 b 307 434 260 411 280 434 l 1039 434 b 1082 385 1065 434 1085 411 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 764 755 l 981 885 b 1023 836 1013 904 1047 864 l 932 727 l 1023 618 b 981 569 1047 589 1013 550 l 764 699 b 764 755 744 712 744 742 z m 322 836 b 364 885 299 864 333 904 l 581 755 b 581 699 602 742 602 712 l 364 569 b 322 618 333 550 299 589 l 413 727 z m 1082 385 b 694 43 1058 193 894 43 l 651 43 b 263 385 452 43 288 193 b 307 434 260 411 280 434 l 1039 434 b 1082 385 1065 434 1085 411 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 727 656 b 890 770 738 725 820 770 b 1052 656 959 770 1041 725 b 999 627 1057 626 1021 607 l 973 650 b 807 650 933 686 847 686 l 781 627 b 727 656 758 607 723 626 z m 456 781 b 543 694 504 781 543 742 b 456 608 543 646 504 608 b 369 694 408 608 369 646 b 456 781 369 742 408 781 z m 1082 385 b 694 43 1058 193 894 43 l 651 43 b 263 385 452 43 288 193 b 307 434 260 411 280 434 l 1039 434 b 1082 385 1065 434 1085 411 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 608 347 l 694 347 l 694 955 l 608 955 b 521 868 560 955 521 916 l 521 434 b 608 347 521 386 560 347 z m 1563 434 l 1563 868 b 1476 955 1563 916 1524 955 l 1389 955 l 1389 347 l 1476 347 b 1563 434 1524 347 1563 386 z m 1693 174 l 347 174 l 347 1172 b 304 1215 347 1196 328 1215 l 43 1215 b 0 1172 19 1215 0 1196 l 0 1085 b 43 1042 0 1061 19 1042 l 174 1042 l 174 43 b 217 0 174 19 193 0 l 442 0 b 434 -43 437 -14 434 -28 b 564 -174 434 -115 492 -174 b 694 -43 636 -174 694 -115 b 686 0 694 -28 691 -14 l 1223 0 b 1215 -43 1218 -14 1215 -28 b 1345 -174 1215 -115 1274 -174 b 1476 -43 1417 -174 1476 -115 b 1468 0 1476 -28 1473 -14 l 1693 0 b 1736 43 1717 0 1736 19 l 1736 130 b 1693 174 1736 154 1717 174 z m 1302 955 l 1302 1085 b 1172 1215 1302 1157 1244 1215 l 911 1215 b 781 1085 840 1215 781 1157 l 781 347 l 1302 347 z m 1172 955 l 911 955 l 911 1085 l 1172 1085 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 781 1215 b 439 873 592 1215 439 1062 b 748 342 439 721 663 443 b 814 342 766 321 797 321 b 1123 873 900 443 1123 721 b 781 1215 1123 1062 970 1215 z m 55 629 b 0 549 24 617 0 581 l 0 -130 b 60 -170 0 -161 31 -182 l 434 0 l 434 632 b 376 758 410 676 390 718 z m 781 240 b 682 286 743 240 707 256 b 521 494 629 349 572 420 l 521 0 l 1042 -174 l 1042 494 b 880 286 990 420 934 349 b 781 240 856 256 819 240 z m 1503 778 l 1128 608 l 1128 -174 l 1508 -22 b 1563 59 1538 -10 1563 26 l 1563 738 b 1503 778 1563 768 1531 789 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 781 1215 b 439 873 592 1215 439 1062 b 748 342 439 721 663 443 b 814 342 766 321 797 321 b 1123 873 900 443 1123 721 b 781 1215 1123 1062 970 1215 z m 781 760 b 667 873 718 760 667 811 b 781 987 667 936 718 987 b 895 873 844 987 895 936 b 781 760 895 811 844 760 z m 55 629 b 0 549 24 617 0 581 l 0 -130 b 60 -170 0 -161 31 -182 l 434 0 l 434 632 b 376 758 410 676 390 718 z m 781 240 b 682 286 743 240 707 256 b 521 494 629 349 572 420 l 521 0 l 1042 -174 l 1042 494 b 880 286 990 420 934 349 b 781 240 856 256 819 240 z m 1503 778 l 1128 608 l 1128 -174 l 1508 -22 b 1563 59 1538 -10 1563 26 l 1563 738 b 1503 778 1563 768 1531 789 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 255 429 b 0 -101 133 307 19 70 l 0 -101 b 72 -173 -4 -143 31 -178 b 602 81 243 -154 481 -40 l 807 286 l 459 633 z m 1317 1143 b 970 1143 1221 1239 1066 1239 l 911 1084 l 811 1183 b 658 1183 769 1226 700 1226 l 376 901 b 376 840 359 884 359 857 l 437 778 b 498 778 454 761 481 761 l 735 1015 l 788 962 l 521 694 l 868 347 l 1317 796 b 1317 1143 1413 892 1413 1047 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 607 861 l 419 1173 b 345 1215 405 1196 372 1215 l 43 1215 b 8 1147 8 1215 -12 1176 l 310 716 b 607 861 390 791 493 843 z m 1345 1215 l 1044 1215 b 969 1173 1013 1215 985 1199 l 782 861 b 1079 716 896 843 999 791 l 1381 1147 b 1345 1215 1401 1176 1381 1215 z m 694 781 b 217 304 431 781 217 567 b 694 -174 217 40 431 -174 b 1172 304 958 -174 1172 40 b 694 781 1172 567 958 781 z m 945 355 l 843 254 l 867 113 b 822 80 871 87 844 68 l 694 147 l 567 80 b 522 113 544 68 518 87 l 546 254 l 443 355 b 461 408 425 373 435 404 l 603 429 l 667 557 b 694 575 672 569 683 575 b 722 557 706 575 717 569 l 786 429 l 928 408 b 945 355 954 404 964 373 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 456 564 b 369 651 408 564 369 603 b 456 738 369 699 408 738 b 543 651 504 738 543 699 b 456 564 543 603 504 564 z m 890 564 b 803 651 842 564 803 603 b 890 738 803 699 842 738 b 977 651 938 738 977 699 b 890 564 977 603 938 564 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 239 608 b 330 760 239 674 276 730 b 326 738 328 752 326 745 b 412 651 326 690 364 651 b 499 738 460 651 499 690 b 495 760 499 745 497 752 b 586 608 549 730 586 674 b 412 434 586 512 508 434 b 239 608 317 434 239 512 z m 846 130 l 499 130 b 499 217 442 130 442 217 l 846 217 b 846 130 904 217 904 130 z m 933 434 b 760 608 837 434 760 512 b 851 760 760 674 797 730 b 846 738 849 752 846 745 b 933 651 846 690 885 651 b 1020 738 981 651 1020 690 b 1016 760 1020 745 1018 752 b 1107 608 1070 730 1107 674 b 933 434 1107 512 1029 434 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 998 0 l 43 0 b 0 -43 19 0 0 -19 l 0 -130 b 43 -174 0 -154 19 -174 l 998 -174 b 1042 -130 1022 -174 1042 -154 l 1042 -43 b 998 0 1042 -19 1022 0 z m 784 942 b 759 995 783 959 771 983 l 552 1203 b 490 1203 535 1220 507 1220 l 282 995 b 257 942 270 983 259 959 l 174 87 l 868 87 z m 651 382 b 616 347 651 365 634 347 l 425 347 b 391 382 408 347 391 365 l 391 486 b 425 521 391 503 408 521 l 616 521 b 651 486 634 521 651 503 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1361 1050 b 1335 1205 1407 1097 1394 1176 b 1292 1215 1324 1211 1305 1215 b 1234 1196 1274 1215 1248 1207 l 681 781 l 1092 781 z m 1345 694 l 43 694 b 0 651 19 694 0 675 l 0 564 b 43 521 0 540 19 521 l 87 521 b 415 37 87 301 223 114 b 348 -122 381 -9 356 -63 b 391 -174 344 -149 364 -174 l 998 -174 b 1041 -122 1025 -174 1045 -149 b 973 37 1032 -63 1008 -9 b 1302 521 1166 114 1302 301 l 1345 521 b 1389 564 1369 521 1389 540 l 1389 651 b 1345 694 1389 675 1369 694 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1128 868 l 1128 1128 b 1042 1215 1128 1176 1090 1215 l 87 1215 b 0 1128 39 1215 0 1176 l 0 868 b 87 781 0 820 39 781 l 1042 781 b 1128 868 1090 781 1128 820 z m 1215 1042 l 1215 694 b 1128 608 1215 647 1176 608 l 694 608 b 521 434 599 608 521 530 l 521 347 b 434 260 473 347 434 308 l 434 -87 b 521 -174 434 -135 473 -174 l 694 -174 b 781 -87 742 -174 781 -135 l 781 260 b 694 347 781 308 742 347 l 694 434 l 1128 434 b 1389 694 1272 434 1389 551 l 1389 868 b 1215 1042 1389 964 1311 1042 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 352 738 l 458 738 b 496 929 462 811 475 877 b 352 738 421 893 366 823 z m 352 651 b 496 460 366 566 421 496 b 458 651 475 512 462 578 z m 608 438 b 671 651 629 458 664 531 l 544 651 b 608 438 551 531 587 458 z m 544 738 l 671 738 b 608 951 664 858 629 931 b 544 738 587 931 551 858 z m 719 460 b 864 651 794 496 849 566 l 758 651 b 719 460 754 578 740 512 z m 1128 1215 l 174 1215 b 0 1042 78 1215 0 1138 l 0 0 b 174 -174 0 -96 78 -174 l 1128 -174 b 1215 -87 1176 -174 1215 -135 l 1215 1128 b 1128 1215 1215 1176 1176 1215 z m 911 87 l 304 87 b 260 130 280 87 260 106 b 304 174 260 154 280 174 l 911 174 b 955 130 935 174 955 154 b 911 87 955 106 935 87 z m 608 347 b 260 694 416 347 260 503 b 608 1042 260 886 416 1042 b 955 694 799 1042 955 886 b 608 347 955 503 799 347 z m 719 929 b 758 738 740 877 754 811 l 864 738 b 719 929 849 823 794 893 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 215 448 b 160 393 192 440 167 416 l 0 -87 l 13 -100 l 265 152 b 260 174 263 159 260 166 b 347 260 260 222 299 260 b 434 174 395 260 434 222 b 347 87 434 126 395 87 b 326 91 340 87 333 89 l 74 -161 l 87 -174 l 566 -14 b 621 41 589 -6 614 18 l 711 269 l 443 537 z m 1002 1138 l 505 597 l 770 332 l 1312 829 b 1002 1138 1542 1032 1204 1367 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 371 839 b 253 727 322 825 269 775 l 0 -33 l 40 -72 l 447 335 b 434 391 439 352 434 371 b 564 521 434 463 492 521 b 694 391 636 521 694 463 b 564 260 694 319 636 260 b 508 273 544 260 525 265 l 101 -134 l 141 -174 l 900 79 b 1012 197 948 95 999 148 l 1128 608 l 781 955 z m 1351 1014 l 1188 1177 b 1003 1177 1137 1228 1054 1228 l 850 1024 l 1197 676 l 1351 830 b 1351 1014 1402 881 1402 963 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 297 553 l 662 918 l 542 1038 l 375 871 b 360 864 372 867 365 864 b 345 871 355 864 348 867 l 314 901 b 314 931 306 909 306 923 l 481 1099 l 390 1190 b 269 1190 357 1224 302 1224 l 25 947 b 25 825 -8 913 -8 859 z m 1351 870 b 1351 1054 1402 921 1402 1003 l 1228 1177 b 1044 1177 1177 1228 1095 1228 l 919 1052 l 1226 745 z m 857 991 l 52 185 l 1 -106 b 68 -173 -6 -145 28 -180 l 359 -122 l 1164 684 z m 1364 217 l 1272 308 l 1105 141 b 1075 141 1097 132 1083 132 l 1044 171 b 1044 201 1036 179 1036 193 l 1212 369 l 1092 488 l 727 123 l 999 -148 b 1120 -148 1032 -182 1087 -182 l 1364 95 b 1364 217 1397 129 1397 183 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1693 0 l 43 0 b 0 -43 19 0 0 -19 l 0 -130 b 43 -174 0 -154 19 -174 l 1693 -174 b 1736 -130 1717 -174 1736 -154 l 1736 -43 b 1693 0 1736 -19 1717 0 z m 122 657 l 362 440 b 431 403 379 425 410 408 l 1212 190 b 1431 186 1283 170 1360 166 b 1559 283 1512 209 1549 244 b 1496 432 1570 323 1555 372 b 1304 541 1444 486 1376 521 l 1039 613 l 767 1133 b 736 1166 763 1149 751 1161 l 559 1214 b 503 1169 530 1222 502 1199 l 633 724 l 356 799 l 281 983 b 251 1010 275 997 264 1007 l 143 1040 b 87 997 115 1047 87 1026 l 87 720 b 122 657 88 696 104 673 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1693 0 l 43 0 b 0 -43 19 0 0 -19 l 0 -130 b 43 -174 0 -154 19 -174 l 1693 -174 b 1736 -130 1717 -174 1736 -154 l 1736 -43 b 1693 0 1736 -19 1717 0 z m 219 290 b 285 260 236 271 259 260 l 639 261 b 719 280 662 261 698 270 l 1508 681 b 1690 839 1581 718 1646 770 b 1726 1012 1740 916 1745 972 b 1568 1086 1706 1051 1658 1080 b 1333 1033 1487 1091 1406 1070 l 1066 898 l 473 1120 b 446 1128 466 1125 454 1128 b 424 1123 440 1128 430 1126 l 246 1033 b 232 955 217 1018 210 979 l 656 689 l 376 547 l 179 646 b 158 651 174 649 164 651 b 136 646 151 651 141 649 l 27 591 b 12 515 -1 576 -9 539 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 817 260 l 1029 472 b 1029 534 1046 489 1046 517 l 968 595 b 906 595 951 612 923 612 l 694 383 l 467 611 b 694 868 595 627 694 735 b 434 1128 694 1012 578 1128 l 43 1128 b 0 1085 19 1128 0 1109 l 0 391 b 43 347 0 367 19 347 l 130 347 b 174 391 154 347 174 367 l 174 608 l 224 608 l 572 260 l 360 49 b 360 -13 343 32 343 4 l 421 -74 b 483 -74 438 -91 466 -91 l 694 138 l 906 -74 b 968 -74 923 -91 951 -91 l 1029 -13 b 1029 49 1046 4 1046 32 z m 174 955 l 434 955 b 521 868 482 955 521 916 b 434 781 521 820 482 781 l 174 781 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 326 -55 0 276 131 63 l 326 434 b 369 477 326 458 345 477 b 412 434 393 477 412 458 l 412 -100 b 673 -152 492 -133 581 -152 b 933 -100 765 -152 853 -133 l 933 434 b 977 477 933 458 953 477 b 1020 434 1000 477 1020 458 l 1020 -55 b 1345 521 1215 63 1345 277 b 673 1194 1345 892 1044 1194 z m 495 606 b 330 606 455 642 370 642 l 304 583 b 267 579 294 574 279 572 b 250 613 255 585 248 599 b 413 727 261 681 343 727 b 575 613 482 727 564 681 b 559 579 577 599 570 585 b 521 583 543 570 528 577 z m 673 87 b 543 260 601 87 543 165 b 673 434 543 356 601 434 b 803 260 745 434 803 356 b 673 87 803 165 745 87 z m 1079 579 b 1042 583 1063 571 1049 577 l 1016 607 b 851 607 976 642 891 642 l 825 583 b 787 579 814 574 799 572 b 771 613 776 585 769 599 b 933 727 782 681 864 727 b 1096 613 1003 727 1085 681 b 1079 579 1098 599 1091 586 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 890 738 b 977 651 938 738 977 699 b 890 564 977 603 938 564 b 803 651 842 564 803 603 b 890 738 803 699 842 738 z m 412 87 b 282 214 340 87 282 144 b 395 425 282 269 359 378 b 430 425 404 437 421 437 b 543 214 465 378 543 269 b 412 87 543 144 484 87 z m 456 564 b 369 651 408 564 369 603 b 456 738 369 699 408 738 b 543 651 504 738 543 699 b 456 564 543 603 504 564 z m 917 146 b 673 260 857 219 767 260 b 673 347 615 260 615 347 b 984 201 793 347 907 294 b 917 146 1021 157 954 102 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1706 644 l 1341 1082 b 1241 1128 1320 1107 1275 1128 l 87 1128 b 0 1042 39 1128 0 1090 l 0 260 b 87 174 0 212 39 174 l 174 174 b 434 -87 174 30 290 -87 b 694 174 578 -87 694 30 l 1042 174 b 1302 -87 1042 30 1158 -87 b 1563 174 1446 -87 1563 30 l 1649 174 b 1736 260 1697 174 1736 212 l 1736 560 b 1706 644 1736 591 1725 620 z m 174 694 l 174 955 l 434 955 l 434 694 z m 434 43 b 304 174 362 43 304 102 b 434 304 304 246 362 304 b 564 174 506 304 564 246 b 434 43 564 102 506 43 z m 868 694 l 608 694 l 608 955 l 868 955 z m 1302 43 b 1172 174 1230 43 1172 102 b 1302 304 1172 246 1230 304 b 1432 174 1374 304 1432 246 b 1302 43 1432 102 1374 43 z m 1042 694 l 1042 955 l 1221 955 l 1438 694 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1691 694 b 1248 500 1550 685 1350 546 b 1042 434 1169 465 1101 434 b 984 575 980 434 971 478 b 868 782 987 597 1016 790 b 408 440 800 777 694 714 l 521 721 b 169 1000 603 927 377 1132 l 20 900 b 7 840 0 887 -6 860 l 53 767 b 113 753 66 746 93 740 l 271 859 b 359 785 321 891 381 839 l 93 119 b 174 0 75 73 100 0 b 235 26 196 0 218 9 b 808 557 349 140 655 434 b 864 329 802 479 802 397 b 1041 261 905 284 965 261 b 1319 342 1138 261 1226 300 b 1694 521 1408 382 1587 512 b 1736 563 1717 522 1736 540 l 1736 650 b 1691 694 1736 675 1715 696 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 304 609 b 456 803 313 723 391 803 b 608 609 520 803 599 723 b 567 597 610 586 578 577 l 541 643 b 456 702 521 680 489 702 b 371 643 423 702 391 680 l 345 597 b 304 609 333 577 302 586 z m 984 276 b 673 130 907 183 793 130 b 361 276 552 130 439 183 b 428 332 325 320 391 376 b 673 217 489 259 578 217 b 917 332 768 217 857 259 b 984 276 954 376 1021 320 z m 1001 597 l 975 643 b 890 702 954 680 923 702 b 804 643 856 702 825 680 l 779 597 b 738 609 767 577 736 586 b 890 803 747 723 825 803 b 1042 609 955 803 1033 723 b 1001 597 1044 586 1012 577 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1172 0 l 1042 0 l 1042 87 l 694 87 l 694 -1 l 564 -1 b 521 -44 540 -1 521 -20 l 521 -130 b 564 -174 521 -154 540 -174 l 1172 -173 b 1215 -130 1196 -173 1215 -153 l 1215 -43 b 1172 0 1215 -19 1196 0 z m 1587 1143 b 1500 1215 1580 1185 1543 1215 l 236 1215 b 149 1143 193 1215 156 1185 b 0 260 -9 214 0 271 b 89 174 0 213 39 174 l 1648 174 b 1736 260 1697 174 1736 213 b 1587 1143 1736 271 1745 214 z m 705 1042 l 1031 1042 l 1058 781 l 678 781 z m 501 347 l 193 347 l 244 651 l 532 651 z m 545 781 l 266 781 l 311 1042 l 572 1042 z m 634 347 l 665 651 l 1071 651 l 1102 347 z m 1164 1042 l 1425 1042 l 1470 781 l 1191 781 z m 1235 347 l 1204 651 l 1492 651 l 1543 347 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1541 694 b 961 469 1463 694 1175 678 b 781 212 872 389 817 298 b 601 469 746 298 691 389 b 21 694 387 678 100 694 b 0 673 9 694 0 685 b 241 133 1 597 19 331 b 781 -87 469 -89 694 -87 b 1322 133 868 -87 1094 -89 b 1563 673 1543 331 1562 597 b 1541 694 1563 685 1553 694 z m 781 394 b 901 531 816 446 856 491 b 1064 654 953 582 1008 622 b 802 1121 1020 845 924 1015 b 761 1121 791 1131 772 1131 b 498 654 639 1015 543 846 b 659 534 554 623 608 584 b 781 394 697 500 752 437 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1281 684 l 1099 746 b 996 850 1047 764 1008 803 l 952 1014 b 654 1075 921 1133 745 1169 l 529 946 b 382 894 493 909 438 889 l 187 909 b 27 684 46 920 -49 787 l 131 542 b 144 406 161 501 166 450 l 67 251 b 266 51 11 139 129 21 l 456 92 b 611 61 510 104 568 92 l 758 -50 b 1041 51 865 -130 1032 -71 l 1054 219 b 1137 335 1058 267 1089 310 l 1305 421 b 1281 684 1427 484 1413 639 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 608 1128 b 521 1215 608 1176 569 1215 l 347 1215 b 260 1128 299 1215 260 1176 l 260 868 l 608 868 z m 1302 868 b 1215 781 1254 868 1215 829 b 1302 694 1215 733 1254 694 b 1389 781 1350 694 1389 733 b 1302 868 1389 829 1350 868 z m 608 781 l 260 781 b 0 521 117 781 0 665 l 0 -87 b 87 -174 0 -135 39 -174 l 781 -174 b 868 -87 829 -174 868 -135 l 868 521 b 608 781 868 665 751 781 z m 434 87 b 217 304 314 87 217 184 b 434 521 217 424 314 521 b 651 304 554 521 651 424 b 434 87 651 184 554 87 z m 1302 955 b 1389 1042 1350 955 1389 994 b 1302 1128 1389 1090 1350 1128 b 1215 1042 1254 1128 1215 1090 b 1302 955 1215 994 1254 955 z m 1042 868 b 955 781 994 868 955 829 b 1042 694 955 733 994 694 b 1128 781 1090 694 1128 733 b 1042 868 1128 829 1090 868 z m 781 1128 b 694 1042 733 1128 694 1090 b 781 955 694 994 733 955 b 868 1042 829 955 868 994 b 781 1128 868 1090 829 1128 z m 1042 1128 b 955 1042 994 1128 955 1090 b 1042 955 955 994 994 955 b 1128 1042 1090 955 1128 994 b 1042 1128 1128 1090 1090 1128 z m 1302 608 b 1215 521 1254 608 1215 569 b 1302 434 1215 473 1254 434 b 1389 521 1350 434 1389 473 b 1302 608 1389 569 1350 608 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 87 -174 l 1302 -174 l 1302 0 l 87 0 z m 1128 521 l 948 521 b 868 601 904 521 868 557 l 868 626 b 926 837 868 701 892 771 b 951 1001 951 885 961 941 b 741 1211 932 1106 846 1193 b 434 955 577 1240 434 1114 b 458 847 434 916 442 880 b 521 601 494 769 521 687 l 521 601 b 441 521 521 557 485 521 l 260 521 b 0 260 117 521 0 404 l 0 174 b 87 87 0 126 39 87 l 1302 87 b 1389 174 1350 87 1389 126 l 1389 260 b 1128 521 1389 404 1272 521 z "
	},
	"": {
		ha: 1454,
		x_min: 0,
		x_max: 0,
		o: "m 1380 750 l 982 808 l 805 1167 b 727 1215 789 1199 758 1215 b 649 1167 696 1215 665 1199 l 472 808 l 74 750 b 26 602 3 740 -25 652 l 314 323 l 246 -72 b 331 -174 236 -128 281 -174 b 372 -163 345 -174 358 -170 l 727 23 l 1082 -163 b 1122 -174 1095 -170 1109 -174 b 1208 -72 1173 -174 1218 -128 l 1140 323 l 1428 602 b 1380 750 1479 652 1451 740 z m 1049 416 l 1000 368 l 1012 301 l 1065 -7 l 788 138 l 727 170 l 727 1030 l 865 750 l 896 689 l 964 679 l 1273 634 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 911 781 l 130 781 b 0 651 58 781 0 723 l 0 43 b 130 -87 0 -29 58 -87 l 174 -87 l 174 -130 b 217 -174 174 -154 193 -174 l 304 -174 b 347 -130 328 -174 347 -154 l 347 -87 l 694 -87 l 694 -130 b 738 -174 694 -154 714 -174 l 825 -174 b 868 -130 849 -174 868 -154 l 868 -87 l 911 -87 b 1042 43 983 -87 1042 -29 l 1042 651 b 911 781 1042 723 983 781 z m 868 195 b 846 174 868 183 858 174 l 195 174 b 174 195 183 174 174 183 l 174 239 b 195 260 174 251 183 260 l 846 260 b 868 239 858 260 868 251 z m 868 456 b 846 434 868 444 858 434 l 195 434 b 174 456 183 434 174 444 l 174 499 b 195 521 174 511 183 521 l 846 521 b 868 499 858 521 868 511 z m 391 1085 l 651 1085 l 651 868 l 781 868 l 781 1085 b 651 1215 781 1157 723 1215 l 391 1215 b 260 1085 319 1215 260 1157 l 260 868 l 391 868 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 369 651 b 456 738 369 699 408 738 b 543 651 504 738 543 699 b 456 564 543 603 504 564 b 369 651 408 564 369 603 z m 673 87 b 499 260 577 87 499 165 b 673 434 499 356 577 434 b 846 260 768 434 846 356 b 673 87 846 165 768 87 z m 890 564 b 803 651 842 564 803 603 b 890 738 803 699 842 738 b 977 651 938 738 977 699 b 890 564 977 603 938 564 z "
	},
	"": {
		ha: 1386,
		x_min: 0,
		x_max: 0,
		o: "m 1300 347 l 1010 347 l 505 -159 b 487 -174 499 -164 493 -168 l 1300 -174 b 1386 -87 1347 -174 1386 -135 l 1386 260 b 1300 347 1386 308 1347 347 z m 1179 762 l 934 1008 b 811 1008 900 1041 845 1041 l 606 802 l 606 87 b 604 63 606 79 605 71 l 1179 639 b 1179 762 1213 673 1213 728 z m 520 1128 b 433 1215 520 1176 481 1215 l 87 1215 b 0 1128 39 1215 0 1176 l 0 87 b 260 -174 0 -57 116 -174 b 520 87 403 -174 520 -57 z m 260 22 b 195 87 224 22 195 51 b 260 152 195 123 224 152 b 325 87 296 152 325 123 b 260 22 325 51 296 22 z m 347 521 l 173 521 l 173 694 l 347 694 l 347 521 z m 347 868 l 173 868 l 173 1042 l 347 1042 l 347 868 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 514 373 b 608 347 524 364 556 347 b 701 373 660 347 691 364 b 846 434 744 413 795 434 l 890 434 b 1035 373 942 434 992 413 b 1128 347 1045 364 1077 347 b 1222 373 1180 347 1212 364 b 1356 433 1262 410 1308 430 l 1050 652 l 1098 686 b 1133 693 1108 693 1121 696 l 1405 635 b 1560 735 1475 620 1544 665 b 1460 890 1575 805 1530 875 l 1187 948 b 947 898 1104 966 1016 948 l 730 743 b 657 670 702 723 677 699 l 471 404 b 514 373 486 395 501 385 z m 1693 260 l 1649 260 b 1497 309 1579 260 1525 283 b 1411 347 1473 331 1444 347 l 1367 347 b 1281 309 1334 347 1305 331 b 1128 260 1253 283 1199 260 b 976 309 1058 260 1004 283 b 890 347 952 331 923 347 l 846 347 b 760 309 813 347 784 331 b 608 260 732 283 678 260 b 455 309 537 260 483 283 b 369 347 431 331 402 347 l 325 347 b 239 309 292 347 263 331 b 87 260 211 283 157 260 l 43 260 b 0 217 19 260 0 241 l 0 130 b 43 87 0 106 19 87 l 87 87 b 347 173 192 87 284 120 b 608 87 410 120 503 87 b 868 173 712 87 805 120 b 1128 87 931 120 1024 87 b 1389 173 1233 87 1326 120 b 1649 87 1452 120 1545 87 l 1693 87 b 1736 130 1717 87 1736 106 l 1736 217 b 1693 260 1736 241 1717 260 z m 304 521 b 521 738 424 521 521 618 b 304 955 521 858 424 955 b 87 738 184 955 87 858 b 304 521 87 618 184 521 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1693 87 l 1649 87 b 1497 135 1579 87 1525 110 b 1411 174 1473 158 1444 174 l 1367 174 b 1281 135 1334 174 1305 158 b 1128 87 1253 110 1199 87 b 976 135 1058 87 1004 110 b 890 174 952 158 923 174 l 846 174 b 760 135 813 174 784 158 b 608 87 732 110 678 87 b 455 135 537 87 483 110 b 369 174 431 158 402 174 l 325 174 b 239 135 292 174 263 158 b 87 87 211 110 157 87 l 43 87 b 0 43 19 87 0 67 l 0 -43 b 43 -87 0 -67 19 -87 l 87 -87 b 347 0 192 -87 284 -54 b 608 -87 410 -54 503 -87 b 868 0 712 -87 805 -54 b 1128 -87 931 -54 1024 -87 b 1389 0 1233 -87 1326 -54 b 1649 -87 1452 -54 1545 -87 l 1693 -87 b 1736 -43 1717 -87 1736 -67 l 1736 43 b 1693 87 1736 67 1717 87 z m 608 174 l 608 434 l 1128 434 l 1128 174 b 1222 199 1180 174 1212 190 b 1302 248 1247 222 1274 238 l 1302 868 b 1389 955 1302 916 1341 955 b 1476 868 1437 955 1476 916 l 1476 825 b 1519 781 1476 801 1495 781 l 1606 781 b 1649 825 1630 781 1649 801 l 1649 868 b 1389 1128 1649 1012 1532 1128 b 1128 868 1245 1128 1128 1012 l 1128 608 l 608 608 l 608 868 b 694 955 608 916 647 955 b 781 868 742 955 781 916 l 781 825 b 825 781 781 801 801 781 l 911 781 b 955 825 935 781 955 801 l 955 868 b 694 1128 955 1012 838 1128 b 434 868 551 1128 434 1012 l 434 248 b 514 199 462 238 489 222 b 608 174 524 190 556 174 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1719 -27 l 1343 264 b 1345 309 1344 279 1345 294 b 947 1155 1345 613 1075 725 b 789 1155 926 1233 813 1237 b 630 814 747 1013 689 906 l 123 1206 b 62 1199 104 1221 77 1217 l 9 1130 b 17 1069 -6 1111 -2 1084 l 1613 -164 b 1674 -157 1632 -179 1659 -176 l 1727 -88 b 1719 -27 1742 -69 1738 -42 z m 391 309 b 868 -174 391 42 604 -174 b 1184 -52 989 -174 1100 -128 l 441 521 b 391 309 410 458 391 390 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 764 679 l 981 809 b 1023 760 1013 828 1047 788 l 932 651 l 1023 542 b 981 493 1047 514 1013 474 l 764 623 b 764 679 744 636 744 666 z m 322 760 b 364 809 299 788 333 828 l 581 679 b 581 623 602 666 602 636 l 364 493 b 322 542 333 474 299 514 l 413 651 z m 673 434 b 1007 145 814 434 986 315 b 959 89 1012 108 985 78 b 673 136 889 119 784 136 b 387 89 561 136 457 119 b 339 145 360 78 334 108 b 673 434 360 315 532 434 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1204 954 b 955 1208 1175 1077 1077 1177 b 713 1142 867 1231 784 1195 b 684 1128 704 1136 694 1132 l 761 1078 b 774 1018 781 1065 787 1038 b 714 1005 762 999 736 991 l 442 1180 b 260 1208 386 1210 324 1225 b 11 954 139 1177 41 1077 b 69 655 -17 841 10 735 b 168 363 133 569 158 468 b 224 -17 180 235 196 108 l 246 -109 b 326 -174 254 -147 288 -174 b 406 -111 364 -174 397 -148 l 500 265 b 608 349 512 314 557 349 b 715 265 659 349 703 314 l 809 -111 b 889 -174 818 -148 851 -174 b 970 -109 928 -174 961 -147 l 991 -17 b 1048 363 1020 108 1036 235 b 1146 655 1057 468 1083 569 b 1204 954 1206 735 1232 841 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 313 844 l 590 743 b 967 1214 685 964 824 1134 b 293 911 707 1228 455 1114 b 313 844 275 888 286 854 z m 672 713 l 1319 478 b 1203 1167 1416 807 1369 1106 b 1140 1178 1183 1174 1162 1178 b 672 713 983 1178 792 990 z m 1415 1051 b 1450 901 1431 1007 1444 957 b 1399 448 1466 766 1447 608 l 1677 347 b 1736 386 1705 337 1735 356 b 1415 1051 1742 645 1622 894 z m 1519 0 l 871 0 l 1047 484 l 884 544 l 686 0 l 43 0 b 0 -43 19 0 0 -19 l 0 -130 b 43 -174 0 -154 19 -174 l 1519 -174 b 1563 -130 1543 -174 1563 -154 l 1563 -43 b 1519 0 1563 -19 1543 0 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1389 868 l 1389 1128 b 1302 1215 1389 1176 1350 1215 l 1042 1215 b 955 1128 994 1215 955 1176 l 434 1128 b 347 1215 434 1176 395 1215 l 87 1215 b 0 1128 39 1215 0 1176 l 0 868 b 87 781 0 820 39 781 l 87 260 b 0 174 39 260 0 222 l 0 -87 b 87 -174 0 -135 39 -174 l 347 -174 b 434 -87 395 -174 434 -135 l 955 -87 b 1042 -174 955 -135 994 -174 l 1302 -174 b 1389 -87 1350 -174 1389 -135 l 1389 174 b 1302 260 1389 222 1350 260 l 1302 781 b 1389 868 1350 781 1389 820 z m 1128 1042 l 1215 1042 l 1215 955 l 1128 955 z m 174 1042 l 260 1042 l 260 955 l 174 955 z m 260 0 l 174 0 l 174 87 l 260 87 z m 1215 0 l 1128 0 l 1128 87 l 1215 87 z m 1128 260 l 1042 260 b 955 174 994 260 955 222 l 955 87 l 434 87 l 434 174 b 347 260 434 222 395 260 l 260 260 l 260 781 l 347 781 b 434 868 395 781 434 820 l 434 955 l 955 955 l 955 868 b 1042 781 955 820 994 781 l 1128 781 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1384 6 l 1186 798 b 1102 868 1176 839 1142 868 l 939 868 b 955 955 949 895 955 924 b 694 1215 955 1099 838 1215 b 434 955 551 1215 434 1099 b 450 868 434 924 440 895 l 287 868 b 203 798 247 868 213 839 l 5 6 b 130 -174 -18 -85 44 -174 l 1259 -174 b 1384 6 1344 -174 1407 -85 z m 694 868 b 608 955 647 868 608 907 b 694 1042 608 1003 647 1042 b 781 955 742 1042 781 1003 b 694 868 781 907 742 868 z "
	},
	"": {
		ha: 781,
		x_min: 0,
		x_max: 0,
		o: "m 586 -43 l 477 -43 l 477 274 b 780 693 663 318 798 491 l 736 1176 b 694 1215 734 1198 716 1215 l 88 1215 b 45 1176 65 1215 47 1198 l 2 693 b 304 274 -16 491 118 318 l 304 -43 l 195 -43 b 87 -152 135 -43 87 -92 b 109 -174 87 -164 97 -174 l 673 -174 b 694 -152 685 -174 694 -164 b 586 -43 694 -92 646 -43 z m 168 1085 l 614 1085 l 633 868 l 148 868 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 1028 343 l 772 608 l 906 608 b 943 682 947 608 970 653 l 636 1025 b 651 1085 645 1043 651 1063 b 521 1215 651 1157 593 1215 b 391 1085 449 1215 391 1157 b 406 1025 391 1063 396 1043 l 98 682 b 136 608 72 653 94 608 l 269 608 l 14 343 b 59 260 -18 310 9 260 l 434 260 l 434 174 l 130 174 b 87 130 106 174 87 154 l 87 -130 b 130 -174 87 -154 106 -174 l 911 -174 b 955 -130 935 -174 955 -154 l 955 130 b 911 174 955 154 935 174 l 608 174 l 608 260 l 983 260 b 1028 343 1033 260 1060 310 z m 521 1129 b 564 1085 545 1129 564 1109 b 521 1042 564 1061 545 1042 b 477 1085 497 1042 477 1061 b 521 1129 477 1109 497 1129 z m 825 43 l 825 -43 l 217 -43 l 217 43 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 952 865 b 773 846 908 873 849 866 b 608 781 708 828 653 807 b 442 846 562 807 507 828 b 263 865 366 866 307 873 b 90 751 193 853 135 815 b 9 540 50 695 23 625 b 12 307 -4 462 -3 384 b 142 -16 33 178 77 71 b 434 -174 219 -121 317 -174 b 529 -146 463 -174 495 -165 b 608 -125 553 -132 579 -125 b 686 -146 637 -125 663 -132 b 781 -174 721 -165 752 -174 b 1074 -16 898 -174 996 -121 b 1204 307 1139 71 1182 178 b 1206 540 1218 384 1219 462 b 1125 751 1192 625 1165 695 b 952 865 1080 815 1022 853 z m 803 977 b 854 1077 826 1002 843 1035 b 868 1172 863 1106 868 1138 l 865 1213 l 824 1215 b 730 1202 790 1215 758 1211 b 629 1150 688 1191 655 1174 b 567 1001 595 1116 574 1066 b 567 914 563 968 563 939 l 608 911 b 702 925 642 911 674 916 b 803 977 744 936 777 953 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1120 521 b 1189 868 1229 670 1233 788 b 873 982 1159 921 1079 1006 b 608 1215 815 1121 726 1215 b 342 982 489 1215 400 1121 b 27 868 137 1005 56 921 b 95 521 -18 788 -14 670 b 27 174 -14 372 -18 253 b 342 60 106 32 302 55 b 608 -174 400 -79 489 -174 b 873 60 726 -174 815 -79 b 1189 174 913 55 1109 32 b 1120 521 1233 253 1229 372 z m 172 260 b 212 391 161 280 171 328 b 272 339 231 374 251 356 b 290 231 276 302 283 266 b 172 260 224 228 182 243 z m 272 702 b 212 650 251 685 231 668 b 172 781 171 714 161 762 b 275 812 181 798 216 812 b 290 811 279 812 285 812 b 272 702 284 781 276 733 z m 608 1042 b 699 941 633 1042 668 1005 b 608 906 669 931 639 919 b 516 941 577 919 546 931 b 608 1042 547 1005 582 1042 z m 608 0 b 516 101 582 0 547 37 b 608 136 546 111 577 123 b 699 101 639 123 669 111 b 608 0 668 37 633 0 z m 777 427 b 608 325 705 375 652 347 b 438 427 563 347 511 375 b 438 615 433 502 433 539 b 608 717 510 667 563 694 b 777 615 652 694 705 667 b 777 427 783 539 783 502 z m 1043 260 b 925 231 1033 242 992 228 b 943 339 933 266 939 302 b 1003 391 964 356 984 374 b 1043 260 1044 328 1054 280 z m 1003 650 b 943 702 984 668 964 685 b 925 811 940 733 931 781 b 940 812 930 812 936 812 b 1043 781 999 812 1034 798 b 1003 650 1054 762 1044 714 z m 608 608 b 521 521 560 608 521 569 b 608 434 521 473 560 434 b 694 521 656 434 694 473 b 608 608 694 569 656 608 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1625 552 b 1736 732 1693 586 1736 656 l 1736 753 b 1534 955 1736 865 1646 955 b 1343 817 1447 955 1370 899 b 1240 694 1322 754 1311 694 l 497 694 b 393 817 422 694 411 764 b 202 955 366 899 289 955 b 0 753 90 955 0 865 l 0 732 b 112 552 0 656 43 586 b 112 490 137 539 137 503 b 0 309 43 456 0 386 l 0 289 b 202 87 0 177 90 87 b 393 225 289 87 366 142 b 497 347 414 287 425 347 l 1240 347 b 1343 225 1314 347 1325 278 b 1534 87 1370 142 1447 87 b 1736 289 1646 87 1736 177 l 1736 309 b 1625 490 1736 386 1693 456 b 1625 552 1599 503 1599 539 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 955 955 b 694 1215 955 1099 838 1215 b 434 955 551 1215 434 1099 b 694 694 434 811 551 694 b 955 955 838 694 955 811 z m 634 561 b 81 694 473 660 212 687 b 0 621 37 697 0 663 l 0 16 b 72 -57 0 -22 31 -55 b 596 -169 190 -63 430 -86 b 651 -137 621 -182 651 -165 l 651 530 b 634 561 651 543 645 554 z m 1308 694 b 755 561 1177 687 916 660 b 738 530 744 554 738 542 l 738 -137 b 794 -169 738 -165 768 -182 b 1317 -57 959 -86 1199 -63 b 1389 16 1357 -55 1389 -22 l 1389 621 b 1308 694 1389 663 1352 697 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 564 1215 b 397 1085 483 1215 416 1160 b 391 1085 394 1085 393 1085 b 217 911 295 1085 217 1007 b 222 873 217 898 219 886 b 87 673 142 841 87 763 b 109 578 87 639 95 607 b 0 391 44 541 0 471 b 134 190 0 300 55 223 b 130 152 132 178 130 165 b 326 -43 130 44 218 -43 b 358 -40 337 -43 347 -42 b 543 -174 384 -117 456 -174 b 738 22 650 -174 738 -86 l 738 1042 b 564 1215 738 1137 660 1215 z m 1563 391 b 1453 578 1563 471 1518 541 b 1476 673 1467 607 1476 639 b 1341 873 1476 763 1420 841 b 1345 911 1344 886 1345 898 b 1172 1085 1345 1007 1268 1085 b 1166 1085 1170 1085 1168 1085 b 998 1215 1147 1160 1079 1215 b 825 1042 903 1215 825 1138 l 825 22 b 1020 -174 825 -86 912 -174 b 1204 -40 1106 -174 1178 -117 b 1237 -43 1215 -42 1226 -43 b 1432 152 1345 -43 1432 44 b 1428 190 1432 165 1431 178 b 1563 391 1507 223 1563 300 z "
	},
	"": {
		ha: 1302,
		x_min: 0,
		x_max: 0,
		o: "m 1190 639 l 1159 716 l 1105 851 b 823 1042 1059 967 948 1042 l 479 1042 b 197 851 354 1042 243 967 l 143 716 l 112 639 b 0 477 47 614 0 552 l 0 347 b 43 233 0 303 17 264 l 43 87 b 130 0 43 39 82 0 l 217 0 b 304 87 265 0 304 39 l 304 174 l 998 174 l 998 87 b 1085 0 998 39 1037 0 l 1172 0 b 1259 87 1220 0 1259 39 l 1259 233 b 1302 347 1285 264 1302 303 l 1302 477 b 1190 639 1302 552 1255 614 z m 358 786 b 479 868 378 836 426 868 l 823 868 b 944 786 877 868 924 836 l 998 651 l 304 651 z m 217 348 b 130 434 165 348 130 382 b 217 521 130 486 165 521 b 347 391 269 521 347 443 b 217 348 347 339 269 348 z m 1085 348 b 955 391 1033 348 955 339 b 1085 521 955 443 1033 521 b 1172 434 1137 521 1172 486 b 1085 348 1172 382 1137 348 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1302 868 l 1215 868 l 1215 998 b 1172 1042 1215 1022 1196 1042 l 911 1042 b 868 998 887 1042 868 1022 l 868 868 l 521 868 l 521 998 b 477 1042 521 1022 501 1042 l 217 1042 b 174 998 193 1042 174 1022 l 174 868 l 87 868 b 0 781 39 868 0 829 l 0 87 b 87 0 0 39 39 0 l 1302 0 b 1389 87 1350 0 1389 39 l 1389 781 b 1302 868 1389 829 1350 868 z m 521 499 b 499 477 521 487 511 477 l 195 477 b 174 499 183 477 174 487 l 174 543 b 195 564 174 555 183 564 l 499 564 b 521 543 511 564 521 555 z m 1215 499 b 1194 477 1215 487 1206 477 l 1085 477 l 1085 369 b 1063 347 1085 357 1075 347 l 1020 347 b 998 369 1008 347 998 357 l 998 477 l 890 477 b 868 499 878 477 868 487 l 868 543 b 890 564 868 555 878 564 l 998 564 l 998 673 b 1020 694 998 685 1008 694 l 1063 694 b 1085 673 1075 694 1085 685 l 1085 564 l 1194 564 b 1215 543 1206 564 1215 555 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 389 616 b 573 800 413 706 484 776 l 602 839 l 690 955 b 711 981 696 964 704 973 l 742 1133 b 702 1157 746 1156 720 1172 l 538 1022 b 502 1028 527 1013 510 1016 l 385 1204 b 340 1193 372 1224 342 1216 l 319 982 b 290 960 318 967 304 957 l 82 1002 b 58 962 59 1007 44 980 l 193 799 b 188 762 202 787 200 771 l 11 645 b 22 600 -8 632 -1 602 l 233 580 b 255 550 248 578 258 565 l 213 342 b 253 319 209 320 235 304 l 348 397 b 355 491 345 428 347 460 z m 1730 257 b 1725 361 1740 293 1737 329 b 1664 442 1715 389 1688 425 l 1654 524 l 1637 668 b 1592 796 1631 715 1615 758 b 1497 889 1571 827 1529 869 b 1414 926 1472 905 1444 918 l 1081 1015 b 990 1025 1051 1023 1020 1026 b 903 1009 960 1024 931 1019 b 862 991 889 1004 875 998 b 789 938 836 977 811 960 b 759 903 779 928 768 916 l 672 787 l 623 721 b 573 708 608 720 586 714 b 472 594 525 688 487 648 l 439 468 b 440 375 430 436 431 405 b 451 347 443 367 448 355 l 413 205 b 411 171 410 194 410 182 b 475 99 416 137 440 108 l 559 76 b 665 138 605 64 652 92 l 687 222 l 1358 42 l 1336 -42 b 1397 -148 1323 -88 1351 -136 l 1481 -171 b 1572 -140 1516 -180 1551 -167 b 1587 -109 1579 -131 1584 -121 l 1625 32 b 1649 51 1632 37 1643 46 b 1697 131 1671 72 1688 99 z m 693 580 b 785 421 744 566 799 471 b 648 412 772 370 699 399 b 587 518 598 426 574 468 b 693 580 601 569 643 593 z m 1482 503 l 1277 558 l 811 683 l 898 799 b 1002 852 920 828 966 852 b 1036 847 1012 852 1027 850 l 1369 758 b 1454 685 1407 748 1438 721 b 1464 648 1459 674 1463 661 z m 1532 355 b 1593 249 1582 341 1607 299 b 1487 188 1580 199 1537 174 b 1372 263 1437 201 1359 213 b 1532 355 1386 313 1481 368 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1476 694 l 1432 694 l 1137 1063 b 1002 1128 1108 1099 1048 1128 l 421 1128 b 260 1019 350 1128 287 1085 l 130 688 b 0 521 55 669 0 602 l 0 217 b 43 174 0 193 19 174 l 174 174 b 434 -87 174 30 290 -87 b 694 174 578 -87 694 30 l 1042 174 b 1302 -87 1042 30 1158 -87 b 1563 174 1446 -87 1563 30 l 1693 174 b 1736 217 1717 174 1736 193 l 1736 434 b 1476 694 1736 578 1620 694 z m 434 43 b 304 174 362 43 304 102 b 434 304 304 245 362 304 b 564 174 506 304 564 245 b 434 43 564 102 506 43 z m 629 694 l 317 694 l 421 955 l 629 955 z m 760 694 l 760 955 l 1002 955 l 1210 694 z m 1302 43 b 1172 174 1230 43 1172 102 b 1302 304 1172 245 1230 304 b 1432 174 1374 304 1432 245 b 1302 43 1432 102 1374 43 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 911 0 l 43 0 b 0 -43 19 0 0 -19 l 0 -130 b 43 -174 0 -154 19 -174 l 911 -174 b 955 -130 935 -174 955 -154 l 955 -43 b 911 0 955 -19 935 0 z m 1476 868 l 1476 998 b 1432 1042 1476 1022 1456 1042 b 1389 998 1408 1042 1389 1022 l 1389 868 l 1302 868 l 1302 998 b 1259 1042 1302 1022 1283 1042 b 1215 998 1235 1042 1215 1022 l 1215 868 l 1172 868 b 1128 825 1148 868 1128 849 l 1128 738 b 1280 532 1128 641 1193 560 l 1280 210 b 1217 131 1280 173 1255 137 b 1128 206 1170 123 1128 160 l 1128 282 b 890 521 1128 414 1022 521 l 868 521 l 868 1042 b 694 1215 868 1138 790 1215 l 260 1215 b 87 1042 165 1215 87 1138 l 87 87 l 868 87 l 868 391 l 890 391 b 998 282 950 391 998 342 l 998 215 b 1184 1 998 108 1077 11 b 1411 206 1307 -11 1411 86 l 1411 532 b 1563 738 1498 560 1563 641 l 1563 825 b 1519 868 1563 849 1543 868 z m 706 738 l 451 361 b 423 347 445 352 435 347 b 392 382 402 347 387 364 l 454 608 l 293 608 b 261 640 273 608 258 623 l 304 930 b 336 955 306 944 320 955 l 521 955 b 552 919 542 955 558 937 l 521 781 l 677 781 b 706 738 702 781 718 757 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1363 582 l 756 1190 b 633 1190 722 1224 667 1224 l 25 582 b 25 459 -8 548 -8 493 l 633 -148 b 756 -148 667 -182 722 -182 l 1363 459 b 1363 582 1397 493 1397 548 z m 1089 548 l 861 337 b 825 353 847 325 825 334 l 825 499 l 564 499 l 564 326 b 543 304 564 314 554 304 l 456 304 b 434 326 444 304 434 314 l 434 543 b 521 629 434 590 473 629 l 825 629 l 825 775 b 861 791 825 794 847 804 l 1089 580 b 1089 548 1099 572 1099 557 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1042 260 b 1039 260 1041 260 1040 260 l 933 437 b 955 521 946 462 955 490 b 933 604 955 551 946 579 l 1039 782 b 1042 781 1040 781 1041 781 b 1215 955 1138 781 1215 859 b 1042 1128 1215 1051 1138 1128 b 892 1042 978 1128 922 1093 l 323 1042 b 174 1128 293 1093 238 1128 b 0 955 78 1128 0 1051 b 87 805 0 891 35 835 l 87 236 b 0 87 35 206 0 151 b 174 -87 0 -9 78 -87 b 323 0 238 -87 293 -52 l 892 0 b 1042 -87 922 -52 978 -87 b 1215 87 1138 -87 1215 -9 b 1042 260 1215 183 1138 260 z m 260 236 l 260 805 b 323 868 282 818 310 846 l 888 868 l 784 694 b 781 694 783 694 782 694 b 608 521 685 694 608 617 b 781 347 608 425 685 347 b 784 347 782 347 783 347 l 888 174 l 323 174 b 260 236 310 196 282 224 z m 738 521 b 781 564 738 545 757 564 b 825 521 805 564 825 545 b 781 477 825 497 805 477 b 738 521 757 477 738 497 z m 1085 955 b 1042 911 1085 931 1066 911 b 998 955 1018 911 998 931 b 1042 998 998 979 1018 998 b 1085 955 1066 998 1085 979 z m 174 998 b 217 955 198 998 217 979 b 174 911 217 931 198 911 b 130 955 150 911 130 931 b 174 998 130 979 150 998 z m 130 87 b 174 130 130 111 150 130 b 217 87 198 130 217 111 b 174 43 217 63 198 43 b 130 87 150 43 130 63 z m 1042 43 b 998 87 1018 43 998 63 b 1042 130 998 111 1018 130 b 1085 87 1066 130 1085 111 b 1042 43 1085 63 1066 43 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 692 506 b 753 506 709 489 736 489 l 784 536 b 784 598 801 553 801 581 l 687 694 l 784 791 b 784 853 801 808 801 836 l 753 883 b 692 883 736 900 709 900 l 534 725 b 534 664 517 708 517 681 z m 952 536 l 983 505 b 1044 505 1000 489 1027 489 l 1203 664 b 1203 725 1220 681 1220 708 l 1044 883 b 983 883 1027 900 1000 900 l 952 853 b 952 791 935 836 935 808 l 1049 694 l 952 598 b 952 536 935 581 935 553 z m 1693 87 l 1035 87 b 946 0 1033 33 995 0 l 781 0 b 692 87 731 0 692 47 l 43 87 b 0 43 20 87 0 67 l 0 0 b 174 -174 0 -95 78 -174 l 1563 -174 b 1736 0 1658 -174 1736 -95 l 1736 43 b 1693 87 1736 67 1717 87 z m 1563 1085 b 1432 1215 1563 1157 1504 1215 l 304 1215 b 174 1085 232 1215 174 1157 l 174 174 l 1563 174 z m 1389 347 l 347 347 l 347 1042 l 1389 1042 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 34 814 l 666 527 b 723 527 684 519 705 519 l 1355 814 b 1355 922 1400 834 1400 902 l 723 1209 b 694 1215 716 1212 703 1215 b 666 1209 686 1215 673 1212 l 34 922 b 34 814 -11 902 -11 834 z m 1355 574 l 1198 646 l 759 447 b 694 433 739 438 717 433 b 630 447 672 433 650 438 l 191 646 l 34 574 b 34 466 -11 554 -11 486 l 666 179 b 723 179 684 171 705 171 l 1355 466 b 1355 574 1400 486 1400 554 z m 1355 228 l 1198 299 l 759 100 b 694 86 739 90 717 86 b 630 100 672 86 650 90 l 191 299 l 34 228 b 34 119 -11 207 -11 139 l 666 -167 b 723 -167 684 -176 705 -176 l 1355 119 b 1355 228 1400 139 1400 207 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 434 347 l 467 347 l 467 304 b 510 260 467 280 486 260 l 618 260 b 662 304 642 260 662 280 l 662 347 l 694 347 b 781 434 742 347 781 386 l 781 1042 b 694 1128 781 1090 742 1128 l 694 1172 b 651 1215 694 1196 675 1215 l 477 1215 b 434 1172 453 1215 434 1196 l 434 1128 b 347 1042 386 1128 347 1090 l 347 434 b 434 347 347 386 386 347 z m 1259 0 l 1255 0 b 1389 347 1338 92 1389 214 b 868 868 1389 634 1155 868 l 868 694 b 1215 347 1060 694 1215 539 b 868 0 1215 156 1060 0 l 130 0 b 0 -130 58 0 0 -58 b 43 -174 0 -154 19 -174 l 1345 -174 b 1389 -130 1369 -174 1389 -154 b 1259 0 1389 -58 1331 0 z m 282 87 l 846 87 b 868 109 858 87 868 97 l 868 152 b 846 174 868 164 858 174 l 282 174 b 260 152 270 174 260 164 l 260 109 b 282 87 260 97 270 87 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1708 780 l 1128 608 l 992 676 b 914 694 971 686 936 694 l 760 694 l 760 825 l 911 825 b 955 868 935 825 955 844 l 955 911 b 911 955 955 935 935 955 l 477 955 b 434 911 453 955 434 935 l 434 868 b 477 825 434 844 453 825 l 629 825 l 629 694 l 477 694 l 102 763 b 87 764 98 763 91 764 b 0 677 40 764 0 726 l 0 420 b 71 334 0 378 30 342 l 260 300 l 260 174 b 347 87 260 126 299 87 l 1092 87 b 1154 113 1115 87 1138 96 l 1730 695 b 1736 711 1734 699 1736 705 l 1736 760 b 1708 780 1736 774 1722 784 z m 260 432 l 130 456 l 130 625 l 260 602 z m 1490 203 b 1606 87 1490 139 1542 87 b 1722 203 1670 87 1722 139 b 1606 434 1722 266 1606 434 b 1490 203 1606 434 1490 266 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1224 214 b 1302 369 1271 250 1302 305 b 1107 564 1302 477 1215 564 l 1069 564 b 1128 694 1105 596 1128 642 b 955 868 1128 790 1051 868 l 939 868 b 955 955 949 895 955 924 b 694 1215 955 1099 838 1215 b 653 1211 680 1215 667 1213 b 694 1085 679 1176 694 1132 b 477 868 694 965 597 868 l 434 868 b 260 694 338 868 260 790 b 320 564 260 642 284 596 l 282 564 b 87 369 174 564 87 477 b 164 214 87 305 118 250 b 0 22 71 199 0 119 b 195 -174 0 -86 87 -174 l 1194 -174 b 1389 22 1301 -174 1389 -86 b 1224 214 1389 119 1318 199 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1389 347 b 1302 434 1389 395 1350 434 l 868 434 b 781 347 820 434 781 395 l 781 -87 b 868 -174 781 -135 820 -174 l 1302 -174 b 1389 -87 1350 -174 1389 -135 z m 347 521 b 0 174 155 521 0 365 b 347 -174 0 -18 155 -174 b 694 174 539 -174 694 -18 b 347 521 694 365 539 521 z m 1299 608 b 1377 738 1368 608 1411 680 l 1119 1172 b 964 1172 1085 1230 999 1230 l 707 738 b 784 608 672 680 715 608 z "
	},
	"": {
		ha: 1302,
		x_min: 0,
		x_max: 0,
		o: "m 1280 308 l 912 521 l 1280 734 b 1296 793 1301 746 1308 772 l 1209 943 b 1150 959 1197 964 1171 971 l 781 746 l 781 1172 b 738 1215 781 1196 762 1215 l 564 1215 b 521 1172 540 1215 521 1196 l 521 746 l 152 959 b 93 943 131 971 105 964 l 6 793 b 22 734 -6 772 1 746 l 390 521 l 22 308 b 6 249 1 296 -6 270 l 93 98 b 152 83 105 78 131 71 l 521 295 l 521 -130 b 564 -174 521 -154 540 -174 l 738 -174 b 781 -130 762 -174 781 -154 l 781 295 l 1150 83 b 1209 98 1171 71 1197 78 l 1296 249 b 1280 308 1308 270 1301 296 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1476 1215 l 260 1215 b 0 955 117 1215 0 1099 l 0 87 b 260 -174 0 -57 117 -174 l 1476 -174 b 1736 87 1620 -174 1736 -57 l 1736 955 b 1476 1215 1736 1099 1620 1215 z m 434 217 b 304 87 434 145 376 87 b 174 217 232 87 174 145 l 174 391 b 217 434 174 415 193 434 l 391 434 b 434 391 415 434 434 415 z m 434 564 b 391 521 434 540 415 521 l 217 521 b 174 564 193 521 174 540 l 174 738 b 304 868 174 810 232 868 b 434 738 376 868 434 810 z m 825 239 b 673 87 825 155 757 87 b 521 239 589 87 521 155 l 521 391 b 564 434 521 415 540 434 l 781 434 b 825 391 805 434 825 415 z m 825 564 b 781 521 825 540 805 521 l 564 521 b 521 564 540 521 521 540 l 521 803 b 673 955 521 887 589 955 b 825 803 757 955 825 887 z m 1215 239 b 1063 87 1215 155 1147 87 b 911 239 979 87 911 155 l 911 391 b 955 434 911 415 931 434 l 1172 434 b 1215 391 1196 434 1215 415 z m 1215 564 b 1172 521 1215 540 1196 521 l 955 521 b 911 564 931 521 911 540 l 911 803 b 1063 955 911 887 979 955 b 1215 803 1147 955 1215 887 z m 1563 217 b 1432 87 1563 145 1504 87 b 1302 217 1360 87 1302 145 l 1302 391 b 1345 434 1302 415 1322 434 l 1519 434 b 1563 391 1543 434 1563 415 z m 1563 564 b 1519 521 1563 540 1543 521 l 1345 521 b 1302 564 1322 521 1302 540 l 1302 738 b 1432 868 1302 810 1360 868 b 1563 738 1504 868 1563 810 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1476 1215 l 260 1215 b 0 955 117 1215 0 1099 l 0 781 b 174 608 0 685 78 608 l 1563 608 b 1736 781 1658 608 1736 685 l 1736 955 b 1476 1215 1736 1099 1620 1215 z m 434 738 b 391 694 434 714 415 694 l 217 694 b 174 738 193 694 174 714 l 174 825 b 304 955 174 897 232 955 b 434 825 376 955 434 897 z m 825 738 b 781 694 825 714 805 694 l 564 694 b 521 738 540 694 521 714 l 521 890 b 673 1042 521 974 589 1042 b 825 890 757 1042 825 974 z m 1215 738 b 1172 694 1215 714 1196 694 l 955 694 b 911 738 931 694 911 714 l 911 890 b 1063 1042 911 974 979 1042 b 1215 890 1147 1042 1215 974 z m 1563 738 b 1519 694 1563 714 1543 694 l 1345 694 b 1302 738 1322 694 1302 714 l 1302 825 b 1432 955 1302 897 1360 955 b 1563 825 1504 955 1563 897 z m 1563 347 l 174 347 b 0 174 78 347 0 270 l 0 87 b 260 -174 0 -57 117 -174 l 1476 -174 b 1736 87 1620 -174 1736 -57 l 1736 174 b 1563 347 1736 270 1658 347 z m 434 130 b 304 0 434 58 376 0 b 174 130 232 0 174 58 l 174 217 b 217 260 174 241 193 260 l 391 260 b 434 217 415 260 434 241 z m 825 152 b 673 0 825 68 757 0 b 521 152 589 0 521 68 l 521 217 b 564 260 521 241 540 260 l 781 260 b 825 217 805 260 825 241 z m 1215 152 b 1063 0 1215 68 1147 0 b 911 152 979 0 911 68 l 911 217 b 955 260 911 241 931 260 l 1172 260 b 1215 217 1196 260 1215 241 z m 1563 130 b 1432 0 1563 58 1504 0 b 1302 130 1360 0 1302 58 l 1302 217 b 1345 260 1302 241 1322 260 l 1519 260 b 1563 217 1543 260 1563 241 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 561 550 b 405 349 464 522 399 439 b 540 433 436 384 483 414 z m 260 415 l 174 902 b 174 909 173 906 175 909 b 687 1042 331 996 508 1042 b 700 1042 692 1042 696 1042 b 840 1112 738 1078 786 1104 b 995 1127 892 1120 944 1124 b 882 1200 968 1164 929 1192 b 687 1215 817 1210 752 1215 b 90 1060 479 1215 274 1163 b 3 872 24 1024 -11 948 l 89 385 b 644 0 128 167 459 0 b 671 2 653 0 662 1 b 564 185 634 48 592 112 b 260 415 434 220 276 325 z m 525 788 b 513 819 523 799 518 809 b 418 769 491 795 457 776 b 311 783 378 762 340 768 b 311 750 309 772 309 762 b 437 663 321 691 377 652 b 525 788 496 673 535 729 z m 1646 887 b 854 1027 1405 1021 1127 1070 b 707 879 780 1015 721 955 l 621 392 b 1051 -170 580 156 880 -140 b 1647 212 1222 -200 1605 -24 l 1733 699 b 1646 887 1747 775 1712 851 z m 905 625 b 1031 713 915 684 972 723 b 1119 587 1090 702 1129 646 b 1108 556 1117 576 1113 566 b 1012 606 1085 580 1051 599 b 905 592 972 613 934 607 b 905 625 903 603 903 614 z m 1096 86 b 854 392 945 112 842 246 b 1124 246 910 323 1007 267 b 1428 291 1242 225 1352 245 b 1096 86 1390 150 1247 59 z m 1449 496 b 1354 546 1427 520 1393 539 b 1247 532 1314 553 1276 547 b 1247 565 1245 542 1245 553 b 1372 652 1257 623 1313 663 b 1461 527 1431 642 1471 586 b 1449 496 1459 516 1454 506 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 1042 694 l 868 694 l 868 797 b 1042 1042 969 833 1042 928 l 868 1042 l 868 1128 b 781 1215 868 1176 829 1215 l 260 1215 b 174 1128 212 1215 174 1176 l 174 1042 l 0 1042 b 174 797 0 928 73 833 l 174 694 l 0 694 b 174 450 0 581 73 486 l 174 347 l 0 347 b 182 100 0 231 77 134 b 521 -174 215 -56 354 -174 b 860 100 687 -174 826 -56 b 1042 347 965 134 1042 231 l 868 347 l 868 450 b 1042 694 969 486 1042 581 z m 521 87 b 391 217 449 87 391 145 b 521 347 391 289 449 347 b 651 217 593 347 651 289 b 521 87 651 145 593 87 z m 521 434 b 391 564 449 434 391 492 b 521 694 391 636 449 694 b 651 564 593 694 651 636 b 521 434 651 492 593 434 z m 521 781 b 391 911 449 781 391 840 b 521 1042 391 983 449 1042 b 651 911 593 1042 651 983 b 521 781 651 840 593 781 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1693 608 l 1649 608 l 1649 781 b 1563 868 1649 829 1610 868 l 1363 868 l 1137 1150 b 1002 1215 1108 1186 1048 1215 l 694 1215 b 608 1128 647 1215 608 1176 l 608 868 l 130 868 b 87 825 106 868 87 849 l 87 608 l 43 608 b 0 564 19 608 0 588 l 0 477 b 43 434 0 453 19 434 l 89 434 b 434 608 168 539 292 608 b 779 434 576 608 700 539 l 957 434 b 1302 608 1036 539 1161 608 b 1647 434 1444 608 1568 539 l 1693 434 b 1736 477 1717 434 1736 453 l 1736 564 b 1693 608 1736 588 1717 608 z m 781 868 l 781 1042 l 1002 1042 l 1141 868 z m 1606 260 l 1592 260 b 1568 317 1586 280 1578 299 l 1578 327 b 1578 388 1595 344 1595 371 l 1517 450 b 1456 450 1500 467 1472 467 l 1446 440 b 1389 463 1431 448 1405 458 l 1389 477 b 1345 521 1389 501 1369 521 l 1259 521 b 1215 477 1235 521 1215 501 l 1215 463 b 1159 440 1195 457 1177 450 l 1149 450 b 1087 450 1132 467 1104 467 l 1026 388 b 1026 327 1009 371 1009 344 l 1036 317 b 1012 260 1028 302 1017 277 l 998 260 b 955 217 974 260 955 241 l 955 130 b 998 87 955 106 974 87 l 1012 87 b 1036 30 1018 67 1026 48 l 1026 20 b 1026 -41 1009 3 1009 -24 l 1087 -103 b 1149 -103 1104 -120 1132 -120 l 1159 -93 b 1215 -116 1177 -102 1195 -110 l 1215 -130 b 1259 -174 1215 -154 1235 -174 l 1346 -174 b 1389 -130 1369 -174 1389 -154 l 1389 -116 b 1446 -93 1409 -110 1428 -102 l 1456 -103 b 1517 -103 1473 -120 1500 -120 l 1578 -41 b 1578 20 1595 -24 1595 3 l 1568 30 b 1592 87 1576 45 1587 71 l 1606 87 b 1649 130 1630 87 1649 106 l 1649 217 b 1606 260 1649 241 1630 260 z m 1302 43 b 1172 174 1230 43 1172 102 b 1302 304 1172 246 1230 304 b 1432 174 1374 304 1432 246 b 1302 43 1432 102 1374 43 z m 738 260 l 724 260 b 700 317 718 280 710 299 l 710 327 b 710 388 727 344 727 371 l 649 450 b 587 450 632 467 604 467 l 578 440 b 521 463 563 448 537 458 l 521 477 b 477 521 521 501 501 521 l 391 521 b 347 477 367 521 347 501 l 347 463 b 291 440 327 457 308 450 l 281 450 b 219 450 264 467 236 467 l 158 388 b 158 327 141 372 141 344 l 168 317 b 144 260 160 302 149 277 l 130 260 b 87 217 106 260 87 241 l 87 130 b 130 87 87 106 106 87 l 144 87 b 168 30 150 67 158 48 l 158 20 b 158 -41 141 3 141 -24 l 219 -103 b 281 -103 236 -120 264 -120 l 291 -93 b 347 -116 308 -102 327 -110 l 347 -130 b 391 -174 347 -154 367 -174 l 477 -174 b 521 -130 501 -174 521 -154 l 521 -116 b 578 -93 541 -110 560 -102 l 588 -103 b 649 -103 604 -120 632 -120 l 710 -41 b 710 20 727 -24 727 3 l 700 30 b 724 87 708 45 719 71 l 738 87 b 781 130 762 87 781 106 l 781 217 b 738 260 781 241 762 260 z m 434 43 b 304 174 362 43 304 102 b 434 304 304 246 362 304 b 564 174 506 304 564 246 b 434 43 564 102 506 43 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1693 434 l 1649 434 l 1649 608 b 1563 694 1649 656 1610 694 l 1432 694 l 1137 1063 b 1002 1128 1108 1099 1048 1128 l 694 1128 b 608 1042 647 1128 608 1090 l 608 694 l 174 694 b 87 608 126 694 87 656 l 87 434 l 43 434 b 0 391 19 434 0 415 l 0 304 b 43 260 0 280 19 260 l 178 260 b 174 217 176 246 174 232 b 477 -87 174 49 310 -87 b 781 217 645 -87 781 49 b 777 260 781 232 779 246 l 959 260 b 955 217 957 246 955 232 b 1259 -87 955 49 1091 -87 b 1563 217 1427 -87 1563 49 b 1558 260 1563 232 1560 246 l 1693 260 b 1736 304 1717 260 1736 280 l 1736 391 b 1693 434 1736 415 1717 434 z m 781 955 l 1002 955 l 1210 694 l 781 694 z m 477 87 b 347 217 406 87 347 145 b 477 347 347 289 406 347 b 608 217 549 347 608 289 b 477 87 608 145 549 87 z m 1259 87 b 1128 217 1187 87 1128 145 b 1259 347 1128 289 1187 347 b 1389 217 1330 347 1389 289 b 1259 87 1389 145 1330 87 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 427 477 l 528 477 l 477 622 z m 955 521 b 890 456 919 521 890 492 b 955 391 890 420 919 391 b 1020 456 991 391 1020 420 b 955 521 1020 492 991 521 z m 1259 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 130 b 130 0 0 58 58 0 l 1259 0 b 1389 130 1331 0 1389 58 l 1389 911 b 1259 1042 1389 983 1331 1042 z m 680 260 l 634 260 b 593 290 615 260 599 272 l 573 347 l 382 347 l 362 290 b 321 260 356 273 338 260 l 321 260 l 275 260 b 234 318 245 260 224 290 l 380 738 b 441 781 388 762 416 781 l 441 781 l 514 781 l 514 781 b 575 737 539 781 567 762 l 721 318 b 680 260 731 290 710 260 z m 1150 304 b 1107 260 1150 280 1131 260 l 1063 260 b 1031 276 1050 260 1039 267 b 955 260 1007 266 982 260 b 760 456 847 260 760 348 b 955 651 760 563 847 651 b 1020 639 978 651 1000 646 l 1020 738 b 1063 781 1020 762 1039 781 l 1107 781 b 1150 738 1131 781 1150 762 z "
	},
	"": {
		ha: 868,
		x_min: 0,
		x_max: 0,
		o: "m 803 521 l 682 521 b 781 825 739 613 781 723 b 434 1215 781 1064 626 1215 b 87 825 242 1215 87 1064 b 186 521 87 723 129 613 l 65 521 b 0 456 29 521 0 492 l 0 369 b 65 304 0 333 29 304 l 326 304 l 326 -109 b 391 -174 326 -144 355 -174 l 477 -174 b 543 -109 513 -174 543 -144 l 543 304 l 803 304 b 868 369 839 304 868 333 l 868 456 b 803 521 868 492 839 521 z m 434 998 b 564 825 514 998 564 932 b 434 551 564 731 491 613 b 304 825 377 613 304 731 b 434 998 304 932 354 998 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1215 243 l 1215 1146 b 1146 1215 1215 1189 1189 1215 l 260 1215 b 0 955 113 1215 0 1102 l 0 87 b 260 -174 0 -61 113 -174 l 1146 -174 b 1215 -104 1181 -174 1215 -148 l 1215 -61 b 1189 -9 1215 -43 1207 -26 b 1189 191 1181 35 1181 156 b 1215 243 1207 200 1215 217 z m 391 825 b 434 868 391 849 410 868 l 564 868 l 564 998 b 608 1042 564 1022 584 1042 l 694 1042 b 738 998 718 1042 738 1022 l 738 868 l 868 868 b 911 825 892 868 911 849 l 911 738 b 868 694 911 714 892 694 l 738 694 l 738 391 b 694 347 738 367 718 347 l 608 347 b 564 391 584 347 564 367 l 564 694 l 434 694 b 391 738 410 694 391 714 z m 1033 0 l 260 0 b 174 87 208 0 174 35 b 260 174 174 139 217 174 l 1033 174 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1345 608 b 955 217 1130 608 955 433 b 1345 -174 955 1 1130 -174 b 1736 217 1561 -174 1736 1 b 1345 608 1736 433 1561 608 z m 1519 200 b 1493 174 1519 185 1507 174 l 1328 174 b 1302 200 1314 174 1302 185 l 1302 408 b 1328 434 1302 422 1314 434 l 1363 434 b 1389 408 1377 434 1389 422 l 1389 260 l 1493 260 b 1519 234 1507 260 1519 249 z m 1345 694 b 1389 692 1360 694 1375 694 l 1389 825 b 1259 955 1389 894 1328 955 l 1042 955 l 1042 1085 b 911 1215 1042 1155 981 1215 l 477 1215 b 347 1085 408 1215 347 1155 l 347 955 l 130 955 b 0 825 61 955 0 894 l 0 608 l 1072 608 b 1345 694 1149 662 1244 694 z m 868 955 l 521 955 l 521 1042 l 868 1042 z m 887 347 l 564 347 b 521 391 540 347 521 367 l 521 521 l 0 521 l 0 130 b 130 0 0 61 61 0 l 921 0 b 868 217 887 65 868 139 b 887 347 868 262 875 306 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1671 694 l 1302 694 l 1302 1150 b 1237 1215 1302 1186 1273 1215 l 846 1215 b 781 1150 810 1215 781 1186 l 781 955 l 608 955 l 608 1172 b 564 1215 608 1196 588 1215 l 521 1215 b 477 1172 497 1215 477 1196 l 477 955 l 304 955 l 304 1172 b 260 1215 304 1196 284 1215 l 217 1215 b 174 1172 193 1215 174 1196 l 174 955 l 65 955 b 0 890 29 955 0 926 l 0 -87 b 87 -174 0 -135 39 -174 l 1649 -174 b 1736 -87 1697 -174 1736 -135 l 1736 629 b 1671 694 1736 665 1707 694 z m 347 119 b 315 87 347 101 333 87 l 206 87 b 174 119 188 87 174 101 l 174 228 b 206 260 174 246 188 260 l 315 260 b 347 228 333 260 347 246 z m 347 380 b 315 347 347 362 333 347 l 206 347 b 174 380 188 347 174 362 l 174 488 b 206 521 174 506 188 521 l 315 521 b 347 488 333 521 347 506 z m 347 640 b 315 608 347 622 333 608 l 206 608 b 174 640 188 608 174 622 l 174 749 b 206 781 174 767 188 781 l 315 781 b 347 749 333 781 347 767 z m 694 119 b 662 87 694 101 680 87 l 553 87 b 521 119 535 87 521 101 l 521 228 b 553 260 521 246 535 260 l 662 260 b 694 228 680 260 694 246 z m 694 380 b 662 347 694 362 680 347 l 553 347 b 521 380 535 347 521 362 l 521 488 b 553 521 521 506 535 521 l 662 521 b 694 488 680 521 694 506 z m 694 640 b 662 608 694 622 680 608 l 553 608 b 521 640 535 608 521 622 l 521 749 b 553 781 521 767 535 781 l 662 781 b 694 749 680 781 694 767 z m 1128 380 b 1096 347 1128 362 1114 347 l 987 347 b 955 380 969 347 955 362 l 955 488 b 987 521 955 506 969 521 l 1096 521 b 1128 488 1114 521 1128 506 z m 1128 640 b 1096 608 1128 622 1114 608 l 987 608 b 955 640 969 608 955 622 l 955 749 b 987 781 955 767 969 781 l 1096 781 b 1128 749 1114 781 1128 767 z m 1128 901 b 1096 868 1128 883 1114 868 l 987 868 b 955 901 969 868 955 883 l 955 1009 b 987 1042 955 1027 969 1042 l 1096 1042 b 1128 1009 1114 1042 1128 1027 z m 1563 119 b 1530 87 1563 101 1548 87 l 1421 87 b 1389 119 1403 87 1389 101 l 1389 228 b 1421 260 1389 246 1403 260 l 1530 260 b 1563 228 1548 260 1563 246 z m 1563 380 b 1530 347 1563 362 1548 347 l 1421 347 b 1389 380 1403 347 1389 362 l 1389 488 b 1421 521 1389 506 1403 521 l 1530 521 b 1563 488 1548 521 1563 506 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1128 b 0 564 311 1128 0 876 b 155 210 0 430 58 307 b 6 -50 121 73 7 -49 b 0 -65 3 -54 0 -60 b 22 -87 0 -77 10 -87 l 22 -87 b 403 53 201 -87 336 -1 b 694 0 492 19 590 0 b 1389 564 1078 0 1389 253 b 694 1128 1389 876 1078 1128 z m 760 308 l 760 260 b 716 217 760 236 740 217 l 673 217 b 629 260 649 217 629 236 l 629 309 b 543 340 598 312 569 323 b 539 389 526 351 525 375 l 586 437 b 625 442 597 447 612 448 b 653 436 633 439 643 436 l 741 436 b 764 459 754 436 764 447 b 748 481 764 469 758 479 l 612 520 b 496 649 552 537 503 587 b 629 820 485 736 547 811 l 629 868 b 673 911 629 892 649 911 l 716 911 b 760 868 740 911 760 892 l 760 820 b 846 789 791 816 820 806 b 850 739 863 778 864 753 l 802 691 b 764 686 792 681 777 680 b 737 692 757 689 745 692 l 736 692 l 647 692 b 625 669 635 692 625 682 b 641 647 625 659 631 650 l 777 608 b 893 479 837 591 885 541 b 760 308 904 392 842 318 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1128 694 b 564 1128 1128 934 876 1128 b 0 694 253 1128 0 934 b 103 445 0 601 38 516 b 6 297 67 363 7 298 b 0 282 3 294 0 287 b 22 260 0 270 10 260 l 22 260 b 262 328 121 260 203 294 b 564 260 350 285 453 260 b 1128 694 876 260 1128 455 z m 521 434 l 521 478 b 436 509 490 480 460 490 b 434 542 425 517 425 533 l 466 572 b 494 574 474 580 485 580 b 528 564 504 568 516 564 l 605 564 b 637 600 622 564 637 580 b 613 635 637 616 627 630 l 491 671 b 405 789 440 686 405 735 b 521 911 405 855 457 909 l 521 955 b 543 977 521 967 531 977 l 586 977 b 608 955 598 977 608 967 l 608 911 b 693 880 638 909 668 898 b 694 847 703 872 704 856 l 662 817 b 635 815 655 809 644 809 b 600 825 624 821 612 825 l 524 825 b 492 789 506 825 492 809 b 516 754 492 773 502 759 l 638 718 b 723 600 688 703 723 654 b 608 478 723 533 672 479 l 608 434 b 586 412 608 422 598 412 l 543 412 b 521 434 531 412 521 422 z m 1459 98 b 1563 347 1524 168 1563 254 b 1212 749 1563 529 1417 684 b 1215 694 1214 731 1215 713 b 564 174 1215 407 923 174 b 478 179 535 174 506 176 b 998 -87 564 23 764 -87 b 1300 -19 1109 -87 1213 -62 b 1541 -87 1359 -54 1442 -87 b 1561 -74 1549 -87 1557 -82 b 1557 -50 1564 -66 1563 -56 b 1459 98 1556 -49 1496 16 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 955 868 l 694 868 l 694 1128 b 608 1215 694 1176 656 1215 l 434 1215 b 347 1128 386 1215 347 1176 l 347 868 l 87 868 b 0 781 39 868 0 829 l 0 608 b 87 521 0 560 39 521 l 347 521 l 347 -87 b 434 -174 347 -135 386 -174 l 608 -174 b 694 -87 656 -174 694 -135 l 694 521 l 955 521 b 1042 608 1003 521 1042 560 l 1042 781 b 955 868 1042 829 1003 868 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1343 605 l 1296 602 b 1177 889 1282 709 1240 807 l 1212 920 b 1214 983 1231 937 1232 965 l 1157 1041 b 1094 1039 1139 1058 1110 1057 l 1063 1003 b 775 1122 981 1066 883 1108 l 778 1169 b 735 1215 780 1194 760 1215 l 654 1215 b 611 1169 629 1215 609 1194 l 613 1122 b 326 1003 506 1108 408 1066 l 295 1039 b 232 1041 279 1057 250 1058 l 175 983 b 177 920 157 965 158 936 l 212 889 b 93 602 149 807 107 709 l 46 605 b 0 561 21 606 0 586 l 0 480 b 46 437 0 455 21 435 l 93 440 b 212 153 107 333 149 235 l 177 122 b 175 59 158 105 157 76 l 232 1 b 295 3 250 -17 279 -16 l 326 38 b 613 -81 408 -24 506 -66 l 611 -127 b 654 -174 609 -152 629 -174 l 735 -174 b 778 -127 760 -174 780 -152 l 775 -81 b 1063 38 883 -66 981 -24 l 1094 3 b 1157 1 1110 -16 1139 -17 l 1214 59 b 1212 122 1232 76 1231 105 l 1177 153 b 1296 440 1240 235 1282 333 l 1343 437 b 1389 480 1368 435 1389 455 l 1389 561 b 1343 605 1389 586 1368 606 z m 765 948 b 947 872 832 937 894 911 l 831 741 b 754 774 808 756 782 767 z m 624 948 l 635 774 b 558 741 607 767 581 756 l 442 872 b 624 948 495 911 557 937 z m 343 773 l 474 658 b 441 580 459 634 448 608 l 267 591 b 343 773 279 658 305 720 z m 267 451 l 441 462 b 474 384 448 434 459 408 l 343 268 b 267 451 305 322 279 383 z m 624 94 b 442 169 557 105 495 131 l 558 300 b 635 268 581 286 607 274 z m 694 434 b 608 521 647 434 608 473 b 694 608 608 569 647 608 b 781 521 742 608 781 569 b 694 434 781 473 742 434 z m 765 94 l 754 268 b 831 300 782 274 808 286 l 947 169 b 765 94 894 131 832 105 z m 1046 268 l 915 384 b 947 462 930 408 941 434 l 1121 451 b 1046 268 1110 383 1084 322 z m 947 580 b 915 658 941 608 930 634 l 1046 773 b 1121 591 1084 720 1110 658 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 477 629 l 911 629 b 955 673 935 629 955 649 l 955 716 b 911 760 955 740 935 760 l 477 760 b 434 716 453 760 434 740 l 434 673 b 477 629 434 649 453 629 z m 434 412 b 477 369 434 388 453 369 l 911 369 b 955 412 935 369 955 388 l 955 456 b 911 499 955 480 935 499 l 477 499 b 434 456 453 499 434 480 z m 694 84 b 567 125 650 84 605 97 l 0 535 l 0 -43 b 130 -174 0 -115 58 -174 l 1259 -174 b 1389 -43 1331 -174 1389 -115 l 1389 535 l 822 125 b 694 84 784 97 739 84 z m 1339 773 b 1259 835 1315 792 1292 810 l 1259 955 b 1128 1085 1259 1027 1200 1085 l 918 1085 b 894 1103 910 1091 902 1097 b 694 1215 848 1136 757 1216 b 495 1103 632 1216 541 1136 b 471 1085 487 1097 479 1091 l 260 1085 b 130 955 189 1085 130 1027 l 130 835 b 50 773 97 810 74 792 b 0 671 22 752 0 706 l 0 642 l 260 454 l 260 955 l 1128 955 l 1128 454 l 1389 642 l 1389 671 b 1339 773 1389 711 1370 749 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1259 868 l 738 868 l 564 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 130 b 130 0 0 58 58 0 l 1259 0 b 1389 130 1331 0 1389 58 l 1389 738 b 1259 868 1389 810 1331 868 z m 998 412 b 955 369 998 388 979 369 l 434 369 b 391 412 410 369 391 388 l 391 456 b 434 499 391 480 410 499 l 955 499 b 998 456 979 499 998 480 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1259 868 l 738 868 l 564 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 130 b 130 0 0 58 58 0 l 1259 0 b 1389 130 1331 0 1389 58 l 1389 738 b 1259 868 1389 810 1331 868 z m 998 412 b 955 369 998 388 979 369 l 760 369 l 760 174 b 716 130 760 150 740 130 l 673 130 b 629 174 649 130 629 150 l 629 369 l 434 369 b 391 412 410 369 391 388 l 391 456 b 434 499 391 480 410 499 l 629 499 l 629 694 b 673 738 629 718 649 738 l 716 738 b 760 694 740 738 760 718 l 760 499 l 955 499 b 998 456 979 499 998 480 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1176 765 l 1450 1069 b 1390 1215 1504 1123 1466 1215 l 86 1215 b 25 1069 10 1215 -29 1123 l 521 521 l 521 98 b 564 11 521 64 537 32 l 781 -152 b 942 -115 838 -194 913 -171 b 781 260 843 -20 781 113 b 1176 765 781 504 950 708 z m 1302 694 b 868 260 1062 694 868 500 b 1302 -174 868 21 1062 -174 b 1736 260 1542 -174 1736 21 b 1302 694 1736 500 1542 694 z m 1345 44 l 1345 0 b 1324 -22 1345 -12 1336 -22 l 1280 -22 b 1259 0 1268 -22 1259 -12 l 1259 44 b 1174 75 1228 46 1198 56 b 1172 108 1163 83 1162 99 l 1204 138 b 1231 140 1211 145 1223 146 b 1266 130 1242 134 1254 130 l 1342 130 b 1374 166 1360 130 1374 146 b 1351 201 1374 182 1365 196 l 1229 237 b 1143 355 1178 252 1143 301 b 1259 477 1143 421 1195 475 l 1259 521 b 1280 543 1259 533 1268 543 l 1324 543 b 1345 521 1336 543 1345 533 l 1345 477 b 1431 446 1376 475 1406 464 b 1432 413 1441 438 1442 422 l 1400 383 b 1373 381 1393 375 1382 375 b 1338 391 1362 387 1350 391 l 1262 391 b 1230 355 1244 391 1230 375 b 1253 320 1230 339 1240 324 l 1376 284 b 1461 166 1426 269 1461 220 b 1345 44 1461 99 1410 45 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1345 260 l 1302 260 l 1302 564 b 1259 608 1302 588 1283 608 l 1215 608 l 1215 825 b 1172 868 1215 849 1196 868 l 1128 868 l 1128 1172 b 1085 1215 1128 1196 1109 1215 b 1042 1172 1061 1215 1042 1196 l 1042 1128 l 868 1128 l 868 1172 b 825 1215 868 1196 849 1215 b 781 1172 801 1215 781 1196 l 781 1128 l 608 1128 l 608 1172 b 564 1215 608 1196 588 1215 b 521 1172 540 1215 521 1196 l 521 1128 l 347 1128 l 347 1172 b 304 1215 347 1196 328 1215 b 260 1172 280 1215 260 1196 l 260 868 l 217 868 b 174 825 193 868 174 849 l 174 608 l 130 608 b 87 564 106 608 87 588 l 87 260 l 43 260 b 0 217 20 260 0 241 l 0 -130 b 43 -174 0 -154 20 -174 l 260 -174 l 260 260 l 347 260 l 347 608 l 434 608 l 434 868 l 521 868 l 521 608 l 434 608 l 434 260 l 347 260 l 347 -174 l 564 -174 l 564 43 b 608 87 564 67 584 87 l 781 87 b 825 43 805 87 825 67 l 825 -174 l 1042 -174 l 1042 260 l 955 260 l 955 608 l 868 608 l 868 868 l 955 868 l 955 608 l 1042 608 l 1042 260 l 1128 260 l 1128 -174 l 1345 -174 b 1389 -130 1369 -174 1389 -154 l 1389 217 b 1345 260 1389 241 1369 260 z m 629 738 b 673 781 629 762 649 781 l 716 781 b 760 738 740 781 760 762 l 760 608 l 629 608 z m 781 260 l 608 260 l 608 434 b 651 477 608 458 627 477 l 738 477 b 781 434 762 477 781 458 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1382 382 b 1302 434 1368 414 1337 434 l 1128 434 l 1128 998 b 1020 1107 1128 1058 1080 1107 b 911 998 960 1107 911 1058 l 911 635 b 884 608 911 620 899 608 l 830 608 b 803 635 815 608 803 620 l 803 1107 b 694 1215 803 1166 754 1215 b 586 1107 635 1215 586 1166 l 586 635 b 559 608 586 620 574 608 l 505 608 b 477 635 490 608 477 620 l 477 998 b 369 1107 477 1058 429 1107 b 260 998 309 1107 260 1058 l 260 434 l 87 434 b 7 382 52 434 21 414 b 0 347 3 373 0 357 b 23 288 0 328 10 302 l 302 -10 b 694 -174 399 -114 542 -174 b 1087 -10 847 -174 990 -114 l 1366 288 b 1389 347 1378 302 1389 328 b 1382 382 1389 357 1386 373 z m 694 87 b 434 260 551 87 434 260 b 694 434 434 260 551 434 b 955 260 838 434 955 260 b 694 87 955 260 838 87 z m 694 347 b 608 260 647 347 608 308 b 694 174 608 212 647 174 b 781 260 742 174 781 212 b 694 347 781 308 742 347 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1346 666 l 1048 708 l 1161 991 b 1089 1052 1179 1036 1130 1078 l 834 890 l 742 1181 b 694 1215 734 1204 714 1215 b 647 1181 675 1215 655 1204 l 555 890 l 300 1052 b 228 991 259 1078 210 1036 l 341 708 l 43 666 b 26 572 -5 659 -16 595 l 293 429 l 91 202 b 127 118 61 169 86 118 b 138 120 130 118 134 119 l 432 184 l 421 -121 b 471 -174 420 -153 445 -174 b 510 -154 485 -174 499 -168 l 694 87 l 879 -154 b 918 -174 890 -168 904 -174 b 968 -121 944 -174 969 -153 l 957 184 l 1251 120 b 1298 202 1298 110 1330 166 l 1096 429 l 1363 572 b 1346 666 1405 595 1394 659 z m 918 370 l 778 400 l 783 255 l 694 370 l 606 255 l 611 400 l 471 370 l 567 478 l 440 546 l 582 566 l 528 702 l 650 624 l 694 763 l 739 624 l 861 702 l 806 566 l 949 546 l 822 478 z "
	},
	"": {
		ha: 1476,
		x_min: 0,
		x_max: 0,
		o: "m 1302 260 l 1462 420 b 1476 561 1471 466 1476 513 b 1473 608 1476 577 1474 592 l 1345 608 l 1457 720 b 1165 1160 1418 895 1315 1053 b 1140 1168 1157 1165 1148 1168 b 1103 1148 1125 1168 1111 1160 b 1099 1107 1095 1136 1093 1121 b 1125 968 1116 1063 1125 1016 b 969 660 1125 846 1068 733 b 951 626 958 651 951 639 b 966 591 950 613 955 600 b 1070 273 1055 510 1094 391 b 806 11 1044 142 938 37 l 799 188 l 873 138 b 900 141 881 133 893 134 b 907 156 904 144 907 151 b 904 167 907 160 906 165 l 849 259 l 964 283 b 981 304 974 285 981 294 b 964 325 981 314 974 323 l 849 349 l 904 440 b 900 467 909 449 908 460 b 873 469 893 474 881 475 l 791 413 l 759 1194 b 738 1215 759 1206 749 1215 l 738 1215 b 716 1194 726 1215 716 1206 l 685 413 l 602 469 b 575 467 594 475 583 474 b 572 440 568 460 566 449 l 626 349 l 512 325 b 495 304 502 323 495 314 b 512 283 495 294 502 285 l 626 259 l 571 167 b 575 140 566 159 568 147 b 602 138 582 134 594 133 l 676 188 l 669 11 b 398 342 515 43 398 179 b 510 591 398 437 438 526 b 525 626 520 600 525 613 b 507 659 524 639 518 651 b 350 968 407 733 350 846 b 377 1107 350 1016 359 1062 b 372 1148 382 1121 381 1136 b 336 1168 364 1160 350 1168 b 311 1160 327 1168 318 1165 b 18 720 159 1052 57 895 l 130 608 l 2 608 b 1 526 0 581 -1 554 b 14 419 2 490 7 454 l 173 260 l 65 260 b 703 -173 177 16 421 -160 b 738 -174 715 -173 726 -174 b 1410 260 1037 -174 1295 5 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1215 243 l 1215 1146 b 1146 1215 1215 1189 1189 1215 l 260 1215 b 0 955 113 1215 0 1102 l 0 87 b 260 -174 0 -61 113 -174 l 1146 -174 b 1215 -104 1181 -174 1215 -148 l 1215 -61 b 1189 -9 1215 -43 1207 -26 b 1189 191 1181 35 1181 156 b 1215 243 1207 200 1215 217 z m 361 824 l 419 766 b 434 760 423 762 428 760 b 449 766 440 760 445 762 b 449 797 458 774 458 788 l 378 868 b 504 1002 405 925 449 971 b 477 911 487 975 477 945 b 542 778 477 857 503 810 b 499 673 515 750 499 714 b 620 524 499 600 551 539 l 623 613 l 595 594 b 589 592 593 593 591 592 b 582 595 586 592 584 593 b 578 603 580 597 578 600 b 580 608 578 604 579 607 l 603 647 l 554 657 b 546 668 549 658 546 663 b 554 679 546 673 549 677 l 603 689 l 580 727 b 582 741 577 732 578 737 b 595 742 585 744 591 745 l 628 720 l 640 1020 b 651 1031 640 1026 645 1031 b 662 1020 657 1031 662 1026 l 675 716 l 713 742 b 719 744 715 743 717 744 b 727 741 722 744 725 743 b 730 733 729 739 730 736 b 729 727 730 731 729 729 l 705 689 l 754 679 b 763 668 759 677 763 673 b 754 657 763 663 759 658 l 705 647 l 729 608 b 727 595 731 604 730 598 b 719 592 725 593 722 592 b 713 594 717 592 715 593 l 679 617 l 683 524 b 803 673 751 539 803 600 b 760 778 803 714 787 750 b 825 911 799 810 825 857 b 799 1002 825 945 815 975 b 924 868 853 971 897 925 l 853 797 b 853 766 844 788 844 774 b 868 760 857 762 863 760 b 883 766 874 760 879 762 l 941 824 b 955 738 949 796 955 768 b 954 734 955 736 954 735 l 854 646 b 846 629 850 642 846 635 b 852 615 846 625 849 618 b 868 608 855 611 863 608 l 868 608 b 882 613 873 608 878 609 l 946 669 b 651 434 915 535 795 434 b 356 669 507 434 387 535 l 420 613 b 434 608 423 610 429 608 l 434 608 b 450 615 440 608 446 610 b 448 646 458 624 457 638 l 348 734 b 347 738 348 735 347 736 b 361 824 347 768 353 796 z m 1033 0 l 260 0 b 174 87 208 0 174 35 b 260 174 174 139 217 174 l 1033 174 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1503 989 l 864 1202 b 781 1215 841 1209 805 1215 b 699 1202 758 1215 721 1209 l 59 989 b 0 906 27 978 0 941 l 0 773 l 719 989 b 844 989 760 1001 803 1001 l 1563 773 l 1563 906 b 1503 989 1563 944 1539 977 z m 744 906 l 0 683 l 0 63 b 68 -21 0 23 28 -13 l 725 -167 b 781 -174 740 -171 765 -174 b 838 -167 797 -174 822 -171 l 1495 -21 b 1563 63 1534 -13 1563 23 l 1563 683 l 819 906 b 744 906 794 913 768 913 z m 347 591 b 331 570 347 581 341 573 l 114 511 b 87 532 100 507 87 518 l 87 577 b 103 598 87 587 93 595 l 320 657 b 347 636 334 661 347 650 z m 738 698 b 722 677 738 688 731 679 l 461 606 b 434 627 448 602 434 612 l 434 672 b 450 693 434 681 441 690 l 710 764 b 738 743 724 767 738 757 z m 1215 636 b 1243 657 1215 650 1229 661 l 1460 598 b 1476 577 1469 595 1476 587 l 1476 532 b 1448 511 1476 518 1462 507 l 1231 570 b 1215 591 1222 573 1215 582 l 1215 591 z m 825 743 b 852 764 825 757 838 767 l 1112 692 b 1128 672 1122 690 1128 681 l 1128 627 b 1101 606 1128 612 1115 602 l 841 677 b 825 698 832 679 825 688 l 825 698 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1128 1036 b 1075 1028 1111 1046 1089 1043 b 1064 998 1069 1021 1064 1008 b 1070 975 1064 991 1067 981 b 1139 738 1115 904 1139 822 b 945 379 1138 594 1066 459 l 737 234 l 737 289 l 857 387 b 865 428 868 398 870 414 l 843 475 b 998 738 935 527 998 625 b 850 997 998 848 938 944 l 864 1028 b 854 1072 870 1043 866 1061 l 694 1215 l 534 1072 b 524 1029 522 1061 518 1044 l 539 997 b 390 738 450 944 390 848 b 545 475 390 625 453 527 l 524 428 b 531 387 518 414 521 398 l 651 289 l 651 235 l 443 379 b 250 738 323 459 251 594 b 319 975 250 822 274 904 b 312 1027 329 992 325 1012 b 257 1034 298 1041 274 1044 b 36 412 44 918 -60 658 b 119 279 55 364 85 319 l 271 102 b 324 92 284 87 306 82 l 539 206 l 618 151 l 491 62 b 477 65 486 63 482 65 b 412 0 441 65 412 36 b 477 -65 412 -36 441 -65 b 540 -9 510 -65 536 -40 l 651 68 l 651 -13 b 607 -87 625 -28 607 -55 b 694 -174 607 -135 646 -174 b 781 -87 742 -174 781 -135 b 737 -13 781 -55 763 -28 l 737 68 l 848 -9 b 912 -65 853 -40 879 -65 b 977 0 948 -65 977 -36 b 912 65 977 36 948 65 b 898 62 907 65 903 63 l 770 151 l 850 206 l 1065 92 b 1118 102 1083 82 1105 87 l 1261 268 b 1373 473 1312 328 1353 397 b 1128 1036 1431 700 1328 927 z m 868 738 b 789 593 868 677 836 624 l 774 627 b 774 832 750 693 750 765 l 795 878 b 868 738 839 847 868 796 z m 521 738 b 593 878 521 796 549 847 l 614 832 b 614 627 638 765 638 693 l 599 593 b 521 738 552 624 521 677 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1361 965 l 725 1210 b 694 1215 717 1213 703 1215 b 664 1210 686 1215 672 1213 l 28 965 b 0 925 13 960 0 941 l 0 825 b 43 781 0 801 19 781 l 1345 781 b 1389 825 1369 781 1389 801 l 1389 925 b 1361 965 1389 943 1378 959 z m 174 694 l 174 260 l 130 260 b 87 217 106 260 87 241 l 87 87 l 1302 87 l 1302 217 b 1259 260 1302 241 1283 260 l 1215 260 l 1215 694 l 1042 694 l 1042 260 l 781 260 l 781 694 l 608 694 l 608 260 l 347 260 l 347 694 z m 1345 0 l 43 0 b 0 -43 19 0 0 -19 l 0 -130 b 43 -174 0 -154 19 -174 l 1345 -174 b 1389 -130 1369 -174 1389 -154 l 1389 -43 b 1345 0 1389 -19 1369 0 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 434 0 b 260 87 365 0 295 61 b 0 278 87 208 35 252 l 0 -87 b 87 -174 0 -135 39 -174 l 781 -174 b 868 -87 829 -174 868 -135 l 868 278 b 608 87 833 252 781 208 b 434 0 573 61 503 0 z m 781 521 l 87 521 b 0 434 39 521 0 482 l 0 391 b 313 156 69 339 61 339 b 434 87 339 139 391 87 b 556 148 477 87 530 139 b 868 382 807 330 799 330 l 868 434 b 781 521 868 482 829 521 z m 1476 781 l 608 781 b 521 694 560 781 521 742 l 521 608 l 781 608 b 954 451 871 608 946 539 l 955 451 l 955 87 l 1476 87 b 1563 174 1524 87 1563 126 l 1563 694 b 1476 781 1563 742 1524 781 z m 1389 434 l 1215 434 l 1215 608 l 1389 608 z m 434 694 b 608 868 434 790 512 868 l 1215 868 l 1215 1128 b 1128 1215 1215 1176 1176 1215 l 260 1215 b 174 1128 212 1215 174 1176 l 174 608 l 434 608 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 391 868 l 304 868 b 260 825 280 868 260 849 l 260 434 l 434 434 l 434 825 b 391 868 434 849 415 868 z m 651 868 l 564 868 b 521 825 540 868 521 849 l 521 434 l 694 434 l 694 825 b 651 868 694 849 675 868 z m 1172 868 l 1085 868 b 1042 825 1061 868 1042 849 l 1042 434 l 1215 434 l 1215 825 b 1172 868 1215 849 1196 868 z m 1432 868 l 1345 868 b 1302 825 1322 868 1302 849 l 1302 434 l 1476 434 l 1476 825 b 1432 868 1476 849 1456 868 z m 1649 955 b 1736 1042 1697 955 1736 994 b 1649 1215 1736 1090 1649 1215 b 1563 1042 1649 1215 1563 1090 b 1649 955 1563 994 1601 955 z m 1389 955 b 1476 1042 1437 955 1476 994 b 1389 1215 1476 1090 1389 1215 b 1302 1042 1389 1215 1302 1090 b 1389 955 1302 994 1341 955 z m 1128 955 b 1215 1042 1176 955 1215 994 b 1128 1215 1215 1090 1128 1215 b 1042 1042 1128 1215 1042 1090 b 1128 955 1042 994 1081 955 z m 868 955 b 955 1042 916 955 955 994 b 868 1215 955 1090 868 1215 b 781 1042 868 1215 781 1090 b 868 955 781 994 820 955 z m 608 955 b 694 1042 656 955 694 994 b 608 1215 694 1090 608 1215 b 521 1042 608 1215 521 1090 b 608 955 521 994 560 955 z m 347 955 b 434 1042 395 955 434 994 b 347 1215 434 1090 347 1215 b 260 1042 347 1215 260 1090 b 347 955 260 994 299 955 z m 87 955 b 174 1042 135 955 174 994 b 87 1215 174 1090 87 1215 b 0 1042 87 1215 0 1090 b 87 955 0 994 39 955 z m 1563 434 b 1476 347 1563 386 1524 347 l 955 347 l 955 825 b 911 868 955 849 935 868 l 825 868 b 781 825 801 868 781 849 l 781 347 l 260 347 b 174 434 212 347 174 386 l 174 825 b 130 868 174 849 154 868 l 43 868 b 0 825 19 868 0 849 l 0 434 b 260 174 0 290 117 174 l 781 174 l 781 0 l 304 0 b 260 -43 280 0 260 -19 l 260 -130 b 304 -174 260 -154 280 -174 l 1432 -174 b 1476 -130 1456 -174 1476 -154 l 1476 -43 b 1432 0 1476 -19 1456 0 l 955 0 l 955 174 l 1476 174 b 1736 434 1620 174 1736 290 l 1736 825 b 1693 868 1736 849 1717 868 l 1606 868 b 1563 825 1582 868 1563 849 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 0 -87 b 87 -174 0 -135 39 -174 l 260 -174 b 347 -87 308 -174 347 -135 l 347 781 l 0 781 z m 1571 434 b 1649 594 1620 481 1649 535 b 1411 927 1649 737 1536 848 b 1107 1188 1297 999 1192 1084 l 1085 1215 l 1063 1188 b 759 927 978 1084 873 999 b 521 594 634 848 521 737 b 599 434 521 535 551 481 z m 1649 347 l 521 347 b 434 260 473 347 434 308 l 434 -87 b 521 -174 434 -135 473 -174 l 608 -174 l 608 0 b 694 87 608 48 647 87 b 781 0 742 87 781 48 l 781 -174 l 955 -174 l 955 22 b 1085 217 955 152 1085 217 b 1215 22 1085 217 1215 152 l 1215 -174 l 1389 -174 l 1389 0 b 1476 87 1389 48 1428 87 b 1563 0 1524 87 1563 48 l 1563 -174 l 1649 -174 b 1736 -87 1697 -174 1736 -135 l 1736 260 b 1649 347 1736 308 1697 347 z m 174 1215 b 0 955 174 1215 0 1128 l 0 868 l 347 868 l 347 955 b 174 1215 347 1128 174 1215 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 978 1050 b 998 1042 983 1045 992 1042 b 1018 1050 1005 1042 1014 1045 l 1077 1108 b 1085 1128 1081 1113 1085 1122 b 1077 1148 1085 1135 1081 1144 l 1018 1207 b 978 1207 1007 1218 989 1218 l 920 1148 b 911 1128 915 1144 911 1135 b 920 1108 911 1122 915 1113 z m 1118 694 b 926 615 1046 694 977 666 l 861 549 b 792 521 842 531 818 521 l 665 521 b 686 725 693 581 703 651 b 443 951 658 840 560 931 b 182 873 345 967 250 937 b 185 805 162 854 163 821 l 256 752 b 308 752 272 740 293 740 b 390 781 331 771 360 781 b 521 651 462 781 521 723 b 390 521 521 579 462 521 l 304 521 b 265 458 271 521 250 487 l 309 370 b 344 347 315 357 329 348 l 434 347 b 608 174 530 347 608 269 b 434 0 608 78 530 0 b 40 250 173 0 102 147 b 0 235 28 270 0 259 b 434 -174 0 87 112 -174 b 781 174 625 -174 781 -18 b 733 347 781 237 763 296 l 792 347 b 983 427 864 347 932 375 l 1049 492 b 1118 521 1067 511 1092 521 b 1215 424 1172 521 1215 477 l 1215 152 b 1128 87 1215 116 1164 87 b 906 203 1022 87 948 153 b 868 190 894 219 868 210 l 868 87 b 1128 -87 868 87 868 -87 b 1389 152 1260 -87 1389 20 l 1389 424 b 1118 694 1389 573 1267 694 z m 1232 1033 b 764 1033 1000 856 774 1025 b 710 1031 748 1045 725 1045 b 700 977 694 1018 690 995 b 1022 781 704 969 809 781 b 1298 883 1239 781 1290 866 b 1302 902 1301 889 1302 896 l 1302 998 b 1232 1033 1302 1034 1261 1054 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1694 272 b 1490 228 1605 306 1539 261 b 1405 197 1443 196 1427 188 b 1361 276 1382 205 1375 222 b 1313 392 1352 312 1341 356 b 1330 415 1319 400 1325 408 b 1443 369 1358 389 1394 369 b 1611 485 1535 369 1581 439 b 1671 543 1640 529 1652 543 b 1736 608 1707 543 1736 572 b 1671 673 1736 644 1707 673 b 1503 557 1579 673 1533 603 b 1443 499 1474 513 1462 499 b 1179 776 1396 499 1341 666 l 1226 869 b 1389 1042 1317 875 1389 949 b 1215 1215 1389 1138 1311 1215 b 1042 1042 1119 1215 1042 1138 b 1070 946 1042 1006 1052 974 l 1021 848 b 868 868 975 860 925 868 b 715 848 811 868 761 860 l 666 946 b 694 1042 684 974 694 1006 b 521 1215 694 1138 617 1215 b 347 1042 425 1215 347 1138 b 510 869 347 949 419 875 l 557 776 b 293 499 395 666 339 499 b 233 557 274 499 262 513 b 65 673 203 603 157 673 b 0 608 29 673 0 644 b 65 543 0 572 29 543 b 125 485 84 543 96 529 b 293 369 155 439 201 369 b 406 415 343 369 379 389 b 423 392 411 408 417 400 b 375 276 395 356 384 312 b 332 197 361 222 354 205 b 246 228 310 188 293 196 b 42 272 197 261 131 306 b 4 188 8 259 -8 222 b 88 150 17 154 55 138 b 173 120 110 159 127 151 b 316 63 210 95 257 63 b 378 75 335 63 356 67 b 501 244 467 109 486 187 b 517 296 507 268 512 284 b 684 203 562 259 617 225 b 521 -43 657 103 597 -43 b 456 -109 485 -43 456 -73 b 521 -174 456 -144 485 -174 b 812 177 702 -174 784 67 b 868 174 830 175 849 174 b 924 177 887 174 906 175 b 1215 -174 952 67 1034 -174 b 1280 -109 1251 -174 1280 -144 b 1215 -43 1280 -73 1251 -43 b 1052 203 1139 -43 1079 103 b 1219 296 1119 225 1174 259 b 1235 244 1224 284 1229 268 b 1358 75 1250 187 1270 109 b 1420 63 1380 67 1401 63 b 1563 120 1479 63 1526 95 b 1648 150 1609 151 1626 159 b 1732 188 1681 138 1719 154 b 1694 272 1745 222 1728 259 z m 1215 1085 b 1259 1042 1239 1085 1259 1066 b 1215 998 1259 1018 1239 998 b 1172 1042 1191 998 1172 1018 b 1215 1085 1172 1066 1191 1085 z m 521 1085 b 564 1042 545 1085 564 1066 b 521 998 564 1018 545 998 b 477 1042 497 998 477 1018 b 521 1085 477 1066 497 1085 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 1172 521 b 1111 282 1172 434 1150 353 l 760 563 l 760 1012 b 1172 521 993 971 1172 766 z m 586 30 b 344 146 494 46 411 87 l 586 340 z m 760 340 l 1002 146 b 760 30 934 87 851 46 z m 586 1012 l 586 563 l 235 282 b 174 521 196 353 174 434 b 586 1012 174 766 352 971 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1684 221 l 1389 347 l 1389 -174 l 1693 -174 b 1736 -130 1717 -174 1736 -154 l 1736 141 b 1684 221 1736 173 1713 208 z m 0 141 l 0 -130 b 43 -174 0 -154 19 -174 l 347 -174 l 347 347 l 53 221 b 0 141 24 208 0 173 z m 1260 546 l 1128 625 l 1128 937 b 1103 998 1128 960 1119 982 l 899 1203 b 837 1203 882 1220 854 1220 l 633 998 b 608 937 617 982 608 960 l 608 625 l 476 546 b 434 472 453 532 434 499 l 434 -174 l 694 -174 l 694 87 b 868 260 694 183 772 260 b 1042 87 964 260 1042 183 l 1042 -174 l 1302 -174 l 1302 472 b 1260 546 1302 502 1286 530 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1085 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -28 58 -87 l 1085 -87 b 1215 43 1157 -87 1215 -28 l 1215 998 b 1085 1128 1215 1070 1157 1128 z m 434 217 b 391 174 434 193 415 174 l 304 174 b 260 217 280 174 260 193 l 260 564 b 304 608 260 588 280 608 l 391 608 b 434 564 415 608 434 588 z m 694 217 b 651 174 694 193 675 174 l 564 174 b 521 217 540 174 521 193 l 521 825 b 564 868 521 849 540 868 l 651 868 b 694 825 675 868 694 849 z m 955 217 b 911 174 955 193 935 174 l 825 174 b 781 217 801 174 781 193 l 781 391 b 825 434 781 415 801 434 l 911 434 b 955 391 935 434 955 415 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1215 43 l 1215 998 b 1085 1128 1215 1070 1157 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -28 58 -87 l 1085 -87 b 1215 43 1157 -87 1215 -28 z m 304 694 b 260 738 280 694 260 714 l 260 825 b 304 868 260 849 280 868 l 651 868 b 694 825 675 868 694 849 l 694 738 b 651 694 694 714 675 694 z m 304 434 b 260 477 280 434 260 453 l 260 564 b 304 608 260 588 280 608 l 911 608 b 955 564 935 608 955 588 l 955 477 b 911 434 955 453 935 434 z m 304 174 b 260 217 280 174 260 193 l 260 304 b 304 347 260 328 280 347 l 477 347 b 521 304 501 347 521 328 l 521 217 b 477 174 521 193 501 174 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 694 868 b 868 1042 790 868 868 946 b 694 1215 868 1138 790 1215 b 521 1042 599 1215 521 1138 b 694 868 521 946 599 868 z m 611 408 b 763 393 649 362 717 356 l 1002 589 b 1017 741 1048 627 1055 695 b 864 757 979 788 911 795 l 709 629 l 603 755 b 429 824 561 805 497 831 b 277 723 364 817 307 779 l 143 474 b 235 138 79 355 119 212 l 357 43 l 109 43 b 0 -65 49 43 0 -5 b 109 -174 0 -125 49 -174 l 673 -174 b 749 12 765 -174 819 -58 l 451 310 l 546 486 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 738 695 b 651 608 690 695 651 656 l 651 391 b 608 347 651 367 632 347 b 564 391 584 347 564 367 l 564 599 b 601 733 564 646 577 692 l 812 1084 b 783 1203 837 1125 824 1178 b 668 1178 743 1227 694 1215 b 666 1177 667 1178 666 1178 l 348 700 b 304 555 319 657 304 607 l 304 338 l 59 256 b 0 174 27 245 0 208 l 0 174 l 0 -87 b 87 -173 0 -116 23 -173 b 109 -171 94 -173 101 -172 l 595 -44 b 825 261 730 -5 825 120 l 825 608 b 738 695 825 656 786 695 z m 1677 256 l 1432 337 l 1432 555 b 1389 699 1432 607 1417 657 l 1070 1177 b 1068 1178 1070 1177 1069 1177 b 954 1203 1042 1215 993 1226 b 924 1084 912 1178 899 1125 l 1135 732 b 1172 598 1159 692 1172 645 l 1172 391 b 1128 347 1172 367 1152 347 b 1085 391 1104 347 1085 367 l 1085 608 b 998 694 1085 655 1046 694 b 911 608 951 694 911 655 l 911 260 b 1141 -44 911 120 1006 -6 l 1627 -171 b 1649 -174 1635 -173 1642 -174 b 1736 -87 1713 -174 1736 -116 l 1736 174 b 1677 256 1736 211 1712 244 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1215 243 l 1215 1146 b 1146 1215 1215 1189 1189 1215 l 260 1215 b 0 955 113 1215 0 1102 l 0 87 b 260 -174 0 -61 113 -174 l 1146 -174 b 1215 -104 1181 -174 1215 -148 l 1215 -61 b 1189 -9 1215 -43 1207 -26 b 1189 191 1181 35 1181 156 b 1215 243 1207 200 1215 217 z m 817 820 b 825 825 818 823 822 825 b 833 820 828 825 831 823 l 863 758 l 931 748 b 936 733 938 747 941 738 l 887 686 l 898 618 b 889 608 899 612 895 608 b 885 609 888 608 887 608 l 825 641 l 764 609 b 760 608 763 608 761 608 b 751 618 755 608 750 612 l 763 686 l 714 733 b 719 748 708 738 711 747 l 786 758 z m 660 1007 b 769 987 697 1007 734 1000 b 788 968 779 987 788 979 b 769 950 788 958 779 950 b 763 951 767 950 764 951 b 752 952 761 951 754 952 b 494 694 610 952 494 836 b 752 437 494 553 610 437 b 763 438 754 437 761 438 b 781 419 773 438 781 429 b 769 402 781 411 776 404 b 660 382 733 389 697 382 b 347 694 487 382 347 522 b 660 1007 347 867 487 1007 z m 1033 0 l 260 0 b 174 87 208 0 174 35 b 260 174 174 139 217 174 l 1033 174 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1370 14 l 1100 285 b 1053 304 1087 297 1071 304 l 1009 304 b 1129 651 1084 400 1129 520 b 564 1215 1129 963 876 1215 b 0 651 253 1215 0 963 b 564 87 0 339 253 87 b 912 206 695 87 816 131 l 912 162 b 931 116 912 145 918 128 l 1201 -154 b 1293 -154 1227 -180 1268 -180 l 1370 -78 b 1370 14 1395 -52 1395 -11 z m 564 261 b 174 651 349 261 174 435 b 564 1042 174 867 348 1042 b 955 651 780 1042 955 867 b 564 261 955 435 780 261 z m 638 674 l 516 711 b 492 745 502 715 492 729 b 524 781 492 765 506 781 l 600 781 b 635 771 613 781 625 778 b 662 773 644 766 655 766 l 694 804 b 693 836 704 813 703 829 b 608 867 668 855 638 866 l 608 911 b 586 933 608 923 598 933 l 543 933 b 521 911 531 933 521 923 l 521 868 b 405 745 457 866 405 812 b 491 628 405 691 440 643 l 613 591 b 637 557 627 587 637 573 b 605 521 637 537 622 521 l 528 521 b 494 531 516 521 504 524 b 466 529 485 536 474 536 l 434 499 b 436 466 425 489 425 474 b 521 435 461 447 490 436 l 521 391 b 543 369 521 379 531 369 l 586 369 b 608 391 598 369 608 379 l 608 434 b 724 557 672 436 724 490 b 638 674 724 611 688 659 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1370 14 l 1100 285 b 1053 304 1087 297 1071 304 l 1009 304 b 1129 651 1084 400 1129 520 b 564 1215 1129 963 876 1215 b 0 651 253 1215 0 963 b 564 87 0 339 253 87 b 912 206 695 87 816 131 l 912 162 b 931 116 912 145 918 128 l 1201 -154 b 1293 -154 1227 -180 1268 -180 l 1370 -78 b 1370 14 1395 -52 1395 -11 z m 564 261 b 174 651 349 261 174 435 b 564 1042 174 867 348 1042 b 955 651 780 1042 955 867 b 564 261 955 435 780 261 z m 564 911 b 364 711 454 911 364 822 b 545 400 364 622 495 459 b 564 391 549 395 558 391 b 584 400 571 391 579 395 b 765 711 634 459 765 622 b 564 911 765 822 675 911 z m 564 651 b 499 716 528 651 499 680 b 564 781 499 752 528 781 b 629 716 600 781 629 752 b 564 651 629 680 600 651 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 582 372 l 781 521 l 781 955 l 347 955 l 347 477 l 112 302 b 33 -43 5 222 -34 73 b 261 -174 82 -128 172 -174 b 417 -122 315 -174 370 -157 l 476 -77 b 582 372 395 73 439 264 z m 781 1128 b 803 1211 781 1158 790 1186 b 781 1215 796 1213 789 1215 l 434 1215 b 347 1128 386 1215 347 1176 l 347 1042 l 781 1042 z m 1302 1215 l 955 1215 b 868 1128 907 1215 868 1176 l 868 1042 l 1389 1042 l 1389 1128 b 1302 1215 1389 1176 1350 1215 z m 868 477 l 634 302 b 555 -43 528 222 488 73 b 781 -174 605 -128 692 -174 b 937 -122 836 -174 891 -157 l 1250 113 b 1389 391 1327 170 1389 295 l 1389 391 l 1389 955 l 868 955 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1550 534 l 1488 595 b 1427 595 1471 612 1444 612 l 1302 470 l 1177 595 b 1116 595 1160 612 1133 612 l 1054 534 b 1054 472 1037 517 1037 489 l 1179 347 l 1054 222 b 1054 161 1037 205 1037 178 l 1116 100 b 1177 100 1133 83 1160 83 l 1302 224 l 1427 100 b 1488 100 1444 83 1471 83 l 1550 161 b 1550 222 1567 178 1567 205 l 1425 347 l 1550 472 b 1550 534 1567 489 1567 517 z m 1497 1215 l 835 1215 b 751 1151 795 1215 761 1189 l 521 275 l 362 563 b 286 608 348 587 314 608 l 65 608 b 0 543 29 608 0 579 l 0 412 b 65 347 0 376 29 347 l 183 347 l 425 -97 b 556 -174 458 -151 505 -174 b 703 -61 603 -174 676 -149 l 969 955 l 1497 955 b 1563 1020 1533 955 1563 984 l 1563 1150 b 1497 1215 1563 1186 1533 1215 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 924 -50 b 899 -51 920 -50 905 -51 b 327 521 583 -51 327 205 b 899 1093 327 836 583 1093 b 924 1091 905 1093 920 1091 b 964 1124 943 1091 960 1105 b 938 1171 968 1144 957 1164 b 694 1215 859 1200 777 1215 b 0 521 312 1215 0 904 b 694 -174 0 138 312 -174 b 937 -129 776 -174 858 -159 b 965 -91 953 -124 965 -109 b 924 -50 965 -69 947 -50 z m 1366 635 l 1159 665 l 1066 853 b 1042 868 1061 863 1051 868 b 1017 853 1032 868 1022 863 l 925 665 l 718 635 b 703 589 695 632 687 605 l 853 443 l 817 237 b 844 205 814 219 828 205 b 856 208 848 205 852 206 l 1042 306 l 1227 208 b 1240 205 1231 206 1235 205 b 1266 237 1255 205 1269 219 l 1231 443 l 1381 589 b 1366 635 1397 605 1388 632 z "
	},
	"": {
		ha: 1259,
		x_min: 0,
		x_max: 0,
		o: "m 1100 521 l 1245 763 b 1158 911 1284 829 1236 911 l 868 911 l 716 1166 b 629 1215 697 1199 663 1215 b 543 1166 596 1215 562 1199 l 391 911 l 101 911 b 14 763 23 911 -25 829 l 158 521 l 14 278 b 101 130 -25 213 23 130 l 391 130 l 543 -125 b 629 -174 562 -157 596 -174 b 716 -125 663 -174 697 -157 l 868 130 l 1158 130 b 1245 278 1236 130 1284 213 z m 1066 760 l 1012 669 l 958 760 z m 924 521 l 782 282 l 477 282 l 335 521 l 477 760 l 782 760 z m 629 1015 l 691 911 l 568 911 z m 193 760 l 300 760 l 247 669 z m 193 282 l 247 372 l 300 282 z m 629 26 l 568 130 l 691 130 z m 958 282 l 1012 372 l 1066 282 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 190 682 l 18 487 b 0 440 8 476 0 455 l 0 440 l 0 -174 l 347 -174 l 347 565 l 244 682 b 190 682 230 699 204 699 z m 1718 487 l 1546 682 b 1492 682 1532 699 1506 699 l 1389 565 l 1389 -174 l 1736 -174 l 1736 440 b 1718 487 1736 457 1730 474 z m 922 1196 b 814 1196 891 1222 846 1222 l 467 918 b 434 851 449 904 434 874 l 434 -174 l 694 -174 l 694 78 b 839 258 694 164 754 244 b 1042 87 948 276 1042 192 l 1042 -174 l 1302 -174 l 1302 851 b 1270 918 1302 877 1290 902 z m 1064 612 b 1053 592 1069 603 1063 592 l 947 592 l 879 483 b 857 483 874 475 862 475 l 789 592 l 683 592 b 673 612 673 592 667 603 l 725 694 l 673 777 b 683 797 667 786 673 797 l 789 797 l 857 905 b 879 905 862 913 874 913 l 947 797 l 1053 797 b 1064 777 1063 797 1069 786 l 1011 694 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 868 221 l 916 302 l 820 302 z m 1137 671 l 1037 671 l 1087 588 z m 130 1215 b 0 1128 58 1215 0 1176 l 0 -87 b 130 -174 0 -135 58 -174 b 260 -87 202 -174 260 -135 l 260 1128 b 130 1215 260 1176 202 1215 z m 599 370 l 699 370 l 649 454 z m 1037 370 l 1137 370 l 1087 454 z m 347 -43 l 1389 -43 l 1389 1085 l 347 1085 z m 528 711 b 578 740 537 727 560 740 l 578 740 l 740 740 l 819 872 b 868 900 828 888 850 900 l 868 900 l 868 900 b 917 873 886 900 908 888 l 996 740 l 1158 740 b 1215 683 1189 740 1215 714 b 1207 653 1215 674 1211 661 l 1127 521 l 1207 388 b 1215 359 1212 380 1215 367 b 1208 331 1215 350 1212 338 b 1158 302 1199 315 1176 302 l 1158 302 l 996 302 l 917 169 b 868 141 908 154 886 141 l 868 141 b 819 169 850 141 828 154 l 740 302 l 578 302 l 578 302 b 521 359 547 302 521 327 b 529 388 521 368 525 381 l 609 521 l 529 654 b 521 683 524 661 521 674 b 528 711 521 691 524 704 z m 1606 1215 b 1476 1128 1534 1215 1476 1176 l 1476 -87 b 1606 -174 1476 -135 1534 -174 b 1736 -87 1678 -174 1736 -135 l 1736 1128 b 1606 1215 1736 1176 1678 1215 z m 868 821 l 820 740 l 916 740 z m 699 671 l 599 671 l 649 588 z m 779 370 l 957 370 l 1047 521 l 957 671 l 779 671 l 689 521 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1021 1128 l 368 1128 b 0 1215 261 1128 96 1167 l 0 955 b 87 868 0 907 39 868 l 174 868 l 174 694 l 43 694 b 0 651 19 694 0 675 l 0 564 b 43 521 0 540 19 521 l 174 521 l 174 -130 b 217 -174 174 -154 193 -174 l 304 -174 b 347 -130 328 -174 347 -154 l 347 521 l 1042 521 l 1042 -130 b 1085 -174 1042 -154 1061 -174 l 1172 -174 b 1215 -130 1196 -174 1215 -154 l 1215 521 l 1345 521 b 1389 564 1369 521 1389 540 l 1389 651 b 1345 694 1389 675 1369 694 l 1215 694 l 1215 868 l 1302 868 b 1389 955 1350 868 1389 907 l 1389 1215 b 1021 1128 1293 1167 1128 1128 z m 347 868 l 608 868 l 608 694 l 347 694 z m 1042 694 l 781 694 l 781 868 l 1042 868 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1717 128 l 1476 260 l 1476 434 l 1625 482 b 1625 560 1657 498 1657 544 l 1302 694 l 1302 868 l 1376 912 b 1365 982 1397 933 1391 969 l 868 1215 l 371 982 b 360 912 345 969 339 933 l 434 868 l 434 694 l 111 560 b 111 482 79 544 79 498 l 260 434 l 260 260 l 19 128 b 0 87 5 118 -2 102 b 24 53 2 73 10 60 l 174 0 l 174 -130 b 217 -174 174 -154 193 -174 l 304 -174 b 347 -130 328 -174 347 -154 l 347 0 l 781 0 l 781 -130 b 825 -174 781 -154 801 -174 l 911 -174 b 955 -130 935 -174 955 -154 l 955 0 l 1389 0 l 1389 -130 b 1432 -174 1389 -154 1408 -174 l 1519 -174 b 1563 -130 1543 -174 1563 -154 l 1563 0 l 1712 53 b 1736 87 1726 60 1734 73 b 1717 128 1738 102 1731 118 z m 608 868 l 1128 868 l 1128 694 l 608 694 z m 434 260 l 434 434 l 1302 434 l 1302 260 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 583 1023 l 342 781 l 65 781 b 0 716 29 781 0 752 l 0 326 b 65 260 0 290 29 260 l 342 260 l 583 19 b 694 65 624 -22 694 7 l 694 977 b 583 1023 694 1035 624 1063 z m 1252 521 l 1376 645 b 1376 707 1393 662 1393 689 l 1314 768 b 1252 768 1297 786 1269 786 l 1128 645 l 1005 768 b 943 768 988 786 960 786 l 881 707 b 881 645 864 689 864 662 l 1005 521 l 881 397 b 881 335 864 380 864 352 l 943 273 b 1005 273 960 256 988 256 l 1128 397 l 1252 273 b 1314 273 1269 256 1297 256 l 1376 335 b 1376 397 1393 352 1393 380 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 673 174 b 586 260 625 174 586 212 b 673 347 586 308 625 347 b 760 260 721 347 760 308 b 673 174 760 212 721 174 z m 673 521 b 412 260 529 521 412 404 b 673 0 412 117 529 0 b 152 521 385 0 152 233 b 673 1042 152 808 385 1042 b 933 781 817 1042 933 925 b 673 521 933 637 817 521 z m 673 868 b 586 781 625 868 586 829 b 673 694 586 733 625 694 b 760 781 721 694 760 733 b 673 868 760 829 721 868 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1063 1042 l 1515 1042 l 1563 1215 l 521 1215 l 521 260 l 1302 260 l 1349 434 l 1063 434 b 1042 456 1051 434 1042 444 l 1042 499 b 1063 521 1042 511 1051 521 l 1373 521 l 1420 694 l 1063 694 b 1042 716 1051 694 1042 704 l 1042 760 b 1063 781 1042 772 1051 781 l 1444 781 l 1491 955 l 1063 955 b 1042 977 1051 955 1042 965 l 1042 1020 b 1063 1042 1042 1032 1051 1042 z m 431 307 l 361 478 b 314 506 353 497 334 508 l 192 494 b 192 895 145 621 144 765 l 314 883 b 361 911 334 881 353 892 l 431 1082 b 413 1137 439 1102 431 1125 l 306 1202 b 187 1183 267 1226 217 1217 b 193 199 -64 902 -63 479 b 306 186 220 170 272 166 l 413 252 b 431 307 431 264 439 286 z m 1302 174 l 521 174 b 347 0 425 174 347 96 l 347 -87 b 434 -174 347 -135 386 -174 l 1389 -174 b 1476 -87 1437 -174 1476 -135 l 1476 0 b 1302 174 1476 96 1398 174 z m 911 -87 b 825 0 864 -87 825 -48 b 911 87 825 48 864 87 b 998 0 959 87 998 48 b 911 -87 998 -48 959 -87 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 738 846 b 781 890 762 846 781 866 b 738 933 781 914 762 933 b 694 890 714 933 694 914 b 738 846 694 866 714 846 z m 1215 243 l 1215 1146 b 1146 1215 1215 1189 1189 1215 l 260 1215 b 0 955 113 1215 0 1102 l 0 87 b 260 -174 0 -61 113 -174 l 1146 -174 b 1215 -104 1181 -174 1215 -148 l 1215 -61 b 1189 -9 1215 -43 1207 -26 b 1189 191 1181 35 1181 156 b 1215 243 1207 200 1215 217 z m 651 1063 b 868 890 771 1063 868 986 b 781 752 868 833 834 783 l 781 716 b 738 673 781 692 762 673 l 564 673 b 521 716 540 673 521 692 l 521 752 b 434 890 468 783 434 833 b 651 1063 434 986 531 1063 z m 339 610 l 356 649 b 384 661 360 661 373 665 l 651 547 l 918 661 b 946 649 929 665 942 661 l 963 610 b 952 581 968 598 963 586 l 761 499 l 952 417 b 963 389 963 413 968 400 l 946 349 b 918 338 942 338 929 333 l 651 452 l 384 337 b 356 349 373 333 361 338 l 339 389 b 350 417 334 400 339 413 l 541 499 l 350 581 b 339 610 339 586 334 598 z m 1033 0 l 260 0 b 174 87 208 0 174 35 b 260 174 174 139 217 174 l 1033 174 z m 564 846 b 608 890 588 846 608 866 b 564 933 608 914 588 933 b 521 890 540 933 521 914 b 564 846 521 866 540 846 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1693 0 l 1626 0 l 975 896 l 1120 1095 b 1111 1156 1134 1115 1130 1142 l 1040 1207 b 980 1197 1021 1221 994 1217 l 868 1044 l 756 1197 b 696 1207 742 1217 715 1221 l 626 1156 b 616 1095 606 1142 602 1115 l 761 896 l 110 0 l 43 0 b 0 -43 19 0 0 -19 l 0 -130 b 43 -174 0 -154 19 -174 l 1693 -174 b 1736 -130 1717 -174 1736 -154 l 1736 -43 b 1693 0 1736 -19 1717 0 z m 868 434 l 1184 0 l 552 0 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 788 694 b 347 461 734 694 499 689 l 347 694 b 87 955 347 838 230 955 b 0 868 39 955 0 916 b 87 781 0 820 39 781 b 174 694 135 781 174 742 l 174 0 b 347 -174 174 -96 251 -174 l 825 -174 b 868 -130 849 -174 868 -154 l 868 -87 b 781 0 868 -39 829 0 l 694 0 l 1042 260 l 1042 -130 b 1085 -174 1042 -154 1061 -174 l 1172 -174 b 1215 -130 1196 -174 1215 -154 l 1215 429 b 1128 417 1187 422 1159 417 b 788 694 961 417 821 536 z m 1215 955 l 1042 955 l 868 1128 l 868 764 b 1128 503 868 620 985 503 b 1389 764 1272 503 1389 620 l 1389 1128 z m 1020 738 b 977 781 996 738 977 757 b 1020 825 977 805 996 825 b 1063 781 1044 825 1063 805 b 1020 738 1063 757 1044 738 z m 1237 738 b 1194 781 1213 738 1194 757 b 1237 825 1194 805 1213 825 b 1280 781 1261 825 1280 805 b 1237 738 1280 757 1261 738 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 304 868 b 412 1055 304 948 348 1017 l 412 521 l 543 521 l 543 1085 l 673 1085 l 673 521 l 803 521 l 803 1055 b 911 868 868 1017 911 948 l 911 521 l 1042 521 l 1042 868 b 694 1215 1042 1060 886 1215 l 521 1215 b 174 868 329 1215 174 1060 l 174 521 l 304 521 z m 1211 288 l 1182 375 b 1099 434 1170 410 1137 434 l 116 434 b 33 375 78 434 45 410 l 4 288 b 87 174 -14 232 28 174 l 87 -130 b 130 -174 87 -154 106 -174 l 217 -174 b 260 -130 241 -174 260 -154 l 260 174 l 955 174 l 955 -130 b 998 -174 955 -154 974 -174 l 1085 -174 b 1128 -130 1109 -174 1128 -154 l 1128 174 b 1211 288 1188 174 1229 232 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 930 259 b 955 347 945 285 955 315 b 781 521 955 443 877 521 b 661 472 735 521 692 502 b 434 608 617 553 532 608 b 174 347 290 608 174 491 b 175 332 174 342 175 337 b 0 87 74 296 0 200 b 260 -174 0 -57 117 -174 l 911 -174 b 1128 43 1031 -174 1128 -76 b 930 259 1128 157 1041 249 z m 1533 406 b 1049 806 1281 358 1049 551 b 1256 1161 1049 954 1128 1089 b 1248 1207 1275 1172 1270 1202 b 1153 1215 1216 1212 1185 1215 b 632 694 866 1215 632 982 b 637 627 632 671 634 649 b 683 588 653 615 669 602 b 781 608 713 601 747 608 b 1042 347 925 608 1042 491 b 1040 318 1042 337 1041 328 b 1185 176 1104 289 1155 238 b 1557 366 1333 185 1466 255 b 1533 406 1571 384 1556 410 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1560 332 b 1563 347 1561 337 1563 342 b 1389 521 1563 443 1485 521 b 1296 493 1355 521 1323 510 b 1020 672 1249 599 1143 672 b 716 369 852 672 716 537 b 719 345 716 361 718 353 b 477 87 584 335 477 224 b 738 -174 477 -57 594 -174 l 1476 -174 b 1736 87 1619 -174 1736 -57 b 1560 332 1736 201 1662 296 z m 393 393 b 393 822 274 512 274 704 b 822 822 511 941 704 941 b 883 734 849 796 868 766 b 966 754 910 744 937 750 l 1047 995 b 995 1047 1058 1028 1027 1058 l 757 968 l 644 1192 b 571 1192 629 1223 586 1223 l 458 968 l 220 1047 b 168 995 188 1058 157 1027 l 247 757 l 23 645 b 23 571 -8 629 -8 586 l 247 459 l 168 221 b 220 168 157 188 188 158 l 426 237 b 490 328 443 271 464 301 b 393 393 455 343 422 364 z m 380 608 b 564 385 380 497 459 405 b 632 416 586 397 608 409 b 815 701 647 537 716 640 b 608 836 779 780 700 836 b 380 608 482 836 380 734 z "
	},
	"": {
		ha: 1302,
		x_min: 0,
		x_max: 0,
		o: "m 290 632 l 3 209 b 19 176 -5 195 4 178 l 584 116 z m 20 360 l 224 692 l 16 816 b 0 807 9 821 0 815 l 0 365 b 20 360 0 354 14 350 z m 50 66 l 577 -172 b 608 -152 591 -178 608 -168 l 608 26 l 55 87 b 50 66 43 88 39 72 z m 270 765 l 487 1153 b 457 1183 498 1172 477 1195 l 48 916 b 49 898 42 912 42 902 z m 651 738 l 947 738 l 688 1195 b 651 1215 680 1208 665 1215 b 614 1195 637 1215 623 1208 l 355 738 z m 1286 816 l 1078 691 l 1282 360 b 1302 365 1288 350 1302 354 l 1302 807 b 1286 816 1302 815 1293 821 z m 1032 765 l 1253 898 b 1254 916 1260 902 1260 912 l 845 1183 b 815 1153 825 1195 804 1172 z m 1247 87 l 694 26 l 694 -152 b 725 -172 694 -168 711 -178 l 1253 66 b 1247 87 1264 72 1259 88 z m 1013 632 l 718 116 l 1283 176 b 1299 209 1299 178 1307 195 z m 651 651 l 379 651 l 651 174 l 923 651 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1145 917 l 695 1191 b 520 1191 641 1223 574 1223 l 70 917 b 70 878 56 908 56 886 l 608 557 l 1145 878 b 1145 917 1160 886 1160 908 z m 1183 796 l 651 479 l 651 -128 b 716 -167 651 -163 687 -185 l 1129 83 b 1215 240 1182 116 1215 176 l 1215 776 b 1183 796 1215 794 1197 805 z m 0 776 l 0 240 b 86 83 0 176 33 116 l 499 -167 b 564 -128 528 -185 564 -163 l 564 479 l 32 796 b 0 776 18 805 0 794 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1345 955 l 1172 955 l 1152 994 b 1075 1042 1139 1020 1104 1042 l 929 1042 l 855 1116 b 781 1085 828 1143 781 1124 l 781 679 l 1128 555 l 1128 651 l 1215 651 b 1389 825 1311 651 1389 729 l 1389 911 b 1345 955 1389 935 1369 955 z m 1042 825 b 998 868 1018 825 998 844 b 1042 911 998 892 1018 911 b 1085 868 1066 911 1085 892 b 1042 825 1085 844 1066 825 z m 260 608 b 174 694 213 608 174 647 b 87 781 174 742 135 781 b 0 694 39 781 0 742 b 174 450 0 581 73 486 l 174 -130 b 217 -174 174 -154 193 -174 l 391 -174 b 434 -130 415 -174 434 -154 l 434 174 l 868 174 l 868 -130 b 911 -174 868 -154 887 -174 l 1085 -174 b 1128 -130 1109 -174 1128 -154 l 1128 462 l 722 608 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 50 521 l 521 608 l 273 421 b 304 347 246 394 265 347 l 908 347 b 868 574 882 419 868 495 l 868 688 l 543 926 b 334 913 479 969 393 964 l 16 598 b 50 521 -18 570 6 515 z m 1560 429 l 1287 565 b 1215 682 1248 585 1215 637 l 1215 781 l 1389 781 l 1465 720 b 1526 695 1481 704 1503 695 l 1610 695 b 1688 743 1640 695 1675 716 l 1727 820 b 1736 859 1732 830 1736 848 b 1719 911 1736 875 1728 898 l 1517 1181 b 1447 1215 1500 1203 1474 1215 l 803 1215 b 788 1178 784 1215 774 1192 l 955 1042 l 793 974 b 793 936 777 966 777 944 l 955 868 l 955 574 b 1215 87 955 378 1053 195 b 38 -78 685 68 281 -24 b 0 -125 16 -83 0 -103 b 48 -174 0 -152 22 -174 l 1402 -174 b 1736 127 1574 -174 1726 -45 b 1560 429 1742 255 1674 372 z m 1327 1036 l 1451 1005 b 1385 955 1443 975 1417 953 b 1327 1036 1350 957 1315 989 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1255 1081 b 931 1215 1181 1155 1036 1215 b 606 1081 826 1215 680 1155 b 434 694 509 985 434 868 l 434 462 l 324 352 b 224 335 298 325 259 321 b 164 347 208 342 181 347 b 0 183 73 347 0 274 b 48 67 0 146 22 94 b 200 27 90 26 147 14 b 241 -126 188 -27 200 -84 b 357 -174 268 -152 320 -174 b 521 -10 448 -174 521 -100 b 509 51 521 7 516 35 b 526 150 495 85 499 124 l 636 260 l 869 260 b 873 260 870 260 871 260 b 1037 288 919 260 993 273 b 1039 697 929 407 929 587 b 1389 743 1133 791 1275 804 b 1255 1081 1392 865 1349 988 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 349 685 l 125 826 b 62 808 103 839 73 832 b 1 566 33 745 6 636 b 45 521 0 542 21 521 l 309 521 b 349 559 330 521 347 538 b 366 632 352 585 358 609 b 349 685 373 652 367 674 z m 865 1194 b 694 1215 811 1208 753 1215 b 523 1194 635 1215 578 1208 b 495 1136 499 1187 486 1159 l 597 853 b 638 825 603 838 621 825 l 751 825 b 792 853 769 825 786 836 l 894 1136 b 865 1194 903 1159 890 1187 z m 304 434 l 43 434 b 0 391 19 434 0 415 l 0 217 b 43 174 0 193 19 174 l 304 174 b 347 217 328 174 347 193 l 347 391 b 304 434 347 415 328 434 z m 304 87 l 43 87 b 0 43 19 87 0 67 l 0 -130 b 43 -174 0 -154 19 -174 l 304 -174 b 347 -130 328 -174 347 -154 l 347 43 b 304 87 347 67 328 87 z m 514 856 l 415 1103 b 354 1125 405 1127 377 1138 b 160 963 292 1090 205 1017 b 171 899 144 943 149 913 l 396 758 b 451 766 414 747 436 751 b 496 804 465 780 480 793 b 514 856 513 816 521 837 z m 1080 521 l 1344 521 b 1387 566 1368 521 1389 542 b 1327 808 1383 636 1356 745 b 1264 826 1316 832 1286 839 l 1040 685 b 1022 632 1022 674 1016 652 b 1039 559 1031 609 1036 585 b 1080 521 1042 538 1059 521 z m 1229 963 b 1035 1125 1184 1017 1097 1090 b 974 1103 1012 1138 983 1127 l 875 856 b 892 804 868 837 875 816 b 938 766 909 793 924 780 b 993 758 953 751 975 747 l 1217 899 b 1229 963 1239 913 1245 943 z m 1345 434 l 1085 434 b 1042 391 1061 434 1042 415 l 1042 217 b 1085 174 1042 193 1061 174 l 1345 174 b 1389 217 1369 174 1389 193 l 1389 391 b 1345 434 1389 415 1369 434 z m 1345 87 l 1085 87 b 1042 43 1061 87 1042 67 l 1042 -130 b 1085 -174 1042 -154 1061 -174 l 1345 -174 b 1389 -130 1369 -174 1389 -154 l 1389 43 b 1345 87 1389 67 1369 87 z m 651 733 l 651 -65 b 673 -87 651 -77 661 -87 l 716 -87 b 738 -65 728 -87 738 -77 l 738 733 b 694 738 724 736 709 738 b 651 733 680 738 665 736 z m 477 621 l 477 -65 b 499 -87 477 -77 487 -87 l 543 -87 b 564 -65 555 -87 564 -77 l 564 702 b 477 621 530 681 500 654 z m 825 702 l 825 -65 b 846 -87 825 -77 834 -87 l 890 -87 b 911 -65 902 -87 911 -77 l 911 621 b 825 702 889 654 859 681 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 608 846 l 608 1215 l 65 1215 b 0 1150 29 1215 0 1186 l 0 -109 b 65 -174 0 -145 29 -174 l 977 -174 b 1042 -109 1013 -174 1042 -145 l 1042 781 l 673 781 b 608 846 637 781 608 811 z m 347 456 b 326 434 347 444 338 434 l 304 434 b 260 391 280 434 260 415 l 260 304 b 304 260 260 280 280 260 l 326 260 b 347 239 338 260 347 251 l 347 195 b 326 174 347 183 338 174 l 304 174 b 174 304 232 174 174 232 l 174 391 b 304 521 174 463 232 521 l 326 521 b 347 499 338 521 347 511 z m 467 174 l 434 174 b 412 195 422 174 412 183 l 412 239 b 434 260 412 251 422 260 l 467 260 b 496 278 483 260 496 270 b 490 289 496 282 494 286 l 430 340 b 394 416 407 359 394 387 b 509 521 394 474 446 521 l 543 521 b 564 499 555 521 564 511 l 564 456 b 543 434 564 444 555 434 l 509 434 b 481 416 493 434 481 425 b 487 406 481 413 483 409 l 546 355 b 582 278 569 335 582 307 b 467 174 582 221 531 174 z m 694 499 l 694 443 b 738 288 694 388 710 334 b 781 443 766 334 781 388 l 781 499 b 803 521 781 511 791 521 l 846 521 b 868 499 858 521 868 511 l 868 443 b 770 187 868 346 833 256 b 738 174 761 179 750 174 b 706 187 726 174 714 179 b 608 443 643 256 608 346 l 608 499 b 629 521 608 511 617 521 l 673 521 b 694 499 685 521 694 511 z m 1023 930 l 757 1196 b 711 1215 745 1208 728 1215 l 694 1215 l 694 868 l 1042 868 l 1042 885 b 1023 930 1042 902 1035 918 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 694 781 l 694 1172 b 651 1215 694 1196 675 1215 l 564 1215 b 521 1172 540 1215 521 1196 l 521 773 b 564 781 534 778 549 781 z m 1042 521 b 868 694 1042 617 964 694 l 564 694 b 521 651 540 694 521 675 l 521 650 b 650 521 521 579 578 521 l 745 521 b 781 499 772 521 781 511 l 781 455 b 760 433 781 444 771 434 b 499 237 639 427 586 366 l 482 211 b 464 201 479 206 471 201 b 452 205 460 201 455 203 l 416 229 b 406 247 411 233 406 241 b 410 259 406 251 408 256 l 427 285 b 555 442 470 349 509 402 b 450 536 509 457 470 492 b 391 521 432 527 412 521 l 304 521 b 217 555 270 521 240 534 b 130 521 194 534 164 521 l 43 521 b 0 529 28 521 14 524 l 0 317 b 102 72 0 225 37 137 l 174 0 l 174 -174 l 868 -174 l 868 0 l 965 97 b 1042 282 1007 139 1042 222 l 1042 282 z m 955 765 l 955 1085 b 911 1128 955 1109 935 1128 l 825 1128 b 781 1085 801 1128 781 1109 l 781 781 l 868 781 b 955 765 899 781 927 775 z m 43 608 l 130 608 b 174 651 154 608 174 627 l 174 998 b 130 1042 174 1022 154 1042 l 43 1042 b 0 998 19 1042 0 1022 l 0 651 b 43 608 0 627 19 608 z m 304 608 l 391 608 b 434 651 415 608 434 627 l 434 1085 b 391 1128 434 1109 415 1128 l 304 1128 b 260 1085 280 1128 260 1109 l 260 651 b 304 608 260 627 280 608 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 505 1215 b 0 673 220 1206 0 958 l 0 -43 b 74 -74 0 -82 47 -101 l 142 -24 b 200 -30 160 -10 185 -13 l 317 -161 b 378 -161 333 -178 361 -178 l 488 -37 b 553 -37 506 -17 536 -17 l 664 -161 b 725 -161 681 -178 708 -178 l 842 -30 b 900 -24 857 -13 882 -10 l 968 -74 b 1042 -43 995 -101 1042 -82 l 1042 694 b 505 1215 1042 987 800 1224 z m 347 608 b 260 694 299 608 260 647 b 347 781 260 742 299 781 b 434 694 395 781 434 742 b 347 608 434 647 395 608 z m 694 608 b 608 694 647 608 608 647 b 694 781 608 742 647 781 b 781 694 742 781 781 742 b 694 608 781 647 742 608 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1550 689 l 1488 751 b 1427 751 1471 768 1444 768 l 1396 720 l 1318 798 b 1274 965 1333 856 1319 920 l 1151 1088 b 537 1088 981 1258 707 1258 l 783 965 l 783 915 b 833 792 783 869 801 824 l 967 659 b 1134 614 1012 613 1076 599 l 1212 536 l 1182 505 b 1182 444 1165 488 1165 461 l 1243 382 b 1304 382 1260 365 1287 365 l 1550 628 b 1550 689 1567 645 1567 672 z m 772 730 b 745 763 762 740 753 752 l 53 117 b 49 -124 -16 52 -18 -57 b 290 -120 116 -192 226 -190 l 936 572 b 905 597 926 579 915 588 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 629 781 b 608 760 617 781 608 772 l 608 434 l 694 434 l 694 760 b 673 781 694 772 685 781 z m 456 781 b 434 760 444 781 434 772 l 434 434 l 521 434 l 521 760 b 499 781 521 772 511 781 z m 1063 781 b 1042 760 1051 781 1042 772 l 1042 434 l 1128 434 l 1128 760 b 1107 781 1128 772 1119 781 z m 1237 781 b 1215 760 1225 781 1215 772 l 1215 434 l 1302 434 l 1302 760 b 1280 781 1302 772 1292 781 z m 1476 760 b 1454 781 1476 772 1466 781 l 1411 781 b 1389 760 1399 781 1389 772 l 1389 434 l 1476 434 z m 282 781 b 260 760 270 781 260 772 l 260 434 l 347 434 l 347 760 b 326 781 347 772 338 781 z m 1693 781 l 1606 781 b 1563 738 1582 781 1563 762 l 1563 434 b 1476 347 1563 386 1524 347 l 955 347 l 955 868 b 911 911 955 892 935 911 l 825 911 b 781 868 801 911 781 892 l 781 347 l 260 347 b 174 434 212 347 174 386 l 174 738 b 130 781 174 762 154 781 l 43 781 b 0 738 19 781 0 762 l 0 434 b 260 174 0 290 117 174 l 781 174 l 781 0 l 304 0 b 260 -43 280 0 260 -19 l 260 -130 b 304 -174 260 -154 280 -174 l 1432 -174 b 1476 -130 1456 -174 1476 -154 l 1476 -43 b 1432 0 1476 -19 1456 0 l 955 0 l 955 174 l 1476 174 b 1736 434 1620 174 1736 290 l 1736 738 b 1693 781 1736 762 1717 781 z m 1649 868 b 1714 940 1685 868 1714 900 b 1649 1085 1714 980 1649 1085 b 1584 940 1649 1085 1584 980 b 1649 868 1584 900 1613 868 z m 87 868 b 152 940 123 868 152 900 b 87 1085 152 980 87 1085 b 22 940 87 1085 22 980 b 87 868 22 900 51 868 z m 868 998 b 933 1071 904 998 933 1031 b 868 1215 933 1111 868 1215 b 803 1071 868 1215 803 1111 b 868 998 803 1031 832 998 z m 304 868 b 369 940 340 868 369 900 b 304 1085 369 980 304 1085 b 239 940 304 1085 239 980 b 304 868 239 900 268 868 z m 477 868 b 543 940 513 868 543 900 b 477 1085 543 980 477 1085 b 412 940 477 1085 412 980 b 477 868 412 900 441 868 z m 651 868 b 716 940 687 868 716 900 b 651 1085 716 980 651 1085 b 586 940 651 1085 586 980 b 651 868 586 900 615 868 z m 1085 868 b 1150 940 1121 868 1150 900 b 1085 1085 1150 980 1085 1085 b 1020 940 1085 1085 1020 980 b 1085 868 1020 900 1049 868 z m 1259 868 b 1324 940 1295 868 1324 900 b 1259 1085 1324 980 1259 1085 b 1194 940 1259 1085 1194 980 b 1259 868 1194 900 1223 868 z m 1432 868 b 1497 940 1468 868 1497 900 b 1432 1085 1497 980 1432 1085 b 1367 940 1432 1085 1367 980 b 1432 868 1367 900 1396 868 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1345 0 l 43 0 b 0 -43 19 0 0 -19 l 0 -130 b 43 -174 0 -154 19 -174 l 1345 -174 b 1389 -130 1369 -174 1389 -154 l 1389 -43 b 1345 0 1389 -19 1369 0 z m 521 174 l 347 260 l 521 347 l 608 521 l 694 347 l 868 260 l 694 174 l 651 87 l 1215 87 l 981 634 b 967 702 973 652 967 682 b 976 757 967 718 971 742 l 1128 1215 l 620 925 b 475 764 565 893 500 821 l 174 87 l 564 87 z m 694 781 l 738 868 l 781 781 l 868 738 l 781 694 l 738 608 l 694 694 l 608 738 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 220 -66 b 283 -171 208 -112 236 -159 b 304 -174 290 -173 297 -174 b 388 -108 343 -174 378 -147 l 457 166 l 313 309 z m 260 466 l 372 898 b 339 950 378 921 363 944 b 70 795 220 978 100 909 l 1 528 b 34 477 -5 505 10 482 l 206 435 b 260 466 230 430 254 443 z m 998 781 l 955 781 b 911 738 931 781 911 762 l 911 694 l 817 694 l 690 821 b 578 868 660 851 620 868 b 423 747 504 868 441 818 l 350 455 b 347 434 348 449 347 440 b 373 373 347 414 359 387 l 608 138 l 608 -87 b 694 -174 608 -135 647 -174 b 781 -87 742 -174 781 -135 l 781 138 b 730 260 781 184 763 228 l 603 387 b 605 391 604 389 605 390 l 659 607 l 720 546 b 781 521 736 530 758 521 l 911 521 l 911 -130 b 955 -174 911 -154 931 -174 l 998 -174 b 1042 -130 1022 -174 1042 -154 l 1042 738 b 998 781 1042 762 1022 781 z m 651 955 b 781 1085 723 955 781 1013 b 651 1215 781 1157 723 1215 b 521 1085 579 1215 521 1157 b 651 955 521 1013 579 955 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1576 954 b 1369 882 1501 955 1434 907 b 1128 1042 1329 976 1237 1042 b 1042 1026 1098 1042 1069 1035 l 1042 1063 b 977 1128 1042 1099 1013 1128 l 933 1128 b 868 1063 897 1128 868 1099 l 868 931 b 521 1042 776 999 654 1042 b 0 608 233 1042 0 847 l 0 -43 b 43 -87 0 -67 19 -87 l 217 -87 b 260 -43 241 -87 260 -67 l 260 149 b 564 87 348 110 452 87 b 868 149 676 87 780 110 l 868 -43 b 911 -87 868 -67 887 -87 l 1085 -87 b 1128 -43 1109 -87 1128 -67 l 1128 434 l 1476 434 l 1476 347 b 1519 304 1476 323 1495 304 l 1606 304 b 1649 347 1630 304 1649 323 l 1649 434 b 1736 521 1697 434 1736 473 l 1736 770 b 1576 954 1736 863 1669 954 z m 1215 738 b 1172 781 1191 738 1172 757 b 1215 825 1172 805 1191 825 b 1259 781 1239 825 1259 805 b 1215 738 1259 757 1239 738 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1562 1007 b 1539 1067 1562 1030 1554 1051 b 1502 1106 1529 1078 1516 1091 b 1562 1189 1532 1125 1555 1153 b 1541 1215 1565 1203 1554 1215 l 1215 1215 b 868 868 1023 1215 868 1060 l 434 868 b 239 778 356 868 286 833 l 239 781 b 0 543 107 781 0 674 l 0 391 b 43 347 0 367 19 347 l 87 347 b 130 391 111 347 130 367 l 130 543 b 176 628 130 578 149 609 b 174 608 175 621 174 615 b 256 418 174 533 206 466 l 187 232 b 176 171 181 216 176 188 b 181 129 176 159 178 140 l 248 -141 b 290 -174 253 -159 272 -174 l 290 -174 l 469 -174 b 511 -120 497 -174 518 -147 l 440 166 l 505 338 l 868 278 l 868 -130 b 911 -174 868 -154 887 -174 l 1085 -174 b 1128 -130 1109 -174 1128 -154 l 1128 352 b 1215 564 1182 407 1215 482 b 1215 566 1215 565 1215 565 l 1215 844 l 1258 825 l 1310 722 b 1420 681 1330 682 1378 664 l 1508 716 b 1562 797 1538 728 1562 764 l 1562 797 z m 1389 955 b 1345 998 1365 955 1345 974 b 1389 1042 1345 1022 1365 1042 b 1432 998 1413 1042 1432 1022 b 1389 955 1432 974 1413 955 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 781 903 l 188 381 b 174 371 184 377 178 375 l 174 -130 b 217 -174 174 -154 193 -174 l 622 -174 l 521 24 l 803 198 l 640 521 l 1042 150 l 759 -24 l 868 -174 l 1345 -174 b 1389 -130 1369 -174 1389 -154 l 1389 371 b 1375 380 1384 375 1379 376 z m 1548 574 l 1389 715 l 1389 1085 b 1345 1128 1389 1109 1369 1128 l 1172 1128 b 1128 1085 1148 1128 1128 1109 l 1128 945 l 854 1187 b 781 1215 833 1206 807 1215 b 709 1187 755 1215 729 1206 l 14 574 b 11 513 -3 558 -5 531 l 69 448 b 130 445 85 431 113 429 l 753 994 b 810 994 769 1008 794 1008 l 1432 445 b 1493 448 1450 429 1477 431 l 1551 513 b 1548 574 1567 531 1566 558 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 998 564 b 1042 608 1022 564 1042 584 l 1042 694 b 998 738 1042 718 1022 738 l 885 738 b 874 993 921 816 922 910 b 624 1128 823 1080 725 1128 l 410 1128 b 229 1063 344 1128 280 1105 l 198 1037 b 189 945 170 1013 166 972 l 245 878 b 337 870 268 850 309 847 l 368 896 b 410 911 380 906 395 911 l 637 911 b 694 854 669 911 694 886 b 676 812 694 838 688 823 l 596 738 l 43 738 b 0 694 19 738 0 718 l 0 608 b 43 564 0 584 19 564 l 408 564 l 314 477 l 43 477 b 0 434 19 477 0 458 l 0 347 b 43 304 0 323 19 304 l 157 304 b 168 49 121 226 120 132 b 418 -87 219 -38 317 -87 l 632 -87 b 812 -21 698 -87 762 -64 l 844 5 b 852 97 872 28 875 69 l 797 164 b 705 172 774 191 733 195 l 673 145 b 632 130 663 137 645 130 l 632 130 l 405 130 b 347 188 373 130 347 156 b 366 230 347 204 354 219 l 446 304 l 998 304 b 1042 347 1022 304 1042 323 l 1042 434 b 998 477 1042 458 1022 477 l 634 477 l 728 564 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 870 1042 b 440 0 -331 1042 -100 0 b 715 152 548 0 650 56 l 785 254 b 955 254 827 316 912 316 l 1025 152 b 1300 0 1090 56 1191 0 b 870 1042 1815 0 2088 1042 z m 499 379 b 282 490 388 379 315 448 b 282 552 268 508 268 534 b 499 663 315 593 388 663 b 716 552 611 663 683 593 b 716 490 731 534 731 508 b 499 379 683 448 611 379 z m 1237 379 b 1020 490 1126 379 1053 448 b 1020 552 1006 508 1006 534 b 1237 663 1053 593 1126 663 b 1454 552 1348 663 1421 593 b 1454 490 1468 534 1468 508 b 1237 379 1421 448 1348 379 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1722 -40 l 941 1175 b 868 1215 925 1200 898 1215 b 795 1175 839 1215 811 1200 l 14 -40 b 0 -87 6 -52 0 -73 b 11 -128 0 -99 5 -118 b 87 -174 24 -153 58 -174 l 87 -174 l 1649 -174 b 1726 -128 1681 -174 1710 -156 b 1736 -87 1731 -118 1736 -99 b 1722 -40 1736 -73 1730 -52 z m 868 968 l 1100 608 l 868 608 l 694 434 l 591 537 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1736 499 l 1736 543 b 1693 586 1736 567 1717 586 l 933 586 l 933 694 l 1128 694 b 1215 781 1176 694 1215 733 l 1215 1128 b 1128 1215 1215 1176 1176 1215 l 608 1215 b 521 1128 560 1215 521 1176 l 521 781 b 608 694 521 733 560 694 l 803 694 l 803 586 l 43 586 b 0 543 19 586 0 567 l 0 499 b 43 456 0 475 19 456 l 326 456 l 326 347 l 174 347 b 87 260 126 347 87 308 l 87 -87 b 174 -174 87 -135 126 -174 l 608 -174 b 694 -87 656 -174 694 -135 l 694 260 b 608 347 694 308 656 347 l 456 347 l 456 456 l 1280 456 l 1280 347 l 1128 347 b 1042 260 1081 347 1042 308 l 1042 -87 b 1128 -174 1042 -135 1081 -174 l 1563 -174 b 1649 -87 1610 -174 1649 -135 l 1649 260 b 1563 347 1649 308 1610 347 l 1411 347 l 1411 456 l 1693 456 b 1736 499 1717 456 1736 475 z m 694 868 l 694 1042 l 1042 1042 l 1042 868 z m 521 0 l 260 0 l 260 174 l 521 174 z m 1476 0 l 1215 0 l 1215 174 l 1476 174 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1649 1128 l 1563 1128 l 1527 1164 b 1404 1215 1498 1192 1444 1215 l 1404 1215 l 1348 1215 b 1262 1192 1318 1215 1288 1207 l 846 955 l 694 955 b 174 434 407 955 174 722 l 174 430 b 66 199 174 341 130 261 b 3 5 17 152 -10 81 b 227 -174 22 -101 120 -174 l 521 -174 b 608 -87 569 -174 608 -135 b 521 0 608 -39 569 0 l 217 0 b 174 43 193 0 174 19 b 217 87 174 67 193 87 l 825 87 b 868 130 849 87 868 106 l 868 174 b 781 260 868 222 829 260 l 608 260 l 1013 479 l 1215 87 l 1432 87 b 1476 130 1456 87 1476 106 l 1476 174 b 1389 260 1476 222 1437 260 l 1312 260 l 1163 559 l 1414 694 l 1476 694 b 1736 955 1620 694 1736 811 l 1736 1042 b 1649 1128 1736 1090 1697 1128 z m 1389 1085 b 1432 1042 1413 1085 1432 1066 b 1389 998 1432 1018 1413 998 b 1345 1042 1365 998 1345 1018 b 1389 1085 1345 1066 1365 1085 z m 1476 825 l 1381 825 l 1105 676 l 1067 751 l 1389 911 l 1598 911 b 1476 825 1580 861 1532 825 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1042 b 0 651 299 1042 0 874 l 0 385 b 694 0 0 172 311 0 b 1389 385 1078 0 1389 172 l 1389 651 b 694 1042 1389 874 1090 1042 z m 694 868 b 1215 651 982 868 1215 771 b 1186 579 1215 626 1205 602 b 694 694 1064 650 890 694 b 203 579 499 694 325 650 b 174 651 184 602 174 626 b 694 868 174 771 407 868 z m 327 497 b 694 564 421 537 547 564 b 1062 497 842 564 968 537 b 694 434 968 458 838 434 b 327 497 551 434 421 458 z "
	},
	"": {
		ha: 1128,
		x_min: 0,
		x_max: 0,
		o: "m 738 955 b 868 1085 810 955 868 1013 b 738 1215 868 1157 810 1215 b 608 1085 666 1215 608 1157 b 738 955 608 1013 666 955 z m 308 354 l 268 260 l 87 260 b 0 174 39 260 0 222 b 87 87 0 126 39 87 l 297 87 b 417 166 349 87 396 118 l 440 221 l 411 238 b 308 354 364 266 330 307 z m 1042 608 l 922 608 l 852 752 b 684 890 818 821 755 872 l 491 948 b 272 901 414 966 334 949 l 164 819 b 148 697 126 789 119 735 b 270 681 177 659 232 652 l 378 763 b 446 780 398 779 425 785 l 486 768 l 384 531 b 456 313 350 451 381 358 l 686 177 l 612 -61 b 669 -170 597 -107 623 -155 b 695 -174 677 -172 686 -174 b 777 -113 731 -174 766 -150 l 863 161 b 804 309 879 218 855 278 l 638 407 l 723 619 l 778 507 b 895 434 800 463 846 434 l 1042 434 b 1128 521 1090 434 1128 473 b 1042 608 1128 569 1090 608 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 130 1215 b 0 1085 58 1215 0 1157 l 0 911 b 43 868 0 887 19 868 l 260 868 l 260 1085 b 130 1215 260 1157 202 1215 z m 694 96 l 694 260 l 1476 260 l 1476 955 b 1215 1215 1476 1098 1359 1215 l 303 1215 b 347 1085 330 1179 347 1134 l 347 87 b 550 -84 347 -19 441 -102 b 694 96 635 -71 694 10 z m 781 174 l 781 87 b 521 -174 781 -57 664 -174 l 1432 -174 b 1736 130 1600 -174 1736 -38 b 1693 174 1736 154 1717 174 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1191 -14 l 806 174 l 1191 361 b 1211 419 1213 372 1221 398 l 1172 497 b 1114 516 1161 518 1135 527 l 608 270 l 102 516 b 43 497 80 527 54 518 l 5 419 b 24 361 -6 398 3 372 l 409 174 l 24 -14 b 5 -72 3 -24 -6 -51 l 43 -150 b 102 -169 54 -171 80 -180 l 608 77 l 1114 -169 b 1172 -150 1135 -180 1161 -171 l 1211 -72 b 1191 -14 1221 -51 1213 -24 z m 407 572 l 392 501 b 437 434 385 467 407 434 l 779 434 b 823 501 809 434 831 467 l 808 572 b 998 868 922 632 998 742 b 608 1215 998 1060 823 1215 b 217 868 392 1215 217 1060 b 407 572 217 742 294 632 z m 760 911 b 846 825 807 911 846 873 b 760 738 846 777 807 738 b 673 825 712 738 673 777 b 760 911 673 873 712 911 z m 456 911 b 543 825 504 911 543 873 b 456 738 543 777 504 738 b 369 825 408 738 369 777 b 456 911 369 873 408 911 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1613 -164 b 1674 -157 1632 -179 1659 -176 l 1727 -88 b 1719 -27 1742 -69 1738 -42 l 123 1206 b 62 1199 104 1221 77 1217 l 9 1130 b 17 1069 -6 1111 -2 1084 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 410 761 l 480 738 l 493 738 l 507 809 b 516 841 509 818 512 829 l 461 951 l 525 1144 b 498 1199 533 1167 521 1192 l 457 1213 b 402 1186 434 1221 410 1208 l 332 975 b 327 947 329 967 327 955 b 336 909 327 936 331 919 z m 1555 268 l 1412 482 b 1340 521 1398 503 1366 521 l 1128 521 l 1316 587 b 1360 621 1331 592 1351 607 l 1506 839 b 1494 899 1519 859 1514 886 l 1458 924 b 1397 911 1438 937 1411 931 l 1260 706 l 1096 651 l 998 651 l 970 792 b 781 955 964 821 923 955 b 592 792 639 955 598 821 l 564 651 l 466 651 l 302 706 l 165 911 b 105 924 152 931 125 937 l 69 899 b 57 839 49 886 43 859 l 202 621 b 247 587 211 607 231 592 l 434 521 l 223 521 b 150 482 197 521 165 504 l 7 268 b 19 207 -6 248 -1 221 l 55 183 b 116 195 75 170 102 175 l 246 391 l 374 391 l 208 127 b 195 81 201 115 195 94 l 195 81 l 195 -130 b 239 -174 195 -154 215 -174 l 282 -174 b 326 -130 306 -174 326 -154 l 326 68 l 526 390 b 521 275 524 352 521 313 b 781 0 521 131 631 0 b 1042 275 931 0 1042 131 b 1036 390 1042 313 1039 352 l 1237 68 l 1237 -130 b 1280 -174 1237 -154 1256 -174 l 1324 -174 b 1367 -130 1348 -174 1367 -154 l 1367 81 b 1354 127 1367 97 1363 113 l 1189 391 l 1317 391 l 1447 195 b 1507 183 1460 175 1487 170 l 1543 207 b 1555 268 1563 221 1568 248 z m 1102 951 l 1047 841 b 1055 809 1050 829 1053 818 l 1069 738 l 1082 738 l 1152 761 l 1226 909 b 1235 947 1231 919 1235 936 b 1231 975 1235 955 1233 967 l 1161 1186 b 1106 1213 1153 1208 1128 1221 l 1064 1199 b 1037 1144 1042 1192 1029 1167 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 347 1215 b 87 694 203 1215 87 982 l 87 228 b 2 -116 87 116 60 57 b 43 -174 -7 -145 14 -174 l 805 -174 b 888 -114 843 -174 876 -150 b 955 228 923 -10 955 82 l 955 694 b 1119 1215 955 921 1019 1111 z m 260 608 b 217 651 236 608 217 627 b 260 694 217 675 236 694 b 304 651 284 694 304 675 b 260 608 304 627 284 608 z m 434 608 b 391 651 410 608 391 627 b 434 694 391 675 410 694 b 477 651 458 694 477 675 b 434 608 477 627 458 608 z m 608 608 b 564 651 584 608 564 627 b 608 694 564 675 584 694 b 651 651 632 694 651 675 b 608 608 651 627 632 608 z m 781 608 b 738 651 757 608 738 627 b 781 694 738 675 757 694 b 825 651 805 694 825 675 b 781 608 825 627 805 608 z m 1302 1215 b 1042 694 1158 1215 1042 982 b 1302 174 1042 407 1158 174 b 1563 694 1446 174 1563 407 b 1302 1215 1563 982 1446 1215 z m 1302 521 b 1215 694 1254 521 1215 599 b 1302 868 1215 790 1254 868 b 1389 694 1350 868 1389 790 b 1302 521 1389 599 1350 521 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1432 304 b 1194 65 1300 304 1194 197 b 1432 -174 1194 -67 1300 -174 b 1671 65 1564 -174 1671 -67 b 1432 304 1671 197 1564 304 z m 1432 0 b 1367 65 1396 0 1367 29 b 1432 130 1367 101 1396 130 b 1497 65 1468 130 1497 101 b 1432 0 1497 29 1468 0 z m 1649 781 l 1476 781 l 1476 890 b 1511 996 1476 929 1488 966 b 1508 1049 1523 1011 1521 1034 l 1449 1114 b 1382 1112 1431 1134 1399 1133 b 1302 890 1331 1050 1302 972 l 1302 781 l 1024 781 l 872 1136 b 753 1215 854 1180 800 1215 l 753 1215 l 391 1215 b 260 1085 319 1215 260 1157 l 260 688 b 182 684 237 706 204 705 l 98 599 b 98 515 74 576 74 538 l 111 501 b 79 423 98 476 87 450 l 60 423 b 0 364 27 423 0 396 l 0 244 b 60 184 0 211 27 184 l 79 184 b 111 107 87 158 98 132 l 98 93 b 98 8 74 70 74 32 l 182 -76 b 266 -76 205 -99 243 -99 l 280 -62 b 358 -95 305 -76 331 -87 l 358 -114 b 418 -174 358 -147 385 -174 l 537 -174 b 597 -114 570 -174 597 -147 l 597 -95 b 675 -62 624 -87 650 -76 l 688 -76 b 773 -76 712 -99 749 -99 l 857 8 b 857 93 880 32 880 69 l 843 107 b 876 184 857 132 868 158 l 895 184 b 955 244 928 184 955 211 l 955 260 l 1173 260 b 1432 391 1233 339 1326 391 b 1573 358 1483 391 1530 378 l 1711 495 b 1736 557 1727 512 1736 534 l 1736 694 b 1649 781 1736 742 1697 781 z m 477 87 b 260 304 358 87 260 184 b 477 521 260 424 358 521 b 694 304 597 521 694 424 b 477 87 694 184 597 87 z m 537 781 l 434 781 l 434 1042 l 724 1042 l 836 781 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 752 1183 b 608 1215 708 1203 659 1215 b 286 998 462 1215 338 1125 l 506 998 z m 929 998 b 830 1133 908 1051 873 1096 l 651 998 z m 608 521 b 955 868 799 521 955 676 b 950 911 955 883 952 897 l 265 911 b 260 868 263 897 260 883 b 608 521 260 676 416 521 z m 217 402 l 217 -174 l 565 -174 l 298 427 b 217 402 275 423 239 412 z m 0 -43 b 130 -174 0 -115 58 -174 l 130 347 b 0 69 51 280 0 181 z m 694 87 l 544 87 l 660 -174 l 694 -174 b 825 -43 766 -174 825 -115 b 694 87 825 28 766 87 z m 851 434 l 805 434 b 608 391 745 406 678 391 b 410 434 537 391 470 406 l 390 434 l 506 174 l 694 174 b 911 -43 814 174 911 76 b 867 -174 911 -92 894 -137 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 69 b 851 434 1215 271 1052 434 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1649 1042 l 87 1042 b 0 955 39 1042 0 1003 l 0 87 b 87 0 0 39 39 0 l 521 0 b 680 102 590 0 652 40 l 755 270 b 868 347 776 317 820 347 b 981 270 916 347 960 317 l 1056 102 b 1215 0 1084 40 1146 0 l 1649 0 b 1736 87 1697 0 1736 39 l 1736 955 b 1649 1042 1736 1003 1697 1042 z m 434 391 b 260 564 338 391 260 468 b 434 738 260 660 338 738 b 608 564 530 738 608 660 b 434 391 608 468 530 391 z m 1302 391 b 1128 564 1206 391 1128 468 b 1302 738 1128 660 1206 738 b 1476 564 1398 738 1476 660 b 1302 391 1476 468 1398 391 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 425 521 l 43 521 b 0 477 20 521 0 501 l 0 391 b 43 347 0 367 20 347 l 429 347 b 520 275 472 347 513 318 b 434 174 529 221 487 174 b 352 233 396 174 363 199 b 310 260 346 250 328 260 l 221 260 b 178 211 195 260 173 237 b 434 0 202 91 307 0 b 692 295 589 0 712 136 b 425 521 675 426 557 521 z m 43 608 l 955 608 b 1209 924 1117 608 1245 756 b 1011 1123 1189 1022 1109 1102 b 699 918 861 1154 726 1055 b 742 868 694 892 716 868 l 831 868 b 872 895 849 868 867 878 b 955 955 884 930 917 955 b 1041 853 1007 955 1050 908 b 950 781 1034 811 993 781 l 43 781 b 0 738 20 781 0 762 l 0 651 b 43 608 0 627 20 608 z m 1085 521 l 661 521 b 769 347 713 476 751 416 l 1085 347 b 1215 217 1157 347 1215 289 b 1085 87 1215 145 1157 87 b 972 153 1037 87 995 114 b 933 174 964 167 949 174 l 841 174 b 800 116 812 174 790 144 b 1151 -80 848 -21 991 -114 b 1380 145 1262 -56 1354 33 b 1085 521 1427 343 1276 521 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1376 1018 l 1192 1203 b 1131 1203 1175 1220 1148 1220 l 1069 1141 b 1069 1080 1052 1124 1052 1097 l 861 872 b 480 798 735 925 583 901 l 51 368 b 51 123 -17 301 -17 191 l 296 -123 b 542 -123 364 -191 474 -191 l 972 307 b 1045 688 1075 410 1099 561 l 1253 896 b 1315 896 1270 879 1298 879 l 1376 957 b 1376 1018 1393 974 1393 1001 z m 486 67 l 241 313 l 572 644 l 817 398 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 130 260 b 0 130 58 260 0 202 b 130 0 0 58 58 0 b 260 130 202 0 260 58 b 130 260 260 202 202 260 z m 1259 260 b 1128 130 1187 260 1128 202 b 1259 0 1128 58 1187 0 b 1389 130 1331 0 1389 58 b 1259 260 1389 202 1331 260 z m 936 230 b 909 345 948 270 940 314 b 795 372 878 376 834 384 b 694 434 776 409 738 434 b 594 372 650 434 613 409 b 480 345 555 384 511 376 b 453 230 448 314 441 270 b 390 130 416 211 390 174 b 453 30 390 86 416 49 b 480 -85 440 -9 448 -53 b 560 -118 502 -107 531 -118 b 594 -111 572 -118 583 -115 b 694 -174 613 -148 650 -174 b 795 -111 738 -174 776 -148 b 829 -118 806 -115 817 -118 b 909 -85 858 -118 887 -107 b 936 30 940 -53 948 -9 b 998 130 972 49 998 86 b 936 230 998 174 972 211 z m 1389 608 b 1128 868 1389 751 1272 868 b 1124 868 1127 868 1125 868 b 1128 911 1127 882 1128 896 b 911 1128 1128 1031 1031 1128 b 746 1050 845 1128 786 1098 b 477 1215 696 1148 595 1215 b 174 911 310 1215 174 1079 b 179 854 174 892 176 873 b 0 608 75 820 0 723 b 260 347 0 464 117 347 l 378 347 b 418 406 388 369 401 389 b 556 464 455 443 504 464 b 694 521 593 500 642 521 b 833 464 747 521 796 500 b 971 406 885 464 934 443 b 1011 347 988 389 1001 369 l 1128 347 b 1389 608 1272 347 1389 464 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 951 604 b 738 781 932 704 844 781 b 645 760 704 781 673 773 b 434 868 598 825 521 868 b 174 608 290 868 174 751 b 174 603 174 606 174 605 b 0 391 75 583 0 496 b 217 174 0 271 97 174 l 911 174 b 1128 391 1031 174 1128 271 b 951 604 1128 497 1052 585 z m 1541 608 b 1174 909 1350 572 1174 718 b 1330 1175 1174 1019 1234 1120 b 1325 1209 1345 1183 1341 1206 b 1253 1215 1301 1213 1277 1215 b 861 841 1041 1215 870 1049 b 1021 674 933 809 992 750 b 1211 438 1121 635 1194 545 b 1252 434 1225 437 1238 434 b 1558 579 1374 434 1485 489 b 1541 608 1569 592 1557 611 z m 989 81 b 930 65 968 93 942 86 l 830 -109 b 847 -168 818 -130 826 -156 b 868 -174 853 -172 861 -174 b 906 -152 883 -174 898 -166 l 1005 22 b 989 81 1017 43 1010 69 z m 728 81 b 669 65 708 93 681 86 l 570 -109 b 586 -168 558 -130 565 -156 b 608 -174 593 -172 600 -174 b 645 -152 623 -174 637 -166 l 745 22 b 728 81 756 43 749 69 z m 468 81 b 409 65 447 93 421 86 l 310 -109 b 326 -168 298 -130 305 -156 b 347 -174 333 -172 340 -174 b 385 -152 362 -174 377 -166 l 484 22 b 468 81 496 43 489 69 z m 208 81 b 148 65 187 93 160 86 l 49 -109 b 65 -168 37 -130 44 -156 b 87 -174 72 -172 79 -174 b 125 -152 102 -174 116 -166 l 224 22 b 208 81 235 43 228 69 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1128 868 b 1124 868 1127 868 1125 868 b 1128 911 1127 882 1128 896 b 911 1128 1128 1031 1031 1128 b 746 1050 845 1128 786 1098 b 477 1215 696 1148 595 1215 b 174 911 310 1215 174 1079 b 179 854 174 892 176 873 b 0 608 75 820 0 723 b 260 347 0 464 117 347 l 1128 347 b 1389 608 1272 347 1389 464 b 1128 868 1389 751 1272 868 z m 239 200 b 130 -38 204 80 130 47 b 260 -174 130 -113 189 -174 b 391 -38 332 -174 391 -113 b 282 200 391 47 317 79 b 239 200 276 222 245 223 z m 673 200 b 564 -38 638 80 564 47 b 694 -174 564 -113 623 -174 b 825 -38 766 -174 825 -113 b 716 200 825 47 751 79 b 673 200 710 222 680 223 z m 1107 200 b 998 -38 1072 80 998 47 b 1128 -174 998 -113 1057 -174 b 1259 -38 1200 -174 1259 -113 b 1150 200 1259 47 1185 79 b 1107 200 1144 222 1114 223 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 499 211 b 440 195 478 223 452 216 l 266 -109 b 282 -168 254 -130 262 -156 b 304 -174 289 -172 296 -174 b 342 -152 319 -174 333 -166 l 515 152 b 499 211 527 173 520 199 z m 759 211 b 700 195 739 223 712 216 l 527 -109 b 543 -168 515 -130 522 -156 b 564 -174 550 -172 557 -174 b 602 -152 579 -174 594 -166 l 776 152 b 759 211 787 173 780 199 z m 238 211 b 179 195 218 223 191 216 l 6 -109 b 22 -168 -6 -130 1 -156 b 43 -174 29 -172 36 -174 b 81 -152 58 -174 73 -166 l 255 152 b 238 211 267 173 259 199 z m 1280 211 b 1221 195 1259 223 1233 216 l 1047 -109 b 1064 -168 1035 -130 1043 -156 b 1085 -174 1070 -172 1078 -174 b 1123 -152 1100 -174 1115 -166 l 1296 152 b 1280 211 1308 173 1301 199 z m 1020 211 b 961 195 999 223 972 216 l 787 -109 b 803 -168 775 -130 782 -156 b 825 -174 810 -172 817 -174 b 862 -152 840 -174 854 -166 l 1036 152 b 1020 211 1048 173 1041 199 z m 1128 868 b 1124 868 1127 868 1125 868 b 1128 911 1127 882 1128 896 b 911 1128 1128 1031 1031 1128 b 746 1050 845 1128 786 1098 b 477 1215 696 1148 595 1215 b 174 911 310 1215 174 1079 b 179 854 174 892 176 873 b 0 608 75 820 0 723 b 260 347 0 464 117 347 l 1128 347 b 1389 608 1272 347 1389 464 b 1128 868 1389 751 1272 868 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1385 604 b 1172 781 1366 704 1278 781 b 1079 760 1139 781 1107 773 b 868 868 1032 825 955 868 b 608 608 724 868 608 751 b 608 603 608 606 608 605 b 434 391 509 583 434 496 b 651 174 434 271 531 174 l 1345 174 b 1563 391 1465 174 1563 271 b 1385 604 1563 497 1486 585 z m 337 510 b 337 879 235 612 235 777 b 680 899 431 972 578 978 b 868 955 735 934 799 955 b 873 954 870 955 872 954 l 898 1027 b 853 1071 907 1054 881 1080 l 649 1003 l 552 1196 b 489 1196 539 1222 502 1222 l 393 1003 l 189 1071 b 144 1026 161 1080 135 1054 l 212 822 l 20 726 b 20 663 -7 713 -7 676 l 212 566 l 144 362 b 189 317 135 334 161 308 l 349 371 b 347 390 349 377 347 384 b 364 488 347 424 354 457 b 337 510 355 495 345 502 z m 526 668 b 613 841 538 734 569 793 b 521 868 586 858 554 868 b 347 694 425 868 347 790 b 404 567 347 644 369 598 b 526 668 435 610 477 645 z m 1423 81 b 1364 65 1402 93 1376 86 l 1264 -109 b 1281 -168 1252 -130 1260 -156 b 1302 -174 1287 -172 1295 -174 b 1340 -152 1317 -174 1332 -166 l 1439 22 b 1423 81 1451 43 1444 69 z m 1162 81 b 1103 65 1142 93 1115 86 l 1004 -109 b 1020 -168 992 -130 999 -156 b 1042 -174 1027 -172 1034 -174 b 1079 -152 1057 -174 1071 -166 l 1179 22 b 1162 81 1190 43 1183 69 z m 902 81 b 843 65 881 93 855 86 l 744 -109 b 760 -168 732 -130 739 -156 b 781 -174 767 -172 774 -174 b 819 -152 796 -174 811 -166 l 918 22 b 902 81 930 43 923 69 z m 642 81 b 582 65 621 93 594 86 l 483 -109 b 499 -168 471 -130 479 -156 b 521 -174 506 -172 514 -174 b 559 -152 536 -174 550 -166 l 658 22 b 642 81 669 43 662 69 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1729 518 l 1676 598 b 1334 781 1599 713 1471 781 l 694 781 l 474 1002 b 495 1202 529 1056 535 1140 b 453 1206 485 1216 465 1218 l 340 1093 l 224 1209 b 190 1206 214 1219 197 1217 b 206 1042 157 1155 162 1086 b 237 1021 215 1033 226 1027 b 220 1007 231 1017 224 1013 l 17 737 b 9 646 -2 711 -5 676 l 48 569 b 126 521 63 540 93 521 l 210 521 b 271 546 233 521 255 530 l 374 640 l 521 347 l 1476 347 l 1476 558 b 1584 422 1520 525 1525 511 b 1645 410 1598 402 1625 397 l 1717 458 b 1729 518 1737 471 1742 498 z m 803 555 l 759 512 l 769 450 b 750 436 771 439 760 431 l 694 465 l 639 436 b 620 450 629 431 618 439 l 630 512 l 585 555 b 593 578 577 563 582 577 l 655 587 l 683 643 b 707 643 687 653 702 653 l 735 587 l 796 578 b 803 555 807 577 812 563 z m 1107 555 l 1063 512 l 1073 450 b 1054 436 1075 439 1063 431 l 998 465 l 943 436 b 923 450 933 431 921 439 l 934 512 l 889 555 b 897 578 881 563 886 577 l 959 587 l 986 643 b 1011 643 991 653 1006 653 l 1038 587 l 1100 578 b 1107 555 1111 577 1115 563 z m 1411 555 l 1366 512 l 1377 450 b 1357 436 1379 439 1367 431 l 1302 465 l 1247 436 b 1227 450 1237 431 1225 439 l 1238 512 l 1193 555 b 1201 578 1185 563 1190 577 l 1262 587 l 1290 643 b 1315 643 1295 653 1309 653 l 1342 587 l 1404 578 b 1411 555 1415 577 1419 563 z m 521 -130 b 564 -174 521 -154 540 -174 l 738 -174 b 781 -130 762 -174 781 -154 l 781 87 l 1215 87 l 1215 -130 b 1259 -174 1215 -154 1235 -174 l 1432 -174 b 1476 -130 1456 -174 1476 -154 l 1476 260 l 521 260 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 87 1215 b 0 1128 39 1215 0 1176 l 0 -130 b 43 -174 0 -154 20 -174 l 130 -174 b 174 -130 154 -174 174 -154 l 174 1128 b 87 1215 174 1176 135 1215 z m 814 392 b 260 362 658 433 511 470 l 260 174 b 1339 236 764 424 862 29 b 1389 314 1370 250 1389 281 l 1389 412 b 1085 347 1271 365 1171 347 b 814 392 983 347 898 369 z m 814 652 b 260 622 658 693 511 730 l 260 455 b 836 476 518 557 680 517 b 1389 506 991 435 1139 398 l 1389 673 b 1085 607 1271 626 1171 607 b 814 652 983 607 898 630 z m 839 994 b 1389 1048 980 951 1104 915 l 1389 1132 b 1255 1204 1389 1200 1316 1235 b 260 1127 790 967 828 1384 l 260 715 b 836 736 518 817 680 777 b 1389 766 991 695 1139 658 l 1389 953 b 1047 868 1243 889 1136 868 b 814 911 958 868 887 889 b 782 920 803 914 792 917 l 782 1010 b 839 994 801 1005 820 999 z m 434 868 b 391 911 410 868 391 887 b 434 955 391 935 410 955 b 477 911 458 955 477 935 b 434 868 477 887 458 868 z m 434 1019 b 391 1063 410 1019 391 1038 b 434 1106 391 1086 410 1106 b 477 1063 458 1106 477 1087 b 434 1019 477 1039 458 1019 z m 608 889 b 564 933 584 889 564 908 b 608 976 564 956 584 976 b 651 933 632 976 651 957 b 608 889 651 909 632 889 z m 608 1041 b 564 1084 584 1041 564 1060 b 608 1128 564 1108 584 1128 b 651 1084 632 1128 651 1108 b 608 1041 651 1060 632 1041 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1332 1213 b 823 1047 1227 1182 1029 1120 b 808 1098 818 1066 812 1084 b 746 1123 800 1124 770 1136 b 164 737 631 1063 331 899 b 155 729 161 734 158 731 b 0 355 52 626 0 490 b 155 -19 0 220 52 84 b 528 -174 258 -122 393 -174 b 903 -19 664 -174 799 -122 b 911 -9 905 -16 908 -12 b 1297 572 1073 158 1237 458 b 1271 634 1310 597 1298 626 b 1221 650 1257 639 1240 644 b 1387 1159 1294 856 1356 1054 b 1332 1213 1397 1192 1366 1223 z m 521 0 b 174 347 329 0 174 155 b 521 694 174 539 329 694 b 868 347 713 694 868 539 b 521 0 868 155 713 0 z m 434 521 b 347 434 386 521 347 482 b 434 347 347 386 386 347 b 521 434 482 347 521 386 b 434 521 521 482 482 521 z m 564 260 b 521 217 540 260 521 241 b 564 174 521 193 540 174 b 608 217 588 174 608 193 b 564 260 608 241 588 260 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 521 -130 b 564 -174 521 -154 540 -174 l 651 -174 b 694 -130 675 -174 694 -154 l 694 347 l 521 347 z m 608 608 l 470 608 l 347 731 b 224 781 314 763 270 781 l 174 781 b 51 730 127 781 84 763 b 0 607 18 697 0 654 l 1 347 l 0 -87 b 87 -174 0 -135 39 -174 b 173 -87 134 -174 173 -135 l 174 186 b 180 182 176 185 178 183 l 259 65 l 259 -87 b 346 -174 259 -135 298 -174 b 433 -87 394 -174 433 -135 l 433 66 b 415 144 433 93 427 120 l 303 310 l 303 529 l 360 472 b 452 434 384 448 417 434 l 608 434 b 694 521 656 434 694 473 b 608 608 694 569 656 608 z m 174 868 b 304 998 245 868 304 926 b 174 1128 304 1070 245 1128 b 43 998 102 1128 43 1070 b 174 868 43 926 102 868 z m 781 1128 l 867 523 l 783 104 b 868 0 771 45 818 0 b 956 82 909 0 944 25 b 1042 0 958 37 995 0 b 1128 87 1090 0 1128 39 b 1215 0 1128 39 1167 0 b 1302 87 1263 0 1302 39 l 1302 1215 l 781 1215 z m 521 1128 l 521 694 l 694 694 l 694 1215 l 608 1215 b 521 1128 560 1215 521 1176 z m 1476 1215 l 1389 1215 l 1389 -130 b 1432 -174 1389 -154 1408 -174 l 1519 -174 b 1563 -130 1543 -174 1563 -154 l 1563 1128 b 1476 1215 1563 1176 1524 1215 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 836 304 l 679 304 l 726 480 b 694 521 731 500 716 521 l 510 521 b 478 493 494 521 480 509 l 434 167 b 467 130 432 148 447 130 l 627 130 l 565 -133 b 597 -174 560 -154 576 -174 b 625 -157 608 -174 619 -168 l 864 255 b 836 304 876 277 860 304 z m 1016 606 b 1042 694 1032 632 1042 662 b 868 868 1042 790 964 868 l 852 868 b 868 955 862 895 868 924 b 608 1215 868 1099 751 1215 b 567 1211 594 1215 580 1213 b 608 1085 592 1176 608 1132 b 391 868 608 965 511 868 l 347 868 b 174 694 251 868 174 790 b 200 606 174 662 184 632 b 0 391 88 597 0 504 b 217 174 0 271 98 174 l 348 174 b 348 178 348 175 348 177 l 391 504 b 510 608 400 563 450 608 l 694 608 b 789 561 732 608 766 591 b 810 458 812 531 819 494 l 792 391 l 835 391 b 939 331 878 391 917 368 b 939 212 960 294 960 248 l 917 174 l 998 174 b 1215 391 1117 174 1215 271 b 1016 606 1215 504 1127 597 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 728 1127 b 0 320 313 1099 0 735 l 0 -43 b 43 -87 0 -67 20 -87 l 130 -87 b 174 -43 154 -87 174 -67 l 174 347 b 861 950 174 709 491 997 b 1389 326 1168 910 1389 635 l 1389 -43 b 1432 -87 1389 -67 1408 -87 l 1519 -87 b 1563 -43 1543 -87 1563 -67 l 1563 347 b 728 1127 1563 796 1183 1157 z m 713 864 b 260 329 450 830 260 594 l 260 -43 b 304 -87 260 -67 280 -87 l 391 -87 b 434 -43 414 -87 434 -67 l 434 347 b 816 693 434 550 609 713 b 1128 332 996 675 1128 513 l 1128 -43 b 1172 -87 1128 -67 1148 -87 l 1259 -87 b 1302 -43 1283 -87 1302 -67 l 1302 347 b 713 864 1302 657 1030 904 z m 729 602 b 521 337 606 578 521 463 l 521 -43 b 564 -87 521 -67 540 -87 l 651 -87 b 694 -43 675 -87 694 -67 l 694 347 b 781 434 694 395 733 434 b 868 347 829 434 868 395 l 868 -43 b 911 -87 868 -67 888 -87 l 998 -87 b 1042 -43 1022 -87 1042 -67 l 1042 347 b 729 602 1042 508 896 635 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1476 694 b 1042 1128 1476 934 1281 1128 l 434 1128 b 0 694 194 1128 0 934 l 0 521 l 1476 521 z m 478 753 l 425 701 l 437 627 b 414 610 439 613 426 603 l 347 645 l 281 610 b 257 627 269 603 255 613 l 270 701 l 216 753 b 225 781 206 763 212 779 l 299 792 l 333 859 b 362 859 339 871 356 871 l 395 792 l 469 781 b 478 753 483 779 488 763 z m 869 753 l 815 701 l 828 627 b 804 610 830 613 816 603 l 738 645 l 671 610 b 648 627 660 603 646 613 l 661 701 l 607 753 b 616 781 597 763 602 779 l 690 792 l 723 859 b 752 859 729 871 747 871 l 785 792 l 860 781 b 869 753 873 779 878 763 z m 1259 753 l 1206 701 l 1219 627 b 1195 610 1221 613 1207 603 l 1128 645 l 1062 610 b 1038 627 1050 603 1036 613 l 1051 701 l 997 753 b 1006 781 988 763 993 779 l 1081 792 l 1114 859 b 1143 859 1120 871 1137 871 l 1176 792 l 1250 781 b 1259 753 1264 779 1269 763 z m 1693 347 l 1606 347 b 1563 304 1582 347 1563 328 l 1563 130 b 1519 87 1563 106 1543 87 b 1476 130 1495 87 1476 106 l 1476 434 l 0 434 l 0 -43 b 43 -87 0 -67 20 -87 l 304 -87 b 347 -43 328 -87 347 -67 l 347 174 l 868 174 l 868 -43 b 911 -87 868 -67 888 -87 l 1172 -87 b 1215 -43 1196 -87 1215 -67 l 1215 260 l 1302 260 l 1302 143 b 1496 -86 1302 30 1383 -74 b 1736 130 1626 -99 1736 3 l 1736 304 b 1693 347 1736 328 1717 347 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1693 217 l 217 217 b 174 174 193 217 174 197 l 174 130 b 217 87 174 106 193 87 l 1693 87 b 1736 130 1717 87 1736 106 l 1736 174 b 1693 217 1736 197 1717 217 z m 391 -43 l 43 -43 b 0 -87 20 -43 0 -63 l 0 -130 b 43 -174 0 -154 20 -174 l 391 -174 b 434 -130 414 -174 434 -154 l 434 -87 b 391 -43 434 -63 414 -43 z m 1519 -43 l 608 -43 b 564 -87 584 -43 564 -63 l 564 -130 b 608 -174 564 -154 584 -174 l 1519 -174 b 1563 -130 1543 -174 1563 -154 l 1563 -87 b 1519 -43 1563 -63 1543 -43 z m 391 434 l 814 434 b 1042 347 875 381 954 347 b 1269 434 1129 347 1208 381 l 1432 434 b 1736 738 1600 434 1736 570 b 1432 1042 1736 906 1600 1042 b 1297 1009 1383 1042 1338 1029 b 955 1215 1232 1131 1104 1215 b 673 1094 844 1215 744 1168 b 391 1215 602 1168 502 1215 b 0 825 175 1215 0 1040 b 391 434 0 609 175 434 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1128 1215 b 868 955 985 1215 868 1098 b 1128 694 868 811 985 694 b 1389 955 1272 694 1389 811 b 1128 1215 1389 1098 1272 1215 z m 1128 868 b 1042 955 1080 868 1042 907 b 1128 1042 1042 1003 1080 1042 b 1215 955 1176 1042 1215 1003 b 1128 868 1215 907 1176 868 z m 694 911 b 391 1215 694 1079 559 1215 b 87 911 223 1215 87 1079 l 87 460 b 0 217 33 393 0 309 b 391 -174 0 1 175 -174 b 781 217 606 -174 781 1 b 694 460 781 309 748 393 z m 391 0 b 174 217 271 0 174 97 b 260 390 174 286 207 350 l 260 911 b 391 1042 260 983 319 1042 b 521 911 463 1042 521 983 l 521 390 b 608 217 575 350 608 286 b 391 0 608 97 510 0 z m 434 339 l 434 911 b 391 955 434 935 414 955 b 347 911 367 955 347 935 l 347 339 b 260 217 297 321 260 274 b 391 87 260 145 319 87 b 521 217 463 87 521 145 b 434 339 521 274 484 321 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1128 1215 b 868 955 985 1215 868 1098 b 1128 694 868 811 985 694 b 1389 955 1272 694 1389 811 b 1128 1215 1389 1098 1272 1215 z m 1128 868 b 1042 955 1080 868 1042 907 b 1128 1042 1042 1003 1080 1042 b 1215 955 1176 1042 1215 1003 b 1128 868 1215 907 1176 868 z m 694 911 b 391 1215 694 1079 559 1215 b 87 911 223 1215 87 1079 l 87 460 b 0 217 33 393 0 309 b 391 -174 0 1 175 -174 b 781 217 606 -174 781 1 b 694 460 781 309 748 393 z m 391 0 b 174 217 271 0 174 97 b 260 390 174 286 207 350 l 260 911 b 391 1042 260 983 319 1042 b 521 911 463 1042 521 983 l 521 390 b 608 217 575 350 608 286 b 391 0 608 97 510 0 z m 434 339 l 434 391 b 391 434 434 414 414 434 b 347 391 367 434 347 414 l 347 339 b 260 217 297 321 260 274 b 391 87 260 145 319 87 b 521 217 463 87 521 145 b 434 339 521 274 484 321 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1649 347 l 1476 347 l 1476 174 l 1536 174 b 1563 152 1551 174 1563 164 l 1563 109 b 1536 87 1563 97 1551 87 l 200 87 b 174 109 185 87 174 97 l 174 152 b 200 174 174 164 185 174 l 260 174 l 260 347 l 87 347 b 0 260 39 347 0 308 l 0 0 b 87 -87 0 -48 39 -87 l 1649 -87 b 1736 0 1697 -87 1736 -48 l 1736 260 b 1649 347 1736 308 1697 347 z m 1389 174 l 1389 1041 b 1301 1128 1389 1089 1350 1128 l 435 1128 b 347 1041 387 1128 347 1089 l 347 174 z m 573 667 l 642 736 b 683 736 653 747 672 747 l 795 623 l 1054 879 b 1095 879 1065 890 1083 890 l 1163 809 b 1163 768 1175 798 1175 780 l 815 423 b 774 423 804 412 785 412 l 573 626 b 573 667 561 637 562 656 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1525 174 b 1368 236 1466 180 1411 202 b 1237 236 1329 266 1275 267 b 842 240 1134 154 946 154 b 715 235 805 270 752 265 b 321 240 612 154 425 154 b 193 234 285 270 231 264 b 37 174 151 201 96 180 b 0 131 15 171 0 152 l 0 44 b 45 0 0 19 21 -2 b 261 71 123 7 197 31 b 782 71 414 -23 632 -22 b 1302 71 935 -23 1153 -22 b 1517 1 1366 32 1440 7 b 1562 44 1542 -2 1562 19 l 1562 130 b 1525 174 1563 151 1547 171 z m 1525 565 b 1368 627 1466 571 1411 593 b 1237 627 1329 657 1275 657 b 842 631 1134 544 946 544 b 715 626 805 661 752 655 b 321 630 612 544 425 544 b 193 625 285 661 231 655 b 37 564 151 592 96 570 b 0 522 15 562 0 543 l 0 434 b 45 391 0 410 21 389 b 261 461 123 398 197 422 b 782 461 414 367 632 369 b 1302 461 935 367 1153 369 b 1517 391 1366 423 1440 398 b 1562 435 1542 389 1562 410 l 1562 520 b 1525 565 1563 542 1547 562 z m 1525 955 b 1368 1017 1466 962 1411 984 b 1237 1017 1329 1047 1275 1048 b 842 1021 1134 935 946 935 b 715 1016 805 1052 752 1046 b 321 1021 612 935 425 935 b 193 1016 285 1051 231 1045 b 37 955 151 983 96 961 b 0 912 15 953 0 934 l 0 825 b 45 782 0 800 21 779 b 261 852 123 788 197 812 b 782 852 414 758 632 759 b 1302 852 935 758 1153 759 b 1517 782 1366 813 1440 789 b 1562 825 1542 780 1562 801 l 1562 911 b 1525 955 1563 933 1547 953 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 521 781 b 738 998 641 781 738 878 b 521 1215 738 1118 641 1215 b 304 998 401 1215 304 1118 b 521 781 304 878 401 781 z m 376 106 l 445 193 l 279 332 l 154 176 b 152 43 123 138 122 83 l 282 -130 b 369 -174 304 -159 336 -174 b 434 -152 391 -174 414 -167 b 456 0 482 -116 492 -48 z m 763 332 l 596 193 l 666 106 l 586 0 b 608 -152 550 -48 560 -116 b 673 -174 627 -167 650 -174 b 760 -130 706 -174 738 -159 l 890 43 b 888 176 919 83 919 138 z m 1022 822 b 871 849 987 871 920 883 l 761 771 b 281 771 618 671 424 671 l 171 848 b 20 822 122 883 54 871 b 46 671 -15 773 -3 705 l 156 594 b 304 518 202 561 252 537 l 304 434 l 738 434 l 738 518 b 885 593 789 537 839 561 l 996 671 b 1022 822 1045 705 1056 773 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 393 1169 b 246 1195 362 1217 292 1231 b 0 694 96 1074 0 895 l 694 694 z m 1345 955 l 1215 955 b 1042 781 1120 955 1042 877 l 1042 608 l 0 608 b 164 253 0 470 62 346 b 0 43 70 229 0 145 b 217 -174 0 -76 97 -174 b 434 43 337 -174 434 -76 b 422 112 434 68 429 90 b 608 87 481 96 543 87 b 793 112 672 87 735 96 b 781 43 786 90 781 68 b 998 -174 781 -76 878 -174 b 1215 43 1118 -174 1215 -76 b 1052 253 1215 145 1146 229 b 1215 608 1153 346 1215 470 l 1215 781 l 1345 781 b 1389 825 1369 781 1389 801 l 1389 911 b 1345 955 1389 935 1369 955 z m 217 -43 b 130 43 169 -43 130 -4 b 217 130 130 91 169 130 b 304 43 265 130 304 91 b 217 -43 304 -4 265 -43 z m 1085 43 b 998 -43 1085 -4 1046 -43 b 911 43 951 -43 911 -4 b 998 130 911 91 951 130 b 1085 43 1046 130 1085 91 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 781 911 b 924 885 831 911 879 901 b 1004 996 960 913 988 951 b 781 1042 936 1025 860 1042 b 559 996 702 1042 627 1026 b 639 886 575 952 602 914 b 781 911 683 901 731 911 z m 444 25 b 510 140 476 57 499 97 b 350 438 421 212 362 317 b 279 450 327 445 304 450 b 218 442 258 450 238 447 b 444 25 229 271 315 122 z m 1052 141 b 1118 26 1063 97 1086 58 b 1344 442 1247 122 1333 272 b 1283 450 1324 447 1304 450 b 1211 437 1258 450 1235 445 b 1052 141 1200 317 1141 212 z m 1360 697 b 1210 751 1312 725 1261 742 b 1237 903 1226 799 1237 850 b 1126 1208 1237 1015 1198 1123 b 1099 1213 1119 1216 1108 1218 b 1089 1186 1090 1207 1086 1196 b 1107 1082 1101 1149 1107 1115 b 781 748 1107 898 961 748 b 456 1082 602 748 456 898 b 474 1186 456 1115 462 1149 b 464 1213 477 1196 473 1207 b 437 1208 455 1218 444 1216 b 326 903 365 1123 326 1015 b 353 751 326 850 336 799 b 203 697 301 742 250 725 b 1 446 109 641 37 552 b 11 420 -2 436 2 425 b 38 424 20 414 31 416 b 117 492 63 453 89 476 b 562 370 272 584 472 529 b 442 -85 651 211 598 7 b 345 -122 415 -102 383 -114 b 328 -144 335 -124 328 -133 b 346 -165 328 -155 335 -164 b 658 -112 453 -187 564 -168 b 781 -3 707 -83 747 -46 b 904 -112 815 -46 855 -83 b 1216 -165 998 -168 1109 -187 b 1234 -144 1227 -164 1234 -155 b 1217 -122 1234 -133 1227 -124 b 1120 -85 1179 -114 1147 -102 b 1000 370 964 7 911 211 b 1445 492 1090 529 1290 584 b 1524 424 1473 476 1499 453 b 1551 420 1531 416 1542 414 b 1561 446 1560 425 1564 436 b 1360 697 1525 552 1454 642 z m 781 347 b 651 477 709 347 651 405 b 781 607 651 549 709 607 b 911 477 853 607 911 549 b 781 347 911 405 853 347 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 467 600 b 391 535 428 608 391 576 l 391 399 b 436 339 391 371 410 349 b 521 217 485 321 521 273 b 391 87 521 145 463 87 b 260 217 319 87 260 145 l 260 890 b 195 955 260 926 231 955 l 65 955 b 0 890 29 955 0 926 l 0 217 b 475 -165 0 -26 223 -218 b 772 133 622 -134 741 -14 b 467 600 819 358 672 559 z m 567 1215 b 521 1172 542 1217 521 1197 l 521 1086 b 562 1043 521 1063 539 1044 b 1215 388 913 1024 1195 739 b 1258 347 1216 365 1235 347 l 1345 347 b 1389 393 1370 347 1390 368 b 567 1215 1366 836 1010 1192 z m 568 955 b 521 911 543 957 521 937 l 521 824 b 561 781 521 801 538 783 b 954 387 769 764 935 596 b 997 347 956 365 975 347 l 1085 347 b 1128 394 1110 347 1130 369 b 568 955 1105 693 866 932 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 694 l 0 694 z m 174 477 b 217 521 174 501 193 521 l 477 521 b 521 477 501 521 521 501 l 521 217 b 477 174 521 193 501 174 l 217 174 b 174 217 193 174 174 193 z m 1085 1042 l 955 1042 l 955 1172 b 911 1215 955 1196 935 1215 l 825 1215 b 781 1172 801 1215 781 1196 l 781 1042 l 434 1042 l 434 1172 b 391 1215 434 1196 414 1215 l 304 1215 b 260 1172 280 1215 260 1196 l 260 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 781 l 1215 781 l 1215 911 b 1085 1042 1215 983 1157 1042 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 0 -43 b 130 -174 0 -115 58 -174 l 1085 -174 b 1215 -43 1157 -174 1215 -115 l 1215 694 l 0 694 z m 174 477 b 217 521 174 501 193 521 l 998 521 b 1042 477 1022 521 1042 501 l 1042 304 b 998 260 1042 280 1022 260 l 217 260 b 174 304 193 260 174 280 z m 1085 1042 l 955 1042 l 955 1172 b 911 1215 955 1196 935 1215 l 825 1215 b 781 1172 801 1215 781 1196 l 781 1042 l 434 1042 l 434 1172 b 391 1215 434 1196 414 1215 l 304 1215 b 260 1172 280 1215 260 1196 l 260 1042 l 130 1042 b 0 911 58 1042 0 983 l 0 781 l 1215 781 l 1215 911 b 1085 1042 1215 983 1157 1042 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1350 966 b 956 1215 1274 1125 1117 1215 b 734 1154 880 1215 803 1195 l 659 1109 b 630 990 618 1085 605 1032 l 719 841 b 793 799 735 814 764 799 b 838 811 808 799 824 803 l 912 856 b 957 868 926 864 942 868 b 1031 826 986 868 1015 853 b 1002 707 1056 785 1042 731 l 42 137 b 12 18 1 112 -12 59 l 101 -131 b 176 -174 118 -158 146 -174 b 221 -161 191 -174 206 -170 l 1163 399 b 1350 966 1357 515 1446 761 z m 868 930 l 861 926 l 805 1093 b 887 1121 831 1106 859 1115 l 943 953 b 868 930 917 951 891 944 z m 396 44 l 232 148 l 315 198 l 479 94 z m 687 217 l 523 322 l 607 371 l 771 267 z m 988 396 l 824 500 l 908 550 l 1072 446 z m 1044 931 l 1168 1056 b 1230 995 1191 1038 1212 1018 l 1106 871 b 1044 931 1090 896 1069 917 z m 1266 639 l 1105 693 b 1125 739 1114 707 1121 722 b 1130 776 1128 751 1130 764 l 1294 721 b 1266 639 1288 693 1279 665 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 809 790 b 401 701 666 860 498 819 l 551 552 b 551 490 568 535 568 507 b 521 478 543 482 532 478 b 490 490 509 478 498 482 l 354 627 l 6 -86 b 6 -139 -2 -102 -2 -122 b 88 -167 21 -170 58 -182 l 450 9 l 317 143 b 317 204 299 160 299 187 b 378 204 334 221 361 221 l 532 50 l 809 185 b 964 340 874 216 930 270 b 809 790 1046 507 976 709 z m 1059 885 b 977 1215 1112 996 1084 1126 b 955 803 840 1102 834 924 l 977 782 b 1389 803 1097 661 1276 667 b 1059 885 1300 911 1169 938 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1386 188 l 1314 622 b 1228 694 1307 664 1271 694 l 564 694 l 564 868 l 825 868 b 868 911 849 868 868 887 l 868 1172 b 825 1215 868 1196 849 1215 l 130 1215 b 87 1172 106 1215 87 1196 l 87 911 b 130 868 87 888 106 868 l 391 868 l 391 694 l 160 694 b 75 622 118 694 82 664 l 2 188 b 0 160 1 178 0 169 l 0 -87 b 87 -174 0 -135 39 -174 l 1302 -174 b 1389 -87 1350 -174 1389 -135 l 1389 159 b 1386 188 1389 169 1388 178 z m 760 543 b 803 586 760 566 779 586 l 846 586 b 890 543 870 586 890 566 l 890 499 b 846 456 890 475 870 456 l 803 456 b 760 499 779 456 760 475 z m 673 369 l 716 369 b 760 326 740 369 760 349 l 760 282 b 716 239 760 258 740 239 l 673 239 b 629 282 649 239 629 258 l 629 326 b 673 369 629 349 649 369 z m 586 586 b 629 543 610 586 629 566 l 629 499 b 586 456 629 475 610 456 l 543 456 b 499 499 519 456 499 475 l 499 543 b 543 586 499 566 519 586 z m 217 998 l 217 1085 l 738 1085 l 738 998 z m 326 456 l 282 456 b 239 499 258 456 239 475 l 239 543 b 282 586 239 566 258 586 l 326 586 b 369 543 349 586 369 566 l 369 499 b 326 456 369 475 349 456 z m 369 282 l 369 326 b 412 369 369 349 388 369 l 456 369 b 499 326 480 369 499 349 l 499 282 b 456 239 499 258 480 239 l 412 239 b 369 282 388 239 369 258 z m 955 -22 b 933 -43 955 -34 945 -43 l 456 -43 b 434 -22 444 -43 434 -34 l 434 22 b 456 43 434 34 444 43 l 933 43 b 955 22 945 43 955 34 z m 1020 282 b 977 239 1020 258 1000 239 l 933 239 b 890 282 909 239 890 258 l 890 326 b 933 369 890 349 909 369 l 977 369 b 1020 326 1000 369 1020 349 z m 1150 499 b 1107 456 1150 475 1131 456 l 1063 456 b 1020 499 1039 456 1020 475 l 1020 543 b 1063 586 1020 566 1039 586 l 1107 586 b 1150 543 1131 586 1150 566 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 543 434 l 239 434 b 193 323 181 434 152 364 l 282 239 l 13 -31 b 13 -92 -4 -47 -4 -75 l 82 -161 b 143 -161 98 -178 126 -178 l 412 109 l 497 19 b 608 65 538 -22 608 7 l 608 369 b 543 434 608 405 579 434 z m 846 608 l 1150 608 b 1196 719 1208 608 1237 678 l 1107 803 l 1376 1072 b 1376 1134 1393 1089 1393 1117 l 1307 1203 b 1246 1203 1290 1219 1263 1219 l 977 933 l 892 1023 b 781 977 851 1064 781 1035 l 781 673 b 846 608 781 637 810 608 z m 1107 239 l 1196 323 b 1150 434 1237 364 1208 434 l 846 434 b 781 369 810 434 781 405 l 781 65 b 892 19 781 7 852 -22 l 977 108 l 1246 -161 b 1307 -161 1263 -178 1290 -178 l 1376 -92 b 1376 -31 1393 -75 1393 -48 z m 496 1022 l 412 933 l 143 1203 b 82 1203 126 1219 98 1219 l 13 1134 b 13 1072 -4 1117 -4 1089 l 282 803 l 193 719 b 239 608 152 678 181 608 l 543 608 b 608 673 579 608 608 637 l 608 977 b 496 1022 608 1034 537 1063 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 1519 781 b 1561 835 1547 781 1568 808 l 1496 1096 b 1454 1128 1491 1115 1474 1128 l 1186 1128 l 1255 781 z m 738 1128 l 465 1128 l 396 781 l 738 781 z m 1097 1128 l 825 1128 l 825 781 l 1167 781 z m 43 781 l 307 781 l 377 1128 l 109 1128 b 66 1096 89 1128 71 1115 l 1 835 b 43 781 -5 808 15 781 z m 1519 608 l 1465 608 l 1476 694 l 87 694 l 98 608 l 43 608 b 0 564 20 608 0 588 l 0 477 b 43 434 0 454 20 434 l 119 434 l 174 0 l 174 -43 b 217 -87 174 -67 193 -87 l 304 -87 b 347 -43 328 -87 347 -67 l 347 0 l 1215 0 l 1215 -43 b 1259 -87 1215 -67 1235 -87 l 1345 -87 b 1389 -43 1369 -87 1389 -67 l 1389 0 l 1443 434 l 1519 434 b 1563 477 1543 434 1563 454 l 1563 564 b 1519 608 1563 588 1543 608 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1136 933 l 1136 933 l 1097 1129 l 825 1129 l 825 782 l 990 782 b 1136 933 1034 834 1082 885 z m 738 1128 l 465 1128 l 396 781 l 738 781 z m 1251 933 b 1390 793 1301 889 1348 841 b 1437 837 1405 808 1421 823 l 1495 889 l 1552 837 b 1561 829 1555 835 1558 832 b 1561 835 1561 831 1561 833 l 1496 1096 b 1454 1128 1491 1115 1474 1128 l 1186 1128 l 1219 962 z m 43 781 l 307 781 l 377 1128 l 109 1128 b 66 1096 89 1128 71 1115 l 1 835 b 43 781 -5 808 15 781 z m 924 694 l 87 694 l 98 608 l 43 608 b 0 564 20 608 0 588 l 0 477 b 43 434 0 454 20 434 l 119 434 l 174 0 l 174 -43 b 217 -87 174 -67 193 -87 l 304 -87 b 347 -43 328 -87 347 -67 l 347 0 l 914 0 b 781 343 832 91 781 211 b 924 694 781 440 835 568 z m 1495 773 b 1386 661 1455 736 1418 699 b 1193 868 1333 731 1268 802 b 868 343 1003 698 868 477 b 1302 -87 868 106 1062 -87 b 1736 343 1542 -87 1736 106 b 1495 773 1736 442 1635 647 z m 1444 150 b 1302 104 1405 121 1355 104 b 1061 343 1169 104 1061 195 b 1195 592 1061 416 1106 481 b 1377 353 1208 577 1377 353 l 1485 481 b 1506 443 1493 468 1500 455 b 1444 150 1556 343 1535 216 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1345 694 l 1215 694 l 1215 825 b 1172 868 1215 849 1196 868 l 1042 868 l 1042 998 b 998 1042 1042 1022 1022 1042 l 391 1042 b 347 998 367 1042 347 1022 l 347 868 l 217 868 b 174 825 193 868 174 849 l 174 694 l 43 694 b 0 651 20 694 0 675 l 0 43 b 43 0 0 20 20 0 l 260 0 l 260 347 l 347 347 l 347 0 l 521 0 l 521 347 l 608 347 l 608 0 l 781 0 l 781 347 l 868 347 l 868 0 l 1042 0 l 1042 347 l 1128 347 l 1128 0 l 1345 0 b 1389 43 1369 0 1389 20 l 1389 651 b 1345 694 1389 675 1369 694 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 653 689 b 772 905 658 772 700 855 b 694 955 758 935 729 955 l 615 955 l 698 1015 b 708 1075 718 1028 722 1056 l 683 1110 b 622 1120 669 1130 642 1134 l 535 1058 l 567 1141 b 541 1197 575 1164 564 1189 l 500 1212 b 445 1187 478 1221 453 1209 l 391 1043 l 337 1187 b 281 1213 328 1210 303 1221 l 240 1197 b 215 1141 218 1189 206 1164 l 246 1058 l 159 1120 b 98 1110 139 1134 112 1130 l 73 1075 b 83 1015 59 1056 64 1028 l 167 955 l 87 955 b 0 868 39 955 0 916 l 0 -87 b 87 -174 0 -135 39 -174 l 545 -174 b 521 -87 530 -148 521 -119 l 521 521 b 653 689 521 602 577 670 z m 608 -87 b 694 -174 608 -135 646 -174 l 1128 -174 l 1128 174 l 608 174 z m 1215 -174 l 1649 -174 b 1736 -87 1697 -174 1736 -135 l 1736 174 l 1215 174 z m 1649 608 l 1594 608 b 1606 672 1601 628 1606 650 b 1410 868 1606 768 1533 868 b 1172 664 1280 868 1204 739 b 933 868 1139 739 1064 868 b 738 672 811 868 738 768 b 750 608 738 650 742 628 l 694 608 b 608 521 646 608 608 569 l 608 260 l 1128 260 l 1128 608 l 1170 608 l 1172 608 l 1174 608 l 1215 608 l 1215 260 l 1736 260 l 1736 521 b 1649 608 1736 569 1697 608 z m 911 608 b 892 621 904 611 898 616 b 868 672 872 638 868 658 b 933 738 868 699 885 738 b 1054 608 984 738 1030 664 z m 1452 621 b 1432 608 1446 616 1440 611 l 1290 608 b 1410 738 1313 663 1360 738 b 1476 672 1458 738 1476 699 b 1452 621 1476 658 1472 638 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1734 39 b 1593 98 1712 94 1648 121 l 1533 73 l 1428 349 b 1522 764 1558 444 1604 621 l 1287 1172 b 1178 1209 1265 1210 1219 1225 l 868 1080 l 558 1209 b 449 1172 518 1226 471 1210 l 215 764 b 308 349 133 621 178 444 l 203 73 l 143 98 b 2 39 88 121 25 95 b 13 11 -3 28 2 15 l 454 -172 b 482 -160 465 -177 477 -171 b 423 -18 505 -105 479 -42 l 363 7 l 470 288 b 506 284 482 286 494 284 b 813 515 646 284 775 374 l 868 720 l 923 515 b 1230 284 961 374 1090 284 b 1266 288 1242 284 1254 286 l 1373 7 l 1313 -18 b 1254 -160 1257 -42 1231 -105 b 1282 -172 1259 -171 1271 -177 l 1723 11 b 1734 39 1734 15 1739 28 z m 748 776 l 444 902 l 543 1074 l 800 967 z m 988 776 l 936 967 l 1193 1074 l 1292 902 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1302 1128 l 87 1128 b 1 1029 34 1128 -7 1082 l 153 62 b 325 -87 165 -23 238 -87 l 1065 -87 b 1237 62 1152 -87 1225 -23 l 1388 1029 b 1302 1128 1395 1082 1355 1128 z m 1201 955 l 1119 434 l 270 434 l 188 955 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 673 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 z m 1215 521 b 1186 346 1215 460 1205 401 l 1131 346 b 1100 359 1119 346 1108 351 l 1013 448 b 1013 493 1001 460 1001 480 l 1047 527 l 1047 550 b 1038 572 1047 559 1044 566 l 1013 598 b 991 607 1007 603 999 607 l 948 607 b 917 637 931 607 917 620 b 926 659 917 645 920 653 l 952 684 b 973 693 957 690 965 693 l 1060 693 b 1091 724 1077 693 1091 707 l 1091 750 b 1060 780 1091 766 1077 780 l 961 780 b 917 737 937 780 917 761 l 917 725 b 888 683 917 706 905 689 l 802 655 b 787 634 793 652 787 644 l 787 628 b 765 607 787 616 777 607 l 722 607 b 700 628 710 607 700 616 b 678 650 700 640 690 650 l 670 650 b 650 638 662 650 654 645 l 625 587 b 586 563 618 573 603 563 l 526 563 b 483 607 502 563 483 583 l 483 675 b 496 706 483 687 487 698 l 550 761 b 570 808 563 773 570 790 b 585 829 570 817 576 826 l 693 865 b 705 872 698 866 702 869 l 778 945 b 787 967 783 951 787 958 b 756 997 787 983 773 997 l 700 997 l 656 954 l 656 932 b 635 910 656 920 647 910 l 591 910 b 570 932 579 910 570 920 l 570 986 b 578 1004 570 993 573 1000 l 657 1063 b 672 1063 662 1063 667 1063 b 1215 521 972 1063 1215 820 z m 353 811 b 362 833 353 819 356 827 l 431 901 b 452 910 436 907 444 910 b 483 880 469 910 483 897 l 483 836 b 474 815 483 828 480 820 l 449 789 b 427 780 443 783 435 780 l 384 780 b 353 811 367 780 353 794 z m 700 -20 l 700 -1 b 657 42 700 23 681 42 l 602 42 b 506 74 573 42 530 57 l 446 120 b 396 218 414 143 396 180 l 396 283 b 456 389 396 327 419 367 l 573 459 b 636 476 592 470 614 476 l 721 476 b 801 447 750 476 779 466 l 918 346 l 968 346 b 1029 321 991 346 1013 337 l 1076 274 b 1111 259 1085 264 1098 259 l 1147 259 b 700 -20 1060 100 893 -11 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1345 434 l 43 434 b 0 391 20 434 0 414 l 0 304 b 43 260 0 280 20 260 l 1345 260 b 1389 304 1369 260 1389 280 l 1389 391 b 1345 434 1389 414 1369 434 z m 1345 781 l 43 781 b 0 738 20 781 0 762 l 0 651 b 43 608 0 627 20 608 l 1345 608 b 1389 651 1369 608 1389 627 l 1389 738 b 1345 781 1389 762 1369 781 z "
	},
	"": {
		ha: 694,
		x_min: 0,
		x_max: 0,
		o: "m 260 -130 l 260 1172 b 217 1215 260 1196 241 1215 l 130 1215 b 87 1172 106 1215 87 1196 l 87 -130 b 130 -174 87 -154 106 -174 l 217 -174 b 260 -130 241 -174 260 -154 z m 608 -130 l 608 1172 b 564 1215 608 1196 588 1215 l 477 1215 b 434 1172 454 1215 434 1196 l 434 -130 b 477 -174 434 -154 454 -174 l 564 -174 b 608 -130 588 -174 608 -154 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1363 1067 l 1241 1190 b 1118 1190 1207 1224 1152 1224 l 934 1006 b 934 883 900 972 900 917 l 787 736 b 425 725 664 815 514 814 b 369 633 398 698 380 667 b 253 538 352 582 305 543 b 81 460 188 532 127 506 b 131 -43 -44 335 -22 110 b 633 -93 283 -195 508 -218 b 712 80 680 -46 705 14 b 806 196 716 132 755 179 b 898 251 841 207 872 225 b 910 613 988 341 988 490 l 1057 760 b 1179 760 1090 726 1146 726 l 1364 945 b 1363 1067 1397 978 1397 1033 z m 564 260 b 434 391 492 260 434 319 b 564 521 434 463 492 521 b 694 391 636 521 694 463 b 564 260 694 319 636 260 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1285 1015 l 1278 1022 b 818 1062 1154 1149 958 1162 l 911 782 l 651 608 l 781 261 l 391 651 l 651 825 l 573 1060 b 110 1022 433 1162 236 1150 l 104 1015 b 84 521 -28 880 -34 665 l 659 -72 b 729 -72 679 -92 710 -92 l 1305 521 b 1285 1015 1423 665 1417 880 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 391 694 b 521 825 463 694 521 753 b 391 955 521 897 463 955 b 260 825 319 955 260 897 b 391 694 260 753 319 694 z m 694 825 b 825 694 694 753 753 694 b 955 825 897 694 955 753 b 825 955 955 897 897 955 b 694 825 753 955 694 897 z m 608 955 b 738 1085 680 955 738 1013 b 608 1215 738 1157 680 1215 b 477 1085 536 1215 477 1157 b 608 955 477 1013 536 955 z m 564 578 b 499 615 564 611 528 632 b 204 516 406 560 305 526 b 173 462 178 514 162 486 b 213 343 190 424 203 384 b 158 248 222 302 198 260 b 35 223 117 235 76 227 b 3 168 9 220 -8 192 b 66 -124 44 79 65 -20 b 141 -167 66 -163 107 -187 b 425 -75 231 -116 327 -85 b 456 -21 451 -73 467 -45 b 416 98 439 17 426 57 b 471 193 407 139 431 181 b 594 218 512 206 553 214 b 626 273 620 221 637 249 b 564 578 584 366 563 470 z m 1180 224 b 1057 248 1139 228 1098 236 b 1002 343 1017 260 993 302 b 1042 462 1012 385 1025 424 b 1011 517 1053 486 1037 515 b 716 615 910 527 809 560 b 651 578 687 632 651 611 b 651 565 651 574 651 569 b 705 310 651 474 670 387 b 703 204 720 276 719 238 b 743 194 716 201 730 198 b 798 99 784 182 808 140 b 758 -20 788 58 775 18 b 790 -75 747 -44 763 -72 b 1074 -167 887 -84 984 -115 b 1149 -123 1108 -186 1149 -162 b 1211 169 1149 -20 1171 80 b 1180 224 1223 193 1206 221 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1383 313 l 1193 759 b 941 974 1153 871 1057 952 b 1034 1197 990 1003 1067 1071 b 1001 1214 1030 1211 1015 1219 l 453 1008 b 0 133 97 881 0 567 l 0 -87 b 87 -174 0 -135 39 -174 l 728 -174 b 805 -48 792 -174 834 -106 l 694 174 l 694 176 b 412 364 571 185 465 259 b 416 390 407 373 409 383 l 449 422 b 484 416 459 433 477 430 b 721 260 524 324 614 260 b 848 296 767 260 810 274 l 972 123 b 1043 87 989 100 1015 87 l 1179 87 b 1241 112 1203 87 1225 96 l 1364 220 b 1383 313 1388 245 1395 281 z m 890 608 b 825 673 854 608 825 637 b 890 738 825 709 854 738 b 955 673 926 738 955 709 b 890 608 955 637 926 608 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1387 1112 b 1302 1215 1397 1166 1356 1215 l 87 1215 b 4 1104 29 1215 -13 1159 l 240 362 b 281 362 247 342 275 342 l 380 699 l 500 190 b 542 190 505 168 537 168 l 668 725 l 761 363 b 802 363 767 342 796 342 l 926 831 l 1107 -156 b 1150 -156 1112 -180 1145 -180 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 868 1123 b 781 1128 840 1127 811 1128 b 132 781 511 1128 272 991 l 868 781 z m 260 694 l 82 694 b 0 347 30 590 0 472 l 260 347 z m 955 1108 l 955 781 l 1431 781 b 955 1108 1322 944 1152 1064 z m 1302 347 l 1563 347 b 1480 694 1563 472 1532 590 l 1302 694 z m 1128 174 l 1128 -87 l 1476 -87 b 1563 0 1524 -87 1563 -48 l 1563 260 l 1116 260 b 1128 174 1123 232 1128 204 z m 1215 694 l 347 694 l 347 347 l 482 347 b 781 521 543 451 653 521 b 1080 347 909 521 1020 451 l 1215 347 z m 0 0 b 87 -87 0 -48 39 -87 l 434 -87 l 434 174 b 446 260 434 204 439 232 l 0 260 z m 781 434 b 521 174 637 434 521 317 l 521 -87 l 1042 -87 l 1042 174 b 781 434 1042 317 925 434 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 998 87 l 130 87 b 87 43 106 87 87 67 l 87 -130 b 130 -174 87 -154 106 -174 l 998 -174 b 1042 -130 1022 -174 1042 -154 l 1042 43 b 998 87 1042 67 1022 87 z m 1153 654 b 908 632 1079 715 970 706 l 852 564 l 771 912 b 303 1205 723 1123 513 1254 b 10 737 93 1157 -39 947 l 140 174 l 978 174 l 1175 410 b 1153 654 1236 483 1226 593 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 345 818 b 388 781 348 797 366 781 l 433 781 b 477 830 460 781 481 804 b 378 1033 467 907 433 977 b 306 1178 339 1073 314 1123 b 263 1215 303 1199 285 1215 l 218 1215 b 174 1166 192 1215 171 1192 b 284 937 184 1080 223 1000 b 345 818 317 904 337 863 z m 649 818 b 692 781 652 797 670 781 l 737 781 b 781 830 763 781 785 804 b 682 1033 771 907 736 977 b 610 1178 643 1073 618 1123 b 566 1215 607 1199 589 1215 l 522 1215 b 478 1166 495 1215 474 1192 b 588 937 488 1080 527 1000 b 649 818 620 904 641 863 z m 1085 694 l 87 694 b 0 608 39 694 0 656 l 0 87 b 260 -174 0 -57 117 -174 l 781 -174 b 1042 87 925 -174 1042 -57 l 1085 87 b 1389 391 1253 87 1389 223 b 1085 694 1389 558 1253 694 z m 1085 260 l 1042 260 l 1042 521 l 1085 521 b 1215 391 1157 521 1215 463 b 1085 260 1215 319 1157 260 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 890 521 l 1302 521 b 1345 567 1326 521 1347 542 b 1067 1067 1332 773 1225 953 b 1006 1054 1047 1081 1019 1074 l 788 705 b 890 521 849 667 890 599 z m 788 338 b 673 304 754 317 715 304 b 558 338 630 304 591 317 l 340 -11 b 355 -72 327 -32 333 -60 b 673 -152 449 -123 558 -152 b 991 -72 788 -152 896 -123 b 1006 -11 1012 -60 1019 -31 z m 673 391 b 803 521 745 391 803 449 b 673 652 803 593 745 652 b 543 521 601 652 543 593 b 673 391 543 449 601 391 z m 44 521 l 455 521 b 558 705 455 599 496 667 l 340 1053 b 278 1066 327 1074 298 1081 b 0 567 120 952 14 772 b 44 521 -2 542 19 521 z "
	},
	"": {
		ha: 1345,
		x_min: 0,
		x_max: 0,
		o: "m 846 521 l 1061 521 b 1104 566 1086 521 1107 542 b 943 859 1092 685 1031 788 b 878 849 922 875 892 871 l 765 667 b 846 521 814 637 846 583 z m 581 374 l 468 193 b 483 132 454 171 461 143 b 673 87 541 103 605 87 b 862 132 741 87 805 103 b 878 193 885 143 891 171 l 765 374 b 673 347 738 357 707 347 b 581 374 639 347 608 357 z m 285 521 l 499 521 b 581 668 499 583 532 637 l 468 849 b 403 859 454 871 423 876 b 241 567 314 789 254 685 b 285 521 238 542 260 521 z m 673 -152 b 1345 521 1044 -152 1345 149 b 673 1194 1345 892 1044 1194 b 0 521 301 1194 0 892 b 673 -152 0 149 301 -152 z m 673 1020 b 1172 521 948 1020 1172 796 b 673 22 1172 245 948 22 b 174 521 397 22 174 245 b 673 1020 174 796 397 1020 z m 673 434 b 760 521 721 434 760 473 b 673 608 760 569 721 608 b 586 521 625 608 586 569 b 673 434 586 473 625 434 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 347 868 b 521 1042 443 868 521 946 b 347 1215 521 1137 443 1215 b 174 1042 251 1215 174 1137 b 347 868 174 946 251 868 z m 1389 868 b 1563 1042 1485 868 1563 946 b 1389 1215 1563 1137 1485 1215 b 1215 1042 1293 1215 1215 1137 b 1389 868 1215 946 1293 868 z m 1734 254 l 1611 758 b 1530 823 1602 794 1568 820 b 1389 781 1489 797 1441 781 b 1248 823 1337 781 1288 797 b 1167 758 1209 820 1176 794 l 1043 254 b 1110 174 1034 213 1066 174 l 1259 174 l 1259 -109 b 1324 -174 1259 -145 1288 -174 l 1454 -174 b 1519 -109 1490 -174 1519 -145 l 1519 174 l 1668 174 b 1734 254 1712 174 1744 213 z m 911 1215 l 825 1215 b 781 1172 801 1215 781 1196 l 781 -130 b 825 -174 781 -154 801 -174 l 911 -174 b 955 -130 935 -174 955 -154 l 955 1172 b 911 1215 955 1196 935 1215 z m 489 824 b 347 781 448 797 399 781 b 206 824 295 781 247 797 b 87 694 139 818 87 763 l 87 326 b 152 260 87 289 116 260 l 174 260 l 174 -109 b 239 -174 174 -145 203 -174 l 456 -174 b 521 -109 492 -174 521 -145 l 521 260 l 543 260 b 608 326 579 260 608 289 l 608 694 b 489 824 608 763 555 818 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1364 496 l 1146 715 l 1275 844 b 1275 973 1311 880 1311 938 l 1147 1102 b 1018 1102 1111 1138 1053 1138 l 888 972 l 670 1190 b 609 1215 653 1207 631 1215 b 548 1190 587 1215 565 1207 l 286 928 b 286 805 252 894 252 839 l 503 587 l 477 561 b 20 546 331 627 162 622 b 12 483 -3 534 -6 502 l 304 192 l 256 143 b 234 148 248 145 242 148 b 148 61 186 148 148 109 b 234 -26 148 13 186 -26 b 321 61 282 -26 321 13 b 317 82 321 68 319 75 l 365 130 l 657 -161 b 720 -154 675 -180 707 -177 b 734 303 796 -12 800 158 l 761 330 l 979 112 b 1040 87 996 95 1018 87 b 1101 112 1062 87 1084 95 l 1363 374 b 1364 496 1397 408 1397 463 z m 409 867 l 609 1067 l 796 880 l 596 680 z m 1040 235 l 853 422 l 1054 622 l 1241 435 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 512 277 l 586 351 b 608 347 594 349 600 347 b 694 434 656 347 694 386 b 608 521 694 482 656 521 b 521 434 560 521 521 482 b 525 413 521 426 523 420 l 451 338 l 134 655 b 66 647 114 675 79 672 b 163 -10 -47 436 -15 168 b 820 -107 341 -188 610 -221 b 828 -39 845 -94 849 -59 z m 567 1215 b 521 1172 542 1217 521 1197 l 521 1086 b 562 1043 521 1063 539 1044 b 1215 388 913 1024 1195 739 b 1258 347 1216 365 1235 347 l 1345 347 b 1389 393 1370 347 1390 368 b 567 1215 1366 836 1010 1192 z m 568 955 b 521 911 543 957 521 937 l 521 824 b 561 781 521 801 538 783 b 954 387 769 764 935 596 b 997 347 956 365 975 347 l 1085 347 b 1128 394 1110 347 1130 369 b 568 955 1105 693 866 932 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 868 1215 l 347 1215 l 0 868 l 0 0 b 174 -174 0 -96 78 -174 l 868 -174 b 1042 0 964 -174 1042 -96 l 1042 1042 b 868 1215 1042 1137 964 1215 z m 434 781 l 304 781 l 304 1042 l 434 1042 z m 651 781 l 521 781 l 521 1042 l 651 1042 z m 868 781 l 738 781 l 738 1042 l 868 1042 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 0 1042 l 0 0 b 174 -174 0 -96 78 -174 l 868 -174 b 1042 0 964 -174 1042 -96 l 1042 868 l 694 1215 l 174 1215 b 0 1042 78 1215 0 1137 z m 608 521 l 434 521 l 434 694 l 608 694 z m 868 521 l 694 521 l 694 694 l 781 694 b 868 608 829 694 868 656 z m 694 174 l 868 174 l 868 87 b 781 0 868 39 829 0 l 694 0 z m 434 174 l 608 174 l 608 0 l 434 0 z m 174 174 l 347 174 l 347 0 l 260 0 b 174 87 212 0 174 39 z m 174 434 l 868 434 l 868 260 l 174 260 z m 174 608 b 260 694 174 656 212 694 l 347 694 l 347 521 l 174 521 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1085 1215 b 955 1085 1013 1215 955 1157 b 1085 955 955 1013 1013 955 b 1215 1085 1157 955 1215 1013 b 1085 1215 1215 1157 1157 1215 z m 1085 0 b 1042 -43 1061 0 1042 -20 b 998 -87 1042 -67 1022 -87 l 738 -87 b 694 -130 714 -87 694 -106 b 738 -174 694 -154 714 -174 l 998 -174 b 1128 -43 1070 -174 1128 -115 b 1085 0 1128 -20 1109 0 z m 320 -23 b 258 -23 303 -40 275 -40 l 74 161 b 13 161 57 178 30 178 b 13 100 -4 144 -4 116 l 197 -85 b 289 -123 222 -110 256 -123 b 381 -85 323 -123 356 -110 b 381 -23 398 -68 398 -40 b 320 -23 364 -7 337 -6 z m 472 464 l 218 210 b 218 88 184 176 184 121 b 279 62 234 71 257 62 b 340 88 301 62 323 71 l 589 337 l 508 419 b 472 464 494 432 482 448 z m 347 781 l 633 781 l 579 735 b 518 610 542 703 520 658 b 569 480 516 561 535 514 l 781 268 l 781 43 b 868 -43 781 -5 820 -43 b 955 43 916 -43 955 -5 l 955 286 b 917 378 955 320 941 354 l 751 543 b 756 546 752 544 754 545 l 979 770 b 1003 888 1010 801 1019 847 b 902 955 986 929 946 955 l 347 955 b 260 868 299 955 260 916 b 347 781 260 820 299 781 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1172 955 b 1302 1085 1244 955 1302 1013 b 1172 1215 1302 1157 1244 1215 b 1042 1085 1100 1215 1042 1157 b 1172 955 1042 1013 1100 955 z m 1370 -11 b 1278 -11 1344 14 1303 14 b 1156 -35 1245 -44 1195 -53 l 787 156 l 921 356 b 904 520 955 407 948 477 l 795 630 l 505 774 b 553 627 497 720 514 666 l 756 423 l 632 236 l 95 514 b 7 486 63 530 24 518 b 35 398 -9 454 3 414 l 1098 -151 b 1199 -173 1131 -166 1165 -173 b 1370 -103 1262 -173 1324 -149 b 1370 -11 1395 -78 1395 -36 z m 326 967 l 294 906 b 410 919 333 886 379 892 l 538 855 b 538 854 538 855 538 855 l 849 700 l 936 736 l 954 684 b 1019 609 965 651 988 624 l 1176 530 b 1293 569 1220 509 1271 526 b 1254 685 1314 612 1297 664 l 1113 756 l 1066 895 b 837 1001 1044 961 955 1048 l 617 913 l 447 997 b 391 1094 448 1037 428 1075 l 361 1035 l 360 1035 l 267 1054 b 253 1049 262 1055 257 1053 b 254 1029 248 1043 248 1034 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 911 955 b 1042 1085 983 955 1042 1013 b 911 1215 1042 1157 983 1215 b 781 1085 840 1215 781 1157 b 911 955 781 1013 840 955 z m 1497 87 b 1432 22 1462 87 1432 58 b 1367 -43 1432 -14 1403 -43 l 1179 -43 l 1248 441 b 1302 521 1280 453 1302 484 b 1216 608 1302 569 1264 608 l 1096 608 l 1025 752 b 858 890 991 821 929 872 l 665 948 b 445 901 588 966 508 949 l 338 819 b 322 697 300 790 292 735 b 328 691 324 694 326 693 l 181 -43 l 65 -43 b 0 -109 29 -43 0 -72 b 65 -174 0 -145 29 -174 l 1367 -174 b 1563 22 1475 -174 1563 -86 b 1497 87 1563 58 1533 87 z m 791 -43 l 528 -43 l 645 203 l 585 239 b 505 314 552 258 526 285 l 336 -43 l 270 -43 l 412 666 b 444 681 423 668 434 674 l 551 763 b 620 780 572 779 598 785 l 660 768 l 558 531 b 629 313 524 451 554 357 l 860 177 z m 1091 -43 l 972 -43 l 1037 161 b 978 309 1053 218 1029 278 l 812 407 l 897 619 l 952 507 b 1069 434 974 463 1019 434 l 1159 434 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1662 264 l 1637 284 b 1576 277 1618 299 1591 296 l 1549 243 b 1555 182 1534 225 1537 197 l 1581 162 b 1606 110 1597 149 1606 130 b 1539 43 1606 73 1576 43 l 130 43 b 87 0 106 43 87 24 l 87 -43 b 130 -87 87 -67 106 -87 l 1530 -87 b 1736 98 1636 -87 1730 -7 b 1662 264 1740 163 1712 224 z m 87 608 b 347 273 87 446 198 311 l 347 130 l 521 130 l 521 260 l 1042 260 l 1042 130 l 1215 130 l 1215 260 b 1476 521 1359 260 1476 377 l 1476 781 b 1563 868 1524 781 1563 820 b 1476 955 1563 916 1524 955 l 1215 955 l 1215 781 b 1042 608 1215 685 1137 608 l 986 608 b 564 868 807 608 644 709 b 143 1128 484 1028 321 1128 l 87 1128 b 0 1042 39 1128 0 1090 b 87 955 0 994 39 955 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1128 b 0 564 311 1128 0 876 b 155 210 0 430 58 307 b 6 -50 121 73 7 -49 b 2 -74 0 -56 -2 -66 b 22 -87 5 -82 13 -87 b 403 53 202 -87 336 -1 b 694 0 492 19 590 0 b 1389 564 1078 0 1389 253 b 694 1128 1389 876 1078 1128 z m 348 391 l 315 391 b 293 412 303 391 293 400 l 293 456 b 315 477 293 468 303 477 l 348 477 b 376 495 364 477 376 487 b 371 506 376 499 374 503 l 311 557 b 275 633 288 576 275 604 b 390 738 275 691 327 738 l 423 738 b 445 716 435 738 445 728 l 445 672 b 423 651 445 661 435 651 l 390 651 b 362 633 374 651 362 641 b 367 623 362 629 364 626 l 427 572 b 463 495 450 552 463 524 b 348 391 463 438 411 391 z m 868 412 b 846 391 868 400 858 391 l 803 391 b 781 412 791 391 781 400 l 781 597 l 714 446 b 675 446 706 430 683 430 l 608 597 l 608 412 b 586 391 608 400 598 391 l 543 391 b 521 412 531 391 521 400 l 521 694 b 564 738 521 718 540 738 l 608 738 b 646 714 624 738 639 729 l 694 618 l 742 714 b 781 738 750 729 765 738 l 825 738 b 868 694 849 738 868 718 z m 999 391 l 966 391 b 944 412 954 391 944 400 l 944 456 b 966 477 944 468 954 477 l 999 477 b 1027 495 1015 477 1027 487 b 1022 506 1027 499 1025 503 l 962 557 b 926 633 939 576 926 604 b 1041 738 926 691 978 738 l 1074 738 b 1096 716 1086 738 1096 728 l 1096 672 b 1074 651 1096 661 1086 651 l 1041 651 b 1013 633 1025 651 1013 641 b 1018 623 1013 629 1015 626 l 1078 572 b 1114 495 1101 552 1114 524 b 999 391 1114 438 1063 391 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1172 955 b 1302 1085 1244 955 1302 1013 b 1172 1215 1302 1157 1244 1215 b 1042 1085 1100 1215 1042 1157 b 1172 955 1042 1013 1100 955 z m 1250 538 b 1302 521 1266 527 1284 521 b 1372 556 1328 521 1354 533 b 1354 677 1400 594 1393 648 l 1052 904 b 929 968 1015 932 973 954 l 756 1026 l 685 1167 b 569 1206 664 1210 612 1227 b 530 1089 526 1184 509 1132 l 609 932 b 684 867 624 901 651 878 l 729 852 l 617 796 b 521 641 558 766 521 707 l 521 497 l 320 430 b 265 320 274 414 250 365 b 289 285 270 306 278 294 l 184 323 b 125 377 157 333 136 352 b 38 409 109 410 71 424 b 7 322 5 394 -9 355 b 139 201 33 265 80 222 l 1129 -159 b 1209 -174 1155 -169 1182 -174 b 1308 -151 1243 -174 1277 -166 b 1340 -65 1341 -136 1355 -97 b 1253 -33 1325 -32 1286 -18 b 1173 -37 1228 -45 1200 -46 l 927 53 b 975 112 951 64 969 85 l 1035 389 b 986 520 1045 438 1026 490 l 844 626 l 1023 709 z m 858 398 l 805 148 b 812 94 801 129 805 111 l 353 261 b 375 265 360 262 368 262 l 605 342 b 694 465 658 359 694 409 l 694 521 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1386 802 l 1370 841 b 1313 865 1361 863 1336 874 l 1237 834 l 1237 913 b 1194 955 1237 936 1217 955 l 1150 955 b 1107 913 1126 955 1107 936 l 1107 785 b 1108 781 1107 784 1108 783 l 955 719 b 895 791 939 746 919 770 b 955 955 932 836 955 892 b 694 1216 955 1099 838 1216 b 434 955 550 1216 434 1099 b 493 791 434 892 457 836 b 434 719 470 770 450 746 l 281 781 b 282 785 281 783 282 784 l 282 913 b 239 955 282 936 263 955 l 195 955 b 152 913 171 955 152 936 l 152 834 l 76 865 b 19 841 53 874 28 863 l 3 802 b 27 747 -6 780 5 756 l 394 597 b 391 564 393 586 391 576 b 402 486 391 537 396 511 b 282 195 328 411 282 309 b 474 -151 282 49 359 -78 b 559 -174 499 -167 529 -174 l 828 -174 b 929 -142 864 -174 900 -162 b 1100 272 1055 -54 1131 101 b 987 486 1085 354 1044 428 b 998 564 994 511 998 537 b 995 597 998 576 996 586 l 1362 747 b 1386 802 1384 755 1395 780 z m 608 955 b 564 998 584 955 564 974 b 608 1042 564 1022 584 1042 b 651 998 632 1042 651 1022 b 608 955 651 974 632 955 z m 694 217 b 651 260 671 217 651 237 b 694 304 651 284 671 304 b 738 260 718 304 738 284 b 694 217 738 237 718 217 z m 694 391 b 651 434 671 391 651 410 b 694 477 651 458 671 477 b 738 434 718 477 738 458 b 694 391 738 410 718 391 z m 694 564 b 651 608 671 564 651 584 b 694 651 651 632 671 651 b 738 608 718 651 738 632 b 694 564 738 584 718 564 z m 694 803 b 651 890 694 803 651 866 b 694 933 651 914 671 933 b 738 890 718 933 738 914 b 694 803 738 866 694 803 z m 781 955 b 738 998 757 955 738 974 b 781 1042 738 1022 757 1042 b 825 998 805 1042 825 1022 b 781 955 825 974 805 955 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 326 195 b 260 130 289 195 260 166 b 326 65 260 94 289 65 b 391 130 362 65 391 94 b 326 195 391 166 362 195 z m 543 195 b 477 130 506 195 477 166 b 543 65 477 94 506 65 b 608 130 579 65 608 94 b 543 195 608 166 579 195 z m 760 195 b 694 130 723 195 694 166 b 760 65 694 94 723 65 b 825 130 796 65 825 94 b 760 195 825 166 796 195 z m 977 195 b 911 130 940 195 911 166 b 977 65 911 94 940 65 b 1042 130 1013 65 1042 94 b 977 195 1042 166 1013 195 z m 1624 61 b 1563 209 1584 101 1563 154 l 1563 485 b 1624 633 1563 541 1584 594 l 1723 733 b 1723 794 1740 750 1740 777 l 1662 855 b 1601 855 1645 872 1618 872 l 1501 756 b 1389 485 1429 684 1389 586 l 1389 434 l 1215 434 l 1215 572 b 1201 640 1215 596 1210 619 l 989 1136 b 869 1215 968 1184 921 1215 l 477 1215 b 347 1085 406 1215 347 1157 l 347 781 l 304 781 b 174 651 232 781 174 723 l 174 404 b 0 130 71 355 0 251 b 304 -174 0 -38 136 -174 l 998 -174 b 1302 130 1166 -174 1302 -38 b 1272 260 1302 177 1291 221 l 1389 260 l 1389 210 b 1501 -61 1389 108 1429 11 l 1601 -161 b 1662 -161 1618 -178 1645 -178 l 1723 -100 b 1723 -38 1740 -83 1740 -55 z m 521 1042 l 840 1042 l 1026 608 l 694 608 l 521 781 z m 998 0 l 304 0 b 174 130 232 0 174 58 b 304 260 174 202 232 260 l 998 260 b 1128 130 1070 260 1128 202 b 998 0 1128 58 1070 0 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 1009 781 l 33 781 b 0 749 15 781 0 767 l 0 597 b 33 564 0 579 15 564 l 412 564 l 412 -54 b 445 -87 412 -72 427 -87 l 597 -87 b 629 -54 615 -87 629 -72 l 629 564 l 1009 564 b 1042 597 1027 564 1042 579 l 1042 749 b 1009 781 1042 767 1027 781 z m 1009 1128 l 33 1128 b 0 1096 15 1128 0 1114 l 0 944 b 33 911 0 926 15 911 l 1009 911 b 1042 944 1027 911 1042 926 l 1042 1096 b 1009 1128 1042 1114 1027 1128 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 998 1085 b 1042 1128 1022 1085 1042 1105 l 1042 1172 b 998 1215 1042 1196 1022 1215 l 43 1215 b 0 1172 20 1215 0 1196 l 0 1128 b 43 1085 0 1105 20 1085 l 87 1085 l 87 660 b 0 564 32 633 0 600 b 235 129 0 382 94 222 l 177 -62 b 260 -174 160 -117 202 -174 l 781 -174 b 864 -62 840 -174 881 -117 l 806 129 b 1042 564 948 222 1042 382 b 955 660 1042 600 1010 633 l 955 1085 z m 217 1020 b 239 1042 217 1032 227 1042 l 369 1042 b 391 1020 381 1042 391 1032 l 391 977 b 369 955 391 965 381 955 l 239 955 b 217 977 227 955 217 965 z m 521 477 b 142 564 312 477 142 516 b 521 651 142 612 312 651 b 900 564 730 651 900 612 b 521 477 900 516 730 477 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1359 142 l 1042 460 b 810 497 979 522 885 534 l 521 786 l 521 955 l 174 1215 l 0 1042 l 260 694 l 429 694 l 718 405 b 756 174 681 330 693 236 l 1073 -144 b 1216 -144 1113 -184 1177 -184 l 1359 -1 b 1359 142 1399 39 1399 103 z m 900 605 b 1103 521 977 605 1049 575 l 1156 468 b 1274 548 1198 487 1239 513 b 1377 919 1375 649 1409 790 b 1323 934 1371 943 1341 952 l 1121 732 l 937 763 l 906 947 l 1108 1149 b 1092 1204 1126 1167 1117 1197 b 722 1101 964 1235 822 1201 b 610 820 645 1023 608 921 l 833 597 b 900 605 855 602 878 605 z m 618 382 l 464 536 l 51 123 b 51 -123 -17 55 -17 -55 b 296 -123 119 -191 228 -191 l 632 212 b 618 382 611 266 605 325 z m 174 -65 b 109 0 138 -65 109 -36 b 174 65 109 36 138 65 b 239 0 210 65 239 36 b 174 -65 239 -36 210 -65 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 781 1042 b 868 1128 829 1042 868 1080 b 781 1215 868 1176 829 1215 b 694 1128 733 1215 694 1176 b 781 1042 694 1080 733 1042 z m 1388 1074 b 1334 1105 1381 1098 1358 1111 l 32 758 b 2 705 9 752 -5 728 b 43 673 7 685 24 673 b 55 674 47 673 51 673 l 651 833 l 651 608 l 174 608 b 87 521 126 608 87 569 l 87 -87 b 174 -174 87 -135 126 -174 l 1215 -174 b 1302 -87 1263 -174 1302 -135 l 1302 521 b 1215 608 1302 569 1263 608 l 738 608 l 738 856 l 1357 1021 b 1388 1074 1380 1028 1394 1051 z m 477 174 l 217 174 l 217 434 l 477 434 z m 911 434 l 1172 434 l 1172 174 l 911 174 z m 825 434 l 825 174 l 564 174 l 564 434 z m 521 955 b 608 1042 569 955 608 994 b 521 1128 608 1090 569 1128 b 434 1042 473 1128 434 1090 b 521 955 434 994 473 955 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 878 1076 b 725 914 821 1024 770 969 b 456 1215 651 1016 560 1119 b 0 451 189 968 0 646 b 608 -174 0 106 272 -174 b 1215 451 943 -174 1215 106 b 878 1076 1215 596 1074 894 z m 825 152 b 615 87 766 111 694 87 b 260 427 420 87 260 216 b 458 781 260 531 326 624 b 726 441 477 760 726 441 l 885 623 b 916 568 896 604 906 586 b 825 152 990 427 958 246 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 594 303 b 740 536 689 397 714 465 b 907 800 769 613 799 692 b 1171 967 1015 908 1094 938 b 1379 1090 1236 991 1298 1015 l 1277 1191 b 1218 1215 1264 1204 1237 1215 b 1168 1199 1203 1215 1180 1208 b 1057 1144 1131 1171 1100 1160 b 544 634 670 1000 568 696 b 277 368 464 421 301 378 b 29 232 209 343 128 313 b 25 110 -8 201 -10 144 l 122 13 b 361 156 217 102 290 130 b 594 303 432 183 499 208 z m 1538 932 l 1441 1028 b 1201 885 1345 940 1272 912 b 968 739 1130 859 1063 833 b 822 505 874 644 848 577 b 655 241 793 429 763 349 b 391 75 547 134 468 104 b 183 -48 327 51 265 27 l 285 -149 b 344 -174 299 -163 325 -174 b 394 -157 360 -174 382 -166 b 505 -102 431 -130 462 -118 b 1018 407 891 42 994 343 b 1286 673 1099 621 1261 664 b 1533 810 1353 698 1434 728 b 1538 932 1571 841 1572 898 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1215 243 l 1215 1146 b 1146 1215 1215 1189 1189 1215 l 260 1215 b 0 955 113 1215 0 1102 l 0 87 b 260 -174 0 -61 113 -174 l 1146 -174 b 1215 -104 1181 -174 1215 -148 l 1215 -61 b 1189 -9 1215 -43 1207 -26 b 1189 191 1181 35 1181 156 b 1215 243 1207 200 1215 217 z m 391 760 b 412 781 391 772 400 781 l 564 781 l 564 933 b 586 955 564 945 574 955 l 716 955 b 738 933 728 955 738 945 l 738 781 l 890 781 b 911 760 902 781 911 772 l 911 629 b 890 608 911 617 902 608 l 738 608 l 738 456 b 716 434 738 444 728 434 l 586 434 b 564 456 574 434 564 444 l 564 608 l 412 608 b 391 629 400 608 391 617 z m 1033 0 l 260 0 b 174 87 208 0 174 35 b 260 174 174 139 217 174 l 1033 174 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 781 1215 b 0 756 293 1215 0 962 b 174 608 0 674 66 608 l 174 -87 b 271 -174 174 -135 217 -174 l 1291 -174 b 1389 -87 1345 -174 1389 -135 l 1389 608 b 1563 756 1497 608 1563 674 b 781 1215 1563 962 1270 1215 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 0 434 l 0 0 b 87 -87 0 -48 39 -87 l 1302 -87 b 1389 0 1350 -87 1389 -48 l 1389 434 z m 813 1128 b 810 1129 812 1129 811 1129 b 756 1109 793 1129 769 1120 l 0 521 l 1389 521 b 813 1128 1389 846 1134 1112 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 781 903 l 188 381 b 174 371 184 377 178 375 l 174 -130 b 217 -174 174 -154 193 -174 l 1345 -174 b 1389 -130 1369 -174 1389 -154 l 1389 371 b 1375 380 1384 375 1379 376 z m 1042 195 b 1020 174 1042 183 1032 174 l 868 174 l 868 22 b 846 0 868 10 858 0 l 716 0 b 694 22 704 0 694 10 l 694 174 l 543 174 b 521 195 531 174 521 183 l 521 326 b 543 347 521 338 531 347 l 694 347 l 694 499 b 716 521 694 511 704 521 l 846 521 b 868 499 858 521 868 511 l 868 347 l 1020 347 b 1042 326 1032 347 1042 338 z m 1548 574 l 854 1187 b 781 1215 837 1203 804 1215 b 709 1187 758 1215 726 1203 l 14 574 b 0 542 6 567 0 553 b 11 513 0 533 5 520 l 69 448 b 101 434 76 440 91 434 b 130 445 111 434 124 439 l 753 994 b 781 1005 759 1000 772 1005 b 810 994 790 1005 803 1000 l 1432 445 b 1461 434 1439 439 1452 434 b 1493 448 1472 434 1486 440 l 1551 513 b 1563 542 1558 520 1563 533 b 1548 574 1563 553 1556 567 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 694 1128 b 0 564 311 1128 0 876 b 155 210 0 430 58 307 b 6 -50 121 73 7 -49 b 0 -65 3 -54 0 -60 b 22 -87 0 -77 10 -87 b 403 53 201 -87 336 -1 b 693 0 481 24 610 0 l 694 0 b 1389 564 1078 0 1389 253 b 694 1128 1389 876 1078 1128 z m 955 499 b 933 477 955 487 945 477 l 781 477 l 781 326 b 760 304 781 314 772 304 l 629 304 b 608 326 617 304 608 314 l 608 477 l 456 477 b 434 499 444 477 434 487 l 434 629 b 456 651 434 641 444 651 l 608 651 l 608 803 b 629 825 608 815 617 825 l 760 825 b 781 803 772 825 781 815 l 781 651 l 933 651 b 955 629 945 651 955 641 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1376 712 l 885 1203 b 854 1215 878 1210 864 1215 b 824 1203 845 1215 831 1210 l 762 1141 b 750 1111 755 1134 750 1120 b 762 1080 750 1101 755 1087 l 1253 589 b 1284 576 1260 582 1274 576 b 1315 589 1294 576 1308 582 l 1376 650 b 1389 681 1383 657 1389 671 b 1376 712 1389 691 1383 705 z m 889 531 l 705 716 l 854 865 l 732 988 l 434 690 b 365 565 405 661 374 605 l 289 238 l 13 -38 b 0 -69 6 -45 0 -59 b 13 -100 0 -79 6 -93 l 74 -161 b 105 -174 81 -168 95 -174 b 135 -161 115 -174 128 -168 l 412 116 l 738 191 b 864 261 778 200 835 231 l 1161 558 l 1039 681 z m 741 383 b 699 360 731 374 713 363 l 484 310 l 534 526 b 557 567 537 539 547 558 l 582 593 l 766 409 z "
	},
	"": {
		ha: 1042,
		x_min: 0,
		x_max: 0,
		o: "m 521 1215 b 0 347 233 1215 0 635 b 521 -174 0 60 233 -174 b 1042 347 808 -174 1042 60 b 521 1215 1042 635 808 1215 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1259 521 l 130 521 b 0 391 58 521 0 463 b 130 260 0 319 58 260 l 1259 260 b 1389 391 1331 260 1389 319 b 1259 521 1389 463 1331 521 z m 1302 174 l 87 174 b 43 130 63 174 43 154 l 43 87 b 217 -87 43 -9 121 -87 l 1172 -87 b 1345 87 1268 -87 1345 -9 l 1345 130 b 1302 174 1345 154 1326 174 z m 159 608 l 1230 608 b 1324 813 1324 608 1378 727 b 694 1128 1215 990 975 1128 b 65 814 414 1128 174 990 b 159 608 11 727 65 608 z m 1042 911 b 1085 868 1066 911 1085 892 b 1042 825 1085 844 1066 825 b 998 868 1018 825 998 844 b 1042 911 998 892 1018 911 z m 694 998 b 738 955 718 998 738 979 b 694 911 738 931 718 911 b 651 955 670 911 651 931 b 694 998 651 979 670 998 z m 347 911 b 391 868 371 911 391 892 b 347 825 391 844 371 825 b 304 868 323 825 304 844 b 347 911 304 892 323 911 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1302 355 b 1225 453 1302 399 1267 443 l 1128 477 l 1128 612 b 1066 676 1128 643 1102 670 l 962 693 b 868 629 913 701 868 670 l 868 543 b 846 521 868 531 858 521 b 825 543 834 521 825 531 l 825 1080 b 698 1215 825 1151 770 1213 b 694 1215 697 1215 696 1215 b 564 1085 623 1215 564 1157 l 564 1085 l 564 543 b 543 521 564 531 555 521 b 521 543 531 521 521 531 l 521 629 b 427 693 521 670 476 701 l 344 679 b 260 594 295 671 260 636 l 260 195 b 239 174 260 183 251 174 b 217 195 227 174 217 183 l 217 401 l 143 361 b 87 270 112 345 87 305 l 87 270 l 87 71 l 87 71 b 116 -1 87 48 100 16 l 200 -85 b 415 -174 249 -134 346 -174 l 998 -174 b 1302 130 1166 -174 1302 -37 l 1302 130 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1302 434 b 979 916 1302 652 1168 838 l 868 694 l 868 998 b 825 1042 868 1022 849 1042 l 564 1042 b 521 998 540 1042 521 1022 l 521 694 l 410 916 b 87 434 220 838 87 652 l 87 260 l 1302 260 z m 1345 174 l 43 174 b 0 130 19 174 0 154 l 0 43 b 43 0 0 19 19 0 l 1345 0 b 1389 43 1369 0 1389 19 l 1389 130 b 1345 174 1389 154 1369 174 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1325 1152 b 1172 1215 1290 1187 1221 1215 b 1018 1152 1122 1215 1054 1187 l 64 197 b -3 41 27 162 -3 92 b 214 -176 -3 -79 95 -176 b 370 -110 265 -176 335 -147 l 1325 845 b 1389 998 1360 880 1389 949 b 1325 1152 1389 1048 1360 1117 z m 1190 893 b 1049 825 1137 840 1088 832 b 955 781 1014 819 988 815 b 911 687 921 748 917 722 b 843 546 904 648 896 599 b 702 478 789 493 741 485 b 608 434 667 472 641 468 b 564 340 574 400 570 375 b 495 199 557 301 549 252 b 355 131 442 146 394 138 b 260 87 319 125 294 120 b 230 74 253 80 240 74 b 186 118 206 74 186 94 b 199 148 186 127 192 141 b 340 216 252 201 301 210 b 434 260 375 222 400 227 b 478 355 468 294 472 319 b 546 495 485 394 493 442 b 687 564 599 549 648 557 b 781 608 722 570 748 574 b 825 702 815 641 819 667 b 893 843 832 741 840 789 b 1034 911 947 896 995 904 b 1128 955 1069 917 1095 921 b 1159 968 1135 962 1149 968 b 1203 924 1183 968 1203 948 b 1190 893 1203 914 1197 900 z m 85 341 l 874 1130 l 843 1161 b 607 1171 775 1229 669 1234 l 44 608 b 54 373 -18 546 -14 440 z m 1304 700 l 515 -88 l 546 -120 b 782 -130 614 -187 719 -192 l 1345 434 b 1335 669 1407 496 1403 601 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 998 781 l 996 781 b 998 826 997 794 998 813 b 608 1216 998 1041 823 1216 b 217 826 392 1216 217 1041 b 220 781 217 813 218 794 l 217 781 b 87 651 145 781 87 723 b 217 521 87 579 145 521 l 998 521 b 1128 651 1070 521 1128 579 b 998 781 1128 723 1070 781 z m 530 -124 b 608 -174 543 -151 577 -174 b 685 -124 638 -174 673 -151 l 955 434 l 260 434 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 629 608 l 781 608 l 781 456 b 803 434 781 444 791 434 l 933 434 b 955 456 945 434 955 444 l 955 608 l 1107 608 b 1128 629 1119 608 1128 617 l 1128 760 b 1107 781 1128 772 1119 781 l 955 781 l 955 933 b 933 955 955 945 945 955 l 803 955 b 781 933 791 955 781 945 l 781 781 l 629 781 b 608 760 617 781 608 772 l 608 629 b 629 608 608 617 617 608 z m 1563 1085 b 1432 1215 1562 1157 1504 1215 l 304 1215 b 174 1085 232 1215 174 1157 l 174 174 l 1563 174 z m 1389 347 l 347 347 l 347 1042 l 1389 1042 z m 1693 87 l 1035 87 b 946 0 1033 33 995 0 l 781 0 b 692 87 731 0 692 47 l 43 87 b 0 43 19 87 0 67 l 0 0 b 174 -174 0 -96 78 -173 l 1563 -174 b 1736 0 1658 -173 1736 -96 l 1736 43 b 1693 87 1736 67 1717 87 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1215 1042 l 174 1042 b 0 868 78 1042 0 964 l 0 174 b 174 0 0 78 78 0 l 1215 0 b 1389 174 1311 0 1389 78 l 1389 868 b 1215 1042 1389 964 1311 1042 z m 434 217 l 217 217 b 174 260 193 217 174 236 l 174 304 b 217 347 174 328 193 347 l 434 347 z m 781 260 b 738 217 781 236 762 217 l 521 217 l 521 347 l 738 347 b 781 304 762 347 781 328 z m 1215 608 b 1128 521 1215 560 1176 521 l 260 521 b 174 608 212 521 174 560 l 174 781 b 260 868 174 829 212 868 l 1128 868 b 1215 781 1176 868 1215 829 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 897 502 l 897 745 l 754 811 b 152 130 595 622 536 130 b 0 -22 68 130 0 62 b 152 -174 0 -106 68 -174 b 1289 397 729 -174 1157 160 l 1189 502 z m 1253 850 b 1244 1207 1340 1012 1294 1143 b 1227 1215 1240 1212 1233 1215 b 1212 1209 1222 1215 1215 1212 l 1150 1146 b 1143 1131 1146 1143 1143 1136 b 1147 1119 1143 1127 1145 1122 b 1143 922 1166 1090 1193 1023 b 988 955 1102 940 1033 955 b 781 894 912 955 842 932 l 984 801 l 984 588 l 1225 588 l 1374 434 b 1389 541 1383 468 1389 504 l 1389 541 b 1253 850 1389 643 1328 781 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 431 1215 b 335 1149 387 1219 346 1192 l 295 997 b 1170 137 804 982 1148 642 l 1324 180 b 1388 277 1367 192 1393 233 b 431 1215 1332 767 924 1168 z m 272 911 l 1 -118 b 0 -130 1 -121 0 -126 b 44 -174 0 -154 20 -174 b 56 -172 47 -174 53 -173 l 1084 113 b 272 911 1072 588 750 906 z m 347 87 b 260 174 299 87 260 126 b 347 260 260 222 299 260 b 434 174 395 260 434 222 b 347 87 434 126 395 87 z m 477 499 b 391 586 430 499 391 538 b 477 673 391 634 430 673 b 564 586 525 673 564 634 b 477 499 564 538 525 499 z m 760 217 b 673 304 712 217 673 256 b 760 391 673 352 712 391 b 846 304 807 391 846 352 b 760 217 846 256 807 217 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 144 -52 b 274 -174 149 -119 207 -174 l 941 -174 b 1071 -52 1009 -174 1067 -119 l 1128 868 l 87 868 z m 334 425 l 577 681 b 608 694 584 688 598 694 b 638 681 618 694 631 688 l 881 425 b 850 347 908 397 889 347 l 694 347 l 694 43 b 651 0 694 19 675 0 l 564 0 b 521 43 540 0 521 19 l 521 347 l 365 347 b 334 425 327 347 307 397 z m 1172 1128 l 846 1128 l 821 1179 b 763 1215 811 1199 785 1215 l 763 1215 l 452 1215 l 452 1215 b 394 1179 430 1215 404 1199 l 369 1128 l 43 1128 b 0 1085 19 1128 0 1109 l 0 998 b 43 955 0 974 19 955 l 1172 955 b 1215 998 1196 955 1215 974 l 1215 1085 b 1172 1128 1215 1109 1196 1128 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 87 -43 b 217 -174 87 -115 145 -174 l 998 -174 b 1128 -43 1070 -174 1128 -115 l 1128 868 l 87 868 z m 334 425 l 577 681 b 608 694 584 688 598 694 b 638 681 618 694 631 688 l 881 425 b 850 347 908 397 889 347 l 694 347 l 694 43 b 651 0 694 19 675 0 l 564 0 b 521 43 540 0 521 19 l 521 347 l 365 347 b 334 425 327 347 307 397 z m 1172 1128 l 846 1128 l 821 1179 b 763 1215 811 1199 785 1215 l 763 1215 l 452 1215 l 452 1215 b 394 1179 430 1215 404 1199 l 369 1128 l 43 1128 b 0 1085 19 1128 0 1109 l 0 998 b 43 955 0 974 19 955 l 1172 955 b 1215 998 1196 955 1215 974 l 1215 1085 b 1172 1128 1215 1109 1196 1128 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 157 434 l 380 434 b 608 347 441 381 520 347 b 835 434 695 347 774 381 l 1059 434 b 1102 477 1082 434 1102 453 b 1097 497 1102 483 1100 492 b 983 756 1056 578 1012 666 b 955 952 962 819 955 885 l 955 1085 l 608 1215 l 260 1085 l 260 952 b 232 756 260 885 253 819 b 118 497 203 666 159 578 b 113 477 115 492 113 483 b 157 434 113 453 133 434 z m 499 1021 b 513 1034 499 1028 505 1034 l 571 1034 l 571 1093 b 585 1107 571 1101 578 1107 l 630 1107 b 644 1093 638 1107 644 1101 l 644 1034 l 703 1034 b 716 1021 710 1034 716 1028 l 716 976 b 703 962 716 968 710 962 l 644 962 l 644 903 b 630 890 644 896 638 890 l 585 890 b 571 903 578 890 571 896 l 571 962 l 513 962 b 499 976 505 962 499 968 z m 391 781 l 825 781 l 825 694 b 608 477 825 575 727 477 b 391 694 488 477 391 575 z m 866 347 l 608 88 l 349 347 b 0 -16 155 339 0 180 b 158 -174 0 -103 71 -174 l 1057 -174 l 1057 -174 b 1215 -16 1145 -174 1215 -103 b 866 347 1215 180 1060 339 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1291 -87 b 1389 11 1345 -87 1389 -43 l 1389 434 l 1693 434 b 1736 477 1717 434 1736 453 l 1736 564 b 1693 608 1736 588 1717 608 l 1313 608 b 1215 510 1259 608 1215 564 l 1215 87 l 955 87 l 955 1031 b 857 1128 955 1085 911 1128 l 445 1128 b 347 1031 391 1128 347 1085 l 347 608 l 43 608 b 0 564 19 608 0 588 l 0 477 b 43 434 0 453 19 434 l 423 434 b 521 532 477 434 521 478 l 521 955 l 781 955 l 781 11 b 879 -87 781 -43 825 -87 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1085 955 b 1215 1085 1157 955 1215 1013 b 1085 1215 1215 1157 1157 1215 b 955 1085 1013 1215 955 1157 b 1085 955 955 1013 1013 955 z m 1074 627 b 1128 608 1087 616 1112 608 l 1128 608 l 1302 608 b 1389 694 1350 608 1389 647 b 1302 781 1389 742 1350 781 l 1159 781 l 966 936 b 911 955 953 946 928 955 b 855 934 894 955 868 946 l 551 674 b 521 608 534 659 521 630 b 560 535 521 582 538 550 l 781 388 l 781 87 b 868 0 781 39 820 0 b 955 87 916 0 955 39 l 955 434 b 916 506 955 460 938 492 l 804 581 l 962 716 z m 1389 521 b 1042 174 1197 521 1042 365 b 1389 -174 1042 -18 1197 -174 b 1736 174 1581 -174 1736 -18 b 1389 521 1736 365 1581 521 z m 1389 0 b 1215 174 1293 0 1215 78 b 1389 347 1215 269 1293 347 b 1563 174 1485 347 1563 269 b 1389 0 1563 78 1485 0 z m 347 521 b 0 174 156 521 0 365 b 347 -174 0 -18 156 -174 b 694 174 539 -174 694 -18 b 347 521 694 365 539 521 z m 347 0 b 174 174 251 0 174 78 b 347 347 174 269 251 347 b 521 174 443 347 521 269 b 347 0 521 78 443 0 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1128 1128 l 87 1128 b 0 1042 39 1128 0 1090 l 0 0 b 87 -87 0 -48 39 -87 l 1128 -87 b 1215 0 1176 -87 1215 -48 l 1215 1042 b 1128 1128 1215 1090 1176 1128 z m 1042 955 l 1042 608 l 694 608 l 694 955 z m 521 955 l 521 608 l 174 608 l 174 955 z m 174 87 l 174 434 l 521 434 l 521 87 z m 694 87 l 694 434 l 1042 434 l 1042 87 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 651 608 l 564 608 b 521 564 540 608 521 588 l 521 477 b 564 434 521 453 540 434 l 651 434 b 694 477 675 434 694 453 l 694 564 b 651 608 694 588 675 608 z m 911 608 l 825 608 b 781 564 801 608 781 588 l 781 477 b 825 434 781 453 801 434 l 911 434 b 955 477 935 434 955 453 l 955 564 b 911 608 955 588 935 608 z m 1172 608 l 1085 608 b 1042 564 1061 608 1042 588 l 1042 477 b 1085 434 1042 453 1061 434 l 1172 434 b 1215 477 1196 434 1215 453 l 1215 564 b 1172 608 1215 588 1196 608 z m 391 608 l 304 608 b 260 564 280 608 260 588 l 260 477 b 304 434 260 453 280 434 l 391 434 b 434 477 415 434 434 453 l 434 564 b 391 608 434 588 415 608 z m 651 87 l 564 87 b 521 43 540 87 521 67 l 521 -43 b 564 -87 521 -67 540 -87 l 651 -87 b 694 -43 675 -87 694 -67 l 694 43 b 651 87 694 67 675 87 z m 911 87 l 825 87 b 781 43 801 87 781 67 l 781 -43 b 825 -87 781 -67 801 -87 l 911 -87 b 955 -43 935 -87 955 -67 l 955 43 b 911 87 955 67 935 87 z m 1172 87 l 1085 87 b 1042 43 1061 87 1042 67 l 1042 -43 b 1085 -87 1042 -67 1061 -87 l 1172 -87 b 1215 -43 1196 -87 1215 -67 l 1215 43 b 1172 87 1215 67 1196 87 z m 1172 347 l 1085 347 b 1042 304 1061 347 1042 328 l 1042 217 b 1085 174 1042 193 1061 174 l 1172 174 b 1215 217 1196 174 1215 193 l 1215 304 b 1172 347 1215 328 1196 347 z m 1172 868 l 1085 868 b 1042 825 1061 868 1042 849 l 1042 738 b 1085 694 1042 714 1061 694 l 1172 694 b 1215 738 1196 694 1215 714 l 1215 825 b 1172 868 1215 849 1196 868 z m 651 347 l 564 347 b 521 304 540 347 521 328 l 521 217 b 564 174 521 193 540 174 l 651 174 b 694 217 675 174 694 193 l 694 304 b 651 347 694 328 675 347 z m 651 868 l 564 868 b 521 825 540 868 521 849 l 521 738 b 564 694 521 714 540 694 l 651 694 b 694 738 675 694 694 714 l 694 825 b 651 868 694 849 675 868 z m 391 87 l 304 87 b 260 43 280 87 260 67 l 260 -43 b 304 -87 260 -67 280 -87 l 391 -87 b 434 -43 415 -87 434 -67 l 434 43 b 391 87 434 67 415 87 z m 651 1128 l 564 1128 b 521 1085 540 1128 521 1109 l 521 998 b 564 955 521 974 540 955 l 651 955 b 694 998 675 955 694 974 l 694 1085 b 651 1128 694 1109 675 1128 z m 911 1128 l 825 1128 b 781 1085 801 1128 781 1109 l 781 998 b 825 955 781 974 801 955 l 911 955 b 955 998 935 955 955 974 l 955 1085 b 911 1128 955 1109 935 1128 z m 1172 1128 l 1085 1128 b 1042 1085 1061 1128 1042 1109 l 1042 998 b 1085 955 1042 974 1061 955 l 1172 955 b 1215 998 1196 955 1215 974 l 1215 1085 b 1172 1128 1215 1109 1196 1128 z m 130 608 l 43 608 b 0 564 19 608 0 588 l 0 477 b 43 434 0 453 19 434 l 130 434 b 174 477 154 434 174 453 l 174 564 b 130 608 174 588 154 608 z m 130 87 l 43 87 b 0 43 19 87 0 67 l 0 -43 b 43 -87 0 -67 19 -87 l 130 -87 b 174 -43 154 -87 174 -67 l 174 43 b 130 87 174 67 154 87 z m 130 347 l 43 347 b 0 304 19 347 0 328 l 0 217 b 43 174 0 193 19 174 l 130 174 b 174 217 154 174 174 193 l 174 304 b 130 347 174 328 154 347 z m 130 868 l 43 868 b 0 825 19 868 0 849 l 0 738 b 43 694 0 714 19 694 l 130 694 b 174 738 154 694 174 714 l 174 825 b 130 868 174 849 154 868 z m 130 1128 l 43 1128 b 0 1085 19 1128 0 1109 l 0 998 b 43 955 0 974 19 955 l 130 955 b 174 998 154 955 174 974 l 174 1085 b 130 1128 174 1109 154 1128 z m 391 1128 l 304 1128 b 260 1085 280 1128 260 1109 l 260 998 b 304 955 260 974 280 955 l 391 955 b 434 998 415 955 434 974 l 434 1085 b 391 1128 434 1109 415 1128 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 651 87 l 564 87 b 521 43 540 87 521 67 l 521 -43 b 564 -87 521 -67 540 -87 l 651 -87 b 694 -43 675 -87 694 -67 l 694 43 b 651 87 694 67 675 87 z m 391 87 l 304 87 b 260 43 280 87 260 67 l 260 -43 b 304 -87 260 -67 280 -87 l 391 -87 b 434 -43 415 -87 434 -67 l 434 43 b 391 87 434 67 415 87 z m 911 87 l 825 87 b 781 43 801 87 781 67 l 781 -43 b 825 -87 781 -67 801 -87 l 911 -87 b 955 -43 935 -87 955 -67 l 955 43 b 911 87 955 67 935 87 z m 1172 608 l 1085 608 b 1042 564 1061 608 1042 588 l 1042 477 b 1085 434 1042 453 1061 434 l 1172 434 b 1215 477 1196 434 1215 453 l 1215 564 b 1172 608 1215 588 1196 608 z m 1172 347 l 1085 347 b 1042 304 1061 347 1042 328 l 1042 217 b 1085 174 1042 193 1061 174 l 1172 174 b 1215 217 1196 174 1215 193 l 1215 304 b 1172 347 1215 328 1196 347 z m 1172 87 l 1085 87 b 1042 43 1061 87 1042 67 l 1042 -43 b 1085 -87 1042 -67 1061 -87 l 1172 -87 b 1215 -43 1196 -87 1215 -67 l 1215 43 b 1172 87 1215 67 1196 87 z m 1172 868 l 1085 868 b 1042 825 1061 868 1042 849 l 1042 738 b 1085 694 1042 714 1061 694 l 1172 694 b 1215 738 1196 694 1215 714 l 1215 825 b 1172 868 1215 849 1196 868 z m 1172 1128 l 87 1128 b 0 1042 39 1128 0 1090 l 0 -43 b 43 -87 0 -67 19 -87 l 130 -87 b 174 -43 154 -87 174 -67 l 174 955 l 1172 955 b 1215 998 1196 955 1215 974 l 1215 1085 b 1172 1128 1215 1109 1196 1128 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 956 868 b 747 833 880 868 810 856 l 781 1167 b 731 1215 784 1195 759 1218 b 347 783 514 1188 347 1005 b 382 574 347 707 359 636 l 48 607 b 0 557 20 610 -3 585 b 432 174 27 341 211 174 b 642 208 509 174 579 186 l 608 -125 b 658 -173 605 -154 630 -177 b 1042 259 874 -146 1042 37 b 1007 468 1042 335 1029 406 l 1341 434 b 1389 485 1369 431 1392 456 b 956 868 1361 701 1178 868 z m 694 434 b 608 521 647 434 608 473 b 694 608 608 569 647 608 b 781 521 742 608 781 569 b 694 434 781 473 742 434 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 316 620 b 347 607 323 613 337 607 b 378 620 357 607 371 613 l 640 891 b 627 1173 717 970 712 1100 b 374 1154 553 1236 442 1225 l 347 1127 l 321 1154 b 67 1173 252 1225 142 1236 b 54 891 -18 1100 -22 970 z m 707 348 l 577 348 l 557 386 b 488 435 547 413 516 435 l 488 435 l 293 435 l 293 435 b 224 386 265 435 234 413 l 205 348 l 74 348 l 74 348 b 0 273 33 348 0 314 l 0 273 l 0 -99 l 0 -99 b 74 -174 0 -140 33 -174 l 707 -174 b 781 -99 748 -174 781 -140 l 781 -99 l 781 273 l 781 273 b 707 348 781 314 748 348 z m 391 -54 b 250 87 313 -54 250 9 b 391 228 250 165 313 228 b 532 87 468 228 532 165 b 391 -54 532 9 468 -54 z m 1355 260 l 1190 260 l 1251 398 b 1218 434 1257 416 1240 434 l 1013 434 b 979 409 996 434 981 423 l 933 119 b 967 87 931 102 947 87 l 1136 87 l 1071 -139 b 1104 -174 1066 -157 1082 -174 b 1105 -174 1104 -174 1105 -174 b 1134 -159 1115 -174 1128 -167 l 1384 218 b 1355 260 1397 236 1381 260 z m 1297 1214 l 894 1152 b 825 1067 854 1146 825 1110 l 825 777 b 781 782 813 779 793 781 b 608 651 685 782 608 723 b 781 521 608 579 685 521 b 955 650 877 521 954 579 l 955 945 l 1259 992 l 1259 864 b 1215 868 1247 866 1227 868 b 1042 738 1119 868 1042 810 b 1215 608 1042 666 1119 608 b 1389 737 1311 608 1388 666 l 1389 1128 b 1297 1214 1389 1181 1345 1222 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 1349 234 l 1045 364 b 1020 369 1039 367 1027 369 b 969 345 1003 369 980 359 l 835 181 b 354 662 655 266 439 481 l 519 796 b 543 847 532 807 543 829 b 537 872 543 854 540 865 l 407 1176 b 347 1215 398 1198 371 1215 b 333 1214 343 1215 337 1215 l 50 1149 b 0 1085 23 1142 0 1114 l 0 1085 b 1259 -174 0 389 564 -174 l 1259 -174 b 1322 -123 1287 -174 1316 -151 l 1387 159 b 1389 173 1388 163 1389 169 b 1349 234 1389 197 1371 224 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 1085 1128 l 130 1128 b 0 998 58 1128 0 1070 l 0 43 b 130 -87 0 -28 58 -87 l 1085 -87 b 1215 43 1157 -87 1215 -28 l 1215 998 b 1085 1128 1215 1070 1157 1128 z m 1041 295 l 1000 118 b 960 87 996 101 978 87 b 174 873 526 87 174 439 b 205 913 174 891 188 909 l 381 954 b 391 955 384 954 388 955 b 428 930 405 954 422 943 l 509 740 b 513 724 511 736 513 729 b 498 693 512 714 505 700 l 395 609 b 696 308 448 496 583 361 l 780 411 b 811 426 787 419 801 425 b 827 423 816 426 823 424 l 1017 341 b 1042 304 1030 335 1041 318 b 1041 295 1042 301 1041 297 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1649 1215 l 434 1215 b 347 1128 386 1215 347 1176 l 347 868 l 781 868 l 781 1042 l 1302 1042 l 1302 174 l 1649 174 b 1736 260 1697 174 1736 212 l 1736 1128 b 1649 1215 1736 1176 1697 1215 z m 629 936 b 605 911 629 922 618 911 l 524 911 b 499 936 510 911 499 922 l 499 1017 b 524 1042 499 1031 510 1042 l 605 1042 b 629 1017 618 1042 629 1031 z m 1584 372 b 1560 347 1584 358 1573 347 l 1478 347 b 1454 372 1465 347 1454 358 l 1454 453 b 1478 477 1454 466 1465 477 l 1560 477 b 1584 453 1573 477 1584 466 z m 1584 654 b 1560 629 1584 640 1573 629 l 1478 629 b 1454 654 1465 629 1454 640 l 1454 735 b 1478 760 1454 749 1465 760 l 1560 760 b 1584 735 1573 760 1584 749 z m 1584 936 b 1560 911 1584 922 1573 911 l 1478 911 b 1454 936 1465 911 1454 922 l 1454 1017 b 1478 1042 1454 1031 1465 1042 l 1560 1042 b 1584 1017 1573 1042 1584 1031 z m 1128 781 l 87 781 b 0 694 39 781 0 742 l 0 -87 b 87 -174 0 -135 39 -174 l 1128 -174 b 1215 -87 1176 -174 1215 -135 l 1215 694 b 1128 781 1215 742 1176 781 z m 260 608 b 347 521 308 608 347 569 b 260 434 347 473 308 434 b 174 521 212 434 174 473 b 260 608 174 569 212 608 z m 1042 0 l 174 0 l 174 87 l 347 260 l 434 174 l 781 521 l 1042 260 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 911 87 l 881 87 l 906 162 l 724 303 l 652 87 l 564 87 b 521 43 540 87 521 67 l 521 -43 b 564 -87 521 -67 540 -87 l 911 -87 b 955 -43 935 -87 955 -67 l 955 43 b 911 87 955 67 935 87 z m 1719 -27 l 1023 511 l 1156 911 l 1476 911 l 1476 825 b 1519 781 1476 801 1495 781 l 1606 781 b 1649 825 1630 781 1649 801 l 1649 1085 b 1606 1128 1649 1109 1630 1128 l 477 1128 b 434 1085 453 1128 434 1109 l 434 966 l 123 1206 b 97 1215 117 1211 105 1215 b 62 1198 85 1215 70 1208 l 9 1130 b 0 1103 4 1123 0 1112 b 17 1069 0 1092 8 1076 l 1613 -164 b 1639 -174 1619 -170 1631 -174 b 1674 -157 1651 -174 1667 -166 l 1727 -88 b 1736 -62 1732 -82 1736 -70 b 1719 -27 1736 -50 1729 -35 z m 841 652 l 608 832 l 608 911 l 927 911 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 477 260 l 347 260 l 347 1085 b 304 1128 347 1109 328 1128 l 217 1128 b 174 1085 193 1128 174 1109 l 174 260 l 43 260 b 13 186 5 260 -15 214 l 230 -74 b 260 -87 237 -81 251 -87 b 291 -74 270 -87 284 -81 l 508 186 b 477 260 535 214 516 260 z m 781 608 l 1128 608 b 1172 651 1152 608 1172 627 l 1172 738 b 1128 781 1172 762 1152 781 l 977 781 l 1143 972 b 1172 1037 1159 987 1172 1016 l 1172 1037 l 1172 1085 b 1128 1128 1172 1109 1152 1128 l 781 1128 b 738 1085 757 1128 738 1109 l 738 998 b 781 955 738 974 757 955 l 933 955 l 767 764 b 738 699 751 749 738 720 l 738 699 l 738 651 b 781 608 738 627 757 608 z m 1213 -29 l 1052 405 b 1011 434 1046 421 1028 434 l 1011 434 l 899 434 l 899 434 b 858 405 882 434 863 421 l 697 -29 b 694 -43 696 -33 694 -39 b 738 -87 694 -67 714 -87 l 805 -87 l 805 -87 b 847 -57 823 -87 841 -73 l 859 -22 l 1051 -22 l 1063 -57 b 1104 -87 1068 -73 1087 -87 l 1104 -87 l 1172 -87 b 1215 -43 1196 -87 1215 -67 b 1213 -29 1215 -39 1214 -33 z m 910 130 l 955 260 l 999 130 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 43 781 l 174 781 l 174 -43 b 217 -87 174 -67 193 -87 l 304 -87 b 347 -43 328 -87 347 -67 l 347 781 l 477 781 b 508 855 516 781 535 828 l 291 1116 b 260 1128 284 1123 270 1128 b 230 1116 250 1128 237 1123 l 13 855 b 43 781 -15 828 5 781 z m 781 608 l 1128 608 b 1172 651 1152 608 1172 627 l 1172 738 b 1128 781 1172 762 1152 781 l 977 781 l 1143 972 b 1172 1037 1159 987 1172 1016 l 1172 1037 l 1172 1085 b 1128 1128 1172 1109 1152 1128 l 781 1128 b 738 1085 757 1128 738 1109 l 738 998 b 781 955 738 974 757 955 l 933 955 l 767 764 b 738 699 751 749 738 720 l 738 699 l 738 651 b 781 608 738 627 757 608 z m 1213 -29 l 1052 405 b 1011 434 1046 421 1028 434 l 1011 434 l 899 434 l 899 434 b 858 405 882 434 863 421 l 697 -29 b 694 -43 696 -33 694 -39 b 738 -87 694 -67 714 -87 l 805 -87 l 805 -87 b 847 -57 823 -87 841 -73 l 859 -22 l 1051 -22 l 1063 -57 b 1104 -87 1068 -73 1087 -87 l 1104 -87 l 1172 -87 b 1215 -43 1196 -87 1215 -67 b 1213 -29 1215 -39 1214 -33 z m 910 130 l 955 260 l 999 130 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 651 955 l 825 955 b 868 998 849 955 868 974 l 868 1085 b 825 1128 868 1109 849 1128 l 651 1128 b 608 1085 627 1128 608 1109 l 608 998 b 651 955 608 974 627 955 z m 651 608 l 998 608 b 1042 651 1022 608 1042 627 l 1042 738 b 998 781 1042 762 1022 781 l 651 781 b 608 738 627 781 608 762 l 608 651 b 651 608 608 627 627 608 z m 1345 87 l 651 87 b 608 43 627 87 608 67 l 608 -43 b 651 -87 608 -67 627 -87 l 1345 -87 b 1389 -43 1369 -87 1389 -67 l 1389 43 b 1345 87 1389 67 1369 87 z m 651 260 l 1172 260 b 1215 304 1196 260 1215 280 l 1215 391 b 1172 434 1215 415 1196 434 l 651 434 b 608 391 627 434 608 415 l 608 304 b 651 260 608 280 627 260 z m 477 260 l 347 260 l 347 1085 b 304 1128 347 1109 328 1128 l 217 1128 b 174 1085 193 1128 174 1109 l 174 260 l 43 260 b 13 186 5 260 -15 214 l 230 -74 b 260 -87 237 -81 251 -87 b 291 -74 270 -87 284 -81 l 508 186 b 477 260 535 214 516 260 z "
	},
	"": {
		ha: 1389,
		x_min: 0,
		x_max: 0,
		o: "m 651 955 l 825 955 b 868 998 849 955 868 974 l 868 1085 b 825 1128 868 1109 849 1128 l 651 1128 b 608 1085 627 1128 608 1109 l 608 998 b 651 955 608 974 627 955 z m 651 608 l 998 608 b 1042 651 1022 608 1042 627 l 1042 738 b 998 781 1042 762 1022 781 l 651 781 b 608 738 627 781 608 762 l 608 651 b 651 608 608 627 627 608 z m 1345 87 l 651 87 b 608 43 627 87 608 67 l 608 -43 b 651 -87 608 -67 627 -87 l 1345 -87 b 1389 -43 1369 -87 1389 -67 l 1389 43 b 1345 87 1389 67 1369 87 z m 651 260 l 1172 260 b 1215 304 1196 260 1215 280 l 1215 391 b 1172 434 1215 415 1196 434 l 651 434 b 608 391 627 434 608 415 l 608 304 b 651 260 608 280 627 260 z m 43 781 l 174 781 l 174 -43 b 217 -87 174 -67 193 -87 l 304 -87 b 347 -43 328 -87 347 -67 l 347 781 l 477 781 b 508 855 516 781 535 828 l 291 1116 b 260 1128 284 1123 270 1128 b 230 1116 250 1128 237 1123 l 13 855 b 43 781 -15 828 5 781 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 477 260 l 347 260 l 347 1085 b 304 1128 347 1109 328 1128 l 217 1128 b 174 1085 193 1128 174 1109 l 174 260 l 43 260 b 13 186 5 260 -15 214 l 230 -74 b 260 -87 237 -81 251 -87 b 291 -74 270 -87 284 -81 l 508 186 b 477 260 535 214 516 260 z m 1085 87 l 1042 87 l 1042 391 b 998 434 1042 415 1022 434 l 868 434 b 829 410 853 434 836 423 l 786 323 b 781 304 783 318 781 309 b 825 260 781 280 801 260 l 868 260 l 868 87 l 825 87 b 781 43 801 87 781 67 l 781 -43 b 825 -87 781 -67 801 -87 l 1085 -87 b 1128 -43 1109 -87 1128 -67 l 1128 43 b 1085 87 1128 67 1109 87 z m 896 1121 b 746 974 832 1103 765 1037 b 933 696 708 835 804 708 b 877 660 919 684 894 668 b 854 605 856 651 847 627 l 881 551 b 938 524 889 528 915 514 b 1172 882 1095 591 1172 691 l 1172 911 b 896 1121 1172 1050 1041 1159 z m 955 857 b 901 911 925 857 901 882 b 955 966 901 941 925 966 b 1009 911 985 966 1009 941 b 955 857 1009 882 985 857 z "
	},
	"": {
		ha: 1215,
		x_min: 0,
		x_max: 0,
		o: "m 291 1116 b 260 1128 284 1123 270 1128 b 230 1116 250 1128 237 1123 l 13 855 b 43 781 -15 828 5 781 l 174 781 l 174 -43 b 217 -87 174 -67 193 -87 l 304 -87 b 347 -43 328 -87 347 -67 l 347 781 l 477 781 b 508 855 516 781 535 828 z m 1085 87 l 1042 87 l 1042 391 b 998 434 1042 415 1022 434 l 868 434 b 829 410 853 434 836 423 l 786 323 b 781 304 783 318 781 309 b 825 260 781 280 801 260 l 868 260 l 868 87 l 825 87 b 781 43 801 87 781 67 l 781 -43 b 825 -87 781 -67 801 -87 l 1085 -87 b 1128 -43 1109 -87 1128 -67 l 1128 43 b 1085 87 1128 67 1109 87 z m 896 1121 b 746 974 832 1103 765 1037 b 933 696 708 835 804 708 b 877 660 919 684 894 668 b 854 605 856 651 847 627 l 881 551 b 938 524 889 528 915 514 b 1172 882 1095 591 1172 691 l 1172 911 b 896 1121 1172 1050 1041 1159 z m 955 857 b 901 911 925 857 901 882 b 955 966 901 941 925 966 b 1009 911 985 966 1009 941 b 955 857 1009 882 985 857 z "
	},
	"": {
		ha: 1563,
		x_min: 0,
		x_max: 0,
		o: "m 738 521 l 986 521 b 1215 725 1103 521 1208 608 b 1215 738 1215 729 1215 734 b 1146 896 1215 790 1184 861 b 1172 998 1160 923 1172 968 b 1171 1018 1172 1004 1171 1013 b 941 1215 1161 1132 1056 1215 l 738 1215 b 694 1172 714 1215 694 1196 l 694 564 b 738 521 694 540 714 521 z m 846 1063 l 955 1063 b 1020 998 991 1063 1020 1034 b 955 933 1020 962 991 933 l 846 933 z m 846 803 l 998 803 b 1063 738 1034 803 1063 774 b 998 673 1063 702 1034 673 l 846 673 z m 421 1155 b 338 1215 410 1188 373 1215 l 338 1215 l 270 1215 l 270 1215 b 187 1155 235 1215 198 1188 l 2 576 b 0 564 1 573 0 567 b 43 521 0 540 19 521 l 111 521 l 111 521 b 153 553 129 521 148 535 l 185 651 l 422 651 l 455 553 b 497 521 460 535 478 521 l 497 521 l 564 521 l 564 521 b 608 564 588 521 608 540 b 606 576 608 567 607 573 z m 242 825 l 304 1011 l 365 825 z m 1550 465 l 1427 588 b 1397 601 1420 595 1407 601 b 1366 588 1387 601 1373 595 l 955 177 l 805 328 b 774 341 798 335 784 341 b 743 328 764 341 750 335 l 621 205 b 608 174 614 198 608 184 b 621 144 608 164 614 151 l 924 -161 b 955 -174 931 -168 945 -174 b 986 -161 965 -174 979 -168 l 1550 404 b 1563 435 1557 411 1563 425 b 1550 465 1563 445 1557 458 z "
	},
	"": {
		ha: 1736,
		x_min: 0,
		x_max: 0,
		o: "m 1345 868 l 1345 868 b 955 477 1130 868 955 693 b 1021 260 955 412 984 315 l 715 260 b 781 477 752 315 781 412 b 391 868 781 693 606 868 b 0 477 175 868 0 693 b 391 87 0 262 175 87 l 1345 87 b 1736 477 1561 87 1736 262 b 1345 868 1736 693 1561 868 z m 174 477 b 391 694 174 597 271 694 b 608 477 510 694 608 597 b 391 260 608 358 510 260 b 174 477 271 260 174 358 z m 1345 260 b 1128 477 1226 260 1128 358 b 1345 694 1128 597 1226 694 b 1563 477 1465 694 1563 597 b 1345 260 1563 358 1465 260 z "
	}
};
var familyName$1 = "Font Awesome 5 Free Solid";
var ascender$1 = 1215;
var descender$1 = -174;
var underlinePosition$1 = -174;
var underlineThickness$1 = 68;
var boundingBox$1 = {
	yMin: -174,
	xMin: -76,
	yMax: 1215,
	xMax: 10829
};
var resolution$1 = 1000;
var original_font_information$1 = {
	format: 0,
	copyright: "Copyright (c) Font Awesome",
	fontFamily: "Font Awesome 5 Free Solid",
	fontSubfamily: "Solid",
	uniqueID: "Font Awesome 5 Free Solid-5.9.0",
	fullName: "Font Awesome 5 Free Solid",
	version: "329.984 (Font Awesome version: 5.9.0)",
	postScriptName: "FontAwesome5Free-Solid",
	description: "The web's most popular icon set and toolkit.",
	manufacturerURL: "https://fontawesome.com",
	preferredFamily: "Font Awesome 5 Free",
	preferredSubfamily: "Solid",
	compatibleFullName: "Font Awesome 5 Free Solid",
	wwsFamily: "Font Awesome 5 Free",
	wwsSubfamily: "Solid"
};
var cssFontWeight$1 = "normal";
var cssFontStyle$1 = "normal";
var font_awesome_5_free_solid = {
	glyphs: glyphs$1,
	familyName: familyName$1,
	ascender: ascender$1,
	descender: descender$1,
	underlinePosition: underlinePosition$1,
	underlineThickness: underlineThickness$1,
	boundingBox: boundingBox$1,
	resolution: resolution$1,
	original_font_information: original_font_information$1,
	cssFontWeight: cssFontWeight$1,
	cssFontStyle: cssFontStyle$1
};

var defaultOptions = {
  range_x: -1000,
  range_y: 1000,
  minZoomDistance: 200,
  width: 100,
  height: 100,
  backgroundColor: '#f8f8fa',
  isSelectionOn: false
};

var parseFonts = function parseFonts(fontLoader) {
  var fontAwesome = fontLoader.parse(font_awesome_5_free_solid);
  var openSans = fontLoader.parse(open_sans_regular_font);
  return {
    fontAwesome: fontAwesome,
    openSans: openSans
  };
};

var initThreeJsCanvas = function initThreeJsCanvas(container, options) {
  if (options === void 0) {
    options = {};
  }

  var toMatch = [/Android/i, /webOS/i, /iPhone/i, /iPad/i, /iPod/i, /BlackBerry/i, /Windows Phone/i];
  window.isMobile = toMatch.some(function (toMatchItem) {
    return navigator.userAgent.match(toMatchItem);
  });
  options = _extends({}, defaultOptions, options);
  var range = Math.max(options.range_x, options.range_y);
  var size = options.width;
  var scale = size * 1.5 / range;
  var scene = new Scene();
  scene.background = new Color(options.backgroundColor);
  var camera = new PerspectiveCamera(75, 1, 0.1, 10000);
  camera.lookAt(new Vector3(0, 0, 0));
  camera.position.set(0, 0, size);
  var renderer = new WebGLRenderer({
    antialias: true
  });
  renderer.setPixelRatio(window.devicePixelRatio);
  renderer.setSize(options.width, options.height);

  if (container.firstChild) {
    container.removeChild(container.firstChild);
  }

  container.appendChild(renderer.domElement);
  Cache.enabled = true;
  var fontLoader = new FontLoader();
  var controls = new MapControls(camera, renderer.domElement);
  controls.screenSpacePanning = true;
  controls.enableRotate = false;
  controls.enablePan = true;
  controls.minDistance = options.minZoomDistance;
  controls.maxDistance = size;
  controls.panBoundaries = new Vector4(-370 * size / 900, 370 * size / 900, -225 * size / 900, 225 * size / 900);
  renderer.domElement.focus = null;
  var fonts = parseFonts(fontLoader);
  var zoomSpeed = 0.1;
  var zoom = 1;

  function animate() {
    if (!animate.cancel) {
      controls.update();
      renderer.render(scene, camera);
      animate.cancel = true;
    }

    window.requestAnimationFrame(animate);
  }

  animate.cancel = false;

  animate.startAnimation = function () {
    return animate.cancel = false;
  };

  animate.cancelAnimation = function () {
    return animate.cancel = true;
  };

  animate();

  if (controls !== null) {
    controls.addEventListener('change', function () {
      animate.startAnimation();
    });
  }

  var controlsReset = function controlsReset() {
    controls.reset();
    animate.startAnimation();
  };

  var updateCameraZoom = function updateCameraZoom(zoom) {
    camera.zoom = zoom;
    camera.updateProjectionMatrix();
    animate.startAnimation();
  };

  var controlsZoomIn = function controlsZoomIn() {
    zoom += zoomSpeed;
    updateCameraZoom(zoom);
  };

  var controlsZoomOut = function controlsZoomOut() {
    zoom -= zoomSpeed;
    updateCameraZoom(zoom);
  };

  return {
    renderer: renderer,
    scene: scene,
    camera: camera,
    fonts: fonts,
    scale: scale,
    controlsReset: controlsReset,
    controlsZoomIn: controlsZoomIn,
    controlsZoomOut: controlsZoomOut,
    startAnimation: animate.startAnimation,
    cancelAnimation: animate.cancelAnimation
  };
};

var SELECT_LIMIT = 30;
var MESSAGES = {
  GROUP_CHANGE: 'You are switching to a different ticket type. Are you sure you want to discard selected seats?',
  ALREADY_RESERVED: 'Selected seat is not available',
  LIMIT_EXCEEDED: "You can buy " + SELECT_LIMIT + " tickets at a time",
  SESSION_EXPIRED: 'Session expired',
  UNEXPECTED_ERROR: 'Unexpected error'
};
var COLORS = {
  BACKGROUND: '#ffffff',
  AVAILABLE: '#80c87f',
  PREDEFINED: '#009999',
  BOOKED: '#e26da0',
  RESERVED: '#7c8fd2',
  RESERVED_BY_ME: '#f5d158',
  BLOCKED: '#808080',
  HOLD: '#b1b1b1',
  STAGE: '#f1f3f7',
  STAGE_BORDER: '#c8d6ec',
  SELECTED_BLOCK_BORDER: "#696969",
  HIGHLIGHT: "#f5d158",
  DARK_GREY: '#bbbbbb',
  SOLD: '#ff0000',
  'ON HOLD': '#b1b1b1'
};
var TIER_COLORS = ['#4a2ac0', '#ff5cb7', '#d86500', '#33c379', '#9d0f3c', '#005985', '#abcb00', '#3b2f2f', '#264653', '#008000', '#2a9d8f', '#e9c46a', '#e76f51', '#9b5de5', '#0063e4', '#540d6e', '#dd3b3b', '#83e377', '#2d3047', '#8da2dd', '#0f39f8', '#8cb369', '#fdf148', '#4fd9b6', '#b92755', '#e1f400', '#968e85', '#fc296d', '#2667c4', '#5c0029', '#857c8d', '#3e4d06', '#153131', '#af5b74', '#616b77', '#02654b', '#38b6fa', '#ee7808', '#eca0ba', '#90251b', '#aaf683', '#8e7dbe', '#01295f', '#ab87ff', '#986b2a', '#789171', '#61304b', '#94f686', '#f9b3c7', '#f0d193'];
var SEAT_STATUSES = {
  AVAILABLE: 'A',
  RESERVED: 'R',
  RESERVED_BY_ME: 'OR',
  BOOKED: 'B',
  BLOCKED: 'BLOCKED',
  PERMANENT_RESERVED: 'IR',
  HOLD: 'H',
  PREDEFINED: 'PREDEFINED'
};
var SEAT_GROUP_TYPES = {
  STANDING: 'STANDING'
};
var OBJECT_TYPES = {
  SEAT: 'SEAT'
};
var SEAT_TYPES = {
  CHAIR: 'chair',
  WHEELCHAIR: 'wheelchair'
};

var incrementInMap = function incrementInMap(map, key) {
  if (map.has(key)) {
    map.set(key, map.get(key) + 1);
  } else {
    map.set(key, 1);
  }
};

var statusesUpdate = function statusesUpdate(newStatuses, seats) {
  var counts = new Map();
  Object.keys(newStatuses).forEach(function (groupRowId) {
    var rowStatuses = newStatuses[groupRowId];

    if (typeof rowStatuses !== 'string') {
      Object.keys(rowStatuses).forEach(function (seatIndex) {
        var newStatus = newStatuses[groupRowId][seatIndex];
        var seatId = groupRowId + ':' + seatIndex;

        if (seats[seatId]) {
          if (Array.isArray(seats[seatId])) {
            seats[seatId].map(function (s) {
              return s.statusUpdate(newStatus);
            });
          } else {
            seats[seatId].statusUpdate(newStatus);
          }
        }

        incrementInMap(counts, newStatus);
      });
    }
  });
  return counts;
};

var blocksStatusesUpdate = function blocksStatusesUpdate(newStatuses, blocks) {
  var statuses = {};
  blocks.forEach(function (block) {
    statuses[block.sgId] = {
      available: 0
    };
  });

  var _loop = function _loop() {
    var _Object$entries$_i = _Object$entries[_i],
        rowId = _Object$entries$_i[0],
        rowStatuses = _Object$entries$_i[1];

    var _rowId$split = rowId.split('-'),
        groupId = _rowId$split[0];

    Object.values(rowStatuses).forEach(function (status) {
      if (status === SEAT_STATUSES.AVAILABLE) {
        ++statuses[groupId].available;
      }
    });
  };

  for (var _i = 0, _Object$entries = Object.entries(newStatuses); _i < _Object$entries.length; _i++) {
    _loop();
  }

  blocks.forEach(function (block) {
    var status = statuses[block.sgId].available > 0 ? SEAT_STATUSES.AVAILABLE : SEAT_STATUSES.BOOKED;
    block.statusUpdate(status, statuses[block.sgId].available);
  });
};

var standingAreaStatusesUpdate = function standingAreaStatusesUpdate(newStatuses, standingAreas) {
  standingAreas.forEach(function (standingArea) {
    newStatuses[standingArea.sgId] && standingArea.statusUpdate(newStatuses[standingArea.sgId]);
  });
};

// A type of promise-like that resolves synchronously and supports only one observer
const _Pact = /*#__PURE__*/(function() {
	function _Pact() {}
	_Pact.prototype.then = function(onFulfilled, onRejected) {
		const result = new _Pact();
		const state = this.s;
		if (state) {
			const callback = state & 1 ? onFulfilled : onRejected;
			if (callback) {
				try {
					_settle(result, 1, callback(this.v));
				} catch (e) {
					_settle(result, 2, e);
				}
				return result;
			} else {
				return this;
			}
		}
		this.o = function(_this) {
			try {
				const value = _this.v;
				if (_this.s & 1) {
					_settle(result, 1, onFulfilled ? onFulfilled(value) : value);
				} else if (onRejected) {
					_settle(result, 1, onRejected(value));
				} else {
					_settle(result, 2, value);
				}
			} catch (e) {
				_settle(result, 2, e);
			}
		};
		return result;
	};
	return _Pact;
})();

// Settles a pact synchronously
function _settle(pact, state, value) {
	if (!pact.s) {
		if (value instanceof _Pact) {
			if (value.s) {
				if (state & 1) {
					state = value.s;
				}
				value = value.v;
			} else {
				value.o = _settle.bind(null, pact, state);
				return;
			}
		}
		if (value && value.then) {
			value.then(_settle.bind(null, pact, state), _settle.bind(null, pact, 2));
			return;
		}
		pact.s = state;
		pact.v = value;
		const observer = pact.o;
		if (observer) {
			observer(pact);
		}
	}
}

function _isSettledPact(thenable) {
	return thenable instanceof _Pact && thenable.s & 1;
}

// Asynchronously iterate through an object that has a length property, passing the index as the first argument to the callback (even as the length property changes)
function _forTo(array, body, check) {
	var i = -1, pact, reject;
	function _cycle(result) {
		try {
			while (++i < array.length && (!check || !check())) {
				result = body(i);
				if (result && result.then) {
					if (_isSettledPact(result)) {
						result = result.v;
					} else {
						result.then(_cycle, reject || (reject = _settle.bind(null, pact = new _Pact(), 2)));
						return;
					}
				}
			}
			if (pact) {
				_settle(pact, 1, result);
			} else {
				pact = result;
			}
		} catch (e) {
			_settle(pact || (pact = new _Pact()), 2, e);
		}
	}
	_cycle();
	return pact;
}

const _iteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.iterator || (Symbol.iterator = Symbol("Symbol.iterator"))) : "@@iterator";

// Asynchronously iterate through an object's values
// Uses for...of if the runtime supports it, otherwise iterates until length on a copy
function _forOf(target, body, check) {
	if (typeof target[_iteratorSymbol] === "function") {
		var iterator = target[_iteratorSymbol](), step, pact, reject;
		function _cycle(result) {
			try {
				while (!(step = iterator.next()).done && (!check || !check())) {
					result = body(step.value);
					if (result && result.then) {
						if (_isSettledPact(result)) {
							result = result.v;
						} else {
							result.then(_cycle, reject || (reject = _settle.bind(null, pact = new _Pact(), 2)));
							return;
						}
					}
				}
				if (pact) {
					_settle(pact, 1, result);
				} else {
					pact = result;
				}
			} catch (e) {
				_settle(pact || (pact = new _Pact()), 2, e);
			}
		}
		_cycle();
		if (iterator.return) {
			var _fixup = function(value) {
				try {
					if (!step.done) {
						iterator.return();
					}
				} catch(e) {
				}
				return value;
			};
			if (pact && pact.then) {
				return pact.then(_fixup, function(e) {
					throw _fixup(e);
				});
			}
			_fixup();
		}
		return pact;
	}
	// No support for Symbol.iterator
	if (!("length" in target)) {
		throw new TypeError("Object is not iterable");
	}
	// Handle live collections properly
	var values = [];
	for (var i = 0; i < target.length; i++) {
		values.push(target[i]);
	}
	return _forTo(values, function(i) { return body(values[i]); }, check);
}

const _asyncIteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.asyncIterator || (Symbol.asyncIterator = Symbol("Symbol.asyncIterator"))) : "@@asyncIterator";

// Asynchronously implement a switch statement
function _switch(discriminant, cases) {
	var dispatchIndex = -1;
	var awaitBody;
	outer: {
		for (var i = 0; i < cases.length; i++) {
			var test = cases[i][0];
			if (test) {
				var testValue = test();
				if (testValue && testValue.then) {
					break outer;
				}
				if (testValue === discriminant) {
					dispatchIndex = i;
					break;
				}
			} else {
				// Found the default case, set it as the pending dispatch case
				dispatchIndex = i;
			}
		}
		if (dispatchIndex !== -1) {
			do {
				var body = cases[dispatchIndex][1];
				while (!body) {
					dispatchIndex++;
					body = cases[dispatchIndex][1];
				}
				var result = body();
				if (result && result.then) {
					awaitBody = true;
					break outer;
				}
				var fallthroughCheck = cases[dispatchIndex][2];
				dispatchIndex++;
			} while (fallthroughCheck && !fallthroughCheck());
			return result;
		}
	}
	const pact = new _Pact();
	const reject = _settle.bind(null, pact, 2);
	(awaitBody ? result.then(_resumeAfterBody) : testValue.then(_resumeAfterTest)).then(void 0, reject);
	return pact;
	function _resumeAfterTest(value) {
		for (;;) {
			if (value === discriminant) {
				dispatchIndex = i;
				break;
			}
			if (++i === cases.length) {
				if (dispatchIndex !== -1) {
					break;
				} else {
					_settle(pact, 1, result);
					return;
				}
			}
			test = cases[i][0];
			if (test) {
				value = test();
				if (value && value.then) {
					value.then(_resumeAfterTest).then(void 0, reject);
					return;
				}
			} else {
				dispatchIndex = i;
			}
		}
		do {
			var body = cases[dispatchIndex][1];
			while (!body) {
				dispatchIndex++;
				body = cases[dispatchIndex][1];
			}
			var result = body();
			if (result && result.then) {
				result.then(_resumeAfterBody).then(void 0, reject);
				return;
			}
			var fallthroughCheck = cases[dispatchIndex][2];
			dispatchIndex++;
		} while (fallthroughCheck && !fallthroughCheck());
		_settle(pact, 1, result);
	}
	function _resumeAfterBody(result) {
		for (;;) {
			var fallthroughCheck = cases[dispatchIndex][2];
			if (!fallthroughCheck || fallthroughCheck()) {
				break;
			}
			dispatchIndex++;
			var body = cases[dispatchIndex][1];
			while (!body) {
				dispatchIndex++;
				body = cases[dispatchIndex][1];
			}
			result = body();
			if (result && result.then) {
				result.then(_resumeAfterBody).then(void 0, reject);
				return;
			}
		}
		_settle(pact, 1, result);
	}
}

// Asynchronously call a function and send errors to recovery continuation
function _catch(body, recover) {
	try {
		var result = body();
	} catch(e) {
		return recover(e);
	}
	if (result && result.then) {
		return result.then(void 0, recover);
	}
	return result;
}

function getGeometryArray(geometry) {
  var _geometry$attributes;

  var _geometry$attributes$ = geometry === null || geometry === void 0 ? void 0 : (_geometry$attributes = geometry.attributes) === null || _geometry$attributes === void 0 ? void 0 : _geometry$attributes.position,
      _geometry$attributes$2 = _geometry$attributes$.array,
      array = _geometry$attributes$2 === void 0 ? [] : _geometry$attributes$2;

  var newArray = [];
  newArray.push(array[0]);
  newArray.push(array[1]);
  newArray.push(array[2]);
  newArray.push(array[3]);
  newArray.push(array[4]);
  newArray.push(array[5]);
  newArray.push(array[9]);
  newArray.push(array[10]);
  newArray.push(array[11]);
  newArray.push(array[6]);
  newArray.push(array[7]);
  newArray.push(array[8]);
  newArray.push(array[0]);
  newArray.push(array[1]);
  newArray.push(array[2]);
  return newArray;
}

var drawRect = function drawRect(group, options) {
  var color = options.color,
      scale = options.scale,
      scene = options.scene,
      center_x = options.center_x,
      center_y = options.center_y,
      borderColor = options.borderColor;
  var y = group.y,
      height = group.height,
      width = group.width,
      x = group.x,
      renderOrder = group.renderOrder;
  var geometry = new PlaneGeometry((width || 1825) * scale, (height || 400) * scale);
  var material = new MeshBasicMaterial({
    color: color
  });
  var rect = new Mesh(geometry, material);
  rect.position.set((x - center_x) * scale, -(y - center_y) * scale, 0);

  if (renderOrder) {
    rect.renderOrder = renderOrder;
  }

  scene.add(rect);

  if (borderColor) {
    var lineBasicMaterial = new LineBasicMaterial({
      color: borderColor
    });
    var lineGeometry = new BufferGeometry();
    var geometryArray = getGeometryArray(geometry);
    var vertices = new Float32Array(geometryArray);
    lineGeometry.setAttribute('position', new BufferAttribute(vertices, 3));
    var line = new Line(lineGeometry, lineBasicMaterial);
    rect.add(line);
  }
};

var drawIcon = function drawIcon(content, options) {
  var _label$position;

  var font = options.font,
      color = options.color,
      position = options.position,
      center_x = options.center_x,
      center_y = options.center_y,
      scale = options.scale,
      scene = options.scene;
  position[0] -= center_x * scale;
  position[1] += center_y * scale;
  var labelGeometry = new TextGeometry(content, {
    font: font,
    size: 16 * scale,
    height: 0
  });
  labelGeometry.center();
  var labelMaterial = new MeshBasicMaterial({
    color: color
  });
  var label = new Mesh(labelGeometry, labelMaterial);

  (_label$position = label.position).set.apply(_label$position, position);

  label.renderOrder = 1;
  scene.add(label);
};

var calculateMeshCountInsideObject = function calculateMeshCountInsideObject(obj) {
  return obj.children.reduce(function (acm, current) {
    if (current.type === 'Mesh') {
      return acm + 1;
    }

    if (current.type === 'Group') {
      return acm + calculateMeshCountInsideObject(current);
    }

    return acm;
  }, 1);
};

var getSeatMaterial = function getSeatMaterial(_ref, materials) {
  var color = _ref.color,
      _ref$opacity = _ref.opacity,
      opacity = _ref$opacity === void 0 ? 1 : _ref$opacity,
      _ref$options = _ref.options,
      options = _ref$options === void 0 ? {} : _ref$options;
  var id = color + ":" + opacity + ":" + JSON.stringify(options);

  if (!materials.seat[id]) {
    materials.seat[id] = new MeshBasicMaterial(_extends({
      color: color,
      opacity: opacity,
      transparent: opacity !== 1
    }, options));
  }

  return materials.seat[id];
};

var getLineMaterial = function getLineMaterial(_ref, materials) {
  var color = _ref.color,
      _ref$linewidth = _ref.linewidth,
      linewidth = _ref$linewidth === void 0 ? 1 : _ref$linewidth,
      _ref$opacity = _ref.opacity,
      opacity = _ref$opacity === void 0 ? 1 : _ref$opacity;
  var id = color + ":" + opacity;

  if (!materials.line[id]) {
    materials.line[id] = new LineBasicMaterial({
      color: color,
      linewidth: linewidth,
      opacity: opacity,
      transparent: opacity !== 1
    });
  }

  return materials.line[id];
};

var calculateTranslateVec = function calculateTranslateVec(edges) {
  var translateVec = new Vector2(0, 0);
  Object.values(edges).forEach(function (v) {
    return translateVec.add(v);
  });
  translateVec.divideScalar(Object.keys(edges).length);
  return translateVec;
};

var translateObj = function translateObj(obj, degree, translateVec) {
  obj.translateX(translateVec.x);
  obj.translateY(translateVec.y);
  obj.rotateOnAxis(new Vector3(0, 0, 1), degree * Math.PI / 180);
  obj.translateX(-translateVec.x);
  obj.translateY(-translateVec.y);
};

var rotateGroup = function rotateGroup(group, degree, edges) {
  var translateVec = calculateTranslateVec(edges);
  translateObj(group, degree, translateVec);
};

var createRectsMeshes = function createRectsMeshes(rects, options) {
  var color = options.color,
      scale = options.scale,
      _options$opacity = options.opacity,
      opacity = _options$opacity === void 0 ? 1 : _options$opacity;
  var material = new MeshBasicMaterial({
    color: color,
    opacity: opacity,
    transparent: true
  });
  var ring = new Group();
  var rectMeshes = rects.map(function (_ref) {
    var width = _ref.width,
        height = _ref.height,
        x = _ref.x,
        y = _ref.y;
    var geometry = new PlaneGeometry(width * scale, height * scale);
    var rect = new Mesh(geometry, material);
    ring.add(rect);
    rect.position.set(x * scale, -y * scale, 0);
    return rect;
  });
  return {
    rectMeshes: rectMeshes,
    ring: ring
  };
};

var drawUnclickableTable = function drawUnclickableTable(_ref) {
  var _tableObject$position;

  var seatId = _ref.seatId,
      options = _ref.options,
      materials = _ref.materials,
      seats = _ref.seats,
      seat = _ref.seat;
  var position = options.position,
      parent = options.parent,
      center_x = options.center_x,
      center_y = options.center_y,
      scale = options.scale,
      sectionLabel = options.sectionLabel,
      rowLabel = options.rowLabel,
      label = options.label,
      _options$opacity = options.opacity,
      opacity = _options$opacity === void 0 ? 1 : _options$opacity,
      sceneMeshes = options.sceneMeshes;
  var _seat$type = seat.type,
      type = _seat$type === void 0 ? SEAT_TYPES.CHAIR : _seat$type,
      size = seat.size,
      _seat$table_shape = seat.table_shape,
      tableShape = _seat$table_shape === void 0 ? 'circle' : _seat$table_shape,
      width = seat.width,
      height = seat.height;
  position[0] -= center_x * scale;
  position[1] += center_y * scale;
  var tableObject = null;

  switch (tableShape) {
    case 'rect':
      {
        var rectGeometry = new PlaneGeometry(width * scale, height * scale);
        var material = new MeshBasicMaterial({
          color: COLORS.DARK_GREY
        });
        var rect = new Mesh(rectGeometry, material);
        tableObject = rect;
        break;
      }

    default:
      {
        var _line$position;

        var radius = (size || 14) * scale;
        var geometry = new CircleGeometry(radius, 32);
        tableObject = new Mesh(geometry, getSeatMaterial({
          color: COLORS.DARK_GREY,
          opacity: opacity
        }, materials));
        var edges = new EdgesGeometry(geometry);
        var line = new LineSegments(edges, getLineMaterial({
          color: COLORS.DARK_GREY,
          opacity: opacity
        }, materials));

        (_line$position = line.position).set.apply(_line$position, position);

        line.renderOrder = -1;
        parent.add(line);
      }
  }

  parent.add(tableObject);
  parent.add(tableObject);
  tableObject.renderOrder = -1;

  (_tableObject$position = tableObject.position).set.apply(_tableObject$position, position);

  tableObject.objectType = 'UNCLICKABLE_TABLE';
  tableObject.opacity = opacity;
  tableObject.disabled = true;
  tableObject.radius = size;
  tableObject.seatId = seatId;
  tableObject.status = SEAT_STATUSES.AVAILABLE;
  tableObject.sectionLabel = sectionLabel;
  tableObject.rowLabel = rowLabel;
  tableObject.label = label;
  tableObject.seatType = type;

  tableObject.changeBorderColor = function (color) {};

  tableObject.mouseOver = null;
  tableObject.mouseOut = null;

  tableObject.changeBorderColorToDefault = function () {
    return tableObject.changeBorderColor(COLORS.DARK_GREY);
  };

  tableObject.changeColor = function (color) {
    tableObject.material = getSeatMaterial({
      color: color,
      opacity: tableObject.opacity
    }, materials);

    tableObject.changeBorderColorToDefault = function () {
      return tableObject.changeBorderColor(color);
    };
  };

  tableObject.updateOpacity = function () {
    tableObject.changeColor("#" + tableObject.material.color.getHexString());
  };

  tableObject.makeDisabled = function () {
    if (tableObject.disabled) {
      return;
    }

    tableObject.disabled = true;
    tableObject.mouseOver = null;
    tableObject.mouseOut = null;
    tableObject.opacity = 0.3;
  };

  tableObject.makeActive = function () {
    if (!tableObject.disabled) {
      return;
    }

    tableObject.disabled = false;
    tableObject.mouseOver = null;
    tableObject.mouseOut = null;
    tableObject.opacity = 1;
  };

  tableObject.statusUpdate = function (status) {};

  tableObject.hasHandlers = true;
  sceneMeshes.push(tableObject);
  seats[seatId] = tableObject;
};

var drawLabel = function drawLabel(content, options, textGeometries, materials) {
  var _label$position;

  var font = options.font,
      color = options.color,
      position = options.position,
      size = options.size,
      center_x = options.center_x,
      center_y = options.center_y,
      scale = options.scale,
      seat_circle_radius = options.seat_circle_radius,
      scene = options.scene,
      _options$rotation = options.rotation,
      rotation = _options$rotation === void 0 ? 0 : _options$rotation,
      _options$opacity = options.opacity,
      opacity = _options$opacity === void 0 ? 1 : _options$opacity;
  position[0] -= center_x * scale;
  position[1] += center_y * scale;
  var labelGeometry;

  if (textGeometries.get(content)) {
    labelGeometry = textGeometries.get(content);
  } else {
    var labelSize = size || (seat_circle_radius || 14) * scale * 4 / 5;
    labelGeometry = new TextGeometry(content, {
      font: font,
      size: labelSize,
      height: 0
    });
    textGeometries.set(content, labelGeometry);
  }

  labelGeometry.center();
  var labelMaterial = getSeatMaterial({
    color: color,
    opacity: opacity
  }, materials);
  var label = new Mesh(labelGeometry, labelMaterial);

  (_label$position = label.position).set.apply(_label$position, position);

  label.rotation.set(0, 0, rotation * Math.PI / 180);
  label.renderOrder = 1;
  scene.add(label);
  return label;
};

var textGeometries = new Map();
var materials = {
  seat: {},
  line: {}
};
var seats = {};
var standingAreas = [];
var blocks = [];

var drawSeat = function drawSeat(seatId, options) {
  var _circle$position, _line$position;

  var color = options.color,
      position = options.position,
      rotation = options.rotation,
      tierId = options.tierId,
      parent = options.parent,
      center_x = options.center_x,
      center_y = options.center_y,
      scale = options.scale,
      seat_circle_radius = options.seat_circle_radius,
      events = options.events,
      sectionLabel = options.sectionLabel,
      rowLabel = options.rowLabel,
      label = options.label,
      fonts = options.fonts,
      _options$opacity = options.opacity,
      opacity = _options$opacity === void 0 ? 1 : _options$opacity,
      _options$disabled = options.disabled,
      disabled = _options$disabled === void 0 ? false : _options$disabled,
      _options$type = options.type,
      type = _options$type === void 0 ? SEAT_TYPES.CHAIR : _options$type,
      seats = options.seats,
      tierColor = options.tierColor,
      enableTicketTypeAssign = options.enableTicketTypeAssign,
      selectedTierId = options.selectedTierId,
      sceneMeshes = options.sceneMeshes;
  var seatColor = enableTicketTypeAssign ? tierColor : COLORS.BOOKED;
  position[0] -= center_x * scale;
  position[1] += center_y * scale;
  var radius = (seat_circle_radius || 14) * scale;
  var geometry = new CircleGeometry(radius, 32);

  if (selectedTierId && selectedTierId !== tierId) {
    opacity = 0.3;
  }

  var circle = new Mesh(geometry, getSeatMaterial({
    color: seatColor,
    opacity: opacity
  }, materials));

  (_circle$position = circle.position).set.apply(_circle$position, position);

  circle.isRequestPending = false;
  var edges = new EdgesGeometry(geometry);
  var line = new LineSegments(edges, getLineMaterial({
    color: seatColor,
    opacity: opacity
  }, materials));

  (_line$position = line.position).set.apply(_line$position, position);

  parent.add(line);
  parent.add(circle);
  parent.add(circle);

  if (type === SEAT_TYPES.WHEELCHAIR) {
    drawLabel("\uF193", {
      font: fonts.fontAwesome,
      color: 'black',
      position: [0, 0, 0],
      center_x: 0,
      center_y: 0,
      scale: scale,
      seat_circle_radius: seat_circle_radius,
      scene: circle,
      size: seat_circle_radius * scale,
      rotation: rotation
    }, textGeometries, materials);
  }

  circle.objectType = OBJECT_TYPES.SEAT;
  circle.opacity = opacity;
  circle.disabled = disabled;
  circle.allowColorUpdateByStatus = true;
  circle.radius = radius;
  circle.seatId = seatId;
  circle.tierId = tierId;
  circle.status = SEAT_STATUSES.BOOKED;
  circle.sectionLabel = sectionLabel;
  circle.rowLabel = rowLabel;
  circle.label = label;
  circle.seatType = type;

  circle.changeBorderColor = function (color) {
    if (!enableTicketTypeAssign) {
      line.material = getLineMaterial({
        color: color,
        opacity: circle.opacity
      }, materials);
    }
  };

  circle.mouseOver = circle.disabled ? null : events.onMouseOver;
  circle.mouseOut = circle.disabled ? null : events.onMouseOut;

  circle.changeBorderColorToDefault = function () {
    return circle.changeBorderColor(COLORS.BOOKED);
  };

  circle.changeColor = function (color) {
    line.material = getLineMaterial({
      color: color,
      opacity: circle.opacity
    }, materials);
    circle.material = getSeatMaterial({
      color: color,
      opacity: circle.opacity
    }, materials);

    circle.changeBorderColorToDefault = function () {
      return circle.changeBorderColor(color);
    };

    circle.color = color;
  };

  circle.changeOpacity = function (opacity, color) {
    if (color === void 0) {
      color = seatColor;
    }

    line.material = getLineMaterial({
      color: color,
      opacity: opacity
    }, materials);
    circle.material = getSeatMaterial({
      color: color,
      opacity: opacity
    }, materials);
  };

  var makeAvailable = function makeAvailable() {
    circle.status = SEAT_STATUSES.AVAILABLE;
  };

  var makeHold = function makeHold() {
    circle.status = SEAT_STATUSES.HOLD;
  };

  var makeReserved = function makeReserved() {
    circle.status = SEAT_STATUSES.RESERVED;
  };

  var makeBooked = function makeBooked() {
    circle.status = SEAT_STATUSES.BOOKED;
  };

  var makeBlocked = function makeBlocked() {
    circle.status = SEAT_STATUSES.BLOCKED;
  };

  var makeReservedByMe = function makeReservedByMe() {
    window.localStorage.setItem('tierId', tierId);
    circle.status = SEAT_STATUSES.RESERVED_BY_ME;
  };

  circle.setColorByStatus = function () {
    if (!circle.allowColorUpdateByStatus) {
      return;
    }

    switch (circle.status) {
      case SEAT_STATUSES.AVAILABLE:
        {
          circle.changeColor(color);
          break;
        }

      case SEAT_STATUSES.HOLD:
        {
          circle.changeColor(COLORS.HOLD);
          break;
        }

      case SEAT_STATUSES.RESERVED_BY_ME:
        {
          circle.changeColor(COLORS.RESERVED_BY_ME);
          break;
        }

      case SEAT_STATUSES.RESERVED:
        {
          circle.changeColor(COLORS.RESERVED);
          break;
        }

      case SEAT_STATUSES.BOOKED:
        {
          circle.changeColor(COLORS.BOOKED);
          break;
        }

      case SEAT_STATUSES.BLOCKED:
        {
          circle.changeColor(COLORS.BLOCKED);
          break;
        }
    }
  };

  circle.updateOpacity = function () {
    circle.changeColor("#" + circle.material.color.getHexString());
  };

  circle.makeDisabled = function () {
    if (circle.disabled) {
      return;
    }

    circle.disabled = true;
    circle.mouseOver = null;
    circle.mouseOut = null;
    circle.opacity = 0.3;
    circle.updateOpacity();
  };

  circle.makeActive = function () {
    if (!circle.disabled) {
      return;
    }

    circle.disabled = false;
    circle.mouseOver = events.onMouseOver;
    circle.mouseOut = events.onMouseOut;
    circle.opacity = 1;
    circle.updateOpacity();
  };

  var select = function select() {
    line.material = getLineMaterial({
      color: '#575656',
      linewidth: 5,
      opacity: circle.opacity
    }, materials);

    circle.changeBorderColor = function () {
      return null;
    };
  };

  var unselect = function unselect() {
    line.material = getLineMaterial({
      color: '#000',
      opacity: circle.opacity
    }, materials);

    circle.changeBorderColor = function (color) {
      line.material = getLineMaterial({
        color: color,
        opacity: circle.opacity
      }, materials);
    };

    circle.changeBorderColorToDefault();
  };

  circle.select = select;
  circle.unselect = unselect;

  circle.unsetLoading = function () {
    return null;
  };

  circle.setLoading = function () {
    if (window.isMobile) {
      var _label = drawLabel("\uF110", {
        font: fonts.fontAwesome,
        color: 'grey',
        position: [0, 0, 0],
        center_x: 0,
        center_y: 0,
        scale: scale,
        seat_circle_radius: seat_circle_radius,
        scene: circle,
        size: seat_circle_radius * scale,
        rotation: rotation
      }, textGeometries, materials);

      circle.unsetLoading = function () {
        return circle.remove(_label);
      };
    }
  };

  circle.statusUpdate = function (status) {
    if (circle.status !== status) {
      switch (status) {
        case SEAT_STATUSES.PERMANENT_RESERVED:
        case SEAT_STATUSES.AVAILABLE:
          {
            makeAvailable();
            circle.unsetLoading();
            break;
          }

        case SEAT_STATUSES.HOLD:
          {
            makeHold();
            circle.unsetLoading();
            break;
          }

        case SEAT_STATUSES.RESERVED_BY_ME:
          {
            makeReservedByMe();
            circle.unsetLoading();
            break;
          }

        case SEAT_STATUSES.RESERVED:
          {
            makeReserved();
            circle.unsetLoading();
            break;
          }

        case SEAT_STATUSES.BOOKED:
          {
            makeBooked();
            circle.unsetLoading();
            break;
          }

        case SEAT_STATUSES.BLOCKED:
          {
            makeBlocked();
            circle.unsetLoading();
            break;
          }
      }

      circle.setColorByStatus();
    }
  };

  circle.hasHandlers = true;
  sceneMeshes.push(circle);
  seats[seatId] = circle;
};

var colorIndex = {
  value: 0
};
var tierColorObj = {};
var resetColorsData = function resetColorsData() {
  colorIndex.value = 0;

  for (var color in tierColorObj) {
    delete tierColorObj[color];
  }
};
var drawGroupSeats = function drawGroupSeats(group, groups, predefinedSeats, options) {
  var scene = options.scene,
      scale = options.scale,
      fonts = options.fonts,
      center_x = options.center_x,
      center_y = options.center_y,
      seat_circle_radius = options.seat_circle_radius,
      events = options.events,
      enableTicketTypeAssign = options.enableTicketTypeAssign,
      selectedTierId = options.selectedTierId,
      renderer = options.renderer,
      camera = options.camera,
      sceneMeshes = options.sceneMeshes;
  var groupObj = new Group();
  group.x = group.x || 0;
  group.y = -group.y || 0;
  groupObj.position.set(group.x * scale, group.y * scale, 0);
  scene.add(groupObj);

  if (group.rotation && group.edges) {
    Object.keys(group.edges).forEach(function (key) {
      group.edges[key].x *= scale;
      group.edges[key].y *= scale;
    });
    group.rotation && rotateGroup(groupObj, group.rotation, group.edges);
  }

  group.children.forEach(function (row, index) {
    if (row.deleted === 0) {
      var rowLabelText = groups[group.id].groups[index].label;
      row.children.forEach(function (seat, seatIndex) {
        if (seat.deleted === 0) {
          var tierId = groups[group.id].groups[index].units[seatIndex].tier;
          var seatLabelText = groups[group.id].groups[index].units[seatIndex].label;
          var seatPosition = [seat.cx * scale, -seat.cy * scale, 0];
          var seatRotation = (seat.rotation || 0) + (row.rotation || 0) + (group.rotation || 0);

          if (!tierColorObj[tierId]) {
            tierColorObj[tierId] = TIER_COLORS[colorIndex.value];
            colorIndex.value++;
          }

          if (seat.type === 'unclickable_table') {
            drawUnclickableTable({
              seats: seats,
              materials: materials,
              textGeometries: textGeometries,
              seatId: seat.id,
              seat: seat,
              options: {
                color: COLORS.AVAILABLE,
                position: seatPosition,
                rotation: seatRotation,
                scene: scene,
                center_x: center_x,
                center_y: center_y,
                scale: scale,
                seat_circle_radius: seat_circle_radius,
                parent: groupObj,
                events: events,
                sectionLabel: groups[group.id].label,
                rowLabel: rowLabelText,
                label: seatLabelText,
                fonts: fonts,
                type: seat.type,
                size: seat.size,
                tableShape: seat.table_shape,
                sceneMeshes: sceneMeshes
              }
            });
            return;
          }

          drawSeat(seat.id, {
            color: predefinedSeats && predefinedSeats.includes(seat.id) ? COLORS.PREDEFINED : COLORS.AVAILABLE,
            position: seatPosition,
            rotation: seatRotation,
            tierId: tierId,
            scene: scene,
            center_x: center_x,
            center_y: center_y,
            scale: scale,
            seat_circle_radius: seat_circle_radius,
            parent: groupObj,
            events: events,
            sectionLabel: groups[group.id].label,
            rowLabel: rowLabelText,
            label: seatLabelText,
            fonts: fonts,
            type: seat.type,
            seats: seats,
            enableTicketTypeAssign: enableTicketTypeAssign,
            tierColor: tierColorObj[tierId],
            selectedTierId: selectedTierId,
            camera: camera,
            renderer: renderer,
            sceneMeshes: sceneMeshes
          });
        }
      });
    }
  });

  if (!group.hideSectionLabel) {
    var labelText = groups[group.id].label;
    var labelPosition = [group.label_x * scale, -group.label_y * scale, 0];
    drawLabel(labelText, {
      font: fonts.openSans,
      color: 'black',
      position: labelPosition,
      center_x: center_x,
      center_y: center_y,
      scale: scale,
      seat_circle_radius: seat_circle_radius,
      scene: groupObj,
      size: group.label_size || seat_circle_radius * scale,
      rotation: group.label_rotation || 0
    }, textGeometries, materials);
  }
};

var calculateCenterOfQuadrangle = function calculateCenterOfQuadrangle(points) {
  var _points$ = points[0],
      x1 = _points$.x,
      y1 = _points$.y,
      _points$2 = points[1],
      x4 = _points$2.x,
      y4 = _points$2.y,
      _points$3 = points[2],
      x2 = _points$3.x,
      y2 = _points$3.y,
      _points$4 = points[3],
      x3 = _points$4.x,
      y3 = _points$4.y;
  var t = ((x1 - x3) * (y3 - y4) - (y1 - y3) * (x3 - x4)) / ((x1 - x2) * (y3 - y4) - (y1 - y2) * (x3 - x4));
  return [x1 + t * (x2 - x1), y1 + t * (y2 - y1)];
};

var drawGroupBlock = function drawGroupBlock(group, groups, creatorInfo, options) {
  var scene = options.scene,
      scale = options.scale,
      fonts = options.fonts,
      center_x = options.center_x,
      center_y = options.center_y,
      seat_circle_radius = options.seat_circle_radius,
      _options$color = options.color,
      color = _options$color === void 0 ? COLORS.AVAILABLE : _options$color,
      events = options.events,
      sceneMeshes = options.sceneMeshes;
  var id = group.id,
      x = group.x,
      y = group.y;
  var edges = creatorInfo.groups[id].edges;
  var circularPoints = [new Vector3(edges.topLeft.x * scale, edges.topLeft.y * scale, 0), new Vector3(edges.topRight.x * scale, edges.topRight.y * scale, 0), new Vector3(edges.bottomRight.x * scale, edges.bottomRight.y * scale, 0), new Vector3(edges.bottomLeft.x * scale, edges.bottomLeft.y * scale, 0)];
  var geometry = new ShapeGeometry(new Shape(circularPoints));
  var material = getSeatMaterial({
    color: color
  }, materials);
  var groupObj = new Mesh(geometry, material);
  groupObj.position.set(x * scale, -y * scale, 0);
  scene.add(groupObj);

  var _calculateCenterOfQua = calculateCenterOfQuadrangle(circularPoints),
      centerX = _calculateCenterOfQua[0],
      centerY = _calculateCenterOfQua[1];

  var labelText = groups[id].label || '';
  var labelPosition = [centerX + x * scale, centerY - y * scale, 0];
  drawLabel(labelText, {
    font: fonts.openSans,
    color: 'black',
    position: labelPosition,
    center_x: center_x,
    center_y: center_y,
    scale: scale,
    seat_circle_radius: seat_circle_radius,
    scene: scene,
    size: 12
  }, textGeometries, materials);
  var lineBasicMaterial = getLineMaterial({
    color: color
  }, materials);
  var lineGeometry = new BufferGeometry().setFromPoints([].concat(circularPoints, [circularPoints[0]]));
  var line = new Line(lineGeometry, lineBasicMaterial);
  groupObj.border = line;
  groupObj.add(line);
  groupObj.click = events.onClick;
  groupObj.mouseOver = events.onMouseOver;
  groupObj.mouseOut = events.onMouseOut;
  groupObj.sgId = id;
  groupObj.tierId = groups[id].groups[0].units[0].tier;
  groupObj.label = groups[id].label;
  groupObj.availableCount = 0;
  groupObj.status = SEAT_STATUSES.BOOKED;
  groupObj.isSelected = false;

  groupObj.changeBorderColor = function (color) {
    groupObj.border && (groupObj.border.material = getLineMaterial({
      color: color
    }, materials));
  };

  groupObj.changeBorderColorToDefault = function () {
    return groupObj.changeBorderColor(color);
  };

  groupObj.select = function () {
    groupObj.isSelected = true;
    groupObj.changeBorderColor(COLORS.SELECTED_BLOCK_BORDER);
  };

  groupObj.unselect = function () {
    groupObj.isSelected = false;
    groupObj.changeBorderColorToDefault();
  };

  var makeAvailable = function makeAvailable() {
    !groupObj.isSelected && groupObj.changeBorderColor(color);
    groupObj.material = getSeatMaterial({
      color: color
    }, materials);
    groupObj.status = SEAT_STATUSES.AVAILABLE;
    groupObj.click = events.onClick;

    groupObj.changeBorderColorToDefault = function () {
      return groupObj.changeBorderColor(color);
    };
  };

  var makeBooked = function makeBooked() {
    groupObj.changeBorderColor(COLORS.BOOKED);
    groupObj.material = getSeatMaterial({
      color: COLORS.BOOKED
    }, materials);
    groupObj.status = SEAT_STATUSES.BOOKED;
    groupObj.click = null;

    groupObj.changeBorderColorToDefault = function () {
      return groupObj.changeBorderColor(COLORS.BOOKED);
    };
  };

  var makeBlocked = function makeBlocked() {
    groupObj.changeBorderColor(COLORS.BLOCKED);
    groupObj.material = getSeatMaterial({
      color: COLORS.BLOCKED
    }, materials);
    groupObj.status = SEAT_STATUSES.BLOCKED;
    groupObj.click = null;

    groupObj.changeBorderColorToDefault = function () {
      return groupObj.changeBorderColor(COLORS.BLOCKED);
    };
  };

  makeBooked();

  groupObj.statusUpdate = function (status, availableCount) {
    if (groupObj.availableCount !== availableCount) {
      groupObj.availableCount = availableCount;

      switch (status) {
        case SEAT_STATUSES.AVAILABLE:
          {
            makeAvailable();
            break;
          }

        case SEAT_STATUSES.BOOKED:
          {
            makeBooked();
            break;
          }

        case SEAT_STATUSES.BLOCKED:
          {
            makeBlocked();
            break;
          }
      }
    }
  };

  groupObj.hasHandlers = true;
  sceneMeshes.push(groupObj);
  blocks.push(groupObj);
};

var SVGLoader = function ( manager ) {

	Loader.call( this, manager );

	// Default dots per inch
	this.defaultDPI = 90;

	// Accepted units: 'mm', 'cm', 'in', 'pt', 'pc', 'px'
	this.defaultUnit = 'px';

};

SVGLoader.prototype = Object.assign( Object.create( Loader.prototype ), {

	constructor: SVGLoader,

	load: function ( url, onLoad, onProgress, onError ) {

		var scope = this;

		var loader = new FileLoader( scope.manager );
		loader.setPath( scope.path );
		loader.setRequestHeader( scope.requestHeader );
		loader.setWithCredentials( scope.withCredentials );
		loader.load( url, function ( text ) {

			try {

				onLoad( scope.parse( text ) );

			} catch ( e ) {

				if ( onError ) {

					onError( e );

				} else {

					console.error( e );

				}

				scope.manager.itemError( url );

			}

		}, onProgress, onError );

	},

	parse: function ( text ) {

		var scope = this;

		function parseNode( node, style ) {

			if ( node.nodeType !== 1 ) return;

			var transform = getNodeTransform( node );

			var traverseChildNodes = true;

			var path = null;

			switch ( node.nodeName ) {

				case 'svg':
					break;

				case 'style':
					parseCSSStylesheet( node );
					break;

				case 'g':
					style = parseStyle( node, style );
					break;

				case 'path':
					style = parseStyle( node, style );
					if ( node.hasAttribute( 'd' ) ) path = parsePathNode( node );
					break;

				case 'rect':
					style = parseStyle( node, style );
					path = parseRectNode( node );
					break;

				case 'polygon':
					style = parseStyle( node, style );
					path = parsePolygonNode( node );
					break;

				case 'polyline':
					style = parseStyle( node, style );
					path = parsePolylineNode( node );
					break;

				case 'circle':
					style = parseStyle( node, style );
					path = parseCircleNode( node );
					break;

				case 'ellipse':
					style = parseStyle( node, style );
					path = parseEllipseNode( node );
					break;

				case 'line':
					style = parseStyle( node, style );
					path = parseLineNode( node );
					break;

				case 'defs':
					traverseChildNodes = false;
					break;

				case 'use':
					style = parseStyle( node, style );
					var usedNodeId = node.href.baseVal.substring( 1 );
					var usedNode = node.viewportElement.getElementById( usedNodeId );
					if ( usedNode ) {

						parseNode( usedNode, style );

					} else {

						console.warn( 'SVGLoader: \'use node\' references non-existent node id: ' + usedNodeId );

					}

					break;
					// console.log( node );

			}

			if ( path ) {

				if ( style.fill !== undefined && style.fill !== 'none' ) {

					path.color.setStyle( style.fill );

				}

				transformPath( path, currentTransform );

				paths.push( path );

				path.userData = { node: node, style: style };

			}

			if ( traverseChildNodes ) {

				var nodes = node.childNodes;

				for ( var i = 0; i < nodes.length; i ++ ) {

					parseNode( nodes[ i ], style );

				}

			}

			if ( transform ) {

				transformStack.pop();

				if ( transformStack.length > 0 ) {

					currentTransform.copy( transformStack[ transformStack.length - 1 ] );

				} else {

					currentTransform.identity();

				}

			}

		}

		function parsePathNode( node ) {

			var path = new ShapePath();

			var point = new Vector2();
			var control = new Vector2();

			var firstPoint = new Vector2();
			var isFirstPoint = true;
			var doSetFirstPoint = false;

			var d = node.getAttribute( 'd' );

			// console.log( d );

			var commands = d.match( /[a-df-z][^a-df-z]*/ig );

			for ( var i = 0, l = commands.length; i < l; i ++ ) {

				var command = commands[ i ];

				var type = command.charAt( 0 );
				var data = command.substr( 1 ).trim();

				if ( isFirstPoint === true ) {

					doSetFirstPoint = true;
					isFirstPoint = false;

				}

				switch ( type ) {

					case 'M':
						var numbers = parseFloats( data );
						for ( var j = 0, jl = numbers.length; j < jl; j += 2 ) {

							point.x = numbers[ j + 0 ];
							point.y = numbers[ j + 1 ];
							control.x = point.x;
							control.y = point.y;

							if ( j === 0 ) {

								path.moveTo( point.x, point.y );

							} else {

								path.lineTo( point.x, point.y );

							}

							if ( j === 0 && doSetFirstPoint === true ) firstPoint.copy( point );

						}

						break;

					case 'H':
						var numbers = parseFloats( data );

						for ( var j = 0, jl = numbers.length; j < jl; j ++ ) {

							point.x = numbers[ j ];
							control.x = point.x;
							control.y = point.y;
							path.lineTo( point.x, point.y );

							if ( j === 0 && doSetFirstPoint === true ) firstPoint.copy( point );

						}

						break;

					case 'V':
						var numbers = parseFloats( data );

						for ( var j = 0, jl = numbers.length; j < jl; j ++ ) {

							point.y = numbers[ j ];
							control.x = point.x;
							control.y = point.y;
							path.lineTo( point.x, point.y );

							if ( j === 0 && doSetFirstPoint === true ) firstPoint.copy( point );

						}

						break;

					case 'L':
						var numbers = parseFloats( data );

						for ( var j = 0, jl = numbers.length; j < jl; j += 2 ) {

							point.x = numbers[ j + 0 ];
							point.y = numbers[ j + 1 ];
							control.x = point.x;
							control.y = point.y;
							path.lineTo( point.x, point.y );

							if ( j === 0 && doSetFirstPoint === true ) firstPoint.copy( point );

						}

						break;

					case 'C':
						var numbers = parseFloats( data );

						for ( var j = 0, jl = numbers.length; j < jl; j += 6 ) {

							path.bezierCurveTo(
								numbers[ j + 0 ],
								numbers[ j + 1 ],
								numbers[ j + 2 ],
								numbers[ j + 3 ],
								numbers[ j + 4 ],
								numbers[ j + 5 ]
							);
							control.x = numbers[ j + 2 ];
							control.y = numbers[ j + 3 ];
							point.x = numbers[ j + 4 ];
							point.y = numbers[ j + 5 ];

							if ( j === 0 && doSetFirstPoint === true ) firstPoint.copy( point );

						}

						break;

					case 'S':
						var numbers = parseFloats( data );

						for ( var j = 0, jl = numbers.length; j < jl; j += 4 ) {

							path.bezierCurveTo(
								getReflection( point.x, control.x ),
								getReflection( point.y, control.y ),
								numbers[ j + 0 ],
								numbers[ j + 1 ],
								numbers[ j + 2 ],
								numbers[ j + 3 ]
							);
							control.x = numbers[ j + 0 ];
							control.y = numbers[ j + 1 ];
							point.x = numbers[ j + 2 ];
							point.y = numbers[ j + 3 ];

							if ( j === 0 && doSetFirstPoint === true ) firstPoint.copy( point );

						}

						break;

					case 'Q':
						var numbers = parseFloats( data );

						for ( var j = 0, jl = numbers.length; j < jl; j += 4 ) {

							path.quadraticCurveTo(
								numbers[ j + 0 ],
								numbers[ j + 1 ],
								numbers[ j + 2 ],
								numbers[ j + 3 ]
							);
							control.x = numbers[ j + 0 ];
							control.y = numbers[ j + 1 ];
							point.x = numbers[ j + 2 ];
							point.y = numbers[ j + 3 ];

							if ( j === 0 && doSetFirstPoint === true ) firstPoint.copy( point );

						}

						break;

					case 'T':
						var numbers = parseFloats( data );

						for ( var j = 0, jl = numbers.length; j < jl; j += 2 ) {

							var rx = getReflection( point.x, control.x );
							var ry = getReflection( point.y, control.y );
							path.quadraticCurveTo(
								rx,
								ry,
								numbers[ j + 0 ],
								numbers[ j + 1 ]
							);
							control.x = rx;
							control.y = ry;
							point.x = numbers[ j + 0 ];
							point.y = numbers[ j + 1 ];

							if ( j === 0 && doSetFirstPoint === true ) firstPoint.copy( point );

						}

						break;

					case 'A':
						var numbers = parseFloats( data, [ 3, 4 ], 7 );

						for ( var j = 0, jl = numbers.length; j < jl; j += 7 ) {

							// skip command if start point == end point
							if ( numbers[ j + 5 ] == point.x && numbers[ j + 6 ] == point.y ) continue;

							var start = point.clone();
							point.x = numbers[ j + 5 ];
							point.y = numbers[ j + 6 ];
							control.x = point.x;
							control.y = point.y;
							parseArcCommand(
								path, numbers[ j ], numbers[ j + 1 ], numbers[ j + 2 ], numbers[ j + 3 ], numbers[ j + 4 ], start, point
							);

							if ( j === 0 && doSetFirstPoint === true ) firstPoint.copy( point );

						}

						break;

					case 'm':
						var numbers = parseFloats( data );

						for ( var j = 0, jl = numbers.length; j < jl; j += 2 ) {

							point.x += numbers[ j + 0 ];
							point.y += numbers[ j + 1 ];
							control.x = point.x;
							control.y = point.y;

							if ( j === 0 ) {

								path.moveTo( point.x, point.y );

							} else {

								path.lineTo( point.x, point.y );

							}

							if ( j === 0 && doSetFirstPoint === true ) firstPoint.copy( point );

						}

						break;

					case 'h':
						var numbers = parseFloats( data );

						for ( var j = 0, jl = numbers.length; j < jl; j ++ ) {

							point.x += numbers[ j ];
							control.x = point.x;
							control.y = point.y;
							path.lineTo( point.x, point.y );

							if ( j === 0 && doSetFirstPoint === true ) firstPoint.copy( point );

						}

						break;

					case 'v':
						var numbers = parseFloats( data );

						for ( var j = 0, jl = numbers.length; j < jl; j ++ ) {

							point.y += numbers[ j ];
							control.x = point.x;
							control.y = point.y;
							path.lineTo( point.x, point.y );

							if ( j === 0 && doSetFirstPoint === true ) firstPoint.copy( point );

						}

						break;

					case 'l':
						var numbers = parseFloats( data );

						for ( var j = 0, jl = numbers.length; j < jl; j += 2 ) {

							point.x += numbers[ j + 0 ];
							point.y += numbers[ j + 1 ];
							control.x = point.x;
							control.y = point.y;
							path.lineTo( point.x, point.y );

							if ( j === 0 && doSetFirstPoint === true ) firstPoint.copy( point );

						}

						break;

					case 'c':
						var numbers = parseFloats( data );

						for ( var j = 0, jl = numbers.length; j < jl; j += 6 ) {

							path.bezierCurveTo(
								point.x + numbers[ j + 0 ],
								point.y + numbers[ j + 1 ],
								point.x + numbers[ j + 2 ],
								point.y + numbers[ j + 3 ],
								point.x + numbers[ j + 4 ],
								point.y + numbers[ j + 5 ]
							);
							control.x = point.x + numbers[ j + 2 ];
							control.y = point.y + numbers[ j + 3 ];
							point.x += numbers[ j + 4 ];
							point.y += numbers[ j + 5 ];

							if ( j === 0 && doSetFirstPoint === true ) firstPoint.copy( point );

						}

						break;

					case 's':
						var numbers = parseFloats( data );

						for ( var j = 0, jl = numbers.length; j < jl; j += 4 ) {

							path.bezierCurveTo(
								getReflection( point.x, control.x ),
								getReflection( point.y, control.y ),
								point.x + numbers[ j + 0 ],
								point.y + numbers[ j + 1 ],
								point.x + numbers[ j + 2 ],
								point.y + numbers[ j + 3 ]
							);
							control.x = point.x + numbers[ j + 0 ];
							control.y = point.y + numbers[ j + 1 ];
							point.x += numbers[ j + 2 ];
							point.y += numbers[ j + 3 ];

							if ( j === 0 && doSetFirstPoint === true ) firstPoint.copy( point );

						}

						break;

					case 'q':
						var numbers = parseFloats( data );

						for ( var j = 0, jl = numbers.length; j < jl; j += 4 ) {

							path.quadraticCurveTo(
								point.x + numbers[ j + 0 ],
								point.y + numbers[ j + 1 ],
								point.x + numbers[ j + 2 ],
								point.y + numbers[ j + 3 ]
							);
							control.x = point.x + numbers[ j + 0 ];
							control.y = point.y + numbers[ j + 1 ];
							point.x += numbers[ j + 2 ];
							point.y += numbers[ j + 3 ];

							if ( j === 0 && doSetFirstPoint === true ) firstPoint.copy( point );

						}

						break;

					case 't':
						var numbers = parseFloats( data );

						for ( var j = 0, jl = numbers.length; j < jl; j += 2 ) {

							var rx = getReflection( point.x, control.x );
							var ry = getReflection( point.y, control.y );
							path.quadraticCurveTo(
								rx,
								ry,
								point.x + numbers[ j + 0 ],
								point.y + numbers[ j + 1 ]
							);
							control.x = rx;
							control.y = ry;
							point.x = point.x + numbers[ j + 0 ];
							point.y = point.y + numbers[ j + 1 ];

							if ( j === 0 && doSetFirstPoint === true ) firstPoint.copy( point );

						}

						break;

					case 'a':
						var numbers = parseFloats( data, [ 3, 4 ], 7 );

						for ( var j = 0, jl = numbers.length; j < jl; j += 7 ) {

							// skip command if no displacement
							if ( numbers[ j + 5 ] == 0 && numbers[ j + 6 ] == 0 ) continue;

							var start = point.clone();
							point.x += numbers[ j + 5 ];
							point.y += numbers[ j + 6 ];
							control.x = point.x;
							control.y = point.y;
							parseArcCommand(
								path, numbers[ j ], numbers[ j + 1 ], numbers[ j + 2 ], numbers[ j + 3 ], numbers[ j + 4 ], start, point
							);

							if ( j === 0 && doSetFirstPoint === true ) firstPoint.copy( point );

						}

						break;

					case 'Z':
					case 'z':
						path.currentPath.autoClose = true;

						if ( path.currentPath.curves.length > 0 ) {

							// Reset point to beginning of Path
							point.copy( firstPoint );
							path.currentPath.currentPoint.copy( point );
							isFirstPoint = true;

						}

						break;

					default:
						console.warn( command );

				}

				// console.log( type, parseFloats( data ), parseFloats( data ).length  )

				doSetFirstPoint = false;

			}

			return path;

		}

		function parseCSSStylesheet( node ) {

			if ( ! node.sheet || ! node.sheet.cssRules || ! node.sheet.cssRules.length ) return;

			for ( var i = 0; i < node.sheet.cssRules.length; i ++ ) {

				var stylesheet = node.sheet.cssRules[ i ];

				if ( stylesheet.type !== 1 ) continue;

				var selectorList = stylesheet.selectorText
					.split( /,/gm )
					.filter( Boolean )
					.map( i => i.trim() );

				for ( var j = 0; j < selectorList.length; j ++ ) {

					stylesheets[ selectorList[ j ] ] = Object.assign(
						stylesheets[ selectorList[ j ] ] || {},
						stylesheet.style
					);

				}

			}

		}

		/**
		 * https://www.w3.org/TR/SVG/implnote.html#ArcImplementationNotes
		 * https://mortoray.com/2017/02/16/rendering-an-svg-elliptical-arc-as-bezier-curves/ Appendix: Endpoint to center arc conversion
		 * From
		 * rx ry x-axis-rotation large-arc-flag sweep-flag x y
		 * To
		 * aX, aY, xRadius, yRadius, aStartAngle, aEndAngle, aClockwise, aRotation
		 */

		function parseArcCommand( path, rx, ry, x_axis_rotation, large_arc_flag, sweep_flag, start, end ) {

			if ( rx == 0 || ry == 0 ) {

				// draw a line if either of the radii == 0
				path.lineTo( end.x, end.y );
				return;

			}

			x_axis_rotation = x_axis_rotation * Math.PI / 180;

			// Ensure radii are positive
			rx = Math.abs( rx );
			ry = Math.abs( ry );

			// Compute (x1', y1')
			var dx2 = ( start.x - end.x ) / 2.0;
			var dy2 = ( start.y - end.y ) / 2.0;
			var x1p = Math.cos( x_axis_rotation ) * dx2 + Math.sin( x_axis_rotation ) * dy2;
			var y1p = - Math.sin( x_axis_rotation ) * dx2 + Math.cos( x_axis_rotation ) * dy2;

			// Compute (cx', cy')
			var rxs = rx * rx;
			var rys = ry * ry;
			var x1ps = x1p * x1p;
			var y1ps = y1p * y1p;

			// Ensure radii are large enough
			var cr = x1ps / rxs + y1ps / rys;

			if ( cr > 1 ) {

				// scale up rx,ry equally so cr == 1
				var s = Math.sqrt( cr );
				rx = s * rx;
				ry = s * ry;
				rxs = rx * rx;
				rys = ry * ry;

			}

			var dq = ( rxs * y1ps + rys * x1ps );
			var pq = ( rxs * rys - dq ) / dq;
			var q = Math.sqrt( Math.max( 0, pq ) );
			if ( large_arc_flag === sweep_flag ) q = - q;
			var cxp = q * rx * y1p / ry;
			var cyp = - q * ry * x1p / rx;

			// Step 3: Compute (cx, cy) from (cx', cy')
			var cx = Math.cos( x_axis_rotation ) * cxp - Math.sin( x_axis_rotation ) * cyp + ( start.x + end.x ) / 2;
			var cy = Math.sin( x_axis_rotation ) * cxp + Math.cos( x_axis_rotation ) * cyp + ( start.y + end.y ) / 2;

			// Step 4: Compute θ1 and Δθ
			var theta = svgAngle( 1, 0, ( x1p - cxp ) / rx, ( y1p - cyp ) / ry );
			var delta = svgAngle( ( x1p - cxp ) / rx, ( y1p - cyp ) / ry, ( - x1p - cxp ) / rx, ( - y1p - cyp ) / ry ) % ( Math.PI * 2 );

			path.currentPath.absellipse( cx, cy, rx, ry, theta, theta + delta, sweep_flag === 0, x_axis_rotation );

		}

		function svgAngle( ux, uy, vx, vy ) {

			var dot = ux * vx + uy * vy;
			var len = Math.sqrt( ux * ux + uy * uy ) * Math.sqrt( vx * vx + vy * vy );
			var ang = Math.acos( Math.max( - 1, Math.min( 1, dot / len ) ) ); // floating point precision, slightly over values appear
			if ( ( ux * vy - uy * vx ) < 0 ) ang = - ang;
			return ang;

		}

		/*
		* According to https://www.w3.org/TR/SVG/shapes.html#RectElementRXAttribute
		* rounded corner should be rendered to elliptical arc, but bezier curve does the job well enough
		*/
		function parseRectNode( node ) {

			var x = parseFloatWithUnits( node.getAttribute( 'x' ) || 0 );
			var y = parseFloatWithUnits( node.getAttribute( 'y' ) || 0 );
			var rx = parseFloatWithUnits( node.getAttribute( 'rx' ) || 0 );
			var ry = parseFloatWithUnits( node.getAttribute( 'ry' ) || 0 );
			var w = parseFloatWithUnits( node.getAttribute( 'width' ) );
			var h = parseFloatWithUnits( node.getAttribute( 'height' ) );

			var path = new ShapePath();
			path.moveTo( x + 2 * rx, y );
			path.lineTo( x + w - 2 * rx, y );
			if ( rx !== 0 || ry !== 0 ) path.bezierCurveTo( x + w, y, x + w, y, x + w, y + 2 * ry );
			path.lineTo( x + w, y + h - 2 * ry );
			if ( rx !== 0 || ry !== 0 ) path.bezierCurveTo( x + w, y + h, x + w, y + h, x + w - 2 * rx, y + h );
			path.lineTo( x + 2 * rx, y + h );

			if ( rx !== 0 || ry !== 0 ) {

				path.bezierCurveTo( x, y + h, x, y + h, x, y + h - 2 * ry );

			}

			path.lineTo( x, y + 2 * ry );

			if ( rx !== 0 || ry !== 0 ) {

				path.bezierCurveTo( x, y, x, y, x + 2 * rx, y );

			}

			return path;

		}

		function parsePolygonNode( node ) {

			function iterator( match, a, b ) {

				var x = parseFloatWithUnits( a );
				var y = parseFloatWithUnits( b );

				if ( index === 0 ) {

					path.moveTo( x, y );

				} else {

					path.lineTo( x, y );

				}

				index ++;

			}

			var regex = /(-?[\d\.?]+)[,|\s](-?[\d\.?]+)/g;

			var path = new ShapePath();

			var index = 0;

			node.getAttribute( 'points' ).replace( regex, iterator );

			path.currentPath.autoClose = true;

			return path;

		}

		function parsePolylineNode( node ) {

			function iterator( match, a, b ) {

				var x = parseFloatWithUnits( a );
				var y = parseFloatWithUnits( b );

				if ( index === 0 ) {

					path.moveTo( x, y );

				} else {

					path.lineTo( x, y );

				}

				index ++;

			}

			var regex = /(-?[\d\.?]+)[,|\s](-?[\d\.?]+)/g;

			var path = new ShapePath();

			var index = 0;

			node.getAttribute( 'points' ).replace( regex, iterator );

			path.currentPath.autoClose = false;

			return path;

		}

		function parseCircleNode( node ) {

			var x = parseFloatWithUnits( node.getAttribute( 'cx' ) || 0 );
			var y = parseFloatWithUnits( node.getAttribute( 'cy' ) || 0 );
			var r = parseFloatWithUnits( node.getAttribute( 'r' ) || 0 );

			var subpath = new Path();
			subpath.absarc( x, y, r, 0, Math.PI * 2 );

			var path = new ShapePath();
			path.subPaths.push( subpath );

			return path;

		}

		function parseEllipseNode( node ) {

			var x = parseFloatWithUnits( node.getAttribute( 'cx' ) || 0 );
			var y = parseFloatWithUnits( node.getAttribute( 'cy' ) || 0 );
			var rx = parseFloatWithUnits( node.getAttribute( 'rx' ) || 0 );
			var ry = parseFloatWithUnits( node.getAttribute( 'ry' ) || 0 );

			var subpath = new Path();
			subpath.absellipse( x, y, rx, ry, 0, Math.PI * 2 );

			var path = new ShapePath();
			path.subPaths.push( subpath );

			return path;

		}

		function parseLineNode( node ) {

			var x1 = parseFloatWithUnits( node.getAttribute( 'x1' ) || 0 );
			var y1 = parseFloatWithUnits( node.getAttribute( 'y1' ) || 0 );
			var x2 = parseFloatWithUnits( node.getAttribute( 'x2' ) || 0 );
			var y2 = parseFloatWithUnits( node.getAttribute( 'y2' ) || 0 );

			var path = new ShapePath();
			path.moveTo( x1, y1 );
			path.lineTo( x2, y2 );
			path.currentPath.autoClose = false;

			return path;

		}

		//

		function parseStyle( node, style ) {

			style = Object.assign( {}, style ); // clone style

			var stylesheetStyles = {};

			if ( node.hasAttribute( 'class' ) ) {

				var classSelectors = node.getAttribute( 'class' )
					.split( /\s/ )
					.filter( Boolean )
					.map( i => i.trim() );

				for ( var i = 0; i < classSelectors.length; i ++ ) {

					stylesheetStyles = Object.assign( stylesheetStyles, stylesheets[ '.' + classSelectors[ i ] ] );

				}

			}

			if ( node.hasAttribute( 'id' ) ) {

				stylesheetStyles = Object.assign( stylesheetStyles, stylesheets[ '#' + node.getAttribute( 'id' ) ] );

			}

			function addStyle( svgName, jsName, adjustFunction ) {

				if ( adjustFunction === undefined ) adjustFunction = function copy( v ) {

					if ( v.startsWith( 'url' ) ) console.warn( 'SVGLoader: url access in attributes is not implemented.' );

					return v;

				};

				if ( node.hasAttribute( svgName ) ) style[ jsName ] = adjustFunction( node.getAttribute( svgName ) );
				if ( stylesheetStyles[ svgName ] ) style[ jsName ] = adjustFunction( stylesheetStyles[ svgName ] );
				if ( node.style && node.style[ svgName ] !== '' ) style[ jsName ] = adjustFunction( node.style[ svgName ] );

			}

			function clamp( v ) {

				return Math.max( 0, Math.min( 1, parseFloatWithUnits( v ) ) );

			}

			function positive( v ) {

				return Math.max( 0, parseFloatWithUnits( v ) );

			}

			addStyle( 'fill', 'fill' );
			addStyle( 'fill-opacity', 'fillOpacity', clamp );
			addStyle( 'opacity', 'opacity', clamp );
			addStyle( 'stroke', 'stroke' );
			addStyle( 'stroke-opacity', 'strokeOpacity', clamp );
			addStyle( 'stroke-width', 'strokeWidth', positive );
			addStyle( 'stroke-linejoin', 'strokeLineJoin' );
			addStyle( 'stroke-linecap', 'strokeLineCap' );
			addStyle( 'stroke-miterlimit', 'strokeMiterLimit', positive );
			addStyle( 'visibility', 'visibility' );

			return style;

		}

		// http://www.w3.org/TR/SVG11/implnote.html#PathElementImplementationNotes

		function getReflection( a, b ) {

			return a - ( b - a );

		}

		// from https://github.com/ppvg/svg-numbers (MIT License)

		function parseFloats( input, flags, stride ) {

			if ( typeof input !== 'string' ) {

				throw new TypeError( 'Invalid input: ' + typeof input );

			}

			// Character groups
			var RE = {
				SEPARATOR: /[ \t\r\n\,.\-+]/,
				WHITESPACE: /[ \t\r\n]/,
				DIGIT: /[\d]/,
				SIGN: /[-+]/,
				POINT: /\./,
				COMMA: /,/,
				EXP: /e/i,
				FLAGS: /[01]/
			};

			// States
			var SEP = 0;
			var INT = 1;
			var FLOAT = 2;
			var EXP = 3;

			var state = SEP;
			var seenComma = true;
			var result = [], number = '', exponent = '';

			function throwSyntaxError( current, i, partial ) {

				var error = new SyntaxError( 'Unexpected character "' + current + '" at index ' + i + '.' );
				error.partial = partial;
				throw error;

			}

			function newNumber() {

				if ( number !== '' ) {

					if ( exponent === '' ) result.push( Number( number ) );
					else result.push( Number( number ) * Math.pow( 10, Number( exponent ) ) );

				}

				number = '';
				exponent = '';

			}

			var current, i = 0, length = input.length;
			for ( i = 0; i < length; i ++ ) {

				current = input[ i ];

				// check for flags
				if ( Array.isArray( flags ) && flags.includes( result.length % stride ) && RE.FLAGS.test( current ) ) {

					state = INT;
					number = current;
					newNumber();
					continue;

				}

				// parse until next number
				if ( state === SEP ) {

					// eat whitespace
					if ( RE.WHITESPACE.test( current ) ) {

						continue;

					}

					// start new number
					if ( RE.DIGIT.test( current ) || RE.SIGN.test( current ) ) {

						state = INT;
						number = current;
						continue;

					}

					if ( RE.POINT.test( current ) ) {

						state = FLOAT;
						number = current;
						continue;

					}

					// throw on double commas (e.g. "1, , 2")
					if ( RE.COMMA.test( current ) ) {

						if ( seenComma ) {

							throwSyntaxError( current, i, result );

						}

						seenComma = true;

					}

				}

				// parse integer part
				if ( state === INT ) {

					if ( RE.DIGIT.test( current ) ) {

						number += current;
						continue;

					}

					if ( RE.POINT.test( current ) ) {

						number += current;
						state = FLOAT;
						continue;

					}

					if ( RE.EXP.test( current ) ) {

						state = EXP;
						continue;

					}

					// throw on double signs ("-+1"), but not on sign as separator ("-1-2")
					if ( RE.SIGN.test( current )
							&& number.length === 1
							&& RE.SIGN.test( number[ 0 ] ) ) {

						throwSyntaxError( current, i, result );

					}

				}

				// parse decimal part
				if ( state === FLOAT ) {

					if ( RE.DIGIT.test( current ) ) {

						number += current;
						continue;

					}

					if ( RE.EXP.test( current ) ) {

						state = EXP;
						continue;

					}

					// throw on double decimal points (e.g. "1..2")
					if ( RE.POINT.test( current ) && number[ number.length - 1 ] === '.' ) {

						throwSyntaxError( current, i, result );

					}

				}

				// parse exponent part
				if ( state === EXP ) {

					if ( RE.DIGIT.test( current ) ) {

						exponent += current;
						continue;

					}

					if ( RE.SIGN.test( current ) ) {

						if ( exponent === '' ) {

							exponent += current;
							continue;

						}

						if ( exponent.length === 1 && RE.SIGN.test( exponent ) ) {

							throwSyntaxError( current, i, result );

						}

					}

				}


				// end of number
				if ( RE.WHITESPACE.test( current ) ) {

					newNumber();
					state = SEP;
					seenComma = false;

				} else if ( RE.COMMA.test( current ) ) {

					newNumber();
					state = SEP;
					seenComma = true;

				} else if ( RE.SIGN.test( current ) ) {

					newNumber();
					state = INT;
					number = current;

				} else if ( RE.POINT.test( current ) ) {

					newNumber();
					state = FLOAT;
					number = current;

				} else {

					throwSyntaxError( current, i, result );

				}

			}

			// add the last number found (if any)
			newNumber();

			return result;

		}

		// Units

		var units = [ 'mm', 'cm', 'in', 'pt', 'pc', 'px' ];

		// Conversion: [ fromUnit ][ toUnit ] (-1 means dpi dependent)
		var unitConversion = {

			'mm': {
				'mm': 1,
				'cm': 0.1,
				'in': 1 / 25.4,
				'pt': 72 / 25.4,
				'pc': 6 / 25.4,
				'px': - 1
			},
			'cm': {
				'mm': 10,
				'cm': 1,
				'in': 1 / 2.54,
				'pt': 72 / 2.54,
				'pc': 6 / 2.54,
				'px': - 1
			},
			'in': {
				'mm': 25.4,
				'cm': 2.54,
				'in': 1,
				'pt': 72,
				'pc': 6,
				'px': - 1
			},
			'pt': {
				'mm': 25.4 / 72,
				'cm': 2.54 / 72,
				'in': 1 / 72,
				'pt': 1,
				'pc': 6 / 72,
				'px': - 1
			},
			'pc': {
				'mm': 25.4 / 6,
				'cm': 2.54 / 6,
				'in': 1 / 6,
				'pt': 72 / 6,
				'pc': 1,
				'px': - 1
			},
			'px': {
				'px': 1
			}

		};

		function parseFloatWithUnits( string ) {

			var theUnit = 'px';

			if ( typeof string === 'string' || string instanceof String ) {

				for ( var i = 0, n = units.length; i < n; i ++ ) {

					var u = units[ i ];

					if ( string.endsWith( u ) ) {

						theUnit = u;
						string = string.substring( 0, string.length - u.length );
						break;

					}

				}

			}

			var scale = undefined;

			if ( theUnit === 'px' && scope.defaultUnit !== 'px' ) {

				// Conversion scale from  pixels to inches, then to default units

				scale = unitConversion[ 'in' ][ scope.defaultUnit ] / scope.defaultDPI;

			} else {

				scale = unitConversion[ theUnit ][ scope.defaultUnit ];

				if ( scale < 0 ) {

					// Conversion scale to pixels

					scale = unitConversion[ theUnit ][ 'in' ] * scope.defaultDPI;

				}

			}

			return scale * parseFloat( string );

		}

		// Transforms

		function getNodeTransform( node ) {

			if ( ! ( node.hasAttribute( 'transform' ) || ( node.nodeName === 'use' && ( node.hasAttribute( 'x' ) || node.hasAttribute( 'y' ) ) ) ) ) {

				return null;

			}

			var transform = parseNodeTransform( node );

			if ( transformStack.length > 0 ) {

				transform.premultiply( transformStack[ transformStack.length - 1 ] );

			}

			currentTransform.copy( transform );
			transformStack.push( transform );

			return transform;

		}

		function parseNodeTransform( node ) {

			var transform = new Matrix3();
			var currentTransform = tempTransform0;

			if ( node.nodeName === 'use' && ( node.hasAttribute( 'x' ) || node.hasAttribute( 'y' ) ) ) {

				var tx = parseFloatWithUnits( node.getAttribute( 'x' ) );
				var ty = parseFloatWithUnits( node.getAttribute( 'y' ) );

				transform.translate( tx, ty );

			}

			if ( node.hasAttribute( 'transform' ) ) {

				var transformsTexts = node.getAttribute( 'transform' ).split( ')' );

				for ( var tIndex = transformsTexts.length - 1; tIndex >= 0; tIndex -- ) {

					var transformText = transformsTexts[ tIndex ].trim();

					if ( transformText === '' ) continue;

					var openParPos = transformText.indexOf( '(' );
					var closeParPos = transformText.length;

					if ( openParPos > 0 && openParPos < closeParPos ) {

						var transformType = transformText.substr( 0, openParPos );

						var array = parseFloats( transformText.substr( openParPos + 1, closeParPos - openParPos - 1 ) );

						currentTransform.identity();

						switch ( transformType ) {

							case 'translate':

								if ( array.length >= 1 ) {

									var tx = array[ 0 ];
									var ty = tx;

									if ( array.length >= 2 ) {

										ty = array[ 1 ];

									}

									currentTransform.translate( tx, ty );

								}

								break;

							case 'rotate':

								if ( array.length >= 1 ) {

									var angle = 0;
									var cx = 0;
									var cy = 0;

									// Angle
									angle = - array[ 0 ] * Math.PI / 180;

									if ( array.length >= 3 ) {

										// Center x, y
										cx = array[ 1 ];
										cy = array[ 2 ];

									}

									// Rotate around center (cx, cy)
									tempTransform1.identity().translate( - cx, - cy );
									tempTransform2.identity().rotate( angle );
									tempTransform3.multiplyMatrices( tempTransform2, tempTransform1 );
									tempTransform1.identity().translate( cx, cy );
									currentTransform.multiplyMatrices( tempTransform1, tempTransform3 );

								}

								break;

							case 'scale':

								if ( array.length >= 1 ) {

									var scaleX = array[ 0 ];
									var scaleY = scaleX;

									if ( array.length >= 2 ) {

										scaleY = array[ 1 ];

									}

									currentTransform.scale( scaleX, scaleY );

								}

								break;

							case 'skewX':

								if ( array.length === 1 ) {

									currentTransform.set(
										1, Math.tan( array[ 0 ] * Math.PI / 180 ), 0,
										0, 1, 0,
										0, 0, 1
									);

								}

								break;

							case 'skewY':

								if ( array.length === 1 ) {

									currentTransform.set(
										1, 0, 0,
										Math.tan( array[ 0 ] * Math.PI / 180 ), 1, 0,
										0, 0, 1
									);

								}

								break;

							case 'matrix':

								if ( array.length === 6 ) {

									currentTransform.set(
										array[ 0 ], array[ 2 ], array[ 4 ],
										array[ 1 ], array[ 3 ], array[ 5 ],
										0, 0, 1
									);

								}

								break;

						}

					}

					transform.premultiply( currentTransform );

				}

			}

			return transform;

		}

		function transformPath( path, m ) {

			function transfVec2( v2 ) {

				tempV3.set( v2.x, v2.y, 1 ).applyMatrix3( m );

				v2.set( tempV3.x, tempV3.y );

			}

			var isRotated = isTransformRotated( m );

			var subPaths = path.subPaths;

			for ( var i = 0, n = subPaths.length; i < n; i ++ ) {

				var subPath = subPaths[ i ];
				var curves = subPath.curves;

				for ( var j = 0; j < curves.length; j ++ ) {

					var curve = curves[ j ];

					if ( curve.isLineCurve ) {

						transfVec2( curve.v1 );
						transfVec2( curve.v2 );

					} else if ( curve.isCubicBezierCurve ) {

						transfVec2( curve.v0 );
						transfVec2( curve.v1 );
						transfVec2( curve.v2 );
						transfVec2( curve.v3 );

					} else if ( curve.isQuadraticBezierCurve ) {

						transfVec2( curve.v0 );
						transfVec2( curve.v1 );
						transfVec2( curve.v2 );

					} else if ( curve.isEllipseCurve ) {

						if ( isRotated ) {

							console.warn( 'SVGLoader: Elliptic arc or ellipse rotation or skewing is not implemented.' );

						}

						tempV2.set( curve.aX, curve.aY );
						transfVec2( tempV2 );
						curve.aX = tempV2.x;
						curve.aY = tempV2.y;

						curve.xRadius *= getTransformScaleX( m );
						curve.yRadius *= getTransformScaleY( m );

					}

				}

			}

		}

		function isTransformRotated( m ) {

			return m.elements[ 1 ] !== 0 || m.elements[ 3 ] !== 0;

		}

		function getTransformScaleX( m ) {

			var te = m.elements;
			return Math.sqrt( te[ 0 ] * te[ 0 ] + te[ 1 ] * te[ 1 ] );

		}

		function getTransformScaleY( m ) {

			var te = m.elements;
			return Math.sqrt( te[ 3 ] * te[ 3 ] + te[ 4 ] * te[ 4 ] );

		}

		//

		var paths = [];
		var stylesheets = {};

		var transformStack = [];

		var tempTransform0 = new Matrix3();
		var tempTransform1 = new Matrix3();
		var tempTransform2 = new Matrix3();
		var tempTransform3 = new Matrix3();
		var tempV2 = new Vector2();
		var tempV3 = new Vector3();

		var currentTransform = new Matrix3();

		var xml = new DOMParser().parseFromString( text, 'image/svg+xml' ); // application/xml

		parseNode( xml.documentElement, {
			fill: '#000',
			fillOpacity: 1,
			strokeOpacity: 1,
			strokeWidth: 1,
			strokeLineJoin: 'miter',
			strokeLineCap: 'butt',
			strokeMiterLimit: 4
		} );

		var data = { paths: paths, xml: xml.documentElement };

		// console.log( paths );
		return data;

	}

} );

SVGLoader.createShapes = function ( shapePath ) {

	// Param shapePath: a shapepath as returned by the parse function of this class
	// Returns Shape object

	const BIGNUMBER = 999999999;

	const IntersectionLocationType = {
		ORIGIN: 0,
		DESTINATION: 1,
		BETWEEN: 2,
		LEFT: 3,
		RIGHT: 4,
		BEHIND: 5,
		BEYOND: 6
	};

	const classifyResult = {
		loc: IntersectionLocationType.ORIGIN,
		t: 0
	};

	function findEdgeIntersection( a0, a1, b0, b1 ) {

		var x1 = a0.x;
		var x2 = a1.x;
		var x3 = b0.x;
		var x4 = b1.x;
		var y1 = a0.y;
		var y2 = a1.y;
		var y3 = b0.y;
		var y4 = b1.y;
		var nom1 = ( x4 - x3 ) * ( y1 - y3 ) - ( y4 - y3 ) * ( x1 - x3 );
		var nom2 = ( x2 - x1 ) * ( y1 - y3 ) - ( y2 - y1 ) * ( x1 - x3 );
		var denom = ( y4 - y3 ) * ( x2 - x1 ) - ( x4 - x3 ) * ( y2 - y1 );
		var t1 = nom1 / denom;
		var t2 = nom2 / denom;

		if ( ( ( denom === 0 ) && ( nom1 !== 0 ) ) || ( t1 <= 0 ) || ( t1 >= 1 ) || ( t2 < 0 ) || ( t2 > 1 ) ) {

			//1. lines are parallel or edges don't intersect

			return null;

		} else if ( ( nom1 === 0 ) && ( denom === 0 ) ) {

			//2. lines are colinear

			//check if endpoints of edge2 (b0-b1) lies on edge1 (a0-a1)
			for ( var i = 0; i < 2; i ++ ) {

				classifyPoint( i === 0 ? b0 : b1, a0, a1 );
				//find position of this endpoints relatively to edge1
				if ( classifyResult.loc == IntersectionLocationType.ORIGIN ) {

					var point = ( i === 0 ? b0 : b1 );
					return { x: point.x, y: point.y, t: classifyResult.t };

				} else if ( classifyResult.loc == IntersectionLocationType.BETWEEN ) {

					var x = + ( ( x1 + classifyResult.t * ( x2 - x1 ) ).toPrecision( 10 ) );
					var y = + ( ( y1 + classifyResult.t * ( y2 - y1 ) ).toPrecision( 10 ) );
					return { x: x, y: y, t: classifyResult.t, };

				}

			}

			return null;

		} else {

			//3. edges intersect

			for ( var i = 0; i < 2; i ++ ) {

				classifyPoint( i === 0 ? b0 : b1, a0, a1 );

				if ( classifyResult.loc == IntersectionLocationType.ORIGIN ) {

					var point = ( i === 0 ? b0 : b1 );
					return { x: point.x, y: point.y, t: classifyResult.t };

				}

			}

			var x = + ( ( x1 + t1 * ( x2 - x1 ) ).toPrecision( 10 ) );
			var y = + ( ( y1 + t1 * ( y2 - y1 ) ).toPrecision( 10 ) );
			return { x: x, y: y, t: t1 };

		}

	}

	function classifyPoint( p, edgeStart, edgeEnd ) {

		var ax = edgeEnd.x - edgeStart.x;
		var ay = edgeEnd.y - edgeStart.y;
		var bx = p.x - edgeStart.x;
		var by = p.y - edgeStart.y;
		var sa = ax * by - bx * ay;

		if ( ( p.x === edgeStart.x ) && ( p.y === edgeStart.y ) ) {

			classifyResult.loc = IntersectionLocationType.ORIGIN;
			classifyResult.t = 0;
			return;

		}

		if ( ( p.x === edgeEnd.x ) && ( p.y === edgeEnd.y ) ) {

			classifyResult.loc = IntersectionLocationType.DESTINATION;
			classifyResult.t = 1;
			return;

		}

		if ( sa < - Number.EPSILON ) {

			classifyResult.loc = IntersectionLocationType.LEFT;
			return;

		}

		if ( sa > Number.EPSILON ) {

			classifyResult.loc = IntersectionLocationType.RIGHT;
			return;


		}

		if ( ( ( ax * bx ) < 0 ) || ( ( ay * by ) < 0 ) ) {

			classifyResult.loc = IntersectionLocationType.BEHIND;
			return;

		}

		if ( ( Math.sqrt( ax * ax + ay * ay ) ) < ( Math.sqrt( bx * bx + by * by ) ) ) {

			classifyResult.loc = IntersectionLocationType.BEYOND;
			return;

		}

		var t;

		if ( ax !== 0 ) {

			t = bx / ax;

		} else {

			t = by / ay;

		}

		classifyResult.loc = IntersectionLocationType.BETWEEN;
		classifyResult.t = t;

	}

	function getIntersections( path1, path2 ) {

		const intersectionsRaw = [];
		const intersections = [];

		for ( let index = 1; index < path1.length; index ++ ) {

			const path1EdgeStart = path1[ index - 1 ];
			const path1EdgeEnd = path1[ index ];

			for ( let index2 = 1; index2 < path2.length; index2 ++ ) {

				const path2EdgeStart = path2[ index2 - 1 ];
				const path2EdgeEnd = path2[ index2 ];

				const intersection = findEdgeIntersection( path1EdgeStart, path1EdgeEnd, path2EdgeStart, path2EdgeEnd );

				if ( intersection !== null && intersectionsRaw.find( i => i.t <= intersection.t + Number.EPSILON && i.t >= intersection.t - Number.EPSILON ) === undefined ) {

					intersectionsRaw.push( intersection );
					intersections.push( new Vector2( intersection.x, intersection.y ) );

				}

			}

		}

		return intersections;

	}

	function getScanlineIntersections( scanline, boundingBox, paths ) {

		const center = new Vector2();
		boundingBox.getCenter( center );

		const allIntersections = [];

		paths.forEach( path => {

			// check if the center of the bounding box is in the bounding box of the paths.
			// this is a pruning method to limit the search of intersections in paths that can't envelop of the current path.
			// if a path envelops another path. The center of that oter path, has to be inside the bounding box of the enveloping path.
			if ( path.boundingBox.containsPoint( center ) ) {

				const intersections = getIntersections( scanline, path.points );

				intersections.forEach( p => {

					allIntersections.push( { identifier: path.identifier, isCW: path.isCW, point: p } );

				} );

			}

		} );

		allIntersections.sort( ( i1, i2 ) => {

			return i1.point.x - i2.point.x;

		} );

		return allIntersections;

	}

	function isHoleTo( simplePath, allPaths, scanlineMinX, scanlineMaxX, _fillRule ) {

		if ( _fillRule === null || _fillRule === undefined || _fillRule === '' ) {

			_fillRule = 'nonzero';

		}

		const centerBoundingBox = new Vector2();
		simplePath.boundingBox.getCenter( centerBoundingBox );

		const scanline = [ new Vector2( scanlineMinX, centerBoundingBox.y ), new Vector2( scanlineMaxX, centerBoundingBox.y ) ];

		const scanlineIntersections = getScanlineIntersections( scanline, simplePath.boundingBox, allPaths );

		scanlineIntersections.sort( ( i1, i2 ) => {

			return i1.point.x - i2.point.x;

		} );

		const baseIntersections = [];
		const otherIntersections = [];

		scanlineIntersections.forEach( i => {

			if ( i.identifier === simplePath.identifier ) {

				baseIntersections.push( i );

			} else {

				otherIntersections.push( i );

			}

		} );

		const firstXOfPath = baseIntersections[ 0 ].point.x;

		// build up the path hierarchy
		const stack = [];
		let i = 0;

		while ( i < otherIntersections.length && otherIntersections[ i ].point.x < firstXOfPath ) {

			if ( stack.length > 0 && stack[ stack.length - 1 ] === otherIntersections[ i ].identifier ) {

				stack.pop();

			} else {

				stack.push( otherIntersections[ i ].identifier );

			}

			i ++;

		}

		stack.push( simplePath.identifier );

		if ( _fillRule === 'evenodd' ) {

			const isHole = stack.length % 2 === 0 ? true : false;
			const isHoleFor = stack[ stack.length - 2 ];

			return { identifier: simplePath.identifier, isHole: isHole, for: isHoleFor };

		} else if ( _fillRule === 'nonzero' ) {

			// check if path is a hole by counting the amount of paths with alternating rotations it has to cross.
			let isHole = true;
			let isHoleFor = null;
			let lastCWValue = null;

			for ( let i = 0; i < stack.length; i ++ ) {

				const identifier = stack[ i ];
				if ( isHole ) {

					lastCWValue = allPaths[ identifier ].isCW;
					isHole = false;
					isHoleFor = identifier;

				} else if ( lastCWValue !== allPaths[ identifier ].isCW ) {

					lastCWValue = allPaths[ identifier ].isCW;
					isHole = true;

				}

			}

			return { identifier: simplePath.identifier, isHole: isHole, for: isHoleFor };

		} else {

			console.warn( 'fill-rule: "' + _fillRule + '" is currently not implemented.' );

		}

	}

	// check for self intersecting paths
	// TODO

	// check intersecting paths
	// TODO

	// prepare paths for hole detection
	let identifier = 0;

	let scanlineMinX = BIGNUMBER;
	let scanlineMaxX = - BIGNUMBER;

	let simplePaths = shapePath.subPaths.map( p => {

		const points = p.getPoints();
		let maxY = - BIGNUMBER;
		let minY = BIGNUMBER;
		let maxX = - BIGNUMBER;
		let minX = BIGNUMBER;

      	//points.forEach(p => p.y *= -1);

		for ( let i = 0; i < points.length; i ++ ) {

			const p = points[ i ];

			if ( p.y > maxY ) {

				maxY = p.y;

			}

			if ( p.y < minY ) {

				minY = p.y;

			}

			if ( p.x > maxX ) {

				maxX = p.x;

			}

			if ( p.x < minX ) {

				minX = p.x;

			}

		}

		//
		if ( scanlineMaxX <= maxX ) {

			scanlineMaxX = maxX + 1;

		}

		if ( scanlineMinX >= minX ) {

			scanlineMinX = minX - 1;

		}

		return { points: points, isCW: ShapeUtils.isClockWise( points ), identifier: identifier ++, boundingBox: new Box2( new Vector2( minX, minY ), new Vector2( maxX, maxY ) ) };

	} );

	simplePaths = simplePaths.filter( sp => sp.points.length > 0 );

	// check if path is solid or a hole
	const isAHole = simplePaths.map( p => isHoleTo( p, simplePaths, scanlineMinX, scanlineMaxX, shapePath.userData.style.fillRule ) );


	const shapesToReturn = [];
	simplePaths.forEach( p => {

		const amIAHole = isAHole[ p.identifier ];

		if ( ! amIAHole.isHole ) {

			const shape = new Shape( p.points );
			const holes = isAHole.filter( h => h.isHole && h.for === p.identifier );
			holes.forEach( h => {

				const path = simplePaths[ h.identifier ];
				shape.holes.push( new Path( path.points ) );

			} );
			shapesToReturn.push( shape );

		}

	} );

	return shapesToReturn;

};

SVGLoader.getStrokeStyle = function ( width, color, lineJoin, lineCap, miterLimit ) {

	// Param width: Stroke width
	// Param color: As returned by THREE.Color.getStyle()
	// Param lineJoin: One of "round", "bevel", "miter" or "miter-limit"
	// Param lineCap: One of "round", "square" or "butt"
	// Param miterLimit: Maximum join length, in multiples of the "width" parameter (join is truncated if it exceeds that distance)
	// Returns style object

	width = width !== undefined ? width : 1;
	color = color !== undefined ? color : '#000';
	lineJoin = lineJoin !== undefined ? lineJoin : 'miter';
	lineCap = lineCap !== undefined ? lineCap : 'butt';
	miterLimit = miterLimit !== undefined ? miterLimit : 4;

	return {
		strokeColor: color,
		strokeWidth: width,
		strokeLineJoin: lineJoin,
		strokeLineCap: lineCap,
		strokeMiterLimit: miterLimit
	};

};

SVGLoader.pointsToStroke = function ( points, style, arcDivisions, minDistance ) {

	// Generates a stroke with some witdh around the given path.
	// The path can be open or closed (last point equals to first point)
	// Param points: Array of Vector2D (the path). Minimum 2 points.
	// Param style: Object with SVG properties as returned by SVGLoader.getStrokeStyle(), or SVGLoader.parse() in the path.userData.style object
	// Params arcDivisions: Arc divisions for round joins and endcaps. (Optional)
	// Param minDistance: Points closer to this distance will be merged. (Optional)
	// Returns BufferGeometry with stroke triangles (In plane z = 0). UV coordinates are generated ('u' along path. 'v' across it, from left to right)

	var vertices = [];
	var normals = [];
	var uvs = [];

	if ( SVGLoader.pointsToStrokeWithBuffers( points, style, arcDivisions, minDistance, vertices, normals, uvs ) === 0 ) {

		return null;

	}

	var geometry = new BufferGeometry();
	geometry.setAttribute( 'position', new Float32BufferAttribute( vertices, 3 ) );
	geometry.setAttribute( 'normal', new Float32BufferAttribute( normals, 3 ) );
	geometry.setAttribute( 'uv', new Float32BufferAttribute( uvs, 2 ) );

	return geometry;

};

SVGLoader.pointsToStrokeWithBuffers = function () {

	var tempV2_1 = new Vector2();
	var tempV2_2 = new Vector2();
	var tempV2_3 = new Vector2();
	var tempV2_4 = new Vector2();
	var tempV2_5 = new Vector2();
	var tempV2_6 = new Vector2();
	var tempV2_7 = new Vector2();
	var lastPointL = new Vector2();
	var lastPointR = new Vector2();
	var point0L = new Vector2();
	var point0R = new Vector2();
	var currentPointL = new Vector2();
	var currentPointR = new Vector2();
	var nextPointL = new Vector2();
	var nextPointR = new Vector2();
	var innerPoint = new Vector2();
	var outerPoint = new Vector2();

	return function ( points, style, arcDivisions, minDistance, vertices, normals, uvs, vertexOffset ) {

		// This function can be called to update existing arrays or buffers.
		// Accepts same parameters as pointsToStroke, plus the buffers and optional offset.
		// Param vertexOffset: Offset vertices to start writing in the buffers (3 elements/vertex for vertices and normals, and 2 elements/vertex for uvs)
		// Returns number of written vertices / normals / uvs pairs
		// if 'vertices' parameter is undefined no triangles will be generated, but the returned vertices count will still be valid (useful to preallocate the buffers)
		// 'normals' and 'uvs' buffers are optional

		arcDivisions = arcDivisions !== undefined ? arcDivisions : 12;
		minDistance = minDistance !== undefined ? minDistance : 0.001;
		vertexOffset = vertexOffset !== undefined ? vertexOffset : 0;

		// First ensure there are no duplicated points
		points = removeDuplicatedPoints( points );

		var numPoints = points.length;

		if ( numPoints < 2 ) return 0;

		var isClosed = points[ 0 ].equals( points[ numPoints - 1 ] );

		var currentPoint;
		var previousPoint = points[ 0 ];
		var nextPoint;

		var strokeWidth2 = style.strokeWidth / 2;

		var deltaU = 1 / ( numPoints - 1 );
		var u0 = 0;

		var innerSideModified;
		var joinIsOnLeftSide;
		var isMiter;
		var initialJoinIsOnLeftSide = false;

		var numVertices = 0;
		var currentCoordinate = vertexOffset * 3;
		var currentCoordinateUV = vertexOffset * 2;

		// Get initial left and right stroke points
		getNormal( points[ 0 ], points[ 1 ], tempV2_1 ).multiplyScalar( strokeWidth2 );
		lastPointL.copy( points[ 0 ] ).sub( tempV2_1 );
		lastPointR.copy( points[ 0 ] ).add( tempV2_1 );
		point0L.copy( lastPointL );
		point0R.copy( lastPointR );

		for ( var iPoint = 1; iPoint < numPoints; iPoint ++ ) {

			currentPoint = points[ iPoint ];

			// Get next point
			if ( iPoint === numPoints - 1 ) {

				if ( isClosed ) {

					// Skip duplicated initial point
					nextPoint = points[ 1 ];

				} else nextPoint = undefined;

			} else {

				nextPoint = points[ iPoint + 1 ];

			}

			// Normal of previous segment in tempV2_1
			var normal1 = tempV2_1;
			getNormal( previousPoint, currentPoint, normal1 );

			tempV2_3.copy( normal1 ).multiplyScalar( strokeWidth2 );
			currentPointL.copy( currentPoint ).sub( tempV2_3 );
			currentPointR.copy( currentPoint ).add( tempV2_3 );

			var u1 = u0 + deltaU;

			innerSideModified = false;

			if ( nextPoint !== undefined ) {

				// Normal of next segment in tempV2_2
				getNormal( currentPoint, nextPoint, tempV2_2 );

				tempV2_3.copy( tempV2_2 ).multiplyScalar( strokeWidth2 );
				nextPointL.copy( currentPoint ).sub( tempV2_3 );
				nextPointR.copy( currentPoint ).add( tempV2_3 );

				joinIsOnLeftSide = true;
				tempV2_3.subVectors( nextPoint, previousPoint );
				if ( normal1.dot( tempV2_3 ) < 0 ) {

					joinIsOnLeftSide = false;

				}

				if ( iPoint === 1 ) initialJoinIsOnLeftSide = joinIsOnLeftSide;

				tempV2_3.subVectors( nextPoint, currentPoint );
				tempV2_3.normalize();
				var dot = Math.abs( normal1.dot( tempV2_3 ) );

				// If path is straight, don't create join
				if ( dot !== 0 ) {

					// Compute inner and outer segment intersections
					var miterSide = strokeWidth2 / dot;
					tempV2_3.multiplyScalar( - miterSide );
					tempV2_4.subVectors( currentPoint, previousPoint );
					tempV2_5.copy( tempV2_4 ).setLength( miterSide ).add( tempV2_3 );
					innerPoint.copy( tempV2_5 ).negate();
					var miterLength2 = tempV2_5.length();
					var segmentLengthPrev = tempV2_4.length();
					tempV2_4.divideScalar( segmentLengthPrev );
					tempV2_6.subVectors( nextPoint, currentPoint );
					var segmentLengthNext = tempV2_6.length();
					tempV2_6.divideScalar( segmentLengthNext );
					// Check that previous and next segments doesn't overlap with the innerPoint of intersection
					if ( tempV2_4.dot( innerPoint ) < segmentLengthPrev && tempV2_6.dot( innerPoint ) < segmentLengthNext ) {

						innerSideModified = true;

					}

					outerPoint.copy( tempV2_5 ).add( currentPoint );
					innerPoint.add( currentPoint );

					isMiter = false;

					if ( innerSideModified ) {

						if ( joinIsOnLeftSide ) {

							nextPointR.copy( innerPoint );
							currentPointR.copy( innerPoint );

						} else {

							nextPointL.copy( innerPoint );
							currentPointL.copy( innerPoint );

						}

					} else {

						// The segment triangles are generated here if there was overlapping

						makeSegmentTriangles();

					}

					switch ( style.strokeLineJoin ) {

						case 'bevel':

							makeSegmentWithBevelJoin( joinIsOnLeftSide, innerSideModified, u1 );

							break;

						case 'round':

							// Segment triangles

							createSegmentTrianglesWithMiddleSection( joinIsOnLeftSide, innerSideModified );

							// Join triangles

							if ( joinIsOnLeftSide ) {

								makeCircularSector( currentPoint, currentPointL, nextPointL, u1, 0 );

							} else {

								makeCircularSector( currentPoint, nextPointR, currentPointR, u1, 1 );

							}

							break;

						case 'miter':
						case 'miter-clip':
						default:

							var miterFraction = ( strokeWidth2 * style.strokeMiterLimit ) / miterLength2;

							if ( miterFraction < 1 ) {

								// The join miter length exceeds the miter limit

								if ( style.strokeLineJoin !== 'miter-clip' ) {

									makeSegmentWithBevelJoin( joinIsOnLeftSide, innerSideModified, u1 );
									break;

								} else {

									// Segment triangles

									createSegmentTrianglesWithMiddleSection( joinIsOnLeftSide, innerSideModified );

									// Miter-clip join triangles

									if ( joinIsOnLeftSide ) {

										tempV2_6.subVectors( outerPoint, currentPointL ).multiplyScalar( miterFraction ).add( currentPointL );
										tempV2_7.subVectors( outerPoint, nextPointL ).multiplyScalar( miterFraction ).add( nextPointL );

										addVertex( currentPointL, u1, 0 );
										addVertex( tempV2_6, u1, 0 );
										addVertex( currentPoint, u1, 0.5 );

										addVertex( currentPoint, u1, 0.5 );
										addVertex( tempV2_6, u1, 0 );
										addVertex( tempV2_7, u1, 0 );

										addVertex( currentPoint, u1, 0.5 );
										addVertex( tempV2_7, u1, 0 );
										addVertex( nextPointL, u1, 0 );

									} else {

										tempV2_6.subVectors( outerPoint, currentPointR ).multiplyScalar( miterFraction ).add( currentPointR );
										tempV2_7.subVectors( outerPoint, nextPointR ).multiplyScalar( miterFraction ).add( nextPointR );

										addVertex( currentPointR, u1, 1 );
										addVertex( tempV2_6, u1, 1 );
										addVertex( currentPoint, u1, 0.5 );

										addVertex( currentPoint, u1, 0.5 );
										addVertex( tempV2_6, u1, 1 );
										addVertex( tempV2_7, u1, 1 );

										addVertex( currentPoint, u1, 0.5 );
										addVertex( tempV2_7, u1, 1 );
										addVertex( nextPointR, u1, 1 );

									}

								}

							} else {

								// Miter join segment triangles

								if ( innerSideModified ) {

									// Optimized segment + join triangles

									if ( joinIsOnLeftSide ) {

										addVertex( lastPointR, u0, 1 );
										addVertex( lastPointL, u0, 0 );
										addVertex( outerPoint, u1, 0 );

										addVertex( lastPointR, u0, 1 );
										addVertex( outerPoint, u1, 0 );
										addVertex( innerPoint, u1, 1 );

									} else {

										addVertex( lastPointR, u0, 1 );
										addVertex( lastPointL, u0, 0 );
										addVertex( outerPoint, u1, 1 );

										addVertex( lastPointL, u0, 0 );
										addVertex( innerPoint, u1, 0 );
										addVertex( outerPoint, u1, 1 );

									}


									if ( joinIsOnLeftSide ) {

										nextPointL.copy( outerPoint );

									} else {

										nextPointR.copy( outerPoint );

									}


								} else {

									// Add extra miter join triangles

									if ( joinIsOnLeftSide ) {

										addVertex( currentPointL, u1, 0 );
										addVertex( outerPoint, u1, 0 );
										addVertex( currentPoint, u1, 0.5 );

										addVertex( currentPoint, u1, 0.5 );
										addVertex( outerPoint, u1, 0 );
										addVertex( nextPointL, u1, 0 );

									} else {

										addVertex( currentPointR, u1, 1 );
										addVertex( outerPoint, u1, 1 );
										addVertex( currentPoint, u1, 0.5 );

										addVertex( currentPoint, u1, 0.5 );
										addVertex( outerPoint, u1, 1 );
										addVertex( nextPointR, u1, 1 );

									}

								}

								isMiter = true;

							}

							break;

					}

				} else {

					// The segment triangles are generated here when two consecutive points are collinear

					makeSegmentTriangles();

				}

			} else {

				// The segment triangles are generated here if it is the ending segment

				makeSegmentTriangles();

			}

			if ( ! isClosed && iPoint === numPoints - 1 ) {

				// Start line endcap
				addCapGeometry( points[ 0 ], point0L, point0R, joinIsOnLeftSide, true, u0 );

			}

			// Increment loop variables

			u0 = u1;

			previousPoint = currentPoint;

			lastPointL.copy( nextPointL );
			lastPointR.copy( nextPointR );

		}

		if ( ! isClosed ) {

			// Ending line endcap
			addCapGeometry( currentPoint, currentPointL, currentPointR, joinIsOnLeftSide, false, u1 );

		} else if ( innerSideModified && vertices ) {

			// Modify path first segment vertices to adjust to the segments inner and outer intersections

			var lastOuter = outerPoint;
			var lastInner = innerPoint;

			if ( initialJoinIsOnLeftSide !== joinIsOnLeftSide ) {

				lastOuter = innerPoint;
				lastInner = outerPoint;

			}

			if ( joinIsOnLeftSide ) {

				if ( isMiter || initialJoinIsOnLeftSide ) {

					lastInner.toArray( vertices, 0 * 3 );
					lastInner.toArray( vertices, 3 * 3 );

					if ( isMiter ) {

						lastOuter.toArray( vertices, 1 * 3 );

					}

				}

			} else {

				if ( isMiter || ! initialJoinIsOnLeftSide ) {

					lastInner.toArray( vertices, 1 * 3 );
					lastInner.toArray( vertices, 3 * 3 );

					if ( isMiter ) {

						lastOuter.toArray( vertices, 0 * 3 );

					}

				}

			}

		}

		return numVertices;

		// -- End of algorithm

		// -- Functions

		function getNormal( p1, p2, result ) {

			result.subVectors( p2, p1 );
			return result.set( - result.y, result.x ).normalize();

		}

		function addVertex( position, u, v ) {

			if ( vertices ) {

				vertices[ currentCoordinate ] = position.x;
				vertices[ currentCoordinate + 1 ] = position.y;
				vertices[ currentCoordinate + 2 ] = 0;

				if ( normals ) {

					normals[ currentCoordinate ] = 0;
					normals[ currentCoordinate + 1 ] = 0;
					normals[ currentCoordinate + 2 ] = 1;

				}

				currentCoordinate += 3;

				if ( uvs ) {

					uvs[ currentCoordinateUV ] = u;
					uvs[ currentCoordinateUV + 1 ] = v;

					currentCoordinateUV += 2;

				}

			}

			numVertices += 3;

		}

		function makeCircularSector( center, p1, p2, u, v ) {

			// param p1, p2: Points in the circle arc.
			// p1 and p2 are in clockwise direction.

			tempV2_1.copy( p1 ).sub( center ).normalize();
			tempV2_2.copy( p2 ).sub( center ).normalize();

			var angle = Math.PI;
			var dot = tempV2_1.dot( tempV2_2 );
			if ( Math.abs( dot ) < 1 ) angle = Math.abs( Math.acos( dot ) );

			angle /= arcDivisions;

			tempV2_3.copy( p1 );

			for ( var i = 0, il = arcDivisions - 1; i < il; i ++ ) {

				tempV2_4.copy( tempV2_3 ).rotateAround( center, angle );

				addVertex( tempV2_3, u, v );
				addVertex( tempV2_4, u, v );
				addVertex( center, u, 0.5 );

				tempV2_3.copy( tempV2_4 );

			}

			addVertex( tempV2_4, u, v );
			addVertex( p2, u, v );
			addVertex( center, u, 0.5 );

		}

		function makeSegmentTriangles() {

			addVertex( lastPointR, u0, 1 );
			addVertex( lastPointL, u0, 0 );
			addVertex( currentPointL, u1, 0 );

			addVertex( lastPointR, u0, 1 );
			addVertex( currentPointL, u1, 1 );
			addVertex( currentPointR, u1, 0 );

		}

		function makeSegmentWithBevelJoin( joinIsOnLeftSide, innerSideModified, u ) {

			if ( innerSideModified ) {

				// Optimized segment + bevel triangles

				if ( joinIsOnLeftSide ) {

					// Path segments triangles

					addVertex( lastPointR, u0, 1 );
					addVertex( lastPointL, u0, 0 );
					addVertex( currentPointL, u1, 0 );

					addVertex( lastPointR, u0, 1 );
					addVertex( currentPointL, u1, 0 );
					addVertex( innerPoint, u1, 1 );

					// Bevel join triangle

					addVertex( currentPointL, u, 0 );
					addVertex( nextPointL, u, 0 );
					addVertex( innerPoint, u, 0.5 );

				} else {

					// Path segments triangles

					addVertex( lastPointR, u0, 1 );
					addVertex( lastPointL, u0, 0 );
					addVertex( currentPointR, u1, 1 );

					addVertex( lastPointL, u0, 0 );
					addVertex( innerPoint, u1, 0 );
					addVertex( currentPointR, u1, 1 );

					// Bevel join triangle

					addVertex( currentPointR, u, 1 );
					addVertex( nextPointR, u, 0 );
					addVertex( innerPoint, u, 0.5 );

				}

			} else {

				// Bevel join triangle. The segment triangles are done in the main loop

				if ( joinIsOnLeftSide ) {

					addVertex( currentPointL, u, 0 );
					addVertex( nextPointL, u, 0 );
					addVertex( currentPoint, u, 0.5 );

				} else {

					addVertex( currentPointR, u, 1 );
					addVertex( nextPointR, u, 0 );
					addVertex( currentPoint, u, 0.5 );

				}

			}

		}

		function createSegmentTrianglesWithMiddleSection( joinIsOnLeftSide, innerSideModified ) {

			if ( innerSideModified ) {

				if ( joinIsOnLeftSide ) {

					addVertex( lastPointR, u0, 1 );
					addVertex( lastPointL, u0, 0 );
					addVertex( currentPointL, u1, 0 );

					addVertex( lastPointR, u0, 1 );
					addVertex( currentPointL, u1, 0 );
					addVertex( innerPoint, u1, 1 );

					addVertex( currentPointL, u0, 0 );
					addVertex( currentPoint, u1, 0.5 );
					addVertex( innerPoint, u1, 1 );

					addVertex( currentPoint, u1, 0.5 );
					addVertex( nextPointL, u0, 0 );
					addVertex( innerPoint, u1, 1 );

				} else {

					addVertex( lastPointR, u0, 1 );
					addVertex( lastPointL, u0, 0 );
					addVertex( currentPointR, u1, 1 );

					addVertex( lastPointL, u0, 0 );
					addVertex( innerPoint, u1, 0 );
					addVertex( currentPointR, u1, 1 );

					addVertex( currentPointR, u0, 1 );
					addVertex( innerPoint, u1, 0 );
					addVertex( currentPoint, u1, 0.5 );

					addVertex( currentPoint, u1, 0.5 );
					addVertex( innerPoint, u1, 0 );
					addVertex( nextPointR, u0, 1 );

				}

			}

		}

		function addCapGeometry( center, p1, p2, joinIsOnLeftSide, start, u ) {

			// param center: End point of the path
			// param p1, p2: Left and right cap points

			switch ( style.strokeLineCap ) {

				case 'round':

					if ( start ) {

						makeCircularSector( center, p2, p1, u, 0.5 );

					} else {

						makeCircularSector( center, p1, p2, u, 0.5 );

					}

					break;

				case 'square':

					if ( start ) {

						tempV2_1.subVectors( p1, center );
						tempV2_2.set( tempV2_1.y, - tempV2_1.x );

						tempV2_3.addVectors( tempV2_1, tempV2_2 ).add( center );
						tempV2_4.subVectors( tempV2_2, tempV2_1 ).add( center );

						// Modify already existing vertices
						if ( joinIsOnLeftSide ) {

							tempV2_3.toArray( vertices, 1 * 3 );
							tempV2_4.toArray( vertices, 0 * 3 );
							tempV2_4.toArray( vertices, 3 * 3 );

						} else {

							tempV2_3.toArray( vertices, 1 * 3 );
							tempV2_3.toArray( vertices, 3 * 3 );
							tempV2_4.toArray( vertices, 0 * 3 );

						}

					} else {

						tempV2_1.subVectors( p2, center );
						tempV2_2.set( tempV2_1.y, - tempV2_1.x );

						tempV2_3.addVectors( tempV2_1, tempV2_2 ).add( center );
						tempV2_4.subVectors( tempV2_2, tempV2_1 ).add( center );

						var vl = vertices.length;

						// Modify already existing vertices
						if ( joinIsOnLeftSide ) {

							tempV2_3.toArray( vertices, vl - 1 * 3 );
							tempV2_4.toArray( vertices, vl - 2 * 3 );
							tempV2_4.toArray( vertices, vl - 4 * 3 );

						} else {

							tempV2_3.toArray( vertices, vl - 2 * 3 );
							tempV2_4.toArray( vertices, vl - 1 * 3 );
							tempV2_4.toArray( vertices, vl - 4 * 3 );

						}

					}

					break;

			}

		}

		function removeDuplicatedPoints( points ) {

			// Creates a new array if necessary with duplicated points removed.
			// This does not remove duplicated initial and ending points of a closed path.

			var dupPoints = false;
			for ( var i = 1, n = points.length - 1; i < n; i ++ ) {

				if ( points[ i ].distanceTo( points[ i + 1 ] ) < minDistance ) {

					dupPoints = true;
					break;

				}

			}

			if ( ! dupPoints ) return points;

			var newPoints = [];
			newPoints.push( points[ 0 ] );

			for ( var i = 1, n = points.length - 1; i < n; i ++ ) {

				if ( points[ i ].distanceTo( points[ i + 1 ] ) >= minDistance ) {

					newPoints.push( points[ i ] );

				}

			}

			newPoints.push( points[ points.length - 1 ] );

			return newPoints;

		}

	};

}();

var loadPromise = function loadPromise(loader, url) {
  return new Promise(function (resolve, reject) {
    loader.load(url, resolve, undefined, reject);
  });
};

var createBorderedMesh = function createBorderedMesh(geometry, material) {
  var edges = new EdgesGeometry(geometry);
  var mesh = new LineSegments(edges, material);
  return mesh;
};

var drawMapWithSVG = function drawMapWithSVG(group, groups, options, predefinedSeats) {
  try {
    var _temp4 = function _temp4() {
      var _temp = _catch(function () {
        var loader = new TextureLoader();
        return Promise.resolve(loadPromise(loader, svgData.bgUrl)).then(function (texture) {
          texture.generateMipmaps = false;
          texture.minFilter = LinearFilter;
          texture.needsUpdate = true;
          var width = texture.image.naturalWidth;
          var height = texture.image.naturalHeight;
          var geometry = new PlaneGeometry(width * 1.3, height * 1.3);
          var material = new MeshBasicMaterial({
            map: texture
          });
          var plane = new Mesh(geometry, material);
          plane.scale.set(scale, scale, 1);
          scene.add(plane);
        });
      }, function (err) {
        console.error('An error happened. ' + err);
      });

      if (_temp && _temp.then) return _temp.then(function () {});
    };

    var scene = options.scene,
        scale = options.scale,
        events = options.events,
        seat_circle_radius = options.seat_circle_radius,
        fonts = options.fonts,
        svgData = options.svgData,
        sceneMeshes = options.sceneMeshes,
        statuses = options.statuses;
    var svgLoader = new SVGLoader();

    var getSvgSeatId = function getSvgSeatId(svgNode) {
      var svgSeatId = svgNode.id;

      if (!svgSeatId) {
        svgSeatId = getSvgSeatId(svgNode.parentElement);
      }

      return svgSeatId;
    };

    var _temp5 = _catch(function () {
      var id = calculateMeshCountInsideObject(scene);
      return Promise.resolve(loadPromise(svgLoader, svgData.elementsUrl)).then(function (data) {
        var paths = data.paths;
        paths.forEach(function (path) {
          var svgSeatId = getSvgSeatId(path.userData.node);
          var isTextEl = !!path.userData.node.attributes['data-istext'];
          var isBlockEl = !!path.userData.node.attributes['data-isblock'];
          var isCoverAll = !!path.userData.node.attributes.iscoverall;
          var isColorChangable = !path.userData.node.attributes.unchangableColor;
          var isBorderedShape = !!path.userData.node.attributes.borderedShape;
          var isFrameShape = !!path.userData.node.attributes.frameShape;
          var borderColor = path.userData.node.attributes.borderColor || 'black';
          var jsonData = null;
          var groupsArr = Object.values(groups);

          for (var i = 0; i < groupsArr.length; ++i) {
            var _group = groupsArr[i];
            var rowArr = Object.values(_group.groups);

            for (var j = 0; j < rowArr.length; ++j) {
              var row = rowArr[j];
              var x = row.units.find(function (el) {
                return el.svgId === svgSeatId;
              });

              if (x) {
                jsonData = x;
                break;
              }
            }

            if (jsonData) {
              break;
            }
          }

          var _jsonData = jsonData,
              tierId = _jsonData.tier,
              label = _jsonData.label,
              seatId = _jsonData.id;

          var _seatId$split = seatId.split(':'),
              groupRowId = _seatId$split[0],
              seatIndex = _seatId$split[1];

          var seatStatus = statuses[groupRowId][seatIndex];
          var color = isColorChangable && predefinedSeats && predefinedSeats.includes(seatId) && seatStatus === SEAT_STATUSES.AVAILABLE ? COLORS.PREDEFINED : path.color.getHex() || 'black';
          var material = getSeatMaterial({
            color: color,
            options: {
              side: DoubleSide,
              depthWrite: false
            }
          }, materials);
          var obj = new Group();
          var shapes = path.toShapes(true);
          shapes.forEach(function (shape) {
            var geometry = new ShapeGeometry(shape);
            var mesh = new Mesh(geometry, material);
            var mesh1 = isBorderedShape || isFrameShape ? createBorderedMesh(geometry, material) : null;
            mesh.rotateY(Math.PI);
            mesh.rotateZ(Math.PI);
            !isBlockEl && !isFrameShape && obj.add(mesh);

            if (!isFrameShape) {
              sceneMeshes.push(mesh);
            }

            if (mesh1) {
              !isBlockEl && obj.add(mesh1);
              sceneMeshes.push(mesh1);
            }

            mesh.sgId = id;
            mesh.isRequestPending = false;
            mesh.objectType = OBJECT_TYPES.SEAT;
            mesh.allowColorUpdateByStatus = !isTextEl;
            mesh.seatId = seatId;
            mesh.tierId = tierId;
            mesh.status = seatStatus;
            mesh.label = label;
            mesh.hasHandlers = true;
            mesh.isColorChangable = isColorChangable;
            mesh.mouseOver = events.onMouseOver;
            mesh.mouseOut = events.onMouseOut;

            mesh.changeBorderColor = function (color) {};

            mesh.changeBorderColorToDefault = function () {};

            var renderOrder = 100;
            renderOrder = isTextEl || !isColorChangable ? 300 : isBlockEl ? 500 : 100;

            if (isCoverAll) {
              renderOrder = 500;
            }

            mesh.renderOrder = renderOrder;

            mesh.getNeighbourSeats = function () {
              return seats[seatId];
            };

            mesh.changeColor = function (color) {
              if (isTextEl || isBlockEl) {
                return;
              }

              mesh.material = getSeatMaterial({
                color: color,
                options: {
                  side: DoubleSide,
                  depthWrite: false
                }
              }, materials);
            };

            if (mesh1) {
              mesh1.rotateY(Math.PI);
              mesh1.rotateZ(Math.PI);
              mesh1.renderOrder = renderOrder + 1;

              mesh1.changeColor = function (color) {
                mesh1.material = getSeatMaterial({
                  color: color,
                  options: {
                    side: DoubleSide,
                    depthWrite: false
                  }
                }, materials);
              };

              mesh1.changeColor(borderColor);
            }

            !(isTextEl || isBlockEl || !isColorChangable) && mesh.changeColor(svgData.booked_color || COLORS.BOOKED);

            var makeAvailable = function makeAvailable() {
              mesh.status = SEAT_STATUSES.AVAILABLE;
            };

            var makeHold = function makeHold() {
              mesh.status = SEAT_STATUSES.HOLD;
            };

            var makeReserved = function makeReserved() {
              mesh.status = SEAT_STATUSES.RESERVED;
            };

            var makeBooked = function makeBooked() {
              mesh.status = SEAT_STATUSES.BOOKED;
            };

            var makeBlocked = function makeBlocked() {
              mesh.status = SEAT_STATUSES.BLOCKED;
            };

            var makeReservedByMe = function makeReservedByMe() {
              window.localStorage.setItem('tierId', tierId);
              mesh.status = SEAT_STATUSES.RESERVED_BY_ME;
            };

            mesh.setColorByStatus = function () {
              if (!mesh.allowColorUpdateByStatus) {
                return;
              }

              switch (mesh.status) {
                case SEAT_STATUSES.AVAILABLE:
                  {
                    mesh.changeColor(color);
                    break;
                  }

                case SEAT_STATUSES.HOLD:
                  {
                    mesh.changeColor(COLORS.HOLD);
                    break;
                  }

                case SEAT_STATUSES.RESERVED_BY_ME:
                  {
                    mesh.changeColor(svgData.own_reservation_color || COLORS.RESERVED_BY_ME);
                    break;
                  }

                case SEAT_STATUSES.RESERVED:
                  {
                    mesh.changeColor(svgData.reservation_color || COLORS.RESERVED);
                    break;
                  }

                case SEAT_STATUSES.BOOKED:
                  {
                    mesh.changeColor(svgData.booked_color || COLORS.BOOKED);
                    break;
                  }

                case SEAT_STATUSES.BLOCKED:
                  {
                    mesh.changeColor(COLORS.BLOCKED);
                    break;
                  }
              }
            };

            if (isColorChangable) {
              mesh.setColorByStatus();
            }

            mesh.updateOpacity = function () {
              mesh.changeColor("#" + mesh.material.color.getHexString());
            };

            mesh.makeDisabled = function () {
              if (mesh.disabled) {
                return;
              }

              mesh.disabled = true;
              mesh.mouseOver = null;
              mesh.mouseOut = null;
              mesh.opacity = 0.3;
              mesh.updateOpacity();
            };

            mesh.makeActive = function () {
              if (!mesh.disabled) {
                return;
              }

              mesh.disabled = false;
              mesh.mouseOver = events.onMouseOver;
              mesh.mouseOut = events.onMouseOut;
              mesh.opacity = 1;
              mesh.updateOpacity();
            };

            var select = function select(id) {
              sceneMeshes.forEach(function (meshItem) {
                if (meshItem.type === 'Mesh' && id === mesh.label && mesh.label === meshItem.label && meshItem.isColorChangable) {
                  meshItem.material = getSeatMaterial({
                    color: COLORS.HIGHLIGHT,
                    options: {
                      side: DoubleSide,
                      depthWrite: false
                    }
                  }, materials);
                }
              });
            };

            var unselect = function unselect(id) {
              sceneMeshes.forEach(function (meshItem) {
                if (meshItem.type === 'Mesh' && id === mesh.label && mesh.label === meshItem.label && meshItem.isColorChangable) {
                  meshItem.material = getSeatMaterial({
                    color: color,
                    options: {
                      side: DoubleSide,
                      depthWrite: false
                    }
                  }, materials);
                }
              });
            };

            mesh.select = select;
            mesh.unselect = unselect;

            mesh.unsetLoading = function () {
              return null;
            };

            mesh.setLoading = function () {
              if (window.isMobile) {
                var _label = drawLabel("\uF110", {
                  font: fonts.fontAwesome,
                  color: 'grey',
                  position: [0, 0, 0],
                  center_x: 0,
                  center_y: 0,
                  scale: scale,
                  seat_circle_radius: seat_circle_radius,
                  scene: mesh,
                  size: seat_circle_radius * scale
                }, textGeometries, materials);

                mesh.unsetLoading = function () {
                  return mesh.remove(_label);
                };
              }
            };

            mesh.statusUpdate = function (status) {
              if (mesh.status !== status) {
                switch (status) {
                  case SEAT_STATUSES.PERMANENT_RESERVED:
                  case SEAT_STATUSES.AVAILABLE:
                    {
                      makeAvailable();
                      mesh.unsetLoading();
                      isBlockEl && obj.remove(mesh);
                      break;
                    }

                  case SEAT_STATUSES.HOLD:
                    {
                      makeHold();
                      mesh.unsetLoading();
                      isBlockEl && obj.remove(mesh);
                      break;
                    }

                  case SEAT_STATUSES.RESERVED_BY_ME:
                    {
                      makeReservedByMe();
                      mesh.unsetLoading();
                      isBlockEl && obj.remove(mesh);
                      break;
                    }

                  case SEAT_STATUSES.RESERVED:
                    {
                      makeReserved();
                      mesh.unsetLoading();
                      isBlockEl && obj.remove(mesh);
                      break;
                    }

                  case SEAT_STATUSES.BOOKED:
                    {
                      makeBooked();
                      mesh.unsetLoading();
                      isBlockEl && obj.remove(mesh);
                      break;
                    }

                  case SEAT_STATUSES.BLOCKED:
                    {
                      makeBlocked();
                      mesh.unsetLoading();
                      isBlockEl && obj.add(mesh);
                      break;
                    }
                }

                if (isColorChangable) {
                  mesh.setColorByStatus();
                }
              }
            };

            if (seats[seatId]) {
              if (Array.isArray(seats[seatId])) {
                seats[seatId].push(mesh);
              } else {
                seats[seatId] = [seats[seatId], mesh];
              }
            } else {
              seats[seatId] = mesh;
            }
          });
          obj.scale.set(scale * 1.3, scale * 1.3, 1);
          scene.add(obj);
        });
      });
    }, function (err) {
      console.log('An error happened. ' + err);
    });

    return Promise.resolve(_temp5 && _temp5.then ? _temp5.then(_temp4) : _temp4(_temp5));
  } catch (e) {
    return Promise.reject(e);
  }
};

var drawRingStanding = function drawRingStanding(group, options) {
  var x = group.x,
      y = group.y,
      width = group.width,
      height = group.height,
      innerWidth = group.inner_width,
      innerHeight = group.inner_height;
  var color = options.color,
      scale = options.scale,
      _options$opacity = options.opacity,
      opacity = _options$opacity === void 0 ? 1 : _options$opacity;
  var horizontal_w = width,
      horizontal_h = (height - innerHeight) / 2;
  var vertical_w = (width - innerWidth) / 2,
      vertical_h = height;
  var rects = [{
    width: horizontal_w,
    height: horizontal_h,
    x: x,
    y: y + (innerHeight / 2 + horizontal_h / 2)
  }, {
    width: horizontal_w,
    height: horizontal_h,
    x: x,
    y: y - (innerHeight / 2 + horizontal_h / 2)
  }, {
    width: vertical_w,
    height: vertical_h,
    x: x + (innerWidth / 2 + vertical_w / 2),
    y: y
  }, {
    width: vertical_w,
    height: vertical_h,
    x: x - (innerWidth / 2 + vertical_w / 2),
    y: y
  }];

  var _createRectsMeshes = createRectsMeshes(rects, options),
      rectMeshes = _createRectsMeshes.rectMeshes,
      ring = _createRectsMeshes.ring;

  var drawRingBorder = function drawRingBorder() {
    var lineBasicMaterial = new LineBasicMaterial({
      color: color,
      opacity: opacity,
      transparent: true
    });
    var geometries = [new PlaneGeometry(width * scale, height * scale), new PlaneGeometry(innerWidth * scale, innerHeight * scale)];
    ring.borderLines = geometries.map(function (geometry) {
      var lineGeometry = new BufferGeometry();
      var geometryArray = getGeometryArray(geometry);
      var vertices = new Float32Array(geometryArray);
      lineGeometry.setAttribute('position', new BufferAttribute(vertices, 3));
      var line = new Line(lineGeometry, lineBasicMaterial);
      ring.add(line);
      return line;
    });
  };

  drawRingBorder();
  return {
    standingArea: ring,
    rectMeshes: rectMeshes
  };
};

var drawRectStanding = function drawRectStanding(group, options) {
  var x = group.x,
      y = group.y,
      width = group.width,
      height = group.height;
  var color = options.color,
      scale = options.scale,
      _options$opacity = options.opacity,
      opacity = _options$opacity === void 0 ? 1 : _options$opacity;
  var rects = [{
    width: width,
    height: height,
    x: x,
    y: y
  }];

  var _createRectsMeshes = createRectsMeshes(rects, options),
      rectMeshes = _createRectsMeshes.rectMeshes,
      ring = _createRectsMeshes.ring;

  var drawRingBorder = function drawRingBorder() {
    var lineBasicMaterial = new LineBasicMaterial({
      color: color,
      opacity: opacity,
      transparent: true
    });
    var geometries = [new PlaneGeometry(width * scale, height * scale)];
    ring.borderLines = geometries.map(function (geometry) {
      var lineGeometry = new BufferGeometry();
      var geometryArray = getGeometryArray(geometry);
      var vertices = new Float32Array(geometryArray);
      lineGeometry.setAttribute('position', new BufferAttribute(vertices, 3));
      var line = new Line(lineGeometry, lineBasicMaterial);
      line.position.set(x * scale, -y * scale, 0);
      ring.add(line);
      return line;
    });
  };

  drawRingBorder();
  return {
    standingArea: ring,
    rectMeshes: rectMeshes
  };
};

var drawOvalStanding = function drawOvalStanding(group, options) {
  var x = group.x,
      y = group.y,
      width = group.width,
      height = group.height,
      innerWidth = group.inner_width,
      innerHeight = group.inner_height,
      rotation = group.rotation,
      rectType = group.rect_type;
  var color = options.color,
      scale = options.scale,
      _options$opacity = options.opacity,
      opacity = _options$opacity === void 0 ? 1 : _options$opacity;
  var isTopRect = rectType === 'top_rect';
  var horizontal_w = width,
      horizontal_h = (height - innerHeight) / 2;
  var vertical_w = (width - innerWidth) / 2,
      vertical_h = height;
  var rects = [{
    width: horizontal_w,
    height: horizontal_h,
    x: x,
    y: y + (innerHeight / 2 + horizontal_h / 2)
  }, {
    width: vertical_w,
    height: vertical_h,
    x: x + (innerWidth / 2 + vertical_w / 2),
    y: y
  }, {
    width: vertical_w,
    height: vertical_h,
    x: x - (innerWidth / 2 + vertical_w / 2),
    y: y
  }];

  if (isTopRect) {
    rects = [{
      width: horizontal_w,
      height: horizontal_h,
      x: x,
      y: -(y + (innerHeight / 2 + horizontal_h / 2))
    }, {
      width: vertical_w,
      height: vertical_h,
      x: x + (innerWidth / 2 + vertical_w / 2),
      y: -y
    }, {
      width: vertical_w,
      height: vertical_h,
      x: x - (innerWidth / 2 + vertical_w / 2),
      y: -y
    }];
  }

  var _createRectsMeshes = createRectsMeshes(rects, options),
      rectMeshes = _createRectsMeshes.rectMeshes,
      ring = _createRectsMeshes.ring;

  var drawRingBorder = function drawRingBorder() {
    var _geometry$attributes;

    var lineBasicMaterial = new LineBasicMaterial({
      color: color,
      opacity: opacity,
      transparent: true
    });
    var geometry = new PlaneGeometry(width * scale, height * scale);
    var delta = (width - innerWidth) * scale / 2;
    var geometryArray = [];

    var _geometry$attributes$ = geometry === null || geometry === void 0 ? void 0 : (_geometry$attributes = geometry.attributes) === null || _geometry$attributes === void 0 ? void 0 : _geometry$attributes.position,
        gArray = _geometry$attributes$.array;

    geometryArray.push(new Vector3(gArray[0], gArray[1], 0), new Vector3(gArray[0] + delta, gArray[1], 0), new Vector3(gArray[6] + delta, gArray[7] + delta, 0), new Vector3(gArray[9] - delta, gArray[10] + delta, 0), new Vector3(gArray[3] - delta, gArray[4], 0), new Vector3(gArray[3], gArray[4], 0), new Vector3(gArray[9], gArray[10], 0), new Vector3(gArray[6], gArray[7], 0), new Vector3(gArray[0], gArray[1], 0));
    var lineGeometry = new BufferGeometry().setFromPoints(geometryArray);
    var line = new Line(lineGeometry, lineBasicMaterial);

    if (isTopRect) {
      line.position.set(x * scale, y * scale, 0);
      line.rotation.set(0, 0, rotation * Math.PI / 180);
    } else {
      line.position.set(x * scale, -y * scale, 0);
    }

    ring.add(line);
    ring.borderLines = [line];
  };

  drawRingBorder();
  return {
    standingArea: ring,
    rectMeshes: rectMeshes
  };
};

var drawMultipleRects = function drawMultipleRects(group, options) {
  var x = group.x,
      y = group.y,
      labelPositions = group.labelPositions;
  var color = options.color,
      scale = options.scale,
      materialGroup = options.materialGroup,
      scene = options.scene,
      seat_circle_radius = options.seat_circle_radius,
      fonts = options.fonts,
      center_x = options.center_x,
      center_y = options.center_y,
      _options$opacity = options.opacity,
      opacity = _options$opacity === void 0 ? 1 : _options$opacity;

  var _createRectsMeshes = createRectsMeshes(group.rects, options),
      rectMeshes = _createRectsMeshes.rectMeshes,
      ring = _createRectsMeshes.ring;

  var drawMultipleRects = function drawMultipleRects() {
    var lineBasicMaterial = new LineBasicMaterial({
      color: color,
      opacity: opacity,
      transparent: true
    });
    ring.borderLines = rectMeshes.map(function (_ref) {
      var geometry = _ref.geometry,
          position = _ref.position;
      var lineGeometry = new BufferGeometry();
      var geometryArray = getGeometryArray(geometry);
      var vertices = new Float32Array(geometryArray);
      lineGeometry.setAttribute('position', new BufferAttribute(vertices, 3));
      var line = new Line(lineGeometry, lineBasicMaterial);
      line.position.set(x * scale + position.x, -y * scale + position.y, 0);
      ring.add(line);
      return line;
    });
  };

  drawMultipleRects();

  if (!group.hideSectionLabel && labelPositions) {
    var labelText = materialGroup.label;
    labelPositions.forEach(function (_ref2) {
      var x = _ref2.x,
          y = _ref2.y,
          _ref2$rotation = _ref2.rotation,
          rotation = _ref2$rotation === void 0 ? 0 : _ref2$rotation;
      drawLabel(labelText, {
        font: fonts.openSans,
        color: 'black',
        position: [x * scale, -y * scale, 0],
        center_x: center_x,
        center_y: center_y,
        scale: scale,
        seat_circle_radius: seat_circle_radius,
        scene: scene,
        size: group.size || 15,
        rotation: rotation
      }, textGeometries, materials);
    });
  }

  return {
    standingArea: ring,
    rectMeshes: rectMeshes
  };
};

var drawStandingArea = function drawStandingArea(group, options) {
  var color = options.color,
      scale = options.scale,
      scene = options.scene,
      center_x = options.center_x,
      center_y = options.center_y,
      events = options.events,
      tierId = options.tierId,
      label = options.label,
      enableTicketTypeAssign = options.enableTicketTypeAssign,
      tierColor = options.tierColor,
      selectedTierId = options.selectedTierId,
      sgId = options.id,
      sceneMeshes = options.sceneMeshes;
  var opacity = options.opacity;
  var seatColor = enableTicketTypeAssign ? tierColor : COLORS.BOOKED;

  if (selectedTierId && selectedTierId !== tierId) {
    opacity = 0.3;
  }

  var drawingFn;

  switch (group.standingAreaShape) {
    case 'ring':
      drawingFn = drawRingStanding;
      break;

    case 'rect':
      drawingFn = drawRectStanding;
      break;

    case 'oval':
      drawingFn = drawOvalStanding;
      break;

    case 'multiple_rects':
      drawingFn = drawMultipleRects;
      break;
  }

  var _drawingFn = drawingFn(group, _extends({}, options, {
    color: seatColor,
    opacity: opacity
  })),
      standingArea = _drawingFn.standingArea,
      rectMeshes = _drawingFn.rectMeshes;

  Object.defineProperty(standingArea, 'material', {
    get: function get() {
      return rectMeshes[0].material;
    }
  });
  standingArea.position.set(-center_x * scale, center_y * scale, 0);
  scene.add(standingArea);
  standingArea.click = events.onClick;
  standingArea.mouseOver = events.onMouseOver;
  standingArea.mouseOut = events.onMouseOut;
  standingArea.sgType = SEAT_GROUP_TYPES.STANDING;
  standingArea.sgId = sgId;
  standingArea.sgLabel = label;
  standingArea.tierId = tierId;
  standingArea.status = SEAT_STATUSES.BOOKED;
  standingArea.sectionLabel = label;
  standingArea.rowLabel = label;
  standingArea.label = label;
  standingArea.opacity = opacity;
  standingArea.color = seatColor;

  if (group.rotation) {
    var degree = group.rotation * Math.PI / 180;
    standingArea.rotateOnAxis(new Vector3(0, 0, 1), degree);
  }

  standingArea.changeBorderColor = function (color) {
    standingArea.borderLines && standingArea.borderLines.forEach(function (line) {
      line.material = getLineMaterial({
        color: color
      }, materials);
    });
  };

  standingArea.changeBorderColorToDefault = function () {
    return standingArea.changeBorderColor(color);
  };

  var makeAvailable = function makeAvailable() {
    standingArea.changeBorderColor(color);
    rectMeshes.forEach(function (rect) {
      rect.material = getSeatMaterial({
        color: color
      }, materials);
    });
    standingArea.status = SEAT_STATUSES.AVAILABLE;
    standingArea.click = events.onClick;

    standingArea.changeBorderColorToDefault = function () {
      return standingArea.changeBorderColor(color);
    };
  };

  standingArea.changeOpacity = function (opacity, color) {
    standingArea.children[0].material = getSeatMaterial({
      color: color,
      opacity: opacity
    }, materials);
  };

  var makeBooked = function makeBooked() {
    standingArea.changeBorderColor(COLORS.BOOKED);
    rectMeshes.forEach(function (rect) {
      rect.material = getSeatMaterial({
        color: COLORS.BOOKED
      }, materials);
    });
    standingArea.status = SEAT_STATUSES.BOOKED;
    standingArea.click = null;

    standingArea.changeBorderColorToDefault = function () {
      return standingArea.changeBorderColor(COLORS.BOOKED);
    };
  };

  var makeBlocked = function makeBlocked() {
    standingArea.changeBorderColor(COLORS.BLOCKED);
    rectMeshes.forEach(function (rect) {
      rect.material = getSeatMaterial({
        color: COLORS.BLOCKED
      }, materials);
    });
    standingArea.status = SEAT_STATUSES.BLOCKED;
    standingArea.click = null;

    standingArea.changeBorderColorToDefault = function () {
      return standingArea.changeBorderColor(COLORS.BLOCKED);
    };
  };

  standingArea.statusUpdate = function (status) {
    if (standingArea.status !== status) {
      switch (status) {
        case SEAT_STATUSES.AVAILABLE:
          {
            makeAvailable();
            break;
          }

        case SEAT_STATUSES.BOOKED:
          {
            makeBooked();
            break;
          }

        case SEAT_STATUSES.BLOCKED:
          {
            makeBlocked();
            break;
          }
      }
    }
  };

  standingArea.hasHandlers = true;
  standingArea.changeOpacity(opacity, seatColor);
  sceneMeshes.push(standingArea);
  standingAreas.push(standingArea);
};

var drawTableHalfCircle = function drawTableHalfCircle(centerPos, faceTo, color, radius, materials, sceneMeshes) {
  var _geometry$attributes;

  var sign = faceTo === 'left' ? 1 : -1;
  var geometry = new CircleGeometry(radius, 32, sign * Math.PI / 2, Math.PI);
  var obj = new Mesh(geometry, new MeshBasicMaterial({
    color: color
  }));
  obj.position.set(-centerPos.x - 0.2 * sign, 0, 0);

  var _geometry$attributes$ = geometry === null || geometry === void 0 ? void 0 : (_geometry$attributes = geometry.attributes) === null || _geometry$attributes === void 0 ? void 0 : _geometry$attributes.position,
      gArray = _geometry$attributes$.array;

  var geometryArray = [];
  geometryArray.push(new Vector3(gArray[3], gArray[4], 0), new Vector3(gArray[6], gArray[7], 0));
  var separationLineGeometry = new BufferGeometry().setFromPoints(geometryArray);
  var separationLineBasicMaterial = getLineMaterial({
    color: '#FFFFFF'
  }, materials);
  var borderLineBasicMaterial = getLineMaterial({
    color: color
  }, materials);
  var separationLine = new Line(separationLineGeometry, separationLineBasicMaterial);
  var edges = new RingGeometry(radius, radius, 32, 1, sign * Math.PI / 2, Math.PI);
  var border = new Line(edges, borderLineBasicMaterial);
  border.position.set(-centerPos.x - 0.1 * sign, 0, 0);
  obj.renderOrder = 100;
  border.renderOrder = 100;
  separationLine.renderOrder = 101;
  var triangle = new Group();
  triangle.add(obj);
  triangle.add(separationLine);
  triangle.add(border);
  triangle.obj = obj;
  triangle.separationLine = separationLine;
  triangle.border = border;
  sceneMeshes.push(triangle);
  return [triangle];
};

var drawTable2Seats = function drawTable2Seats(seatId, options) {
  var _seat$position;

  var color = options.color,
      position = options.position,
      rotation = options.rotation,
      tierId = options.tierId,
      parent = options.parent,
      center_x = options.center_x,
      center_y = options.center_y,
      scale = options.scale,
      seat_circle_radius = options.seat_circle_radius,
      events = options.events,
      sectionLabel = options.sectionLabel,
      rowLabel = options.rowLabel,
      label = options.label,
      fonts = options.fonts,
      _options$disabled = options.disabled,
      disabled = _options$disabled === void 0 ? false : _options$disabled,
      _options$type = options.type,
      type = _options$type === void 0 ? SEAT_TYPES.CHAIR : _options$type,
      _options$faceTo = options.faceTo,
      faceTo = _options$faceTo === void 0 ? 'left' : _options$faceTo,
      enableTicketTypeAssign = options.enableTicketTypeAssign,
      tierColor = options.tierColor,
      selectedTierId = options.selectedTierId,
      sceneMeshes = options.sceneMeshes;
  var _options$opacity = options.opacity,
      opacity = _options$opacity === void 0 ? 1 : _options$opacity;
  var seatColor = enableTicketTypeAssign ? tierColor : COLORS.BOOKED;
  position[0] -= center_x * scale;
  position[1] += center_y * scale;

  if (selectedTierId && selectedTierId !== tierId) {
    opacity = 0.3;
  }

  var radius = (seat_circle_radius || 14 - 2) * scale;
  var seat = new Group();

  (_seat$position = seat.position).set.apply(_seat$position, position);

  var circleGeometry = new CircleGeometry(radius, 32);
  var circleMaterial = getSeatMaterial({
    color: seatColor,
    opacity: opacity
  }, materials);
  var circle1 = new Mesh(circleGeometry, circleMaterial);
  var circle2 = new Mesh(circleGeometry, circleMaterial);
  var circleBorder1 = new LineSegments(new EdgesGeometry(circleGeometry), getLineMaterial({
    color: COLORS.BOOKED,
    opacity: opacity
  }, materials));
  var circleBorder2 = circleBorder1.clone();
  var sign = faceTo === 'left' ? 1 : -1;
  var tableRadius = 2.5 * radius;
  var c1p = new Vector3(sign * (0.2 - tableRadius), 0, 0).sub(seat.position);
  var c2p = new Vector3(sign * (0.2 - tableRadius) / 2, (0.2 - tableRadius) * Math.sqrt(3) / 2, 0).sub(seat.position);
  circle1.position.copy(c1p);
  circleBorder1.position.copy(c1p);
  circle2.position.copy(c2p);
  circleBorder2.position.copy(c2p);

  var _drawTableHalfCircle = drawTableHalfCircle(seat.position, faceTo, seatColor, tableRadius, materials, sceneMeshes),
      triangle = _drawTableHalfCircle[0];

  seat.add(triangle, circle1, circle2, circleBorder1, circleBorder2);
  seat.isRequestPending = false;
  seat.objectType = OBJECT_TYPES.SEAT;
  seat.opacity = opacity;
  seat.disabled = disabled;
  seat.allowColorUpdateByStatus = true;
  seat.radius = radius;
  seat.seatId = seatId;
  seat.tierId = tierId;
  seat.status = SEAT_STATUSES.BOOKED;
  seat.sectionLabel = sectionLabel;
  seat.rowLabel = rowLabel;
  seat.label = label;
  seat.seatType = type;
  seat.color = seatColor;
  seat.material = triangle.obj.material;

  seat.changeBorderColor = function (color) {
    circleBorder1.material = circleBorder2.material = triangle.border.material = getLineMaterial({
      color: color
    }, materials);
  };

  seat.mouseOver = seat.disabled ? null : events.onMouseOver;
  seat.mouseOut = seat.disabled ? null : events.onMouseOut;

  seat.changeBorderColorToDefault = function () {
    seat.changeBorderColor(COLORS.BOOKED);
  };

  seat.changeColor = function (color, opacity) {
    if (opacity === void 0) {
      opacity = 1;
    }

    seat.changeBorderColor(color);
    circle1.material = circle2.material = getSeatMaterial({
      color: color,
      opacity: opacity
    }, materials);
    triangle.obj.material = getSeatMaterial({
      color: color,
      opacity: opacity
    }, materials);

    seat.changeBorderColorToDefault = function () {
      seat.changeBorderColor(color);
    };

    seat.material = triangle.obj.material;
  };

  var makeAvailable = function makeAvailable() {
    seat.status = SEAT_STATUSES.AVAILABLE;
  };

  var makeHold = function makeHold() {
    seat.status = SEAT_STATUSES.HOLD;
  };

  var makeReserved = function makeReserved() {
    seat.status = SEAT_STATUSES.RESERVED;
  };

  var makeBooked = function makeBooked() {
    seat.status = SEAT_STATUSES.BOOKED;
  };

  var makeBlocked = function makeBlocked() {
    seat.status = SEAT_STATUSES.BLOCKED;
  };

  var makeReservedByMe = function makeReservedByMe() {
    window.localStorage.setItem('tierId', tierId);
    seat.status = SEAT_STATUSES.RESERVED_BY_ME;
  };

  seat.setColorByStatus = function () {
    if (!seat.allowColorUpdateByStatus) {
      return;
    }

    switch (seat.status) {
      case SEAT_STATUSES.AVAILABLE:
        {
          seat.changeColor(color);
          break;
        }

      case SEAT_STATUSES.HOLD:
        {
          seat.changeColor(COLORS.HOLD);
          break;
        }

      case SEAT_STATUSES.RESERVED_BY_ME:
        {
          seat.changeColor(COLORS.RESERVED_BY_ME);
          break;
        }

      case SEAT_STATUSES.RESERVED:
        {
          seat.changeColor(COLORS.RESERVED);
          break;
        }

      case SEAT_STATUSES.BOOKED:
        {
          seat.changeColor(COLORS.BOOKED);
          break;
        }

      case SEAT_STATUSES.BLOCKED:
        {
          seat.changeColor(COLORS.BLOCKED);
          break;
        }
    }
  };

  seat.changeOpacity = function (opacity, color) {
    seat.changeColor(color, opacity);
  };

  seat.makeDisabled = function () {
    if (seat.disabled) {
      return;
    }

    seat.disabled = true;
    seat.mouseOver = null;
    seat.mouseOut = null;
    seat.opacity = 0.3;
    seat.changeOpacity(0.3, seatColor);
  };

  seat.makeActive = function () {
    if (!seat.disabled) {
      return;
    }

    seat.disabled = false;
    seat.mouseOver = events.onMouseOver;
    seat.mouseOut = events.onMouseOut;
    seat.opacity = 1;
    seat.changeOpacity(1, seatColor);
  };

  var select = function select() {
    circleBorder1.material = circleBorder2.material = triangle.border.material = getLineMaterial({
      color: '#575656',
      linewidth: 5,
      opacity: seat.opacity
    }, materials);

    seat.changeBorderColor = function () {
      return null;
    };
  };

  var unselect = function unselect() {
    circleBorder1.material = circleBorder2.material = triangle.border.material = getLineMaterial({
      color: '#000',
      opacity: seat.opacity
    }, materials);

    seat.changeBorderColor = function (color) {
      circleBorder1.material = circleBorder2.material = triangle.border.material = getLineMaterial({
        color: color,
        opacity: seat.opacity
      }, materials);
    };

    seat.changeBorderColorToDefault();
  };

  seat.select = select;
  seat.unselect = unselect;

  seat.unsetLoading = function () {
    return null;
  };

  seat.setLoading = function () {
    if (window.isMobile) {
      var _label = drawLabel("\uF110", {
        font: fonts.fontAwesome,
        color: 'grey',
        position: [0, 0, 0],
        center_x: 0,
        center_y: 0,
        scale: scale,
        seat_circle_radius: seat_circle_radius,
        scene: seat,
        size: seat_circle_radius * scale,
        rotation: rotation
      }, textGeometries, materials);

      seat.unsetLoading = function () {
        return seat.remove(_label);
      };
    }
  };

  seat.statusUpdate = function (status) {
    if (seat.status !== status) {
      switch (status) {
        case SEAT_STATUSES.PERMANENT_RESERVED:
        case SEAT_STATUSES.AVAILABLE:
          {
            makeAvailable();
            seat.unsetLoading();
            break;
          }

        case SEAT_STATUSES.HOLD:
          {
            makeHold();
            seat.unsetLoading();
            break;
          }

        case SEAT_STATUSES.RESERVED_BY_ME:
          {
            makeReservedByMe();
            seat.unsetLoading();
            break;
          }

        case SEAT_STATUSES.RESERVED:
          {
            makeReserved();
            seat.unsetLoading();
            break;
          }

        case SEAT_STATUSES.BOOKED:
          {
            makeBooked();
            seat.unsetLoading();
            break;
          }

        case SEAT_STATUSES.BLOCKED:
          {
            makeBlocked();
            seat.unsetLoading();
            break;
          }
      }

      seat.setColorByStatus();
    }
  };

  seats[seatId] = seat;
  seat.changeOpacity(opacity, seatColor);
  seat.hasHandlers = true;
  sceneMeshes.push(seat);
  parent.add(seat);
};

var drawTableCircle = function drawTableCircle(centerPos, color, radius, group, opacity, sceneMeshes) {
  if (opacity === void 0) {
    opacity = 1;
  }

  var geometry = new CircleGeometry(radius, 32, 0, 2 * Math.PI);
  var obj = new Mesh(geometry, new MeshBasicMaterial({
    color: color,
    opacity: opacity,
    transparent: true
  }));
  obj.position.set(centerPos.x, centerPos.y, centerPos.z);
  var triangle = new Group();
  triangle.add(obj);
  triangle.obj = obj;
  sceneMeshes.push(triangle);

  if (group.renderOrder) {
    triangle.renderOrder = 2;
  }

  return [triangle];
};

var drawRect$1 = function drawRect(group, options) {
  var color = options.color,
      borderColor = options.borderColor,
      position = options.position,
      scale = options.scale,
      _options$opacity = options.opacity,
      opacity = _options$opacity === void 0 ? 1 : _options$opacity,
      sceneMeshes = options.sceneMeshes;
  var height = group.height,
      width = group.width;
  var geometry = new PlaneGeometry(width * scale, height * scale);
  var material = new MeshBasicMaterial({
    color: color,
    opacity: opacity,
    transparent: true
  });
  var rect = new Mesh(geometry, material);

  if (borderColor) {
    var lineBasicMaterial = new LineBasicMaterial({
      color: borderColor
    });
    var lineGeometry = new BufferGeometry();
    var geometryArray = getGeometryArray(geometry);
    var vertices = new Float32Array(geometryArray);
    geometry.setAttribute('position', new BufferAttribute(vertices, 3));
    var line = new Line(lineGeometry, lineBasicMaterial);
    rect.add(line);
  }

  rect.position.set(position.x, position.y, position.z);
  var triangle = new Group();
  triangle.add(rect);
  triangle.obj = rect;
  sceneMeshes.push(triangle);
  return [triangle];
};

var drawIcon$1 = function drawIcon(content, options) {
  var _label$position;

  var font = options.font,
      color = options.color,
      position = options.position,
      center_x = options.center_x,
      center_y = options.center_y,
      scale = options.scale,
      scene = options.scene,
      rotation = options.rotation,
      width = options.width,
      height = options.height,
      pSize = options.size;
  position[0] -= center_x * scale;
  position[1] += center_y * scale;
  var geometry = new PlaneGeometry(width * scale, height * scale);
  var material = new MeshBasicMaterial({
    color: color
  });
  var rect = new Mesh(geometry, material);
  rect.position.set(position.x, position.y, position.z);
  var labelGeometry = new TextGeometry(content, {
    font: font,
    size: pSize || 16 * scale,
    height: 0
  });
  labelGeometry.center();
  var labelMaterial = new MeshBasicMaterial({
    color: color
  });
  var label = new Mesh(labelGeometry, labelMaterial);

  (_label$position = label.position).set.apply(_label$position, position);

  label.renderOrder = 1;
  var triangle = new Group();
  triangle.add(rect);
  triangle.obj = rect;
  triangle.add(label);
  triangle.obj = label;

  if (rotation) {
    triangle.rotateOnAxis(new Vector3(0, 0, 1), rotation * Math.PI / 180);
  }

  scene.add(triangle);
  return triangle;
};

var drawTableSeats = function drawTableSeats(seatId, options, seats, materials, textGeometries, group) {
  var _seat$position;

  var scene = options.scene,
      color = options.color,
      position = options.position,
      rotation = options.rotation,
      tierId = options.tierId,
      parent = options.parent,
      center_x = options.center_x,
      center_y = options.center_y,
      scale = options.scale,
      seat_circle_radius = options.seat_circle_radius,
      events = options.events,
      sectionLabel = options.sectionLabel,
      rowLabel = options.rowLabel,
      label = options.label,
      fonts = options.fonts,
      _options$disabled = options.disabled,
      disabled = _options$disabled === void 0 ? false : _options$disabled,
      _options$type = options.type,
      type = _options$type === void 0 ? SEAT_TYPES.CHAIR : _options$type,
      isExternalLink = options.isExternalLink,
      seatExternalLink = options.seatExternalLink,
      enableTicketTypeAssign = options.enableTicketTypeAssign,
      tierColor = options.tierColor,
      selectedTierId = options.selectedTierId,
      sceneMeshes = options.sceneMeshes;
  var _options$opacity = options.opacity,
      opacity = _options$opacity === void 0 ? 1 : _options$opacity;
  var seatColor = enableTicketTypeAssign ? tierColor : COLORS.BOOKED;
  position[0] -= center_x * scale;
  position[1] += center_y * scale;
  var radius = (seat_circle_radius || 14 - 2) * scale;
  var seat = new Group();

  if (selectedTierId && selectedTierId !== tierId) {
    opacity = 0.3;
  }

  (_seat$position = seat.position).set.apply(_seat$position, position);

  var tableRadius = 2.5 * radius;

  var _ref = group.seatShape === 'rect' ? drawRect$1({
    width: group.width,
    height: group.height
  }, {
    color: seatColor,
    position: seat.position,
    scale: scale,
    opacity: opacity,
    sceneMeshes: sceneMeshes
  }) : drawTableCircle(seat.position, isExternalLink ? COLORS.AVAILABLE : seatColor, tableRadius, group, opacity, sceneMeshes),
      triangle = _ref[0];

  if (rotation) {
    triangle.rotateOnAxis(new Vector3(0, 0, 1), rotation * Math.PI / 180);
  }

  var labelText = group.label || '';
  var labelPosition = [center_x + group.x * scale + group.label_x * scale, center_y - group.y * scale + group.label_y * scale, 0];
  drawLabel(labelText, {
    font: fonts.openSans,
    color: 'black',
    position: labelPosition,
    center_x: center_x,
    center_y: center_y,
    scale: scale,
    seat_circle_radius: seat_circle_radius,
    scene: scene,
    size: 12
  }, textGeometries, materials);

  if (type === SEAT_TYPES.WHEELCHAIR) {
    drawIcon$1("\uF193", {
      font: fonts.fontAwesome,
      color: 'black',
      position: position,
      center_x: center_x,
      center_y: center_y,
      scale: scale,
      seat_circle_radius: seat_circle_radius,
      scene: seat,
      size: group.width * scale / 2,
      width: group.width,
      height: group.height,
      rotation: rotation
    });
  }

  seat.hasHandlers = true;
  seat.add(triangle);
  sceneMeshes.push(seat);
  seat.isRequestPending = false;
  seat.objectType = OBJECT_TYPES.SEAT;
  seat.color = seatColor;
  seat.opacity = opacity;
  seat.disabled = disabled;
  seat.allowColorUpdateByStatus = true;
  seat.radius = radius;
  seat.seatId = seatId;
  seat.tierId = tierId;
  seat.status = SEAT_STATUSES.BOOKED;
  seat.sectionLabel = sectionLabel;
  seat.rowLabel = rowLabel;
  seat.label = label;
  seat.seatType = type;
  seat.isExternalLink = isExternalLink;
  seat.seatExternalLink = seatExternalLink;
  seat.material = triangle.obj.material;

  seat.changeBorderColor = function (color) {
    triangle.material = getLineMaterial({
      color: color
    }, materials);
  };

  seat.mouseOver = seat.disabled ? null : events.onMouseOver;
  seat.mouseOut = seat.disabled ? null : events.onMouseOut;

  seat.changeBorderColorToDefault = function () {
    seat.changeBorderColor(COLORS.BOOKED);
  };

  seat.changeColor = function (color, opacity) {
    if (opacity === void 0) {
      opacity = 1;
    }

    seat.changeBorderColor(color);
    triangle.material = getSeatMaterial({
      color: color,
      opacity: opacity
    }, materials);
    triangle.obj.material = getSeatMaterial({
      color: color,
      opacity: opacity
    }, materials);

    seat.changeBorderColorToDefault = function () {
      seat.changeBorderColor(color);
    };

    seat.material = triangle.obj.material;
  };

  var makeAvailable = function makeAvailable() {
    seat.status = SEAT_STATUSES.AVAILABLE;
  };

  var makeHold = function makeHold() {
    seat.status = SEAT_STATUSES.HOLD;
  };

  var makeReserved = function makeReserved() {
    seat.status = SEAT_STATUSES.RESERVED;
  };

  var makeBooked = function makeBooked() {
    seat.status = SEAT_STATUSES.BOOKED;
  };

  var makeBlocked = function makeBlocked() {
    seat.status = SEAT_STATUSES.BLOCKED;
  };

  var makeReservedByMe = function makeReservedByMe() {
    window.localStorage.setItem('tierId', tierId);
    seat.status = SEAT_STATUSES.RESERVED_BY_ME;
  };

  seat.setColorByStatus = function () {
    if (!seat.allowColorUpdateByStatus || seat.isExternalLink) {
      return;
    }

    switch (seat.status) {
      case SEAT_STATUSES.AVAILABLE:
        {
          seat.changeColor(color);
          break;
        }

      case SEAT_STATUSES.HOLD:
        {
          seat.changeColor(COLORS.HOLD);
          break;
        }

      case SEAT_STATUSES.RESERVED_BY_ME:
        {
          seat.changeColor(COLORS.RESERVED_BY_ME);
          break;
        }

      case SEAT_STATUSES.RESERVED:
        {
          seat.changeColor(COLORS.RESERVED);
          break;
        }

      case SEAT_STATUSES.BOOKED:
        {
          seat.changeColor(COLORS.BOOKED);
          break;
        }

      case SEAT_STATUSES.BLOCKED:
        {
          seat.changeColor(COLORS.BLOCKED);
          break;
        }
    }
  };

  seat.changeOpacity = function (opacity, color) {
    seat.changeColor(color, opacity);
  };

  seat.makeDisabled = function () {
    if (seat.disabled) {
      return;
    }

    seat.disabled = true;
    seat.mouseOver = null;
    seat.mouseOut = null;
    seat.opacity = 0.3;
    seat.changeOpacity(0.3, seat.color);
  };

  seat.makeActive = function () {
    if (!seat.disabled) {
      return;
    }

    seat.disabled = false;
    seat.mouseOver = events.onMouseOver;
    seat.mouseOut = events.onMouseOut;
    seat.opacity = 1;
    seat.changeOpacity(1, seat.color);
  };

  var select = function select() {
    triangle.material = getLineMaterial({
      color: '#575656',
      linewidth: 5,
      opacity: seat.opacity
    }, materials);

    seat.changeBorderColor = function () {
      return null;
    };
  };

  var unselect = function unselect() {
    triangle.material = getLineMaterial({
      color: '#000',
      opacity: seat.opacity
    }, materials);

    seat.changeBorderColor = function (color) {
      triangle.material = getLineMaterial({
        color: color,
        opacity: seat.opacity
      }, materials);
    };

    seat.changeBorderColorToDefault();
  };

  seat.select = select;
  seat.unselect = unselect;

  seat.unsetLoading = function () {
    return null;
  };

  seat.setLoading = function () {
    if (window.isMobile) {
      var _label = drawIcon$1("\uF110", {
        font: fonts.fontAwesome,
        color: 'grey',
        position: [seat.position.x, seat.position.y, seat.position.z],
        center_x: 0,
        center_y: 0,
        scale: scale,
        seat_circle_radius: seat_circle_radius,
        scene: seat,
        size: seat_circle_radius * scale,
        width: group.width,
        height: group.height,
        rotation: rotation
      });

      seat.unsetLoading = function () {
        return seat.remove(_label);
      };
    }
  };

  seat.statusUpdate = function (status) {
    if (seat.status !== status) {
      switch (status) {
        case SEAT_STATUSES.PERMANENT_RESERVED:
        case SEAT_STATUSES.AVAILABLE:
          {
            makeAvailable();
            seat.unsetLoading();
            break;
          }

        case SEAT_STATUSES.HOLD:
          {
            makeHold();
            seat.unsetLoading();
            break;
          }

        case SEAT_STATUSES.RESERVED_BY_ME:
          {
            makeReservedByMe();
            seat.unsetLoading();
            break;
          }

        case SEAT_STATUSES.RESERVED:
          {
            makeReserved();
            seat.unsetLoading();
            break;
          }

        case SEAT_STATUSES.BOOKED:
          {
            makeBooked();
            seat.unsetLoading();
            break;
          }

        case SEAT_STATUSES.BLOCKED:
          {
            makeBlocked();
            seat.unsetLoading();
            break;
          }
      }

      seat.setColorByStatus();
    }
  };

  seats[seatId] = seat;
  seat.changeOpacity(opacity, seatColor);
  parent.add(seat);
};

var drawTable = function drawTable(group, groups, options, circlePartsCount) {
  var scene = options.scene,
      scale = options.scale,
      fonts = options.fonts,
      center_x = options.center_x,
      center_y = options.center_y,
      seat_circle_radius = options.seat_circle_radius,
      events = options.events,
      enableTicketTypeAssign = options.enableTicketTypeAssign,
      selectedTierId = options.selectedTierId,
      sceneMeshes = options.sceneMeshes;
  var groupObj = new Group();
  group.x = group.x || 0;
  group.y = -group.y || 0;
  groupObj.position.set(group.x * scale, group.y * scale, 0);
  scene.add(groupObj);
  var table = group.children[0];
  table.children.forEach(function (seat, seatIndex) {
    var tierId = groups[group.id].groups[0].units[seatIndex].tier;
    var seatLabelText = groups[group.id].groups[0].units[seatIndex].label;
    var seatExternalLink = groups[group.id].groups[0].units[seatIndex].external_link;
    var faceTo = seatIndex === 0 ? 'left' : 'right';

    if (!tierColorObj[tierId]) {
      tierColorObj[tierId] = TIER_COLORS[colorIndex.value];
      colorIndex.value++;
    }

    switch (circlePartsCount) {
      case 2:
        drawTable2Seats(seat.id, {
          color: COLORS.AVAILABLE,
          position: [seat.cx * scale, -seat.cy * scale, 0],
          rotation: 0,
          tierId: tierId,
          scene: scene,
          center_x: center_x,
          center_y: center_y,
          scale: scale,
          seat_circle_radius: seat_circle_radius - 2,
          parent: groupObj,
          events: events,
          sectionLabel: groups[group.id].label,
          rowLabel: groups[group.id].groups[0].label,
          label: seatLabelText,
          fonts: fonts,
          type: seat.type,
          faceTo: faceTo,
          enableTicketTypeAssign: enableTicketTypeAssign,
          tierColor: tierColorObj[tierId],
          selectedTierId: selectedTierId,
          sceneMeshes: sceneMeshes
        });
        break;

      default:
        drawTableSeats(seat.id, {
          color: COLORS.AVAILABLE,
          position: [seat.cx * scale, -seat.cy * scale, 0],
          rotation: group.rotation || 0,
          tierId: tierId,
          scene: scene,
          center_x: center_x,
          center_y: center_y,
          scale: scale,
          seat_circle_radius: seat_circle_radius - 2,
          parent: groupObj,
          events: events,
          sectionLabel: groups[group.id].label,
          rowLabel: groups[group.id].groups[0].label,
          label: seatLabelText,
          fonts: fonts,
          type: seat.type,
          isExternalLink: group.isExternalLink,
          seatExternalLink: seatExternalLink,
          enableTicketTypeAssign: enableTicketTypeAssign,
          tierColor: tierColorObj[tierId],
          selectedTierId: selectedTierId,
          sceneMeshes: sceneMeshes
        }, seats, materials, textGeometries, group);
        break;
    }
  });
};

var findObjectWithHandlers = function findObjectWithHandlers(meshObject) {
  if (!meshObject) return null;

  if (meshObject.object) {
    if (meshObject.object.hasHandlers) {
      return meshObject.object;
    }

    return findObjectWithHandlers(meshObject.object.parent);
  } else if (meshObject.hasHandlers) {
    return meshObject;
  } else {
    return findObjectWithHandlers(meshObject.parent);
  }
};

var raycastMouseEvents = function raycastMouseEvents(_ref) {
  var camera = _ref.camera,
      renderer = _ref.renderer,
      sceneMeshes = _ref.sceneMeshes,
      events = _ref.events;
  var raycaster = new Raycaster();
  var lastIntersected = null;
  var lastIntersectedTap = null;

  var clickHandler = function clickHandler(event) {
    var mouse = {
      x: event.offsetX / renderer.domElement.clientWidth * 2 - 1,
      y: -(event.offsetY / renderer.domElement.clientHeight) * 2 + 1
    };
    var rect = renderer.domElement.getBoundingClientRect();
    var positionOfSeat = {
      x: event.clientX - rect.x,
      y: event.clientY - rect.y
    };
    raycaster.setFromCamera(mouse, camera);
    var intersected = raycaster.intersectObjects(sceneMeshes, true);

    if (intersected[0]) {
      if (window.isMobile) {
        if (lastIntersectedTap) {
          events.onMouseOut({
            seat: lastIntersectedTap
          });
        }

        events.onTap({
          seat: findObjectWithHandlers(intersected[0]),
          position: positionOfSeat
        });
        lastIntersectedTap = findObjectWithHandlers(intersected[0]);
      } else {
        events.onClick({
          seat: findObjectWithHandlers(intersected[0])
        });
      }
    }
  };

  var mouseOverHandler = function mouseOverHandler(event) {
    var mouse = {
      x: event.offsetX / renderer.domElement.clientWidth * 2 - 1,
      y: -(event.offsetY / renderer.domElement.clientHeight) * 2 + 1
    };
    var rect = renderer.domElement.getBoundingClientRect();
    var positionOfSeat = {
      x: event.clientX - rect.x,
      y: event.clientY - rect.y
    };
    raycaster.setFromCamera(mouse, camera);
    var intersected = raycaster.intersectObjects(sceneMeshes, true);

    if (intersected[0] && !lastIntersected) {
      lastIntersected = findObjectWithHandlers(intersected[0]);
      events.onMouseOver({
        seat: findObjectWithHandlers(intersected[0]),
        position: positionOfSeat
      });
    } else if (intersected[0] && lastIntersected) {
      events.onMouseOut({
        seat: lastIntersected
      });
      lastIntersected = findObjectWithHandlers(intersected[0]);
      events.onMouseOver({
        seat: findObjectWithHandlers(intersected[0]),
        position: positionOfSeat
      });
    } else if (!intersected[0] && lastIntersected) {
      events.onMouseOut({
        seat: lastIntersected
      });
      lastIntersected = null;
    }
  };

  return {
    handleClick: clickHandler,
    handleMouseOver: mouseOverHandler
  };
};

var drawMap = function drawMap(seatmap, groups, creatorInfo, predefinedSeats, options) {
  try {
    var _temp5 = function _temp5() {
      var mouseHandlers = raycastMouseEvents({
        camera: camera,
        renderer: renderer,
        sceneMeshes: sceneMeshes,
        events: events
      });
      var canvasContainer = document.getElementById('seatCanvasContainer');

      if (canvasContainer) {
        canvasContainer.addEventListener('mousemove', mouseHandlers.handleMouseOver);
        canvasContainer.addEventListener('pointerdown', mouseHandlers.handleClick);
      }
    };

    var scene = options.scene,
        scale = options.scale,
        fonts = options.fonts,
        center_x = options.center_x,
        center_y = options.center_y,
        seat_circle_radius = options.seat_circle_radius,
        events = options.events,
        isBlockMap = options.isBlockMap,
        enableTicketTypeAssign = options.enableTicketTypeAssign,
        selectedTierId = options.selectedTierId,
        renderer = options.renderer,
        camera = options.camera;
    seatmap = _cloneDeep(seatmap);
    var sceneMeshes = [];
    resetColorsData();

    var _temp6 = _forOf(seatmap, function (group) {
      var _temp2 = function () {
        if (group.deleted === 0) {
          var _temp7 = _switch(group.type, [[function () {
            return 'section';
          }, function () {
            {
              isBlockMap ? drawGroupBlock(group, groups, creatorInfo, _extends({
                sceneMeshes: sceneMeshes
              }, options)) : drawGroupSeats(group, groups, predefinedSeats, _extends({
                sceneMeshes: sceneMeshes
              }, options));
              return;
            }
          }], [function () {
            return 'with_svg';
          }, function () {
            {
              return Promise.resolve(drawMapWithSVG(group, groups, _extends({
                sceneMeshes: sceneMeshes
              }, options), predefinedSeats)).then(function () {});
            }
          }], [function () {
            return 'stencil';
          }, function () {
            {
              drawRect(group, {
                color: group.bg_color || COLORS.STAGE,
                borderColor: group.border_color || COLORS.STAGE_BORDER,
                scale: scale,
                scene: scene,
                center_x: center_x,
                center_y: center_y
              });
              var iconCode = group.stencil === 'ring' || group.stencil === 'dance_floor' ? null : group.stencil === 'basketball' ? "\uF434" : group.stencil === 'dj' ? "\uF025" : "\uF130";
              var iconPosition = iconCode ? [group.x * scale, -group.y * scale, 0] : [0, 0];
              iconCode && drawIcon(iconCode, {
                font: fonts.fontAwesome,
                color: 0x6a727a,
                position: iconPosition,
                center_x: center_x,
                center_y: center_y,
                scale: scale,
                scene: scene
              });
              var labelText = group.label || 'Stage';
              var labelPosition = !_isUndefined(group.label_x) && !_isUndefined(group.label_y) ? [group.label_x * scale, group.label_y * scale, 0] : [group.x * scale, -(group.y + 24) * scale, 0];
              drawLabel(labelText, {
                font: fonts.openSans,
                color: 'black',
                position: labelPosition,
                center_x: center_x,
                center_y: center_y,
                scale: scale,
                seat_circle_radius: seat_circle_radius,
                scene: scene,
                size: group.label_size * (scale || 1) || 12,
                rotation: group.label_rotation || 0
              }, textGeometries, materials);
              return;
            }
          }], [function () {
            return 'standing';
          }, function () {
            {
              var _groups$group$id = groups[group.id],
                  tierId = _groups$group$id.tier,
                  label = _groups$group$id.label;

              if (!tierColorObj[tierId]) {
                tierColorObj[tierId] = TIER_COLORS[colorIndex.value];
                colorIndex.value++;
              }

              drawStandingArea(group, {
                color: COLORS.AVAILABLE,
                scale: scale,
                scene: scene,
                opacity: 1,
                center_x: center_x,
                center_y: center_y,
                events: events,
                tierId: tierId,
                label: label,
                materialGroup: groups[group.id],
                id: group.id,
                fonts: fonts,
                seat_circle_radius: seat_circle_radius,
                enableTicketTypeAssign: enableTicketTypeAssign,
                tierColor: tierColorObj[tierId],
                selectedTierId: selectedTierId,
                sceneMeshes: sceneMeshes
              });

              if (!group.hideSectionLabel && !group.labelPositions) {
                var labelText = groups[group.id].label;
                var height = group.height || 0;
                var innerHeight = group.inner_height || 0;
                var labelPosition = [group.x * scale, -group.y * scale, 0];

                if (innerHeight) {
                  var y = -(group.y + (innerHeight + (height - innerHeight) / 2) / 2) * scale;

                  if (group.rect_type === 'top_rect') {
                    y = -y;
                  }

                  labelPosition = [group.x * scale, y, 0];
                }

                if (group.rotation === 90 || group.rotation === -90) {
                  labelPosition = [group.y * scale * group.rotation / 90, group.x * scale * group.rotation / 90, 0];
                }

                var labelX = group.label_x || 0;
                var labelY = group.label_y || 0;
                labelPosition = [labelPosition[0] + labelX * scale, labelPosition[1] + labelY * scale, 0];
                drawLabel(labelText, {
                  font: fonts.openSans,
                  color: 'black',
                  position: labelPosition,
                  center_x: center_x,
                  center_y: center_y,
                  scale: scale,
                  seat_circle_radius: seat_circle_radius,
                  scene: scene,
                  opacity: 0.9,
                  size: group.size || 12,
                  rotation: group.label_rotation
                }, textGeometries, materials);
              }

              return;
            }
          }], [function () {
            return 'general_admission_section';
          }, function () {
            {
              var rectColor, iconCode, iconColor;

              if (group.seatingType === 'seated') {
                rectColor = '#EC00BE';
                iconColor = '#f8f8fa';
                iconCode = String.fromCharCode(0xf6c0);
              } else {
                rectColor = '#56D36A';
                iconColor = '#6a727a';
                iconCode = String.fromCharCode(0xf183);
              }

              drawRect(group, {
                color: rectColor,
                scale: scale,
                scene: scene,
                center_x: center_x,
                center_y: center_y
              });
              var iconPosition = [group.x * scale, -group.y * scale, 0];
              drawIcon(iconCode, {
                font: fonts.fontAwesome,
                color: iconColor,
                position: iconPosition,
                center_x: center_x,
                center_y: center_y,
                scale: scale,
                scene: scene
              });
              var labelText = groups[group.id].label;
              var labelPosition = [group.x * scale, -(group.y + 24) * scale, 0];
              drawLabel(labelText, {
                font: fonts.openSans,
                color: 'black',
                position: labelPosition,
                center_x: center_x,
                center_y: center_y,
                scale: scale,
                seat_circle_radius: seat_circle_radius,
                scene: scene
              }, textGeometries, materials);
              return;
            }
          }], [function () {
            return 'table_2';
          }, function () {
            {
              drawTable(group, groups, _extends({
                sceneMeshes: sceneMeshes
              }, options), 2);
              return;
            }
          }], [function () {
            return 'table_circle';
          }, function () {
            {
              drawTable(group, groups, _extends({
                sceneMeshes: sceneMeshes
              }, options));
              return;
            }
          }], [void 0, function () {
            console.log(group.type);
          }]]);

          if (_temp7 && _temp7.then) return _temp7.then(function () {});
        }
      }();

      if (_temp2 && _temp2.then) return _temp2.then(function () {});
    });

    return Promise.resolve(_temp6 && _temp6.then ? _temp6.then(_temp5) : _temp5(_temp6));
  } catch (e) {
    return Promise.reject(e);
  }
};

var arrayToObject = function arrayToObject(array) {
  return array.reduce(function (obj, item) {
    obj[item.id] = item;
    return obj;
  }, {});
};

var activateGroups = function activateGroups(groups) {
  for (var _i = 0, _Object$entries = Object.entries(seats); _i < _Object$entries.length; _i++) {
    var _Object$entries$_i = _Object$entries[_i],
        seatId = _Object$entries$_i[0],
        seat = _Object$entries$_i[1];

    var _seatId$split = seatId.split('-'),
        groupId = _seatId$split[0];

    if (!groups || groups.length === 0 || groups.includes(groupId)) {
      Array.isArray(seat) ? seat.forEach(function (e) {
        return e.makeActive();
      }) : seat.makeActive();
    } else {
      Array.isArray(seat) ? seat.forEach(function (e) {
        return e.makeDisabled();
      }) : seat.makeDisabled();
    }
  }
};

var recolorSeats = function recolorSeats(seatsToRecolor, seatsToRecover) {
  if (seatsToRecover === void 0) {
    seatsToRecover = [];
  }

  seatsToRecolor.forEach(function (_ref) {
    var seatId = _ref.seatId,
        color = _ref.color;
    var meshes = seats[seatId];

    if (meshes && meshes.length) {
      meshes.forEach(function (meshItem) {
        meshItem.allowColorUpdateByStatus = false;
        meshItem.changeColor(color);
      });
    } else {
      seats[seatId].allowColorUpdateByStatus = false;
      seats[seatId].changeColor(color);
    }
  });
  seatsToRecover.forEach(function (_ref2) {
    var seatId = _ref2.seatId;
    var meshes = seats[seatId];

    if (meshes && meshes.length) {
      meshes.forEach(function (meshItem) {
        meshItem.allowColorUpdateByStatus = true;
        meshItem.setColorByStatus();
      });
    } else {
      seats[seatId].allowColorUpdateByStatus = true;
      seats[seatId].setColorByStatus();
    }
  });
};

var createMap = function createMap(container, seatmap, groups, creatorInfo, predefinedSeats, options) {
  try {
    groups = arrayToObject(groups);

    var _initThreeJsCanvas = initThreeJsCanvas(container, options),
        renderer = _initThreeJsCanvas.renderer,
        camera = _initThreeJsCanvas.camera,
        scene = _initThreeJsCanvas.scene,
        scale = _initThreeJsCanvas.scale,
        fonts = _initThreeJsCanvas.fonts,
        cancelAnimation = _initThreeJsCanvas.cancelAnimation,
        controlsReset = _initThreeJsCanvas.controlsReset,
        controlsZoomIn = _initThreeJsCanvas.controlsZoomIn,
        controlsZoomOut = _initThreeJsCanvas.controlsZoomOut,
        startAnimation = _initThreeJsCanvas.startAnimation;

    return Promise.resolve(drawMap(seatmap, groups, creatorInfo, predefinedSeats, _extends({}, options, {
      scene: scene,
      scale: scale,
      fonts: fonts,
      renderer: renderer,
      camera: camera
    }))).then(function () {
      var updateStatuses = function updateStatuses(statuses) {
        standingAreaStatusesUpdate(statuses, standingAreas);
        return options.isBlockMap ? blocksStatusesUpdate(statuses, blocks) : statusesUpdate(statuses, seats);
      };

      if (!options.enableTicketTypeAssign) {
        updateStatuses(options.statuses);
      }

      return {
        activateGroups: activateGroups,
        recolorSeats: recolorSeats,
        cancelAnimation: cancelAnimation,
        controlsReset: controlsReset,
        controlsZoomIn: controlsZoomIn,
        controlsZoomOut: controlsZoomOut,
        startAnimation: startAnimation,
        statusesUpdate: updateStatuses
      };
    });
  } catch (e) {
    return Promise.reject(e);
  }
};

var ColorLabels = function ColorLabels(_ref) {
  var showStatusesLabel = _ref.showStatusesLabel,
      isTableMap = _ref.isTableMap,
      isSVGMap = _ref.isSVGMap,
      svgData = _ref.svgData;
  return /*#__PURE__*/React__default.createElement("div", {
    className: "colorLabelsContainer",
    style: {
      color: isSVGMap ? 'white' : 'black'
    }
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "colorLabelLine",
    style: {
      backgroundColor: svgData.bgColor || 'transparent'
    }
  }, showStatusesLabel[SEAT_STATUSES.HOLD] && /*#__PURE__*/React__default.createElement("div", {
    className: "colorLabel",
    style: {
      background: svgData.bgColor || 'white'
    }
  }, /*#__PURE__*/React__default.createElement("div", {
    style: {
      backgroundColor: "" + COLORS.HOLD
    },
    className: "colorLabelColor"
  }), /*#__PURE__*/React__default.createElement("span", {
    className: "colorLabelText"
  }, "On Hold")), showStatusesLabel[SEAT_STATUSES.BLOCKED] && /*#__PURE__*/React__default.createElement("div", {
    className: "colorLabel",
    style: {
      background: svgData.bgColor || 'white'
    }
  }, /*#__PURE__*/React__default.createElement("div", {
    style: {
      backgroundColor: "" + COLORS.BLOCKED
    },
    className: "colorLabelColor"
  }), /*#__PURE__*/React__default.createElement("span", {
    className: "colorLabelText"
  }, "Blocked")), /*#__PURE__*/React__default.createElement("div", {
    className: "colorLabel",
    style: {
      background: svgData.bgColor || 'white'
    }
  }, /*#__PURE__*/React__default.createElement("div", {
    style: {
      backgroundColor: isSVGMap ? 'white' : "" + COLORS.AVAILABLE
    },
    className: "colorLabelColor"
  }), /*#__PURE__*/React__default.createElement("span", {
    className: "colorLabelText"
  }, "Available")), showStatusesLabel[SEAT_STATUSES.PREDEFINED] && /*#__PURE__*/React__default.createElement("div", {
    className: "colorLabel",
    style: {
      background: svgData.bgColor || 'white'
    }
  }, /*#__PURE__*/React__default.createElement("div", {
    style: {
      backgroundColor: "" + COLORS.PREDEFINED
    },
    className: "colorLabelColor"
  }), /*#__PURE__*/React__default.createElement("span", {
    className: "colorLabelText"
  }, "Predefined")), showStatusesLabel[SEAT_STATUSES.RESERVED] && /*#__PURE__*/React__default.createElement("div", {
    className: "colorLabel",
    style: {
      background: svgData.bgColor || 'white'
    }
  }, /*#__PURE__*/React__default.createElement("div", {
    style: {
      backgroundColor: "" + COLORS.RESERVED
    },
    className: "colorLabelColor"
  }), /*#__PURE__*/React__default.createElement("span", {
    className: "colorLabelText"
  }, isTableMap ? 'Locked for Purchase' : 'Reserved')), /*#__PURE__*/React__default.createElement("div", {
    className: "colorLabel",
    style: {
      background: svgData.bgColor || 'white'
    }
  }, /*#__PURE__*/React__default.createElement("div", {
    style: {
      backgroundColor: "" + COLORS.BOOKED
    },
    className: "colorLabelColor"
  }), /*#__PURE__*/React__default.createElement("span", {
    className: "colorLabelText"
  }, "Sold"))), showStatusesLabel[SEAT_TYPES.WHEELCHAIR] && /*#__PURE__*/React__default.createElement("div", {
    className: "colorLabelLine",
    style: {
      justifyContent: 'left',
      backgroundColor: '#dae5f1',
      width: '100%'
    }
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "colorLabel",
    style: {
      backgroundColor: svgData.bgColor || 'transparent',
      marginLeft: '8px'
    }
  }, /*#__PURE__*/React__default.createElement("div", {
    style: {
      backgroundColor: "transparent"
    },
    className: "colorLabelColor"
  }, /*#__PURE__*/React__default.createElement(FontAwesomeIcon, {
    className: "colorLabelColorWheelchairIcon",
    icon: faWheelchair
  })), /*#__PURE__*/React__default.createElement("span", {
    className: "colorLabelText"
  }, "Accessible seating is available"))));
};

var ColorLabelsPPP = function ColorLabelsPPP(_ref) {
  var showStatusesLabel = _ref.showStatusesLabel,
      _ref$colors = _ref.colors,
      colors = _ref$colors === void 0 ? {} : _ref$colors;
  return /*#__PURE__*/React__default.createElement("div", {
    className: "colorLabelsContainer"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "colorLabelLine colorLabelLine-ppp"
  }, showStatusesLabel[SEAT_STATUSES.BLOCKED] && /*#__PURE__*/React__default.createElement("div", {
    className: "colorLabel colorLabel-ppp"
  }, /*#__PURE__*/React__default.createElement(FontAwesomeIcon, {
    className: "colorLabelColorTimesIcon",
    icon: faTimes
  }), /*#__PURE__*/React__default.createElement("span", {
    className: "colorLabelText"
  }, "Blocked")), /*#__PURE__*/React__default.createElement("div", {
    className: "colorLabel colorLabel-ppp"
  }, /*#__PURE__*/React__default.createElement("div", {
    style: {
      backgroundColor: '#9bb4c0'
    },
    className: "colorLabelColor"
  }), /*#__PURE__*/React__default.createElement("div", {
    style: {
      backgroundColor: '#fff'
    },
    className: "colorLabelColor"
  }), /*#__PURE__*/React__default.createElement("span", {
    className: "colorLabelText"
  }, "Available")), showStatusesLabel[SEAT_STATUSES.RESERVED] && /*#__PURE__*/React__default.createElement("div", {
    className: "colorLabel colorLabel-ppp"
  }, /*#__PURE__*/React__default.createElement("div", {
    style: {
      backgroundColor: "" + (colors.RESERVED || COLORS.RESERVED)
    },
    className: "colorLabelColor"
  }), /*#__PURE__*/React__default.createElement("span", {
    className: "colorLabelText"
  }, "Reserved")), /*#__PURE__*/React__default.createElement("div", {
    className: "colorLabel colorLabel-ppp"
  }, /*#__PURE__*/React__default.createElement("div", {
    style: {
      backgroundColor: "" + (colors.BOOKED || COLORS.BOOKED)
    },
    className: "colorLabelColor"
  }), /*#__PURE__*/React__default.createElement("span", {
    className: "colorLabelText"
  }, "Sold"))));
};

var ControlButtons = function ControlButtons(props) {
  return /*#__PURE__*/React__default.createElement("div", {
    className: "controlButtonsContainer",
    style: {
      bottom: props.isWheelChairAvailable ? '30px' : '10px'
    }
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "controlButton",
    onClick: props.reset
  }, "\u21BB"), /*#__PURE__*/React__default.createElement("div", {
    className: "controlButton",
    onClick: props.zoomIn
  }, "+"), /*#__PURE__*/React__default.createElement("div", {
    className: "controlButton",
    onClick: props.zoomOut
  }, "-"));
};

var StandingAreaLabel = function StandingAreaLabel(props) {
  var price = props.price,
      status = props.status,
      section = props.section,
      color = props.color,
      top = props.top,
      left = props.left,
      visible = props.visible,
      showReserveButton = props.showReserveButton,
      initializeReservationButtonListeners = props.initializeReservationButtonListeners,
      tierNames = props.tierNames,
      initializeListeners = props.initializeListeners,
      _props$seatLabelPrice = props.seatLabelPriceText,
      seatLabelPriceText = _props$seatLabelPrice === void 0 ? 'Price' : _props$seatLabelPrice;
  var showPrice = seatLabelPriceText && price;
  return visible ? /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelContainer",
    style: {
      top: top,
      left: left
    },
    ref: initializeListeners
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelRect"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelHeader"
  }, /*#__PURE__*/React__default.createElement(FontAwesomeIcon, {
    className: "seatLabelIcon",
    icon: faMale
  }), /*#__PURE__*/React__default.createElement("span", null, "Section Information")), /*#__PURE__*/React__default.createElement("div", {
    className: classNames({
      'seatLabelContent seatLabelContent-large': showReserveButton,
      seatLabelContent: !showReserveButton
    })
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelRow"
  }, showPrice && /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItem"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, seatLabelPriceText), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemText"
  }, price)), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItem"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, "Status"), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemText",
    style: {
      color: color
    }
  }, status))), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelRow"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: classNames('seatLabelItem', 'flex2')
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, "Section"), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemText"
  }, section + (tierNames ? " (" + tierNames + ")" : '')))), showReserveButton && /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelRow"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItem"
  }, /*#__PURE__*/React__default.createElement("button", {
    className: "seatLabelButton",
    ref: initializeReservationButtonListeners
  }, "Reserve"))))), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelTriangle"
  })) : null;
};

var SeatLabel = function SeatLabel(props) {
  var price = props.price,
      row = props.row,
      seat = props.seat,
      status = props.status,
      section = props.section,
      color = props.color,
      top = props.top,
      left = props.left,
      visible = props.visible,
      showReserveButton = props.showReserveButton,
      tierNames = props.tierNames,
      _props$seatLabelPrice = props.seatLabelPriceText,
      seatLabelPriceText = _props$seatLabelPrice === void 0 ? 'Price' : _props$seatLabelPrice,
      buyerInfo = props.buyerInfo,
      initializeListeners = props.initializeListeners,
      initializeReservationButtonListeners = props.initializeReservationButtonListeners,
      _props$seatType = props.seatType,
      seatType = _props$seatType === void 0 ? SEAT_TYPES.CHAIR : _props$seatType;
  var showPrice = seatLabelPriceText && price;
  return visible ? /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelContainer",
    style: {
      top: top,
      left: left
    },
    ref: initializeListeners
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelRect"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelHeader"
  }, /*#__PURE__*/React__default.createElement(FontAwesomeIcon, {
    className: "seatLabelIcon",
    icon: seatType === SEAT_TYPES.WHEELCHAIR ? faWheelchair : faChair
  }), /*#__PURE__*/React__default.createElement("span", null, "Seat Information")), /*#__PURE__*/React__default.createElement("div", {
    className: classNames({
      'seatLabelContent seatLabelContent-large': showReserveButton,
      seatLabelContent: !showReserveButton
    })
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelRow"
  }, showPrice && /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItem"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, seatLabelPriceText), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemText"
  }, price)), Boolean((row || '').trim()) && /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItem"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, "Row"), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemText"
  }, row || seat.match(/[a-zA-Z]+/g))), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItem"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, "Seat No."), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemText"
  }, row ? seat : seat.match(/\d+/g)))), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelRow"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItem"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, "Status"), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemText",
    style: {
      color: color
    }
  }, status)), /*#__PURE__*/React__default.createElement("div", {
    className: classNames({
      'seatLabelItem flex2': showPrice,
      seatLabelItem: !showPrice
    })
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, "Section"), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemText"
  }, section + (tierNames ? " (" + tierNames + ")" : '')))), showReserveButton && /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelRow"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItem"
  }, /*#__PURE__*/React__default.createElement("button", {
    className: "seatLabelButton",
    ref: initializeReservationButtonListeners
  }, "Reserve")))), buyerInfo && /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelContent withBuyerInfo"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelRow"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItem flex3"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, "Buyer"), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemText"
  }, buyerInfo.name))), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelRow"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItem flex3"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, "Buyer Email"), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemText"
  }, buyerInfo.email))), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelRow"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItem flex3"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, "Order ID"), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemText"
  }, buyerInfo.order)), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItem flex3"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, "Ticket Hash"), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemText"
  }, buyerInfo.ticket))))), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelTriangle"
  })) : null;
};

var _g;

function _extends$1() {
  _extends$1 = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };
  return _extends$1.apply(this, arguments);
}

function SvgSofa(props) {
  return /*#__PURE__*/createElement("svg", _extends$1({
    xmlns: "http://www.w3.org/2000/svg",
    viewBox: "0 0 36.13 23.28"
  }, props), _g || (_g = /*#__PURE__*/createElement("g", {
    "data-name": "Layer 2"
  }, /*#__PURE__*/createElement("g", {
    "data-name": "Layer 1"
  }, /*#__PURE__*/createElement("path", {
    d: "M26.2 15.17c-.1 3.6-4.2 5.7-9.2 5.7S7.9 18.57 7.9 15s4.2-5.7 9.2-5.7 9.1 2.3 9.1 5.9",
    fill: "#fafafa"
  }), /*#__PURE__*/createElement("path", {
    d: "M1.1 22.67l3.8-.9s-.2-8.2 1.3-10.7c3.4-5.6 11.7-4.8 15.6-3.8A8.78 8.78 0 0127.2 11l4.1 9.4 4.2-1.1s-2.4-8-3.6-11.4S24.5 1 18.5.57c-7-.5-14.5 1.8-16.8 6.7s-.6 15.4-.6 15.4z",
    fill: "none",
    stroke: "#c0c2c4",
    strokeMiterlimit: 10
  })))));
}

var _g$1;

function _extends$2() {
  _extends$2 = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };
  return _extends$2.apply(this, arguments);
}

function SvgTableDiamond(props) {
  return /*#__PURE__*/createElement("svg", _extends$2({
    xmlns: "http://www.w3.org/2000/svg",
    viewBox: "0 0 14.2 14.2"
  }, props), _g$1 || (_g$1 = /*#__PURE__*/createElement("g", {
    "data-name": "Layer 2"
  }, /*#__PURE__*/createElement("path", {
    d: "M7.1 0l7.1 7.1-7.1 7.1L0 7.1z",
    fill: "#9bb4c0",
    "data-name": "Layer 1"
  }))));
}

var _defs, _g$2;

function _extends$3() {
  _extends$3 = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };
  return _extends$3.apply(this, arguments);
}

function SvgTableBig(props) {
  return /*#__PURE__*/createElement("svg", _extends$3({
    xmlns: "http://www.w3.org/2000/svg",
    viewBox: "0 0 30.21 15.4"
  }, props), _defs || (_defs = /*#__PURE__*/createElement("defs", null, /*#__PURE__*/createElement("style", null, ".tableBig_svg__cls-3{fill:#fafafa}"))), _g$2 || (_g$2 = /*#__PURE__*/createElement("g", {
    id: "tableBig_svg__Layer_2",
    "data-name": "Layer 2"
  }, /*#__PURE__*/createElement("g", {
    id: "tableBig_svg__Layer_1-2",
    "data-name": "Layer 1"
  }, /*#__PURE__*/createElement("path", {
    d: "M.61 12.3l-.1-9.2h29.1l.1 9.2z",
    fill: "#fff"
  }), /*#__PURE__*/createElement("path", {
    d: "M.61 12.3l-.1-9.2h29.1l.1 9.2z",
    fill: "none",
    stroke: "#fff",
    strokeMiterlimit: 10
  }), /*#__PURE__*/createElement("path", {
    className: "tableBig_svg__cls-3",
    d: "M27.41 0H3.11l-.8 1.8 25-.1zM2.41 13.5l24.3.1.9 1.8-25.1-.2z"
  })))));
}

var _g$3;

function _extends$4() {
  _extends$4 = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };
  return _extends$4.apply(this, arguments);
}

function SvgTableSquare(props) {
  return /*#__PURE__*/createElement("svg", _extends$4({
    xmlns: "http://www.w3.org/2000/svg",
    viewBox: "0 0 18.7 17.7"
  }, props), _g$3 || (_g$3 = /*#__PURE__*/createElement("g", {
    "data-name": "Layer 2"
  }, /*#__PURE__*/createElement("g", {
    "data-name": "Layer 1"
  }, /*#__PURE__*/createElement("path", {
    d: "M18.2 17.2H.5V.5h17.7z",
    fill: "#f2f2f3"
  }), /*#__PURE__*/createElement("path", {
    d: "M18.2 17.2H.5V.5h17.7z",
    fill: "none",
    stroke: "#f2f2f3",
    strokeMiterlimit: 10
  })))));
}

var _g$4;

function _extends$5() {
  _extends$5 = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };
  return _extends$5.apply(this, arguments);
}

function SvgTableSquareBlue(props) {
  return /*#__PURE__*/createElement("svg", _extends$5({
    xmlns: "http://www.w3.org/2000/svg",
    viewBox: "0 0 12.4 12.4"
  }, props), _g$4 || (_g$4 = /*#__PURE__*/createElement("g", {
    "data-name": "Layer 2"
  }, /*#__PURE__*/createElement("path", {
    d: "M0 0h12.4v12.4H0z",
    fill: "#9bb4c0",
    "data-name": "Layer 1"
  }))));
}

var getTicketTypeIcon = function getTicketTypeIcon(ticket, ticketGroupName) {
  if (ticket === void 0) {
    ticket = {};
  }

  var _ticket = ticket,
      ticketTypeName = _ticket.ticket_type_name;

  switch (ticketTypeName || ticketGroupName) {
    case 'Booths':
      return /*#__PURE__*/React__default.createElement(SvgSofa, {
        className: "seatLabelIcon",
        width: 40
      });

    case 'Mesas | Tables':
      return /*#__PURE__*/React__default.createElement(SvgTableDiamond, {
        className: "seatLabelIcon",
        width: 30
      });

    case 'Playa | Beach':
    case 'Sala':
      return /*#__PURE__*/React__default.createElement(SvgTableBig, {
        className: "seatLabelIcon",
        width: 40
      });

    case 'Pista | Dance Floor':
      return /*#__PURE__*/React__default.createElement(SvgTableSquare, {
        className: "seatLabelIcon",
        width: 30
      });

    case 'Ocean Front Tables':
      return /*#__PURE__*/React__default.createElement(SvgTableSquareBlue, {
        className: "seatLabelIcon",
        width: 30
      });

    default:
      return /*#__PURE__*/React__default.createElement("span", {
        style: {
          padding: '0 5px'
        }
      });
  }
};

var getTicketTypeGroupAndName = function getTicketTypeGroupAndName(ticketTypeFullName) {
  if (ticketTypeFullName === void 0) {
    ticketTypeFullName = '';
  }

  var nameArray = (ticketTypeFullName || '').split(/\[|\]/);
  var ticketTypeGroupName = nameArray[1] || '';
  var ticketTypeName = nameArray[2] || '';
  return {
    ticketTypeName: ticketTypeName,
    ticketTypeGroupName: ticketTypeGroupName
  };
};

var SeatLabelMin = function SeatLabelMin(props) {
  var top = props.top,
      left = props.left,
      tierId = props.tierId,
      _props$ticketTypeTire = props.ticketTypeTireRelationsArray,
      ticketTypeTireRelationsArray = _props$ticketTypeTire === void 0 ? [] : _props$ticketTypeTire,
      _props$tierPrices = props.tierPrices,
      tierPrices = _props$tierPrices === void 0 ? {} : _props$tierPrices;

  var visibleTickets = _filter(ticketTypeTireRelationsArray, {
    ticket_type_tier_id: tierId
  });

  var _ref = visibleTickets[0] ? getTicketTypeGroupAndName(visibleTickets[0].ticket_type_name) : {},
      ticketTypeGroupName = _ref.ticketTypeGroupName;

  return /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelContainer",
    style: {
      top: top,
      left: left
    }
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelRect seatLabelRect-ppp"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelHeader seatLabelHeader-ppp"
  }, getTicketTypeIcon(undefined, ticketTypeGroupName), /*#__PURE__*/React__default.createElement("span", null, (ticketTypeGroupName || '').toUpperCase())), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelContent seatLabelContent-ppp"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelRow"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItem"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, "TABLE"), _map(visibleTickets, function (item) {
    var _getTicketTypeGroupAn = getTicketTypeGroupAndName(item.ticket_type_name),
        ticketTypeName = _getTicketTypeGroupAn.ticketTypeName;

    return /*#__PURE__*/React__default.createElement("div", {
      key: item.ticket_type_id,
      className: "seatLabelItemText"
    }, (ticketTypeName || '').toUpperCase());
  })), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItem"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, "PRICE"), _map(visibleTickets, function (item) {
    return /*#__PURE__*/React__default.createElement("div", {
      key: item.ticket_type_id,
      className: "seatLabelItemText"
    }, tierPrices[item.ticket_type_tier_id]);
  }))))), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelTriangle seatLabelTriangle-ppp"
  }));
};

var TableLabel = function TableLabel(props) {
  var price = props.price,
      status = props.status,
      top = props.top,
      left = props.left,
      visible = props.visible,
      showReserveButton = props.showReserveButton,
      _props$seatLabelPrice = props.seatLabelPriceText,
      seatLabelPriceText = _props$seatLabelPrice === void 0 ? 'Price' : _props$seatLabelPrice,
      buyerInfo = props.buyerInfo,
      initializeListeners = props.initializeListeners,
      initializeReservationButtonListeners = props.initializeReservationButtonListeners,
      ticketTypeTireRelationsArray = props.ticketTypeTireRelationsArray,
      tierId = props.tierId,
      isReservedByMe = props.isReservedByMe;
  var showPrice = seatLabelPriceText && price;

  var visibleTickets = _filter(ticketTypeTireRelationsArray, {
    ticket_type_tier_id: tierId
  });

  var showDeposit = visibleTickets[0] && visibleTickets[0].ticket_type_deposit;

  var showNumberOfGuests = function showNumberOfGuests() {
    if (!visibleTickets[0]) {
      return false;
    }

    return visibleTickets[0].ticket_type_min_number_of_guests && visibleTickets[0].ticket_type_max_number_of_guests;
  };

  var numberOfGuests = function numberOfGuests() {
    var minGuests = visibleTickets[0].ticket_type_min_number_of_guests;
    var maxGuests = visibleTickets[0].ticket_type_max_number_of_guests;

    if (minGuests && maxGuests && minGuests === maxGuests) {
      return maxGuests + ' Guests';
    } else {
      return minGuests + '-' + maxGuests + ' Guests';
    }
  };

  return visible ? /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelContainer",
    style: {
      top: top,
      left: left
    },
    ref: initializeListeners
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelRect"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelHeader"
  }, /*#__PURE__*/React__default.createElement("span", {
    style: {
      marginLeft: '10px'
    }
  }, "Table Information")), /*#__PURE__*/React__default.createElement("div", {
    className: classNames({
      'seatLabelContent seatLabelContent-large': showReserveButton,
      seatLabelContent: !showReserveButton
    })
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelRow"
  }, showPrice && /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItem"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, seatLabelPriceText), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemText"
  }, price)), /*#__PURE__*/React__default.createElement("div", {
    className: classNames({
      'seatLabelItem flex2': showPrice,
      seatLabelItem: !showPrice
    })
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, "Table"), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemText"
  }, visibleTickets[0] ? visibleTickets[0].ticket_type_name : ''))), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelRow"
  }, showDeposit && /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItem"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, "Deposit"), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemText"
  }, visibleTickets[0] ? visibleTickets[0].ticket_type_deposit + '%' : '')), /*#__PURE__*/React__default.createElement("div", {
    className: classNames({
      'seatLabelItem flex2': showDeposit,
      seatLabelItem: !showDeposit
    })
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, "Status"), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemText",
    style: {
      color: isReservedByMe ? '#ffa500' : COLORS[status.toUpperCase()]
    }
  }, status.toUpperCase() === 'RESERVED' ? 'Locked for Purchase' : status))), showNumberOfGuests() && /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelRow"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItem"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, "Number of Guests"), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemText"
  }, numberOfGuests()))), showReserveButton && /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelRow"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItem"
  }, /*#__PURE__*/React__default.createElement("button", {
    className: "seatLabelButton",
    ref: initializeReservationButtonListeners
  }, "Reserve")))), buyerInfo && /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelContent withBuyerInfo"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelRow"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItem flex3"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, "Buyer"), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemText"
  }, buyerInfo.name))), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelRow"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItem flex3"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, "Buyer Email"), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemText"
  }, buyerInfo.email))), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelRow"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItem flex3"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, "Order ID"), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemText"
  }, buyerInfo.order)), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItem flex3"
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemTitle"
  }, "Ticket Hash"), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelItemText"
  }, buyerInfo.ticket))))), /*#__PURE__*/React__default.createElement("div", {
    className: "seatLabelTriangle"
  })) : null;
};

var MouseLabel = React__default.memo(function (props) {
  var onReservationButtonClick = props.onReservationButtonClick;
  var reservationButtonRef = null;
  var labelRef = null;

  var avoidBubblingEvent = function avoidBubblingEvent(e) {
    e.preventDefault();
  };

  var initializeListeners = function initializeListeners(element) {
    if (element !== null) {
      element.addEventListener('touchstart', avoidBubblingEvent);
      element.addEventListener('touchend', avoidBubblingEvent);
    } else if (labelRef !== null) {
      labelRef.removeEventListener('touchstart', avoidBubblingEvent);
      labelRef.removeEventListener('touchend', avoidBubblingEvent);
    }

    labelRef = element;
  };

  var handleOnTouchEnd = function handleOnTouchEnd(e) {
    e.isFromLabelReservationButton = true;
    onReservationButtonClick(e);
  };

  var initializeReservationButtonListeners = function initializeReservationButtonListeners(element) {
    if (element !== null) {
      element.addEventListener('touchend', handleOnTouchEnd);
    } else if (reservationButtonRef !== null) {
      reservationButtonRef.removeEventListener('touchend', handleOnTouchEnd);
    }

    reservationButtonRef = element;
  };

  if (props.visible) {
    if (props.sgType === SEAT_GROUP_TYPES.STANDING) {
      return /*#__PURE__*/React__default.createElement(StandingAreaLabel, _extends({}, props, {
        initializeListeners: initializeListeners,
        initializeReservationButtonListeners: initializeReservationButtonListeners
      }));
    } else if (props.is_PPP) {
      return /*#__PURE__*/React__default.createElement(SeatLabelMin, props);
    } else if (props.isTableMap) {
      return /*#__PURE__*/React__default.createElement(TableLabel, _extends({}, props, {
        initializeListeners: initializeListeners,
        initializeReservationButtonListeners: initializeReservationButtonListeners
      }));
    } else {
      return /*#__PURE__*/React__default.createElement(SeatLabel, _extends({}, props, {
        initializeListeners: initializeListeners,
        initializeReservationButtonListeners: initializeReservationButtonListeners
      }));
    }
  } else {
    return null;
  }
});

var nullFn = function nullFn() {
  return null;
};

var SeatMapView = /*#__PURE__*/function (_Component) {
  _inheritsLoose(SeatMapView, _Component);

  function SeatMapView(props) {
    var _showStatusesLabel;

    var _this;

    _this = _Component.call(this, props) || this;

    _this.preEventHandler = function (eventFn) {
      return function () {
        if (_this.props.disabled) {
          return null;
        }

        return eventFn.apply(void 0, arguments);
      };
    };

    _this.drawMapComponent = function () {
      try {
        var _this$state = _this.state,
            events = _this$state.events,
            statuses = _this$state.statuses,
            _this$state$isBlockMa = _this$state.isBlockMap,
            isBlockMap = _this$state$isBlockMa === void 0 ? false : _this$state$isBlockMa;
        var _this$props = _this.props,
            width = _this$props.width,
            height = _this$props.height,
            predefinedSeats = _this$props.predefinedSeats,
            isSelectionOn = _this$props.isSelectionOn,
            activeGroups = _this$props.activeGroups,
            _this$props$seatsToRe = _this$props.seatsToRecolor,
            seatsToRecolor = _this$props$seatsToRe === void 0 ? [] : _this$props$seatsToRe,
            _this$props$enableTic = _this$props.enableTicketTypeAssign,
            enableTicketTypeAssign = _this$props$enableTic === void 0 ? false : _this$props$enableTic,
            disabled = _this$props.disabled,
            selectedTierId = _this$props.selectedTierId,
            _this$props$seatData = _this$props.seatData,
            _this$props$seatData$ = _this$props$seatData.meta,
            center_x = _this$props$seatData$.center_x,
            center_y = _this$props$seatData$.center_y,
            range_x = _this$props$seatData$.range_x,
            range_y = _this$props$seatData$.range_y,
            minZoomDistance = _this$props$seatData$.minZoomDistance,
            seat_circle_radius = _this$props$seatData$.seat_circle_radius,
            name = _this$props$seatData$.name,
            seatmap = _this$props$seatData.seatmap,
            groups = _this$props$seatData.groups,
            creatorInfo = _this$props$seatData.creatorInfo,
            svgData = _this$props$seatData.svgData;

        var backgroundColor = _get(svgData, 'bgColor', COLORS.BACKGROUND);

        var container = document.getElementById('seatCanvasContainer');
        return Promise.resolve(createMap(container, seatmap, groups, creatorInfo, predefinedSeats, {
          width: width,
          height: height,
          center_x: center_x,
          center_y: center_y,
          range_x: range_x,
          range_y: range_y,
          minZoomDistance: minZoomDistance,
          backgroundColor: backgroundColor,
          seat_circle_radius: seat_circle_radius,
          statuses: statuses,
          events: events,
          isSelectionOn: isSelectionOn,
          isBlockMap: isBlockMap,
          svgData: svgData,
          enableTicketTypeAssign: enableTicketTypeAssign,
          disabled: disabled,
          selectedTierId: selectedTierId
        })).then(function (_ref) {
          var _extends2;

          var recolorSeats = _ref.recolorSeats,
              cancelAnimation = _ref.cancelAnimation,
              statusesUpdate = _ref.statusesUpdate,
              controlsReset = _ref.controlsReset,
              controlsZoomIn = _ref.controlsZoomIn,
              controlsZoomOut = _ref.controlsZoomOut,
              activateGroups = _ref.activateGroups,
              startAnimation = _ref.startAnimation;
          window.document.dispatchEvent(new CustomEvent('mapDrawn'));
          _this.controlsReset = controlsReset;
          _this.controlsZoomIn = controlsZoomIn;
          _this.controlsZoomOut = controlsZoomOut;
          _this.cancelAnimation = cancelAnimation;
          _this.startAnimation = startAnimation;
          _this.statusesUpdate = statusesUpdate;
          _this.activateGroups = activateGroups;
          _this.recolorSeats = recolorSeats;

          if (name === 'papaya') {
            _this.is_PPP = true;
          }

          _this.activateGroups(activeGroups);

          _this.recolorSeats(seatsToRecolor);

          _this.processNewStatuses(statuses);

          _this.setState({
            showStatusesLabel: _extends({}, _this.state.showStatusesLabel, (_extends2 = {}, _extends2[SEAT_TYPES.WHEELCHAIR] = _this.areThereWheelchairSeats(), _extends2))
          });
        });
      } catch (e) {
        return Promise.reject(e);
      }
    };

    _this.mouseOverSeat = function (e, isTab) {
      var _e$seat;

      if (isTab === void 0) {
        isTab = false;
      }

      if (((_e$seat = e.seat) === null || _e$seat === void 0 ? void 0 : _e$seat.objectType) === 'UNCLICKABLE_TABLE') {
        return;
      }

      if (_this.selectionInProgress || _this.props.disableHoverPopup) {
        return;
      }

      var seat = e.seat,
          position = e.position;
      if (!seat) return;

      if (_this.state.isBlockMap) {
        seat.status === SEAT_STATUSES.AVAILABLE && seat.changeBorderColor(COLORS.SELECTED_BLOCK_BORDER);

        if (seat.status === SEAT_STATUSES.AVAILABLE) {
          document.body.style.cursor = _this.props.loading ? 'progress' : 'pointer';
        }
      } else {
        var enableTicketTypeAssign = _this.props.enableTicketTypeAssign;
        seat.changeBorderColor('#000000');

        var seatLabelParams = _extends({}, _this.state.seatLabelParams);

        seatLabelParams.showReserveButton = false;

        seatLabelParams.onReservationButtonClick = function () {};

        if (!seat.isExternalLink) {
          switch (seat.status) {
            case SEAT_STATUSES.AVAILABLE:
              seatLabelParams.status = 'Available';
              seatLabelParams.isReservedByMe = false;
              seatLabelParams.showReserveButton = isTab;

              seatLabelParams.onReservationButtonClick = function () {
                _this.seatClick(e);

                _this.clearTap();
              };

              document.body.style.cursor = _this.props.loading ? 'progress' : 'pointer';
              break;

            case SEAT_STATUSES.HOLD:
              seatLabelParams.status = 'On Hold';
              seatLabelParams.isReservedByMe = false;
              document.body.style.cursor = _this.props.loading ? 'progress' : 'pointer';
              break;

            case SEAT_STATUSES.RESERVED:
              seatLabelParams.status = 'Reserved';
              seatLabelParams.isReservedByMe = false;
              break;

            case SEAT_STATUSES.RESERVED_BY_ME:
              seatLabelParams.status = 'Reserved';
              seatLabelParams.isReservedByMe = true;
              document.body.style.cursor = _this.props.loading ? 'progress' : 'pointer';
              break;

            case SEAT_STATUSES.BOOKED:
              seatLabelParams.status = 'Sold';
              seatLabelParams.isReservedByMe = false;
              break;

            case SEAT_STATUSES.BLOCKED:
              seatLabelParams.status = 'BLOCKED';
              seatLabelParams.isReservedByMe = false;
              break;

            default:
              seatLabelParams.status = 'Sold';
              seatLabelParams.isReservedByMe = false;
              break;
          }

          seatLabelParams.visible = true;
        } else {
          seatLabelParams.visible = false;
          document.body.style.cursor = 'pointer';
        }

        if (enableTicketTypeAssign) {
          document.body.style.cursor = 'pointer';
        }

        seatLabelParams.color = "#" + seat.material.color.getHexString();
        seatLabelParams.sgType = seat.sgType;
        seatLabelParams.top = position.y;
        seatLabelParams.left = position.x;
        seatLabelParams.seat = seat.label;
        seatLabelParams.row = seat.rowLabel;
        seatLabelParams.seatType = seat.seatType;
        seatLabelParams.section = seat.sectionLabel;
        seatLabelParams.tierId = seat.tierId;
        seatLabelParams.price = _get(window, "tierPrices." + seat.tierId, '');
        seatLabelParams.tierNames = _get(window, "tierNames." + seat.tierId, '');
        seatLabelParams.seatLabelPriceText = _get(window, 'seatLabelPriceText', 'Price');
        seatLabelParams.buyerInfo = _get(_this.props, "buyersInfo." + seat.seatId, null);

        _this.setSeatLabelParameter(seatLabelParams);
      }
    };

    _this.mouseOutSeat = function (e) {
      var seat = e.seat;

      if (seat.objectType === 'UNCLICKABLE_TABLE') {
        return;
      }

      !seat.isSelected && seat.changeBorderColorToDefault();
      document.body.style.cursor = _this.props.loading ? 'progress' : 'auto';

      _this.setSeatLabelParameter({
        visible: false
      });
    };

    _this.seatClick = function (e) {
      if (!e.seat || e.seat && e.seat.disabled) {
        return;
      }

      if (e.seat.objectType === 'UNCLICKABLE_TABLE') {
        return;
      }

      _this.setSeatLabelParameter({
        visible: false
      });

      _this.props.events.onSeatClick && _this.props.events.onSeatClick(e);
      e && e.seat && !e.seat.isSelected && e.seat.changeBorderColorToDefault();
    };

    _this.clearTap = function () {
      _this.lastTappedSeat = null;

      _this.setSeatLabelParameter({
        visible: false
      });
    };

    _this.seatTap = function (e) {
      var seat = e.seat;

      if (seat.objectType === 'UNCLICKABLE_TABLE') {
        return;
      }

      if (_this.lastTappedSeat === seat && _includes([SEAT_STATUSES.AVAILABLE, SEAT_STATUSES.RESERVED_BY_ME], seat.status) || seat.isExternalLink) {
        _this.lastTappedSeat = null;

        _this.seatClick(e);
      } else {
        _this.lastTappedSeat = seat;

        _this.mouseOverSeat(e, true);
      }
    };

    _this.setSeatLabelParameter = function (params) {
      _this.setState({
        seatLabelParams: _extends({}, _this.state.seatLabelParams, params)
      });
    };

    _this.onSelectStart = function () {
      _this.selectionInProgress = true;
    };

    _this.onSelectUp = function (e) {
      _this.selectionInProgress = false;
      _this.props.events.onSelectUp && _this.props.events.onSelectUp(e);
    };

    _this.state = {
      statuses: props.statuses,
      events: _this.createEvents(),
      seatLabelParams: {
        price: null,
        row: null,
        seat: null,
        status: null,
        section: null,
        color: null,
        top: null,
        left: null,
        visible: false,
        showReserveButton: false,
        onReservationButtonClick: null,
        tierId: null,
        seatType: SEAT_TYPES.CHAIR,
        seatLabelPriceText: 'Price'
      },
      showStatusesLabel: (_showStatusesLabel = {}, _showStatusesLabel[SEAT_STATUSES.HOLD] = false, _showStatusesLabel[SEAT_STATUSES.BLOCKED] = false, _showStatusesLabel[SEAT_STATUSES.AVAILABLE] = true, _showStatusesLabel[SEAT_STATUSES.RESERVED] = !props.isBlockMap, _showStatusesLabel[SEAT_STATUSES.BOOKED] = true, _showStatusesLabel[SEAT_STATUSES.PREDEFINED] = !_isEmpty(props.predefinedSeats), _showStatusesLabel[SEAT_TYPES.WHEELCHAIR] = false, _showStatusesLabel),
      selectionInProgress: false,
      isBlockMap: props.isBlockMap || false,
      ticketTypeTireRelationsArray: props.ticketTypeTireRelationsArray,
      tierPrices: props.tierPrices
    };
    _this.controlsReset = nullFn;
    _this.is_PPP = false;
    _this.lastTappedSeat = null;
    return _this;
  }

  var _proto = SeatMapView.prototype;

  _proto.componentDidMount = function componentDidMount() {
    try {
      var _this3 = this;

      var _this3$props$exportTi = _this3.props.exportTiersColors,
          exportTiersColors = _this3$props$exportTi === void 0 ? function () {} : _this3$props$exportTi;
      return Promise.resolve(_this3.drawMapComponent()).then(function () {
        _this3.startAnimation && _this3.startAnimation();
        exportTiersColors(tierColorObj);
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.componentDidUpdate = function componentDidUpdate(prevProps) {
    var _this$props2 = this.props,
        statuses = _this$props2.statuses,
        loading = _this$props2.loading,
        activeGroups = _this$props2.activeGroups,
        selectedTierId = _this$props2.selectedTierId;

    if (!_isEqual(prevProps.statuses, statuses)) {
      this.processNewStatuses(statuses);
    }

    if (prevProps.loading !== loading) {
      document.body.style.cursor = loading ? 'progress' : 'auto';
    }

    if (prevProps.activeGroups !== this.props.activeGroups && this.activateGroups) {
      this.activateGroups(activeGroups);
    }

    var _this$props$seatsToRe2 = this.props.seatsToRecolor,
        nextSeatsToRecolor = _this$props$seatsToRe2 === void 0 ? [] : _this$props$seatsToRe2;
    var _prevProps$seatsToRec = prevProps.seatsToRecolor,
        seatsToRecolor = _prevProps$seatsToRec === void 0 ? [] : _prevProps$seatsToRec;

    if (nextSeatsToRecolor !== seatsToRecolor && this.recolorSeats) {
      var seatIdsToRecolor = nextSeatsToRecolor.map(function (_ref2) {
        var seatId = _ref2.seatId;
        return seatId;
      });
      var seatsToRecover = seatsToRecolor.filter(function (_ref3) {
        var seatId = _ref3.seatId;
        return !seatIdsToRecolor.includes(seatId);
      });

      if (this.recolorSeats) {
        this.recolorSeats(nextSeatsToRecolor, seatsToRecover);
      }
    }

    if (selectedTierId && prevProps.selectedTierId !== selectedTierId) {
      this.cancelAnimation();
      this.drawMapComponent();
    }

    if (!_isEqual(prevProps.seatData, this.props.seatData)) {
      this.cancelAnimation();
      this.drawMapComponent();
    }

    this.startAnimation && this.startAnimation();
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    if (this.cancelAnimation) {
      this.cancelAnimation();
    }
  };

  _proto.createEvents = function createEvents() {
    var props = this.props,
        preEventHandler = this.preEventHandler,
        seatClick = this.seatClick,
        mouseOutSeat = this.mouseOutSeat,
        mouseOverSeat = this.mouseOverSeat,
        onSelectUp = this.onSelectUp,
        clearTap = this.clearTap,
        seatTap = this.seatTap,
        onSelectStart = this.onSelectStart;
    return {
      onClick: preEventHandler(seatClick),
      onTap: preEventHandler(seatTap),
      onMouseOver: preEventHandler(mouseOverSeat),
      onMouseOut: preEventHandler(mouseOutSeat),
      onTierIdChange: preEventHandler(props.events.onTierIdChange),
      onSelectUp: preEventHandler(onSelectUp),
      onClearTap: preEventHandler(clearTap),
      onSelectStart: preEventHandler(onSelectStart)
    };
  };

  _proto.areThereWheelchairSeats = function areThereWheelchairSeats() {
    var seatmap = this.props.seatData.seatmap;
    return !!seatmap.find(function (group) {
      return group.children && !!group.children.find(function (row) {
        return row.children && !!row.children.find(function (seat) {
          return seat.type === SEAT_TYPES.WHEELCHAIR;
        });
      });
    });
  };

  _proto.processNewStatuses = function processNewStatuses(statuses) {
    var enableTicketTypeAssign = this.props.enableTicketTypeAssign;

    if (!enableTicketTypeAssign && this.statusesUpdate) {
      var _extends3;

      var statusesCount = this.statusesUpdate(statuses);
      this.setState({
        statuses: statuses,
        showStatusesLabel: _extends({}, this.state.showStatusesLabel, (_extends3 = {}, _extends3[SEAT_STATUSES.HOLD] = statusesCount && statusesCount.get(SEAT_STATUSES.HOLD) > 0, _extends3[SEAT_STATUSES.BLOCKED] = statusesCount && statusesCount.get(SEAT_STATUSES.BLOCKED) > 0, _extends3))
      });
    }
  };

  _proto.render = function render() {
    var _this$state2 = this.state,
        seatLabelParams = _this$state2.seatLabelParams,
        isBlockMap = _this$state2.isBlockMap,
        showStatusesLabel = _this$state2.showStatusesLabel,
        ticketTypeTireRelationsArray = _this$state2.ticketTypeTireRelationsArray,
        tierPrices = _this$state2.tierPrices,
        _this$props3 = this.props,
        isTableMap = _this$props3.isTableMap,
        enableTicketTypeAssign = _this$props3.enableTicketTypeAssign,
        _this$props3$seatData = _this$props3.seatData.svgData,
        svgData = _this$props3$seatData === void 0 ? {} : _this$props3$seatData,
        controlsReset = this.controlsReset,
        controlsZoomIn = this.controlsZoomIn,
        controlsZoomOut = this.controlsZoomOut;
    var isSVGMap = !_isEmpty(svgData);
    return /*#__PURE__*/React__default.createElement("div", {
      className: "seatMapContainer"
    }, !enableTicketTypeAssign && /*#__PURE__*/React__default.createElement(Fragment, null, this.is_PPP ? /*#__PURE__*/React__default.createElement(ColorLabelsPPP, {
      showStatusesLabel: showStatusesLabel,
      colors: {
        RESERVED: svgData.reservation_color,
        BOOKED: svgData.booked_color
      }
    }) : /*#__PURE__*/React__default.createElement(ColorLabels, {
      showStatusesLabel: showStatusesLabel,
      isTableMap: isTableMap,
      isSVGMap: isSVGMap,
      svgData: svgData
    })), /*#__PURE__*/React__default.createElement(ControlButtons, {
      isWheelChairAvailable: showStatusesLabel[SEAT_TYPES.WHEELCHAIR],
      reset: controlsReset,
      zoomIn: controlsZoomIn,
      zoomOut: controlsZoomOut
    }), /*#__PURE__*/React__default.createElement("div", {
      id: "seatCanvasContainer",
      className: "seatMapCanvasContainer"
    }, !isBlockMap && /*#__PURE__*/React__default.createElement(MouseLabel, _extends({}, seatLabelParams, {
      is_PPP: this.is_PPP,
      isTableMap: isTableMap,
      ticketTypeTireRelationsArray: ticketTypeTireRelationsArray,
      tierPrices: tierPrices
    }))));
  };

  return SeatMapView;
}(Component);

export default SeatMapView;
export { COLORS, MESSAGES, SEAT_GROUP_TYPES, SEAT_STATUSES, SEAT_TYPES, SELECT_LIMIT };
//# sourceMappingURL=index.modern.js.map
